`timescale 1ns/1ps



`define AXI_ALEN_1            4'b0000
`define AXI_ALEN_2            4'b0001
`define AXI_ALEN_3            4'b0010
`define AXI_ALEN_4            4'b0011
`define AXI_ALEN_5            4'b0100
`define AXI_ALEN_6            4'b0101
`define AXI_ALEN_7            4'b0110
`define AXI_ALEN_8            4'b0111
`define AXI_ALEN_9            4'b1000
`define AXI_ALEN_10           4'b1001
`define AXI_ALEN_11           4'b1010
`define AXI_ALEN_12           4'b1011
`define AXI_ALEN_13           4'b1100
`define AXI_ALEN_14           4'b1101
`define AXI_ALEN_15           4'b1110
`define AXI_ALEN_16           4'b1111

`define AXI_ASIZE_8           3'b000
`define AXI_ASIZE_16          3'b001
`define AXI_ASIZE_32          3'b010
`define AXI_ASIZE_64          3'b011
`define AXI_ASIZE_128         3'b100
`define AXI_ASIZE_256         3'b101
`define AXI_ASIZE_512         3'b110
`define AXI_ASIZE_1024        3'b111

`define AXI_ABURST_FIXED      2'b00
`define AXI_ABURST_INCR       2'b01
`define AXI_ABURST_WRAP       2'b10

`define AXI_ALOCK_NOLOCK      2'b00
`define AXI_ALOCK_EXCL        2'b01
`define AXI_ALOCK_LOCKED      2'b10

`define AXI_RESP_OKAY         2'b00
`define AXI_RESP_EXOKAY       2'b01
`define AXI_RESP_SLVERR       2'b10
`define AXI_RESP_DECERR       2'b11



module A11AhbLiteMToAxi
  (
   CLK,
   RESETn,

   HADDR,
   HTRANS,
   HWRITE,
   HSIZE,
   HBURST,
   HPROT,
   HMASTLOCK,
   HBSTRB,
   HAUSER,
   HREADYOUT,
   HRESP,

   
   AWADDR, 
   AWLEN, 
   AWSIZE, 
   AWBURST, 
   AWLOCK, 
   AWCACHE, 
   AWPROT, 
   AWUSER,
   AWVALID, 
   AWREADY, 
   
   WSTRB, 
   WLAST, 
   WVALID, 
   WREADY, 
   
   BRESP, 
   BVALID, 
   BREADY, 
   
   ARADDR, 
   ARLEN, 
   ARSIZE, 
   ARBURST, 
   ARLOCK, 
   ARCACHE, 
   ARPROT, 
   ARUSER,
   ARVALID, 
   ARREADY, 
   
   RRESP,
   RLAST,
   RVALID, 
   RREADY, 
   
   CSYSREQ, 
   CSYSACK, 
   CACTIVE,

   SCANENABLE,
   SCANINCLK,
   SCANOUTCLK
   );


  parameter DATA_WIDTH = 64;           // data bus width, default = 64-bit

  parameter AUSER_WIDTH = 1; 
  parameter AUSER_MAX   = AUSER_WIDTH-1;

  parameter STRB_WIDTH = DATA_WIDTH/8; // WSTRB width
  parameter STRB_MAX   = STRB_WIDTH-1; // WSTRB max index


  input               CLK;        // common AHB/AXI Clock
  input               RESETn;     // common AHB/AXI Reset

  input        [31:0] HADDR;      // address bus
  input         [1:0] HTRANS;     // transfer type
  input               HWRITE;     // transfer direction
  input         [2:0] HSIZE;      // transfer size
  input         [2:0] HBURST;     // burst type
  input         [5:0] HPROT;      // protection control
  input               HMASTLOCK;  // HIGH for locked transfer
  input  [STRB_MAX:0] HBSTRB;     // byte lane strobes
  input [AUSER_MAX:0] HAUSER;     // user signals
  output              HREADYOUT;  // transfer done
  output        [2:0] HRESP;      // slave response

  output        [31:0] AWADDR;     // write address
  output         [3:0] AWLEN;      // burst length
  output         [2:0] AWSIZE;     // burst size
  output         [1:0] AWBURST;    // burst type
  output         [1:0] AWLOCK;     // burst lock type
  output         [3:0] AWCACHE;    // caching attributes
  output         [2:0] AWPROT;     // protection attributes
  output [AUSER_MAX:0] AWUSER;     // write user signals 
  output               AWVALID;    // write address valid
  input                AWREADY;    // write address ready

  output [STRB_MAX:0] WSTRB;      // write strobes
  output              WLAST;      // write last
  output              WVALID;     // write valid
  input               WREADY;     // write ready
  
  input         [1:0] BRESP;      // write response
  input               BVALID;     // response valid
  output              BREADY;     // response ready

  output        [31:0] ARADDR;     // write address
  output         [3:0] ARLEN;      // burst length
  output         [2:0] ARSIZE;     // burst size
  output         [1:0] ARBURST;    // burst type
  output         [1:0] ARLOCK;     // burst lock type
  output         [3:0] ARCACHE;    // caching attributes
  output         [2:0] ARPROT;     // protection attributes
  output [AUSER_MAX:0] ARUSER;     // read user signals 
  output               ARVALID;    // write address valid
  input                ARREADY;    // write address ready

  input        [1:0]  RRESP;      // read response
  input               RLAST;      // last read transfer
  input               RVALID;     // read valid
  output              RREADY;     // read ready

  input               CSYSREQ;    // system low-power request
  output              CSYSACK;    // system low-power acknowledge
  output              CACTIVE;    // clock active

  input               SCANENABLE; // enable scan test mode
  input               SCANINCLK;  // scan data input
  output              SCANOUTCLK; // scan data output


  `define AHB_TRN_IDLE        2'b00   // idle transfer
  `define AHB_TRN_BUSY        2'b01   // busy transfer
  `define AHB_TRN_NONSEQ      2'b10   // non-sequential
  `define AHB_TRN_SEQ         2'b11   // sequential

  `define AHB_BUR_SINGLE      3'b000  // single
  `define AHB_BUR_INCR        3'b001  // incrementing
  `define AHB_BUR_WRAP4       3'b010  // 4-beat wrap
  `define AHB_BUR_INCR4       3'b011  // 4-beat incr
  `define AHB_BUR_WRAP8       3'b100  // 8-beat wrap
  `define AHB_BUR_INCR8       3'b101  // 8-beat incr
  `define AHB_BUR_WRAP16      3'b110  // 16-beat wrap
  `define AHB_BUR_INCR16      3'b111  // 16-beat incr

  `define AHB_RESP_OKAY       3'b000   // okay response
  `define AHB_RESP_ERROR      3'b001   // error response
  `define AHB_RESP_RETRY      3'b010   // retry response
  `define AHB_RESP_SPLIT      3'b011   // split response
  `define AHB_RESP_XFAIL      3'b100   // xfail response
  

  wire              CLK;        // common AHB/AXI Clock
  wire              RESETn;     // common AHB/AXI Reset

  wire        [31:0] HADDR;      // address bus
  wire         [1:0] HTRANS;     // transfer type
  wire               HWRITE;     // transfer direction
  wire         [2:0] HSIZE;      // transfer size
  wire         [2:0] HBURST;     // burst type
  wire         [5:0] HPROT;      // protection control
  wire               HMASTLOCK;  // HIGH for locked transfer
  wire  [STRB_MAX:0] HBSTRB;     // byte lane strobes for reads and writes
  wire [AUSER_MAX:0] HAUSER;     // user signals
  wire               HREADYOUT;  // transfer done
  wire         [2:0] HRESP;      // slave response

  wire        [31:0] AWADDR;     // write address
  wire         [3:0] AWLEN;      // burst length
  wire         [2:0] AWSIZE;     // burst size
  wire         [1:0] AWBURST;    // burst type
  wire         [1:0] AWLOCK;     // burst lock type
  wire         [3:0] AWCACHE;    // caching attributes
  wire         [2:0] AWPROT;     // protection attributes
  wire [AUSER_MAX:0] AWUSER;     // write user signals 
  wire               AWVALID;    // write address valid
  wire               AWREADY;    // write address ready

  wire [STRB_MAX:0] WSTRB;      // write strobes
  wire              WLAST;      // write last
  wire              WVALID;     // write valid
  wire              WREADY;     // write ready

  wire        [1:0] BRESP;      // write response
  wire              BVALID;     // response valid
  wire              BREADY;     // response ready

  wire        [31:0] ARADDR;     // write address
  wire         [3:0] ARLEN;      // burst length
  wire         [2:0] ARSIZE;     // burst size
  wire         [1:0] ARBURST;    // burst type
  wire         [1:0] ARLOCK;     // burst lock type
  wire         [3:0] ARCACHE;    // caching attributes
  wire         [2:0] ARPROT;     // protection attributes
  wire [AUSER_MAX:0] ARUSER;     // read user signals 
  wire               ARVALID;    // write address valid
  wire               ARREADY;    // write address ready

  wire       [1:0]  RRESP;      // read response
  wire              RVALID;     // read valid
  wire              RREADY;     // read ready

  wire              CSYSREQ;    // system low-power request    
  reg               CSYSACK;    // system low-power acknowledge
  wire              CACTIVE;    // block active


  wire              iHREADYOUT;   // transfer done
  wire        [2:0] iHRESP;       // slave response

  reg               iWVALID;      // write data channel valid
  reg  [STRB_MAX:0] iWSTRB;       // write strobes
  reg               iWLAST;       // last write data item
  reg               iRREADY;      // read data channel ready
  reg               iCACTIVE;     // block active
                                  
  wire              AValidNxt;    // D input for AVALID register
  wire              AWriteNxt;    // D input for AWRITE register
  reg         [3:0] ALenNxt;      // D input for ALEN register
  reg         [1:0] ABurstNxt;    // D input for ABURST register
  wire        [3:0] ACacheNxt;    // D input for ACACHE register
  wire        [2:0] AProtNxt;     // D input for APROT register
  wire        [1:0] ALockNxt;     // D input for ALOCK register
                                  
  wire              RReadyNxt;    // D input for RREADY output register
                                  
  wire              WValidNxt;    // D input for WVALID output register
  wire              WLastNxt;     // D input for WLAST output register
  wire [STRB_MAX:0] WStrbNxt;     // D input for WSTRB output register
                                
  wire              CActiveNxt;   // D input for CACTIVE output register
                                  
  reg               AWRITE;       // burst direction
  reg        [31:0] ADDR;         // write address
  reg         [3:0] ALEN;         // burst length
  reg         [2:0] ASIZE;        // burst size
  reg         [1:0] ABURST;       // burst type
  reg         [1:0] ALOCK;        // burst lock type
  reg         [3:0] ACACHE;       // caching attributes
  reg         [2:0] APROT;        // protection attributes
  reg [AUSER_MAX:0] AUSER;        // auser signals
  reg               AVALID;       // write address valid
  wire              AREADY;       // write address ready

  reg               AhbDataValid; // Valid data phase on AHB
  wire              NewAddr;      // new AXI address should be generated

  reg               ReadXfer;     // High when read burst in progress
  wire              ReadXferNxt;  // D input for ReadXfer register
  reg         [3:0] WCount;       // write data counter for generating WLAST
  wire        [3:0] WCountNxt;    // D input for WCount register
  reg               Exclusive;    // indicates an exclusive operation
  
  reg               BRespWait;    // high when waiting for a write response
  wire              BRespWaitNxt; // D input for BRespWait
  
  reg         [2:0] HRespReg;     // registered HRESP
  reg               HReadyReg;    // registered HREADYOUT

  reg               SwpRead;      // Detects a read transfer from an ARM SWP
  wire              SwpReadNxt;   // D input for SwpRead register
  wire              SwpReadError; // ARM SWP read receives an error response
  wire              DummyWrite;   // Indicates the dummy write is in progress
  reg               DummyWriteReg;// Dummy write register
  
  reg               ReadErrorMidBurst;    // ERROR response received mid-burst
  reg               ReadTerminate;        // AHB master has terminated a read  
  wire              ReadErrorMidBurstNxt; // D-input to ReadErrorMidBurst reg
  wire              ReadTerminateNxt;     // D-input to ReadTerminate register

  reg               MissedAddr;           // high when an address is missed 
                    
  wire              MissedAddrNxt;        // D-input to MissedAddr register


`ifdef ASSERT_ON
  reg               UnalignedAddr;        // used to detect a burst with an 
`endif                                    // unaligned address
  

  
  always @(negedge RESETn or posedge CLK)
  begin : p_AhbDataValidSeq
    if (~RESETn)
      AhbDataValid <= 1'b0;
    else
      if (iHREADYOUT)
        AhbDataValid <= HTRANS[1]; // valid AHB address
  end
  
  assign NewAddr = (iHREADYOUT & 
                    ((HTRANS == `AHB_TRN_NONSEQ) | 
                     ((HTRANS == `AHB_TRN_SEQ) & (HBURST == `AHB_BUR_INCR))));
                     
                     
  assign AREADY = AWRITE ? AWREADY : ARREADY;

  assign AValidNxt = (RVALID & RLAST & MissedAddr ? 1'b1 :
                      
                      NewAddr & ~ReadTerminateNxt ? 1'b1 : 

                      SwpReadError ? 1'b1 :
                      
                      AREADY ? 1'b0 :
                      
                      AVALID); // otherwise keep stable
                     
  assign AWriteNxt = (NewAddr ? HWRITE :
                      
                      SwpReadError ? 1'b1 :

                      AWRITE); // otherwise keep stable

 
  always @ (HBURST)
  begin : p_ALenBurNxtComb
    case (HBURST) 
      `AHB_BUR_SINGLE: begin
        ALenNxt   = `AXI_ALEN_1;
        ABurstNxt = `AXI_ABURST_INCR;
      end
      
      `AHB_BUR_INCR: begin
        ALenNxt   = `AXI_ALEN_1;
        ABurstNxt = `AXI_ABURST_INCR;
      end
      
      `AHB_BUR_INCR4: begin
        ALenNxt   = `AXI_ALEN_4;
        ABurstNxt = `AXI_ABURST_INCR;
      end
      
      `AHB_BUR_INCR8: begin
        ALenNxt   = `AXI_ALEN_8;
        ABurstNxt = `AXI_ABURST_INCR;
      end

      `AHB_BUR_INCR16: begin
        ALenNxt   = `AXI_ALEN_16;
        ABurstNxt = `AXI_ABURST_INCR;
      end

      `AHB_BUR_WRAP4: begin
        ALenNxt   = `AXI_ALEN_4;
        ABurstNxt = `AXI_ABURST_WRAP;
      end

      `AHB_BUR_WRAP8: begin
        ALenNxt   = `AXI_ALEN_8;
        ABurstNxt = `AXI_ABURST_WRAP;
      end
      
      `AHB_BUR_WRAP16: begin
        ALenNxt   = `AXI_ALEN_16;
        ABurstNxt = `AXI_ABURST_WRAP;
      end

      default : begin // illegal switch, propogate x's
        ALenNxt   = 4'bxxxx;
        ABurstNxt = 3'bxxx;
      end
      
    endcase // case(HBURST)
  end

  

  assign ACacheNxt[0] = HPROT[2];                        // bufferable
  
  assign ACacheNxt[1] = HPROT[3];                        // cacheable
  
  assign ACacheNxt[2] = (HPROT[4] & HPROT[3]) |          // read allocate
                        (~HPROT[4] & HPROT[3] & HPROT[2]);
  
  assign ACacheNxt[3] = ~HPROT[4] & HPROT[3] & HPROT[2]; // write allocate

  
  assign AProtNxt[0] = HPROT[1];  // normal/privileged
  
  assign AProtNxt[1] = 1'b0; // Tie to make bridge secure
  
  assign AProtNxt[2] = ~HPROT[0]; // data/instruction

 
  assign ALockNxt = (~AREADY & AVALID ? ALOCK :
  
                     NewAddr & HPROT[5] ? `AXI_ALOCK_EXCL :

                     NewAddr & ~HMASTLOCK ? `AXI_ALOCK_NOLOCK :
                     
                     SwpReadNxt ? `AXI_ALOCK_LOCKED :
                     
                     AVALID & (ALOCK == `AXI_ALOCK_LOCKED) ? `AXI_ALOCK_NOLOCK :
                     
                     ALOCK);

  always @(negedge RESETn or posedge CLK)
  begin : p_ACtrlSeq
    if (~RESETn)
      Exclusive  <= 1'b0;
    else
      if (NewAddr)
        Exclusive  <= HPROT[5];
  end
  

  always @(negedge RESETn or posedge CLK)
  begin : p_AddrSeq
    if (~RESETn)
    begin
      ADDR   <= {32{1'b0}};
      AUSER  <= {AUSER_WIDTH{1'b0}};
      ALEN   <= 4'b0000;
      ASIZE  <= 3'b000;
      ABURST <= 2'b00;
      ACACHE <= 4'b0000;
      APROT  <= 3'b000;
    end
    else
      if (NewAddr) // control signals only change when new address on AHB
      begin

        AUSER  <= HAUSER;
        ADDR   <= HADDR;
        ALEN   <= ALenNxt;
        ASIZE  <= HSIZE;
        ABURST <= ABurstNxt;
        ACACHE <= ACacheNxt;
        APROT  <= AProtNxt;
    end
  end

  always @(negedge RESETn or posedge CLK)
  begin : p_AValidSeq
    if (~RESETn)
    begin
      AVALID <= 1'b0;
      AWRITE <= 1'b0;
      ALOCK  <= 2'b00;
    end
    else
    begin
      AVALID <= AValidNxt;
      AWRITE <= AWriteNxt;
      ALOCK  <= ALockNxt;
    end
  end

  
  

  assign SwpReadNxt = ~HWRITE & (HTRANS == `AHB_TRN_NONSEQ) & HMASTLOCK &
                      iHREADYOUT;

  always @ (negedge RESETn or posedge CLK)
  begin : p_SwpReadReg
    if (~RESETn)
      SwpRead <= 1'b0;
    else
      if (iHREADYOUT)
        SwpRead <= SwpReadNxt;
  end

  assign SwpReadError = SwpRead & RVALID & RRESP[1] & ~ReadTerminate;

  assign DummyWrite = (SwpReadError ? 1'b1 :

                       BVALID ? 1'b0 :

                       DummyWriteReg);
                       
  always @ (negedge RESETn or posedge CLK)
  begin : p_DummyWriteReg
    if (~RESETn)
      DummyWriteReg <= 1'b0;
    else
      DummyWriteReg <= DummyWrite;
  end

  
  
  assign WValidNxt = ((HTRANS[1] & HWRITE & iHREADYOUT) |

                      SwpReadError) ? 1'b1 :
         
                      WREADY ? 1'b0 :
                      
                      iWVALID; 
  
  assign WCountNxt = ((NewAddr & HWRITE) ? ALenNxt :

                      SwpReadError ? 4'h0 :
                      
                      ((HTRANS == `AHB_TRN_SEQ) & iHREADYOUT) ? WCount - 4'h1 :

                      
                      WCount);

  assign WLastNxt = (WCountNxt == 4'h0) & WValidNxt;

  assign WStrbNxt = (HTRANS[1] & HWRITE & iHREADYOUT ? HBSTRB : 

                     SwpReadError ? {STRB_WIDTH{1'b0}} :
                       
                     iWVALID & ~WREADY ? iWSTRB :
                     
                     {STRB_WIDTH{1'b0}}); // strobes default low

  assign BRespWaitNxt = (iWLAST ? 1'b1 :
                         BVALID ? 1'b0 :
                         BRespWait);

  always @(negedge RESETn or posedge CLK)
  begin : p_WOutputsSeq
    if (~RESETn)
    begin
      iWVALID   <= 1'b0;
      iWLAST    <= 1'b0;
      iWSTRB    <= {STRB_WIDTH{1'b0}};
      WCount    <= 4'h0;
      BRespWait <= 1'b0;
    end
    else
    begin
      iWVALID   <= WValidNxt;
      iWLAST    <= WLastNxt;
      iWSTRB    <= WStrbNxt;
      WCount    <= WCountNxt;
      BRespWait <= BRespWaitNxt;
    end // else: !if(~RESETn)
  end // block: p_WOutputsSeq


  assign BREADY = 1'b1;


  assign ReadXferNxt = ((HTRANS[1] & ~HWRITE & iHREADYOUT)  ? 1'b1 :
                         
                         (RVALID & iRREADY & ~ReadTerminate) ? 1'b0 :
                         
                         ReadXfer); // maintain state
                         
  assign RReadyNxt = (~HWRITE & HTRANS == `AHB_TRN_BUSY & iHREADYOUT) | 
                     
                     (iHRESP != `AHB_RESP_OKAY & ~iHREADYOUT) ? 1'b0 : 1'b1;


  
  assign ReadErrorMidBurstNxt = RVALID & ~RLAST & iRREADY & // read data valid
                                RRESP[1] &                  // error response
                                ~ReadTerminate; 
  
  assign ReadTerminateNxt = ReadErrorMidBurst &  
                            
                            ~HTRANS[0] ? 1'b1 : 
                            
                            RVALID & iRREADY & RLAST ? 1'b0 :
                            
                            ReadTerminate;

  assign MissedAddrNxt = ReadTerminateNxt & NewAddr ? 1'b1 : 
                         
                         RVALID & RLAST & iRREADY ? 1'b0 :
                         
                         MissedAddr;
                         
  always @ (negedge RESETn or posedge CLK)
  begin : p_RBusReg
    if (~RESETn)
    begin
      ReadXfer <= 1'b0;
      iRREADY  <= 1'b0;
      ReadErrorMidBurst <= 1'b0;
      ReadTerminate <= 1'b0;
      MissedAddr <= 1'b0;
    end
    else
    begin
      ReadXfer <= ReadXferNxt;
      iRREADY  <= RReadyNxt;
      ReadErrorMidBurst <= ReadErrorMidBurstNxt;
      ReadTerminate <= ReadTerminateNxt;
      MissedAddr <= MissedAddrNxt;
    end
  end 


  assign iHREADYOUT = ~AhbDataValid ? 1'b1 :
  
                      AVALID & ~AREADY ? 1'b0 :

                      ReadTerminate ? 1'b0 :
                      
                      RVALID & iRREADY &
                      (RRESP[1] | 
                       (Exclusive & RRESP == `AXI_RESP_OKAY)) ? 1'b0 :
                   
                      BVALID & 
                      (BRESP[1] | DummyWriteReg | 
                       (Exclusive & BRESP == `AXI_RESP_OKAY)) ? 1'b0 :
                  
                      iWVALID & ~WREADY ? 1'b0 :
                  
                      BRespWaitNxt ? 1'b0 :
                  
                      ReadXfer & ~RVALID ? 1'b0 :
                   
                      1'b1;

  assign iHRESP = (~HReadyReg & HRespReg != `AHB_RESP_OKAY ? HRespReg :
                   
                   ReadTerminate ? `AHB_RESP_OKAY :
                  
                   BVALID & (BRESP[1] | DummyWriteReg) ? `AHB_RESP_ERROR :

                   RVALID & iRREADY & RRESP[1] & ~DummyWrite ? `AHB_RESP_ERROR :
                   
                   Exclusive & 
                   ((RVALID & (RRESP == `AXI_RESP_OKAY)) | 
                    (BVALID & (BRESP == `AXI_RESP_OKAY))) ? `AHB_RESP_XFAIL :
                   
                   `AHB_RESP_OKAY);
                        
  always @(negedge RESETn or posedge CLK)
  begin : p_HRespSeq
    if (~RESETn)
    begin
      HRespReg  <= `AHB_RESP_OKAY;
      HReadyReg <= 1'b1;
    end
    else
    begin
      HRespReg  <= iHRESP;
      HReadyReg <= iHREADYOUT;
    end
  end


  assign HREADYOUT = iHREADYOUT;
  assign HRESP     = iHRESP;


  assign AWVALID =  AWRITE & AVALID;
  assign ARVALID = ~AWRITE & AVALID;

  assign AWADDR  = ADDR;
  assign AWLEN   = ALEN;
  assign AWSIZE  = ASIZE;
  assign AWBURST = ABURST;
  assign AWLOCK  = ALOCK;
  assign AWCACHE = ACACHE;
  assign AWPROT  = APROT;
  assign AWUSER = AUSER;
  
  assign ARADDR  = ADDR;
  assign ARLEN   = ALEN;
  assign ARSIZE  = ASIZE;
  assign ARBURST = ABURST;
  assign ARLOCK  = ALOCK;
  assign ARCACHE = ACACHE;
  assign ARPROT  = APROT;
  assign ARUSER = AUSER;

  assign WVALID  = iWVALID;
  assign WSTRB   = iWSTRB;
  assign WLAST   = iWLAST;
  assign RREADY  = iRREADY;
  assign CACTIVE = iCACTIVE;
  
  
  assign CActiveNxt = (HTRANS != `AHB_TRN_IDLE ? 1'b1 :
                       
                       (HTRANS == `AHB_TRN_IDLE) & 
                       iHREADYOUT & ~ReadTerminateNxt ? 1'b0 :
                       
                       iCACTIVE); // else hold state
                       
  always @(negedge RESETn or posedge CLK)
  begin : p_CSysAckSeq
    if (!RESETn)
    begin
      CSYSACK  <= 1'b0;
      iCACTIVE <= 1'b0;
    end
    else
    begin
      CSYSACK  <= CSYSREQ;
      iCACTIVE <= CActiveNxt;
    end
  end

`ifdef ASSERT_ON


     assert_proposition #(0, 0, "AUSER_WIDTH should be between 1 and 32")
       ahb_to_axi_auser_width(1'b1, AUSER_WIDTH > 0 && AUSER_WIDTH < 33);



  assert_proposition #(0, 0, "DATA_WIDTH should be 32 or 64")
    ahb_to_axi_data_width
    (
      .reset_n (1'b1), 
      .test_expr (DATA_WIDTH == 32 || DATA_WIDTH == 64)
    );

  

  assert_never #(2, 0,
    "Locked exclusive accesses are not supported by this bridge") 
    ahb_to_axi_locked_excl
    (
      .clk (CLK),
      .reset_n (RESETn),
      .test_expr (HTRANS[1] & HPROT[5] & HMASTLOCK)
    );



  assert_never #(1, 0, "The bridge only supports locked bursts of length 1")
    ahb_to_axi_locked_burst
    (
      .clk (CLK),
      .reset_n (RESETn),
      .test_expr ((HTRANS == `AHB_TRN_SEQ) & HMASTLOCK)
    );



  always @(HSIZE or HADDR)
  begin : p_UnalignedAddrComb
    case (HSIZE)
      `AXI_ASIZE_8  : UnalignedAddr = 1'b0;
      `AXI_ASIZE_16 : UnalignedAddr = HADDR[0];
      `AXI_ASIZE_32 : UnalignedAddr = (HADDR[0] | HADDR[1]);
      `AXI_ASIZE_64 : UnalignedAddr = (HADDR[0] | HADDR[1] | HADDR[2]);
      default       : UnalignedAddr = 1'bx; // illegal switch
    endcase // case(HSIZE)
  end
    
  assert_never #(1, 0, "Unaligned addressing is not supported by this bridge")
    ahb_to_axi_unaligned_address
    (
      .clk (CLK),
      .reset_n (RESETn),
      .test_expr (HTRANS[1] & UnalignedAddr)
    );


`endif // ASSERT_ON

endmodule

`timescale 1ns / 1ps

module AhbLiteMToAxi
  (
   CLK,
   RESETn,

   HMASTLOCK,
   HADDR,
   HTRANS,
   HWRITE,
   HSIZE,
   HBURST,
   HPROT,
   HAUSER,
   HREADY,
   HRESP,


   AWADDR,
   AWLEN,
   AWSIZE,
   AWBURST,
   AWLOCK,
   AWCACHE,
   AWPROT,
   AWUSER,
   AWVALID,
   AWREADY,

   WSTRB,
   WLAST,
   WVALID,
   WREADY,

   BRESP,
   BVALID,
   BREADY,

   ARADDR,
   ARLEN,
   ARSIZE,
   ARBURST,
   ARLOCK,
   ARCACHE,
   ARPROT,
   ARUSER,
   ARVALID,
   ARREADY,

   RRESP,
   RLAST,
   RVALID,
   RREADY,

   CSYSREQ,
   CSYSACK,
   CACTIVE,

   SCANENABLE,
   SCANINCLK,
   SCANOUTCLK
   );

parameter DATA_WIDTH = 64;              // Data bus-width

parameter AUSER_WIDTH = 1;
parameter AUSER_MAX   = AUSER_WIDTH-1;

parameter STRB_WIDTH = (DATA_WIDTH/8);  // Strobe bus-width
parameter STRB_MAX   = (STRB_WIDTH-1);  // Upper bound of Strobe bus

  input                CLK;        // common AHB/AXI Clock
  input                RESETn;     // common AHB/AXI Reset

  input                HMASTLOCK;  // HIGH for locked transfer
  input         [31:0] HADDR;      // address bus
  input  [AUSER_MAX:0] HAUSER;     // user signals
  input          [1:0] HTRANS;     // transfer type
  input                HWRITE;     // transfer direction
  input          [2:0] HSIZE;      // transfer size
  input          [2:0] HBURST;     // burst type
  input          [3:0] HPROT;      // protection control
  output               HREADY;     // transfer done
  output         [1:0] HRESP;      // slave response

  output        [31:0] AWADDR;     // write address
  output [AUSER_MAX:0] AWUSER;     // write user signals
  output         [3:0] AWLEN;      // burst length
  output         [2:0] AWSIZE;     // burst size
  output         [1:0] AWBURST;    // burst type
  output         [1:0] AWLOCK;     // burst lock type
  output         [3:0] AWCACHE;    // caching attributes
  output         [2:0] AWPROT;     // protection attributes
  output               AWVALID;    // write address valid
  input                AWREADY;    // write address ready

  output  [STRB_MAX:0] WSTRB;      // write strobes
  output               WLAST;      // write last
  output               WVALID;     // write valid
  input                WREADY;     // write ready

  input          [1:0] BRESP;      // write response
  input                BVALID;     // response valid
  output               BREADY;     // response ready

  output        [31:0] ARADDR;     // write address
  output [AUSER_MAX:0] ARUSER;     // read user signals
  output         [3:0] ARLEN;      // burst length
  output         [2:0] ARSIZE;     // burst size
  output         [1:0] ARBURST;    // burst type
  output         [1:0] ARLOCK;     // burst lock type
  output         [3:0] ARCACHE;    // caching attributes
  output         [2:0] ARPROT;     // protection attributes
  output               ARVALID;    // write address valid
  input                ARREADY;    // write address ready

  input          [1:0] RRESP;      // read response
  input                RLAST;      // last read data in burst
  input                RVALID;     // read valid
  output               RREADY;     // read ready

  input                CSYSREQ;    // system low-power request
  output               CSYSACK;    // system low-power acknowledge
  output               CACTIVE;    // clock active

  input                SCANENABLE; // enable scan test mode
  input                SCANINCLK;  // scan data input
  output               SCANOUTCLK; // scan data output


  wire                 CLK;        // common AHB/AXI Clock
  wire                 RESETn;     // common AHB/AXI Reset

  wire                 HMASTLOCK;  // high for a locked transfer
  wire          [31:0] HADDR;      // address bus
  wire   [AUSER_MAX:0] HAUSER;     // user signals
  wire           [1:0] HTRANS;     // transfer type
  wire                 HWRITE;     // transfer direction
  wire           [2:0] HSIZE;      // transfer size
  wire           [2:0] HBURST;     // burst type
  wire           [3:0] HPROT;      // protection control
  wire                 HREADY;     // transfer done
  wire           [1:0] HRESP;      // slave response

  wire          [31:0] AWADDR;     // write address
  wire   [AUSER_MAX:0] AWUSER;     // write user signals
  wire           [3:0] AWLEN;      // burst length
  wire           [2:0] AWSIZE;     // burst size
  wire           [1:0] AWBURST;    // burst type
  wire           [1:0] AWLOCK;     // burst lock type
  wire           [3:0] AWCACHE;    // caching attributes
  wire           [2:0] AWPROT;     // protection attributes
  wire                 AWVALID;    // write address valid
  wire                 AWREADY;    // write address ready

  wire    [STRB_MAX:0] WSTRB;      // write strobes
  wire                 WLAST;      // write last
  wire                 WVALID;     // write valid
  wire                 WREADY;     // write ready

  wire           [1:0] BRESP;      // write response
  wire                 BVALID;     // response valid
  wire                 BREADY;     // response ready

  wire          [31:0] ARADDR;     // write address
  wire   [AUSER_MAX:0] ARUSER;     // read user signals
  wire           [3:0] ARLEN;      // burst length
  wire           [2:0] ARSIZE;     // burst size
  wire           [1:0] ARBURST;    // burst type
  wire           [1:0] ARLOCK;     // burst lock type
  wire           [3:0] ARCACHE;    // caching attributes
  wire           [2:0] ARPROT;     // protection attributes
  wire                 ARVALID;    // write address valid
  wire                 ARREADY;    // write address ready

  wire          [1:0]  RRESP;      // read response
  wire                 RLAST;      // last read data in burst
  wire                 RVALID;     // read valid
  wire                 RREADY;     // read ready

  wire                 CSYSREQ;    // system low-power request
  wire                 CSYSACK;    // system low-power acknowledge
  wire                 CACTIVE;    // block active




  wire   [STRB_MAX:0] HBSTRB;      // byte lane strobes for reads and writes

  wire          [2:0] HrespAll;    // ARM11-AHB response signal
  wire          [5:0] HprotPadded; // ARM11-AHB HPROT signal


  AhbToAxiStrbGen
    #(DATA_WIDTH)
  uStrbGen
    (
      .HADDR       (HADDR[2:0]),
      .HSIZE       (HSIZE),

      .HBSTRB      (HBSTRB)
    );


  A11AhbLiteMToAxi
    #(DATA_WIDTH, AUSER_WIDTH)
  uA11AhbLiteMToAxi
    (
     .CLK         (CLK),
     .RESETn      (RESETn),

     .HADDR       (HADDR),
     .HAUSER      (HAUSER),
     .HTRANS      (HTRANS),
     .HWRITE      (HWRITE),

     .HSIZE       (HSIZE),
     .HBURST      (HBURST),
     .HPROT       (HprotPadded),        // Bits 5:4 = '00', bits 3:0 = HPROT
     .HMASTLOCK   (HMASTLOCK),
     .HBSTRB      (HBSTRB),

     .HREADYOUT   (HREADY),
     .HRESP       (HrespAll),           // Bits 2..1 are connected but not used

     .AWADDR      (AWADDR),
     .AWUSER      (AWUSER),
     .AWLEN       (AWLEN),
     .AWSIZE      (AWSIZE),
     .AWBURST     (AWBURST),
     .AWLOCK      (AWLOCK),
     .AWCACHE     (AWCACHE),
     .AWPROT      (AWPROT),
     .AWVALID     (AWVALID),

     .AWREADY     (AWREADY),

     .WSTRB       (WSTRB),
     .WLAST       (WLAST),
     .WVALID      (WVALID),

     .WREADY      (WREADY),

     .BRESP       (BRESP),
     .BVALID      (BVALID),

     .BREADY      (BREADY),

     .ARADDR      (ARADDR),
     .ARUSER      (ARUSER),
     .ARLEN       (ARLEN),
     .ARSIZE      (ARSIZE),
     .ARBURST     (ARBURST),
     .ARLOCK      (ARLOCK),
     .ARCACHE     (ARCACHE),
     .ARPROT      (ARPROT),
     .ARVALID     (ARVALID),

     .ARREADY     (ARREADY),

     .RRESP       (RRESP),
     .RLAST       (RLAST),
     .RVALID      (RVALID),

     .RREADY      (RREADY),

     .CSYSREQ     (CSYSREQ),

     .CSYSACK     (CSYSACK),
     .CACTIVE     (CACTIVE),

     .SCANENABLE  (SCANENABLE),
     .SCANINCLK   (SCANINCLK),

     .SCANOUTCLK  ()
     );

  assign HprotPadded = { 2'b00, HPROT };



  assign HRESP = { 1'b0, HrespAll[0] };

endmodule



`timescale 1ns / 1ps

module AhbMToAxi 
  (
   CLK,
   RESETn,
   
   HLOCK,
   HADDR,
   HTRANS,
   HWRITE,
   HSIZE,
   HBURST,
   HPROT,
   HAUSER,
   HGRANT,
   HREADY,
   HRESP,

   
   AWADDR, 
   AWLEN, 
   AWSIZE, 
   AWBURST, 
   AWLOCK, 
   AWCACHE, 
   AWPROT, 
   AWUSER,
   AWVALID, 
   AWREADY, 
   
   WSTRB, 
   WLAST, 
   WVALID, 
   WREADY, 
   
   BRESP, 
   BVALID, 
   BREADY, 
   
   ARADDR, 
   ARLEN, 
   ARSIZE, 
   ARBURST, 
   ARLOCK, 
   ARCACHE, 
   ARPROT, 
   ARUSER,
   ARVALID, 
   ARREADY, 
   
   RRESP, 
   RLAST,
   RVALID, 
   RREADY, 
   
   CSYSREQ,
   CSYSACK,
   CACTIVE,
  
   SCANENABLE,
   SCANINCLK,
   SCANOUTCLK
   );

parameter DATA_WIDTH = 64;              // Data bus-width

parameter AUSER_WIDTH = 1; 
parameter AUSER_MAX   = AUSER_WIDTH-1;

parameter STRB_WIDTH = (DATA_WIDTH/8);  // Strobe bus-width
parameter STRB_MAX   = (STRB_WIDTH-1);  // Upper bound of Strobe bus

  input               CLK;        // common AHB/AXI Clock
  input               RESETn;     // common AHB/AXI Reset

  input                HLOCK;      // HIGH for locked transfer
  input         [31:0] HADDR;      // address bus
  input  [AUSER_MAX:0] HAUSER;     // user signals
  input          [1:0] HTRANS;     // transfer type
  input                HWRITE;     // transfer direction
  input          [2:0] HSIZE;      // transfer size
  input          [2:0] HBURST;     // burst type
  input          [3:0] HPROT;      // protection control
  output               HGRANT;     // master granted
  output               HREADY;     // transfer done
  output         [1:0] HRESP;      // slave response

  output        [31:0] AWADDR;     // write address
  output [AUSER_MAX:0] AWUSER;     // write user signals 
  output         [3:0] AWLEN;      // burst length
  output         [2:0] AWSIZE;     // burst size
  output         [1:0] AWBURST;    // burst type
  output         [1:0] AWLOCK;     // burst lock type
  output         [3:0] AWCACHE;    // caching attributes
  output         [2:0] AWPROT;     // protection attributes
  output               AWVALID;    // write address valid
  input                AWREADY;    // write address ready

  output [STRB_MAX:0] WSTRB;      // write strobes
  output              WLAST;      // write last
  output              WVALID;     // write valid
  input               WREADY;     // write ready
  
  input         [1:0] BRESP;      // write response
  input               BVALID;     // response valid
  output              BREADY;     // response ready

  output        [31:0] ARADDR;     // write address
  output [AUSER_MAX:0] ARUSER;     // read user signals 
  output         [3:0] ARLEN;      // burst length
  output         [2:0] ARSIZE;     // burst size
  output         [1:0] ARBURST;    // burst type
  output         [1:0] ARLOCK;     // burst lock type
  output         [3:0] ARCACHE;    // caching attributes
  output         [2:0] ARPROT;     // protection attributes
  output               ARVALID;    // write address valid
  input                ARREADY;    // write address ready

  input         [1:0] RRESP;      // read response
  input               RLAST;      // last read data in burst
  input               RVALID;     // read valid
  output              RREADY;     // read ready

  input               CSYSREQ;    // system low-power request
  output              CSYSACK;    // system low-power acknowledge
  output              CACTIVE;    // clock active

  input               SCANENABLE; // enable scan test mode
  input               SCANINCLK;  // scan data input
  output              SCANOUTCLK; // scan data output
  
  
  wire                CLK;        // common AHB/AXI Clock
  wire                RESETn;     // common AHB/AXI Reset

  wire                HLOCK;      // high for a locked transfer, arbiter class
  wire         [31:0] HADDR;      // address bus
  wire          [1:0] HTRANS;     // transfer type
  wire                HWRITE;     // transfer direction
  wire          [2:0] HSIZE;      // transfer size
  wire          [2:0] HBURST;     // burst type
  wire          [3:0] HPROT;      // protection control
  wire                HREADY;     // transfer done
  wire          [1:0] HRESP;      // slave response

  wire         [31:0] AWADDR;     // write address
  wire          [3:0] AWLEN;      // burst length
  wire          [2:0] AWSIZE;     // burst size
  wire          [1:0] AWBURST;    // burst type
  wire          [1:0] AWLOCK;     // burst lock type
  wire          [3:0] AWCACHE;    // caching attributes
  wire          [2:0] AWPROT;     // protection attributes
  wire                AWVALID;    // write address valid
  wire                AWREADY;    // write address ready

  wire   [STRB_MAX:0] WSTRB;      // write strobes
  wire                WLAST;      // write last
  wire                WVALID;     // write valid
  wire                WREADY;     // write ready

  wire          [1:0] BRESP;      // write response
  wire                BVALID;     // response valid
  wire                BREADY;     // response ready

  wire         [31:0] ARADDR;     // write address
  wire          [3:0] ARLEN;      // burst length
  wire          [2:0] ARSIZE;     // burst size
  wire          [1:0] ARBURST;    // burst type
  wire          [1:0] ARLOCK;     // burst lock type
  wire          [3:0] ARCACHE;    // caching attributes
  wire          [2:0] ARPROT;     // protection attributes
  wire                ARVALID;    // write address valid
  wire                ARREADY;    // write address ready

  wire         [1:0]  RRESP;      // read response
  wire                RLAST;      // last read data in burst
  wire                RVALID;     // read valid
  wire                RREADY;     // read ready

  wire                CSYSREQ;    // system low-power request    
  wire                CSYSACK;    // system low-power acknowledge
  wire                CACTIVE;    // block active
  



  reg                 HMASTLOCK;   // HIGH for locked transfer, address class
  wire   [STRB_MAX:0] HBSTRB;      // byte lane strobes for reads and writes
  
  wire                iHREADY;     // internal version of HREADY output
  wire          [2:0] HrespAll;    // ARM11-AHB response signal
  wire          [5:0] HprotPadded; // ARM11-AHB HPROT signal


  AhbToAxiStrbGen
    #(DATA_WIDTH)
  uStrbGen
    (
      .HADDR       (HADDR[2:0]),
      .HSIZE       (HSIZE),

      .HBSTRB      (HBSTRB)
    );



  always @ (negedge RESETn or posedge CLK)
  begin : p_HmastlockSeq
    if (~RESETn)
      HMASTLOCK <= 1'b0;
    else
      if (iHREADY)
        HMASTLOCK <= HLOCK;
  end


  A11AhbLiteMToAxi
    #(DATA_WIDTH, AUSER_WIDTH
     )
  uA11AhbLiteMToAxi
    (
     .CLK         (CLK),
     .RESETn      (RESETn),

     .HADDR       (HADDR),
     .HAUSER      (HAUSER),
     .HTRANS      (HTRANS),
     .HWRITE      (HWRITE),
     
     .HSIZE       (HSIZE),
     .HBURST      (HBURST),
     .HPROT       (HprotPadded),           // Bits 6:4 = '000', bits 3:0 = HPROT
     .HMASTLOCK   (HMASTLOCK),
     .HBSTRB      (HBSTRB),
     
     .HREADYOUT   (iHREADY),
     .HRESP       (HrespAll),              // Bit 2 is connected but not used
     
     .AWADDR      (AWADDR),
     .AWUSER      (AWUSER),
     .AWLEN       (AWLEN),
     .AWSIZE      (AWSIZE),
     .AWBURST     (AWBURST),
     .AWLOCK      (AWLOCK),
     .AWCACHE     (AWCACHE),
     .AWPROT      (AWPROT),
     .AWVALID     (AWVALID),
     
     .AWREADY     (AWREADY),
     
     .WSTRB       (WSTRB),
     .WLAST       (WLAST),
     .WVALID      (WVALID),
     
     .WREADY      (WREADY),
     
     .BRESP       (BRESP),
     .BVALID      (BVALID),
     
     .BREADY      (BREADY),

     .ARADDR      (ARADDR),
     .ARUSER      (ARUSER),
     .ARLEN       (ARLEN),
     .ARSIZE      (ARSIZE),
     .ARBURST     (ARBURST),
     .ARLOCK      (ARLOCK),
     .ARCACHE     (ARCACHE),
     .ARPROT      (ARPROT),
     .ARVALID     (ARVALID),
     
     .ARREADY     (ARREADY),
     
     .RRESP       (RRESP),
     .RLAST       (RLAST),
     .RVALID      (RVALID),
     
     .RREADY      (RREADY),
     
     .CSYSREQ     (CSYSREQ),
     
     .CSYSACK     (CSYSACK),
     .CACTIVE     (CACTIVE),
    
      
     .SCANENABLE  (SCANENABLE),
     .SCANINCLK   (SCANINCLK),
     
     .SCANOUTCLK  ()
     );

  assign HprotPadded = { 2'b00, HPROT };



  assign HREADY = iHREADY;

  assign HRESP = HrespAll[1:0];

  assign HGRANT = 1'b1;

endmodule



`timescale 1ns / 1ps

module AhbSToAxi 
  (
   CLK,
   RESETn,
   
   HREADY,
   HSEL,
   HADDR,
   HTRANS,
   HWRITE,
   HSIZE,
   HBURST,
   HPROT,
   HMASTLOCK,
   HAUSER,
   HREADYOUT,
   HRESP,

   
   AWADDR, 
   AWLEN, 
   AWSIZE, 
   AWBURST, 
   AWLOCK, 
   AWCACHE, 
   AWPROT, 
   AWUSER,
   AWVALID, 
   AWREADY, 
   
   WSTRB, 
   WLAST, 
   WVALID, 
   WREADY, 
   
   BRESP, 
   BVALID, 
   BREADY, 
   
   ARADDR, 
   ARLEN, 
   ARSIZE, 
   ARBURST, 
   ARLOCK, 
   ARCACHE, 
   ARPROT, 
   ARUSER,
   ARVALID, 
   ARREADY, 
   
   RRESP, 
   RLAST,
   RVALID, 
   RREADY, 
   
   CSYSREQ,
   CSYSACK,
   CACTIVE,
  
   SCANENABLE,
   SCANINCLK,
   SCANOUTCLK
   );

parameter DATA_WIDTH = 64;              // Data bus-width
parameter INCR_OVERRIDE = 0;            // enable for INCR override block

parameter AUSER_WIDTH = 1; 
parameter AUSER_MAX   = AUSER_WIDTH-1;

parameter STRB_WIDTH = (DATA_WIDTH/8);  // Strobe bus-width
parameter STRB_MAX   = (STRB_WIDTH-1);  // Upper bound of Strobe bus

  input               CLK;        // common AHB/AXI Clock
  input               RESETn;     // common AHB/AXI Reset

  input               HSEL;       // slave select               
  input        [31:0] HADDR;      // address bus
  input         [1:0] HTRANS;     // transfer type
  input               HWRITE;     // transfer direction
  input         [2:0] HSIZE;      // transfer size
  input         [2:0] HBURST;     // burst type
  input         [3:0] HPROT;      // protection control
  input               HMASTLOCK;  // HIGH for locked transfer
  input [AUSER_MAX:0] HAUSER;     // auser signals
  input               HREADY;     // HREADY input from slave mux
  output              HREADYOUT;  // transfer done
  output        [1:0] HRESP;      // slave response

  output        [31:0] AWADDR;     // write address
  output         [3:0] AWLEN;      // burst length
  output         [2:0] AWSIZE;     // burst size
  output         [1:0] AWBURST;    // burst type
  output         [1:0] AWLOCK;     // burst lock type
  output         [3:0] AWCACHE;    // caching attributes
  output         [2:0] AWPROT;     // protection attributes
  output [AUSER_MAX:0] AWUSER;     // write user signals 
  output               AWVALID;    // write address valid
  input                AWREADY;    // write address ready

  output [STRB_MAX:0] WSTRB;      // write strobes
  output              WLAST;      // write last
  output              WVALID;     // write valid
  input               WREADY;     // write ready
  
  input         [1:0] BRESP;      // write response
  input               BVALID;     // response valid
  output              BREADY;     // response ready

  output        [31:0] ARADDR;     // write address
  output         [3:0] ARLEN;      // burst length
  output         [2:0] ARSIZE;     // burst size
  output         [1:0] ARBURST;    // burst type
  output         [1:0] ARLOCK;     // burst lock type
  output         [3:0] ARCACHE;    // caching attributes
  output         [2:0] ARPROT;     // protection attributes
  output [AUSER_MAX:0] ARUSER;     // read user signals 
  output               ARVALID;    // write address valid
  input                ARREADY;    // write address ready

  input         [1:0] RRESP;      // read response
  input               RLAST;      // last read data in burst
  input               RVALID;     // read valid
  output              RREADY;     // read ready

  input               CSYSREQ;    // system low-power request
  output              CSYSACK;    // system low-power acknowledge
  output              CACTIVE;    // clock active

  input               SCANENABLE; // enable scan test mode
  input               SCANINCLK;  // scan data input
  output              SCANOUTCLK; // scan data output
  
  
  wire                CLK;        // common AHB/AXI Clock
  wire                RESETn;     // common AHB/AXI Reset

  wire         [31:0] HADDR;      // address bus
  wire          [1:0] HTRANS;     // transfer type
  wire                HWRITE;     // transfer direction
  wire          [2:0] HSIZE;      // transfer size
  wire          [2:0] HBURST;     // burst type
  wire          [3:0] HPROT;      // protection control
  wire                HMASTLOCK;  // HIGH for locked transfer
  wire   [STRB_MAX:0] HBSTRB;     // byte lane strobes for reads and writes
  wire                HREADYOUT;  // transfer done
  wire          [1:0] HRESP;      // slave response

  wire         [31:0] AWADDR;     // write address
  wire          [3:0] AWLEN;      // burst length
  wire          [2:0] AWSIZE;     // burst size
  wire          [1:0] AWBURST;    // burst type
  wire          [1:0] AWLOCK;     // burst lock type
  wire          [3:0] AWCACHE;    // caching attributes
  wire          [2:0] AWPROT;     // protection attributes
  wire                AWVALID;    // write address valid
  wire                AWREADY;    // write address ready

  wire   [STRB_MAX:0] WSTRB;      // write strobes
  wire                WLAST;      // write last
  wire                WVALID;     // write valid
  wire                WREADY;     // write ready

  wire          [1:0] BRESP;      // write response
  wire                BVALID;     // response valid
  wire                BREADY;     // response ready

  wire         [31:0] ARADDR;     // write address
  wire          [3:0] ARLEN;      // burst length
  wire          [2:0] ARSIZE;     // burst size
  wire          [1:0] ARBURST;    // burst type
  wire          [1:0] ARLOCK;     // burst lock type
  wire          [3:0] ARCACHE;    // caching attributes
  wire          [2:0] ARPROT;     // protection attributes
  wire                ARVALID;    // write address valid
  wire                ARREADY;    // write address ready

  wire         [1:0]  RRESP;      // read response
  wire                RLAST;      // last read data in burst
  wire                RVALID;     // read valid
  wire                RREADY;     // read ready

  wire                CSYSREQ;    // system low-power request    
  wire                CSYSACK;    // system low-power acknowledge
  wire                CACTIVE;    // block active
  


  wire        [1:0] HtransSquelched;    // mod'd HTRANS based on HSEL & HREADY
  wire        [1:0] HtransIncrOvrrde;   // mod'd HTRANS from burst conversion
  wire        [1:0] HtransA11Wrap;      // mux'd HTRANS based on parameter
  wire        [2:0] HburstIncrOvrrde;   // mod'd HBURST from burst conversion
  wire        [2:0] HburstA11Wrap;      // mux'd HBURST based on parameter
  wire        [2:0] HrespAll;           // A11 response signal
  wire        [5:0] HprotPadded;        // A11 HPROT signal
  wire              iHREADYOUT;         // internal version of HREADYOUT


  AhbToAxiHtransSquelch uHtransSquelch
    (
      .HCLK        (CLK),
      .HRESETn     (RESETn),

      .HSELS       (HSEL),
      .HREADYS     (HREADY),
      .HTRANSS     (HTRANS),

      .SCANENABLE  (SCANENABLE),
      .SCANINHCLK  (SCANINCLK),

      .HTRANSM     (HtransSquelched),

      .SCANOUTHCLK ()
    );

  AhbToAxiIncrOverride uIncrOverride
    (
      .HTRANSS     (HtransSquelched),
      .HBURSTS     (HBURST),

      .HTRANSM     (HtransIncrOvrrde),
      .HBURSTM     (HburstIncrOvrrde)
    );

  AhbToAxiStrbGen
    #(DATA_WIDTH)
  uStrbGen
    (
      .HADDR       (HADDR[2:0]),
      .HSIZE       (HSIZE),

      .HBSTRB      (HBSTRB)
    );


  A11AhbLiteMToAxi
    #(DATA_WIDTH, AUSER_WIDTH)
  uA11AhbLiteMToAxi
    (
     .CLK         (CLK),
     .RESETn      (RESETn),

     .HADDR       (HADDR),
     .HAUSER      (HAUSER),
     .HTRANS      (HtransA11Wrap),
     .HWRITE      (HWRITE),
     
     .HSIZE       (HSIZE),
     .HBURST      (HburstA11Wrap),
     .HPROT       (HprotPadded),           // Bits 6:4 = '000', bits 3:0 = HPROT
     .HMASTLOCK   (HMASTLOCK),
     .HBSTRB      (HBSTRB),
     
     .HREADYOUT   (iHREADYOUT),
     .HRESP       (HrespAll),              // Bit 2 is connected but not used
     
     .AWADDR      (AWADDR),
     .AWUSER      (AWUSER),
     .AWLEN       (AWLEN),
     .AWSIZE      (AWSIZE),
     .AWBURST     (AWBURST),
     .AWLOCK      (AWLOCK),
     .AWCACHE     (AWCACHE),
     .AWPROT      (AWPROT),
     .AWVALID     (AWVALID),
     
     .AWREADY     (AWREADY),
     
     .WSTRB       (WSTRB),
     .WLAST       (WLAST),
     .WVALID      (WVALID),
     
     .WREADY      (WREADY),
     
     .BRESP       (BRESP),
     .BVALID      (BVALID),
     
     .BREADY      (BREADY),
    
     .ARADDR      (ARADDR),
     .ARUSER      (ARUSER),
     .ARLEN       (ARLEN),
     .ARSIZE      (ARSIZE),
     .ARBURST     (ARBURST),
     .ARLOCK      (ARLOCK),
     .ARCACHE     (ARCACHE),
     .ARPROT      (ARPROT),
     .ARVALID     (ARVALID),
     
     .ARREADY     (ARREADY),
     
     .RRESP       (RRESP),
     .RLAST       (RLAST),
     .RVALID      (RVALID),
     
     .RREADY      (RREADY),
     
     .CSYSREQ     (CSYSREQ),
     
     .CSYSACK     (CSYSACK),
     .CACTIVE     (CACTIVE),
     
     .SCANENABLE  (SCANENABLE),
     .SCANINCLK   (SCANINCLK),
     
     .SCANOUTCLK  ()
     );

  assign HprotPadded = { 2'b00, HPROT };


  assign HREADYOUT = iHREADYOUT;

  assign HRESP = HrespAll[1:0];

  assign HtransA11Wrap = (INCR_OVERRIDE == 0) ? HtransSquelched : // bypass
                                                HtransIncrOvrrde; // override

  assign HburstA11Wrap = (INCR_OVERRIDE == 0) ? HBURST :          // bypass
                                                HburstIncrOvrrde; // override

endmodule



`timescale 1ns / 1ps

module AhbToAxiHtransSquelch 
  (
   HCLK, 
   HRESETn, 
   HSELS, 
   HREADYS, 
   HTRANSS, 
   SCANENABLE,
   SCANINHCLK,

   HTRANSM, 
   SCANOUTHCLK
   );

  input         HCLK;          // Clock input
  input         HRESETn;       // Reset async input active low
  input         HSELS;         // Select input
  input         HREADYS;       // Transfer valid input
  input   [1:0] HTRANSS;       // Transfer type input
  input         SCANENABLE;    // Scan Test Mode Enable
  input         SCANINHCLK;    // Scan Chain Input (HCLK domain)

  output  [1:0] HTRANSM;       // Transfer type output
  output        SCANOUTHCLK;   // Scan Chain Output (HCLK domain)




`define AHB_TRN_IDLE   2'b00   // Idle transfer
`define AHB_TRN_BUSY   2'b01   // Busy transfer
`define AHB_TRN_NONSEQ 2'b10   // Non-sequential
`define AHB_TRN_SEQ    2'b11   // Sequential



  wire        HCLK;            // Clock input                     
  wire        HRESETn;         // Reset async input active low
  wire        HSELS;           // Select input
  wire        HREADYS;         // Transfer valid input
  wire  [1:0] HTRANSS;         // Transfer type input
  wire        SCANENABLE;      // Scan Test Mode Enable
  wire        SCANINHCLK;      // Scan Chain Input (HCLK domain)
  
  wire  [1:0] HTRANSM;         // Transfer type output
  wire        SCANOUTHCLK;     // Scan Chain Output (HCLK domain)
                               
  wire        Valid;           // Indicates a transfer directed at the bridge
  reg         ValidReg;        // Registered version of Valid



  assign Valid = (HSELS & (HREADYS | ValidReg));
  
  always @ (negedge HRESETn or posedge HCLK)
    begin : p_ValidReg
      if (!HRESETn)
        ValidReg <= 1'b0;
      else
        ValidReg <= Valid;
    end
    
  assign HTRANSM = (!Valid) ? `AHB_TRN_IDLE :   // Assert IDLE

                   HTRANSS; 


endmodule



`timescale 1ns/1ps

module AhbToAxiIncrOverride 
  (
   HTRANSS,
   HBURSTS,
   
   HTRANSM,
   HBURSTM
   );

  input [1:0]  HTRANSS;   // Transfer type
  input [2:0]  HBURSTS;   // Burst length
  
  output [1:0]  HTRANSM;  // Transfer type
  output [2:0]  HBURSTM;  // Burst length



`define AHB_BUR_SINGLE 3'b000  // single
`define AHB_BUR_INCR   3'b001  // incrementing

  wire [1:0]    HTRANSS;    // transfer type
  wire [2:0]    HBURSTS;    // burst length
  wire [1:0]    HTRANSM;    // transfer type
  wire [2:0]    HBURSTM;    // burst length



  assign HBURSTM = HBURSTS != `AHB_BUR_SINGLE ? `AHB_BUR_INCR :
                   `AHB_BUR_SINGLE;
  

  assign HTRANSM = {HTRANSS[1],1'b0};


endmodule // AhbToAxiIncrOverride

`timescale 1ns / 1ps

module AhbToAxiStrbGen
  (
    HADDR,
    HSIZE,

    HBSTRB
  );


  parameter DATA_WIDTH = 64;           // data bus width, default = 64-bit

  parameter STRB_WIDTH = DATA_WIDTH/8; // WSTRB width
  parameter STRB_MAX   = STRB_WIDTH-1; // WSTRB max index



input         [2:0] HADDR;              // Address
input         [2:0] HSIZE;              // Data size

output [STRB_MAX:0] HBSTRB;             // Write data strobe


`define AHB_SZ_BYTE  3'b000           //  8 bit transfer
`define AHB_SZ_HALF  3'b001           // 16 bit transfer
`define AHB_SZ_WORD  3'b010           // 32 bit transfer
`define AHB_SZ_DWORD 3'b011           // 64 bit transfer

wire        [2:0] HADDR;                // Address
wire        [2:0] HSIZE;                // Data size
reg  [STRB_MAX:0] HBSTRB;               // Write data strobe



  always @(HSIZE or HADDR)
    begin : p_EncodeHbstrbComb

      if (DATA_WIDTH == 64)     // 64-bit mode
        begin
          case (HSIZE)

            `AHB_SZ_BYTE  :         // Byte
              case (HADDR[2:0])
                3'b000  : HBSTRB = 8'b00000001;
                3'b001  : HBSTRB = 8'b00000010;
                3'b010  : HBSTRB = 8'b00000100;
                3'b011  : HBSTRB = 8'b00001000;
                3'b100  : HBSTRB = 8'b00010000;
                3'b101  : HBSTRB = 8'b00100000;
                3'b110  : HBSTRB = 8'b01000000;
                3'b111  : HBSTRB = 8'b10000000;
                default : HBSTRB = 8'bxxxxxxxx; // illegal switch, propagate X
              endcase // case(HADDR[2:0])

            `AHB_SZ_HALF  :         // Half-word
              case (HADDR[2:1])
                2'b00   : HBSTRB = 8'b00000011;
                2'b01   : HBSTRB = 8'b00001100;
                2'b10   : HBSTRB = 8'b00110000;
                2'b11   : HBSTRB = 8'b11000000;
                default : HBSTRB = 8'bxxxxxxxx; // illegal switch, propagate X
              endcase // case(HADDR[2:1])

            `AHB_SZ_WORD  :         // Word
              case (HADDR[2])
                1'b0    : HBSTRB = 8'b00001111;
                1'b1    : HBSTRB = 8'b11110000;
                default : HBSTRB = 8'bxxxxxxxx; // illegal switch, propagate X
              endcase // case(HADDR[2])

            `AHB_SZ_DWORD :         // Dword
                HBSTRB = 8'b11111111;

            default   :
                HBSTRB = 8'bxxxxxxxx; // illegal switch, propagate X

          endcase // case(HSIZE)

      end // if (DATA_WIDTH == 64)

      else                      // 32-bit mode (validated by OVL assertion)
        begin

          case (HSIZE)

            `AHB_SZ_BYTE  :         // Byte
              case (HADDR[1:0])
                2'b00   : HBSTRB = 4'b0001;
                2'b01   : HBSTRB = 4'b0010;
                2'b10   : HBSTRB = 4'b0100;
                2'b11   : HBSTRB = 4'b1000;
                default : HBSTRB = 4'bxxxx; // illegal switch, propagate X
              endcase

            `AHB_SZ_HALF  :         // Half-word
              case (HADDR[1])
                1'b0    : HBSTRB = 4'b0011;
                1'b1    : HBSTRB = 4'b1100;
                default : HBSTRB = 4'bxxxx; // illegal switch, propagate X
              endcase

            `AHB_SZ_WORD  :         // Word
                HBSTRB = 4'b1111;

            default   :
                HBSTRB = 4'bxxxx; // illegal switch, propagate X

          endcase // case(HSIZE)
        end // else: !if(DATA_WIDTH == 64)
    end // block


`ifdef ASSERT_ON


  assert_proposition #(0, 0, "DATA_WIDTH should be 32 or 64")
    uA11AhbLiteToAxiDataWidth
    (
      .reset_n (1'b1),
      .test_expr (DATA_WIDTH == 32 || DATA_WIDTH == 64)
    );


`endif

endmodule