`define ARM_CM3_ETM_LICENSE

module CORTEXM3INTEGRATION
  (
`ifdef ARM_TestBench
   TBENCHINPUTS, TBENCHOUTPUTS,
`endif
   /*AUTOARG*/
   ISOLATEn, RETAINn, nTRST, SWCLKTCK, SWDITMS, TDI, PORESETn, SYSRESETn,
   RSTBYPASS, CGBYPASS, FCLK, HCLK, TRACECLKIN, STCLK, STCALIB, AUXFAULT, BIGEND,
   INTISR, INTNMI, HREADYI, HRDATAI, HRESPI, IFLUSH, HREADYD, HRDATAD, HRESPD,
   EXRESPD, SE, HREADYS, HRDATAS, HRESPS, EXRESPS, EDBGRQ, DBGRESTART, RXEV,
   SLEEPHOLDREQn, WICENREQ, FIXMASTERTYPE, TSVALUEB, TSCLKCHANGE, MPUDISABLE,
   DBGEN, CDBGPWRUPACK,
   TDO, nTDOEN, SWDOEN, SWDO, SWV, JTAGNSW, TRACECLK, TRACEDATA, TRCENA, HTRANSI,
   HSIZEI, HADDRI, HBURSTI, HPROTI, MEMATTRI, HTRANSD, HSIZED, HADDRD, HBURSTD,
   HPROTD, MEMATTRD, HMASTERD, EXREQD, HWRITED, HWDATAD, HTRANSS, HSIZES,
   HADDRS, HBURSTS, HPROTS, MEMATTRS, HMASTERS, EXREQS, HWRITES, HWDATAS,
   HMASTLOCKS, BRCHSTAT, HALTED, LOCKUP, SLEEPING, SLEEPDEEP, ETMINTNUM,
   ETMINTSTAT, SYSRESETREQ, TXEV, CURRPRI, DBGRESTARTED, SLEEPHOLDACKn,
   GATEHCLK, INTERNALSTATE, HTMDHADDR, HTMDHTRANS, HTMDHSIZE, HTMDHBURST,
   HTMDHPROT, HTMDHWDATA, HTMDHWRITE, HTMDHRDATA, HTMDHREADY, HTMDHRESP,
   WICENACK, WAKEUP, CDBGPWRUPREQ
  );


  parameter MPU_PRESENT     = 1;
  parameter NUM_IRQ         = 16;
  parameter LVL_WIDTH       = 3;
  parameter TRACE_LVL       = 1;
  parameter DEBUG_LVL       = 3;
  parameter JTAG_PRESENT    = 1;
  parameter CLKGATE_PRESENT = 0;
  parameter RESET_ALL_REGS  = 0;
  parameter OBSERVATION     = 0;
  parameter WIC_PRESENT     = 0;
  parameter WIC_LINES       = 3;
  parameter BB_PRESENT      = 1;
  parameter CONST_AHB_CTRL  = 0;



  parameter DP_PRESENT = DEBUG_LVL > 0 ? 1 : 0;


`ifdef ARM_TestBench
  input  [128:0] TBENCHINPUTS;       // Testbench inputs for stimulation
  output [141:0] TBENCHOUTPUTS;      // Testbench outputs for observation
`endif

  input          ISOLATEn;           // Isolate core power domain
  input          RETAINn;            // Retain core state during power-down

  input          nTRST;              // Test reset
  input          SWDITMS;            // Test Mode Select/SWDIN
  input          SWCLKTCK;           // Test clock / SWCLK
  input          TDI;                // Test Data In
  input          CDBGPWRUPACK;       // Debug power up acknowledge

  input          PORESETn;           // PowerOn reset
  input          SYSRESETn;          // System reset
  input          RSTBYPASS;          // Reset Bypass
  input          CGBYPASS;           // Architectural clock gate bypass
  input          FCLK;               // Free running clock
  input          HCLK;               // System clock
  input          TRACECLKIN;         // TPIU trace port clock
  input          STCLK;              // System Tick clock
  input   [25:0] STCALIB;            // System Tick calibration
  input   [31:0] AUXFAULT;           // Auxillary FSR pulse inputs
  input          BIGEND;             // Static endianess select

  input  [239:0] INTISR;             // Interrupts
  input          INTNMI;             // Non-maskable Interrupt

  input          HREADYI;            // ICode-bus ready
  input   [31:0] HRDATAI;            // ICode-bus read data
  input    [1:0] HRESPI;             // ICode-bus transfer response
  input          IFLUSH;             // ICode-bus buffer flush
  input          HREADYD;            // DCode-bus ready
  input   [31:0] HRDATAD;            // DCode-bus read data
  input    [1:0] HRESPD;             // DCode-bus transfer response
  input          EXRESPD;            // DCode-bus exclusive response

  input          HREADYS;            // System-bus ready
  input   [31:0] HRDATAS;            // System-bus read data
  input    [1:0] HRESPS;             // System-bus transfer response
  input          EXRESPS;            // System-bus exclusive response

  input          RXEV;               // Wait for exception input
  input          SLEEPHOLDREQn;      // Hold core in sleep mode

  input          EDBGRQ;             // Debug Request
  input          DBGRESTART;         // External Debug Restart request

  input          FIXMASTERTYPE;      // Override HMASTER for AHB-AP accesses

  input          WICENREQ;           // WIC mode Request from PMU

  input [47:0]   TSVALUEB;           // Binary coded timestamp value
  input          TSCLKCHANGE;        // Timestamp clock ratio change

  input          SE;                 // Scan Enable

  input          MPUDISABLE;         // Disable the MPU (act as default)
  input          DBGEN;              // Enable debug

  output         TDO;                // Test Data Out
  output         nTDOEN;             // Test Data Out Enable
  output         CDBGPWRUPREQ;       // Debug power up request

  output         SWDO;               // SingleWire data out
  output         SWDOEN;             // SingleWire output enable
  output         JTAGNSW;            // JTAG mode(1) or SW mode(0)

  output         SWV;                // SingleWire Viewer Data

  output         TRACECLK;           // TracePort clock reference
  output   [3:0] TRACEDATA;          // TracePort Data
  output         TRCENA;             // Trace Enable

  output  [31:0] HTMDHADDR;          // HTM data HADDR
  output   [1:0] HTMDHTRANS;         // HTM data HTRANS
  output   [2:0] HTMDHSIZE;          // HTM data HSIZE
  output   [2:0] HTMDHBURST;         // HTM data HBURST
  output   [3:0] HTMDHPROT;          // HTM data HPROT
  output  [31:0] HTMDHWDATA;         // HTM data HWDATA
  output         HTMDHWRITE;         // HTM data HWRITE
  output  [31:0] HTMDHRDATA;         // HTM data HRDATA
  output         HTMDHREADY;         // HTM data HREADY
  output   [1:0] HTMDHRESP;          // HTM data HRESP

  output   [1:0] HTRANSI;            // ICode-bus transfer type
  output   [2:0] HSIZEI;             // ICode-bus transfer size
  output  [31:0] HADDRI;             // ICode-bus address
  output   [2:0] HBURSTI;            // ICode-bus burst length
  output   [3:0] HPROTI;             // ICode-bus protection
  output   [1:0] MEMATTRI;           // ICode-bus memory attributes
  output   [1:0] HMASTERD;           // DCode-bus master
  output   [1:0] HTRANSD;            // DCode-bus transfer type
  output   [2:0] HSIZED;             // DCode-bus transfer size
  output  [31:0] HADDRD;             // DCode-bus address
  output   [2:0] HBURSTD;            // DCode-bus burst length
  output   [3:0] HPROTD;             // DCode-bus protection
  output   [1:0] MEMATTRD;           // ICode-bus memory attributes
  output         EXREQD;             // ICode-bus exclusive request
  output         HWRITED;            // DCode-bus write not read
  output  [31:0] HWDATAD;            // DCode-bus write data

  output   [1:0] HMASTERS;           // System-bus master
  output   [1:0] HTRANSS;            // System-bus transfer type
  output         HWRITES;            // System-bus write not read
  output   [2:0] HSIZES;             // System-bus transfer size
  output         HMASTLOCKS;         // System-bus lock
  output  [31:0] HADDRS;             // System-bus address
  output  [31:0] HWDATAS;            // System-bus write data
  output   [2:0] HBURSTS;            // System-bus burst length
  output   [3:0] HPROTS;             // System-bus protection
  output   [1:0] MEMATTRS;           // System-bus memory attributes
  output         EXREQS;             // System-bus exclusive request

  output   [3:0] BRCHSTAT;           // Branch status
  output         HALTED;             // Core is halted via debug
  output         DBGRESTARTED;       // External Debug Restart Ready
  output         LOCKUP;             // Lockup indication
  output         SLEEPING;           // Core is sleeping
  output         SLEEPDEEP;          // System can enter deep sleep
  output         SLEEPHOLDACKn;      // Indicate core is force in sleep mode
  output   [8:0] ETMINTNUM;          // Interrupt that is currently active
  output   [2:0] ETMINTSTAT;         // Interrupt activation status
  output   [7:0] CURRPRI;            // Current Int Priority

  output         SYSRESETREQ;        // System reset request

  output         TXEV;               // Event output

  output         GATEHCLK;           // when high, HCLK can be turned off

  output [148:0] INTERNALSTATE;      // Observation of internal state

  output         WICENACK;           // WIC mode acknowledge from WIC
  output         WAKEUP;             // Wake-up request from WIC


  wire           ISOLATEn;           // Isolate core power domain
  wire           RETAINn;            // Retain core state during power-down

  wire           nTDOEN;             // Test Data Out Enable
  wire           SWDO;               // SingleWire data out
  wire           SWDOEN;             // SingleWire output enable
  wire           SWV;                // SingleWire Viewer Data

  wire           PORESETn;           // PowerOn reset
  wire           SYSRESETn;          // System reset
  wire           RSTBYPASS;          // Reset Bypass
  wire           CGBYPASS;           // Architectural clock gate bypass
  wire           FCLK;               // Free running clock
  wire           TRACECLKIN;         // TPIU trace port clock
  wire           STCLK;              // System Tick clock
  wire    [25:0] STCALIB;            // System Tick calibration
  wire           BIGEND;             // Static endianess select
  wire     [3:0] BRCHSTAT;           // Branch status
  wire           HALTED;             // Core is halted via debug
  wire     [7:0] CURRPRI;            // Current Int Priority
  wire           tpiu_en;            // Control for TPIU clock gating
  wire     [9:0] vect_addr;          // Reserved
  wire           vect_addr_en;       // Reserved
  wire           stk_align_init;     // Reserved

  wire     [1:0] HTRANSI;            // ICode-bus transfer type
  wire     [2:0] HSIZEI;             // ICode-bus transfer size
  wire    [31:0] HADDRI;             // ICode-bus address
  wire     [2:0] HBURSTI;            // ICode-bus burst length
  wire     [3:0] HPROTI;             // ICode-bus protection
  wire     [1:0] MEMATTRI;           // ICode-bus memory attributes
  wire           HREADYI;            // ICode-bus ready
  wire    [31:0] HRDATAI;            // ICode-bus read data
  wire     [1:0] HRESPI;             // ICode-bus transfer response
  wire           IFLUSH;             // ICode-bus buffer flush
  wire           d_not_i_trans;      // I/DCode arbitration control

  wire     [1:0] HMASTERD;           // DCode-bus master
  wire     [1:0] HTRANSD;            // DCode-bus transfer type
  wire     [2:0] HSIZED;             // DCode-bus transfer size
  wire    [31:0] HADDRD;             // DCode-bus address
  wire     [2:0] HBURSTD;            // DCode-bus burst length
  wire     [3:0] HPROTD;             // DCode-bus protection
  wire     [1:0] MEMATTRD;           // DCode-bus memory attributes
  wire           EXREQD;             // DCode-bus exclusive request
  wire           HWRITED;            // DCode-bus write not read
  wire    [31:0] HWDATAD;            // DCode-bus write data
  wire           HREADYD;            // DCode-bus ready
  wire    [31:0] HRDATAD;            // DCode-bus read data
  wire     [1:0] HRESPD;             // DCode-bus transfer response
  wire           EXRESPD;            // DCode-bus exclusive response

  wire     [1:0] HMASTERS;           // System-bus master
  wire     [1:0] HTRANSS;            // System-bus transfer type
  wire     [2:0] HSIZES;             // System-bus transfer size
  wire    [31:0] HADDRS;             // System-bus address
  wire     [2:0] HBURSTS;            // System-bus burst length
  wire     [3:0] HPROTS;             // System-bus protection
  wire     [1:0] MEMATTRS;           // System-bus memory attributes
  wire           EXREQS;             // System-bus exclusive request
  wire           HWRITES;            // System-bus write not read
  wire    [31:0] HWDATAS;            // System-bus write data
  wire           HREADYS;            // System-bus ready
  wire    [31:0] HRDATAS;            // System-bus read data
  wire     [1:0] HRESPS;             // System-bus transfer response
  wire           EXRESPS;            // System-bus exclusive response

  wire           dbg_en_sync;        // DBGEN synchronised to DAPCLK
  wire           dap_en;             // DAP-Enable
  wire           dap_clk_en;         // DAP-ClockEn (clock divider)
  wire           dap_reset_n;        // DAP-reset
  reg            dap_sel;            // DAP-select
  reg            dap_enable;         // DAP-enable
  reg            dap_write;          // DAP-write
  reg            dap_abort;          // DAP-abort
  reg     [31:0] dap_addr_mux;       // Muxed DAPADDR from various sources
  wire    [31:0] dap_addr;           // DAP-address
  reg     [31:0] dap_wdata;          // DAP-write data
  reg            dap_ready;          // DAP-transfer response
  reg            dap_slverr;         // DAP-slave error
  reg     [31:0] dap_rdata;          // DAP-read data
  wire           dap_sel_core;       // DAP-select core

  wire           dap_ready_core;     // DAP-transfer response
  wire           dap_slverr_core;    // DAP-slave error
  wire    [31:0] dap_rdata_core;     // DAP-read data

  wire           c_sys_power_up;     // System Powerup (SWCLK)
  wire           c_sys_power_up_sync;// Synchronised to FCLK
  wire           c_dbg_power_up_ack_sync; // Synchronised to DAPCLK

  wire           dap_sel_swj;        // DAP-select
  wire           dap_enable_swj;     // DAP-enable
  wire           dap_write_swj;      // DAP-write
  wire           dap_abort_swj;      // DAP-abort
  wire    [31:0] dap_addr_swj;       // DAP-address
  wire    [31:0] dap_wdata_swj;      // DAP-wdata
  wire           CDBGPWRUPREQ;       // Debug Powerup
  wire           c_sys_power_up_req; // System Powerup
  wire           c_sys_power_up_req_sync; // System Powerup sync to FCLK
  wire           CDBGPWRUPACK;       // Debug Powerup acknowledge
  wire           c_sys_power_up_ack; // System Powerup acknowledge

  wire           atb_valid_port1;    // ATB valid signal source 1
  wire    [6:0]  atb_id1;            // ATB ID source 1
  wire    [7:0]  atb_data_port1;     // ATB data source 1
  wire           atb_valid_port2;    // ATB valid signal source 2
  wire    [6:0]  atb_id2;            // ATB ID source 2
  wire    [7:0]  atb_data_port2;     // ATB data source 2
  wire           atb_ready_port1;    // ATB ready signal source 1
  wire           atb_ready_port2;    // ATB ready signal source 2
  wire           t_reset_n;          // TRACECLKIN asychronous reset active low
  wire           TRACECLK;           // Trace Clock
  wire     [3:0] TRACEDATA;          // Trace Data
  wire           trace_swo;          // Trace SingleWire Output
  wire           tpiu_active;        // TPIU is active
  wire           tpiu_baud;          // TPIU baud indicator (in TRACECLK)

  wire           at_valid_itm;       // ATB Valid
  wire           af_ready_itm;       // ATB Flush
  wire     [7:0] at_data_itm;        // ATB Data
  wire           at_ready_itm;       // ATB Ready
  wire     [6:0] at_id_itm;          // ATM ID

  wire           SLEEPING;           // Core is sleeping
  wire           SLEEPDEEP;          // System can enter deep sleep

  wire           SYSRESETREQ;        // System Reset Request

  wire           d_sync;             // DWT generated sync pulse
  wire     [3:0] etm_trigger;        // DWT generated trigger
  wire     [3:0] etm_trig_i_not_d;   // DWT generates trigger on PC match
  wire           atb_atvalid_etm;    // ATB Valid
  wire           atb_afready_etm;    // ATB Flush
  wire     [7:0] atb_atdata_etm;     // ATB Data
  wire           atb_atready_etm;    // ATB Ready
  wire     [6:0] atb_atid_etm;       // ATM ID
  wire           etm_dbg_req;        // Debug request from ETM
  wire           etm_fifo_full;      // ETM FIFO is full
  wire    [31:1] etm_ia;             // Instruction Address
  wire           etm_ivalid;         // Instruction data is valid
  wire           etm_istall;         // Stall IVALID from prev. cycle
  wire           etm_dvalid;         // Data Matches are valid
  wire           etm_fold;           // 2 instructions this cycle
  wire           etm_cancel;         // Previous started instruction did not complete
  wire           etm_ccfail;         // Instruction failed condition code
  wire           etm_branch;         // Instruction is Branch
  wire           etm_indirect_branch;// Instruction is indirect branch
  wire           etm_isb;            // Instruction is ISB
  wire     [2:0] ETMINTSTAT;         // Branch is due to int entry[x] or exit[y]
  wire     [8:0] ETMINTNUM;          // Interrupt Source
  wire           etm_flush;          // Pipe flush hint
  wire           etm_find_br;        // Flush indirect branch
  wire           etm_power_up;       // The ETM is powered-up
  wire           etm_en;             // ETM Traceport Enabled
  wire           etm_trigger_out;    // ETM Trigger Status Signal to TPIU
  wire           niden;              // Non-invasive debug enable
  wire           etm_fifo_full_en;   // System supports FIFOFULL
  wire           combined_dbg_req;   // Combined EDBGRQ and ETM request

  wire           ppb_psel;           // APB select from Cortex-M3
  wire           ppb_penable;        // APB Enable
  wire           ppb_pwrite;         // APB is write
  wire           ppb_paddr31;        // APB Addr bit 31
  wire    [19:2] ppb_paddr;          // APB address
  wire    [31:0] ppb_pwdata;         // APB write data
  wire    [31:0] ppb_prdata;         // APB read data
  wire           ppb_pready;         // APB Ready response
  wire           ppb_pslverr;        // APB Slave Error Response
  wire           ppb_pseltpiu;       // APB Select for TPIU
  wire           ppb_psel_etm;       // APB Select for ETM
  wire           ppb_psel_rom;       // APB Select for ROM Table
  wire    [31:0] ppb_prdata_tpiu;    // APB Read data from TPIU
  wire    [31:0] ppb_prdata_mux_tpiu;// APB Read data from TPIU (in mux)
  wire    [31:0] ppb_prdata_etm;     // APB Read data from ETM
  wire    [31:0] ppb_prdata_mux_etm; // APB Read data from ETM (in mux)
  wire    [31:0] ppb_prdata_rom;     // APB Read data from ROM Table
  wire    [31:0] ppb_prdata_mux_rom; // APB Read data from ROM Table (in mux)
  wire     [5:0] ppb_lock;           // APB Lock flag

  reg            poreset_n_q;        // Power-On Reset out of first register
  reg            poreset_n_qq;       // Power-On Reset out of second register
  reg            refreset_n_q;       // Power-On Reset sync'd to TRACECLKIN out of first register
  reg            refreset_n_qq;      // Power-On Reset  sync'd to TRACECLKINout of second register
  reg            sysreset_n_q;       // System Reset out of first register
  reg            sysreset_n_qq;      // System Reset out of second register
  reg            po_trst_n_q;        // Power-On Test Reset out of first register
  reg            po_trst_n_qq;       // Power-On Test Reset out of second register

  wire           int_poreset_n;      // Above signal with bypass for test
  wire           int_sysreset_n;     // Above signal with bypass for test
  wire           int_po_trst_n;      // Above signal with bypass for test

  wire           dap_clk_src;        // DAP-clock source
  wire           dap_clk;            // DAP-clock gated
  wire           dclk_g;             // Trace Clock gated
  wire           HCLK;               // FCLK gated if sleeping
  wire           cclk;               // Core clock
  wire           int_gate_hclk;      // Internal GATEHCLK output
  wire           trace_clk_in_g;     // TRACECLKIN gated

  wire    [31:0] int_prdata_rom;
  wire           int_etm_dbg_req;
  wire           int_etm_fifo_full;
  wire           int_etm_power_up;
  wire           int_etm_en;
  wire    [31:0] int_prdata_etm;
  wire     [7:0] int_atb_atdata_etm;
  wire           int_atb_atvalid_etm;
  wire           int_atb_afready_etm;
  wire           int_etm_trigger_out;
  wire     [6:0] int_atb_atid_etm;
  wire    [31:0] int_prdata_tpiu;
  wire           int_trace_clk;
  wire     [3:0] int_trace_data;
  wire           int_trace_swo;
  wire           int_tpiu_active;
  wire           int_tpiu_baud;
  wire           int_atb_ready_port1;
  wire           int_atb_ready_port2;
  wire           int_tdo;
  wire           int_tdoen_n;
  wire           int_dap_sel_swj;
  wire           int_dap_enable_swj;
  wire           int_dap_write_swj;
  wire           int_dap_abort_swj;
  wire    [31:0] int_dap_addr_swj;
  wire    [31:0] int_dap_wdata_swj;
  wire           int_sw_do;
  wire           int_sw_do_en;
  wire           int_c_dbg_power_up_req;
  wire           int_c_sys_power_up_req;
  wire           int_dap_clk;
  wire           int_dclk_g;
  wire           int_cclk;
  wire           int_trace_clk_in;
  wire   [148:0] int_internal_state;

`ifdef ARM_TestBench

  wire           dap_clk_tb;
  wire           dap_clk_en_tb;

  wire    [31:0] ppb_prdata_tb;
  wire           ppb_pready_tb;
  wire           ppb_pslverr_tb;
  wire           ppb_psel_tb;

  wire    [28:0] test_bench_cfg;
`endif

  wire    [31:0] dap_addr_tb;
  wire           dap_sel_tb;
  wire           dap_enable_tb;
  wire           dap_write_tb;
  wire    [31:0] dap_wdata_tb;
  wire           dap_abort_tb;

  wire           dap_en_tb;
  wire     [1:0] etm_ext_in;
  wire     [1:0] etm_max_ext_in;

  wire    [31:0] ppb_prdata_mux_tbox;

  wire     [1:1] config_check_0;  // IRQ number
  wire     [1:1] config_check_1;  // Priority register width
  wire     [1:1] config_check_2;  // Debug level
  wire     [1:1] config_check_3;  // Trace level
  wire     [1:1] config_check_4;  // MPU present
  wire     [1:1] config_check_5;  // JTAG present
  wire     [1:1] config_check_6;  // Clock gating
  wire     [1:1] config_check_7;  // Reset all registers
  wire     [1:1] config_check_8;  // Observation
  wire     [1:1] config_check_9;  // Debug level and trace level combination
  wire     [1:1] config_check_10; // WIC lines
  wire     [1:1] config_check_11; // WIC Present
  wire     [1:1] config_check_12; // Bit-banding present
  wire     [1:1] config_check_13; // Constant AHB control

  wire                 mst_wic_en;     // logic removal term
  wire                 int_wake_up;    // Wake-up request from WIC
  wire                 int_wic_en_ack; // WIC mode acknowledge from WIC
  wire                 wic_nmi_pend;   // NMI  pended by WIC
  wire                 wic_rxev_pend;  // RXEV pended by WIC
  wire         [239:0] int_int_isr;    // Incoming IRQs OR'd with WIC pended IRQs
  wire                 int_int_nmi;    // Incoming NMI  OR'd with WIC pended NMI
  wire                 int_rxev;       // Incoming RXEV OR'd with WIC pended RXEV
  wire                 int_ext_dbg_req;// Incoming EDBGRQ OR'd with WIC pended EDBGRQ
  wire                 wic_ds_req_n;   // WIC mode request to core
  wire                 wic_ds_ack_n;   // WIC mode acknowledge from core
  wire                 wic_load;       // Load IRQ/MON/NMI/RXEV sensitivity list
  wire                 wic_clear;      // Clear WIC sensitivity list
  wire         [239:0] wic_mask_isr;   // WIC IRQ sensitivity list
  wire                 wic_mask_mon;   // WIC DBG MON sensitivity
  wire                 wic_mask_nmi;   // WIC NMI sensitivity
  wire                 wic_mask_rxev;  // WIC RXEV sensitivity
  wire [WIC_LINES-1:0] wic_pend;       // IRQ/NMI/RXEV pended by WIC
  wire [WIC_LINES:0]   int_wic_pend;   // IRQ/NMI/RXEV pended by WIC
  wire [WIC_LINES-1:0] wic_int;        // IRQ/NMI/RXEV input to WIC
  wire         [242:0] int_wic_int;    // expanded WICINT
  wire [WIC_LINES-1:0] wic_sense;      // sensitivity list from WIC
  wire [WIC_LINES:0]   int_wic_sense;  // sensitivity list from WIC
  wire         [242:0] wic_mask;       // sensitivity list from core
  wire [WIC_LINES-1:0] int_wic_mask;   // sensitivity list from core
  wire         [241:0] int_wic_irq_pend;
  wire         [239:0] wic_irq_pend;
  wire                 wic_mon_pend;


  always @ (posedge FCLK or negedge PORESETn)
    if (!PORESETn)
      begin
        poreset_n_q  <= 1'b0;
        poreset_n_qq <= 1'b0;
      end
    else
      begin
        poreset_n_q  <= 1'b1;
        poreset_n_qq <= poreset_n_q;
      end

  always @ (posedge FCLK or negedge SYSRESETn)
    if (!SYSRESETn)
      begin
        sysreset_n_q  <= 1'b0;
        sysreset_n_qq <= 1'b0;
      end
    else
      begin
        sysreset_n_q  <= 1'b1;
        sysreset_n_qq <= sysreset_n_q;
      end

  always @ (posedge SWCLKTCK or negedge PORESETn)
    if (!PORESETn)
      begin
        po_trst_n_q  <= 1'b0;
        po_trst_n_qq <= 1'b0;
      end
    else
      begin
        po_trst_n_q  <= 1'b1;
        po_trst_n_qq <= po_trst_n_q;
      end

  assign int_poreset_n  = RSTBYPASS ? PORESETn : poreset_n_qq;
  assign int_sysreset_n = RSTBYPASS ? PORESETn : sysreset_n_qq;
  assign int_po_trst_n  = RSTBYPASS ? PORESETn : po_trst_n_qq;

`ifdef ARM_TestBench

  reg  dap_reset_n_q;
  reg  dap_reset_n_qq;

  always @(posedge dap_clk_src or negedge int_poreset_n)
  begin
    if (!int_poreset_n)
      begin
      dap_reset_n_q  <= 1'b0;
      dap_reset_n_qq <= 1'b0;
      end
    else
      begin
      dap_reset_n_q  <= int_poreset_n;
      dap_reset_n_qq <= dap_reset_n_q;
      end
  end

  assign dap_reset_n  = dap_reset_n_qq;

`else
  assign dap_reset_n  = int_poreset_n;
`endif

  always @ (posedge TRACECLKIN or negedge int_poreset_n)
    if (!int_poreset_n)
      begin
        refreset_n_q  <= 1'b0;
        refreset_n_qq <= 1'b0;
      end
    else
      begin
        refreset_n_q  <= int_poreset_n;
        refreset_n_qq <= refreset_n_q;
      end

  assign t_reset_n = RSTBYPASS ? PORESETn : refreset_n_qq;




`ifdef ARM_TestBench
  assign dap_clk_en  = c_dbg_power_up_ack_sync & dap_clk_en_tb;
  assign dap_clk_src = dap_clk_tb;
`else
  assign dap_clk_en  = c_dbg_power_up_ack_sync;
  assign dap_clk_src = FCLK;
`endif

  assign int_gate_hclk   = ((SLEEPING | ~SLEEPHOLDACKn | ~ISOLATEn) &
                       ~(c_dbg_power_up_ack_sync | c_sys_power_up_sync));
  assign GATEHCLK = int_gate_hclk;

  cm3_sync #(1)
    u_cm3_sync_hclken
      (//Inputs
       .clk                       (FCLK),
       .reset_n                   (int_poreset_n),
       .d_async_i                 (c_sys_power_up),
       .q_o                       (c_sys_power_up_sync)
      );

  cm3_sync #(1)
    u_cm3_sync_dappwrup
      (//Inputs
       .clk                       (dap_clk_src),
       .reset_n                   (dap_reset_n),
       .d_async_i                 (CDBGPWRUPACK),
       .q_o                       (c_dbg_power_up_ack_sync)
      );

  cm3_sync #(1)
    u_cm3_sync_dbgen
      (//Inputs
       .clk                       (dap_clk_src),
       .reset_n                   (dap_reset_n),
       .d_async_i                 (DBGEN),
       .q_o                       (dbg_en_sync)
      );

  assign dap_en = c_dbg_power_up_ack_sync & dbg_en_sync;

  cm3_clk_gate #(CLKGATE_PRESENT)
    u_cm3_clk_gate_dapclk
      (// Inputs
       .clk                       (dap_clk_src),
       .clk_enable_i              (dap_clk_en),
       .global_disable_i          (CGBYPASS),
       .clk_gated_o               (int_dap_clk)
      );

  assign tpiu_en = (TRCENA | etm_en) & ~int_gate_hclk;
  cm3_clk_gate #(CLKGATE_PRESENT)
    u_cm3_clk_gate_dclk
      (// Inputs
       .clk                       (FCLK),
       .clk_enable_i              (tpiu_en),
       .global_disable_i          (CGBYPASS),
       .clk_gated_o               (int_dclk_g)
      );

  cm3_clk_gate #(CLKGATE_PRESENT)
    u_cm3_clk_gate_cclk
      (// Inputs
       .clk                       (FCLK),
       .clk_enable_i              (ISOLATEn),
       .global_disable_i          (CGBYPASS),
       .clk_gated_o               (int_cclk)
      );

  cm3_clk_gate #(CLKGATE_PRESENT)
    u_cm3_clk_gate_traceclkin
      (// Inputs
       .clk                       (TRACECLKIN),
       .clk_enable_i              (ISOLATEn),
       .global_disable_i          (CGBYPASS),
       .clk_gated_o               (int_trace_clk_in)
      );

  assign dap_clk        = (CLKGATE_PRESENT != 0) ? int_dap_clk        : dap_clk_src;
  assign dclk_g         = (CLKGATE_PRESENT != 0) ? int_dclk_g         : HCLK;
  assign cclk           = ((CLKGATE_PRESENT != 0) &
                           (WIC_PRESENT     != 0)) ? int_cclk         : FCLK;
  assign trace_clk_in_g = ((CLKGATE_PRESENT != 0) &
                           (WIC_PRESENT     != 0)) ? int_trace_clk_in : TRACECLKIN;



`ifdef CM3_ARM_CODEMUX
  assign d_not_i_trans = 1'b1;
`else
  assign d_not_i_trans = 1'b0;
`endif



  CORTEXM3 #(MPU_PRESENT, NUM_IRQ, LVL_WIDTH, TRACE_LVL, DEBUG_LVL,
             CLKGATE_PRESENT, RESET_ALL_REGS, OBSERVATION, WIC_PRESENT,
             WIC_LINES, BB_PRESENT, CONST_AHB_CTRL)
    uCORTEXM3
      (/*AUTOINST*/
       .PORESETn            (int_poreset_n),
       .SYSRESETn           (int_sysreset_n),
       .RSTBYPASS           (RSTBYPASS),
       .CGBYPASS            (CGBYPASS),
       .FCLK                (cclk),
       .HCLK                (HCLK),
       .STCLK               (STCLK),
       .STCALIB             (STCALIB),
       .AUXFAULT            (AUXFAULT),
       .BIGEND              (BIGEND),
       .INTNMI              (int_int_nmi),
       .INTISR              (int_int_isr),
       .ETMPWRUP            (etm_power_up),
       .VECTADDR            (vect_addr),
       .VECTADDREN          (vect_addr_en),
       .EDBGRQ              (combined_dbg_req),
       .DBGRESTART          (DBGRESTART),
       .ETMFIFOFULL         (etm_fifo_full),
       .TPIUACTV            (tpiu_active),
       .TPIUBAUD            (tpiu_baud),
       .RXEV                (int_rxev),
       .SLEEPHOLDREQn       (SLEEPHOLDREQn),
       .STKALIGNINIT        (stk_align_init),
       .FIXMASTERTYPE       (FIXMASTERTYPE),
       .TSVALUEB             (TSVALUEB),
       .TSCLKCHANGE         (TSCLKCHANGE),
       .HREADYI             (HREADYI),
       .HRDATAI             (HRDATAI),
       .HRESPI              (HRESPI),
       .IFLUSH              (IFLUSH),
       .HREADYD             (HREADYD),
       .HRDATAD             (HRDATAD),
       .HRESPD              (HRESPD),
       .EXRESPD             (EXRESPD),
       .HREADYS             (HREADYS),
       .HRDATAS             (HRDATAS),
       .HRESPS              (HRESPS),
       .EXRESPS             (EXRESPS),
       .PRDATA              (ppb_prdata),
       .PREADY              (ppb_pready),
       .PSLVERR             (ppb_pslverr),
       .DAPEN               (dap_en),
       .DAPCLKEN            (dap_clk_en),
       .DAPCLK              (dap_clk),
       .DAPRESETn           (dap_reset_n),
       .DAPSEL              (dap_sel_core),
       .DAPENABLE           (dap_enable),
       .DAPWRITE            (dap_write),
       .DAPABORT            (dap_abort),
       .DAPADDR             (dap_addr),
       .DAPWDATA            (dap_wdata),
       .ATREADY             (at_ready_itm),
       .PPBLOCK             (ppb_lock),
       .DNOTITRANS          (d_not_i_trans),
       .WICDSREQn           (wic_ds_req_n),
       .SE                  (SE),
       .MPUDISABLE          (MPUDISABLE),
       .DBGEN               (DBGEN),
       .SYSRESETREQ         (SYSRESETREQ),
       .TXEV                (TXEV),
       .HTRANSI             (HTRANSI),
       .HSIZEI              (HSIZEI),
       .HADDRI              (HADDRI),
       .HBURSTI             (HBURSTI),
       .HPROTI              (HPROTI),
       .MEMATTRI            (MEMATTRI),
       .HMASTERD            (HMASTERD),
       .HTRANSD             (HTRANSD),
       .HSIZED              (HSIZED),
       .HADDRD              (HADDRD),
       .HBURSTD             (HBURSTD),
       .HPROTD              (HPROTD),
       .MEMATTRD            (MEMATTRD),
       .EXREQD              (EXREQD),
       .HWRITED             (HWRITED),
       .HWDATAD             (HWDATAD),
       .HMASTERS            (HMASTERS),
       .HTRANSS             (HTRANSS),
       .HWRITES             (HWRITES),
       .HSIZES              (HSIZES),
       .HMASTLOCKS          (HMASTLOCKS),
       .HADDRS              (HADDRS),
       .HWDATAS             (HWDATAS),
       .HBURSTS             (HBURSTS),
       .HPROTS              (HPROTS),
       .MEMATTRS            (MEMATTRS),
       .EXREQS              (EXREQS),
       .PADDR31             (ppb_paddr31),
       .PADDR               (ppb_paddr),
       .PSEL                (ppb_psel),
       .PENABLE             (ppb_penable),
       .PWRITE              (ppb_pwrite),
       .PWDATA              (ppb_pwdata),
       .DAPREADY            (dap_ready_core),
       .DAPSLVERR           (dap_slverr_core),
       .DAPRDATA            (dap_rdata_core),
       .ATVALID             (at_valid_itm),
       .AFREADY             (af_ready_itm),
       .ATDATA              (at_data_itm),
       .ETMTRIGGER          (etm_trigger),
       .ETMTRIGINOTD        (etm_trig_i_not_d),
       .ETMIVALID           (etm_ivalid),
       .ETMISTALL           (etm_istall),
       .ETMDVALID           (etm_dvalid),
       .ETMFOLD             (etm_fold),
       .ETMCANCEL           (etm_cancel),
       .ETMIA               (etm_ia[31:1]),
       .ETMICCFAIL          (etm_ccfail),
       .ETMIBRANCH          (etm_branch),
       .ETMIINDBR           (etm_indirect_branch),
       .ETMISB              (etm_isb),
       .ETMINTSTAT          (ETMINTSTAT),
       .ETMINTNUM           (ETMINTNUM),
       .ETMFLUSH            (etm_flush),
       .ETMFINDBR           (etm_find_br),
       .DSYNC               (d_sync),
       .HTMDHADDR           (HTMDHADDR) ,
       .HTMDHTRANS          (HTMDHTRANS),
       .HTMDHSIZE           (HTMDHSIZE) ,
       .HTMDHBURST          (HTMDHBURST),
       .HTMDHPROT           (HTMDHPROT) ,
       .HTMDHWDATA          (HTMDHWDATA),
       .HTMDHRDATA          (HTMDHRDATA),
       .HTMDHWRITE          (HTMDHWRITE),
       .HTMDHREADY          (HTMDHREADY),
       .HTMDHRESP           (HTMDHRESP) ,
       .ATIDITM             (at_id_itm),
       .BRCHSTAT            (BRCHSTAT),
       .HALTED              (HALTED),
       .DBGRESTARTED        (DBGRESTARTED),
       .LOCKUP              (LOCKUP),
       .SLEEPING            (SLEEPING),
       .SLEEPDEEP           (SLEEPDEEP),
       .SLEEPHOLDACKn       (SLEEPHOLDACKn),
       .CURRPRI             (CURRPRI),
       .TRCENA              (TRCENA),
       .INTERNALSTATE       (int_internal_state),
       .WICDSACKn           (wic_ds_ack_n),
       .WICLOAD             (wic_load),
       .WICCLEAR            (wic_clear),
       .WICMASKISR          (wic_mask_isr),
       .WICMASKMON          (wic_mask_mon),
       .WICMASKNMI          (wic_mask_nmi),
       .WICMASKRXEV         (wic_mask_rxev)
       );

  assign INTERNALSTATE = (OBSERVATION != 0) ? int_internal_state : {149{1'b0}};


  assign wic_mask  = {wic_mask_isr, wic_mask_mon, wic_mask_nmi, wic_mask_rxev};
  assign int_wic_mask = wic_mask[WIC_LINES-1:0];

  assign int_wic_int  = {INTISR, EDBGRQ, INTNMI, RXEV};
  assign wic_int   = int_wic_int[WIC_LINES-1:0];

  cm3_wic #(WIC_PRESENT, WIC_LINES)
    u_cm3_wic
      (// Inputs
       .FCLK                (FCLK),
       .RESETn              (int_sysreset_n),
       .WICLOAD             (wic_load),
       .WICCLEAR            (wic_clear),
       .WICINT              (wic_int),
       .WICMASK             (int_wic_mask),
       .WICENREQ            (WICENREQ),
       .WICDSACKn           (wic_ds_ack_n),
       .WAKEUP              (int_wake_up),
       .WICSENSE            (wic_sense),
       .WICPEND             (wic_pend),
       .WICDSREQn           (wic_ds_req_n),
       .WICENACK            (int_wic_en_ack)
       );

  assign int_wic_pend    = {1'b0, wic_pend};
  assign int_wic_sense   = {1'b0, wic_sense};

  assign int_wic_irq_pend = {{244-WIC_LINES{1'b0}},int_wic_pend[WIC_LINES:3]};
  assign wic_irq_pend     = mst_wic_en ? int_wic_irq_pend[239:0] : {240{1'b0}};
  assign wic_mon_pend     = mst_wic_en ? int_wic_pend[2]         : 1'b0;
  assign wic_nmi_pend     = mst_wic_en ? int_wic_pend[1]         : 1'b0;
  assign wic_rxev_pend    = mst_wic_en ? int_wic_pend[0]         : 1'b0;

  assign int_int_isr      = INTISR                 | wic_irq_pend;
  assign int_int_nmi      = INTNMI                 | wic_nmi_pend;
  assign int_rxev         = (RXEV & ~wic_sense[0]) | wic_rxev_pend;
  assign int_ext_dbg_req  = EDBGRQ                 | wic_mon_pend;

  assign combined_dbg_req = int_ext_dbg_req | etm_dbg_req;

  assign WAKEUP     = (WIC_PRESENT != 0) ? int_wake_up    : 1'b0;
  assign WICENACK   = (WIC_PRESENT != 0) ? int_wic_en_ack : 1'b0;
  assign mst_wic_en = (WIC_PRESENT != 0) ? 1'b1           : 1'b0;

  DAPSWJDP #(DP_PRESENT, JTAG_PRESENT, RESET_ALL_REGS)
    uDAPSWJDP
      (// Inputs
       .nPOTRST             (int_po_trst_n),
       .nTRST               (nTRST),
       .SWCLKTCK            (SWCLKTCK),
       .SWDITMS             (SWDITMS),
       .TDI                 (TDI),
       .DAPRESETn           (dap_reset_n),
       .DAPCLK              (dap_clk),
       .DAPCLKEN            (dap_clk_en),
       .DAPRDATA            (dap_rdata),
       .DAPREADY            (dap_ready),
       .DAPSLVERR           (dap_slverr),
       .nCDBGPWRDN          (1'b1),
       .CDBGPWRUPACK        (CDBGPWRUPACK),
       .CSYSPWRUPACK        (c_sys_power_up_ack),
       .CDBGRSTACK          (1'b0),
       .TDO                 (int_tdo),
       .nTDOEN              (int_tdoen_n),
       .DAPSEL              (int_dap_sel_swj),
       .DAPENABLE           (int_dap_enable_swj),
       .DAPWRITE            (int_dap_write_swj),
       .DAPABORT            (int_dap_abort_swj),
       .DAPADDR             (int_dap_addr_swj),
       .DAPWDATA            (int_dap_wdata_swj),
       .CDBGPWRUPREQ        (int_c_dbg_power_up_req),
       .CSYSPWRUPREQ        (int_c_sys_power_up_req),
       .CDBGRSTREQ          (), // Debug reset request to reset controller
       .SWDO                (int_sw_do),
       .SWDOEN              (int_sw_do_en),
       .JTAGNSW             (JTAGNSW), // JTAG/not Serial Wire Mode
       .JTAGTOP             ()  // JTAG-DP TAP controller in one of four top states
     );

  assign c_sys_power_up_ack = c_sys_power_up_req & c_sys_power_up;

  assign TDO                = (DP_PRESENT != 0) ? int_tdo                : 1'b0;
  assign nTDOEN             = (DP_PRESENT != 0) ? int_tdoen_n            : 1'b1;
  assign dap_sel_swj        = (DP_PRESENT != 0) ? int_dap_sel_swj        : 1'b0;
  assign dap_enable_swj     = (DP_PRESENT != 0) ? int_dap_enable_swj     : 1'b0;
  assign dap_write_swj      = (DP_PRESENT != 0) ? int_dap_write_swj      : 1'b0;
  assign dap_abort_swj      = (DP_PRESENT != 0) ? int_dap_abort_swj      : 1'b0;
  assign dap_addr_swj       = (DP_PRESENT != 0) ? int_dap_addr_swj       : {32{1'b0}};
  assign dap_wdata_swj      = (DP_PRESENT != 0) ? int_dap_wdata_swj      : {32{1'b0}};
  assign CDBGPWRUPREQ       = (DP_PRESENT != 0) ? int_c_dbg_power_up_req : 1'b0;
  assign c_sys_power_up_req = (DP_PRESENT != 0) ? int_c_sys_power_up_req : 1'b0;
  assign SWDO               = (DP_PRESENT != 0) ? int_sw_do              : 1'b0;
  assign SWDOEN             = (DP_PRESENT != 0) ? int_sw_do_en           : 1'b0;

  always @(dap_sel_tb or dap_enable_tb or dap_write_tb or dap_addr_tb or dap_wdata_tb or
           dap_abort_tb or dap_sel_swj or dap_enable_swj or dap_write_swj or
           dap_addr_swj or dap_wdata_swj or dap_abort_swj)
    begin
      if (dap_sel_tb)
        begin
          dap_sel      = dap_sel_tb;
          dap_enable   = dap_enable_tb;
          dap_write    = dap_write_tb;
          dap_abort    = dap_abort_tb;
          dap_addr_mux = dap_addr_tb;
          dap_wdata    = dap_wdata_tb;
        end
      else
        begin
          dap_sel      = dap_sel_swj;
          dap_enable   = dap_enable_swj;
          dap_write    = dap_write_swj;
          dap_abort    = dap_abort_swj;
          dap_addr_mux = dap_addr_swj;
          dap_wdata    = dap_wdata_swj;
        end
    end

  assign c_sys_power_up = dap_en_tb | c_sys_power_up_req;

  assign dap_addr = {8'h00, 16'h0000, dap_addr_mux[7:0]};


`ifdef ARM_CM3_ETM_LICENSE

  CM3ETM #(TRACE_LVL, CLKGATE_PRESENT, RESET_ALL_REGS)
    uCM3ETM
      (// Inputs
       .FCLK                (HCLK),
       .PORESETn            (int_poreset_n),
       .NIDEN               (niden),
       .FIFOFULLEN          (etm_fifo_full_en),
       .ETMIA               (etm_ia[31:1]),
       .ETMIVALID           (etm_ivalid),
       .ETMISTALL           (etm_istall),
       .ETMDVALID           (etm_dvalid),
       .ETMFOLD             (etm_fold),
       .ETMCANCEL           (etm_cancel),
       .ETMICCFAIL          (etm_ccfail),
       .ETMIBRANCH          (etm_branch),
       .ETMIINDBR           (etm_indirect_branch),
       .ETMISB              (etm_isb),
       .ETMINTSTAT          (ETMINTSTAT),
       .ETMINTNUM           (ETMINTNUM),
       .COREHALT            (HALTED),
       .EXTIN               (etm_ext_in),
       .MAXEXTIN            (etm_max_ext_in),
       .DWTMATCH            (etm_trigger),
       .DWTINOTD            (etm_trig_i_not_d),
       .ATREADYM            (atb_atready_etm),
       .PSEL                (ppb_psel_etm),
       .PENABLE             (ppb_penable),
       .PADDR               (ppb_paddr[11:2]),
       .PWRITE              (ppb_pwrite),
       .PWDATA              (ppb_pwdata),
       .TSVALUEB            (TSVALUEB),
       .TSCLKCHANGE         (TSCLKCHANGE),
       .SE                  (SE),
       .CGBYPASS            (CGBYPASS),
       .ETMPWRUP            (int_etm_power_up),
       .ETMEN               (int_etm_en),
       .PRDATA              (int_prdata_etm),
       .ATDATAM             (int_atb_atdata_etm[7:0]),
       .ATVALIDM            (int_atb_atvalid_etm),
       .AFREADYM            (int_atb_afready_etm),
       .ETMTRIGOUT          (int_etm_trigger_out),
       .ATIDM               (int_atb_atid_etm),
       .ETMDBGRQ            (int_etm_dbg_req),
       .FIFOFULL            (int_etm_fifo_full)
     );


  assign etm_dbg_req     = (TRACE_LVL > 1) ? int_etm_dbg_req     : 1'b0;
  assign etm_fifo_full   = (TRACE_LVL > 1) ? int_etm_fifo_full   : 1'b0;
  assign etm_power_up    = (TRACE_LVL > 1) ? int_etm_power_up    : 1'b0;
  assign etm_en          = (TRACE_LVL > 1) ? int_etm_en          : 1'b0;
  assign ppb_prdata_etm  = (TRACE_LVL > 1) ? int_prdata_etm      : {32{1'b0}};
  assign atb_atdata_etm  = (TRACE_LVL > 1) ? int_atb_atdata_etm  : {8{1'b0}};
  assign atb_atvalid_etm = (TRACE_LVL > 1) ? int_atb_atvalid_etm : 1'b0;
  assign atb_afready_etm = (TRACE_LVL > 1) ? int_atb_afready_etm : 1'b0;
  assign etm_trigger_out = (TRACE_LVL > 1) ? int_etm_trigger_out : 1'b0;
  assign atb_atid_etm    = (TRACE_LVL > 1) ? int_atb_atid_etm    : {7{1'b0}};

`else

  assign etm_dbg_req     = 1'b0;
  assign etm_fifo_full   = 1'b0;
  assign etm_power_up    = 1'b0;
  assign etm_en          = 1'b0;
  assign ppb_prdata_etm  = {32{1'b0}};
  assign atb_atdata_etm  = {8{1'b0}};
  assign atb_atvalid_etm = 1'b0;
  assign atb_afready_etm = 1'b0;
  assign etm_trigger_out = 1'b0;
  assign atb_atid_etm    = {7{1'b0}};

`endif

  assign atb_valid_port1 = TRACE_LVL > 1 ? atb_atvalid_etm : at_valid_itm;
  assign atb_id1         = TRACE_LVL > 1 ? atb_atid_etm    : at_id_itm;
  assign atb_data_port1  = TRACE_LVL > 1 ? atb_atdata_etm  : at_data_itm;
  assign atb_valid_port2 = TRACE_LVL > 1 ? at_valid_itm    : 1'b0;
  assign atb_id2         = TRACE_LVL > 1 ? at_id_itm       : {7{1'b0}};
  assign atb_data_port2  = TRACE_LVL > 1 ? at_data_itm     : {8{1'b0}};

  cm3_tpiu #(TRACE_LVL,0)
    u_cm3_tpiu
      (/*AUTOINST*/
       .CLK                 (dclk_g),
       .CLKEN               (tpiu_en),
       .TRACECLKIN          (trace_clk_in_g),
       .RESETn              (int_poreset_n),
       .TRESETn             (t_reset_n),
       .PWRITE              (ppb_pwrite),
       .PENABLE             (ppb_penable),
       .PSEL                (ppb_pseltpiu),
       .PADDR               (ppb_paddr[11:2]),
       .PWDATA              (ppb_pwdata[12:0]),
       .ATVALID1S           (atb_valid_port1),
       .ATID1S              (atb_id1),
       .ATDATA1S            (atb_data_port1),
       .ATVALID2S           (atb_valid_port2),
       .ATID2S              (atb_id2),
       .ATDATA2S            (atb_data_port2),
       .SYNCREQ             (d_sync),
       .MAXPORTSIZE         (2'b11),
       .TRIGGER             (etm_trigger_out),
       .PRDATA              (int_prdata_tpiu),
       .TPIUACTV            (int_tpiu_active),
       .TPIUBAUD            (int_tpiu_baud),
       .SWOACTIVE           (/*Can use for pin Muxing*/),
       .ATREADY1S           (int_atb_ready_port1),
       .ATREADY2S           (int_atb_ready_port2),
       .TRACECLK            (int_trace_clk),
       .TRACEDATA           (int_trace_data),
       .TRACESWO            (int_trace_swo)
       );

  assign atb_atready_etm = atb_ready_port1;
  assign at_ready_itm    = TRACE_LVL > 1 ? atb_ready_port2 : atb_ready_port1;

  assign ppb_prdata_tpiu = (TRACE_LVL > 0) ? int_prdata_tpiu     : {32{1'b0}};
  assign tpiu_active     = (TRACE_LVL > 0) ? int_tpiu_active     : 1'b0;
  assign tpiu_baud       = (TRACE_LVL > 0) ? int_tpiu_baud       : 1'b0;
  assign atb_ready_port1 = (TRACE_LVL > 0) ? int_atb_ready_port1 : 1'b1;
  assign atb_ready_port2 = (TRACE_LVL > 0) ? int_atb_ready_port2 : 1'b1;
  assign TRACECLK        = (TRACE_LVL > 0) ? int_trace_clk       : 1'b0;
  assign TRACEDATA       = (TRACE_LVL > 0) ? int_trace_data      : {4{1'b0}};
  assign trace_swo       = (TRACE_LVL > 0) ? int_trace_swo       : 1'b0;

  assign SWV = trace_swo;


  assign dap_sel_core = (dap_addr_mux[31:24]==8'h00) && dap_sel;

  always @(dap_sel_core or dap_ready_core or dap_rdata_core or dap_slverr_core)
    begin
      if (dap_sel_core)
        begin
          dap_ready  = dap_ready_core;
          dap_rdata  = dap_rdata_core;
          dap_slverr = dap_slverr_core;
        end
      else
        begin
          dap_ready  = 1'b1;
          dap_rdata  = {32{1'b0}};
          dap_slverr = 1'b0;
        end
    end


`ifdef ARM_TestBench

  assign ppb_psel_tb         = ppb_psel & (ppb_paddr[19:12] == 8'h42);
  assign ppb_prdata_mux_tbox = (ppb_psel_tb) ? ppb_prdata_tb  : {32{1'b0}};
  assign ppb_pslverr         = (ppb_psel_tb) ? ppb_pslverr_tb : 1'b0;
  assign ppb_pready          = (ppb_psel_tb) ? ppb_pready_tb  : 1'b1;

`else

  assign ppb_prdata_mux_tbox = {32{1'b0}};
  assign ppb_pslverr         = 1'b0;
  assign ppb_pready          = 1'b1;

`endif

  assign ppb_pseltpiu        = ppb_psel & ppb_paddr[19:12]==8'h40;
  assign ppb_psel_etm        = ppb_psel & ppb_paddr[19:12]==8'h41;
  assign ppb_psel_rom        = ppb_psel & ppb_paddr[19:12]==8'hFF;

  assign ppb_prdata_mux_tpiu = (ppb_pseltpiu) ? ppb_prdata_tpiu : {32{1'b0}};
  assign ppb_prdata_mux_etm  = (ppb_psel_etm) ? ppb_prdata_etm  : {32{1'b0}};
  assign ppb_prdata_mux_rom  = (ppb_psel_rom) ? ppb_prdata_rom  : {32{1'b0}};

  assign ppb_prdata          = (ppb_prdata_mux_tpiu | ppb_prdata_mux_etm |
                                ppb_prdata_mux_rom  | ppb_prdata_mux_tbox);

  cm3_rom_table #(DEBUG_LVL, TRACE_LVL)
    u_cm3_rom_table
      (/*AUTOINST*/
       .PCLK                (HCLK),
       .PRESETn             (int_poreset_n),
       .PSEL                (ppb_psel_rom),
       .PENABLE             (ppb_penable),
       .PADDR               (ppb_paddr[11:2]),
       .PWRITE              (ppb_pwrite),
       .PRDATA              (int_prdata_rom)
       );

  assign ppb_prdata_rom = (DEBUG_LVL > 0) ? int_prdata_rom : {32{1'b0}};

`ifdef ARM_TestBench


  assign test_bench_cfg[28]       = (BB_PRESENT > 0) ? 1'b1 : 1'b0;

  assign test_bench_cfg[27]       = 1'b0; // Reserved

  assign test_bench_cfg[26:18]    = WIC_LINES;

  assign test_bench_cfg[17]       = (WIC_PRESENT > 0) ? 1'b1 : 1'b0;

  assign test_bench_cfg[16]       = 1'b0; // Reserved
  assign test_bench_cfg[15]       = 1'b0; // Reserved
  assign test_bench_cfg[14]       = 1'b0; // Reserved

  assign test_bench_cfg[13]       = (OBSERVATION > 0) ? 1'b1 : 1'b0;
  assign test_bench_cfg[12]       = (CLKGATE_PRESENT > 0) ? 1'b1 : 1'b0;
  assign test_bench_cfg[11:8]     = TRACE_LVL;
  assign test_bench_cfg[7:4]      = DEBUG_LVL;
  assign test_bench_cfg[3]        = 1'b1;
  assign test_bench_cfg[2]        = (MPU_PRESENT  > 0) ? 1'b1 : 1'b0;
  assign test_bench_cfg[1]        = (JTAG_PRESENT > 0) ? 1'b1 : 1'b0;
  assign test_bench_cfg[0]        = (TRACE_LVL    > 1) ? 1'b1 : 1'b0;


  assign dap_clk_tb               = TBENCHINPUTS[0];
  assign dap_clk_en_tb            = TBENCHINPUTS[1];
  assign dap_addr_tb[31:0]        = TBENCHINPUTS[33:2];
  assign dap_sel_tb               = TBENCHINPUTS[34];
  assign dap_enable_tb            = TBENCHINPUTS[35];
  assign dap_write_tb             = TBENCHINPUTS[36];
  assign dap_wdata_tb[31:0]       = TBENCHINPUTS[68:37];
  assign dap_abort_tb             = TBENCHINPUTS[69];

  assign ppb_prdata_tb[31:0]      = TBENCHINPUTS[101:70];
  assign ppb_pready_tb            = TBENCHINPUTS[102];
  assign ppb_pslverr_tb           = TBENCHINPUTS[103];

  assign dap_en_tb                = TBENCHINPUTS[104];
  assign etm_ext_in[1:0]          = TBENCHINPUTS[106:105];
  assign etm_max_ext_in[1:0]      = TBENCHINPUTS[108:107];

  assign ppb_lock[5:0]            = TBENCHINPUTS[114:109];
  assign vect_addr[9:0]           = TBENCHINPUTS[124:115];
  assign vect_addr_en             = TBENCHINPUTS[125];
  assign niden                    = TBENCHINPUTS[126];
  assign etm_fifo_full_en         = TBENCHINPUTS[127];
  assign stk_align_init           = TBENCHINPUTS[128];


  assign TBENCHOUTPUTS[31:0]      = dap_rdata[31:0];
  assign TBENCHOUTPUTS[32]        = dap_ready;
  assign TBENCHOUTPUTS[33]        = dap_slverr;
  assign TBENCHOUTPUTS[34]        = dap_abort;

  assign TBENCHOUTPUTS[35]        = at_valid_itm;
  assign TBENCHOUTPUTS[36]        = at_ready_itm;
  assign TBENCHOUTPUTS[44:37]     = at_data_itm[7:0];

  assign TBENCHOUTPUTS[45]        = ppb_paddr31;
  assign TBENCHOUTPUTS[63:46]     = ppb_paddr[19:2];
  assign TBENCHOUTPUTS[64]        = ppb_psel_tb;
  assign TBENCHOUTPUTS[65]        = ppb_penable;
  assign TBENCHOUTPUTS[97:66]     = ppb_pwdata[31:0];
  assign TBENCHOUTPUTS[98]        = ppb_pwrite;

  assign TBENCHOUTPUTS[102:99]    = etm_trigger[3:0];
  assign TBENCHOUTPUTS[103]       = etm_power_up;
  assign TBENCHOUTPUTS[104]       = etm_en;

  assign TBENCHOUTPUTS[105]       = wic_sense[0];
  assign TBENCHOUTPUTS[106]       = wic_sense[1];
  assign TBENCHOUTPUTS[107]       = wic_sense[2];
  assign TBENCHOUTPUTS[108]       = |(int_wic_sense[WIC_LINES:3]);
  assign TBENCHOUTPUTS[109]       = wic_ds_req_n;
  assign TBENCHOUTPUTS[110]       = wic_ds_ack_n;
  assign TBENCHOUTPUTS[112:111]   = 2'b00; // reserved

  assign TBENCHOUTPUTS[141:113]   = test_bench_cfg[28:0];

`else

  assign dap_addr_tb              = {32{1'b0}};
  assign dap_sel_tb               = 1'b0;
  assign dap_enable_tb            = 1'b0;
  assign dap_write_tb             = 1'b0;
  assign dap_wdata_tb             = {32{1'b0}};
  assign dap_abort_tb             = 1'b0;
  assign dap_en_tb                = 1'b0;

  assign etm_ext_in               = 2'b00;
  assign etm_max_ext_in           = 2'b10;
  assign niden                    = 1'b1;
  assign etm_fifo_full_en         = 1'b1;

  assign stk_align_init           = 1'b1;

  assign ppb_lock                 = 6'b000000;
  assign vect_addr                = 10'b0000000000;
  assign vect_addr_en             = 1'b0;

`endif


  assign config_check_0[((NUM_IRQ > 0) & (NUM_IRQ < 241))] = 1'b1;

  assign config_check_1[((LVL_WIDTH > 2) & (LVL_WIDTH < 9))] = 1'b1;

  assign config_check_2[(DEBUG_LVL >= 0) & (DEBUG_LVL < 4)] = 1'b1;

  assign config_check_3[(TRACE_LVL >= 0) & (TRACE_LVL < 4)] = 1'b1;

  assign config_check_4[((MPU_PRESENT == 0) | (MPU_PRESENT == 1))] = 1'b1;

  assign config_check_5[((JTAG_PRESENT == 0) | (JTAG_PRESENT == 1))] = 1'b1;

  assign config_check_6[((CLKGATE_PRESENT == 0) |(CLKGATE_PRESENT == 1))] = 1'b1;

  assign config_check_7[((RESET_ALL_REGS == 0) | (RESET_ALL_REGS == 1))] = 1'b1;

  assign config_check_8[((OBSERVATION == 0) | (OBSERVATION == 1))] = 1'b1;

  assign config_check_9[((DEBUG_LVL != 0) | (TRACE_LVL == 0))] = 1'b1;

  assign config_check_10[(WIC_LINES > 2) & (WIC_LINES < (NUM_IRQ + 4))] = 1'b1;

  assign config_check_11[((WIC_PRESENT == 0) | (WIC_PRESENT == 1))] = 1'b1;

  assign config_check_12[((BB_PRESENT == 0) | (BB_PRESENT == 1))] = 1'b1;

  assign config_check_13[((CONST_AHB_CTRL == 0) | (CONST_AHB_CTRL == 1))] = 1'b1;

endmodule
