`define CM3_ITCMSIZE 4'b0110

`define CM3_DTCMSIZE 4'b0110

`define CM3_ITCM_INIT    1

`define CM3_DTCM_INIT    0

`define CM3_NUM_IRQ    16

`define CM3_JTAG_PRESENT 1

`define CM3_ARM_CODEMUX  1

`define CM3_MPU_PRESENT  1

`define CM3_LVL_WIDTH    3

`define CM3_TRACE_LVL    1

`define CM3_DEBUG_LVL    3

`define CM3_WIC_PRESENT  0

`define CM3_WIC_LINES    3

`define CM3_BB_PRESENT   1


module CortexM3DbgAXI (
  DBGRESTARTED, JTAGNSW, JTAGTOP, TDO, nTDOEN, SWDO, SWDOEN, LOCKUP, 
  HALTED, SYSRESETREQ, TRCENA, TRACECLK, TRACEDATA, WAKEUP, WICENACK,
  HCLK, STCLK, SYSRESETn, IRQ, NMI, DBGRESETn, EDBGRQ, DBGRESTART, nTRST, 
  SWCLKTCK, SWDITMS, TDI, CFGITCMEN, WICENREQ,
  
  AWADDRS, AWLENS, AWSIZES, AWBURSTS, AWLOCKS, AWCACHES, AWPROTS, AWUSERS, AWVALIDS, AWREADYS, 
   
  WSTRBS, WLASTS, WVALIDS, WREADYS, HWDATAS, 
   
  BRESPS, BVALIDS, BREADYS, 
   
  ARADDRS, ARLENS, ARSIZES, ARBURSTS, ARLOCKS, ARCACHES, ARPROTS, ARUSERS, ARVALIDS, ARREADYS, 
   
  RRESPS, RLASTS, RVALIDS, RREADYS, HRDATAS,

  AWADDRC, AWLENC, AWSIZEC, AWBURSTC, AWLOCKC, AWCACHEC, AWPROTC, AWUSERC, AWVALIDC, AWREADYC, 
   
  WSTRBC, WLASTC, WVALIDC, WREADYC, HWDATAC, 
   
  BRESPC, BVALIDC, BREADYC, 
   
  ARADDRC, ARLENC, ARSIZEC, ARBURSTC, ARLOCKC, ARCACHEC, ARPROTC, ARUSERC, ARVALIDC, ARREADYC, 
   
  RRESPC, RLASTC, RVALIDC, RREADYC,  HRDATAC
  ); 



  parameter             MPU_PRESENT    = `CM3_MPU_PRESENT;
  parameter             NUM_IRQ        = `CM3_NUM_IRQ;
  parameter             LVL_WIDTH      = `CM3_LVL_WIDTH;
  parameter             TRACE_LVL      = `CM3_TRACE_LVL;
  parameter             DEBUG_LVL      = `CM3_DEBUG_LVL;
  parameter             JTAG_PRESENT   = `CM3_JTAG_PRESENT;
  parameter             WIC_PRESENT    = `CM3_WIC_PRESENT;
  parameter             WIC_LINES      = `CM3_WIC_LINES;
  parameter             BB_PRESENT     = `CM3_BB_PRESENT;

  parameter             ITCM_SIZE      = `CM3_ITCMSIZE;     // ITCM size
  parameter             DTCM_SIZE      = `CM3_DTCMSIZE;     // DTCM size
  parameter             ITCM_INIT_RAM  = `CM3_ITCM_INIT;    // Initialise TCM if 1
  parameter             DTCM_INIT_RAM  = `CM3_DTCM_INIT;    // Initialise TCM if 1
  parameter             ITCM_INIT_FILE = "";                // File to init TCM only used ixTCM_RAM is set
  parameter             DTCM_INIT_FILE = "";                // File to init TCM only used ixTCM_RAM is set


  parameter             AUSER_WIDTH = 1; 
  parameter             AUSER_MAX   = AUSER_WIDTH-1;
  
  parameter             STRB_WIDTH = 32/8;          // WSTRB width - Data width fixed to 32
  parameter             STRB_MAX   = STRB_WIDTH-1;  // WSTRB max index

  

  input                 HCLK;          // System clock
  input                 STCLK;         // System tick clock, (1MHz).  REVISIT, check 1M okay
  input                 SYSRESETn;     // System reset
  input                 DBGRESETn;     // Power on debug reset

  output                LOCKUP;        // Core is in lockup state
  output                HALTED;        // Core is in Halt debug state
  
  output         [31:0] AWADDRS;       // write address
  output          [3:0] AWLENS;        // burst length
  output          [2:0] AWSIZES;       // burst size
  output          [1:0] AWBURSTS;      // burst type
  output          [1:0] AWLOCKS;       // burst lock type
  output          [3:0] AWCACHES;      // caching attributes
  output          [2:0] AWPROTS;       // protection attributes
  output  [AUSER_MAX:0] AWUSERS;       // write user signals 
  output                AWVALIDS;      // write address valid
  input                 AWREADYS;      // write address ready

  output   [STRB_MAX:0] WSTRBS;        // write strobes
  output                WLASTS;        // write last
  output                WVALIDS;       // write valid
  input                 WREADYS;       // write ready
  
  input           [1:0] BRESPS;        // write response
  input                 BVALIDS;       // response valid
  output                BREADYS;       // response ready

  output         [31:0] ARADDRS;       // write address
  output          [3:0] ARLENS;        // burst length
  output          [2:0] ARSIZES;       // burst size
  output          [1:0] ARBURSTS;      // burst type
  output          [1:0] ARLOCKS;       // burst lock type
  output          [3:0] ARCACHES;      // caching attributes
  output          [2:0] ARPROTS;       // protection attributes
  output  [AUSER_MAX:0] ARUSERS;       // read user signals 
  output                ARVALIDS;      // write address valid
  input                 ARREADYS;      // write address ready

  input          [1:0]  RRESPS;        // read response
  input                 RLASTS;        // last read transfer
  input                 RVALIDS;       // read valid
  output                RREADYS;       // read ready

  output         [31:0] HWDATAS;       // Data from master to slave
  input          [31:0] HRDATAS;       // Data from slave to master

  output         [31:0] AWADDRC;       // write address
  output          [3:0] AWLENC;        // burst length
  output          [2:0] AWSIZEC;       // burst size
  output          [1:0] AWBURSTC;      // burst type
  output          [1:0] AWLOCKC;       // burst lock type
  output          [3:0] AWCACHEC;      // caching attributes
  output          [2:0] AWPROTC;       // protection attributes
  output  [AUSER_MAX:0] AWUSERC;       // write user signals 
  output                AWVALIDC;      // write address valid
  input                 AWREADYC;      // write address ready

  output   [STRB_MAX:0] WSTRBC;        // write strobes
  output                WLASTC;        // write last
  output                WVALIDC;       // write valid
  input                 WREADYC;       // write ready
  
  input           [1:0] BRESPC;        // write response
  input                 BVALIDC;       // response valid
  output                BREADYC;       // response ready

  output         [31:0] ARADDRC;       // write address
  output          [3:0] ARLENC;        // burst length
  output          [2:0] ARSIZEC;       // burst size
  output          [1:0] ARBURSTC;      // burst type
  output          [1:0] ARLOCKC;       // burst lock type
  output          [3:0] ARCACHEC;      // caching attributes
  output          [2:0] ARPROTC;       // protection attributes
  output  [AUSER_MAX:0] ARUSERC;       // read user signals 
  output                ARVALIDC;      // write address valid
  input                 ARREADYC;      // write address ready

  input           [1:0] RRESPC;        // read response
  input                 RLASTC;        // last read transfer
  input                 RVALIDC;       // read valid
  output                RREADYC;       // read ready

  output         [31:0] HWDATAC;       // Data from master to slave
  input          [31:0] HRDATAC;       // Data from slave to master

  input                 nTRST;         // JTAG reset
  input                 SWCLKTCK;      // Serial wire and JTAG clock
  input                 SWDITMS;       // SW Data / JTAG Test Mode Select
  input                 TDI;           // JTAG data input
  output                JTAGNSW;       // JTAG = 1, serial wire = 0
  output                JTAGTOP;       // state controller indicator
  output                TDO;           // JTAG data output
  output                nTDOEN;        // JTAG data out enable
  output                SWDO;          // Serial wire data out
  output                SWDOEN;        // Serial data output enable

  input   [NUM_IRQ-1:0] IRQ;           // External interrupts
  input                 NMI;           // Non-maskable interrupt
  output                SYSRESETREQ;   // From u_cortexm1 of CortexM1Dbg.v
  
  input           [1:0] CFGITCMEN;     // Reset value for ITCM alias enables

  input                 EDBGRQ;        // External debug request
  input                 DBGRESTART;    // Restart from halt request
  output                DBGRESTARTED;  // Restart from halt acknowledge

  output wire           TRACECLK;      // Trace clock
  output wire           TRCENA;        // Trace enable
  output wire     [3:0] TRACEDATA;     // Trace data

  input                 WICENREQ;      // WIC mode Request from PMU
  output                WICENACK;      // WIC mode acknowledge from WIC
  output                WAKEUP;        // Wake-up request from WIC


  localparam ITCM_ADDR_WIDTH = ITCM_SIZE + 7;
  localparam DTCM_ADDR_WIDTH = DTCM_SIZE + 7;
  
  localparam ITCM_MEM_SIZE   = (2**(ITCM_ADDR_WIDTH+2));
  localparam DTCM_MEM_SIZE   = (2**(DTCM_ADDR_WIDTH+2));

  wire [1:1]            check_itcmsize;
  wire [1:1]            check_dtcmsize;

  assign check_itcmsize[(0 <= ITCM_SIZE) && (ITCM_SIZE <= 4'b1011)] = 1'b1;
  assign check_dtcmsize[(0 <= DTCM_SIZE) && (DTCM_SIZE <= 4'b1011)] = 1'b1;


  wire [DTCM_ADDR_WIDTH+1:2]    DTCMADDR;
  wire [3:0]                    DTCMBYTEWR;
  wire                          DTCMEN;
  wire [31:0]                   DTCMRDATA;
  wire [31:0]                   DTCMWDATA;
  wire [ITCM_ADDR_WIDTH+1:2]    ITCMADDR;
  wire [3:0]                    ITCMBYTEWR;
  wire                          ITCMEN;
  wire [31:0]                   ITCMRDATA;
  wire [31:0]                   ITCMWDATA;

  wire [3:0]                    CFGITCMSZ;
  wire [3:0]                    CFGDTCMSZ;

  wire                          DTCMRD;
  assign  DTCMRD = DTCMEN & (DTCMBYTEWR == 4'h0);

  
  assign CFGITCMSZ = ITCM_SIZE;
  assign CFGDTCMSZ = DTCM_SIZE;

/*  
  assign CFGITCMEN[0] = (ITCM_LA_EN!=0);
  assign CFGITCMEN[1] = (ITCM_UA_EN!=0);
*/


  wire   [31:0] haddri;
  wire    [1:0] htransi;
  wire    [2:0] hsizei;
  wire    [2:0] hbursti;
  wire    [3:0] hproti;
  wire    [1:0] memattri;
  wire   [31:0] hrdatai;
  wire          hreadyi;
  wire    [1:0] hrespi;

  wire   [31:0] haddrd;
  wire    [1:0] htransd;
  wire    [1:0] hmasterd;
  wire    [2:0] hsized;
  wire    [2:0] hburstd;
  wire    [3:0] hprotd;
  wire    [1:0] memattrd;
  wire   [31:0] hwdatad;
  wire          hwrited;
  wire          exreqd;
  wire   [31:0] hrdatad;
  wire          hreadyd;
  wire    [1:0] hrespd;
  wire          exrespd;

  wire   [31:0] haddrc;
  wire    [1:0] htransc;
  wire    [2:0] hsizec;
  wire    [2:0] hburstc;
  wire    [3:0] hprotc;
  wire          hwritec;
  wire          exreqc;

  wire   [31:0] hrdatac_int;
  wire   [31:0] hrdatac_itcm;
  wire   [31:0] hrdatac_axi;

  wire          hreadyc;
  wire          hreadyc_axi;
  wire          hreadyc_itcm;

  wire    [1:0] hrespc;
  wire    [1:0] hrespc_axi;
  wire          hrespc_itcm;     // The cmsdk_ahb_to_sram peripheral only supports single bit hresp

  wire   [31:0] haddrs;
  wire    [1:0] htranss;
  wire    [1:0] hmasters;       // Unused
  wire          hwrites;
  wire    [2:0] hsizes;
  wire          hmastlocks;
  wire    [2:0] hbursts;
  wire    [3:0] hprots;
  wire    [1:0] memattrs;       // Unused
  wire          exreqs;         // Unused
  wire          hreadys;
  wire          hreadys_axi;
  wire          hreadys_dtcm;

  wire   [31:0] hrdatas_int;
  wire   [31:0] hrdatas_dtcm;
  wire   [31:0] hrdatas_axi;

  wire    [1:0] hresps;
  wire    [1:0] hresps_axi;
  wire          hresps_dtcm;     // The cmsdk_ahb_to_sram peripheral only supports single bit hresp
  wire          exresps;

    wire    dtcm_memory_sel;
    reg     dtcm_phase_reg;
    
    assign  dtcm_memory_sel = (haddrs >= 32'h2000_0000) && (haddrs < (32'h2000_0000 + DTCM_MEM_SIZE));
    
    always @(posedge HCLK)
        if ( SYSRESETn == 1'b0 )
            dtcm_phase_reg <= 1'b1;
        else if( (htranss != 2'b00) && (hreadys == 1'b1) )
            dtcm_phase_reg <= dtcm_memory_sel;

    assign hrdatas_int = (dtcm_phase_reg) ? hrdatas_dtcm         : hrdatas_axi;
    assign hreadys     = (dtcm_phase_reg) ? hreadys_dtcm         : hreadys_axi;
    assign hresps      = (dtcm_phase_reg) ? {1'b0, hresps_dtcm}  : hresps_axi;
    
    AhbSToAxi #(
        .DATA_WIDTH     (32),
        .AUSER_WIDTH    (AUSER_WIDTH),
        .INCR_OVERRIDE  (0)
    ) u_s_AhbSToAxi (
        .CLK            (HCLK),
        .RESETn         (SYSRESETn),

        .HREADY         (1'b1),
        .HSEL           (~dtcm_memory_sel),         // AHB spec states that HSEL is a simple decode of address bus
        .HADDR          (haddrs),
        .HTRANS         (htranss),
        .HWRITE         (hwrites),
        .HSIZE          (hsizes),
        .HBURST         (hbursts),
        .HPROT          (hprots),
        .HMASTLOCK      (hmastlocks),
        .HAUSER         ({AUSER_WIDTH{1'b0}}),      // User signaling not used
        .HREADYOUT      (hreadys_axi),
        .HRESP          (hresps_axi),


        .AWADDR         (AWADDRS), 
        .AWLEN          (AWLENS), 
        .AWSIZE         (AWSIZES), 
        .AWBURST        (AWBURSTS), 
        .AWLOCK         (AWLOCKS), 
        .AWCACHE        (AWCACHES), 
        .AWPROT         (AWPROTS), 
        .AWUSER         (AWUSERS),
        .AWVALID        (AWVALIDS), 
        .AWREADY        (AWREADYS), 

        .WSTRB          (WSTRBS), 
        .WLAST          (WLASTS), 
        .WVALID         (WVALIDS), 
        .WREADY         (WREADYS), 

        .BRESP          (BRESPS), 
        .BVALID         (BVALIDS), 
        .BREADY         (BREADYS), 

        .ARADDR         (ARADDRS), 
        .ARLEN          (ARLENS), 
        .ARSIZE         (ARSIZES), 
        .ARBURST        (ARBURSTS), 
        .ARLOCK         (ARLOCKS), 
        .ARCACHE        (ARCACHES), 
        .ARPROT         (ARPROTS), 
        .ARUSER         (ARUSERS),
        .ARVALID        (ARVALIDS), 
        .ARREADY        (ARREADYS), 

        .RRESP          (RRESPS), 
        .RLAST          (RLASTS),
        .RVALID         (RVALIDS), 
        .RREADY         (RREADYS), 

        .CSYSREQ        (1'b0),
        .CSYSACK        (/* Unused */),
        .CACTIVE        (/* Unused */),

        .SCANENABLE     (1'b0),
        .SCANINCLK      (1'b0),
        .SCANOUTCLK     (/* Unused */)
    );

    assign hrdatas_axi = HRDATAS;

    assign exresps = 1'b1;

    wire [47:0]   tsvalueb;           // Timestamp for debug

    wire enablecnt = TRCENA & ~HALTED;

    cm3_tscnt_48 u_cm3_tscnt_48 (

        .clk            (HCLK),
        .resetn         (DBGRESETn),

        .enablecnt_i    (enablecnt),
        .tsvalueb_o     (tsvalueb)
    );

    cm3_code_mux u_cm3_code_mux (
        .HCLK           (HCLK),
        .HRESETn        (DBGRESETn),

        .HADDRI         (haddri),
        .HTRANSI        (htransi),
        .HSIZEI         (hsizei),
        .HBURSTI        (hbursti),
        .HPROTI         (hproti),
        .HRDATAI        (hrdatai),
        .HREADYI        (hreadyi),
        .HRESPI         (hrespi),

        .HADDRD         (haddrd),
        .HTRANSD        (htransd),
        .HSIZED         (hsized),
        .HBURSTD        (hburstd),
        .HPROTD         (hprotd),
        .HWDATAD        (hwdatad),
        .HWRITED        (hwrited),
        .EXREQD         (exreqd),
        .HRDATAD        (hrdatad),
        .HREADYD        (hreadyd),
        .HRESPD         (hrespd),
        .EXRESPD        (exrespd),

        .HADDRC         (haddrc),
        .HWDATAC        (HWDATAC),
        .HTRANSC        (htransc),
        .HWRITEC        (hwritec),
        .HSIZEC         (hsizec),
        .HBURSTC        (hburstc),
        .HPROTC         (hprotc),
        .EXREQC         (exreqc),
        .HRDATAC        (hrdatac_int),
        .HREADYC        (hreadyc),
        .HRESPC         (hrespc),
        .EXRESPC        (1'b0)

        );

    wire    itcm_memory_sel;
    reg     itcm_phase_reg;
    
    assign  itcm_memory_sel = ( ((haddrc >= 32'h0000_0000) && (haddrc < (32'h0000_0000 + ITCM_MEM_SIZE))) && (CFGITCMEN[0] == 1'b1) ) ||
                              ( ((haddrc >= 32'h1000_0000) && (haddrc < (32'h1000_0000 + ITCM_MEM_SIZE))) && (CFGITCMEN[1] == 1'b1) );
    
    always @(posedge HCLK)
        if ( SYSRESETn == 1'b0 )
            itcm_phase_reg <= 1'b1;
        else if( (htransc != 2'b00) && (hreadyc == 1'b1) )
            itcm_phase_reg <= itcm_memory_sel;

    assign hrdatac_int = (itcm_phase_reg) ? hrdatac_itcm         : hrdatac_axi;
    assign hreadyc     = (itcm_phase_reg) ? hreadyc_itcm         : hreadyc_axi;
    assign hrespc      = (itcm_phase_reg) ? {1'b0, hrespc_itcm}  : hrespc_axi;
    
    AhbSToAxi #(
        .DATA_WIDTH     (32),
        .AUSER_WIDTH    (AUSER_WIDTH),
        .INCR_OVERRIDE  (0)
    ) u_c_AhbSToAxi (
        .CLK            (HCLK),
        .RESETn         (SYSRESETn),

        .HREADY         (1'b1),
        .HSEL           (~itcm_memory_sel),         // AHB spec states that HSEL is a simple decode of address bus
        .HADDR          (haddrc),
        .HTRANS         (htransc),
        .HWRITE         (hwritec),
        .HSIZE          (hsizec),
        .HBURST         (hburstc),
        .HPROT          (hprotc),
        .HMASTLOCK      (1'b1),                     // All code accesses should be locked, no other accesses interleaved
        .HAUSER         ({AUSER_WIDTH{1'b0}}),      // User signaling not used
        .HREADYOUT      (hreadyc_axi),
        .HRESP          (hrespc_axi),


        .AWADDR         (AWADDRC), 
        .AWLEN          (AWLENC), 
        .AWSIZE         (AWSIZEC), 
        .AWBURST        (AWBURSTC), 
        .AWLOCK         (AWLOCKC), 
        .AWCACHE        (AWCACHEC), 
        .AWPROT         (AWPROTC), 
        .AWUSER         (AWUSERC),
        .AWVALID        (AWVALIDC), 
        .AWREADY        (AWREADYC), 

        .WSTRB          (WSTRBC), 
        .WLAST          (WLASTC), 
        .WVALID         (WVALIDC), 
        .WREADY         (WREADYC), 

        .BRESP          (BRESPC), 
        .BVALID         (BVALIDC), 
        .BREADY         (BREADYC), 

        .ARADDR         (ARADDRC), 
        .ARLEN          (ARLENC), 
        .ARSIZE         (ARSIZEC), 
        .ARBURST        (ARBURSTC), 
        .ARLOCK         (ARLOCKC), 
        .ARCACHE        (ARCACHEC), 
        .ARPROT         (ARPROTC), 
        .ARUSER         (ARUSERC),
        .ARVALID        (ARVALIDC), 
        .ARREADY        (ARREADYC), 

        .RRESP          (RRESPC), 
        .RLAST          (RLASTC),
        .RVALID         (RVALIDC), 
        .RREADY         (RREADYC), 

        .CSYSREQ        (1'b0),
        .CSYSACK        (/* Unused */),
        .CACTIVE        (/* Unused */),

        .SCANENABLE     (1'b0),
        .SCANINCLK      (1'b0),
        .SCANOUTCLK     (/* Unused */)
    );

    assign hrdatac_axi = HRDATAC;


    
    cmsdk_ahb_to_sram #(
        .AW             (ITCM_ADDR_WIDTH+2)
    ) u_cmsdk_ahb_to_itcm (
        .HCLK           (HCLK),
        .HRESETn        (DBGRESETn),
        .HSEL           (1'b1),
        .HREADY         (1'b1),
        .HTRANS         (htransc),
        .HSIZE          (hsizec),
        .HWRITE         (hwritec),
        .HADDR          (haddrc[ITCM_ADDR_WIDTH+1:0]),
        .HWDATA         (HWDATAC),
        .HREADYOUT      (hreadyc_itcm),
        .HRESP          (hrespc_itcm),
        .HRDATA         (hrdatac_itcm),

        .SRAMRDATA      (ITCMRDATA),
        .SRAMADDR       (ITCMADDR),
        .SRAMWEN        (ITCMBYTEWR),
        .SRAMWDATA      (ITCMWDATA),
        .SRAMCS         (ITCMEN)
    );

    cmsdk_ahb_to_sram #(
        .AW             (DTCM_ADDR_WIDTH+2)
    ) u_cmsdk_ahb_to_dtcm (
        .HCLK           (HCLK),
        .HRESETn        (DBGRESETn),
        .HSEL           (dtcm_memory_sel),
        .HREADY         (1'b1),
        .HTRANS         (htranss),
        .HSIZE          (hsizes),
        .HWRITE         (hwrites),
        .HADDR          (haddrs[DTCM_ADDR_WIDTH+1:0]),
        .HWDATA         (HWDATAS),
        .HREADYOUT      (hreadys_dtcm),
        .HRESP          (hresps_dtcm),
        .HRDATA         (hrdatas_dtcm),

        .SRAMRDATA      (DTCMRDATA),
        .SRAMADDR       (DTCMADDR),
        .SRAMWEN        (DTCMBYTEWR),
        .SRAMWDATA      (DTCMWDATA),
        .SRAMCS         (DTCMEN)
    );

    
  wire [239:0] INTISR      = {{(240-NUM_IRQ){1'b0}}, IRQ};
  
  wire core_bigend         = 1'b0;           // Peripherals in this system do not support BIGEND

  wire [25:0]  STCALIB;                      // Calibration signal for alternative clock source of SysTick timer
  assign STCALIB           = 25'h0;          // REVISIT - need to set correct value

  wire         cdbgpwrupack;                 // Debug Power Domain up acknowledge.
  wire         cdbgpwrupreq;                 // Debug Power Domain up request.
  assign cdbgpwrupack      = cdbgpwrupreq;   // Loop back power domain request, as power domain will always be available.
  
  CORTEXM3INTEGRATION #(
        .MPU_PRESENT   (MPU_PRESENT),
        .NUM_IRQ       (NUM_IRQ),
        .LVL_WIDTH     (LVL_WIDTH),
        .TRACE_LVL     (TRACE_LVL),
        .DEBUG_LVL     (DEBUG_LVL),
        .JTAG_PRESENT  (JTAG_PRESENT),
        .WIC_PRESENT   (WIC_PRESENT),
        .WIC_LINES     (WIC_LINES),
        .BB_PRESENT    (BB_PRESENT)
  ) u_CORTEXM3INTEGRATION (
       .ISOLATEn       (1'b1),               // Isolate core power domain
       .RETAINn        (1'b1),               // Retain core state during power-down

       .PORESETn       (DBGRESETn),          // Power on reset - reset processor and debug. Synchronous to FCLK and HCLK
       .SYSRESETn      (SYSRESETn),          // System reset   - reset processor only. Synchronous to FCLK and HCLK
       .RSTBYPASS      (1'b0),               // Reset bypass - disable internal generated reset for testing (e.gATPG)
       .CGBYPASS       (1'b0),               // Clock gating bypass - disable internal clock gating for testing.
       .SE             (1'b0),

       .FCLK           (HCLK),               // Free running clock - NVIC, SysTick, debug
       .HCLK           (HCLK),               // System clock - AHB, processor
       .TRACECLKIN     (HCLK),               // Trace clock input.

       .STCLK          (STCLK),              // External reference clock for SysTick (Not really a clock, it is sampled by DFF)
       .STCALIB        (STCALIB),            // Calibration info for SysTick

       .AUXFAULT       (32'h0),              // Auxiliary Fault Status Register inputs (1 cycle pulse)

       .BIGEND         (core_bigend),        // Big Endian - select when exiting system reset

       .nTRST          (nTRST),              // JTAG TAP Reset
       .SWCLKTCK       (SWCLKTCK),           // SW/JTAG Clock
       .SWDITMS        (SWDITMS),            // SW Debug Data In / JTAG Test Mode Select
       .TDI            (TDI),                // JTAG TAP Data In / Alternative input function
       .CDBGPWRUPACK   (cdbgpwrupack),       // Debug Power Domain up acknowledge.

       .INTISR         (INTISR),             // Interrupts
       .INTNMI         (NMI),                // Non-maskable Interrupt

       .HREADYI        (hreadyi),            // I-CODE bus ready
       .HRDATAI        (hrdatai),            // I-CODE bus read data
       .HRESPI         (hrespi),             // I-CODE bus response
       .IFLUSH         (1'b0),               // Reserved input

       .HREADYD        (hreadyd),            // D-CODE bus ready
       .HRDATAD        (hrdatad),            // D-CODE bus read data
       .HRESPD         (hrespd),             // D-CODE bus response
       .EXRESPD        (exrespd),            // D-CODE bus exclusive response

       .HREADYS        (hreadys),            // System bus ready
       .HRDATAS        (hrdatas_int),        // System bus read data
       .HRESPS         (hresps),             // System bus response
       .EXRESPS        (exresps),            // System bus exclusive response

       .RXEV           (1'b0),               // Receive Event input
       .SLEEPHOLDREQn  (1'b1),               // Extend Sleep request

       .EDBGRQ         (EDBGRQ),             // External Debug Request
       .DBGRESTART     (DBGRESTART),         // Debug Restart request

       .FIXMASTERTYPE  (1'b1),               // Override HMASTER for AHB-AP accesses. REVISIT - check this fixes AHB-AP master = 1.

       .WICENREQ       (WICENREQ),           // Enable WIC interface function.

       .TSVALUEB       (tsvalueb),           // Binary coded timestamp value for trace
       .TSCLKCHANGE    (1'b0),               // Timestamp clock ratio change is not used

       .DBGEN          (1'b1),               // Halting Debug Enable
       .MPUDISABLE     (1'b0),               // Disable MPU functionality


       .TDO            (TDO),                // JTAG TAP Data Out
       .nTDOEN         (nTDOEN),             // TDO enable
       .CDBGPWRUPREQ   (cdbgpwrupreq),       // Debug Power Domain up request
       .SWDO           (SWDO),               // SW Data Out
       .SWDOEN         (SWDOEN),             // SW Data Out Enable
       .JTAGNSW        (JTAGNSW),            // JTAG/not Serial Wire Mode

       .SWV            (),                   // SingleWire Viewer Data

       .TRACECLK       (TRACECLK),           // TRACECLK output
       .TRACEDATA      (TRACEDATA),          // Trace Data

       .HTMDHADDR      (),                   // HTM data HADDR
       .HTMDHTRANS     (),                   // HTM data HTRANS
       .HTMDHSIZE      (),                   // HTM data HSIZE
       .HTMDHBURST     (),                   // HTM data HBURST
       .HTMDHPROT      (),                   // HTM data HPROT
       .HTMDHWDATA     (),                   // HTM data HWDATA
       .HTMDHWRITE     (),                   // HTM data HWRITE
       .HTMDHRDATA     (),                   // HTM data HRDATA
       .HTMDHREADY     (),                   // HTM data HREADY
       .HTMDHRESP      (),                   // HTM data HRESP

       .HADDRI         (haddri),             // I-CODE bus address
       .HTRANSI        (htransi),            // I-CODE bus transfer type
       .HSIZEI         (hsizei),             // I-CODE bus transfer size
       .HBURSTI        (hbursti),            // I-CODE bus burst length
       .HPROTI         (hproti),             // i-code bus protection
       .MEMATTRI       (memattri),           // I-CODE bus memory attributes - REVISIT - unused

       .HADDRD         (haddrd),             // D-CODE bus address
       .HTRANSD        (htransd),            // D-CODE bus transfer type
       .HSIZED         (hsized),             // D-CODE bus transfer size
       .HWRITED        (hwrited),            // D-CODE bus write not read
       .HBURSTD        (hburstd),            // D-CODE bus burst length
       .HPROTD         (hprotd),             // D-CODE bus protection
       .MEMATTRD       (memattrd),           // D-CODE bus memory attributes - REVISIT - unused
       .HMASTERD       (hmasterd),           // D-CODE bus master - REVISIT - unused
       .HWDATAD        (hwdatad),            // D-CODE bus write data
       .EXREQD         (exreqd),             // D-CODE bus exclusive request

       .HADDRS         (haddrs),             // System bus address
       .HTRANSS        (htranss),            // System bus transfer type
       .HSIZES         (hsizes),             // System bus transfer size
       .HWRITES        (hwrites),            // System bus write not read
       .HBURSTS        (hbursts),            // System bus burst length
       .HPROTS         (hprots),             // System bus protection
       .HMASTLOCKS     (hmastlocks),         // System bus lock
       .MEMATTRS       (memattrs),           // System bus memory attributes
       .HMASTERS       (hmasters),           // System bus master
       .HWDATAS        (HWDATAS),            // System bus write data
       .EXREQS         (exreqs),             // System bus exclusive request

       .BRCHSTAT       (),                   // Branch State
       .HALTED         (HALTED),             // The processor is halted
       .DBGRESTARTED   (DBGRESTARTED),       // Debug Restart interface handshaking
       .LOCKUP         (LOCKUP),             // The processor is locked up
       .SLEEPING       (),                   // The processor is in sleep mdoe (sleep/deep sleep)
       .SLEEPDEEP      (),                   // The processor is in deep sleep mode
       .SLEEPHOLDACKn  (),                   // Acknowledge for SLEEPHOLDREQn
       .ETMINTNUM      (),                   // Current exception number
       .ETMINTSTAT     (),                   // Exception/Interrupt activation status
       .CURRPRI        (),                   // Current exception priority
       .TRCENA         (TRCENA),             // Trace Enable

       .SYSRESETREQ    (SYSRESETREQ),        // System Reset Request

       .TXEV           (),                   // Transmit Event

       .GATEHCLK       (),                   // when high, HCLK can be turned off

       .WAKEUP         (WAKEUP),             // Wake up request from WIC to PMU
       .WICENACK       (WICENACK)            // Acknowledge for WICENREQ - WIC operation deep sleep mode
   );




X_TCMDBG      #(
                 .ADDR_WIDTH                      (ITCM_ADDR_WIDTH),
                 .INIT_RAM                        (ITCM_INIT_RAM),
                 .INIT_FILE                       (ITCM_INIT_FILE)
               )                                            
      u_x_itcm (                                                              
                 .doutA                           (ITCMRDATA[31:0]),
                 .doutB                           (),
                 .clkA                            (HCLK),
                 .enaA                            (ITCMEN),
                 .weA                             (ITCMBYTEWR[3:0]),
                 .addrA                           (ITCMADDR),
                 .dinA                            (ITCMWDATA[31:0]),
                 .clkB                            (HCLK),
                 .enaB                            (1'b0),
                 .weB                             (4'h0),
                 .addrB                           ({ITCM_ADDR_WIDTH{1'b0}}),
                 .dinB                            (32'h0)
               );

               

X_TCMDBG      #(
                 .ADDR_WIDTH                      (DTCM_ADDR_WIDTH),
                 .INIT_RAM                        (DTCM_INIT_RAM),
                 .INIT_FILE                       (DTCM_INIT_FILE)
               )                                            
      u_x_dtcm (                                                              
                 .doutA                           (DTCMRDATA[31:0]),
                 .doutB                           (),
                 .clkA                            (HCLK),
                 .enaA                            (1'b1),
                 .weA                             (DTCMBYTEWR[3:0]),
                 .addrA                           (DTCMADDR),
                 .dinA                            (DTCMWDATA[31:0]),
                 .clkB                            (HCLK),
                 .enaB                            (1'b0),
                 .weB                             (4'h0),
                 .addrB                           ({DTCM_ADDR_WIDTH{1'b0}}),
                 .dinB                            (32'h0)
               );


endmodule       
