module DAPDpIMux
  (/*AUTOARG*/
  DBGCLK, nPOTRST, JTAGNSW, JBusreq, JBuswrite, JBusaddr, JBusabort,
  JBuswdata, JBusmode, JBusmask, JBustrncnt, JCDbgPwrUpReq,
  JCSysPwrUpReq, JCDbgRstReq, SBusreq, SBuswrite, SBusaddr,
  SBusabort, SBuswdata, SBusmode, SBusmask, SBustrncnt,
  SCDbgPwrUpReq, SCSysPwrUpReq, SCDbgRstReq,
  Busreq, Buswrite, Busaddr, Busabort, Buswdata, Busmode, Busmask,
  Bustrncnt, CDBGPWRUPREQ, CSYSPWRUPREQ, CDBGRSTREQ
  );

  parameter DP_PRESENT    = 1;
  parameter JTAG_PRESENT  = 1;


  input         DBGCLK;     // Debug clock
  input         nPOTRST;    // Power on reset
  input         JTAGNSW;    // JTAG select (not Serial Wire)

  input         JBusreq;    // JTAG Bus Transaction Request
  input         JBuswrite;  // JTAG Bus Write
  input  [31:0] JBusaddr;   // JTAG Bus Address
  input         JBusabort;  // JTAG Bus Abort
  input  [31:0] JBuswdata;  // JTAG Bus Write Data Port
  input   [1:0] JBusmode;   // JTAG Bus Mode
  input   [3:0] JBusmask;   // JTAG Bus Compare Mask
  input  [11:0] JBustrncnt; // JTAG Bus Transfer Cnt
  input         JCDbgPwrUpReq;  // JTAG-DP Debug Power Domain up request
  input         JCSysPwrUpReq;  // JTAG-DP System Power Domain up request
  input         JCDbgRstReq;    // JTAG-DP Debug reset request to reset controller

  input         SBusreq;    // SW   Bus Transaction Request
  input         SBuswrite;  // SW   Bus Write
  input  [31:0] SBusaddr;   // SW   Bus Address
  input         SBusabort;  // SW   Bus Abort
  input  [31:0] SBuswdata;  // SW   Bus Write Data Port
  input   [1:0] SBusmode;   // SW   Bus Mode
  input   [3:0] SBusmask;   // SW   Bus Compare Mask
  input  [11:0] SBustrncnt; // SW   Bus Transfer Cnt
  input         SCDbgPwrUpReq;  // SW-DP Debug Power Domain up request
  input         SCSysPwrUpReq;  // SW-DP System Power Domain up request
  input         SCDbgRstReq;    // SW-DP Debug reset request to reset controller

  output        Busreq;     // Bus Transaction Request
  output        Buswrite;   // Bus Write
  output [31:0] Busaddr;    // Bus Address
  output        Busabort;   // Bus Abort
  output [31:0] Buswdata;   // Bus Write Data Port
  output  [1:0] Busmode;    // Bus Mode
  output  [3:0] Busmask;    // Bus Compare Mask
  output [11:0] Bustrncnt;  // Bus Transfer Cnt
  output        CDBGPWRUPREQ;  // Debug Power Domain up request
  output        CSYSPWRUPREQ;  // System Power Domain up request
  output        CDBGRSTREQ;    // Debug reset request to reset controller

  wire          JBusreq;    // JTAG Bus Transaction Request
  wire          JBuswrite;  // JTAG Bus Write
  wire   [31:0] JBusaddr;   // JTAG Bus Address
  wire          JBusabort;  // JTAG Bus Abort
  wire   [31:0] JBuswdata;  // JTAG Bus Write Data Port
  wire    [1:0] JBusmode;   // JTAG Bus Mode
  wire    [3:0] JBusmask;   // JTAG Bus Compare Mask
  wire   [11:0] JBustrncnt; // JTAG Bus Transfer Cnt
  wire          JCDbgPwrUpReq;  // JTAG-DP Debug Power Domain up request
  wire          JCSysPwrUpReq;  // JTAG-DP System Power Domain up request
  wire          JCDbgRstReq;    // JTAG-DP Debug reset request to reset controller
  wire          SBusreq;    // SW   Bus Transaction Request
  wire          SBuswrite;  // SW   Bus Write
  wire   [31:0] SBusaddr;   // SW   Bus Address
  wire          SBusabort;  // SW   Bus Abort
  wire   [31:0] SBuswdata;  // SW   Bus Write Data Port
  wire    [1:0] SBusmode;   // SW   Bus Mode
  wire    [3:0] SBusmask;   // SW   Bus Compare Mask
  wire   [11:0] SBustrncnt; // SW   Bus Transfer Cnt
  wire          SCDbgPwrUpReq;  // SW-DP Debug Power Domain up request
  wire          SCSysPwrUpReq;  // SW-DP System Power Domain up request
  wire          SCDbgRstReq;    // SW-DP Debug reset request to reset controller
  wire           Busreq;     // Bus Transaction Request (registered)
  reg           Busreq_cdc_check;     // Bus Transaction Request (registered)
  wire           Busabort;   // Bus Abort (registered)
  reg           Busabort_cdc_check;   // Bus Abort (registered)
  wire          NextBusreq;     // Bus Transaction Request
  wire          NextBusabort;   // Bus Abort
  wire          Buswrite;   // Bus Write
  wire   [31:0] Busaddr;    // Bus Address
  wire   [31:0] Buswdata;   // Bus Write Data Port
  wire    [1:0] Busmode;    // Bus Mode
  wire    [3:0] Busmask;    // Bus Compare Mask
  wire   [11:0] Bustrncnt;  // Bus Transfer Cnt

  wire          ICDbgPwrUpReq; // Unregistered Debug Power Domain up request
  wire          ICSysPwrUpReq; // UnregisteredSystem Power Domain up request
  wire          ICDbgRstReq;   // UnregisteredDebug reset request to reset controller

  wire          CDBGPWRUPREQ;  // Debug Power Domain up request
  wire          CSYSPWRUPREQ;  // System Power Domain up request
  wire          CDBGRSTREQ;    // Debug reset request to reset controller

  wire          MstJTAGEn;
  wire          iBusreq;
  wire          iBuswrite;
  wire [31:0]   iBusaddr;
  wire          iBusabort;
  wire [31:0]   iBuswdata;
  wire [1:0]    iBusmode;
  wire [3:0]    iBusmask;
  wire [11:0]   iBustrncnt;
  reg           RegCDBGPWRUPREQ;
  reg           RegCSYSPWRUPREQ;
  reg           RegCDBGRSTREQ;

  assign MstJTAGEn    = (JTAG_PRESENT != 0) ? 1'b1          : 1'b0;

  assign Busreq       = (DP_PRESENT != 0)   ? iBusreq       : 1'b0;
  assign Buswrite     = (DP_PRESENT != 0)   ? iBuswrite     : 1'b0;
  assign Busaddr      = (DP_PRESENT != 0)   ? iBusaddr      : {32{1'b0}};
  assign Busabort     = (DP_PRESENT != 0)   ? iBusabort     : 1'b0;
  assign Buswdata     = (DP_PRESENT != 0)   ? iBuswdata     : {32{1'b0}};
  assign Busmode      = (DP_PRESENT != 0)   ? iBusmode      : 2'b00;
  assign Busmask      = (DP_PRESENT != 0)   ? iBusmask      : 4'b0000;
  assign Bustrncnt    = (DP_PRESENT != 0)   ? iBustrncnt    : {12{1'b0}};
  assign CDBGPWRUPREQ = (DP_PRESENT != 0)   ? RegCDBGPWRUPREQ : 1'b0;
  assign CSYSPWRUPREQ = (DP_PRESENT != 0)   ? RegCSYSPWRUPREQ : 1'b0;
  assign CDBGRSTREQ   = (DP_PRESENT != 0)   ? RegCDBGRSTREQ   : 1'b0;





  always @ (posedge DBGCLK or negedge nPOTRST)
    begin : p_BusRequest
      if (!nPOTRST)
         Busreq_cdc_check  <= 1'b0;
      else
         Busreq_cdc_check  <= NextBusreq;
    end

  always @ (posedge DBGCLK or negedge nPOTRST)
    begin : p_BusAbort
      if (!nPOTRST)
         Busabort_cdc_check  <= 1'b0;
      else
         Busabort_cdc_check  <= NextBusabort;
    end

  assign NextBusreq    = JTAGNSW & MstJTAGEn ? JBusreq    : SBusreq;
  assign NextBusabort  = JTAGNSW & MstJTAGEn ? JBusabort  : SBusabort;
  assign iBuswrite  = JTAGNSW & MstJTAGEn ? JBuswrite  : SBuswrite;
  assign iBusaddr   = JTAGNSW & MstJTAGEn ? JBusaddr   : SBusaddr;
  assign iBuswdata  = JTAGNSW & MstJTAGEn ? JBuswdata  : SBuswdata;
  assign iBusmode   = JTAGNSW & MstJTAGEn ? JBusmode   : SBusmode;
  assign iBusmask   = JTAGNSW & MstJTAGEn ? JBusmask   : SBusmask;
  assign iBustrncnt = JTAGNSW & MstJTAGEn ? JBustrncnt : SBustrncnt;


  assign ICDbgPwrUpReq = JTAGNSW & MstJTAGEn ? JCDbgPwrUpReq : SCDbgPwrUpReq;
  assign ICSysPwrUpReq = JTAGNSW & MstJTAGEn ? JCSysPwrUpReq : SCSysPwrUpReq;
  assign ICDbgRstReq   = JTAGNSW & MstJTAGEn ? JCDbgRstReq   : SCDbgRstReq;


  always @ (posedge DBGCLK or negedge nPOTRST)
    begin : p_DbgPwrUpReqReg
      if (!nPOTRST)
        RegCDBGPWRUPREQ <= 1'b0;
      else
        RegCDBGPWRUPREQ <= ICDbgPwrUpReq;
    end

  always @ (posedge DBGCLK or negedge nPOTRST)
    begin : p_SysPwrUpReqReg
      if (!nPOTRST)
        RegCSYSPWRUPREQ <= 1'b0;
      else
        RegCSYSPWRUPREQ <= ICSysPwrUpReq;
    end

  always @ (posedge DBGCLK or negedge nPOTRST)
    begin : p_DbgRstReqReg
      if (!nPOTRST)
        RegCDBGRSTREQ <= 1'b0;
      else
        RegCDBGRSTREQ <= ICDbgRstReq;
    end


`ifdef ARM_ASSERT_ON

  assert_win_unchange #(0, 88, 0, "SWD Protocol signals changed when JTAG selected")
  SWDSigChange(
               .clk           (DBGCLK),
               .reset_n       (nPOTRST),
               .start_event   (JTAGNSW & MstJTAGEn),
               .test_expr     ({SBusreq,SBuswrite,SBusaddr,SBusabort,
                                SBuswdata,SBusmode,SBusmask,SBustrncnt,
                                SCDbgPwrUpReq,SCSysPwrUpReq,SCDbgRstReq}),
               .end_event     (~JTAGNSW & MstJTAGEn));

  assert_win_unchange #(0, 88, 0, "JTAG Protocol signals changed when SWD selected")
  JTAGSigChange(
               .clk           (DBGCLK),
               .reset_n       (nPOTRST),
               .start_event   (~JTAGNSW & MstJTAGEn),
               .test_expr     ({JBusreq,JBuswrite,JBusaddr,JBusabort,
                                JBuswdata,JBusmode,JBusmask,JBustrncnt,
                                JCDbgPwrUpReq,JCSysPwrUpReq,JCDbgRstReq}),
               .end_event     (JTAGNSW & MstJTAGEn));

`endif

`ifdef ARM_CDC_ON
        cdc_launch uxcdc_Busabort(
                .clk_launch(DBGCLK),
                .clk_capture(DAPSWJDP.uDAPDpApbSync.uSyncBusAbort.SYNCCLK),
                .d_in(Busabort_cdc_check),
                .d_out(Busabort)
                );
`else
        assign iBusabort = Busabort_cdc_check;
`endif
`ifdef ARM_CDC_ON
        cdc_launch uxcdc_Busreq(
                .clk_launch(DBGCLK),
                .clk_capture(DAPSWJDP.uDAPDpApbSync.uSyncBusReq.SYNCCLK),
                .d_in(Busreq_cdc_check),
                .d_out(Busreq)
                );
`else
        assign iBusreq = Busreq_cdc_check;
`endif
endmodule
