`define DAP_APBIDLE     2'b00
`define DAP_APBSETUP    2'b01
`define DAP_APBENABLE   2'b11
`define DAP_APBEND      2'b10

`define DAP_M_NORMAL    2'b00
`define DAP_M_VERIFY    2'b01
`define DAP_M_FIND      2'b10
`define DAP_M_RSVD      2'b11


`define JTAGDP_E2D     4'b0000
`define JTAGDP_E1D     4'b0001
`define JTAGDP_SHD     4'b0010
`define JTAGDP_PDR     4'b0011
`define JTAGDP_SIR     4'b0100
`define JTAGDP_UDR     4'b0101
`define JTAGDP_CDR     4'b0110
`define JTAGDP_SDR     4'b0111
`define JTAGDP_E2I     4'b1000
`define JTAGDP_E1I     4'b1001
`define JTAGDP_SHI     4'b1010
`define JTAGDP_PIR     4'b1011
`define JTAGDP_RTI     4'b1100
`define JTAGDP_UIR     4'b1101
`define JTAGDP_CIR     4'b1110
`define JTAGDP_TLR     4'b1111

`define JTAGDP_EXTEST  4'b0000
`define JTAGDP_ABORT   4'b1000
`define JTAGDP_DPACC   4'b1010
`define JTAGDP_APACC   4'b1011
`define JTAGDP_IDCODE  4'b1110
`define JTAGDP_BYPASS  4'b1111

`define JTAGDP_CTRL    2'b01
`define JTAGDP_STAT    2'b01
`define JTAGDP_ADDR    2'b10
`define JTAGDP_DAPW    1'b0
`define JTAGDP_DAPR    1'b1
`define JTAGDP_APBR    1'b0
`define JTAGDP_APBW    1'b1
`define JTAGDP_MUXSTAT 2'b01
`define JTAGDP_MUXADDR 2'b10
`define JTAGDP_MUXRBUF 2'b11
`define JTAGDP_AP      1'b0
`define JTAGDP_DP      1'b1

`define JTAGDP_ZERO12  12'b000000000000

`define JTAGDP_DEVICEID 32'h4BA00477


module DAPJtagDpProtocol
  (/*AUTOARG*/
  nPOTRST, nTRST, TCK, TMS, TDI, BUSACK, BUSRDATA, BUSCMP, BUSERROR,
  BUSENDCNT, CDBGPWRUPACK, CSYSPWRUPACK, CDBGRSTACK,
  TDO, nTDOEN, BUSREQ, BUSWRITE, BUSABORT, BUSADDR, BUSWDATA,
  BUSMODE, BUSMASK, BUSTRNCNT, CDBGPWRUPREQ, CSYSPWRUPREQ,
  CDBGRSTREQ, JTAGTLR, JTAGTOP
  );

  parameter DP_PRESENT    = 1;
  parameter JTAG_PRESENT  = 1;

  input         nPOTRST;    // JTAG Reset - (Pulsed once at PowerOn)

  input         nTRST;      // JTAG Reset - (External)
  input         TCK;        // JTAG Test Clock Enable
  input         TMS;        // JTAG Test Mode Select
  input         TDI;        // JTAG Test Data In

  input         BUSACK;     // Bus Trasnaction Acknowledge
  input  [31:0] BUSRDATA;   // Bus Read Data Port
  input         BUSCMP;     // Bus Pushed Verify/Find
  input         BUSERROR;   // Bus Error
  input  [11:0] BUSENDCNT;  // Bus End Count

  input         CDBGPWRUPACK;  // Debug Power Domain up acknowledge
  input         CSYSPWRUPACK;  // System Power Domain up acknowledge
  input         CDBGRSTACK;    // Debug reset acknowledge to reset controller

  output        TDO;        // JTAG Test Data Out
  output        nTDOEN;     // JTAG Test Data Out Enable

  output        BUSREQ;     // Bus Transaction Request
  output        BUSWRITE;   // Bus Write
  output        BUSABORT;   // Bus Abort
  output [31:0] BUSADDR;    // Bus Address Port
  output [31:0] BUSWDATA;   // Bus Write Data Port
  output  [1:0] BUSMODE;    // Bus Mode
  output  [3:0] BUSMASK;    // Bus Compare Mask
  output [11:0] BUSTRNCNT;  // Bus Transfer Cnt

  output        CDBGPWRUPREQ;  // Debug Power Domain up request
  output        CSYSPWRUPREQ;  // System Power Domain up request
  output        CDBGRSTREQ;    // Debug reset request to reset controller
  output        JTAGTLR;       // JTAG TAP state machine in Test Logic Reset
  output        JTAGTOP;       // JTAG-DP TAP controller in one of four top states
  wire          nPOTRST;   // JTAG Reset - (Pulsed once at PowerOn)
  wire          nTRST;      // JTAG Reset - (External)
  wire          TCK;        // JTAG Test Clock Enable
  wire          TMS;        // JTAG Test Mode Select
  wire          TDI;        // JTAG Test Data In
  wire          BUSACK;     // Bus Trasnaction Acknowledge
  wire   [31:0] BUSRDATA;   // Bus Read Data Port
  wire          BUSCMP;     // Bus Pushed Verify/Find
  wire          BUSERROR;   // Bus Error
  wire   [11:0] BUSENDCNT;  // Bus End Count
  wire          CDBGPWRUPACK;  // Debug Power Domain up acknowledge
  wire          CSYSPWRUPACK;  // System Power Domain up acknowledge
  wire          CDBGRSTACK;    // Debug reset acknowledge to reset controller
  wire          TDO;        // JTAG Test Data Out
  wire          BUSREQ;     // Bus Transaction Request
  wire          BUSWRITE;   // Bus Write
  wire          BUSABORT;   // Bus Abort
  wire   [31:0] BUSADDR;    // Bus Address Port
  wire   [31:0] BUSWDATA;   // Bus Write Data Port
  wire    [1:0] BUSMODE;    // Bus Mode
  wire    [3:0] BUSMASK;    // Bus Compare Mask
  wire   [11:0] BUSTRNCNT;  // Bus Transfer Cnt
  wire          CDBGPWRUPREQ;  // Debug Power Domain up request
  wire          CSYSPWRUPREQ;  // System Power Domain up request
  wire          CDBGRSTREQ;    // Debug reset request to reset controller
  wire          JTAGTLR;       // JTAG TAP state machine in Test Logic Reset

  reg     [3:0] JTAGcurr;   // JTAG current state
  reg     [3:0] JTAGnext;   // JTAG next state
  reg     [3:0] JTAGir;     // JTAG Instruction register
  reg     [3:0] JTAGirD;    // JTAG IR DFF input
  reg           JTAGirEn;   // JTAG IR DFF enable
  reg     [3:0] JTAGsir;    // JTAG Scan instruction register
  reg     [3:0] JTAGsirD;   // JTAG SIR DFF input
  reg           JTAGsirEn;  // JTAG SIR DFF enable
  reg    [34:0] JTAGsdr;    // JTAG Scan data register
  reg    [34:0] JTAGsdrD;   // JTAG SDR DFF input
  reg           JTAGsdrEn;  // JTAG SDR DFF enable
  wire          TDOinstruction;// Instruction TDO ouput
  reg           TDOdata;    // Data TDO output
  reg           TDODi;      // TDO DFF input
  reg           TDOi;       // TDO output
  wire          TDOenDi;    // TDO Enable DFF input
  reg           TDOeni;     // TDO output Enable
  reg           APabort;    // AP abort
  reg           APabortD;   // AP abort DFF input
  reg           APabortEn;  // AP abort DFF enable
  wire     [1:0] Trnmode;    // transfer mode
  reg     [1:0] Trnmode_cdc_check;    // transfer mode
  wire     [3:0] Masklane;   // mask lanes for compare operations
  reg     [3:0] Masklane_cdc_check;   // mask lanes for compare operations
  wire          CtrlEn;     // control register enable
  wire    [11:0] Trncnt;     // transfer count
  reg    [11:0] Trncnt_cdc_check;     // transfer count
  reg    [11:0] TrncntD;    // transfer count DFF input
  reg           TrncntEn;   // transfer count DFF enable
  wire          Stickystatus;  // Sticky status
  reg           Stickycmp;  // Sticky compare register
  reg           StickycmpD; // Sticky compare DFF input
  reg           StickycmpEn;// Sticky compare DFF enable
  reg           Stickyerr;  // Sticky error register
  reg           StickyerrD; // Sticky error DFF input
  reg           StickyerrEn;// Sticky error DFF enable
  wire           APdir;      // AP access direction
  reg           APdir_cdc_check;      // AP access direction
  reg     [7:0] APsel;      // AP access selection
  wire          APselEn;    // AP selection enable
  reg     [7:4] APbanksel;  // AP bank selection
  reg     [1:0] DPaddr;     // DP address
  reg     [1:0] Rdmux;      // read multiplexor
  reg     [1:0] RdmuxD;     // read multiplexor DFF input
  wire          RdmuxEn;    // read multiplexor DFF enable
  reg           Busreqi;    // bus request to APB interface
  wire          BusreqDi;   // bus request DFF input
  reg           BusreqEni;  // bus request DFF enable
  wire    [31:0] Buswdatai;  // bus request write holding register
  reg    [31:0] Buswdatai_cdc_check;  // bus request write holding register
  wire          APaccEn;    // AP Access enable
  reg           Facerdycdri;  // interface ready @ CaptureDR
  wire          FacerdycdrDi; // interface ready DFF input
  wire          FacerdycdrEni;// interface ready DFF enable
  reg           Contdetect; // continuation detection
  wire          NCONT;      // nCONT output bit
  wire    [2:0] NCONTvec;   // nCONT output bit vector
  reg           NCONTerr;   // nCONT overrun detect
  reg           NCONTerrD;  // nCONT overrun DFF input
  reg           NCONTerrEn; // nCONT overrun DFF enable
  wire          TCKn;       // invert TCK clock for falling edge output
  reg           Transapdp;  // transaction to AP/DP
  wire          TransapdpD; // transaction DFF input
  wire          TransapdpEn;// transaction DFF enable
  wire          DPacc;      // DP Access enable
  wire          APacc;      // AP Access enable
  reg           Iptr;       // FIFO Pointer In
  wire          IptrD;      // FIFO Pointer In DFF input
  wire          IptrEn;     // FIFO Pointer In DFF enable
  reg           Optr;       // FIFO Pointer Out
  wire          OptrD;      // FIFO Pointer Out DFF input
  wire          OptrEn;     // FIFO Pointer Out DFF enable
  wire          Eptr;       // FIFO empty (write fifo)
  wire          Fptr;       // FIFO full

  wire          iTDO;
  wire          inTDOEN;
  wire          iBUSREQ;
  wire          iBUSWRITE;
  wire          iBUSABORT;
  wire [31:0]   iBUSADDR;
  wire [31:0]   iBUSWDATA;
  wire [1:0]    iBUSMODE;
  wire [3:0]    iBUSMASK;
  wire [11:0]   iBUSTRNCNT;
  reg           iCDBGPWRUPREQ;
  reg           iCSYSPWRUPREQ;
  reg           iCDBGRSTREQ;
  wire          iJTAGTLR;
  wire          iJTAGTOP;
  wire          MstJTAGEn;

  assign TDO          = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iTDO          : 1'b0;
  assign nTDOEN       = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? inTDOEN       : 1'b1;
  assign BUSREQ       = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iBUSREQ       : 1'b0;
  assign BUSWRITE     = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iBUSWRITE     : 1'b0;
  assign BUSABORT     = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iBUSABORT     : 1'b0;
  assign BUSADDR      = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iBUSADDR      : {32{1'b0}};
  assign BUSWDATA     = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iBUSWDATA     : {32{1'b0}};
  assign BUSMODE      = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iBUSMODE      : 2'b00;
  assign BUSMASK      = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iBUSMASK      : 4'b0000;
  assign BUSTRNCNT    = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iBUSTRNCNT    : {12{1'b0}};
  assign CDBGPWRUPREQ = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iCDBGPWRUPREQ : 1'b0;
  assign CSYSPWRUPREQ = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iCSYSPWRUPREQ : 1'b0;
  assign CDBGRSTREQ   = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iCDBGRSTREQ   : 1'b0;
  assign JTAGTLR      = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iJTAGTLR      : 1'b0;
  assign JTAGTOP      = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iJTAGTOP      : 1'b0;
  assign MstJTAGEn    = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? 1'b1          : 1'b0;



  always @ (JTAGcurr or TMS)
    case (JTAGcurr)
      `JTAGDP_TLR : JTAGnext = TMS ? `JTAGDP_TLR : `JTAGDP_RTI;
      `JTAGDP_RTI : JTAGnext = TMS ? `JTAGDP_SDR : `JTAGDP_RTI;
      `JTAGDP_SDR : JTAGnext = TMS ? `JTAGDP_SIR : `JTAGDP_CDR;
      `JTAGDP_CDR : JTAGnext = TMS ? `JTAGDP_E1D : `JTAGDP_SHD;
      `JTAGDP_SHD : JTAGnext = TMS ? `JTAGDP_E1D : `JTAGDP_SHD;
      `JTAGDP_E1D : JTAGnext = TMS ? `JTAGDP_UDR : `JTAGDP_PDR;
      `JTAGDP_PDR : JTAGnext = TMS ? `JTAGDP_E2D : `JTAGDP_PDR;
      `JTAGDP_E2D : JTAGnext = TMS ? `JTAGDP_UDR : `JTAGDP_SHD;
      `JTAGDP_UDR : JTAGnext = TMS ? `JTAGDP_SDR : `JTAGDP_RTI;
      `JTAGDP_SIR : JTAGnext = TMS ? `JTAGDP_TLR : `JTAGDP_CIR;
      `JTAGDP_CIR : JTAGnext = TMS ? `JTAGDP_E1I : `JTAGDP_SHI;
      `JTAGDP_SHI : JTAGnext = TMS ? `JTAGDP_E1I : `JTAGDP_SHI;
      `JTAGDP_E1I : JTAGnext = TMS ? `JTAGDP_UIR : `JTAGDP_PIR;
      `JTAGDP_PIR : JTAGnext = TMS ? `JTAGDP_E2I : `JTAGDP_PIR;
      `JTAGDP_E2I : JTAGnext = TMS ? `JTAGDP_UIR : `JTAGDP_SHI;
      `JTAGDP_UIR : JTAGnext = TMS ? `JTAGDP_SDR : `JTAGDP_RTI;
       default : JTAGnext = 4'bxxxx;
    endcase

  always @ (negedge nTRST or posedge TCK)
    if (!nTRST)
      JTAGcurr <= `JTAGDP_TLR;
    else
      JTAGcurr <= JTAGnext;


    assign iJTAGTLR = (JTAGcurr==`JTAGDP_TLR);


    assign iJTAGTOP = (JTAGcurr==`JTAGDP_TLR | JTAGcurr==`JTAGDP_RTI |
                      JTAGcurr==`JTAGDP_SDR | JTAGcurr==`JTAGDP_SIR);

  always @ (JTAGcurr or TDI or JTAGsir)
    case (JTAGcurr)
      `JTAGDP_TLR : begin
        JTAGsirD  = 4'b0001;
        JTAGsirEn = 1'b1;
      end
      `JTAGDP_CIR : begin
        JTAGsirD  = 4'b0001;
        JTAGsirEn = 1'b1;
      end
      `JTAGDP_SHI : begin
        JTAGsirD  = {TDI,JTAGsir[3:1]};
        JTAGsirEn = 1'b1;
      end
      `JTAGDP_E2D,
      `JTAGDP_E1D,
      `JTAGDP_SHD,
      `JTAGDP_PDR,
      `JTAGDP_SIR,
      `JTAGDP_UDR,
      `JTAGDP_CDR,
      `JTAGDP_SDR,
      `JTAGDP_E2I,
      `JTAGDP_E1I,
      `JTAGDP_PIR,
      `JTAGDP_RTI,
      `JTAGDP_UIR: begin
        JTAGsirD  = 4'b0000;
        JTAGsirEn = 1'b0;
      end
      default: begin
        JTAGsirD  = 4'bxxxx;
        JTAGsirEn = 1'bx;
      end
    endcase

  always @ (negedge nTRST or posedge TCK)
    if (!nTRST)
      JTAGsir <= 4'b0000;
    else if (JTAGsirEn)
      JTAGsir <= JTAGsirD;

  always @ (JTAGcurr or JTAGsir)
    case (JTAGcurr)
      `JTAGDP_TLR : begin
        JTAGirD  = `JTAGDP_IDCODE;
        JTAGirEn = 1'b1;
      end
      `JTAGDP_UIR : begin
        JTAGirD  = JTAGsir;
        JTAGirEn = 1'b1;
      end
      `JTAGDP_E2D,
      `JTAGDP_E1D,
      `JTAGDP_SHD,
      `JTAGDP_PDR,
      `JTAGDP_SIR,
      `JTAGDP_UDR,
      `JTAGDP_CDR,
      `JTAGDP_SDR,
      `JTAGDP_E2I,
      `JTAGDP_E1I,
      `JTAGDP_SHI,
      `JTAGDP_PIR,
      `JTAGDP_RTI,
      `JTAGDP_CIR: begin
        JTAGirD  = 4'b0000;
        JTAGirEn = 1'b0;
      end
      default: begin
        JTAGirD  = 4'bxxxx;
        JTAGirEn = 1'bx;
      end
    endcase

  always @ (negedge nTRST or posedge TCK)
    if (!nTRST)
      JTAGir <= `JTAGDP_IDCODE;
    else if (JTAGirEn & MstJTAGEn)
      JTAGir <= JTAGirD;

  assign TDOinstruction = JTAGsir[0];

  always @ (JTAGcurr or JTAGir or BUSRDATA or NCONT or Rdmux or
            APsel or APbanksel or Trncnt or Masklane or
            Stickycmp or Stickyerr or Trnmode or NCONTerr or Contdetect or
            TDI or JTAGsdr or Transapdp or CSYSPWRUPACK or CSYSPWRUPREQ or
            CDBGPWRUPACK or CDBGPWRUPREQ or CDBGRSTACK or CDBGRSTREQ)
    case (JTAGcurr)
      `JTAGDP_TLR : begin
        JTAGsdrD  = {35{1'b0}};
        JTAGsdrEn = 1'b1;
      end
      `JTAGDP_CDR : begin
        case (JTAGir)
          `JTAGDP_IDCODE : begin
            JTAGsdrD  = {`JTAGDP_DEVICEID,3'b000};
            JTAGsdrEn = 1'b1;
          end
          `JTAGDP_APACC,`JTAGDP_DPACC : begin
            if (Transapdp==`JTAGDP_AP)
              begin
                JTAGsdrD  = {BUSRDATA,1'b0,~NCONT,NCONT};
                JTAGsdrEn = 1'b1;
              end
            else
              begin
                case (Rdmux)
                  `JTAGDP_MUXADDR : begin
                    JTAGsdrD  = {APsel,8'h00,8'h00,APbanksel,
                                 4'h0,1'b0,~NCONT,NCONT};
                    JTAGsdrEn = 1'b1;
                  end
                  `JTAGDP_MUXSTAT : begin
                    JTAGsdrD  = {CSYSPWRUPACK, CSYSPWRUPREQ,
                                 CDBGPWRUPACK, CDBGPWRUPREQ,
                                 CDBGRSTACK,   CDBGRSTREQ,
                                 2'b00,Trncnt,Masklane,2'b00,
                                 Stickyerr,Stickycmp,Trnmode,
                                 NCONTerr,Contdetect,1'b0,~NCONT,NCONT};
                    JTAGsdrEn = 1'b1;
                  end
                  `JTAGDP_MUXRBUF,2'b00 : begin
                    JTAGsdrD  = {{32{1'b0}},1'b0,~NCONT,NCONT};
                    JTAGsdrEn = 1'b1;
                  end
                  default : begin
                    JTAGsdrD  = {35{1'bx}};
                    JTAGsdrEn = 1'bx;
                  end
                endcase
              end
          end
          `JTAGDP_EXTEST,
          `JTAGDP_BYPASS,
          `JTAGDP_ABORT,
          4'b0001,
          4'b0010,
          4'b0011,
          4'b0100,
          4'b0101,
          4'b0110,
          4'b0111,
          4'b1001,
          4'b1100,
          4'b1101: begin
            JTAGsdrD  = {35{1'b0}};
            JTAGsdrEn = 1'b1;
          end
          default: begin
            JTAGsdrD  = {35{1'bx}};
            JTAGsdrEn = 1'bx;
          end
        endcase
      end
      `JTAGDP_SHD : begin
        JTAGsdrD  = {TDI,JTAGsdr[34:1]};
        JTAGsdrEn = (JTAGir !=`JTAGDP_DPACC & JTAGir !=`JTAGDP_APACC) |
                    (JTAGir==`JTAGDP_DPACC | JTAGir==`JTAGDP_APACC);
      end
      `JTAGDP_E2D,
      `JTAGDP_E1D,
      `JTAGDP_PDR,
      `JTAGDP_SIR,
      `JTAGDP_UDR,
      `JTAGDP_SDR,
      `JTAGDP_E2I,
      `JTAGDP_E1I,
      `JTAGDP_SHI,
      `JTAGDP_PIR,
      `JTAGDP_RTI,
      `JTAGDP_UIR,
      `JTAGDP_CIR: begin
        JTAGsdrD  = {35{1'b0}};
        JTAGsdrEn = 1'b0;
      end
      default: begin
        JTAGsdrD  = {35{1'bx}};
        JTAGsdrEn = 1'bx;
      end
    endcase

  always @ (negedge nPOTRST or posedge TCK)
    if (!nPOTRST)
      JTAGsdr <= {35{1'b0}};
    else if (JTAGsdrEn)
      JTAGsdr <= JTAGsdrD;

  always @ (JTAGir or JTAGsdr)
    case (JTAGir)
      `JTAGDP_IDCODE : TDOdata = JTAGsdr[3];   // 32-bit register
      `JTAGDP_ABORT,
      `JTAGDP_DPACC,
      `JTAGDP_APACC  : TDOdata = JTAGsdr[0];   // 35-bit registers
      `JTAGDP_EXTEST,
      `JTAGDP_BYPASS,
      4'b0001,
      4'b0010,
      4'b0011,
      4'b0100,
      4'b0101,
      4'b0110,
      4'b0111,
      4'b1001,
      4'b1100,
      4'b1101 : TDOdata = JTAGsdr[34];  // 1-bit  register
      default : TDOdata = 1'bx;
    endcase

  always @ (JTAGcurr or TDOinstruction or TDOdata)
    case (JTAGcurr)
      `JTAGDP_SHI,`JTAGDP_E1I : TDODi = TDOinstruction;
      `JTAGDP_SHD,`JTAGDP_E1D : TDODi = TDOdata;
      `JTAGDP_E2D,`JTAGDP_PDR,
      `JTAGDP_SIR,`JTAGDP_UDR,
      `JTAGDP_CDR,`JTAGDP_SDR,
      `JTAGDP_E2I,`JTAGDP_PIR,
      `JTAGDP_RTI,`JTAGDP_UIR,
      `JTAGDP_CIR,`JTAGDP_TLR : TDODi = 1'b1;
      default   : TDODi = 1'bx;
    endcase

  assign TCKn = ~TCK;

  always @ (negedge nPOTRST or posedge TCKn)
    if (!nPOTRST)
      TDOi <= 1'b0;
    else
      TDOi <= TDODi;

  assign iTDO = TDOi;

  assign TDOenDi = (JTAGcurr==`JTAGDP_SHD | JTAGcurr==`JTAGDP_SHI);

  always @ (negedge nPOTRST or posedge TCKn)
    if (!nPOTRST)
      TDOeni <= 1'b0;
    else
      TDOeni <= TDOenDi;

  assign inTDOEN = ~TDOeni;

  assign DPacc = JTAGcurr==`JTAGDP_UDR & JTAGir==`JTAGDP_DPACC & Facerdycdri;
  assign APacc = JTAGcurr==`JTAGDP_UDR & JTAGir==`JTAGDP_APACC & Facerdycdri;

  assign RdmuxEn = DPacc;

  always @ (JTAGsdr)
    case (JTAGsdr[2:1])
      `JTAGDP_ADDR   : RdmuxD  = `JTAGDP_MUXADDR;
      `JTAGDP_STAT   : RdmuxD  = `JTAGDP_MUXSTAT;
      2'b00,
      2'b11   : RdmuxD  = `JTAGDP_MUXRBUF;
      default : RdmuxD  = 2'bxx;
    endcase

  always @ (negedge nPOTRST or posedge TCK)
    if (!nPOTRST)
      Rdmux <= `JTAGDP_MUXRBUF;
    else if (RdmuxEn & MstJTAGEn)
      Rdmux <= RdmuxD;


  assign FacerdycdrDi  = ~NCONT;
  assign FacerdycdrEni = JTAGcurr==`JTAGDP_CDR;

  always @ (negedge nPOTRST or posedge TCK)
    if (!nPOTRST)
      Facerdycdri <= 1'b0;
    else if (FacerdycdrEni & MstJTAGEn)
      Facerdycdri <= FacerdycdrDi;

  always @ (Contdetect or DPacc or JTAGcurr or JTAGir or JTAGsdr or
            NCONTerr or NCONT)
    if (DPacc)
      begin
        NCONTerrD  = 1'b0;
        NCONTerrEn = (JTAGsdr[4] | ~JTAGsdr[3]) & NCONTerr &
                     JTAGsdr[2:1]==`JTAGDP_CTRL & JTAGsdr[0]==`JTAGDP_DAPW;
      end
    else
      begin
        NCONTerrD  = 1'b1;
        NCONTerrEn = Contdetect & NCONT & ~NCONTerr & JTAGcurr==`JTAGDP_CDR &
                     (JTAGir==`JTAGDP_APACC | JTAGir==`JTAGDP_DPACC);
      end

  always @ (negedge nPOTRST or posedge TCK)
    if (!nPOTRST)
      NCONTerr <= 1'b0;
    else if (NCONTerrEn & MstJTAGEn)
      NCONTerr <= NCONTerrD;

  always @ (JTAGcurr or JTAGir or APabort or Busreqi or BUSACK)
    if (JTAGir==`JTAGDP_ABORT & JTAGcurr==`JTAGDP_UDR)
      begin
        APabortD  = 1'b1;
        APabortEn = Busreqi & ~BUSACK;
      end
    else
      begin
        APabortD  = 1'b0;
        APabortEn = APabort & BUSACK;
      end

  always @ (negedge nPOTRST or posedge TCK)
    if (!nPOTRST)
      APabort <= 1'b0;
    else if (APabortEn & MstJTAGEn)
      APabort <= APabortD;

  assign APselEn = DPacc & JTAGsdr[2:1]==`JTAGDP_ADDR & JTAGsdr[0]==`JTAGDP_DAPW;

  always @ (negedge nPOTRST or posedge TCK)
    if (!nPOTRST)
      begin
        APsel     <= 8'b00000000;
        APbanksel <= 4'b0000;
      end
    else if (APselEn & MstJTAGEn)
      begin
        APsel     <= JTAGsdr[34:27];
        APbanksel <= JTAGsdr[10:7];
      end

  assign CtrlEn = DPacc & JTAGsdr[2:1]==`JTAGDP_CTRL & JTAGsdr[0]==`JTAGDP_DAPW;

  always @ (negedge nPOTRST or posedge TCK)
    if (!nPOTRST)
      begin
        Contdetect   <= 1'b0;
        Trnmode_cdc_check       <= 2'b00;
        Masklane_cdc_check      <= 4'b0000;
        iCDBGPWRUPREQ <= 1'b0;
        iCSYSPWRUPREQ <= 1'b0;
        iCDBGRSTREQ   <= 1'b0;
      end
    else if (CtrlEn & MstJTAGEn)
      begin
        Contdetect   <= JTAGsdr[3];
        Trnmode_cdc_check       <= JTAGsdr[6:5];
        Masklane_cdc_check      <= JTAGsdr[14:11];
        iCDBGRSTREQ   <= JTAGsdr[29];
        iCSYSPWRUPREQ <= JTAGsdr[33];
        iCDBGPWRUPREQ <= JTAGsdr[31];
      end

  always @ (JTAGcurr or JTAGir or Facerdycdri or JTAGsdr or BUSACK or
            BUSENDCNT or Busreqi)
    if (JTAGcurr==`JTAGDP_UDR & JTAGir==`JTAGDP_DPACC & Facerdycdri)
      begin
        TrncntD  = JTAGsdr[26:15];
        TrncntEn = JTAGsdr[2:1]==`JTAGDP_CTRL & JTAGsdr[0]==`JTAGDP_DAPW;
      end
    else
      begin
        TrncntD  = BUSENDCNT;
        TrncntEn = BUSACK & Busreqi;
      end

  always @ (negedge nPOTRST or posedge TCK)
    if (!nPOTRST)
       Trncnt_cdc_check  <= 12'b000000000000;
    else if (TrncntEn & MstJTAGEn)
       Trncnt_cdc_check  <= TrncntD;

  assign APaccEn = APacc & ~Stickystatus;

  always @ (negedge nPOTRST or posedge TCK)
    if (!nPOTRST)
      begin
        Buswdatai_cdc_check  <= 32'h00000000;
        DPaddr               <= 2'b00;
        APdir_cdc_check      <= 1'b0;
      end
    else if (APaccEn & MstJTAGEn)
      begin
        Buswdatai_cdc_check  <= JTAGsdr[34:3];
        DPaddr               <= JTAGsdr[2:1];
        APdir_cdc_check      <= (~JTAGsdr[0]);
      end

  assign IptrEn = APacc & ~Stickystatus & CDBGPWRUPACK & CDBGPWRUPREQ;
  assign IptrD  = ~Iptr;

  always @ (negedge nPOTRST or posedge TCK)
    if (!nPOTRST)
      Iptr <= 1'b0;
    else if (IptrEn & MstJTAGEn)
      Iptr <= IptrD;

  assign OptrEn = Busreqi & BUSACK;
  assign OptrD  = ~Optr;

  always @ (negedge nPOTRST or posedge TCK)
    if (!nPOTRST)
      Optr <= 1'b0;
    else if (OptrEn & MstJTAGEn)
      Optr <= OptrD;

  assign Eptr = Iptr==Optr;

  assign Fptr = Iptr!=Optr;

  assign NCONTvec = {// DP access must wait for transaction buffer to clear.
                     JTAGir==`JTAGDP_DPACC & ~Eptr,
                     JTAGir==`JTAGDP_APACC &  Fptr &  APdir &
                       ~(OptrEn & ~IptrEn & OptrD==Iptr),
                     JTAGir==`JTAGDP_APACC & ~Eptr & ~APdir &
                       ~(OptrEn & ~IptrEn & OptrD==Iptr)};

  assign NCONT = |NCONTvec;

  always @ (BUSCMP or BUSACK or Stickycmp or Busreqi or JTAGcurr or
            JTAGir or Facerdycdri or JTAGsdr or Trnmode)
    begin
      StickycmpD  = 1'b0;
      StickycmpEn = 1'b0;
      if (BUSACK & Busreqi)
        begin
          StickycmpD  = 1'b1;
          StickycmpEn = (BUSCMP & Trnmode==`DAP_M_FIND) |
                        (!BUSCMP & Trnmode==`DAP_M_VERIFY);
        end
      else if (JTAGcurr==`JTAGDP_UDR & JTAGir==`JTAGDP_DPACC & Facerdycdri)
        if (JTAGsdr[2:1]==`JTAGDP_CTRL & JTAGsdr[0]==`JTAGDP_DAPW)
          begin
            StickycmpD  = 1'b0;
            StickycmpEn = JTAGsdr[7] & Stickycmp;
          end
    end

  always @ (negedge nPOTRST or posedge TCK)
    if (!nPOTRST)
      Stickycmp <= 1'b0;
    else if (StickycmpEn & MstJTAGEn)
      Stickycmp <= StickycmpD;

  always @ (BUSERROR or BUSACK or Stickyerr or Busreqi or JTAGcurr or
            JTAGir or Facerdycdri or JTAGsdr or APacc or Eptr or
            CDBGPWRUPACK or CDBGPWRUPREQ)
    begin
      StickyerrD  = 1'b0;
      StickyerrEn = 1'b0;
      if (BUSACK & Busreqi)
        begin
          StickyerrD  = 1'b1;
          StickyerrEn = BUSERROR;
        end
      else if (APacc & Eptr)
        begin
          StickyerrD  = 1'b1;
          StickyerrEn = ~Stickyerr & ~(CDBGPWRUPACK & CDBGPWRUPREQ);
        end
      else if (JTAGcurr==`JTAGDP_UDR & JTAGir==`JTAGDP_DPACC & Facerdycdri)
        if (JTAGsdr[2:1]==`JTAGDP_CTRL & JTAGsdr[0]==`JTAGDP_DAPW)
          begin
            StickyerrD  = 1'b0;
            StickyerrEn = JTAGsdr[8] & Stickyerr;
          end
    end

  always @ (negedge nPOTRST or posedge TCK)
    if (!nPOTRST)
      Stickyerr <= 1'b0;
    else if (StickyerrEn & MstJTAGEn)
      Stickyerr <= StickyerrD;

  assign Stickystatus = Stickycmp | Stickyerr | (Contdetect & NCONTerr);


  always @ (Stickystatus or APacc or BUSACK or Busreqi or
            Eptr or CDBGPWRUPACK or CDBGPWRUPREQ)
    if (Busreqi)
      begin
        BusreqEni = BUSACK;
      end
    else if ((APacc & Eptr) | ~Eptr)
      begin
        BusreqEni = ~Stickystatus & ~BUSACK &
                    CDBGPWRUPACK & CDBGPWRUPREQ;
      end
    else
      begin
        BusreqEni = 1'b0;
      end

  assign BusreqDi = (Busreqi & ~BUSACK) |
                    ((APacc | ~Eptr) & ~Stickystatus & ~BUSACK &
                     CDBGPWRUPACK & CDBGPWRUPREQ);

  always @ (negedge nPOTRST or posedge TCK)
    if (!nPOTRST)
      Busreqi <= 1'b0;
    else if (BusreqEni & MstJTAGEn)
      Busreqi <= BusreqDi;

  assign TransapdpD  = JTAGir==`JTAGDP_APACC ? `JTAGDP_AP : `JTAGDP_DP;
  assign TransapdpEn = JTAGcurr==`JTAGDP_UDR & Facerdycdri;

  always @ (negedge nPOTRST or posedge TCK)
    if (!nPOTRST)
      Transapdp <= 1'b0;
    else if (TransapdpEn)
      Transapdp <= TransapdpD;

  assign iBUSREQ     = BusreqDi;
  assign iBUSADDR    = {APsel,8'b00000000,8'b00000000,APbanksel,
                       DPaddr,2'b00};
  assign iBUSWDATA   = Buswdatai;
  assign iBUSABORT   = BusreqDi ? APabort : 1'b0;
  assign iBUSWRITE   = APdir;
  assign iBUSMODE    = Trnmode;
  assign iBUSMASK    = Masklane;
  assign iBUSTRNCNT  = Trncnt;


`ifdef ARM_ASSERT_ON

  assert_never_unknown #(`OVL_ERROR, 4, `OVL_ASSERT, "nTRST must be pulsed for simulation to run correctly.")
  JTAGdp_ntrst(
               .clk       (TCK),
               .reset_n   (nTRST),
               .qualifier (1'b1),
               .test_expr (JTAGcurr));

  assert_never_unknown #(`OVL_ERROR, 12, `OVL_ASSERT, "Initialise transfer counterbefore using APACC interface.")
  JTAGdp_trncnt(
                .clk       (TCK),
                .reset_n   (nPOTRST),
                .qualifier (BUSREQ),
                .test_expr ({Trncnt}));

  assert_never_unknown #(`OVL_ERROR, 12, `OVL_ASSERT, "Initialise address register before using APACC interface.")
  JTAGdp_apsel(
               .clk       (TCK),
               .reset_n   (nPOTRST),
               .qualifier (BUSREQ),
               .test_expr ({APsel,APbanksel}));

  assert_never_unknown #(`OVL_ERROR, 2, `OVL_ASSERT, "ReadMux must not be X during transaction.")
  JTAGdp_Rdmux(
               .clk       (TCK),
               .reset_n   (nPOTRST),
               .qualifier (JTAGcurr==`JTAGDP_CDR&JTAGir==`JTAGDP_DPACC),
               .test_expr (Rdmux));

  assert_never #(`OVL_ERROR, `OVL_ASSERT, "FIFO full has been pushed.")
    fifofull_check(TCK, nPOTRST, IptrEn & Fptr);

  assert_never #(`OVL_ERROR, `OVL_ASSERT, "FIFO empty has been popped.")
    fifoempty_check(TCK, nPOTRST, OptrEn & Eptr);

  assert_never #(`OVL_ERROR, `OVL_ASSERT, "Incorrect application of BUSREQ controls.")
    busreq_check(TCK, nPOTRST, BusreqEni & (BusreqDi==Busreqi));

  assert_never_unknown #(`OVL_ERROR, 3, `OVL_ASSERT, "NCONT vector must be exclusive.")
    ncont_exclusive(TCK, nPOTRST, 1'b1, NCONTvec);

  assert_never #(`OVL_ERROR, `OVL_ASSERT, "FIFO cannot not be incremented and decremented in same cycle.")
    fifoctrl_check(TCK, nPOTRST, IptrEn & OptrEn & ~Stickystatus);

  reg  [8*20:0] mnemonic;

  always @ (JTAGcurr)
    case (JTAGcurr)
      `JTAGDP_TLR : mnemonic = {"TLR"};
      `JTAGDP_RTI : mnemonic = {"RTI"};
      `JTAGDP_SDR : mnemonic = {"SDR"};
      `JTAGDP_CDR : mnemonic = {"CDR"};
      `JTAGDP_SHD : mnemonic = {"SHD"};
      `JTAGDP_E1D : mnemonic = {"E1D"};
      `JTAGDP_PDR : mnemonic = {"PDR"};
      `JTAGDP_E2D : mnemonic = {"E2D"};
      `JTAGDP_UDR : mnemonic = {"UDR"};
      `JTAGDP_SIR : mnemonic = {"SIR"};
      `JTAGDP_CIR : mnemonic = {"CIR"};
      `JTAGDP_SHI : mnemonic = {"SHI"};
      `JTAGDP_E1I : mnemonic = {"E1I"};
      `JTAGDP_PIR : mnemonic = {"PIR"};
      `JTAGDP_E2I : mnemonic = {"E2I"};
      `JTAGDP_UIR : mnemonic = {"UIR"};
    endcase

`endif

`ifdef ARM_CDC_ON
        cdc_launch #(31,0) uxcdc_Buswdatai(
                .clk_launch(TCK),
                .clk_capture(DAPSWJDP.uDAPDpApbIf.DAPCLK),
                .d_in(Buswdatai_cdc_check),
                .d_out(Buswdatai)
                );
`else
        assign Buswdatai = Buswdatai_cdc_check;
`endif
`ifdef ARM_CDC_ON
        cdc_launch #(3,0) uxcdc_Masklane(
                .clk_launch(TCK),
                .clk_capture(DAPSWJDP.uDAPDpApbIf.DAPCLK),
                .d_in(Masklane_cdc_check),
                .d_out(Masklane)
                );
`else
        assign Masklane = Masklane_cdc_check;
`endif
`ifdef ARM_CDC_ON
        cdc_launch #(1,0) uxcdc_Trnmode(
                .clk_launch(TCK),
                .clk_capture(DAPSWJDP.uDAPDpApbIf.DAPCLK),
                .d_in(Trnmode_cdc_check),
                .d_out(Trnmode)
                );
`else
        assign Trnmode = Trnmode_cdc_check;
`endif
`ifdef ARM_CDC_ON
        cdc_launch uxcdc_APdir(
                .clk_launch(TCK),
                .clk_capture(DAPSWJDP.uDAPDpApbIf.DAPCLK),
                .d_in(APdir_cdc_check),
                .d_out(APdir)
                );
`else
        assign APdir = APdir_cdc_check;
`endif
`ifdef ARM_CDC_ON
        cdc_launch #(11,0) uxcdc_Trncnt(
                .clk_launch(TCK),
                .clk_capture(DAPSWJDP.uDAPDpApbIf.DAPCLK),
                .d_in(Trncnt_cdc_check),
                .d_out(Trncnt)
                );
`else
        assign Trncnt = Trncnt_cdc_check;
`endif
endmodule
