`define DAP_APBIDLE     2'b00
`define DAP_APBSETUP    2'b01
`define DAP_APBENABLE   2'b11
`define DAP_APBEND      2'b10

`define DAP_M_NORMAL    2'b00
`define DAP_M_VERIFY    2'b01
`define DAP_M_FIND      2'b10
`define DAP_M_RSVD      2'b11


module DAPSwDpApbIf
  (/*AUTOARG*/
   DAPRESETn, DAPCLK, DAPCLKEN, DAPRDATA, DAPREADY, DAPSLVERR,
   BUSREQ, BUSWRITE, BUSADDR, BUSABORT, BUSWDATA, BUSMODE, BUSMASK, BUSTRNCNT,
   DAPSEL, DAPENABLE, DAPWRITE, DAPABORT, DAPADDR, DAPWDATA,
   BUSACK, BUSRDATA, BUSCMP, BUSERROR, BUSENDCNT
  );

  parameter DP_PRESENT    = 1;

  input         DAPRESETn;  // Debug APB Reset
  input         DAPCLK;     // Debug APB Clock
  input         DAPCLKEN;   // Debug APB Clock Enable
  input  [31:0] DAPRDATA;   // DAP Read bus
  input         DAPREADY;   // Ready signal for DAP slaves
  input         DAPSLVERR;  // Error signal for DAP slaves

  input         BUSREQ;     // Bus Transaction Request
  input         BUSWRITE;   // Bus Write
  input  [31:0] BUSADDR;    // Bus Address
  input         BUSABORT;   // Bus Abort
  input  [31:0] BUSWDATA;   // Bus Write Data Port
  input   [1:0] BUSMODE;    // Bus Mode
  input   [3:0] BUSMASK;    // Bus Compare Mask
  input  [11:0] BUSTRNCNT;  // Bus Transfer Cnt

  output        DAPSEL;     // DAP transaction select
  output        DAPENABLE;  // DAP transaction enable
  output        DAPWRITE;   // APB transaction direction
  output        DAPABORT;   // DAP transaction abort
  output [31:0] DAPADDR;    // APB Address
  output [31:0] DAPWDATA;   // APB write bus

  output        BUSACK;     // Bus Trasnaction Acknowledge
  output [31:0] BUSRDATA;   // Bus Read Data Port
  output        BUSCMP;     // Bus Pushed Verify/Find
  output        BUSERROR;   // Bus Error
  output [11:0] BUSENDCNT;  // Bus endCount

  wire          DAPRESETn;  // Debug APB Reset
  wire          DAPCLK;     // Debug APB Clock
  wire          DAPCLKEN;   // Debug APB Clock Enable
  wire   [31:0] DAPRDATA;   // DAP Read bus
  wire          DAPREADY;   // Ready signal for DAP slaves
  wire          DAPSLVERR;  // Error signal for DAP slaves
  wire          BUSREQ;     // Bus Transaction Request
  wire          BUSWRITE;   // Bus Write
  wire   [31:0] BUSADDR;    // Bus Address
  wire          BUSABORT;   // Bus Abort
  wire   [31:0] BUSWDATA;   // Bus Write Data Port
  wire    [1:0] BUSMODE;    // Bus Mode
  wire    [3:0] BUSMASK;    // Bus Compare Mask
  wire   [11:0] BUSTRNCNT;  // Bus Transfer Cnt
  wire          DAPSEL;     // DAP transaction select
  wire          DAPENABLE;  // DAP transaction enable
  wire          DAPWRITE;   // APB transaction direction
  wire          DAPABORT;   // DAP transaction abort
  wire   [31:0] DAPADDR;    // APB Address
  wire   [31:0] DAPWDATA;   // APB write bus
  wire          BUSACK;     // Bus Trasnaction Acknowledge
  wire   [31:0] BUSRDATA;   // Bus Read Data Port
  wire          BUSCMP;     // Bus Pushed Verify/Find
  wire          BUSERROR;   // Bus Error
  wire   [11:0] BUSENDCNT;  // Bus endCount

  reg     [1:0] APBcurr;    // APB state machine current
  reg     [1:0] APBnext;    // APB state machine next
  reg           APBnextEn;  // APB state machine next enable
  wire           Busacki;    // bus acknowledge
  reg           Busacki_cdc_check;    // bus acknowledge
  reg           BusackDi;   // bus acknowledge DFF input
  reg           BusackEni;  // bus acknowledge DFF enable
  wire           Buscmpi;    // bus compare operation
  reg           Buscmpi_cdc_check;    // bus compare operation
  reg           BuscmpDi;   // bus compare DFF input
  reg           BuscmpEni;  // bus compare DFF enable
  wire           Buserrori;  // bus error operation
  reg           Buserrori_cdc_check;  // bus error operation
  wire          BuserrorDi; // bus error DFF input
  wire          BuserrorEni;// bus error DFF enable
  wire    [31:0] Rdbuff;     // APB read buffer
  reg    [31:0] Rdbuff_cdc_check;     // APB read buffer
  wire          RdbuffEn;   // APB read buffer enable
  wire    [3:0] Cmplane;    // compare byte lanes
  reg           Dapwritei;  // write control for normal operations
  wire    [11:0] Buscnt;     // transaction counter
  reg    [11:0] Buscnt_cdc_check;     // transaction counter
  reg    [11:0] BuscntD;    // transaction counter DFF input
  reg           BuscntEn;   // transaction counter DFF enable
  wire          Compare;    // compare status

  wire          iDAPSEL;
  wire          iDAPENABLE;
  wire          iDAPWRITE;
  wire          iDAPABORT;
  wire [31:0]   iDAPADDR;
  wire [31:0]   iDAPWDATA;
  wire          iBUSACK;
  wire [31:0]   iBUSRDATA;
  wire          iBUSCMP;
  wire          iBUSERROR;
  wire [11:0]   iBUSENDCNT;
  wire          MstDPEn;

  assign DAPSEL    = (DP_PRESENT != 0) ? iDAPSEL    : 1'b0;
  assign DAPENABLE = (DP_PRESENT != 0) ? iDAPENABLE : 1'b0;
  assign DAPWRITE  = (DP_PRESENT != 0) ? iDAPWRITE  : 1'b0;
  assign DAPABORT  = (DP_PRESENT != 0) ? iDAPABORT  : 1'b0;
  assign DAPADDR   = (DP_PRESENT != 0) ? iDAPADDR   : {32{1'b0}};
  assign DAPWDATA  = (DP_PRESENT != 0) ? iDAPWDATA  : {32{1'b0}};
  assign BUSACK    = (DP_PRESENT != 0) ? iBUSACK    : 1'b0;
  assign BUSRDATA  = (DP_PRESENT != 0) ? iBUSRDATA  : {32{1'b0}};
  assign BUSCMP    = (DP_PRESENT != 0) ? iBUSCMP    : 1'b0;
  assign BUSERROR  = (DP_PRESENT != 0) ? iBUSERROR  : 1'b0;
  assign BUSENDCNT = (DP_PRESENT != 0) ? iBUSENDCNT : {12{1'b0}};
  assign MstDPEn   = (DP_PRESENT != 0) ? 1'b1       : 1'b0;


  always @ (DAPCLKEN or APBcurr or BUSREQ or DAPREADY or Compare or
            Buscnt or BUSMODE or DAPSLVERR or DAPABORT)
    case (APBcurr)
      `DAP_APBIDLE   : begin
        APBnext   = `DAP_APBSETUP;
        APBnextEn = DAPCLKEN & BUSREQ;
      end
      `DAP_APBSETUP  : begin
        APBnext   = `DAP_APBENABLE;
        APBnextEn = DAPCLKEN;
      end
      `DAP_APBENABLE : begin
        if (DAPREADY)
          begin
            APBnextEn = DAPCLKEN;
            if (|Buscnt)
              if ((BUSMODE==`DAP_M_FIND & Compare) | (BUSMODE==`DAP_M_VERIFY &
                                                  !Compare))
                APBnext = `DAP_APBEND;
              else if (DAPSLVERR | DAPABORT)
                APBnext = `DAP_APBEND;
              else
                APBnext = `DAP_APBSETUP;
            else
                APBnext = `DAP_APBEND;
          end
        else
          begin
            APBnext   = 2'b00;
            APBnextEn = 1'b0;
          end
      end
      `DAP_APBEND : begin
        APBnext   = `DAP_APBIDLE;
        APBnextEn = DAPCLKEN & ~BUSREQ;
      end
      default : begin
        APBnext   = 2'bxx;
        APBnextEn = 1'bx;
      end
    endcase

  always @ (negedge DAPRESETn or posedge DAPCLK)
    if (!DAPRESETn)
      APBcurr <= `DAP_APBIDLE;
    else if (APBnextEn & MstDPEn)
      APBcurr <= APBnext;

  assign RdbuffEn = DAPCLKEN & APBcurr==`DAP_APBENABLE & DAPREADY & ~DAPWRITE;

  always @ (negedge DAPRESETn or posedge DAPCLK)
    if (!DAPRESETn)
       Rdbuff_cdc_check  <= {32{1'b0}};
    else if (RdbuffEn & MstDPEn)
       Rdbuff_cdc_check  <= DAPRDATA;

  always @ (BUSMODE or BUSWRITE)
    case (BUSMODE)
      `DAP_M_VERIFY,`DAP_M_FIND : Dapwritei = 1'b0;
      `DAP_M_NORMAL,`DAP_M_RSVD : Dapwritei = BUSWRITE;
      default           : Dapwritei = 1'bx;
    endcase

  assign Cmplane[0] = (BUSWDATA[7:0]  ==DAPRDATA[7:0])   ? BUSMASK[0] : 1'b0;
  assign Cmplane[1] = (BUSWDATA[15:8] ==DAPRDATA[15:8])  ? BUSMASK[1] : 1'b0;
  assign Cmplane[2] = (BUSWDATA[23:16]==DAPRDATA[23:16]) ? BUSMASK[2] : 1'b0;
  assign Cmplane[3] = (BUSWDATA[31:24]==DAPRDATA[31:24]) ? BUSMASK[3] : 1'b0;
  assign Compare    = Cmplane==BUSMASK;

  always @ (DAPCLKEN or APBcurr or Buscnt or DAPREADY or BUSREQ or BUSTRNCNT or
            BUSABORT or BUSMODE or Compare)
    begin
      BuscntD  = {12{1'b0}};
      BuscntEn = 1'b0;
      if (APBcurr==`DAP_APBIDLE & BUSREQ)
        begin
          BuscntD  = BUSTRNCNT;
          BuscntEn = DAPCLKEN;
        end
      else if (APBcurr==`DAP_APBENABLE & DAPREADY)
        begin
          if (BUSABORT)
            begin
              BuscntD  = {12{1'b0}};
              BuscntEn = DAPCLKEN;
            end
          else if (|Buscnt & ~((BUSMODE==`DAP_M_FIND & Compare) |
                          (BUSMODE==`DAP_M_VERIFY & !Compare)))
            begin
              BuscntD  = (Buscnt - 1);
              BuscntEn = DAPCLKEN;
            end
        end
    end

  always @ (negedge DAPRESETn or posedge DAPCLK)
    if (!DAPRESETn)
       Buscnt_cdc_check  <= {12{1'b0}};
    else if (BuscntEn & MstDPEn)
       Buscnt_cdc_check  <= BuscntD;

  always @ (DAPCLKEN or APBcurr or BUSREQ)
    case (APBcurr)
      `DAP_APBEND : begin
        BusackDi  = BUSREQ;
        BusackEni = DAPCLKEN;
      end
      `DAP_APBIDLE,
      `DAP_APBSETUP,
      `DAP_APBENABLE : begin
        BusackDi  = 1'b0;
        BusackEni = DAPCLKEN;
      end
      default: begin
        BusackDi  = 1'bx;
        BusackEni = 1'bx;
      end
    endcase

  always @ (negedge DAPRESETn or posedge DAPCLK)
    if (!DAPRESETn)
       Busacki_cdc_check  <= 1'b0;
    else if (BusackEni & MstDPEn)
       Busacki_cdc_check  <= BusackDi;

  always @ (DAPCLKEN or APBcurr or DAPREADY or BUSMODE or Compare or Buscmpi)
    case (BUSMODE)
      `DAP_M_VERIFY,`DAP_M_FIND: begin
        BuscmpDi  = Compare;
        BuscmpEni = DAPCLKEN & APBcurr==`DAP_APBENABLE & DAPREADY &
                    (Buscmpi ^ Compare);
      end
      `DAP_M_NORMAL,`DAP_M_RSVD: begin
        BuscmpDi  = 1'b0;
        BuscmpEni = DAPCLKEN & Buscmpi;
      end
      default: begin
        BuscmpDi  = 1'bx;
        BuscmpEni = 1'bx;
      end
    endcase


  always @ (negedge DAPRESETn or posedge DAPCLK)
    if (!DAPRESETn)
       Buscmpi_cdc_check  <= 1'b0;
    else if (BuscmpEni & MstDPEn)
       Buscmpi_cdc_check  <= BuscmpDi;

  assign BuserrorDi  = DAPSLVERR;
  assign BuserrorEni = DAPCLKEN & APBcurr==`DAP_APBENABLE & DAPREADY &
                       (Buserrori ^ DAPSLVERR);

  always @ (negedge DAPRESETn or posedge DAPCLK)
    if (!DAPRESETn)
       Buserrori_cdc_check  <= 1'b0;
    else if (BuserrorEni & MstDPEn)
       Buserrori_cdc_check  <= BuserrorDi;

  assign iBUSRDATA  = Rdbuff;
  assign iBUSCMP    = Buscmpi;
  assign iBUSERROR  = Buserrori;
  assign iBUSACK    = Busacki;
  assign iBUSENDCNT = Buscnt;

  assign iDAPSEL    = APBcurr==`DAP_APBENABLE | APBcurr==`DAP_APBSETUP;
  assign iDAPENABLE = APBcurr==`DAP_APBENABLE;
  assign iDAPWRITE  = (APBcurr==`DAP_APBENABLE | APBcurr==`DAP_APBSETUP) &
                     Dapwritei;
  assign iDAPABORT  = BUSABORT & APBcurr==`DAP_APBENABLE;
  assign iDAPADDR   = (APBcurr==`DAP_APBENABLE | APBcurr==`DAP_APBSETUP) ?
                      BUSADDR   : {32{1'b0}};
  assign iDAPWDATA  = (APBcurr==`DAP_APBENABLE | APBcurr==`DAP_APBSETUP) ?
                      BUSWDATA  : {32{1'b0}};


`ifdef ARM_ASSERT_ON

  wire hs_start;
  wire hs_end;

  assign hs_start = BUSREQ & APBcurr==`DAP_APBIDLE;
  assign hs_end   = BUSREQ & APBcurr==`DAP_APBEND & !Busacki;

  `ifdef ARM_CDC_ON
  `else
  assert_never #(`OVL_ERROR, `OVL_ASSERT, "BUSABORT must only be asserted with BUSREQ.")
    dapapb_busabort(.clk       (DAPCLK),
                    .reset_n   (DAPRESETn),
                    .test_expr (BUSABORT & ~BUSREQ));
  `endif

  assert_handshake #(`OVL_ERROR, 0, 0, 0, 0, 0, `OVL_ASSERT, "DAPAPB handshake.")
    dapapb_handshake(.clk      (DAPCLK),
                    .reset_n   (DAPRESETn),
                     .req      (BUSREQ),
                     .ack      (Busacki));

  assert_win_unchange #(`OVL_ERROR, 1, `OVL_ASSERT, "BUSWRITE must remain static during handshake.")
    dapapb_hs_buswrite(
                       .clk         (DAPCLK),
                       .reset_n     (DAPRESETn),
                       .start_event (hs_start),
                       .test_expr   (BUSWRITE),
                       .end_event  (hs_end));

  assert_win_unchange #(`OVL_ERROR, 32, `OVL_ASSERT, "BUSADDR must remain static during handshake.")
  dapapb_hs_busaddr(
                    .clk         (DAPCLK),
                    .reset_n     (DAPRESETn),
                    .start_event (hs_start),
                    .test_expr   (BUSADDR),
                    .end_event   (hs_end));

  assert_win_unchange #(`OVL_ERROR, 32, `OVL_ASSERT, "BUSWDATA must remain static during handshake.")
    dapapb_hs_buswdata(.clk      (DAPCLK),
                    .reset_n     (DAPRESETn),
                    .start_event (hs_start),
                    .test_expr   (BUSWDATA),
                    .end_event   (hs_end));

  assert_win_unchange #(`OVL_ERROR, 2, `OVL_ASSERT, "BUSMODE must remain static during handshake.")
    dapapb_hs_busmode(.clk         (DAPCLK),
                      .reset_n     (DAPRESETn),
                      .start_event ( hs_start),
                      .test_expr   (BUSMODE),
                      .end_event   (hs_end));

  assert_win_unchange #(`OVL_ERROR, 4, `OVL_ASSERT, "BUSMASK must remain static during handshake.")
  dapapb_hs_busmask(.clk           (DAPCLK),
                      .reset_n     (DAPRESETn),
                      .start_event (hs_start),
                      .test_expr   (BUSMASK),
                      .end_event   (hs_end));

  assert_win_unchange #(`OVL_ERROR, 12, `OVL_ASSERT, "BUSTRNCNT must remain static during handshake.")
  dapapb_hs_bustrncnt(.clk         (DAPCLK),
                      .reset_n     (DAPRESETn),
                      .start_event (hs_start),
                      .test_expr   (BUSTRNCNT),
                      .end_event   (hs_end));

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "Caught BUSREQ X propogation")
    dapapb_x_busreq(
                    .clk       (DAPCLK),
                    .reset_n   (DAPRESETn),
                    .qualifier (1'b1),
                    .test_expr (BUSREQ));

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "Caught DAPREADY X propogation")
    dapapb_x_dapready(
                    .clk       (DAPCLK),
                    .reset_n   (DAPRESETn),
                    .qualifier (1'b1),
                    .test_expr (DAPREADY));

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "Caught BUSABORT X propogation")
    dapapb_x_dapabort(
                      .clk       (DAPCLK),
                      .reset_n   (DAPRESETn),
                      .qualifier (1'b1),
                      .test_expr (BUSABORT));

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "Caught DAPABORT X propogation")
    dapapb_x_DAPABORT(
                      .clk       (DAPCLK),
                      .reset_n   (DAPRESETn),
                      .qualifier (1'b1),
                      .test_expr (DAPABORT));

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "Caught RdbuffEn X propogation")
    dapapb_x_RdbuffEn (
                      .clk       (DAPCLK),
                      .reset_n   (DAPRESETn),
                      .qualifier (1'b1),
                      .test_expr (RdbuffEn));

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "Caught BuscmpEni X propogation")
    dapapb_x_BuscmpEni(
                      .clk       (DAPCLK),
                      .reset_n   (DAPRESETn),
                      .qualifier (1'b1),
                      .test_expr (BuscmpEni));

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "CaughtBuserrorEni  X propogation")
    dapapb_x_BuserrorEni(
                      .clk       (DAPCLK),
                      .reset_n   (DAPRESETn),
                      .qualifier (1'b1),
                      .test_expr (BuserrorEni));

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "Caught DAPSLVERR X propogation")
    dapapb_x_DAPSLVERR(
                      .clk       (DAPCLK),
                      .reset_n   (DAPRESETn),
                      .qualifier (1'b1),
                      .test_expr (DAPSLVERR));

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "Caught compare X propogation")
  dapapb_x_compare(
                   .clk       (DAPCLK),
                   .reset_n   (DAPRESETn),
                   .qualifier (APBcurr==`DAP_APBENABLE&
                               ((BUSMODE==`DAP_M_VERIFY)|
                                (BUSMODE==`DAP_M_FIND))),
                   .test_expr (Compare));

  assert_never_unknown #(`OVL_ERROR, 2, `OVL_ASSERT, "Caught APBcurr X propogation")
    dapapb_x_APBcurr(
                    .clk       (DAPCLK),
                    .reset_n   (DAPRESETn),
                    .qualifier (1'b1),
                    .test_expr (APBcurr));

  assert_never_unknown #(`OVL_ERROR, 12, `OVL_ASSERT, "Caught Buscnt X propogation")
    dapapb_x_Buscnt(
                    .clk       (DAPCLK),
                    .reset_n   (DAPRESETn),
                    .qualifier (APBcurr==`DAP_APBENABLE),
                    .test_expr (Buscnt));

  assert_never_unknown #(`OVL_ERROR, 2, `OVL_ASSERT, "Caught BUSMODE X propogation")
    dapapb_x_busmode(
                    .clk       (DAPCLK),
                    .reset_n   (DAPRESETn),
                    .qualifier ( APBcurr==`DAP_APBENABLE |
                                 APBcurr==`DAP_APBSETUP),
                    .test_expr ( BUSMODE));

`endif

`ifdef ARM_CDC_ON
        cdc_launch uxcdc_Busacki(
                .clk_launch(DAPCLK),
                .clk_capture(DAPSWJDP.uDAPSwDpSync.uSyncBusAck.SYNCCLK),
                .d_in(Busacki_cdc_check),
                .d_out(Busacki)
                );
`else
        assign Busacki = Busacki_cdc_check;
`endif
`ifdef ARM_CDC_ON
        cdc_launch #(11,0) uxcdc_Buscnt(
                .clk_launch(DAPCLK),
                .clk_capture(DAPSWJDP.uDAPJtagDpProtocol.TCK),
                .d_in(Buscnt_cdc_check),
                .d_out(Buscnt)
                );
`else
        assign Buscnt = Buscnt_cdc_check;
`endif
`ifdef ARM_CDC_ON
        cdc_launch #(31,0) uxcdc_Rdbuff(
                .clk_launch(DAPCLK),
                .clk_capture(DAPSWJDP.uDAPSwDpProtocol.DBGCLK),
                .d_in(Rdbuff_cdc_check),
                .d_out(Rdbuff)
                );
`else
        assign Rdbuff = Rdbuff_cdc_check;
`endif
`ifdef ARM_CDC_ON
        cdc_launch uxcdc_Buscmpi(
                .clk_launch(DAPCLK),
                .clk_capture(DAPSWJDP.uDAPJtagDpProtocol.TCK),
                .d_in(Buscmpi_cdc_check),
                .d_out(Buscmpi)
                );
`else
        assign Buscmpi = Buscmpi_cdc_check;
`endif
`ifdef ARM_CDC_ON
        cdc_launch uxcdc_Buserrori(
                .clk_launch(DAPCLK),
                .clk_capture(DAPSWJDP.uDAPJtagDpProtocol.TCK),
                .d_in(Buserrori_cdc_check),
                .d_out(Buserrori)
                );
`else
        assign Buserrori = Buserrori_cdc_check;
`endif
endmodule
