`define DAP_APBIDLE     2'b00
`define DAP_APBSETUP    2'b01
`define DAP_APBENABLE   2'b11
`define DAP_APBEND      2'b10

`define DAP_M_NORMAL    2'b00
`define DAP_M_VERIFY    2'b01
`define DAP_M_FIND      2'b10
`define DAP_M_RSVD      2'b11


`define SWWM_IDLE    2'b00
`define SWWM_RESET   2'b01
`define SWWM_W4TRAIN 2'b10
`define SWWM_TRAIN   2'b11

`define SWDP_SLERESET        5'b00000
`define SWDP_SLEIDLE         5'b00001
`define SWDP_SLEBANK         5'b00010
`define SWDP_SLEDIR          5'b00011
`define SWDP_SLEADDR0        5'b00100
`define SWDP_SLEADDR1        5'b00101
`define SWDP_SLEPARITYH      5'b00110
`define SWDP_SLESTOPH        5'b00111
`define SWDP_SLEPARKH        5'b01000
`define SWDP_SLETRNH2        5'b01010
`define SWDP_SLETRNH1        5'b01011
`define SWDP_SLETRNH0        5'b01100
`define SWDP_UNUSED01        5'b01101
`define SWDP_SLEACK0         5'b10000
`define SWDP_SLEACK1         5'b10001
`define SWDP_SLEACK2         5'b10010
`define SWDP_SLEPARKW        5'b10011
`define SWDP_SLETRNW3        5'b10100
`define SWDP_SLETRNW2        5'b10101
`define SWDP_SLETRNW1        5'b10110
`define SWDP_SLETRNW0        5'b10111
`define SWDP_SLERESW         5'b11000
`define SWDP_SLEDATAW        5'b11001
`define SWDP_SLEPARITYW      5'b11010
`define SWDP_SLEMARKW        5'b11011
`define SWDP_SLESTOPW        5'b11100
`define SWDP_SLEDATAR        5'b11101
`define SWDP_SLEPARITYR      5'b11110
`define SWDP_SLESTOPR        5'b11111
`define SWDP_SLETRNC         5'b01110
`define SWDP_SLERESC         5'b01111
`define SWDP_SLEERROR        5'b01001

`define SWDP_SLEUNUSED `SWDP_SLEMARKW,`SWDP_SLESTOPR,`SWDP_SLERESW, `SWDP_UNUSED01
`define SWDP_SLEXXXX         5'bXXXXX
`define SWDP_ABORT           2'b00   // Wr
`define SWDP_IDCODE          2'b00   // Rd
`define SWDP_CTRL            2'b01   // Wr
`define SWDP_STAT            2'b01   // Rd
`define SWDP_ADDR            2'b10   // Wr
`define SWDP_RESEND          2'b10   // Rd
`define SWDP_RDBUFF          2'b11   // Rd

`define SWDP_STARTBIT        1'b1
`define SWDP_SERREAD         1'b1
`define SWDP_SERWRITE        1'b0
`define SWDP_APBREAD         1'b0
`define SWDP_APBWRITE        1'b1
`define SWDP_DPREG           1'b0
`define SWDP_APREG           1'b1

`define SWDP_CNT32           5'b11111

`define SWDP_ZERO12          12'b000000000000


module DAPSwDpProtocol
  (
   DBGRESETn, DBGCLK, DBGDI,
   BUSACK, BUSRDATA, BUSCMP, BUSERROR, BUSENDCNT,
   CDBGPWRUPACK, CSYSPWRUPACK, CDBGRSTACK,
   DBGDO, DBGDOEN, BUSREQ, BUSWRITE, BUSABORT, BUSADDR,
   BUSWDATA, BUSMODE, BUSMASK, BUSTRNCNT,
   CDBGPWRUPREQ, CSYSPWRUPREQ, CDBGRSTREQ, WIRERESET
  );

  parameter DP_PRESENT     = 1;
  parameter RESET_ALL_REGS = 0;


  input         DBGRESETn;  // Power-On Reset
  input         DBGCLK;     // SW-DP Clock
  input         DBGDI;      // SW-DP Data In
  input         BUSACK;     // Bus Trasnaction Acknowledge
  input  [31:0] BUSRDATA;   // Bus Read Data Port
  input         BUSCMP;     // Bus Pushed Verify/Find
  input         BUSERROR;   // Bus Error
  input  [11:0] BUSENDCNT;  // Bus end count
  input         CDBGPWRUPACK;  // Debug Power Domain up acknowledge
  input         CSYSPWRUPACK;  // System Power Domain up acknowledge
  input         CDBGRSTACK;    // Debug reset acknowledge to reset controller
  output        DBGDO;      // SW-DP Data Out
  output        DBGDOEN;    // SW-DP Data Out Enable
  output        BUSREQ;     // Bus Transaction Request
  output        BUSWRITE;   // Bus Write
  output        BUSABORT;   // Bus Abort
  output [31:0] BUSADDR;    // Bus Address Port
  output [31:0] BUSWDATA;   // Bus Write Data Port
  output  [1:0] BUSMODE;    // Bus Mode
  output  [3:0] BUSMASK;    // Bus Compare Mask
  output [11:0] BUSTRNCNT;  // Bus Transfer Cnt
  output        CDBGPWRUPREQ;  // Debug Power Domain up request
  output        CSYSPWRUPREQ;  // System Power Domain up request
  output        CDBGRSTREQ;    // Debug reset request to reset controller
  output        WIRERESET;     // JTAG-DP is in it's standby state.


  wire          DBGRESETn;  // Power-On Reset
  wire          DBGCLK;     // SW-DP Clock
  wire          DbgClkEn;   // SW-DP Clock Enable
  wire          DBGDI;      // SW-DP Data In
  wire          BUSACK;     // Bus Trasnaction Acknowledge
  wire   [31:0] BUSRDATA;   // Bus RD Data Port
  wire          BUSCMP;     // Bus Pushed Verify/Find
  wire          BUSERROR;   // Bus Error
  wire   [11:0] BUSENDCNT;  // Bus end count
  wire          CDBGPWRUPACK;  // Debug Power Domain up acknowledge
  wire          CSYSPWRUPACK;  // System Power Domain up acknowledge
  wire          CDBGRSTACK;    // Debug reset acknowledge to reset controller
  reg           DBGDOReg;      // SW-DP Data Out
  wire          DBGDOEN;    // SW-DP Data Out Enable
  wire          BUSREQ;     // Bus Transaction Request
  wire          BUSWRITE;   // Bus Write
  wire          BUSABORT;   // Bus Abort
  wire   [31:0] BUSADDR;    // Bus Address Port
  wire   [31:0] BUSWDATA;   // Bus Write Data Port
  wire    [1:0] BUSMODE;    // Bus Mode
  wire    [3:0] BUSMASK;    // Bus Compare Mask
  wire   [11:0] BUSTRNCNT;  // Bus Transfer Cnt
  reg           CDBGPWRUPREQReg;  // Debug Power Domain up request
  reg           CSYSPWRUPREQReg;  // System Power Domain up request
  reg           CDBGRSTREQReg;    // Debug reset request to reset controller

  reg     [4:0] State;      // SLE current state
  reg     [4:0] NxtState;   // SLE next state
  wire    [4:0] RstState;   // SLE next state with wire reset
  reg           SerDir;     // DP serial direction
  wire          SerDirEn;   // DP serial direction enable
  reg           SerBank;    // DP serial bank
  wire          SerBankEn;  // DP serial bank enable
  reg     [1:0] SerAddr;    // DP serial address
  wire    [1:0] SerAddrD;   // DP serial data input
  wire          SerAddrEn;  // DP serial address enable
  reg     [4:0] SerCnt;     // DP serial count, 32-bits
  wire    [4:0] SerCntD;    // DP serial count DFF input
  wire          SerCntEn;   // DP serial count DFF enable
  reg           WBufReq;    // write buffer request
  reg           WBufReqD;   // write buffer request DFF input
  reg           WBufReqEn;  // write buffer request DFF enable
  reg           WBufAck;    // write buffer acknowledge
  reg           WBufAckD;   // write buffer acknowledge DFF input
  reg           WBufAckEn;  // write buffer acknowledge DFF enable
  wire          WBufCancel; // write buffer cancel
  reg           IfaceRdyC;   // interface ready capture DFF_Q
  wire    [4:0] IfaceRdyCD;  // interface ready capture (~nCONT)
  wire          IfaceRdyCEn; // interface ready capture DFF_En
  reg           IfaceRdyAc;  // interface ready acknowledged DFF_Q
  wire          AcStateEn;   // IfaceRdyAc & RespFaultReg En

  reg    [31:0] ShiftReg;   // shift register
  reg    [31:0] ShiftRegD;  // shift register DFF input
  reg           ShiftRegEn; // shift register DFF enable
  reg           iBusReq;    // bus request
  reg           iBusReqD;   // bus request DFF input
  reg           iBusReqEn;  // bus request DFF enable
  wire   [31:0] iBusWdata;  // bus write data register
  reg    [31:0] iBusWdata_cdc_check;  // bus write data register
  wire          iBusWdataEn;// bus write data DFF enable
  reg  [31:0]   RetryReg;   // Bus read data shadow
  wire [31:0]   RetryRegD;  // Bus read data shadow input
  reg           RetryRegEn; // Bus read data shadow enable

  reg           Parity;     // Parity bit for read/write
  wire          ParityD;    // Parity DFF.D
  wire          ParityEn;   // Parity DFF.En
  reg           ErrorChk;   // Error Check
  wire          ErrorChkD;  // Error Check DFF.D
  wire          ErrorChkEn; // Error Check DFF.En
  reg           ErrorTrack;    // Error Track
  wire          ErrorTrackD;   // Error Track DFF.D
  wire          ErrorTrackEn;  // Error Track DFF.En
  reg           WriteErrReg;   // Write Error DFF
  reg           WriteErrD;     // Write Error DFF.D
  reg           WriteErrEn;    // Write Error DFF.En
  wire          PreventWrite;  // Error before write completes

  reg           ReadOKReg;     // Previous read was accepted
  reg           ReadOKD;       // ReadOK Dff.D
  reg           ReadOKEn;      // ReadOK Dff.En
  reg           NeedIDReadReg; // ID register read must be next
  wire          NeedIDReadEn;  // NeedIDReadReg Dff.En
  wire          NeedIDReadD;   // NeedIDReadReg Dff.D


  wire          RespFaultD;    // Resp is fault Dff.D
  reg           RespFaultReg;  // Resp is fault

  wire          CtrlStatEn;   // control register enable
  wire          StickyStatus; // sticky status
  reg           StickyCmp;    // sticky compare status
  reg           StickyCmpD;   // sticky compare DFF input
  reg           StickyCmpEn;  // sticky compare DFF enable
  reg           StickyErr;    // sticky error register
  reg           StickyErrD;   // sticky error DFF input
  reg           StickyErrEn;  // sticky error DFF enable
  wire     [3:0] MaskLane;     // compare operation byte lane masks
  reg     [3:0] MaskLane_cdc_check;     // compare operation byte lane masks
  wire     [1:0] TrnferMode;   // transfer mode
  reg     [1:0] TrnferMode_cdc_check;   // transfer mode
  wire    [11:0] TrnferCnt;    // transfer counter
  reg    [11:0] TrnferCnt_cdc_check;    // transfer counter
  reg    [11:0] TrnferCntD;   // transfer counter DFF input
  reg           TrnferCntEn;  // transfer counter DFF enable
  reg           ContDetectReg;// fixed packet length mode
  reg           ContErrReg;   // wait was sent
  reg           ContErrD;     // D input for ContErrReg
  reg           ContErrEn;    // enable for ContErrReg

  wire          WireCtrlEn;   // Wire Control Enable
  reg     [1:0] PrescalerReg; // DBGCLK prescaler
  wire    [1:0] PrescalerD;   // DBGCLK prescaler DFF.D
  reg     [1:0] TurnCnt;      // Turn Around Counter
  wire    [1:0] TurnCntD;     // Turn Around Counter DFF.D
  reg     [3:0] ApBankSel;    // AP bank select
  wire           ApDir;        // AP direction
  reg           ApDir_cdc_check;        // AP direction
  wire          ApDirD;       // AP direction DFF input
  reg     [1:0] ApRegSel;     // AP register select
  wire    [1:0] ApRegSelD;    // AP register select DFF input
  wire          ApRegSelEn;   // AP register select DFF enable
  reg     [7:0] ApSel;        // AP select
  wire          ApSelEn;      // AP select enable
  reg           CtrlSel;      // address bank select
  reg           DapDir;       // DAP APB direction
  wire          DapDirEn;     // DAP APB direction DFF input
  reg           DapBank;      // DAP APB direction DFF enable
  wire          DapBankEn;    // DAP APB bank
  reg     [1:0] DapRegSel;    // DAP AP bank register select
  reg           DapAbort;     // DAP AP abort
  reg           DapAbortD;    // DAP AP abort DFF input
  reg           DapAbortEn;   // DAP AP abort DFF enable
  reg           NxtDbgDo;     // Next DBGDO
  reg           NxtDbgDoEn;   // Next DBGDOEN

  wire          iWireReset;    // From (optional) wire interface
  wire [5:0]    ResetCountD;  // Line reset DFF input
  reg  [5:0]    ResetCountReg;// Line Reset counter
  wire          ResetCountEn; // Line Reset DFF Enabled


  wire          iDBGDO;
  reg           iDBGDOEN;
  wire          iBUSWRITE;
  wire          iBUSABORT;
  wire [31:0]   iBUSADDR;
  wire [1:0]    iBUSMODE;
  wire [3:0]    iBUSMASK;
  wire [11:0]   iBUSTRNCNT;
  wire          iCDBGPWRUPREQ;
  wire          iCSYSPWRUPREQ;
  wire          iCDBGRSTREQ;
  wire          MstDPEn;
  wire          FullDBGRESETn;

  assign FullDBGRESETn = (RESET_ALL_REGS == 1) ? DBGRESETn : 1'b1;

  assign DBGDO        = (DP_PRESENT != 0) ? iDBGDO        : 1'b0;
  assign DBGDOEN      = (DP_PRESENT != 0) ? iDBGDOEN      : 1'b0;
  assign BUSREQ       = (DP_PRESENT != 0) ? iBusReqD       : 1'b0;
  assign BUSWRITE     = (DP_PRESENT != 0) ? iBUSWRITE     : 1'b0;
  assign BUSABORT     = (DP_PRESENT != 0) ? iBUSABORT     : 1'b0;
  assign BUSADDR      = (DP_PRESENT != 0) ? iBUSADDR      : {32{1'b0}};
  assign BUSWDATA     = (DP_PRESENT != 0) ? iBusWdata     : {32{1'b0}};
  assign BUSMODE      = (DP_PRESENT != 0) ? iBUSMODE      : 2'b00;
  assign BUSMASK      = (DP_PRESENT != 0) ? iBUSMASK      : 4'b0000;
  assign BUSTRNCNT    = (DP_PRESENT != 0) ? iBUSTRNCNT    : {12{1'b0}};
  assign CDBGPWRUPREQ = (DP_PRESENT != 0) ? iCDBGPWRUPREQ : 1'b0;
  assign CSYSPWRUPREQ = (DP_PRESENT != 0) ? iCSYSPWRUPREQ : 1'b0;
  assign CDBGRSTREQ   = (DP_PRESENT != 0) ? iCDBGRSTREQ   : 1'b0;
  assign WIRERESET    = (DP_PRESENT != 0) ? iWireReset    : 1'b0;
  assign MstDPEn      = (DP_PRESENT != 0) ? 1'b1          : 1'b0;



  assign DbgClkEn  = ~iWireReset | ~ResetCountReg[0];

  assign ResetCountEn = DBGDI & ~DBGDOEN ?
                        (~iWireReset|~ResetCountReg[0]) : (|ResetCountReg);
  assign ResetCountD  = DBGDI & ~DBGDOEN ? (ResetCountReg+6'd1) : {6{1'b0}};

  always @(posedge DBGCLK or negedge DBGRESETn)
    if (~DBGRESETn)
      ResetCountReg <= {6{1'b0}};
    else if (ResetCountEn & MstDPEn)
      ResetCountReg <= ResetCountD;

  assign iWireReset = ResetCountReg[5:1] == 5'd25;

  always @ (State or DBGDI or IfaceRdyAc or SerDir or SerCnt or TurnCnt or
            ErrorChk or ErrorTrack or RespFaultReg or ContDetectReg)
    case (State)
      `SWDP_SLERESET     : NxtState = DBGDI ? `SWDP_SLERESET : `SWDP_SLEIDLE;
      `SWDP_SLEIDLE      : NxtState = DBGDI ? `SWDP_SLEBANK  : `SWDP_SLEIDLE;
      `SWDP_SLEBANK      : NxtState = `SWDP_SLEDIR;
      `SWDP_SLEDIR       : NxtState = `SWDP_SLEADDR0;
      `SWDP_SLEADDR0     : NxtState = `SWDP_SLEADDR1;
      `SWDP_SLEADDR1     : NxtState = `SWDP_SLEPARITYH;
      `SWDP_SLEPARITYH   : NxtState = `SWDP_SLESTOPH;
      `SWDP_SLESTOPH     : NxtState = `SWDP_SLEPARKH;
      `SWDP_SLEPARKH     : case (TurnCnt)
                        2'b11 : NxtState = `SWDP_SLETRNH2;
                        2'b10 : NxtState = `SWDP_SLETRNH1;
                        2'b01 : NxtState = `SWDP_SLETRNH0;
                        2'b00 : NxtState = `SWDP_SLEACK0;
                        default:NxtState = `SWDP_SLEXXXX;
                      endcase // case (TurnCnt)
      `SWDP_SLETRNH2     : NxtState = `SWDP_SLETRNH1;
      `SWDP_SLETRNH1     : NxtState = `SWDP_SLETRNH0;
      `SWDP_SLETRNH0     : NxtState = `SWDP_SLEACK0;
      `SWDP_SLEACK0      : NxtState = `SWDP_SLEACK1;
      `SWDP_SLEACK1      : NxtState = `SWDP_SLEACK2;
      `SWDP_SLEACK2      : NxtState = (ErrorChk | ~IfaceRdyAc | RespFaultReg|
                                       (SerDir==`SWDP_SERWRITE)) ?
                                      `SWDP_SLEPARKW :`SWDP_SLEDATAR;
      `SWDP_SLEPARKW     : case (TurnCnt)
                             2'b11 : NxtState = `SWDP_SLETRNW3;
                             2'b10 : NxtState = `SWDP_SLETRNW2;
                             2'b01 : NxtState = `SWDP_SLETRNW1;
                             2'b00 : NxtState = `SWDP_SLETRNW0;
                             default:NxtState = `SWDP_SLEXXXX;
                           endcase // case(TurnCnt)
      `SWDP_SLETRNW3     : NxtState = `SWDP_SLETRNW2;
      `SWDP_SLETRNW2     : NxtState = `SWDP_SLETRNW1;
      `SWDP_SLETRNW1     : NxtState = `SWDP_SLETRNW0;
      `SWDP_SLETRNW0     : NxtState = (((SerDir==`SWDP_SERREAD) &
                                        ~((ErrorChk | ~IfaceRdyAc |
                                           RespFaultReg)&ContDetectReg)) |
                                       ((SerDir==`SWDP_SERWRITE) &
                                        ((ErrorChk | ~IfaceRdyAc | RespFaultReg)
                                         &~ContDetectReg))) ?
                                      `SWDP_SLERESC:`SWDP_SLEDATAW;
      `SWDP_SLEDATAW     : NxtState = (SerCnt==`SWDP_CNT32) ?
                                      `SWDP_SLEPARITYW : `SWDP_SLEDATAW;
      `SWDP_SLEPARITYW   : NxtState = `SWDP_SLESTOPW;
      `SWDP_SLESTOPW     : NxtState = DBGDI ? `SWDP_SLEBANK : `SWDP_SLEIDLE;
      `SWDP_SLEDATAR     : NxtState = (SerCnt==`SWDP_CNT32) ?
                                      `SWDP_SLEPARITYR : `SWDP_SLEDATAR;
      `SWDP_SLEPARITYR   : NxtState = `SWDP_SLEPARKW;
      `SWDP_SLERESC      : NxtState = ErrorChk & ErrorTrack ?
                                      `SWDP_SLEERROR :
                                      (DBGDI?(ErrorChk ?
                                              `SWDP_SLERESC : `SWDP_SLEBANK)
                                       : `SWDP_SLEIDLE);
      `SWDP_SLEERROR     : NxtState = `SWDP_SLEERROR;
      `SWDP_SLEUNUSED : NxtState = `SWDP_SLEIDLE;
      default : NxtState = `SWDP_SLEXXXX;
    endcase

  assign RstState = iWireReset ? `SWDP_SLERESET : NxtState;

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      State <= `SWDP_SLERESET;
    else if (DbgClkEn & MstDPEn)
      State <= RstState;

  assign NeedIDReadEn = (SerDir==`SWDP_SERREAD & ~ErrorChk &
                         SerBank==`SWDP_DPREG & SerAddr==`SWDP_IDCODE) |
                          iWireReset;
  assign NeedIDReadD = iWireReset;

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      NeedIDReadReg <= 1'b1;
    else if (NeedIDReadEn & MstDPEn)
      NeedIDReadReg <= NeedIDReadD;

  assign RespFaultD = StickyStatus &
                       ~(SerBank==`SWDP_DPREG & (SerAddr==`SWDP_ABORT|
                                                 (SerDir==`SWDP_SERREAD &
                                                  CtrlSel == 1'b0 &
                                                  SerAddr==`SWDP_STAT)));

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      RespFaultReg <= 1'b0;
    else if (AcStateEn & MstDPEn)
      RespFaultReg <= RespFaultD;

  always @ (State or ErrorChk or IfaceRdyAc or Parity or
            RespFaultReg or ShiftReg)
    begin
      case (State)
        `SWDP_SLEIDLE,`SWDP_SLEBANK,`SWDP_SLEDIR,`SWDP_SLEADDR0,`SWDP_SLEADDR1,
        `SWDP_SLEPARITYH,`SWDP_SLESTOPH,
        `SWDP_SLEPARKH,`SWDP_SLETRNH2,`SWDP_SLETRNH1,`SWDP_SLETRNH0,
        `SWDP_SLETRNC,`SWDP_SLERESC,
        `SWDP_SLEDATAW,`SWDP_SLEPARITYW,`SWDP_SLESTOPW,`SWDP_SLEERROR,
        `SWDP_SLERESET : begin
          NxtDbgDo   = 1'b0;
          NxtDbgDoEn = 1'b0;
        end
        `SWDP_SLEACK0 : begin
          NxtDbgDo   = ErrorChk | (IfaceRdyAc&~RespFaultReg);
          NxtDbgDoEn = ~ErrorChk;
        end
        `SWDP_SLEACK1 : begin
          NxtDbgDo   = ErrorChk | (~IfaceRdyAc&~RespFaultReg);
          NxtDbgDoEn = ~ErrorChk;
        end
        `SWDP_SLEACK2 : begin
          NxtDbgDo   = ErrorChk | RespFaultReg;
          NxtDbgDoEn = ~ErrorChk;
        end
        `SWDP_SLEPARKW,`SWDP_SLETRNW3, `SWDP_SLETRNW2,
          `SWDP_SLETRNW1,`SWDP_SLETRNW0: begin
          NxtDbgDo   = 1'b1;
          NxtDbgDoEn = 1'b0;
          end
        `SWDP_SLEDATAR : begin
          NxtDbgDo   = ShiftReg[0];
          NxtDbgDoEn = 1'b1;
        end
        `SWDP_SLEPARITYR : begin
          NxtDbgDo   = Parity;
          NxtDbgDoEn = 1'b1;
        end
        `SWDP_SLEUNUSED :  begin
            NxtDbgDo   = 1'bX;
            NxtDbgDoEn = 1'bX;
          end
        default: begin
          NxtDbgDo   = 1'bx;
          NxtDbgDoEn = 1'bx;
        end
      endcase
    end

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      begin
        DBGDOReg <= 1'b0;
        iDBGDOEN  <= 1'b0;
      end
    else
      begin
        DBGDOReg <= NxtDbgDo;
        iDBGDOEN  <= NxtDbgDoEn;
      end

  assign iDBGDO = DBGDOReg;



  assign SerBankEn = DbgClkEn & State==`SWDP_SLEBANK;

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      SerBank <= 1'b0;
    else if (SerBankEn & MstDPEn)
      SerBank <= DBGDI;

  assign SerDirEn = DbgClkEn & State==`SWDP_SLEDIR;

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      SerDir <= 1'b0;
    else if (SerDirEn & MstDPEn)
      SerDir <= DBGDI;

  assign SerAddrEn = DbgClkEn & (State==`SWDP_SLEADDR0 | State==`SWDP_SLEADDR1);
  assign SerAddrD  = {DBGDI,SerAddr[1]};

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      SerAddr <= 2'b00;
    else if (SerAddrEn & MstDPEn)
      SerAddr <= SerAddrD;

  always @ (DbgClkEn or State or SerDir or SerBank or SerAddr or
            TrnferCnt or MaskLane or StickyCmp or StickyErr or TrnferMode or
            BUSRDATA or DBGDI or ShiftReg or ContErrReg or ContDetectReg or
            WriteErrReg or IfaceRdyAc or ErrorChk or CtrlSel or TurnCnt or
            PrescalerReg or RetryReg or ReadOKReg or CSYSPWRUPACK or
            CSYSPWRUPREQReg or CDBGPWRUPACK or CDBGPWRUPREQReg or
            CDBGRSTACK or CDBGRSTREQReg or RespFaultReg or StickyStatus)
    begin
      if (DbgClkEn)
        case (State)
          `SWDP_SLEACK1 : begin
            if (SerDir==`SWDP_SERREAD & ~ErrorChk)
              begin
                if (IfaceRdyAc&~RespFaultReg)
                  begin
                    if (SerBank==`SWDP_APREG)
                      begin
                        ShiftRegD  = BUSRDATA;
                        ShiftRegEn = 1'b1;
                        RetryRegEn = 1'b1;
                        ReadOKD    = 1'b1;
                        ReadOKEn   = 1'b1;
                      end
                    else // if (SerBank==`SWDP_DPREG)
                      begin
                        case (SerAddr)
                          `SWDP_IDCODE : begin
                            ShiftRegD  = {32'h2BA01477}; // DeviceID
                            ShiftRegEn = 1'b1;
                            RetryRegEn = 1'b0;
                            ReadOKD    = 1'b0;
                            ReadOKEn   = 1'b0;
                          end
                          `SWDP_STAT : begin
                            ShiftRegD  = CtrlSel ? {20'h00000,2'b00,
                                                    TurnCnt,1'b0,1'b1,
                                                    4'b0000,PrescalerReg} :
                                         {CSYSPWRUPACK, CSYSPWRUPREQReg,
                                          CDBGPWRUPACK, CDBGPWRUPREQReg,
                                          CDBGRSTACK,   CDBGRSTREQReg,
                                          2'b00,TrnferCnt,MaskLane,
                                          WriteErrReg,ReadOKReg,StickyErr,
                                          StickyCmp,
                                          TrnferMode,ContErrReg,ContDetectReg};
                            ShiftRegEn = 1'b1;
                            RetryRegEn = 1'b0;
                            ReadOKD    = 1'b0;
                            ReadOKEn   = 1'b0;
                          end
                          `SWDP_RESEND: begin
                            ShiftRegD  = RetryReg;
                            ShiftRegEn = 1'b1;
                            RetryRegEn = 1'b0;
                            ReadOKD    = 1'b0;
                            ReadOKEn   = 1'b0;
                          end
                          `SWDP_RDBUFF : begin
                            ShiftRegD  = BUSRDATA;
                            ShiftRegEn = 1'b1;
                            RetryRegEn = 1'b1;
                            ReadOKD  = 1'b1;
                            ReadOKEn = 1'b1;
                          end
                          default : begin
                            ShiftRegD  = {32{1'bx}};
                            ShiftRegEn = 1'bx;
                            RetryRegEn = 1'bx;
                            ReadOKD    = 1'bx;
                            ReadOKEn   = 1'bx;
                          end
                        endcase
                      end // else: !if(SerBank==`SWDP_APREG)
                  end else // if (IfaceRdyAc)
                    begin
                      ShiftRegD  = {32{1'b0}};
                      ShiftRegEn = 1'b0;
                      RetryRegEn = 1'b0;
                      ReadOKD    = 1'b0;
                      ReadOKEn   = ((SerBank==`SWDP_APREG)|
                                    (SerAddr==`SWDP_RDBUFF));
                    end
              end // if (SerDir==`SWDP_SERREAD & ~ErrorChk)
            else
              begin
                ShiftRegD  = {32{1'b0}};
                ShiftRegEn = 1'b0;
                RetryRegEn = 1'b0;
                ReadOKD    = 1'b0;
                ReadOKEn   = 1'b0;
              end // else: !if(SerDir==`SWDP_SERREAD & ~ErrorChk)

          end
          `SWDP_SLEDATAW : begin
            ShiftRegD  = {DBGDI,ShiftReg[31:1]};
            ShiftRegEn = (SerAddr==`SWDP_ABORT & SerBank==`SWDP_DPREG &
                          SerDir==`SWDP_SERWRITE) |
                         ~StickyStatus;
            RetryRegEn = 1'b0;
            ReadOKD    = 1'b0;
            ReadOKEn   = 1'b0;
          end
          `SWDP_SLEDATAR : begin
            ShiftRegD  = {1'b0,ShiftReg[31:1]};
            ShiftRegEn = 1'b1;
            RetryRegEn = 1'b0;
            ReadOKD    = 1'b0;
            ReadOKEn   = 1'b0;
          end
          `SWDP_SLERESET, `SWDP_SLEIDLE, `SWDP_SLEBANK, `SWDP_SLEDIR ,
            `SWDP_SLEADDR0, `SWDP_SLEADDR1,
            `SWDP_SLEPARITYH, `SWDP_SLESTOPH, `SWDP_SLEPARKH,
            `SWDP_SLETRNH0, `SWDP_SLETRNH1, `SWDP_SLETRNH2,
            `SWDP_SLEACK0, `SWDP_SLEACK2,
            `SWDP_SLEPARKW, `SWDP_SLETRNW3, `SWDP_SLETRNW2, `SWDP_SLETRNW1,
            `SWDP_SLETRNW0, `SWDP_SLEPARITYW, `SWDP_SLESTOPW,
            `SWDP_SLEPARITYR,`SWDP_SLETRNC, `SWDP_SLERESC, `SWDP_SLEERROR:
            begin
              ShiftRegD  = {32{1'b0}};
              ShiftRegEn = 1'b0;
              RetryRegEn = 1'b0;
              ReadOKD    = 1'b0;
              ReadOKEn   = 1'b0;
            end
          `SWDP_SLEUNUSED :  begin
            ShiftRegD  = {32{1'b0}};
            ShiftRegEn = 1'b0;
            RetryRegEn = 1'b0;
            ReadOKD    = 1'b0;
            ReadOKEn   = 1'b0;
          end
          default: begin
            ShiftRegD  = {32{1'bx}};
            ShiftRegEn = 1'bx;
            RetryRegEn = 1'bx;
            ReadOKD    = 1'bx;
            ReadOKEn   = 1'bx;
         end
        endcase // case(State)
      else
        begin
          ShiftRegD  = {32{1'b0}};
          ShiftRegEn = 1'b0;
          RetryRegEn = 1'b0;
          ReadOKD    = 1'b0;
          ReadOKEn   = 1'b0;
        end // else: !if(DbgClkEn)
    end


  always @ (posedge DBGCLK or negedge FullDBGRESETn)
    if(!FullDBGRESETn)
      ShiftReg <= {32{1'b0}};
    else if (ShiftRegEn & MstDPEn)
      ShiftReg <= ShiftRegD;

  assign RetryRegD  = BUSRDATA;

  always @ (posedge DBGCLK or negedge DBGRESETn)
    if(!DBGRESETn)
       RetryReg <= 32'h00000000;
    else
      if (RetryRegEn & MstDPEn)
        RetryReg <= RetryRegD;

  always @ (posedge DBGCLK or negedge DBGRESETn)
    if(!DBGRESETn)
      ReadOKReg <= 1'b0;
    else
      if (ReadOKEn & MstDPEn)
        ReadOKReg <= ReadOKD;

  assign SerCntEn = iWireReset | (DbgClkEn & (State==`SWDP_SLEDATAW |
                                              State==`SWDP_SLEDATAR));
  assign SerCntD  = iWireReset ? 5'b0000 : (SerCnt + 5'b00001);

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      SerCnt <= 5'b00000;
    else if (SerCntEn & MstDPEn)
      SerCnt <= SerCntD;

  assign ParityEn = DbgClkEn & (// clear the parity bit
                                (State==`SWDP_SLERESC     |
                                 State==`SWDP_SLESTOPW    |
                                 State==`SWDP_SLEIDLE     |
                                 State==`SWDP_SLEPARITYH) |
                                (State==`SWDP_SLEBANK  |
                                 State==`SWDP_SLEDIR   |
                                 State==`SWDP_SLEADDR0 |
                                 State==`SWDP_SLEADDR1 |
                                 State==`SWDP_SLEDATAW |
                                 State==`SWDP_SLEDATAR));
  assign ParityD  = ((Parity ^ ShiftReg[0]) & (State==`SWDP_SLEDATAR)) |
                    ((Parity ^ DBGDI) & (State==`SWDP_SLEDATAW |
                                         State==`SWDP_SLEBANK  |
                                         State==`SWDP_SLEDIR   |
                                         State==`SWDP_SLEADDR0 |
                                         State==`SWDP_SLEADDR1));

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      Parity <= 1'b0;
    else if (ParityEn & MstDPEn)
      Parity <= ParityD;

  assign ErrorChkEn = DbgClkEn & (// Set
                                  ErrorChkD |
                                  (State==`SWDP_SLEBANK & ErrorChk));
  assign ErrorChkD  = // Parity error
                      ((DBGDI!=Parity) & (State==`SWDP_SLEPARITYH)) |
                      ((DBGDI==1'b1)   & (State==`SWDP_SLESTOPH )) |
                      ((DBGDI==1'b0)   & (State==`SWDP_SLEPARKH)) |
                      ((ErrorTrack | NeedIDReadReg)
                       & ((State==`SWDP_SLEBANK  &  DBGDI) |
                          (State==`SWDP_SLEDIR   & ~DBGDI) |
                          (State==`SWDP_SLEADDR0 &  DBGDI) |
                          (State==`SWDP_SLEADDR1 &  DBGDI)));

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      ErrorChk <= 1'b0;
    else if (ErrorChkEn & MstDPEn)
      ErrorChk <= ErrorChkD;

  assign ErrorTrackEn = // Set
                        (ErrorTrackD) |
                        (DbgClkEn & ErrorTrack & State==`SWDP_SLEACK1 &
                         IfaceRdyAc &
                         SerBank==`SWDP_DPREG & SerDir==`SWDP_SERREAD &
                         SerAddr==`SWDP_IDCODE) |
                        (DbgClkEn & ErrorTrack & State==`SWDP_SLEBANK &
                         ErrorChk);

  assign ErrorTrackD  = // ID code not read out of reset error
                        (DbgClkEn & State==`SWDP_SLEBANK & ErrorChk &
                         ~ErrorTrack);

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      ErrorTrack <= 1'b0;
    else if (ErrorTrackEn & MstDPEn)
      ErrorTrack <= ErrorTrackD;

  always @ (DbgClkEn or IfaceRdyAc or State or ContErrReg or
            SerDir or SerBank or SerAddr or ShiftReg or WBufCancel or
            ErrorChk or Parity or DBGDI or WriteErrReg)
    begin
      if (DbgClkEn & (State==`SWDP_SLEPARITYW |
                      State==`SWDP_SLESTOPW   |
                      WBufCancel))
        begin
          WriteErrD = ((DBGDI!=Parity) & (State==`SWDP_SLEPARITYW)) |
                      WBufCancel;
          WriteErrEn= (~ContErrReg &
                       ((DBGDI!=Parity) & (State==`SWDP_SLEPARITYW))) |
                      (IfaceRdyAc & State==`SWDP_SLESTOPW &
                       SerDir==`SWDP_SERWRITE &
                       SerBank==`SWDP_DPREG & SerAddr==`SWDP_ABORT &
                       ~ErrorChk & ShiftReg[3] & WriteErrReg) |
                        WBufCancel;
                        end // if (DbgClkEn & (State==`SWDP_SLEPARITYW |...
      else
        begin
          WriteErrD  = 1'b0;
          WriteErrEn = 1'b0;
        end
    end // always @ (DbgClkEn or IfaceRdyAc or State or...

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      WriteErrReg <=  1'b0;
    else if (WriteErrEn & MstDPEn)
      WriteErrReg <= WriteErrD;

  assign PreventWrite = WriteErrReg | WriteErrD;


  always @ (DbgClkEn or State or IfaceRdyAc or SerBank or SerDir or SerAddr or
            WBufAck or WBufCancel or RespFaultReg or
            CDBGPWRUPACK or CDBGPWRUPREQReg or PreventWrite)
    begin
      if (DbgClkEn & IfaceRdyAc & State==`SWDP_SLESTOPW &
          SerDir==`SWDP_SERWRITE)
        begin
          if (SerBank==`SWDP_DPREG & SerAddr==`SWDP_ABORT)
            begin
              WBufReqD  = 1'b0;
              WBufReqEn = 1'b1;
            end
          else if (SerBank==`SWDP_APREG & ~RespFaultReg &
                   CDBGPWRUPACK & CDBGPWRUPREQReg & ~PreventWrite)
            begin
              WBufReqD  = 1'b1;
              WBufReqEn = 1'b1;
            end
          else
            begin
              WBufReqD  = 1'b0;
              WBufReqEn = 1'b0;
            end
        end
      else if (WBufAck | WBufCancel)
        begin
          WBufReqD  = 1'b0;
          WBufReqEn = 1'b1;
        end
      else
        begin
          WBufReqD  = 1'b0;
          WBufReqEn = 1'b0;
        end
    end

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      WBufReq <= 1'b0;
    else if (WBufReqEn & MstDPEn)
      WBufReq <= WBufReqD;

  always @ (WBufReq or WBufCancel or WBufAck or iBusReq or BUSACK)
    if (WBufReq & ~WBufCancel & ~WBufAck & ~iBusReq & ~BUSACK)
      begin
        WBufAckD  = 1'b1;
        WBufAckEn = 1'b1;
      end
    else
      begin
        WBufAckD  = 1'b0;
        WBufAckEn = WBufAck;
      end

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      WBufAck <= 1'b0;
    else if (WBufAckEn & MstDPEn)
      WBufAck <= WBufAckD;

  assign WBufCancel = WBufReq & ((
                                   (SerBank==`SWDP_DPREG) & IfaceRdyC &
                                   (State==`SWDP_SLEACK0)) |
                                 (StickyCmpD & StickyCmpEn) |
                                 (StickyErrD & StickyErrEn)
                                 );



  assign IfaceRdyCEn = DbgClkEn & State==`SWDP_SLESTOPH;
  assign IfaceRdyCD  = { // Read
                        (~WBufReq & SerDir==`SWDP_SERREAD  & ~(iBusReq|BUSACK)),
                        (~WBufReq & SerDir==`SWDP_SERWRITE & ~(iBusReq|BUSACK)),
                        (~WBufReq & SerDir==`SWDP_SERWRITE &
                         SerBank==`SWDP_APREG &
                         (TrnferCnt=={12{1'b0}})),
                        (SerBank==`SWDP_DPREG & SerAddr==`SWDP_ABORT),
                        (SerBank==`SWDP_DPREG & SerAddr==`SWDP_STAT &
                         SerDir == `SWDP_SERREAD) };

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      IfaceRdyC <= 1'b0;
    else if (IfaceRdyCEn & MstDPEn)
      IfaceRdyC <= |IfaceRdyCD;

  assign AcStateEn = (NxtState==`SWDP_SLEACK0);

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      IfaceRdyAc <= 1'b0;
    else if (AcStateEn & MstDPEn)
      IfaceRdyAc <= IfaceRdyC;



  always @ (DbgClkEn or State or SerAddr or SerDir or SerBank or ErrorChk or
            iBusReq or BUSACK or ShiftReg or PreventWrite)
    begin
      if (DbgClkEn & State==`SWDP_SLESTOPW & SerDir==`SWDP_SERWRITE &
              SerBank==`SWDP_DPREG & SerAddr==`SWDP_ABORT & ~ErrorChk
          & ~PreventWrite )
        begin
          if (ShiftReg[0]==1'b1)
            begin
              DapAbortD  = 1'b1;
              DapAbortEn = 1'b1;
            end
          else
            begin
              DapAbortD  = 1'b0;
              DapAbortEn = 1'b0;
            end
        end
      else if (~iBusReq & ~BUSACK)
        begin
          DapAbortD  = 1'b0;
          DapAbortEn = 1'b1;
        end
      else
        begin
          DapAbortD  = 1'b0;
          DapAbortEn = 1'b0;
        end
    end

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      DapAbort <= 1'b0;
    else if (DapAbortEn & MstDPEn)
      DapAbort <= DapAbortD;

  assign ApSelEn = DbgClkEn & IfaceRdyAc & State==`SWDP_SLESTOPW &
         SerDir==`SWDP_SERWRITE & SerBank==`SWDP_DPREG &
         SerAddr==`SWDP_ADDR & ~ErrorChk & ~RespFaultReg & ~PreventWrite;

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      begin
        ApSel     <= {8{1'b0}};
        ApBankSel <= {4{1'b0}};
        CtrlSel   <= 1'b0;
      end
    else if (ApSelEn & MstDPEn)
      begin
        ApSel     <= ShiftReg[31:24];
        ApBankSel <= ShiftReg[7:4];
        CtrlSel   <= ShiftReg[0];
      end

  assign DapBankEn = DbgClkEn & State==`SWDP_SLEACK1 & IfaceRdyAc & ~ErrorChk
                     & ~RespFaultReg;
  assign DapDirEn  = DapBankEn & SerBank==`SWDP_APREG;

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      DapBank <= 1'b0;
    else if (DapBankEn & MstDPEn)
      DapBank <= SerBank;

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      begin
        DapDir    <= 1'b0;
        DapRegSel <= 2'b00;
      end
    else if (DapDirEn & MstDPEn)
      begin
        DapDir    <= ~SerDir;
        DapRegSel <= SerAddr;
      end

  assign CtrlStatEn = DbgClkEn & IfaceRdyAc & State==`SWDP_SLESTOPW &
                      SerDir==`SWDP_SERWRITE &
                      SerBank==`SWDP_DPREG & SerAddr==`SWDP_CTRL & ~CtrlSel
                      & ~RespFaultReg & ~ErrorChk & ~PreventWrite;

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      begin
        ContDetectReg   <= 1'b0;
         TrnferMode_cdc_check    <= 2'b00;
         MaskLane_cdc_check      <= 4'b0000;
        CDBGRSTREQReg   <= 1'b0;
        CDBGPWRUPREQReg <= 1'b0;
        CSYSPWRUPREQReg <= 1'b0;
      end
    else if (CtrlStatEn & MstDPEn)
      begin
        ContDetectReg   <= ShiftReg[0];
         TrnferMode_cdc_check       <= ShiftReg[3:2];
         MaskLane_cdc_check         <= ShiftReg[11:8];
        CDBGRSTREQReg   <= ShiftReg[26];
        CDBGPWRUPREQReg <= ShiftReg[28];
        CSYSPWRUPREQReg <= ShiftReg[30];
      end

  assign iCDBGPWRUPREQ    = CDBGPWRUPREQReg;
  assign iCDBGRSTREQ      = CDBGRSTREQReg;
  assign iCSYSPWRUPREQ    = CSYSPWRUPREQReg;

  always @ (CtrlStatEn or BUSACK or iBusReq or BUSENDCNT or ShiftReg)
    begin
      if (CtrlStatEn)
        begin
          TrnferCntD  = ShiftReg[23:12];
          TrnferCntEn = 1'b1;
        end
      else if (BUSACK & iBusReq)
        begin
          TrnferCntD  = BUSENDCNT;
          TrnferCntEn = 1'b1;
        end
      else
        begin
          TrnferCntD  = {12{1'b0}};
          TrnferCntEn = 1'b0;
        end
    end // always @ (CtrlStatEn or BUSACK or iBusReq or BUSENDCNT or ShiftReg)

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
       TrnferCnt_cdc_check  <= {12{1'b0}};
    else if (TrnferCntEn & MstDPEn)
       TrnferCnt_cdc_check  <= TrnferCntD;

  assign WireCtrlEn = (DbgClkEn & IfaceRdyAc & State==`SWDP_SLESTOPW &
                       SerDir==`SWDP_SERWRITE & ~RespFaultReg &
                       SerBank==`SWDP_DPREG & SerAddr==`SWDP_CTRL & CtrlSel &
                       ~ErrorChk & ~PreventWrite) |
                      (iWireReset & (PrescalerReg!=2'b11 | TurnCnt!=2'b00));

  assign PrescalerD = iWireReset ? 2'b11 : ShiftReg[1:0];
  assign TurnCntD   = iWireReset ? 2'b00 : ShiftReg[9:8];

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      begin
        PrescalerReg <= 2'b11;
        TurnCnt   <= 2'b00;
      end
    else if (WireCtrlEn & MstDPEn)
      begin
        PrescalerReg <= PrescalerD;
        TurnCnt   <= TurnCntD;
      end


  always @ (BUSACK or iBusReq or BUSCMP or DbgClkEn or IfaceRdyAc or State or
            SerDir or SerBank or SerAddr or ShiftReg or StickyCmp or
            TrnferMode or ErrorChk or RespFaultReg or
            ContErrD)
    begin
      if (BUSACK & iBusReq)
        begin
          StickyCmpD  = 1'b1;
          StickyCmpEn = ( BUSCMP & TrnferMode==`DAP_M_FIND) |
                        (~BUSCMP & TrnferMode==`DAP_M_VERIFY);
        end
      else if (DbgClkEn & IfaceRdyAc & State==`SWDP_SLESTOPW &
               SerDir==`SWDP_SERWRITE & SerBank==`SWDP_DPREG &
               SerAddr==`SWDP_ABORT & ~ErrorChk & ~RespFaultReg
               & ~ContErrD)
        begin
          StickyCmpD  = 1'b0;
          StickyCmpEn = ShiftReg[1] & StickyCmp;
        end
      else
        begin
          StickyCmpD  = 1'b0;
          StickyCmpEn = 1'b0;
        end
    end // always @ (BUSACK or iBusReq or BUSCMP or...

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      StickyCmp <= 1'b0;
    else if (StickyCmpEn & MstDPEn)
      StickyCmp <= StickyCmpD;

  always @ (DbgClkEn or State or BUSACK or iBusReq or BUSERROR or StickyErr or
            IfaceRdyAc or SerBank or SerDir or SerAddr or ShiftReg or ErrorChk
            or CDBGPWRUPACK or CDBGPWRUPREQReg or RespFaultReg
            or ContErrD)
    begin
      if (BUSACK & iBusReq)
        begin
          StickyErrD  = 1'b1;
          StickyErrEn = BUSERROR;
        end
      else if (DbgClkEn & IfaceRdyAc & State==`SWDP_SLESTOPW &
               SerDir==`SWDP_SERWRITE & SerBank==`SWDP_APREG &
               ~ErrorChk & ~RespFaultReg)
        begin
          StickyErrD  = 1'b1;
          StickyErrEn = ~(CDBGPWRUPACK & CDBGPWRUPREQReg);
        end
      else if (DbgClkEn & IfaceRdyAc & State==`SWDP_SLESTOPW &
               SerDir==`SWDP_SERWRITE & SerBank==`SWDP_DPREG &
               SerAddr==`SWDP_ABORT & ~ErrorChk & ~RespFaultReg
               & ~ContErrD)
        begin
          StickyErrD  = 1'b0;
          StickyErrEn = ShiftReg[2] & StickyErr;
        end
      else
        begin
          StickyErrD  = 1'b0;
          StickyErrEn = 1'b0;
        end
    end

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      StickyErr <= 1'b0;
    else if (StickyErrEn & MstDPEn)
      StickyErr <= StickyErrD;

  always @ (DbgClkEn or State or ContErrReg or ContDetectReg or
            IfaceRdyAc or SerBank or SerDir or SerAddr or ShiftReg or
            ErrorChk or StickyStatus)
    begin
      if (ContDetectReg & DbgClkEn & State==`SWDP_SLEACK2 &
          (StickyStatus|ErrorChk|~IfaceRdyAc))
        begin
          ContErrD  = 1'b1;
          ContErrEn = ~ContErrReg;
        end
      else if (DbgClkEn & IfaceRdyAc & State==`SWDP_SLESTOPW &
               SerDir==`SWDP_SERWRITE & SerBank==`SWDP_DPREG &
               SerAddr==`SWDP_ABORT & ~ErrorChk)
        begin
          ContErrD  = 1'b0;
          ContErrEn = ShiftReg[4] & ContErrReg;
        end
      else
        begin
          ContErrD  = 1'b0;
          ContErrEn = 1'b0;
        end
    end

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      ContErrReg <= 1'b0;
    else if (ContErrEn & MstDPEn)
      ContErrReg <= ContErrD;


  assign StickyStatus = StickyCmp | StickyErr | WriteErrReg | ContErrReg;



  always @ (StickyStatus or DbgClkEn or State or SerDir or SerBank or
            ErrorChk or WBufAck or BUSACK or iBusReq or IfaceRdyAc or
            CDBGPWRUPACK or CDBGPWRUPREQReg or TrnferMode or
            PreventWrite)
    if (iBusReq)
      begin
        iBusReqD  = ~BUSACK;
        iBusReqEn = BUSACK;
      end
    else if ((WBufAck) |
             (DbgClkEn & State==`SWDP_SLEACK1 & SerDir==`SWDP_SERREAD &
              SerBank==`SWDP_APREG & IfaceRdyAc & ~ErrorChk & ~StickyStatus
              & (TrnferMode==`DAP_M_NORMAL) & ~PreventWrite))
      begin
        iBusReqD  = CDBGPWRUPACK & CDBGPWRUPREQReg & ~BUSACK;
        iBusReqEn = CDBGPWRUPACK & CDBGPWRUPREQReg & ~BUSACK;
      end
    else
      begin
        iBusReqD  = iBusReq;
        iBusReqEn = 1'b0;
      end

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      iBusReq <= 1'b0;
    else if (iBusReqEn & MstDPEn)
      iBusReq <= iBusReqD;

  assign iBusWdataEn = WBufAck & DapBank==`SWDP_APREG;

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
       iBusWdata_cdc_check  <= 32'h00000000;
    else if (iBusWdataEn & MstDPEn)
       iBusWdata_cdc_check  <= ShiftReg;

  assign ApRegSelEn = iBusReqD & iBusReqEn;
  assign ApDirD     = WBufAck ? DapDir    : ~SerDir;
  assign ApRegSelD  = WBufAck ? DapRegSel : SerAddr;

  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      begin
         ApDir_cdc_check     <= 1'b0;
        ApRegSel <= 2'b00;
      end
    else if (ApRegSelEn & MstDPEn)
      begin
         ApDir_cdc_check     <= ApDirD;
        ApRegSel <= ApRegSelD;
      end

  assign iBUSADDR   = {ApSel,8'h00,8'h00,ApBankSel,ApRegSel,2'b00};
  assign iBUSABORT  = iBusReqD & DapAbort;
  assign iBUSWRITE  = ApDir;
  assign iBUSMODE   = TrnferMode;
  assign iBUSMASK   = MaskLane;
  assign iBUSTRNCNT = TrnferCnt;

`ifdef ARM_ASSERT_ON

  reg           WBufAckQ; // for checking
  always @ (negedge DBGRESETn or posedge DBGCLK)
    if (!DBGRESETn)
      WBufAckQ <= 1'b0;
    else
      WBufAckQ <= WBufAck;

  assert_never #(`OVL_ERROR, `OVL_ASSERT, "WBufAck has been incorrectly formed.")
    WBufAck_check(
                  .clk       (DBGCLK),
                  .reset_n   (DBGRESETn),
                  .test_expr (WBufAckQ & ~iBusReq));

  assert_never #(`OVL_ERROR, `OVL_ASSERT, "WBufAck transaction must not start if previous faulted.")
  WBufAck_error(
                .clk       (DBGCLK),
                .reset_n   (DBGRESETn),
                .test_expr (WBufAck &
                            (StickyCmp | StickyErr | WriteErrReg)));

  assert_never_unknown #(`OVL_ERROR, 5, `OVL_ASSERT, "State variable must not be X")
  swdp_dbgresetn(
                 .clk       (DBGCLK),
                 .reset_n   (DBGRESETn),
                 .qualifier (1'b1),
                 .test_expr (State));

  assert_never_unknown #(`OVL_ERROR, 12, `OVL_ASSERT,
      "Initialise transfer counter via software before using APACC interface.")
    swdp_trncnt(
                .clk       (DBGCLK),
                .reset_n   (DBGRESETn),
                .qualifier (iBusReq),
                .test_expr (TrnferCnt));

  assert_never_unknown #(`OVL_ERROR, 14, `OVL_ASSERT,
      "Initialise address register via software before using APACC interface.")
    swdp_ApSel(
               .clk       (DBGCLK),
               .reset_n   (DBGRESETn),
               .qualifier (iBusReq),
               .test_expr ({ApSel,ApBankSel,DapRegSel}));

  assert_never_unknown #(`OVL_ERROR, 32, `OVL_ASSERT, "ShiftReg must not be X during transaction.")
    swdp_ShiftReg(
                  .clk       (DBGCLK),
                  .reset_n   (DBGRESETn),
                  .qualifier (iBusReq),
                  .test_expr (ShiftReg));

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "ShiftRegEn must not be X.")
    swdp_ShiftRegEn(
                    .clk (DBGCLK),
                    .reset_n (DBGRESETn),
                    .qualifier (1'b1),
                    .test_expr (ShiftRegEn)
                    );


  assert_never_unknown #(`OVL_ERROR, 32, `OVL_ASSERT, "iBusWdata must not be X during transaction.")
    swdp_iBusWdata(
                   .clk       (DBGCLK),
                   .reset_n   (DBGRESETn),
                   .qualifier (iBusReq),
                   .test_expr (iBusWdata)
                   );

  assert_handshake #(`OVL_ERROR, 0, 0, 0, 0, 0, `OVL_ASSERT, "Write buffer handshake.")
    swdp_wbufhs(
                .clk       (DBGCLK),
                .reset_n   (DBGRESETn),
                .req       (WBufReq),
                .ack       (WBufAck | WBufCancel));

  assert_never #(`OVL_ERROR, `OVL_ASSERT, "SerCnt must not be non-zero outside of shift states.")
    SerCnt_check(
                 .clk       (DBGCLK),
                 .reset_n   (DBGRESETn),
                 .test_expr ((|SerCnt) & ~(State==`SWDP_SLEDATAW |
                               State==`SWDP_SLEDATAR | iWireReset)));

  assert_never #(`OVL_ERROR, `OVL_ASSERT, "ErrorTrack has been incorrectly formed.")
    ErrorTrack2_check(
                      .clk        (DBGCLK),
                      .reset_n    (DBGRESETn),
                      .test_expr  (ErrorTrackEn &
                                   (ErrorTrack==ErrorTrackD)));

  assert_never #(`OVL_ERROR, `OVL_ASSERT, "WriteErr has been incorrectly formed.")
    WriteErr2_check(
                    .clk        (DBGCLK),
                    .reset_n    (DBGRESETn),
                    .test_expr  (WriteErrEn & (WriteErrReg==WriteErrD)));

assert_cycle_sequence #(`OVL_ERROR, 3, 2'b00, `OVL_ASSERT, "ACK none corrupted")
  AckChk_none(
              .clk           (DBGCLK),
              .reset_n       (DBGRESETn),
              .event_sequence ({(State==`SWDP_SLEACK0&NxtDbgDo),
                               NxtDbgDo,NxtDbgDo}));

assert_cycle_sequence #(`OVL_ERROR, 3, 2'b00, `OVL_ASSERT, "ACK OK corrupted")
  AckChk_ok(
            .clk           (DBGCLK),
            .reset_n       (DBGRESETn),
            .event_sequence({(State==`SWDP_SLEACK0&NxtDbgDo),~NxtDbgDo,
                             ~NxtDbgDo}));

assert_cycle_sequence #(`OVL_ERROR, 3, 2'b00, `OVL_ASSERT, "ACK Wait corrupted")
  AckChk_wait(
              .clk           (DBGCLK),
              .reset_n       (DBGRESETn),
              .event_sequence({(State==`SWDP_SLEACK0&~NxtDbgDo),
                               NxtDbgDo,~NxtDbgDo}));

assert_cycle_sequence #(`OVL_ERROR, 3, 2'b00, `OVL_ASSERT, "ACK Fault corrupted")
  AckChk_fault(
               .clk           (DBGCLK),
               .reset_n       (DBGRESETn),
               .event_sequence({(State==`SWDP_SLEACK0&~NxtDbgDo),
                               ~NxtDbgDo,NxtDbgDo}));

  assert_win_unchange #(`OVL_ERROR, 3, `OVL_ASSERT, "Response type changed during ACK")
  AckSigChange(
               .clk           (DBGCLK),
               .reset_n       (DBGRESETn),
               .start_event   (State==`SWDP_SLEACK0),
               .test_expr     ({ErrorChk,IfaceRdyAc,RespFaultReg}),
               .end_event     (State==`SWDP_SLEACK2));

  reg          StateUndef;
  always @(State)
    case (State)
      `SWDP_SLEUNUSED: StateUndef = 1'b1;
      default  : StateUndef = 1'b0;
    endcase // case(State)

  assert_never #(`OVL_ERROR, `OVL_ASSERT, "State variable has taken an un-used value")
  SWStateUndef(
               .clk        (DBGCLK),
               .reset_n    (DBGRESETn),
               .test_expr  (StateUndef)
               );

  assert_never_unknown #(`OVL_ERROR, 5, `OVL_ASSERT, "State variable must not be X")
  SWStateX (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (State)
            );
  assert_never #(`OVL_ERROR, `OVL_ASSERT, "Parity not reset for header")
  SWParReset(
               .clk        (DBGCLK),
               .reset_n    (DBGRESETn),
               .test_expr  (State == `SWDP_SLEBANK & Parity)
               );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "ErrorChk must not be X")
  SW_x_ErrorChk (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (ErrorChk)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "ResetCountEn must not be X")
  SW_x_ResetCountEn(
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (ResetCountEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "NeedIDReadEn must not be X")
  SW_x_NeedIDReadEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (NeedIDReadEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "SerBankEn must not be X")
  SW_x_SerBankEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (SerBankEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "SerDirEn must not be X")
  SW_x_SerDirEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (SerDirEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "SerAddrEn must not be X")
  SW_x_SerAddrEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (SerAddrEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "DbgClkEn must not be X")
  SW_x_DbgClkEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (DbgClkEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "SerDir must not be X")
  SW_x_SerDir (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (SerDir)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "RespFaultReg must not be X")
  SW_x_RespFaultReg (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (RespFaultReg)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "IfaceRdyAc must not be X")
  SW_x_IfaceRdyAc (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (IfaceRdyAc)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "RetryRegEn must not be X")
  SW_x_RetryRegEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (RetryRegEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "ReadOKEn must not be X")
  SW_x_ReadOKEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (ReadOKEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "SerCntEn must not be X")
  SW_x_SerCntEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (SerCntEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "ParityEn must not be X")
  SW_x_ParityEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (ParityEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "ErrorChkEn must not be X")
  SW_x_ErrorChkEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (ErrorChkEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "ErrorTrackEn must not be X")
  SW_x_ErrorTrackEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (ErrorTrackEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "WBufCancel must not be X")
  SW_x_WBufCancel (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (WBufCancel)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "WriteErrEn must not be X")
  SW_x_WriteErrEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (WriteErrEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "CDBGPWRUPACK must not be X")
  SW_x_CDBGPWRUPACK (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (CDBGPWRUPACK)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "CDBGPWRUPREQReg must not be X")
  SW_x_CDBGPWRUPREQReg (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (CDBGPWRUPREQReg)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "IfaceRdyCEn must not be X")
  SW_x_IfaceRdyCEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (IfaceRdyCEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "PreventWrite must not be X")
  SW_x_PreventWrite (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (PreventWrite)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "WBufReqEn must not be X")
  SW_x_WBufReqEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (WBufReqEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "AcStateEn must not be X")
  SW_x_AcStateEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (AcStateEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "SerBank must not be X")
  SW_x_SerBank (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (SerBank)
            );

  assert_never_unknown #(`OVL_ERROR, 2, `OVL_ASSERT, "SerAddr must not be X")
  SW_x_SerAddr (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (SerAddr)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "BUSACK must not be X")
  SW_x_BUSACK (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (BUSACK)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "DapAbortEn must not be X")
  SW_x_DapAbortEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (DapAbortEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "ApSelEn must not be X")
  SW_x_ApSelEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (ApSelEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "DapBankEn must not be X")
  SW_x_DapBankEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (DapBankEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "DapDirEn must not be X")
  SW_x_DapDirEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (DapDirEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "CtrlStatEn must not be X")
  SW_x_CtrlStatEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (CtrlStatEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "TrnferCntEn must not be X")
  SW_x_TrnferCntEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (TrnferCntEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "StickyCmpEn must not be X")
  SW_x_StickyCmpEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (StickyCmpEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "StickyErrEn must not be X")
  SW_x_StickyErrEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (StickyErrEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "ContDetectReg must not be X")
  SW_x_ContDetectReg (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (ContDetectReg)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "ContErrEn must not be X")
  SW_x_ContErrEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (ContErrEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "WBufAck must not be X")
  SW_x_WBufAck (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (WBufAck)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "iBusReqEn must not be X")
  SW_x_iBusReqEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (iBusReqEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "iBusWdataEn must not be X")
  SW_x_iBusWdataEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (iBusWdataEn)
            );

  assert_never_unknown #(`OVL_ERROR, 1, `OVL_ASSERT, "ApRegSelEn must not be X")
  SW_x_ApRegSelEn (
            .clk       (DBGCLK),
            .reset_n   (DBGRESETn),
            .qualifier (1'b1),
            .test_expr (ApRegSelEn)
            );
  reg  [8*20:0] mnemonic; // Debug test for current state

  always @ (State)
    case (State)
      `SWDP_SLERESET     : mnemonic = {"SLERESET"};
      `SWDP_SLEIDLE      : mnemonic = {"IDLE"};
      `SWDP_SLEBANK      : mnemonic = {"BANK"};
      `SWDP_SLEDIR       : mnemonic = {"DIR"};
      `SWDP_SLEADDR0     : mnemonic = {"ADDR0"};
      `SWDP_SLEADDR1     : mnemonic = {"ADDR1"};
      `SWDP_SLEPARITYH   : mnemonic = {"PARITYH"};
      `SWDP_SLESTOPH     : mnemonic = {"STOPH"};
      `SWDP_SLEPARKH     : mnemonic = {"PARKH"};
      `SWDP_SLETRNH2     : mnemonic = {"TRNH2"};
      `SWDP_SLETRNH1     : mnemonic = {"TRNH1"};
      `SWDP_SLEACK0      : mnemonic = {"ACK0"};
      `SWDP_SLEACK1      : mnemonic = {"ACK1"};
      `SWDP_SLEACK2      : mnemonic = {"ACK2"};
      `SWDP_SLEPARKW     : mnemonic = {"PARKW"};
      `SWDP_SLETRNW3     : mnemonic = {"TRNW3"};
      `SWDP_SLETRNW2     : mnemonic = {"TRNW2"};
      `SWDP_SLETRNW1     : mnemonic = {"TRNW1"};
      `SWDP_SLETRNW0     : mnemonic = {"TRNW0"};
      `SWDP_SLERESW      : mnemonic = {"RESW"};
      `SWDP_SLEDATAW     : mnemonic = {"DATAW"};
      `SWDP_SLEPARITYW   : mnemonic = {"PARITYW"};
      `SWDP_SLEMARKW     : mnemonic = {"MARKW"};
      `SWDP_SLESTOPW     : mnemonic = {"STOPW"};
      `SWDP_SLEDATAR     : mnemonic = {"DATAR"};
      `SWDP_SLEPARITYR   : mnemonic = {"PARITYR"};
      `SWDP_SLESTOPR     : mnemonic = {"STOPR"};
      `SWDP_SLETRNC      : mnemonic = {"TRNC"};
      `SWDP_SLERESC      : mnemonic = {"RESC"};
      `SWDP_SLEERROR     : mnemonic = {"ERROR"};
      default       : mnemonic = {"UNKNOWN"};
    endcase

  reg [(8*20)-1:0]  DecodeHeader; // Debug text for header decode

  always @(posedge DBGCLK or negedge DBGRESETn)
    if (~DBGRESETn)
      DecodeHeader <= {"RESET"};
    else
      begin
        if (State == `SWDP_SLEACK0)
          begin
            if(ErrorChk)
              DecodeHeader <= {"ERROR"};
            else
              if (SerBank==`SWDP_DPREG)
                case ({SerDir,SerAddr})
                  {`SWDP_SERREAD,`SWDP_IDCODE} :
                                       DecodeHeader <= {"Read ID"};
                  {`SWDP_SERREAD,`SWDP_STAT}   :
                                       DecodeHeader <= {"Read STATUS"};
                  {`SWDP_SERREAD,`SWDP_RESEND} :
                                       DecodeHeader <= {"Read Resend"};
                  {`SWDP_SERREAD,`SWDP_RDBUFF} :
                                       DecodeHeader <= {"Read RDBUFF"};
                  {`SWDP_SERWRITE,`SWDP_ABORT} :
                                       DecodeHeader <= {"Write Abort"};
                  {`SWDP_SERWRITE,`SWDP_CTRL } :
                                       DecodeHeader <= {"Write Control"};
                  {`SWDP_SERWRITE,`SWDP_ADDR } :
                                       DecodeHeader <= {"Write ADDR"};
                  default: DecodeHeader <= {"Write undefined"};
                endcase // case({SerDir,SerAddr})
              else
                begin
                  if (SerDir==`SWDP_SERREAD)
                    begin
                      DecodeHeader <= {"AP Read xxxxx"};
                      DecodeHeader[7:0] <= 8'd48+SerAddr;
                    end
                  else
                    begin
                      DecodeHeader <= {"AP Write xxxxx"};
                      DecodeHeader[7:0] <= 8'd48+SerAddr;
                    end
                  case ({ApBankSel,SerAddr})
                    6'b000000 : DecodeHeader[39:0] <= " CSW";
                    6'b000001 : DecodeHeader[39:0] <= " TAR";
                    6'b000010 : DecodeHeader[39:0] <= " xxx";
                    6'b000011 : DecodeHeader[39:0] <= " DRW";

                    6'b000100 : DecodeHeader[39:0] <= " BD0";
                    6'b000101 : DecodeHeader[39:0] <= " BD1";
                    6'b000110 : DecodeHeader[39:0] <= " BD2";
                    6'b000111 : DecodeHeader[39:0] <= " BD3";

                    6'b111100 : DecodeHeader[39:0] <= "aaaa";
                    6'b111101 : DecodeHeader[39:0] <= " CFG";
                    6'b111110 : DecodeHeader[39:0] <= "Base";
                    6'b111111 : DecodeHeader[39:0] <= " IDR";
                    endcase // case({ApBankSel,SerAddr})

                  DecodeHeader[47:40] <= 8'd32;

                end
          end // if (State == `SWDP_SLEACK0)
      end // else: !if(~DBGRESETn)
`endif

`ifdef ARM_CDC_ON
        cdc_launch #(3,0) uxcdc_MaskLane(
                .clk_launch(DBGCLK),
                .clk_capture(DAPSWJDP.uDAPDpApbIf.DAPCLK),
                .d_in(MaskLane_cdc_check),
                .d_out(MaskLane)
                );
`else
        assign MaskLane = MaskLane_cdc_check;
`endif
`ifdef ARM_CDC_ON
        cdc_launch #(1,0) uxcdc_TrnferMode(
                .clk_launch(DBGCLK),
                .clk_capture(DAPSWJDP.uDAPDpApbIf.DAPCLK),
                .d_in(TrnferMode_cdc_check),
                .d_out(TrnferMode)
                );
`else
        assign TrnferMode = TrnferMode_cdc_check;
`endif
`ifdef ARM_CDC_ON
        cdc_launch #(31,0) uxcdc_iBusWdata(
                .clk_launch(DBGCLK),
                .clk_capture(DAPSWJDP.uDAPDpApbIf.DAPCLK),
                .d_in(iBusWdata_cdc_check),
                .d_out(iBusWdata)
                );
`else
        assign iBusWdata = iBusWdata_cdc_check;
`endif
`ifdef ARM_CDC_ON
        cdc_launch uxcdc_ApDir(
                .clk_launch(DBGCLK),
                .clk_capture(DAPSWJDP.uDAPDpApbIf.DAPCLK),
                .d_in(ApDir_cdc_check),
                .d_out(ApDir)
                );
`else
        assign ApDir = ApDir_cdc_check;
`endif
`ifdef ARM_CDC_ON
        cdc_launch #(11,0) uxcdc_TrnferCnt(
                .clk_launch(DBGCLK),
                .clk_capture(DAPSWJDP.uDAPDpApbIf.DAPCLK),
                .d_in(TrnferCnt_cdc_check),
                .d_out(TrnferCnt)
                );
`else
        assign TrnferCnt = TrnferCnt_cdc_check;
`endif
endmodule
