  `define SWJ_JTLR        6'b000000    // JTAG Test-Logic-Reset - also entered from reset

  `define SWJ_JDET        6'b000001    // JTAG-Selected - SW sequence detecting
  `define SWJ_SSEQ2       6'b000010    // Looking for 2nd bit in SWD Select sequence
  `define SWJ_SSEQ3       6'b000011    // Looking for 3rd bit in SWD Select sequence
  `define SWJ_SSEQ4       6'b000100    // Looking for 4th bit in SWD Select sequence
  `define SWJ_SSEQ5       6'b000101    // Looking for 5th bit in SWD Select sequence
  `define SWJ_SSEQ6       6'b000110    // Looking for 6th bit in SWD Select sequence
  `define SWJ_SSEQ7       6'b000111    // Looking for 7th bit in SWD Select sequence
  `define SWJ_SSEQ8       6'b001000    // Looking for 8th bit in SWD Select sequence
  `define SWJ_SSEQ9       6'b001001    // Looking for 9th bit in SWD Select sequence
  `define SWJ_SSEQ10      6'b001010    // Looking for 10th bit in SWD Select sequence
  `define SWJ_SSEQ11      6'b001011    // Looking for 11th bit in SWD Select sequence
  `define SWJ_SSEQ12      6'b001100    // Looking for 12th bit in SWD Select sequence
  `define SWJ_SSEQ13      6'b001101    // Looking for 13th bit in SWD Select sequence
  `define SWJ_SSEQ14      6'b001110    // Looking for 14th bit in SWD Select sequence
  `define SWJ_SSEQ15      6'b001111    // Looking for 15th bit in SWD Select sequence

  `define SWJ_SSLP        6'b111111    // SW selected - SW-DP performing transactions

  `define SWJ_SLRST       6'b100000    // SW-Selected - SW-DP in line reset

  `define SWJ_SDET        6'b100001    // SW-Selected - JTAG sequence detecting
  `define SWJ_JSEQ2       6'b100010    // Looking for 2nd bit in JTAG Select sequence
  `define SWJ_JSEQ3       6'b100011    // Looking for 3rd bit in JTAG Select sequence
  `define SWJ_JSEQ4       6'b100100    // Looking for 4th bit in JTAG Select sequence
  `define SWJ_JSEQ5       6'b100101    // Looking for 5th bit in JTAG Select sequence
  `define SWJ_JSEQ6       6'b100110    // Looking for 6th bit in JTAG Select sequence
  `define SWJ_JSEQ7       6'b100111    // Looking for 7th bit in JTAG Select sequence
  `define SWJ_JSEQ8       6'b101000    // Looking for 8th bit in JTAG Select sequence
  `define SWJ_JSEQ9       6'b101001    // Looking for 9th bit in JTAG Select sequence
  `define SWJ_JSEQ10      6'b101010    // Looking for 10th bit in JTAG Select sequence
  `define SWJ_JSEQ11      6'b101011    // Looking for 11th bit in JTAG Select sequence
  `define SWJ_JSEQ12      6'b101100    // Looking for 12th bit in JTAG Select sequence
  `define SWJ_JSEQ13      6'b101101    // Looking for 13th bit in JTAG Select sequence
  `define SWJ_JSEQ14      6'b101110    // Looking for 14th bit in JTAG Select sequence
  `define SWJ_JSEQ15      6'b101111    // Looking for 15th bit in JTAG Select sequence

  `define SWJ_JSLP        6'b010000    // JTAG-Selected - JTAG-DP performing transactions


  `define SWJUNUSED1   6'b010011,6'b010100,6'b010101,6'b010110,6'b010111,6'b011000
  `define SWJUNUSED2   6'b011001,6'b011010,6'b011011,6'b011100,6'b011101,6'b011110,6'b011111
  `define SWJUNUSED3   6'b110000,6'b110001,6'b110010,6'b110011,6'b110100,6'b110101
  `define SWJUNUSED4   6'b110110,6'b110111,6'b111000,6'b111001,6'b111010,6'b111011
  `define SWJUNUSED5   6'b111100,6'b111101,6'b111110,6'b010010,6'b010001

  `define SWJ_UNUSED `SWJUNUSED1, `SWJUNUSED2, `SWJUNUSED3, `SWJUNUSED4, `SWJUNUSED5


  `define SWJ_UNDEF    6'bXXXXXX     // Undefined state


module DAPSwjWatcher
  (/*AUTOARG*/
  SWCLKTCK, SWDITMS, nPOTRST, WIRERESET, JTAGTLR,
  JTAGNSW, DBGDI, TMS
  );

  parameter DP_PRESENT   = 1;
  parameter JTAG_PRESENT = 1;

  input         SWCLKTCK;       // Serial Wire DBGCLK / JTAG TCK input
  input         SWDITMS;        // Serial Wire Data In / JTAG TMS
  input         nPOTRST;        // Power on Reset


  input         WIRERESET;      // SW-DP in line reset
  input         JTAGTLR;        // JTAG-DP TAP controller in Test-Logic-Reset


  output        JTAGNSW;        // Select JTAG, not Serial Wire
  output        DBGDI;          // Serial Wire Data Input
  output        TMS;            // TMS input to JTAG-DP



  reg  [5:0]    NextState;      // Next state of mode select state machine
  wire  [5:0]    State;          // Mode select state machine state
  reg  [5:0]    State_cdc_check;          // Mode select state machine state

  wire          JTAGNSW;        // Mode select signal
  wire          StateSeqEn;     // State machine clock enable

  wire          iJTAGNSW;
  wire          iDBGDI;
  wire          iTMS;
  wire          MstDPEn;

  assign JTAGNSW = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iJTAGNSW : 1'b0;
  assign DBGDI   = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iDBGDI   : SWDITMS;
  assign TMS     = (JTAG_PRESENT != 0 & DP_PRESENT != 0) ? iTMS     : SWDITMS;

  assign MstDPEn = (DP_PRESENT != 0) ? 1'b1 : 1'b0;


   always @(/*AUTOSENSE*/JTAGTLR or SWDITMS or State or WIRERESET)

     begin: p_SwjWatcher_State
      case (State)


       `SWJ_JTLR : // JTAG selected - reset state; also entered on power on reset

         NextState = (SWDITMS) ? `SWJ_JTLR : `SWJ_JDET;


       `SWJ_JDET  : // JTAG-Selected - Look for 1st bit in SWD Select sequence
         NextState = (SWDITMS)  ? `SWJ_SSEQ2  : `SWJ_JSLP;
       `SWJ_SSEQ2 : // Look for 2nd bit in SWD Select sequence
         NextState = (SWDITMS)  ? `SWJ_SSEQ3  : `SWJ_JSLP;
       `SWJ_SSEQ3 : // Look for 3rd bit in SWD Select sequence
         NextState = (SWDITMS) ? `SWJ_SSEQ4  : `SWJ_JSLP;
       `SWJ_SSEQ4 : // Look for 4th bit in SWD Select sequence
         NextState = (SWDITMS)  ? `SWJ_SSEQ5  : `SWJ_JSLP;
       `SWJ_SSEQ5 : // Look for 5th bit in SWD Select sequence
         NextState = (~SWDITMS) ? `SWJ_SSEQ6  : `SWJ_JSLP;
       `SWJ_SSEQ6 : // Look for 6th bit in SWD Select sequence
         NextState = (~SWDITMS)  ? `SWJ_SSEQ7  : `SWJ_JSLP;
       `SWJ_SSEQ7 : // Look for 7th bit in SWD Select sequence
         NextState = (SWDITMS) ? `SWJ_SSEQ8  : `SWJ_JSLP;
       `SWJ_SSEQ8 : // Look for 8th bit in SWD Select sequence
         NextState = (SWDITMS)  ? `SWJ_SSEQ9  : `SWJ_JSLP;
       `SWJ_SSEQ9 : // Look for 9th bit in SWD Select sequence
         NextState = (SWDITMS)  ? `SWJ_SSEQ10 : `SWJ_JSLP;
       `SWJ_SSEQ10 : // Look for 10th bit in SWD Select sequence
         NextState = (SWDITMS) ? `SWJ_SSEQ11 : `SWJ_JSLP;
       `SWJ_SSEQ11 : // Look for 11th bit in SWD Select sequence
         NextState = (~SWDITMS)  ? `SWJ_SSEQ12 : `SWJ_JSLP;
       `SWJ_SSEQ12 : // Look for 12th bit in SWD Select sequence
         NextState = (~SWDITMS) ? `SWJ_SSEQ13 : `SWJ_JSLP;
       `SWJ_SSEQ13 : // Look for 13th bit in SWD Select sequence
         NextState = (SWDITMS)  ? `SWJ_SSEQ14 : `SWJ_JSLP;
       `SWJ_SSEQ14 : // Look for 14th bit in SWD Select sequence
         NextState = (SWDITMS)  ? `SWJ_SSEQ15 : `SWJ_JSLP;
       `SWJ_SSEQ15 : // Look for 15th bit in SWD Select sequence
         NextState = (SWDITMS) ? ((WIRERESET) ? `SWJ_SLRST : `SWJ_SSLP) : `SWJ_JSLP;

       `SWJ_JSLP : //JTAG selected - sleeping state - remain here until TAP goes to Test-Logic-Reset
         NextState = (JTAGTLR) ? `SWJ_JTLR : `SWJ_JSLP;


       `SWJ_SLRST : // Serial Wire selected - line reset state

         NextState = (SWDITMS) ? `SWJ_SLRST : `SWJ_SDET;

       `SWJ_SDET : // Serial Wire selected - JTAG sequence detecting state
         NextState = (~SWDITMS)  ? `SWJ_JSEQ2  : `SWJ_SSLP;
       `SWJ_JSEQ2 : // Look for 2nd bit in SWD Select sequence
         NextState = (SWDITMS)  ? `SWJ_JSEQ3  : `SWJ_SSLP;
       `SWJ_JSEQ3 : // Look for 3rd bit in SWD Select sequence
         NextState = (SWDITMS) ? `SWJ_JSEQ4  : `SWJ_SSLP;
       `SWJ_JSEQ4 : // Look for 4th bit in SWD Select sequence
         NextState = (SWDITMS)  ? `SWJ_JSEQ5  : `SWJ_SSLP;
       `SWJ_JSEQ5 : // Look for 5th bit in SWD Select sequence
         NextState = (SWDITMS) ? `SWJ_JSEQ6  : `SWJ_SSLP;
       `SWJ_JSEQ6 : // Look for 6th bit in SWD Select sequence
         NextState = (~SWDITMS)  ? `SWJ_JSEQ7  : `SWJ_SSLP;
       `SWJ_JSEQ7 : // Look for 7th bit in SWD Select sequence
         NextState = (~SWDITMS) ? `SWJ_JSEQ8  : `SWJ_SSLP;
       `SWJ_JSEQ8 : // Look for 8th bit in SWD Select sequence
         NextState = (SWDITMS)  ? `SWJ_JSEQ9  : `SWJ_SSLP;
       `SWJ_JSEQ9 : // Look for 9th bit in SWD Select sequence
         NextState = (SWDITMS)  ? `SWJ_JSEQ10 : `SWJ_SSLP;
       `SWJ_JSEQ10 : // Look for 10th bit in SWD Select sequence
         NextState = (SWDITMS) ? `SWJ_JSEQ11 : `SWJ_SSLP;
       `SWJ_JSEQ11 : // Look for 11th bit in SWD Select sequence
         NextState = (~SWDITMS)  ? `SWJ_JSEQ12 : `SWJ_SSLP;
       `SWJ_JSEQ12 : // Look for 12th bit in SWD Select sequence
         NextState = (~SWDITMS) ? `SWJ_JSEQ13 : `SWJ_SSLP;
       `SWJ_JSEQ13 : // Look for 13th bit in SWD Select sequence
         NextState = (SWDITMS)  ? `SWJ_JSEQ14 : `SWJ_SSLP;
       `SWJ_JSEQ14 : // Look for 14th bit in SWD Select sequence
         NextState = (SWDITMS)  ? `SWJ_JSEQ15 : `SWJ_SSLP;
       `SWJ_JSEQ15 : // Look for 15th bit in SWD Select sequence
         NextState = (SWDITMS) ? ((JTAGTLR) ? `SWJ_JTLR : `SWJ_JSLP) : `SWJ_SSLP;


       `SWJ_SSLP : //Serial Wire selected - sleeping state - remain here until SW-DP goes to line reset

         NextState = (WIRERESET) ? `SWJ_SLRST : `SWJ_SSLP;

       `SWJ_UNUSED   : NextState = `SWJ_UNDEF;

        default     : NextState = `SWJ_UNDEF;
      endcase
    end // block: p_SwjWatcher_State


  assign StateSeqEn = ~(((State == `SWJ_JSLP) & ~JTAGTLR) | ((State == `SWJ_SSLP) & ~WIRERESET));

  always @ (posedge SWCLKTCK or negedge nPOTRST)
    begin : p_StateSeq
      if (!nPOTRST)
         State_cdc_check  <= `SWJ_JTLR;
      else if (StateSeqEn & MstDPEn)
         State_cdc_check  <= NextState;
    end


  assign iJTAGNSW = (State == `SWJ_JSLP | State[5:4] == 2'b00);

  assign iDBGDI = JTAGNSW | SWDITMS; // Force Serial Wire Data Input high in JTAG mode

  assign iTMS   = ~JTAGNSW | SWDITMS; // Force TMS high in Serial Wire Mode


`ifdef ARM_ASSERT_ON

  assert_never_unknown #(0, 6, 0, "SWJ Watcher state machine Next State must not be X")
  swjdp_state(
               .clk       (SWCLKTCK),
               .reset_n   (nPOTRST),
               .qualifier (1'b1),
               .test_expr (NextState));

  assert_never_unknown #(0, 1, 0, "SWJ Watcher state machine enable must not be X")
  swjdp_state_en(
               .clk       (SWCLKTCK),
               .reset_n   (nPOTRST),
               .qualifier (1'b1),
               .test_expr (StateSeqEn));

  assert_always_on_edge #(0, 2, 0, "SWD not in line-reset when switching from JTAG to SWD ")
  Switch_to_SWD(
               .clk            (SWCLKTCK),
               .reset_n        (nPOTRST),
               .sampling_event (JTAGNSW),
               .test_expr      (WIRERESET));

  assert_always_on_edge #(0, 1, 0, "JTAG-DP not in TLR when switching from SWD to JTAG")
  Switch_to_JTAG(
               .clk            (SWCLKTCK),
               .reset_n        (nPOTRST),
               .sampling_event (JTAGNSW),
               .test_expr      (JTAGTLR));

  reg  [8*20:0] mnemonic; // Debug test for current state

  always @ (State)
    case (State)
      `SWJ_JTLR         : mnemonic = {"SWJ_JTLR"};
      `SWJ_JDET         : mnemonic = {"SWJ_JDET"};
      `SWJ_SSEQ2        : mnemonic = {"SWJ_SSEQ2"};
      `SWJ_SSEQ3        : mnemonic = {"SWJ_SSEQ3"};
      `SWJ_SSEQ4        : mnemonic = {"SWJ_SSEQ4"};
      `SWJ_SSEQ5        : mnemonic = {"SWJ_SSEQ5"};
      `SWJ_SSEQ6        : mnemonic = {"SWJ_SSEQ6"};
      `SWJ_SSEQ7        : mnemonic = {"SWJ_SSEQ7"};
      `SWJ_SSEQ8        : mnemonic = {"SWJ_SSEQ8"};
      `SWJ_SSEQ9        : mnemonic = {"SWJ_SSEQ9"};
      `SWJ_SSEQ10       : mnemonic = {"SWJ_SSEQ10"};
      `SWJ_SSEQ11       : mnemonic = {"SWJ_SSEQ11"};
      `SWJ_SSEQ12       : mnemonic = {"SWJ_SSEQ12"};
      `SWJ_SSEQ13       : mnemonic = {"SWJ_SSEQ13"};
      `SWJ_SSEQ14       : mnemonic = {"SWJ_SSEQ14"};
      `SWJ_SSEQ15       : mnemonic = {"SWJ_SSEQ15"};
      `SWJ_SSLP         : mnemonic = {"SWJ_SSLP"};
      `SWJ_SLRST        : mnemonic = {"SWJ_SLRST"};
      `SWJ_SDET         : mnemonic = {"SWJ_SDET"};
      `SWJ_JSEQ2        : mnemonic = {"SWJ_JSEQ2"};
      `SWJ_JSEQ3        : mnemonic = {"SWJ_JSEQ3"};
      `SWJ_JSEQ4        : mnemonic = {"SWJ_JSEQ4"};
      `SWJ_JSEQ5        : mnemonic = {"SWJ_JSEQ5"};
      `SWJ_JSEQ6        : mnemonic = {"SWJ_JSEQ6"};
      `SWJ_JSEQ7        : mnemonic = {"SWJ_JSEQ7"};
      `SWJ_JSEQ8        : mnemonic = {"SWJ_JSEQ8"};
      `SWJ_JSEQ9        : mnemonic = {"SWJ_JSEQ9"};
      `SWJ_JSEQ10       : mnemonic = {"SWJ_JSEQ10"};
      `SWJ_JSEQ11       : mnemonic = {"SWJ_JSEQ11"};
      `SWJ_JSEQ12       : mnemonic = {"SWJ_JSEQ12"};
      `SWJ_JSEQ13       : mnemonic = {"SWJ_JSEQ13"};
      `SWJ_JSEQ14       : mnemonic = {"SWJ_JSEQ14"};
      `SWJ_JSEQ15       : mnemonic = {"SWJ_JSEQ15"};
      `SWJ_JSLP         : mnemonic = {"SWJ_JSLP"};
      default       : mnemonic = {"UNKNOWN"};
    endcase // case(State)

`endif

`ifdef ARM_CDC_ON
        cdc_launch #(5,0) uxcdc_State(
                .clk_launch(SWCLKTCK),
                .clk_capture(DAPSWJDP.uDAPDpApbIf.DAPCLK),
                .d_in(State_cdc_check),
                .d_out(State)
                );
`else
        assign State = State_cdc_check;
`endif

endmodule
  