module cm3_bus_matrix
  (// Inputs
   hclk, hreset_n, dreset_n, fpb_matched_i, fpb_matchedi_i, fpb_matched_addri_i,
   fpb_matched_addrd_i, fpb_matched_proti_i, fpb_matched_mem_attri_i,
   fpb_matched_protd_i, fpb_matched_mem_attrd_i, fpb_matched_exreqd_i,
   fpb_matched_sized_i, fpb_remap_addr_i, fpb_remap_func_i, mpu_canceli_i,
   mpu_canceld_i, dpu_ahb_haddrd_i, dpu_ahb_htransd_i, dpu_ahb_hwrited_i,
   dpu_ahb_hsized_i, dpu_ahb_hburstd_i, dpu_ahb_hprotd_i, mpu_mem_attrd_i,
   dpu_exreqd_i, dpu_ahb_hwdatad_i, dpu_dbg_snapstall_i, data_be8_i,
   dpu_force_non_buf_i, mpu_addrd_non_buf_i, dpu_ahb_haddri_i,
   dpu_ahb_htransi_i, dpu_ahb_hsizei_i, dpu_ahb_hbursti_i, dpu_ahb_hproti_i,
   mpu_mem_attri_i, d_not_i_trans_i, dap_ahb_hmaster_i, dap_ahb_haddr_i,
   dap_ahb_htrans_i, dap_ahb_hwrite_i, dap_ahb_hsize_i, dap_ahb_hburst_i,
   dap_ahb_hprot_i, dap_ahb_hwdata_i, dap_ahb_habort_i, mtx_ahb_hrdatai_i,
   mtx_ahb_hreadyi_i, mtx_ahb_hrespi_i, mtx_ahb_hrdatad_i,
   mtx_ahb_hreadyd_i, mtx_ahb_hrespd_i, mtx_exrespd_i, mtx_ahb_hrdatas_i,
   mtx_ahb_hreadys_i, mtx_ahb_hresps_i, mtx_exresps_i, mtx_ppb_hrdata_i,
   mtx_ppb_hready_i, mtx_ppb_hresp_i,
   mtx_ahb_haddri_o, mtx_ahb_htransi_o, mtx_ahb_hsizei_o, mtx_ahb_hbursti_o,
   mtx_ahb_hproti_o, mtx_mem_attri_o, mtx_ahb_hmasterd_o, mtx_ahb_haddrd_o,
   mtx_ahb_htransd_o, mtx_ahb_hwrited_o, mtx_ahb_hsized_o, mtx_ahb_hburstd_o,
   mtx_ahb_hprotd_o, mtx_mem_attrd_o, mtx_exreqd_o, mtx_ahb_hwdatad_o,
   mtx_ahb_hmasters_o, mtx_ahb_hmastlocks_o, mtx_ahb_haddrs_o,
   mtx_ahb_htranss_o, mtx_ahb_hwrites_o, mtx_ahb_hsizes_o, mtx_ahb_hbursts_o,
   mtx_ahb_hprots_o, mtx_mem_attrs_o, mtx_exreqs_o, mtx_ahb_hwdatas_o,
   mtx_ppb_hmaster_o, mtx_ppb_haddr_o, mtx_ppb_htrans_o, mtx_ppb_hwrite_o,
   mtx_ppb_hsize_o, mtx_ppb_hburst_o, mtx_ppb_hprot_o, mtx_ppb_hwdata_o,
   mtx_dpu_ahb_hrdatad_o, mtx_mpu_ahb_hreadyoutd_o, mtx_dpu_ahb_haddraccd_o,
   mtx_dpu_ahb_hrespd_o, mtx_dpu_exrespd_o, mtx_dpu_ahb_hrdatai_o,
   mtx_mpu_ahb_hreadyouti_o, mtx_dpu_ahb_haddracci_o, mtx_dpu_ahb_hrespi_o,
   mtx_dap_ahb_hrdata_o, mtx_dap_ahb_hreadyout_o, mtx_dap_ahb_haddracc_o,
   mtx_dap_ahb_hresp_o, mtx_bm_active_o, mtx_write_buff_abort_o,
   mtx_mpu_canceli_reg_o, mtx_mpu_canceld_reg_o, mtx_fpb_active_o
  );


  parameter DEBUG_LVL      = 3;  // Debug support level, between 0 and 3
  parameter RESET_ALL_REGS = 0;  // Reset all registers if 1
  parameter BB_PRESENT     = 1;  // Bit banding present if 1
  parameter CONST_AHB_CTRL = 0;  // Constant AHB control info if 1


  input         hclk;                     // AHB system clock
  input         hreset_n;                 // AHB system reset
  input         dreset_n;                 // Debug reset

  input   [1:0] mpu_mem_attri_i;          // I-Side Memory Attributes
  input   [1:0] mpu_mem_attrd_i;          // D-Side Memory Attributes
  input         mpu_addrd_non_buf_i;      // Address non-bufferable
  input         mpu_canceli_i;            // Core D-Side MPU lookup failed
  input         mpu_canceld_i;            // Core I-Side MPU lookup failed

  input   [1:0] fpb_matched_i;            // FPB has matched
  input         fpb_matchedi_i;           // IFPB match - data phase
  input   [4:0] fpb_matched_addri_i;      // Instruction address
  input   [2:0] fpb_matched_addrd_i;      // Data address
  input   [3:0] fpb_matched_proti_i;      // Instruction prot
  input   [1:0] fpb_matched_mem_attri_i;  // Instruction memory attributes
  input   [3:0] fpb_matched_protd_i;      // Data prot
  input   [1:0] fpb_matched_mem_attrd_i;  // Data memory attributes
  input         fpb_matched_exreqd_i;     // Data exclusive request
  input   [1:0] fpb_matched_sized_i;      // Data size
  input  [23:0] fpb_remap_addr_i;         // Remapped address
  input   [1:0] fpb_remap_func_i;         // Remap function
  output        mtx_mpu_canceli_reg_o;    // Cancel trans following mpu_canceli
  output        mtx_mpu_canceld_reg_o;    // Cancel trans following mpu_canceld
  output        mtx_fpb_active_o;         // Servicing FPB request

  input  [31:0] dpu_ahb_haddrd_i;         // Address bus
  input   [1:0] dpu_ahb_htransd_i;        // Transfer type
  input         dpu_ahb_hwrited_i;        // Transfer direction
  input   [1:0] dpu_ahb_hsized_i;         // Transfer size
  input   [2:0] dpu_ahb_hburstd_i;        // Burst type
  input   [3:0] dpu_ahb_hprotd_i;         // Protection control
  input  [31:0] dpu_ahb_hwdatad_i;        // Write data
  output [31:0] mtx_dpu_ahb_hrdatad_o;    // Read data bus
  output        mtx_mpu_ahb_hreadyoutd_o; // HREADY feedback, through MPU
  output        mtx_dpu_ahb_haddraccd_o;  // HADDR accept
  output  [1:0] mtx_dpu_ahb_hrespd_o;     // Transfer response
  input         dpu_exreqd_i;             // Exclusive Request
  output        mtx_dpu_exrespd_o;        // Exclusive response

  input  [31:0] dpu_ahb_haddri_i;         // Address bus
  input   [1:0] dpu_ahb_htransi_i;        // Tranafer type
  input   [1:0] dpu_ahb_hsizei_i;         // Transfer size
  input   [2:0] dpu_ahb_hbursti_i;        // Burst type
  input   [3:0] dpu_ahb_hproti_i;         // Protection control
  output [31:0] mtx_dpu_ahb_hrdatai_o;    // Read data bus
  output        mtx_mpu_ahb_hreadyouti_o; // HREADY feedback
  output        mtx_dpu_ahb_haddracci_o;  // HADDR accept
  output  [1:0] mtx_dpu_ahb_hrespi_o;     // Transfer response

  input  [31:0] dap_ahb_haddr_i;          // Address bus
  input   [1:0] dap_ahb_hmaster_i;        // Master
  input   [1:0] dap_ahb_htrans_i;         // Transfer type
  input         dap_ahb_hwrite_i;         // Transfer direction
  input   [1:0] dap_ahb_hsize_i;          // Transfer size
  input   [2:0] dap_ahb_hburst_i;         // Burst type
  input   [3:0] dap_ahb_hprot_i;          // Protection control
  input  [31:0] dap_ahb_hwdata_i;         // Write data
  input         dap_ahb_habort_i;         // Abort
  output [31:0] mtx_dap_ahb_hrdata_o;     // Read data bus
  output        mtx_dap_ahb_hreadyout_o;  // HREADY feedback
  output        mtx_dap_ahb_haddracc_o;   // HADDR accept
  output  [1:0] mtx_dap_ahb_hresp_o;      // Transfer response

  output [31:0] mtx_ahb_haddri_o;         // Address bus
  output  [1:0] mtx_ahb_htransi_o;        // Transfer type
  output  [2:0] mtx_ahb_hsizei_o;         // Transfer size
  output  [2:0] mtx_ahb_hbursti_o;        // Burst type
  output  [3:0] mtx_ahb_hproti_o;         // Protection control
  input  [31:0] mtx_ahb_hrdatai_i;        // Read data bus
  input         mtx_ahb_hreadyi_i;        // HREADY feedback
  input   [1:0] mtx_ahb_hrespi_i;         // Transfer response
  output  [1:0] mtx_mem_attri_o;          // Memory Attributes

  output [31:0] mtx_ahb_haddrd_o;         // Address bus
  output  [1:0] mtx_ahb_hmasterd_o;       // Master
  output  [1:0] mtx_ahb_htransd_o;        // Transfer type
  output        mtx_ahb_hwrited_o;        // Transfer direction
  output  [2:0] mtx_ahb_hsized_o;         // Transfer size
  output  [2:0] mtx_ahb_hburstd_o;        // Burst type
  output  [3:0] mtx_ahb_hprotd_o;         // Protection control
  output [31:0] mtx_ahb_hwdatad_o;        // Write data
  input  [31:0] mtx_ahb_hrdatad_i;        // Read data bus
  input         mtx_ahb_hreadyd_i;        // HREADY feedback
  input   [1:0] mtx_ahb_hrespd_i;         // Transfer response
  output  [1:0] mtx_mem_attrd_o;          // Memory Attributes
  output        mtx_exreqd_o;             // Exclusive Request
  input         mtx_exrespd_i;            // Exclusive response

  output [31:0] mtx_ahb_haddrs_o;         // Address bus
  output  [1:0] mtx_ahb_hmasters_o;       // Master
  output        mtx_ahb_hmastlocks_o;     // Locked transaction
  output  [1:0] mtx_ahb_htranss_o;        // Transfer type
  output        mtx_ahb_hwrites_o;        // Transfer direction
  output  [2:0] mtx_ahb_hsizes_o;         // Transfer size
  output  [2:0] mtx_ahb_hbursts_o;        // Burst type
  output  [3:0] mtx_ahb_hprots_o;         // Protection control
  output [31:0] mtx_ahb_hwdatas_o;        // Write data
  input  [31:0] mtx_ahb_hrdatas_i;        // Read data bus
  input         mtx_ahb_hreadys_i;        // HREADY feedback
  input   [1:0] mtx_ahb_hresps_i;         // Transfer response
  output  [1:0] mtx_mem_attrs_o;          // Memory Attributes
  output        mtx_exreqs_o;             // Exclusive Request
  input         mtx_exresps_i;            // Exclusive response

  output [31:0] mtx_ppb_haddr_o;          // Address bus
  output  [1:0] mtx_ppb_hmaster_o;        // Master
  output  [1:0] mtx_ppb_htrans_o;         // Transfer type
  output        mtx_ppb_hwrite_o;         // Transfer direction
  output  [2:0] mtx_ppb_hsize_o;          // Transfer size
  output  [2:0] mtx_ppb_hburst_o;         // Burst type
  output  [3:0] mtx_ppb_hprot_o;          // Protection control
  output [31:0] mtx_ppb_hwdata_o;         // Write data
  input  [31:0] mtx_ppb_hrdata_i;         // Read data bus
  input         mtx_ppb_hready_i;         // HREADY feedback
  input   [1:0] mtx_ppb_hresp_i;          // Transfer response

  input         data_be8_i;               // Endianess
  input         d_not_i_trans_i;          // D/ICODE arbitration control
  input         dpu_force_non_buf_i;      // Force non-bufferable
  input         dpu_dbg_snapstall_i;      // SNAPSTALL from debug
  output        mtx_bm_active_o;          // BitMaster is active
  output        mtx_write_buff_abort_o;   // WriteBuffer imprecise abort



  wire          bm_icore_to_sys;
  wire          bm_mpu_unalign_cancel;     // Unaligned MPU cancel

  wire   [31:0] is_dcore_addr;
  wire    [1:0] is_dcore_master;
  wire    [1:0] is_dcore_trans;
  wire          is_dcore_write;
  wire    [1:0] is_dcore_size;
  wire    [1:0] is_dcore_size_unalign;
  wire    [2:0] is_dcore_burst;
  wire    [3:0] is_dcore_prot;
  wire    [1:0] is_dcore_mem_attr;
  wire          is_dcore_exreq;
  wire          is_dcore_held_trans;       // Transaction held
  wire    [2:0] is_dcore_unalign;          // DCore unaligned access encoding
  wire    [1:0] is_dcore_buffered_write;   // DCore buffered write
  wire          mtx_mpu_canceld_reg;

  wire          dec_dcore_accept;          // DCore -> * transaction accepted
  wire          dec_dcore_active;          // DCore -> * transaction active
  wire          dec_dcore_active_data;     // Data phase of active signal
  wire          dec_dcore_readyout;        // HREADYOUT from active output stage
  wire    [1:0] dec_dcore_resp;            // HRESP from active output stage
  wire          dec_dcore_exresp;          // EXRESP from active output stage

  wire   [31:0] is_icore_addr;
  wire    [1:0] is_icore_trans;
  wire    [1:0] is_icore_size;
  wire    [2:0] is_icore_burst;
  wire    [3:0] is_icore_prot;
  wire    [1:0] is_icore_mem_attr;
  wire          is_icore_held_trans;       // Transaction held
  wire          mtx_mpu_canceli_reg;

  wire          dec_icore_accept;          // ICore -> * transaction accepted
  wire          dec_icore_active;          // ICore -> * transaction active
  wire          dec_icore_readyout;        // HREADYOUT from active output stage
  wire    [1:0] dec_icore_resp;            // HRESP from active output stage

  wire    [1:0] is_dap_master;
  wire   [31:0] is_dap_addr;
  wire    [1:0] is_dap_trans;
  wire          is_dap_write;
  wire    [1:0] is_dap_size;
  wire    [1:0] is_dap_size_unalign;
  wire    [2:0] is_dap_burst;
  wire    [3:0] is_dap_prot;
  wire          is_dap_held_trans;         // Transaction held
  wire    [2:0] is_dap_unalign;            // DAP unaligned access encoding

  wire          dec_dap_accept;            // DAP -> * transaction accepted
  wire          dec_dap_active;            // DAP -> * transaction active
  wire          dec_dap_readyout;          // HREADYOUT from active output stage
  wire    [1:0] dec_dap_resp;              // HRESP from active output stage

  wire    [1:0] bm_master;
  wire   [31:0] bm_addr;
  wire    [1:0] bm_trans;
  wire          bm_write;
  wire    [1:0] bm_size;
  wire    [2:0] bm_burst;
  wire    [3:0] bm_prot;
  wire    [1:0] bm_mem_attr;
  wire          bm_exreq;
  wire   [31:0] bm_wdata;
  wire   [31:0] bm_rdata;
  wire          bm_readyout;
  wire    [1:0] bm_readyout_sel;
  wire          bm_resp;

  wire          bm_dcore_active;           // DCore -> Bit Master active
  wire          bm_icore_active;           // ICore -> Bit Master active
  wire          bm_dap_active;             // DAP   -> Bit Master active
  wire          bm_dcore_active_data;      // DCore -> Bit Master active data phase
  wire          bm_dap_active_data;        // DAP   -> Bit Master active data phase
  wire    [1:0] bm_dcore_active_sel;       // DCore active slave
  wire          bm_icore_active_sel;       // ICore active slave
  wire    [1:0] bm_dap_active_sel;         // DAP active slave
  wire          bm_dcore_fpb_active;       // FPB remap on DCore
  wire          bm_icore_fpb_active;       // FPB remap on ICore
  wire          bm_dcore_fpb_stall;        // DCore FPB stall
  wire          bm_icore_fpb_stall;        // ICore FPB stall
  wire          bm_fpb_active;             // FPB remap
  wire          bm_cancel_valid;           // Bit Master late cancel valid
  wire          bm_buff_write_accept;      // Buffered Write owns Bit Master
  wire          bm_state_bb_wr1;           // Bit Master in bit-band write 1 state
  wire          bm_state_bb_wr2;           // Bit Master in bit-band write 2 state
  wire          bm_dcore_stall;            // DCore stalled waiting for Bit Master
  wire          bm_icore_stall;            // ICore stalled waiting for Bit Master
  wire          bm_dap_stall;              // DAP stalled waiting for Bit Master
  wire          bm_sel_dcode;              // Bit Master -> DCode request
  wire          bm_sel_dcode_data;         // Bit Master -> DCode request (data port)
  wire          bm_sel_sys;                // Bit Master -> SYS request
  wire          bm_sel_sys_data;           // Bit Master -> SYS request (data port)
  wire          mtx_write_buff_abort;      // Imprecise abort from buffered write
  wire          mtx_bm_active;             // Bit Master is active

  wire          dec_dcore_sel_dcode;       // DCore -> DCode
  wire          dec_dcore_sel_dcode_held;  // DCore -> DCode held
  wire          dec_dcore_sel_sys;         // DCore -> SYS
  wire          dec_dcore_sel_sys_held;    // DCore -> SYS held
  wire          dec_dcore_sel_bb;          // DCore -> SYS bit-band
  wire          dec_dcore_sel_ppb;         // DCore -> PPB
  wire          dec_dcore_sel_ppb_held;    // DCore -> PPB held
  wire          dec_icore_sel_icode;       // ICore -> ICode
  wire          dec_icore_sel_icode_held;  // ICore -> ICode held
  wire          dec_icore_sel_sys;         // ICore -> SYS
  wire          dec_icore_sel_sys_held;    // ICore -> SYS held
  wire          dec_dap_sel_dcode;         // DAP -> DCode
  wire          dec_dap_sel_dcode_held;    // DAP -> DCode held
  wire          dec_dap_sel_sys;           // DAP -> SYS
  wire          dec_dap_sel_sys_held;      // DAP -> SYS held
  wire          dec_dap_sel_bb;            // DAP -> SYS bit-band
  wire          dec_dap_sel_ppb;           // DAP -> PPB
  wire          dec_dap_sel_ppb_held;      // DAP -> PPB held

  wire          os_dcore_to_dcode_accept;  // DCore -> DCode accepted
  wire          os_dcore_to_dcode_active;  // DCore -> DCode active
  wire          os_dcore_to_sys_accept;    // DCore -> SYS accepted
  wire          os_dcore_to_sys_active;    // DCore -> SYS active
  wire          os_dcore_to_ppb_accept;    // DCore -> PPB accepted
  wire          os_dcore_to_ppb_active;    // DCore -> PPB active
  wire          os_icore_to_icode_accept;  // ICore -> ICode accepted
  wire          os_icore_to_icode_active;  // ICore -> ICode active
  wire          os_icore_to_icode_held;    // ICore -> ICode waited request
  wire          os_dap_to_dcode_accept;    // DAP -> DCode accepted
  wire          os_dap_to_dcode_active;    // DAP -> DCode active
  wire          os_dap_to_dcode_held_req;  // DAP -> DCode waited request
  wire          os_dap_to_sys_accept;      // DAP -> SYS accepted
  wire          os_dap_to_sys_active;      // DAP -> SYS active
  wire          os_dap_to_sys_held_req;    // DAP -> SYS waited request
  wire          os_dap_to_ppb_accept;      // DAP -> PPB accepted
  wire          os_dap_to_ppb_active;      // DAP -> PPB active
  wire          os_sys_bm_wait;            // SYS -> BM wait

  wire   [31:0] mtx_dpu_ahb_hrdatai;
  wire          mtx_mpu_ahb_hreadyouti;
  wire          mtx_dpu_ahb_haddracci;
  wire    [1:0] mtx_dpu_ahb_hrespi;

  wire   [31:0] mtx_dpu_ahb_hrdatad;
  wire          mtx_mpu_ahb_hreadyoutd;
  wire          mtx_dpu_ahb_haddraccd;
  wire    [1:0] mtx_dpu_ahb_hrespd;
  wire          mtx_dpu_exrespd;

  wire   [31:0] mtx_dap_ahb_hrdata;
  wire          mtx_dap_ahb_hreadyout;
  wire          mtx_dap_ahb_haddracc;
  wire    [1:0] mtx_dap_ahb_hresp;

  wire   [31:0] mtx_ahb_haddri;
  wire    [1:0] mtx_ahb_htransi;
  wire    [2:0] mtx_ahb_hsizei;
  wire    [2:0] mtx_ahb_hbursti;
  wire    [3:0] mtx_ahb_hproti;
  wire    [1:0] mtx_mem_attri;

  wire   [31:0] mtx_ahb_haddrd;
  wire    [1:0] mtx_ahb_hmasterd;
  wire    [1:0] mtx_ahb_htransd;
  wire          mtx_ahb_hwrited;
  wire    [2:0] mtx_ahb_hsized;
  wire    [2:0] mtx_ahb_hburstd;
  wire    [3:0] mtx_ahb_hprotd;
  wire   [31:0] mtx_ahb_hwdatad;
  wire    [1:0] mtx_mem_attrd;
  wire          mtx_exreqd;

  wire   [31:0] mtx_ahb_haddrs;
  wire    [1:0] mtx_ahb_hmasters;
  wire          mtx_ahb_hmastlocks;
  wire    [1:0] mtx_ahb_htranss;
  wire          mtx_ahb_hwrites;
  wire    [2:0] mtx_ahb_hsizes;
  wire    [2:0] mtx_ahb_hbursts;
  wire    [3:0] mtx_ahb_hprots;
  wire   [31:0] mtx_ahb_hwdatas;
  wire    [1:0] mtx_mem_attrs;
  wire          mtx_exreqs;

  wire   [31:0] mtx_ppb_haddr;
  wire    [1:0] mtx_ppb_hmaster;
  wire    [1:0] mtx_ppb_htrans;
  wire          mtx_ppb_hwrite;
  wire    [2:0] mtx_ppb_hsize;
  wire    [2:0] mtx_ppb_hburst;
  wire    [3:0] mtx_ppb_hprot;
  wire   [31:0] mtx_ppb_hwdata;

  wire          opt_mtx_dap_ahb_hreadyout;
  wire          opt_mtx_dap_ahb_haddracc;
  wire    [1:0] opt_mtx_dap_ahb_hresp;
  wire    [1:0] opt_is_dap_master;
  wire   [31:0] opt_is_dap_addr;
  wire    [1:0] opt_is_dap_trans;
  wire          opt_is_dap_write;
  wire    [1:0] opt_is_dap_size;
  wire    [1:0] opt_is_dap_size_unalign;
  wire    [2:0] opt_is_dap_unalign;
  wire    [2:0] opt_is_dap_burst;
  wire    [3:0] opt_is_dap_prot;
  wire          opt_is_dap_held_trans;



  cm3_mtx_input_stage_dcore
    u_cm3_mtx_input_stage_dcore
      (// Inputs
       .hclk                      (hclk),
       .hreset_n                  (hreset_n),
       .dpu_ahb_haddrd_i          (dpu_ahb_haddrd_i),
       .dpu_ahb_htransd_i         (dpu_ahb_htransd_i),
       .dpu_ahb_hwrited_i         (dpu_ahb_hwrited_i),
       .dpu_ahb_hsized_i          (dpu_ahb_hsized_i),
       .dpu_ahb_hburstd_i         (dpu_ahb_hburstd_i),
       .dpu_ahb_hprotd_i          (dpu_ahb_hprotd_i),
       .mpu_mem_attrd_i           (mpu_mem_attrd_i),
       .dpu_exreqd_i              (dpu_exreqd_i),
       .dpu_dbg_snapstall_i       (dpu_dbg_snapstall_i),
       .dpu_force_non_buf_i       (dpu_force_non_buf_i),
       .mpu_addrd_non_buf_i       (mpu_addrd_non_buf_i),
       .mpu_canceld_i             (mpu_canceld_i),
       .buff_write_accept_i       (bm_buff_write_accept),
       .dec_dcore_sel_bb_i        (dec_dcore_sel_bb),
       .dec_dcore_sel_ppb_i       (dec_dcore_sel_ppb),
       .dec_dcore_accept_i        (dec_dcore_accept),
       .dec_dcore_active_i        (dec_dcore_active),
       .dec_dcore_readyout_i      (dec_dcore_readyout),
       .dec_dcore_resp_i          (dec_dcore_resp),
       .dec_dcore_exresp_i        (dec_dcore_exresp),
       .dec_dcore_sel_dcode_i     (dec_dcore_sel_dcode),
       .dec_dcore_sel_sys_i       (dec_dcore_sel_sys),
       .mtx_mpu_ahb_hreadyoutd_o  (mtx_mpu_ahb_hreadyoutd),
       .mtx_dpu_ahb_haddraccd_o   (mtx_dpu_ahb_haddraccd),
       .mtx_dpu_ahb_hrespd_o      (mtx_dpu_ahb_hrespd),
       .mtx_dpu_exrespd_o         (mtx_dpu_exrespd),
       .is_dcore_addr_o           (is_dcore_addr),
       .is_dcore_master_o         (is_dcore_master),
       .is_dcore_trans_o          (is_dcore_trans),
       .is_dcore_write_o          (is_dcore_write),
       .is_dcore_size_o           (is_dcore_size),
       .is_dcore_size_unalign_o   (is_dcore_size_unalign),
       .is_dcore_burst_o          (is_dcore_burst),
       .is_dcore_prot_o           (is_dcore_prot),
       .is_dcore_mem_attr_o       (is_dcore_mem_attr),
       .is_dcore_exreq_o          (is_dcore_exreq),
       .is_dcore_held_trans_o     (is_dcore_held_trans),
       .is_dcore_unalign_o        (is_dcore_unalign),
       .is_dcore_buffered_write_o (is_dcore_buffered_write),
       .mtx_mpu_canceld_reg_o     (mtx_mpu_canceld_reg)
      );

  cm3_mtx_input_stage_icore
    u_cm3_mtx_input_stage_icore
      (// Inputs
       .hclk                      (hclk),
       .hreset_n                  (hreset_n),
       .dpu_ahb_haddri_i          (dpu_ahb_haddri_i),
       .dpu_ahb_htransi_i         (dpu_ahb_htransi_i),
       .dpu_ahb_hsizei_i          (dpu_ahb_hsizei_i),
       .dpu_ahb_hbursti_i         (dpu_ahb_hbursti_i),
       .dpu_ahb_hproti_i          (dpu_ahb_hproti_i),
       .mpu_mem_attri_i           (mpu_mem_attri_i),
       .mpu_canceli_i             (mpu_canceli_i),
       .dec_icore_accept_i        (dec_icore_accept),
       .dec_icore_active_i        (dec_icore_active),
       .dec_icore_readyout_i      (dec_icore_readyout),
       .dec_icore_resp_i          (dec_icore_resp),
       .bm_icore_to_sys_i         (bm_icore_to_sys),
       .mtx_mpu_ahb_hreadyouti_o  (mtx_mpu_ahb_hreadyouti),
       .mtx_dpu_ahb_haddracci_o   (mtx_dpu_ahb_haddracci),
       .mtx_dpu_ahb_hrespi_o      (mtx_dpu_ahb_hrespi),
       .is_icore_addr_o           (is_icore_addr),
       .is_icore_trans_o          (is_icore_trans),
       .is_icore_size_o           (is_icore_size),
       .is_icore_burst_o          (is_icore_burst),
       .is_icore_prot_o           (is_icore_prot),
       .is_icore_mem_attr_o       (is_icore_mem_attr),
       .is_icore_held_trans_o     (is_icore_held_trans),
       .mtx_mpu_canceli_reg_o     (mtx_mpu_canceli_reg)
      );

  cm3_mtx_input_stage_dap #(DEBUG_LVL)
    u_cm3_mtx_input_stage_dap
      (// Inputs
       .hclk                      (hclk),
       .dreset_n                  (dreset_n),
       .dap_ahb_haddr_i           (dap_ahb_haddr_i),
       .dap_ahb_hmaster_i         (dap_ahb_hmaster_i),
       .dap_ahb_htrans_i          (dap_ahb_htrans_i),
       .dap_ahb_hwrite_i          (dap_ahb_hwrite_i),
       .dap_ahb_hsize_i           (dap_ahb_hsize_i),
       .dap_ahb_hburst_i          (dap_ahb_hburst_i),
       .dap_ahb_hprot_i           (dap_ahb_hprot_i),
       .dap_ahb_habort_i          (dap_ahb_habort_i),
       .dec_dap_accept_i          (dec_dap_accept),
       .dec_dap_active_i          (dec_dap_active),
       .dec_dap_readyout_i        (dec_dap_readyout),
       .dec_dap_resp_i            (dec_dap_resp),
       .mtx_dap_ahb_hreadyout_o   (mtx_dap_ahb_hreadyout),
       .mtx_dap_ahb_haddracc_o    (mtx_dap_ahb_haddracc),
       .mtx_dap_ahb_hresp_o       (mtx_dap_ahb_hresp),
       .is_dap_master_o           (is_dap_master),
       .is_dap_addr_o             (is_dap_addr),
       .is_dap_trans_o            (is_dap_trans),
       .is_dap_write_o            (is_dap_write),
       .is_dap_size_o             (is_dap_size),
       .is_dap_size_unalign_o     (is_dap_size_unalign),
       .is_dap_burst_o            (is_dap_burst),
       .is_dap_prot_o             (is_dap_prot),
       .is_dap_held_trans_o       (is_dap_held_trans),
       .is_dap_unalign_o          (is_dap_unalign)
      );



`ifdef ARM_DSM

  wire [1:0]    dsm_debug_lvl;

  assign opt_mtx_dap_ahb_haddracc  = (dsm_debug_lvl > 0) ? mtx_dap_ahb_haddracc  : 1'b1;
  assign opt_mtx_dap_ahb_hreadyout = (dsm_debug_lvl > 0) ? mtx_dap_ahb_hreadyout : 1'b1;
  assign opt_mtx_dap_ahb_hresp     = (dsm_debug_lvl > 0) ? mtx_dap_ahb_hresp     : 2'b00;
  assign opt_is_dap_master         = (dsm_debug_lvl > 0) ? is_dap_master         : 2'b00;
  assign opt_is_dap_addr           = (dsm_debug_lvl > 0) ? is_dap_addr           : {32{1'b0}};
  assign opt_is_dap_trans          = (dsm_debug_lvl > 0) ? is_dap_trans          : 2'b00;
  assign opt_is_dap_write          = (dsm_debug_lvl > 0) ? is_dap_write          : 1'b0;
  assign opt_is_dap_size           = (dsm_debug_lvl > 0) ? is_dap_size           : 2'b00;
  assign opt_is_dap_size_unalign   = (dsm_debug_lvl > 0) ? is_dap_size_unalign   : 2'b00;
  assign opt_is_dap_burst          = (dsm_debug_lvl > 0) ? is_dap_burst          : 3'b000;
  assign opt_is_dap_prot           = (dsm_debug_lvl > 0) ? is_dap_prot           : 4'b0000;
  assign opt_is_dap_held_trans     = (dsm_debug_lvl > 0) ? is_dap_held_trans     : 1'b0;
  assign opt_is_dap_unalign        = (dsm_debug_lvl > 0) ? is_dap_unalign        : 3'b000;

`else

  assign opt_mtx_dap_ahb_haddracc  = (DEBUG_LVL > 0) ? mtx_dap_ahb_haddracc  : 1'b1;
  assign opt_mtx_dap_ahb_hreadyout = (DEBUG_LVL > 0) ? mtx_dap_ahb_hreadyout : 1'b1;
  assign opt_mtx_dap_ahb_hresp     = (DEBUG_LVL > 0) ? mtx_dap_ahb_hresp     : 2'b00;
  assign opt_is_dap_master         = (DEBUG_LVL > 0) ? is_dap_master         : 2'b00;
  assign opt_is_dap_addr           = (DEBUG_LVL > 0) ? is_dap_addr           : {32{1'b0}};
  assign opt_is_dap_trans          = (DEBUG_LVL > 0) ? is_dap_trans          : 2'b00;
  assign opt_is_dap_write          = (DEBUG_LVL > 0) ? is_dap_write          : 1'b0;
  assign opt_is_dap_size           = (DEBUG_LVL > 0) ? is_dap_size           : 2'b00;
  assign opt_is_dap_size_unalign   = (DEBUG_LVL > 0) ? is_dap_size_unalign   : 2'b00;
  assign opt_is_dap_burst          = (DEBUG_LVL > 0) ? is_dap_burst          : 3'b000;
  assign opt_is_dap_prot           = (DEBUG_LVL > 0) ? is_dap_prot           : 4'b0000;
  assign opt_is_dap_held_trans     = (DEBUG_LVL > 0) ? is_dap_held_trans     : 1'b0;
  assign opt_is_dap_unalign        = (DEBUG_LVL > 0) ? is_dap_unalign        : 3'b000;

`endif



  cm3_mtx_decode_dcore #(BB_PRESENT)
    u_cm3_mtx_decode_dcore
      (// Inputs
       .hclk                      (hclk),
       .hreset_n                  (hreset_n),
       .mtx_mpu_ahb_hreadyoutd_i  (mtx_mpu_ahb_hreadyoutd),
       .dpu_dbg_snapstall_i       (dpu_dbg_snapstall_i),
       .is_dcore_addr_i           (is_dcore_addr),
       .is_dcore_unalign_i        (is_dcore_unalign),
       .os_dcore_to_dcode_accept_i(os_dcore_to_dcode_accept),
       .os_dcore_to_dcode_active_i(os_dcore_to_dcode_active),
       .mtx_ahb_hreadyd_i         (mtx_ahb_hreadyd_i),
       .mtx_ahb_hrespd_i          (mtx_ahb_hrespd_i),
       .mtx_exrespd_i             (mtx_exrespd_i),
       .mtx_ahb_hrdatad_i         (mtx_ahb_hrdatad_i),
       .os_dcore_to_sys_accept_i  (os_dcore_to_sys_accept),
       .os_dcore_to_sys_active_i  (os_dcore_to_sys_active),
       .mtx_ahb_hreadys_i         (mtx_ahb_hreadys_i),
       .mtx_ahb_hresps_i          (mtx_ahb_hresps_i),
       .mtx_exresps_i             (mtx_exresps_i),
       .mtx_ahb_hrdatas_i         (mtx_ahb_hrdatas_i),
       .os_dcore_to_ppb_accept_i  (os_dcore_to_ppb_accept),
       .os_dcore_to_ppb_active_i  (os_dcore_to_ppb_active),
       .mtx_ppb_hready_i          (mtx_ppb_hready_i),
       .mtx_ppb_hresp_i           (mtx_ppb_hresp_i),
       .mtx_ppb_hrdata_i          (mtx_ppb_hrdata_i),
       .bm_state_bb_wr2_i         (bm_state_bb_wr2),
       .bm_dcore_active_i         (bm_dcore_active),
       .bm_dcore_active_sel_i     (bm_dcore_active_sel),
       .bm_dcore_active_data_i    (bm_dcore_active_data),
       .bm_dcore_fpb_active_i     (bm_dcore_fpb_active),
       .bm_dcore_stall_i          (bm_dcore_stall),
       .bm_dcore_fpb_stall_i      (bm_dcore_fpb_stall),
       .bm_readyout_sel_i         (bm_readyout_sel),
       .bm_resp_i                 (bm_resp),
       .bm_rdata_i                (bm_rdata),
       .dec_dcore_sel_dcode_o     (dec_dcore_sel_dcode),
       .dec_dcore_sel_sys_o       (dec_dcore_sel_sys),
       .dec_dcore_sel_ppb_o       (dec_dcore_sel_ppb),
       .dec_dcore_sel_bb_o        (dec_dcore_sel_bb),
       .dec_dcore_sel_dcode_held_o(dec_dcore_sel_dcode_held),
       .dec_dcore_sel_sys_held_o  (dec_dcore_sel_sys_held),
       .dec_dcore_sel_ppb_held_o  (dec_dcore_sel_ppb_held),
       .dec_dcore_accept_o        (dec_dcore_accept),
       .dec_dcore_active_o        (dec_dcore_active),
       .dec_dcore_active_data_o   (dec_dcore_active_data),
       .dec_dcore_readyout_o      (dec_dcore_readyout),
       .dec_dcore_resp_o          (dec_dcore_resp),
       .dec_dcore_exresp_o        (dec_dcore_exresp),
       .mtx_dpu_ahb_hrdatad_o     (mtx_dpu_ahb_hrdatad)
      );

  cm3_mtx_decode_icore
    u_cm3_mtx_decode_icore
      (// Inputs
       .hclk                      (hclk),
       .hreset_n                  (hreset_n),
       .mtx_mpu_ahb_hreadyouti_i  (mtx_mpu_ahb_hreadyouti),
       .fpb_remap_func_i          (fpb_remap_func_i),
       .fpb_matchedi_i            (fpb_matchedi_i),
       .d_not_i_trans_i           (d_not_i_trans_i),
       .mtx_ahb_htransd_i         (mtx_ahb_htransd),
       .is_icore_addr_i           (is_icore_addr),
       .os_icore_to_icode_accept_i(os_icore_to_icode_accept),
       .os_icore_to_icode_active_i(os_icore_to_icode_active),
       .mtx_ahb_hreadyi_i         (mtx_ahb_hreadyi_i),
       .mtx_ahb_hrespi_i          (mtx_ahb_hrespi_i),
       .mtx_ahb_hrdatai_i         (mtx_ahb_hrdatai_i),
       .mtx_ahb_hreadys_i         (mtx_ahb_hreadys_i),
       .mtx_ahb_hresps_i          (mtx_ahb_hresps_i),
       .mtx_ahb_hrdatas_i         (mtx_ahb_hrdatas_i),
       .bm_icore_fpb_stall_i      (bm_icore_fpb_stall),
       .bm_icore_active_i         (bm_icore_active),
       .bm_icore_active_sel_i     (bm_icore_active_sel),
       .bm_icore_fpb_active_i     (bm_icore_fpb_active),
       .bm_icore_stall_i          (bm_icore_stall),
       .bm_readyout_i             (bm_readyout),
       .bm_resp_i                 (bm_resp),
       .bm_rdata_i                (bm_rdata),
       .dec_icore_sel_icode_o     (dec_icore_sel_icode),
       .dec_icore_sel_sys_o       (dec_icore_sel_sys),
       .dec_icore_sel_icode_held_o(dec_icore_sel_icode_held),
       .dec_icore_sel_sys_held_o  (dec_icore_sel_sys_held),
       .dec_icore_accept_o        (dec_icore_accept),
       .dec_icore_active_o        (dec_icore_active),
       .dec_icore_readyout_o      (dec_icore_readyout),
       .dec_icore_resp_o          (dec_icore_resp),
       .mtx_dpu_ahb_hrdatai_o     (mtx_dpu_ahb_hrdatai)
      );

  cm3_mtx_decode_dap #(BB_PRESENT)
    u_cm3_mtx_decode_dap
      (// Inputs
       .hclk                      (hclk),
       .dreset_n                  (dreset_n),
       .mtx_dap_ahb_hreadyout_i   (mtx_dap_ahb_hreadyout),
       .dpu_dbg_snapstall_i       (dap_ahb_habort_i),
       .is_dap_addr_i             (opt_is_dap_addr),
       .is_dap_unalign_i          (opt_is_dap_unalign),
       .is_dap_held_trans_i       (opt_is_dap_held_trans),
       .os_dap_to_dcode_accept_i  (os_dap_to_dcode_accept),
       .os_dap_to_dcode_active_i  (os_dap_to_dcode_active),
       .mtx_ahb_hreadyd_i         (mtx_ahb_hreadyd_i),
       .mtx_ahb_hrespd_i          (mtx_ahb_hrespd_i),
       .mtx_ahb_hrdatad_i         (mtx_ahb_hrdatad_i),
       .os_dap_to_sys_accept_i    (os_dap_to_sys_accept),
       .os_dap_to_sys_active_i    (os_dap_to_sys_active),
       .mtx_ahb_hreadys_i         (mtx_ahb_hreadys_i),
       .mtx_ahb_hresps_i          (mtx_ahb_hresps_i),
       .mtx_ahb_hrdatas_i         (mtx_ahb_hrdatas_i),
       .os_dap_to_ppb_accept_i    (os_dap_to_ppb_accept),
       .os_dap_to_ppb_active_i    (os_dap_to_ppb_active),
       .mtx_ppb_hready_i          (mtx_ppb_hready_i),
       .mtx_ppb_hresp_i           (mtx_ppb_hresp_i),
       .mtx_ppb_hrdata_i          (mtx_ppb_hrdata_i),
       .bm_state_bb_wr2_i         (bm_state_bb_wr2),
       .bm_dap_active_i           (bm_dap_active),
       .bm_dap_active_sel_i       (bm_dap_active_sel),
       .bm_dap_active_data_i      (bm_dap_active_data),
       .bm_dap_stall_i            (bm_dap_stall),
       .bm_readyout_i             (bm_readyout),
       .bm_resp_i                 (bm_resp),
       .bm_rdata_i                (bm_rdata),
       .dec_dap_sel_dcode_o       (dec_dap_sel_dcode),
       .dec_dap_sel_sys_o         (dec_dap_sel_sys),
       .dec_dap_sel_ppb_o         (dec_dap_sel_ppb),
       .dec_dap_sel_bb_o          (dec_dap_sel_bb),
       .dec_dap_sel_dcode_held_o  (dec_dap_sel_dcode_held),
       .dec_dap_sel_sys_held_o    (dec_dap_sel_sys_held),
       .dec_dap_sel_ppb_held_o    (dec_dap_sel_ppb_held),
       .dec_dap_accept_o          (dec_dap_accept),
       .dec_dap_active_o          (dec_dap_active),
       .dec_dap_readyout_o        (dec_dap_readyout),
       .dec_dap_resp_o            (dec_dap_resp),
       .mtx_dap_ahb_hrdata_o      (mtx_dap_ahb_hrdata)
      );



  cm3_mtx_bit_master #(DEBUG_LVL, RESET_ALL_REGS, CONST_AHB_CTRL)
    u_cm3_mtx_bit_master
      (// Inputs
       .hclk                      (hclk),
       .hreset_n                  (hreset_n),
       .data_be8_i                (data_be8_i),
       .dpu_dbg_snapstall_i       (dpu_dbg_snapstall_i),
       .dap_ahb_habort_i          (dap_ahb_habort_i),
       .mpu_canceli_i             (mpu_canceli_i),
       .mpu_canceld_i             (mpu_canceld_i),
       .fpb_matched_i             (fpb_matched_i),
       .fpb_matched_addri_i       (fpb_matched_addri_i),
       .fpb_matched_mem_attri_i   (fpb_matched_mem_attri_i),
       .fpb_matched_mem_attrd_i   (fpb_matched_mem_attrd_i),
       .fpb_matched_exreqd_i      (fpb_matched_exreqd_i),
       .fpb_matched_addrd_i       (fpb_matched_addrd_i),
       .fpb_matched_proti_i       (fpb_matched_proti_i),
       .fpb_matched_protd_i       (fpb_matched_protd_i),
       .fpb_matched_sized_i       (fpb_matched_sized_i),
       .fpb_remap_addr_i          (fpb_remap_addr_i),
       .is_icore_addr_i           (is_icore_addr),
       .is_icore_prot_i           (is_icore_prot),
       .is_icore_mem_attr_i       (is_icore_mem_attr),
       .is_icore_held_trans_i     (is_icore_held_trans),
       .is_dcore_addr_i           (is_dcore_addr),
       .is_dcore_prot_i           (is_dcore_prot),
       .is_dcore_size_i           (is_dcore_size),
       .is_dcore_write_i          (is_dcore_write),
       .is_dcore_mem_attr_i       (is_dcore_mem_attr),
       .is_dcore_exreq_i          (is_dcore_exreq),
       .is_dcore_held_trans_i     (is_dcore_held_trans),
       .is_dcore_unalign_i        (is_dcore_unalign),
       .is_dap_addr_i             (opt_is_dap_addr),
       .is_dap_master_i           (opt_is_dap_master),
       .is_dap_prot_i             (opt_is_dap_prot),
       .is_dap_size_i             (opt_is_dap_size),
       .is_dap_write_i            (opt_is_dap_write),
       .is_dap_held_trans_i       (opt_is_dap_held_trans),
       .is_dap_unalign_i          (opt_is_dap_unalign),
       .dec_dcore_sel_dcode_i     (dec_dcore_sel_dcode),
       .dec_dcore_sel_sys_i       (dec_dcore_sel_sys),
       .dec_dcore_sel_bb_i        (dec_dcore_sel_bb),
       .dec_dcore_active_data_i   (dec_dcore_active_data),
       .dec_icore_sel_sys_i       (dec_icore_sel_sys),
       .dec_icore_sel_sys_held_i  (dec_icore_sel_sys_held),
       .dec_dap_sel_dcode_i       (dec_dap_sel_dcode),
       .dec_dap_sel_sys_i         (dec_dap_sel_sys),
       .dec_dap_sel_bb_i          (dec_dap_sel_bb),
       .os_dcore_to_dcode_accept_i(os_dcore_to_dcode_accept),
       .os_dcore_to_sys_accept_i  (os_dcore_to_sys_accept),
       .os_dap_to_dcode_accept_i  (os_dap_to_dcode_accept),
       .os_dap_to_dcode_held_req_i(os_dap_to_dcode_held_req),
       .os_dap_to_sys_accept_i    (os_dap_to_sys_accept),
       .os_dap_to_sys_held_req_i  (os_dap_to_sys_held_req),
       .os_sys_bm_wait_i          (os_sys_bm_wait),
       .dpu_ahb_hwdatad_i         (dpu_ahb_hwdatad_i),
       .dap_ahb_hwdata_i          (dap_ahb_hwdata_i),
       .mtx_ahb_hreadyd_i         (mtx_ahb_hreadyd_i),
       .mtx_ahb_hrespd_i          (mtx_ahb_hrespd_i),
       .mtx_ahb_hrdatad_i         (mtx_ahb_hrdatad_i),
       .mtx_ahb_hreadys_i         (mtx_ahb_hreadys_i),
       .mtx_ahb_hresps_i          (mtx_ahb_hresps_i),
       .mtx_ahb_hrdatas_i         (mtx_ahb_hrdatas_i),
       .is_dcore_buffered_write_i (is_dcore_buffered_write),
       .bm_state_bb_wr1_o         (bm_state_bb_wr1),
       .bm_state_bb_wr2_o         (bm_state_bb_wr2),
       .bm_addr_o                 (bm_addr),
       .bm_master_o               (bm_master),
       .bm_trans_o                (bm_trans),
       .bm_cancel_valid_o         (bm_cancel_valid),
       .bm_write_o                (bm_write),
       .bm_size_o                 (bm_size),
       .bm_burst_o                (bm_burst),
       .bm_prot_o                 (bm_prot),
       .bm_mem_attr_o             (bm_mem_attr),
       .bm_exreq_o                (bm_exreq),
       .bm_wdata_o                (bm_wdata),
       .bm_dcore_active_o         (bm_dcore_active),
       .bm_icore_active_o         (bm_icore_active),
       .bm_dap_active_o           (bm_dap_active),
       .bm_dcore_fpb_active_o     (bm_dcore_fpb_active),
       .bm_icore_fpb_active_o     (bm_icore_fpb_active),
       .bm_fpb_active_o           (bm_fpb_active),
       .bm_dcore_active_sel_o     (bm_dcore_active_sel),
       .bm_icore_active_sel_o     (bm_icore_active_sel),
       .bm_dap_active_sel_o       (bm_dap_active_sel),
       .bm_dcore_active_data_o    (bm_dcore_active_data),
       .bm_dap_active_data_o      (bm_dap_active_data),
       .bm_dcore_stall_o          (bm_dcore_stall),
       .bm_icore_stall_o          (bm_icore_stall),
       .bm_dap_stall_o            (bm_dap_stall),
       .bm_dcore_fpb_stall_o      (bm_dcore_fpb_stall),
       .bm_icore_fpb_stall_o      (bm_icore_fpb_stall),
       .bm_sel_dcode_o            (bm_sel_dcode),
       .bm_sel_dcode_data_o       (bm_sel_dcode_data),
       .bm_sel_sys_o              (bm_sel_sys),
       .bm_sel_sys_data_o         (bm_sel_sys_data),
       .bm_buff_write_accept_o    (bm_buff_write_accept),
       .bm_rdata_o                (bm_rdata),
       .bm_readyout_o             (bm_readyout),
       .bm_readyout_sel_o         (bm_readyout_sel),
       .bm_resp_o                 (bm_resp),
       .bm_icore_to_sys_o         (bm_icore_to_sys),
       .bm_mpu_unalign_cancel_o   (bm_mpu_unalign_cancel),
       .mtx_write_buff_abort_o    (mtx_write_buff_abort),
       .mtx_bm_active_o           (mtx_bm_active)
    );



  cm3_mtx_output_stage_icode #(CONST_AHB_CTRL)
    u_cm3_mtx_output_stage_icode
      (// Inputs
       .hclk                      (hclk),
       .hreset_n                  (hreset_n),
       .d_not_i_trans_i           (d_not_i_trans_i),
       .dec_icore_sel_icode_i     (dec_icore_sel_icode),
       .dec_icore_sel_icode_held_i(dec_icore_sel_icode_held),
       .is_icore_addr_i           (is_icore_addr),
       .is_icore_trans_i          (is_icore_trans),
       .is_icore_size_i           (is_icore_size),
       .is_icore_burst_i          (is_icore_burst),
       .is_icore_prot_i           (is_icore_prot),
       .is_icore_held_trans_i     (is_icore_held_trans),
       .mtx_ahb_hreadyi_i         (mtx_ahb_hreadyi_i),
       .mtx_mpu_canceli_reg_i     (mtx_mpu_canceli_reg),
       .os_icore_to_icode_accept_o(os_icore_to_icode_accept),
       .os_icore_to_icode_active_o(os_icore_to_icode_active),
       .os_icore_to_icode_held_o  (os_icore_to_icode_held),
       .mtx_ahb_haddri_o          (mtx_ahb_haddri),
       .mtx_ahb_htransi_o         (mtx_ahb_htransi),
       .mtx_ahb_hsizei_o          (mtx_ahb_hsizei),
       .mtx_ahb_hbursti_o         (mtx_ahb_hbursti),
       .mtx_ahb_hproti_o          (mtx_ahb_hproti),
       .mtx_mem_attri_o           (mtx_mem_attri)
      );

  cm3_mtx_output_stage_dcode #(CONST_AHB_CTRL)
    u_cm3_mtx_output_stage_dcode
      (// Inputs
       .hclk                      (hclk),
       .hreset_n                  (hreset_n),
       .mpu_canceld_i             (mpu_canceld_i),
       .bm_mpu_unalign_cancel_i   (bm_mpu_unalign_cancel),
       .dec_dcore_sel_dcode_i     (dec_dcore_sel_dcode),
       .dec_dcore_sel_dcode_held_i(dec_dcore_sel_dcode_held),
       .dec_dap_sel_dcode_i       (dec_dap_sel_dcode),
       .dec_dap_sel_dcode_held_i  (dec_dap_sel_dcode_held),
       .is_dcore_addr_i           (is_dcore_addr),
       .is_dcore_master_i         (is_dcore_master),
       .is_dcore_trans_i          (is_dcore_trans),
       .is_dcore_write_i          (is_dcore_write),
       .is_dcore_size_unalign_i   (is_dcore_size_unalign),
       .is_dcore_burst_i          (is_dcore_burst),
       .is_dcore_prot_i           (is_dcore_prot),
       .is_dcore_exreq_i          (is_dcore_exreq),
       .is_dcore_held_trans_i     (is_dcore_held_trans),
       .dpu_ahb_hwdatad_i         (dpu_ahb_hwdatad_i),
       .is_dap_addr_i             (opt_is_dap_addr),
       .is_dap_master_i           (opt_is_dap_master),
       .is_dap_trans_i            (opt_is_dap_trans),
       .is_dap_write_i            (opt_is_dap_write),
       .is_dap_size_unalign_i     (opt_is_dap_size_unalign),
       .is_dap_burst_i            (opt_is_dap_burst),
       .is_dap_prot_i             (opt_is_dap_prot),
       .is_dap_held_trans_i       (opt_is_dap_held_trans),
       .os_icore_to_icode_held_i  (os_icore_to_icode_held),
       .dap_ahb_hwdata_i          (dap_ahb_hwdata_i),
       .bm_sel_dcode_i            (bm_sel_dcode),
       .bm_sel_dcode_data_i       (bm_sel_dcode_data),
       .bm_addr_i                 (bm_addr),
       .bm_master_i               (bm_master),
       .bm_trans_i                (bm_trans),
       .bm_cancel_valid_i         (bm_cancel_valid),
       .bm_write_i                (bm_write),
       .bm_size_i                 (bm_size),
       .bm_burst_i                (bm_burst),
       .bm_prot_i                 (bm_prot),
       .bm_wdata_i                (bm_wdata),
       .mtx_ahb_hreadyd_i         (mtx_ahb_hreadyd_i),
       .os_dcore_to_dcode_accept_o(os_dcore_to_dcode_accept),
       .os_dcore_to_dcode_active_o(os_dcore_to_dcode_active),
       .os_dap_to_dcode_accept_o  (os_dap_to_dcode_accept),
       .os_dap_to_dcode_active_o  (os_dap_to_dcode_active),
       .os_dap_to_dcode_held_req_o(os_dap_to_dcode_held_req),
       .mtx_ahb_hmasterd_o        (mtx_ahb_hmasterd),
       .mtx_ahb_haddrd_o          (mtx_ahb_haddrd),
       .mtx_ahb_htransd_o         (mtx_ahb_htransd),
       .mtx_ahb_hwrited_o         (mtx_ahb_hwrited),
       .mtx_ahb_hsized_o          (mtx_ahb_hsized),
       .mtx_ahb_hburstd_o         (mtx_ahb_hburstd),
       .mtx_ahb_hprotd_o          (mtx_ahb_hprotd),
       .mtx_mem_attrd_o           (mtx_mem_attrd),
       .mtx_exreqd_o              (mtx_exreqd),
       .mtx_ahb_hwdatad_o         (mtx_ahb_hwdatad)
      );

  cm3_mtx_output_stage_sys #(CONST_AHB_CTRL)
    u_cm3_mtx_output_stage_sys
      (// Inputs
       .hclk                      (hclk),
       .dreset_n                  (dreset_n),
       .mpu_canceld_i             (mpu_canceld_i),
       .bm_mpu_unalign_cancel_i   (bm_mpu_unalign_cancel),
       .dec_dcore_sel_sys_i       (dec_dcore_sel_sys),
       .dec_dcore_sel_sys_held_i  (dec_dcore_sel_sys_held),
       .is_dcore_master_i         (is_dcore_master),
       .is_dcore_addr_i           (is_dcore_addr),
       .is_dcore_trans_i          (is_dcore_trans),
       .is_dcore_write_i          (is_dcore_write),
       .is_dcore_size_i           (is_dcore_size),
       .is_dcore_size_unalign_i   (is_dcore_size_unalign),
       .is_dcore_burst_i          (is_dcore_burst),
       .is_dcore_prot_i           (is_dcore_prot),
       .is_dcore_mem_attr_i       (is_dcore_mem_attr),
       .is_dcore_exreq_i          (is_dcore_exreq),
       .is_dcore_held_trans_i     (is_dcore_held_trans),
       .dpu_ahb_hwdatad_i         (dpu_ahb_hwdatad_i),
       .dec_dap_sel_sys_i         (dec_dap_sel_sys),
       .dec_dap_sel_sys_held_i    (dec_dap_sel_sys_held),
       .is_dap_master_i           (opt_is_dap_master),
       .is_dap_addr_i             (opt_is_dap_addr),
       .is_dap_trans_i            (opt_is_dap_trans),
       .is_dap_write_i            (opt_is_dap_write),
       .is_dap_size_i             (opt_is_dap_size),
       .is_dap_size_unalign_i     (opt_is_dap_size_unalign),
       .is_dap_burst_i            (opt_is_dap_burst),
       .is_dap_prot_i             (opt_is_dap_prot),
       .is_dap_held_trans_i       (opt_is_dap_held_trans),
       .dap_ahb_hwdata_i          (dap_ahb_hwdata_i),
       .bm_state_bb_wr1_i         (bm_state_bb_wr1),
       .dec_dcore_sel_bb_i        (dec_dcore_sel_bb),
       .dec_dap_sel_bb_i          (dec_dap_sel_bb),
       .bm_sel_sys_i              (bm_sel_sys),
       .bm_sel_sys_data_i         (bm_sel_sys_data),
       .bm_master_i               (bm_master),
       .bm_addr_i                 (bm_addr),
       .bm_trans_i                (bm_trans),
       .bm_cancel_valid_i         (bm_cancel_valid),
       .bm_write_i                (bm_write),
       .bm_size_i                 (bm_size),
       .bm_burst_i                (bm_burst),
       .bm_prot_i                 (bm_prot),
       .bm_mem_attr_i             (bm_mem_attr),
       .bm_exreq_i                (bm_exreq),
       .bm_wdata_i                (bm_wdata),
       .mtx_ahb_hreadys_i         (mtx_ahb_hreadys_i),
       .os_dcore_to_sys_accept_o  (os_dcore_to_sys_accept),
       .os_dcore_to_sys_active_o  (os_dcore_to_sys_active),
       .os_dap_to_sys_accept_o    (os_dap_to_sys_accept),
       .os_dap_to_sys_active_o    (os_dap_to_sys_active),
       .os_dap_to_sys_held_req_o  (os_dap_to_sys_held_req),
       .os_sys_bm_wait_o          (os_sys_bm_wait),
       .mtx_ahb_hmasters_o        (mtx_ahb_hmasters),
       .mtx_ahb_hmastlocks_o      (mtx_ahb_hmastlocks),
       .mtx_ahb_haddrs_o          (mtx_ahb_haddrs),
       .mtx_ahb_htranss_o         (mtx_ahb_htranss),
       .mtx_ahb_hwrites_o         (mtx_ahb_hwrites),
       .mtx_ahb_hsizes_o          (mtx_ahb_hsizes),
       .mtx_ahb_hbursts_o         (mtx_ahb_hbursts),
       .mtx_ahb_hprots_o          (mtx_ahb_hprots),
       .mtx_ahb_hwdatas_o         (mtx_ahb_hwdatas),
       .mtx_mem_attrs_o           (mtx_mem_attrs),
       .mtx_exreqs_o              (mtx_exreqs)
      );

  cm3_mtx_output_stage_ppb
    u_cm3_mtx_output_stage_ppb
      (// Inputs
       .hclk                      (hclk),
       .hreset_n                  (dreset_n),
       .mpu_canceld_i             (mpu_canceld_i),
       .dec_dcore_sel_ppb_i       (dec_dcore_sel_ppb),
       .dec_dcore_sel_ppb_held_i  (dec_dcore_sel_ppb_held),
       .is_dcore_addr_i           (is_dcore_addr),
       .is_dcore_master_i         (is_dcore_master),
       .is_dcore_trans_i          (is_dcore_trans),
       .is_dcore_write_i          (is_dcore_write),
       .is_dcore_size_i           (is_dcore_size),
       .is_dcore_burst_i          (is_dcore_burst),
       .is_dcore_prot_i           (is_dcore_prot),
       .dpu_ahb_hwdatad_i         (dpu_ahb_hwdatad_i),
       .is_dcore_held_trans_i     (is_dcore_held_trans),
       .dec_dap_sel_ppb_i         (dec_dap_sel_ppb),
       .dec_dap_sel_ppb_held_i    (dec_dap_sel_ppb_held),
       .is_dap_addr_i             (opt_is_dap_addr),
       .is_dap_master_i           (opt_is_dap_master),
       .is_dap_trans_i            (opt_is_dap_trans),
       .is_dap_write_i            (opt_is_dap_write),
       .is_dap_size_i             (opt_is_dap_size),
       .is_dap_burst_i            (opt_is_dap_burst),
       .is_dap_prot_i             (opt_is_dap_prot),
       .is_dap_held_trans_i       (opt_is_dap_held_trans),
       .dap_ahb_hwdata_i          (dap_ahb_hwdata_i),
       .mtx_ppb_hready_i          (mtx_ppb_hready_i),
       .os_dcore_to_ppb_accept_o  (os_dcore_to_ppb_accept),
       .os_dcore_to_ppb_active_o  (os_dcore_to_ppb_active),
       .os_dap_to_ppb_accept_o    (os_dap_to_ppb_accept),
       .os_dap_to_ppb_active_o    (os_dap_to_ppb_active),
       .mtx_ppb_haddr_o           (mtx_ppb_haddr),
       .mtx_ppb_hmaster_o         (mtx_ppb_hmaster),
       .mtx_ppb_htrans_o          (mtx_ppb_htrans),
       .mtx_ppb_hwrite_o          (mtx_ppb_hwrite),
       .mtx_ppb_hsize_o           (mtx_ppb_hsize),
       .mtx_ppb_hburst_o          (mtx_ppb_hburst),
       .mtx_ppb_hprot_o           (mtx_ppb_hprot),
       .mtx_ppb_hwdata_o          (mtx_ppb_hwdata)
      );



  assign mtx_mpu_canceli_reg_o    = mtx_mpu_canceli_reg;
  assign mtx_mpu_canceld_reg_o    = mtx_mpu_canceld_reg;
  assign mtx_fpb_active_o         = bm_fpb_active;

  assign mtx_dpu_ahb_hrdatad_o    = mtx_dpu_ahb_hrdatad;
  assign mtx_mpu_ahb_hreadyoutd_o = mtx_mpu_ahb_hreadyoutd;
  assign mtx_dpu_ahb_haddraccd_o  = mtx_dpu_ahb_haddraccd;
  assign mtx_dpu_ahb_hrespd_o     = mtx_dpu_ahb_hrespd;
  assign mtx_dpu_exrespd_o        = mtx_dpu_exrespd;

  assign mtx_dpu_ahb_hrdatai_o    = mtx_dpu_ahb_hrdatai;
  assign mtx_mpu_ahb_hreadyouti_o = mtx_mpu_ahb_hreadyouti;
  assign mtx_dpu_ahb_haddracci_o  = mtx_dpu_ahb_haddracci;
  assign mtx_dpu_ahb_hrespi_o     = mtx_dpu_ahb_hrespi;

  assign mtx_dap_ahb_hrdata_o     = mtx_dap_ahb_hrdata;
  assign mtx_dap_ahb_hreadyout_o  = opt_mtx_dap_ahb_hreadyout;
  assign mtx_dap_ahb_haddracc_o   = opt_mtx_dap_ahb_haddracc;
  assign mtx_dap_ahb_hresp_o      = opt_mtx_dap_ahb_hresp;

  assign mtx_ahb_haddri_o         = mtx_ahb_haddri;
  assign mtx_ahb_htransi_o        = mtx_ahb_htransi;
  assign mtx_ahb_hsizei_o         = mtx_ahb_hsizei;
  assign mtx_ahb_hbursti_o        = mtx_ahb_hbursti;
  assign mtx_ahb_hproti_o         = mtx_ahb_hproti;
  assign mtx_mem_attri_o          = mtx_mem_attri;

  assign mtx_ahb_haddrd_o         = mtx_ahb_haddrd;
  assign mtx_ahb_hmasterd_o       = mtx_ahb_hmasterd;
  assign mtx_ahb_htransd_o        = mtx_ahb_htransd;
  assign mtx_ahb_hwrited_o        = mtx_ahb_hwrited;
  assign mtx_ahb_hsized_o         = mtx_ahb_hsized;
  assign mtx_ahb_hburstd_o        = mtx_ahb_hburstd;
  assign mtx_ahb_hprotd_o         = mtx_ahb_hprotd;
  assign mtx_ahb_hwdatad_o        = mtx_ahb_hwdatad;
  assign mtx_mem_attrd_o          = mtx_mem_attrd;
  assign mtx_exreqd_o             = mtx_exreqd;

  assign mtx_ahb_hmasters_o       = mtx_ahb_hmasters;
  assign mtx_ahb_hmastlocks_o     = mtx_ahb_hmastlocks;
  assign mtx_ahb_haddrs_o         = mtx_ahb_haddrs;
  assign mtx_ahb_htranss_o        = mtx_ahb_htranss;
  assign mtx_ahb_hwrites_o        = mtx_ahb_hwrites;
  assign mtx_ahb_hsizes_o         = mtx_ahb_hsizes;
  assign mtx_ahb_hbursts_o        = mtx_ahb_hbursts;
  assign mtx_ahb_hprots_o         = mtx_ahb_hprots;
  assign mtx_ahb_hwdatas_o        = mtx_ahb_hwdatas;
  assign mtx_mem_attrs_o          = mtx_mem_attrs;
  assign mtx_exreqs_o             = mtx_exreqs;

  assign mtx_ppb_haddr_o          = mtx_ppb_haddr;
  assign mtx_ppb_hmaster_o        = mtx_ppb_hmaster;
  assign mtx_ppb_htrans_o         = mtx_ppb_htrans;
  assign mtx_ppb_hwrite_o         = mtx_ppb_hwrite;
  assign mtx_ppb_hsize_o          = mtx_ppb_hsize;
  assign mtx_ppb_hburst_o         = mtx_ppb_hburst;
  assign mtx_ppb_hprot_o          = mtx_ppb_hprot;
  assign mtx_ppb_hwdata_o         = mtx_ppb_hwdata;

  assign mtx_bm_active_o          = mtx_bm_active;
  assign mtx_write_buff_abort_o   = mtx_write_buff_abort;



`ifdef AHB_ASSERT_ON

  wire   ahbpc_reset_n;
  assign ahbpc_reset_n = dreset_n & hreset_n;


  AhbPCWrapper #(32,32,0,0,0,2,2,2,CONST_AHB_CTRL)
    u_ahbpc_icode
      (// Inputs
       .HCLK                      (hclk),
       .HCLKEN                    (1'b1),
       .HRESETn                   (ahbpc_reset_n),
       .SnapStall                 (dpu_dbg_snapstall_i),
       .HADDR                     (mtx_ahb_haddri),
       .HTRANS                    (mtx_ahb_htransi),
       .HWRITE                    (1'b0),
       .HSIZE                     (mtx_ahb_hsizei),
       .HBURST                    (mtx_ahb_hbursti),
       .HPROT                     (mtx_ahb_hproti),
       .HWDATA                    (32'h00000000),
       .HSELx                     (1'b1),
       .HREADY                    (mtx_ahb_hreadyi_i),
       .HREADYOUT                 (mtx_ahb_hreadyi_i),
       .HRESP                     (mtx_ahb_hrespi_i),
       .HLOCKx                    (1'b0),
       .HGRANTx                   (1'b1),
       .HMASTLOCK                 (1'b0)
      );

  AhbPCWrapper #(32,32,0,0,0,2,2,2,CONST_AHB_CTRL)
    u_ahbpc_dcode
      (// Inputs
       .HCLK                      (hclk),
       .HCLKEN                    (1'b1),
       .HRESETn                   (ahbpc_reset_n),
       .SnapStall                 (dpu_dbg_snapstall_i),
       .HADDR                     (mtx_ahb_haddrd),
       .HTRANS                    (mtx_ahb_htransd),
       .HWRITE                    (mtx_ahb_hwrited),
       .HSIZE                     (mtx_ahb_hsized),
       .HBURST                    (mtx_ahb_hburstd),
       .HPROT                     (mtx_ahb_hprotd),
       .HWDATA                    (mtx_ahb_hwdatad),
       .HSELx                     (1'b1),
       .HREADY                    (mtx_ahb_hreadyd_i),
       .HREADYOUT                 (mtx_ahb_hreadyd_i),
       .HRESP                     (mtx_ahb_hrespd_i),
       .HLOCKx                    (1'b0),
       .HGRANTx                   (1'b1),
       .HMASTLOCK                 (1'b0)
      );

  AhbPCWrapper #(32,32,0,0,0,2,2,2,CONST_AHB_CTRL)
    u_ahbpc_sys
      (// Inputs
       .HCLK                      (hclk),
       .HCLKEN                    (1'b1),
       .HRESETn                   (dreset_n),
       .SnapStall                 (dpu_dbg_snapstall_i),
       .HADDR                     (mtx_ahb_haddrs),
       .HTRANS                    (mtx_ahb_htranss),
       .HWRITE                    (mtx_ahb_hwrites),
       .HSIZE                     (mtx_ahb_hsizes),
       .HBURST                    (mtx_ahb_hbursts),
       .HPROT                     (mtx_ahb_hprots),
       .HWDATA                    (mtx_ahb_hwdatas),
       .HSELx                     (1'b1),
       .HREADY                    (mtx_ahb_hreadys_i),
       .HREADYOUT                 (mtx_ahb_hreadys_i),
       .HRESP                     (mtx_ahb_hresps_i),
       .HLOCKx                    (1'b0),
       .HGRANTx                   (1'b1),
       .HMASTLOCK                 (mtx_ahb_hmastlocks)
      );


`endif



`ifdef ARM_ASSERT_ON

  `include "cm3_mtx_defs.v"


  wire       ovl_suppress_dcode_addr_range_assert_s0;
  wire       ovl_suppress_dcode_addr_range_assert_s1;
  wire       ovl_suppress_dcode_addr_range_assert_s2;
  wire       ovl_suppress_sys_addr_range_assert_s0;
  wire       ovl_suppress_sys_addr_range_assert_s1;
  wire       ovl_suppress_sys_addr_range_assert_s2;

  reg        ovl_dcode_unalign_s1;
  reg        ovl_dcode_unalign_s2;
  reg        ovl_sys_unalign_s1;
  reg        ovl_sys_unalign_s2;

  reg  [9:0] ovl_mtx_ahb_haddri_reg;
  reg  [9:0] ovl_mtx_ahb_haddrd_reg;
  reg  [9:0] ovl_mtx_ahb_haddrs_reg;
  reg  [9:0] ovl_mtx_ppb_haddr_reg;


  always @ (posedge hclk)
    if (mtx_ahb_hreadyi_i)
       ovl_mtx_ahb_haddri_reg <= mtx_ahb_haddri[9:0];

  always @ (posedge hclk)
    if (mtx_ahb_hreadyd_i)
       ovl_mtx_ahb_haddrd_reg <= mtx_ahb_haddrd[9:0];

  always @ (posedge hclk)
    if (mtx_ahb_hreadys_i)
       ovl_mtx_ahb_haddrs_reg <= mtx_ahb_haddrs[9:0];

  always @ (posedge hclk)
    if (mtx_ppb_hready_i)
       ovl_mtx_ppb_haddr_reg <= mtx_ppb_haddr[9:0];


  always @ (posedge hclk)
    if (!hreset_n)
      begin
        ovl_dcode_unalign_s1 <= 1'b0;
        ovl_dcode_unalign_s2 <= 1'b0;
      end
    else if (mtx_ahb_hreadyd_i)
      begin
        ovl_dcode_unalign_s1 <= ovl_suppress_dcode_addr_range_assert_s0;
        ovl_dcode_unalign_s2 <= ovl_dcode_unalign_s1;
      end

  always @ (posedge hclk)
    if (!hreset_n)
      begin
        ovl_sys_unalign_s1 <= 1'b0;
        ovl_sys_unalign_s2 <= 1'b0;
      end
    else if (mtx_ahb_hreadys_i)
      begin
        ovl_sys_unalign_s1 <= ovl_suppress_sys_addr_range_assert_s0;
        ovl_sys_unalign_s2 <= ovl_sys_unalign_s1;
      end



  assign ovl_suppress_dcode_addr_range_assert_s0 =
                                      (ovl_mtx_ahb_haddrd_reg > mtx_ahb_haddrd[9:0]) & bm_sel_dcode;
  assign ovl_suppress_dcode_addr_range_assert_s1 =
                                            (mtx_ahb_haddrd >= 32'h20000000) & ovl_dcode_unalign_s1;
  assign ovl_suppress_dcode_addr_range_assert_s2 =
                                            (mtx_ahb_haddrd >= 32'h20000000) & ovl_dcode_unalign_s2;

  assign ovl_suppress_sys_addr_range_assert_s0 =
                                        (ovl_mtx_ahb_haddrs_reg > mtx_ahb_haddrs[9:0]) & bm_sel_sys;
  assign ovl_suppress_sys_addr_range_assert_s1 =
         ((mtx_ahb_haddrs >= 32'hE0000000) | (mtx_ahb_haddrs >= 32'h00000000)) & ovl_sys_unalign_s1;
  assign ovl_suppress_sys_addr_range_assert_s2 =
         ((mtx_ahb_haddrs >= 32'hE0000000) | (mtx_ahb_haddrs >= 32'h00000000)) & ovl_sys_unalign_s2;



  reg ovl_dpu_ahb_hrespi_reg;
  reg ovl_dpu_ahb_hreadyouti_reg;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        ovl_dpu_ahb_hrespi_reg     <= 1'b0;
        ovl_dpu_ahb_hreadyouti_reg <= 1'b0;
      end
    else
      begin
        ovl_dpu_ahb_hrespi_reg     <= mtx_dpu_ahb_hrespi[0];
        ovl_dpu_ahb_hreadyouti_reg <= mtx_mpu_ahb_hreadyouti;
      end

  wire ovl_nxt_icore_trans_state;
  reg  ovl_icore_trans_state;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_icore_trans_state <= 1'b0;
    else
      ovl_icore_trans_state <= ovl_nxt_icore_trans_state;

  assign ovl_nxt_icore_trans_state = // New data phase
                                     (dpu_ahb_htransi_i[1]   &
                                      mtx_mpu_ahb_hreadyouti &
                                      mtx_dpu_ahb_haddracci) |
                                     (ovl_icore_trans_state & ~mtx_mpu_ahb_hreadyouti);

  assert_never #(0,0,"mtx_ahb_htransi and mtx_ahb_htransd both asserted when DNOTITRANS set.")
    ovl_dnotitrans_check(hclk, hreset_n,
       (d_not_i_trans_i & (mtx_ahb_htransi != 2'b00) & (mtx_ahb_htransd != 2'b00)));

  assert_always #(0,0,"ICore accesses must always be word size.")
    ovl_icore_size_check(hclk, hreset_n,
      ((dpu_ahb_htransi_i != 2'b00) ? (dpu_ahb_hsizei_i == 2'b10) : 1'b1));

  assert_never #(0,0,"ICore HADDR must be word aligned.")
    ovl_icore_addr_wordaligned(hclk, hreset_n, (dpu_ahb_haddri_i[1:0] != 2'b00));

  assert_always #(0,0,"ICore->SYS accesses must always be word size.")
    ovl_icore2sys_size_check(hclk, hreset_n,
      (((mtx_ahb_htranss != 2'b00) && (mtx_ahb_hprots[0] == 1'b0) && (mtx_ahb_hmasters == 2'b10)) ?
        (mtx_ahb_hsizes == 3'b010) : 1'b1));

  assert_always #(0,0,"ICore->SYS accesses must always be word aligned.")
    ovl_icore2sys_wordalign_check(hclk, hreset_n,
      (((mtx_ahb_htranss != 2'b00) && (mtx_ahb_hprots[0] == 1'b0) && (mtx_ahb_hmasters == 2'b10)) ?
        (mtx_ahb_haddrs[1:0] == 2'b00) : 1'b1));

  assert_next #(0,1,1,0,0,
    "dpu_ahb_hreadyout should be deasserted in cycle after fpb_matched_i[0]")
      ovl_ifpb_hready(hclk, hreset_n, (fpb_matched_i[0] & (fpb_remap_func_i == 2'b00)),
        (~mtx_mpu_ahb_hreadyouti));

  assert_always #(0,0,"Invalid 2-cycle Error response on ICore - HRESP.")
    ovl_icore_resp(hclk, hreset_n,
      ((mtx_mpu_ahb_hreadyouti & mtx_dpu_ahb_hrespi[0] & ovl_icore_trans_state) ?
        ovl_dpu_ahb_hrespi_reg : 1'b1));

  assert_always #(0,0,"Invalid 2-cycle Error response on ICore - HREADY.")
    ovl_icore_resp2(hclk, hreset_n,
      ((mtx_mpu_ahb_hreadyouti & mtx_dpu_ahb_hrespi[0] & ovl_icore_trans_state) ?
         ~ovl_dpu_ahb_hreadyouti_reg : 1'b1));

  assert_never #(0,0,"HRESP was not held for 2 cycles during ICore error response.")
    ovl_icore_resp3(hclk, hreset_n,
      (~ovl_dpu_ahb_hreadyouti_reg & ovl_dpu_ahb_hrespi_reg & ~mtx_dpu_ahb_hrespi[0] &
        ovl_icore_trans_state));



  reg [31:0] ovl_dpu_haddrd_reg;

  reg ovl_dpu_ahb_hrespd_reg;
  reg ovl_dpu_ahb_hreadyoutd_reg;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        ovl_dpu_ahb_hrespd_reg     <= 1'b0;
        ovl_dpu_ahb_hreadyoutd_reg <= 1'b0;
        ovl_dpu_haddrd_reg         <= {32{1'b0}};
      end
    else
      begin
        ovl_dpu_ahb_hrespd_reg     <= mtx_dpu_ahb_hrespd[0];
        ovl_dpu_ahb_hreadyoutd_reg <= mtx_mpu_ahb_hreadyoutd;
        ovl_dpu_haddrd_reg         <= dpu_ahb_haddrd_i;
      end

  wire ovl_nxt_dcore_trans_state;
  reg  ovl_dcore_trans_state;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_dcore_trans_state <= 1'b0;
    else
      ovl_dcore_trans_state <= ovl_nxt_dcore_trans_state;

  assign ovl_nxt_dcore_trans_state = // New data phase
                                     (dpu_ahb_htransd_i[1]   &
                                      mtx_mpu_ahb_hreadyoutd &
                                      mtx_dpu_ahb_haddracci) |
                                     (ovl_dcore_trans_state & ~mtx_mpu_ahb_hreadyoutd);


  assert_next #(0,1,1,0,0,
   "mtx_mpu_ahb_hreadyoutd should be deasserted in cycle after fpb_matched_i[1]")
      ovl_dfpb_hready(hclk, hreset_n, fpb_matched_i[1], (~mtx_mpu_ahb_hreadyoutd));

  assert_next #(0,1,1,0,0,
    "dpu_ahb_haddrd_i must not change in the cycle following mtx_dpu_ahb_haddraccd = 0")
      ovl_haddraccs0_addr(hclk, hreset_n, (~mtx_dpu_ahb_haddraccd),
        (dpu_ahb_haddrd_i == ovl_dpu_haddrd_reg));

  assert_always #(0,0,"Invalid 2-cycle Error response on DCore - HRESP.")
    ovl_dcore_resp(hclk, hreset_n,
      ((mtx_mpu_ahb_hreadyoutd & mtx_dpu_ahb_hrespd[0] & ovl_dcore_trans_state) ?
         ovl_dpu_ahb_hrespd_reg : 1'b1));

  assert_always #(0,0,"Invalid 2-cycle Error response on DCore - HREADY.")
    ovl_dcore_resp2(hclk, hreset_n,
      ((mtx_mpu_ahb_hreadyoutd & mtx_dpu_ahb_hrespd[0] & ovl_dcore_trans_state) ?
        ~ovl_dpu_ahb_hreadyoutd_reg : 1'b1));

  assert_never #(0,0,"HRESP is not held for 2 cycles during error response on DCore.")
    ovl_dcore_resp3(hclk, hreset_n,
      (~ovl_dpu_ahb_hreadyoutd_reg & ovl_dpu_ahb_hrespd_reg & ~mtx_dpu_ahb_hrespd[0] &
        ovl_dcore_trans_state));

  assert_never #(0,0, "mtx_write_buff_abort should only be driven when HREADYDCore is asserted.")
    ovl_wb_abort_check(hclk, hreset_n, (mtx_write_buff_abort & ~mtx_mpu_ahb_hreadyoutd));

  assert_next #(0,1,1,0,0, "mtx_write_buff_abort should only be a 1-cycle pulse")
    ovl_wb_abort_pulse_check(hclk, hreset_n, mtx_write_buff_abort, (~mtx_write_buff_abort));

  assert_always #(0,0,
    "dpu_ahb_haddraddc must be deasserted when (bm_buff_write_accept & !dec_dcore_readyout)")
      ovl_haddracc_check(hclk, hreset_n,
        ((dpu_ahb_htransd_i[1] & bm_buff_write_accept & ~dec_dcore_readyout) ?
          ~mtx_dpu_ahb_haddraccd : 1'b1));

  assert_never #(0,0,
    "bm_dcore_fpb_stall and bm_buff_write_accept should not be asserted at same time.")
      ovl_illegal_simultaneous1(hclk, hreset_n, (bm_dcore_fpb_stall & bm_buff_write_accept));



  reg [31:0] ovl_dap_haddr_reg;

  always @ (posedge hclk)
    ovl_dap_haddr_reg  <= dap_ahb_haddr_i;

  reg ovl_dap_ahb_hresp_reg;
  reg ovl_dap_ahb_hreadyout_reg;

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      begin
        ovl_dap_ahb_hresp_reg     <= 1'b0;
        ovl_dap_ahb_hreadyout_reg <= 1'b0;
      end
    else
      begin
        ovl_dap_ahb_hresp_reg     <= opt_mtx_dap_ahb_hresp[0];
        ovl_dap_ahb_hreadyout_reg <= opt_mtx_dap_ahb_hreadyout;
      end

  wire ovl_nxt_dap_trans_state;
  reg  ovl_dap_trans_state;

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      ovl_dap_trans_state <= 1'b0;
    else
      ovl_dap_trans_state <= ovl_nxt_dap_trans_state;

  assign ovl_nxt_dap_trans_state = // New data phase
                                   (dap_ahb_htrans_i[1]   &
                                    mtx_dap_ahb_hreadyout &
                                    mtx_dap_ahb_haddracc) |
                                   (ovl_dap_trans_state & ~mtx_dap_ahb_hreadyout);


  assert_next #(0,1,1,0,0,
    "dap_ahb_haddr_i must not change in the cycle following mtx_dap_ahb_haddracc = 0")
      ovl_haddraccs2_addr(hclk, dreset_n, (~mtx_dap_ahb_haddracc & ~dap_ahb_habort_i),
        (dap_ahb_haddr_i == ovl_dap_haddr_reg));

  assert_next #(0,1,1,0,0,
    "dap_ahb_haddr_i must not change in the cycle following !HREADY, unless prev IDLE")
      ovl_hreadys2_addr(hclk, dreset_n,
        (~mtx_dap_ahb_hreadyout & (dap_ahb_htrans_i != `MTX_TRN_IDLE)),
          (dap_ahb_haddr_i == ovl_dap_haddr_reg));

  assert_always #(0,0,"Invalid 2-cycle Error response on DAP port - HRESP.")
    ovl_dap_resp(hclk, dreset_n,
      ((mtx_dap_ahb_hreadyout & mtx_dap_ahb_hresp[0] & ovl_dap_trans_state) ?
        ovl_dap_ahb_hresp_reg : 1'b1));

  assert_always #(0,0,"Invalid 2-cycle Error response on DAP port - HREADY.")
    ovl_dap_resp2(hclk, dreset_n,
      ((mtx_dap_ahb_hreadyout & mtx_dap_ahb_hresp[0] & ovl_dap_trans_state) ?
         ~ovl_dap_ahb_hreadyout_reg : 1'b1));

  assert_never #(0,0,"HRESP is not held for 2 cycles.")
    ovl_dap_resp3(hclk, dreset_n,
      (~ovl_dap_ahb_hreadyout_reg & ovl_dap_ahb_hresp_reg & ~mtx_dap_ahb_hresp[0] &
        ovl_dap_trans_state));



  assert_always #(0,0,"ICode accesses must always be word size.")
    ovl_icode_size_check(hclk, hreset_n,
      ((mtx_ahb_htransi != 2'b00) ? (mtx_ahb_hsizei == 3'b010) : 1'b1));

  assert_never #(0,0,"ICode HADDR must be word aligned.")
    ovl_icode_addr_wordaligned(hclk, hreset_n, (mtx_ahb_haddri[1:0] != 2'b00));

  assert_never #(0,0,"ICode HADDR must be between 0x0 and 0x1FFFFFFF.")
    ovl_icode_addr_valid(hclk, hreset_n,
        ((mtx_ahb_htransi != 2'b00) & (mtx_ahb_haddri >= 32'h20000000)));

  assert_never #(0,0,"ICode burst crosses 1k boundary.")
    ovl_icode_burst_crosses_1k_boundary(hclk, hreset_n,
      ((mtx_ahb_htransi == 2'b11) & (ovl_mtx_ahb_haddri_reg > mtx_ahb_haddri[9:0])));



  assert_always #(0,0,"DCode half-word accesses must be half-word aligned.")
    ovl_dcode_hwalign_check(hclk, hreset_n,
      (((mtx_ahb_htransd != 2'b00) & (mtx_ahb_hsized == 3'b001)) ?
        (mtx_ahb_haddrd[0] == 1'b0) : 1'b1));

  assert_always #(0,0,"DCode word accesses must be word aligned.")
    ovl_dcode_walign_check(hclk, hreset_n,
      (((mtx_ahb_htransd != 2'b00) & (mtx_ahb_hsized == 3'b010)) ?
        (mtx_ahb_haddrd[1:0] == 2'b00) : 1'b1));

  assert_never #(0,0,"DCode HADDR must be between 0x0 and 0x1FFFFFFF.")
    ovl_dcode_addr_valid(hclk, hreset_n,
        ((mtx_ahb_htransd != 2'b00) & (mtx_ahb_haddrd >= 32'h20000000) &
         ~(ovl_suppress_dcode_addr_range_assert_s0 |
           ovl_suppress_dcode_addr_range_assert_s1 |
           ovl_suppress_dcode_addr_range_assert_s2)));

  assert_never #(0,0,"DCode burst crosses 1k boundary.")
    ovl_dcode_burst_crosses_1k_boundary(hclk, hreset_n,
      ((mtx_ahb_htransd == 2'b11) & (ovl_mtx_ahb_haddrd_reg > mtx_ahb_haddrd[9:0])));



  assert_always #(0,0,"SYS half-word accesses must be half-word aligned.")
    ovl_sys_hwalign_check(hclk, hreset_n,
      (((mtx_ahb_htranss != 2'b00) & (mtx_ahb_hsizes == 3'b001)) ?
        (mtx_ahb_haddrs[0] == 1'b0) : 1'b1));

  assert_always #(0,0,"SYS word accesses must be word aligned.")
    ovl_sys_walign_check(hclk, hreset_n,
      (((mtx_ahb_htranss != 2'b00) & (mtx_ahb_hsizes == 3'b010)) ?
        (mtx_ahb_haddrs[1:0] == 2'b00) : 1'b1));

  assert_never #(0,0,
     "HADDRS must be between 0x20000000 - 0xDFFFFFFF, 0xE0100000 - 0xFFFFFFFF.")
       ovl_sys_addr_valid(hclk, hreset_n,
         ((mtx_ahb_htranss != 2'b00) &
           ((mtx_ahb_haddrs <= 32'h1FFFFFFF) |
            ((mtx_ahb_haddrs >= 32'hE0000000) & (mtx_ahb_haddrs <= 32'hE00FFFFF))) &
           ~(ovl_suppress_sys_addr_range_assert_s0 |
             ovl_suppress_sys_addr_range_assert_s1 |
             ovl_suppress_sys_addr_range_assert_s2)));

  assert_never #(0,0,"System burst crosses 1k boundary.")
    ovl_system_burst_crosses_1k_boundary(hclk, hreset_n,
      ((mtx_ahb_htranss == 2'b11) & (ovl_mtx_ahb_haddrs_reg > mtx_ahb_haddrs[9:0])));



  assert_never #(0,0,"PPB HADDR must be between 0xE0000000 and 0xE00FFFFF.")
    ovl_ppb_addr_valid(hclk, hreset_n,
      ((mtx_ppb_htrans != 2'b00) &
        ((mtx_ppb_haddr <= 32'hDFFFFFFF) | (mtx_ppb_haddr >= 32'hE0100000))));

  assert_never #(0,0,"PPB burst crosses 1k boundary.")
    ovl_ppb_burst_crosses_1k_boundary(hclk, hreset_n,
      ((mtx_ppb_htrans == 2'b11) & (ovl_mtx_ppb_haddr_reg > mtx_ppb_haddr[9:0])));



  reg [1:0] ovl_mtx_ahb_hrespi_reg;
  reg       ovl_mtx_ahb_hreadyi_reg;
  reg [1:0] ovl_mtx_ahb_htransi_reg;

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      begin
        ovl_mtx_ahb_hrespi_reg  <= 2'b00;
        ovl_mtx_ahb_hreadyi_reg <= 1'b0;
        ovl_mtx_ahb_htransi_reg <= 2'b00;
      end
    else
      begin
        ovl_mtx_ahb_hrespi_reg  <= mtx_ahb_hrespi_i;
        ovl_mtx_ahb_hreadyi_reg <= mtx_ahb_hreadyi_i;
        ovl_mtx_ahb_htransi_reg <= mtx_ahb_htransi;
      end

  reg [1:0] ovl_mtx_ahb_hrespd_reg;
  reg       ovl_mtx_ahb_hreadyd_reg;
  reg [1:0] ovl_mtx_ahb_htransd_reg;

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      begin
        ovl_mtx_ahb_hrespd_reg  <= 1'b0;
        ovl_mtx_ahb_hreadyd_reg <= 1'b0;
        ovl_mtx_ahb_htransd_reg <= 2'b00;
      end
    else
      begin
        ovl_mtx_ahb_hrespd_reg  <= mtx_ahb_hrespd_i;
        ovl_mtx_ahb_hreadyd_reg <= mtx_ahb_hreadyd_i;
        ovl_mtx_ahb_htransd_reg <= mtx_ahb_htransd;
      end

  reg [1:0] ovl_mtx_ahb_hresps_reg;
  reg       ovl_mtx_ahb_hreadys_reg;
  reg [1:0] ovl_mtx_ahb_htranss_reg;

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      begin
        ovl_mtx_ahb_hresps_reg  <= 1'b0;
        ovl_mtx_ahb_hreadys_reg <= 1'b0;
        ovl_mtx_ahb_htranss_reg <= 2'b00;
      end
    else
      begin
        ovl_mtx_ahb_hresps_reg  <= mtx_ahb_hresps_i;
        ovl_mtx_ahb_hreadys_reg <= mtx_ahb_hreadys_i;
        ovl_mtx_ahb_htranss_reg <= mtx_ahb_htranss;
      end

  reg [1:0] ovl_mtx_ppb_hresp_reg;
  reg       ovl_mtx_ppb_hready_reg;
  reg [1:0] ovl_mtx_ppb_htrans_reg;

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      begin
        ovl_mtx_ppb_hresp_reg  <= 1'b0;
        ovl_mtx_ppb_hready_reg <= 1'b0;
        ovl_mtx_ppb_htrans_reg <= 2'b00;
      end
    else
      begin
        ovl_mtx_ppb_hresp_reg  <= mtx_ppb_hresp_i;
        ovl_mtx_ppb_hready_reg <= mtx_ppb_hready_i;
        ovl_mtx_ppb_htrans_reg <= mtx_ppb_htrans;
      end


  reg  ovl_icode_trans_state;
  wire ovl_nxt_icode_trans_state;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_icode_trans_state <= 1'b0;
    else
      ovl_icode_trans_state <= ovl_nxt_icode_trans_state;

  assign ovl_nxt_icode_trans_state = ((mtx_ahb_htransi != 2'b00) & mtx_ahb_hreadyi_i) |
                                     (ovl_icode_trans_state & ~mtx_ahb_hreadyi_i);

  reg  ovl_dcode_trans_state;
  wire ovl_nxt_dcode_trans_state;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_dcode_trans_state <= 1'b0;
    else
      ovl_dcode_trans_state <= ovl_nxt_dcode_trans_state;

  assign ovl_nxt_dcode_trans_state = ((mtx_ahb_htransd != 2'b00) & mtx_ahb_hreadyd_i) |
                                     (ovl_dcode_trans_state & ~mtx_ahb_hreadyd_i);

  reg  ovl_sys_trans_state;
  wire ovl_nxt_sys_trans_state;

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      ovl_sys_trans_state <= 1'b0;
    else
      ovl_sys_trans_state <= ovl_nxt_sys_trans_state;

  assign ovl_nxt_sys_trans_state = ((mtx_ahb_htranss != 2'b00) & mtx_ahb_hreadys_i) |
                                   (ovl_sys_trans_state & ~mtx_ahb_hreadys_i);

  reg  ovl_ppb_trans_state;
  wire ovl_nxt_ppb_trans_state;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_ppb_trans_state <= 1'b0;
    else
      ovl_ppb_trans_state <= ovl_nxt_ppb_trans_state;

  assign ovl_nxt_ppb_trans_state = ((mtx_ppb_htrans != 2'b00) & mtx_ppb_hready_i) |
                                   (ovl_ppb_trans_state & ~mtx_ppb_hready_i);


  assert_implication #(0,1,
    "mtx_ahb_hreadyi_i should be low in 1st cycle of ERROR/RETRY/SPLIT response")
      ovl_error_ready_constraint_0(hclk, hreset_n,
                                   (((ovl_mtx_ahb_htransi_reg != 2'b00)  &
                                     (ovl_mtx_ahb_hreadyi_reg == 1'b1)   &
                                     (mtx_ahb_hrespi_i        != 2'b00)) |
                                    ((mtx_ahb_hrespi_i        != 2'b00)  &
                                     (ovl_mtx_ahb_hrespi_reg  == 2'b00))),
                                   ~mtx_ahb_hreadyi_i);

  assert_implication #(0,1,
    "mtx_ahb_hreadyd_i should be low in 1st cycle of ERROR/RETRY/SPLIT response")
      ovl_error_ready_constraint_1(hclk, hreset_n,
                                   (((ovl_mtx_ahb_htransd_reg != 2'b00)  &
                                     (ovl_mtx_ahb_hreadyd_reg == 1'b1)   &
                                     (mtx_ahb_hrespd_i        != 2'b00)) |
                                    ((mtx_ahb_hrespd_i        != 2'b00)  &
                                     (ovl_mtx_ahb_hrespd_reg  == 2'b00))),
                                   ~mtx_ahb_hreadyd_i);

  assert_implication #(0,1,
    "mtx_ahb_hreadys_i should be low in 1st cycle of ERROR/RETRY/SPLIT response")
      ovl_error_ready_constraint_2(hclk, dreset_n,
                                   (((ovl_mtx_ahb_htranss_reg != 2'b00)  &
                                     (ovl_mtx_ahb_hreadys_reg == 1'b1)   &
                                     (mtx_ahb_hresps_i        != 2'b00)) |
                                    ((mtx_ahb_hresps_i        != 2'b00)  &
                                     (ovl_mtx_ahb_hresps_reg  == 2'b00))),
                                   ~mtx_ahb_hreadys_i);

  assert_implication #(0,1,
    "mtx_ppb_hready should be low in 1st cycle of ERROR/RETRY/SPLIT response")
      ovl_error_ready_constraint_3(hclk, hreset_n,
                                   (((ovl_mtx_ppb_htrans_reg != 2'b00)  &
                                     (ovl_mtx_ppb_hready_reg == 1'b1)   &
                                     (mtx_ppb_hresp_i        != 2'b00)) |
                                    ((mtx_ppb_hresp_i        != 2'b00)  &
                                     (ovl_mtx_ppb_hresp_reg  == 2'b00))),
                                   ~mtx_ppb_hready_i);


  assert_implication #(0,1,"mtx_ahb_hrespi_i should be OKAY if no transaction occuring")
    ovl_notrans_resp_constraint_0(hclk, hreset_n,
                                  (~ovl_icode_trans_state),
                                  (mtx_ahb_hrespi_i == 2'b00));

  assert_implication #(0,1,"mtx_ahb_hrespd_i should be OKAY if no transaction occuring")
    ovl_notrans_resp_constraint_1(hclk, hreset_n, (~ovl_dcode_trans_state),
      (mtx_ahb_hrespd_i == 2'b00));

  assert_implication #(0,1,"mtx_ahb_hresps_i should be OKAY if no transaction occuring")
    ovl_notrans_resp_constraint_2(hclk, dreset_n, (~ovl_sys_trans_state),
      (mtx_ahb_hresps_i == 2'b00));

  assert_implication #(0,1,"mtx_ppb_hresp_i should be OKAY if no transaction occuring")
    ovl_notrans_resp_constraint_3(hclk, hreset_n, (~ovl_ppb_trans_state),
      (mtx_ppb_hresp_i == 2'b00));


  assert_implication #(0,1,"mtx_ahb_hrespi_i ERROR/RETRY/SPLIT should not change during data phase")
    ovl_resp_hold_constraint_0(hclk, hreset_n,
                               (ovl_icode_trans_state &
                                 ~((ovl_mtx_ahb_htransi_reg != 2'b00) &
                                    ovl_mtx_ahb_hreadyi_reg)          &
                               (ovl_mtx_ahb_hrespi_reg != 2'b00)),
                               (mtx_ahb_hrespi_i == ovl_mtx_ahb_hrespi_reg));

  assert_implication #(0,1,"mtx_ahb_hrespd_i ERROR/RETRY/SPLIT should not change during data phase")
    ovl_resp_hold_constraint_1(hclk, hreset_n,
      (ovl_dcode_trans_state &
        ~((ovl_mtx_ahb_htransd_reg != 2'b00) & ovl_mtx_ahb_hreadyd_reg) &
          (ovl_mtx_ahb_hrespd_reg != 2'b00)), (mtx_ahb_hrespd_i == ovl_mtx_ahb_hrespd_reg));

  assert_implication #(0,1,"mtx_ahb_hresps_i ERROR/RETRY/SPLIT should not change during data phase")
    ovl_resp_hold_constraint_2(hclk, dreset_n,
      (ovl_sys_trans_state &
        ~((ovl_mtx_ahb_htranss_reg != 2'b00) & ovl_mtx_ahb_hreadys_reg) &
          (ovl_mtx_ahb_hresps_reg != 2'b00)), (mtx_ahb_hresps_i == ovl_mtx_ahb_hresps_reg));

  assert_implication #(0,1,
    "mtx_ppb_hresp_i ERROR/RETRY/SPLIT should not change during data phase")
      ovl_resp_hold_constraint_3(hclk, hreset_n,
        (ovl_ppb_trans_state &
          ~((ovl_mtx_ppb_htrans_reg != 2'b00) & ovl_mtx_ppb_hready_reg) &
            (ovl_mtx_ppb_hresp_reg != 2'b00)),
              (mtx_ppb_hresp_i == ovl_mtx_ppb_hresp_reg));


  assert_implication #(0,1,"HRESP should be OKAY on RESET")
    ovl_resp_reset_constraint_0(hclk, dreset_n, (~hreset_n),
      ((mtx_ahb_hrespi_i | mtx_ahb_hrespd_i) == 2'b00));


  `include "cm3_mtx_undefs.v"

`endif

endmodule
