module cm3_dap_ahb_ap_sync
  (// Inputs
   clk, reset_n, d_async_i,
   q_o
  );


  parameter DEBUG_LVL    = 3;  // Debug support level (between 0 and 3)
  parameter GLITCH_CHECK = 0;  // Check for glitches if 1



  input  clk;        // Clock to which q_o output will be synchronized
  input  reset_n;    // reset_n for syncronisation registers
  input  d_async_i;  // Signal to be syncronized

  output q_o;        // Synronized output



  reg    d_sync1;    // 1st stage synchronized version of d_async_i
  reg    q;          // 2nd stage synchronized version of d_async_i

  wire   opt_q;      // Logic removal term for q_o output


`ifdef ARM_DSM

  wire [1:0]    dsm_debug_lvl;


  assign opt_q = (dsm_debug_lvl > 0) ? q : 1'b0;

`else


  assign opt_q = (DEBUG_LVL > 0) ? q : 1'b0;

`endif


  always @ (posedge clk or negedge reset_n)
    if (!reset_n)
      begin
        d_sync1 <= 1'b0;
        q       <= 1'b0;
      end
    else
      begin
        d_sync1 <= d_async_i;
        q       <= d_sync1;
      end



  assign q_o = opt_q;



`ifdef ARM_ASSERT_ON

  assert_never #(0,0,"Glitching input undersireable in handshake.")
    ovl_ap_sync_glitch(clk, reset_n,
      ((GLITCH_CHECK == 1) & ((d_sync1 != d_async_i) & (q != d_sync1))));

  assert_never_unknown #(0,2,0,"Halt X propogation at synchroniser level.")
    ovl_ap_sync_x_check(clk, reset_n, 1'b1, {d_async_i, q});

`endif

endmodule
