`include "cm3_dpu_defs.vp"

module cm3_dpu_16bit_dec
  (// Inputs
   hclk, hreset_n, instr_valid_de_i, instr_de_i, instr_size_de_i,
   instr_16_alu_1_cyc_de_o, instr_16_algn_de_o, instr_16_mul_ilock_de_o,
   instr_16_mul_de_o, instr_16_ls_allow_skid_de_o, instr_16_immed_de_o,
   instr_16_lsm_immed_de_o, instr_16_imm_valid_de_o, instr_16_data_size_de_o,
   instr_16_sign_de_o, instr_16_exit_op_de_o, instr_16_wfi_de_o,
   instr_16_wfe_de_o, instr_16_undef_de_o, instr_16_txev_de_o,
   instr_16_pc_lr_de_o, instr_16_br_t_changer_de_o, instr_16_noncc_br_de_o,
   instr_16_flg_set_de_o, instr_16_it_de_o, instr_16_cps_ctl_de_o,
   instr_16_czb_de_o, instr_16_bkpt_de_o, instr_16_svc_de_o,
   instr_16_cond_code_de_o, instr_16_alu_ctl_de_o, instr_16_lsu_ctl_de_o,
   instr_16_lsu_rd_a_addr_de_o, instr_16_lsu_rd_b_addr_de_o,
   instr_16_rd_a_en_de_o, instr_16_rd_a_addr_de_o, instr_16_rd_b_en_de_o,
   instr_16_rd_b_addr_de_o, instr_16_wr_d_en_de_o, instr_16_wr_d_addr_de_o
   );

  parameter      EXT_PRESENT = 0;             // Extensions present if 1

  input          hclk;
  input          hreset_n;
  input          instr_valid_de_i;            // Instruction in decode is valid
  input [31:0]   instr_de_i;                  // Instruction in decode
  input          instr_size_de_i;             // Size of instruction in decode

  output         instr_16_alu_1_cyc_de_o;     // ALU operation will take 1 cycle
  output         instr_16_algn_de_o;          // Align data
  output         instr_16_mul_ilock_de_o;     // Multiply interlock
  output         instr_16_mul_de_o;           // Multiply decoded
  output [1:0]   instr_16_ls_allow_skid_de_o; // LS pipeline/advance controls
  output [31:0]  instr_16_immed_de_o;         // Immediate value
  output [31:0]  instr_16_lsm_immed_de_o;     // LSM immediate value
  output         instr_16_imm_valid_de_o;     // Immediate valid
  output [1:0]   instr_16_data_size_de_o;     // Data size
  output         instr_16_sign_de_o;          // Signed operation or sign extend
  output         instr_16_exit_op_de_o;       // Valid exit instruction
  output         instr_16_wfe_de_o;           // Instruction is WFE
  output         instr_16_wfi_de_o;           // Instruction is WFI
  output         instr_16_undef_de_o;         // Instruction is an UNDEF
  output         instr_16_txev_de_o;          // Instruction is SEV
  output [1:0]   instr_16_pc_lr_de_o;         // Link LR into PC
  output         instr_16_br_t_changer_de_o;  // Branch changes T bit
  output         instr_16_noncc_br_de_o;      // Non-conditional branch
  output         instr_16_flg_set_de_o;       // Can set flags in IT
  output         instr_16_it_de_o;            // Instruction is IT
  output [2:0]   instr_16_cps_ctl_de_o;       // CPS control
  output         instr_16_czb_de_o;           // Instruction is CZB
  output         instr_16_bkpt_de_o;          // Instruction is BKPT
  output         instr_16_svc_de_o;           // Instruction is SVC
  output [3:0]   instr_16_cond_code_de_o;     // Condition code
  output [20:0]  instr_16_alu_ctl_de_o;       // ALU control
  output [14:0]  instr_16_lsu_ctl_de_o;       // LSU control
  output [3:0]   instr_16_lsu_rd_a_addr_de_o; // RF A port read address for LSU
  output [3:0]   instr_16_lsu_rd_b_addr_de_o; // RF A port read address for LSU
  output         instr_16_rd_a_en_de_o;       // RF A port read enable
  output [3:0]   instr_16_rd_a_addr_de_o;     // RF A port read address
  output         instr_16_rd_b_en_de_o;       // RF B port read enable
  output [3:0]   instr_16_rd_b_addr_de_o;     // RF B port read address
  output         instr_16_wr_d_en_de_o;       // RF D port write enable
  output [3:0]   instr_16_wr_d_addr_de_o;     // RF D port write address

  wire           opt_ext_en;                  // Logic removal for extensions
  wire           i16_adc_de;                  // ADC
  wire           i16_add1_de;                 // ADD (1)
  wire           i16_add2_de;                 // ADD (2)
  wire           i16_add3_de;                 // ADD (3)
  wire           i16_add4_de;                 // ADD (4)
  wire           i16_addpc_de;                // ADD PC
  wire           i16_add5_de;                 // ADD (5)
  wire           i16_add6_de;                 // ADD (6)
  wire           i16_add7_de;                 // ADD (7)
  wire           i16_and_de;                  // AND
  wire           i16_asr1_de;                 // ASR (1)
  wire           i16_asr2_de;                 // ASR (2)
  wire           i16_b1_de;                   // B (1)
  wire           i16_b2_de;                   // B (2)
  wire           i16_bic_de;                  // BIC
  wire           i16_bkpt_de;                 // BKPT
  wire           i16_blx_de;                  // BLX
  wire           i16_cmn_de;                  // CMN
  wire           i16_cmp1_de;                 // CMP (1)
  wire           i16_cmp2_de;                 // CMP (2)
  wire           i16_cmp3_de;                 // CMP (3)
  wire           i16_cps_de;                  // CPS
  wire           i16_czb_de;                  // CZB
  wire           i16_eor_de;                  // EOR
  wire           i16_it_de;                   // IT
  wire           i16_ldmia_de;                // LDMIA
  wire           i16_ldr1_de;                 // LDR (1)
  wire           i16_ldr2_de;                 // LDR (2)
  wire           i16_ldr3_de;                 // LDR (3)
  wire           i16_ldr4_de;                 // LDR (4)
  wire           i16_ldrb1_de;                // LDRB (1)
  wire           i16_ldrb2_de;                // LDRB (2)
  wire           i16_ldrh1_de;                // LDRH (1)
  wire           i16_ldrh2_de;                // LDRH (2)
  wire           i16_ldrsb_de;                // LDRSB
  wire           i16_ldrsh_de;                // LDRSH
  wire           i16_lsl1_de;                 // LSL (1)
  wire           i16_lsl2_de;                 // LSL (2)
  wire           i16_lsr1_de;                 // LSR (1)
  wire           i16_lsr2_de;                 // LSR (2)
  wire           i16_mov1_de;                 // MOV (1)
  wire           i16_mov2_de;                 // MOV (2)
  wire           i16_mov3_de;                 // MOV (3)
  wire           i16_mul_de;                  // MUL
  wire           i16_mvn_de;                  // MVN
  wire           i16_neg_de;                  // NEG
  wire           i16_nop_de;                  // NOP
  wire           i16_orr_de;                  // ORR
  wire           i16_pop_de;                  // POP
  wire           i16_push_de;                 // PUSH
  wire           i16_rev_de;                  // REV
  wire           i16_revsh_de;                // REVSH
  wire           i16_rev16_de;                // REV16
  wire           i16_ror_de;                  // ROR
  wire           i16_sbc_de;                  // SBC
  wire           i16_sev_de;                  // SEV
  wire           i16_stmia_de;                // STMIA
  wire           i16_str1_de;                 // STR (1)
  wire           i16_str2_de;                 // STR (2)
  wire           i16_str3_de;                 // STR (3)
  wire           i16_strb1_de;                // STRB (1)
  wire           i16_strb2_de;                // STRB (2)
  wire           i16_strh1_de;                // STRH (1)
  wire           i16_strh2_de;                // STRH (2)
  wire           i16_sub1_de;                 // SUB (1)
  wire           i16_sub2_de;                 // SUB (2)
  wire           i16_sub3_de;                 // SUB (3)
  wire           i16_sub4_de;                 // SUB (4)
  wire           i16_sxth_de;                 // SXTH
  wire           i16_sxtb_de;                 // SXTB
  wire           i16_svc_de;                  // SVC
  wire           i16_tst_de;                  // TST
  wire           i16_uxth_de;                 // UXTH
  wire           i16_uxtb_de;                 // UXTB
  wire           i16_wfe_de;                  // WFE
  wire           i16_wfi_de;                  // WFI
  wire           i16_alu_bw_and_de;           // ALU bitwise AND operation
  wire           i16_alu_bw_or_de;            // ALU bitwise OR  operation
  wire           i16_alu_bw_eor_de;           // ALU bitwise EOR operation
  wire           i16_alu_shft_de;             // ALU shift operation
  wire           i16_alu_mrs_pri_de;          // ALU MRS to PRIMASK
  wire           i16_alu_mrs_fault_de;        // ALU MRS to FAULTMASK
  wire           i16_alu_mov_norm_de;         // ALU MOV
  wire           i16_alu_mov_rev_de;          // ALU REV
  wire           i16_alu_mul_de;              // ALU MUL
  wire [4:0]     i16_alu_type;                // ALU operation type
  wire [1:0]     i16_flag_wr_de;              // ALU flag writes
  wire           i16_alu_invert_in1;          // ALU invert input 1
  wire           i16_alu_invert_in2;          // ALU invert input 2
  wire           i16_use_adder1;              // ALU use adder input 1
  wire           i16_use_adder2;              // ALU use adder input 2
  wire [1:0]     i16_shft_ctl;                // ALU shifter control
  wire [1:0]     i16_shiftin1_sel;            // ALU shifter input 1 select
  wire [1:0]     i16_shiftin2_sel;            // ALU shifter input 2 select
  wire           i16_use_carry_de;            // Use carry in
  wire [20:0]    instr_16_alu_ctl_de;         // ALU control
  wire           i16_lsu_lss_de;              // LSU load/store single
  wire           i16_lsu_lsm_de;              // LSU load/store multiple
  wire           i16_lsu_valid_de;            // LSU operation valid
  wire           i16_lsu_isld_de;             // LSU operation is a load
  wire [2:0]     i16_lsu_wb_ctl_de;           // LSU write back control
  wire [14:0]    instr_16_lsu_ctl_de;         // LSU control
  wire [1:0]     instr_16_ls_allow_skid_de;   // LS pipeline/advance controls
  wire           instr_16_mul_ilock_de;       // Multiply interlock
  wire           instr_16_mul_de;             // Instruction is a MUL
  wire           instr_16_wfe_de;             // Instruction is a WFE
  wire           instr_16_wfi_de;             // Instruction is a WFI
  wire           instr_16_undef_de;           // Instruction is UNDEF
  wire           instr_16_txev_de;            // Instruction is a SEV
  wire [1:0]     instr_16_pc_lr_de;           // Link LR into PC
  wire           instr_16_br_t_changer_de;    // Branch changes T bit
  wire           instr_16_czb_de;             // Instruction is a CZB
  wire           instr_16_alu_1_cyc_de;       // ALU operation will take 1 cycle
  wire           instr_16_flg_set_de;         // Can set flags in IT
  wire           instr_16_it_de;              // Instruction is a IT
  wire [2:0]     instr_16_cps_ctl_de;         // CPS control
  wire [1:0]     instr_16_data_size_de;       // Data size
  wire           instr_16_sign_de;            // Signed operation or sign extend
  wire           instr_16_exit_op_de;         // Valid exit instruction
  wire           instr_16_algn_de;            // Align data
  wire           instr_16_bkpt_de;            // Instruction is a BKPT
  wire           instr_16_svc_de;             // Instruction is a SVC
  wire [3:0]     instr_16_cond_code_de;       // Condition code
  wire           instr_16_noncc_br_de;        // Non-conditional branch
  wire           instr_16_rd_a_en_de;         // RF A port read enable
  wire           instr_16_rd_b_en_de;         // RF B port read enable
  wire           instr_16_wr_d_en_de;         // RF D port write enable
  wire [18:0]    reg_addr_sel_de;             // RF address mux select
  wire [3:0]     instr_16_lsu_rd_a_addr_de;   // RF A port read address for LSU
  wire [3:0]     instr_16_lsu_rd_b_addr_de;   // RF B port read address for LSU
  wire [3:0]     instr_16_lsu_wr_d_addr_de;   // RF D port write address for LSU
  wire [3:0]     instr_16_rd_a_addr_de;       // RF A port read address
  wire [3:0]     instr_16_rd_b_addr_de;       // RF B port read address
  wire [3:0]     instr_16_wr_d_addr_de;       // RF D port write address
  wire           instr_16_imm_valid_de;       // Immediate valid
  wire [13:0]    reg_immed_sel_de;            // Immediate term mux select
  wire [31:0]    instr_16_immed_de;           // Immediate value
  wire [31:0]    instr_16_lsm_immed_de;       // LSM immediate value

  assign opt_ext_en     = (EXT_PRESENT == 1)    ? 1'b1     : 1'b0;

  assign i16_adc_de   = (instr_de_i[15:6]  == 10'b0100000101);
  assign i16_add1_de  = (instr_de_i[15:9]  == 7'b0001110) & (|instr_de_i[8:6]);
  assign i16_add2_de  = (instr_de_i[15:11] == 5'b00110);
  assign i16_add3_de  = (instr_de_i[15:9]  == 7'b0001100);
  assign i16_add4_de  = ((instr_de_i[15:8] == 8'b01000100) &
                         ({instr_de_i[7],instr_de_i[2:0]} != 4'b1111));
  assign i16_addpc_de = ((instr_de_i[15:8] == 8'b01000100) &
                         ({instr_de_i[7],instr_de_i[2:0]} == 4'b1111));
  assign i16_add5_de  = (instr_de_i[15:11] == 5'b10100);
  assign i16_add6_de  = (instr_de_i[15:11] == 5'b10101);
  assign i16_add7_de  = (instr_de_i[15:7]  == 9'b101100000);
  assign i16_and_de   = (instr_de_i[15:6]  == 10'b0100000000);
  assign i16_asr1_de  = (instr_de_i[15:11] == 5'b00010);
  assign i16_asr2_de  = (instr_de_i[15:6]  == 10'b0100000100);
  assign i16_b1_de    = ((instr_de_i[15:12] == 4'b1101) &
                         (instr_de_i[11:9] != 3'b111));
  assign i16_b2_de    = (instr_de_i[15:11] == 5'b11100);
  assign i16_bic_de   = (instr_de_i[15:6]  == 10'b0100001110);
  assign i16_bkpt_de  = (instr_de_i[15:8]  == 8'b10111110);
  assign i16_blx_de   = (instr_de_i[15:8]  == 8'b01000111);
  assign i16_cmn_de   = (instr_de_i[15:6]  == 10'b0100001011);
  assign i16_cmp1_de  = (instr_de_i[15:11] == 5'b00101);
  assign i16_cmp2_de  = (instr_de_i[15:6]  == 10'b0100001010);
  assign i16_cmp3_de  = (instr_de_i[15:8]  == 8'b01000101);
  assign i16_eor_de   = (instr_de_i[15:6]  == 10'b0100000001);
  assign i16_ldmia_de = (instr_de_i[15:11] == 5'b11001);
  assign i16_ldr1_de  = (instr_de_i[15:11] == 5'b01101);
  assign i16_ldr2_de  = (instr_de_i[15:9]  == 7'b0101100);
  assign i16_ldr3_de  = (instr_de_i[15:11] == 5'b01001);
  assign i16_ldr4_de  = (instr_de_i[15:11] == 5'b10011);
  assign i16_ldrb1_de = (instr_de_i[15:11] == 5'b01111);
  assign i16_ldrb2_de = (instr_de_i[15:9]  == 7'b0101110);
  assign i16_ldrh1_de = (instr_de_i[15:11] == 5'b10001);
  assign i16_ldrh2_de = (instr_de_i[15:9]  == 7'b0101101);
  assign i16_ldrsb_de = (instr_de_i[15:9]  == 7'b0101011);
  assign i16_ldrsh_de = (instr_de_i[15:9]  == 7'b0101111);
  assign i16_lsl1_de  = (instr_de_i[15:11] == 5'b00000) & ~instr_size_de_i;
  assign i16_lsl2_de  = (instr_de_i[15:6]  == 10'b0100000010);
  assign i16_lsr1_de  = (instr_de_i[15:11] == 5'b00001);
  assign i16_lsr2_de  = (instr_de_i[15:6]  == 10'b0100000011);
  assign i16_mov1_de  = (instr_de_i[15:11] == 5'b00100);
  assign i16_mov2_de  = (instr_de_i[15:6]  == 10'b0001110000);
  assign i16_mov3_de  = (instr_de_i[15:8]  == 8'b01000110);
  assign i16_mul_de   = (instr_de_i[15:6]  == 10'b0100001101) & ~opt_ext_en;
  assign i16_mvn_de   = (instr_de_i[15:6]  == 10'b0100001111);
  assign i16_neg_de   = (instr_de_i[15:6]  == 10'b0100001001);
  assign i16_orr_de   = (instr_de_i[15:6]  == 10'b0100001100);
  assign i16_pop_de   = (instr_de_i[15:9]  == 7'b1011110);
  assign i16_push_de  = (instr_de_i[15:9]  == 7'b1011010);
  assign i16_rev_de   = (instr_de_i[15:6]  == 10'b1011101000);
  assign i16_revsh_de = (instr_de_i[15:6]  == 10'b1011101011);
  assign i16_rev16_de = (instr_de_i[15:6]  == 10'b1011101001);
  assign i16_ror_de   = (instr_de_i[15:6]  == 10'b0100000111);
  assign i16_sbc_de   = (instr_de_i[15:6]  == 10'b0100000110);
  assign i16_stmia_de = (instr_de_i[15:11] == 5'b11000);
  assign i16_str1_de  = (instr_de_i[15:11] == 5'b01100);
  assign i16_str2_de  = (instr_de_i[15:9]  == 7'b0101000);
  assign i16_str3_de  = (instr_de_i[15:11] == 5'b10010);
  assign i16_strb1_de = (instr_de_i[15:11] == 5'b01110);
  assign i16_strb2_de = (instr_de_i[15:9]  == 7'b0101010);
  assign i16_strh1_de = (instr_de_i[15:11] == 5'b10000);
  assign i16_strh2_de = (instr_de_i[15:9]  == 7'b0101001);
  assign i16_sub1_de  = (instr_de_i[15:9]  == 7'b0001111);
  assign i16_sub2_de  = (instr_de_i[15:11] == 5'b00111);
  assign i16_sub3_de  = (instr_de_i[15:9]  == 7'b0001101);
  assign i16_sub4_de  = (instr_de_i[15:7]  == 9'b101100001);
  assign i16_sxth_de  = (instr_de_i[15:6]  == 10'b1011001000);
  assign i16_sxtb_de  = (instr_de_i[15:6]  == 10'b1011001001);
  assign i16_svc_de   = (instr_de_i[15:8]  == 8'b11011111);
  assign i16_tst_de   = (instr_de_i[15:6]  == 10'b0100001000);
  assign i16_uxth_de  = (instr_de_i[15:6]  == 10'b1011001010);
  assign i16_uxtb_de  = (instr_de_i[15:6]  == 10'b1011001011);
  assign i16_cps_de   = (instr_de_i[15:5]  == 11'b10110110011);
  assign i16_czb_de   = ((instr_de_i[15:12] == 4'b1011) &
                         ~instr_de_i[10] & instr_de_i[8]);
  assign i16_it_de    = ((instr_de_i[15:8] == 8'b10111111) &
                         (instr_de_i[3:0] != 4'b0000));
  assign i16_wfe_de   = ((instr_de_i[15:8] == 8'b10111111) &
                         (instr_de_i[7:4] == 4'b0010)  &
                         (instr_de_i[3:0] == 4'b0000));
  assign i16_wfi_de   = ((instr_de_i[15:8] == 8'b10111111) &
                         (instr_de_i[7:4] == 4'b0011) &
                         (instr_de_i[3:0] == 4'b0000));
  assign i16_sev_de   = ((instr_de_i[15:8] == 8'b10111111) &
                         (instr_de_i[7:4] == 4'b0100)  &
                         (instr_de_i[3:0] == 4'b0000));
  assign i16_nop_de   = ((instr_de_i[15:8] == 8'b10111111) &
                         ~(i16_wfe_de | i16_wfi_de | i16_sev_de) &
                         (instr_de_i[3:0] == 4'b0000));


  assign i16_alu_bw_and_de    = (i16_and_de  | i16_tst_de  | i16_bic_de);
  assign i16_alu_bw_or_de     = (i16_orr_de);
  assign i16_alu_bw_eor_de    = (i16_eor_de);

  assign i16_alu_shft_de      = (i16_ror_de  | i16_lsr1_de | i16_lsr2_de |
                                 i16_lsl1_de | i16_lsl2_de | i16_asr1_de |
                                 i16_asr2_de);

  assign i16_alu_mrs_pri_de   = (i16_cps_de & instr_de_i[1]);

  assign i16_alu_mrs_fault_de = (i16_cps_de & instr_de_i[0]);

  assign i16_alu_mov_norm_de  = (i16_mov1_de | i16_mov3_de | i16_mvn_de  |
                                 i16_blx_de  | i16_sxth_de | i16_sxtb_de |
                                 i16_uxth_de | i16_uxtb_de);

  assign i16_alu_mov_rev_de   = (i16_rev_de  | i16_revsh_de | i16_rev16_de);

  assign i16_alu_mul_de       = (i16_mul_de);




  assign i16_alu_type[4] = (i16_alu_mov_norm_de  | i16_alu_mov_rev_de   |
                            i16_alu_mul_de);

  assign i16_alu_type[3] = (i16_alu_mrs_pri_de   | i16_alu_mrs_fault_de |
                            i16_alu_mul_de);

  assign i16_alu_type[2] = (i16_alu_shft_de      | i16_alu_mrs_pri_de   |
                            i16_alu_mrs_fault_de | i16_alu_mov_norm_de  |
                            i16_alu_mov_rev_de);

  assign i16_alu_type[1] = (i16_alu_bw_or_de     | i16_alu_bw_eor_de    |
                            i16_alu_mrs_fault_de | i16_alu_mov_rev_de   |
                            i16_alu_mul_de);

  assign i16_alu_type[0] = (i16_alu_bw_and_de    | i16_alu_bw_eor_de    |
                            i16_alu_mrs_pri_de   | i16_alu_mul_de);


  assign i16_flag_wr_de[1] = (i16_adc_de  | i16_add1_de | i16_add2_de |
                              i16_add3_de | i16_cmn_de  | i16_cmp1_de |
                              i16_cmp2_de | i16_cmp3_de | i16_mov2_de |
                              i16_neg_de  | i16_sbc_de  | i16_sub1_de |
                              i16_sub2_de | i16_sub3_de | i16_asr1_de |
                              i16_asr2_de | i16_lsl1_de | i16_lsl2_de |
                              i16_lsr1_de | i16_lsr2_de | i16_ror_de);

  assign i16_flag_wr_de[0] = (i16_adc_de  | i16_add1_de | i16_add2_de |
                              i16_add3_de | i16_cmn_de  | i16_cmp1_de |
                              i16_cmp2_de | i16_cmp3_de | i16_mov2_de |
                              i16_neg_de  | i16_sbc_de  | i16_sub1_de |
                              i16_sub2_de | i16_sub3_de | i16_and_de  |
                              i16_bic_de  | i16_eor_de  | i16_mov1_de |
                              i16_mul_de  | i16_mvn_de  | i16_orr_de  |
                              i16_tst_de);

  assign i16_alu_invert_in1 = 1'b0;

  assign i16_alu_invert_in2 = (i16_neg_de  | i16_sub1_de | i16_sub2_de |
                               i16_sub3_de | i16_sub4_de | i16_sbc_de  |
                               i16_cmp1_de | i16_bic_de  | i16_cmp2_de |
                               i16_cmp3_de | i16_mvn_de);

  assign i16_use_adder1 = (i16_sub1_de | i16_sub2_de | i16_sub3_de |
                           i16_sub4_de | i16_sbc_de  | i16_cmp1_de |
                           i16_cmp2_de | i16_cmp3_de | i16_add1_de |
                           i16_add2_de | i16_add3_de | i16_add4_de |
                           i16_add5_de | i16_add6_de | i16_add7_de |
                           i16_adc_de  | i16_cmn_de  | i16_mov2_de |
                           i16_b1_de   | i16_b2_de   | i16_czb_de);

  assign i16_use_adder2 = (i16_use_adder1 | i16_neg_de | i16_mvn_de);


  assign i16_shft_ctl[1] = (i16_asr1_de | i16_asr2_de | i16_ror_de);

  assign i16_shft_ctl[0] = (i16_lsr1_de | i16_lsr2_de | i16_ror_de);


  assign i16_shiftin1_sel[1] = instr_16_alu_1_cyc_de | i16_blx_de | i16_mov3_de;

  assign i16_shiftin1_sel[0] = 1'b0;


  assign i16_shiftin2_sel[1] = (i16_lsl2_de | i16_lsr2_de | i16_asr2_de |
                                i16_ror_de);

  assign i16_shiftin2_sel[0] = (i16_asr1_de | i16_lsl1_de | i16_lsr1_de);

  assign i16_use_carry_de    = i16_adc_de  | i16_sbc_de;


  assign instr_16_alu_ctl_de = { i16_use_carry_de, i16_flag_wr_de, i16_alu_type,
                                 1'b0, i16_shiftin2_sel, i16_shiftin1_sel,
                                 i16_shft_ctl, 2'b00, i16_use_adder2,
                                 i16_use_adder1, i16_alu_invert_in2,
                                 i16_alu_invert_in1 };


  assign i16_lsu_lss_de = (i16_ldr1_de  | i16_ldr2_de  | i16_ldr3_de  |
                           i16_ldr4_de  | i16_ldrb1_de | i16_ldrb2_de |
                           i16_ldrh1_de | i16_ldrh2_de | i16_ldrsb_de |
                           i16_ldrsh_de | i16_str1_de  | i16_str2_de  |
                           i16_str3_de  | i16_strb1_de | i16_strb2_de |
                           i16_strh1_de | i16_strh2_de);

  assign i16_lsu_lsm_de = (((i16_ldmia_de | i16_stmia_de) & (|instr_de_i[7:0])) |
                           ((i16_pop_de   | i16_push_de)  & (|instr_de_i[8:0])));

  assign i16_lsu_valid_de = (i16_addpc_de | i16_lsu_lsm_de | i16_lsu_lss_de);

  assign i16_lsu_isld_de  = (i16_ldr1_de  | i16_ldr2_de  | i16_ldr3_de  |
                             i16_ldr4_de  | i16_ldrb1_de | i16_ldrb2_de |
                             i16_ldrh1_de | i16_ldrh2_de | i16_ldrsb_de |
                             i16_ldrsh_de | i16_ldmia_de | i16_pop_de);

  assign i16_lsu_wb_ctl_de[0] = (i16_ldmia_de | i16_stmia_de |
                                 i16_pop_de   | i16_push_de);

  assign i16_lsu_wb_ctl_de[1] = (i16_ldmia_de | i16_stmia_de | i16_pop_de);

  assign i16_lsu_wb_ctl_de[2] = (i16_lsu_lss_de | i16_ldmia_de | i16_stmia_de |
                                 i16_pop_de);


  assign instr_16_lsu_ctl_de = { 3'b000, i16_lsu_wb_ctl_de, 2'b00,
                                 i16_lsu_isld_de, i16_lsu_lsm_de, 1'b0,
                                 i16_lsu_lss_de, 1'b0, i16_addpc_de,
                                 i16_lsu_valid_de };

  assign instr_16_ls_allow_skid_de = { // Next opcode not taken. Always taken
                                       ~i16_bkpt_de & ~i16_czb_de & ~i16_it_de,
                                       i16_nop_de };

  assign instr_16_mul_ilock_de = i16_b1_de;

  assign instr_16_mul_de = i16_mul_de;

  assign instr_16_wfe_de = i16_wfe_de;

  assign instr_16_wfi_de = i16_wfi_de;

  assign instr_16_undef_de = ~(i16_adc_de   | i16_add1_de  | i16_add2_de  |
                               i16_add3_de  | i16_add4_de  | i16_add5_de  |
                               i16_add6_de  | i16_add7_de  | i16_and_de   |
                               i16_asr1_de  | i16_asr2_de  | i16_b1_de    |
                               i16_b2_de    | i16_bic_de   | i16_bkpt_de  |
                               i16_blx_de   | i16_cmn_de   | i16_cmp1_de  |
                               i16_cmp2_de  | i16_cmp3_de  | i16_eor_de   |
                               i16_ldmia_de | i16_ldr1_de  | i16_ldr2_de  |
                               i16_ldr3_de  | i16_ldr4_de  | i16_ldrb1_de |
                               i16_ldrb2_de | i16_ldrh1_de | i16_ldrh2_de |
                               i16_ldrsb_de | i16_ldrsh_de | i16_lsl1_de  |
                               i16_lsl2_de  | i16_lsr1_de  | i16_lsr2_de  |
                               i16_mov1_de  | i16_mov2_de  | i16_mov3_de  |
                               i16_mul_de   | i16_mvn_de   | i16_neg_de   |
                               i16_orr_de   | i16_pop_de   | i16_push_de  |
                               i16_rev_de   | i16_revsh_de | i16_rev16_de |
                               i16_ror_de   | i16_sbc_de   | i16_stmia_de |
                               i16_str1_de  | i16_str2_de  | i16_str3_de  |
                               i16_strb1_de | i16_strb2_de | i16_strh1_de |
                               i16_strh2_de | i16_sub1_de  | i16_sub2_de  |
                               i16_sub3_de  | i16_sub4_de  | i16_sxth_de  |
                               i16_sxtb_de  | i16_svc_de   | i16_tst_de   |
                               i16_uxth_de  | i16_uxtb_de  | i16_czb_de   |
                               i16_it_de    | i16_wfe_de   | i16_wfi_de   |
                               i16_sev_de   | i16_nop_de   | i16_cps_de   |
                               i16_addpc_de);

  assign instr_16_txev_de = i16_sev_de;

  assign instr_16_pc_lr_de  = {i16_blx_de & instr_de_i[7],
                               i16_blx_de & instr_de_i[7]};

  assign instr_16_br_t_changer_de = i16_blx_de | i16_pop_de;

  assign instr_16_czb_de = i16_czb_de;

  assign instr_16_alu_1_cyc_de = (// Single Cycle ALU
                                  i16_add1_de  | i16_add2_de  | i16_add3_de |
                                  i16_add4_de  | i16_add5_de  | i16_add6_de |
                                  i16_add7_de  | i16_adc_de   | i16_cmn_de  |
                                  i16_mov2_de  | i16_and_de   | i16_tst_de  |
                                  i16_bic_de   | i16_eor_de   | i16_mvn_de  |
                                  i16_neg_de   | i16_orr_de   | i16_cmp1_de |
                                  i16_cmp2_de  | i16_cmp3_de  | i16_sub1_de |
                                  i16_sub2_de  | i16_sub3_de  | i16_sub4_de |
                                  i16_asr1_de  | i16_asr2_de  | i16_lsl1_de |
                                  i16_lsl2_de  | i16_lsr1_de  | i16_lsr2_de |
                                  i16_sbc_de   | i16_mov1_de  | i16_rev_de  |
                                  i16_ror_de   | i16_sxth_de  | i16_sxtb_de |
                                  i16_uxth_de  | i16_uxtb_de  | i16_nop_de  |
                                  i16_revsh_de | i16_rev16_de | i16_mul_de  |
                                  (i16_mov3_de &
                                   {instr_de_i[7],instr_de_i[2:0]}!=4'b1111));

  assign instr_16_flg_set_de = (i16_cmn_de | i16_cmp1_de | i16_cmp2_de |
                                i16_cmp3_de | i16_tst_de);

  assign instr_16_it_de      = i16_it_de;

  assign instr_16_cps_ctl_de = ({3{i16_cps_de}} &
                                {// PRIMASK
                                 instr_de_i[1],
                                 instr_de_i[0],
                                 instr_de_i[4]});


  assign instr_16_data_size_de[1] = (i16_ldmia_de | i16_pop_de   | i16_push_de  |
                                     i16_stmia_de | i16_ldr1_de  | i16_ldr2_de  |
                                     i16_ldr3_de  | i16_ldr4_de  | i16_str1_de  |
                                     i16_str2_de  | i16_str3_de  | i16_mov1_de  |
                                     i16_mov2_de  | i16_mov3_de  | i16_blx_de   |
                                     i16_mul_de   | i16_mvn_de);

  assign instr_16_data_size_de[0] = (i16_ldrh1_de | i16_ldrh2_de | i16_ldrsh_de |
                                     i16_strh1_de | i16_strh2_de | i16_revsh_de |
                                     i16_rev16_de | i16_sxth_de  | i16_uxth_de);

  assign instr_16_sign_de    = (i16_ldrsb_de | i16_ldrsh_de | i16_revsh_de |
                                i16_sxth_de  | i16_sxtb_de );

  assign instr_16_exit_op_de = (i16_blx_de & ~instr_de_i[7]) | i16_pop_de;

  assign instr_16_algn_de    = i16_add5_de | i16_ldr3_de;

  assign instr_16_bkpt_de    = i16_bkpt_de;
  assign instr_16_svc_de     = i16_svc_de;

  assign instr_16_cond_code_de = (({4{i16_b1_de}} & instr_de_i[11:8]) |
                                  ({4{i16_czb_de}} & {3'b000,instr_de_i[11]}));

  assign instr_16_noncc_br_de  = i16_b1_de | i16_czb_de;

  assign instr_16_rd_a_en_de = (i16_adc_de   | i16_add1_de  | i16_add2_de  |
                                i16_add3_de  | i16_add4_de  | i16_add5_de  |
                                i16_add6_de  | i16_add7_de  | i16_and_de   |
                                i16_asr2_de  | i16_b1_de    | i16_b2_de    |
                                i16_bic_de   | i16_cmn_de   | i16_cmp1_de  |
                                i16_cmp2_de  | i16_cmp3_de  | i16_eor_de   |
                                i16_ldmia_de | i16_ldr1_de  | i16_ldr4_de  |
                                i16_ldrh1_de | i16_ldr2_de  | i16_ldr3_de  |
                                i16_ldrb1_de | i16_ldrb2_de | i16_ldrh2_de |
                                i16_ldrsb_de | i16_ldrsh_de | i16_lsl2_de  |
                                i16_lsr2_de  | i16_mov2_de  | i16_mul_de   |
                                i16_orr_de   | i16_pop_de   | i16_push_de  |
                                i16_sub1_de  | i16_sub2_de  | i16_sub3_de  |
                                i16_ror_de   | i16_sbc_de   | i16_tst_de   |
                                i16_stmia_de | i16_str1_de  | i16_str2_de  |
                                i16_str3_de  | i16_strb1_de | i16_strb2_de |
                                i16_strh1_de | i16_strh2_de | i16_sub4_de  |
                                i16_czb_de   | i16_addpc_de);

  assign instr_16_rd_b_en_de = (i16_adc_de   | i16_add3_de  | i16_add4_de  |
                                i16_and_de   | i16_asr2_de  | i16_bic_de   |
                                i16_blx_de   | i16_cmn_de   | i16_cmp2_de  |
                                i16_cmp3_de  | i16_eor_de   | i16_ldr2_de  |
                                i16_ldrb2_de | i16_ldrh2_de | i16_ldrsb_de |
                                i16_ldrsh_de | i16_lsl2_de  | i16_lsr2_de  |
                                i16_mov3_de  | i16_mul_de   | i16_mvn_de   |
                                i16_neg_de   | i16_orr_de   | i16_ror_de   |
                                i16_sbc_de   | i16_tst_de   | i16_str1_de  |
                                i16_strb1_de | i16_strh1_de | i16_str3_de  |
                                i16_sxth_de  | i16_sub3_de  | i16_sxtb_de  |
                                i16_uxth_de  | i16_uxtb_de  | i16_rev_de   |
                                i16_revsh_de | i16_rev16_de | i16_czb_de   |
                                i16_addpc_de | i16_asr1_de  | i16_lsl1_de  |
                                i16_lsr1_de);

  assign instr_16_wr_d_en_de = (i16_adc_de   | i16_add1_de  | i16_add2_de  |
                                i16_add3_de  | i16_add4_de  | i16_add5_de  |
                                i16_add6_de  | i16_add7_de  | i16_and_de   |
                                i16_asr1_de  | i16_asr2_de  | i16_b1_de    |
                                i16_b2_de    | i16_bic_de   | i16_blx_de   |
                                i16_eor_de   | i16_lsl1_de  | i16_lsr1_de  |
                                i16_lsl2_de  | i16_lsr2_de  | i16_mov1_de  |
                                i16_mov2_de  | i16_mov3_de  | i16_mul_de   |
                                i16_mvn_de   | i16_neg_de   | i16_orr_de   |
                                i16_rev_de   | i16_revsh_de | i16_rev16_de |
                                i16_ror_de   | i16_sbc_de   | i16_sub1_de  |
                                i16_sub2_de  | i16_sub3_de  | i16_sub4_de  |
                                i16_sxth_de  | i16_sxtb_de  | i16_uxth_de  |
                                i16_uxtb_de  | i16_czb_de);

  assign reg_addr_sel_de = {// Add/subtract register
                            (i16_add3_de  | i16_sub3_de),
                            (i16_add1_de  | i16_mov2_de | i16_sub1_de),
                            (i16_rev_de   | i16_revsh_de | i16_rev16_de |
                             i16_sxth_de  | i16_sxtb_de  | i16_uxth_de  |
                             i16_uxtb_de  | i16_neg_de   | i16_mvn_de   |
                             i16_sbc_de   | i16_adc_de   | i16_and_de   |
                             i16_bic_de   | i16_cmn_de   | i16_cmp2_de  |
                             i16_eor_de   | i16_mul_de   | i16_orr_de   |
                             i16_tst_de),
                            (i16_asr1_de  | i16_lsl1_de  | i16_lsr1_de),
                            (i16_add2_de  | i16_cmp1_de  | i16_mov1_de  |
                             i16_sub2_de),
                            (i16_asr2_de  | i16_lsl2_de  | i16_lsr2_de  |
                             i16_ror_de),
                            i16_blx_de,
                            (i16_add4_de  | i16_addpc_de | i16_cmp3_de  |
                             i16_mov3_de),
                            i16_ldr3_de,
                            (i16_ldr2_de  | i16_ldrb2_de | i16_ldrh2_de |
                             i16_ldrsb_de | i16_ldrsh_de | i16_str2_de  |
                             i16_strb2_de | i16_strh2_de),
                            (i16_ldr1_de  | i16_ldrb1_de | i16_str1_de  |
                             i16_strb1_de | i16_ldrh1_de | i16_strh1_de),
                            (i16_ldr4_de  | i16_str3_de),
                            (i16_ldmia_de | i16_stmia_de),
                            (i16_push_de  | i16_pop_de),
                            i16_add5_de,
                            i16_add6_de,
                            (i16_add7_de  | i16_sub4_de),
                            (i16_b1_de    | i16_b2_de),
                            i16_czb_de };

  assign {  instr_16_lsu_rd_a_addr_de,
            instr_16_lsu_rd_b_addr_de,
            instr_16_lsu_wr_d_addr_de  } =

        ({12{reg_addr_sel_de[10]}} & { 4'b1111,
                                       4'b1111,
                                       1'b0,instr_de_i[10:8]         }) |
        ({12{reg_addr_sel_de[9]}} & { 1'b0,instr_de_i[5:3],
                                       1'b0,instr_de_i[8:6],
                                       1'b0,instr_de_i[2:0]          }) |
        ({12{reg_addr_sel_de[8]}} & { 1'b0,instr_de_i[5:3],
                                      1'b0,instr_de_i[2:0],
                                      1'b0,instr_de_i[2:0]           }) |
        ({12{reg_addr_sel_de[7]}} & { 4'b1101,
                                      1'b0,instr_de_i[10:8],
                                      1'b0,instr_de_i[10:8]          }) |
        ({12{reg_addr_sel_de[6]}} & { 1'b0,instr_de_i[10:8],
                                      `CORE_UNUSED,
                                      `CORE_UNUSED                   }) |
        ({12{reg_addr_sel_de[5]}} & { 4'b1101,
                                      `CORE_UNUSED,
                                      4'b1101                        });

  assign {  instr_16_rd_a_addr_de,
            instr_16_rd_b_addr_de,
            instr_16_wr_d_addr_de  } =
         ({12{reg_addr_sel_de[18]}} & { 1'b0,instr_de_i[5:3],
                                        1'b0,instr_de_i[8:6],
                                        1'b0,instr_de_i[2:0]          }) |
         ({12{reg_addr_sel_de[17]}} & { 1'b0,instr_de_i[5:3],
                                        `CORE_UNUSED,
                                        1'b0,instr_de_i[2:0]          }) |
         ({12{reg_addr_sel_de[16]}} & { 1'b0,instr_de_i[2:0],
                                        1'b0,instr_de_i[5:3],
                                        1'b0,instr_de_i[2:0]          }) |
         ({12{reg_addr_sel_de[15]}} & { `CORE_UNUSED,
                                        1'b0,instr_de_i[5:3],
                                        1'b0,instr_de_i[2:0]          }) |
         ({12{reg_addr_sel_de[14]}} & { 1'b0,instr_de_i[10:8],
                                        `CORE_UNUSED,
                                        1'b0,instr_de_i[10:8]         }) |
         ({12{reg_addr_sel_de[13]}} & { 1'b0,instr_de_i[5:3],
                                        1'b0,instr_de_i[2:0],
                                        1'b0,instr_de_i[2:0]         }) |
         ({12{reg_addr_sel_de[12]}} & { `CORE_UNUSED,
                                        instr_de_i[6:3],
                                        4'b1111                       }) |
         ({12{reg_addr_sel_de[11]}} & { instr_de_i[7],instr_de_i[2:0],
                                        instr_de_i[6],instr_de_i[5:3],
                                        instr_de_i[7],instr_de_i[2:0] }) |
         ({12{|reg_addr_sel_de[10:5]}} & { instr_16_lsu_rd_a_addr_de,
                                           instr_16_lsu_rd_b_addr_de,
                                           instr_16_lsu_wr_d_addr_de  }) |
         ({12{reg_addr_sel_de[4]}} & { 4'b1111,
                                       `CORE_UNUSED,
                                       1'b0,instr_de_i[10:8]          }) |
         ({12{reg_addr_sel_de[3]}} & { 4'b1101,
                                       `CORE_UNUSED,
                                       1'b0,instr_de_i[10:8]          }) |
         ({12{reg_addr_sel_de[2]}} & { 4'b1101,
                                       `CORE_UNUSED,
                                       4'b1101                        }) |

         ({12{reg_addr_sel_de[1]}} & { 4'b1111,
                                       4'b1111,
                                       4'b1111                        }) |
         ({12{reg_addr_sel_de[0]}} & { 4'b1111,
                                       1'b0,instr_de_i[2:0],
                                       4'b1111                        });

  assign instr_16_imm_valid_de = (i16_add1_de  | i16_add2_de  | i16_add5_de  |
                                  i16_add6_de  | i16_add7_de  | i16_asr1_de  |
                                  i16_b1_de    | i16_b2_de    | i16_cmp1_de  |
                                  i16_ldr1_de  | i16_ldr3_de  | i16_ldr4_de  |
                                  i16_ldrb1_de | i16_ldrh1_de | i16_lsl1_de  |
                                  i16_lsr1_de  | i16_mov1_de  | i16_mov2_de  |
                                  i16_str1_de  | i16_str3_de  | i16_strb1_de |
                                  i16_strh1_de | i16_sub1_de  | i16_sub2_de  |
                                  i16_ldmia_de | i16_stmia_de | i16_push_de  |
                                  i16_pop_de   | i16_sub4_de  | i16_czb_de);

  assign reg_immed_sel_de = {
        (i16_add1_de  | i16_mov2_de  | i16_sub1_de),
        (i16_asr1_de  | i16_lsl1_de  | i16_lsr1_de),
        (i16_add2_de  | i16_cmp1_de  | i16_mov1_de | i16_sub2_de),
        (i16_ldr3_de),
        (i16_ldr1_de  | i16_ldrb1_de | i16_str1_de | i16_strb1_de),
        (i16_ldrh1_de | i16_strh1_de),
        (i16_ldr4_de  | i16_str3_de),
        (i16_add5_de  | i16_add6_de),
        (i16_add7_de  | i16_sub4_de),
        i16_b1_de,
        i16_b2_de,
        (i16_ldmia_de | i16_stmia_de),
        (i16_push_de  | i16_pop_de),
        i16_czb_de };

  assign instr_16_immed_de =
        ({32{reg_immed_sel_de[13]}} & {{29{1'b0}},instr_de_i[8:6]})        |
        ({32{reg_immed_sel_de[12]}} & {{27{1'b0}},instr_de_i[10:6]})       |
        ({32{reg_immed_sel_de[11]}} & {{24{1'b0}},instr_de_i[7:0]})        |
        ({32{reg_immed_sel_de[10]}} & {{22{1'b0}},instr_de_i[7:0],2'b00})  |
        ({32{reg_immed_sel_de[9] &  instr_de_i[12]}} &
                                      {{27{1'b0}},instr_de_i[10:6]})       |
        ({32{reg_immed_sel_de[9] & ~instr_de_i[12]}} &
                                      {{25{1'b0}},instr_de_i[10:6],2'b00}) |
        ({32{reg_immed_sel_de[8]}} & {{26{1'b0}},instr_de_i[10:6],1'b0})   |
        ({32{reg_immed_sel_de[7]}} & {{22{1'b0}},instr_de_i[7:0],2'b00})   |
        ({32{reg_immed_sel_de[6]}} & {{22{1'b0}},instr_de_i[7:0],2'b00})   |
        ({32{reg_immed_sel_de[5]}} & {{23{1'b0}},instr_de_i[6:0],2'b00})   |
        ({32{reg_immed_sel_de[4]}} & {{23{instr_de_i[7]}},instr_de_i[7:0],1'b0})   |
        ({32{reg_immed_sel_de[3]}} & {{20{instr_de_i[10]}},instr_de_i[10:0],1'b0}) |
        ({32{|reg_immed_sel_de[2:1]}} & instr_16_lsm_immed_de)             |
        ({32{reg_immed_sel_de[0]}} & {{25{1'b0}},instr_de_i[9],instr_de_i[7:3],1'b0});

  assign instr_16_lsm_immed_de = (// Load/store multiple
                                  ({32{reg_immed_sel_de[2]}} &
                                   {{24{1'b0}},instr_de_i[7:0]}) |
                                  ({32{reg_immed_sel_de[1]}} &
                                   {{16{1'b0}},instr_de_i[8]&instr_de_i[11],
                                    instr_de_i[8] & ~instr_de_i[11],{6{1'b0}},
                                    instr_de_i[7:0]}));

  assign instr_16_alu_1_cyc_de_o     = instr_16_alu_1_cyc_de;
  assign instr_16_algn_de_o          = instr_16_algn_de;
  assign instr_16_mul_ilock_de_o     = instr_16_mul_ilock_de;
  assign instr_16_mul_de_o           = instr_16_mul_de;
  assign instr_16_ls_allow_skid_de_o = instr_16_ls_allow_skid_de;
  assign instr_16_immed_de_o         = instr_16_immed_de;
  assign instr_16_lsm_immed_de_o     = instr_16_lsm_immed_de;
  assign instr_16_imm_valid_de_o     = instr_16_imm_valid_de;
  assign instr_16_data_size_de_o     = instr_16_data_size_de;
  assign instr_16_sign_de_o          = instr_16_sign_de;
  assign instr_16_exit_op_de_o       = instr_16_exit_op_de;
  assign instr_16_wfe_de_o           = instr_16_wfe_de;
  assign instr_16_wfi_de_o           = instr_16_wfi_de;
  assign instr_16_undef_de_o         = instr_16_undef_de;
  assign instr_16_txev_de_o          = instr_16_txev_de;
  assign instr_16_pc_lr_de_o         = instr_16_pc_lr_de;
  assign instr_16_br_t_changer_de_o  = instr_16_br_t_changer_de;
  assign instr_16_noncc_br_de_o      = instr_16_noncc_br_de;
  assign instr_16_flg_set_de_o       = instr_16_flg_set_de;
  assign instr_16_it_de_o            = instr_16_it_de;
  assign instr_16_cps_ctl_de_o       = instr_16_cps_ctl_de;
  assign instr_16_czb_de_o           = instr_16_czb_de;
  assign instr_16_bkpt_de_o          = instr_16_bkpt_de;
  assign instr_16_svc_de_o           = instr_16_svc_de;
  assign instr_16_cond_code_de_o     = instr_16_cond_code_de;
  assign instr_16_alu_ctl_de_o       = instr_16_alu_ctl_de;
  assign instr_16_lsu_ctl_de_o       = instr_16_lsu_ctl_de;
  assign instr_16_lsu_rd_a_addr_de_o = instr_16_lsu_rd_a_addr_de;
  assign instr_16_lsu_rd_b_addr_de_o = instr_16_lsu_rd_b_addr_de;
  assign instr_16_rd_a_en_de_o       = instr_16_rd_a_en_de;
  assign instr_16_rd_a_addr_de_o     = instr_16_rd_a_addr_de;
  assign instr_16_rd_b_en_de_o       = instr_16_rd_b_en_de;
  assign instr_16_rd_b_addr_de_o     = instr_16_rd_b_addr_de;
  assign instr_16_wr_d_en_de_o       = instr_16_wr_d_en_de;
  assign instr_16_wr_d_addr_de_o     = instr_16_wr_d_addr_de;


`ifdef ARM_ASSERT_ON

  assert_zero_one_hot #(0,76,0,"OpCodes must be exclusive")
    ovl_opcode_excl(hclk, hreset_n, {76{instr_valid_de_i}} &
                    {i16_adc_de,   i16_add1_de,  i16_add2_de,  i16_add3_de,
                     i16_add4_de,  i16_add5_de,  i16_add6_de,  i16_add7_de,
                     i16_and_de,   i16_asr1_de,  i16_asr2_de,  i16_addpc_de,
                     i16_b1_de,    i16_b2_de,    i16_bic_de,   i16_bkpt_de,
                     i16_blx_de,   i16_cmn_de,   i16_cmp1_de,  i16_cmp2_de,
                     i16_cmp3_de,  i16_eor_de,   i16_ldmia_de, i16_ldr1_de,
                     i16_ldr2_de,  i16_ldr3_de,  i16_ldr4_de,  i16_ldrb1_de,
                     i16_ldrb2_de, i16_ldrh1_de, i16_ldrh2_de, i16_ldrsb_de,
                     i16_ldrsh_de, i16_lsl1_de,  i16_lsl2_de,  i16_lsr1_de,
                     i16_lsr2_de,  i16_mov1_de,  i16_mov2_de,  i16_mov3_de,
                     i16_mul_de,   i16_mvn_de,   i16_neg_de,   i16_orr_de,
                     i16_pop_de,   i16_push_de,  i16_rev_de,   i16_revsh_de,
                     i16_rev16_de, i16_ror_de,   i16_sbc_de,   i16_stmia_de,
                     i16_str1_de,  i16_str2_de,  i16_str3_de,  i16_strb1_de,
                     i16_strb2_de, i16_strh1_de, i16_strh2_de, i16_sub1_de,
                     i16_sub2_de,  i16_sub3_de,  i16_sub4_de,  i16_sxth_de,
                     i16_sxtb_de,  i16_svc_de,   i16_tst_de,   i16_uxth_de,
                     i16_uxtb_de,  i16_czb_de,   i16_it_de,    i16_wfe_de,
                     i16_wfi_de,   i16_sev_de,   i16_nop_de,   i16_cps_de});

  assert_zero_one_hot #(0,19,0,"RF address selector should be exclusive")
    ovl_addr_excl(hclk, hreset_n, (reg_addr_sel_de &
                                   {19{instr_valid_de_i}}));

  assert_zero_one_hot #(0,14,0, "RF immediate selector should be exclusive")
    ovl_immed_excl(hclk, hreset_n, (reg_immed_sel_de &
                                    {14{instr_valid_de_i}}));

`endif

`include "cm3_dpu_undefs.vp"

endmodule
