`include "cm3_dpu_defs.vp"

module cm3_dpu_32bit_dec
  (// Inputs
   hclk, hreset_n, instr_valid_de_i, instr_32_de_i, instr_privileged_de_i,
   e_shft_inlinec_de_i,
   instr_32_alu_1_cyc_de_o, instr_32_algn_de_o, instr_32_mla_de_o,
   instr_32_mul_ilock_de_o, instr_32_ls_single_wb_de_o,
   instr_32_ls_allow_skid_de_o, instr_32_lsd_baseinlist_de_o,
   instr_32_immed_de_o, instr_32_lsm_immed_de_o, instr_32_imm_valid_de_o,
   instr_32_data_size_de_o, instr_32_sign_de_o, instr_32_exit_op_de_o,
   instr_32_undef_de_o, instr_32_txev_de_o, instr_32_pc_lr_de_o,
   instr_32_isb_de_o, instr_32_br_t_changer_de_o, instr_32_noncc_br_de_o,
   instr_32_flg_set_de_o, instr_32_msr_ctl_de_o, instr_32_barrier_de_o,
   instr_32_cond_code_de_o, instr_32_copro_de_o, instr_32_wfe_de_o,
   instr_32_wfi_de_o, instr_32_has_shft_imm_de_o, instr_32_alu_ctl_de_o,
   instr_32_lsu_ctl_de_o, instr_32_sp_rd_de_o, instr_32_sp_wr_de_o,
   instr_32_lsu_rd_a_addr_de_o, instr_32_lsu_rd_b_addr_de_o,
   instr_32_rd_a_en_de_o, instr_32_rd_a_addr_de_o, instr_32_rd_b_en_de_o,
   instr_32_rd_b_addr_de_o, instr_32_rd_c_addr_de_o, instr_32_wr_d_en_de_o,
   instr_32_wr_d_addr_de_o
   );

  parameter      EXT_PRESENT = 0;             // Extensions present if 1

  input          hclk;
  input          hreset_n;
  input          instr_valid_de_i;            // Instruction in decode is valid
  input [31:0]   instr_32_de_i;               // 32 bit instruction in decode
  input          instr_privileged_de_i;       // Privileged (vs user) mode
  input          e_shft_inlinec_de_i;         // E shift with inline carry

  output         instr_32_alu_1_cyc_de_o;     // ALU operation will take 1 cycle
  output         instr_32_algn_de_o;          // Align data
  output         instr_32_mla_de_o;           // Instruction is MLA
  output         instr_32_mul_ilock_de_o;     // Multiply interlock
  output         instr_32_ls_single_wb_de_o;  // Load/store single has writeback
  output [1:0]   instr_32_ls_allow_skid_de_o; // LS pipeline/advance controls
  output         instr_32_lsd_baseinlist_de_o;// LSD has base in list
  output [31:0]  instr_32_immed_de_o;         // Immediate value
  output [31:0]  instr_32_lsm_immed_de_o;     // LSM immediate value
  output         instr_32_imm_valid_de_o;     // Immediate valid
  output [1:0]   instr_32_data_size_de_o;     // Data size
  output         instr_32_sign_de_o;          // Signed operation or sign extend
  output         instr_32_exit_op_de_o;       // Valid exit instruction
  output         instr_32_undef_de_o;         // Instruction is an UNDEF
  output         instr_32_txev_de_o;          // Instruction is SEV
  output [1:0]   instr_32_pc_lr_de_o;         // Link LR into PC
  output         instr_32_isb_de_o;           // ISB instruction for ETM
  output         instr_32_br_t_changer_de_o;  // Branch changes T bit
  output         instr_32_noncc_br_de_o;      // Non-conditional branch
  output         instr_32_flg_set_de_o;       // Can set flags in IT
  output [4:0]   instr_32_msr_ctl_de_o;       // MSR control
  output [1:0]   instr_32_barrier_de_o;       // Barrier control
  output [3:0]   instr_32_cond_code_de_o;     // Condition code
  output         instr_32_copro_de_o;         // Coprocessor instruction
  output         instr_32_wfe_de_o;           // Instruction is WFE
  output         instr_32_wfi_de_o;           // Instruction is WFI
  output         instr_32_has_shft_imm_de_o;  // Instruction has immediate shift
  output [20:0]  instr_32_alu_ctl_de_o;       // ALU control
  output [14:0]  instr_32_lsu_ctl_de_o;       // LSU control
  output [1:0]   instr_32_sp_rd_de_o;         // SP read control
  output [1:0]   instr_32_sp_wr_de_o;         // SP write control
  output [3:0]   instr_32_lsu_rd_a_addr_de_o; // RF A port read address for LSU
  output [3:0]   instr_32_lsu_rd_b_addr_de_o; // RF A port read address for LSU
  output         instr_32_rd_a_en_de_o;       // RF A port read enable
  output [3:0]   instr_32_rd_a_addr_de_o;     // RF A port read address
  output         instr_32_rd_b_en_de_o;       // RF B port read enable
  output [3:0]   instr_32_rd_b_addr_de_o;     // RF B port read address
  output [3:0]   instr_32_rd_c_addr_de_o;     // RF secondary read address
  output         instr_32_wr_d_en_de_o;       // RF D port write enable
  output [3:0]   instr_32_wr_d_addr_de_o;     // RF D port write address

  wire           opt_ext_en;                  // Optimisation of extensions
  wire           i32_spec_de;                 // Special case
  wire           i32_sp_wr_de;                // SP write
  wire           i32_pc_wr_de;                // PC write
  wire           i32_pc_op_de;                // Op1 = 0xF
  wire           i32_fmt_0_de;                // Format 0
  wire           i32_adc1_de;                 // ADC (1)
  wire           i32_add1_de;                 // ADD (1)
  wire           i32_and1_de;                 // AND (1)
  wire           i32_bic1_de;                 // BIC (1)
  wire           i32_cmn1_de;                 // CMN (1)
  wire           i32_cmp1_de;                 // CMP (1)
  wire           i32_eor1_de;                 // EOR (1)
  wire           i32_mov1_de;                 // MOV (1)
  wire           i32_mvn1_de;                 // MVN (1)
  wire           i32_orn1_de;                 // ORN (1)
  wire           i32_orr1_de;                 // ORR (1)
  wire           i32_rsb1_de;                 // RSB (1)
  wire           i32_sbc1_de;                 // SBC (1)
  wire           i32_sub1_de;                 // SUB (1)
  wire           i32_teq1_de;                 // TEQ (1)
  wire           i32_tst1_de;                 // TST (1)
  wire           i32_fmt_1_de;                // Format 1
  wire           i32_addw_de;                 // ADDW
  wire           i32_subw_de;                 // SUBW
  wire           i32_fmt_2_de;                // Format 2
  wire           i32_movw_de;                 // MOVW
  wire           i32_movt_de;                 // MOVT
  wire           i32_fmt_3_de;                // Format 3
  wire           i32_ubfx_de;                 // UBFX
  wire           i32_sbfx_de;                 // SBFX
  wire           i32_bfc_de;                  // BFC
  wire           i32_bfi_de;                  // BFI
  wire           i32_usat_de;                 // USAT
  wire           i32_ssat_de;                 // SSAT
  wire           i32_sat16_de;                // SAT16
  wire           i32_fmt_t0_de;               // Format T0
  wire           i32_adc2_de;                 // ADC (2)
  wire           i32_add2_de;                 // ADD (2)
  wire           i32_and2_de;                 // AND (2)
  wire           i32_bic2_de;                 // BIC (2)
  wire           i32_cmn2_de;                 // CMN (2)
  wire           i32_cmp2_de;                 // CMP (2)
  wire           i32_eor2_de;                 // EOR (2)
  wire           i32_mov2_de;                 // MOV (2)
  wire           i32_lsl1_de;                 // LSL (1)
  wire           i32_lsr1_de;                 // LSR (1)
  wire           i32_asr1_de;                 // ASR (1)
  wire           i32_ror1_de;                 // ROR (1)
  wire           i32_mvn2_de;                 // MVN (2)
  wire           i32_orn2_de;                 // ORN (2)
  wire           i32_orr2_de;                 // ORR (2)
  wire           i32_rsb2_de;                 // RSB (2)
  wire           i32_sbc2_de;                 // SBC (2)
  wire           i32_sub2_de;                 // SUB (2)
  wire           i32_teq2_de;                 // TEQ (2)
  wire           i32_tst2_de;                 // TST (2)
  wire           i32_fmt_t1_de;               // Format T1
  wire           i32_lsl2_de;                 // LSL (2)
  wire           i32_lsr2_de;                 // LSR (2)
  wire           i32_asr2_de;                 // ASR (2)
  wire           i32_ror2_de;                 // ROR (2)
  wire           i32_fmt_t2_de;               // Format T2
  wire           i32_sxth_de;                 // SXTH
  wire           i32_sxtb_de;                 // SXTB
  wire           i32_uxth_de;                 // UXTH
  wire           i32_uxtb_de;                 // UXTB
  wire           i32_fmt_t4_de;               // Format T4
  wire           i32_rbit_de;                 // RBIT
  wire           i32_rev_de;                  // REV
  wire           i32_rev16_de;                // REV16
  wire           i32_revsh_de;                // REVSH
  wire           i32_clz_de;                  // CLZ
  wire           i32_fmt_f0_de;               // Format F0
  wire           i32_mul_de;                  // MUL
  wire           i32_mla_de;                  // MLA
  wire           i32_mls_de;                  // MLS
  wire           i32_fmt_f1_de;               // Format F1
  wire           i32_umull_de;                // UMULL
  wire           i32_smull_de;                // SMULL
  wire           i32_fmt_f2_de;               // Format F2
  wire           i32_umlal_de;                // UMLAL
  wire           i32_smlal_de;                // SMLAL
  wire           i32_div_de;                  // SDIV/UDIV
  wire           i32_udiv_de;                 // UDIV
  wire           i32_sdiv_de;                 // SDIV
  wire           i32_bl_de;                   // Unconditional B/BL
  wire           i32_b0_de;                   // Conditional B
  wire [3:0]     i32_ls_sls_de;               // Load/store single (Size Load Sign)
  wire           i32_hint_de;                 // HINT/PLD
  wire           i32_ldrsb_de;                // LDRSB
  wire           i32_ls_fmt0_de;              // Load/store format 0
  wire           i32_ls_fmt1_de;              // Load/store format 1
  wire           i32_ls_fmt2_de;              // Load/store format 2
  wire           i32_ls_fmt3_de;              // Load/store format 3
  wire           i32_ldrsh_de;                // LDRSH
  wire           i32_ls_single_de;            // Load/store single
  wire           i32_ls_iadj_de;              // Load/store index adjust
  wire           i32_ls_pc_base_de;           // Base = 0xF
  wire           i32_ls_double_de;            // Load/store double
  wire           i32_ls_multiple_de;          // Load/store multiple
  wire           i32_tb_de;                   // Table branch (TBB/TBH)
  wire           i32_clrex_de;                // CLREX
  wire           i32_ls_excl_de;              // Load/store exclusive
  wire           i32_ls_excl_b_de;            // LDREXB/STREXB
  wire           i32_ls_excl_h_de;            // LDREXH/STREXH
  wire           i32_ls_excl_w_de;            // LDREX/STREX
  wire           i32_copro_de;                // Coprocessor instruction
  wire           i32_wfe_de;                  // WFE
  wire           i32_wfi_de;                  // WFI
  wire           i32_sev_de;                  // SEV
  wire           i32_nop_de;                  // NOP
  wire           i32_mxx_de;                  // MSR/MRS
  wire           i32_msr_de;                  // MSR
  wire           i32_mrs_de;                  // MRS
  wire           i32_xxb_de;                  // ISB/DSB/DMB
  wire           i32_dmb_de;                  // DMB
  wire           i32_dsb_de;                  // DSB
  wire           i32_isb_de;                  // ISB
  wire           i32_div_mask_de;             // DIV unpredictable mask
  wire           i32_mul_mask_de;             // MUL unpredictable mask
  wire           instr_32_mla_de;             // MLA
  wire           i32_alu_bfx_de;              // ALU BFX
  wire           i32_alu_bf_de;               // ALU bitfield op
  wire           i32_alu_sat_de;              // ALU saturation op
  wire           i32_alu_clz_de;              // ALU CLZ
  wire           i32_alu_bw_and_de;           // ALU bitwise AND
  wire           i32_alu_bw_or_de;            // ALU bitwise OR
  wire           i32_alu_bw_eor_de;           // ALU bitwise EOR
  wire           i32_alu_shft_de;             // ALU op requires shift
  wire           i32_alu_msr_flag_de;         // ALU MSR flag write
  wire           i32_alu_mrs_de;              // ALU predictable MRS
  wire           i32_alu_mrs_ctl_de;          // ALU MRS control
  wire           i32_alu_mrs_pri_de;          // ALU MRS to PRIMASK
  wire           i32_alu_mrs_fault_de;        // ALU MRS to FAULTMASK
  wire           i32_alu_mrs_base_de;         // ALU MRS to BASEPRI
  wire           i32_alu_mrs_flg1_de;         // ALU MRS flag write (1)
  wire           i32_alu_mrs_flg2_de;         // ALU MRS flag write (2)
  wire           i32_alu_mrs_sp_de;           // ALU MRS to SP
  wire           i32_alu_mrs_unpred_de;       // ALU MRS unpredictable
  wire           i32_alu_mov_norm_de;         // ALU move
  wire           i32_alu_mov_rbit_de;         // ALU RBIT
  wire           i32_alu_mov_rev_de;          // ALU REV
  wire           i32_alu_mov_movt_de;         // ALU MOVT
  wire           i32_alu_div_de;              // ALU SDIV/UDIV
  wire           i32_alu_mlal_de;             // ALU SMLAL/UMLAL
  wire           i32_alu_mull_de;             // ALU SMULL/UMULL
  wire           i32_alu_mul_de;              // ALU MUL
  wire           i32_alu_mla_de;              // ALU MLA
  wire           i32_alu_mls_de;              // ALU MLS
  wire [4:0]     i32_alu_type;                // ALU operation type
  wire           i32_flag_wr_all_de;          // ALU op writes to all of NZCV
  wire [1:0]     i32_flag_wr_de;              // ALU flag write control
  wire           i32_alu_invert_in1;          // ALU invert input 1
  wire           i32_alu_invert_in2;          // ALU invert input 2
  wire           i32_use_adder1;              // ALU use adder input 1
  wire           i32_use_adder2;              // ALU use adder input 2
  wire [1:0]     i32_shft_ctl;                // ALU shifter control
  wire [1:0]     i32_shiftin1_sel;            // ALU shifter input 1 select
  wire [1:0]     i32_shiftin2_sel;            // ALU shifter input 2 select
  wire [1:0]     i32_shft_inlinec_de;         // ALU shift has in line carry
  wire           i32_shft_rrx_de;             // ALU shift is of the form RRX
  wire           i32_use_carry_de;            // ALU use carry in
  wire [20:0]    instr_32_alu_ctl_de;         // ALU control
  wire           i32_lsu_lss_de;              // LSU load/store single
  wire           i32_lsu_lsd_de;              // LSU load/store double
  wire           i32_lsu_lsm_de;              // LSU load/store multiple
  wire           i32_lsu_valid_de;            // LSU operation valid
  wire           i32_lsu_isld_de;             // LSU operation is a load
  wire [1:0]     i32_lsu_immshft_de;          // LSU 2 bit immediate shift
  wire [2:0]     i32_lsu_wb_ctl_de;           // LSU write back control
  wire           i32_lsu_translate_de;        // LSU translate privilege to user
  wire           i32_lsu_excl_de;             // LSU op is an exclusive
  wire [14:0]    instr_32_lsu_ctl_de;         // LSU control
  wire           instr_32_ls_single_wb_de;    // LSS has writeback
  wire [1:0]     instr_32_ls_allow_skid_de;   // LS pipeline/advance controls
  wire           instr_32_mul_ilock_de;       // Multiply interlock
  wire [7:0]     i32_ror_immed_l_de;          // ROR immediate (LSL version)
  wire [7:0]     i32_ror_immed_r_de;          // ROR immediate (LSR version)
  wire [31:0]    i32_shft_ror_immed_de;       // ROR immediate
  wire           i32_shft_ror_cimmed_de;      // ROR has carry in
  wire           instr_32_has_shft_imm_de;    // Instruction has immediate shift
  wire           instr_32_copro_de;           // Coprocessor instruction
  wire           instr_32_wfe_de;             // Instruction is WFE
  wire           instr_32_wfi_de;             // Instruction is WFI
  wire           instr_32_isb_de;             // Instruction is ISB
  wire           instr_32_undef_de;           // Instruction is an UNDEF
  wire           instr_32_txev_de;            // Instruction is SEV
  wire [1:0]     instr_32_pc_lr_de;           // Link LR into PC
  wire           instr_32_br_t_changer_de;    // Branch changes T bit
  wire           instr_32_alu_1_cyc_de;       // ALU operation will take 1 cycle
  wire           instr_32_flg_set_de;         // Can set flags in IT
  wire [1:0]     instr_32_sp_rd_de;           // SP read control
  wire [1:0]     instr_32_sp_wr_de;           // SP write control
  wire [4:0]     instr_32_msr_ctl_de;         // MSR control
  wire [1:0]     instr_32_data_size_de;       // Data size
  wire           instr_32_sign_de;            // Signed operation or sign extend
  wire           instr_32_exit_op_de;         // Valid exit instruction
  wire           instr_32_algn_de;            // Align data
  wire [1:0]     instr_32_barrier_de;         // Barrier control
  wire           instr_32_lsd_baseinlist_de;  // LSD has base in list
  wire [3:0]     instr_32_cond_code_de;       // Condition code
  wire           instr_32_noncc_br_de;        // Non-conditional branch
  wire           instr_32_rd_a_en_de;         // RF A port read enable
  wire           instr_32_rd_b_en_de;         // RF B port read enable
  wire           instr_32_wr_d_en_de;         // RF D port write enable
  wire [12:0]    reg_addr_sel_de;             // RF address mux select
  wire [3:0]     instr_32_lsu_rd_a_addr_de;   // RF A port read address
  wire [3:0]     instr_32_lsu_rd_b_addr_de;   // RF B port read address
  wire [3:0]     instr_32_lsu_rd_c_addr_de;   // RF secondary read address
  wire [3:0]     instr_32_lsu_wr_d_addr_de;   // RF D port write address
  wire [3:0]     instr_32_rd_a_addr_de;       // RF A port read address
  wire [3:0]     instr_32_rd_b_addr_de;       // RF B port read address
  wire [3:0]     instr_32_rd_c_addr_de;       // RF secondary read address
  wire [3:0]     instr_32_wr_d_addr_de;       // RF D port write address
  wire           instr_32_imm_valid_de;       // Immediate valid
  wire [18:0]    reg_immed_sel_de;            // Immediate term mux select
  wire [31:0]    instr_32_immed_de;           // Immediate value
  wire [31:0]    instr_32_lsm_immed_de;       // LSM immediate value

  assign opt_ext_en    = (EXT_PRESENT == 1)    ? 1'b1     : 1'b0;


  assign i32_spec_de  = (instr_32_de_i[4] & (instr_32_de_i[27:24] == 4'b1111));

  assign i32_sp_wr_de = (instr_32_de_i[27:24] == 4'b1101);

  assign i32_pc_wr_de = ((instr_32_de_i[27:24] == 4'b1111) &
                         ~(i32_msr_de |
                           ((i32_fmt_0_de | i32_fmt_t0_de) &
                            ((instr_32_de_i[8:5] == 4'b0000) |
                             (instr_32_de_i[8:5] == 4'b0100) |
                             (instr_32_de_i[8:5] == 4'b1000) |
                             (instr_32_de_i[8:5] == 4'b1101)))));

  assign i32_pc_op_de = (instr_32_de_i[3:0] == 4'b1111);

  assign i32_fmt_0_de = ((instr_32_de_i[15:11] == 5'b11110) &
                         ~instr_32_de_i[9] & ~instr_32_de_i[31]);
  assign i32_adc1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b1010));
  assign i32_add1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b1000) &
                         ~i32_spec_de);
  assign i32_and1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b0000) &
                         ~i32_spec_de);
  assign i32_bic1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b0001));
  assign i32_cmn1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b1000) &
                         i32_spec_de);
  assign i32_cmp1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b1101) &
                         i32_spec_de);
  assign i32_eor1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b0100) &
                         ~i32_spec_de);
  assign i32_mov1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b0010) &
                         i32_pc_op_de);
  assign i32_mvn1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b0011) &
                         i32_pc_op_de);
  assign i32_orn1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b0011) &
                         ~i32_pc_op_de);
  assign i32_orr1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b0010) &
                         ~i32_pc_op_de);
  assign i32_rsb1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b1110));
  assign i32_sbc1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b1011));
  assign i32_sub1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b1101) &
                         ~i32_spec_de);
  assign i32_teq1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b0100) &
                         i32_spec_de);
  assign i32_tst1_de  = (i32_fmt_0_de & (instr_32_de_i[8:5] == 4'b0000) &
                         i32_spec_de);

  assign i32_fmt_1_de = ((instr_32_de_i[15:11] == 5'b11110) &
                         (instr_32_de_i[9:8] == 2'b10) &
                         ~instr_32_de_i[6] & ~instr_32_de_i[31]);
  assign i32_addw_de  = (i32_fmt_1_de & (instr_32_de_i[8:4] == 5'b00000));
  assign i32_subw_de  = (i32_fmt_1_de & (instr_32_de_i[8:4] == 5'b01010));

  assign i32_fmt_2_de = ((instr_32_de_i[15:11] == 5'b11110) &
                         (instr_32_de_i[9:8] == 2'b10) &
                         instr_32_de_i[6] & ~instr_32_de_i[31]);
  assign i32_movw_de  = (i32_fmt_2_de & (instr_32_de_i[7:4] == 4'b0100));
  assign i32_movt_de  = (i32_fmt_2_de & (instr_32_de_i[7:4] == 4'b1100));

  assign i32_fmt_3_de = ((instr_32_de_i[15:11] == 5'b11110) &
                         (instr_32_de_i[9:8] == 2'b11) &
                         ~instr_32_de_i[4] & ~instr_32_de_i[31]);
  assign i32_ubfx_de  = (i32_fmt_3_de & (instr_32_de_i[7:5] == 3'b110));
  assign i32_sbfx_de  = (i32_fmt_3_de & (instr_32_de_i[7:5] == 3'b010));
  assign i32_bfc_de   = (i32_fmt_3_de & (instr_32_de_i[7:5] == 3'b011) &
                         i32_pc_op_de);
  assign i32_bfi_de   = (i32_fmt_3_de & (instr_32_de_i[7:5] == 3'b011) &
                         ~i32_pc_op_de);
  assign i32_usat_de  = (i32_fmt_3_de & (instr_32_de_i[7:6] == 2'b10) &
                         (~instr_32_de_i[5]       |
                          (|instr_32_de_i[30:28]) |
                          (|instr_32_de_i[23:22])));
  assign i32_ssat_de  = (i32_fmt_3_de & (instr_32_de_i[7:6] == 2'b00) &
                         (~instr_32_de_i[5]       |
                          (|instr_32_de_i[30:28]) |
                          (|instr_32_de_i[23:22])));
  assign i32_sat16_de = (i32_fmt_3_de & (instr_32_de_i[6:5] == 2'b01) &
                         ~((|instr_32_de_i[30:28]) |
                           (|instr_32_de_i[23:22])));

  assign i32_fmt_t0_de = (instr_32_de_i[15:9] == 7'b1110101);
  assign i32_adc2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b1010));
  assign i32_add2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b1000) &
                         ~i32_spec_de);
  assign i32_and2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b0000) &
                         ~i32_spec_de);
  assign i32_bic2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b0001));
  assign i32_cmn2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b1000) &
                         i32_spec_de);
  assign i32_cmp2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b1101) &
                         i32_spec_de);
  assign i32_eor2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b0100) &
                         ~i32_spec_de);
  assign i32_mov2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b0010) &
                         i32_pc_op_de);
  assign i32_lsl1_de  = (i32_mov2_de   & (instr_32_de_i[21:20] == 2'b00));
  assign i32_lsr1_de  = (i32_mov2_de   & (instr_32_de_i[21:20] == 2'b01));
  assign i32_asr1_de  = (i32_mov2_de   & (instr_32_de_i[21:20] == 2'b10));
  assign i32_ror1_de  = (i32_mov2_de   & (instr_32_de_i[21:20] == 2'b11));
  assign i32_mvn2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b0011) &
                         i32_pc_op_de);
  assign i32_orn2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b0011) &
                         ~i32_pc_op_de);
  assign i32_orr2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b0010) &
                         ~i32_pc_op_de);
  assign i32_rsb2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b1110));
  assign i32_sbc2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b1011));
  assign i32_sub2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b1101) &
                         ~i32_spec_de);
  assign i32_teq2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b0100) &
                         i32_spec_de);
  assign i32_tst2_de  = (i32_fmt_t0_de & (instr_32_de_i[8:5] == 4'b0000) &
                         i32_spec_de);

  assign i32_fmt_t1_de = ((instr_32_de_i[15:7] == 9'b111110100) &
                          (instr_32_de_i[31:28] == 4'b1111) &
                          (instr_32_de_i[23:20] == 4'b0000));
  assign i32_lsl2_de  = (i32_fmt_t1_de & (instr_32_de_i[6:5] == 2'b00));
  assign i32_lsr2_de  = (i32_fmt_t1_de & (instr_32_de_i[6:5] == 2'b01));
  assign i32_asr2_de  = (i32_fmt_t1_de & (instr_32_de_i[6:5] == 2'b10));
  assign i32_ror2_de  = (i32_fmt_t1_de & (instr_32_de_i[6:5] == 2'b11));

  assign i32_fmt_t2_de = ((instr_32_de_i[15:7] == 9'b111110100) &
                          (instr_32_de_i[3:0] == 4'b1111) &
                          (instr_32_de_i[31:28] == 4'b1111) & instr_32_de_i[23]);
  assign i32_sxth_de  = (i32_fmt_t2_de & (instr_32_de_i[6:4] == 3'b000));
  assign i32_sxtb_de  = (i32_fmt_t2_de & (instr_32_de_i[6:4] == 3'b100));
  assign i32_uxth_de  = (i32_fmt_t2_de & (instr_32_de_i[6:4] == 3'b001));
  assign i32_uxtb_de  = (i32_fmt_t2_de & (instr_32_de_i[6:4] == 3'b101));

  assign i32_fmt_t4_de = ((instr_32_de_i[15:6] == 10'b1111101010) &
                          (instr_32_de_i[31:28] == 4'b1111) &
                          (instr_32_de_i[23:22] == 2'b10));
  assign i32_rbit_de  = (i32_fmt_t4_de & (instr_32_de_i[5:4] == 2'b01) &
                         (instr_32_de_i[21:20] == 2'b10));
  assign i32_rev_de   = (i32_fmt_t4_de & (instr_32_de_i[5:4] == 2'b01) &
                         (instr_32_de_i[21:20] == 2'b00));
  assign i32_rev16_de = (i32_fmt_t4_de & (instr_32_de_i[5:4] == 2'b01) &
                         (instr_32_de_i[21:20] == 2'b01));
  assign i32_revsh_de = (i32_fmt_t4_de & (instr_32_de_i[5:4] == 2'b01) &
                         (instr_32_de_i[21:20] == 2'b11));
  assign i32_clz_de   = (i32_fmt_t4_de & (instr_32_de_i[5:4] == 2'b11) &
                         (instr_32_de_i[21:20] == 2'b00));

  assign i32_fmt_f0_de = ((instr_32_de_i[15:4] == 12'b111110110000) &
                          (instr_32_de_i[23:21] == 3'b000) & ~opt_ext_en);
  assign i32_mul_de   = (i32_fmt_f0_de & ~instr_32_de_i[20] &
                         (instr_32_de_i[31:28] == 4'b1111) & ~opt_ext_en);
  assign i32_mla_de   = (i32_fmt_f0_de & ~instr_32_de_i[20] &
                         (instr_32_de_i[31:28] != 4'b1111) & ~opt_ext_en);
  assign i32_mls_de   = (i32_fmt_f0_de &  instr_32_de_i[20] & ~opt_ext_en);

  assign i32_fmt_f1_de = ((instr_32_de_i[15:6] == 10'b1111101110) &
                          ~instr_32_de_i[4] &
                          (instr_32_de_i[23:20] == 4'b0000) & ~opt_ext_en);
  assign i32_umull_de = (i32_fmt_f1_de &  instr_32_de_i[5] & ~opt_ext_en);
  assign i32_smull_de = (i32_fmt_f1_de & ~instr_32_de_i[5] & ~opt_ext_en);

  assign i32_fmt_f2_de = ((instr_32_de_i[15:6] == 10'b1111101111) &
                          ~instr_32_de_i[4] &
                          (instr_32_de_i[23:20] == 4'b0000) & ~opt_ext_en);
  assign i32_umlal_de = (i32_fmt_f2_de &  instr_32_de_i[5] & ~opt_ext_en);
  assign i32_smlal_de = (i32_fmt_f2_de & ~instr_32_de_i[5] & ~opt_ext_en);

  assign i32_div_de   = ((instr_32_de_i[15:6] == 10'b1111101110) &
                          instr_32_de_i[4] &
                         (instr_32_de_i[23:20] == 4'b1111));
  assign i32_udiv_de  = i32_div_de &  instr_32_de_i[5];
  assign i32_sdiv_de  = i32_div_de & ~instr_32_de_i[5];

  assign i32_bl_de    = ((instr_32_de_i[15:11] == 5'b11110) &
                         instr_32_de_i[31] & instr_32_de_i[28]);

  assign i32_b0_de    = ((instr_32_de_i[15:11] == 5'b11110) &
                         (instr_32_de_i[9:7] != 3'b111) &
                         (instr_32_de_i[31:30] == 2'b10) & ~instr_32_de_i[28]);

  assign i32_ls_sls_de = {instr_32_de_i[6:4],instr_32_de_i[8]};
  assign i32_hint_de  = ((instr_32_de_i[31:28] == 4'b1111) &
                         ((i32_ls_sls_de == 4'b0000) |
                          (i32_ls_sls_de == 4'b0010) |
                          (i32_ls_sls_de == 4'b0011) |
                          (i32_ls_sls_de == 4'b0100) |
                          (i32_ls_sls_de == 4'b1000) |
                          (i32_ls_sls_de[3:1] == 3'b011)));
  assign i32_ls_fmt0_de = (instr_32_de_i[4:0] == 5'b11111);
  assign i32_ls_fmt1_de = ( instr_32_de_i[7] & (instr_32_de_i[3:0] != 4'b1111));
  assign i32_ls_fmt2_de = (~instr_32_de_i[7] & (instr_32_de_i[3:0] != 4'b1111) &
                           instr_32_de_i[27] & (instr_32_de_i[26] |
                                                instr_32_de_i[24]));
  assign i32_ls_fmt3_de = (~instr_32_de_i[7] & (instr_32_de_i[3:0] != 4'b1111) &
                          ( instr_32_de_i[27:22]==6'b000000));
  assign i32_ldrsb_de = ((instr_32_de_i[15:9] == 7'b1111100) &
                         ~(instr_32_de_i[8] & ~instr_32_de_i[4]) &
                         ~(&instr_32_de_i[6:5]) &
                         ~(instr_32_de_i[8] & instr_32_de_i[4] &
                           (instr_32_de_i[6:5] == 2'b10)) &
                         (i32_ls_fmt0_de | i32_ls_fmt1_de |
                          i32_ls_fmt2_de | i32_ls_fmt3_de));
  assign i32_ldrsh_de     = (i32_ldrsb_de &  i32_hint_de);
  assign i32_ls_single_de = (i32_ldrsb_de & ~i32_hint_de);
  assign i32_ls_pc_base_de = (instr_32_de_i[4:0] == 5'b11111);
  assign i32_ls_iadj_de   = (i32_ls_single_de & ~instr_32_de_i[7] &
                             ~(|instr_32_de_i[27:22]));

  assign i32_ls_double_de = ((instr_32_de_i[15:9] == 7'b1110100) &
                             instr_32_de_i[6] &
                             (instr_32_de_i[8] | instr_32_de_i[5]));

  assign i32_ls_multiple_de = ((instr_32_de_i[15:9] == 7'b1110100) &
                               (instr_32_de_i[8] != instr_32_de_i[7]) &
                               ~instr_32_de_i[6]);

  assign i32_tb_de    = ((instr_32_de_i[15:4] == 12'b111010001101) &
                         ~(|instr_32_de_i[23:21]));

  assign i32_clrex_de = ((instr_32_de_i[15:4] == 12'b111100111011) &
                         (instr_32_de_i[31:30] == 2'b10) & ~instr_32_de_i[28] &
                         (instr_32_de_i[23:20] == 4'b0010));
  assign i32_ls_excl_de   = (instr_32_de_i[15:5] == 11'b11101000110);
  assign i32_ls_excl_b_de = (i32_ls_excl_de & (instr_32_de_i[23:20] == 4'b0100));
  assign i32_ls_excl_h_de = (i32_ls_excl_de & (instr_32_de_i[23:20] == 4'b0101));
  assign i32_ls_excl_w_de = (instr_32_de_i[15:5] == 11'b11101000010);

  assign i32_copro_de = ((instr_32_de_i[15:9] == 7'b1110110)  |
                         (instr_32_de_i[15:8] == 8'b11101110) |
                         (instr_32_de_i[15:9] == 7'b1111110)  |
                         (instr_32_de_i[15:8] == 8'b11111110));

  assign i32_wfe_de   = ((instr_32_de_i[15:4] == 12'b111100111010) &
                         (instr_32_de_i[31:30] == 2'b10) &
                         ~instr_32_de_i[28] & (instr_32_de_i[26:24] == 3'b000) &
                         (instr_32_de_i[23:16] == 8'b00000010));

  assign i32_wfi_de   = ((instr_32_de_i[15:4] == 12'b111100111010) &
                         (instr_32_de_i[31:30] == 2'b10) &
                         ~instr_32_de_i[28] & (instr_32_de_i[26:24] == 3'b000) &
                         (instr_32_de_i[23:16] == 8'b00000011));
  assign i32_sev_de   = ((instr_32_de_i[15:4] == 12'b111100111010) &
                         (instr_32_de_i[31:30] == 2'b10) &
                         ~instr_32_de_i[28] & (instr_32_de_i[26:24] == 3'b000) &
                         (instr_32_de_i[23:16] == 8'b00000100));
  assign i32_nop_de   = ((instr_32_de_i[15:4] == 12'b111100111010) &
                         (instr_32_de_i[31:30] == 2'b10) &
                         ~instr_32_de_i[28] & (instr_32_de_i[26:24] == 3'b000) &
                         ~(i32_wfe_de | i32_wfi_de | i32_sev_de));

  assign i32_mxx_de   = ((instr_32_de_i[15:7] == 9'b111100111) &
                         ~instr_32_de_i[4] & ~instr_32_de_i[28] &
                         (instr_32_de_i[31:30] == 2'b10));
  assign i32_msr_de   = (i32_mxx_de & (instr_32_de_i[6:5] == 2'b00) &
                         (instr_32_de_i[25:24] == 2'b00));
  assign i32_mrs_de   = (i32_mxx_de & (instr_32_de_i[6:5] == 2'b11));

  assign i32_xxb_de   = ((instr_32_de_i[15:4] == 12'b111100111011) &
                         (instr_32_de_i[31:30] == 2'b10) & ~instr_32_de_i[28]);
  assign i32_dmb_de   = (i32_xxb_de & (instr_32_de_i[23:20] == 4'b0101));
  assign i32_dsb_de   = (i32_xxb_de & (instr_32_de_i[23:20] == 4'b0100));
  assign i32_isb_de   = (i32_xxb_de & (instr_32_de_i[23:20] == 4'b0110));

  assign i32_div_mask_de = ((instr_32_de_i[27:24] != 4'b1111) &
                            (instr_32_de_i[27:24] != 4'b1101));

  assign i32_mul_mask_de = ((instr_32_de_i[31:28] != 4'b1111) &
                            (instr_32_de_i[31:28] != 4'b1101) &
                            i32_div_mask_de);

  assign instr_32_mla_de = i32_alu_mla_de;


  assign i32_alu_bw_and_de = (i32_and1_de | i32_tst1_de | i32_and2_de |
                              i32_tst2_de | i32_bic1_de | i32_bic2_de);
  assign i32_alu_bw_or_de  = (i32_orr1_de | i32_orr2_de | i32_orn1_de |
                              i32_orn2_de);
  assign i32_alu_bw_eor_de = (i32_eor1_de | i32_teq1_de | i32_eor2_de |
                              i32_teq2_de);

  assign i32_alu_shft_de   = (~i32_pc_wr_de &
                              (i32_ror1_de | i32_ror2_de | i32_lsr1_de |
                               i32_lsr2_de | i32_lsl1_de | i32_lsl2_de |
                               i32_asr1_de | i32_asr2_de));

  assign i32_alu_bfx_de = ~i32_pc_wr_de & (i32_sbfx_de | i32_ubfx_de);
  assign i32_alu_bf_de  = ~i32_pc_wr_de & (i32_bfi_de  | i32_bfc_de);

  assign i32_alu_sat_de = ~i32_pc_wr_de & (i32_ssat_de | i32_usat_de);

  assign i32_alu_clz_de = ~i32_pc_wr_de & i32_clz_de;

  assign i32_alu_msr_flag_de = (i32_msr_de & (instr_32_de_i[23:18]==6'b000000) &
                                (instr_32_de_i[3:0]!=4'b1111));

  assign i32_alu_mrs_de       = (i32_mrs_de & ~i32_pc_wr_de);
  assign i32_alu_mrs_ctl_de   = (i32_alu_mrs_de &
                                 (instr_32_de_i[23:16]==8'b00010100));
  assign i32_alu_mrs_pri_de   = (i32_alu_mrs_de &
                                 (instr_32_de_i[23:16]==8'b00010000));
  assign i32_alu_mrs_fault_de = (i32_alu_mrs_de &
                                 (instr_32_de_i[23:16]==8'b00010011));
  assign i32_alu_mrs_base_de  = (i32_alu_mrs_de &
                                 (instr_32_de_i[23:18]==6'b000100) &
                                 (instr_32_de_i[17] ^ instr_32_de_i[16]));
  assign i32_alu_mrs_flg1_de  = (i32_alu_mrs_de &
                                 (instr_32_de_i[23:18]==6'b000000));
  assign i32_alu_mrs_flg2_de  = (i32_alu_mrs_de &
                                 (instr_32_de_i[23:19]==5'b00000) &
                                 instr_32_de_i[16]);
  assign i32_alu_mrs_sp_de    = (i32_alu_mrs_de &
                                 (instr_32_de_i[23:17]==7'b000100));

  assign i32_alu_mrs_unpred_de  = (i32_mrs_de & ~i32_pc_wr_de &
                                   ~(i32_alu_mrs_ctl_de   | i32_alu_mrs_pri_de  |
                                     i32_alu_mrs_fault_de | i32_alu_mrs_base_de |
                                     i32_alu_mrs_flg1_de  | i32_alu_mrs_flg2_de |
                                     i32_alu_mrs_sp_de));

  assign i32_alu_mov_norm_de = (i32_isb_de |
                                (~i32_pc_wr_de &
                                 (i32_mov1_de | i32_movw_de | i32_sxth_de |
                                  i32_sxtb_de | i32_uxth_de | i32_uxtb_de |
                                  i32_mvn1_de |i32_mvn2_de  |
                                  i32_alu_mrs_sp_de |
                                  (i32_msr_de & ~i32_alu_msr_flag_de &
                                   ~(&instr_32_de_i[3:0])))));

  assign i32_alu_mov_rbit_de = ~i32_pc_wr_de & i32_rbit_de;

  assign i32_alu_mov_rev_de  = (~i32_pc_wr_de &
                                (i32_rev_de | i32_revsh_de |  i32_rev16_de));

  assign i32_alu_mov_movt_de = ~i32_pc_wr_de & i32_movt_de;

  assign i32_alu_div_de      = (~i32_pc_wr_de &
                                ((i32_sdiv_de  & i32_div_mask_de) |
                                 (i32_udiv_de  & i32_div_mask_de)));

  assign i32_alu_mlal_de = (~i32_pc_wr_de &
                            ((i32_smlal_de & i32_mul_mask_de) |
                             (i32_umlal_de & i32_mul_mask_de)));

  assign i32_alu_mull_de = (~i32_pc_wr_de &
                            ((i32_smull_de & i32_mul_mask_de) |
                             (i32_umull_de & i32_mul_mask_de)));

  assign i32_alu_mul_de  = (~(i32_pc_wr_de | i32_sp_wr_de) & i32_mul_de);

  assign i32_alu_mla_de  = (~(i32_pc_wr_de | i32_sp_wr_de) & i32_mla_de);

  assign i32_alu_mls_de  = (~(i32_pc_wr_de | i32_sp_wr_de) & i32_mls_de);



  assign i32_alu_type[4] = (i32_alu_mrs_flg1_de  | i32_alu_mrs_flg2_de   |
                            i32_alu_mov_norm_de  | i32_alu_mov_rbit_de   |
                            i32_alu_mov_rev_de   | i32_alu_mov_movt_de   |
                            i32_alu_div_de       | i32_alu_mlal_de       |
                            i32_alu_mull_de      | i32_alu_mul_de        |
                            i32_alu_mla_de       | i32_alu_mls_de        |
                            i32_alu_clz_de);

  assign i32_alu_type[3] = (i32_alu_msr_flag_de  | i32_alu_mrs_unpred_de |
                            i32_alu_mrs_base_de  | i32_alu_mrs_ctl_de    |
                            i32_alu_mrs_pri_de   | i32_alu_mrs_fault_de  |
                            i32_alu_div_de       | i32_alu_mlal_de       |
                            i32_alu_mull_de      | i32_alu_mul_de        |
                            i32_alu_mla_de       | i32_alu_mls_de        |
                            i32_alu_clz_de);

  assign i32_alu_type[2] = (i32_alu_shft_de      | i32_alu_bfx_de        |
                            i32_alu_bf_de        | i32_alu_sat_de        |
                            i32_alu_mrs_pri_de   | i32_alu_mrs_fault_de  |
                            i32_alu_mov_norm_de  | i32_alu_mov_rbit_de   |
                            i32_alu_mov_rev_de   | i32_alu_mov_movt_de   |
                            i32_alu_mla_de       | i32_alu_mls_de        |
                            i32_alu_clz_de);

  assign i32_alu_type[1] = (i32_alu_bw_or_de     | i32_alu_bw_eor_de     |
                            i32_alu_bf_de        | i32_alu_sat_de        |
                            i32_alu_mrs_ctl_de   | i32_alu_mrs_base_de   |
                            i32_alu_mrs_fault_de | i32_alu_mrs_base_de   |
                            i32_alu_mrs_flg2_de  | i32_alu_mov_rev_de    |
                            i32_alu_mov_movt_de  | i32_alu_mull_de       |
                            i32_alu_mul_de       | i32_alu_clz_de);

  assign i32_alu_type[0] = (i32_alu_bw_and_de    | i32_alu_bw_eor_de     |
                            i32_alu_bfx_de       | i32_alu_sat_de        |
                            i32_alu_mrs_base_de  | i32_alu_mrs_pri_de    |
                            i32_alu_mrs_base_de  | i32_alu_mrs_unpred_de |
                            i32_alu_mrs_flg1_de  | i32_alu_mov_rbit_de   |
                            i32_alu_mov_movt_de  | i32_alu_mlal_de       |
                            i32_alu_mul_de       | i32_alu_mls_de        |
                            i32_alu_clz_de);

  assign i32_flag_wr_all_de = (i32_adc1_de | i32_add1_de | i32_cmn1_de |
                               i32_cmp1_de | i32_rsb1_de | i32_sbc1_de |
                               i32_sub1_de | i32_adc2_de | i32_add2_de |
                               i32_cmn2_de | i32_cmp2_de | i32_rsb2_de |
                               i32_sbc2_de | i32_sub2_de);


  assign i32_flag_wr_de[1] = (instr_32_de_i[4] & ~i32_pc_wr_de &
                              (i32_flag_wr_all_de |
                               (i32_lsl1_de | i32_lsr1_de | i32_asr1_de |
                                i32_ror1_de | i32_lsl2_de | i32_lsr2_de |
                                i32_asr2_de | i32_ror2_de |
                                ((i32_and1_de | i32_bic1_de | i32_eor1_de |
                                  i32_mov1_de | i32_mvn1_de | i32_orn1_de |
                                  i32_orr1_de | i32_teq1_de | i32_tst1_de) &
                                 i32_shft_ror_cimmed_de)  |
                                (i32_and2_de | i32_bic2_de | i32_eor2_de |
                                 i32_mvn2_de | i32_orn2_de | i32_orr2_de |
                                 i32_teq2_de | i32_tst2_de))));

  assign i32_flag_wr_de[0] = (instr_32_de_i[4] & ~i32_pc_wr_de &
                              (i32_flag_wr_all_de |
                               ((i32_and1_de | i32_bic1_de | i32_eor1_de |
                                 i32_mov1_de | i32_mvn1_de | i32_orn1_de |
                                 i32_orr1_de | i32_teq1_de | i32_tst1_de) &
                                ~i32_shft_ror_cimmed_de)));

  assign i32_alu_invert_in1 = (i32_rsb1_de | i32_rsb2_de);

  assign i32_alu_invert_in2 = (i32_cmp1_de | i32_sbc1_de | i32_sub1_de |
                               i32_subw_de | i32_cmp2_de | i32_sbc2_de |
                               i32_sub2_de | i32_bic1_de | i32_bic2_de |
                               i32_orn1_de | i32_orn2_de | i32_mvn1_de |
                               i32_mvn2_de);

  assign i32_use_adder1     = (i32_bl_de   | i32_b0_de   | i32_adc1_de |
                               i32_add1_de | i32_cmn1_de | i32_addw_de |
                               i32_adc2_de | i32_add2_de | i32_cmn2_de |
                               i32_cmp1_de | i32_sbc1_de | i32_sub1_de |
                               i32_subw_de | i32_cmp2_de | i32_sbc2_de |
                               i32_sub2_de | i32_rsb1_de | i32_rsb2_de);

  assign i32_use_adder2     = (i32_use_adder1 | i32_mvn1_de | i32_mvn2_de);


  assign i32_shft_ctl[1] = (i32_asr1_de | i32_asr2_de   | i32_ror1_de |
                            i32_ror2_de | i32_fmt_t2_de |
                            ((i32_ssat_de | i32_usat_de) & instr_32_de_i[5]) |
                            (i32_fmt_t0_de & instr_32_de_i[21]));

  assign i32_shft_ctl[0] = (i32_lsr1_de | i32_lsr2_de   | i32_ror1_de |
                            i32_ror2_de | i32_fmt_t2_de |
                            (i32_fmt_t0_de & instr_32_de_i[20]) |
                            ((i32_sbfx_de | i32_ubfx_de) &
                             (|{instr_32_de_i[30:28],instr_32_de_i[23:22]})));


  assign i32_shiftin1_sel[1] = (i32_mxx_de | i32_fmt_3_de | i32_fmt_0_de |
                                i32_fmt_t0_de | i32_fmt_t1_de | i32_isb_de |
                                i32_fmt_t2_de | i32_div_de |
                                i32_fmt_t4_de);

  assign i32_shiftin1_sel[0] = 1'b0;


  assign i32_shiftin2_sel[1] = (i32_fmt_t1_de);

  assign i32_shiftin2_sel[0] = (i32_fmt_3_de  | i32_fmt_t0_de |
                                i32_fmt_t2_de | i32_div_de);

  assign i32_shft_inlinec_de = { i32_shft_ror_cimmed_de &
                                 (i32_and1_de | i32_tst1_de | i32_bic1_de |
                                  i32_eor1_de | i32_teq1_de | i32_orr1_de |
                                  i32_mov1_de | i32_orn1_de | i32_mvn1_de),
                                 (i32_and2_de | i32_tst2_de | i32_bic2_de |
                                  i32_eor2_de | i32_teq2_de | i32_orr2_de |
                                  i32_mov2_de | i32_orn2_de | i32_mvn2_de) |
                                 e_shft_inlinec_de_i};

  assign i32_shft_rrx_de = (i32_fmt_t0_de &
                            ({instr_32_de_i[30:28],instr_32_de_i[23:20]} ==
                             7'b0000011));

  assign i32_use_carry_de = (i32_adc1_de | i32_sbc1_de |
                             i32_adc2_de | i32_sbc2_de);


  assign instr_32_alu_ctl_de = { i32_use_carry_de, i32_flag_wr_de, i32_alu_type,
                                 i32_shft_rrx_de, i32_shiftin2_sel,
                                 i32_shiftin1_sel, i32_shft_ctl,
                                 i32_shft_inlinec_de, i32_use_adder2,
                                 i32_use_adder1, i32_alu_invert_in2,
                                 i32_alu_invert_in1 };


  assign i32_lsu_lss_de = (// Exclusive LSS (byte and half)
                           ((i32_ls_excl_b_de | i32_ls_excl_h_de) &
                            ~(~instr_32_de_i[4] &
                              ((instr_32_de_i[19:16]==4'b1111) |
                               (instr_32_de_i[19:16]==4'b1101))) &
                            ~( instr_32_de_i[4] &
                              ((instr_32_de_i[31:28]==4'b1111) |
                               (instr_32_de_i[31:28]==4'b1101)))) |
                           ((i32_ls_excl_w_de)  &
                            ~(~instr_32_de_i[4] &
                              ((instr_32_de_i[27:24]==4'b1111) |
                               (instr_32_de_i[27:24]==4'b1101))) &
                            ~( instr_32_de_i[4] &
                              ((instr_32_de_i[31:28]==4'b1111) |
                               (instr_32_de_i[31:28]==4'b1101)))) |
                           (i32_ls_single_de &
                            ~(i32_ls_fmt2_de & instr_32_de_i[24] &
                             ( instr_32_de_i[3:0]==instr_32_de_i[31:28]))));

  assign i32_lsu_lsd_de = (i32_ls_double_de &
                           ~(instr_32_de_i[4] &
                             ((instr_32_de_i[31:28]==4'b1111) |
                              (instr_32_de_i[31:28]==4'b1101) |
                              (instr_32_de_i[27:24]==4'b1111) |
                              (instr_32_de_i[27:24]==4'b1101))));

  assign i32_lsu_lsm_de = (i32_ls_multiple_de &
                           (|{instr_32_de_i[31:30],instr_32_de_i[28:16]}));

  assign i32_lsu_valid_de = (i32_tb_de      | i32_lsu_lsm_de |
                             i32_lsu_lsd_de | i32_lsu_lss_de);

  assign i32_lsu_isld_de = ((i32_ls_excl_w_de | i32_ls_excl_b_de |
                             i32_ls_excl_h_de | i32_ls_single_de |
                             i32_ls_double_de | i32_ls_multiple_de) &
                            instr_32_de_i[4]);

  assign i32_lsu_immshft_de = (({2{i32_ls_iadj_de}} & instr_32_de_i[21:20]) |
                               ({1'b0,i32_tb_de} & {1'b0,instr_32_de_i[20]}));

  assign i32_lsu_wb_ctl_de[0] = ((i32_ls_single_de & ~instr_32_de_i[7] &
                                  instr_32_de_i[27] & instr_32_de_i[24] &
                                  ~i32_ls_pc_base_de) |
                                 (i32_ls_double_de & instr_32_de_i[5] &
                                  ~(&instr_32_de_i[3:0])) |
                                 (i32_ls_multiple_de & instr_32_de_i[5] &
                                  ~(&instr_32_de_i[3:0])));

  assign i32_lsu_wb_ctl_de[1] = ((i32_ls_single_de & ~instr_32_de_i[7] &
                                  instr_32_de_i[27] & ~instr_32_de_i[26] &
                                  ~i32_ls_pc_base_de) |
                                 (i32_ls_double_de & ~instr_32_de_i[8]) |
                                 (i32_ls_multiple_de & instr_32_de_i[7]));

  assign i32_lsu_wb_ctl_de[2] = ((i32_lsu_lss_de &
                                  ~(i32_ls_single_de & ~instr_32_de_i[7] &
                                    ((&instr_32_de_i[4:0]) |
                                     instr_32_de_i[26:24]==3'b001 |
                                     instr_32_de_i[26:24]==3'b100 |
                                     instr_32_de_i[26:24]==3'b101))) |
                                 (i32_ls_double_de & instr_32_de_i[7]) |
                                 (i32_ls_multiple_de & instr_32_de_i[7]));

  assign i32_lsu_translate_de = (i32_ls_single_de & i32_ls_fmt2_de &
                                 (instr_32_de_i[26:24]==3'b110) &
                                 ~(instr_32_de_i[4] &
                                   (instr_32_de_i[31:28]==4'b1111)));

  assign i32_lsu_excl_de = (i32_ls_excl_w_de |
                            i32_ls_excl_b_de |
                            i32_ls_excl_h_de);


  assign instr_32_lsu_ctl_de = { i32_clrex_de, i32_lsu_excl_de,
                                 i32_lsu_translate_de, i32_lsu_wb_ctl_de,
                                 i32_lsu_immshft_de, i32_lsu_isld_de,
                                 i32_lsu_lsm_de, i32_lsu_lsd_de, i32_lsu_lss_de,
                                 i32_tb_de, 1'b0, i32_lsu_valid_de };

  assign instr_32_ls_single_wb_de = (i32_ls_single_de & ~instr_32_de_i[7] &
                                     instr_32_de_i[27] & instr_32_de_i[24] &
                                     ~i32_ls_pc_base_de);

  assign instr_32_ls_allow_skid_de = {// next opcode not taken
                                      1'b0,
                                      (i32_nop_de | i32_ldrsh_de)};

  assign instr_32_mul_ilock_de = i32_b0_de;

  assign i32_ror_immed_l_de     = ({instr_32_de_i[22:16],1'b0} <<
                                   ~{instr_32_de_i[29:28],instr_32_de_i[23]});

  assign i32_ror_immed_r_de     = ({1'b1,instr_32_de_i[22:16]} >>
                                   {instr_32_de_i[29:28],instr_32_de_i[23]});

  assign i32_shft_ror_immed_de  = {({8{{instr_32_de_i[10],instr_32_de_i[30]}==2'b01}} &
                                    i32_ror_immed_r_de),
                                   ({8{{instr_32_de_i[10],instr_32_de_i[30]}==2'b10}} &
                                    i32_ror_immed_r_de) |
                                   ({8{{instr_32_de_i[10],instr_32_de_i[30]}==2'b01}} &
                                    i32_ror_immed_l_de),
                                   ({8{{instr_32_de_i[10],instr_32_de_i[30]}==2'b11}} &
                                    i32_ror_immed_r_de) |
                                   ({8{{instr_32_de_i[10],instr_32_de_i[30]}==2'b10}} &
                                    i32_ror_immed_l_de),
                                   ({8{{instr_32_de_i[10],instr_32_de_i[30]}==2'b11}} &
                                    i32_ror_immed_l_de)};

  assign i32_shft_ror_cimmed_de = instr_32_de_i[10] | instr_32_de_i[30];

  assign instr_32_has_shft_imm_de = (// Format T0
                                     i32_adc2_de | i32_add2_de | i32_and2_de |
                                     i32_bic2_de | i32_cmn2_de | i32_cmp2_de |
                                     i32_eor2_de | i32_mvn2_de | i32_orn2_de |
                                     i32_orr2_de | i32_rsb2_de | i32_sbc2_de |
                                     i32_sub2_de | i32_teq2_de | i32_tst2_de |
                                     i32_sxth_de | i32_sxtb_de | i32_uxth_de |
                                     i32_uxtb_de |
                                     e_shft_inlinec_de_i);

  assign instr_32_copro_de = i32_copro_de;

  assign instr_32_wfe_de   = i32_wfe_de;

  assign instr_32_wfi_de   = i32_wfi_de;

  assign instr_32_isb_de   = i32_isb_de;

  assign instr_32_undef_de = ~(i32_adc1_de  | i32_add1_de  | i32_and1_de  |
                               i32_bic1_de  | i32_cmn1_de  | i32_cmp1_de  |
                               i32_eor1_de  | i32_mov1_de  | i32_mvn1_de  |
                               i32_orn1_de  | i32_orr1_de  | i32_rsb1_de  |
                               i32_sbc1_de  | i32_sub1_de  | i32_teq1_de  |
                               i32_tst1_de  | i32_addw_de  | i32_subw_de  |
                               i32_movw_de  | i32_movt_de  | i32_ubfx_de  |
                               i32_sbfx_de  | i32_bfc_de   | i32_bfi_de   |
                               i32_usat_de  | i32_ssat_de  | i32_adc2_de  |
                               i32_add2_de  | i32_and2_de  | i32_bic2_de  |
                               i32_cmn2_de  | i32_cmp2_de  | i32_eor2_de  |
                               i32_lsl1_de  | i32_lsr1_de  | i32_asr1_de  |
                               i32_ror1_de  | i32_mvn2_de  | i32_orn2_de  |
                               i32_orr2_de  | i32_rsb2_de  | i32_sbc2_de  |
                               i32_sub2_de  | i32_teq2_de  | i32_tst2_de  |
                               i32_lsl2_de  | i32_lsr2_de  | i32_asr2_de  |
                               i32_ror2_de  | i32_sxth_de  | i32_sxtb_de  |
                               i32_uxth_de  | i32_uxtb_de  | i32_rbit_de  |
                               i32_rev_de   | i32_rev16_de | i32_revsh_de |
                               i32_clz_de   | i32_mul_de   | i32_mla_de   |
                               i32_mls_de   | i32_umull_de | i32_smull_de |
                               i32_umlal_de | i32_smlal_de | i32_udiv_de  |
                               i32_sdiv_de  | i32_bl_de    | i32_b0_de    |
                               i32_ldrsh_de | i32_tb_de    | i32_clrex_de |
                               i32_copro_de | i32_wfe_de   | i32_sev_de   |
                               i32_wfi_de   | i32_nop_de   | i32_msr_de   |
                               i32_mrs_de   | i32_dmb_de   | i32_dsb_de   |
                               i32_isb_de   |
                               i32_ls_single_de   | i32_ls_double_de |
                               i32_ls_multiple_de | i32_ls_excl_w_de |
                               i32_ls_excl_b_de   | i32_ls_excl_h_de);

  assign instr_32_txev_de = i32_sev_de;

  assign instr_32_pc_lr_de = { 1'b0, (i32_bl_de & instr_32_de_i[30])};

  assign instr_32_br_t_changer_de = ((i32_ls_single_de & instr_32_de_i[4] &
                                      (&instr_32_de_i[31:28])) |
                                     (i32_ls_multiple_de & instr_32_de_i[4] &
                                      instr_32_de_i[31]));

  assign instr_32_alu_1_cyc_de = (// Single Cycle ALU
                                  ~i32_pc_wr_de &
                                  (i32_fmt_0_de  | i32_fmt_1_de  |
                                   i32_fmt_2_de  | i32_fmt_3_de  |
                                   i32_fmt_t0_de | i32_fmt_t1_de |
                                   i32_fmt_t2_de | i32_fmt_t4_de |
                                   i32_mul_de | i32_nop_de));

  assign instr_32_flg_set_de   = (((i32_fmt_0_de | i32_fmt_1_de  |
                                    i32_fmt_2_de | i32_fmt_t0_de |
                                    i32_fmt_t1_de) &
                                   instr_32_de_i[4]) |
                                  (i32_msr_de & ~(|instr_32_de_i[23:19]) &
                                   ~instr_32_de_i[18]) |
                                  i32_usat_de | i32_ssat_de);

  assign instr_32_sp_rd_de = {instr_32_de_i[16],
                              i32_mrs_de & instr_32_de_i[23:17]==7'b0000100};

  assign instr_32_sp_wr_de = {instr_32_de_i[16], i32_msr_de &
                              instr_32_de_i[23:17]==7'b0000100};

  assign instr_32_msr_ctl_de = ({5{i32_msr_de & instr_32_de_i[3:0]!=4'b1111}} &
                                { instr_32_de_i[23:16]==8'b00010100,
                                  {4{instr_32_de_i[23:18]==6'b000100}} &
                                  {instr_32_de_i[17:16]==2'b11,
                                   instr_32_de_i[17:16]==2'b10,
                                   instr_32_de_i[17:16]==2'b01,
                                   instr_32_de_i[17:16]==2'b00}});



  assign instr_32_data_size_de[1] = (i32_movt_de   | i32_movw_de   | i32_mov1_de |
                                     i32_fmt_f0_de | i32_fmt_f1_de | i32_mvn1_de |
                                     i32_mvn2_de   | i32_fmt_f2_de | i32_mrs_de  |
                                     i32_isb_de    |
                                     (i32_ls_single_de & instr_32_de_i[6:5] == 2'b10) |
                                     i32_ls_double_de   |
                                     i32_ls_multiple_de |
                                     i32_ls_excl_w_de   |
                                     (i32_msr_de & ~i32_alu_msr_flag_de) |
                                     (i32_alu_msr_flag_de & instr_32_de_i[27]));

  assign instr_32_data_size_de[0] = (i32_revsh_de | i32_rev16_de | i32_sxth_de  |
                                     i32_uxth_de  |
                                     (i32_ls_single_de & instr_32_de_i[6:5] == 2'b01) |
                                     (i32_tb_de & instr_32_de_i[20]) |
                                     i32_ls_excl_h_de |
                                     (i32_alu_msr_flag_de & instr_32_de_i[26]));

  assign instr_32_sign_de = (i32_revsh_de | i32_sxth_de  | i32_sxtb_de |
                             i32_sbfx_de  | i32_ssat_de  | i32_sdiv_de |
                             i32_smlal_de | i32_smull_de |
                             (i32_ls_single_de & instr_32_de_i[8]));

  assign instr_32_exit_op_de = ((i32_ls_multiple_de &
                                 instr_32_de_i[4] & instr_32_de_i[31]) |
                                (i32_ls_single_de &
                                 instr_32_de_i[4] & (&instr_32_de_i[31:28])));

  assign instr_32_algn_de = ((i32_fmt_1_de & (&instr_32_de_i[3:0])) |
                             (i32_ls_single_de & (&instr_32_de_i[3:0])));

  assign instr_32_barrier_de = {i32_dmb_de, i32_dsb_de};

  assign instr_32_lsd_baseinlist_de = (i32_ls_double_de & instr_32_de_i[4] &
                                       (instr_32_de_i[31:28] == instr_32_de_i[3:0]));

  assign instr_32_cond_code_de = ({4{i32_b0_de}} & instr_32_de_i[9:6]);
  assign instr_32_noncc_br_de  = i32_b0_de;

  assign instr_32_rd_a_en_de = (i32_bl_de     | i32_b0_de     | i32_fmt_0_de  |
                                i32_fmt_1_de  | i32_fmt_3_de  | i32_fmt_t0_de |
                                i32_fmt_t1_de | i32_fmt_t2_de | i32_fmt_f0_de |
                                i32_fmt_f1_de | i32_fmt_f2_de | i32_div_de    |
                                i32_tb_de     |
                                i32_ls_single_de   | i32_ls_double_de |
                                i32_ls_multiple_de | i32_ls_excl_b_de |
                                i32_ls_excl_h_de   | i32_ls_excl_w_de);

  assign instr_32_rd_b_en_de = (i32_fmt_2_de  | i32_fmt_t0_de | i32_fmt_t1_de |
                                i32_fmt_t2_de | i32_fmt_t4_de | i32_fmt_f0_de |
                                i32_fmt_f1_de | i32_fmt_f2_de | i32_div_de    |
                                i32_tb_de     | i32_msr_de    | i32_mrs_de    |
                                i32_isb_de    |
                                i32_ls_excl_b_de | i32_ls_excl_h_de |
                                i32_ls_excl_w_de |
                                ((i32_ls_single_de &
                                  (i32_ls_fmt1_de |
                                   i32_ls_fmt2_de |
                                   i32_ls_fmt3_de))) |
                                (i32_fmt_3_de & ~i32_sat16_de));

  assign instr_32_wr_d_en_de = (i32_alu_mull_de | i32_alu_mlal_de |
                                i32_alu_div_de  | i32_alu_mul_de  |
                                i32_alu_mla_de  | i32_alu_mls_de  |
                                i32_bl_de    | i32_b0_de    | i32_isb_de   |
                                ((i32_asr1_de  | i32_lsl1_de  | i32_lsr1_de  |
                                  i32_ror1_de  | i32_asr2_de  | i32_lsr2_de  |
                                  i32_lsl2_de  | i32_ror2_de  | i32_bic1_de  |
                                  i32_bic2_de  | i32_adc1_de  | i32_adc2_de  |
                                  i32_subw_de  | i32_addw_de  | i32_movw_de  |
                                  i32_movt_de  | i32_mov1_de  | i32_sbfx_de  |
                                  i32_ubfx_de  | i32_bfi_de   | i32_bfc_de   |
                                  i32_usat_de  | i32_ssat_de  | i32_rsb1_de  |
                                  i32_rsb2_de  | i32_orn1_de  | i32_orn2_de  |
                                  i32_orr1_de  | i32_orr2_de  | i32_mvn1_de  |
                                  i32_mvn2_de  | i32_sub1_de  | i32_sub2_de  |
                                  i32_add1_de  | i32_add2_de  | i32_and1_de  |
                                  i32_and2_de  | i32_eor1_de  | i32_eor2_de  |
                                  i32_sbc1_de  | i32_sbc2_de  | i32_clz_de   |
                                  i32_rbit_de  | i32_rev_de   | i32_rev16_de |
                                  i32_revsh_de | i32_sxth_de  | i32_uxth_de  |
                                  i32_sxtb_de  | i32_uxtb_de) &
                                 (instr_32_de_i[27:24]!=4'b1111))  |
                                (i32_alu_mrs_de &
                                 (instr_32_de_i[23:16] < 8'b00010101) &
                                 (instr_32_de_i[19:16]!=4'b1011) &
                                 (instr_32_de_i[19:18]!=2'b11)) |
                                (i32_msr_de & ~i32_alu_msr_flag_de &
                                 ~(&instr_32_de_i[3:0]) &
                                 (instr_32_de_i[23:17]==7'b0000100) &
                                 instr_privileged_de_i));

  assign reg_addr_sel_de = {// Conditional branch and ISB
                            i32_isb_de,
                            i32_fmt_t1_de,
                            ((i32_fmt_t0_de & ~i32_mov2_de) |
                             i32_fmt_0_de | i32_fmt_1_de  | i32_fmt_t2_de |
                             i32_div_de   | i32_fmt_f1_de | i32_mul_de    |
                             i32_fmt_f2_de),
                            i32_mov2_de | i32_fmt_t4_de,
                            i32_fmt_2_de,
                            i32_fmt_3_de,
                            (i32_mla_de | i32_mls_de),
                            (i32_bl_de | i32_b0_de),
                            ((i32_ls_single_de & instr_32_imm_valid_de) |
                             i32_ls_excl_w_de |
                             ((i32_ls_excl_b_de | i32_ls_excl_h_de) &
                              instr_32_de_i[4])),
                            ((i32_ls_single_de & ~instr_32_imm_valid_de) |
                             i32_tb_de),
                            (i32_ls_double_de | i32_ls_multiple_de |
                             ((i32_ls_excl_b_de | i32_ls_excl_h_de) &
                              ~instr_32_de_i[4])),
                            i32_mrs_de,
                            i32_msr_de };

  assign {  instr_32_lsu_rd_a_addr_de,
            instr_32_lsu_rd_b_addr_de,
            instr_32_lsu_rd_c_addr_de,
            instr_32_lsu_wr_d_addr_de  } =

         ({16{reg_addr_sel_de[4]}}  & { instr_32_de_i[3:0],
                                        instr_32_de_i[31:28],
                                        instr_32_de_i[27:24],
                                        instr_32_de_i[31:28] }) |

         ({16{reg_addr_sel_de[3]}}  & { instr_32_de_i[3:0],
                                        instr_32_de_i[19:16],
                                        `CORE_UNUSED,
                                        instr_32_de_i[31:28] }) |

         ({16{reg_addr_sel_de[2]}}  & { instr_32_de_i[3:0],
                                        instr_32_de_i[31:28],
                                        instr_32_de_i[19:16],
                                        instr_32_de_i[27:24] });

  assign {  instr_32_rd_a_addr_de,
            instr_32_rd_b_addr_de,
            instr_32_rd_c_addr_de,
            instr_32_wr_d_addr_de  } =

        ({16{reg_addr_sel_de[12]}} & { 4'b1111,
                                       4'b1111,
                                       `CORE_UNUSED,
                                       4'b1111              }) |
        ({16{reg_addr_sel_de[11]}} & { instr_32_de_i[19:16],
                                       instr_32_de_i[3:0],
                                       instr_32_de_i[31:28],
                                       instr_32_de_i[27:24] }) |
         ({16{reg_addr_sel_de[10]}} & { instr_32_de_i[3:0],
                                        instr_32_de_i[19:16],
                                        instr_32_de_i[31:28],
                                        instr_32_de_i[27:24] }) |
         ({16{reg_addr_sel_de[9]}}  & { instr_32_de_i[3:0],
                                        instr_32_de_i[19:16],
                                        `CORE_UNUSED,
                                        instr_32_de_i[27:24] }) |
         ({16{reg_addr_sel_de[8]}}  & { instr_32_de_i[3:0],
                                        instr_32_de_i[27:24],
                                        `CORE_UNUSED,
                                        instr_32_de_i[27:24] }) |
         ({16{reg_addr_sel_de[7]}}  & { instr_32_de_i[27:24],
                                        instr_32_de_i[3:0],
                                        `CORE_UNUSED,
                                        instr_32_de_i[27:24] }) |
         ({16{reg_addr_sel_de[6]}}  & { instr_32_de_i[3:0],
                                        instr_32_de_i[19:16],
                                        instr_32_de_i[31:28],
                                        instr_32_de_i[27:24] }) |

         ({16{reg_addr_sel_de[5]}}  & { 4'b1111,
                                        `CORE_UNUSED,
                                        `CORE_UNUSED,
                                        4'b1111              }) |

         ({16{|reg_addr_sel_de[4:2]}} & { instr_32_lsu_rd_a_addr_de,
                                          instr_32_lsu_rd_b_addr_de,
                                          instr_32_lsu_rd_c_addr_de,
                                          instr_32_lsu_wr_d_addr_de }) |

         ({16{reg_addr_sel_de[1]}}  & { `CORE_UNUSED,
                                        4'b1101,
                                        `CORE_UNUSED,
                                        instr_32_de_i[27:24] }) |

         ({16{reg_addr_sel_de[0]}} &  { `CORE_UNUSED,
                                        instr_32_de_i[3:0],
                                        `CORE_UNUSED,
                                        4'b1101              });


  assign instr_32_imm_valid_de = (i32_bl_de    | i32_b0_de    | i32_fmt_0_de |
                                  i32_fmt_1_de | i32_fmt_2_de | i32_fmt_3_de |
                                  i32_lsl1_de  | i32_lsr1_de  | i32_asr1_de  |
                                  i32_ror1_de  |
                                  i32_ls_double_de | i32_ls_multiple_de |
                                  i32_ls_excl_w_de | i32_ls_excl_b_de   |
                                  i32_ls_excl_h_de |
                                  (i32_ls_single_de &
                                   (instr_32_de_i[7]  |
                                    i32_ls_pc_base_de |
                                    instr_32_de_i[27])) |
                                  (i32_mrs_de &
                                   ((instr_32_de_i[23:16]==8'b00001010) |
                                    ~instr_privileged_de_i)));

  assign reg_immed_sel_de = {
        i32_fmt_0_de & ({instr_32_de_i[10],instr_32_de_i[30]}==2'b00) & (instr_32_de_i[29:28]==2'b00),
        i32_fmt_0_de & ({instr_32_de_i[10],instr_32_de_i[30]}==2'b00) & (instr_32_de_i[29:28]==2'b01),
        i32_fmt_0_de & ({instr_32_de_i[10],instr_32_de_i[30]}==2'b00) & (instr_32_de_i[29:28]==2'b10),
        i32_fmt_0_de & ({instr_32_de_i[10],instr_32_de_i[30]}==2'b00) & (instr_32_de_i[29:28]==2'b11),
        i32_fmt_0_de & ({instr_32_de_i[10],instr_32_de_i[30]}!=2'b00),
        i32_fmt_1_de,
        i32_ubfx_de | i32_sbfx_de,
        i32_bfi_de,
        i32_bfc_de,
        i32_fmt_2_de &  instr_32_de_i[7],
        i32_fmt_2_de & ~instr_32_de_i[7],
        (i32_usat_de | i32_ssat_de | i32_sat16_de),
        i32_fmt_t0_de,
        i32_fmt_t2_de,
        i32_bl_de,
        i32_b0_de,
        i32_ls_single_de,
        i32_ls_double_de | i32_ls_excl_w_de,
        i32_ls_multiple_de};

  assign instr_32_immed_de =
  (({32{reg_immed_sel_de[18]}} & {{24{1'b0}},instr_32_de_i[23:16]})   |
   ({32{reg_immed_sel_de[17]}} & {2{{8{1'b0}},instr_32_de_i[23:16]}}) |
   ({32{reg_immed_sel_de[16]}} & {2{instr_32_de_i[23:16],{8{1'b0}}}}) |
   ({32{reg_immed_sel_de[15]}} & {4{instr_32_de_i[23:16]}})           |
   ({32{reg_immed_sel_de[14]}} & i32_shft_ror_immed_de)               |
   ({32{reg_immed_sel_de[13]}} & {{20{1'b0}},instr_32_de_i[10],
                                  instr_32_de_i[30:28],instr_32_de_i[23:16]})  |
   ({32{reg_immed_sel_de[12]}} & {instr_32_de_i[20:16],{22{1'b0}},
                                  instr_32_de_i[30:28],instr_32_de_i[23:22]})  |
   ({32{reg_immed_sel_de[11]}} & {instr_32_de_i[20:16],{14{1'b0}},
                                  instr_32_de_i[30:28],instr_32_de_i[23:22],
                                  3'b000,instr_32_de_i[30:28],
                                  instr_32_de_i[23:22]})                       |
   ({32{reg_immed_sel_de[10]}} & {instr_32_de_i[20:16],{14{1'b0}},
                                  instr_32_de_i[30:28],instr_32_de_i[23:22],
                                  8'b00100000})                                |
   ({32{reg_immed_sel_de[9]}} & {instr_32_de_i[3:0],instr_32_de_i[10],
                                 instr_32_de_i[30:28],instr_32_de_i[23:16],
                                 {16{1'b0}}})                                  |
   ({32{reg_immed_sel_de[8]}} & {{16{1'b0}},instr_32_de_i[3:0],
                                 instr_32_de_i[10],instr_32_de_i[30:28],
                                 instr_32_de_i[23:16]})                        |
   ({32{reg_immed_sel_de[7]}} & {instr_32_de_i[20:16],{22{1'b0}},
                                 instr_32_de_i[30:28],instr_32_de_i[23:22]})   |
   ({32{reg_immed_sel_de[6]}} & {{26{1'b0}},^instr_32_de_i[21:20] &
                                 ~|{instr_32_de_i[30:28],instr_32_de_i[23:22]},
                                 instr_32_de_i[30:28],instr_32_de_i[23],
                                 instr_32_de_i[22] | i32_shft_rrx_de})         |
   ({32{reg_immed_sel_de[5]}} & {{27{1'b0}},instr_32_de_i[21:20],3'b000})      |
   ({32{reg_immed_sel_de[4]}} & {{8{instr_32_de_i[10]}},
                                 ~(instr_32_de_i[29]^instr_32_de_i[10]),
                                 ~(instr_32_de_i[27]^instr_32_de_i[10]),
                                 instr_32_de_i[9:0],instr_32_de_i[26:16],1'b0}) |
   ({32{reg_immed_sel_de[3]}} & {{11{instr_32_de_i[10]}},instr_32_de_i[10],
                                 instr_32_de_i[27],instr_32_de_i[29],
                                 instr_32_de_i[5:0],instr_32_de_i[26:16],1'b0}) |
   ({32{reg_immed_sel_de[2]}} & {{20{1'b0}},
                                 {4{(&instr_32_de_i[4:0]) | instr_32_de_i[7]}} &
                                 instr_32_de_i[27:24], instr_32_de_i[23:16]})   |
   ({32{reg_immed_sel_de[1]}} & {{22{1'b0}},instr_32_de_i[23:16],2'b00})        |
   ({32{reg_immed_sel_de[0]}} & instr_32_lsm_immed_de));

  assign instr_32_lsm_immed_de = ({32{reg_immed_sel_de[0]}} &
                                  {{16{1'b0}},instr_32_de_i[31:30],
                                   1'b0,instr_32_de_i[28:16]});

  assign instr_32_alu_1_cyc_de_o      = instr_32_alu_1_cyc_de;
  assign instr_32_algn_de_o           = instr_32_algn_de;
  assign instr_32_mla_de_o            = instr_32_mla_de;
  assign instr_32_mul_ilock_de_o      = instr_32_mul_ilock_de;
  assign instr_32_ls_single_wb_de_o   = instr_32_ls_single_wb_de;
  assign instr_32_ls_allow_skid_de_o  = instr_32_ls_allow_skid_de;
  assign instr_32_lsd_baseinlist_de_o = instr_32_lsd_baseinlist_de;
  assign instr_32_immed_de_o          = instr_32_immed_de;
  assign instr_32_lsm_immed_de_o      = instr_32_lsm_immed_de;
  assign instr_32_imm_valid_de_o      = instr_32_imm_valid_de;
  assign instr_32_data_size_de_o      = instr_32_data_size_de;
  assign instr_32_sign_de_o           = instr_32_sign_de;
  assign instr_32_exit_op_de_o        = instr_32_exit_op_de;
  assign instr_32_undef_de_o          = instr_32_undef_de;
  assign instr_32_txev_de_o           = instr_32_txev_de;
  assign instr_32_pc_lr_de_o          = instr_32_pc_lr_de;
  assign instr_32_isb_de_o            = instr_32_isb_de;
  assign instr_32_br_t_changer_de_o   = instr_32_br_t_changer_de;
  assign instr_32_noncc_br_de_o       = instr_32_noncc_br_de;
  assign instr_32_flg_set_de_o        = instr_32_flg_set_de;
  assign instr_32_msr_ctl_de_o        = instr_32_msr_ctl_de;
  assign instr_32_barrier_de_o        = instr_32_barrier_de;
  assign instr_32_cond_code_de_o      = instr_32_cond_code_de;
  assign instr_32_copro_de_o          = instr_32_copro_de;
  assign instr_32_wfe_de_o            = instr_32_wfe_de;
  assign instr_32_wfi_de_o            = instr_32_wfi_de;
  assign instr_32_has_shft_imm_de_o   = instr_32_has_shft_imm_de;
  assign instr_32_alu_ctl_de_o        = instr_32_alu_ctl_de;
  assign instr_32_lsu_ctl_de_o        = instr_32_lsu_ctl_de;
  assign instr_32_sp_rd_de_o          = instr_32_sp_rd_de;
  assign instr_32_sp_wr_de_o          = instr_32_sp_wr_de;
  assign instr_32_lsu_rd_a_addr_de_o  = instr_32_lsu_rd_a_addr_de;
  assign instr_32_lsu_rd_b_addr_de_o  = instr_32_lsu_rd_b_addr_de;
  assign instr_32_rd_a_en_de_o        = instr_32_rd_a_en_de;
  assign instr_32_rd_b_en_de_o        = instr_32_rd_b_en_de;
  assign instr_32_wr_d_en_de_o        = instr_32_wr_d_en_de;
  assign instr_32_rd_c_addr_de_o      = instr_32_rd_c_addr_de;
  assign instr_32_rd_a_addr_de_o      = instr_32_rd_a_addr_de;
  assign instr_32_rd_b_addr_de_o      = instr_32_rd_b_addr_de;
  assign instr_32_wr_d_addr_de_o      = instr_32_wr_d_addr_de;


`ifdef ARM_ASSERT_ON

  assert_zero_one_hot #(0,88,0,"OpCodes must be exclusive")
    ovl_opcode_excl(hclk, hreset_n, {88{instr_valid_de_i}} &
                    {i32_adc1_de,  i32_add1_de,  i32_and1_de,  i32_bic1_de,
                     i32_cmn1_de,  i32_cmp1_de,  i32_eor1_de,  i32_mov1_de,
                     i32_mvn1_de,  i32_orn1_de,  i32_orr1_de,  i32_rsb1_de,
                     i32_sbc1_de,  i32_sub1_de,  i32_teq1_de,  i32_tst1_de,
                     i32_addw_de,  i32_subw_de,  i32_movw_de,  i32_movt_de,
                     i32_ubfx_de,  i32_sbfx_de,  i32_bfc_de,   i32_bfi_de,
                     i32_adc2_de,  i32_add2_de,  i32_and2_de,  i32_bic2_de,
                     i32_cmn2_de,  i32_cmp2_de,  i32_eor2_de,  i32_lsl1_de,
                     i32_lsr1_de,  i32_asr1_de,  i32_ror1_de,  i32_mvn2_de,
                     i32_orn2_de,  i32_orr2_de,  i32_rsb2_de,  i32_sbc2_de,
                     i32_sub2_de,  i32_teq2_de,  i32_tst2_de,  i32_lsl2_de,
                     i32_lsr2_de,  i32_asr2_de,  i32_ror2_de,  i32_sxth_de,
                     i32_sxtb_de,  i32_uxth_de,  i32_uxtb_de,  i32_rbit_de,
                     i32_rev_de,   i32_rev16_de, i32_revsh_de, i32_clz_de,
                     i32_mul_de,   i32_mla_de,   i32_mls_de,   i32_umull_de,
                     i32_smull_de, i32_umlal_de, i32_smlal_de, i32_udiv_de,
                     i32_sdiv_de,  i32_usat_de,  i32_ssat_de,  i32_bl_de,
                     i32_b0_de,    i32_ldrsh_de, i32_tb_de,    i32_clrex_de,
                     i32_copro_de, i32_wfe_de,   i32_wfi_de,   i32_sev_de,
                     i32_nop_de,   i32_msr_de,   i32_mrs_de,   i32_dmb_de,
                     i32_dsb_de,   i32_isb_de,
                     i32_ls_single_de, i32_ls_double_de, i32_ls_multiple_de,
                     i32_ls_excl_w_de, i32_ls_excl_b_de, i32_ls_excl_h_de
                     });

  assert_zero_one_hot #(0,4,0,"Load store single formats must be exclusive")
    ovl_ldst_excl(hclk, hreset_n, ({i32_ls_fmt0_de,i32_ls_fmt1_de,
                                    i32_ls_fmt2_de,i32_ls_fmt3_de} &
                                   {4{instr_valid_de_i}}));

  assert_zero_one_hot #(0,13,0,"RF Address selector should be exclusive")
    ovl_addr_excl(hclk, hreset_n, ({13{instr_valid_de_i}} &
                                   reg_addr_sel_de));

  assert_zero_one_hot #(0,19,0, "Immediate selector should be exclusive")
    ovl_immed_excl(hclk, hreset_n, (reg_immed_sel_de &
                                    {19{instr_valid_de_i}}));

`endif

`include "cm3_dpu_undefs.vp"

endmodule
