`include "cm3_dpu_defs.vp"

module cm3_dpu_alu
  (// Inputs
   hreset_n, hclk, ext_cg_bypass_i, instr_alu_ctl_de_i, instr_mla_track_de_i,
   instr_immed_de_i, instr_rf_rd_c_addr_de_i, instr_rf_wr_d_en_de_i,
   instr_rf_wr_d_addr_de_i, instr_de_2_ex_en_i, instr_ccpass_ex_i,
   instr_immed_valid_ex_i, instr_immed_ex_i, instr_data_size_ex_i,
   instr_algn_ex_i, instr_sign_ex_i, instr_exit_op_ex_i, rf_rd_a_data_ex_i,
   rf_rd_a_zero_ex_i, rf_rd_b_data_ex_i, rf_rd_b_zero_ex_i, e_simd_res_ex_i,
   privileged_i, nvic_basepri_i, nvic_fmask_i, nvic_pmask_i, nvic_div_0_trp_i,
   nvic_int_invoke_i, int_entry_i, int_isr_ex_i, psr_ex_i, process_sp_ex_i,
   user_thread_ex_i, fpu_active_ex_i,
   instr_flag_wr_ex_o, instr_rf_wr_d_addr_ex_o, alu_shift_res_ex_o,
   alu_will_wr_rf_ex_o, alu_rf_rd_a_en_ex_o, alu_rf_rd_a_addr_ex_o,
   alu_rf_wr_d_en_ex_o, alu_rf_wr_d_addr_ex_o, alu_rf_wr_d_data_ex_o,
   alu_zero_res_ex_o, alu_flag_wr_ex_o, alu_flag_res_ex_o, alu_ge_res_ex_o,
   alu_mul_flag_res_ex_o, alu_exit_valid_ex_o, alu_exit_code_ex_o,
   alu_stall_ex_o, alu_mcyc_invk_msk_ex_o, alu_mcyc_ilock_ex_o,
   fault_divby0_ex_o
   );

  parameter CLKGATE_PRESENT = 0; // Instantiate architectural clock gating if 1
  parameter RESET_ALL_REGS  = 0; // Reset all registers if 1
  parameter EXT_PRESENT     = 0; // Extensions present if 1
  parameter FPU_PRESENT     = 0; // FPU present if 1


  input          hreset_n;
  input          hclk;
  input          ext_cg_bypass_i;          // Architectural clock gate bypass
  input [20:0]   instr_alu_ctl_de_i;       // ALU unit control
  input          instr_mla_track_de_i;     // MLA instr in progress
  input [31:0]   instr_immed_de_i;         // Immediate value (de version)
  input [3:0]    instr_rf_rd_c_addr_de_i;  // RF secondary read address
  input          instr_rf_wr_d_en_de_i;    // RF D port write enable
  input [3:0]    instr_rf_wr_d_addr_de_i;  // RF D port write address
  input          instr_de_2_ex_en_i;       // Decode -> execute advance
  input          instr_ccpass_ex_i;        // Condition code pass
  input          instr_immed_valid_ex_i;   // Immediate valid
  input [31:0]   instr_immed_ex_i;         // Immediate value (ex version)
  input [1:0]    instr_data_size_ex_i;     // Data size
  input          instr_algn_ex_i;          // Align data
  input          instr_sign_ex_i;          // Signed operation or sign extend
  input          instr_exit_op_ex_i;       // Valid exit instruction in execute
  input [31:0]   rf_rd_a_data_ex_i;        // RF A port read data
  input [1:0]    rf_rd_a_zero_ex_i;        // RF A port read data HW zero
  input [31:0]   rf_rd_b_data_ex_i;        // RF B port read data
  input [1:0]    rf_rd_b_zero_ex_i;        // RF B port read data HW zero
  input [31:0]   e_simd_res_ex_i;          // SIMD unit result
  input          privileged_i;             // Privileged (vs user) mode
  input [7:0]    nvic_basepri_i;           // BASEPRI value
  input          nvic_fmask_i;             // FAULTMASK value
  input          nvic_pmask_i;             // PRIMASK value
  input          nvic_div_0_trp_i;         // DIV by zero fault enable
  input          nvic_int_invoke_i;        // Interrupt stack request
  input          int_entry_i;              // Interrupt ISR entered
  input [8:0]    int_isr_ex_i;             // Current ISR number
  input [31:0]   psr_ex_i;                 // PSR
  input [1:0]    process_sp_ex_i;          // Current SP is process (vs main)
  input          user_thread_ex_i;         // Thread is (or will be) user mode
  input          fpu_active_ex_i;          // FPU active in current context

  output         instr_flag_wr_ex_o;       // Instruction writes to the flags
  output [3:0]   instr_rf_wr_d_addr_ex_o;  // RF D port write address from instr
  output [31:0]  alu_shift_res_ex_o;       // ALU shifter result
  output         alu_will_wr_rf_ex_o;      // ALU will write to RF
  output         alu_rf_rd_a_en_ex_o;      // ALU RF A port read enable
  output [3:0]   alu_rf_rd_a_addr_ex_o;    // ALU RF A port read address
  output         alu_rf_wr_d_en_ex_o;      // ALU RF D port write enable
  output [3:0]   alu_rf_wr_d_addr_ex_o;    // ALU RF D port write address
  output [31:0]  alu_rf_wr_d_data_ex_o;    // ALU RF D port write data
  output         alu_zero_res_ex_o;        // ALU result is zero
  output [5:0]   alu_flag_wr_ex_o;         // ALU NZCVQ GE result write enables
  output [4:0]   alu_flag_res_ex_o;        // ALU flag result
  output [3:0]   alu_ge_res_ex_o;          // ALU GE bits result
  output [4:0]   alu_mul_flag_res_ex_o;    // ALU MUL flag result (critical)
  output         alu_exit_valid_ex_o;      // ALU operation is an ISR exit
  output [4:0]   alu_exit_code_ex_o;       // ALU ISR exit code
  output         alu_stall_ex_o;           // ALU operation requires stall
  output         alu_mcyc_invk_msk_ex_o;   // ALU multi-cycle op interrupt mask
  output         alu_mcyc_ilock_ex_o;      // ALU multi-cycle op interlock
  output         fault_divby0_ex_o;        // DIV causes a divide by 0 fault


  wire           alu_cg_en;                // ALU clock gate enable
  wire           aclk;                     // ALU multi-cycle operation clock
  wire           iaclk;                    // ALU multi-cycle operation clock
  wire [20:0]    instr_alu_ctl_ex;         // ALU unit control
  wire           instr_flag_wr_ex;         // Instruction writes to the flags
  wire [3:0]     instr_rf_wr_d_addr_ex;    // RF D port write address from instr
  wire [3:0]     alu_mcyc_state_ex;        // Multi-cycle state
  wire [5:0]     mul_input_sel_ex;         // Multiplier input selects
  wire [31:0]    shift_in1_ex;             // ALU shifter input 1
  wire [7:0]     shift_in2_fast_ex;        // Shifter input 2 (fast version)
  wire [31:0]    shift_l_mask_ex;          // Shifter mask (left shift only)
  wire [31:0]    shift_r_mask_ex;          // Shifter mask (right shift only)
  wire [31:0]    shift_lr_mask_ex;         // Shifter mask (left & right shift)
  wire [6:0]     shift_sel_ex;             // Shifter result select
  wire           mul_neg_in2_ex;           // Invert multiplier input 2
  wire           mul_add_valid_ex;         // Multiply result input to adder
  wire           mul_cin_ex;               // Adder carry in for multiply
  wire           mul_mla_track_ex;         // MLA instr in progress
  wire           mul_mull_extend_ex;       // Extend *MULL operation
  wire [1:0]     div_srt_init_ex;          // DIV SRT unit intialisation value
  wire           div_srt_sign_ex;          // DIV SRT unit sign value
  wire [5:0]     div_cnt_ex;               // DIV operation cycle count
  wire [1:0]     div_cnt_reg_ex;           // Registered div_cnt_ex
  wire [31:0]    shift_res_ex;             // ALU shifter result
  wire [5:0]     clz_res_ex;               // ALU count leading zeros result
  wire           div_fast_zero_ex;         // Early zero detection for DIV
  wire           alu_cout_ex;              // ALU carry out
  wire           alu_will_wr_rf_ex;        // ALU will write to RF
  wire           alu_rf_rd_a_en_ex;        // ALU RF A port read enable
  wire [3:0]     alu_rf_rd_a_addr_ex;      // ALU RF A port read address
  wire           alu_rf_wr_d_en_ex;        // ALU RF D port write enable
  wire [3:0]     alu_rf_wr_d_addr_ex;      // ALU RF D port write address
  wire [31:0]    alu_rf_wr_d_data_ex;      // ALU RF D port write data
  wire           alu_zero_res_ex;          // ALU result is zero
  wire [5:0]     alu_flag_wr_ex;           // ALU NZCVQ GE result write enables
  wire [4:0]     alu_flag_res_ex;          // ALU flag result
  wire [3:0]     alu_ge_res_ex;            // ALU GE bits result
  wire [4:0]     alu_mul_flag_res_ex;      // ALU MUL flag result (critical)
  wire           alu_exit_valid_ex;        // ALU operation is an ISR exit
  wire [4:0]     alu_exit_code_ex;         // ALU ISR exit code
  wire           alu_stall_ex;             // ALU operation requires stall
  wire           alu_mcyc_invk_msk_ex;     // ALU multi-cycle op mask stack req
  wire           alu_mcyc_ilock_ex;        // ALU multi-cycle op interlock
  wire           fault_divby0_ex;          // DIV causes a divide by 0 fault
  wire           opt_clk_gate_en;          // Clock Gate logic removal

`ifdef ARM_DSM

  wire           dsm_clkgate_present;

  assign opt_clk_gate_en   = (dsm_clkgate_present) ? 1'b1 : 1'b0;

`else

  assign opt_clk_gate_en   = (CLKGATE_PRESENT == 1) ? 1'b1 : 1'b0;

`endif

  cm3_clk_gate #(CLKGATE_PRESENT)
    u_cm3_clk_gate_alu
      (// Inputs
       .clk                        (hclk),
       .clk_enable_i               (alu_cg_en),
       .global_disable_i           (ext_cg_bypass_i),
       .clk_gated_o                (iaclk)
       );

  assign aclk = (opt_clk_gate_en) ? iaclk : hclk;

  cm3_dpu_alu_ctl #(RESET_ALL_REGS, EXT_PRESENT, FPU_PRESENT)
    u_cm3_dpu_alu_ctl
      (// Inputs
       .hreset_n                   (hreset_n),
       .hclk                       (hclk),
       .aclk                       (aclk),
       .nvic_div_0_trp_i           (nvic_div_0_trp_i),
       .nvic_int_invoke_i          (nvic_int_invoke_i),
       .int_entry_i                (int_entry_i),
       .instr_de_2_ex_en_i         (instr_de_2_ex_en_i),
       .instr_alu_ctl_de_i         (instr_alu_ctl_de_i),
       .instr_mla_track_de_i       (instr_mla_track_de_i),
       .instr_rf_rd_c_addr_de_i    (instr_rf_rd_c_addr_de_i),
       .instr_rf_wr_d_en_de_i      (instr_rf_wr_d_en_de_i),
       .instr_rf_wr_d_addr_de_i    (instr_rf_wr_d_addr_de_i),
       .instr_immed_de_i           (instr_immed_de_i),
       .instr_immed_ex_i           (instr_immed_ex_i),
       .instr_ccpass_ex_i          (instr_ccpass_ex_i),
       .instr_sign_ex_i            (instr_sign_ex_i),
       .instr_data_size_ex_i       (instr_data_size_ex_i),
       .instr_exit_op_ex_i         (instr_exit_op_ex_i),
       .rf_rd_a_data_ex_i          (rf_rd_a_data_ex_i),
       .rf_rd_a_zero_ex_i          (rf_rd_a_zero_ex_i),
       .rf_rd_b_data_ex_i          (rf_rd_b_data_ex_i),
       .rf_rd_b_zero_ex_i          (rf_rd_b_zero_ex_i),
       .shift_in1_ex_i             (shift_in1_ex),
       .shift_res_ex_i             (shift_res_ex),
       .clz_res_ex_i               (clz_res_ex),
       .alu_cout_ex_i              (alu_cout_ex),
       .alu_cg_en_o                (alu_cg_en),
       .instr_flag_wr_ex_o         (instr_flag_wr_ex),
       .instr_alu_ctl_ex_o         (instr_alu_ctl_ex),
       .alu_mcyc_state_ex_o        (alu_mcyc_state_ex),
       .mul_input_sel_ex_o         (mul_input_sel_ex),
       .div_srt_init_ex_o          (div_srt_init_ex),
       .div_srt_sign_ex_o          (div_srt_sign_ex),
       .div_fast_zero_ex_o         (div_fast_zero_ex),
       .div_cnt_ex_o               (div_cnt_ex),
       .div_cnt_reg_ex_o           (div_cnt_reg_ex),
       .mul_neg_in2_ex_o           (mul_neg_in2_ex),
       .mul_add_valid_ex_o         (mul_add_valid_ex),
       .mul_mull_extend_ex_o       (mul_mull_extend_ex),
       .mul_mla_track_ex_o         (mul_mla_track_ex),
       .mul_cin_ex_o               (mul_cin_ex),
       .shift_in2_fast_ex_o        (shift_in2_fast_ex),
       .shift_sel_ex_o             (shift_sel_ex),
       .shift_l_mask_ex_o          (shift_l_mask_ex),
       .shift_r_mask_ex_o          (shift_r_mask_ex),
       .shift_lr_mask_ex_o         (shift_lr_mask_ex),
       .alu_rf_rd_a_en_ex_o        (alu_rf_rd_a_en_ex),
       .alu_rf_rd_a_addr_ex_o      (alu_rf_rd_a_addr_ex),
       .alu_rf_wr_d_en_ex_o        (alu_rf_wr_d_en_ex),
       .alu_rf_wr_d_addr_ex_o      (alu_rf_wr_d_addr_ex),
       .alu_will_wr_rf_ex_o        (alu_will_wr_rf_ex),
       .instr_rf_wr_d_addr_ex_o    (instr_rf_wr_d_addr_ex),
       .alu_exit_valid_ex_o        (alu_exit_valid_ex),
       .alu_exit_code_ex_o         (alu_exit_code_ex),
       .alu_stall_ex_o             (alu_stall_ex),
       .alu_mcyc_invk_msk_ex_o     (alu_mcyc_invk_msk_ex),
       .alu_mcyc_ilock_ex_o        (alu_mcyc_ilock_ex),
       .alu_zero_res_ex_o          (alu_zero_res_ex),
       .alu_flag_wr_ex_o           (alu_flag_wr_ex),
       .fault_divby0_ex_o          (fault_divby0_ex)
       );

  cm3_dpu_alu_dp #(RESET_ALL_REGS, EXT_PRESENT)
    u_cm3_dpu_alu_dp
      (// Inputs
       .hreset_n                   (hreset_n),
       .hclk                       (hclk),
       .aclk                       (aclk),
       .alu_cg_en_i                (alu_cg_en),
       .instr_immed_ex_i           (instr_immed_ex_i),
       .instr_immed_valid_ex_i     (instr_immed_valid_ex_i),
       .instr_algn_ex_i            (instr_algn_ex_i),
       .instr_sign_ex_i            (instr_sign_ex_i),
       .instr_data_size_ex_i       (instr_data_size_ex_i),
       .instr_alu_ctl_ex_i         (instr_alu_ctl_ex),
       .alu_mcyc_state_ex_i        (alu_mcyc_state_ex),
       .mul_input_sel_ex_i         (mul_input_sel_ex),
       .mul_neg_in2_ex_i           (mul_neg_in2_ex),
       .mul_add_valid_ex_i         (mul_add_valid_ex),
       .mul_mla_track_ex_i         (mul_mla_track_ex),
       .mul_cin_ex_i               (mul_cin_ex),
       .mul_mull_extend_ex_i       (mul_mull_extend_ex),
       .div_fast_zero_ex_i         (div_fast_zero_ex),
       .div_srt_init_ex_i          (div_srt_init_ex),
       .div_srt_sign_ex_i          (div_srt_sign_ex),
       .div_cnt_ex_i               (div_cnt_ex),
       .div_cnt_reg_ex_i           (div_cnt_reg_ex),
       .shift_in2_fast_ex_i        (shift_in2_fast_ex),
       .shift_l_mask_ex_i          (shift_l_mask_ex),
       .shift_r_mask_ex_i          (shift_r_mask_ex),
       .shift_lr_mask_ex_i         (shift_lr_mask_ex),
       .shift_sel_ex_i             (shift_sel_ex),
       .rf_rd_a_data_ex_i          (rf_rd_a_data_ex_i),
       .rf_rd_b_data_ex_i          (rf_rd_b_data_ex_i),
       .e_simd_res_ex_i            (e_simd_res_ex_i),
       .psr_ex_i                   (psr_ex_i),
       .privileged_i               (privileged_i),
       .process_sp_ex_i            (process_sp_ex_i),
       .user_thread_ex_i           (user_thread_ex_i),
       .fpu_active_ex_i            (fpu_active_ex_i),
       .int_isr_ex_i               (int_isr_ex_i),
       .nvic_basepri_i             (nvic_basepri_i),
       .nvic_fmask_i               (nvic_fmask_i),
       .nvic_pmask_i               (nvic_pmask_i),
       .shift_in1_ex_o             (shift_in1_ex),
       .shift_res_ex_o             (shift_res_ex),
       .clz_res_ex_o               (clz_res_ex),
       .alu_cout_ex_o              (alu_cout_ex),
       .alu_flag_res_ex_o          (alu_flag_res_ex),
       .alu_ge_res_ex_o            (alu_ge_res_ex),
       .alu_mul_flag_res_ex_o      (alu_mul_flag_res_ex),
       .alu_rf_wr_d_data_ex_o      (alu_rf_wr_d_data_ex)
       );

  assign instr_flag_wr_ex_o      = instr_flag_wr_ex;
  assign instr_rf_wr_d_addr_ex_o = instr_rf_wr_d_addr_ex;
  assign alu_shift_res_ex_o      = shift_res_ex;
  assign alu_will_wr_rf_ex_o     = alu_will_wr_rf_ex;
  assign alu_rf_rd_a_en_ex_o     = alu_rf_rd_a_en_ex;
  assign alu_rf_rd_a_addr_ex_o   = alu_rf_rd_a_addr_ex;
  assign alu_rf_wr_d_en_ex_o     = alu_rf_wr_d_en_ex;
  assign alu_rf_wr_d_addr_ex_o   = alu_rf_wr_d_addr_ex;
  assign alu_rf_wr_d_data_ex_o   = alu_rf_wr_d_data_ex;
  assign alu_zero_res_ex_o       = alu_zero_res_ex;
  assign alu_flag_wr_ex_o        = alu_flag_wr_ex;
  assign alu_flag_res_ex_o       = alu_flag_res_ex;
  assign alu_ge_res_ex_o         = alu_ge_res_ex;
  assign alu_mul_flag_res_ex_o   = alu_mul_flag_res_ex;
  assign alu_exit_valid_ex_o     = alu_exit_valid_ex;
  assign alu_exit_code_ex_o      = alu_exit_code_ex;
  assign alu_stall_ex_o          = alu_stall_ex;
  assign alu_mcyc_invk_msk_ex_o  = alu_mcyc_invk_msk_ex;
  assign alu_mcyc_ilock_ex_o     = alu_mcyc_ilock_ex;
  assign fault_divby0_ex_o       = fault_divby0_ex;

`include "cm3_dpu_undefs.vp"

endmodule
