`include "cm3_dpu_defs.vp"

module cm3_dpu_alu_ctl
  (// Inputs
   hreset_n, hclk, aclk, nvic_div_0_trp_i, nvic_int_invoke_i, int_entry_i,
   instr_de_2_ex_en_i, instr_alu_ctl_de_i, instr_mla_track_de_i,
   instr_rf_rd_c_addr_de_i, instr_rf_wr_d_en_de_i, instr_rf_wr_d_addr_de_i,
   instr_immed_de_i, instr_immed_ex_i, instr_ccpass_ex_i, instr_sign_ex_i,
   instr_data_size_ex_i, instr_exit_op_ex_i, rf_rd_a_data_ex_i,
   rf_rd_a_zero_ex_i, rf_rd_b_data_ex_i, rf_rd_b_zero_ex_i, shift_in1_ex_i,
   shift_res_ex_i, clz_res_ex_i, alu_cout_ex_i,
   alu_cg_en_o, instr_flag_wr_ex_o, instr_alu_ctl_ex_o, alu_mcyc_state_ex_o,
   mul_input_sel_ex_o, div_srt_init_ex_o, div_srt_sign_ex_o, div_fast_zero_ex_o,
   div_cnt_ex_o, div_cnt_reg_ex_o, mul_neg_in2_ex_o, mul_add_valid_ex_o,
   mul_mull_extend_ex_o, mul_mla_track_ex_o, mul_cin_ex_o, shift_in2_fast_ex_o,
   shift_sel_ex_o, shift_l_mask_ex_o, shift_r_mask_ex_o, shift_lr_mask_ex_o,
   alu_rf_rd_a_en_ex_o, alu_rf_rd_a_addr_ex_o, alu_rf_wr_d_en_ex_o,
   alu_rf_wr_d_addr_ex_o, alu_will_wr_rf_ex_o, instr_rf_wr_d_addr_ex_o,
   alu_exit_valid_ex_o, alu_exit_code_ex_o, alu_stall_ex_o,
   alu_mcyc_invk_msk_ex_o, alu_mcyc_ilock_ex_o, alu_zero_res_ex_o,
   alu_flag_wr_ex_o, fault_divby0_ex_o
   );

  parameter      RESET_ALL_REGS = 0;       // Reset all registers if 1
  parameter      EXT_PRESENT    = 0;       // Extensions present if 1
  parameter      FPU_PRESENT    = 0;       // FPU present if 1

  input          hreset_n;
  input          hclk;
  input          aclk;                     // ALU multi-cycle operation clock
  input          nvic_div_0_trp_i;         // DIV by zero fault enable
  input          nvic_int_invoke_i;        // Interrupt stack request
  input          int_entry_i;              // Interrupt ISR entered
  input          instr_de_2_ex_en_i;       // Decode -> execute advance
  input [20:0]   instr_alu_ctl_de_i;       // ALU unit control
  input          instr_mla_track_de_i;     // MLA instr in progress
  input [3:0]    instr_rf_rd_c_addr_de_i;  // RF secondary read address
  input          instr_rf_wr_d_en_de_i;    // RF D port write enable
  input [3:0]    instr_rf_wr_d_addr_de_i;  // RF D port write address
  input [31:0]   instr_immed_de_i;         // Immediate value (de version)
  input [31:0]   instr_immed_ex_i;         // Immediate value (ex version)
  input          instr_ccpass_ex_i;        // Condition code pass
  input          instr_sign_ex_i;          // Signed operation or sign extend
  input [1:0]    instr_data_size_ex_i;     // Data size
  input          instr_exit_op_ex_i;       // Valid exit instruction in execute
  input [31:0]   rf_rd_a_data_ex_i;        // RF A port read data
  input [1:0]    rf_rd_a_zero_ex_i;        // RF A port read data HW zero
  input [31:0]   rf_rd_b_data_ex_i;        // RF B port read data
  input [1:0]    rf_rd_b_zero_ex_i;        // RF B port read data HW zero
  input [31:0]   shift_in1_ex_i;           // ALU shifter input 1
  input [31:0]   shift_res_ex_i;           // ALU shifter result
  input [5:0]    clz_res_ex_i;             // ALU count leading zeros result
  input          alu_cout_ex_i;            // ALU carry out

  output         alu_cg_en_o;              // ALU clock gate enable
  output         instr_flag_wr_ex_o;       // Instruction writes to the flags
  output [20:0]  instr_alu_ctl_ex_o;       // ALU unit control
  output [3:0]   alu_mcyc_state_ex_o;      // ALU multi-cycle operation state
  output [5:0]   mul_input_sel_ex_o;       // Multiplier input selects
  output [1:0]   div_srt_init_ex_o;        // DIV SRT unit intialisation value
  output         div_srt_sign_ex_o;        // DIV SRT unit sign value
  output         div_fast_zero_ex_o;       // Early zero detection for DIV
  output [5:0]   div_cnt_ex_o;             // DIV operation cycle count
  output [1:0]   div_cnt_reg_ex_o;         // Registered div_cnt_ex
  output         mul_neg_in2_ex_o;         // Invert multiplier input 2
  output         mul_add_valid_ex_o;       // Multiply result input to adder
  output         mul_mull_extend_ex_o;     // Extend *MULL operation
  output         mul_mla_track_ex_o;       // MLA instr in progress
  output         mul_cin_ex_o;             // Adder carry in for multiply
  output [7:0]   shift_in2_fast_ex_o;      // Shifter input 2 (fast version)
  output [6:0]   shift_sel_ex_o;           // Shifter result select
  output [31:0]  shift_l_mask_ex_o;        // Shifter mask (left shift only)
  output [31:0]  shift_r_mask_ex_o;        // Shifter mask (right shift only)
  output [31:0]  shift_lr_mask_ex_o;       // Shifter mask (left & right shift)
  output         alu_rf_rd_a_en_ex_o;      // ALU RF A port read enable
  output [3:0]   alu_rf_rd_a_addr_ex_o;    // ALU RF A port read address
  output         alu_rf_wr_d_en_ex_o;      // ALU RF D port write enable
  output [3:0]   alu_rf_wr_d_addr_ex_o;    // ALU RF D port write address
  output         alu_will_wr_rf_ex_o;      // ALU will write to RF
  output [3:0]   instr_rf_wr_d_addr_ex_o;  // RF D port write address
  output         alu_exit_valid_ex_o;      // ALU operation is an ISR exit
  output [4:0]   alu_exit_code_ex_o;       // ALU ISR exit code
  output         alu_stall_ex_o;           // ALU operation requires stall
  output         alu_mcyc_invk_msk_ex_o;   // ALU multi-cycle op mask stack req
  output         alu_mcyc_ilock_ex_o;      // ALU multi-cycle op interlock
  output         alu_zero_res_ex_o;        // ALU result is zero
  output [5:0]   alu_flag_wr_ex_o;         // ALU NZCVQ GE result write enables
  output         fault_divby0_ex_o;        // DIV causes a divide by 0 fault


  wire           full_hreset_n;            // Apply hreset_n to all registers
  wire           opt_ext_en;               // Master enable for extensions
  wire           opt_fpu_en;               // Master enable for FPU
  wire           alu_mul_de;               // MUL instruction
  wire           alu_ml_de;                // MLA/MLS instruction
  wire           alu_mull_de;              // SMULL/UMULL instruction
  wire           alu_mlal_de;              // SMLAL/UMLAL instruction
  wire           alu_div_de;               // SDIV/UDIV instruction
  wire           alu_actv_de;              // ALU active
  wire           alu_multicyc_de;          // ALU multi-cycle operation decoded
  wire           alu_mcyc_op_valid_de;     // ALU multi-cycle operation valid
  wire [3:0]     alu_mcyc_op_de;           // ALU operation is multi-cycle
  wire           mul_mla_pipe_de;          // Allow MLA pipe-lining
  wire [7:0]     shift_in2_fast_de;        // Shifter input 2 (fast version)
  wire           alu_rf_clr_ex;            // Clear RF information
  wire           alu_de_2_ex_en;           // ALU decode -> execute
  wire           alu_rf_de_2_ex_en;        // ALU decode -> execute
  wire [3:0]     nxt_alu_rf_rd_c_addr_ex;  // Next RF secondary read address
  reg  [3:0]     alu_rf_rd_c_addr_ex;      // RF secondary read address
  reg  [3:0]     instr_rf_wr_d_addr_ex;    // RF D port write address
  reg            instr_rf_wr_d_en_ex;      // RF D port write enable
  reg  [20:0]    instr_alu_ctl_ex;         // ALU unit control
  wire           alu_shift_ex;             // Shifter operation
  wire           alu_clz_ex;               // CLZ instruction
  wire           alu_mov_ex;               // MOV instruction
  wire           alu_muldiv_ex;            // Multiply/divide instruction
  wire           alu_bfx_ex;               // SBFX/UBFX instruction
  wire           alu_bfield_ex;            // Bitfield instruction
  wire           alu_reg_shft_ex;          // Regular ALU shift
  wire           alu_sat_ex;               // Saturation instruction
  wire           alu_msr_flag_ex;          // MSR operation to flags
  wire           alu_mlal_ex;              // SMLAL/UMLAL instruction
  wire           alu_mul_ex;               // MUL instruction
  wire           alu_ml_ex;                // MLA/MLS instruction
  wire           alu_mls_ex;               // MLS instruction
  wire           alu_mull_ex;              // SMULL/UMULL instruction
  wire           alu_div_ex;               // SDIV/UDIV instruction
  wire           alu_sdiv_ex;              // SDIV instruction
  wire           alu_udiv_ex;              // UDIV instruction
  wire           alu_multicyc_ex;          // ALU operation is multi-cycle
  wire           alu_actv_ex;              // ALU active
  wire           alu_nflag_wr_ex;          // ALU operation writes to N flag
  wire           alu_zflag_wr_ex;          // ALU operation writes to Z flag
  wire           alu_cflag_wr_ex;          // ALU operation writes to C flag
  wire           alu_vflag_wr_ex;          // ALU operation writes to V flag
  wire           instr_flag_wr_ex;         // ALU operation writes to the flags
  wire           shift_in2_fast_we;        // Shifter input 2 reg write enable
  reg  [7:0]     shift_in2_fast_ex;        // Shifter input 2 (fast version)
  wire [4:0]     sbfx_msb_pos_ex;          // MSB bit position for SBFX
  wire           sbfx_msb_ex;              // MSB for SBFZ operation
  wire [5:0]     shift_width_in1_ex;       // Shifter amount calculation input 1
  wire           shift_width_in2_ex;       // Shifter amount calculation input 2
  wire [5:0]     shift_width_ex;           // Shifter amount result
  wire           shift_l_imm32_ex;         // Left shift mask - select 32
  wire           shift_l_width_ex;         // Left shift mask - select width
  wire [5:0]     shift_mask_lsl_ex;        // Left shift mask - LSL amount
  wire [31:0]    shift_l_mask_ex;          // Left shift mask for sat/bitfield
  wire           shift_r_immed_ex;         // Right shift mask - select immediate
  wire           shift_r_width_ex;         // Right shift mask - select width
  wire [5:0]     shift_mask_lsr_ex;        // Right shift mask - LSR amount
  wire [31:0]    shift_r_mask_ex;          // Right shift mask for sat/bitfield
  wire [31:0]    shift_lr_mask_ex;         // Shifter mask (left & right shift)
  wire           sat_pos_tst_ex;           // Positive saturation required test
  wire           sat_neg_tst_ex;           // Negative saturation required test
  wire [6:0]     shift_sel_ex;             // Shifter result format select
  wire           mcyc_not_int;             // Interrupt stack request w/o entry
  wire           alu_cg_en;                // Multi-cycle op clock gate enable
  wire           mul_signed_ex;            // Signed multiply operation
  wire           mul_has_neg_ex;           // Signed multiply has a -ve input
  reg  [3:0]     mcyc_mull1_adv_ex;        // Multiply optimisation for xMULL_1
  wire [3:0]     mcyc_mull2_adv_ex;        // Multiply optimisation for xMULL_2
  wire [3:0]     mcyc_mull4_adv_ex;        // Multiply optimisation for xMULL_4
  wire [3:0]     mcyc_mull5_adv_ex;        // Multiply optimisation for xMULL_5
  wire [3:0]     mcyc_mlap_adv_ex;         // Multiply optimisation for MLA_P
  wire [3:0]     mcyc_clz1_adv_ex;         // Divide optimisation for CLZ_1
  wire [3:0]     mcyc_clz2_adv_ex;         // Divide optimisation for CLZ_2
  wire [3:0]     mcyc_srt_adv_ex;          // Divide optimisation for SRT
  reg  [3:0]     nxt_alu_mcyc_op_ex;       // Next multi-cycle operation
  wire           mcyc_cc_check_ex;         // In a state which requires CC check
  wire           mcyc_kill_ex;             // Kill multi-cycle operation
  wire [3:0]     nxt_mcyc_state_ex;        // Next multi-cycle state
  wire           alu_mcyc_state_we;        // Multi-cycle state write enable
  wire           alu_mcyc_state_mask;      // Multi-cycle state mask to remove E
  reg  [3:0]     ialu_mcyc_state_ex;       // Multi-cycle state (before masking)
  wire [3:0]     alu_mcyc_state_ex;        // Multi-cycle state
  reg  [5:0]     nxt_mul_input_sel_ex;     // Next multiplier input selects
  wire           mul_input_sel_we;         // Multiplier input select wr enable
  reg  [5:0]     mul_input_sel_ex;         // Multiplier input selects
  wire [5:0]     opt_mul_input_sel_ex;     // Opt multiplier input selects
  wire           nxt_mul_mla_track_ex;     // Next MLA pipe-line track
  wire           mul_mla_track_we;         // MLA pipe-line track write enable
  reg            mul_mla_track_ex;         // MLA pipe-line track
  wire           nxt_mul_mull_extend_ex;   // Next extend MULL for MLA opt
  reg            mul_mull_extend_ex;       // Extend MULL for MLA opt
  wire           alu_mcyc_actv_ex;         // Multi-cycle operation active
  wire           mcyc_stall_ex;            // M-cyc op requires pipe-line stall
  wire           mul_neg_in2_ex;           // Invert input 2 on multiplier
  wire           mul_add_valid_ex;         // Pass multiply result to adder
  wire           mul_carry_we;             // Stored carry result write enable
  reg            mul_carry_ex;             // Stored carry result
  wire           mul_cin_ex;               // Muliply carry in to adder
  wire           div_s_max_neg_ex;         // SDIV input is max negative
  wire           div_srt_ctl_we;           // SRT divider control write enable
  wire [1:0]     nxt_div_srt_init_ex;      // Next SRT divider initialisation
  reg  [1:0]     div_srt_init_ex;          // SRT divider initialisation
  wire           nxt_div_srt_sign_ex;      // Next SRT divider sign info
  reg            div_srt_sign_ex;          // SRT divider sign info
  wire           clz_res_reg_we;           // CLZ result for DIV write enable
  reg  [5:0]     clz_res_reg_ex;           // CLZ result for DIV cycle info
  wire [5:0]     div_clz_sub_in1_ex;       // DIV CLZ result sub in 1
  wire [5:0]     div_clz_sub_in2_ex;       // DIV CLZ result sub in 2
  wire           div_clz_sub_in3_ex;       // DIV CLZ result sub in 3
  wire [5:0]     div_clz_sub_res_ex;       // DIV CLZ result (diff of CLZ A/B)
  wire [5:0]     nxt_div_cnt_ex;           // Next divide cycle count
  wire           div_cnt_we;               // Divide cycle count write enable
  reg  [5:0]     div_cnt_ex;               // Divide cycle count
  wire [1:0]     nxt_div_cnt_reg_ex;       // Next registered divide cycle count
  wire           div_cnt_reg_we;           // Registered div cycle count enable
  reg  [1:0]     div_cnt_reg_ex;           // Registered divide cycle count
  wire           nxt_fault_divby0_ex;      // Next divide by 0 fault
  wire           fault_divby0_we;          // Divide by 0 fault write enable
  reg            fault_divby0_ex;          // Divide by 0 fault
  wire           fault_divby0_held_we;     // Held divide by 0 fault enable
  reg            fault_divby0_held_ex;     // Held divide by 0 fault
  wire           alu_stall_ex;             // ALU requires pipe-line stall
  wire           mcyc_mlx_1_rd_en_ex;      // MLx_1 stage RF A port read enable
  wire           mcyc_mla_p_rd_en_ex;      // MLA_P stage RF A port read enable
  wire           mcyc_xmull_1_rd_en_ex;    // xMULL_1 stage RF A port read enable
  wire           mcyc_xmull_4_rd_en_ex;    // xMULL_4 stage RF A port read enable
  wire           mcyc_xmull_5_rd_en_ex;    // xMULL_5 stage RF A port read enable
  wire           mcyc_xmlal_1_rd_en_ex;    // xMLAL_1 stage RF A port read enable
  wire           alu_rf_rd_a_en_ex;        // ALU RF A port read enable
  wire [3:0]     alu_rf_rd_a_addr_ex;      // ALU RF A port read address
  wire           alu_rf_wr_d_en_ex;        // ALU RF D port write enable
  wire [3:0]     alu_rf_wr_d_addr_ex;      // ALU RF D port write address
  wire           alu_will_wr_rf_ex;        // ALU will write to RF
  wire           alu_zero_res_ex;          // ALU result is zero
  wire           div_fast_zero_ex;         // Divide result is zero (fast)
  wire [5:0]     alu_flag_wr_ex;           // ALU GE NZCVQ result write enables
  wire [4:0]     alu_exit_code_ex;         // ALU operation ISR exit code
  wire           alu_exit_valid_ex;        // ALU operation is a valid ISR exit
  wire           alu_mcyc_invk_msk_ex;     // ALU multi-cycle op mask stack req
  wire           alu_mcyc_ilock_ex;        // ALU multi-cycle op interlock

`ifdef ARM_DSM

  wire           dsm_reset_all_regs;
  wire           dsm_fpu_present;

  assign full_hreset_n = (dsm_reset_all_regs)  ? hreset_n : 1'b1;

  assign opt_fpu_en    = (dsm_fpu_present)     ? 1'b1     : 1'b0;

`else

  assign full_hreset_n = (RESET_ALL_REGS == 1) ? hreset_n : 1'b1;

  assign opt_fpu_en    = (FPU_PRESENT == 1)    ? 1'b1     : 1'b0;

`endif

  assign opt_ext_en    = (EXT_PRESENT == 1)    ? 1'b1     : 1'b0;

  assign alu_mcyc_state_mask = (opt_ext_en &
                                ~((ialu_mcyc_state_ex == `CORE_DIV_CLZ_1) |
                                  (ialu_mcyc_state_ex == `CORE_DIV_CLZ_2) |
                                  (ialu_mcyc_state_ex == `CORE_DIV_INIT ) |
                                  (ialu_mcyc_state_ex == `CORE_DIV_SRT  ) |
                                  (ialu_mcyc_state_ex == `CORE_DIV_WREG)));


  assign alu_de_2_ex_en = ((instr_de_2_ex_en_i & (alu_actv_de | alu_actv_ex)));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      instr_alu_ctl_ex    <= {21{1'b0}};
    else if (alu_de_2_ex_en)
      instr_alu_ctl_ex    <= instr_alu_ctl_de_i;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      alu_rf_rd_c_addr_ex <= 4'b0000;
    else if (alu_de_2_ex_en & ~opt_ext_en)
      alu_rf_rd_c_addr_ex <= nxt_alu_rf_rd_c_addr_ex;

  assign alu_rf_clr_ex     = ~alu_actv_ex & instr_rf_wr_d_en_ex & ~alu_stall_ex;

  assign alu_rf_de_2_ex_en = ((instr_de_2_ex_en_i &
                              (instr_rf_wr_d_en_de_i | instr_rf_wr_d_en_ex)) |
                              alu_rf_clr_ex);

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        instr_rf_wr_d_addr_ex <= 4'b0000;
        instr_rf_wr_d_en_ex   <= 1'b0;
      end
    else if (alu_rf_de_2_ex_en)
      begin
        instr_rf_wr_d_addr_ex <= instr_rf_wr_d_addr_de_i;
        instr_rf_wr_d_en_ex   <= instr_rf_wr_d_en_de_i;
      end

  assign alu_actv_de = (|instr_alu_ctl_de_i[17:13]) | (|instr_alu_ctl_de_i[4:3]);
  assign alu_actv_ex = (|instr_alu_ctl_ex[17:13])   | (|instr_alu_ctl_ex[4:3]);



  assign alu_shift_ex    = (instr_alu_ctl_ex[17:15] == 3'b001);
  assign alu_reg_shft_ex = alu_shift_ex & (instr_alu_ctl_ex[14:13] == 2'b00);

  assign alu_sat_ex      = alu_shift_ex & (instr_alu_ctl_ex[14:13] == 2'b11);

  assign alu_clz_ex      = (instr_alu_ctl_ex[17:13] == 5'b11111);

  assign alu_mov_ex      = (instr_alu_ctl_ex[17:13] == 5'b10100);

  assign alu_bfx_ex      = alu_shift_ex & (instr_alu_ctl_ex[14:13] == 2'b01);
  assign alu_bfield_ex   = alu_shift_ex & (instr_alu_ctl_ex[14:13] == 2'b10);

  assign alu_msr_flag_ex = (instr_alu_ctl_ex[17:13] == 5'b01000);

  assign alu_muldiv_ex   = (instr_alu_ctl_ex[17:16] == 2'b11 & ~alu_clz_ex);
  assign alu_mlal_de     = (instr_alu_ctl_de_i[17:13] == 5'b11001);
  assign alu_mlal_ex     = (instr_alu_ctl_ex[17:13] == 5'b11001);
  assign alu_mul_de      = (instr_alu_ctl_de_i[17:13] == 5'b11011);
  assign alu_mul_ex      = (instr_alu_ctl_ex[17:13] == 5'b11011);
  assign alu_mull_de     = (instr_alu_ctl_de_i[17:13] == 5'b11010);
  assign alu_mull_ex     = ((instr_alu_ctl_ex[17:13] == 5'b11010) |
                            mul_mull_extend_ex);
  assign alu_div_de      = (instr_alu_ctl_de_i[17:13] == 5'b11000);
  assign alu_div_ex      = (instr_alu_ctl_ex[17:13] == 5'b11000);
  assign alu_sdiv_ex     = alu_div_ex & instr_sign_ex_i;
  assign alu_udiv_ex     = alu_div_ex & ~instr_sign_ex_i;
  assign alu_multicyc_de = ((instr_alu_ctl_de_i[17:16] == 2'b11) &
                            ~(instr_alu_ctl_de_i[14:13] == 2'b11));
  assign alu_multicyc_ex = alu_muldiv_ex & ~alu_mul_ex;
  assign alu_ml_de       = (instr_alu_ctl_de_i[17:14] == 4'b1110);
  assign alu_ml_ex       = ((instr_alu_ctl_ex[17:14] == 4'b1110) &
                            ~mul_mull_extend_ex);
  assign alu_mls_ex      = alu_ml_ex &  instr_alu_ctl_ex[13];


  assign alu_nflag_wr_ex  = (|instr_alu_ctl_ex[19:18]);
  assign alu_zflag_wr_ex  = (|instr_alu_ctl_ex[19:18]);
  assign alu_cflag_wr_ex  = instr_alu_ctl_ex[19];
  assign alu_vflag_wr_ex  = (&instr_alu_ctl_ex[19:18]);
  assign instr_flag_wr_ex = (|instr_alu_ctl_ex[19:18]);


  assign shift_in2_fast_de = (div_cnt_we ? {3'b000, nxt_div_cnt_ex[4:0]} :
                                           instr_immed_de_i[7:0]);

  assign shift_in2_fast_we = alu_de_2_ex_en | div_cnt_we;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      shift_in2_fast_ex <= {8{1'b0}};
    else if (shift_in2_fast_we)
      shift_in2_fast_ex <= shift_in2_fast_de;


  assign sbfx_msb_pos_ex    = instr_immed_ex_i[31:27] + instr_immed_ex_i[4:0];
  assign sbfx_msb_ex        = shift_in1_ex_i[sbfx_msb_pos_ex] & instr_sign_ex_i;

  assign shift_width_in1_ex = {1'b0,instr_immed_ex_i[31:27]};
  assign shift_width_in2_ex = alu_bfx_ex | alu_bfield_ex;
  assign shift_width_ex     = (shift_width_in1_ex +
                               {5'b00000, shift_width_in2_ex});

  assign shift_l_imm32_ex   = alu_bfx_ex & ~sbfx_msb_ex;
  assign shift_l_width_ex   = (alu_bfield_ex | alu_sat_ex |
                               (alu_bfx_ex & sbfx_msb_ex));

  assign shift_mask_lsl_ex  = (({6{shift_l_imm32_ex}} & 6'b100000) |
                               ({6{shift_l_width_ex}} & shift_width_ex));

  assign shift_l_mask_ex    = 32'hFFFFFFFF << shift_mask_lsl_ex;

  assign shift_r_immed_ex   = alu_bfield_ex;
  assign shift_r_width_ex   = alu_bfx_ex | alu_sat_ex;

  assign shift_mask_lsr_ex  = (6'b100000 -
                               (({6{shift_r_width_ex}} & shift_width_ex) |
                                ({6{shift_r_immed_ex}} &
                                 {1'b0,instr_immed_ex_i[12:8]})));

  assign shift_r_mask_ex    = 32'hFFFFFFFF >> shift_mask_lsr_ex;

  assign shift_lr_mask_ex   = shift_l_mask_ex | shift_r_mask_ex;

  assign sat_pos_tst_ex =  (|(shift_res_ex_i & shift_l_mask_ex));
  assign sat_neg_tst_ex = ~(&(shift_res_ex_i | shift_r_mask_ex));


  assign shift_sel_ex = {// USAT -ve saturation (zero)
                         alu_sat_ex & ~instr_sign_ex_i & shift_res_ex_i[31],
                         (alu_sat_ex &  instr_sign_ex_i & shift_res_ex_i[31] &
                          sat_neg_tst_ex),
                         alu_sat_ex & ~shift_res_ex_i[31] & sat_pos_tst_ex,
                         (alu_sat_ex & ((~shift_res_ex_i[31] & ~sat_pos_tst_ex) |
                                        (instr_sign_ex_i & shift_res_ex_i[31] &
                                         ~sat_neg_tst_ex))),
                         alu_bfield_ex,
                         alu_bfx_ex,
                         alu_reg_shft_ex};


  assign mcyc_not_int = ~(nvic_int_invoke_i & ~int_entry_i);

  assign alu_mcyc_op_valid_de = (alu_multicyc_de ?
                                 mcyc_not_int :
                                 (alu_mcyc_state_ex!=`CORE_MUL_NORMAL));

  assign alu_mcyc_op_de = ({4{alu_mcyc_op_valid_de}} &
                           (({4{alu_div_de}}                & `CORE_DIV_CLZ_1)   |
                            ({4{alu_mlal_de | alu_mull_de}} & `CORE_MUL_xMULL_1) |
                            ({4{alu_ml_de}}                 & `CORE_MUL_MLx_1)));

  assign alu_cg_en = alu_multicyc_ex;


  assign mul_signed_ex  = (alu_mlal_ex | alu_mull_ex) & instr_sign_ex_i;

  assign mul_has_neg_ex = ((rf_rd_a_data_ex_i[31] | rf_rd_b_data_ex_i[31]) &
                           mul_signed_ex);




  always @ (rf_rd_a_zero_ex_i or rf_rd_b_zero_ex_i or mul_has_neg_ex or
            alu_mlal_ex)
    case ({rf_rd_b_zero_ex_i,rf_rd_a_zero_ex_i})
      4'b1111,
      4'b1110,
      4'b1101,
      4'b1100,
      4'b1011,
      4'b1010,
      4'b0111,
      4'b0011 : mcyc_mull1_adv_ex = alu_mlal_ex    ? `CORE_MUL_xMLAL_1 :
                                                     `CORE_MUL_xMULL_4;
      4'b0101 : mcyc_mull1_adv_ex = mul_has_neg_ex ? `CORE_MUL_xMULL_2 :
                                                     `CORE_MUL_xMULL_4;
      4'b0110,
      4'b0100,
      4'b0010 : mcyc_mull1_adv_ex = mul_has_neg_ex ? `CORE_MUL_xMULL_2 :
                                                     `CORE_MUL_xMULL_3;
      4'b0000,
      4'b0001,
      4'b1000,
      4'b1001 : mcyc_mull1_adv_ex = `CORE_MUL_xMULL_2;
      default : mcyc_mull1_adv_ex = 4'bxxxx;
    endcase

  assign mcyc_mull2_adv_ex = (((~rf_rd_b_zero_ex_i[1] &
                                (rf_rd_a_zero_ex_i == 2'b01)) |
                               (~rf_rd_a_zero_ex_i[1] &
                                (rf_rd_b_zero_ex_i == 2'b10))) ?
                              `CORE_MUL_xMULL_4 : `CORE_MUL_xMULL_3);

  assign mcyc_mull4_adv_ex = (((rf_rd_b_zero_ex_i[1] |
                                rf_rd_a_zero_ex_i[1]) & alu_mlal_ex) ?
                              `CORE_MUL_xMLAL_1 : `CORE_MUL_xMULL_5);

  assign mcyc_mull5_adv_ex = (alu_mlal_ex ? `CORE_MUL_xMLAL_1 :
                              (mul_mla_track_ex ? `CORE_MUL_MLA_P :
                                                  alu_mcyc_op_de));

  assign mcyc_mlap_adv_ex  = (mul_mla_track_ex ? `CORE_MUL_MLx_2 :
                                                 alu_mcyc_op_de);



  assign mcyc_clz1_adv_ex  = (div_fast_zero_ex   ? `CORE_DIV_WREG :
                                                   `CORE_DIV_CLZ_2);

  assign mcyc_clz2_adv_ex  = (~nxt_div_cnt_ex[5] ? `CORE_DIV_INIT :
                                                   `CORE_DIV_WREG);

  assign mcyc_srt_adv_ex   = (div_cnt_ex[5]      ? `CORE_DIV_WREG :
                                                   `CORE_DIV_SRT);

  always @ (alu_mcyc_state_ex or alu_mcyc_op_de or mcyc_mull1_adv_ex or
            mcyc_mull2_adv_ex or mcyc_mull4_adv_ex or mcyc_mull5_adv_ex or
            mcyc_mlap_adv_ex or mcyc_clz1_adv_ex or mcyc_clz2_adv_ex or
            mcyc_srt_adv_ex)
    case (alu_mcyc_state_ex)
      `CORE_MUL_NORMAL  : nxt_alu_mcyc_op_ex = alu_mcyc_op_de;
      `CORE_MUL_MLx_1   : nxt_alu_mcyc_op_ex = `CORE_MUL_MLx_2;
      `CORE_MUL_MLx_2   : nxt_alu_mcyc_op_ex = alu_mcyc_op_de;
      `CORE_MUL_xMULL_1 : nxt_alu_mcyc_op_ex = mcyc_mull1_adv_ex;
      `CORE_MUL_xMULL_2 : nxt_alu_mcyc_op_ex = mcyc_mull2_adv_ex;
      `CORE_MUL_xMULL_3 : nxt_alu_mcyc_op_ex = `CORE_MUL_xMULL_4;
      `CORE_MUL_xMULL_4 : nxt_alu_mcyc_op_ex = mcyc_mull4_adv_ex;
      `CORE_MUL_xMULL_5 : nxt_alu_mcyc_op_ex = mcyc_mull5_adv_ex;
      `CORE_MUL_MLA_P   : nxt_alu_mcyc_op_ex = mcyc_mlap_adv_ex;
      `CORE_MUL_xMLAL_1 : nxt_alu_mcyc_op_ex = `CORE_MUL_xMLAL_2;
      `CORE_MUL_xMLAL_2 : nxt_alu_mcyc_op_ex = alu_mcyc_op_de;
      `CORE_DIV_CLZ_1   : nxt_alu_mcyc_op_ex = mcyc_clz1_adv_ex;
      `CORE_DIV_CLZ_2   : nxt_alu_mcyc_op_ex = mcyc_clz2_adv_ex;
      `CORE_DIV_INIT    : nxt_alu_mcyc_op_ex = `CORE_DIV_SRT;
      `CORE_DIV_SRT     : nxt_alu_mcyc_op_ex = mcyc_srt_adv_ex;
      `CORE_DIV_WREG    : nxt_alu_mcyc_op_ex = alu_mcyc_op_de;
      default           : nxt_alu_mcyc_op_ex = 4'bxxxx;
    endcase

  assign mcyc_cc_check_ex  = ((alu_mcyc_state_ex == `CORE_MUL_MLx_1)   |
                              (alu_mcyc_state_ex == `CORE_MUL_xMULL_1) |
                              (alu_mcyc_state_ex == `CORE_DIV_CLZ_1));

  assign mcyc_kill_ex      = ((mcyc_cc_check_ex & ~instr_ccpass_ex_i) |
                              ~mcyc_not_int);

  assign nxt_mcyc_state_ex = (({4{~mcyc_kill_ex}} & nxt_alu_mcyc_op_ex) |
                              ({4{ mcyc_kill_ex}} & alu_mcyc_op_de));


  assign alu_mcyc_state_we = (alu_mcyc_op_valid_de |
                              (alu_mcyc_state_ex!=`CORE_MUL_NORMAL));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ialu_mcyc_state_ex <= 4'b0000;
    else if (alu_mcyc_state_we)
      ialu_mcyc_state_ex <= nxt_mcyc_state_ex;

  assign alu_mcyc_state_ex = (alu_mcyc_state_mask ? `CORE_MUL_NORMAL :
                                                    ialu_mcyc_state_ex);




  always @ (nxt_mcyc_state_ex or alu_mul_de)
    case (nxt_mcyc_state_ex)
      `CORE_MUL_NORMAL :
        nxt_mul_input_sel_ex = {2{alu_mul_de, 1'b0, alu_mul_de}};
      `CORE_MUL_xMULL_1 :
        nxt_mul_input_sel_ex = 6'b001001;
      `CORE_MUL_xMULL_2 :
        nxt_mul_input_sel_ex = 6'b001010;
      `CORE_MUL_xMULL_3 :
        nxt_mul_input_sel_ex = 6'b010001;
      `CORE_MUL_xMULL_4 :
        nxt_mul_input_sel_ex = 6'b010010;
      `CORE_MUL_MLx_1,        `CORE_MUL_xMULL_5,      `CORE_MUL_MLA_P :
        nxt_mul_input_sel_ex = 6'b101101;
      `CORE_MUL_MLx_2,        `CORE_MUL_xMLAL_1,      `CORE_MUL_xMLAL_2,
      `CORE_DIV_CLZ_1,        `CORE_DIV_CLZ_2,        `CORE_DIV_INIT,
      `CORE_DIV_SRT,          `CORE_DIV_WREG :
        nxt_mul_input_sel_ex = 6'b000000;
      default :
        nxt_mul_input_sel_ex = 6'bxxxxxx;
    endcase

  assign mul_input_sel_we = (~opt_ext_en &
                             (alu_mul_de | alu_mul_ex | alu_mcyc_state_we));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      mul_input_sel_ex <= 6'b000000;
    else if (mul_input_sel_we)
      mul_input_sel_ex <= nxt_mul_input_sel_ex & {6{~opt_ext_en}};

  assign opt_mul_input_sel_ex = {6{~opt_ext_en}} & mul_input_sel_ex;


  assign mul_mla_pipe_de        = (instr_mla_track_de_i & alu_muldiv_ex &
                                   ~alu_div_ex & ~nvic_int_invoke_i &
                                   instr_ccpass_ex_i);

  assign mul_mla_track_we       = (// Clear
                                   alu_de_2_ex_en |
                                   (mul_mla_track_ex &
                                    ((alu_mcyc_state_ex==`CORE_MUL_xMULL_5) |
                                     (alu_mcyc_state_ex==`CORE_MUL_MLA_P))));

  assign nxt_mul_mull_extend_ex = ((alu_mcyc_state_ex==`CORE_MUL_xMULL_4) &
                                   ~nvic_int_invoke_i);

  assign nxt_mul_mla_track_ex   = (mul_mla_pipe_de &
                                   (nxt_mul_mull_extend_ex |
                                    ((alu_mcyc_state_ex==`CORE_MUL_xMULL_5) &
                                     mul_mla_track_ex)));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        mul_mull_extend_ex <= 1'b0;
        mul_mla_track_ex   <= 1'b0;
      end
    else if (mul_mla_track_we)
      begin
        mul_mull_extend_ex <= nxt_mul_mull_extend_ex;
        mul_mla_track_ex   <= nxt_mul_mla_track_ex;
      end


  assign alu_mcyc_actv_ex = (~(&alu_rf_wr_d_addr_ex) &
                             ((alu_mcyc_state_ex==`CORE_MUL_xMULL_4)  &
                              alu_mull_ex & ~nvic_int_invoke_i)        |
                             ((alu_mcyc_state_ex==`CORE_MUL_xMULL_5)  &
                              alu_mull_ex)                             |
                             ((alu_mcyc_state_ex==`CORE_MUL_xMLAL_1)  &
                              ~nvic_int_invoke_i)                      |
                             (alu_mcyc_state_ex==`CORE_MUL_xMLAL_2)    |
                             (alu_mcyc_state_ex==`CORE_DIV_WREG));

  assign mcyc_stall_ex    = ~(((alu_mcyc_state_ex==`CORE_MUL_NORMAL)  &
                               ~alu_multicyc_ex)                       |
                              (alu_mcyc_state_ex==`CORE_MUL_MLx_2)     |
                              (alu_mcyc_state_ex==`CORE_DIV_WREG)      |
                              (alu_mcyc_state_ex==`CORE_MUL_xMLAL_2)   |
                              ((alu_mcyc_state_ex==`CORE_MUL_MLA_P)   &
                               ~mul_mla_track_ex)                      |
                              ((alu_mcyc_state_ex==`CORE_MUL_xMULL_4) &
                               alu_mull_ex & mul_mla_pipe_de)          |
                              ((alu_mcyc_state_ex==`CORE_MUL_xMULL_5) &
                               alu_mull_ex &
                               (mul_mla_pipe_de | ~mul_mla_track_ex)));


  assign mul_neg_in2_ex   = (((alu_mcyc_state_ex == `CORE_MUL_MLx_2)    &
                              alu_mls_ex )                               |
                             (alu_mcyc_state_ex == `CORE_MUL_xMULL_2)    |
                             ((alu_mcyc_state_ex == `CORE_DIV_CLZ_1)    &
                              alu_sdiv_ex & rf_rd_a_data_ex_i[31])       |
                             ((alu_mcyc_state_ex == `CORE_DIV_WREG)     &
                              alu_sdiv_ex &
                              (rf_rd_a_data_ex_i[31]^rf_rd_b_data_ex_i[31])));

  assign mul_add_valid_ex = ((alu_mcyc_state_ex == `CORE_MUL_MLx_2)   |
                             (alu_mcyc_state_ex == `CORE_MUL_MLA_P)   |
                             (alu_mcyc_state_ex == `CORE_MUL_xMULL_1) |
                             (alu_mcyc_state_ex == `CORE_MUL_xMULL_2) |
                             (alu_mcyc_state_ex == `CORE_MUL_xMULL_3) |
                             (alu_mcyc_state_ex == `CORE_MUL_xMULL_4) |
                             (alu_mcyc_state_ex == `CORE_MUL_xMULL_5) |
                             (alu_mcyc_state_ex == `CORE_MUL_xMLAL_1) |
                             (alu_mcyc_state_ex == `CORE_MUL_xMLAL_2) |
                             (alu_mcyc_state_ex == `CORE_DIV_CLZ_1)   |
                             (alu_mcyc_state_ex == `CORE_DIV_WREG));

  assign mul_carry_we     = (alu_mcyc_state_ex==`CORE_MUL_xMLAL_1);

  always @ (posedge aclk or negedge hreset_n)
    if (!hreset_n)
      mul_carry_ex <= 1'b0;
    else if (mul_carry_we)
      mul_carry_ex <= alu_cout_ex_i;

  assign mul_cin_ex = (alu_mcyc_state_ex==`CORE_MUL_xMLAL_2) & mul_carry_ex;

  assign div_s_max_neg_ex = (alu_sdiv_ex & rf_rd_a_data_ex_i[31] &
                             ~(|rf_rd_a_data_ex_i[30:16]) &
                             rf_rd_a_zero_ex_i[0]);



  assign div_srt_ctl_we      = (// Set
                                ((alu_mcyc_state_ex==`CORE_DIV_INIT) &
                                 ~nvic_int_invoke_i) |
                                (|div_srt_init_ex));

  assign nxt_div_srt_init_ex = ({2{alu_mcyc_state_ex==`CORE_DIV_INIT}} &
                                {(div_s_max_neg_ex &  rf_rd_b_data_ex_i[31]),
                                 (div_s_max_neg_ex & ~rf_rd_b_data_ex_i[31]) |
                                 alu_udiv_ex});

  assign nxt_div_srt_sign_ex = alu_sdiv_ex & rf_rd_b_data_ex_i[31];

  always @ (posedge aclk or negedge hreset_n)
    if (!hreset_n)
      begin
        div_srt_init_ex <= 2'b00;
        div_srt_sign_ex <= 1'b0;
      end
    else if (div_srt_ctl_we)
      begin
        div_srt_init_ex <= nxt_div_srt_init_ex;
        div_srt_sign_ex <= nxt_div_srt_sign_ex;
      end


  assign clz_res_reg_we = (alu_mcyc_state_ex==`CORE_DIV_CLZ_1);

  always @ (posedge aclk or negedge full_hreset_n)
    if (!full_hreset_n)
      clz_res_reg_ex <= {6{1'b0}};
    else if (clz_res_reg_we)
      clz_res_reg_ex <= clz_res_ex_i;

  assign div_clz_sub_in1_ex = ({6{alu_mcyc_state_ex==`CORE_DIV_CLZ_2}} &
                               clz_res_reg_ex);
  assign div_clz_sub_in2_ex = ({6{alu_mcyc_state_ex==`CORE_DIV_CLZ_2}} &
                               clz_res_ex_i);
  assign div_clz_sub_in3_ex = div_s_max_neg_ex & rf_rd_b_data_ex_i[31];

  assign div_clz_sub_res_ex = (div_clz_sub_in1_ex - div_clz_sub_in2_ex -
                               {5'b00000, div_clz_sub_in3_ex});

  assign nxt_div_cnt_ex = (((alu_mcyc_state_ex == `CORE_DIV_INIT) |
                            (alu_mcyc_state_ex == `CORE_DIV_SRT)) ?
                           ({6{~div_cnt_ex[5]}} & (div_cnt_ex - 6'b000100)) :
                           div_clz_sub_res_ex);

  assign div_cnt_we     = ((alu_mcyc_state_ex==`CORE_DIV_CLZ_2) |
                           (alu_mcyc_state_ex==`CORE_DIV_INIT)  |
                           (alu_mcyc_state_ex==`CORE_DIV_SRT));

  always @ (posedge aclk or negedge hreset_n)
    if (!hreset_n)
      div_cnt_ex <= {6{1'b0}};
    else if (div_cnt_we)
      div_cnt_ex <= nxt_div_cnt_ex;

  assign div_cnt_reg_we     = (alu_mcyc_state_ex==`CORE_DIV_INIT);
  assign nxt_div_cnt_reg_ex = 2'b11 - div_cnt_ex[1:0];

  always @ (posedge aclk or negedge hreset_n)
    if (!hreset_n)
      div_cnt_reg_ex <= 2'b00;
    else if (div_cnt_reg_we)
      div_cnt_reg_ex <= nxt_div_cnt_reg_ex;


  assign fault_divby0_we     = (((alu_mcyc_state_ex==`CORE_DIV_CLZ_1) &
                                 instr_ccpass_ex_i & nvic_div_0_trp_i &
                                 ~nvic_int_invoke_i) | fault_divby0_ex);

  assign nxt_fault_divby0_ex = ((alu_mcyc_state_ex==`CORE_DIV_CLZ_1) &
                                (&rf_rd_b_zero_ex_i));

  always @ (posedge aclk or negedge hreset_n)
    if (!hreset_n)
      fault_divby0_ex <= 1'b0;
    else if (fault_divby0_we)
      fault_divby0_ex <= nxt_fault_divby0_ex;

  assign fault_divby0_held_we = (((alu_mcyc_state_ex==`CORE_DIV_CLZ_1) &
                                  instr_ccpass_ex_i & nvic_div_0_trp_i &
                                  ~nvic_int_invoke_i) |
                                 ((alu_mcyc_state_ex==`CORE_MUL_NORMAL) &
                                  fault_divby0_held_ex));

  always @ (posedge aclk or negedge hreset_n)
    if (!hreset_n)
      fault_divby0_held_ex <= 1'b0;
    else if (fault_divby0_held_we)
      fault_divby0_held_ex <= nxt_fault_divby0_ex;


  assign alu_stall_ex = ((instr_ccpass_ex_i & mcyc_stall_ex) | fault_divby0_ex |
                         (instr_ccpass_ex_i & nvic_int_invoke_i &
                          (alu_rf_wr_d_addr_ex==4'b1101) & instr_rf_wr_d_en_ex));


  assign mcyc_mlx_1_rd_en_ex   = ((alu_mcyc_state_ex==`CORE_MUL_MLx_1) &
                                  instr_ccpass_ex_i);

  assign mcyc_mla_p_rd_en_ex   = ((alu_mcyc_state_ex==`CORE_MUL_MLA_P) &
                                  mul_mla_track_ex);

  assign mcyc_xmull_1_rd_en_ex = ((alu_mcyc_state_ex==`CORE_MUL_xMULL_1) &
                                  instr_ccpass_ex_i & alu_mlal_ex &
                                  ((rf_rd_b_zero_ex_i[1] &
                                    rf_rd_a_zero_ex_i[1]) |
                                   (&rf_rd_b_zero_ex_i) |
                                   (&rf_rd_a_zero_ex_i)));

  assign mcyc_xmull_4_rd_en_ex = ((alu_mcyc_state_ex==`CORE_MUL_xMULL_4) &
                                  alu_mlal_ex &
                                  (rf_rd_b_zero_ex_i[1] |
                                   rf_rd_a_zero_ex_i[1]));

  assign mcyc_xmull_5_rd_en_ex = ((alu_mcyc_state_ex==`CORE_MUL_xMULL_5) &
                                  alu_mlal_ex);

  assign mcyc_xmlal_1_rd_en_ex = (alu_mcyc_state_ex==`CORE_MUL_xMLAL_1);

  assign alu_rf_rd_a_en_ex     = (~nvic_int_invoke_i &
                                  (mcyc_mlx_1_rd_en_ex   |
                                   mcyc_mla_p_rd_en_ex   |
                                   mcyc_xmull_1_rd_en_ex |
                                   mcyc_xmull_4_rd_en_ex |
                                   mcyc_xmull_5_rd_en_ex |
                                   mcyc_xmlal_1_rd_en_ex));

  assign alu_rf_rd_a_addr_ex   = ((((alu_mcyc_state_ex==`CORE_MUL_MLx_1)   |
                                    (alu_mcyc_state_ex==`CORE_MUL_MLA_P)   |
                                    (alu_mcyc_state_ex==`CORE_MUL_xMULL_1) |
                                    (alu_mcyc_state_ex==`CORE_MUL_xMULL_4) |
                                    (alu_mcyc_state_ex==`CORE_MUL_xMULL_5)) &
                                   ~opt_ext_en) ?
                                  alu_rf_rd_c_addr_ex : instr_rf_wr_d_addr_ex);

  assign nxt_alu_rf_rd_c_addr_ex = ({4{~opt_ext_en}} &
                                    (((alu_mcyc_state_ex==`CORE_MUL_xMULL_4) &
                                      ~nvic_int_invoke_i) ?
                                     instr_rf_wr_d_addr_ex :
                                     instr_rf_rd_c_addr_de_i));

  assign alu_rf_wr_d_en_ex = (instr_ccpass_ex_i & instr_rf_wr_d_en_ex &
                              (// If ALU is not active then it will be due to
                               (~alu_actv_ex &
                                ~(nvic_int_invoke_i &
                                  (alu_rf_wr_d_addr_ex==4'b1101))) |
                               (alu_actv_ex & ~fault_divby0_ex &
                                ((~(alu_div_ex | alu_mlal_ex | alu_mull_ex) &
                                  (mul_mla_track_ex | ~alu_stall_ex)) |
                                 ( (alu_div_ex | alu_mlal_ex | alu_mull_ex) &
                                   alu_mcyc_actv_ex)))));

  assign alu_rf_wr_d_addr_ex = ((((alu_mcyc_state_ex==`CORE_MUL_xMULL_4)   |
                                  (alu_mcyc_state_ex==`CORE_MUL_xMLAL_1)   |
                                  ((alu_mcyc_state_ex==`CORE_MUL_xMULL_5) &
                                   mul_mla_track_ex)                       |
                                  ((alu_mcyc_state_ex==`CORE_MUL_MLA_P)   &
                                   mul_mla_track_ex)) &
                                 ~opt_ext_en) ?
                                alu_rf_rd_c_addr_ex : instr_rf_wr_d_addr_ex);

  assign alu_will_wr_rf_ex = (instr_ccpass_ex_i & ~alu_stall_ex &
                              instr_rf_wr_d_en_ex);


  assign alu_zero_res_ex  = (&rf_rd_b_zero_ex_i);

  assign div_fast_zero_ex = alu_zero_res_ex | (&rf_rd_a_zero_ex_i);


  assign alu_flag_wr_ex = ({6{instr_ccpass_ex_i & ~alu_stall_ex}} &
                            {// GE
                             alu_msr_flag_ex & instr_data_size_ex_i[0],
                             {4{alu_msr_flag_ex & instr_data_size_ex_i[1]}} |
                             ({alu_nflag_wr_ex, alu_zflag_wr_ex,
                               alu_cflag_wr_ex, alu_vflag_wr_ex}),
                             ((alu_msr_flag_ex & instr_data_size_ex_i[1]) |
                              (alu_sat_ex & (|shift_sel_ex[6:4])))});


  assign alu_exit_code_ex  = rf_rd_b_data_ex_i[4:0];

  assign alu_exit_valid_ex = (instr_ccpass_ex_i & alu_mov_ex &
                              instr_exit_op_ex_i & (&instr_rf_wr_d_addr_ex) &
                              (&rf_rd_b_data_ex_i[31:5]) &
                              (opt_fpu_en | rf_rd_b_data_ex_i[4]));


  assign alu_mcyc_invk_msk_ex = (alu_multicyc_ex & ~instr_de_2_ex_en_i &
                                 ~fault_divby0_held_ex & instr_ccpass_ex_i);

  assign alu_mcyc_ilock_ex    = (alu_muldiv_ex & ~fault_divby0_held_ex &
                                 instr_ccpass_ex_i);


  assign alu_cg_en_o             = alu_cg_en;
  assign instr_flag_wr_ex_o      = instr_flag_wr_ex;
  assign instr_alu_ctl_ex_o      = instr_alu_ctl_ex;
  assign alu_mcyc_state_ex_o     = alu_mcyc_state_ex;
  assign mul_input_sel_ex_o      = opt_mul_input_sel_ex;
  assign div_srt_init_ex_o       = div_srt_init_ex;
  assign div_srt_sign_ex_o       = div_srt_sign_ex;
  assign div_fast_zero_ex_o      = div_fast_zero_ex;
  assign div_cnt_ex_o            = div_cnt_ex;
  assign div_cnt_reg_ex_o        = div_cnt_reg_ex;
  assign mul_neg_in2_ex_o        = mul_neg_in2_ex;
  assign mul_add_valid_ex_o      = mul_add_valid_ex;
  assign mul_mull_extend_ex_o    = mul_mull_extend_ex;
  assign mul_mla_track_ex_o      = mul_mla_track_ex;
  assign mul_cin_ex_o            = mul_cin_ex;
  assign shift_in2_fast_ex_o     = shift_in2_fast_ex;
  assign shift_sel_ex_o          = shift_sel_ex;
  assign shift_l_mask_ex_o       = shift_l_mask_ex;
  assign shift_r_mask_ex_o       = shift_r_mask_ex;
  assign shift_lr_mask_ex_o      = shift_lr_mask_ex;
  assign alu_rf_rd_a_en_ex_o     = alu_rf_rd_a_en_ex;
  assign alu_rf_rd_a_addr_ex_o   = alu_rf_rd_a_addr_ex;
  assign alu_rf_wr_d_en_ex_o     = alu_rf_wr_d_en_ex;
  assign alu_rf_wr_d_addr_ex_o   = alu_rf_wr_d_addr_ex;
  assign alu_will_wr_rf_ex_o     = alu_will_wr_rf_ex;
  assign instr_rf_wr_d_addr_ex_o = instr_rf_wr_d_addr_ex;
  assign alu_exit_valid_ex_o     = alu_exit_valid_ex;
  assign alu_exit_code_ex_o      = alu_exit_code_ex;
  assign alu_stall_ex_o          = alu_stall_ex;
  assign alu_mcyc_invk_msk_ex_o  = alu_mcyc_invk_msk_ex;
  assign alu_mcyc_ilock_ex_o     = alu_mcyc_ilock_ex;
  assign alu_zero_res_ex_o       = alu_zero_res_ex;
  assign alu_flag_wr_ex_o        = alu_flag_wr_ex;
  assign fault_divby0_ex_o       = fault_divby0_ex;


`ifdef ARM_ASSERT_ON

  wire ovl_alu_mla_ex = (alu_ml_ex & ~instr_alu_ctl_ex[13]);

  assert_never #(0,0,"mul_cin_ex should not be without alu_muldiv_ex")
    ovl_usecarry_wo_mulcode(hclk, hreset_n, mul_cin_ex & ~alu_muldiv_ex);

  assert_never #(0,0,"alu_mcyc_state_ex should not be without alu_muldiv_ex")
    ovl_mulstate_wo_mulcode(hclk, hreset_n,
                            (|alu_mcyc_state_ex) & ~alu_muldiv_ex);

  assert_never #(0,0,"Decode and ALU use carry must be exclusive")
    ovl_carryin_excl(hclk, hreset_n, instr_alu_ctl_ex[20] & mul_cin_ex);

  assert_zero_one_hot #(0,7,0,"Shifter select must be exclusive")
    ovl_shift_sel_ex_excl(hclk, hreset_n, shift_sel_ex);

  assert_next #(0,1,0,0,0,"div_srt_init_ex must be pulsed")
    ovl_srt_init_ex_pulsed(hclk, hreset_n, (|div_srt_init_ex),
                           ~(|div_srt_init_ex));

  assert_next #(0,1,0,0,0,"fault_divby0_ex must be pulsed")
    ovl_fault_divby0_pulsed(hclk, hreset_n, fault_divby0_ex,
                            ~fault_divby0_ex);

  assert_never #(0,0,"division[2] has been incorrectly generated")
    ovl_mult_opt_check(hclk, hreset_n,
                       ((alu_mcyc_state_ex==`CORE_MUL_xMULL_2)        &
                        (rf_rd_b_zero_ex_i[0] | rf_rd_a_zero_ex_i[1]) &
                        ~mul_has_neg_ex)                               |
                       ((alu_mcyc_state_ex==`CORE_MUL_xMULL_3)        &
                        (rf_rd_b_zero_ex_i[1] | rf_rd_a_zero_ex_i[0])) |
                       ((alu_mcyc_state_ex==`CORE_MUL_xMULL_4)        &
                        (rf_rd_b_zero_ex_i[1] & rf_rd_a_zero_ex_i[1]) &
                        alu_mlal_ex)                                   |
                       ((alu_mcyc_state_ex==`CORE_MUL_xMULL_5)        &
                        (rf_rd_b_zero_ex_i[1] | rf_rd_a_zero_ex_i[1]) &
                        alu_mlal_ex));

  assert_never #(0,0,"mul_mla_track_ex set outside of valid multiply states")
    ovl_instr_mla_track_de_check(hclk, hreset_n,
                                 mul_mla_track_ex &
                                 ~(((alu_mcyc_state_ex==`CORE_MUL_xMULL_5) &
                                    alu_mull_ex) |
                                   (alu_mcyc_state_ex==`CORE_MUL_MLA_P)));

  assert_never #(0,0,"MLA controls are invalid")
    ovl_mla_ctrl_check(hclk, hreset_n,
                       ((alu_mcyc_state_ex==`CORE_MUL_MLA_P) & ~ovl_alu_mla_ex) |
                       ((alu_mcyc_state_ex==`CORE_MUL_MLx_1) & ~alu_ml_ex)      |
                       ((alu_mcyc_state_ex==`CORE_MUL_MLx_2) & ~alu_ml_ex));

  assert_never #(0,0,"Mul controls outside of valid multiply states")
    ovl_mcyc_state_check1(hclk, hreset_n,
                          (alu_mull_ex &
                           ~((alu_mcyc_state_ex==`CORE_MUL_xMULL_1) |
                             (alu_mcyc_state_ex==`CORE_MUL_xMULL_2) |
                             (alu_mcyc_state_ex==`CORE_MUL_xMULL_3) |
                             (alu_mcyc_state_ex==`CORE_MUL_xMULL_4) |
                             (alu_mcyc_state_ex==`CORE_MUL_xMULL_5) |
                             (alu_mcyc_state_ex==`CORE_MUL_xMLAL_1) |
                             (alu_mcyc_state_ex==`CORE_MUL_xMLAL_2) |
                             (alu_mcyc_state_ex==`CORE_MUL_NORMAL))));

  assert_never #(0,0,"Mul controls outside of valid multiply states")
    ovl_mcyc_state_check2(hclk, hreset_n,
                          ((alu_mlal_ex | alu_mull_ex) &
                           ~((alu_mcyc_state_ex==`CORE_MUL_xMULL_1) |
                             (alu_mcyc_state_ex==`CORE_MUL_xMULL_2) |
                             (alu_mcyc_state_ex==`CORE_MUL_xMULL_3) |
                             (alu_mcyc_state_ex==`CORE_MUL_xMULL_4) |
                             (alu_mcyc_state_ex==`CORE_MUL_xMULL_5) |
                             (alu_mcyc_state_ex==`CORE_MUL_xMLAL_1) |
                             (alu_mcyc_state_ex==`CORE_MUL_xMLAL_2) |
                             (alu_mcyc_state_ex==`CORE_MUL_xMLAL_1) |
                             (alu_mcyc_state_ex==`CORE_MUL_xMLAL_2) |
                             (alu_mcyc_state_ex==`CORE_MUL_NORMAL))));

  assert_never #(0,0,"Mul controls outside of valid multiply states")
    ovl_mcyc_state_check4(hclk, hreset_n,
                          (alu_div_ex &
                           ~((alu_mcyc_state_ex==`CORE_DIV_CLZ_1) |
                             (alu_mcyc_state_ex==`CORE_DIV_CLZ_2) |
                             (alu_mcyc_state_ex==`CORE_DIV_INIT)  |
                             (alu_mcyc_state_ex==`CORE_DIV_SRT)   |
                             (alu_mcyc_state_ex==`CORE_DIV_WREG)  |
                             (alu_mcyc_state_ex==`CORE_MUL_NORMAL))));

  assert_never #(0,0,"ALU clock gate control has been incorrectly formed")
    ovl_alu_cg_en_check(hclk, hreset_n,
                        (~alu_cg_en &
                         (div_srt_ctl_we | fault_divby0_we |
                          mul_carry_we   | clz_res_reg_we  |
                          div_cnt_we     | div_cnt_reg_we)));

  assert_never_unknown #(0,5,0,"X propagate protect default statements")
    ovl_alu_ctrl_x_check1(hclk, hreset_n, 1'b1,
                          {alu_mcyc_state_ex, alu_de_2_ex_en});

  assert_never_unknown #(0,4,0,"X propagate protect default statements")
    ovl_alu_ctrl_x_check2(hclk, hreset_n, alu_muldiv_ex,
                          {rf_rd_b_zero_ex_i, rf_rd_a_zero_ex_i});

  assert_never #(0,0,"Condition codes C/V without N/Z")
    ovl_flags1_en(hclk, hreset_n,
                   (alu_cflag_wr_ex | alu_vflag_wr_ex) &
                  ~(alu_nflag_wr_ex & alu_zflag_wr_ex));

  assert_never #(0,0,"Condition N & Z must be set at the same time")
    ovl_flags2_en(hclk, hreset_n, alu_nflag_wr_ex != alu_zflag_wr_ex);

`endif

`include "cm3_dpu_undefs.vp"

endmodule
