module cm3_dpu_alu_srtdiv
  (// Inputs
   div_srt_init_ex_i, div_srt_sign_ex_i, div_srt_in1_ex_i, div_srt_in2_ex_i,
   div_srt_rem_ex_o, div_srt_res_ex_o
   );

  input [1:0]    div_srt_init_ex_i;        // DIV SRT unit intialisation value
  input          div_srt_sign_ex_i;        // DIV SRT unit sign value
  input [31:0]   div_srt_in1_ex_i;         // DIV SRT unit input 1
  input [31:0]   div_srt_in2_ex_i;         // DIV SRT unit input 2

  output [31:0]  div_srt_rem_ex_o;         // DIV SRT unit remainder
  output [3:0]   div_srt_res_ex_o;         // DIV SRT unit result


  wire [31:0]    div_srt_in2_lsr1_ex;      // div_srt_in2_ex shifted right one
  wire [31:0]    div_srt_in2_lsr2_ex;      // div_srt_in2_ex shifted right two
  wire [31:0]    div_srt_in2_lsr3_ex;      // div_srt_in2_ex shifted right three
  wire           srt_sign_ex;              // SRT sign value
  wire           addsub_a_add_ex;          // Add/sub A is add
  wire [32:0]    addsub_a_in1_ex;          // Add/sub A, input 1
  wire [32:0]    addsub_a_in2_ex;          // Add/sub A, input 2
  wire [32:0]    addsub_a_res_ex;          // Add/sub A result
  wire           addsub_b_add_ex;          // Add/sub B is add
  wire [32:0]    addsub_b_in1_ex;          // Add/sub B, input 1
  wire [32:0]    addsub_b_in2_ex;          // Add/sub B, input 2
  wire [32:0]    addsub_b_res_ex;          // Add/sub B result
  wire           addsub_c_add_ex;          // Add/sub C is add
  wire [31:0]    addsub_c_res_ex;          // Add/sub C result
  wire           addsub_d_add_ex;          // Add/sub D is add
  wire [31:0]    addsub_d_res_ex;          // Add/sub D result
  wire [31:0]    addsub_a_lsr1_ex;         // addsub_a_res_ex shifted right one
  wire [31:0]    addsub_b_lsr1_ex;         // addsub_b_res_ex shifted right one
  wire [31:0]    div_srt_rem_ex;           // SRT remainder
  wire [3:0]     div_srt_res_ex;           // SRT result


  assign srt_sign_ex  = div_srt_in2_ex_i[31] & div_srt_sign_ex_i;


  assign div_srt_in2_lsr1_ex = {{1{srt_sign_ex}},div_srt_in2_ex_i[31:1]};
  assign div_srt_in2_lsr2_ex = {{2{srt_sign_ex}},div_srt_in2_ex_i[31:2]};
  assign div_srt_in2_lsr3_ex = {{3{srt_sign_ex}},div_srt_in2_ex_i[31:3]};


  assign addsub_a_add_ex = (((div_srt_in1_ex_i[31] ^ div_srt_sign_ex_i) &
                             ~div_srt_init_ex_i[0]) | div_srt_init_ex_i[1]);


  assign addsub_a_in1_ex[32:0]  = {div_srt_in1_ex_i, 1'b1};
  assign addsub_a_in2_ex[32:1]  = {32{~addsub_a_add_ex}} ^ div_srt_in2_ex_i;
  assign addsub_a_in2_ex[   0]  = ~addsub_a_add_ex;
  assign addsub_a_res_ex[32:0]  = addsub_a_in1_ex[32:0] + addsub_a_in2_ex[32:0];
  assign addsub_a_lsr1_ex[31:0] = addsub_a_res_ex[32:1];


  assign addsub_b_add_ex = addsub_a_lsr1_ex[31] ^ div_srt_sign_ex_i;


  assign addsub_b_in1_ex[32:0]  = {addsub_a_lsr1_ex, 1'b1};
  assign addsub_b_in2_ex[32:1]  = {32{~addsub_b_add_ex}} ^ div_srt_in2_lsr1_ex;
  assign addsub_b_in2_ex[   0]  = ~addsub_b_add_ex;
  assign addsub_b_res_ex[32:0]  = addsub_b_in1_ex[32:0] + addsub_b_in2_ex[32:0];
  assign addsub_b_lsr1_ex[31:0] = addsub_b_res_ex[32:1];


  assign addsub_c_add_ex  = addsub_b_lsr1_ex[31] ^ div_srt_sign_ex_i;

  assign addsub_c_res_ex  = (({32{ addsub_c_add_ex}} &
                              (addsub_b_lsr1_ex + div_srt_in2_lsr2_ex)) |
                             ({32{~addsub_c_add_ex}} &
                              (addsub_b_lsr1_ex - div_srt_in2_lsr2_ex)));


  assign addsub_d_add_ex  = addsub_c_res_ex[31] ^ div_srt_sign_ex_i;

  assign addsub_d_res_ex  = (({32{ addsub_d_add_ex}} &
                              (addsub_c_res_ex + div_srt_in2_lsr3_ex)) |
                             ({32{~addsub_d_add_ex}} &
                              (addsub_c_res_ex - div_srt_in2_lsr3_ex)));

  assign div_srt_rem_ex   = addsub_d_res_ex;
  assign div_srt_res_ex   = {~addsub_a_lsr1_ex[31], ~addsub_b_lsr1_ex[31],
                             ~addsub_c_res_ex[31],  ~addsub_d_res_ex[31]};

  assign div_srt_rem_ex_o = div_srt_rem_ex;
  assign div_srt_res_ex_o = div_srt_res_ex;

endmodule
