module cm3_dpu_etmintf
  (// Inputs
   hreset_n, hclk, etm_ifen_i, etm_fifofull_i, etm_instr_valid_de_i,
   instr_ext_pflush_fe_i, instr_isb_de_i, instr_immed_valid_ex_i,
   instr_ccpass_ex_i, instr_stall_ex_i, instr_not_bkpt_ex_i,
   instr_br_pflush_ex_i, instr_itfold_ex_i, instr_complete_ex_i, lsu_valid_ex_i,
   lsu_stack_start_i, lsu_dbl_fault_ex_i, fpu_lazy_stack_op_i, nvic_lockup_i,
   nvic_int_curr_isr_i, nvic_int_nxt_isr_i, nvic_int_invoke_i, int_stack_op_i,
   int_entry_i, int_exit_i, int_return_i, rst_pc_valid_ex_i, ahb_htransd_i,
   nvic_slp_asleep_i, rf_pc_fwd_ex_i, fault_lsu_precise_ex_i, dbg_halted_i,
   nvic_dbg_mon_req_i, nvic_dbg_halt_req_i,
   etm_ivalid_o, etm_ia_o, etm_istall_o, etm_ibranch_o, etm_iindrbr_o,
   etm_iccfail_o, etm_isb_o, etm_dvalid_o, etm_pstat_o, etm_fold_o,
   etm_lsu_o, etm_cancel_o, etm_intstat_o, etm_intnum_o, etm_flush_o,
   etm_findbr_o, etm_stall_o
   );

  input          hreset_n;
  input          hclk;
  input          etm_ifen_i;               // ETM interface enable
  input          etm_fifofull_i;           // ETM FIFO full
  input          etm_instr_valid_de_i;     // ETM instruction valid
  input          instr_ext_pflush_fe_i;    // Pipeline flush for external event
  input          instr_isb_de_i;           // ISB decoded
  input          instr_immed_valid_ex_i;   // Immediate valid
  input          instr_ccpass_ex_i;        // Condition code pass
  input [3:0]    instr_stall_ex_i;         // Instruction stall information
  input          instr_not_bkpt_ex_i;      // Instruction is not a BKPT
  input [2:0]    instr_br_pflush_ex_i;     // Branch pipeline flush
  input          instr_itfold_ex_i;        // IT folded
  input          instr_complete_ex_i;      // Instruction has completed
  input          lsu_valid_ex_i;           // LSU op is valid
  input          lsu_stack_start_i;        // LSU stack op start
  input          lsu_dbl_fault_ex_i;       // LSU double fault detected
  input          fpu_lazy_stack_op_i;      // Lazy stacking
  input [1:0]    nvic_lockup_i;            // NVIC lock-up
  input [8:0]    nvic_int_curr_isr_i;      // NVIC current ISR number
  input [7:0]    nvic_int_nxt_isr_i;       // NVIC next ISR number
  input          nvic_int_invoke_i;        // Interrupt stack request
  input          int_stack_op_i;           // LSU is stacking/unstacking
  input          int_entry_i;              // ISR entry
  input          int_exit_i;               // ISR exit
  input          int_return_i;             // Unstacking complete
  input          rst_pc_valid_ex_i;        // PC reset vector fetch complete
  input [1:0]    ahb_htransd_i;            // AHB D-side HTRANS
  input          nvic_slp_asleep_i;        // DPU asleep
  input [31:0]   rf_pc_fwd_ex_i;           // PC advanced to decode-time
  input          fault_lsu_precise_ex_i;   // Precise LSU fault
  input          dbg_halted_i;             // Core halted
  input          nvic_dbg_mon_req_i;       // Core monitor request
  input          nvic_dbg_halt_req_i;      // Core halt request

  output         etm_ivalid_o;             // ETM instr valid
  output [31:1]  etm_ia_o;                 // ETM instr address
  output         etm_istall_o;             // ETM instr stall
  output         etm_ibranch_o;            // ETM instr is a branch
  output         etm_iindrbr_o;            // ETM instr is an indirect branch
  output         etm_iccfail_o;            // ETM instr condition code failed
  output         etm_isb_o;                // ETM instr is an ISB
  output         etm_dvalid_o;             // ETM data match valid (for DWT)
  output [1:0]   etm_pstat_o;              // ETM pipeline status
  output         etm_fold_o;               // ETM IT fold
  output         etm_lsu_o;                // ETM LSU operation
  output         etm_cancel_o;             // ETM instruction cancelled
  output [2:0]   etm_intstat_o;            // ETM interrupt status
  output [8:0]   etm_intnum_o;             // ETM interrupt number
  output         etm_flush_o;              // ETM pipeline flush
  output         etm_findbr_o;             // ETM flush due to indirect branch
  output         etm_stall_o;              // ETM stall pipe

  wire           pipe_flush_ex;            // Pipeline flush
  wire           pc_chng_ex;               // PC changed
  wire           pc_chng_we;               // PC change write enable
  reg            pc_chng_reg;              // Registered pc_chng_ex
  wire           pc_indir_ex;              // Indirect PC modification
  wire           pc_indir_we;              // Indirect PC mod write enable
  reg            pc_indir_reg;             // Registered pc_indir_ex
  wire           nxt_etm_ivalid;           // Next ETM instr valid
  wire           etm_ivalid_we;            // ETM instr valid write enable
  reg            etm_ivalid;               // ETM instr valid
  wire           nxt_instr_isb_reg;        // ETM ISB instruction deccoded
  reg            instr_isb_reg;            // ETM ISB instruction deccoded
  wire           rst_pc_wr;                // Reset PC write
  wire           etm_istall;               // ETM instr stall
  wire           etm_ibranch;              // ETM instr is a branch
  wire           etm_iindrbr;              // ETM instr is an indirect branch
  wire           etm_iccfail;              // ETM instr condition code failed
  wire           etm_isb;                  // ETM instr is ISB
  wire           etm_dvalid;               // ETM data match valid (for DWT)
  wire [31:1]    pc_fe;                    // Fetch time PC
  wire           etm_ia_we;                // ETM instr address write enable
  reg  [31:1]    etm_ia;                   // ETM instr address
  wire           etm_intstat_we;           // ETM interrupt status write enable
  wire [2:0]     nxt_etm_intstat;          // Next ETM interrupt status
  reg  [2:0]     etm_intstat;              // ETM interrupt status
  wire [8:0]     etm_intnum;               // ETM interrupt number
  wire [1:0]     etm_pstat;                // ETM pipeline status
  wire           etm_fold;                 // ETM IT fold
  wire           etm_lsu;                  // ETM LSU operation
  wire           etm_cancel_we;            // ETM instr cancelled write enable
  wire           nxt_etm_cancel;           // Next ETM instruction cancelled
  reg            etm_cancel;               // ETM instruction cancelled
  wire           etm_flush;                // ETM pipeline flush
  wire           etm_findbr;               // ETM flush due to indirect branch
  wire           nxt_etm_stall;            // Next ETM stall pipe
  wire           etm_stall_we;             // ETM stall pipe write enable
  reg            etm_stall;                // ETM stall pipe


  assign pipe_flush_ex = (((|instr_br_pflush_ex_i) & ~int_stack_op_i) |
                          lsu_stack_start_i |
                          (etm_ivalid & ~(|instr_br_pflush_ex_i)));

  assign pc_chng_ex    = (etm_ifen_i & ((|instr_br_pflush_ex_i) |
                                        lsu_stack_start_i));

  assign pc_chng_we    = ((pipe_flush_ex & (pc_chng_ex ^ pc_chng_reg)) |
                          (pc_chng_ex & ~etm_ifen_i));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      pc_chng_reg <= 1'b0;
    else if (pc_chng_we)
      pc_chng_reg <= pc_chng_ex;

  assign pc_indir_ex   = (etm_ifen_i &
                          ((instr_br_pflush_ex_i[0] & ~instr_immed_valid_ex_i) |
                           (|instr_br_pflush_ex_i[2:1])                        |
                           lsu_stack_start_i));

  assign pc_indir_we   = ((pipe_flush_ex & (pc_indir_ex ^ pc_indir_reg)) |
                          (pc_indir_reg & ~etm_ifen_i));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      pc_indir_reg <= 1'b0;
    else if (pc_indir_we)
      pc_indir_reg <= pc_indir_ex;

  assign nxt_etm_ivalid = etm_ifen_i & etm_instr_valid_de_i;
  assign etm_ivalid_we  = etm_ifen_i | (etm_ivalid & ~etm_ifen_i);

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      etm_ivalid <= 1'b0;
    else if (etm_ivalid_we)
      etm_ivalid <= nxt_etm_ivalid;

  assign nxt_instr_isb_reg = etm_ifen_i & instr_isb_de_i;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      instr_isb_reg <= 1'b0;
    else if (etm_ivalid_we)
      instr_isb_reg <= nxt_instr_isb_reg;

  assign rst_pc_wr   = ~rst_pc_valid_ex_i & instr_br_pflush_ex_i[2];

  assign etm_istall  = etm_ifen_i & fault_lsu_precise_ex_i & ~etm_ivalid;
  assign etm_ibranch = etm_ivalid & pc_chng_reg;
  assign etm_iindrbr = etm_ivalid & pc_indir_reg;
  assign etm_iccfail = etm_ivalid & ~instr_ccpass_ex_i;
  assign etm_isb     = etm_ivalid & instr_isb_reg;
  assign etm_dvalid  = etm_ifen_i & ahb_htransd_i[1];

  assign pc_fe       = rf_pc_fwd_ex_i[31:1] - {{29{1'b0}}, 2'b10};

  assign etm_ia_we   = (etm_ifen_i &
                        (etm_instr_valid_de_i |
                         int_entry_i          |
                         int_return_i         |
                         (dbg_halted_i & ~nvic_dbg_halt_req_i)));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      etm_ia <= {31{1'b0}};
    else if (etm_ia_we)
      etm_ia <= pc_fe[31:1];

  assign etm_intstat_we  = etm_ifen_i | (|etm_intstat);
  assign nxt_etm_intstat = ({3{etm_ifen_i}} &
                            {(nvic_int_invoke_i | ~rst_pc_valid_ex_i) &
                             ~(rst_pc_wr  | int_entry_i |
                               int_exit_i | int_return_i),
                             ({2{rst_pc_wr   }} & 2'b01) |
                             ({2{int_entry_i }} & 2'b01) |
                             ({2{int_exit_i  }} & 2'b10) |
                             ({2{int_return_i}} & 2'b11)});

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      etm_intstat <= 3'b000;
    else if (etm_intstat_we)
      etm_intstat <= nxt_etm_intstat;

  assign etm_intnum = (etm_intstat[2] ? { 1'b0, nvic_int_nxt_isr_i } :
                                        nvic_int_curr_isr_i);

  assign etm_pstat = { (int_stack_op_i | fpu_lazy_stack_op_i),
                       instr_not_bkpt_ex_i };

  assign etm_fold  = instr_itfold_ex_i & etm_ivalid;
  assign etm_lsu   = lsu_valid_ex_i;

  assign nxt_etm_cancel = ((lsu_stack_start_i & ~fpu_lazy_stack_op_i &
                            (((|instr_stall_ex_i[2:0]) &
                              ~instr_br_pflush_ex_i[1] &
                              ~(instr_ext_pflush_fe_i & lsu_dbl_fault_ex_i)) |
                             (nvic_lockup_i[0] & ~nvic_lockup_i[1] &
                              nvic_int_invoke_i & ~instr_complete_ex_i))) |
                           rst_pc_wr);

  assign etm_cancel_we  = (nxt_etm_cancel & etm_ifen_i) | etm_cancel;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      etm_cancel <= 1'b0;
    else if (etm_cancel_we)
      etm_cancel <= nxt_etm_cancel;

  assign etm_flush  = pc_chng_reg  & ~etm_ivalid;
  assign etm_findbr = pc_indir_reg & ~etm_ivalid;


  assign nxt_etm_stall = (etm_ifen_i & etm_fifofull_i);
  assign etm_stall_we  = (etm_stall ^ nxt_etm_stall);

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      etm_stall <= 1'b0;
    else if (etm_stall_we)
      etm_stall <= nxt_etm_stall;

  assign etm_ivalid_o  = etm_ivalid;
  assign etm_ia_o      = etm_ia;
  assign etm_istall_o  = etm_istall;
  assign etm_ibranch_o = etm_ibranch;
  assign etm_iindrbr_o = etm_iindrbr;
  assign etm_iccfail_o = etm_iccfail;
  assign etm_isb_o     = etm_isb;
  assign etm_dvalid_o  = etm_dvalid;
  assign etm_pstat_o   = etm_pstat;
  assign etm_fold_o    = etm_fold;
  assign etm_lsu_o     = etm_lsu;
  assign etm_cancel_o  = etm_cancel;
  assign etm_intstat_o = etm_intstat;
  assign etm_intnum_o  = etm_intnum;
  assign etm_flush_o   = etm_flush;
  assign etm_findbr_o  = etm_findbr;
  assign etm_stall_o   = etm_stall;


`ifdef ARM_ASSERT_ON

  reg            ovl_dbg_mon_req_reg;
  reg            ovl_etm_findbr_reg;
  reg            ovl_etm_flush_reg;
  reg            ovl_etm_flush_valid;
  reg  [31:0]    ovl_etm_ia_check_val;
  reg            ovl_instr_br_pflush_track;
  reg            ovl_rst_pc_wr_reg;
  wire [31:0]    ovl_etm_full_ia;

  assign ovl_etm_full_ia = {etm_ia,1'b0};

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        ovl_dbg_mon_req_reg <= 1'b0;
        ovl_etm_findbr_reg  <= 1'b0;
        ovl_etm_flush_reg   <= 1'b0;
        ovl_rst_pc_wr_reg   <= 1'b0;
      end
    else
      begin
        ovl_dbg_mon_req_reg <= nvic_dbg_mon_req_i;
        ovl_etm_findbr_reg  <= etm_findbr;
        ovl_etm_flush_reg   <= etm_flush;
        ovl_rst_pc_wr_reg   <= rst_pc_wr;
      end

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_etm_flush_valid <= 1'b0;
    else if (etm_instr_valid_de_i | lsu_stack_start_i | ~etm_ifen_i)
      ovl_etm_flush_valid <= etm_ifen_i & instr_not_bkpt_ex_i;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_etm_ia_check_val <= {32{1'b0}};
    else if (rst_pc_wr & etm_ifen_i)
      ovl_etm_ia_check_val <= {pc_fe[31:1],1'b0};
    else if (etm_ivalid | lsu_dbl_fault_ex_i)
      ovl_etm_ia_check_val <= ovl_etm_full_ia;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_instr_br_pflush_track <= 1'b0;
    else if (|instr_br_pflush_ex_i)
      ovl_instr_br_pflush_track <= 1'b1;
    else if (etm_ivalid)
      ovl_instr_br_pflush_track <= 1'b0;

  assert_never #(0,0,
    "pc_chng_reg enable should only be asserted when pc_chng_reg is changed")
    ovl_pc_chng_excl(hclk, hreset_n, (pc_chng_we & (pc_chng_ex==pc_chng_reg)));

  assert_never #(0,0,
    "pc_indir_reg enable should only be asserted when pc_indir_reg is changed")
    ovl_pc_indir_excl(hclk, hreset_n, (pc_indir_we &
                                       (pc_indir_ex==pc_indir_reg)));

  assert_never #(0,0,
    "etm_ivalid and etm_flush should never be asserted together")
    ovl_etmctrl(hclk, hreset_n, etm_ivalid & etm_flush);

  assert_never #(0,0,
    "ETM indirect branch flush and valid sidebands do not match")
    ovl_etm_iindrbr(hclk, hreset_n, etm_ivalid & (etm_iindrbr^ovl_etm_findbr_reg));

  assert_never #(0,0,
    "Deassertion of etm_flush must be followed by etm_ivalid")
    ovl_flush_check(hclk, hreset_n, (etm_ifen_i & ovl_etm_flush_reg &
                                     ~etm_flush & ~etm_ivalid));

  assert_never #(0,0, "pc_indir_reg cannot be set without pc_chng_reg")
    ovl_pc_indir_check(hclk, hreset_n, pc_indir_reg & ~pc_chng_reg);

  assert_never #(0,0, "etm_findbr cannot be set without etm_flush")
    ovl_etm_findbr(hclk, hreset_n, etm_findbr & ~etm_flush);

  assert_never #(0,0, "int_stack_op cannot be set without etm_flush")
    ovl_etm_flush(hclk, hreset_n, (int_stack_op_i & ovl_etm_flush_valid &
                                   ~etm_flush));

  assert_never #(0,0, "etm_cancel has been incorrectly formed")
    ovl_etm_cancel(hclk, hreset_n,
                   (etm_cancel & ~ovl_rst_pc_wr_reg &
                    ((ovl_etm_full_ia!=ovl_etm_ia_check_val) |
                     (etm_pstat!=2'b10))));

  assert_zero_one_hot #(0,2,0, "ETM pipe status must be exclusive")
    ovl_etm_pstat_excl(hclk, hreset_n, etm_pstat);

  assert_never #(0,0,"etm_pstat has been incorrectly formed")
    ovl_etm_pstat(hclk, hreset_n, ( (|etm_pstat) & dbg_halted_i) |
                                  (~(^etm_pstat) & nvic_slp_asleep_i));

  assert_never #(0,0,"dbg_halted and nvic_slp_asleep should be exclusive")
    ovl_etm_sleephalt(hclk, hreset_n, dbg_halted_i & nvic_slp_asleep_i);

  assert_never #(0,0, "etm_intstat has been incorrectly formed")
    ovl_etm_intstat_1(hclk, hreset_n, etm_intstat > 4);

  assert_never #(0,0, "etm_ivalid has been incorrectly formed")
    ovl_etm_ivalid(hclk, hreset_n, (etm_ivalid &
                                    (dbg_halted_i |
                                     ((ovl_etm_full_ia==ovl_etm_ia_check_val) &
                                      ~ovl_instr_br_pflush_track))));

  assert_never #(0,0, "etm_istall has been incorrectly formed")
    ovl_etm_istall(hclk, hreset_n, etm_ivalid & etm_istall);

  assert_never #(0,0, "ETM interface has been incorrectly formed")
    ovl_etm_if(hclk, hreset_n,
               (({etm_intstat[1:0],etm_pstat,dbg_halted_i,etm_ivalid}==6'b000011) |
                ({etm_intstat[1:0],etm_pstat,dbg_halted_i,etm_ivalid}==6'b001001) |
                ({etm_intstat[1:0],etm_pstat,dbg_halted_i,etm_ivalid}==6'b001010) |
                ({etm_intstat[1:0],etm_pstat,dbg_halted_i,etm_ivalid}==6'b001011) |
                ({etm_intstat[1:0],etm_pstat,dbg_halted_i,etm_ivalid}==6'b010011) |
                ({etm_intstat[1:0],etm_pstat,dbg_halted_i,etm_ivalid}==6'b011001) |
                ({etm_intstat[1:0],etm_pstat,dbg_halted_i,etm_ivalid}==6'b011010) |
                ({etm_intstat[1:0],etm_pstat,dbg_halted_i,etm_ivalid}==6'b011011)));

`endif

endmodule
