module cm3_dpu_fetch
  (// Inputs
   hreset_n, hclk, ext_iflush_i, instr_valid_de_i, instr_use_word_de_i,
   instr_br_lr_sel_de_i, instr_br_imm_de_i, instr_br_ccpass_de_i,
   instr_de_br_det_de_i, instr_br_req_de_i, instr_br_pflush_fast_de_i,
   instr_br_pflush_de_i, instr_br_imm_ex_i, instr_br_pflush_ex_i,
   instr_pflush_ex_i, instr_lsu_br_det_ex_i, instr_br_type_ex_i,
   instr_advance_ex_i, instr_detect_exit_ex_i, instr_dblfault_ex_i,
   instr_det_erly_br_trk_i, instr_det_erly_br_trk_v_i, privileged_i,
   nvic_int_invoke_i, nvic_int_tail_i, nvic_int_addr_i, int_entry_i, int_exit_i,
   int_return_i, nvic_int_newarrival_i, nvic_int_in_nmihf_i, mtx_ahb_hreadyi_i,
   mtx_ahb_haddracci_i, mtx_ahb_hrespi_i, mpu_hrespi_i, mtx_ahb_hrdatai_i,
   mtx_ahb_hreadyd_i, etm_stall_i, etm_fifofull_i, nvic_slp_asleep_i,
   nvic_slp_sleeponexit_i, nvic_data_be8_i, lsu_stackpush_rdy_ex_i,
   lsu_pflush_ex_i, fpu_lazy_stack_op_i, fpu_lazy_stall_de_i, rf_rd_lr_de_i,
   rf_pc_wr_data_de_i, rf_rd_b_data_ex_i, rf_pc_fwd_ex_i, dbg_halted_i,
   nvic_dbg_halt_req_i, nvic_dbg_mon_req_i,
   state_fe_o, fe_address_add_res_o, fe_buf_has_valid_instr_o, instr_valid_fe_o,
   instr_word_present_fe_o, instr_stop_adv_fe_o, instr_fault_fe_o,
   instr_ext_pflush_fe_o, instr_de_o, int_first_instr_valid_o,
   int_vector_fault_o, rf_wr_d_en_fe_o, rf_wr_d_addr_fe_o, rf_wr_d_data_fe_o,
   ahb_htransi_o, ahb_hsizei_o, ahb_haddri_o, ahb_hproti_o, ahb_htransi_track_o,
   fe_haddracci_o
   );

  parameter      RESET_ALL_REGS = 0;        // Reset all registers if 1
  parameter      CONST_AHB_CTRL = 0;        // Constant AHB control info if 1

  input          hreset_n;
  input          hclk;
  input          ext_iflush_i;              // Instruction flush request
  input          instr_valid_de_i;          // Instruction in decode is valid
  input          instr_use_word_de_i;       // Consumed 32 bits of instruction
  input          instr_br_lr_sel_de_i;      // Select LR for BLX and MOV PC LR
  input [31:0]   instr_br_imm_de_i;         // Branch immediate offset
  input          instr_br_ccpass_de_i;      // Branch passed condition code
  input          instr_de_br_det_de_i;      // Detected a branch in decode
  input          instr_br_req_de_i;         // Branch fetch request
  input [1:0]    instr_br_pflush_fast_de_i; // Branch pipeline flush (fast)
  input [1:0]    instr_br_pflush_de_i;      // Branch pipeline flush
  input [31:0]   instr_br_imm_ex_i;         // Branch immediate offset
  input [2:0]    instr_br_pflush_ex_i;      // Branch pipeline flush
  input          instr_pflush_ex_i;         // Pipeline flush due to instr
  input          instr_lsu_br_det_ex_i;     // LSU branch detected
  input [1:0]    instr_br_type_ex_i;        // Type of branch in execute
  input          instr_advance_ex_i;        // Advance instruction
  input          instr_detect_exit_ex_i;    // ISR exit detected
  input          instr_dblfault_ex_i;       // Double fault
  input          instr_det_erly_br_trk_i;   // Early branch detected
  input          instr_det_erly_br_trk_v_i; // Validated early branch detected
  input          privileged_i;              // Privileged (vs user) mode
  input          nvic_int_invoke_i;         // Interrupt stack request
  input          nvic_int_tail_i;           // Interrupt tail-chain request
  input [31:0]   nvic_int_addr_i;           // ISR vector address
  input          int_entry_i;               // ISR entry
  input          int_exit_i;                // ISR exit
  input          int_return_i;              // Unstacking complete
  input          nvic_int_newarrival_i;     // Interrupt new arrival
  input [2:0]    nvic_int_in_nmihf_i;       // In NMI/HF ISR
  input          mtx_ahb_hreadyi_i;         // AHB I-side HREADY
  input          mtx_ahb_haddracci_i;       // AHB I-side HADDRACC
  input [1:0]    mtx_ahb_hrespi_i;          // AHB I-side HRESP
  input          mpu_hrespi_i;              // MPU I-side HRESP
  input [31:0]   mtx_ahb_hrdatai_i;         // AHB I-side HRDATA
  input          mtx_ahb_hreadyd_i;         // AHB D-side HREADY
  input          etm_stall_i;               // ETM stall pipe
  input          etm_fifofull_i;            // ETM FIFO full
  input          nvic_slp_asleep_i;         // DPU asleep
  input          nvic_slp_sleeponexit_i;    // Sleep-on-exit request
  input          nvic_data_be8_i;           // Data is BE8
  input          lsu_stackpush_rdy_ex_i;    // LSU ready to push stack
  input          lsu_pflush_ex_i;           // LSU pipe flush
  input          fpu_lazy_stack_op_i;       // Lazy stacking in progress
  input          fpu_lazy_stall_de_i;       // FPU lazy stall
  input [31:0]   rf_rd_lr_de_i;             // R14
  input [31:1]   rf_pc_wr_data_de_i;        // RF PC write data
  input [31:0]   rf_rd_b_data_ex_i;         // RF B port read data
  input [31:0]   rf_pc_fwd_ex_i;            // PC advanced to decode-time
  input          dbg_halted_i;              // Core halted
  input          nvic_dbg_halt_req_i;       // Core halt request
  input          nvic_dbg_mon_req_i;        // Core monitor request

  output [3:0]   state_fe_o;                // Fetch state
  output [31:1]  fe_address_add_res_o;      // Fetch address adder result
  output         fe_buf_has_valid_instr_o;  // Fetch buffer has valid instr
  output         instr_valid_fe_o;          // Instruction in fetch is valid
  output         instr_word_present_fe_o;   // Word present in fetch buffer
  output         instr_stop_adv_fe_o;       // Stop decode time advance
  output [1:0]   instr_fault_fe_o;          // Instruction in fetch has faulted
  output         instr_ext_pflush_fe_o;     // External event caused pipe flush
  output [31:0]  instr_de_o;                // Instruction in decode
  output         int_first_instr_valid_o;   // 1st instr ready for entry/return
  output         int_vector_fault_o;        // Vector fetch fault
  output         rf_wr_d_en_fe_o;           // Fetch RF D port write enable
  output [3:0]   rf_wr_d_addr_fe_o;         // Fetch RF D port write address
  output [31:0]  rf_wr_d_data_fe_o;         // Fetch RF D port write data
  output [1:0]   ahb_htransi_o;             // AHB I-side HTRANS
  output [1:0]   ahb_hsizei_o;              // AHB I-side HSIZE
  output [31:0]  ahb_haddri_o;              // AHB I-side HADDR
  output [1:0]   ahb_hproti_o;              // AHB I-side HPROT
  output         ahb_htransi_track_o;       // Track I-side AHB transactions
  output         fe_haddracci_o;            // Fetch version of HADDRACCI

  wire           instr_valid_fe;            // Instruction in fetch is valid
  wire           instr_word_present_fe;     // Word present in fetch buffer
  wire           instr_stop_adv_fe;         // Stop decode time advance
  wire [1:0]     instr_fault_fe;            // Instruction in fetch has faulted
  wire           instr_ext_pflush_fe;       // External event caused pipe flush
  wire [31:0]    instr_de;                  // Instruction in decode
  wire           int_vector_err_fe;         // Vector fetch error
  wire           int_vector_fault;          // Vector fetch fault
  wire           int_first_instr_valid;     // 1st instr ready for entry/return
  wire           rf_wr_d_en_fe;             // Fetch RF D port write enable
  wire [3:0]     rf_wr_d_addr_fe;           // Fetch RF D port write address
  wire [31:0]    rf_wr_d_data_fe;           // Fetch RF D port write data
  wire [1:0]     ahb_htransi;               // AHB I-side HTRANS
  wire [1:0]     ahb_hsizei;                // AHB I-side HSIZE
  wire [31:0]    ahb_haddri;                // AHB I-side HADDR
  wire [1:0]     ahb_hproti;                // AHB I-side HPROT
  wire           ahb_htransi_track;         // Track I-side AHB transactions
  wire [3:0]     state_fe;                  // Fetch state
  wire [31:1]    fe_address_add_res;        // Fetch address adder result
  wire           fe_buf_has_valid_instr;    // Fetch buffer has valid instr
  wire           fe_haddracci;              // Fetch version of HADDRACCI

  cm3_dpu_fetch_ctl
    u_cm3_dpu_fetch_ctl
      (// Inputs
       .hreset_n                   (hreset_n),
       .hclk                       (hclk),
       .ext_iflush_i               (ext_iflush_i),
       .instr_ext_pflush_fe_i      (instr_ext_pflush_fe),
       .instr_br_pflush_ex_i       (instr_br_pflush_ex_i),
       .instr_dblfault_ex_i        (instr_dblfault_ex_i),
       .instr_detect_exit_ex_i     (instr_detect_exit_ex_i),
       .lsu_stackpush_rdy_ex_i     (lsu_stackpush_rdy_ex_i),
       .nvic_int_invoke_i          (nvic_int_invoke_i),
       .nvic_int_tail_i            (nvic_int_tail_i),
       .int_entry_i                (int_entry_i),
       .int_exit_i                 (int_exit_i),
       .int_return_i               (int_return_i),
       .int_vector_err_fe_i        (int_vector_err_fe),
       .nvic_int_newarrival_i      (nvic_int_newarrival_i),
       .nvic_int_in_nmihf_i        (nvic_int_in_nmihf_i),
       .mtx_ahb_hreadyi_i          (mtx_ahb_hreadyi_i),
       .mtx_ahb_hreadyd_i          (mtx_ahb_hreadyd_i),
       .fe_haddracci_i             (fe_haddracci),
       .nvic_slp_asleep_i          (nvic_slp_asleep_i),
       .nvic_slp_sleeponexit_i     (nvic_slp_sleeponexit_i),
       .dbg_halted_i               (dbg_halted_i),
       .state_fe_o                 (state_fe),
       .rf_wr_d_en_fe_o            (rf_wr_d_en_fe),
       .rf_wr_d_addr_fe_o          (rf_wr_d_addr_fe));

  cm3_dpu_fetch_ahbintf #(RESET_ALL_REGS,CONST_AHB_CTRL)
    u_cm3_dpu_fetch_ahbintf
      (// Inputs
       .hreset_n                   (hreset_n),
       .hclk                       (hclk),
       .state_fe_i                 (state_fe),
       .ext_iflush_i               (ext_iflush_i),
       .instr_valid_de_i           (instr_valid_de_i),
       .instr_use_word_de_i        (instr_use_word_de_i),
       .instr_advance_ex_i         (instr_advance_ex_i),
       .instr_dblfault_ex_i        (instr_dblfault_ex_i),
       .instr_detect_exit_ex_i     (instr_detect_exit_ex_i),
       .privileged_i               (privileged_i),
       .nvic_int_invoke_i          (nvic_int_invoke_i),
       .nvic_int_tail_i            (nvic_int_tail_i),
       .nvic_int_addr_i            (nvic_int_addr_i),
       .int_entry_i                (int_entry_i),
       .int_return_i               (int_return_i),
       .nvic_int_newarrival_i      (nvic_int_newarrival_i),
       .nvic_int_in_nmihf_i        (nvic_int_in_nmihf_i),
       .int_exit_i                 (int_exit_i),
       .nvic_data_be8_i            (nvic_data_be8_i),
       .lsu_stackpush_rdy_ex_i     (lsu_stackpush_rdy_ex_i),
       .lsu_pflush_ex_i            (lsu_pflush_ex_i),
       .fpu_lazy_stack_op_i        (fpu_lazy_stack_op_i),
       .fpu_lazy_stall_de_i        (fpu_lazy_stall_de_i),
       .mtx_ahb_hreadyi_i          (mtx_ahb_hreadyi_i),
       .mtx_ahb_haddracci_i        (mtx_ahb_haddracci_i),
       .mtx_ahb_hrespi_i           (mtx_ahb_hrespi_i),
       .mpu_hrespi_i               (mpu_hrespi_i),
       .mtx_ahb_hrdatai_i          (mtx_ahb_hrdatai_i),
       .ahb_htransi_i              (ahb_htransi),
       .etm_stall_i                (etm_stall_i),
       .etm_fifofull_i             (etm_fifofull_i),
       .nvic_slp_asleep_i          (nvic_slp_asleep_i),
       .rf_rd_lr_de_i              (rf_rd_lr_de_i),
       .rf_rd_b_data_ex_i          (rf_rd_b_data_ex_i),
       .rf_pc_fwd_ex_i             (rf_pc_fwd_ex_i),
       .instr_br_lr_sel_de_i       (instr_br_lr_sel_de_i),
       .instr_br_imm_de_i          (instr_br_imm_de_i),
       .instr_br_type_ex_i         (instr_br_type_ex_i),
       .instr_br_ccpass_de_i       (instr_br_ccpass_de_i),
       .instr_de_br_det_de_i       (instr_de_br_det_de_i),
       .instr_br_req_de_i          (instr_br_req_de_i),
       .instr_br_imm_ex_i          (instr_br_imm_ex_i),
       .instr_pflush_ex_i          (instr_pflush_ex_i),
       .instr_lsu_br_det_ex_i      (instr_lsu_br_det_ex_i),
       .instr_br_pflush_fast_de_i  (instr_br_pflush_fast_de_i),
       .instr_br_pflush_de_i       (instr_br_pflush_de_i),
       .rf_pc_wr_data_de_i         (rf_pc_wr_data_de_i),
       .instr_det_erly_br_trk_i    (instr_det_erly_br_trk_i),
       .instr_det_erly_br_trk_v_i  (instr_det_erly_br_trk_v_i),
       .dbg_halted_i               (dbg_halted_i),
       .nvic_dbg_halt_req_i        (nvic_dbg_halt_req_i),
       .nvic_dbg_mon_req_i         (nvic_dbg_mon_req_i),
       .instr_valid_fe_o           (instr_valid_fe),
       .instr_word_present_fe_o    (instr_word_present_fe),
       .instr_stop_adv_fe_o        (instr_stop_adv_fe),
       .instr_fault_fe_o           (instr_fault_fe),
       .instr_ext_pflush_fe_o      (instr_ext_pflush_fe),
       .instr_de_o                 (instr_de),
       .int_first_instr_valid_o    (int_first_instr_valid),
       .int_vector_err_fe_o        (int_vector_err_fe),
       .int_vector_fault_o         (int_vector_fault),
       .rf_wr_d_data_fe_o          (rf_wr_d_data_fe),
       .ahb_htransi_o              (ahb_htransi),
       .ahb_hsizei_o               (ahb_hsizei),
       .ahb_haddri_o               (ahb_haddri),
       .ahb_hproti_o               (ahb_hproti),
       .ahb_htransi_track_o        (ahb_htransi_track),
       .fe_address_add_res_o       (fe_address_add_res),
       .fe_buf_has_valid_instr_o   (fe_buf_has_valid_instr),
       .fe_haddracci_o             (fe_haddracci));

  assign instr_valid_fe_o         = instr_valid_fe;
  assign instr_word_present_fe_o  = instr_word_present_fe;
  assign instr_stop_adv_fe_o      = instr_stop_adv_fe;
  assign instr_fault_fe_o         = instr_fault_fe;
  assign instr_ext_pflush_fe_o    = instr_ext_pflush_fe;
  assign instr_de_o               = instr_de;
  assign int_vector_fault_o       = int_vector_fault;
  assign int_first_instr_valid_o  = int_first_instr_valid;
  assign rf_wr_d_en_fe_o          = rf_wr_d_en_fe;
  assign rf_wr_d_addr_fe_o        = rf_wr_d_addr_fe;
  assign rf_wr_d_data_fe_o        = rf_wr_d_data_fe;
  assign ahb_htransi_o            = ahb_htransi;
  assign ahb_hsizei_o             = ahb_hsizei;
  assign ahb_haddri_o             = ahb_haddri;
  assign ahb_hproti_o             = ahb_hproti;
  assign ahb_htransi_track_o      = ahb_htransi_track;
  assign fe_address_add_res_o     = fe_address_add_res;
  assign state_fe_o               = state_fe;
  assign fe_buf_has_valid_instr_o = fe_buf_has_valid_instr;
  assign fe_haddracci_o           = fe_haddracci;

endmodule
