`include "cm3_dpu_defs.vp"

module cm3_dpu_fetch_ahbintf
  (// Inputs
   hreset_n, hclk, ext_iflush_i, state_fe_i, instr_det_erly_br_trk_i,
   instr_det_erly_br_trk_v_i, instr_de_br_det_de_i, instr_valid_de_i,
   instr_use_word_de_i, instr_br_lr_sel_de_i, instr_br_imm_de_i,
   instr_br_req_de_i, instr_br_ccpass_de_i, instr_br_pflush_fast_de_i,
   instr_br_pflush_de_i, instr_br_imm_ex_i, instr_advance_ex_i,
   instr_lsu_br_det_ex_i, instr_pflush_ex_i, instr_br_type_ex_i,
   instr_dblfault_ex_i, instr_detect_exit_ex_i, privileged_i,
   nvic_int_invoke_i, nvic_int_newarrival_i, nvic_int_addr_i, int_entry_i,
   nvic_int_tail_i, int_exit_i, int_return_i, nvic_int_in_nmihf_i,
   nvic_data_be8_i, lsu_stackpush_rdy_ex_i, lsu_pflush_ex_i,
   fpu_lazy_stack_op_i, fpu_lazy_stall_de_i, mtx_ahb_hreadyi_i,
   mtx_ahb_haddracci_i, mtx_ahb_hrespi_i, mpu_hrespi_i, mtx_ahb_hrdatai_i,
   ahb_htransi_i, etm_stall_i, etm_fifofull_i, nvic_slp_asleep_i,
   rf_pc_wr_data_de_i, rf_rd_lr_de_i, rf_rd_b_data_ex_i, rf_pc_fwd_ex_i,
   dbg_halted_i, nvic_dbg_halt_req_i, nvic_dbg_mon_req_i,
   fe_address_add_res_o, fe_buf_has_valid_instr_o, instr_valid_fe_o,
   instr_word_present_fe_o, instr_stop_adv_fe_o, instr_fault_fe_o,
   instr_ext_pflush_fe_o, instr_de_o, int_first_instr_valid_o,
   int_vector_err_fe_o, int_vector_fault_o, rf_wr_d_data_fe_o,
   ahb_htransi_o, ahb_hsizei_o, ahb_haddri_o, ahb_hproti_o,
   ahb_htransi_track_o, fe_haddracci_o
   );

  parameter      RESET_ALL_REGS = 0;        // Reset all registers if 1
  parameter      AHB_CONST_CTRL = 0;        // Constant AHB control info if 1

  input          hreset_n;
  input          hclk;
  input [3:0]    state_fe_i;                // Fetch state
  input          ext_iflush_i;              // Instruction flush request
  input          instr_det_erly_br_trk_i;   // Early branch detected
  input          instr_det_erly_br_trk_v_i; // Validated early branch detected
  input          instr_br_lr_sel_de_i;      // Select LR for BLX and MOV PC LR
  input [31:0]   instr_br_imm_de_i;         // Branch immediate offset
  input          instr_valid_de_i;          // Instruction in decode is valid
  input          instr_use_word_de_i;       // Consumed 32 bits of instruction
  input          instr_br_ccpass_de_i;      // Branch passed condition code
  input          instr_de_br_det_de_i;      // Detected a branch in decode
  input          instr_br_req_de_i;         // Branch fetch request
  input [1:0]    instr_br_pflush_fast_de_i; // Branch pipeline flush (fast)
  input [1:0]    instr_br_pflush_de_i;      // Branch pipeline flush
  input [31:0]   instr_br_imm_ex_i;         // Branch immediate offset
  input          instr_advance_ex_i;        // Advance instruction
  input          instr_detect_exit_ex_i;    // ISR exit detected
  input          instr_dblfault_ex_i;       // Double fault
  input [1:0]    instr_br_type_ex_i;        // Type of branch in execute
  input          instr_pflush_ex_i;         // Pipeline flush due to instr
  input          instr_lsu_br_det_ex_i;     // LSU branch detected
  input          privileged_i;              // Privileged (vs user) mode
  input          nvic_int_invoke_i;         // Interrupt stack request
  input          nvic_int_newarrival_i;     // Interrupt new arrival
  input [31:0]   nvic_int_addr_i;           // ISR vector address
  input          int_entry_i;               // ISR entry
  input          nvic_int_tail_i;           // Interrupt tail-chain request
  input          int_exit_i;                // ISR exit
  input          int_return_i;              // Unstacking complete
  input [2:0]    nvic_int_in_nmihf_i;       // In NMI/HF ISR
  input          nvic_data_be8_i;           // Data is BE8
  input          lsu_stackpush_rdy_ex_i;    // LSU ready to push stack
  input          lsu_pflush_ex_i;           // LSU pipe flush
  input          fpu_lazy_stall_de_i;       // FPU lazy stall
  input          fpu_lazy_stack_op_i;       // Lazy stacking in progress
  input          mtx_ahb_hreadyi_i;         // AHB I-side HREADY
  input          mtx_ahb_haddracci_i;       // AHB I-side HADDRACC
  input [1:0]    mtx_ahb_hrespi_i;          // AHB I-side HRESP
  input          mpu_hrespi_i;              // MPU I-side HRESP
  input [31:0]   mtx_ahb_hrdatai_i;         // AHB I-side HRDATA
  input [1:0]    ahb_htransi_i;             // Final AHB I-side HTRANS
  input          etm_stall_i;               // ETM stall pipe
  input          etm_fifofull_i;            // ETM FIFO full
  input          nvic_slp_asleep_i;         // DPU asleep
  input [31:1]   rf_pc_wr_data_de_i;        // RF PC write data
  input [31:0]   rf_rd_lr_de_i;             // R14
  input [31:0]   rf_rd_b_data_ex_i;         // RF B port read data
  input [31:0]   rf_pc_fwd_ex_i;            // PC advanced to decode-time
  input          dbg_halted_i;              // Core halted
  input          nvic_dbg_halt_req_i;       // Core halt request
  input          nvic_dbg_mon_req_i;        // Core monitor request

  output [31:1]  fe_address_add_res_o;      // Fetch address adder result
  output         fe_buf_has_valid_instr_o;  // Fetch buffer has valid instr
  output         instr_valid_fe_o;          // Instruction in fetch is valid
  output         instr_word_present_fe_o;   // Word present in fetch buffer
  output         instr_stop_adv_fe_o;       // Stop decode time advance
  output [1:0]   instr_fault_fe_o;          // Instruction in fetch has faulted
  output         instr_ext_pflush_fe_o;     // External event caused pipe flush
  output [31:0]  instr_de_o;                // Instruction in decode
  output         int_vector_err_fe_o;       // Vector fetch error
  output         int_vector_fault_o;        // Vector fetch fault
  output         int_first_instr_valid_o;   // 1st instr ready for entry/return
  output [31:0]  rf_wr_d_data_fe_o;         // RF D port write data
  output [1:0]   ahb_htransi_o;             // AHB I-side HTRANS
  output [1:0]   ahb_hsizei_o;              // AHB I-side HSIZE
  output [31:0]  ahb_haddri_o;              // AHB I-side HADDR
  output [1:0]   ahb_hproti_o;              // AHB I-side HPROT
  output         ahb_htransi_track_o;       // Track I-side AHB transactions
  output         fe_haddracci_o;            // Fetch version of HADDRACCI

  wire           full_hreset_n;             // Apply hreset_n to all registers
  wire [31:1]    fe_address_add_res;        // Fetch address adder result
  wire           fe_buf_has_valid_instr;    // Fetch buffer has valid instr
  wire           instr_valid_fe;            // Instruction in fetch is valid
  wire           instr_word_present_fe;     // Word present in fetch buffer
  wire [1:0]     instr_fault_fe;            // Instruction in fetch has faulted
  wire           instr_ext_pflush_fe;       // External event caused pipe flush
  wire [95:0]    fe_buf_fe;                 // Fetch buffer contents
  wire [31:0]    instr_fast_fe;             // Sequential instruction in fetch
  wire [31:0]    instr_fe;                  // Instruction in fetch
  wire           fetch_event_fe;            // Fetch event
  wire           instr_copro_fe;            // Instruction is a coprocessor instr
  wire           instr_bkpt_fe;             // Instruction is BKPT
  wire           instr_stop_adv_fe;         // Stop fetching
  reg [31:0]     instr_de;                  // Instruction in decode
  wire           int_first_instr_valid;     // 1st instr ready for entry/return
  wire [31:0]    rf_wr_d_data_fe;           // RF D port write data
  wire [1:0]     ahb_hsizei;                // AHB I-side HSIZE
  wire [31:0]    ahb_req_haddri;            // Requested AHB I-side HADDR
  wire [1:0]     ahb_req_hproti;            // Requested AHB I-side HPROT
  reg  [1:0]     ahb_req_htransi;           // Requested AHB I-side HTRANS
  wire [31:0]    ahb_haddri;                // AHB I-side HADDR
  wire [1:0]     ahb_hproti;                // AHB I-side HPROT
  wire [1:0]     ahb_htransi;               // AHB I-side HTRANS
  reg            ahb_htransi_issued_fe;     // I-side AHB transaction issued
  reg            fe_htransi_track;          // Track I-side AHB transactions
  wire           ahb_htransi_track;         // Track I and constant ctl trans
  reg            int_vector_err_fe;         // Vector fetch error
  wire           int_vector_fault_we;       // Vector fetch fault write enable
  reg            int_vector_fault;          // Vector fetch fault
  wire           iflush_invalid;            // Ignore external flush
  wire           nxt_instr_ext_pflush_fe;   // Next external event pipe flush
  wire           instr_ext_pflush_fe_we;    // External event pipe flush wr en
  reg            iflush_detected;           // External flush detected
  reg            ahb_htransi_issued_fe_we;  // AHB trans issued wr en
  reg            int_vect_trk_fe;           // Vector fetch track
  wire           nxt_int_vect_trk_fe;       // Next vector fetch track
  wire           int_vect_trk_fe_we;        // Vector fetch track write enable
  reg  [31:0]    fe_address;                // Fetch address
  reg  [31:0]    nxt_fe_address;            // Next fetch address
  reg  [31:0]    nxt_default_fe_address;    // Default next fetch address
  reg            fe_address_we;             // Fetch address write enable
  reg            default_fe_address_we;     // Default fe address write enable
  wire           fe_buf_wr_ptr_restart;     // Restart buffer write pointer
  wire           fe_buf_wr_ptr_reset;       // Reset buffer write pointer
  wire           fe_buf_wr_ptr_interrupt;   // Interrupt restart write pointer
  wire           ahb_hrdatai_valid_fe;      // AHB I-side HRDATA valid
  reg  [2:0]     fe_buf_wr_ptr;             // Fetch buffer write pointer
  wire [2:0]     nxt_fe_buf_wr_ptr;         // Next fetch buffer write pointer
  wire           fe_buf_wr_ptr_we;          // Fetch buffer write pointer wr en
  reg            sel_br_rd_ptr;             // Select buffer branch read pointer
  reg  [5:0]     fe_buf_rd_ptr;             // Fetch buffer read pointer
  reg  [5:0]     nxt_fe_buf_rd_ptr_erly;    // Next buffer read pointer (early)
  wire [5:0]     nxt_fe_buf_rd_ptr;         // Next fetch buffer read pointer
  reg            fe_buf_rd_ptr_fast_we;     // Buffer read pointer wr en (fast)
  reg            fe_buf_rd_ptr_we;          // Fetch buffer read pointer wr en
  wire [5:0]     nxt_default_fe_buf_rd_ptr; // Default next buffer read pointer
  wire           instr_br_valid_de;         // Branch valid
  wire [5:0]     fe_buf_fault;              // Fetch buffer fault
  reg  [31:0]    fe_buf_word_0;             // Fetch buffer word 0
  reg  [1:0]     fe_buf_fault_0;            // Fetch buffer fault 0
  wire           fe_buf_word_0_we;          // Fetch buffer word 0 write enable
  reg  [31:0]    fe_buf_word_1;             // Fetch buffer word 1
  reg  [1:0]     fe_buf_fault_1;            // Fetch buffer fault 1
  wire           fe_buf_word_1_we;          // Fetch buffer word 1 write enable
  reg  [31:0]    fe_buf_word_2;             // Fetch buffer word 2
  reg  [1:0]     fe_buf_fault_2;            // Fetch buffer fault 2
  wire           fe_buf_word_2_fast_we;     // Fetch buffer word 2 wr en (fast)
  wire           fe_buf_word_2_we;          // Fetch buffer word 2 write enable
  wire [31:0]    nxt_fe_buf_word_2;         // Next fetch buffer word 2
  wire [1:0]     nxt_fe_buf_fault_2;        // Next fetch buffer fault 2
  reg            fe_fault_trk;              // Fetch fault track
  wire           nxt_fe_fault_trk;          // Next fetch fault track
  wire           fe_fault_trk_we;           // Fetch fault track write enable
  wire           fe_address_add_sel_1;      // Select fetch address 1
  wire           fe_address_add_sel_2;      // Select fetch address 2
  wire           fe_address_add_sel_3;      // Select fetch address 3
  wire [31:1]    fe_address_add_1;          // PC + branch immediate (de)
  wire [31:1]    fe_address_add_2;          // PC + branch immediate (ex)
  wire [31:1]    fe_address_add_3;          // Sequential address
  reg  [2:0]     nxt_fe_buf_status;         // Next fetch buffer status
  reg  [2:0]     fe_buf_status;             // Fetch buffer status
  wire           fe_buf_status_we;          // Fetch buffer status write enable
  wire           fe_buf_inc;                // Increment fetch buffer
  wire           fe_buf_dec;                // Decrement detch buffer
  wire           fe_buf_clr;                // Clear fetch buffer
  wire [2:0]     nxt_fe_buf_ctrl;           // Next fetch buffer control
  reg  [2:0]     fe_buf_ctrl;               // Fetch buffer control
  wire           fe_buf_data_req;           // Fetch buffer data request
  wire           fe_buf_advance;            // Advance fetch buffer
  wire [6:0]     ahb_haddri_sel;            // HADDR selection
  wire [1:0]     ahb_hrespi;                // Combined AHB I-side HRESP
  wire           int_vector_err_fe_we;      // Vector fetch error write enable
  wire           nxt_int_vector_err_fe;     // Next vector fetch error
  wire           instr_size_de;             // Instruction size
  wire           fe_buf_has_instr;          // Buffer contains instruction
  wire [31:0]    ahb_hrdatai_be8;           // BE8 swizzle of vector
  wire           htransi_issued;            // AHB I-side trans issued
  wire [5:0]     instr_sel_fe;              // Instruction select
  wire [5:0]     instr_sel_fast_fe;         // Instruction select (fast)
  reg  [5:0]     instr_sel_de;              // Instruction select
  wire [3:0]     fe_buf_fault_word_sel;     // Buffer fault selection
  wire [1:0]     fe_buf_fault_present;      // Fault present in fetch buffer
  wire           ahb_trans_waited;          // Spotted a wait-stated transfer
  wire           capt_ahb_ctrl_we;          // Capture control info write enable
  wire           ahb_const_ctrl_we;         // Tracking write enable
  reg            ahb_const_ctrl;            // Keep AHB control info constant
  wire           capt_ahb_valid;            // Captured information valid
  reg  [31:0]    capt_ahb_haddri;           // Captured address information
  reg  [1:0]     capt_ahb_htransi;          // Captured transfer information
  reg  [1:0]     capt_ahb_hproti;           // Captured protection information
  wire           ahb_const_iactive;         // Constant AHB I active
  wire           ahb_const_haddracci;       // Modified HADDRACCI
  wire           fe_haddracci;              // Fetch version of HADDRACCI
  wire           opt_const_en;              // Constant control logic removal

`ifdef ARM_DSM

  wire          dsm_reset_all_regs;
  wire          dsm_const_ahb_ctrl;

  assign full_hreset_n = dsm_reset_all_regs    ? hreset_n : 1'b1;
  assign opt_const_en = dsm_const_ahb_ctrl     ? 1'b1     : 1'b0;

`else

  assign full_hreset_n = (RESET_ALL_REGS == 1) ? hreset_n : 1'b1;
  assign opt_const_en  = (AHB_CONST_CTRL > 0)  ? 1'b1     : 1'b0;

`endif

  assign iflush_invalid = (dbg_halted_i | instr_dblfault_ex_i | int_exit_i |
                           lsu_pflush_ex_i | instr_detect_exit_ex_i |
                           nvic_int_newarrival_i |
                           (state_fe_i==`CORE_FTCH_RST_START)  |
                           (state_fe_i==`CORE_FTCH_PREFETCH_1) |
                           (state_fe_i==`CORE_FTCH_PRELOAD_2));

  assign nxt_instr_ext_pflush_fe = ext_iflush_i & ~iflush_invalid;
  assign instr_ext_pflush_fe_we  = ((iflush_detected & iflush_invalid) |
                                    nxt_instr_ext_pflush_fe);

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      iflush_detected <= 1'b0;
    else if (instr_ext_pflush_fe_we)
      iflush_detected <= nxt_instr_ext_pflush_fe;

  assign instr_ext_pflush_fe = iflush_detected & ~nvic_slp_asleep_i;


  assign ahb_hrespi = { mpu_hrespi_i, mtx_ahb_hrespi_i[0] };



  assign fe_address_add_sel_1  = instr_de_br_det_de_i & ~instr_br_type_ex_i[1];
  assign fe_address_add_1      = rf_pc_fwd_ex_i[31:1] + instr_br_imm_de_i[31:1];

  assign fe_address_add_sel_2  = instr_br_type_ex_i[1];
  assign fe_address_add_2      = rf_pc_fwd_ex_i[31:1] + instr_br_imm_ex_i[31:1];

  assign fe_address_add_sel_3  = ~(instr_de_br_det_de_i | instr_br_type_ex_i[1]);
  assign fe_address_add_3      = {fe_address[31:2], 1'b0} + {{29{1'b0}},2'b10};

  assign fe_address_add_res    = (({31{fe_address_add_sel_1}} & fe_address_add_1) |
                                  ({31{fe_address_add_sel_2}} & fe_address_add_2) |
                                  ({31{fe_address_add_sel_3}} & fe_address_add_3));

  assign instr_size_de    = ((instr_de[15:13] == 3'b111) &
                             (instr_de[12:11] != 2'b00));

  assign fe_buf_has_instr = (~nvic_slp_asleep_i & ~etm_stall_i & ~dbg_halted_i &
                             (// Branch FDATA in buffer
                              instr_det_erly_br_trk_i |
                              nxt_fe_buf_status[1] |
                              (nxt_fe_buf_status[0] & ~instr_size_de) |
                              (nxt_fe_buf_status[0] &  instr_size_de &
                               (fe_buf_rd_ptr[0] |
                                fe_buf_rd_ptr[2] |
                                fe_buf_rd_ptr[4]))));

  assign fe_buf_has_valid_instr = fe_buf_has_instr & ~(|fe_buf_fault_present);

  assign instr_valid_fe = ((fe_buf_has_instr & ~ext_iflush_i) |
                           (fe_fault_trk & ~nvic_slp_asleep_i & ~etm_stall_i &
                            ~dbg_halted_i &
                            (nxt_fe_buf_status[0] & (|fe_buf_fault_word_sel[1:0]) |
                             nxt_fe_buf_status[1] & (|fe_buf_fault_word_sel[3:2]))));

  assign instr_word_present_fe = (// Two or more entries in Fetch Buffer
                                  (nxt_fe_buf_status[1] &
                                   ~((instr_sel_de[5] & (|fe_buf_fault[1:0]))   |
                                     (instr_sel_de[1] & (|fe_buf_fault[3:2]))   |
                                     (instr_sel_de[3] & (|fe_buf_fault[5:4])))) |
                                  (nxt_fe_buf_status[0] &
                                   (fe_buf_rd_ptr[0] |
                                    fe_buf_rd_ptr[2] |
                                    fe_buf_rd_ptr[4])));

  assign int_first_instr_valid = ((instr_valid_fe | instr_ext_pflush_fe) &
                                  ~nvic_int_newarrival_i &
                                  (state_fe_i==`CORE_FTCH_INT_ENTRY |
                                   state_fe_i==`CORE_FTCH_INT_RETURN));

  assign fe_buf_inc = (ahb_htransi_issued_fe & mtx_ahb_hreadyi_i);
  assign fe_buf_dec = (instr_valid_de_i & ~instr_det_erly_br_trk_i &
                       (fe_buf_rd_ptr[1] | fe_buf_rd_ptr[3] |
                        fe_buf_rd_ptr[5] | instr_use_word_de_i));
  assign fe_buf_clr = (((|instr_br_pflush_de_i) &
                        ((state_fe_i==`CORE_FTCH_FETCH)        |
                         (state_fe_i==`CORE_FTCH_INT_ENTRY)    |
                         (state_fe_i==`CORE_FTCH_INT_RETURN))) |
                       (state_fe_i==`CORE_FTCH_PREFETCH_1) |
                       (state_fe_i==`CORE_FTCH_PRELOAD_1)  |
                       instr_detect_exit_ex_i | dbg_halted_i | ext_iflush_i);
  assign nxt_fe_buf_ctrl = { fe_buf_clr, fe_buf_inc, fe_buf_dec };

  assign fe_buf_status_we = (|nxt_fe_buf_ctrl);

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        fe_buf_ctrl   <= 3'b000;
        fe_buf_status <= 3'b000;
      end
    else if (fe_buf_status_we)
      begin
        fe_buf_ctrl   <= nxt_fe_buf_ctrl;
        fe_buf_status <= nxt_fe_buf_status;
      end

  always @ (fe_buf_ctrl or fe_buf_status)
    case (fe_buf_ctrl)
      3'b100,3'b101,3'b110,3'b111 : begin
        nxt_fe_buf_status  = 3'b000;
      end
      3'b010 : begin
        nxt_fe_buf_status  = {fe_buf_status[1:0],1'b1};
      end
      3'b001 : begin
        nxt_fe_buf_status  = {1'b0,fe_buf_status[2:1]};
      end
       3'b011 : begin
        nxt_fe_buf_status  = fe_buf_status[2:0];
      end
      3'b000 : begin
        nxt_fe_buf_status  = fe_buf_status[2:0];
      end
      default : begin
        nxt_fe_buf_status  = 3'bxxx;
      end
    endcase

  assign fe_buf_advance  = (instr_advance_ex_i & ~etm_fifofull_i &
                            ~fpu_lazy_stall_de_i & ~instr_det_erly_br_trk_i &
                            ~nvic_dbg_mon_req_i &
                            (fe_buf_rd_ptr[1] | fe_buf_rd_ptr[3] |
                             fe_buf_rd_ptr[5] | instr_size_de));

  assign fe_buf_data_req = (~nvic_slp_asleep_i & ~fe_fault_trk &
                            ~instr_lsu_br_det_ex_i & ~instr_ext_pflush_fe &
                            (// FIFO is empty
                             (nxt_fe_buf_status==3'b000) |
                             (nxt_fe_buf_status==3'b001 &
                              ~(ahb_htransi_issued_fe &
                                instr_det_erly_br_trk_i)) |
                             (nxt_fe_buf_status==3'b011 &
                              ((~ahb_htransi_issued_fe &
                                ~instr_det_erly_br_trk_i) |
                               fe_buf_advance)) |
                             (nxt_fe_buf_status==3'b111 &
                              ~ahb_htransi_issued_fe & fe_buf_advance)));

  always @ (state_fe_i or nvic_int_invoke_i or instr_detect_exit_ex_i or
            dbg_halted_i or int_vector_err_fe or fe_buf_data_req or
            instr_br_req_de_i or nvic_int_tail_i or fpu_lazy_stack_op_i)
    case (state_fe_i)
      `CORE_FTCH_RESET:
        ahb_req_htransi = `CORE_AHB_IDLE;
      `CORE_FTCH_RST_START,   `CORE_FTCH_RST_LR:
        ahb_req_htransi = (int_vector_err_fe ? `CORE_AHB_IDLE :
                                               `CORE_AHB_NONSEQ);
      `CORE_FTCH_RST_SP,      `CORE_FTCH_RST_PC:
        ahb_req_htransi = `CORE_AHB_IDLE;
      `CORE_FTCH_PREFETCH_1:
        ahb_req_htransi = ((nvic_int_invoke_i | nvic_int_tail_i) ?
                           `CORE_AHB_NONSEQ : `CORE_AHB_IDLE);
      `CORE_FTCH_PREFETCH_2:
        ahb_req_htransi = `CORE_AHB_IDLE;
      `CORE_FTCH_PREFETCH_3:
        ahb_req_htransi = `CORE_AHB_NONSEQ;
      `CORE_FTCH_PRELOAD_1:
        ahb_req_htransi = `CORE_AHB_IDLE;
      `CORE_FTCH_PRELOAD_2:
        ahb_req_htransi = `CORE_AHB_NONSEQ;
      `CORE_FTCH_INT_ENTRY,   `CORE_FTCH_INT_RETURN:
        ahb_req_htransi = ((fe_buf_data_req | instr_br_req_de_i) ?
                           `CORE_AHB_NONSEQ : `CORE_AHB_IDLE);
      `CORE_FTCH_FETCH:
        ahb_req_htransi = ((fe_buf_data_req | instr_br_req_de_i) &
                           ~(nvic_int_invoke_i |
                             fpu_lazy_stack_op_i |
                             instr_detect_exit_ex_i |
                             dbg_halted_i)) ?
                          `CORE_AHB_NONSEQ : `CORE_AHB_IDLE;
      `CORE_FTCH_SLEEP :
        ahb_req_htransi = `CORE_AHB_IDLE;
      default:
        ahb_req_htransi = 2'bxx;
    endcase

  assign ahb_hrdatai_be8 = (int_vector_err_fe ?
                            32'hFFFFFFFF :
                            (nvic_data_be8_i ?
                             {mtx_ahb_hrdatai_i[7:0],   mtx_ahb_hrdatai_i[15:8],
                              mtx_ahb_hrdatai_i[23:16], mtx_ahb_hrdatai_i[31:24]} :
                             mtx_ahb_hrdatai_i));

  assign htransi_issued = (ahb_req_htransi[1] & mtx_ahb_hreadyi_i &
                           fe_haddracci & ~dbg_halted_i &
                           ~instr_br_pflush_fast_de_i[1]);

  always @ (instr_br_pflush_fast_de_i or rf_pc_wr_data_de_i or
            instr_br_pflush_de_i or instr_br_ccpass_de_i or
            instr_detect_exit_ex_i or fe_address_add_res or
            instr_pflush_ex_i or instr_br_req_de_i or htransi_issued)
    case ({(|instr_br_pflush_fast_de_i), htransi_issued})
      2'b11, 2'b10 : begin
        nxt_default_fe_address = {rf_pc_wr_data_de_i[31:2],2'b00};
        default_fe_address_we  = ((instr_br_pflush_de_i[1] |
                                   instr_br_ccpass_de_i) &
                                  ~instr_detect_exit_ex_i);
      end
      2'b01 : begin
        nxt_default_fe_address = {fe_address_add_res[31:2],2'b00};
        default_fe_address_we  = (~instr_pflush_ex_i &
                                  ~(instr_br_req_de_i & ~instr_br_ccpass_de_i));
      end
      2'b00 : begin
        nxt_default_fe_address = {fe_address_add_res[31:2],2'b00};
        default_fe_address_we  = 1'b0;
      end
      default : begin
        nxt_default_fe_address = {32{1'bx}};
        default_fe_address_we  = 1'bx;
      end
    endcase

  always @ (state_fe_i or mtx_ahb_hreadyi_i or ahb_hrdatai_be8 or int_vect_trk_fe or
            rf_pc_wr_data_de_i or instr_br_pflush_de_i or
            nxt_default_fe_address or default_fe_address_we)
    case (state_fe_i)
      `CORE_FTCH_RST_START : begin
        nxt_fe_address = 32'hFFFFFFFF;
        fe_address_we  = mtx_ahb_hreadyi_i;
      end
      `CORE_FTCH_RST_LR,      `CORE_FTCH_RST_SP,      `CORE_FTCH_PREFETCH_2 : begin
        nxt_fe_address = ahb_hrdatai_be8;
        fe_address_we  = mtx_ahb_hreadyi_i & int_vect_trk_fe;
      end
      `CORE_FTCH_RST_PC : begin
        nxt_fe_address = {rf_pc_wr_data_de_i[31:2],2'b00};
        fe_address_we  = 1'b0;
      end
      `CORE_FTCH_PREFETCH_1 : begin
        nxt_fe_address = {rf_pc_wr_data_de_i[31:2],2'b00};
        fe_address_we  = 1'b0;
      end
      `CORE_FTCH_PRELOAD_1 : begin
        nxt_fe_address = {rf_pc_wr_data_de_i[31:2],2'b00};
        fe_address_we  = |instr_br_pflush_de_i;
      end
      `CORE_FTCH_PRELOAD_2,   `CORE_FTCH_PREFETCH_3 : begin
        nxt_fe_address = {rf_pc_wr_data_de_i[31:2],2'b00};
        fe_address_we  = 1'b0;
      end
      `CORE_FTCH_RESET,       `CORE_FTCH_FETCH,       `CORE_FTCH_INT_ENTRY,
      `CORE_FTCH_INT_RETURN,  `CORE_FTCH_SLEEP : begin
        nxt_fe_address = nxt_default_fe_address;
        fe_address_we  = default_fe_address_we;
      end
      default : begin
        nxt_fe_address = {32{1'bx}};
        fe_address_we  = 1'bx;
      end
    endcase

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fe_address <= {32{1'b0}};
    else if (fe_address_we)
      fe_address <= nxt_fe_address;

  assign nxt_int_vect_trk_fe = (mtx_ahb_hreadyi_i & fe_haddracci &
                                ((state_fe_i==`CORE_FTCH_PREFETCH_1) |
                                 (state_fe_i==`CORE_FTCH_RST_START)  |
                                 (state_fe_i==`CORE_FTCH_RST_LR)));
  assign int_vect_trk_fe_we = (// Set
                               nxt_int_vect_trk_fe |
                               (int_vect_trk_fe & mtx_ahb_hreadyi_i));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      int_vect_trk_fe <= 1'b0;
    else if (int_vect_trk_fe_we)
      int_vect_trk_fe <= nxt_int_vect_trk_fe;

  always @ (instr_br_pflush_de_i or int_entry_i or int_return_i or
            ahb_htransi_issued_fe or ahb_req_htransi or mtx_ahb_hreadyi_i or
            fe_haddracci or ext_iflush_i or instr_br_req_de_i or
            instr_br_ccpass_de_i or state_fe_i)
    if ((instr_br_pflush_de_i[1] & ~int_entry_i & ~int_return_i) | ext_iflush_i)
      ahb_htransi_issued_fe_we = ahb_htransi_issued_fe;
    else if (instr_br_req_de_i)
      ahb_htransi_issued_fe_we = (instr_br_ccpass_de_i &
                                  ((ahb_req_htransi[1] &
                                    mtx_ahb_hreadyi_i  &
                                    fe_haddracci) ^ ahb_htransi_issued_fe)) |
                                 (~instr_br_ccpass_de_i & mtx_ahb_hreadyi_i &
                                  ahb_htransi_issued_fe);
    else
      ahb_htransi_issued_fe_we = (~((state_fe_i==`CORE_FTCH_RST_START)  |
                                    (state_fe_i==`CORE_FTCH_RST_LR)     |
                                    (state_fe_i==`CORE_FTCH_PREFETCH_1) |
                                    (state_fe_i==`CORE_FTCH_PRELOAD_1)) ?
                                  (mtx_ahb_hreadyi_i &
                                   ((ahb_req_htransi[1] & fe_haddracci) ^
                                    ahb_htransi_issued_fe)) :
                                  (mtx_ahb_hreadyi_i & ahb_htransi_issued_fe));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ahb_htransi_issued_fe <= 1'b0;
    else if (ahb_htransi_issued_fe_we)
      ahb_htransi_issued_fe <= ~ahb_htransi_issued_fe;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fe_htransi_track <= 1'b0;
    else if (mtx_ahb_hreadyi_i)
      fe_htransi_track <= ahb_htransi_i[1] & fe_haddracci;

  assign ahb_htransi_track = fe_htransi_track | ahb_const_iactive;

  assign rf_wr_d_data_fe         = fe_address;


  assign fe_buf_wr_ptr_reset     = (state_fe_i == `CORE_FTCH_RESET);

  assign fe_buf_wr_ptr_interrupt = ((state_fe_i == `CORE_FTCH_PREFETCH_3) |
                                    (state_fe_i == `CORE_FTCH_PRELOAD_1));

  assign fe_buf_wr_ptr_restart   = (// Reset
                                    fe_buf_wr_ptr_reset |
                                    fe_buf_wr_ptr_interrupt |
                                    (|instr_br_pflush_de_i));

  assign ahb_hrdatai_valid_fe    = ahb_htransi_issued_fe & mtx_ahb_hreadyi_i;

  assign nxt_fe_buf_wr_ptr       = (fe_buf_wr_ptr_restart ?
                                    3'b001 :
                                    {fe_buf_wr_ptr[1:0],fe_buf_wr_ptr[2]});

  assign fe_buf_wr_ptr_we = (fe_buf_wr_ptr_reset | ahb_hrdatai_valid_fe    |
                             (fe_buf_wr_ptr_interrupt & ~fe_buf_wr_ptr[0]) |
                             (|instr_br_pflush_de_i));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fe_buf_wr_ptr <= 3'b000;
    else if (fe_buf_wr_ptr_we)
      fe_buf_wr_ptr <= nxt_fe_buf_wr_ptr;


  assign instr_br_valid_de = (instr_de_br_det_de_i & instr_br_ccpass_de_i &
                              ~instr_valid_de_i);

  assign fe_buf_word_0_we      = ahb_hrdatai_valid_fe & fe_buf_wr_ptr[0];
  assign fe_buf_word_1_we      = ahb_hrdatai_valid_fe & fe_buf_wr_ptr[1];
  assign fe_buf_word_2_fast_we = ahb_hrdatai_valid_fe & fe_buf_wr_ptr[2];
  assign fe_buf_word_2_we      = fe_buf_word_2_fast_we | instr_br_valid_de;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      fe_buf_word_0 <= {32{1'b0}};
    else if (fe_buf_word_0_we)
      fe_buf_word_0 <= mtx_ahb_hrdatai_i;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fe_buf_fault_0 <= 2'b00;
    else if (fe_buf_word_0_we)
      fe_buf_fault_0 <= ahb_hrespi;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      fe_buf_word_1 <= {32{1'b0}};
    else if (fe_buf_word_1_we)
      fe_buf_word_1 <= mtx_ahb_hrdatai_i;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fe_buf_fault_1 <= 2'b00;
    else if (fe_buf_word_1_we)
      fe_buf_fault_1 <= ahb_hrespi;

  assign nxt_fe_buf_word_2 = instr_br_valid_de ? instr_de : mtx_ahb_hrdatai_i;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      fe_buf_word_2 <= {32{1'b0}};
    else if (fe_buf_word_2_we)
      fe_buf_word_2 <= nxt_fe_buf_word_2;

  assign nxt_fe_buf_fault_2 = instr_br_valid_de ? 2'b00 : ahb_hrespi;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fe_buf_fault_2 <= 2'b00;
    else if (fe_buf_word_2_we)
      fe_buf_fault_2 <= nxt_fe_buf_fault_2;


  assign nxt_fe_fault_trk = (// normal fetch fault
                             (ahb_htransi_issued_fe & (|ahb_hrespi) &
                              ~mtx_ahb_hreadyi_i & ~|instr_br_pflush_de_i &
                              ~ext_iflush_i) |
                             ((state_fe_i==`CORE_FTCH_PREFETCH_2) &
                              (|ahb_hrespi) & ~mtx_ahb_hreadyi_i) & ~ext_iflush_i);

  assign fe_fault_trk_we  = (// Fetch queue hits ahb_hrespi_i outside branch
                             (nxt_fe_fault_trk & ~fe_fault_trk) |
                             (instr_br_pflush_de_i[0] & fe_fault_trk) |
                             (instr_br_pflush_de_i[1] & fe_fault_trk &
                              ~int_entry_i & ~int_return_i) |
                             (ext_iflush_i & fe_fault_trk) |
                             ((state_fe_i==`CORE_FTCH_PREFETCH_1) &
                              mtx_ahb_hreadyi_i & fe_haddracci &
                              fe_fault_trk));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fe_fault_trk <= 1'b0;
    else if (fe_fault_trk_we)
      fe_fault_trk <= nxt_fe_fault_trk;

  assign fe_buf_fault = { fe_buf_fault_2, fe_buf_fault_1, fe_buf_fault_0 };


  assign nxt_default_fe_buf_rd_ptr = ((instr_use_word_de_i & nxt_fe_buf_status[0]) ?
                                      {fe_buf_rd_ptr[3:0],fe_buf_rd_ptr[5:4]} :
                                      {fe_buf_rd_ptr[4:0],fe_buf_rd_ptr[5]});

  always @ (state_fe_i or fe_address or rf_pc_wr_data_de_i or
            instr_br_pflush_de_i or nxt_default_fe_buf_rd_ptr or
            instr_valid_de_i or instr_det_erly_br_trk_i)
    case (state_fe_i)
      `CORE_FTCH_RESET :
        begin
          sel_br_rd_ptr          = 1'b0;
          nxt_fe_buf_rd_ptr_erly = 6'b000001;
          fe_buf_rd_ptr_we       = 1'b1;
          fe_buf_rd_ptr_fast_we  = 1'b1;
        end
      `CORE_FTCH_RST_PC,      `CORE_FTCH_PREFETCH_3 :
        begin
          sel_br_rd_ptr          = 1'b0;
          nxt_fe_buf_rd_ptr_erly = fe_address[1] ? 6'b000010 : 6'b000001;
          fe_buf_rd_ptr_we       = 1'b1;
          fe_buf_rd_ptr_fast_we  = 1'b1;
        end
      `CORE_FTCH_PRELOAD_1 :
        begin
          sel_br_rd_ptr          = 1'b0;
          nxt_fe_buf_rd_ptr_erly = (rf_pc_wr_data_de_i[1] ? 6'b000010 :
                                                            6'b000001);
          fe_buf_rd_ptr_we       = (|instr_br_pflush_de_i);
          fe_buf_rd_ptr_fast_we  = 1'b0;
        end
      `CORE_FTCH_RST_START,   `CORE_FTCH_RST_LR,      `CORE_FTCH_RST_SP,
      `CORE_FTCH_FETCH,       `CORE_FTCH_INT_ENTRY,   `CORE_FTCH_INT_RETURN,
      `CORE_FTCH_PREFETCH_1,  `CORE_FTCH_PREFETCH_2,  `CORE_FTCH_PRELOAD_2,
      `CORE_FTCH_SLEEP :
        begin
          sel_br_rd_ptr          = (|instr_br_pflush_de_i);
          nxt_fe_buf_rd_ptr_erly = nxt_default_fe_buf_rd_ptr;
          fe_buf_rd_ptr_we       = ((|instr_br_pflush_de_i) |
                                    (instr_valid_de_i &
                                     ~instr_det_erly_br_trk_i));
          fe_buf_rd_ptr_fast_we  = instr_valid_de_i;
        end
      default :
        begin
          sel_br_rd_ptr          = 1'bx;
          nxt_fe_buf_rd_ptr_erly = 6'bxxxxxx;
          fe_buf_rd_ptr_we       = 1'bx;
          fe_buf_rd_ptr_fast_we  = 1'bx;
        end
    endcase

  assign nxt_fe_buf_rd_ptr = (({6{ sel_br_rd_ptr}} &
                               {4'b0000,
                                rf_pc_wr_data_de_i[1],
                                ~rf_pc_wr_data_de_i[1]}) |
                              ({6{~sel_br_rd_ptr}} &
                               nxt_fe_buf_rd_ptr_erly));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fe_buf_rd_ptr <= 6'b000000;
    else if (fe_buf_rd_ptr_we)
      fe_buf_rd_ptr <= nxt_fe_buf_rd_ptr;

  assign instr_sel_fe = (instr_det_erly_br_trk_v_i ?
                         6'b010000 :
                         (fe_buf_rd_ptr_we ?
                          nxt_fe_buf_rd_ptr :
                          fe_buf_rd_ptr));

  assign instr_sel_fast_fe = (fe_buf_rd_ptr_fast_we ?
                              nxt_fe_buf_rd_ptr_erly : fe_buf_rd_ptr);

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      instr_sel_de <= 6'b000000;
    else
      instr_sel_de <= instr_sel_fe;

  assign nxt_int_vector_err_fe = ((|ahb_hrespi) & ~mtx_ahb_hreadyi_i &
                                  ~int_vector_err_fe & ~ext_iflush_i &
                                  ((state_fe_i==`CORE_FTCH_RST_LR) |
                                   (state_fe_i==`CORE_FTCH_RST_SP) |
                                   ((state_fe_i==`CORE_FTCH_PREFETCH_2) &
                                    ~nvic_int_newarrival_i)));

  assign int_vector_err_fe_we  = (// Set
                                  (nxt_int_vector_err_fe) |
                                  (int_vector_err_fe & ext_iflush_i) |
                                  (int_vector_err_fe &
                                   (state_fe_i==`CORE_FTCH_RST_PC)) |
                                  (int_vector_err_fe &
                                   (state_fe_i==`CORE_FTCH_PREFETCH_2) &
                                   mtx_ahb_hreadyi_i & lsu_stackpush_rdy_ex_i &
                                   nvic_int_in_nmihf_i[2]) |
                                  (int_vector_err_fe &
                                   (state_fe_i==`CORE_FTCH_PREFETCH_2) &
                                   nvic_int_newarrival_i));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      int_vector_err_fe <= 1'b0;
    else if (int_vector_err_fe_we)
      int_vector_err_fe <= nxt_int_vector_err_fe;

  assign int_vector_fault_we = nxt_int_vector_err_fe | int_vector_fault;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      int_vector_fault <= 1'b0;
    else if (int_vector_fault_we)
      int_vector_fault <= nxt_int_vector_err_fe;

  assign ahb_haddri_sel[0] = instr_br_type_ex_i[0];
  assign ahb_haddri_sel[1] = instr_br_lr_sel_de_i;
  assign ahb_haddri_sel[2] = (state_fe_i==`CORE_FTCH_PREFETCH_1);
  assign ahb_haddri_sel[3] = ((instr_pflush_ex_i &
                               (state_fe_i!=`CORE_FTCH_PREFETCH_1)) |
                              ((state_fe_i==`CORE_FTCH_PREFETCH_3) |
                               (state_fe_i==`CORE_FTCH_PRELOAD_2)));
  assign ahb_haddri_sel[4] = (state_fe_i==`CORE_FTCH_RST_START);
  assign ahb_haddri_sel[5] = (state_fe_i==`CORE_FTCH_RST_LR);
  assign ahb_haddri_sel[6] = ~(|ahb_haddri_sel[5:0]);

  assign ahb_req_haddri = (({32{ahb_haddri_sel[0]}} & {rf_rd_b_data_ex_i[31:2],2'b00}) |
                           ({32{ahb_haddri_sel[1]}} & {rf_rd_lr_de_i[31:2],2'b00})     |
                           ({32{ahb_haddri_sel[2]}} & {nvic_int_addr_i[31:2],2'b00})   |
                           ({32{ahb_haddri_sel[3]}} & {fe_address[31:2],2'b00})        |
                           ({32{ahb_haddri_sel[5]}} & 32'h00000004)                    |
                           ({32{ahb_haddri_sel[6]}} & {fe_address_add_res[31:2],2'b00}));

  assign ahb_hsizei     = `CORE_HSIZE_32;
  assign ahb_req_hproti = {(privileged_i |
                            ~((state_fe_i == `CORE_FTCH_FETCH)      |
                              (state_fe_i == `CORE_FTCH_INT_RETURN) |
                              (state_fe_i == `CORE_FTCH_PRELOAD_2))),
                           (state_fe_i==`CORE_FTCH_PREFETCH_1) |
                           (state_fe_i==`CORE_FTCH_RST_START)  |
                           (state_fe_i==`CORE_FTCH_RST_LR)};

  assign fe_buf_fe = { fe_buf_word_2_fast_we ? mtx_ahb_hrdatai_i : fe_buf_word_2,
                       fe_buf_word_1_we      ? mtx_ahb_hrdatai_i : fe_buf_word_1,
                       fe_buf_word_0_we      ? mtx_ahb_hrdatai_i : fe_buf_word_0  };

  assign instr_fast_fe =
         (({32{instr_sel_fast_fe[0]}} & {fe_buf_fe[31:16],fe_buf_fe[15:0]})  |
          ({32{instr_sel_fast_fe[1]}} & {fe_buf_fe[47:32],fe_buf_fe[31:16]}) |
          ({32{instr_sel_fast_fe[2]}} & {fe_buf_fe[63:48],fe_buf_fe[47:32]}) |
          ({32{instr_sel_fast_fe[3]}} & {fe_buf_fe[79:64],fe_buf_fe[63:48]}) |
          ({32{instr_sel_fast_fe[5]}} & {fe_buf_fe[15:0], fe_buf_fe[95:80]}));

  assign instr_fe       = ((instr_sel_fast_fe[4] | instr_det_erly_br_trk_v_i) ?
                           {fe_buf_fe[95:80],fe_buf_fe[79:64]} :
                           instr_fast_fe);

  assign fetch_event_fe = (~instr_br_valid_de &
                           (ahb_hrdatai_valid_fe |
                            fe_buf_rd_ptr_we | instr_det_erly_br_trk_i));

  always @(posedge hclk or negedge hreset_n)
    if (!hreset_n)
      instr_de <= 32'b0;
    else if (fetch_event_fe & ~instr_br_valid_de)
      instr_de <= instr_fe;

  assign instr_copro_fe    = ((instr_fe[15:10] == 6'b111011) &
                              ~(instr_fe[9:8] == 2'b11) & fetch_event_fe);

  assign instr_bkpt_fe     = (instr_fe[15:8]  == 8'b10111110);

  assign instr_stop_adv_fe = (instr_copro_fe | instr_bkpt_fe) & fetch_event_fe;

  assign fe_buf_fault_word_sel =
         {// Unaligned T2 fetch entry
          ({2{(instr_sel_de[5] & instr_size_de)}} & fe_buf_fault[1:0]) |
          ({2{(instr_sel_de[1] & instr_size_de)}} & fe_buf_fault[3:2]) |
          ({2{(instr_sel_de[3] & instr_size_de)}} & fe_buf_fault[5:4]),
          ({2{(|instr_sel_de[1:0])}} & fe_buf_fault[1:0]) |
          ({2{(|instr_sel_de[3:2])}} & fe_buf_fault[3:2]) |
          ({2{(|instr_sel_de[5:4])}} & fe_buf_fault[5:4])};

  assign fe_buf_fault_present = (({2{(|fe_buf_rd_ptr[1:0]) |
                                     (fe_buf_rd_ptr[5] & instr_size_de)}} &
                                  fe_buf_fault[1:0]) |
                                 ({2{(|fe_buf_rd_ptr[3:2]) |
                                     (fe_buf_rd_ptr[1] & instr_size_de)}} &
                                  fe_buf_fault[3:2]) |
                                 ({2{(|fe_buf_rd_ptr[5:4]) |
                                     (fe_buf_rd_ptr[3] & instr_size_de)}} &
                                  fe_buf_fault[5:4]));

  assign instr_fault_fe = ((|fe_buf_fault_word_sel[1:0]) ?
                           fe_buf_fault_word_sel[1:0] :
                           fe_buf_fault_word_sel[3:2]);


  assign ahb_trans_waited  = (opt_const_en & (|ahb_req_htransi) &
                              ~(mtx_ahb_hreadyi_i & mtx_ahb_haddracci_i));

  assign capt_ahb_ctrl_we  = ahb_trans_waited & ~ahb_const_ctrl;

  assign ahb_const_ctrl_we = (capt_ahb_ctrl_we |
                              (mtx_ahb_hreadyi_i & mtx_ahb_haddracci_i &
                               capt_ahb_valid));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ahb_const_ctrl <= 1'b0;
    else if (ahb_const_ctrl_we)
      ahb_const_ctrl <= ahb_trans_waited;

  assign capt_ahb_valid = ahb_const_ctrl & opt_const_en;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        capt_ahb_haddri  <= {32{1'b0}};
        capt_ahb_htransi <= 2'b00;
        capt_ahb_hproti  <= 2'b00;
      end
    else if (capt_ahb_ctrl_we)
      begin
        capt_ahb_haddri  <= ahb_req_haddri;
        capt_ahb_htransi <= ahb_req_htransi;
        capt_ahb_hproti  <= ahb_req_hproti;
      end

  assign ahb_const_iactive   = capt_ahb_valid & opt_const_en;

  assign ahb_haddri          = (capt_ahb_valid ? capt_ahb_haddri :
                                                 ahb_req_haddri);
  assign ahb_htransi         = (capt_ahb_valid ? capt_ahb_htransi :
                                                 ahb_req_htransi);
  assign ahb_hproti          = (capt_ahb_valid ? capt_ahb_hproti :
                                                 ahb_req_hproti);

  assign ahb_const_haddracci = (mtx_ahb_haddracci_i &
                                (((capt_ahb_haddri == ahb_req_haddri) &
                                  (capt_ahb_hproti == ahb_req_hproti)) |
                                 ~capt_ahb_valid));

  assign fe_haddracci        = (opt_const_en ? ahb_const_haddracci :
                                               mtx_ahb_haddracci_i);

  assign instr_valid_fe_o         = instr_valid_fe;
  assign instr_word_present_fe_o  = instr_word_present_fe;
  assign instr_stop_adv_fe_o      = instr_stop_adv_fe;
  assign int_first_instr_valid_o  = int_first_instr_valid;
  assign instr_fault_fe_o         = instr_fault_fe;
  assign instr_ext_pflush_fe_o    = instr_ext_pflush_fe;
  assign int_vector_err_fe_o      = int_vector_err_fe;
  assign int_vector_fault_o       = int_vector_fault;
  assign instr_de_o               = instr_de;
  assign rf_wr_d_data_fe_o        = rf_wr_d_data_fe;
  assign ahb_htransi_o            = ahb_htransi;
  assign ahb_hsizei_o             = ahb_hsizei;
  assign ahb_haddri_o             = ahb_haddri;
  assign ahb_hproti_o             = ahb_hproti;
  assign ahb_htransi_track_o      = ahb_htransi_track;
  assign fe_haddracci_o           = fe_haddracci;
  assign fe_address_add_res_o     = fe_address_add_res;
  assign fe_buf_has_valid_instr_o = fe_buf_has_valid_instr;


`ifdef ARM_ASSERT_ON

  reg           ovl_hreadi_reg;           // Reg version of hreadyi
  reg           ovl_hrespi_reg;           // Reg version of hrespi
  reg           ovl_chk_htransi;          // Check htransi signal is correct
  wire          ovl_ifetch_valid;         // Indicates instr fetch address phase
  wire    [1:0] ovl_prot_priv_de;         // Privilege of instruction in decode
  wire          ovl_prot_priv_de_mask;    // Mask for ovl_prot_priv_de
  wire          ovl_msr_ctl_de;           // MSR to CONTROL register in decode
  reg           ovl_msr_ctl_ex;           // MSR to CONTROL register in execute
  wire          ovl_priv_de;              // Decode time privilege
  reg           ovl_hproti_buf0;          // Priv attribute of fe_buf_word_0
  reg           ovl_hproti_buf1;          // Priv attribute of fe_buf_word_1
  reg           ovl_hproti_buf2;          // Priv attribute of fe_buf_word_2
  wire          ovl_hproti_buf0_mask_set; // Set ovl_hproti_buf_0_mask
  wire          ovl_hproti_buf0_mask_clr; // Clear ovl_hproti_buf_0_mask
  wire          ovl_hproti_buf0_mask_we;  // ovl_hproti_buf0_mask write enable
  wire          ovl_nxt_hproti_buf0_mask; // Next ovl_hproti_buf0_mask
  wire          ovl_hproti_buf1_mask_set; // Set ovl_hproti_buf1_mask
  wire          ovl_hproti_buf1_mask_clr; // Clear ovl_hproti_buf1_mask
  wire          ovl_hproti_buf1_mask_we;  // ovl_hproti_buf1_mask write enable
  wire          ovl_nxt_hproti_buf1_mask; // Next ovl_hproti_buf1_mask
  wire          ovl_hproti_buf2_mask_set; // Set ovl_hproti_buf2_mask
  wire          ovl_hproti_buf2_mask_clr; // Clear ovl_hproti_buf2_mask
  wire          ovl_hproti_buf2_mask_we;  // ovl_hproti_buf2_mask write enable
  wire          ovl_nxt_hproti_buf2_mask; // Next ovl_hproti_buf2_mask
  reg           ovl_hproti_buf0_mask;     // Mask for ovl_hproti_buf0
  reg           ovl_hproti_buf1_mask;     // Mask for ovl_hproti_buf1
  reg           ovl_hproti_buf2_mask;     // Mask for ovl_hproti_buf2
  reg           ovl_hproti_buf0_mask_reg; // Previous mask for ovl_hproti_buf0
  reg           ovl_hproti_buf1_mask_reg; // Previous mask for ovl_hproti_buf1
  reg           ovl_hproti_buf2_mask_reg; // Previous mask for ovl_hproti_buf2
  reg [1:0]     ovl_hproti_reg;           // Last hproti transaction
  reg           ovl_hreadyi_d1;
  reg [1:0]     ovl_htransi_d1;
  reg [31:0]    ovl_haddri_d1;
  reg [1:0]     ovl_hproti_d1;
  reg           ovl_hrespi_d1;
  reg [2:0]     ovl_hsizei_d1;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_chk_htransi <= 1'b0;
    else
      begin
        if (ext_iflush_i)
          ovl_chk_htransi <= 1'b0;
        else if (instr_br_pflush_de_i[1] & ~int_entry_i & ~int_return_i)
          ovl_chk_htransi <= 1'b0;
        else if (instr_br_pflush_de_i[0])
          ovl_chk_htransi <= ahb_req_htransi[1] & mtx_ahb_hreadyi_i & fe_haddracci;
        else if (instr_br_req_de_i & mtx_ahb_hreadyi_i)
          ovl_chk_htransi <= (ahb_req_htransi[1] & mtx_ahb_hreadyi_i &
                              fe_haddracci & instr_br_ccpass_de_i);
        else if (mtx_ahb_hreadyi_i)
          ovl_chk_htransi <= (ahb_req_htransi[1] & fe_haddracci &
                              ~((state_fe_i==`CORE_FTCH_RST_START)  |
                                (state_fe_i==`CORE_FTCH_RST_LR)     |
                                (state_fe_i==`CORE_FTCH_PREFETCH_1) |
                                (state_fe_i==`CORE_FTCH_PRELOAD_1)));
      end

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        ovl_hreadi_reg <= 1'b0;
        ovl_hrespi_reg <= 1'b0;
      end
    else
      begin
        ovl_hreadi_reg <= mtx_ahb_hreadyi_i;
        ovl_hrespi_reg <= (|ahb_hrespi);
      end

  always @(posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        ovl_hreadyi_d1 <= 1'b0;
        ovl_hsizei_d1  <= 2'b00;
        ovl_htransi_d1 <= 2'b00;
        ovl_haddri_d1  <= {32{1'b0}};
        ovl_hproti_d1  <= 2'b00;
        ovl_hrespi_d1  <= 1'b0;
      end
    else
      begin
        ovl_hreadyi_d1 <= mtx_ahb_hreadyi_i & mtx_ahb_haddracci_i;
        ovl_hsizei_d1  <= ahb_hsizei;
        ovl_htransi_d1 <= ahb_htransi;
        ovl_haddri_d1  <= ahb_haddri;
        ovl_hproti_d1  <= ahb_hproti;
        ovl_hrespi_d1  <= mpu_hrespi_i | (|mtx_ahb_hrespi_i);
      end

  assign ovl_priv_de = nvic_int_invoke_i | privileged_i;

  assign ovl_msr_ctl_de = (// in privileged mode
                           ovl_priv_de &
                           ((instr_de[15:4]  == 12'b111100111000) &
                            (instr_de[31:30] == 2'b10) & ~instr_de[28] &
                            (instr_de[25:16] == 10'b0000010100)) &
                           instr_valid_de_i & instr_size_de);

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_msr_ctl_ex <= 1'b0;
    else if (instr_valid_de_i)
      ovl_msr_ctl_ex <= ovl_msr_ctl_de;

  assign ovl_ifetch_valid = (~ahb_req_hproti[0] & ahb_req_htransi[1] &
                             mtx_ahb_hreadyi_i & fe_haddracci);

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_hproti_reg <= 2'b00;
    else if (ovl_ifetch_valid)
      ovl_hproti_reg <= ahb_req_hproti;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_hproti_buf0 <= 1'b0;
    else if (fe_buf_word_0_we)
      ovl_hproti_buf0 <= ovl_hproti_reg[1];

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_hproti_buf1 <= 1'b0;
    else if (fe_buf_word_1_we)
      ovl_hproti_buf1 <= ovl_hproti_reg[1];

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_hproti_buf2 <= 1'b0;
    else if (fe_buf_word_2_we)
      ovl_hproti_buf2 <= ovl_hproti_reg[1];

  assign ovl_prot_priv_de = (({2{instr_sel_de[0]}} &
                              {ovl_hproti_buf0, ovl_hproti_buf0})  |
                             ({2{instr_sel_de[1]}} &
                              {ovl_hproti_buf1, ovl_hproti_buf0})  |
                             ({2{instr_sel_de[2]}} &
                              {ovl_hproti_buf1, ovl_hproti_buf1})  |
                             ({2{instr_sel_de[3]}} &
                              {ovl_hproti_buf2, ovl_hproti_buf1})  |
                             ({2{instr_sel_de[4]}} &
                              {ovl_hproti_buf2, ovl_hproti_buf2})  |
                             ({2{instr_sel_de[5]}} &
                              {ovl_hproti_buf0, ovl_hproti_buf2}));

  assign ovl_hproti_buf0_mask_set = ovl_msr_ctl_ex;
  assign ovl_hproti_buf0_mask_clr = (ovl_hproti_buf0_mask_reg &
                                     ovl_hproti_buf0_mask &
                                     fe_buf_word_0_we);
  assign ovl_hproti_buf0_mask_we  = (ovl_hproti_buf0_mask_set |
                                     ovl_hproti_buf0_mask_clr);
  assign ovl_nxt_hproti_buf0_mask = ovl_hproti_buf0_mask_set;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_hproti_buf0_mask <= 1'b0;
    else if (ovl_hproti_buf0_mask_we)
      ovl_hproti_buf0_mask <= ovl_nxt_hproti_buf0_mask;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_hproti_buf0_mask_reg <= 1'b0;
    else if (fe_buf_word_0_we | ovl_msr_ctl_ex | instr_br_valid_de)
      ovl_hproti_buf0_mask_reg <= (// Clear if new MSR to CONTROL reg in
                                   ovl_msr_ctl_ex ? 1'b0 :
                                   (instr_br_valid_de ? 1'b0 :
                                    ovl_hproti_buf0_mask));

  assign ovl_hproti_buf1_mask_set = ovl_msr_ctl_ex;
  assign ovl_hproti_buf1_mask_clr = (ovl_hproti_buf1_mask_reg &
                                     ovl_hproti_buf1_mask &
                                     fe_buf_word_1_we);
  assign ovl_hproti_buf1_mask_we  = (ovl_hproti_buf1_mask_set |
                                     ovl_hproti_buf1_mask_clr);
  assign ovl_nxt_hproti_buf1_mask = ovl_hproti_buf1_mask_set;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_hproti_buf1_mask <= 1'b0;
    else if (ovl_hproti_buf1_mask_we)
      ovl_hproti_buf1_mask <= ovl_nxt_hproti_buf1_mask;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_hproti_buf1_mask_reg <= 1'b0;
    else if (fe_buf_word_1_we |  ovl_msr_ctl_ex | instr_br_valid_de)
      ovl_hproti_buf1_mask_reg <= (// Clear if new MSR to CONTROL reg in execute
                                   ovl_msr_ctl_ex ? 1'b0 :
                                   (instr_br_valid_de ? 1'b0 :
                                    ovl_hproti_buf1_mask));

  assign ovl_hproti_buf2_mask_set = ovl_msr_ctl_ex;
  assign ovl_hproti_buf2_mask_clr = (ovl_hproti_buf2_mask_reg &
                                     ovl_hproti_buf2_mask &
                                     fe_buf_word_2_we & ~instr_br_valid_de);
  assign ovl_hproti_buf2_mask_we  = (ovl_hproti_buf2_mask_set |
                                     ovl_hproti_buf2_mask_clr);
  assign ovl_nxt_hproti_buf2_mask = ovl_hproti_buf2_mask_set;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_hproti_buf2_mask <= 1'b0;
    else if (ovl_hproti_buf2_mask_we)
      ovl_hproti_buf2_mask <= ovl_nxt_hproti_buf2_mask;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_hproti_buf2_mask_reg <= 1'b0;
    else if (fe_buf_word_2_we | ovl_msr_ctl_ex | instr_br_valid_de)
      ovl_hproti_buf2_mask_reg <= (// Clear if new MSR to CONTROL reg in execute
                                   ovl_msr_ctl_ex ? 1'b0 :
                                   (instr_br_valid_de ? 1'b0 :
                                    ovl_hproti_buf2_mask));

  assign ovl_prot_priv_de_mask = (((|instr_sel_de[1:0]) & ovl_hproti_buf0_mask)  |
                                  ((|instr_sel_de[3:2]) & ovl_hproti_buf1_mask)  |
                                  ((|instr_sel_de[5:4]) & ovl_hproti_buf2_mask));

  assert_implication #(0,0,
                       "Privilege in fetch must match privilege in decode")
    ovl_priv_chk(hclk, hreset_n,
                 instr_valid_de_i & ~ovl_prot_priv_de_mask,
                 (ovl_priv_de == ovl_prot_priv_de[0]));

  assert_never #(0,0,"Pipeline must be filled on first opcode of ISR")
    ovl_chk_int_1st(hclk, hreset_n, (int_entry_i | int_return_i) &
                    ~(instr_valid_fe | instr_ext_pflush_fe));

  assert_never #(0,0,"FIFO must not be incremented when full")
    ovl_fe_buf_inc(hclk, hreset_n,
                   nxt_fe_buf_status==3'b111 & fe_buf_inc & ~fe_buf_dec &
                   ~fe_buf_clr & ~((state_fe_i==`CORE_FTCH_FETCH) &
                                   (instr_valid_fe | etm_stall_i) &
                                   (nvic_int_invoke_i | nvic_dbg_halt_req_i)));

  assert_never #(0,0,"FIFO must not be decremented when empty")
    ovl_fe_buf_dec(hclk, hreset_n, nxt_fe_buf_status==3'b000 & fe_buf_dec &
                   ~fe_buf_inc);

  assert_never #(0,0,"FIFO must be filled and empties in order")
    ovl_fifofill_never(hclk, hreset_n,
                       (nxt_fe_buf_status[1] & ~nxt_fe_buf_status[0]) |
                       (nxt_fe_buf_status[2] & ~nxt_fe_buf_status[1]));

  assert_never #(0,0,"Fault trk has been incorrectly formed")
    ovl_fe_fault_trk(hclk, hreset_n,
                     (fe_fault_trk_we & (fe_fault_trk==nxt_fe_fault_trk)) |
                     (fe_fault_trk & instr_ext_pflush_fe));

  assert_never #(0,0,"ahb_req_htransi should never emit BUSY")
    ovl_htrans_chk(hclk, hreset_n, ahb_req_htransi==2'b01);

  assert_zero_one_hot #(0,3,0,"instr_de in pointer must be exclusive")
    ovl_fdatain_excl(hclk, hreset_n, fe_buf_wr_ptr);

  assert_zero_one_hot #(0,6,0,"instr_de out pointer must be exclusive")
    ovl_fdataout_excl(hclk, hreset_n, fe_buf_rd_ptr);

  assert_zero_one_hot #(0,7,0,
                        "ahb_haddri_sel datapath selector must be exclusive")
    ovl_ahb_haddri_sel_excl(hclk, hreset_n, ahb_haddri_sel);

  assert_never #(0,0,"ahb_hrespi_i response incorrectly generated")
    ovl_hresp_chk(hclk, hreset_n,
                  ((|ahb_hrespi) & mtx_ahb_hreadyi_i &
                   ~(ovl_hrespi_reg & ~ovl_hreadi_reg)) |
                  (ovl_hrespi_reg & ~ovl_hreadi_reg & ~(|ahb_hrespi)));

  assert_next #(0,1,0,0,0,"int_vector_fault must be pulsed")
    ovl_vfault_pulsed(hclk, hreset_n, int_vector_fault, ~int_vector_fault);

  assert_never #(0,0,"nvic_slp_asleep and int_invoke must be exclusive")
    ovl_sleepinvoke(hclk, hreset_n, nvic_slp_asleep_i & nvic_int_invoke_i);

  assert_never #(0,0,
                 "Branch controls have overlapped into invalid interrupt state")
    ovl_brchctrl(hclk, hreset_n, ((state_fe_i==`CORE_FTCH_PREFETCH_2) &
                                  (instr_pflush_ex_i |
                                   instr_det_erly_br_trk_i)));

  assert_never #(0,0,"instr_lsu_br_det_ex should not be set at interrupt time")
    ovl_instr_lsu_br_det_1(hclk, hreset_n, instr_lsu_br_det_ex_i &
                           ~((state_fe_i==`CORE_FTCH_FETCH) |
                             (state_fe_i==`CORE_FTCH_PREFETCH_1)));

  assert_never #(0,0,"int_entry_i must only be pulsed in FTCH_INT_ENTRY state")
    ovl_int_entry(hclk, hreset_n, (int_entry_i &
                                   (state_fe_i!=`CORE_FTCH_INT_ENTRY)));

  assert_never #(0,0,"int_return_i must only be pulsed in FTCH_INT_RETURN state")
    ovl_int_ret(hclk, hreset_n, (int_return_i &
                                 (state_fe_i!=`CORE_FTCH_INT_RETURN)));

  assert_never #(0,0,"FIFO cannot be full and branch tracking active")
    ovl_br_trk_chk(hclk, hreset_n, ((nxt_fe_buf_status==3'b111) &
                                    instr_det_erly_br_trk_i));

  assert_never #(0,0,"ahb_htransi_issued_fe has been incorrectly formed")
    ovl_htransi_iss_fe(hclk, hreset_n, ahb_htransi_issued_fe ^ ovl_chk_htransi);

  assert_never #(0,0,"int_vector_fault has been incorrectly formed")
    ovl_vector_fault(hclk, hreset_n, (int_vector_err_fe &
                                      ~((state_fe_i==`CORE_FTCH_PREFETCH_2) |
                                        (state_fe_i==`CORE_FTCH_RST_LR)     |
                                        (state_fe_i==`CORE_FTCH_RST_SP)     |
                                        (state_fe_i==`CORE_FTCH_RST_PC))));

  assert_never #(0,0,"Interrupt state should not issue a branch request")
    ovl_instr_br_req(hclk, hreset_n,
                     (state_fe_i==`CORE_FTCH_PREFETCH_1) & instr_br_req_de_i);

  assert_never #(0,0,"instr_valid_de asserted when etm_fifofull is asserted")
    ovl_etm_stall_check(hclk, hreset_n, etm_stall_i & instr_valid_de_i);

  assert_never_unknown #(0,13,0,"X propagate protect default statements")
    ovl_default_x(hclk, hreset_n, 1'b1,
                  {state_fe_i, nxt_fe_buf_ctrl,
                   ahb_req_htransi, mtx_ahb_hreadyi_i, fe_haddracci,
                   dbg_halted_i, ahb_htransi_issued_fe});


  assert_implication #(0,0,
      "For a stable waited transfer all control info must remain constant")
    ahb_ctl_const
      (hclk, hreset_n,
       (opt_const_en & ~ovl_hreadyi_d1 & ahb_htransi[1] & ovl_htransi_d1[1]),
       ((ovl_haddri_d1  == ahb_haddri) & (ovl_htransi_d1 == ahb_htransi) &
        (ovl_hsizei_d1  == ahb_hsizei) & (ovl_hproti_d1  == ahb_hproti)));

  assert_implication #(0,0, "Cancel waited NSEQ/SEQ only if error")
    ahb_errm_trans1
      (hclk, hreset_n,
       (opt_const_en & ~ovl_hreadyi_d1 & (|ovl_htransi_d1) &
        (ovl_htransi_d1 != ahb_htransi)),
       ((ahb_htransi == 2'b00) & ovl_hrespi_d1));


  assert_implication #(0,0,"For a SEQ transfer control/addr must be identical")
    ahb_errm_high_addr
      (hclk, hreset_n,
       (ovl_hreadyi_d1 & (ahb_htransi == 2'b11)),
       ((ovl_haddri_d1[31:10] == ahb_haddri[31:10]) &
        (ovl_hsizei_d1        == ahb_hsizei)        &
        (ovl_hproti_d1        == ahb_hproti)));

  assert_implication #(0,0,"A SEQ transfer can only follow a NONSEQ or SEQ")
    ahb_errm_seq
      (hclk, hreset_n,
       (ahb_htransi == 2'b11),
       (ovl_htransi_d1 != 2'b00));

`endif

`include "cm3_dpu_undefs.vp"

endmodule
