module cm3_dpu_lsu
  (// Inputs
   hreset_n, hclk, instr_valid_de_i, instr_lsu_ctl_de_i, instr_immed_de_i,
   instr_lsm_immed_de_i, instr_imm_valid_fast_de_i, instr_ccpass_de_i,
   instr_algn_de_i, instr_ls_allow_pline_de_i, instr_ls_single_adv_de_i,
   instr_lsu_int_ilock_de_i, instr_lsm_baseinlist_de_i, instr_data_size_de_i,
   instr_itblock_ex_i, instr_immed_valid_ex_i, instr_immed_ex_i,
   instr_data_size_ex_i, instr_sign_ex_i, instr_exit_op_ex_i,
   instr_detect_exit_ex_i, instr_atbase_ex_i, psr_ex_i, privileged_i,
   process_sp_ex_i, user_thread_ex_i, nvic_data_be8_i, rf_rd_a_addr_de_i,
   rf_rd_b_addr_de_i, instr_rf_rd_c_addr_de_i, rf_wr_d_addr_de_i,
   rf_rd_a_data_de_i, rf_rd_b_data_de_i, rf_rd_a_data_ex_i, rf_rd_b_data_ex_i,
   mtx_ahb_hreadyd_i, mtx_ahb_haddraccd_i, mtx_ahb_hrespd_i, mpu_hrespd_i,
   mtx_ahb_hrdatad_i, mtx_ahb_exrespd_i, mpu_canceld_i, mpu_addrd_non_buf_i,
   mtx_bm_active_i, fpu_enabled_i, fpu_first_reg_de_i, fpu_str_fwd_de_i,
   fpu_active_ex_i, fpu_wr_data_ex_i, fpu_lazy_start_i, fpu_lazy_stack_op_i,
   fpu_int_cp_fault_i, fpu_flip_stack_i, fpu_stack_ctl_i, fpu_unstack_ctl_i,
   nvic_basepri_i, nvic_fmask_i, nvic_pmask_i, nvic_bfhfnmign_i,
   nvic_int_stk_algn_i, nvic_unalign_trp_i, nvic_int_invoke_i, nvic_int_tail_i,
   nvic_fpccr_i, nvic_fpcar_i, int_first_instr_valid_i, int_isr_ex_i,
   int_lr_exit_code_ex_i, nvic_int_in_nmihf_i, rst_pc_valid_ex_i,
   iflush_pflush_ex_i, nvic_slp_asleep_i, nvic_slp_sleeponexit_i,
   fault_dblfault_ex_i, nvic_dbg_reg_addr_i, nvic_dbg_trans_i,
   nvic_dbg_reg_wr_i, dbg_halt_set_i, dbg_halted_i,
   instr_excl_ex_o, lsu_valid_ex_o, instr_lsu_ctl_ex_o, lsu_state_ex_o,
   lsu_active_ex_o, lsu_sp_wb_ex_o, lsu_stall_ex_o, lsu_stall_late_ex_o,
   lsu_str_buf_imm_ex_o, lsu_flag_wr_ex_o, lsu_psr_ex_o, lsu_it_valid_ex_o,
   lsu_it_ex_o, lsu_nxt_lsm_fpu_ex_o, lsu_lsm_reg_ex_o, lsu_lsm_last_ex_o,
   lsu_lsm_list_valid_ex_o, lsu_stackpush_rdy_ex_o, lsu_exit_code_ex_o,
   lsu_exit_valid_ex_o, lsu_pflush_ex_o, lsu_fix_pc_ex_o,
   lsu_mcyc_invk_msk_ex_o, lsu_ahb_invk_msk_o, lsu_mcyc_ilock_ex_o,
   lsu_local_fault_ex_o, lsu_dbl_fault_ex_o, lsu_fp_ls_rf_addr_ex_o,
   lsu_will_wr_rf_ex_o, lsu_rf_rd_a_en_ex_o, lsu_rf_rd_a_en_fast_ex_o,
   lsu_rf_rd_a_addr_ex_o, lsu_rf_wr_d_en_ex_o, lsu_rf_wr_d_en_fast_ex_o,
   lsu_rf_wr_d_addr_ex_o, lsu_rf_wr_d_data_ex_o, lsu_rf_wr_z_addr_ex_o,
   lsu_rf_wr_z_en_ex_o, lsu_rf_wr_z_data_ex_o, lsu_fpu_rd_data_ex_o,
   lsu_nxt_fpcar_o, lsu_main_sp_en_o, lsu_real_sp_rd_o, lsu_ici_cont_o,
   lsu_stack_start_o, lsu_addrd_non_buf_o, ahb_htransd_o, ahb_hwrited_o,
   ahb_hsized_o, ahb_haddrd_o, ahb_hwdatad_o, ahb_hprotd_o, ahb_exreqd_o,
   dpu_addrd_valid_o, int_entry_o, int_exit_o, int_return_o, int_retisr_o,
   int_stack_op_o, int_unstacking_o, fault_lsu_precise_ex_o, fault_lsu_ex_o,
   dbg_ctrl_wr_ex_o, dbg_wr_isr_o, dbg_ready_o
  );

  parameter      RESET_ALL_REGS = 0;       // Reset all registers if 1
  parameter      EXT_PRESENT    = 0;       // Extensions present if 1
  parameter      FPU_PRESENT    = 0;       // FPU present if 1
  parameter      CONST_AHB_CTRL = 0;       // Constant AHB control info if 1

  input          hreset_n;
  input          hclk;
  input          instr_valid_de_i;         // Instruction in decode is valid
  input [15:0]   instr_lsu_ctl_de_i;       // LSU unit control (De)
  input [31:0]   instr_immed_de_i;         // Immediate value
  input [31:0]   instr_lsm_immed_de_i;     // LSM immediate value
  input          instr_imm_valid_fast_de_i;// Immediate valid (fast)
  input          instr_ccpass_de_i;        // Condition code pass
  input          instr_algn_de_i;          // Align data
  input          instr_ls_allow_pline_de_i;// Allow LSS to overlap
  input          instr_ls_single_adv_de_i; // Allow fail/nop underneath LS
  input          instr_lsu_int_ilock_de_i; // Interrupt interlock
  input          instr_lsm_baseinlist_de_i;// LSM has base in list (De)
  input [1:0]    instr_data_size_de_i;     // Data size
  input          instr_itblock_ex_i;       // In IT block
  input          instr_immed_valid_ex_i;   // Immediate valid
  input [31:0]   instr_immed_ex_i;         // Immediate value
  input [1:0]    instr_data_size_ex_i;     // Data size
  input          instr_sign_ex_i;          // Signed operation or sign extend
  input          instr_exit_op_ex_i;       // Valid exit instruction in execute
  input          instr_detect_exit_ex_i;   // ISR exit detected
  input          instr_atbase_ex_i;        // In thread
  input [31:0]   psr_ex_i;                 // PSR
  input          privileged_i;             // Privileged (vs user) mode
  input [1:0]    process_sp_ex_i;          // Current SP is process (vs main)
  input          user_thread_ex_i;         // Thread is (or will be) user mode
  input          nvic_data_be8_i;          // Data is BE8
  input [3:0]    rf_rd_a_addr_de_i;        // RF A port read address
  input [3:0]    rf_rd_b_addr_de_i;        // RF B port read address
  input [3:0]    instr_rf_rd_c_addr_de_i;  // RF secondary read address
  input [3:0]    rf_wr_d_addr_de_i;        // RF D port write address
  input [31:0]   rf_rd_a_data_de_i;        // RF A port read data
  input [31:0]   rf_rd_b_data_de_i;        // RF B port read data
  input [31:0]   rf_rd_a_data_ex_i;        // RF A port read data
  input [31:0]   rf_rd_b_data_ex_i;        // RF B port read data
  input          mtx_ahb_hreadyd_i;        // AHB D-side HREADY
  input          mtx_ahb_haddraccd_i;      // AHB D-side HADDRACC
  input [1:0]    mtx_ahb_hrespd_i;         // AHB D-side HRESP
  input          mpu_hrespd_i;             // MPU D-side HRESP
  input [31:0]   mtx_ahb_hrdatad_i;        // AHB D-side HRDATA
  input          mtx_ahb_exrespd_i;        // AHB D-side EXRESP
  input          mpu_canceld_i;            // MPU D-side cancel
  input          mpu_addrd_non_buf_i;      // MPU D-side addr non-bufferable
  input          mtx_bm_active_i;          // Bus-matrix active
  input          fpu_enabled_i;            // FPU enabled
  input [4:0]    fpu_first_reg_de_i;       // FPU first reg for LSM
  input [1:0]    fpu_str_fwd_de_i;         // FPU store forwarding control
  input          fpu_active_ex_i;          // Current CONTROL[2]
  input   [31:0] fpu_wr_data_ex_i;         // Data from FPU
  input          fpu_lazy_start_i;         // Lazy stack start
  input          fpu_lazy_stack_op_i;      // Lazy stacking in progress
  input          fpu_int_cp_fault_i;       // FPU interrupt NOCP fault
  input          fpu_flip_stack_i;         // When stacking FPU flip stack order
  input [1:0]    fpu_stack_ctl_i;          // FPU stack control
  input [1:0]    fpu_unstack_ctl_i;        // FPU unstack control
  input [7:0]    nvic_basepri_i;           // BASEPRI value
  input          nvic_fmask_i;             // FAULTMASK value
  input          nvic_pmask_i;             // PRIMASK value
  input          nvic_bfhfnmign_i;         // Ignore BF in NMI and HF ISR
  input          nvic_int_stk_algn_i;      // Align stack to 8-bytes
  input          nvic_unalign_trp_i;       // Trap unaligned accesses
  input          nvic_int_invoke_i;        // Interrupt stack request
  input          nvic_int_tail_i;          // Interrupt tail-chain request
  input [31:0]   nvic_fpccr_i;             // Floating-point context control reg
  input [31:0]   nvic_fpcar_i;             // Floating-point context address reg
  input          int_first_instr_valid_i;  // 1st instr ready for entry/return
  input [8:0]    int_isr_ex_i;             // Current ISR number
  input [4:0]    int_lr_exit_code_ex_i;    // Generated exit code for LR
  input [2:0]    nvic_int_in_nmihf_i;      // In NMI/HF ISR
  input          rst_pc_valid_ex_i;        // PC reset vector fetch complete
  input          iflush_pflush_ex_i;       // IFLUSH pipe flush
  input          nvic_slp_asleep_i;        // DPU asleep
  input          nvic_slp_sleeponexit_i;   // Sleep-on-exit request
  input          fault_dblfault_ex_i;      // Double fault detected
  input [6:0]    nvic_dbg_reg_addr_i;      // Debug register transfer reg num
  input          nvic_dbg_trans_i;         // Debug register transfer request
  input          nvic_dbg_reg_wr_i;        // Debug register transfer W/nR
  input          dbg_halt_set_i;           // Core halt set
  input          dbg_halted_i;             // Core halted

  output         instr_excl_ex_o;          // Instruction is an exclusive
  output         lsu_valid_ex_o;           // LSU op is valid
  output [15:0]  instr_lsu_ctl_ex_o;       // LSU unit control (Ex)
  output [5:0]   lsu_state_ex_o;           // LSU state
  output         lsu_active_ex_o;          // LSU is active
  output         lsu_sp_wb_ex_o;           // LSU SP WB
  output         lsu_stall_ex_o;           // LSU stall pipe
  output         lsu_stall_late_ex_o;      // LSU very late stall
  output         lsu_str_buf_imm_ex_o;     // Store status is an immediate
  output         lsu_flag_wr_ex_o;         // LSU flag result write enable
  output [31:0]  lsu_psr_ex_o;             // LSU PSR write value
  output         lsu_it_valid_ex_o;        // LSU write to IT is valid
  output [7:0]   lsu_it_ex_o;              // LSU IT bits to write
  output         lsu_lsm_list_valid_ex_o;  // LSM list non-empty
  output [4:0]   lsu_nxt_lsm_fpu_ex_o;     // Next LSM register number for FPU
  output [4:0]   lsu_lsm_reg_ex_o;         // Current LSM register number
  output         lsu_lsm_last_ex_o;        // Last register in LSM list
  output         lsu_stackpush_rdy_ex_o;   // LSU ready for pushing stack
  output [4:0]   lsu_exit_code_ex_o;       // LSU ISR exit code
  output         lsu_exit_valid_ex_o;      // LSU operation is an ISR exit
  output         lsu_pflush_ex_o;          // LSU pipe flush
  output         lsu_fix_pc_ex_o;          // LSU requires PC fix
  output         lsu_mcyc_invk_msk_ex_o;   // LSU multi-cycle op interrupt mask
  output         lsu_ahb_invk_msk_o;       // LSU mask interrupts for const AHB
  output         lsu_mcyc_ilock_ex_o;      // LSU multi-cycle op interlock
  output [1:0]   lsu_local_fault_ex_o;     // LSU detection of faults
  output         lsu_dbl_fault_ex_o;       // LSU double fault detected
  output [4:0]   lsu_fp_ls_rf_addr_ex_o;   // FPU RF address
  output         lsu_will_wr_rf_ex_o;      // LSU will write to RF imminently
  output         lsu_rf_rd_a_en_ex_o;      // LSU RF A port read enable
  output         lsu_rf_rd_a_en_fast_ex_o; // LSU RF A port read enable (fast)
  output [3:0]   lsu_rf_rd_a_addr_ex_o;    // LSU RF A port read address
  output         lsu_rf_wr_d_en_ex_o;      // LSU RF D port write enable
  output         lsu_rf_wr_d_en_fast_ex_o; // LSU RF D port write enable (fast)
  output [3:0]   lsu_rf_wr_d_addr_ex_o;    // LSU RF D port write address
  output [31:0]  lsu_rf_wr_d_data_ex_o;    // LSU RF D port write data
  output [3:0]   lsu_rf_wr_z_addr_ex_o;    // LSU RF Z port write address
  output         lsu_rf_wr_z_en_ex_o;      // LSU RF Z port write enable
  output [31:0]  lsu_rf_wr_z_data_ex_o;    // LSU RF Z port write data
  output [31:0]  lsu_fpu_rd_data_ex_o;     // LSU data to FPU
  output [31:0]  lsu_nxt_fpcar_o;          // Write value for FPCAR
  output         lsu_main_sp_en_o;         // Force r13 to be main SP
  output         lsu_real_sp_rd_o;         // Force r13 read to be real SP
  output [3:0]   lsu_ici_cont_o;           // ICI bits from stack
  output         lsu_stack_start_o;        // LSU stack op start
  output         lsu_addrd_non_buf_o;      // AHB D-side address non-bufferable
  output [1:0]   ahb_htransd_o;            // AHB D-side HTRANS
  output         ahb_hwrited_o;            // AHB D-side HWRITE
  output [1:0]   ahb_hsized_o;             // AHB D-side HSIZE
  output [31:0]  ahb_haddrd_o;             // AHB D-side HADDR
  output [31:0]  ahb_hwdatad_o;            // AHB D-side HWDATA
  output [1:0]   ahb_hprotd_o;             // AHB D-side HPROT
  output         ahb_exreqd_o;             // AHB D-side EXREQ
  output         dpu_addrd_valid_o;        // Address valid for FAR use
  output         int_entry_o;              // ISR entry
  output         int_exit_o;               // ISR exit
  output         int_return_o;             // Unstacking complete
  output [8:0]   int_retisr_o;             // Returned to ISR number
  output         int_stack_op_o;           // LSU is stacking/unstacking
  output         int_unstacking_o;         // LSU is unstacking
  output         fault_lsu_precise_ex_o;   // Precise LSU fault
  output [10:0]  fault_lsu_ex_o;           // Collection of LSU faults
  output         dbg_ctrl_wr_ex_o;         // Debug write to special control
  output         dbg_wr_isr_o;             // Debug write to ISR number
  output         dbg_ready_o;              // Debug register transfer ready

  wire           instr_excl_ex;            // Instruction is an exclusive
  wire [15:0]    instr_lsu_ctl_ex;         // LSU unit control (Ex)
  wire           instr_lsm_baseinlist_ex;  // LSM has base in list
  wire           lsu_valid_ex;             // LSU op is valid
  wire [5:0]     lsu_state_ex;             // LSU state
  wire           lsu_active_ex;            // LSU is active
  wire           lsu_sp_wb_ex;             // LSU SP WB
  wire           lsu_stall_ex;             // LSU stall pipe
  wire           lsu_stall_late_ex;        // LSU very late stall
  wire [1:0]     lsu_lsm_base_status_ex;   // LSM base in list status
  wire           lsm_fpu_brestore_ex;      // Restore base for FP LSM
  wire [4:0]     lsu_nxt_lsm_fpu_ex;       // Next LSM register number for FPU
  wire [4:0]     lsu_lsm_reg_ex;           // Current LSM register number
  wire           lsu_lsm_last_ex;          // Last register in LSM list
  wire [4:0]     lsm_list_length_de;       // LSM register list length (De)
  wire [5:0]     lsm_list_length_ex;       // LSM register list length (Ex)
  wire [5:0]     lsm_ici_remaining_ex;     // ICI LSM number of remaining reg
  wire           lsu_ld_excl_ex;           // LDREX in execute
  wire [1:0]     lsu_st_excl_ex;           // STREX in execute
  wire           lsu_pline_valid_de;       // Allow LSS to overlap
  wire           lsu_st_imm_nonbuf_ex;     // Non bufferable store immediate
  wire           lsu_excl_mon_ex;          // Exclusive monitor status
  wire           lsu_stack_algn_64_ex;     // Align stack at 8-byte boundary
  wire           lsu_unalgn_ex;            // Access is unaligned
  wire           lsu_flag_wr_ex;           // LSU flag result write enable
  wire [31:0]    lsu_psr_ex;               // LSU PSR value
  wire           lsu_it_valid_ex;          // LSU write to IT is valid
  wire [7:0]     lsu_it_ex;                // LSU IT bits to write
  wire           lsu_lsm_list_valid_ex;    // LSM list non-empty
  wire           lsu_stackpush_rdy_ex;     // LSU ready for pushing stack
  wire [4:0]     lsu_exit_code_ex;         // LSU ISR exit code
  wire           lsu_exit_valid_ex;        // LSU operation is an ISR exit
  wire [31:0]    lsu_nxt_fpcar;            // Next FPCAR
  wire           lsu_pflush_ex;            // LSU pipe flush
  wire           lsu_fix_pc_ex;            // LSU requires PC fix
  wire           lsu_mcyc_invk_msk_ex;     // Mask intinvoke for multi-cycle op
  wire           lsu_ahb_invk_msk;         // Mask interrupts for const AHB
  wire           lsu_mcyc_ilock_ex;        // Multi-cycle op interlock
  wire           lsu_dbl_fault_ex;         // LSU double fault detected
  wire [4:0]     lsu_fp_ls_rf_addr_ex;     // FPU RF address
  wire [31:0]    lsu_fpu_rd_data_ex;       // Data to FPU
  wire           lsu_will_wr_rf_ex;        // LSU will write to RF imminently
  wire           lsu_rf_rd_a_en_ex;        // LSU RF A port read enable
  wire           lsu_rf_rd_a_en_fast_ex;   // LSU RF A port read enable (fast)
  wire [3:0]     lsu_rf_rd_a_addr_ex;      // LSU RF A port read address
  wire           lsu_rf_wr_d_en_ex;        // LSU RF D port write enable
  wire           lsu_rf_wr_d_en_fast_ex;   // LSU RF D port write enable (fast)
  wire [3:0]     lsu_rf_wr_d_addr_ex;      // LSU RF D port write address
  wire [31:0]    lsu_rf_wr_d_data_ex;      // LSU RF D port write data
  wire [3:0]     lsu_rf_wr_z_addr_ex;      // LSU RF Z port write address
  wire           lsu_rf_wr_z_en_ex;        // LSU RF Z port write enable
  wire [31:0]    lsu_rf_wr_z_data_ex;      // LSU RF Z port write data
  wire [3:0]     rf_rd_a_addr_ex;          // RF A port read address
  wire [3:0]     rf_rd_b_addr_ex;          // RF B port read address
  wire [3:0]     instr_rf_rd_c_addr_ex;    // RF secondary read address
  wire [3:0]     rf_wr_d_addr_ex;          // RF D port write address
  wire           lsu_main_sp_en;           // Force r13 to be main SP
  wire           lsu_real_sp_rd;           // Force r13 read to be real SP
  wire [3:0]     lsu_ici_cont;             // ICI bits from stack
  wire           lsu_stack_start;          // LSU stack op start
  wire           lsu_stack_end;            // LSU stack op complete
  wire           lsu_addrd_non_buf;        // AHB D-side address non-bufferable
  wire           lsu_addracc_wait_reg;     // Waiting for address acceptance
  wire           str_buf_imm_ex;           // Store status is an immediate
  wire           ahb_const_unstk;          // Keep trans for unstack
  wire [1:0]     ahb_htransd;              // AHB D-side HTRANS
  wire           ahb_hwrited;              // AHB D-side HWRITE
  wire [1:0]     ahb_hsized;               // AHB D-side HSIZE
  wire [31:0]    ahb_haddrd;               // AHB D-side HADDR
  wire [31:0]    ahb_hwdatad;              // AHB D-side HWDATA
  wire [1:0]     ahb_hprotd;               // AHB D-side HPROT
  wire           ahb_exreqd;               // AHB D-side EXREQ
  wire           dpu_addrd_valid;          // Address valid for FAR use
  wire           int_stack_op;             // LSU is stacking/unstacking
  wire           int_unstacking;           // LSU is unstacking
  wire           int_entry;                // ISR entry
  wire           int_exit;                 // ISR exit
  wire           int_return;               // Unstacking complete
  wire [8:0]     int_retisr;               // Returned to ISR number
  wire [1:0]     fault_lsu_local_ex;       // LSU detection of faults
  wire           fault_lsu_precise_ex;     // Precise LSU fault
  wire [10:0]    fault_lsu_ex;             // Collection of LSU faults
  wire           dbg_ctrl_wr_ex;           // Debug write to special control
  wire           dbg_wr_isr;               // Debug write to ISR number
  wire           dbg_ready;                // Debug register transfer ready

  cm3_dpu_lsu_ctl #(RESET_ALL_REGS, EXT_PRESENT, FPU_PRESENT)
    u_cm3_dpu_lsu_ctl
      (// Inputs
       .hreset_n                   (hreset_n),
       .hclk                       (hclk),
       .instr_lsu_ctl_de_i         (instr_lsu_ctl_de_i),
       .instr_valid_de_i           (instr_valid_de_i),
       .instr_ccpass_de_i          (instr_ccpass_de_i),
       .instr_lsm_baseinlist_de_i  (instr_lsm_baseinlist_de_i),
       .instr_ls_allow_pline_de_i  (instr_ls_allow_pline_de_i),
       .instr_ls_single_adv_de_i   (instr_ls_single_adv_de_i),
       .instr_lsu_int_ilock_de_i   (instr_lsu_int_ilock_de_i),
       .instr_lsm_immed_de_i       (instr_lsm_immed_de_i),
       .rf_rd_a_addr_de_i          (rf_rd_a_addr_de_i),
       .rf_rd_b_addr_de_i          (rf_rd_b_addr_de_i),
       .instr_rf_rd_c_addr_de_i    (instr_rf_rd_c_addr_de_i),
       .rf_wr_d_addr_de_i          (rf_wr_d_addr_de_i),
       .instr_itblock_ex_i         (instr_itblock_ex_i),
       .instr_immed_valid_ex_i     (instr_immed_valid_ex_i),
       .instr_immed_ex_i           (instr_immed_ex_i),
       .instr_atbase_ex_i          (instr_atbase_ex_i),
       .instr_detect_exit_ex_i     (instr_detect_exit_ex_i),
       .psr_ex_i                   (psr_ex_i),
       .lsu_ld_excl_ex_i           (lsu_ld_excl_ex),
       .lsu_st_excl_ex_i           (lsu_st_excl_ex),
       .lsu_unalgn_ex_i            (lsu_unalgn_ex),
       .lsu_addracc_wait_reg_i     (lsu_addracc_wait_reg),
       .lsu_stack_algn_64_ex_i     (lsu_stack_algn_64_ex),
       .lsu_psr_ex_i               (lsu_psr_ex),
       .lsu_exit_valid_ex_i        (lsu_exit_valid_ex),
       .ahb_const_unstk_i          (ahb_const_unstk),
       .rst_pc_valid_ex_i          (rst_pc_valid_ex_i),
       .iflush_pflush_ex_i         (iflush_pflush_ex_i),
       .mtx_ahb_haddraccd_i        (mtx_ahb_haddraccd_i),
       .mtx_ahb_hreadyd_i          (mtx_ahb_hreadyd_i),
       .mpu_canceld_i              (mpu_canceld_i),
       .mpu_addrd_non_buf_i        (mpu_addrd_non_buf_i),
       .mtx_bm_active_i            (mtx_bm_active_i),
       .fpu_enabled_i              (fpu_enabled_i),
       .fpu_first_reg_de_i         (fpu_first_reg_de_i),
       .fpu_lazy_start_i           (fpu_lazy_start_i),
       .fpu_int_cp_fault_i         (fpu_int_cp_fault_i),
       .fpu_stack_ctl_i            (fpu_stack_ctl_i),
       .fpu_unstack_ctl_i          (fpu_unstack_ctl_i),
       .fpu_lazy_stack_op_i        (fpu_lazy_stack_op_i),
       .nvic_int_invoke_i          (nvic_int_invoke_i),
       .nvic_int_tail_i            (nvic_int_tail_i),
       .int_first_instr_valid_i    (int_first_instr_valid_i),
       .nvic_int_in_nmihf_i        (nvic_int_in_nmihf_i),
       .nvic_fpccr_i               (nvic_fpccr_i),
       .nvic_slp_asleep_i          (nvic_slp_asleep_i),
       .nvic_slp_sleeponexit_i     (nvic_slp_sleeponexit_i),
       .fault_lsu_local_ex_i       (fault_lsu_local_ex),
       .fault_dblfault_ex_i        (fault_dblfault_ex_i),
       .dbg_halt_set_i             (dbg_halt_set_i),
       .dbg_halted_i               (dbg_halted_i),
       .nvic_dbg_reg_addr_i        (nvic_dbg_reg_addr_i),
       .nvic_dbg_reg_wr_i          (nvic_dbg_reg_wr_i),
       .nvic_dbg_trans_i           (nvic_dbg_trans_i),
       .instr_excl_ex_o            (instr_excl_ex),
       .instr_lsu_ctl_ex_o         (instr_lsu_ctl_ex),
       .instr_lsm_baseinlist_ex_o  (instr_lsm_baseinlist_ex),
       .lsu_valid_ex_o             (lsu_valid_ex),
       .lsu_state_ex_o             (lsu_state_ex),
       .lsu_nxt_lsm_fpu_ex_o       (lsu_nxt_lsm_fpu_ex),
       .lsu_lsm_reg_ex_o           (lsu_lsm_reg_ex),
       .lsu_lsm_list_valid_ex_o    (lsu_lsm_list_valid_ex),
       .lsu_lsm_last_ex_o          (lsu_lsm_last_ex),
       .lsm_list_length_de_o       (lsm_list_length_de),
       .lsm_list_length_ex_o       (lsm_list_length_ex),
       .lsm_ici_remaining_ex_o     (lsm_ici_remaining_ex),
       .lsu_stall_ex_o             (lsu_stall_ex),
       .lsu_stall_late_ex_o        (lsu_stall_late_ex),
       .lsu_excl_mon_ex_o          (lsu_excl_mon_ex),
       .lsu_pline_valid_de_o       (lsu_pline_valid_de),
       .lsu_st_imm_nonbuf_ex_o     (lsu_st_imm_nonbuf_ex),
       .lsu_lsm_base_status_ex_o   (lsu_lsm_base_status_ex),
       .lsm_fpu_brestore_ex_o      (lsm_fpu_brestore_ex),
       .lsu_dbl_fault_ex_o         (lsu_dbl_fault_ex),
       .lsu_stackpush_rdy_ex_o     (lsu_stackpush_rdy_ex),
       .lsu_pflush_ex_o            (lsu_pflush_ex),
       .lsu_it_valid_ex_o          (lsu_it_valid_ex),
       .lsu_it_ex_o                (lsu_it_ex),
       .lsu_fix_pc_ex_o            (lsu_fix_pc_ex),
       .lsu_mcyc_invk_msk_ex_o     (lsu_mcyc_invk_msk_ex),
       .lsu_mcyc_ilock_ex_o        (lsu_mcyc_ilock_ex),
       .lsu_fp_ls_rf_addr_ex_o     (lsu_fp_ls_rf_addr_ex),
       .lsu_rf_rd_a_en_fast_ex_o   (lsu_rf_rd_a_en_fast_ex),
       .lsu_rf_rd_a_en_ex_o        (lsu_rf_rd_a_en_ex),
       .lsu_rf_rd_a_addr_ex_o      (lsu_rf_rd_a_addr_ex),
       .lsu_rf_wr_d_en_ex_o        (lsu_rf_wr_d_en_ex),
       .lsu_rf_wr_d_en_fast_ex_o   (lsu_rf_wr_d_en_fast_ex),
       .lsu_rf_wr_d_addr_ex_o      (lsu_rf_wr_d_addr_ex),
       .lsu_rf_wr_z_addr_ex_o      (lsu_rf_wr_z_addr_ex),
       .lsu_rf_wr_z_en_ex_o        (lsu_rf_wr_z_en_ex),
       .lsu_will_wr_rf_ex_o        (lsu_will_wr_rf_ex),
       .lsu_ici_cont_o             (lsu_ici_cont),
       .lsu_stack_start_o          (lsu_stack_start),
       .lsu_stack_end_o            (lsu_stack_end),
       .lsu_addrd_non_buf_o        (lsu_addrd_non_buf),
       .lsu_main_sp_en_o           (lsu_main_sp_en),
       .lsu_real_sp_rd_o           (lsu_real_sp_rd),
       .rf_rd_a_addr_ex_o          (rf_rd_a_addr_ex),
       .rf_rd_b_addr_ex_o          (rf_rd_b_addr_ex),
       .instr_rf_rd_c_addr_ex_o    (instr_rf_rd_c_addr_ex),
       .rf_wr_d_addr_ex_o          (rf_wr_d_addr_ex),
       .int_entry_o                (int_entry),
       .int_exit_o                 (int_exit),
       .int_return_o               (int_return),
       .int_stack_op_o             (int_stack_op),
       .int_unstacking_o           (int_unstacking),
       .dbg_ready_o                (dbg_ready)
      );

  cm3_dpu_lsu_ahbintf #(RESET_ALL_REGS, EXT_PRESENT, FPU_PRESENT, CONST_AHB_CTRL)
    u_cm3_dpu_lsu_ahbintf
      (// Inputs
       .hreset_n                   (hreset_n),
       .hclk                       (hclk),
       .instr_lsu_ctl_de_i         (instr_lsu_ctl_de_i),
       .instr_ccpass_de_i          (instr_ccpass_de_i),
       .instr_immed_de_i           (instr_immed_de_i),
       .instr_imm_valid_fast_de_i  (instr_imm_valid_fast_de_i),
       .instr_algn_de_i            (instr_algn_de_i),
       .instr_ls_single_adv_de_i   (instr_ls_single_adv_de_i),
       .instr_data_size_de_i       (instr_data_size_de_i),
       .instr_lsu_ctl_ex_i         (instr_lsu_ctl_ex),
       .instr_immed_ex_i           (instr_immed_ex_i),
       .instr_immed_valid_ex_i     (instr_immed_valid_ex_i),
       .instr_itblock_ex_i         (instr_itblock_ex_i),
       .instr_lsm_baseinlist_ex_i  (instr_lsm_baseinlist_ex),
       .instr_data_size_ex_i       (instr_data_size_ex_i),
       .instr_sign_ex_i            (instr_sign_ex_i),
       .instr_atbase_ex_i          (instr_atbase_ex_i),
       .instr_exit_op_ex_i         (instr_exit_op_ex_i),
       .instr_detect_exit_ex_i     (instr_detect_exit_ex_i),
       .psr_ex_i                   (psr_ex_i),
       .lsu_valid_ex_i             (lsu_valid_ex),
       .lsu_state_ex_i             (lsu_state_ex),
       .lsu_lsm_reg_ex_i           (lsu_lsm_reg_ex),
       .lsu_lsm_last_ex_i          (lsu_lsm_last_ex),
       .lsm_list_length_de_i       (lsm_list_length_de),
       .lsm_list_length_ex_i       (lsm_list_length_ex),
       .lsm_ici_remaining_ex_i     (lsm_ici_remaining_ex),
       .lsu_lsm_base_status_ex_i   (lsu_lsm_base_status_ex),
       .lsm_fpu_brestore_ex_i      (lsm_fpu_brestore_ex),
       .lsu_stack_start_i          (lsu_stack_start),
       .lsu_stack_end_i            (lsu_stack_end),
       .lsu_excl_mon_ex_i          (lsu_excl_mon_ex),
       .lsu_pline_valid_de_i       (lsu_pline_valid_de),
       .lsu_st_imm_nonbuf_ex_i     (lsu_st_imm_nonbuf_ex),
       .lsu_will_wr_rf_ex_i        (lsu_will_wr_rf_ex),
       .lsu_rf_wr_d_addr_ex_i      (lsu_rf_wr_d_addr_ex),
       .lsu_rf_wr_z_en_ex_i        (lsu_rf_wr_z_en_ex),
       .lsu_rf_wr_z_addr_ex_i      (lsu_rf_wr_z_addr_ex),
       .rf_rd_a_addr_de_i          (rf_rd_a_addr_de_i),
       .rf_rd_b_addr_de_i          (rf_rd_b_addr_de_i),
       .rf_wr_d_addr_de_i          (rf_wr_d_addr_de_i),
       .rf_rd_a_data_de_i          (rf_rd_a_data_de_i),
       .rf_rd_b_data_de_i          (rf_rd_b_data_de_i),
       .rf_rd_a_addr_ex_i          (rf_rd_a_addr_ex),
       .rf_rd_b_addr_ex_i          (rf_rd_b_addr_ex),
       .instr_rf_rd_c_addr_ex_i    (instr_rf_rd_c_addr_ex),
       .rf_wr_d_addr_ex_i          (rf_wr_d_addr_ex),
       .rf_rd_a_data_ex_i          (rf_rd_a_data_ex_i),
       .rf_rd_b_data_ex_i          (rf_rd_b_data_ex_i),
       .mtx_ahb_hreadyd_i          (mtx_ahb_hreadyd_i),
       .mtx_ahb_haddraccd_i        (mtx_ahb_haddraccd_i),
       .mtx_ahb_hrespd_i           (mtx_ahb_hrespd_i),
       .mpu_hrespd_i               (mpu_hrespd_i),
       .mtx_ahb_hrdatad_i          (mtx_ahb_hrdatad_i),
       .mtx_ahb_exrespd_i          (mtx_ahb_exrespd_i),
       .mpu_canceld_i              (mpu_canceld_i),
       .mpu_addrd_non_buf_i        (mpu_addrd_non_buf_i),
       .fpu_enabled_i              (fpu_enabled_i),
       .fpu_str_fwd_de_i           (fpu_str_fwd_de_i),
       .fpu_wr_data_ex_i           (fpu_wr_data_ex_i),
       .fpu_active_ex_i            (fpu_active_ex_i),
       .fpu_flip_stack_i           (fpu_flip_stack_i),
       .fpu_stack_ctl_i            (fpu_stack_ctl_i),
       .fpu_unstack_ctl_i          (fpu_unstack_ctl_i),
       .fpu_lazy_start_i           (fpu_lazy_start_i),
       .fpu_lazy_stack_op_i        (fpu_lazy_stack_op_i),
       .fpu_int_cp_fault_i         (fpu_int_cp_fault_i),
       .nvic_data_be8_i            (nvic_data_be8_i),
       .rst_pc_valid_ex_i          (rst_pc_valid_ex_i),
       .nvic_int_invoke_i          (nvic_int_invoke_i),
       .nvic_int_tail_i            (nvic_int_tail_i),
       .int_isr_ex_i               (int_isr_ex_i),
       .int_stack_op_i             (int_stack_op),
       .nvic_int_in_nmihf_i        (nvic_int_in_nmihf_i),
       .int_lr_exit_code_ex_i      (int_lr_exit_code_ex_i),
       .process_sp_ex_i            (process_sp_ex_i),
       .user_thread_ex_i           (user_thread_ex_i),
       .privileged_i               (privileged_i),
       .nvic_basepri_i             (nvic_basepri_i),
       .nvic_fmask_i               (nvic_fmask_i),
       .nvic_pmask_i               (nvic_pmask_i),
       .nvic_bfhfnmign_i           (nvic_bfhfnmign_i),
       .nvic_int_stk_algn_i        (nvic_int_stk_algn_i),
       .nvic_unalign_trp_i         (nvic_unalign_trp_i),
       .nvic_fpcar_i               (nvic_fpcar_i),
       .nvic_fpccr_i               (nvic_fpccr_i),
       .nvic_slp_asleep_i          (nvic_slp_asleep_i),
       .nvic_slp_sleeponexit_i     (nvic_slp_sleeponexit_i),
       .dbg_halted_i               (dbg_halted_i),
       .nvic_dbg_reg_addr_i        (nvic_dbg_reg_addr_i),
       .nvic_dbg_reg_wr_i          (nvic_dbg_reg_wr_i),
       .lsu_active_ex_o            (lsu_active_ex),
       .lsu_sp_wb_ex_o             (lsu_sp_wb_ex),
       .lsu_flag_wr_ex_o           (lsu_flag_wr_ex),
       .lsu_psr_ex_o               (lsu_psr_ex),
       .lsu_unalgn_ex_o            (lsu_unalgn_ex),
       .lsu_stack_algn_64_ex_o     (lsu_stack_algn_64_ex),
       .lsu_ld_excl_ex_o           (lsu_ld_excl_ex),
       .lsu_st_excl_ex_o           (lsu_st_excl_ex),
       .lsu_exit_code_ex_o         (lsu_exit_code_ex),
       .lsu_exit_valid_ex_o        (lsu_exit_valid_ex),
       .lsu_rf_wr_d_data_ex_o      (lsu_rf_wr_d_data_ex),
       .lsu_rf_wr_z_data_ex_o      (lsu_rf_wr_z_data_ex),
       .lsu_fpu_rd_data_ex_o       (lsu_fpu_rd_data_ex),
       .lsu_nxt_fpcar_o            (lsu_nxt_fpcar),
       .lsu_addracc_wait_reg_o     (lsu_addracc_wait_reg),
       .lsu_ahb_invk_msk_o         (lsu_ahb_invk_msk),
       .str_buf_imm_ex_o           (str_buf_imm_ex),
       .ahb_const_unstk_o          (ahb_const_unstk),
       .ahb_htransd_o              (ahb_htransd),
       .ahb_hwrited_o              (ahb_hwrited),
       .ahb_hsized_o               (ahb_hsized),
       .ahb_haddrd_o               (ahb_haddrd),
       .ahb_hprotd_o               (ahb_hprotd),
       .ahb_hwdatad_o              (ahb_hwdatad),
       .ahb_exreqd_o               (ahb_exreqd),
       .dpu_addrd_valid_o          (dpu_addrd_valid),
       .int_retisr_o               (int_retisr),
       .fault_lsu_local_ex_o       (fault_lsu_local_ex),
       .fault_lsu_precise_ex_o     (fault_lsu_precise_ex),
       .fault_lsu_ex_o             (fault_lsu_ex),
       .dbg_ctrl_wr_ex_o           (dbg_ctrl_wr_ex),
       .dbg_wr_isr_o               (dbg_wr_isr));

  assign instr_excl_ex_o          = instr_excl_ex;
  assign lsu_valid_ex_o           = lsu_valid_ex;
  assign instr_lsu_ctl_ex_o       = instr_lsu_ctl_ex;
  assign lsu_state_ex_o           = lsu_state_ex;
  assign lsu_active_ex_o          = lsu_active_ex;
  assign lsu_sp_wb_ex_o           = lsu_sp_wb_ex;
  assign lsu_stall_ex_o           = lsu_stall_ex;
  assign lsu_stall_late_ex_o      = lsu_stall_late_ex;
  assign lsu_str_buf_imm_ex_o     = str_buf_imm_ex;
  assign lsu_flag_wr_ex_o         = lsu_flag_wr_ex;
  assign lsu_psr_ex_o             = lsu_psr_ex;
  assign lsu_it_valid_ex_o        = lsu_it_valid_ex;
  assign lsu_it_ex_o              = lsu_it_ex;
  assign lsu_nxt_lsm_fpu_ex_o     = lsu_nxt_lsm_fpu_ex;
  assign lsu_lsm_reg_ex_o         = lsu_lsm_reg_ex;
  assign lsu_lsm_last_ex_o        = lsu_lsm_last_ex;
  assign lsu_lsm_list_valid_ex_o  = lsu_lsm_list_valid_ex;
  assign lsu_stackpush_rdy_ex_o   = lsu_stackpush_rdy_ex;
  assign lsu_exit_code_ex_o       = lsu_exit_code_ex;
  assign lsu_exit_valid_ex_o      = lsu_exit_valid_ex;
  assign lsu_pflush_ex_o          = lsu_pflush_ex;
  assign lsu_fix_pc_ex_o          = lsu_fix_pc_ex;
  assign lsu_mcyc_invk_msk_ex_o   = lsu_mcyc_invk_msk_ex;
  assign lsu_ahb_invk_msk_o       = lsu_ahb_invk_msk;
  assign lsu_mcyc_ilock_ex_o      = lsu_mcyc_ilock_ex;
  assign lsu_local_fault_ex_o     = fault_lsu_local_ex;
  assign lsu_dbl_fault_ex_o       = lsu_dbl_fault_ex;
  assign lsu_fp_ls_rf_addr_ex_o   = lsu_fp_ls_rf_addr_ex;
  assign lsu_will_wr_rf_ex_o      = lsu_will_wr_rf_ex;
  assign lsu_rf_rd_a_en_ex_o      = lsu_rf_rd_a_en_ex;
  assign lsu_rf_rd_a_en_fast_ex_o = lsu_rf_rd_a_en_fast_ex;
  assign lsu_rf_rd_a_addr_ex_o    = lsu_rf_rd_a_addr_ex;
  assign lsu_rf_wr_d_en_ex_o      = lsu_rf_wr_d_en_ex;
  assign lsu_rf_wr_d_en_fast_ex_o = lsu_rf_wr_d_en_fast_ex;
  assign lsu_rf_wr_d_addr_ex_o    = lsu_rf_wr_d_addr_ex;
  assign lsu_rf_wr_d_data_ex_o    = lsu_rf_wr_d_data_ex;
  assign lsu_fpu_rd_data_ex_o     = lsu_fpu_rd_data_ex;
  assign lsu_rf_wr_z_addr_ex_o    = lsu_rf_wr_z_addr_ex;
  assign lsu_rf_wr_z_en_ex_o      = lsu_rf_wr_z_en_ex;
  assign lsu_rf_wr_z_data_ex_o    = lsu_rf_wr_z_data_ex;
  assign lsu_nxt_fpcar_o          = lsu_nxt_fpcar;
  assign lsu_main_sp_en_o         = lsu_main_sp_en;
  assign lsu_real_sp_rd_o         = lsu_real_sp_rd;
  assign lsu_ici_cont_o           = lsu_ici_cont;
  assign lsu_stack_start_o        = lsu_stack_start;
  assign lsu_addrd_non_buf_o      = lsu_addrd_non_buf;
  assign ahb_htransd_o            = ahb_htransd;
  assign ahb_hwrited_o            = ahb_hwrited;
  assign ahb_hsized_o             = ahb_hsized;
  assign ahb_haddrd_o             = ahb_haddrd;
  assign ahb_hwdatad_o            = ahb_hwdatad;
  assign ahb_hprotd_o             = ahb_hprotd;
  assign ahb_exreqd_o             = ahb_exreqd;
  assign dpu_addrd_valid_o        = dpu_addrd_valid;
  assign int_entry_o              = int_entry;
  assign int_exit_o               = int_exit;
  assign int_return_o             = int_return;
  assign int_retisr_o             = int_retisr;
  assign int_stack_op_o           = int_stack_op;
  assign int_unstacking_o         = int_unstacking;
  assign fault_lsu_precise_ex_o   = fault_lsu_precise_ex;
  assign fault_lsu_ex_o           = fault_lsu_ex;
  assign dbg_ctrl_wr_ex_o         = dbg_ctrl_wr_ex;
  assign dbg_wr_isr_o             = dbg_wr_isr;
  assign dbg_ready_o              = dbg_ready;

endmodule
