`include "cm3_dpu_defs.vp"

module cm3_dpu_lsu_ahbintf
  (// Inputs
   hreset_n, hclk, instr_lsu_ctl_de_i, instr_ccpass_de_i, instr_immed_de_i,
   instr_imm_valid_fast_de_i, instr_algn_de_i, instr_ls_single_adv_de_i,
   instr_data_size_de_i, instr_lsu_ctl_ex_i, instr_immed_ex_i,
   instr_immed_valid_ex_i, instr_itblock_ex_i, instr_lsm_baseinlist_ex_i,
   instr_data_size_ex_i, instr_sign_ex_i, instr_atbase_ex_i, instr_exit_op_ex_i,
   instr_detect_exit_ex_i, psr_ex_i, lsu_valid_ex_i, lsu_state_ex_i,
   lsu_lsm_reg_ex_i, lsu_lsm_last_ex_i, lsm_list_length_de_i,
   lsm_list_length_ex_i, lsm_ici_remaining_ex_i, lsu_lsm_base_status_ex_i,
   lsm_fpu_brestore_ex_i, lsu_stack_start_i, lsu_stack_end_i, lsu_excl_mon_ex_i,
   lsu_pline_valid_de_i, lsu_st_imm_nonbuf_ex_i, lsu_will_wr_rf_ex_i,
   lsu_rf_wr_d_addr_ex_i, lsu_rf_wr_z_en_ex_i, lsu_rf_wr_z_addr_ex_i,
   rf_rd_a_addr_de_i, rf_rd_b_addr_de_i, rf_wr_d_addr_de_i, rf_rd_a_data_de_i,
   rf_rd_b_data_de_i, rf_rd_a_addr_ex_i, rf_rd_b_addr_ex_i,
   instr_rf_rd_c_addr_ex_i, rf_wr_d_addr_ex_i, rf_rd_a_data_ex_i,
   rf_rd_b_data_ex_i, mtx_ahb_hreadyd_i, mtx_ahb_haddraccd_i, mtx_ahb_hrespd_i,
   mpu_hrespd_i, mtx_ahb_hrdatad_i, mtx_ahb_exrespd_i, mpu_canceld_i,
   mpu_addrd_non_buf_i, fpu_enabled_i, fpu_wr_data_ex_i, fpu_lazy_start_i,
   fpu_flip_stack_i, fpu_stack_ctl_i, fpu_unstack_ctl_i, fpu_lazy_stack_op_i,
   fpu_int_cp_fault_i, fpu_str_fwd_de_i, fpu_active_ex_i, nvic_data_be8_i,
   rst_pc_valid_ex_i, nvic_int_invoke_i, nvic_int_tail_i, int_isr_ex_i,
   int_stack_op_i, nvic_int_in_nmihf_i, int_lr_exit_code_ex_i, process_sp_ex_i,
   user_thread_ex_i, privileged_i, nvic_basepri_i, nvic_fmask_i, nvic_pmask_i,
   nvic_bfhfnmign_i, nvic_int_stk_algn_i, nvic_unalign_trp_i, nvic_fpcar_i,
   nvic_fpccr_i, nvic_slp_asleep_i, nvic_slp_sleeponexit_i, dbg_halted_i,
   nvic_dbg_reg_addr_i, nvic_dbg_reg_wr_i,
   lsu_active_ex_o, lsu_sp_wb_ex_o, lsu_flag_wr_ex_o, lsu_psr_ex_o,
   lsu_unalgn_ex_o, lsu_stack_algn_64_ex_o, lsu_ld_excl_ex_o, lsu_st_excl_ex_o,
   lsu_exit_code_ex_o, lsu_exit_valid_ex_o, lsu_rf_wr_d_data_ex_o,
   lsu_rf_wr_z_data_ex_o, lsu_fpu_rd_data_ex_o, lsu_nxt_fpcar_o,
   lsu_addracc_wait_reg_o, lsu_ahb_invk_msk_o, str_buf_imm_ex_o,
   ahb_const_unstk_o, ahb_htransd_o, ahb_hwrited_o, ahb_hsized_o, ahb_haddrd_o,
   ahb_hprotd_o, ahb_hwdatad_o, ahb_exreqd_o, dpu_addrd_valid_o, int_retisr_o,
   fault_lsu_local_ex_o, fault_lsu_precise_ex_o, fault_lsu_ex_o,
   dbg_ctrl_wr_ex_o, dbg_wr_isr_o
   );

  parameter      RESET_ALL_REGS = 0;       // Reset all registers if 1
  parameter      EXT_PRESENT    = 0;       // Extensions present if 1
  parameter      FPU_PRESENT    = 0;       // FPU present if 1
  parameter      CONST_AHB_CTRL = 0;        // Constant AHB control info if 1

  input          hreset_n;
  input          hclk;
  input [15:0]   instr_lsu_ctl_de_i;       // LSU unit control (De)
  input          instr_ccpass_de_i;        // Condition code pass
  input [31:0]   instr_immed_de_i;         // Immediate value
  input          instr_imm_valid_fast_de_i;// Immediate valid (fast version)
  input          instr_algn_de_i;          // Align data
  input          instr_ls_single_adv_de_i; // Advance LSS to data-phase
  input [1:0]    instr_data_size_de_i;     // Data size
  input [15:0]   instr_lsu_ctl_ex_i;       // LSU unit control (Ex)
  input [31:0]   instr_immed_ex_i;         // Immediate value
  input          instr_immed_valid_ex_i;   // Immediate valid
  input          instr_itblock_ex_i;       // In IT block
  input          instr_lsm_baseinlist_ex_i;// LSM has base in list
  input [1:0]    instr_data_size_ex_i;     // Data size
  input          instr_sign_ex_i;          // Signed operation or sign extend
  input          instr_atbase_ex_i;        // In thread
  input          instr_exit_op_ex_i;       // Valid exit instruction in execute
  input          instr_detect_exit_ex_i;   // ISR exit detected
  input [31:0]   psr_ex_i;                 // PSR
  input          lsu_valid_ex_i;           // LSU op is valid
  input [5:0]    lsu_state_ex_i;           // LSU state
  input [4:0]    lsu_lsm_reg_ex_i;         // Current LSM register number
  input          lsu_lsm_last_ex_i;        // Last register in LSM list
  input [4:0]    lsm_list_length_de_i;     // LSM register list length (De)
  input [5:0]    lsm_list_length_ex_i;     // LSM register list length (Ex)
  input [5:0]    lsm_ici_remaining_ex_i;   // ICI LSM number of remaining reg
  input [1:0]    lsu_lsm_base_status_ex_i; // LSM base in list status
  input          lsm_fpu_brestore_ex_i;    // Restore base for FP LSM
  input          lsu_stack_start_i;        // LSU stack op start
  input          lsu_stack_end_i;          // LSU stack op complete
  input          lsu_excl_mon_ex_i;        // Exclusive monitor status
  input          lsu_pline_valid_de_i;     // Allow LSS to overlap
  input          lsu_st_imm_nonbuf_ex_i;   // Non bufferable store immediate
  input          lsu_will_wr_rf_ex_i;      // LSU will write to RF imminently
  input [3:0]    lsu_rf_wr_d_addr_ex_i;    // LSU RF D port write address
  input          lsu_rf_wr_z_en_ex_i;      // LSU RF Z port write enable
  input [3:0]    lsu_rf_wr_z_addr_ex_i;    // LSU RF Z port write address
  input [3:0]    rf_rd_a_addr_de_i;        // RF A port read address
  input [3:0]    rf_rd_b_addr_de_i;        // RF B port read address
  input [3:0]    rf_wr_d_addr_de_i;        // RF D port write address
  input [31:0]   rf_rd_a_data_de_i;        // RF A port read data
  input [31:0]   rf_rd_b_data_de_i;        // RF B port read data
  input [3:0]    rf_rd_a_addr_ex_i;        // RF A port read address
  input [3:0]    rf_rd_b_addr_ex_i;        // RF B port read address
  input [3:0]    instr_rf_rd_c_addr_ex_i;  // RF secondary read address
  input [3:0]    rf_wr_d_addr_ex_i;        // RF D port write address
  input [31:0]   rf_rd_a_data_ex_i;        // RF A port read data
  input [31:0]   rf_rd_b_data_ex_i;        // RF B port read data
  input          mtx_ahb_hreadyd_i;        // AHB D-side HREADY
  input          mtx_ahb_haddraccd_i;      // AHB D-side HADDRACC
  input [1:0]    mtx_ahb_hrespd_i;         // AHB D-side HRESP
  input          mpu_hrespd_i;             // MPU D-side HRESP
  input [31:0]   mtx_ahb_hrdatad_i;        // AHB D-side HRDATA
  input          mtx_ahb_exrespd_i;        // AHB D-side EXRESP
  input          mpu_canceld_i;            // MPU D-side cancel
  input          mpu_addrd_non_buf_i;      // MPU D-side addr non-bufferable
  input          fpu_enabled_i;            // FPU enabled
  input [31:0]   fpu_wr_data_ex_i;         // Data from FPU
  input [1:0]    fpu_stack_ctl_i;          // [1] Stack FPU frame
  input [1:0]    fpu_unstack_ctl_i;        // [1] Unstack FPU frame
  input          fpu_lazy_start_i;         // Start lazy FPU state storage
  input          fpu_lazy_stack_op_i;      // Lazy FPU storage in progress
  input          fpu_flip_stack_i;         // When stacking FPU, flip stack order
  input          fpu_int_cp_fault_i;       // NOCP fault on stacking
  input [1:0]    fpu_str_fwd_de_i;         // FPU store forwarding control
  input          fpu_active_ex_i;          // FPU context active
  input [31:0]   nvic_fpcar_i;             // Floating-point context address reg
  input [31:0]   nvic_fpccr_i;             // Floating-point context control reg
  input          nvic_data_be8_i;          // Data is BE8
  input          rst_pc_valid_ex_i;        // PC reset vector fetch complete
  input          nvic_int_invoke_i;        // Interrupt stack request
  input          nvic_int_tail_i;          // Interrupt tail-chain request
  input [8:0]    int_isr_ex_i;             // Current ISR number
  input          int_stack_op_i;           // LSU is stacking/unstacking
  input [2:0]    nvic_int_in_nmihf_i;      // In NMI/HF ISR
  input [4:0]    int_lr_exit_code_ex_i;    // Generated exit code for LR
  input [1:0]    process_sp_ex_i;          // Current SP is process (vs main)
  input          user_thread_ex_i;         // Thread is (or will be) user mode
  input          privileged_i;             // Privileged (vs user) mode
  input [7:0]    nvic_basepri_i;           // BASEPRI value
  input          nvic_fmask_i;             // FAULTMASK value
  input          nvic_pmask_i;             // PRIMASK value
  input          nvic_bfhfnmign_i;         // Ignore BF in NMI and HF ISR
  input          nvic_int_stk_algn_i;      // Align stack to 8-bytes
  input          nvic_unalign_trp_i;       // Trap unaligned accesses
  input          nvic_slp_asleep_i;        // Core asleep
  input          nvic_slp_sleeponexit_i;   // Sleep-on-exit request
  input          dbg_halted_i;             // Core halted
  input [6:0]    nvic_dbg_reg_addr_i;      // Debug register transfer reg num
  input          nvic_dbg_reg_wr_i;        // Debug register transfer W/nR

  output         lsu_active_ex_o;          // LSU is active
  output         lsu_sp_wb_ex_o;           // LSU SP WB
  output         lsu_flag_wr_ex_o;         // LSU flag result write enable
  output [31:0]  lsu_psr_ex_o;             // LSU PSR write value
  output         lsu_unalgn_ex_o;          // Access is unaligned
  output         lsu_stack_algn_64_ex_o;   // Align stack at 8-byte boundary
  output         lsu_ld_excl_ex_o;         // LDREX in execute
  output [1:0]   lsu_st_excl_ex_o;         // STREX in execute
  output [4:0]   lsu_exit_code_ex_o;       // LSU ISR exit code
  output         lsu_exit_valid_ex_o;      // LSU operation is an ISR exit
  output [31:0]  lsu_rf_wr_d_data_ex_o;    // LSU RF D port write data
  output [31:0]  lsu_rf_wr_z_data_ex_o;    // LSU RF D port write data
  output [31:0]  lsu_fpu_rd_data_ex_o;     // Data to FPU
  output [31:0]  lsu_nxt_fpcar_o;          // Next value for FPCAR reg
  output         lsu_addracc_wait_reg_o;   // Waiting for address acceptance
  output         lsu_ahb_invk_msk_o;       // Mask interrupts for const AHB
  output         str_buf_imm_ex_o;         // Store status is an immediate
  output         ahb_const_unstk_o;        // Keep trans for unstack
  output [1:0]   ahb_htransd_o;            // AHB D-side HTRANS
  output         ahb_hwrited_o;            // AHB D-side HWRITE
  output [1:0]   ahb_hsized_o;             // AHB D-side HSIZE
  output [31:0]  ahb_haddrd_o;             // AHB D-side HADDR
  output [1:0]   ahb_hprotd_o;             // AHB D-side HPROT
  output [31:0]  ahb_hwdatad_o;            // AHB D-side HWDATA
  output         ahb_exreqd_o;             // AHB D-side EXREQ
  output         dpu_addrd_valid_o;        // Address valid for FAR use
  output [8:0]   int_retisr_o;             // Returned to ISR number
  output [1:0]   fault_lsu_local_ex_o;     // LSU detection of faults
  output         fault_lsu_precise_ex_o;   // Precise LSU fault
  output [10:0]  fault_lsu_ex_o;           // Collection of LSU faults
  output         dbg_ctrl_wr_ex_o;         // Debug write to special control
  output         dbg_wr_isr_o;             // Debug write to ISR number

  wire           full_hreset_n;            // Apply hreset_n to all registers
  wire           opt_ext_en;               // Master enable for extensions
  wire           opt_fpu_en;               // Master enable for FPU logic
  wire           opt_const_en;             // Constant control logic removal
  wire           lsu_ldst_de;              // LSU op is a load or store
  wire           lsu_tb_de;                // LSU op is a table branch
  wire           lsu_wb_before_de;         // LSU op has writeback before
  wire           lsu_wb_neg_de;            // LSU op writeback is negative
  wire [1:0]     lsu_immshft_de;           // LSU op ahs a 2 bit immediate shift
  wire           lsu_excl_de;              // LSU op is an exclusive
  wire           lsu_lsm_de;               // LSU op is a load or store multiple
  wire           lsu_lsd_de;               // LSU op is a load or store double
  wire           lsu_lss_de;               // LSU op is a load or store single
  wire           lsu_fpu_de;               // LSU has an FPU operation in De
  wire           lsu_flsm_de;              // FP LSM in decode
  wire           lsu_excl_ex;              // LSU op is an exclusive
  wire           lsu_isld_ex;              // LSU op is a load
  wire           lsu_translate_ex;         // LSU translate from priv -> user
  wire           lsu_wb_valid_ex;          // LSU op has writeback aapplied
  wire           lsu_wb_after_ex;          // LSU op has writeback after
  wire           lsu_wb_pos_ex;            // LSU op writeback is positive
  wire           lsu_fpu_ex;               // LSU op is for FPU
  wire           lsu_reg_data_valid_de;    // LSU op requires RF data
  wire           lsu_imm_data_valid_de;    // LSU op requires immediate data
  wire           str_imm_ex;               // Store with imm based offset
  wire           lsu_push_ex;              // LSU op is a PUSH
  wire           lsu_pop_ex;               // LSU op is a POP
  wire           lsu_sp_wb_ex;             // LSU op is SP WB
  wire           lss_cphase_algn_ex;       // LSS control phase (aligned)
  wire           lss_cphase_unalgn_ex;     // LSS control phase (unaligned)
  wire           lss_cphase_ex;            // LSS control phase (all)
  wire           lsd_cphase_ex;            // LSD control phase
  wire           lss_lsd_cphase_ex;        // LSS and LSD control phase combined
  wire           lsm_cphase_ex;            // LSM control phase
  wire           lsu_cphase_ex;            // LSU opening control phases
  wire           lss_cphase_negimm_ex;     // LSS -ve imm offset control phase
  wire           lsu_lfault_ex;            // LSU has detected a fault
  wire           lsu_excp_ex;              // LSU has detected an exception
  wire           lsu_ahb_rdy_ex;           // AHB is ready
  wire           lsu_active_ex;            // LSU is active
  wire           sp_adder_ctl_ex;          // SP fix adder control
  reg  [4:0]     sp_adder_in1_ex;          // SP fix adder input 1
  wire [9:0]     sp_adder_in2_ex;          // SP fix adder input 2
  wire [9:0]     sp_adder_ex;              // SP fix adder result
  wire           sp_fix_we;                // Write enable to reg sp_adder_ex
  wire [2:0]     nxt_offset_sel;           // Next SP fix offset selector
  reg  [2:0]     sp_offset_sel_ex;         // SP fix offset selector
  reg  [9:0]     sp_fix_ex;                // Registered sp_adder_ex
  wire [9:0]     sp_offset_ex;             // SP offset for address adder inputs
  reg            addr_adder_ctl;           // Address adder add[0]/sub[1] ctl
  wire [31:0]    addr_offset;              // Address offset for adder input 2
  reg            use_addr_offset;          // Use addr_offset for adder input 2
  reg  [31:0]    addr_adder_in1;           // Address adder input 1
  reg  [31:0]    addr_adder_in2;           // Address adder input 2
  wire [31:0]    addr_adder_algn_in1;      // Align address adder input 1
  wire [31:0]    addr_adder_late_in2;      // Address adder input 2 (late)
  reg            addr_reg_ex_we;           // Address reg write enable (Ex)
  wire           addr_reg_we;              // Address reg write enable (all)
  wire           addr_adder_algn;          // Align to 8-byte boundary
  reg            addr_adder_ctl_ex;        // Registered addr_adder_ctl
  reg            addr_adder_algn_ex;       // Registered addr_adder_algn
  reg  [31:0]    addr_adder_in1_ex;        // Registered addr_adder_in1
  reg  [31:0]    addr_adder_in2_ex;        // Registered addr_adder_in2
  wire [31:0]    addr_adder_ex;            // Address adder result
  wire           nxt_fpcar_valid;          // Next FPCAR is valid
  wire [31:0]    lsu_nxt_fpcar;            // Next FPCAR
  wire           nxt_addrd_1k_cross;       // Next 1K boundary cross
  wire           addrd_1k_cross_we;        // 1K boundary cross write enable
  reg            addrd_1k_cross;           // 1K boundary cross
  wire [31:0]    ahb_haddrd;               // AHB D-side HADDR
  reg  [1:0]     ahb_htransd;              // AHB D-side HTRANS
  reg            ahb_hwrited;              // AHB D-side HWRITE
  reg  [1:0]     ahb_hsized;               // AHB D-side HSIIZE
  wire [1:0]     ahb_hprotd;               // AHB D-side HPROT
  wire           ahb_exreqd;               // AHB D-side EXREQ
  wire           dpu_addrd_valid;          // Address valid for FAR use
  wire           nvic_lazy_stkalign;       // STKALIGN inc. 64-bit-aligned FPU
  wire           nxt_int_chk_algn_64;      // Next check alignment for stacking
  wire           int_chk_algn_64_we;       // Check alignment write enable
  reg            int_chk_algn_64;          // Check alignment register
  wire           int_req_algn_64_clr;      // Clear int_req_algn_64
  wire           int_req_algn_64_we;       // int_req_algn_64 write enable
  wire           nxt_int_req_algn_64;      // Next align stack register request
  reg            int_req_algn_64;          // Align stack register request
  wire           int_algn_stk_64_valid;    // Stack alignment required
  wire           int_algn_unstk_64_set;    // Set int_algn_unstk_64_valid
  wire           int_algn_unstk_64_clr;    // Clear int_algn_unstk_64_valid
  wire           int_algn_unstk_64_we;     // int_algn_unstk_64_valid write en
  wire           nxt_int_algn_unstk_64;    // Next int_algn_unstk_64_valid
  reg            int_algn_unstk_64_valid;  // Unstacking alignment required
  wire           int_algn_stk_64;          // Align stack to 8-byte boundary
  wire           lsu_stack_algn_le_64_ex;  // 64-bit stack alignment select (LE)
  wire           lsu_stack_algn_be_64_ex;  // 64-bit stack alignment select (BE)
  wire           lsu_stack_algn_64_ex;     // 64-bit stack alignment
  wire           addracc_wait_ex;          // Transaction req but not accepted
  wire           addracc_wait_we;          // HADDRACC tracking write enable
  reg            lsu_addracc_wait_reg;     // HADDRACC tracking register
  wire           nxt_lsu_be8_valid_ex;     // Next LSU BE8 status
  wire           lsu_be8_valid_we;         // LSU BE8 write enable
  reg            lsu_be8_valid_ex;         // LSU BE8 status for transaction
  wire           ahb_ctl_2_data_we;        // AHB control -> data phase
  reg            htransd_reg;              // Data phase version of htransd
  reg  [1:0]     haddrd_reg;               // Data phase version of haddrd
  reg            hwrited_reg;              // Data phase version of hwrited
  reg  [1:0]     hsized_reg;               // Data phase version of hsized
  reg            instr_sign_reg;           // Data phase version of instr_sign
  reg            fpu_state_reg;            // Data phase version of fpu_state_ex
  reg            lsm_last_reg;             // Data phase version of lsm_last_ex
  wire           ahb_frc_psr_reg;          // Data phase version of ahb_frc_psr_trns
  reg            opt_ahb_frc_psr_reg;      // Removable ahb_frc_psr_reg
  wire [31:0]    b_hrdatad;                // HRDATA endianness swizzle (byte)
  wire [31:0]    h_hrdatad;                // HRDATA endianness swizzle (hword)
  wire [31:0]    w_hrdatad;                // HRDATA endianness swizzle (word)
  wire           str_fwd_test_ex;          // Potential store forwarding test
  wire           set_str_fwd_ex;           // Store forwarding detected
  wire           clr_str_fwd_ex;           // Clearance of store forwarding
  wire           lsu_str_fwd_we;           // Store forwarding write enable
  reg            poss_str_fwd_ex;          // Store forwarding register
  wire           lsu_str_fwd_ex;           // Validated store forwarding case
  wire           nxt_int_fix_sp;           // Next need to fix SP
  wire           int_fix_sp_we;            // Need to fix SP write enable
  reg            int_fix_sp;               // Need to fix SP
  reg            data_adder_ctl;           // Data adder control
  wire [31:0]    data_adder_in1;           // Data adder input 1
  reg  [31:0]    data_adder_in2;           // Data adder input 2
  wire [31:0]    data_adder_ex;            // Data adder
  reg  [31:0]    lsu_tb_data_ex;           // HRDATA selection for table branch
  reg            data_reg_we;              // Data register write enable
  reg  [31:0]    nxt_data_reg_ex;          // Next data register value
  reg  [31:0]    data_reg_ex;              // Data register
  wire           nxt_str_stat_valid_ex;    // Next str_stat_valid_ex
  wire [2:0]     nxt_str_stat_ex;          // Next str_stat_ex
  wire           str_stat_we;              // Store immediate status write en
  reg            str_stat_valid_ex;        // Store immediate status valid
  reg  [2:0]     str_stat_ex;              // Store immediate status
  wire           str_buf_imm_ex;           // Store status is an immediate
  wire           sel_str_reg_ex;           // Select store register data
  wire           str_fwd_valid_we;         // str_fwd_valid_ex write enable
  wire           nxt_str_fwd_valid_ex;     // Next str_fwd_valid_ex
  reg            opt_str_fwd_valid_ex;     // Optimised str_fwd_valid_ex
  wire           str_fwd_valid_ex;         // Second data register required
  reg  [31:0]    str_fwd_reg_ex;           // Second data register
  wire [31:0]    str_data_reg_ex;          // Select required data register
  wire [31:0]    str_mux_ex;               // Selected data to store
  reg  [31:0]    str_mux_rot_ex;           // Rotation of store data
  reg  [31:0]    str_mux_rep_ex;           // Replication of store data
  wire [31:0]    str_mux_mask_ex;          // Masked store mux
  wire [31:0]    dbg_special_reg;          // Speical ctl reg for debug trans
  wire [31:0]    str_psr_ex;               // PSR store data
  reg  [31:0]    str_data_ex;              // Store data
  wire [31:0]    ahb_hwdatad;              // AHB D-side HWDATA
  reg  [23:0]    rdata_sel;                // Read data byte selection
  wire           rdata_sign_hword;         // Sign extension value (hword)
  wire           rdata_sign_byte;          // Sign extension value (byte)
  wire [31:0]    rdata_sign_mask_ex;       // Read data sign mask
  wire [31:0]    rdata_ex;                 // Read data
  wire [31:0]    lsu_rd_data_ex;           // LSU read data
  wire           fwd_a_from_d_ex;          // Forward to A port from D
  wire           fwd_a_from_z_ex;          // Forward to A port from Z
  wire [31:0]    lsu_a_fwd_data_ex;        // LSU forwarded data to A
  wire           fwd_b_from_d_ex;          // Forward to B port from D
  wire           fwd_b_from_z_ex;          // Forward to B port from Z
  wire [31:0]    lsu_b_fwd_data_ex;        // LSU forwarded data to B
  wire [31:0]    lsu_fpu_rd_data_ex;       // Data to FPU
  wire [31:0]    rdata_addr_fwd_a_ex;      // Read data to address forward (A)
  wire [31:0]    rdata_addr_fwd_b_ex;      // Read data to address forward (B)
  wire [31:0]    rdata_addr_b_lsl_ex;      // Shift B read data left by imm
  wire           nxt_lsu_ld_excl_ex;       // Next load exclusive marker
  wire           lsu_ld_excl_we;           // Load exclusive marker write enable
  reg            lsu_ld_excl_ex;           // Load exclusive marker register
  wire [1:0]     nxt_lsu_st_excl_ex;       // Next store exclusive marker
  wire           lsu_st_excl_we;           // Store exclusive marker write en
  reg  [1:0]     lsu_st_excl_ex;           // Store exclusive marker register
  wire           lsm_passed_brestore_ex;   // LSM base in list has been passed
  wire           lsm_curreg_brestore_ex;   // LSM base in list is current reg
  reg  [31:0]    lsu_rf_wr_d_data_ex;      // LSU D port write data
  wire [31:0]    lsu_msk_rf_wr_d_data_ex;  // Masked version of LSU D write data
  wire [31:0]    lsu_rf_wr_z_data_ex;      // LSU Z port write data
  wire           lsu_flag_wr_ex;           // LSU flag result write enable
  wire [31:0]    lsu_psr_ex;               // LSU PSR write value
  reg  [8:0]     int_retisr;               // Returned to ISR number
  wire           dbg_psr_wr_ex;            // Debug write to PSR
  wire           dbg_ctrl_wr_ex;           // Debug write to special control
  wire           dbg_wr_isr_we;            // Debug write to ISR number write en
  reg            dbg_wr_isr;               // Debug write to ISR number
  wire [7:0]     hrdatad_1_det;            // Detect bytes=F in HRDATA
  wire           lsu_exit_we;              // Exit information write enable
  wire           lsu_exit_in_b0;           // Exit code is located in byte 0
  wire           lsu_exit_in_b1;           // Exit code is located in byte 1
  wire           lsu_exit_in_b2;           // Exit code is located in byte 2
  wire           lsu_exit_in_b3;           // Exit code is located in byte 3
  wire           lsu_exit_ones;            // Non exit code portion is all ones
  wire [4:0]     nxt_lsu_exit_code_ex;     // Next exit code
  wire           nxt_lsu_exit_valid_ex;    // Next exit code is valid
  reg            lsu_exit_valid_ex;        // Exit code is valid
  reg  [4:0]     lsu_exit_code_ex;         // Exit code
  wire           ahb_trans_waited;         // Spotted a wait-stated transfer
  wire           capt_ahb_ctrl_we;         // Capture control info write enable
  wire           ahb_const_ctrl_we;        // Tracking write enable
  reg            ahb_const_trk;            // AHB constant tracking
  wire           ahb_const_ctrl;           // Keep AHB control info constant
  wire           lsu_ahb_invk_msk;         // Mask interrupts for const AHB
  wire           ahb_const_unstk;          // Keep is PSR unstack state
  wire           ahb_frc_psr_trns;         // Keep trans for unstack
  wire           nxt_fault_lsu_resp;       // Next bus fault response
  wire           fault_lsu_resp_we;        // Bus fault response write enable
  reg            fault_lsu_resp_ex;        // Bus fault response
  wire           fault_lsu_precise_set;    // Set precise fault
  wire           fault_lsu_precise_clr;    // Clear precise fault
  wire           fault_lsu_precise_we;     // Precise fault write enable
  reg            fault_lsu_precise_ex;     // Precise fault register
  wire           nxt_chk_unalgn_ex;        // Next check alignment
  wire [1:0]     nxt_algn_size_ex;         // Next alignement size
  wire           chk_unalgn_we;            // Check alignement write enable
  reg  [1:0]     algn_size_ex;             // Alignment size
  reg            chk_unalgn_ex;            // Check alignement
  wire           lsu_unalgn_ex;            // Unaligned access detected
  wire           fault_lsu_unalgn_ex;      // Raise unaligned fault if required
  wire [3:0]     bus_fault_sel;            // Fault selection
  wire           fault_lsu_bus_we;         // Bus fault type write enable
  wire [9:0]     nxt_fault_lsu_bus;        // Next bus fault type
  reg  [9:0]     fault_lsu_bus_ex;         // Bus fault type register
  wire           nxt_fault_lsu_mmatch;     // Unstacking mismatch vs PSR
  wire           fault_lsu_mmatch_we;      // Unstacking mismatch write enable
  reg            fault_lsu_mmatch_ex;      // Unstacking mismatch register
  wire [1:0]     fault_lsu_local_ex;       // LSU detection of faults
  wire [10:0]    fault_lsu_ex;             // Collection of LSU faults

`ifdef ARM_DSM

  wire           dsm_reset_all_regs;
  wire           dsm_fpu_present;
  wire           dsm_const_ahb_ctrl;

  assign full_hreset_n = (dsm_reset_all_regs)  ? hreset_n : 1'b1;

  assign opt_fpu_en    = (dsm_fpu_present)     ? 1'b1     : 1'b0;

  assign opt_const_en  = (dsm_const_ahb_ctrl)  ? 1'b1     : 1'b0;

`else

  assign full_hreset_n = (RESET_ALL_REGS == 1) ? hreset_n : 1'b1;

  assign opt_fpu_en    = (FPU_PRESENT == 1)    ? 1'b1     : 1'b0;

  assign opt_const_en  = (CONST_AHB_CTRL == 1) ? 1'b1     : 1'b0;

`endif

  assign opt_ext_en    = (EXT_PRESENT == 1)    ? 1'b1     : 1'b0;

  assign lsu_ldst_de      = (|instr_lsu_ctl_de_i[5:3]);
  assign lsu_tb_de        = instr_lsu_ctl_de_i[2];
  assign lsu_wb_before_de = ~instr_lsu_ctl_de_i[10];
  assign lsu_wb_neg_de    = ~instr_lsu_ctl_de_i[11];
  assign lsu_immshft_de   = instr_lsu_ctl_de_i[8:7];
  assign lsu_excl_de      = instr_lsu_ctl_de_i[13];
  assign lsu_lsm_de       = instr_lsu_ctl_de_i[5];
  assign lsu_lsd_de       = instr_lsu_ctl_de_i[4];
  assign lsu_lss_de       = instr_lsu_ctl_de_i[3];
  assign lsu_fpu_de       = instr_lsu_ctl_de_i[15] & opt_fpu_en;
  assign lsu_flsm_de      = lsu_fpu_de & lsu_lsm_de;

  assign lsu_excl_ex      = instr_lsu_ctl_ex_i[13] & lsu_valid_ex_i;
  assign lsu_isld_ex      = instr_lsu_ctl_ex_i[6]  & lsu_valid_ex_i;
  assign lsu_translate_ex = instr_lsu_ctl_ex_i[12] & lsu_valid_ex_i;
  assign lsu_wb_valid_ex  = instr_lsu_ctl_ex_i[9]  & lsu_valid_ex_i;
  assign lsu_wb_after_ex  = instr_lsu_ctl_ex_i[10];
  assign lsu_wb_pos_ex    = instr_lsu_ctl_ex_i[11];
  assign lsu_fpu_ex       = instr_lsu_ctl_ex_i[15] & opt_fpu_en;


  assign lsu_reg_data_valid_de = (lsu_tb_de |
                                  (lsu_lss_de & ~instr_imm_valid_fast_de_i &
                                   ~lsu_fpu_de));

  assign lsu_imm_data_valid_de = (lsu_lsd_de |
                                  (lsu_lss_de & instr_imm_valid_fast_de_i));

  assign str_imm_ex            = instr_immed_valid_ex_i & ~lsu_isld_ex;

  assign lsu_push_ex           = (lsu_wb_valid_ex & ~lsu_wb_after_ex &
                                  ~lsu_wb_pos_ex);

  assign lsu_pop_ex            = (lsu_wb_valid_ex & lsu_wb_after_ex &
                                  lsu_wb_pos_ex);

  assign lsu_sp_wb_ex          = ((rf_rd_a_addr_ex_i==4'b1101) &
                                  lsu_wb_valid_ex);

  assign lss_cphase_algn_ex    = ((lsu_state_ex_i==`CORE_LSU_LSS_N) |
                                  (lsu_state_ex_i==`CORE_LSU_LSS_P) |
                                  (lsu_state_ex_i==`CORE_LSU_LSS_B));

  assign lss_cphase_unalgn_ex  = (lsu_state_ex_i==`CORE_LSU_LSS_U);

  assign lss_cphase_ex         = (lss_cphase_algn_ex |
                                  lss_cphase_unalgn_ex);

  assign lsd_cphase_ex         = (lsu_state_ex_i==`CORE_LSU_LSD_N);

  assign lss_lsd_cphase_ex     = lss_cphase_ex | lsd_cphase_ex;

  assign lsm_cphase_ex         = (lsu_state_ex_i==`CORE_LSU_LSM_NONSEQ);

  assign lsu_cphase_ex         = (lsm_cphase_ex                     |
                                  lsd_cphase_ex                     |
                                  (lsu_state_ex_i==`CORE_LSU_LSS_N) |
                                  (lsu_state_ex_i==`CORE_LSU_LTB_N));

  assign lss_cphase_negimm_ex  = (lss_cphase_ex & ~lsu_wb_pos_ex &
                                  (|instr_immed_ex_i[1:0]));

  assign lsu_lfault_ex         = (|fault_lsu_local_ex);

  assign lsu_excp_ex           = lsu_lfault_ex | nvic_int_invoke_i;

  assign lsu_ahb_rdy_ex        = mtx_ahb_hreadyd_i & mtx_ahb_haddraccd_i;

  assign lsu_active_ex         = (// In CC passing control phase
                                  (lsu_cphase_ex) |
                                  str_stat_valid_ex                   |
                                  ((lsu_state_ex_i!=`CORE_LSU_IDLE) &
                                   ~lsu_cphase_ex));


  assign sp_adder_ctl_ex = (~lsu_wb_pos_ex &
                            (lss_lsd_cphase_ex | lsm_cphase_ex));

  always @ (lss_cphase_negimm_ex or fpu_stack_ctl_i or lsm_cphase_ex or
            lss_lsd_cphase_ex)
    if (lss_cphase_negimm_ex)
      sp_adder_in1_ex = ((|fpu_stack_ctl_i) ?
                           5'b1_0011 : // +19 (1 word and FPU space)
                           5'b0_0001); //  +4 (1 word)

    else if (lsm_cphase_ex | lss_lsd_cphase_ex)
      sp_adder_in1_ex = ((|fpu_stack_ctl_i) ?
                           5'b1_0100 : // +20 (2 words and FPU space)
                           5'b0_0010); //  +8 (2 words)
    else
      sp_adder_in1_ex = 5'b0_0000;     // +0

  assign sp_adder_in2_ex = (// LSS or LSD
                            ({10{lss_lsd_cphase_ex}} &
                             {2'b00,instr_immed_ex_i[9:2]}) |
                            ({10{lsm_cphase_ex}}     &
                             {4'b0000, lsm_list_length_ex_i}));

  assign sp_adder_ex     = ({{5{1'b0}}, sp_adder_in1_ex } +
                            (sp_adder_in2_ex ^ {10{sp_adder_ctl_ex}}) +
                            ({{9{1'b0}}, sp_adder_ctl_ex }));

  assign sp_fix_we       = (// AHB close
                            (lsu_ahb_rdy_ex &
                             (lss_cphase_algn_ex |
                              lsd_cphase_ex      |
                              lsm_cphase_ex)) |
                            (lss_cphase_unalgn_ex & mpu_canceld_i));

  assign nxt_offset_sel  = ({3{rf_rd_a_addr_ex_i==4'b1101}} &
                            {// FPU push interrupt between 1st and 2nd reg
                             (lsu_fpu_ex & ~lsu_lsm_last_ex_i & lsu_push_ex),
                             (// PUSH
                              (lsm_cphase_ex & lsu_push_ex) |
                              (lsu_wb_valid_ex & lss_cphase_algn_ex &
                               (lsu_isld_ex | mpu_canceld_i | mpu_addrd_non_buf_i)) |
                              (lsu_wb_valid_ex & mpu_canceld_i &
                               ~mtx_ahb_hreadyd_i & lss_cphase_unalgn_ex)),
                             (// POP
                              (lsm_cphase_ex & lsu_pop_ex) |
                              (lsm_cphase_ex & lsu_push_ex & instr_itblock_ex_i) |
                              (lsd_cphase_ex & lsu_wb_valid_ex))});

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        sp_offset_sel_ex <= 3'b000;
        sp_fix_ex        <= {10{1'b0}};
      end
    else if (sp_fix_we)
      begin
        sp_offset_sel_ex <= nxt_offset_sel;
        sp_fix_ex        <= sp_adder_ex;
      end

  assign sp_offset_ex = (((sp_offset_sel_ex[0] & nvic_int_invoke_i) |
                          (sp_offset_sel_ex[1] & fault_lsu_local_ex[0]) |
                          (sp_offset_sel_ex[2] & nvic_int_invoke_i &
                           lsm_fpu_brestore_ex_i)) ?
                         sp_fix_ex :
                         ((|fpu_stack_ctl_i) ? 10'b0000010100 :
                                               10'b0000000010));



  always @ (lsu_state_ex_i or nvic_int_invoke_i or lsu_wb_pos_ex or
            lsu_tb_de or lsu_wb_neg_de or lsu_ldst_de or lsu_lsm_reg_ex_i or
            fpu_flip_stack_i)
    case (lsu_state_ex_i)
      `CORE_LSU_INT_S_FLAGS,  `CORE_LSU_INT_S_FPSCR,  `CORE_LSU_INT_L_PC :
        addr_adder_ctl = 1'b1;
      `CORE_LSU_INT_S_PC,     `CORE_LSU_INT_S_RX,     `CORE_LSU_INT_EXIT,
      `CORE_LSU_FP_L_FLAGS,   `CORE_LSU_INT_L_R14,    `CORE_LSU_INT_L_RX,
      `CORE_LSU_INT_L_FLAGS :
        addr_adder_ctl = lsu_ldst_de & lsu_wb_neg_de;
      `CORE_LSU_FP_S_SX,      `CORE_LSU_FP_L_SX :
        addr_adder_ctl = fpu_flip_stack_i & (lsu_lsm_reg_ex_i[3:0] == 4'h7);
      `CORE_LSU_LSM_INTC1 :
        addr_adder_ctl = nvic_int_invoke_i | lsu_wb_pos_ex;
      `CORE_LSU_INT_SFAULT_1, `CORE_LSU_INT_SFAULT_2 :
        addr_adder_ctl = 1'b0;
      `CORE_LSU_LSS_N,        `CORE_LSU_LSS_P,        `CORE_LSU_LSS_B,
      `CORE_LSU_LSD_N,        `CORE_LSU_LSM_NONSEQ,   `CORE_LSU_INT_UFAULT_1,
      `CORE_LSU_DFAULT_1,     `CORE_LSU_DBG_PC_2,     `CORE_LSU_DFAULT_2,
      `CORE_LSU_FAULT,        `CORE_LSU_IDLE,         `CORE_LSU_LSS_U,
      `CORE_LSU_LSS_D,        `CORE_LSU_LSD_S,        `CORE_LSU_LSD_D,
      `CORE_LSU_LSM_INTC2,    `CORE_LSU_LSM_Rx,       `CORE_LSU_SLEEP,
      `CORE_LSU_LTB_N,        `CORE_LSU_LTB_D,        `CORE_LSU_LTB_W,
      `CORE_LSU_ADD4,         `CORE_LSU_INT_RETURN,   `CORE_LSU_INT_ENTRY,
      `CORE_LSU_INT_ENLCK,    `CORE_LSU_INT_UFAULT_2, `CORE_LSU_DBG_PC_1,
      `CORE_LSU_DBG_R_0,      `CORE_LSU_DBG_R_1,      `CORE_LSU_DBG_R_2,
      `CORE_LSU_LAZY_EXIT,    `CORE_LSU_LAZY_FAULT_1, `CORE_LSU_LAZY_FAULT_2,
      `CORE_LSU_LAZY_ENTRY,   `CORE_LSU_LAZY_S_S0,    `CORE_LSU_LAZY_S_SX,
      `CORE_LSU_LAZY_S_FPSCR :
        addr_adder_ctl = ((nvic_int_invoke_i & ~lsu_ldst_de & ~lsu_tb_de) |
                          (lsu_ldst_de & lsu_wb_neg_de));
      default :
        addr_adder_ctl = 1'bx;
    endcase

  assign addr_offset = (// Use RF data
                        ({32{lsu_reg_data_valid_de}} & rdata_addr_b_lsl_ex) |
                        ({32{(lsu_imm_data_valid_de | lsu_flsm_de) &
                             (lsu_wb_before_de | (lsu_fpu_de & ~lsu_flsm_de))}} &
                         {{16{1'b0}}, instr_immed_de_i[15:0]})          |
                        ({32{lsu_lsm_de & ~lsu_fpu_de}}     &
                         {{26{1'b0}},{4{lsu_wb_before_de}}  &
                          lsm_list_length_de_i[3:0],2'b00}));

  always @ (lsu_state_ex_i or lsu_tb_de or lsu_ldst_de or nvic_int_invoke_i or
            lsu_lfault_ex or lsu_excp_ex or rdata_addr_fwd_a_ex or
            ahb_haddrd or lsm_ici_remaining_ex_i or lsm_list_length_ex_i or
            sp_offset_ex or instr_detect_exit_ex_i or lsu_lsm_last_ex_i or
            lsu_lsm_reg_ex_i or fpu_stack_ctl_i or fpu_flip_stack_i or
            nvic_fpcar_i)
    case (lsu_state_ex_i)
      `CORE_LSU_INT_S_FLAGS,  `CORE_LSU_INT_L_PC:
        begin
          addr_adder_in1  = ahb_haddrd;
          addr_adder_in2  = 32'h0000001C;
          use_addr_offset = 1'b0;
        end
      `CORE_LSU_INT_S_FPSCR :
        begin
          addr_adder_in1  = ahb_haddrd;
          addr_adder_in2  = 32'h00000040;
          use_addr_offset = 1'b0;
        end
      `CORE_LSU_INT_S_PC,     `CORE_LSU_INT_EXIT,     `CORE_LSU_SLEEP,
      `CORE_LSU_FP_L_FLAGS,   `CORE_LSU_INT_L_FLAGS :
        begin
          addr_adder_in1  = ahb_haddrd;
          addr_adder_in2  = 32'h00000004;
          use_addr_offset = 1'b0;
        end
      `CORE_LSU_FP_S_SX,      `CORE_LSU_FP_L_SX :
        begin
          addr_adder_in1  = ahb_haddrd;

          case (fpu_flip_stack_i)
            1'b0: addr_adder_in2  = 32'h00000004;
            1'b1: addr_adder_in2  = (lsu_lsm_reg_ex_i[3:0]==4'h7) ?
                                     32'h0000003C :  // 8th reg
                                     ((lsu_lsm_reg_ex_i[3:0]==4'hF) ?
                                     32'h00000024 :  // 15th reg
                                     32'h00000004);
            default: addr_adder_in2  = 32'hXXXXXXXX;
          endcase

          use_addr_offset = 1'b0;
        end
      `CORE_LSU_LAZY_ENTRY :
        begin
          addr_adder_in1  = (nvic_int_invoke_i ?
                            rdata_addr_fwd_a_ex : nvic_fpcar_i);
          addr_adder_in2  = (nvic_int_invoke_i ?
                            ((|fpu_stack_ctl_i) ? 32'h00000050 : 32'h00000008) :
                            32'h00000000);
          use_addr_offset = 1'b0;
        end
      `CORE_LSU_LAZY_EXIT :
        begin
          addr_adder_in1  = rdata_addr_fwd_a_ex;
          addr_adder_in2  = (|fpu_stack_ctl_i) ? 32'h00000050: 32'h00000008;
          use_addr_offset = ~nvic_int_invoke_i;
        end
      `CORE_LSU_LAZY_S_S0,    `CORE_LSU_LAZY_S_SX,    `CORE_LSU_LAZY_S_FPSCR:
        begin
          addr_adder_in1  = (nvic_int_invoke_i ?
                            rdata_addr_fwd_a_ex : ahb_haddrd);
          addr_adder_in2  = (nvic_int_invoke_i ?
                            ((|fpu_stack_ctl_i) ? 32'h00000050 : 32'h00000008) :
                            32'h00000004);
          use_addr_offset = 1'b0;
        end
      `CORE_LSU_INT_S_RX :
        begin
          addr_adder_in1  = ahb_haddrd;

          case ({(lsu_lsm_reg_ex_i[3:0]==4'hE), fpu_flip_stack_i})
            2'b00, 2'b01: addr_adder_in2  = 32'h00000004;

            2'b10:        addr_adder_in2  = 32'h0000000C;

            2'b11:        addr_adder_in2  = 32'h0000002C;

            default:      addr_adder_in2  = 32'hXXXXXXXX;
          endcase

          use_addr_offset = 1'b0;
        end
      `CORE_LSU_INT_L_RX :
        begin
          addr_adder_in1  = ahb_haddrd;
          addr_adder_in2  = ((lsu_lsm_reg_ex_i[3:0]==4'hC) ?
                             32'h0000000C : 32'h00000004);

          use_addr_offset = 1'b0;
        end
      `CORE_LSU_LSM_INTC1 :
        begin
          addr_adder_in1  = (nvic_int_invoke_i ?
                            rdata_addr_fwd_a_ex : ahb_haddrd);
          addr_adder_in2  = (nvic_int_invoke_i ?
                            ((|fpu_stack_ctl_i) ? 32'h00000050 : 32'h00000008) :
                            {{24{1'b0}},lsm_list_length_ex_i,2'b00});
          use_addr_offset = 1'b0;
        end
      `CORE_LSU_LSM_INTC2 :
        begin
          addr_adder_in1  = (nvic_int_invoke_i ?
                            rdata_addr_fwd_a_ex : ahb_haddrd);
          addr_adder_in2  = (nvic_int_invoke_i ?
                            ((|fpu_stack_ctl_i) ? 32'h00000050 : 32'h00000008) :
                            {{24{1'b0}},lsm_ici_remaining_ex_i,2'b00});
          use_addr_offset = 1'b0;
      end
      `CORE_LSU_LSM_NONSEQ,   `CORE_LSU_LSD_N :
        begin
          addr_adder_in1  = (nvic_int_invoke_i ?
                            rdata_addr_fwd_a_ex : ahb_haddrd);
          addr_adder_in2  = (nvic_int_invoke_i ?
                            ((|fpu_stack_ctl_i) ? 32'h00000050 : 32'h00000008) :
                            32'h00000004);
          use_addr_offset = 1'b0;
        end
      `CORE_LSU_LSD_S :
        begin
          addr_adder_in1  = rdata_addr_fwd_a_ex;
          addr_adder_in2  = {{20{sp_offset_ex[9]}},sp_offset_ex[9:0],2'b00};
          use_addr_offset = 1'b0;
        end
      `CORE_LSU_LSS_P,        `CORE_LSU_LSS_D,        `CORE_LSU_LSD_D,
      `CORE_LSU_LSS_B :
        begin
          addr_adder_in1  = rdata_addr_fwd_a_ex;
          addr_adder_in2  = (lsu_lfault_ex ?
                            {{20{sp_offset_ex[9]}},sp_offset_ex[9:0],2'b00} :
                            ((|fpu_stack_ctl_i) ? 32'h00000050 : 32'h00000008));
          use_addr_offset = ~lsu_excp_ex;
        end
      `CORE_LSU_LSM_Rx :
        begin
          addr_adder_in1  = ((lsu_excp_ex |
                             lsu_lsm_last_ex_i) ?
                            rdata_addr_fwd_a_ex : ahb_haddrd);
          addr_adder_in2  = (lsu_excp_ex ?
                            ((lsu_lsm_last_ex_i & ~lsu_lfault_ex) ?
                             ((|fpu_stack_ctl_i) ? 32'h00000050 : 32'h00000008) :
                             {{20{sp_offset_ex[9]}},sp_offset_ex[9:0],2'b00}) :
                            32'h00000004);
          use_addr_offset = ~lsu_excp_ex & lsu_lsm_last_ex_i;
        end
      `CORE_LSU_DBG_R_0 :
        begin
          addr_adder_in1  = 32'hE000EDF8;
          addr_adder_in2  = 32'h00000000;
          use_addr_offset = 1'b0;
        end
      `CORE_LSU_INT_ENTRY,    `CORE_LSU_INT_ENLCK,    `CORE_LSU_INT_L_R14,
      `CORE_LSU_INT_RETURN :
        begin
          addr_adder_in1  = rdata_addr_fwd_a_ex;
          addr_adder_in2  = 32'h00000000;
          use_addr_offset = 1'b1;
        end
      `CORE_LSU_INT_SFAULT_1, `CORE_LSU_INT_SFAULT_2 :
        begin
          addr_adder_in1  = rdata_addr_fwd_a_ex;
          addr_adder_in2  = 32'h00000020;
          use_addr_offset = 1'b0;
        end
      `CORE_LSU_IDLE,         `CORE_LSU_LSS_N,        `CORE_LSU_LSS_U,
      `CORE_LSU_LTB_N,        `CORE_LSU_LTB_D,        `CORE_LSU_LTB_W,
      `CORE_LSU_ADD4,         `CORE_LSU_INT_UFAULT_1, `CORE_LSU_INT_UFAULT_2,
      `CORE_LSU_DBG_PC_1,     `CORE_LSU_DBG_PC_2,     `CORE_LSU_DBG_R_1,
      `CORE_LSU_DBG_R_2,      `CORE_LSU_FAULT,        `CORE_LSU_DFAULT_1,
      `CORE_LSU_DFAULT_2,     `CORE_LSU_LAZY_FAULT_1, `CORE_LSU_LAZY_FAULT_2 :
        begin
          addr_adder_in1  = ((lsu_ldst_de  | lsu_tb_de |
                             nvic_int_invoke_i | instr_detect_exit_ex_i) ?
                           rdata_addr_fwd_a_ex : 32'h00000000);

          addr_adder_in2  = (nvic_int_invoke_i ?
                            ((|fpu_stack_ctl_i) ? 32'h00000050 : 32'h00000008) :
                            32'h00000018);

          use_addr_offset = ~nvic_int_invoke_i & ~instr_detect_exit_ex_i;

        end
      default :
        begin
          addr_adder_in1  = 32'hxxxxxxxx;
          addr_adder_in2  = 32'hxxxxxxxx;
          use_addr_offset = 1'bx;
        end
    endcase

  assign addr_adder_algn_in1 = { addr_adder_in1[31:2],
                                 addr_adder_in1[1:0] & {2{~instr_algn_de_i}} };

  assign addr_adder_late_in2 = use_addr_offset ? addr_offset : addr_adder_in2;

  always @ (mtx_ahb_hreadyd_i or mtx_ahb_haddraccd_i or lsu_ahb_rdy_ex or
            nvic_int_invoke_i or lsu_lfault_ex or nvic_slp_sleeponexit_i or
            nvic_slp_asleep_i or lsu_lsm_last_ex_i or lsu_wb_after_ex or
            lsu_wb_valid_ex or rf_rd_a_addr_ex_i or lsu_isld_ex or
            instr_lsm_baseinlist_ex_i or instr_detect_exit_ex_i or
            lsu_state_ex_i or fpu_lazy_start_i or lsu_excp_ex)
    case (lsu_state_ex_i)
      `CORE_LSU_DBG_R_0,      `CORE_LSU_LSM_INTC2 :
        addr_reg_ex_we = 1'b1;
      `CORE_LSU_LSD_N,        `CORE_LSU_LSM_NONSEQ,   `CORE_LSU_LAZY_S_S0,
      `CORE_LSU_LAZY_S_SX :
        addr_reg_ex_we = lsu_ahb_rdy_ex & (nvic_int_invoke_i | ~lsu_lfault_ex);
      `CORE_LSU_INT_EXIT :
        addr_reg_ex_we = lsu_ahb_rdy_ex & ~nvic_slp_sleeponexit_i;
      `CORE_LSU_SLEEP :
        addr_reg_ex_we = lsu_ahb_rdy_ex & ~nvic_slp_asleep_i & ~nvic_int_invoke_i;
      `CORE_LSU_LSD_S,        `CORE_LSU_LSD_D :
        addr_reg_ex_we = (mtx_ahb_hreadyd_i &
                          (lsu_excp_ex | fpu_lazy_start_i));
      `CORE_LSU_LSM_Rx :
        addr_reg_ex_we = (mtx_ahb_hreadyd_i &
                          ((mtx_ahb_haddraccd_i & ~lsu_lsm_last_ex_i) |
                           lsu_excp_ex | fpu_lazy_start_i));
      `CORE_LSU_DBG_R_1,      `CORE_LSU_INT_ENTRY,    `CORE_LSU_INT_ENLCK :
        addr_reg_ex_we = 1'b0;
      `CORE_LSU_INT_S_PC,     `CORE_LSU_INT_S_FLAGS,  `CORE_LSU_INT_S_RX,
      `CORE_LSU_INT_L_R14,    `CORE_LSU_INT_L_RX,     `CORE_LSU_INT_L_PC,
      `CORE_LSU_INT_L_FLAGS,  `CORE_LSU_FP_S_SX,      `CORE_LSU_FP_L_FLAGS,
      `CORE_LSU_FP_L_SX,      `CORE_LSU_INT_S_FPSCR,  `CORE_LSU_LAZY_ENTRY :
        addr_reg_ex_we = lsu_ahb_rdy_ex;
      `CORE_LSU_LSM_INTC1 :
        addr_reg_ex_we = ((// Do not correct base if writeback
                           lsu_wb_valid_ex &
                           ~(lsu_wb_after_ex & (rf_rd_a_addr_ex_i==4'b1101)) &
                           ~(lsu_isld_ex & instr_lsm_baseinlist_ex_i) |
                          nvic_int_invoke_i));
      `CORE_LSU_FAULT,        `CORE_LSU_LAZY_FAULT_1, `CORE_LSU_LAZY_FAULT_2 :
        addr_reg_ex_we = nvic_int_invoke_i;
      `CORE_LSU_IDLE,         `CORE_LSU_LSS_N,        `CORE_LSU_LSS_P,
      `CORE_LSU_LSS_B,        `CORE_LSU_LSS_U,        `CORE_LSU_LSS_D,
      `CORE_LSU_LTB_N,        `CORE_LSU_LTB_D,        `CORE_LSU_LTB_W,
      `CORE_LSU_ADD4,         `CORE_LSU_INT_RETURN,   `CORE_LSU_INT_SFAULT_1,
      `CORE_LSU_DBG_R_2,      `CORE_LSU_DBG_PC_2,     `CORE_LSU_INT_SFAULT_2,
      `CORE_LSU_DBG_PC_1,     `CORE_LSU_INT_UFAULT_1, `CORE_LSU_DFAULT_1,
      `CORE_LSU_DFAULT_2,     `CORE_LSU_INT_UFAULT_2, `CORE_LSU_LAZY_EXIT,
      `CORE_LSU_LAZY_S_FPSCR :
        addr_reg_ex_we = nvic_int_invoke_i | instr_detect_exit_ex_i;
      default :
        addr_reg_ex_we = 1'bx;
    endcase

  assign addr_reg_we     = ((lsu_ldst_de | lsu_tb_de | addr_reg_ex_we) &
                            rst_pc_valid_ex_i);

  assign addr_adder_algn = ~(((nvic_int_stk_algn_i | (|fpu_stack_ctl_i)) &
                              nvic_int_invoke_i & ~int_stack_op_i) |
                             fpu_lazy_start_i);

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        addr_adder_ctl_ex  <= 1'b0;
        addr_adder_algn_ex <= 1'b0;
        addr_adder_in1_ex  <= {32{1'b0}};
        addr_adder_in2_ex  <= {32{1'b0}};
      end
    else if (addr_reg_we)
      begin
        addr_adder_ctl_ex  <= addr_adder_ctl;
        addr_adder_algn_ex <= addr_adder_algn;
        addr_adder_in1_ex  <= addr_adder_algn_in1;
        addr_adder_in2_ex  <= addr_adder_late_in2 ^ {32{addr_adder_ctl}};
      end

  assign addr_adder_ex  = (addr_adder_in1_ex + addr_adder_in2_ex +
                           addr_adder_ctl_ex);

  assign ahb_haddrd = { addr_adder_ex[31:3],
                        addr_adder_ex[2] & addr_adder_algn_ex,
                        addr_adder_ex[1:0] };



  assign nxt_fpcar_valid      = (((lsu_state_ex_i == `CORE_LSU_INT_ENTRY) |
                                  (lsu_state_ex_i == `CORE_LSU_INT_ENLCK)) &
                                 fpu_stack_ctl_i[0]);

  assign lsu_nxt_fpcar[31:3] = ({29{nxt_fpcar_valid}} & addr_adder_ex[31:3]);

  assign lsu_nxt_fpcar[2:0]  = 3'b000;


  assign nxt_addrd_1k_cross = ahb_htransd[1] & (&ahb_haddrd[9:2]);
  assign addrd_1k_cross_we  = (lsu_ahb_rdy_ex &
                               (// Set
                                nxt_addrd_1k_cross |
                                addrd_1k_cross));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      addrd_1k_cross <= 1'b0;
    else if (addrd_1k_cross_we)
      addrd_1k_cross <= nxt_addrd_1k_cross;

  always @ (lsu_state_ex_i or lsu_lsm_last_ex_i or nvic_int_invoke_i or
            nvic_int_tail_i or nvic_slp_asleep_i or nvic_slp_sleeponexit_i or
            htransd_reg or lsu_lfault_ex or lsu_excl_ex or lsu_isld_ex or
            addrd_1k_cross or lsu_excl_mon_ex_i or lsu_exit_valid_ex or
            lsu_lsm_reg_ex_i or fpu_flip_stack_i or lsm_last_reg or
            lsu_excp_ex or ahb_frc_psr_trns)
    case (lsu_state_ex_i)
      `CORE_LSU_DBG_R_1 :
        ahb_htransd = `CORE_AHB_NONSEQ;
      `CORE_LSU_INT_S_PC:
        ahb_htransd = lsu_exit_valid_ex ? `CORE_AHB_IDLE : `CORE_AHB_NONSEQ;
      `CORE_LSU_INT_S_RX :
        ahb_htransd = (lsu_lfault_ex ? `CORE_AHB_IDLE :
                       ((htransd_reg & ~addrd_1k_cross &
                         (|lsu_lsm_reg_ex_i[3:0])) ?
                        `CORE_AHB_SEQ : `CORE_AHB_NONSEQ));
      `CORE_LSU_FP_S_SX :
        ahb_htransd = (lsu_lfault_ex ? `CORE_AHB_IDLE :
                       ((htransd_reg & ~addrd_1k_cross &
                         (|lsu_lsm_reg_ex_i[3:0]) &
                         ~(fpu_flip_stack_i & (lsu_lsm_reg_ex_i[3:0]==4'h8))) ?
                        `CORE_AHB_SEQ : `CORE_AHB_NONSEQ));
      `CORE_LSU_FP_L_SX :
        ahb_htransd = (lsu_excp_ex ? `CORE_AHB_IDLE :
                       ((htransd_reg & ~addrd_1k_cross &
                         ~(fpu_flip_stack_i & (lsu_lsm_reg_ex_i[3:0]==4'h8))) ?
                        `CORE_AHB_SEQ : `CORE_AHB_NONSEQ));
      `CORE_LSU_INT_L_FLAGS :
        ahb_htransd = ((lsu_excp_ex & ~ahb_frc_psr_trns) ?
                       `CORE_AHB_IDLE : `CORE_AHB_NONSEQ);
      `CORE_LSU_INT_S_FLAGS :
        ahb_htransd = (lsu_lfault_ex ? `CORE_AHB_IDLE :
                       ((htransd_reg & ~addrd_1k_cross) ?
                        `CORE_AHB_SEQ : `CORE_AHB_NONSEQ));
      `CORE_LSU_INT_S_FPSCR :
        ahb_htransd = (lsu_lfault_ex ? `CORE_AHB_IDLE :
                       ((htransd_reg & ~addrd_1k_cross & ~fpu_flip_stack_i) ?
                        `CORE_AHB_SEQ : `CORE_AHB_NONSEQ));
      `CORE_LSU_FP_L_FLAGS :
        ahb_htransd = (lsu_excp_ex ?
                       `CORE_AHB_IDLE :
                       ((htransd_reg & ~addrd_1k_cross & ~fpu_flip_stack_i) ?
                        `CORE_AHB_SEQ : `CORE_AHB_NONSEQ));
      `CORE_LSU_INT_L_PC :
        ahb_htransd = (lsu_excp_ex ?
                       `CORE_AHB_IDLE :
                       ((htransd_reg & ~addrd_1k_cross) ?
                        `CORE_AHB_SEQ : `CORE_AHB_NONSEQ));
      `CORE_LSU_INT_L_RX :
        ahb_htransd = (lsu_excp_ex ?
                       `CORE_AHB_IDLE :
                       ((htransd_reg & ~addrd_1k_cross &
                         (lsu_lsm_reg_ex_i[3:0] != 4'hE)) ?
                        `CORE_AHB_SEQ : `CORE_AHB_NONSEQ));
      `CORE_LSU_INT_EXIT :
        ahb_htransd = ((nvic_int_invoke_i | nvic_int_tail_i |
                        nvic_slp_sleeponexit_i) ?
                       `CORE_AHB_IDLE : `CORE_AHB_NONSEQ);
      `CORE_LSU_SLEEP :
        ahb_htransd = ((~nvic_slp_asleep_i & ~nvic_int_invoke_i) ?
                       `CORE_AHB_NONSEQ : `CORE_AHB_IDLE);
      `CORE_LSU_LSS_N,        `CORE_LSU_LSS_P,        `CORE_LSU_LSS_B :
        ahb_htransd = ((lsu_excp_ex |
                        (lsu_excl_ex & ~lsu_isld_ex & ~lsu_excl_mon_ex_i)) ?
                       `CORE_AHB_IDLE : `CORE_AHB_NONSEQ);
      `CORE_LSU_LTB_N,        `CORE_LSU_LSM_NONSEQ,   `CORE_LSU_LSD_N :
        ahb_htransd = (lsu_excp_ex ? `CORE_AHB_IDLE : `CORE_AHB_NONSEQ);
      `CORE_LSU_LSD_S:
        ahb_htransd = (lsu_excp_ex ?
                       `CORE_AHB_IDLE :
                       ((htransd_reg & ~addrd_1k_cross) ?
                        `CORE_AHB_SEQ : `CORE_AHB_NONSEQ));
      `CORE_LSU_LSM_Rx:
        ahb_htransd = ((lsu_excp_ex | lsu_lsm_last_ex_i | lsm_last_reg) ?
                       `CORE_AHB_IDLE :
                       ((htransd_reg & ~addrd_1k_cross) ?
                        `CORE_AHB_SEQ : `CORE_AHB_NONSEQ));
      `CORE_LSU_LAZY_S_S0,    `CORE_LSU_LAZY_S_SX :
        ahb_htransd = (lsu_excp_ex ? `CORE_AHB_IDLE :
                       ((htransd_reg & ~addrd_1k_cross & (|lsu_lsm_reg_ex_i[3:0])) ?
                        `CORE_AHB_SEQ : `CORE_AHB_NONSEQ));
      `CORE_LSU_LAZY_S_FPSCR :
        ahb_htransd = (lsu_excp_ex ? `CORE_AHB_IDLE :
                       ((htransd_reg & ~addrd_1k_cross) ?
                        `CORE_AHB_SEQ : `CORE_AHB_NONSEQ));
      `CORE_LSU_IDLE,         `CORE_LSU_LSS_U,        `CORE_LSU_LSS_D,
      `CORE_LSU_LSD_D,        `CORE_LSU_LSM_INTC1,    `CORE_LSU_LSM_INTC2,
      `CORE_LSU_LTB_D,        `CORE_LSU_LTB_W,        `CORE_LSU_ADD4,
      `CORE_LSU_INT_L_R14,    `CORE_LSU_INT_RETURN,   `CORE_LSU_INT_ENTRY,
      `CORE_LSU_INT_ENLCK,    `CORE_LSU_INT_SFAULT_1, `CORE_LSU_INT_SFAULT_2,
      `CORE_LSU_INT_UFAULT_1, `CORE_LSU_INT_UFAULT_2, `CORE_LSU_DBG_PC_1,
      `CORE_LSU_DBG_PC_2,     `CORE_LSU_DBG_R_0,      `CORE_LSU_DBG_R_2,
      `CORE_LSU_FAULT,        `CORE_LSU_DFAULT_1,     `CORE_LSU_DFAULT_2,
      `CORE_LSU_LAZY_ENTRY,   `CORE_LSU_LAZY_EXIT,    `CORE_LSU_LAZY_FAULT_1,
      `CORE_LSU_LAZY_FAULT_2 :
        ahb_htransd = `CORE_AHB_IDLE;
      default:
        ahb_htransd = 2'bxx;
    endcase


  always @ (lsu_state_ex_i or lsu_isld_ex or nvic_dbg_reg_wr_i)
    case (lsu_state_ex_i)
      `CORE_LSU_INT_S_RX,     `CORE_LSU_INT_S_FLAGS,  `CORE_LSU_INT_S_PC,
      `CORE_LSU_INT_S_FPSCR,  `CORE_LSU_FP_S_SX,      `CORE_LSU_LAZY_S_FPSCR,
      `CORE_LSU_LAZY_S_S0,    `CORE_LSU_LAZY_S_SX :
        ahb_hwrited = 1'b1;
      `CORE_LSU_DBG_R_1 :
        ahb_hwrited = ~nvic_dbg_reg_wr_i;
      `CORE_LSU_LSM_NONSEQ,   `CORE_LSU_LSS_N,        `CORE_LSU_LSS_P,
      `CORE_LSU_LSS_B,        `CORE_LSU_LSD_N,        `CORE_LSU_LSD_S,
      `CORE_LSU_LSM_Rx :
        ahb_hwrited = ~lsu_isld_ex;
      `CORE_LSU_IDLE,         `CORE_LSU_LSS_U,        `CORE_LSU_LSS_D,
      `CORE_LSU_LSD_D,        `CORE_LSU_LSM_INTC1,    `CORE_LSU_LSM_INTC2,
      `CORE_LSU_LTB_N,        `CORE_LSU_LTB_D,        `CORE_LSU_LTB_W,
      `CORE_LSU_ADD4,         `CORE_LSU_INT_L_PC,     `CORE_LSU_INT_L_RX,
      `CORE_LSU_INT_L_R14,    `CORE_LSU_INT_L_FLAGS,  `CORE_LSU_INT_RETURN,
      `CORE_LSU_INT_ENTRY,    `CORE_LSU_INT_ENLCK,    `CORE_LSU_INT_EXIT,
      `CORE_LSU_INT_SFAULT_1, `CORE_LSU_INT_SFAULT_2, `CORE_LSU_INT_UFAULT_1,
      `CORE_LSU_INT_UFAULT_2, `CORE_LSU_DBG_PC_1,     `CORE_LSU_DBG_PC_2,
      `CORE_LSU_DBG_R_0,      `CORE_LSU_DBG_R_2,      `CORE_LSU_SLEEP,
      `CORE_LSU_FAULT,        `CORE_LSU_DFAULT_1,     `CORE_LSU_DFAULT_2,
      `CORE_LSU_FP_L_FLAGS,   `CORE_LSU_FP_L_SX,      `CORE_LSU_LAZY_ENTRY,
      `CORE_LSU_LAZY_EXIT,    `CORE_LSU_LAZY_FAULT_1, `CORE_LSU_LAZY_FAULT_2 :
        ahb_hwrited = 1'b0;
      default :
        ahb_hwrited = 1'bx;
    endcase


  always @ (lsu_state_ex_i or instr_data_size_ex_i or nvic_int_invoke_i or
            nvic_int_tail_i or nvic_slp_asleep_i)
    case (lsu_state_ex_i)
      `CORE_LSU_INT_EXIT :
        ahb_hsized = ((nvic_int_invoke_i | nvic_int_tail_i) ?
                      `CORE_HSIZE_8 : `CORE_HSIZE_32);
      `CORE_LSU_SLEEP :
        ahb_hsized = ((nvic_int_invoke_i | nvic_slp_asleep_i) ?
                      `CORE_HSIZE_8 : `CORE_HSIZE_32);
      `CORE_LSU_INT_S_RX,     `CORE_LSU_INT_S_PC,     `CORE_LSU_INT_S_FLAGS,
      `CORE_LSU_FP_S_SX,      `CORE_LSU_INT_S_FPSCR,  `CORE_LSU_INT_L_RX,
      `CORE_LSU_INT_L_R14,    `CORE_LSU_INT_L_PC,     `CORE_LSU_INT_L_FLAGS,
      `CORE_LSU_DBG_R_1,      `CORE_LSU_FP_L_FLAGS,   `CORE_LSU_FP_L_SX,
      `CORE_LSU_LAZY_S_FPSCR, `CORE_LSU_LAZY_S_S0,    `CORE_LSU_LAZY_S_SX :
        ahb_hsized = `CORE_HSIZE_32;
      `CORE_LSU_LSD_N,        `CORE_LSU_LSD_S,        `CORE_LSU_LSM_Rx,
      `CORE_LSU_LSM_NONSEQ,   `CORE_LSU_LSS_N,        `CORE_LSU_LSS_P,
      `CORE_LSU_LSS_B,        `CORE_LSU_LTB_N :
        ahb_hsized = instr_data_size_ex_i;
      `CORE_LSU_IDLE,         `CORE_LSU_LSS_U,        `CORE_LSU_LSS_D,
      `CORE_LSU_LSD_D,        `CORE_LSU_LSM_INTC1,    `CORE_LSU_LSM_INTC2,
      `CORE_LSU_LTB_D,        `CORE_LSU_LTB_W,        `CORE_LSU_ADD4,
      `CORE_LSU_INT_RETURN,   `CORE_LSU_INT_ENTRY,    `CORE_LSU_INT_ENLCK,
      `CORE_LSU_INT_SFAULT_1, `CORE_LSU_INT_SFAULT_2, `CORE_LSU_INT_UFAULT_1,
      `CORE_LSU_INT_UFAULT_2, `CORE_LSU_DBG_PC_1,     `CORE_LSU_DBG_PC_2,
      `CORE_LSU_DBG_R_0,      `CORE_LSU_DBG_R_2,      `CORE_LSU_FAULT,
      `CORE_LSU_DFAULT_1,     `CORE_LSU_DFAULT_2,     `CORE_LSU_LAZY_ENTRY,
      `CORE_LSU_LAZY_EXIT,    `CORE_LSU_LAZY_FAULT_1, `CORE_LSU_LAZY_FAULT_2 :
        ahb_hsized = `CORE_HSIZE_8;
      default :
        ahb_hsized = 2'bxx;
    endcase

  assign ahb_hprotd = { (fpu_lazy_stack_op_i ?
                         ~nvic_fpccr_i[1] :
                         ((privileged_i &
                           (int_stack_op_i | ~lsu_translate_ex)) |
                          (lsu_state_ex_i==`CORE_LSU_DBG_R_1))),
                        1'b1 };

  assign ahb_exreqd      = (lsu_excl_ex & ~int_stack_op_i &
                            ~fpu_lazy_stack_op_i & ~dbg_halted_i);

  assign dpu_addrd_valid = (ahb_htransd[1] & lsu_ahb_rdy_ex &
                            ~mpu_hrespd_i & ~mtx_ahb_hrespd_i[0]);


  assign nvic_lazy_stkalign = (|{fpu_stack_ctl_i, nvic_int_stk_algn_i});


  assign nxt_int_chk_algn_64 = (lsu_stack_start_i & nvic_int_invoke_i &
                                nvic_lazy_stkalign);

  assign int_chk_algn_64_we  = (nxt_int_chk_algn_64 |
                                (int_chk_algn_64 & (lsu_stack_start_i |
                                                    lsu_stack_end_i   |
                                                    ~addr_adder_ex[2])));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      int_chk_algn_64 <= 1'b0;
    else if (int_chk_algn_64_we)
      int_chk_algn_64 <= nxt_int_chk_algn_64;

  assign int_req_algn_64_clr = (int_algn_stk_64_valid & lsu_stack_end_i);

  assign int_req_algn_64_we  = (// Set
                                int_algn_stk_64_valid |
                                int_req_algn_64_clr);

  assign nxt_int_req_algn_64 = int_algn_stk_64_valid & ~int_req_algn_64_clr;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      int_req_algn_64 <= 1'b0;
    else if (int_req_algn_64_we)
      int_req_algn_64 <= nxt_int_req_algn_64;

  assign int_algn_stk_64_valid = ((int_chk_algn_64 & addr_adder_ex[2]) |
                                  int_req_algn_64);

  assign int_algn_unstk_64_set = ((lsu_state_ex_i==`CORE_LSU_INT_L_FLAGS) &
                                  mtx_ahb_hreadyd_i & ~lsu_addracc_wait_reg &
                                  ~lsu_excp_ex & lsu_stack_algn_64_ex);

  assign int_algn_unstk_64_clr = int_algn_unstk_64_valid & lsu_stack_end_i;
  assign int_algn_unstk_64_we  = int_algn_unstk_64_set |  int_algn_unstk_64_clr;
  assign nxt_int_algn_unstk_64 = int_algn_unstk_64_set & ~int_algn_unstk_64_clr;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      int_algn_unstk_64_valid <= 1'b0;
    else if (int_algn_unstk_64_we)
      int_algn_unstk_64_valid <= nxt_int_algn_unstk_64;

  assign int_algn_stk_64 = int_algn_stk_64_valid | int_algn_unstk_64_valid;

  assign lsu_stack_algn_le_64_ex = (|{fpu_unstack_ctl_i, nvic_int_stk_algn_i}) &
                                   ~rf_rd_a_data_ex_i[2]                       &
                                   ~lsu_be8_valid_ex;
  assign lsu_stack_algn_be_64_ex = (|{fpu_unstack_ctl_i, nvic_int_stk_algn_i}) &
                                   ~rf_rd_a_data_ex_i[2]                       &
                                   lsu_be8_valid_ex;
  assign lsu_stack_algn_64_ex    = (lsu_stack_algn_le_64_ex & mtx_ahb_hrdatad_i[9]) |
                                   (lsu_stack_algn_be_64_ex & mtx_ahb_hrdatad_i[17]);


  assign addracc_wait_ex = (mtx_ahb_hreadyd_i & ~mtx_ahb_haddraccd_i &
                            ahb_htransd[1]);

  assign addracc_wait_we = (// Set
                            (~lsu_addracc_wait_reg & addracc_wait_ex) |
                            ( lsu_addracc_wait_reg & lsu_ahb_rdy_ex));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      lsu_addracc_wait_reg <= 1'b0;
    else if (addracc_wait_we)
      lsu_addracc_wait_reg <= addracc_wait_ex;


  assign nxt_lsu_be8_valid_ex = (nvic_data_be8_i &
                                 ~(ahb_haddrd[31:20] == `CORE_PPBREGION));

  assign lsu_be8_valid_we     = (ahb_htransd[1] & lsu_ahb_rdy_ex &
                                 (nxt_lsu_be8_valid_ex ^ lsu_be8_valid_ex));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      lsu_be8_valid_ex <= 1'b0;
    else if (lsu_be8_valid_we)
      lsu_be8_valid_ex <= nxt_lsu_be8_valid_ex;

  assign ahb_ctl_2_data_we = ahb_htransd[1] & lsu_ahb_rdy_ex;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        htransd_reg    <= 1'b0;
        haddrd_reg     <= 2'b00;
        hwrited_reg    <= 1'b0;
        hsized_reg     <= 2'b00;
        instr_sign_reg <= 1'b0;
        fpu_state_reg  <= 1'b0;
        lsm_last_reg   <= 1'b0;
      end
    else if (ahb_ctl_2_data_we)
      begin
        htransd_reg    <= ahb_htransd[1];
        haddrd_reg     <= ahb_haddrd[1:0];
        hwrited_reg    <= ahb_hwrited;
        hsized_reg     <= ahb_hsized;
        instr_sign_reg <= instr_sign_ex_i;
        fpu_state_reg  <= lsu_fpu_ex;
        lsm_last_reg   <= lsu_lsm_last_ex_i;
      end

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      opt_ahb_frc_psr_reg <= 1'b0;
    else if (ahb_ctl_2_data_we & opt_const_en)
      opt_ahb_frc_psr_reg <= ahb_frc_psr_trns;

  assign ahb_frc_psr_reg = opt_ahb_frc_psr_reg & opt_const_en;


  assign b_hrdatad = mtx_ahb_hrdatad_i;

  assign h_hrdatad = (lsu_be8_valid_ex ?
                      { mtx_ahb_hrdatad_i[23:16], mtx_ahb_hrdatad_i[31:24],
                        mtx_ahb_hrdatad_i[7:0],   mtx_ahb_hrdatad_i[15:8]} :
                      mtx_ahb_hrdatad_i);

  assign w_hrdatad = (lsu_be8_valid_ex ?
                      { mtx_ahb_hrdatad_i[7:0],   mtx_ahb_hrdatad_i[15:8],
                        mtx_ahb_hrdatad_i[23:16], mtx_ahb_hrdatad_i[31:24]} :
                      mtx_ahb_hrdatad_i);


  assign str_fwd_test_ex = (// Overlap of LSS allowed, ensure its associated
                            lsu_pline_valid_de_i &
                            ((// FPU load -> store
                              fpu_str_fwd_de_i[1]) |
                             (// Non FPU load
                              lsu_isld_ex & ~lsu_fpu_ex & ~fpu_str_fwd_de_i[0] &
                              (instr_imm_valid_fast_de_i ?
                               (rf_rd_b_addr_de_i==rf_wr_d_addr_ex_i) :
                               (rf_wr_d_addr_de_i==rf_wr_d_addr_ex_i))) |
                             (// Store exclusive
                              (lsu_excl_ex & ~lsu_isld_ex) & ~fpu_str_fwd_de_i[0] &
                              (instr_imm_valid_fast_de_i ?
                               (rf_rd_b_addr_de_i==instr_rf_rd_c_addr_ex_i) :
                               (rf_wr_d_addr_de_i==instr_rf_rd_c_addr_ex_i)))));

  assign set_str_fwd_ex  = (str_fwd_test_ex &
                            (// Aligned case
                             ( mtx_ahb_hreadyd_i & lss_cphase_algn_ex) |
                             (~mtx_ahb_hreadyd_i & lss_cphase_unalgn_ex)));

  assign clr_str_fwd_ex  = lsu_ahb_rdy_ex & poss_str_fwd_ex;

  assign lsu_str_fwd_we  = set_str_fwd_ex | clr_str_fwd_ex;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      poss_str_fwd_ex <= 1'b0;
    else if (lsu_str_fwd_we)
      poss_str_fwd_ex <= set_str_fwd_ex;

  assign lsu_str_fwd_ex = poss_str_fwd_ex & ~lsu_isld_ex;


  assign nxt_int_fix_sp = (// Reading SP for POP and ...
                           (rf_rd_a_addr_ex_i==4'b1101) & lsu_pop_ex &
                           (// Fault detected outside of NMI/HF or ..
                            ((lsu_state_ex_i==`CORE_LSU_LSM_Rx) & ~int_fix_sp &
                             lsu_lfault_ex & ~(|nvic_int_in_nmihf_i[1:0])) |
                            (nvic_int_invoke_i & ~lsu_lfault_ex &
                             mtx_ahb_hreadyd_i & ~lsu_lsm_last_ex_i &
                             ((lsu_state_ex_i==`CORE_LSU_LSM_Rx) |
                              (lsu_state_ex_i==`CORE_LSU_LSD_S)))));

  assign int_fix_sp_we  = (// Set
                           nxt_int_fix_sp |
                           (int_fix_sp & (lsu_state_ex_i==`CORE_LSU_INT_S_PC) &
                            lsu_ahb_rdy_ex));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      int_fix_sp <= 1'b0;
    else if (int_fix_sp_we)
      int_fix_sp <= nxt_int_fix_sp;


  always @ (lsu_state_ex_i or lsu_wb_after_ex or lsu_wb_pos_ex or
            lsu_lsm_reg_ex_i or rf_rd_a_addr_ex_i or lsu_wb_pos_ex)
    case (lsu_state_ex_i)
      `CORE_LSU_LSS_N,        `CORE_LSU_LSS_P,        `CORE_LSU_LSS_B,
      `CORE_LSU_LSD_N :
        data_adder_ctl = ~lsu_wb_pos_ex;
      `CORE_LSU_LSM_NONSEQ :
        data_adder_ctl = ((|lsu_lsm_reg_ex_i) &
                          ((rf_rd_a_addr_ex_i!=4'b1101) |
                           ((rf_rd_a_addr_ex_i==4'b1101) & ~lsu_wb_after_ex))) ?
                         lsu_wb_pos_ex :
                         ~lsu_wb_pos_ex;
      `CORE_LSU_INT_S_PC,     `CORE_LSU_INT_S_RX,     `CORE_LSU_INT_S_FLAGS,
      `CORE_LSU_INT_UFAULT_1, `CORE_LSU_DBG_PC_2,     `CORE_LSU_LAZY_S_S0,
      `CORE_LSU_DFAULT_2 :
        data_adder_ctl = 1'b1;
      `CORE_LSU_IDLE,         `CORE_LSU_LSS_U,        `CORE_LSU_LSS_D,
      `CORE_LSU_LSD_S,        `CORE_LSU_LSD_D,        `CORE_LSU_LSM_INTC1,
      `CORE_LSU_LSM_INTC2,    `CORE_LSU_LSM_Rx,       `CORE_LSU_LTB_N,
      `CORE_LSU_LTB_D,        `CORE_LSU_LTB_W,        `CORE_LSU_ADD4,
      `CORE_LSU_INT_L_PC,     `CORE_LSU_INT_L_RX,     `CORE_LSU_INT_L_R14,
      `CORE_LSU_INT_L_FLAGS,  `CORE_LSU_INT_RETURN,   `CORE_LSU_INT_ENTRY,
      `CORE_LSU_INT_ENLCK,    `CORE_LSU_INT_EXIT,     `CORE_LSU_INT_SFAULT_1,
      `CORE_LSU_INT_SFAULT_2, `CORE_LSU_INT_UFAULT_2, `CORE_LSU_DBG_PC_1,
      `CORE_LSU_DBG_R_0,      `CORE_LSU_DBG_R_1,      `CORE_LSU_DBG_R_2,
      `CORE_LSU_SLEEP,        `CORE_LSU_FAULT,        `CORE_LSU_DFAULT_1,
      `CORE_LSU_INT_S_FPSCR,  `CORE_LSU_FP_S_SX,      `CORE_LSU_FP_L_SX,
      `CORE_LSU_FP_L_FLAGS,   `CORE_LSU_LAZY_ENTRY,   `CORE_LSU_LAZY_EXIT,
      `CORE_LSU_LAZY_S_FPSCR, `CORE_LSU_LAZY_S_SX,    `CORE_LSU_LAZY_FAULT_1,
      `CORE_LSU_LAZY_FAULT_2 :
        data_adder_ctl = 1'b0;
      default :
        data_adder_ctl = 1'bx;
    endcase

  assign data_adder_in1 = (((lsu_state_ex_i==`CORE_LSU_INT_S_PC) & int_fix_sp) ?
                           data_reg_ex :
                           rf_rd_a_data_ex_i);

  always @ (lsu_state_ex_i or lsm_list_length_ex_i or rf_rd_b_data_ex_i or
            instr_immed_ex_i or instr_immed_valid_ex_i or data_reg_ex or
            fpu_stack_ctl_i or fpu_unstack_ctl_i)
    case (lsu_state_ex_i)
      `CORE_LSU_LSS_N,        `CORE_LSU_LSS_P,        `CORE_LSU_LSS_B,
      `CORE_LSU_LSD_N,        `CORE_LSU_ADD4 :
        data_adder_in2 = (instr_immed_valid_ex_i ?
                          instr_immed_ex_i : rf_rd_b_data_ex_i);
      `CORE_LSU_LTB_W :
        data_adder_in2 = {{15{1'b0}},data_reg_ex[15:0],1'b0};
      `CORE_LSU_LSM_NONSEQ :
        data_adder_in2 = {{24{1'b0}}, lsm_list_length_ex_i,   2'b00};
      `CORE_LSU_INT_S_PC :
        data_adder_in2 = (|fpu_stack_ctl_i) ? 32'h00000068 : 32'h00000020;
      `CORE_LSU_SLEEP,        `CORE_LSU_INT_EXIT,     `CORE_LSU_INT_UFAULT_1 :
        data_adder_in2 = (|fpu_unstack_ctl_i) ? 32'h00000068 : 32'h00000020;
      `CORE_LSU_INT_L_FLAGS :
        data_adder_in2 = 32'h00000004;
      `CORE_LSU_IDLE,         `CORE_LSU_LSS_U,        `CORE_LSU_LSS_D,
      `CORE_LSU_LSD_S,        `CORE_LSU_LSD_D,        `CORE_LSU_LSM_INTC1,
      `CORE_LSU_LSM_INTC2,    `CORE_LSU_LSM_Rx,       `CORE_LSU_LTB_N,
      `CORE_LSU_LTB_D,        `CORE_LSU_INT_S_FLAGS,  `CORE_LSU_INT_S_RX,
      `CORE_LSU_INT_L_PC,     `CORE_LSU_INT_L_RX,     `CORE_LSU_INT_L_R14,
      `CORE_LSU_INT_RETURN,   `CORE_LSU_INT_ENTRY,    `CORE_LSU_INT_ENLCK,
      `CORE_LSU_INT_SFAULT_1, `CORE_LSU_INT_SFAULT_2, `CORE_LSU_INT_UFAULT_2,
      `CORE_LSU_DBG_PC_1,     `CORE_LSU_DBG_R_0,      `CORE_LSU_DBG_R_1,
      `CORE_LSU_DBG_R_2,      `CORE_LSU_FAULT,        `CORE_LSU_DFAULT_1,
      `CORE_LSU_DBG_PC_2,     `CORE_LSU_DFAULT_2,     `CORE_LSU_INT_S_FPSCR,
      `CORE_LSU_FP_S_SX,      `CORE_LSU_FP_L_FLAGS,   `CORE_LSU_FP_L_SX,
      `CORE_LSU_LAZY_ENTRY,   `CORE_LSU_LAZY_EXIT,    `CORE_LSU_LAZY_S_FPSCR,
      `CORE_LSU_LAZY_S_S0,    `CORE_LSU_LAZY_S_SX,    `CORE_LSU_LAZY_FAULT_1,
      `CORE_LSU_LAZY_FAULT_2 :
        data_adder_in2 = {32{1'b0}};
      default :
        data_adder_in2 = {32{1'bx}};
    endcase

  assign data_adder_ex = (data_adder_in1 +
                          (data_adder_in2 ^ {32{data_adder_ctl}}) +
                          data_adder_ctl);

  always @ (instr_data_size_ex_i or haddrd_reg or b_hrdatad or h_hrdatad or
            w_hrdatad)
    case ({instr_data_size_ex_i[0],haddrd_reg})
      3'b000  : lsu_tb_data_ex = {{24{1'b0}},b_hrdatad[7:0]};
      3'b001  : lsu_tb_data_ex = {{24{1'b0}},b_hrdatad[15:8]};
      3'b010  : lsu_tb_data_ex = {{24{1'b0}},b_hrdatad[23:16]};
      3'b011  : lsu_tb_data_ex = {{24{1'b0}},b_hrdatad[31:24]};
      3'b100  : lsu_tb_data_ex = {{16{1'b0}},h_hrdatad[15:0]};
      3'b101  : lsu_tb_data_ex = {{16{1'b0}},w_hrdatad[23:8]};
      3'b110  : lsu_tb_data_ex = {{16{1'b0}},h_hrdatad[31:16]};
      3'b111  : lsu_tb_data_ex = {{16{1'b0}},{w_hrdatad[7:0],w_hrdatad[31:24]}};
      default : lsu_tb_data_ex = {32{1'bx}};
    endcase

  always @ (lsu_state_ex_i or mtx_ahb_hreadyd_i or mtx_ahb_haddraccd_i or
            lsu_ahb_rdy_ex or mpu_canceld_i or lsu_wb_valid_ex or
            lsu_addracc_wait_reg or instr_immed_valid_ex_i or
            nvic_int_invoke_i or lsu_str_fwd_ex or lsu_addracc_wait_reg)
    case (lsu_state_ex_i)
      `CORE_LSU_LSS_P,        `CORE_LSU_LSS_B,        `CORE_LSU_LSS_N :
        data_reg_we = (// WB used - maintain in case needed for restore
                       (mtx_ahb_hreadyd_i & mtx_ahb_haddraccd_i &
                        lsu_wb_valid_ex &
                        (mpu_canceld_i | ~lsu_addracc_wait_reg)) |
                       (instr_immed_valid_ex_i & ~lsu_str_fwd_ex &
                        mtx_ahb_hreadyd_i) |
                       (instr_immed_valid_ex_i &  lsu_str_fwd_ex &
                        mtx_ahb_hreadyd_i & ~lsu_addracc_wait_reg));
      `CORE_LSU_LSS_U :
        data_reg_we = mpu_canceld_i;
      `CORE_LSU_LSD_N,        `CORE_LSU_LSM_NONSEQ :
        data_reg_we = lsu_ahb_rdy_ex;
      `CORE_LSU_LTB_D :
        data_reg_we = mtx_ahb_hreadyd_i & ~nvic_int_invoke_i;
      `CORE_LSU_IDLE,         `CORE_LSU_LSS_D,        `CORE_LSU_LSD_S,
      `CORE_LSU_LSD_D,        `CORE_LSU_LSM_INTC1,    `CORE_LSU_LSM_INTC2,
      `CORE_LSU_LSM_Rx,       `CORE_LSU_LTB_N,        `CORE_LSU_LTB_W,
      `CORE_LSU_ADD4,         `CORE_LSU_INT_S_PC,     `CORE_LSU_INT_S_RX,
      `CORE_LSU_INT_S_FLAGS,  `CORE_LSU_INT_L_PC,     `CORE_LSU_INT_L_RX,
      `CORE_LSU_INT_L_R14,    `CORE_LSU_INT_L_FLAGS,  `CORE_LSU_INT_RETURN,
      `CORE_LSU_INT_ENTRY,    `CORE_LSU_INT_ENLCK,    `CORE_LSU_INT_EXIT,
      `CORE_LSU_INT_SFAULT_1, `CORE_LSU_INT_SFAULT_2, `CORE_LSU_INT_UFAULT_1,
      `CORE_LSU_INT_UFAULT_2, `CORE_LSU_DBG_PC_1,     `CORE_LSU_DBG_PC_2,
      `CORE_LSU_DBG_R_0,      `CORE_LSU_DBG_R_1,      `CORE_LSU_DBG_R_2,
      `CORE_LSU_SLEEP,        `CORE_LSU_FAULT,        `CORE_LSU_DFAULT_1,
      `CORE_LSU_DFAULT_2,     `CORE_LSU_INT_S_FPSCR,  `CORE_LSU_FP_S_SX,
      `CORE_LSU_FP_L_FLAGS,   `CORE_LSU_FP_L_SX,      `CORE_LSU_LAZY_ENTRY,
      `CORE_LSU_LAZY_EXIT,    `CORE_LSU_LAZY_S_FPSCR, `CORE_LSU_LAZY_S_S0,
      `CORE_LSU_LAZY_S_SX,    `CORE_LSU_LAZY_FAULT_1, `CORE_LSU_LAZY_FAULT_2 :
        data_reg_we = 1'b0;
      default :
        data_reg_we = 1'bx;
    endcase

  assign lsu_msk_rf_wr_d_data_ex = {lsu_rf_wr_d_data_ex[31:2],
                                    lsu_rf_wr_d_data_ex[1:0] &
                                    ~{2{instr_immed_valid_ex_i ?
                                        (rf_rd_b_addr_ex_i==4'b1101) :
                                        (rf_wr_d_addr_ex_i==4'b1101)}}};

  always @ (lsu_state_ex_i or lsu_str_fwd_ex or lsu_msk_rf_wr_d_data_ex or
            instr_immed_valid_ex_i or lsu_isld_ex or fpu_wr_data_ex_i or
            lsu_st_imm_nonbuf_ex_i or lsu_wb_pos_ex or lsu_wb_valid_ex or
            mpu_canceld_i or rf_rd_b_data_ex_i or rf_rd_a_data_ex_i or
            lsu_lsm_reg_ex_i or rf_rd_a_addr_ex_i or lsu_fpu_ex or
            instr_lsm_baseinlist_ex_i or data_adder_ex or lsu_tb_data_ex)
    case (lsu_state_ex_i)
      `CORE_LSU_LSS_P,        `CORE_LSU_LSS_B,        `CORE_LSU_LSS_N :
        nxt_data_reg_ex = ((lsu_str_fwd_ex & ~lsu_st_imm_nonbuf_ex_i &
                            ~mpu_canceld_i) ?
                           lsu_msk_rf_wr_d_data_ex :
                           (// Store immediate write data buffering
                            (instr_immed_valid_ex_i & ~lsu_isld_ex &
                             ~lsu_st_imm_nonbuf_ex_i & ~mpu_canceld_i) ?
                            (lsu_fpu_ex ? fpu_wr_data_ex_i : rf_rd_b_data_ex_i) :
                            rf_rd_a_data_ex_i));
      `CORE_LSU_LSS_U,        `CORE_LSU_LSD_N :
        nxt_data_reg_ex = rf_rd_a_data_ex_i;
      `CORE_LSU_LSM_NONSEQ :
        nxt_data_reg_ex = (((|lsu_lsm_reg_ex_i) & lsu_wb_valid_ex &
                            ~(lsu_isld_ex & instr_lsm_baseinlist_ex_i) &
                            (rf_rd_a_addr_ex_i!=4'b1101 | ~lsu_wb_pos_ex)) ?
                           data_adder_ex : rf_rd_a_data_ex_i);
      `CORE_LSU_LTB_D :
        nxt_data_reg_ex = lsu_tb_data_ex;
      default :
        nxt_data_reg_ex = {32{1'bx}};
    endcase

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      data_reg_ex <= {32{1'b0}};
    else if (data_reg_we)
      data_reg_ex <= nxt_data_reg_ex;

  assign nxt_str_stat_valid_ex = (~lsu_isld_ex & ~lsu_excp_ex &
                                  mtx_ahb_haddraccd_i &
                                  ((lsu_state_ex_i==`CORE_LSU_LSS_N) |
                                   (lsu_state_ex_i==`CORE_LSU_LSS_P) |
                                   (lsu_state_ex_i==`CORE_LSU_LSS_B)));

  assign nxt_str_stat_ex       = ({3{nxt_str_stat_valid_ex}} &
                                  { // Non-exclusive & bufferable
                                    (~mpu_addrd_non_buf_i & ~lsu_excl_ex),
                                    (instr_immed_valid_ex_i & ~mpu_canceld_i &
                                     ~(mpu_addrd_non_buf_i & lsu_wb_valid_ex)),
                                    (instr_immed_valid_ex_i & ~mpu_canceld_i &
                                     mpu_addrd_non_buf_i & lsu_wb_valid_ex) });

  assign str_stat_we           = (mtx_ahb_hreadyd_i &
                                  (// Set
                                  nxt_str_stat_valid_ex |
                                  str_stat_valid_ex));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        str_stat_valid_ex <= 1'b0;
        str_stat_ex       <= 3'b000;
      end
    else if (str_stat_we)
      begin
        str_stat_valid_ex <= nxt_str_stat_valid_ex;
        str_stat_ex       <= nxt_str_stat_ex;
      end

  assign str_buf_imm_ex = str_stat_valid_ex & (&str_stat_ex[2:1]);



  assign nxt_str_fwd_valid_ex = (opt_ext_en & mpu_addrd_non_buf_i &
                                 (lsu_state_ex_i == `CORE_LSU_LSS_B) &
                                 lsu_str_fwd_ex & lsu_wb_valid_ex &
                                 str_imm_ex & mtx_ahb_hreadyd_i &
                                 ~lsu_excp_ex);

  assign str_fwd_valid_we     = ((nxt_str_fwd_valid_ex & mtx_ahb_haddraccd_i) |
                                 (str_fwd_valid_ex & mtx_ahb_hreadyd_i));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      opt_str_fwd_valid_ex <= 1'b0;
    else if (str_fwd_valid_we)
      opt_str_fwd_valid_ex <= nxt_str_fwd_valid_ex;

  assign str_fwd_valid_ex     = opt_str_fwd_valid_ex & opt_ext_en;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      str_fwd_reg_ex <= {32{1'b0}};
    else if (nxt_str_fwd_valid_ex & ~lsu_addracc_wait_reg)
      str_fwd_reg_ex <= lsu_msk_rf_wr_d_data_ex;


  assign sel_str_reg_ex = ((str_stat_valid_ex & str_stat_ex[1]) |
                           ~lsu_active_ex |
                           (((lsu_state_ex_i==`CORE_LSU_LSM_Rx) &
                             (lsu_lsm_reg_ex_i[3:0]==rf_rd_a_addr_ex_i)) |
                            ((lsu_state_ex_i==`CORE_LSU_LSD_S) &
                             (rf_rd_a_addr_ex_i==rf_rd_b_addr_ex_i) &
                             lsu_wb_valid_ex & lsu_wb_after_ex) |
                            ((lsu_state_ex_i==`CORE_LSU_LSD_D) &
                             (rf_rd_a_addr_ex_i==rf_wr_d_addr_ex_i) &
                             lsu_wb_valid_ex & lsu_wb_after_ex)));

  assign str_data_reg_ex = (({32{sel_str_reg_ex  }} & data_reg_ex) |
                            ({32{str_fwd_valid_ex}} & str_fwd_reg_ex));

  assign str_mux_ex = ((sel_str_reg_ex | str_fwd_valid_ex) ? str_data_reg_ex :
                       (str_stat_ex[0] ? rf_rd_b_data_ex_i : rf_rd_a_data_ex_i));


  always @ (lsu_be8_valid_ex or haddrd_reg or str_mux_ex)
    case ({lsu_be8_valid_ex, haddrd_reg})
      3'b000, 3'b100 :
        str_mux_rot_ex = {str_mux_ex[31:24],str_mux_ex[23:16],
                          str_mux_ex[15:8], str_mux_ex[7:0]};
      3'b001, 3'b111 :
        str_mux_rot_ex = {str_mux_ex[23:16],str_mux_ex[15:8],
                          str_mux_ex[7:0],  str_mux_ex[31:24]};
      3'b010, 3'b110 :
        str_mux_rot_ex = {str_mux_ex[15:8], str_mux_ex[7:0],
                          str_mux_ex[31:24],str_mux_ex[23:16]};
      3'b011, 3'b101 :
        str_mux_rot_ex = {str_mux_ex[7:0],  str_mux_ex[31:24],
                          str_mux_ex[23:16],str_mux_ex[15:8]};
      default :
        str_mux_rot_ex = {32{1'bx}};
    endcase

  always @ (hsized_reg or str_mux_ex or haddrd_reg or str_mux_rot_ex)
    case (hsized_reg)
      2'b00 :
        str_mux_rep_ex = {str_mux_ex[7:0],str_mux_ex[7:0],
                          str_mux_ex[7:0],str_mux_ex[7:0]};
      2'b01 :
        str_mux_rep_ex = (haddrd_reg[0] ?
                          {2{str_mux_ex[7:0],  str_mux_ex[15:8]}} :
                          {2{str_mux_ex[15:8], str_mux_ex[7:0]}});
      2'b10, 2'b11 :
        str_mux_rep_ex = str_mux_rot_ex;
      default:
        str_mux_rep_ex = {32{1'bx}};
    endcase

  assign str_mux_mask_ex = {32{str_stat_valid_ex}} & str_mux_rep_ex;

  assign dbg_special_reg = {5'b00000,         fpu_active_ex_i, process_sp_ex_i[0],
                            user_thread_ex_i, 7'b0000000,      nvic_fmask_i,
                            nvic_basepri_i,   7'b0000000,      nvic_pmask_i};

  assign str_psr_ex      = { psr_ex_i[31:16],
                             ((|psr_ex_i[11:10]) | (|psr_ex_i[26:25]) |
                              (nvic_int_invoke_i & ~dbg_halted_i)) ?
                             psr_ex_i[15:12] :
                             lsu_lsm_reg_ex_i[3:0],
                             psr_ex_i[11:10],
                             int_algn_stk_64,
                             int_isr_ex_i };

  always @ (lsu_state_ex_i or hwrited_reg or str_mux_ex or rf_rd_a_data_ex_i or
            str_psr_ex or dbg_special_reg or str_mux_mask_ex or
            nvic_dbg_reg_wr_i or nvic_dbg_reg_addr_i or lsu_lsm_reg_ex_i or
            fpu_wr_data_ex_i or fpu_stack_ctl_i or fpu_state_reg or
            fpu_enabled_i or opt_fpu_en)
    case (lsu_state_ex_i)
      `CORE_LSU_DBG_R_2 :
        str_data_ex = nvic_dbg_reg_wr_i ?
                      str_mux_mask_ex :
                      ((|nvic_dbg_reg_addr_i[6:5]) ?
                       fpu_wr_data_ex_i :
                       ((nvic_dbg_reg_addr_i[4] & ~(|nvic_dbg_reg_addr_i[1:0])) ?
                        (nvic_dbg_reg_addr_i[2] ?
                         dbg_special_reg : str_psr_ex) :
                        rf_rd_a_data_ex_i));
      `CORE_LSU_INT_S_RX :
        str_data_ex = ((|lsu_lsm_reg_ex_i[3:0]) ?
                       rf_rd_a_data_ex_i : str_psr_ex);
      `CORE_LSU_FP_S_SX,      `CORE_LSU_LAZY_S_S0,    `CORE_LSU_LAZY_S_SX :
        str_data_ex = ((|lsu_lsm_reg_ex_i[4:0]) ?
                       fpu_wr_data_ex_i : rf_rd_a_data_ex_i);
      `CORE_LSU_INT_S_FPSCR,  `CORE_LSU_LAZY_EXIT,    `CORE_LSU_LAZY_S_FPSCR :
        str_data_ex = fpu_wr_data_ex_i;
      `CORE_LSU_INT_S_FLAGS,  `CORE_LSU_INT_ENLCK :
        str_data_ex = rf_rd_a_data_ex_i;
      `CORE_LSU_INT_ENTRY :
        str_data_ex = ((fpu_stack_ctl_i[1] & fpu_enabled_i) ?
                       fpu_wr_data_ex_i : rf_rd_a_data_ex_i);
      `CORE_LSU_LSD_S,        `CORE_LSU_LSD_D,        `CORE_LSU_LSM_Rx :
        str_data_ex = {32{hwrited_reg}} &
                      ((fpu_state_reg & opt_fpu_en) ?
                       fpu_wr_data_ex_i : str_mux_ex);
      `CORE_LSU_IDLE,         `CORE_LSU_LSS_N,        `CORE_LSU_LSS_P,
      `CORE_LSU_LSS_B,        `CORE_LSU_LSS_U,        `CORE_LSU_LSS_D,
      `CORE_LSU_LSD_N,        `CORE_LSU_LSM_INTC1,    `CORE_LSU_LSM_INTC2,
      `CORE_LSU_LSM_NONSEQ,   `CORE_LSU_LTB_N,        `CORE_LSU_LTB_D,
      `CORE_LSU_LTB_W,        `CORE_LSU_ADD4,         `CORE_LSU_INT_S_PC,
      `CORE_LSU_INT_L_PC,     `CORE_LSU_INT_L_RX,     `CORE_LSU_INT_L_R14,
      `CORE_LSU_INT_L_FLAGS,  `CORE_LSU_INT_RETURN,   `CORE_LSU_INT_EXIT,
      `CORE_LSU_INT_SFAULT_1, `CORE_LSU_INT_SFAULT_2, `CORE_LSU_INT_UFAULT_1,
      `CORE_LSU_INT_UFAULT_2, `CORE_LSU_DBG_PC_1,     `CORE_LSU_DBG_PC_2,
      `CORE_LSU_DBG_R_0,      `CORE_LSU_DBG_R_1,      `CORE_LSU_SLEEP,
      `CORE_LSU_FAULT,        `CORE_LSU_DFAULT_1,     `CORE_LSU_DFAULT_2,
      `CORE_LSU_FP_L_SX,      `CORE_LSU_FP_L_FLAGS,   `CORE_LSU_LAZY_ENTRY,
      `CORE_LSU_LAZY_FAULT_1, `CORE_LSU_LAZY_FAULT_2 :
        str_data_ex = str_mux_mask_ex;
      default :
        str_data_ex = {32{1'bx}};
    endcase

  assign ahb_hwdatad = (lsu_be8_valid_ex ?
                        { str_data_ex[7:0],   str_data_ex[15:8],
                          str_data_ex[23:16], str_data_ex[31:24] } :
                        str_data_ex);



  always @ (haddrd_reg or hsized_reg or instr_sign_reg or lsu_be8_valid_ex)
    case ({hsized_reg, instr_sign_reg, haddrd_reg, lsu_be8_valid_ex})
      `def_00000x : rdata_sel = 24'b0000_0000_0000_0001_0000_0000;
      `def_00001x : rdata_sel = 24'b0000_0000_0000_0010_0000_0000;
      `def_00010x : rdata_sel = 24'b0000_0000_0000_0100_0000_0000;
      `def_00011x : rdata_sel = 24'b0000_0000_0000_1000_0000_0000;
      `def_00100x : rdata_sel = 24'b0000_0000_0000_0001_0001_0001;
      `def_00101x : rdata_sel = 24'b0000_0000_0000_0010_0010_0010;
      `def_00110x : rdata_sel = 24'b0000_0000_0000_0100_0100_0100;
      `def_00111x : rdata_sel = 24'b0000_0000_0000_1000_1000_1000;
      6'b010000   : rdata_sel = 24'b0000_0000_0010_0001_0000_0000;
      6'b010001   : rdata_sel = 24'b0000_0000_0001_0010_0000_0000;
      6'b010010   : rdata_sel = 24'b0000_0000_0100_0010_0000_0000;
      6'b010011   : rdata_sel = 24'b0000_0000_0010_0100_0000_0000;
      6'b010100   : rdata_sel = 24'b0000_0000_1000_0100_0000_0000;
      6'b010101   : rdata_sel = 24'b0000_0000_0100_1000_0000_0000;
      6'b010110   : rdata_sel = 24'b0000_0000_0001_1000_0000_0000;
      6'b010111   : rdata_sel = 24'b0000_0000_1000_0001_0000_0000;
      6'b011000   : rdata_sel = 24'b0000_0000_0010_0001_0010_0000;
      6'b011001   : rdata_sel = 24'b0000_0000_0001_0010_0001_0000;
      6'b011010   : rdata_sel = 24'b0000_0000_0100_0010_0100_0000;
      6'b011011   : rdata_sel = 24'b0000_0000_0010_0100_0010_0000;
      6'b011100   : rdata_sel = 24'b0000_0000_1000_0100_1000_0000;
      6'b011101   : rdata_sel = 24'b0000_0000_0100_1000_0100_0000;
      6'b011110   : rdata_sel = 24'b0000_0000_0001_1000_0001_0000;
      6'b011111   : rdata_sel = 24'b0000_0000_1000_0001_1000_0000;
      `def_1xx000 : rdata_sel = 24'b1000_0100_0010_0001_0000_0000;
      `def_1xx001 : rdata_sel = 24'b0001_0010_0100_1000_0000_0000;
      `def_1xx010 : rdata_sel = 24'b0001_1000_0100_0010_0000_0000;
      `def_1xx011 : rdata_sel = 24'b0010_0100_1000_0001_0000_0000;
      `def_1xx100 : rdata_sel = 24'b0010_0001_1000_0100_0000_0000;
      `def_1xx101 : rdata_sel = 24'b0100_1000_0001_0010_0000_0000;
      `def_1xx110 : rdata_sel = 24'b0100_0010_0001_1000_0000_0000;
      `def_1xx111 : rdata_sel = 24'b1000_0001_0010_0100_0000_0000;
      default     : rdata_sel = {24{1'bx}};
    endcase

  assign rdata_sign_hword = ((rdata_sel[4] & mtx_ahb_hrdatad_i[7])  |
                             (rdata_sel[5] & mtx_ahb_hrdatad_i[15]) |
                             (rdata_sel[6] & mtx_ahb_hrdatad_i[23]) |
                             (rdata_sel[7] & mtx_ahb_hrdatad_i[31]));

  assign rdata_sign_byte = ((rdata_sel[0] & mtx_ahb_hrdatad_i[7])  |
                            (rdata_sel[1] & mtx_ahb_hrdatad_i[15]) |
                            (rdata_sel[2] & mtx_ahb_hrdatad_i[23]) |
                            (rdata_sel[3] & mtx_ahb_hrdatad_i[31]));

  assign rdata_sign_mask_ex = { {16{rdata_sign_hword}},
                                { 8{rdata_sign_byte}},
                                { 8{1'b0}} };

  assign rdata_ex[7:0]   = (({8{rdata_sel[8]}}  & mtx_ahb_hrdatad_i[7:0])   |
                            ({8{rdata_sel[9]}}  & mtx_ahb_hrdatad_i[15:8])  |
                            ({8{rdata_sel[10]}} & mtx_ahb_hrdatad_i[23:16]) |
                            ({8{rdata_sel[11]}} & mtx_ahb_hrdatad_i[31:24]));

  assign rdata_ex[15:8]  = (({8{rdata_sel[12]}} & mtx_ahb_hrdatad_i[7:0])   |
                            ({8{rdata_sel[13]}} & mtx_ahb_hrdatad_i[15:8])  |
                            ({8{rdata_sel[14]}} & mtx_ahb_hrdatad_i[23:16]) |
                            ({8{rdata_sel[15]}} & mtx_ahb_hrdatad_i[31:24]));

  assign rdata_ex[23:16] = (({8{rdata_sel[16]}} & mtx_ahb_hrdatad_i[7:0])   |
                            ({8{rdata_sel[17]}} & mtx_ahb_hrdatad_i[15:8])  |
                            ({8{rdata_sel[18]}} & mtx_ahb_hrdatad_i[23:16]) |
                            ({8{rdata_sel[19]}} & mtx_ahb_hrdatad_i[31:24]));

  assign rdata_ex[31:24] = (({8{rdata_sel[20]}} & mtx_ahb_hrdatad_i[7:0])   |
                            ({8{rdata_sel[21]}} & mtx_ahb_hrdatad_i[15:8])  |
                            ({8{rdata_sel[22]}} & mtx_ahb_hrdatad_i[23:16]) |
                            ({8{rdata_sel[23]}} & mtx_ahb_hrdatad_i[31:24]));

  assign lsu_rd_data_ex  = rdata_ex | rdata_sign_mask_ex;


  assign fwd_a_from_d_ex   = (lsu_will_wr_rf_ex_i &
                              (lsu_rf_wr_d_addr_ex_i==rf_rd_a_addr_de_i));

  assign fwd_a_from_z_ex   = (lsu_rf_wr_z_en_ex_i & opt_ext_en &
                              (lsu_rf_wr_z_addr_ex_i==rf_rd_a_addr_de_i));

  assign lsu_a_fwd_data_ex = ((fwd_a_from_d_ex | ~opt_ext_en) ?
                              lsu_rd_data_ex : lsu_rf_wr_z_data_ex);

  assign rdata_addr_fwd_a_ex = ((~nvic_int_invoke_i &
                                 (fwd_a_from_d_ex | fwd_a_from_z_ex)) ?
                                { lsu_a_fwd_data_ex[31:2],
                                  ({2{rf_rd_a_addr_de_i!=4'b1101}} &
                                   lsu_a_fwd_data_ex[1:0])} :
                                rf_rd_a_data_de_i);

  assign fwd_b_from_d_ex   = (lsu_will_wr_rf_ex_i &
                              (lsu_rf_wr_d_addr_ex_i==rf_rd_b_addr_de_i));

  assign fwd_b_from_z_ex   = (lsu_rf_wr_z_en_ex_i & opt_ext_en &
                              (lsu_rf_wr_z_addr_ex_i==rf_rd_b_addr_de_i));

  assign lsu_b_fwd_data_ex = ((fwd_b_from_d_ex | ~opt_ext_en) ?
                              lsu_rd_data_ex : lsu_rf_wr_z_data_ex);

  assign rdata_addr_fwd_b_ex = ((~nvic_int_invoke_i &
                                 (fwd_b_from_d_ex | fwd_b_from_z_ex)) ?
                                { lsu_b_fwd_data_ex[31:2],
                                 ({2{rf_rd_b_addr_de_i!=4'b1101}} &
                                  lsu_b_fwd_data_ex[1:0])} :
                                rf_rd_b_data_de_i);

  assign rdata_addr_b_lsl_ex = (rdata_addr_fwd_b_ex << lsu_immshft_de);


  assign nxt_lsu_ld_excl_ex = (lsu_excl_ex & lsu_isld_ex & lsu_ahb_rdy_ex &
                               ~lsu_excp_ex & lss_cphase_algn_ex);

  assign lsu_ld_excl_we     = (nxt_lsu_ld_excl_ex |
                               (lsu_ld_excl_ex & mtx_ahb_hreadyd_i &
                                ((lsu_state_ex_i==`CORE_LSU_LSS_P) |
                                 (lsu_state_ex_i==`CORE_LSU_LSS_U) |
                                 (lsu_state_ex_i==`CORE_LSU_LSS_D))));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      lsu_ld_excl_ex <= 1'b0;
    else if (lsu_ld_excl_we)
      lsu_ld_excl_ex <= nxt_lsu_ld_excl_ex;


  assign nxt_lsu_st_excl_ex = { ~lsu_excl_mon_ex_i,
                                (lsu_excl_ex & ~lsu_isld_ex & lsu_ahb_rdy_ex &
                                 ~lsu_excp_ex & lss_cphase_algn_ex) };

  assign lsu_st_excl_we     = (nxt_lsu_st_excl_ex[0] |
                               (lsu_st_excl_ex[0] & mtx_ahb_hreadyd_i &
                                ((lsu_state_ex_i==`CORE_LSU_LSS_P) |
                                 (lsu_state_ex_i==`CORE_LSU_LSS_U) |
                                 (lsu_state_ex_i==`CORE_LSU_LSS_D))));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      lsu_st_excl_ex <= 2'b00;
    else if (lsu_st_excl_we)
      lsu_st_excl_ex <= nxt_lsu_st_excl_ex;


  assign lsm_passed_brestore_ex = (lsu_isld_ex & nvic_int_invoke_i &
                                   ~lsu_lsm_last_ex_i &
                                   lsu_lsm_base_status_ex_i[0]);

  assign lsm_curreg_brestore_ex = (lsu_isld_ex & nvic_int_invoke_i &
                                   ~lsu_lsm_last_ex_i &
                                   lsu_lsm_base_status_ex_i[1]);


  always @ (lsu_state_ex_i or data_adder_ex or w_hrdatad or data_reg_ex or
            int_lr_exit_code_ex_i or int_algn_stk_64 or nvic_int_invoke_i or
            lsu_lfault_ex or lsu_wb_valid_ex or data_reg_ex or lsu_excp_ex or
            lsu_lsm_last_ex_i or instr_itblock_ex_i or fault_lsu_resp_ex or
            lsu_st_excl_ex or mtx_ahb_exrespd_i or lsu_addracc_wait_reg or
            instr_lsm_baseinlist_ex_i or lsm_passed_brestore_ex or
            lsm_curreg_brestore_ex or lsu_rd_data_ex or lsm_fpu_brestore_ex_i)
    case (lsu_state_ex_i)
      `CORE_LSU_DBG_PC_2,     `CORE_LSU_LAZY_S_S0,    `CORE_LSU_DFAULT_2,
      `CORE_LSU_INT_L_FLAGS,  `CORE_LSU_SLEEP,        `CORE_LSU_INT_EXIT,
      `CORE_LSU_LSM_NONSEQ,   `CORE_LSU_LTB_W,        `CORE_LSU_ADD4 :
        lsu_rf_wr_d_data_ex = data_adder_ex;
      `CORE_LSU_DBG_R_2,      `CORE_LSU_INT_L_RX,     `CORE_LSU_INT_L_R14,
      `CORE_LSU_INT_L_PC :
        lsu_rf_wr_d_data_ex = w_hrdatad;
      `CORE_LSU_INT_S_PC,     `CORE_LSU_INT_UFAULT_1:
        lsu_rf_wr_d_data_ex = { data_adder_ex[31:3],
                                data_adder_ex[2] & ~int_algn_stk_64,
                                data_adder_ex[1:0] };
      `CORE_LSU_INT_ENTRY:
        lsu_rf_wr_d_data_ex = { {27{1'b1}}, int_lr_exit_code_ex_i };
      `CORE_LSU_INT_SFAULT_2, `CORE_LSU_INT_UFAULT_2, `CORE_LSU_LAZY_FAULT_2 :
        lsu_rf_wr_d_data_ex = 32'hFFFFFFFF;
      `CORE_LSU_LSD_S:
        lsu_rf_wr_d_data_ex = ((lsu_excp_ex &
                                (lsu_wb_valid_ex | instr_lsm_baseinlist_ex_i)) ?
                               data_reg_ex : w_hrdatad);
      `CORE_LSU_LSD_D:
        lsu_rf_wr_d_data_ex = ((lsu_lfault_ex &
                                (lsu_wb_valid_ex | instr_lsm_baseinlist_ex_i)) ?
                               data_reg_ex : w_hrdatad);
      `CORE_LSU_LSM_Rx:
        lsu_rf_wr_d_data_ex = ((lsu_lfault_ex |
                                lsm_passed_brestore_ex |
                                (lsm_curreg_brestore_ex & lsu_addracc_wait_reg) |
                                ((nvic_int_invoke_i & ~lsu_lsm_last_ex_i) &
                                  ((instr_itblock_ex_i & lsu_wb_valid_ex) |
                                   lsm_fpu_brestore_ex_i))) ?
                              data_reg_ex : w_hrdatad);
      `CORE_LSU_LSS_N,        `CORE_LSU_LSD_N:
        lsu_rf_wr_d_data_ex = data_adder_ex & {32{lsu_wb_valid_ex}};
      `CORE_LSU_LSS_P,        `CORE_LSU_LSS_B,        `CORE_LSU_LSS_U,
      `CORE_LSU_LSS_D :
        lsu_rf_wr_d_data_ex = (fault_lsu_resp_ex ? data_reg_ex :
                               lsu_st_excl_ex[0] ?
                               {{31{1'b0}},
                                lsu_st_excl_ex[0] &
                                (lsu_st_excl_ex[1] | mtx_ahb_exrespd_i) } :
                               lsu_rd_data_ex);
      `CORE_LSU_IDLE,         `CORE_LSU_LSM_INTC1,    `CORE_LSU_LSM_INTC2,
      `CORE_LSU_LTB_N,        `CORE_LSU_LTB_D,        `CORE_LSU_INT_S_RX,
      `CORE_LSU_INT_S_FLAGS,  `CORE_LSU_INT_RETURN,   `CORE_LSU_INT_ENLCK,
      `CORE_LSU_INT_SFAULT_1, `CORE_LSU_DBG_PC_1,     `CORE_LSU_DBG_R_0,
      `CORE_LSU_DBG_R_1,      `CORE_LSU_FAULT,        `CORE_LSU_DFAULT_1,
      `CORE_LSU_INT_S_FPSCR,  `CORE_LSU_FP_S_SX,      `CORE_LSU_FP_L_FLAGS,
      `CORE_LSU_FP_L_SX,      `CORE_LSU_LAZY_ENTRY,   `CORE_LSU_LAZY_EXIT,
      `CORE_LSU_LAZY_S_FPSCR, `CORE_LSU_LAZY_S_SX,    `CORE_LSU_LAZY_FAULT_1 :
        lsu_rf_wr_d_data_ex = {32{1'b0}};
      default :
        lsu_rf_wr_d_data_ex = {32{1'bx}};
    endcase

  assign lsu_rf_wr_z_data_ex = (// SP alignment
                                { data_adder_ex[31:2],
                                  ({2{lsu_rf_wr_z_addr_ex_i!=4'b1101}} &
                                   data_adder_ex[1:0]) } &
                                 {32{lsu_wb_valid_ex}} &
                                {32{lsu_state_ex_i == `CORE_LSU_LSS_B}});

  assign lsu_fpu_rd_data_ex = w_hrdatad;


  assign dbg_psr_wr_ex   = ((lsu_state_ex_i == `CORE_LSU_DBG_R_2) &
                            mtx_ahb_hreadyd_i & nvic_dbg_reg_wr_i &
                            (nvic_dbg_reg_addr_i == 7'b001_0000));

  assign lsu_flag_wr_ex  = (// Debug write to PSR
                            dbg_psr_wr_ex |
                            ((lsu_state_ex_i == `CORE_LSU_INT_L_FLAGS) &
                             mtx_ahb_hreadyd_i & ~lsu_addracc_wait_reg &
                             ~lsu_excp_ex));

  assign lsu_psr_ex      = w_hrdatad;

  assign dbg_wr_isr_we   = (// Set
                            dbg_psr_wr_ex |
                            dbg_wr_isr);

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      dbg_wr_isr <= 1'b0;
    else if (dbg_wr_isr_we)
      dbg_wr_isr <= dbg_psr_wr_ex;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      int_retisr <= 9'b000000000;
    else if (lsu_flag_wr_ex)
      int_retisr <= w_hrdatad[8:0];

  assign dbg_ctrl_wr_ex  = ((lsu_state_ex_i==`CORE_LSU_DBG_R_2) &
                            mtx_ahb_hreadyd_i & nvic_dbg_reg_wr_i &
                            (nvic_dbg_reg_addr_i == 7'b001_0100));


  assign hrdatad_1_det = { (&mtx_ahb_hrdatad_i[31:28]), (&mtx_ahb_hrdatad_i[27:24]),
                           (&mtx_ahb_hrdatad_i[23:20]), (&mtx_ahb_hrdatad_i[19:16]),
                           (&mtx_ahb_hrdatad_i[15:12]), (&mtx_ahb_hrdatad_i[11:8]),
                           (&mtx_ahb_hrdatad_i[7:4]),   (&mtx_ahb_hrdatad_i[3:0]) };

  assign lsu_exit_we   = lsu_exit_valid_ex | nxt_lsu_exit_valid_ex;

  assign lsu_exit_in_b0 = (((haddrd_reg==2'b00) & ~lsu_be8_valid_ex) |
                           ((haddrd_reg==2'b01) &  lsu_be8_valid_ex));

  assign lsu_exit_in_b1 = (((haddrd_reg==2'b01) & ~lsu_be8_valid_ex) |
                           ((haddrd_reg==2'b10) &  lsu_be8_valid_ex));

  assign lsu_exit_in_b2 = (((haddrd_reg==2'b10) & ~lsu_be8_valid_ex) |
                           ((haddrd_reg==2'b11) &  lsu_be8_valid_ex));

  assign lsu_exit_in_b3 = (((haddrd_reg==2'b11) & ~lsu_be8_valid_ex) |
                           ((haddrd_reg==2'b00) &  lsu_be8_valid_ex));

  assign lsu_exit_ones  = (opt_fpu_en ?
                           (// Exit in byte 0
                            (lsu_exit_in_b0 &
                             (&hrdatad_1_det[7:2]) & (&mtx_ahb_hrdatad_i[7:5])) |
                            (lsu_exit_in_b1 &
                             (&hrdatad_1_det[7:4]) & (&hrdatad_1_det[1:0]) &
                             (&mtx_ahb_hrdatad_i[15:13])) |
                            (lsu_exit_in_b2 &
                             (&hrdatad_1_det[7:6]) & (&hrdatad_1_det[3:0]) &
                             (&mtx_ahb_hrdatad_i[23:21])) |
                            (lsu_exit_in_b3 &
                             (&hrdatad_1_det[5:0]) &
                             (&mtx_ahb_hrdatad_i[31:29]))) :
                           (// Exit in byte 0
                           (lsu_exit_in_b0 &
                            (&hrdatad_1_det[7:1])) |
                           (lsu_exit_in_b1 &
                            (&hrdatad_1_det[7:3]) & (&hrdatad_1_det[1:0])) |
                           (lsu_exit_in_b2 &
                            (&hrdatad_1_det[7:5]) & (&hrdatad_1_det[3:0])) |
                           (lsu_exit_in_b3 &
                            (hrdatad_1_det[7]) & (&hrdatad_1_det[5:0]))));

  assign nxt_lsu_exit_code_ex = (({5{lsu_exit_in_b0}} & mtx_ahb_hrdatad_i[ 4: 0]) |
                                 ({5{lsu_exit_in_b1}} & mtx_ahb_hrdatad_i[12: 8]) |
                                 ({5{lsu_exit_in_b2}} & mtx_ahb_hrdatad_i[20:16]) |
                                 ({5{lsu_exit_in_b3}} & mtx_ahb_hrdatad_i[28:24]));

  assign nxt_lsu_exit_valid_ex = (instr_exit_op_ex_i & mtx_ahb_hreadyd_i &
                                  ~lsu_lfault_ex & lsu_exit_ones &
                                  (// LSM PC write
                                   ((&lsu_lsm_reg_ex_i[3:0])   &
                                    (lsu_state_ex_i==`CORE_LSU_LSM_Rx)) |
                                   ((&rf_rd_b_addr_ex_i)  &
                                    (lsu_state_ex_i==`CORE_LSU_LSD_S)) |
                                   ((&rf_wr_d_addr_ex_i)  &
                                    (lsu_state_ex_i==`CORE_LSU_LSD_D)) |
                                   ((&rf_wr_d_addr_ex_i) & ~lsu_ld_excl_ex &
                                    (instr_data_size_ex_i==2'b10) &
                                    ((lsu_state_ex_i==`CORE_LSU_LSS_U) |
                                     (lsu_state_ex_i==`CORE_LSU_LSS_D)))));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        lsu_exit_valid_ex <= 1'b0;
        lsu_exit_code_ex  <= 5'b00000;
      end
    else if (lsu_exit_we)
      begin
        lsu_exit_valid_ex <= nxt_lsu_exit_valid_ex;
        lsu_exit_code_ex  <= nxt_lsu_exit_code_ex;
      end


  assign ahb_trans_waited  = (opt_const_en & (|ahb_htransd) &
                              ~(mtx_ahb_hreadyd_i & mtx_ahb_haddraccd_i));

  assign capt_ahb_ctrl_we  = ahb_trans_waited & ~ahb_const_ctrl;

  assign ahb_const_ctrl_we = (capt_ahb_ctrl_we |
                              (mtx_ahb_hreadyd_i & mtx_ahb_haddraccd_i &
                               ahb_const_ctrl));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ahb_const_trk <= 1'b0;
    else if (ahb_const_ctrl_we)
      ahb_const_trk <= ahb_trans_waited;

  assign ahb_const_ctrl   = ahb_const_trk & opt_const_en;

  assign lsu_ahb_invk_msk = (opt_const_en &
                             (ahb_htransd[1] | ahb_const_ctrl) &
                             ~(mtx_ahb_hreadyd_i & mtx_ahb_haddraccd_i));

  assign ahb_frc_psr_trns = ahb_const_ctrl & fault_lsu_mmatch_ex;

  assign ahb_const_unstk  = (ahb_const_ctrl & fault_lsu_mmatch_ex &
                             ~mtx_ahb_haddraccd_i);


  assign nxt_fault_lsu_resp = (// Error response received
                               |(({2{htransd_reg & ~mtx_ahb_hreadyd_i}} &
                                  { mpu_hrespd_i, mtx_ahb_hrespd_i[0]}) &
                                 {~ahb_frc_psr_reg,
                                  (~(int_stack_op_i | fpu_lazy_stack_op_i) &
                                   ~(|nvic_int_in_nmihf_i[1:0]) &
                                   ~str_buf_imm_ex) |
                                  (~(int_stack_op_i | fpu_lazy_stack_op_i) &
                                   (|nvic_int_in_nmihf_i[1:0]) &
                                   ~str_buf_imm_ex & ~nvic_bfhfnmign_i) |
                                  ((int_stack_op_i & ~ahb_frc_psr_reg) |
                                   fpu_lazy_stack_op_i) }));

  assign fault_lsu_resp_we = (nxt_fault_lsu_resp |
                              (fault_lsu_resp_ex & mtx_ahb_hreadyd_i));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fault_lsu_resp_ex <= 1'b0;
    else if (fault_lsu_resp_we)
      fault_lsu_resp_ex <= nxt_fault_lsu_resp;


  assign fault_lsu_precise_set = ((lsu_pline_valid_de_i |
                                   instr_ls_single_adv_de_i) &
                                  (// Set by LSS control phase
                                   (lsu_ahb_rdy_ex &
                                    ~(instr_immed_valid_ex_i & ~lsu_isld_ex &
                                      ~lsu_excl_ex & ~mpu_addrd_non_buf_i) &
                                    ~lsu_unalgn_ex & ~lsu_excp_ex &
                                    ~mpu_canceld_i & lss_cphase_algn_ex) |
                                   (~mtx_ahb_hreadyd_i & ~fault_lsu_local_ex[0] &
                                    ~mpu_canceld_i & lss_cphase_unalgn_ex)));

  assign fault_lsu_precise_clr = ((mtx_ahb_hreadyd_i &
                                   (// Clear by non-fault data phase
                                    (~fault_lsu_resp_ex &
                                     ((lsu_state_ex_i==`CORE_LSU_LSS_P) |
                                      (lsu_state_ex_i==`CORE_LSU_LSS_B) |
                                      (lsu_state_ex_i==`CORE_LSU_LSS_U) |
                                      (lsu_state_ex_i==`CORE_LSU_LSS_D) |
                                      str_buf_imm_ex))                     |
                                    (lsu_state_ex_i==`CORE_LSU_INT_ENTRY)  |
                                    (lsu_state_ex_i==`CORE_LSU_INT_ENLCK)  |
                                    (lsu_state_ex_i==`CORE_LSU_DFAULT_2))) |
                                  (lss_cphase_unalgn_ex & mpu_canceld_i));

  assign fault_lsu_precise_we  = fault_lsu_precise_set | fault_lsu_precise_clr;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fault_lsu_precise_ex <= 1'b0;
    else if (fault_lsu_precise_we)
      fault_lsu_precise_ex <= fault_lsu_precise_set;


  assign nxt_chk_unalgn_ex = (~fault_lsu_unalgn_ex & instr_ccpass_de_i &
                              (lsu_lsm_de | lsu_lsd_de |
                               (lsu_tb_de & nvic_unalign_trp_i) |
                               (lsu_lss_de &
                                (lsu_fpu_de | nvic_unalign_trp_i | lsu_excl_de))));

  assign nxt_algn_size_ex  = (instr_data_size_de_i[1:0] &
                              {2{(lsu_ldst_de | lsu_tb_de) & instr_ccpass_de_i}});

  assign chk_unalgn_we     = ((instr_ccpass_de_i & (lsu_ldst_de | lsu_tb_de)) |
                              (lsu_unalgn_ex & mtx_ahb_hreadyd_i &
                               mtx_ahb_haddraccd_i) |
                              ((|algn_size_ex) & ~lsu_unalgn_ex));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        algn_size_ex  <= 2'b00;
        chk_unalgn_ex <= 1'b0;
      end
    else if (chk_unalgn_we)
      begin
        algn_size_ex  <= nxt_algn_size_ex;
        chk_unalgn_ex <= nxt_chk_unalgn_ex;
      end

  assign lsu_unalgn_ex       = ((algn_size_ex[1] & (|addr_adder_ex[1:0])) |
                                (algn_size_ex[0] & addr_adder_ex[0]));

  assign fault_lsu_unalgn_ex = chk_unalgn_ex & lsu_unalgn_ex;


  assign bus_fault_sel = ({4{mtx_ahb_hreadyd_i}} &
                          {// Alignment
                           fault_lsu_unalgn_ex &
                           (~((mpu_hrespd_i | mtx_ahb_hrespd_i[0]) &
                              ~((|nvic_int_in_nmihf_i[1:0]) & nvic_bfhfnmign_i) &
                              ((hwrited_reg & ~str_buf_imm_ex) | ~hwrited_reg))),
                           mpu_hrespd_i & ~ahb_frc_psr_reg,
                           mtx_ahb_hrespd_i[0] & ~ahb_frc_psr_reg &
                           ((hwrited_reg & ~str_buf_imm_ex) | ~hwrited_reg),
                           mtx_ahb_hrespd_i[0] &
                           (hwrited_reg & str_buf_imm_ex) });

  assign fault_lsu_bus_we = (|nxt_fault_lsu_bus) | (|fault_lsu_bus_ex);

  assign nxt_fault_lsu_bus = { // MemManage access unaligned
                               bus_fault_sel[3] & ~int_stack_op_i,
                               bus_fault_sel[2] &
                               ~(int_stack_op_i | fpu_lazy_stack_op_i),
                               (bus_fault_sel[2] & hwrited_reg &
                                fpu_lazy_stack_op_i & opt_fpu_en),
                               bus_fault_sel[2] & hwrited_reg & int_stack_op_i,
                               bus_fault_sel[2] & int_stack_op_i & ~hwrited_reg,
                               (bus_fault_sel[1] & hwrited_reg &
                                fpu_lazy_stack_op_i & opt_fpu_en),
                               bus_fault_sel[1] & hwrited_reg & int_stack_op_i,
                               bus_fault_sel[1] & int_stack_op_i & ~hwrited_reg,
                               bus_fault_sel[0] & ~int_stack_op_i,
                               bus_fault_sel[1] &
                               ~(int_stack_op_i |
                                 (fpu_lazy_stack_op_i & opt_fpu_en)) };

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fault_lsu_bus_ex <= {10{1'b0}};
    else if (fault_lsu_bus_we)
      fault_lsu_bus_ex <= nxt_fault_lsu_bus;


  assign nxt_fault_lsu_mmatch = (mtx_ahb_hreadyd_i & ~lsu_addracc_wait_reg &
                                 ~fault_lsu_resp_ex &
                                 (lsu_state_ex_i==`CORE_LSU_INT_L_FLAGS) &
                                 (instr_atbase_ex_i ^
                                  ~(|lsu_psr_ex[8:0])));

  assign fault_lsu_mmatch_we = (nxt_fault_lsu_mmatch |
                                (fault_lsu_mmatch_ex & mtx_ahb_hreadyd_i));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fault_lsu_mmatch_ex <= 1'b0;
    else if (fault_lsu_mmatch_we)
      fault_lsu_mmatch_ex <= nxt_fault_lsu_mmatch;


  assign fault_lsu_local_ex  = { (fault_lsu_unalgn_ex |
                                  fault_lsu_mmatch_ex |
                                  fpu_int_cp_fault_i),
                                 fault_lsu_resp_ex };

  assign fault_lsu_ex        = { // Unstacking mismatch fault (PSR IntNum !=)
                                 int_stack_op_i & fault_lsu_mmatch_ex,
                                 fault_lsu_bus_ex };

  assign lsu_active_ex_o        = lsu_active_ex;
  assign lsu_sp_wb_ex_o         = lsu_sp_wb_ex;
  assign lsu_flag_wr_ex_o       = lsu_flag_wr_ex;
  assign lsu_psr_ex_o           = lsu_psr_ex;
  assign lsu_unalgn_ex_o        = lsu_unalgn_ex;
  assign lsu_stack_algn_64_ex_o = lsu_stack_algn_64_ex;
  assign lsu_ld_excl_ex_o       = lsu_ld_excl_ex;
  assign lsu_st_excl_ex_o       = lsu_st_excl_ex;
  assign lsu_exit_code_ex_o     = lsu_exit_code_ex;
  assign lsu_exit_valid_ex_o    = lsu_exit_valid_ex;
  assign lsu_rf_wr_d_data_ex_o  = lsu_rf_wr_d_data_ex;
  assign lsu_rf_wr_z_data_ex_o  = lsu_rf_wr_z_data_ex;
  assign lsu_fpu_rd_data_ex_o   = lsu_fpu_rd_data_ex;
  assign lsu_nxt_fpcar_o        = lsu_nxt_fpcar;
  assign lsu_addracc_wait_reg_o = lsu_addracc_wait_reg;
  assign lsu_ahb_invk_msk_o     = lsu_ahb_invk_msk;
  assign str_buf_imm_ex_o       = str_buf_imm_ex;
  assign ahb_const_unstk_o      = ahb_const_unstk;
  assign ahb_htransd_o          = ahb_htransd;
  assign ahb_hwrited_o          = ahb_hwrited;
  assign ahb_hsized_o           = ahb_hsized;
  assign ahb_haddrd_o           = ahb_haddrd;
  assign ahb_hprotd_o           = ahb_hprotd;
  assign ahb_hwdatad_o          = ahb_hwdatad;
  assign ahb_exreqd_o           = ahb_exreqd;
  assign dpu_addrd_valid_o      = dpu_addrd_valid;
  assign int_retisr_o           = int_retisr;
  assign fault_lsu_local_ex_o   = fault_lsu_local_ex;
  assign fault_lsu_precise_ex_o = fault_lsu_precise_ex;
  assign fault_lsu_ex_o         = fault_lsu_ex;
  assign dbg_ctrl_wr_ex_o       = dbg_ctrl_wr_ex;
  assign dbg_wr_isr_o           = dbg_wr_isr;


`ifdef ARM_ASSERT_ON

  wire          ovl_sample_trans;
  reg    [31:0] ovl_haddrd_reg;
  reg     [1:0] ovl_htransd_reg;
  reg           ovl_hwrited_reg;
  reg     [1:0] ovl_hsized_reg;
  reg     [7:0] ovl_bfmask;
  reg           ovl_hreadyd_d1;
  reg           ovl_await_d1;
  reg     [1:0] ovl_htransd_d1;
  reg     [1:0] ovl_hsized_d1;
  reg    [31:0] ovl_haddrd_d1;
  reg     [1:0] ovl_hprotd_d1;
  reg           ovl_hrespd_d1;
  reg    [31:0] ovl_hwdatad_d1;
  reg           ovl_hwrited_d1;

  assign ovl_sample_trans = (mtx_ahb_haddraccd_i & mtx_ahb_hreadyd_i &
                             ((|ahb_htransd) |
                              (ahb_htransd!=ovl_htransd_d1)));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        ovl_haddrd_reg  <= {32{1'b0}};
        ovl_htransd_reg <= 2'b00;
        ovl_hwrited_reg <= 1'b0;
        ovl_hsized_reg  <= 2'b00;
      end
    else if (ovl_sample_trans)
      begin
        ovl_haddrd_reg  <= ahb_haddrd;
        ovl_htransd_reg <= ahb_htransd;
        ovl_hwrited_reg <= ahb_hwrited;
        ovl_hsized_reg  <= ahb_hsized;
      end

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        ovl_htransd_d1 <= 2'b00;
        ovl_haddrd_d1  <= {32{1'b0}};
        ovl_hsized_d1  <= 2'b00;
        ovl_hprotd_d1  <= 2'b00;
        ovl_hwrited_d1 <= 1'b0;
        ovl_hwdatad_d1 <= {32{1'b0}};
      end
    else
      begin
        ovl_htransd_d1 <= ahb_htransd;
        ovl_haddrd_d1  <= ahb_haddrd;
        ovl_hsized_d1  <= ahb_hsized;
        ovl_hprotd_d1  <= ahb_hprotd;
        ovl_hwrited_d1 <= ahb_hwrited;
        ovl_hwdatad_d1 <= ahb_hwdatad;
      end

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_bfmask <= 8'b11111111;
    else if (fault_lsu_bus_we)
      ovl_bfmask <= {7'b1111111,~((|nvic_int_in_nmihf_i[1:0]) &
                                  nvic_bfhfnmign_i)};

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        ovl_await_d1   <= 1'b0;
        ovl_hreadyd_d1 <= 1'b0;
        ovl_hrespd_d1  <= 1'b0;
      end
    else
      begin
        ovl_await_d1   <= mtx_ahb_hreadyd_i & mtx_ahb_haddraccd_i;
        ovl_hreadyd_d1 <= mtx_ahb_hreadyd_i;
        ovl_hrespd_d1  <= mpu_hrespd_i | (|mtx_ahb_hrespd_i);
      end

  assert_never #(0,0,"ahb_htransd should never emit BUSY")
    ovl_htransd(hclk, hreset_n, ahb_htransd==2'b01);

  assert_never #(0,0,"ahb_htransd SEQ should only follow NSEQ or SEQ")
    ovl_htrans_seq(hclk, hreset_n,
                   ((ahb_htransd==2'b11) &
                    ~((ovl_htransd_reg == 2'b10) |
                      (ovl_htransd_reg == 2'b11))));

  assert_implication #(0,0,
    "During a waited transfer can only change an IDLE into a NONSEQ transfer")
    ovl_ahb_errm_trans2
      (hclk, hreset_n,
       (~ovl_await_d1 & (ovl_htransd_d1 == 2'b00)),
       ((ahb_htransd == 2'b00) | (ahb_htransd == 2'b10)));

  assert_implication #(0,0,
    "During a waited write transfer the write data must be kept stable")
    ahb_errm_wdata
      (hclk, hreset_n,
       (ovl_hwrited_d1 & ~ovl_hreadyd_d1),
       (ovl_hwdatad_d1 == ahb_hwdatad));

  assert_never #(0,0,"ahb_hsized must be stable during sequential transfer")
    ovl_hsize_seq(hclk, hreset_n, ((ahb_htransd == 2'b11) &
                                   (ovl_hsized_reg != ahb_hsized)));

  assert_never #(0,0,"ahb_hwrited must be stable during sequential transfer")
    ovl_hwrite_seq(hclk, hreset_n, ((ahb_htransd == 2'b11) &
                                    (ovl_hwrited_reg != ahb_hwrited)));

  assert_never #(0,0,"ahb_haddrd should increment by 4 if ahb_htransd = SEQ")
    ovl_haddr_seq(hclk, hreset_n, (mtx_ahb_haddraccd_i & mtx_ahb_hreadyd_i &
                                   (ahb_htransd == 2'b11) &
                                   ((ovl_haddrd_reg + 4) != ahb_haddrd)));

  assert_never #(0,0,"mtx_ahb_hrespd_i two cycle response incorrectly generated")
    ovl_hresp(hclk, hreset_n,
              (htransd_reg &
               (((mpu_hrespd_i | (|mtx_ahb_hrespd_i)) & mtx_ahb_hreadyd_i &
                 ~(ovl_hrespd_d1 & ~ovl_hreadyd_d1)) |
                (ovl_hrespd_d1 & ~ovl_hreadyd_d1 &
                 ~(mpu_hrespd_i | (|mtx_ahb_hrespd_i))))));

  assert_never #(0,0,"LSU must not fault when in debug")
    ovl_debugfault(hclk, hreset_n, (lsu_lfault_ex &
                                    ((lsu_state_ex_i==`CORE_LSU_DBG_PC_2) |
                                     (lsu_state_ex_i==`CORE_LSU_DBG_R_0)  |
                                     (lsu_state_ex_i==`CORE_LSU_DBG_R_1)  |
                                     (lsu_state_ex_i==`CORE_LSU_DBG_R_2))));

  assert_never #(0,0,"Debug {HREADY,HRESP} not allowed to be !=2'b10")
    ovl_debug(hclk, hreset_n, ((lsu_state_ex_i==`CORE_LSU_DBG_R_2) &
                               (mpu_hrespd_i | (|mtx_ahb_hrespd_i) |
                                ~mtx_ahb_hreadyd_i)));

  assert_never #(0,0,
    "int_fix_sp must not be set outside of faults and first HTRANS of interrupt")
    ovl_int_fix_sp(hclk, hreset_n, (int_fix_sp &
                                    ~((lsu_state_ex_i==`CORE_LSU_LSM_Rx)   |
                                      (lsu_state_ex_i==`CORE_LSU_INT_S_PC) |
                                      (lsu_state_ex_i==`CORE_LSU_IDLE)     |
                                      (lsu_state_ex_i==`CORE_LSU_FAULT))));

  assert_never #(0,0,"int_fix_sp should toggle when written")
    ovl_int_fix_sp_2(hclk, hreset_n, (int_fix_sp_we &
                                      (int_fix_sp==nxt_int_fix_sp)));

  assert_zero_one_hot #(0,10,0,"fault_lsu_ex must be exclusive")
    ovl_fault_lsu_ex_excl(hclk, hreset_n, ({fault_lsu_ex[10:2],fault_lsu_ex[0]} &
                                           ovl_bfmask));

  assert_never #(0,0,"Unstack fault incorrectly generated")
    ovl_unstk_fault(hclk, hreset_n,
                    (fault_lsu_ex[10] &
                     ~((lsu_state_ex_i==`CORE_LSU_INT_L_FLAGS) |
                       (lsu_state_ex_i==`CORE_LSU_INT_L_RX)    |
                       (lsu_state_ex_i==`CORE_LSU_INT_UFAULT_1))));

  assert_never #(0,0,"str_stat cannot be set into fault states")
    ovl_str_stat_1(hclk, hreset_n, ((str_stat_valid_ex | (|str_stat_ex)) &
                                    ((lsu_state_ex_i==`CORE_LSU_FAULT) |
                                     (lsu_state_ex_i==`CORE_LSU_LAZY_FAULT_1) |
                                     (lsu_state_ex_i==`CORE_LSU_LAZY_FAULT_2))));

  assert_never #(0,0,"str_stat cannot be set into MPU error")
    ovl_str_stat_2(hclk, hreset_n,
                   (str_buf_imm_ex & mtx_ahb_hreadyd_i & mpu_hrespd_i &
                    ~(str_stat_we & ~(|nxt_str_stat_ex) &
                      ~nxt_str_stat_valid_ex)));

  assert_zero_one_hot #(0,2,0,"multiple data register selection must be exclusive")
    ovl_data_reg_sel_excl(hclk, hreset_n, ({sel_str_reg_ex,str_fwd_valid_ex}));

  assert_never #(0,0,"LSS pipelined state should not be enter when MPU faults")
    ovl_lsspipe_check(hclk, hreset_n, (((lsu_state_ex_i==`CORE_LSU_LSS_P) |
                                        (lsu_state_ex_i==`CORE_LSU_LSS_B)) &
                                       mtx_ahb_hreadyd_i & mpu_hrespd_i));

  assert_never #(0,0,"fault_lsu_ex incorrectly formed")
    ovl_fault_lsu_ex_check(hclk, hreset_n,
                           ((fault_lsu_ex[0]) & ~
                            ((|nvic_int_in_nmihf_i[1:0]) & nvic_bfhfnmign_i) &
                            ~((lsu_state_ex_i==`CORE_LSU_FAULT) |
                              (lsu_state_ex_i==`CORE_LSU_INT_S_PC))) |
                           ((fault_lsu_ex[8] | fault_lsu_ex[9]) &
                            ~((lsu_state_ex_i==`CORE_LSU_FAULT) |
                              (lsu_state_ex_i==`CORE_LSU_INT_S_PC) |
                              (lsu_state_ex_i==`CORE_LSU_LAZY_FAULT_2))) |
                           ((fault_lsu_ex[2] | fault_lsu_ex[5]) &
                            ~((lsu_state_ex_i==`CORE_LSU_INT_UFAULT_1) |
                              (lsu_state_ex_i==`CORE_LSU_INT_UFAULT_2))) |
                           ((fault_lsu_ex[3] | fault_lsu_ex[6]) &
                            ~((lsu_state_ex_i==`CORE_LSU_INT_S_PC) |
                              (lsu_state_ex_i==`CORE_LSU_INT_SFAULT_1))) |
                           ((fault_lsu_ex[4] | fault_lsu_ex[7]) &
                            ~((lsu_state_ex_i==`CORE_LSU_INT_S_PC)     |
                              (lsu_state_ex_i==`CORE_LSU_LAZY_FAULT_1) |
                              (lsu_state_ex_i==`CORE_LSU_LAZY_FAULT_2))) |
                           ((fault_lsu_ex[10]) &
                            ~((lsu_state_ex_i==`CORE_LSU_INT_L_FLAGS) |
                              (lsu_state_ex_i==`CORE_LSU_INT_L_RX) |
                              (lsu_state_ex_i==`CORE_LSU_INT_UFAULT_1))));

  assert_never #(0,0,
                 "Stack alignment must not be set outside of interrupt processing")
     ovl_stk_align_check(hclk, hreset_n, (int_algn_stk_64 & ~int_stack_op_i));

  assert_never #(0,0, "Only aligned transactions for int_stack_op")
    ovl_stack_op0_check(hclk, hreset_n, (int_stack_op_i & (|ahb_haddrd[1:0]) &
                                        (|ahb_htransd) & mtx_ahb_haddraccd_i &
                                        mtx_ahb_hreadyd_i));

  assert_never #(0,0, "Only word transactions for int_stack_op")
    ovl_stack_op1_check(hclk, hreset_n, (int_stack_op_i &
                                        (ahb_hsized != 2'b10) &
                                         (|ahb_htransd) &
                                         mtx_ahb_haddraccd_i & mtx_ahb_hreadyd_i));

  assert_never_unknown #(0,6,0,"X propagate protect default statements")
    ovl_ahbintf_x_chk(hclk, hreset_n, 1'b1, lsu_state_ex_i);

  assert_implication #(0,0,
      "For a stable waited transfer all control info must remain constant")
    ovl_lsu_ctl_const
      (hclk, hreset_n,
       (opt_const_en & ~ovl_await_d1 & ahb_htransd[1] & ovl_htransd_d1[1]),
       ((ovl_haddrd_d1  == ahb_haddrd)  &
        (ovl_htransd_d1 == ahb_htransd) &
        (ovl_hwrited_d1 == ahb_hwrited) &
        (ovl_hsized_d1  == ahb_hsized)  &
        (ovl_hprotd_d1  == ahb_hprotd)));

  assert_implication #(0,0, "Cancel waited NSEQ/SEQ only if error")
    ovl_lsu_errm_trans1
      (hclk, hreset_n,
       (opt_const_en & ~ovl_await_d1 & (|ovl_htransd_d1) &
        (ovl_htransd_d1 != ahb_htransd)),
       ((ahb_htransd == 2'b00) & ovl_hrespd_d1));

`endif

`include "cm3_dpu_undefs.vp"

endmodule
