`include "cm3_dpu_defs.vp"

module cm3_dpu_lsu_ctl
  (// Inputs
   hreset_n, hclk, instr_valid_de_i, instr_lsu_ctl_de_i, instr_ccpass_de_i,
   instr_lsm_baseinlist_de_i, instr_ls_allow_pline_de_i,
   instr_ls_single_adv_de_i, instr_lsu_int_ilock_de_i, instr_lsm_immed_de_i,
   rf_rd_a_addr_de_i, rf_rd_b_addr_de_i, instr_rf_rd_c_addr_de_i,
   rf_wr_d_addr_de_i, instr_itblock_ex_i, instr_immed_valid_ex_i,
   instr_immed_ex_i, instr_atbase_ex_i, instr_detect_exit_ex_i, psr_ex_i,
   lsu_ld_excl_ex_i, lsu_st_excl_ex_i, lsu_unalgn_ex_i, lsu_addracc_wait_reg_i,
   lsu_stack_algn_64_ex_i, lsu_psr_ex_i, lsu_exit_valid_ex_i, ahb_const_unstk_i,
   rst_pc_valid_ex_i, iflush_pflush_ex_i, mtx_ahb_haddraccd_i, mtx_ahb_hreadyd_i,
   mpu_canceld_i, mpu_addrd_non_buf_i, mtx_bm_active_i, fpu_enabled_i,
   fpu_first_reg_de_i, fpu_lazy_start_i, fpu_int_cp_fault_i, fpu_stack_ctl_i,
   fpu_unstack_ctl_i, fpu_lazy_stack_op_i, nvic_int_invoke_i, nvic_int_tail_i,
   int_first_instr_valid_i, nvic_int_in_nmihf_i, nvic_fpccr_i,
   nvic_slp_asleep_i, nvic_slp_sleeponexit_i, fault_lsu_local_ex_i,
   fault_dblfault_ex_i, dbg_halt_set_i, dbg_halted_i, nvic_dbg_reg_addr_i,
   nvic_dbg_reg_wr_i, nvic_dbg_trans_i,
   instr_excl_ex_o, instr_lsu_ctl_ex_o, instr_lsm_baseinlist_ex_o,
   lsu_valid_ex_o, lsu_state_ex_o, lsu_nxt_lsm_fpu_ex_o, lsu_lsm_reg_ex_o,
   lsu_lsm_list_valid_ex_o, lsu_lsm_last_ex_o, lsm_list_length_de_o,
   lsm_list_length_ex_o, lsm_ici_remaining_ex_o, lsu_stall_ex_o,
   lsu_stall_late_ex_o, lsu_excl_mon_ex_o, lsu_pline_valid_de_o,
   lsu_st_imm_nonbuf_ex_o, lsu_lsm_base_status_ex_o, lsm_fpu_brestore_ex_o,
   lsu_dbl_fault_ex_o, lsu_stackpush_rdy_ex_o, lsu_pflush_ex_o,
   lsu_it_valid_ex_o, lsu_it_ex_o, lsu_fix_pc_ex_o, lsu_mcyc_invk_msk_ex_o,
   lsu_mcyc_ilock_ex_o, lsu_fp_ls_rf_addr_ex_o, lsu_rf_rd_a_en_fast_ex_o,
   lsu_rf_rd_a_en_ex_o, lsu_rf_rd_a_addr_ex_o, lsu_rf_wr_d_en_ex_o,
   lsu_rf_wr_d_en_fast_ex_o, lsu_rf_wr_d_addr_ex_o, lsu_rf_wr_z_addr_ex_o,
   lsu_rf_wr_z_en_ex_o, lsu_will_wr_rf_ex_o, lsu_ici_cont_o, lsu_stack_start_o,
   lsu_stack_end_o, lsu_addrd_non_buf_o, lsu_main_sp_en_o, lsu_real_sp_rd_o,
   rf_rd_a_addr_ex_o, rf_rd_b_addr_ex_o, instr_rf_rd_c_addr_ex_o,
   rf_wr_d_addr_ex_o, int_entry_o, int_exit_o, int_return_o, int_stack_op_o,
   int_unstacking_o, dbg_ready_o
  );

  parameter      RESET_ALL_REGS = 0;       // Reset all registers if 1
  parameter      EXT_PRESENT    = 0;       // Extensions present if 1
  parameter      FPU_PRESENT    = 0;       // FPU present if 1

  input          hreset_n;
  input          hclk;
  input          instr_valid_de_i;         // Instruction in decode is valid
  input [15:0]   instr_lsu_ctl_de_i;       // LSU unit control (De)
  input          instr_ccpass_de_i;        // Condition code pass (De)
  input          instr_lsm_baseinlist_de_i;// LSM has base in list (De)
  input          instr_ls_allow_pline_de_i;// Allow LSS to overlap
  input          instr_ls_single_adv_de_i; // Advance LSS to data-phase
  input          instr_lsu_int_ilock_de_i; // Interrupt interlock
  input [31:0]   instr_lsm_immed_de_i;     // LSM immediate value
  input [3:0]    rf_rd_a_addr_de_i;        // RF A port read address
  input [3:0]    rf_rd_b_addr_de_i;        // RF B port read address
  input [3:0]    instr_rf_rd_c_addr_de_i;  // RF secondary read address
  input [3:0]    rf_wr_d_addr_de_i;        // RF D port write address
  input          instr_itblock_ex_i;       // In IT block
  input          instr_immed_valid_ex_i;   // Immediate valid
  input [31:0]   instr_immed_ex_i;         // Immediate value
  input          instr_atbase_ex_i;        // In thread
  input          instr_detect_exit_ex_i;   // Instruction caused an ISR exit
  input [31:0]   psr_ex_i;                 // PSR
  input          lsu_ld_excl_ex_i;         // LDREX in execute
  input [1:0]    lsu_st_excl_ex_i;         // STREX in execute
  input          lsu_unalgn_ex_i;          // Access is unaligned
  input          lsu_addracc_wait_reg_i;   // Waiting for address acceptance
  input          lsu_stack_algn_64_ex_i;   // Align stack at 8-byte boundary
  input [31:0]   lsu_psr_ex_i;             // LSU PSR write value
  input          lsu_exit_valid_ex_i;      // LSU instruction is valid exit
  input          ahb_const_unstk_i;        // Keep trans for unstack
  input          rst_pc_valid_ex_i;        // PC reset vector fetch complete
  input          iflush_pflush_ex_i;       // IFLUSH pipe flush
  input          mtx_bm_active_i;          // Bus-matrix active
  input          mtx_ahb_haddraccd_i;      // AHB D-side HADDRACC
  input          mtx_ahb_hreadyd_i;        // AHB D-side HREADY
  input          fpu_enabled_i;            // FPU enabled
  input  [4:0]   fpu_first_reg_de_i;       // FPU first reg for LSM
  input          fpu_lazy_start_i;         // Lazy stack start
  input [1:0]    fpu_stack_ctl_i;          // FPU stack control
  input [1:0]    fpu_unstack_ctl_i;        // FPU unstack control
  input          fpu_lazy_stack_op_i;      // Lazy FPU stack in progress
  input          fpu_int_cp_fault_i;       // FPU interrupt NOCP fault
  input          mpu_canceld_i;            // MPU D-side cancel
  input          mpu_addrd_non_buf_i;      // MPU D-side addr non-bufferable
  input [2:0]    nvic_int_in_nmihf_i;      // In NMI/HF ISR
  input          nvic_int_invoke_i;        // Interrupt stack request
  input          nvic_int_tail_i;          // Interrupt tail-chain request
  input          int_first_instr_valid_i;  // 1st instr ready for entry/return
  input [31:0]   nvic_fpccr_i;             // Floating-point context control reg
  input          nvic_slp_asleep_i;        // DPU asleep
  input          nvic_slp_sleeponexit_i;   // Sleep-on-exit request
  input [1:0]    fault_lsu_local_ex_i;     // LSU detection of faults
  input          fault_dblfault_ex_i;      // Double fault detected
  input          dbg_halt_set_i;           // Core halt set
  input          dbg_halted_i;             // Core halted
  input [6:0]    nvic_dbg_reg_addr_i;      // Debug register transfer reg num
  input          nvic_dbg_reg_wr_i;        // Debug register transfer W/nR
  input          nvic_dbg_trans_i;         // Debug register transfer request

  output         instr_excl_ex_o;          // Instruction is an exclusive
  output [15:0]  instr_lsu_ctl_ex_o;       // LSU unit control (Ex)
  output         instr_lsm_baseinlist_ex_o;// LSM has base in list (Ex)
  output [5:0]   lsu_state_ex_o;           // LSU state
  output         lsu_valid_ex_o;           // LSU op is valid
  output [4:0]   lsu_nxt_lsm_fpu_ex_o;     // Next LSM register number for FPU
  output [4:0]   lsu_lsm_reg_ex_o;         // Current LSM register number
  output         lsu_lsm_list_valid_ex_o;  // LSM lsit valid
  output         lsu_lsm_last_ex_o;        // Last register in LSM list
  output [4:0]   lsm_list_length_de_o;     // LSM register list length (De)
  output [5:0]   lsm_list_length_ex_o;     // LSM register list length (Ex)
  output [5:0]   lsm_ici_remaining_ex_o;   // ICI LSM number of remaining reg
  output         lsu_stall_ex_o;           // LSU stall pipe
  output         lsu_stall_late_ex_o;      // LSU very late stall
  output         lsu_excl_mon_ex_o;        // Exclusive monitor status
  output         lsu_pline_valid_de_o;     // Allow LSS to overlap
  output         lsu_st_imm_nonbuf_ex_o;   // Non bufferable store immediate
  output [1:0]   lsu_lsm_base_status_ex_o; // LSM base in list status
  output         lsm_fpu_brestore_ex_o;    // Restore base for FP LSM
  output         lsu_dbl_fault_ex_o;       // LSU double fault detected
  output         lsu_stackpush_rdy_ex_o;   // LSU ready for pushing stack
  output         lsu_pflush_ex_o;          // Flush pipe
  output         lsu_it_valid_ex_o;        // LSU write to IT is valid
  output [7:0]   lsu_it_ex_o;              // LSU IT bits to write
  output         lsu_fix_pc_ex_o;          // LSU requires PC fix
  output         lsu_mcyc_invk_msk_ex_o;   // Mask intinvoke for multi-cycle op
  output         lsu_mcyc_ilock_ex_o;      // Multi-cycle op interlock
  output [4:0]   lsu_fp_ls_rf_addr_ex_o;   // FPU RF address
  output         lsu_rf_rd_a_en_fast_ex_o; // LSU RF A port read enable (fast)
  output         lsu_rf_rd_a_en_ex_o;      // LSU RF A port read enable
  output [3:0]   lsu_rf_rd_a_addr_ex_o;    // LSU RF A port read address
  output         lsu_rf_wr_d_en_ex_o;      // LSU RF D port write enable
  output         lsu_rf_wr_d_en_fast_ex_o; // LSU RF D port write enable (fast)
  output [3:0]   lsu_rf_wr_d_addr_ex_o;    // LSU RF D port write address
  output [3:0]   lsu_rf_wr_z_addr_ex_o;    // LSU RF Z port write address
  output         lsu_rf_wr_z_en_ex_o;      // LSU RF Z port write enable
  output         lsu_will_wr_rf_ex_o;      // LSU will write to RF imminently
  output [3:0]   lsu_ici_cont_o;           // ICI bits from stack
  output         lsu_stack_start_o;        // LSU stack op start
  output         lsu_stack_end_o;          // LSU stack op complete
  output         lsu_addrd_non_buf_o;      // AHB D-side address non-bufferable
  output         lsu_main_sp_en_o;         // Force r13 to be main SP
  output         lsu_real_sp_rd_o;         // Force r13 read to be real SP
  output [3:0]   rf_rd_a_addr_ex_o;        // RF A port read address
  output [3:0]   rf_rd_b_addr_ex_o;        // RF B port read address
  output [3:0]   instr_rf_rd_c_addr_ex_o;  // RF secondary read address
  output [3:0]   rf_wr_d_addr_ex_o;        // RF D port write address
  output         int_entry_o;              // ISR entry
  output         int_exit_o;               // ISR exit
  output         int_return_o;             // Unstacking complete
  output         int_stack_op_o;           // LSU is stacking/unstacking
  output         int_unstacking_o;         // LSU is unstacking
  output         dbg_ready_o;              // Debug register transfer ready

  wire           full_hreset_n;            // Apply hreset_n to all registers
  wire           opt_ext_en;               // Master enable for extensions
  wire           opt_fpu_en;               // Master enable for FPU logic
  wire           lsu_addpc_de;             // LSU op is ADD PC
  wire           lsu_tb_de;                // LSU op is TBH/TBB
  wire           lsu_lss_de;               // LSU op is LSS
  wire           lsu_lsd_de;               // LSU op is LSD
  wire           lsu_lsm_de;               // LSU op is LSM
  wire           lsu_lsm_cc_de;            // LSU op is a CC passing LSM
  wire           lsu_clrex_de;             // LSU op is CLREX
  wire           lsu_op_de;                // LSU has operation in De
  wire           lsu_fpu_de;               // LSU has an FPU operation in De
  wire           lsu_wb_valid_de;          // LSU op has write-back
  wire           lsu_de_2_ex_en;           // LSU control De -> Ex
  reg            lsu_valid_ex;             // LSU operation in Ex is valid
  wire [15:0]    nxt_instr_lsu_ctl_ex;     // Next Ex LSU control
  reg  [15:0]    instr_lsu_ctl_ex;         // LSU unit control (Ex)
  reg            instr_lsm_baseinlist_ex;  // LSM has base in list (Ex)
  wire           lsu_rf_addr_we;           // RF addresses write enable
  reg  [3:0]     rf_rd_a_addr_ex;          // RF A port read address
  reg  [3:0]     rf_rd_b_addr_ex;          // RF B port read address
  reg  [3:0]     instr_rf_rd_c_addr_ex;    // RF secondary read address
  reg  [3:0]     rf_wr_d_addr_ex;          // RF D port write address
  wire           lsu_lsm_ex;               // LSU op is a LSM
  wire           lsu_flsm_ex;              // LSU op is a FP LSM
  wire           lsu_isld_ex;              // LSU op is a load
  wire           lsu_wb_valid_ex;          // LSU op has write-back
  wire           lsu_wb_after_ex;          // LSU WB is IA
  wire           lsu_excl_ex;              // LSU op is an exclusive
  wire           lsu_clrex_ex;             // LSU op is CLREX
  wire           lsu_fpu_ex;               // LSU has an FPU operation in Ex
  wire           instr_excl_ex;            // Instruction is an exclusive
  wire           str_imm_ex;               // Store with imm based offset
  wire           lsu_lfault_ex;            // LSU has detected a fault
  wire           lsu_excp_ex;              // LSU has detected an exception
  wire           lsu_ahb_rdy_ex;           // AHB is ready
  wire           lsu_ahb_close_ex;         // AHB data phase close
  wire           str_buf_valid_ex;         // Store buffer can be used
  wire           int_first_instr_rdy;      // 1st instr ready for/after int
  wire           psr_it_valid_ex;          // PSR contains valid IT block
  wire           psr_ici_valid_ex;         // PSR contains valid ICI code
  wire           psr_it_nonzero_ex;        // PSR IT register is non-zero
  wire           lsu_res_it_valid_ex;      // Stack PSR contains valid IT block
  wire           lsu_st_imm_nonbuf_ex;     // Non bufferable store immediate
  wire           lsu_pline_valid_de;       // Allow LSS to overlap
  wire           lsu_in_nmihf;             // In NMI or HF handler
  wire           lsu_ici_ret_ex;           // LSU is returning to continued LSM
  wire           lsu_fp_immed_stk;         // Full immediate FP stack
  wire           lsu_fp_immed_unstk;       // Full immediate FP unstack
  wire [4:0]     lsu_request;              // LSU operation request
  wire           lsm_continue;             // Continue LSM based on ICI
  wire           lsu_ret_lsm_ex;           // LSU returning to a continued LSM
  wire [5:0]     lsu_ret_state_ex;         // Select between ICI or normal LSM
  wire [11:0]    idle_sel;                 // Idle state select lines
  wire [5:0]     idle_nxt_state;           // Idle state for next state logic
  wire           idle_valid;               // idle_nxt_state can be used
  wire [5:0]     lss_pipe_state_ex;        // LSS overlap state decisision
  wire [5:0]     lss_data_state_ex;        // Unaligned/overlap state decision
  wire           lss_pipe_adv_de;          // LSS pipe line advance
  wire           lss_stall_de;             // LSS stall close for SP and PC
  reg            stall;                    // LSU stall
  wire           lsu_stall_late_ex;        // LSU very late stall
  reg            state_we;                 // LSU state write enable
  reg  [5:0]     nxt_state;                // LSU next state
  reg  [5:0]     state;                    // LSU state
  wire           lsu_stackpush_rdy_ex;     // LSU interrupt push is ready
  wire           dbg_ready_we;             // dbg_ready write enable
  wire           nxt_dbg_ready;            // Next dbg_ready
  reg            dbg_ready;                // Debug register transfer ready
  wire           stack_op_int;             // Integer stack op
  wire           stack_op_fp;              // FPU stack op
  wire [31:0]    stack_regs_int;           // Register list to stack
  wire [31:0]    stack_regs_fp;            // FPU stack list
  wire [31:0]    stack_regs;               // Register list for stack ops
  reg  [31:0]    non_r13_immed_ex;         // Immediate with R13 masked
  wire [2:0]     lsm_mask_0;               // LSM list mask (step 0)
  wire [30:0]    lsm_mask_1;               // LSM list mask (step 1)
  wire [30:0]    lsm_mask_2;               // LSM list mask (step 2)
  wire [31:0]    lsm_masked_list;          // LSM list with completed masked
  wire [30:0]    lsm_ripple_list;          // LSM one-hot mask list
  wire [31:0]    lsm_reg_sel;              // LSM one-hot list for next reg
  wire [31:0]    fp_lsm_regs_upper;        // FP reg list computation, upper regs
  wire [31:0]    fp_lsm_regs_lower;        // FP reg list computation, lower regs
  wire [31:0]    fp_lsm_reg_list;          // FP register list for LSM
  wire [4:0]     lsm_nxt_reg_ex;           // LSM list converted to reg number
  wire           lsm_list_valid_ex;        // LSM list non-empty
  reg            lsm_reg_we;               // LSM register write enable
  reg  [4:0]     nxt_lsm_reg_ex;           // Next LSM register value
  reg            nxt_lsm_reg_valid_ex;     // Next LSM register valid
  reg  [4:0]     lsm_reg_ex;               // LSM register value
  reg            lsm_reg_valid_ex;         // LSM register valid
  wire           lsu_it_valid_ex;          // LSU write to IT is valid
  wire [7:0]     lsu_it_ex;                // LSU IT bits to write
  wire [3:0]     lsm_reg_inv_ex;           // 15 - lsu_lsm_reg_ex
  wire [15:0]    lsm_inv_shft_ex;          // 6'h7FFF >> lsm_reg_inv_ex
  wire [15:0]    immed_mask_ex;            // LSM immediate mask
  wire           sel_mask_list_ex;         // Select between decoded and cont
  wire [31:0]    lsm_immed;                // LSM immediate based reg list
  wire [4:0]     lsm_list_length_de;       // LSM register list length (De)
  wire [5:0]     nxt_lsm_list_length_ex;   // Next LSM reg list length (Ex)
  reg  [5:0]     lsm_list_length_ex;       // LSM register list length (Ex)
  wire [5:0]     nxt_lsm_ici_remain_ex;    // Next ICI LSM remaining reg num
  reg  [5:0]     lsm_ici_remaining_ex;     // ICI LSM number of remaining reg
  wire [1:0]     lsu_lsm_base_status_ex;   // LSM base in list status
  wire           ldm_base_updated_ex;      // LDM base has been updated
  wire           lsm_fpu_first_ex;         // First reg in FP LSM in progress
  wire [4:0]     nxt_lsm_fpu_ex;           // Next LSM register for FPU
  wire           lsm_fpu_brestore_ex;      // FP LSM requires base restoral
  wire           lsm_reg_pc_ex;            // Next LSM register is the PC
  wire           lsm_reg_nonzero_ex;       // Next LSM register is > r0
  wire           lsm_ici_last_ex;          // Last reg in LSM ICI case
  wire           lsm_last_we;              // lsm_last_ex write enable
  wire           nxt_lsm_last_ex;          // Next lsm_last_ex
  reg            lsm_last_ex;              // Last register in LSM list
  wire [3:0]     lsu_ici_cont;             // ICI bits from stack
  wire           lsu_excl_mon_we;          // Exclusive monitor write enable
  wire           nxt_lsu_excl_mon_ex;      // Next exclusive monitor status
  reg            lsu_excl_mon_ex;          // Exclusive monitor status
  wire [3:0]     nxt_rf_wr_d_addr_wr;      // Next rf_wr_d_addr_wr
  wire           lsu_ex_2_wr_en;           // LSU Ex -> Wr enable
  reg  [3:0]     rf_rd_a_addr_wr;          // RF A port read address (Wr)
  reg  [3:0]     rf_wr_d_addr_wr;          // RF D port write address (Wr)
  reg            lsu_isld_wr;              // LSU op is a load (Wr)
  reg            lsu_wb_valid_wr;          // LSU op has WB (Wr)
  reg            opt_lsu_fpu_wr;           // LSU op is for FPU (Wr)
  wire           lsu_fpu_wr;               // LSU op is for FPU (Wr)
  wire           lsm_rf_d_en_ex;           // LSM base update D port write en
  wire           int_entry;                // ISR entry
  wire           int_exit;                 // ISR exit
  wire           int_return;               // Unstacking complete
  wire           lsu_stack_start;          // LSU stack op start
  wire           lsu_stack_end;            // LSU stack op complete
  wire           int_stacking_we;          // int_stacking write enable
  wire           set_int_stacking;         // Set int_stacking
  reg            int_stacking;             // LSU is stacking
  wire           int_unstacking_we;        // int_unstacking write enable
  reg            int_unstacking;           // LSU is unstacking
  wire           int_stack_op;             // LSU is stacking/unstacking
  wire           lsu_addrd_non_buf;        // AHB D-side address non-bufferable
  wire           lsu_main_sp_en;           // Force r13 to be main SP
  wire           lsu_real_sp_rd;           // Force r13 read to be real SP
  reg            lsu_rf_rd_a_en_fast_ex;   // LSU RF A port read enable (fast)
  reg            lsu_rf_rd_a_en_ex;        // LSU RF A port read enable
  reg  [3:0]     lsu_rf_rd_a_addr_ex;      // LSU RF A port read address
  reg            lsu_rf_wr_en_ex;          // LSU RF write enable
  reg            wr_hreadyd_chk_ex;        // Validate D wr against hreadyd
  reg            wr_haddraccd_chk_ex;      // Validate D wr against haddraccd
  wire           lsu_rf_wr_d_en_ex;        // LSU RF D port write enable
  wire           lsu_rf_wr_d_en_fast_ex;   // LSU RF D port write enable (fast)
  reg  [3:0]     lsu_rf_wr_d_addr_ex;      // LSU RF D port write address
  wire [3:0]     lsu_rf_wr_z_addr_ex;      // LSU RF Z port write address
  wire           lsu_rf_wr_z_en_ex;        // LSU RF Z port write enable
  reg  [4:0]     opt_fp_ls_rf_addr_ex;     // FPU ld/st RF address
  wire [4:0]     lsu_fp_ls_rf_addr_ex;     // FPU ld/st RF address
  wire           lsu_wb_will_occur_ex;     // Writeback imminent
  wire           lsu_ld_will_occur_ex;     // Load of data imminent
  wire           lsu_last_will_occur_ex;   // Last load for multiple op imminent
  wire           lsu_entry_will_occur_ex;  // Interrupt entry imminent
  wire           lsu_excl_will_occur_ex;   // Exclusive result imminent
  wire           lsu_will_wr_rf_ex;        // LSU will write to RF imminently
  wire           lsu_dbl_fault_ex;         // LSU double fault detected
  wire           lsu_pflush_ex;            // Flush pipe
  wire           lsu_fix_pc_ex;            // LSU requires PC fix
  wire           lsu_mcyc_invk_msk_ex;     // Mask intinvoke for multi-cycle op
  wire           lsu_mcyc_ilock_ex;        // Multi-cycle op interlock

`ifdef ARM_DSM

  wire   dsm_reset_all_regs;
  wire   dsm_fpu_present;

  assign full_hreset_n = (dsm_reset_all_regs)  ? hreset_n : 1'b1;

  assign opt_fpu_en    = (dsm_fpu_present)     ? 1'b1     : 1'b0;

`else

  assign full_hreset_n = (RESET_ALL_REGS == 1) ? hreset_n : 1'b1;

  assign opt_fpu_en    = (FPU_PRESENT == 1)    ? 1'b1     : 1'b0;

`endif

  assign opt_ext_en    = (EXT_PRESENT == 1)    ? 1'b1     : 1'b0;

  assign lsu_addpc_de  = instr_lsu_ctl_de_i[1] & instr_ccpass_de_i;
  assign lsu_tb_de     = instr_lsu_ctl_de_i[2] & instr_ccpass_de_i;
  assign lsu_lss_de    = instr_lsu_ctl_de_i[3] & instr_ccpass_de_i;
  assign lsu_lsd_de    = (instr_lsu_ctl_de_i[4] & (~lsu_fpu_de | ~lsu_lsm_de));
  assign lsu_lsm_de    = instr_lsu_ctl_de_i[5];
  assign lsu_lsm_cc_de = instr_lsu_ctl_de_i[5] & instr_ccpass_de_i;
  assign lsu_clrex_de  = instr_lsu_ctl_de_i[14];
  assign lsu_op_de     = ((instr_lsu_ctl_de_i[0] | lsu_clrex_de) &
                          instr_ccpass_de_i);
  assign lsu_fpu_de    = instr_lsu_ctl_de_i[15] & opt_fpu_en;
  assign lsu_wb_valid_de = instr_lsu_ctl_de_i[9];


  assign lsu_de_2_ex_en       = instr_valid_de_i & (lsu_op_de | lsu_valid_ex);

   always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      lsu_valid_ex <= 1'b0;
    else if (lsu_de_2_ex_en)
      lsu_valid_ex <= lsu_op_de;

  assign nxt_instr_lsu_ctl_ex = (instr_lsu_ctl_de_i & {16{lsu_op_de}});

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        instr_lsu_ctl_ex        <= {16{1'b0}};
        instr_lsm_baseinlist_ex <= 1'b0;
      end
    else if (lsu_op_de)
      begin
        instr_lsu_ctl_ex        <= nxt_instr_lsu_ctl_ex;
        instr_lsm_baseinlist_ex <= instr_lsm_baseinlist_de_i;
      end

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      opt_fp_ls_rf_addr_ex <= 5'b0000;
    else if (lsu_op_de & opt_fpu_en)
      opt_fp_ls_rf_addr_ex <= {5{lsu_fpu_de}} & fpu_first_reg_de_i;

  assign lsu_fp_ls_rf_addr_ex = {5{opt_fpu_en}} & opt_fp_ls_rf_addr_ex;

  assign lsu_rf_addr_we = lsu_op_de & ~lsu_clrex_de;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        rf_rd_a_addr_ex         <= 4'b0000;
        rf_rd_b_addr_ex         <= 4'b0000;
        instr_rf_rd_c_addr_ex   <= 4'b0000;
        rf_wr_d_addr_ex         <= 4'b0000;
      end
    else if (lsu_rf_addr_we)
      begin
        rf_rd_a_addr_ex         <= rf_rd_a_addr_de_i;
        rf_rd_b_addr_ex         <= rf_rd_b_addr_de_i;
        instr_rf_rd_c_addr_ex   <= instr_rf_rd_c_addr_de_i;
        rf_wr_d_addr_ex         <= rf_wr_d_addr_de_i;
      end

  assign lsu_fpu_ex       = instr_lsu_ctl_ex[15] & lsu_valid_ex & opt_fpu_en;
  assign lsu_lsm_ex       = instr_lsu_ctl_ex[5]  & lsu_valid_ex;
  assign lsu_flsm_ex      = (instr_lsu_ctl_ex[5]  & instr_lsu_ctl_ex[15]  &
                             lsu_valid_ex);
  assign lsu_isld_ex      = instr_lsu_ctl_ex[6]  & lsu_valid_ex;
  assign lsu_wb_valid_ex  = instr_lsu_ctl_ex[9]  & lsu_valid_ex;
  assign lsu_wb_after_ex  = instr_lsu_ctl_ex[10];
  assign lsu_excl_ex      = instr_lsu_ctl_ex[13] & lsu_valid_ex;
  assign lsu_clrex_ex     = instr_lsu_ctl_ex[14] & lsu_valid_ex;
  assign instr_excl_ex    = lsu_excl_ex;


  assign str_imm_ex           =  instr_immed_valid_ex_i & ~lsu_isld_ex;

  assign lsu_lfault_ex        = (|fault_lsu_local_ex_i);

  assign lsu_excp_ex          = lsu_lfault_ex | nvic_int_invoke_i;

  assign lsu_ahb_rdy_ex       = mtx_ahb_hreadyd_i & mtx_ahb_haddraccd_i;

  assign lsu_ahb_close_ex     = (mtx_ahb_hreadyd_i &
                                 (mtx_ahb_haddraccd_i | lsu_excp_ex));

  assign str_buf_valid_ex     = (str_imm_ex & ~lsu_excl_ex & ~lsu_unalgn_ex_i);

  assign int_first_instr_rdy  = int_first_instr_valid_i & ~mtx_bm_active_i;

  assign psr_it_valid_ex      = (|psr_ex_i[11:10]) | (|psr_ex_i[26:25]);

  assign psr_ici_valid_ex     = ~psr_it_valid_ex & (|psr_ex_i[15:12]);

  assign psr_it_nonzero_ex    =  (|psr_ex_i[26:25]) | (|psr_ex_i[15:10]);

  assign lsu_res_it_valid_ex  = ((|lsu_psr_ex_i[11:10]) |
                                 (|lsu_psr_ex_i[26:25]));

  assign lsu_st_imm_nonbuf_ex = (mpu_addrd_non_buf_i & lsu_wb_valid_ex &
                                 str_imm_ex);

  assign lsu_pline_valid_de   = (instr_ls_allow_pline_de_i &
                                 ~lsu_st_imm_nonbuf_ex);

  assign lsu_in_nmihf         = (|nvic_int_in_nmihf_i[1:0]);

  assign lsu_ici_ret_ex       = (state==`CORE_LSU_LSM_INTC1);

  assign lsu_fp_immed_stk     = fpu_stack_ctl_i[1] & fpu_enabled_i & opt_fpu_en;

  assign lsu_fp_immed_unstk   = (fpu_unstack_ctl_i[1] & fpu_enabled_i &
                                 opt_fpu_en);

  assign lsu_request = { // Double Fault
                         ((fault_dblfault_ex_i & ~nvic_int_invoke_i &
                           ~dbg_halted_i) |
                          iflush_pflush_ex_i),
                         (nvic_dbg_trans_i & dbg_halted_i),
                         dbg_halt_set_i,
                         (~instr_detect_exit_ex_i & nvic_int_invoke_i &
                          ~int_entry & ~dbg_halted_i),
                         ( instr_detect_exit_ex_i & ~fault_dblfault_ex_i) };


  assign idle_sel[11:0]   = { // Store FPU state
                              fpu_lazy_start_i & instr_ccpass_de_i & ~lsu_lfault_ex,
                              lsu_lfault_ex & ~(|lsu_request),
                              lsu_addpc_de  & instr_ccpass_de_i & ~lsu_lfault_ex,
                              lsu_tb_de     & instr_ccpass_de_i & ~lsu_lfault_ex,
                              lsu_lsm_de    & instr_ccpass_de_i & ~lsu_lfault_ex,
                              lsu_lsd_de    & instr_ccpass_de_i & ~lsu_lfault_ex,
                              lsu_lss_de    & instr_ccpass_de_i & ~lsu_lfault_ex,
                              lsu_request };

  assign lsm_continue     = (// LSM is decode
                             lsu_lsm_de &
                             ((((state==`CORE_LSU_INT_L_R14) |
                                (state==`CORE_LSU_LAZY_EXIT)) & psr_ici_valid_ex) |
                              lsm_reg_valid_ex));

  assign lsu_ret_lsm_ex   = ((state==`CORE_LSU_IDLE      |
                              state==`CORE_LSU_INT_L_R14 |
                              state==`CORE_LSU_LAZY_EXIT |
                              state==`CORE_LSU_INT_RETURN) & lsm_continue);

  assign lsu_ret_state_ex = (lsu_ret_lsm_ex ? `CORE_LSU_LSM_INTC1 :
                                              `CORE_LSU_LSM_NONSEQ);

  assign idle_nxt_state   = (({6{idle_sel[0]}}  & `CORE_LSU_INT_EXIT) |
                             ({6{idle_sel[1]}}  & `CORE_LSU_INT_S_PC) |
                             ({6{idle_sel[2]}}  & `CORE_LSU_DBG_PC_1) |
                             ({6{idle_sel[3]}}  & `CORE_LSU_DBG_R_0)  |
                             ({6{idle_sel[4]}}  & `CORE_LSU_DFAULT_1) |
                             ({6{idle_sel[5]}}  & `CORE_LSU_LSS_N)    |
                             ({6{idle_sel[6]}}  & `CORE_LSU_LSD_N)    |
                             ({6{idle_sel[7]}}  & lsu_ret_state_ex)   |
                             ({6{idle_sel[8]}}  & `CORE_LSU_LTB_N)    |
                             ({6{idle_sel[9]}}  & `CORE_LSU_ADD4)     |
                             ({6{idle_sel[10]}} & `CORE_LSU_FAULT)    |
                             ({6{idle_sel[11]}} & `CORE_LSU_LAZY_ENTRY));

  assign idle_valid      = (rst_pc_valid_ex_i &
                            (|(idle_sel &
                               { {9{1'b1}}, {2{mtx_ahb_hreadyd_i}}, 1'b1 })));


  assign lss_pipe_state_ex = ((instr_ccpass_de_i & lsu_pline_valid_de &
                               ~mpu_canceld_i) ?
                              ((opt_ext_en & lsu_wb_valid_de) ?
                               `CORE_LSU_LSS_B : `CORE_LSU_LSS_P) :
                              `CORE_LSU_LSS_D);

  assign lss_data_state_ex = ((lsu_unalgn_ex_i & ~mpu_canceld_i &
                               ~lsu_st_imm_nonbuf_ex) ?
                              `CORE_LSU_LSS_U : lss_pipe_state_ex);

  assign lss_pipe_adv_de   = lsu_pline_valid_de | instr_ls_single_adv_de_i;

  assign lss_stall_de      = (((rf_wr_d_addr_ex==4'b1111) & lsu_isld_wr & ~lsu_fpu_wr) |
                              ((rf_wr_d_addr_ex==4'b1101) & lsu_isld_wr & ~lsu_fpu_wr &
                               nvic_int_invoke_i));


  always @ (state or mtx_ahb_hreadyd_i or mtx_ahb_haddraccd_i or
            mpu_canceld_i or str_buf_valid_ex or lsu_ahb_close_ex or
            lsu_isld_ex or rst_pc_valid_ex_i or idle_nxt_state or idle_valid or
            lss_data_state_ex or lss_pipe_state_ex or nvic_int_invoke_i or
            nvic_int_tail_i or int_first_instr_rdy or lsu_excp_ex or
            nvic_slp_asleep_i or nvic_slp_sleeponexit_i or lsu_lfault_ex or
            lsu_in_nmihf or lsm_reg_pc_ex or lsu_unalgn_ex_i or
            lsu_exit_valid_ex_i or lsm_list_valid_ex or lsu_fp_immed_stk or
            instr_lsu_int_ilock_de_i or lss_stall_de or lsu_ahb_rdy_ex or
            lss_pipe_adv_de or mpu_addrd_non_buf_i or lsu_fp_immed_unstk or
            ahb_const_unstk_i)

    begin

      case (state)

        `CORE_LSU_IDLE :
          begin
            stall     = ~rst_pc_valid_ex_i;
            state_we  = idle_valid;
            nxt_state = idle_nxt_state;
          end

        `CORE_LSU_LSS_N,        `CORE_LSU_LSS_P,      `CORE_LSU_LSS_B :
          begin
            stall     = (lsu_excp_ex | lsu_unalgn_ex_i | ~lsu_ahb_rdy_ex |
                         (~lss_pipe_adv_de & ~str_buf_valid_ex) |
                         (~lss_pipe_adv_de & mpu_addrd_non_buf_i));
            state_we  = lsu_ahb_close_ex;
            nxt_state = ((lsu_excp_ex |
                          (str_buf_valid_ex & ~mpu_canceld_i &
                           ~mpu_addrd_non_buf_i)) ?
                         idle_nxt_state : lss_data_state_ex);
          end
        `CORE_LSU_LSS_U :
          begin
            stall     = (// Control phase link
                         ~(mtx_ahb_hreadyd_i | lss_pipe_adv_de) |
                         lsu_lfault_ex |
                         lss_stall_de);
            state_we  = 1'b1;
            nxt_state = mtx_ahb_hreadyd_i ? idle_nxt_state : lss_pipe_state_ex;
          end
        `CORE_LSU_LSS_D :
          begin
            stall     = ~mtx_ahb_hreadyd_i | lsu_lfault_ex | lss_stall_de;
            state_we  = mtx_ahb_hreadyd_i;
            nxt_state = idle_nxt_state;
          end

        `CORE_LSU_LSD_N :
          begin
            stall     = 1'b1;
            state_we  = lsu_ahb_close_ex;
            nxt_state = lsu_excp_ex ? idle_nxt_state : `CORE_LSU_LSD_S;
          end
        `CORE_LSU_LSD_S :
          begin
            stall     = 1'b1;
            state_we  = lsu_ahb_close_ex;
            nxt_state = lsu_excp_ex ? idle_nxt_state : `CORE_LSU_LSD_D;
          end
        `CORE_LSU_LSD_D :
          begin
            stall     = ~mtx_ahb_hreadyd_i | lsu_lfault_ex;
            state_we  = mtx_ahb_hreadyd_i;
            nxt_state = idle_nxt_state;
          end

        `CORE_LSU_LSM_INTC1 :
          begin
            stall     = 1'b1;
            state_we  = 1'b1;
            nxt_state = lsu_excp_ex ? idle_nxt_state : `CORE_LSU_LSM_INTC2;
          end
        `CORE_LSU_LSM_INTC2 :
          begin
            stall     = 1'b1;
            state_we  = 1'b1;
            nxt_state = nvic_int_invoke_i ? idle_nxt_state : `CORE_LSU_LSM_NONSEQ ;
          end
        `CORE_LSU_LSM_NONSEQ :
          begin
            stall     = 1'b1;
            state_we  = lsu_ahb_close_ex;
            nxt_state = lsu_excp_ex ? idle_nxt_state : `CORE_LSU_LSM_Rx;
          end
        `CORE_LSU_LSM_Rx :
          begin
            stall     = (~mtx_ahb_hreadyd_i |
                         lsm_list_valid_ex |
                         (lsu_isld_ex & lsm_reg_pc_ex) |
                         lsu_lfault_ex);
            state_we  = mtx_ahb_hreadyd_i & (~lsm_list_valid_ex | lsu_excp_ex);
            nxt_state = idle_nxt_state;
          end

        `CORE_LSU_LTB_N :
          begin
            stall     = 1'b1;
            state_we  = lsu_ahb_rdy_ex;
            nxt_state = lsu_excp_ex ? idle_nxt_state : `CORE_LSU_LTB_D;
          end
        `CORE_LSU_LTB_D :
          begin
            stall     = 1'b1;
            state_we  = mtx_ahb_hreadyd_i;
            nxt_state = lsu_excp_ex ? idle_nxt_state : `CORE_LSU_LTB_W;
          end
        `CORE_LSU_LTB_W :
          begin
            stall     = ~mtx_ahb_hreadyd_i;
            state_we  = mtx_ahb_hreadyd_i;
            nxt_state = idle_nxt_state;
          end

        `CORE_LSU_ADD4 :
          begin
            stall     = 1'b0;
            state_we  = 1'b1;
            nxt_state = mtx_ahb_hreadyd_i ? idle_nxt_state : `CORE_LSU_IDLE;
          end


        `CORE_LSU_INT_S_PC :
          begin
            stall     = 1'b1;
            state_we  = lsu_ahb_rdy_ex | lsu_exit_valid_ex_i;
            nxt_state = (lsu_exit_valid_ex_i ? `CORE_LSU_INT_EXIT :
                                               `CORE_LSU_INT_S_FLAGS);
          end
        `CORE_LSU_INT_S_FLAGS :
          begin
            stall     = 1'b1;
            state_we  = (mtx_ahb_hreadyd_i &
                         (mtx_ahb_haddraccd_i | lsu_lfault_ex));
            nxt_state = (lsu_lfault_ex ? `CORE_LSU_INT_SFAULT_1 :
                                         `CORE_LSU_INT_S_RX);
          end
        `CORE_LSU_INT_S_RX :
          begin
            stall     = 1'b1;
            state_we  = (mtx_ahb_hreadyd_i &
                         ((mtx_ahb_haddraccd_i & ~lsm_list_valid_ex) |
                          lsu_lfault_ex));
            nxt_state = (lsu_lfault_ex ? `CORE_LSU_INT_SFAULT_1 :
                          (lsu_fp_immed_stk ?
                           `CORE_LSU_FP_S_SX : `CORE_LSU_INT_ENTRY));
          end
        `CORE_LSU_FP_S_SX :
          begin
            stall     = 1'b1;
            state_we  = (mtx_ahb_hreadyd_i &
                         ((mtx_ahb_haddraccd_i & ~lsm_list_valid_ex) |
                          lsu_lfault_ex));
            nxt_state = (lsu_lfault_ex ? `CORE_LSU_INT_SFAULT_1 :
                                         `CORE_LSU_INT_S_FPSCR);
          end
        `CORE_LSU_INT_S_FPSCR :
          begin
            stall     = 1'b1;
            state_we  = mtx_ahb_hreadyd_i & (mtx_ahb_haddraccd_i | lsu_lfault_ex);
            nxt_state = (lsu_lfault_ex ? `CORE_LSU_INT_SFAULT_1 :
                                         `CORE_LSU_INT_ENTRY);
          end
        `CORE_LSU_INT_ENTRY :
          begin
            stall     = (~(mtx_ahb_hreadyd_i & int_first_instr_rdy) |
                         lsu_lfault_ex | instr_lsu_int_ilock_de_i);
            state_we  =  (mtx_ahb_hreadyd_i & (int_first_instr_rdy | lsu_lfault_ex));
            nxt_state = (lsu_lfault_ex ? `CORE_LSU_INT_SFAULT_1 :
                         (instr_lsu_int_ilock_de_i ? `CORE_LSU_INT_ENLCK :
                                                     idle_nxt_state));
          end
        `CORE_LSU_INT_ENLCK :
          begin
            stall     = ~int_first_instr_rdy;
            state_we  = int_first_instr_rdy;
            nxt_state = idle_nxt_state;
          end


        `CORE_LSU_LAZY_ENTRY :
          begin
            stall     = 1'b1;
            state_we  = lsu_ahb_rdy_ex;
            nxt_state = (nvic_int_invoke_i ? `CORE_LSU_INT_S_PC :
                                             `CORE_LSU_LAZY_S_S0);
          end
        `CORE_LSU_LAZY_S_S0 :
          begin
            stall     = 1'b1;
            state_we  = lsu_ahb_rdy_ex;
            nxt_state = (nvic_int_invoke_i ? `CORE_LSU_INT_S_PC :
                                             `CORE_LSU_LAZY_S_SX);
          end
        `CORE_LSU_LAZY_S_SX :
          begin
            stall     = 1'b1;
            state_we  = (mtx_ahb_hreadyd_i &
                         ((mtx_ahb_haddraccd_i & ~lsm_list_valid_ex) | lsu_excp_ex));
            nxt_state = (nvic_int_invoke_i ? `CORE_LSU_INT_S_PC :
                         (lsu_lfault_ex ? `CORE_LSU_LAZY_FAULT_1 :
                                          `CORE_LSU_LAZY_S_FPSCR));
          end
        `CORE_LSU_LAZY_S_FPSCR :
          begin
            stall     = 1'b1;
            state_we  = mtx_ahb_hreadyd_i & (mtx_ahb_haddraccd_i | lsu_excp_ex);
            nxt_state = (nvic_int_invoke_i ? `CORE_LSU_INT_S_PC :
                         (lsu_lfault_ex ? `CORE_LSU_LAZY_FAULT_1 :
                                          `CORE_LSU_LAZY_EXIT));
          end
        `CORE_LSU_LAZY_EXIT :
          begin
            stall     = (lsu_excp_ex | ~mtx_ahb_hreadyd_i);
            state_we  = mtx_ahb_hreadyd_i;
            nxt_state = (nvic_int_invoke_i ? `CORE_LSU_INT_S_PC :
                         (lsu_lfault_ex ? `CORE_LSU_LAZY_FAULT_1 : idle_nxt_state));
          end
        `CORE_LSU_LAZY_FAULT_1 :
          begin
            stall     = 1'b1;
            state_we  = 1'b1;
            nxt_state = (nvic_int_invoke_i ? `CORE_LSU_INT_S_PC :
                                             `CORE_LSU_LAZY_FAULT_2);
          end

        `CORE_LSU_LAZY_FAULT_2 :
          begin
            stall     = 1'b1;
            state_we  = 1'b1;
            nxt_state = idle_nxt_state;
          end


        `CORE_LSU_INT_EXIT :
          begin
            stall     = 1'b1;
            state_we  = (mtx_ahb_hreadyd_i &
                         (mtx_ahb_haddraccd_i | nvic_int_tail_i |
                          nvic_int_invoke_i    | nvic_slp_sleeponexit_i));
            nxt_state = (nvic_slp_sleeponexit_i ? `CORE_LSU_SLEEP :
                         ((nvic_int_invoke_i | nvic_int_tail_i) ?
                          `CORE_LSU_INT_ENTRY : `CORE_LSU_INT_L_PC));
          end
        `CORE_LSU_INT_L_PC :
          begin
            stall     = 1'b1;
            state_we  = lsu_ahb_close_ex;
            nxt_state = (lsu_excp_ex ?
                         (lsu_in_nmihf ? `CORE_LSU_INT_UFAULT_2 :
                                         `CORE_LSU_INT_UFAULT_1) :
                         `CORE_LSU_INT_L_FLAGS);
          end
        `CORE_LSU_INT_L_FLAGS :
          begin
            stall     = 1'b1;
            state_we  = lsu_ahb_close_ex & ~ahb_const_unstk_i;
            nxt_state = (lsu_excp_ex ? `CORE_LSU_INT_UFAULT_1 :
                                       `CORE_LSU_INT_L_RX);
          end
        `CORE_LSU_INT_L_RX :
          begin
            stall     = 1'b1;
            state_we  = (mtx_ahb_hreadyd_i &
                         ((mtx_ahb_haddraccd_i & ~lsm_list_valid_ex) |
                          lsu_excp_ex));
            nxt_state = (lsu_excp_ex ? `CORE_LSU_INT_UFAULT_1 :
                          (lsu_fp_immed_unstk ?
                           `CORE_LSU_FP_L_SX : `CORE_LSU_INT_L_R14));
          end
        `CORE_LSU_INT_L_R14 :
          begin
            stall     = (~(mtx_ahb_hreadyd_i & int_first_instr_rdy) | lsu_excp_ex |
                         instr_lsu_int_ilock_de_i);
            state_we  = mtx_ahb_hreadyd_i;
            nxt_state = (lsu_excp_ex ? `CORE_LSU_INT_UFAULT_1 :
                         ((int_first_instr_rdy & ~instr_lsu_int_ilock_de_i) ?
                          idle_nxt_state : `CORE_LSU_INT_RETURN));
          end
        `CORE_LSU_FP_L_SX :
          begin
            stall     = 1'b1;
            state_we  = (mtx_ahb_hreadyd_i &
                         ((mtx_ahb_haddraccd_i & ~lsm_list_valid_ex) |
                          lsu_excp_ex));
            nxt_state = (lsu_excp_ex ? `CORE_LSU_INT_UFAULT_1 :
                                       `CORE_LSU_FP_L_FLAGS);
          end
        `CORE_LSU_FP_L_FLAGS :
          begin
            stall     = 1'b1;
            state_we  = lsu_ahb_close_ex;
            nxt_state = (lsu_excp_ex ? `CORE_LSU_INT_UFAULT_1 :
                                       `CORE_LSU_INT_L_R14);
          end
        `CORE_LSU_INT_RETURN :
          begin
            stall     = (~(int_first_instr_rdy & ~nvic_int_invoke_i) |
                         lsu_lfault_ex);
            state_we  = (nvic_int_invoke_i | int_first_instr_rdy);
            nxt_state = (nvic_int_invoke_i ? `CORE_LSU_INT_UFAULT_1 :
                                             idle_nxt_state);
          end

        `CORE_LSU_INT_SFAULT_1 :
          begin
            stall     = 1'b1;
            state_we  = int_first_instr_rdy | ~lsu_in_nmihf;
            nxt_state = `CORE_LSU_INT_SFAULT_2;
          end
        `CORE_LSU_INT_SFAULT_2 :
          begin
            stall     = 1'b1;
            state_we  = 1'b1;
            nxt_state = `CORE_LSU_INT_ENTRY;
          end
        `CORE_LSU_INT_UFAULT_1 :
          begin
            stall     = 1'b1;
            state_we  = (nvic_int_invoke_i | (lsu_in_nmihf & int_first_instr_rdy));
            nxt_state = (nvic_int_invoke_i ? `CORE_LSU_INT_ENTRY :
                                             `CORE_LSU_INT_UFAULT_2);
          end
        `CORE_LSU_INT_UFAULT_2 :
          begin
            stall     = 1'b1;
            state_we  = 1'b1;
            nxt_state = (nvic_int_invoke_i ? `CORE_LSU_INT_UFAULT_1 :
                                             `CORE_LSU_INT_RETURN);
        end

        `CORE_LSU_FAULT :
          begin
            stall     = 1'b0; // do not stall
            state_we  = 1'b1;
            nxt_state = idle_nxt_state;
          end

        `CORE_LSU_DFAULT_1 :
          begin
            stall     = 1'b1;
            state_we  = mtx_ahb_hreadyd_i;
            nxt_state = nvic_int_invoke_i ? idle_nxt_state : `CORE_LSU_DFAULT_2;
          end
        `CORE_LSU_DFAULT_2 :
          begin
            stall     = 1'b1;
            state_we  = lsu_ahb_rdy_ex;
            nxt_state = idle_nxt_state;
          end

        `CORE_LSU_SLEEP :
          begin
            stall     = 1'b1;
            state_we  = (~nvic_slp_asleep_i &
                         (nvic_int_invoke_i | mtx_ahb_haddraccd_i));
            nxt_state = (nvic_int_invoke_i ? `CORE_LSU_INT_ENTRY :
                                             `CORE_LSU_INT_L_PC);
          end

        `CORE_LSU_DBG_PC_1 :
          begin
            stall     = 1'b1;
            state_we  = mtx_ahb_hreadyd_i;
            nxt_state = `CORE_LSU_DBG_PC_2;
        end
        `CORE_LSU_DBG_PC_2 :
          begin
            stall     = 1'b1;
            state_we  = 1'b1;
            nxt_state = idle_nxt_state;
       end
        `CORE_LSU_DBG_R_0 :
          begin
            stall     = 1'b1;
            state_we  = 1'b1;
            nxt_state = `CORE_LSU_DBG_R_1;
          end
        `CORE_LSU_DBG_R_1 :
          begin
            stall     = 1'b1;
            state_we  = lsu_ahb_rdy_ex;
            nxt_state = `CORE_LSU_DBG_R_2;
          end
        `CORE_LSU_DBG_R_2 :
          begin
            stall     = 1'b1;
            state_we  = mtx_ahb_hreadyd_i;
            nxt_state = `CORE_LSU_IDLE;
          end

        default :
          begin
            stall     = 1'bx;
            state_we  = 1'bx;
            nxt_state = 6'bxxxxxx;
          end
      endcase
    end

  assign lsu_stall_late_ex = ((// In LSS state
                               ((state == `CORE_LSU_LSS_N) |
                                (state == `CORE_LSU_LSS_P) |
                                (state == `CORE_LSU_LSS_B)) &
                               (// with MPU cancel
                                mpu_canceld_i |
                                (~lss_pipe_adv_de & mpu_addrd_non_buf_i))) |
                              ((state == `CORE_LSU_LSS_U) & mpu_canceld_i));


  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      state     <= `CORE_LSU_IDLE;
    else if (state_we)
      state     <= nxt_state;

  assign lsu_stackpush_rdy_ex = ((state==`CORE_LSU_INT_SFAULT_1)                  |
                                 (state==`CORE_LSU_INT_SFAULT_2)                  |
                                 (state==`CORE_LSU_INT_S_RX & lsm_reg_nonzero_ex) |
                                 (state==`CORE_LSU_INT_ENTRY)                     |
                                 (state==`CORE_LSU_INT_ENLCK));

  assign dbg_ready_we  = (dbg_ready & lsu_request[3]) | nxt_dbg_ready;
  assign nxt_dbg_ready = (mtx_ahb_hreadyd_i & ~dbg_ready &
                          (state==`CORE_LSU_DBG_R_2));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      dbg_ready <= 1'b1;
    else if (dbg_ready_we)
      dbg_ready <= nxt_dbg_ready;



  assign stack_op_int  = ((state == `CORE_LSU_INT_L_RX) |
                          (state == `CORE_LSU_INT_S_RX));
  assign stack_op_fp   = ((state == `CORE_LSU_FP_L_SX) |
                          (state == `CORE_LSU_FP_S_SX) |
                          (state == `CORE_LSU_LAZY_S_S0) |
                          (state == `CORE_LSU_LAZY_S_SX)) & opt_fpu_en;

  assign stack_regs_int = (((state==`CORE_LSU_INT_L_RX) &
                            ~lsu_fp_immed_unstk) ?
                           {{16{1'b0}}, 16'b0001000000001111} :
                           {{16{1'b0}}, 16'b0101000000001111});
  assign stack_regs_fp  =  {{16{1'b0}}, 16'b1111111111111111};

  assign stack_regs     = ((stack_op_int | ~opt_fpu_en) ? stack_regs_int :
                                                          stack_regs_fp);

  assign fp_lsm_regs_upper = 32'hFFFF_FFFF << lsu_fp_ls_rf_addr_ex;
  assign fp_lsm_regs_lower = ~(fp_lsm_regs_upper << instr_immed_ex_i[9:2]);
  assign fp_lsm_reg_list   = (fp_lsm_regs_upper & fp_lsm_regs_lower &
                              {32{opt_fpu_en}});

  always @ (stack_op_int or stack_op_fp or stack_regs or
            fp_lsm_reg_list or instr_immed_ex_i or lsu_flsm_ex)
    case ({(stack_op_int | stack_op_fp), lsu_flsm_ex})
      2'b10,
      2'b11:   non_r13_immed_ex = stack_regs;
      2'b01:   non_r13_immed_ex = fp_lsm_reg_list;
      2'b00:   non_r13_immed_ex = {{16{1'b0}},
                                   (instr_immed_ex_i[15:0] &
                                    16'b1101111111111111)};
      default: non_r13_immed_ex = 32'hXXXXXXXX;
    endcase


  assign lsm_mask_0 = (({ 3{lsm_reg_ex[1:0] == 2'b00}} & 3'b111) |
                       ({ 3{lsm_reg_ex[1:0] == 2'b01}} & 3'b110) |
                       ({ 3{lsm_reg_ex[1:0] == 2'b10}} & 3'b100) |
                       ({ 3{lsm_reg_ex[1:0] == 2'b11}} & 3'b000));

  assign lsm_mask_1 = (({31{lsm_reg_ex[3:2] == 2'b00}} & { 28'hFFFFFFF, lsm_mask_0        }) |
                       ({31{lsm_reg_ex[3:2] == 2'b01}} & { 24'hFFFFFF,  lsm_mask_0,  4'h0 }) |
                       ({31{lsm_reg_ex[3:2] == 2'b10}} & { 20'hFFFFF,   lsm_mask_0,  8'h0 }) |
                       ({31{lsm_reg_ex[3:2] == 2'b11}} & { 16'hFFFF,    lsm_mask_0, 12'h0 }));

  assign lsm_mask_2 = ((lsm_reg_ex[4] & opt_fpu_en) ?
                       {lsm_mask_1[14:0], {16{1'b0}}} : lsm_mask_1);

  assign lsm_masked_list = ({ lsm_mask_2, ~lsm_reg_valid_ex } &
                            non_r13_immed_ex);

  assign lsm_ripple_list = { (|lsm_masked_list[30:0]), (|lsm_masked_list[29:0]),
                             (|lsm_masked_list[28:0]), (|lsm_masked_list[27:0]),
                             (|lsm_masked_list[26:0]), (|lsm_masked_list[25:0]),
                             (|lsm_masked_list[24:0]), (|lsm_masked_list[23:0]),
                             (|lsm_masked_list[22:0]), (|lsm_masked_list[21:0]),
                             (|lsm_masked_list[20:0]), (|lsm_masked_list[19:0]),
                             (|lsm_masked_list[18:0]), (|lsm_masked_list[17:0]),
                             (|lsm_masked_list[16:0]), (|lsm_masked_list[15:0]),
                             (|lsm_masked_list[14:0]), (|lsm_masked_list[13:0]),
                             (|lsm_masked_list[12:0]), (|lsm_masked_list[11:0]),
                             (|lsm_masked_list[10:0]), (|lsm_masked_list[ 9:0]),
                             (|lsm_masked_list[ 8:0]), (|lsm_masked_list[ 7:0]),
                             (|lsm_masked_list[ 6:0]), (|lsm_masked_list[ 5:0]),
                             (|lsm_masked_list[ 4:0]), (|lsm_masked_list[ 3:0]),
                             (|lsm_masked_list[ 2:0]), (|lsm_masked_list[ 1:0]),
                             lsm_masked_list[0] };

  assign lsm_reg_sel    = lsm_masked_list & ~{lsm_ripple_list[30:0], 1'b1};

  assign lsm_nxt_reg_ex = (({5{lsm_reg_sel[ 0]}} & 5'b0_0000) |
                           ({5{lsm_reg_sel[ 1]}} & 5'b0_0001) |
                           ({5{lsm_reg_sel[ 2]}} & 5'b0_0010) |
                           ({5{lsm_reg_sel[ 3]}} & 5'b0_0011) |
                           ({5{lsm_reg_sel[ 4]}} & 5'b0_0100) |
                           ({5{lsm_reg_sel[ 5]}} & 5'b0_0101) |
                           ({5{lsm_reg_sel[ 6]}} & 5'b0_0110) |
                           ({5{lsm_reg_sel[ 7]}} & 5'b0_0111) |
                           ({5{lsm_reg_sel[ 8]}} & 5'b0_1000) |
                           ({5{lsm_reg_sel[ 9]}} & 5'b0_1001) |
                           ({5{lsm_reg_sel[10]}} & 5'b0_1010) |
                           ({5{lsm_reg_sel[11]}} & 5'b0_1011) |
                           ({5{lsm_reg_sel[12]}} & 5'b0_1100) |
                           ({5{lsm_reg_sel[13]}} & 5'b0_1101) | // Used for FP stack only
                           ({5{lsm_reg_sel[14]}} & 5'b0_1110) |
                           ({5{lsm_reg_sel[15]}} & 5'b0_1111) |
                           ({5{lsm_reg_sel[16]}} & 5'b1_0000) | // Reg16-31 for FP only
                           ({5{lsm_reg_sel[17]}} & 5'b1_0001) |
                           ({5{lsm_reg_sel[18]}} & 5'b1_0010) |
                           ({5{lsm_reg_sel[19]}} & 5'b1_0011) |
                           ({5{lsm_reg_sel[20]}} & 5'b1_0100) |
                           ({5{lsm_reg_sel[21]}} & 5'b1_0101) |
                           ({5{lsm_reg_sel[22]}} & 5'b1_0110) |
                           ({5{lsm_reg_sel[23]}} & 5'b1_0111) |
                           ({5{lsm_reg_sel[24]}} & 5'b1_1000) |
                           ({5{lsm_reg_sel[25]}} & 5'b1_1001) |
                           ({5{lsm_reg_sel[26]}} & 5'b1_1010) |
                           ({5{lsm_reg_sel[27]}} & 5'b1_1011) |
                           ({5{lsm_reg_sel[28]}} & 5'b1_1100) |
                           ({5{lsm_reg_sel[29]}} & 5'b1_1101) |
                           ({5{lsm_reg_sel[30]}} & 5'b1_1110) |
                           ({5{lsm_reg_sel[31]}} & 5'b1_1111));

  assign nxt_lsm_fpu_ex    = ((state == `CORE_LSU_LSM_NONSEQ) ?
                              lsu_fp_ls_rf_addr_ex : (lsm_reg_ex + 1'b1));

  assign lsm_list_valid_ex = (|lsm_masked_list);

  always @ (state or lsu_ahb_rdy_ex or lsu_excp_ex or lsm_reg_valid_ex or
            mtx_ahb_hreadyd_i or lsu_lfault_ex or nvic_int_invoke_i or
            mtx_ahb_haddraccd_i or lsm_last_ex or lsu_in_nmihf or
            int_first_instr_rdy or psr_it_valid_ex or ldm_base_updated_ex or
            nvic_dbg_reg_wr_i or nvic_dbg_reg_addr_i or lsm_nxt_reg_ex or
            lsm_list_valid_ex or lsu_psr_ex_i or psr_ex_i or lsu_flsm_ex or
            lsu_res_it_valid_ex or psr_ici_valid_ex or lsm_reg_nonzero_ex or
            lsu_fp_immed_stk or lsu_fp_immed_unstk or lsm_reg_ex or
            lsm_fpu_first_ex or fault_dblfault_ex_i or fault_lsu_local_ex_i)
    case (state)
      `CORE_LSU_LSM_NONSEQ :
        begin
          lsm_reg_we           = (lsu_ahb_rdy_ex & ~lsm_reg_nonzero_ex &
                                  ~lsu_excp_ex);
          nxt_lsm_reg_ex       = lsm_nxt_reg_ex;
          nxt_lsm_reg_valid_ex = lsm_list_valid_ex;
        end
      `CORE_LSU_LSM_Rx :
        begin
          lsm_reg_we           = (mtx_ahb_hreadyd_i &
                                  (lsu_lfault_ex | nvic_int_invoke_i |
                                   mtx_ahb_haddraccd_i | lsm_last_ex));
          nxt_lsm_reg_ex       = ((((ldm_base_updated_ex |
                                     lsm_fpu_first_ex) &
                                    nvic_int_invoke_i) |
                                   lsu_lfault_ex) ?
                                  5'b0_0000 : lsm_nxt_reg_ex);
          nxt_lsm_reg_valid_ex = (lsm_list_valid_ex &
                                  ~(((ldm_base_updated_ex |
                                      lsm_fpu_first_ex) &
                                     nvic_int_invoke_i) |
                                    lsu_lfault_ex));
        end
       `CORE_LSU_FAULT :
        begin
          lsm_reg_we           = (fault_dblfault_ex_i & ~nvic_int_invoke_i &
                                  lsu_flsm_ex);
          nxt_lsm_reg_ex       = {1'b0,lsm_reg_ex[4:1]};
          nxt_lsm_reg_valid_ex = lsm_reg_valid_ex;
        end
      `CORE_LSU_INT_ENTRY :
        begin
          lsm_reg_we           = ((lsm_reg_nonzero_ex | lsm_reg_valid_ex) &
                                  mtx_ahb_hreadyd_i);
          nxt_lsm_reg_ex       = 5'b0_0000;
          nxt_lsm_reg_valid_ex = 1'b0;
        end
      `CORE_LSU_LAZY_EXIT,    `CORE_LSU_LAZY_FAULT_1 :
        begin
          lsm_reg_we           = (((lsm_reg_nonzero_ex | lsm_reg_valid_ex) &
                                   mtx_ahb_hreadyd_i) |
                                  (~lsm_reg_valid_ex & lsu_ahb_rdy_ex &
                                   ~psr_it_valid_ex & ~fault_lsu_local_ex_i[0]));
          nxt_lsm_reg_ex       = {1'b0, psr_ex_i[15:12]};
          nxt_lsm_reg_valid_ex = psr_ici_valid_ex;
        end
      `CORE_LSU_INT_L_FLAGS,  `CORE_LSU_INT_S_FLAGS,  `CORE_LSU_INT_S_FPSCR :
        begin
          lsm_reg_we           = lsu_ahb_rdy_ex;
          nxt_lsm_reg_ex       = 5'b0_0000;
          nxt_lsm_reg_valid_ex = 1'b1;
        end
      `CORE_LSU_FP_S_SX,      `CORE_LSU_FP_L_SX :
        begin
          lsm_reg_we           = lsu_ahb_rdy_ex;
          nxt_lsm_reg_ex       = lsm_nxt_reg_ex;
          nxt_lsm_reg_valid_ex = lsm_list_valid_ex;
        end
      `CORE_LSU_INT_L_RX :
        begin
          lsm_reg_we           = lsu_ahb_rdy_ex;
          nxt_lsm_reg_ex       = (lsu_fp_immed_unstk &
                                  (lsm_reg_ex==5'b0_1110)) ? 5'b0_0001 :
                                                             lsm_nxt_reg_ex;
          nxt_lsm_reg_valid_ex = ((lsu_fp_immed_unstk &
                                   (lsm_reg_ex==5'b0_1110)) |
                                  lsm_list_valid_ex);
        end
      `CORE_LSU_INT_S_RX :
        begin
          lsm_reg_we           = lsu_ahb_rdy_ex;
          nxt_lsm_reg_ex       = (lsu_fp_immed_stk &
                                  (lsm_reg_ex==5'b0_1110)) ? 5'b0_0000 :
                                                             lsm_nxt_reg_ex;
          nxt_lsm_reg_valid_ex = ((lsu_fp_immed_stk & (lsm_reg_ex==5'b0_1110)) |
                                  lsm_list_valid_ex);
        end
      `CORE_LSU_LAZY_ENTRY :
        begin
          lsm_reg_we           = lsu_ahb_rdy_ex;
          nxt_lsm_reg_ex       = 5'b0_0000;
          nxt_lsm_reg_valid_ex = ~nvic_int_invoke_i;
        end
       `CORE_LSU_LAZY_S_S0,   `CORE_LSU_LAZY_S_SX :
        begin
          lsm_reg_we           = lsu_ahb_rdy_ex;
          nxt_lsm_reg_ex       = lsu_excp_ex ? 5'b0_0000 : lsm_nxt_reg_ex;
          nxt_lsm_reg_valid_ex = lsm_list_valid_ex & ~lsu_excp_ex;
        end
      `CORE_LSU_INT_L_R14 :
        begin
          lsm_reg_we           = (~lsm_reg_valid_ex & lsu_ahb_rdy_ex &
                                  ~psr_it_valid_ex & ~lsu_lfault_ex);
          nxt_lsm_reg_ex       = {1'b0, psr_ex_i[15:12]};
          nxt_lsm_reg_valid_ex = psr_ici_valid_ex;
        end
      `CORE_LSU_INT_UFAULT_1 :
        begin
          lsm_reg_we           = (nvic_int_invoke_i |
                                  (lsu_in_nmihf & int_first_instr_rdy));
          nxt_lsm_reg_ex       = psr_ici_valid_ex ?
                                 {1'b0, psr_ex_i[15:12]} :
                                 5'b0_0000;
          nxt_lsm_reg_valid_ex = psr_ici_valid_ex;
        end
      `CORE_LSU_LSM_INTC1 :
        begin
          lsm_reg_we           = lsu_flsm_ex;
          nxt_lsm_reg_ex       = {lsm_reg_ex[3:0], 1'b0};
          nxt_lsm_reg_valid_ex = 1'b1;
        end
      `CORE_LSU_DBG_R_2 :
        begin
          lsm_reg_we           = (nvic_dbg_reg_wr_i & mtx_ahb_hreadyd_i &
                                  (nvic_dbg_reg_addr_i==7'b001_0000));
          nxt_lsm_reg_ex       = (lsu_res_it_valid_ex ?
                                  5'b0_0000 :
                                  {1'b0, lsu_psr_ex_i[15:12]});
          nxt_lsm_reg_valid_ex = (~lsu_res_it_valid_ex &
                                  (|lsu_psr_ex_i[15:12]));
        end
      `CORE_LSU_IDLE,         `CORE_LSU_LSS_N,        `CORE_LSU_LSS_P,
      `CORE_LSU_LSS_B,        `CORE_LSU_LSS_U,        `CORE_LSU_LSS_D,
      `CORE_LSU_LSD_N,        `CORE_LSU_LSD_S,        `CORE_LSU_LSD_D,
      `CORE_LSU_LSM_INTC2,    `CORE_LSU_LTB_N,        `CORE_LSU_LTB_D,
      `CORE_LSU_LTB_W,        `CORE_LSU_ADD4,         `CORE_LSU_INT_L_PC,
      `CORE_LSU_INT_RETURN,   `CORE_LSU_INT_ENLCK,    `CORE_LSU_INT_EXIT,
      `CORE_LSU_INT_SFAULT_1, `CORE_LSU_INT_SFAULT_2, `CORE_LSU_INT_UFAULT_2,
      `CORE_LSU_DBG_PC_1,     `CORE_LSU_DBG_PC_2,     `CORE_LSU_DBG_R_0,
      `CORE_LSU_DBG_R_1,      `CORE_LSU_SLEEP,        `CORE_LSU_DFAULT_1,
      `CORE_LSU_DFAULT_2,     `CORE_LSU_LAZY_S_FPSCR, `CORE_LSU_FP_L_FLAGS,
      `CORE_LSU_LAZY_FAULT_2, `CORE_LSU_INT_S_PC :
        begin
          lsm_reg_we           = 1'b0;
          nxt_lsm_reg_ex       = 5'b0_0000;
          nxt_lsm_reg_valid_ex = 1'b0;
        end
      default :
        begin
          lsm_reg_we           = 1'bx;
          nxt_lsm_reg_ex       = 5'bx_xxxx;
          nxt_lsm_reg_valid_ex = 1'bx;
        end
    endcase

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        lsm_reg_ex       <= 5'b0_0000;
        lsm_reg_valid_ex <= 1'b0;
      end
    else if (lsm_reg_we)
      begin
        lsm_reg_ex       <= nxt_lsm_reg_ex;
        lsm_reg_valid_ex <= nxt_lsm_reg_valid_ex;
      end


  assign lsm_ici_last_ex = ((state == `CORE_LSU_LSM_NONSEQ) &
                            lsu_ahb_rdy_ex & lsm_reg_nonzero_ex &
                            (lsm_list_length_de == 5'b00000));

  assign nxt_lsm_last_ex = (// Normal case
                            ((lsm_list_length_de == 5'b00001) & lsu_lsm_ex &
                             ~lsu_excp_ex & ~lsu_lsm_de & ~lsu_ici_ret_ex &
                             ~lsm_last_ex & ~fault_dblfault_ex_i &
                             ~dbg_halted_i & ~int_stack_op &
                             ~fpu_lazy_stack_op_i) |
                            lsm_ici_last_ex & ~lsu_excp_ex);

  assign lsm_last_we = lsm_reg_we | lsm_ici_last_ex;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      lsm_last_ex <= 1'b0;
    else if (lsm_last_we)
      lsm_last_ex <= nxt_lsm_last_ex;


  assign lsu_it_valid_ex = (// Stacking copy of ICI into IT
                            ((state==`CORE_LSU_INT_S_FLAGS) & lsu_ahb_rdy_ex &
                             lsm_reg_nonzero_ex & ~psr_it_valid_ex) |
                            ((state==`CORE_LSU_LAZY_ENTRY) & lsu_ahb_rdy_ex &
                             lsm_reg_nonzero_ex & ~psr_it_valid_ex) |
                            ((state == `CORE_LSU_INT_ENTRY) &
                             psr_it_nonzero_ex) |
                            ((((state == `CORE_LSU_LAZY_EXIT) &
                               ~fault_lsu_local_ex_i[0]) |
                              (state == `CORE_LSU_LAZY_FAULT_1)) &
                             lsu_ahb_rdy_ex & psr_it_nonzero_ex &
                             ~psr_it_valid_ex) |
                            ((state == `CORE_LSU_INT_L_FLAGS) & mtx_ahb_hreadyd_i &
                             ~lsu_addracc_wait_reg_i & ~lsu_excp_ex) |
                            ((state == `CORE_LSU_INT_UFAULT_1) &
                             ~psr_it_valid_ex & (|psr_ex_i[15:12]) &
                             (nvic_int_invoke_i |
                              (lsu_in_nmihf & int_first_instr_rdy))) |
                            ((state == `CORE_LSU_INT_L_R14) & ~lsm_reg_valid_ex &
                             lsu_ahb_rdy_ex & ~lsu_lfault_ex &
                             ~psr_it_valid_ex & (|psr_ex_i[15:12])));

  assign lsu_it_ex = ({8{~((state == `CORE_LSU_INT_ENTRY)    |
                           (state == `CORE_LSU_INT_L_R14)    |
                           (state == `CORE_LSU_LAZY_EXIT)    |
                           (state == `CORE_LSU_LAZY_FAULT_1) |
                           (state == `CORE_LSU_INT_UFAULT_1))}} &
                      ((state == `CORE_LSU_INT_L_FLAGS) ?
                       (lsu_res_it_valid_ex ?
                        {lsu_psr_ex_i[15:10],lsu_psr_ex_i[26:25]} :
                        {lsu_psr_ex_i[15:12], 4'b0000}) :
                       {(lsu_fpu_ex & lsu_valid_ex) ?
                        lsm_reg_ex[4:1] : lsm_reg_ex[3:0],
                        4'b0000}));

  assign lsu_lsm_base_status_ex = ({2{instr_lsm_baseinlist_ex}} &
                                   { (rf_rd_a_addr_ex == lsm_reg_ex[3:0]),
                                     (rf_rd_a_addr_ex <  lsm_reg_ex[3:0]) });

  assign ldm_base_updated_ex   = (lsu_isld_ex & (|lsu_lsm_base_status_ex));

  assign lsm_fpu_first_ex      = (lsu_fpu_ex & ~lsm_last_ex & opt_fpu_en &
                                  (nxt_lsm_fpu_ex[4:1] == lsu_fp_ls_rf_addr_ex[4:1]));

  assign lsm_fpu_brestore_ex  = (lsm_fpu_first_ex & lsu_wb_valid_ex);

  assign lsm_reg_pc_ex        = (lsm_reg_valid_ex & (&lsm_reg_ex[3:0]) &
                                 ~lsu_fpu_ex);

  assign lsm_reg_nonzero_ex   = (|lsm_reg_ex);

  assign lsu_ici_cont         = (// Direct from register
                                 ({4{lsm_reg_valid_ex &
                                     ((state==`CORE_LSU_IDLE)      |
                                      (state==`CORE_LSU_INT_L_R14) |
                                      (state==`CORE_LSU_INT_RETURN))}} &
                                  lsm_reg_ex[3:0]) |
                                 ({4{~psr_it_valid_ex &
                                     ((state==`CORE_LSU_LAZY_EXIT) |
                                      (state==`CORE_LSU_INT_L_R14) |
                                      (state==`CORE_LSU_INT_RETURN))}} &
                                  psr_ex_i[15:12]));


  assign lsm_reg_inv_ex     = 15 - lsm_reg_ex[3:0];
  assign lsm_inv_shft_ex    = 16'h7FFF >> lsm_reg_inv_ex;
  assign immed_mask_ex      = lsm_inv_shft_ex & instr_immed_ex_i[15:0];

  assign sel_mask_list_ex   = (((state == `CORE_LSU_LSM_NONSEQ) |
                                (state == `CORE_LSU_LSM_Rx) |
                                stack_op_int |
                                stack_op_fp) & ~lsm_last_ex);

  assign lsm_immed          = (lsu_ici_ret_ex ? {{16{1'b0}}, immed_mask_ex} :
                                                (sel_mask_list_ex ? lsm_masked_list :
                                                 {{16{1'b0}}, instr_lsm_immed_de_i[15:0]}));

  assign lsm_list_length_de = (lsm_immed[0]  +
                               lsm_immed[1]  +
                               lsm_immed[2]  +
                               lsm_immed[3]  +
                               lsm_immed[4]  +
                               lsm_immed[5]  +
                               lsm_immed[6]  +
                               lsm_immed[7]  +
                               lsm_immed[8]  +
                               lsm_immed[9]  +
                               lsm_immed[10] +
                               lsm_immed[11] +
                               lsm_immed[12] +
                               lsm_immed[13] +
                               lsm_immed[14] +
                               lsm_immed[15] +
                               (lsm_immed[16] & opt_fpu_en) +
                               (lsm_immed[17] & opt_fpu_en) +
                               (lsm_immed[18] & opt_fpu_en) +
                               (lsm_immed[19] & opt_fpu_en) +
                               (lsm_immed[20] & opt_fpu_en) +
                               (lsm_immed[21] & opt_fpu_en) +
                               (lsm_immed[22] & opt_fpu_en) +
                               (lsm_immed[23] & opt_fpu_en) +
                               (lsm_immed[24] & opt_fpu_en) +
                               (lsm_immed[25] & opt_fpu_en) +
                               (lsm_immed[26] & opt_fpu_en) +
                               (lsm_immed[27] & opt_fpu_en) +
                               (lsm_immed[28] & opt_fpu_en) +
                               (lsm_immed[29] & opt_fpu_en) +
                               (lsm_immed[30] & opt_fpu_en) +
                               (lsm_immed[31] & opt_fpu_en));

  assign nxt_lsm_list_length_ex = (lsu_fpu_de ?
                                   instr_lsm_immed_de_i[7:2] :
                                   {2'b00, lsm_list_length_de[3:0]});

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      lsm_list_length_ex <= {6{1'b0}};
    else if (lsu_lsm_cc_de)
      lsm_list_length_ex <= nxt_lsm_list_length_ex;

  assign nxt_lsm_ici_remain_ex = (lsu_fpu_ex ?
                                  ({lsm_reg_ex, 1'b0} -
                                   {1'b0, lsu_fp_ls_rf_addr_ex}) :
                                  {2'b00, lsm_list_length_de[3:0]});

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      lsm_ici_remaining_ex <= {6{1'b0}};
    else if (lsu_ici_ret_ex)
      lsm_ici_remaining_ex <= nxt_lsm_ici_remain_ex;


  assign lsu_excl_mon_we     = (// Set
                                nxt_lsu_excl_mon_ex |
                                (lsu_excl_mon_ex &
                                 (// Clear on store exclusive
                                  (lsu_excl_ex & ~lsu_isld_ex & lsu_ahb_rdy_ex &
                                   ((state==`CORE_LSU_LSS_N) |
                                    (state==`CORE_LSU_LSS_P))) |
                                  (lsu_clrex_ex) |
                                  (|lsu_request[1:0]))));

  assign nxt_lsu_excl_mon_ex = (// Executing LDREX
                                lsu_excl_ex & lsu_isld_ex &
                                ~lsu_excp_ex & lsu_ahb_rdy_ex &
                                ((state==`CORE_LSU_LSS_N) |
                                 (state==`CORE_LSU_LSS_P)));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      lsu_excl_mon_ex <= 1'b0;
    else if (lsu_excl_mon_we)
      lsu_excl_mon_ex <= nxt_lsu_excl_mon_ex;


  assign nxt_rf_wr_d_addr_wr = (lsu_isld_ex ? rf_wr_d_addr_ex :
                                              instr_rf_rd_c_addr_ex);

  assign lsu_ex_2_wr_en      = (lsu_ahb_rdy_ex & ((state==`CORE_LSU_LSS_N) |
                                                  (state==`CORE_LSU_LSS_P) |
                                                  (state==`CORE_LSU_LSS_B)));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        rf_rd_a_addr_wr <= 4'b0000;
        rf_wr_d_addr_wr <= 4'b0000;
        lsu_isld_wr     <= 1'b0;
        lsu_wb_valid_wr <= 1'b0;
        opt_lsu_fpu_wr  <= 1'b0;
      end
    else if (lsu_ex_2_wr_en)
      begin
        rf_rd_a_addr_wr <= rf_rd_a_addr_ex;
        rf_wr_d_addr_wr <= nxt_rf_wr_d_addr_wr;
        lsu_isld_wr     <= lsu_isld_ex;
        lsu_wb_valid_wr <= lsu_wb_valid_ex;
        opt_lsu_fpu_wr  <= lsu_fpu_ex;
      end

  assign lsu_fpu_wr = opt_lsu_fpu_wr & opt_fpu_en;

  assign lsm_rf_d_en_ex = (// In ITBlock base restoral
                           (lsu_wb_valid_ex & instr_itblock_ex_i &
                            (lsu_lfault_ex | (nvic_int_invoke_i & ~lsm_last_ex)) &
                            ~(lsu_isld_ex & instr_lsm_baseinlist_ex)) |
                           (~ldm_base_updated_ex &
                            lsu_lfault_ex & lsu_wb_valid_ex &
                            ~(lsu_isld_ex & instr_lsm_baseinlist_ex)) |
                           ( ldm_base_updated_ex & ~lsm_last_ex &
                             ( ( lsu_excp_ex & lsu_lsm_base_status_ex[0]) |
                               ( lsu_excp_ex & lsu_lsm_base_status_ex[1] &
                                               lsu_addracc_wait_reg_i) |
                               (~lsu_excp_ex & ~lsu_addracc_wait_reg_i))) |
                           ( lsu_isld_ex &  lsm_last_ex &
                             ( ( lsu_lfault_ex &  lsu_lsm_base_status_ex[0]) |
                               (~lsu_lfault_ex & (|lsu_lsm_base_status_ex) &
                                                 ~lsu_addracc_wait_reg_i))) |
                           (nvic_int_invoke_i & lsm_fpu_brestore_ex) |
                           ( lsu_isld_ex & ~(|lsu_lsm_base_status_ex) &
                             ~lsu_fpu_ex & ~lsu_lfault_ex &
                             ~lsu_addracc_wait_reg_i &
                             ~(instr_itblock_ex_i & nvic_int_invoke_i &
                               ~lsm_last_ex)));


  assign int_entry  = (// First ISR instruction in decode
                       int_first_instr_rdy &
                       (((state==`CORE_LSU_INT_ENTRY) & mtx_ahb_hreadyd_i &
                         ~lsu_lfault_ex & ~instr_lsu_int_ilock_de_i) |
                        (state==`CORE_LSU_INT_ENLCK)));

  assign int_exit   = ((state==`CORE_LSU_INT_EXIT) & mtx_ahb_hreadyd_i &
                       ~lsu_addracc_wait_reg_i);

  assign int_return = (// First instruction in decode and no new interrupt
                       int_first_instr_rdy & ~nvic_int_invoke_i &
                       (((state==`CORE_LSU_INT_L_R14) & mtx_ahb_hreadyd_i &
                         ~lsu_lfault_ex & ~instr_lsu_int_ilock_de_i) |
                        (state==`CORE_LSU_INT_RETURN)));

  assign lsu_stack_start = (~int_stack_op &
                            (// Start of unstacking with no handler exit
                             (instr_detect_exit_ex_i & ~fault_dblfault_ex_i) |
                             (nvic_int_invoke_i & rst_pc_valid_ex_i &
                              ~dbg_halted_i & mtx_ahb_hreadyd_i)));

  assign lsu_stack_end   = (int_stack_op &
                            (// Stacking complete
                             int_entry |
                             int_return));

  assign set_int_stacking = (~int_stacking & nvic_int_invoke_i &
                             rst_pc_valid_ex_i & ~dbg_halted_i &
                             mtx_ahb_hreadyd_i);

  assign int_stacking_we = (// Set
                            set_int_stacking |
                            (int_stacking & int_entry));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      int_stacking <= 1'b0;
    else if (int_stacking_we)
      int_stacking <= ~int_stacking;

  assign int_unstacking_we = (// Set
                              (~int_unstacking & instr_detect_exit_ex_i &
                               ~fault_dblfault_ex_i) |
                              (int_unstacking &
                               (int_return |
                                (nvic_int_invoke_i & rst_pc_valid_ex_i &
                                 ~dbg_halted_i & mtx_ahb_hreadyd_i))));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      int_unstacking <= 1'b0;
    else if (int_unstacking_we)
      int_unstacking <= ~int_unstacking;

  assign int_stack_op      = int_stacking | int_unstacking;

  assign lsu_addrd_non_buf = int_stack_op | fpu_lazy_stack_op_i;

  assign lsu_main_sp_en = (// Interrupt preemption
                           ((state == `CORE_LSU_INT_S_PC) & lsu_ahb_rdy_ex) |
                           ((state == `CORE_LSU_INT_EXIT) & mtx_ahb_hreadyd_i &
                            (nvic_int_tail_i | nvic_int_invoke_i))          |
                           ((state == `CORE_LSU_INT_UFAULT_1) &
                            nvic_int_invoke_i)                              |
                           ((state == `CORE_LSU_SLEEP) & nvic_int_invoke_i));


  assign lsu_real_sp_rd = (opt_fpu_en &
                           ((state == `CORE_LSU_INT_SFAULT_1) |
                            (state == `CORE_LSU_INT_SFAULT_2)));

  always @ (state or instr_detect_exit_ex_i or nvic_int_invoke_i or
            lsu_isld_ex or instr_immed_valid_ex_i or lsm_last_ex or
            lsu_excp_ex or nvic_dbg_reg_wr_i or lsu_isld_ex or
            nvic_dbg_reg_addr_i)
    case (state)
      `CORE_LSU_DBG_PC_1,     `CORE_LSU_INT_EXIT,     `CORE_LSU_INT_L_PC,
      `CORE_LSU_INT_S_RX,     `CORE_LSU_INT_S_PC,     `CORE_LSU_DFAULT_1,
      `CORE_LSU_LAZY_ENTRY,   `CORE_LSU_LTB_D :
        lsu_rf_rd_a_en_fast_ex = 1'b1;
      `CORE_LSU_DBG_R_1 :
        lsu_rf_rd_a_en_fast_ex = (~nvic_dbg_reg_wr_i &
                                  (nvic_dbg_reg_addr_i[6:5] == 2'b00));
      `CORE_LSU_INT_S_FLAGS,  `CORE_LSU_INT_ENTRY,    `CORE_LSU_INT_ENLCK,
      `CORE_LSU_INT_S_FPSCR,  `CORE_LSU_FP_S_SX :
        lsu_rf_rd_a_en_fast_ex = 1'b0;
      `CORE_LSU_LSS_N,        `CORE_LSU_LSS_P,        `CORE_LSU_LSS_B :
        lsu_rf_rd_a_en_fast_ex = ((~lsu_isld_ex & ~instr_immed_valid_ex_i) |
                                  nvic_int_invoke_i);
      `CORE_LSU_LSS_U,        `CORE_LSU_LSS_D,        `CORE_LSU_LSD_D :
        lsu_rf_rd_a_en_fast_ex = lsu_excp_ex;
      `CORE_LSU_LSD_N,        `CORE_LSU_LSD_S :
        lsu_rf_rd_a_en_fast_ex = ~lsu_isld_ex | lsu_excp_ex;
      `CORE_LSU_LSM_NONSEQ,   `CORE_LSU_LSM_Rx :
        lsu_rf_rd_a_en_fast_ex = ((~lsu_isld_ex & ~lsm_last_ex) |
                                  instr_detect_exit_ex_i | lsu_excp_ex);
      `CORE_LSU_IDLE,         `CORE_LSU_LSM_INTC1,    `CORE_LSU_LSM_INTC2,
      `CORE_LSU_LTB_N,        `CORE_LSU_LTB_W,        `CORE_LSU_ADD4,
      `CORE_LSU_INT_L_RX,     `CORE_LSU_INT_L_R14,    `CORE_LSU_INT_L_FLAGS,
      `CORE_LSU_INT_RETURN,   `CORE_LSU_INT_SFAULT_1, `CORE_LSU_INT_SFAULT_2,
      `CORE_LSU_INT_UFAULT_1, `CORE_LSU_INT_UFAULT_2, `CORE_LSU_DBG_PC_2,
      `CORE_LSU_DBG_R_0,      `CORE_LSU_DBG_R_2,      `CORE_LSU_SLEEP,
      `CORE_LSU_FAULT,        `CORE_LSU_DFAULT_2,     `CORE_LSU_FP_L_SX,
      `CORE_LSU_FP_L_FLAGS,   `CORE_LSU_LAZY_FAULT_1, `CORE_LSU_LAZY_FAULT_2,
      `CORE_LSU_LAZY_EXIT,    `CORE_LSU_LAZY_S_S0,    `CORE_LSU_LAZY_S_SX,
      `CORE_LSU_LAZY_S_FPSCR :
        lsu_rf_rd_a_en_fast_ex = (instr_detect_exit_ex_i | nvic_int_invoke_i);
      default :
        lsu_rf_rd_a_en_fast_ex = 1'bx;
    endcase

  always @ (state or instr_detect_exit_ex_i or nvic_int_invoke_i or
            mtx_ahb_hreadyd_i or lsu_excp_ex or instr_immed_valid_ex_i or
            mtx_ahb_haddraccd_i or lsu_isld_ex or lsu_ahb_rdy_ex or
            lsm_last_ex or nvic_dbg_reg_wr_i or lsu_isld_ex or
            nvic_dbg_reg_addr_i)
    case (state)
      `CORE_LSU_DBG_PC_1,     `CORE_LSU_DFAULT_1,     `CORE_LSU_LTB_D,
      `CORE_LSU_LAZY_ENTRY :
        lsu_rf_rd_a_en_ex = mtx_ahb_hreadyd_i;
      `CORE_LSU_INT_EXIT,     `CORE_LSU_INT_L_PC,     `CORE_LSU_INT_S_RX,
      `CORE_LSU_INT_S_PC :
        lsu_rf_rd_a_en_ex = lsu_ahb_rdy_ex;
      `CORE_LSU_DBG_R_1 :
        lsu_rf_rd_a_en_ex = (lsu_ahb_rdy_ex & ~nvic_dbg_reg_wr_i &
                             (nvic_dbg_reg_addr_i[6:5] == 2'b00));
      `CORE_LSU_INT_S_FLAGS,  `CORE_LSU_INT_ENTRY,    `CORE_LSU_INT_ENLCK,
      `CORE_LSU_INT_S_FPSCR,  `CORE_LSU_FP_S_SX :
        lsu_rf_rd_a_en_ex = 1'b0;
      `CORE_LSU_LSS_N,        `CORE_LSU_LSS_P,        `CORE_LSU_LSS_B :
        lsu_rf_rd_a_en_ex = (mtx_ahb_hreadyd_i &
                             ((~lsu_isld_ex & ~instr_immed_valid_ex_i) |
                              nvic_int_invoke_i));
      `CORE_LSU_LSS_U,        `CORE_LSU_LSS_D,        `CORE_LSU_LSD_D :
        lsu_rf_rd_a_en_ex = mtx_ahb_hreadyd_i & lsu_excp_ex;
      `CORE_LSU_LSD_N,        `CORE_LSU_LSD_S :
        lsu_rf_rd_a_en_ex = (mtx_ahb_hreadyd_i &
                             ((mtx_ahb_haddraccd_i & ~lsu_isld_ex) |
                              lsu_excp_ex));
      `CORE_LSU_LSM_NONSEQ,   `CORE_LSU_LSM_Rx :
        lsu_rf_rd_a_en_ex = (mtx_ahb_hreadyd_i &
                             ((mtx_ahb_haddraccd_i & ~lsu_isld_ex &
                               ~lsm_last_ex) |
                              instr_detect_exit_ex_i | lsu_excp_ex));
      `CORE_LSU_IDLE,         `CORE_LSU_LSM_INTC1,    `CORE_LSU_LSM_INTC2,
      `CORE_LSU_LTB_N,        `CORE_LSU_LTB_W,        `CORE_LSU_ADD4,
      `CORE_LSU_INT_L_RX,     `CORE_LSU_INT_L_R14,    `CORE_LSU_INT_L_FLAGS,
      `CORE_LSU_INT_RETURN,   `CORE_LSU_INT_SFAULT_1, `CORE_LSU_INT_SFAULT_2,
      `CORE_LSU_INT_UFAULT_1, `CORE_LSU_INT_UFAULT_2, `CORE_LSU_DBG_PC_2,
      `CORE_LSU_DBG_R_0,      `CORE_LSU_DBG_R_2,      `CORE_LSU_SLEEP,
      `CORE_LSU_FAULT,        `CORE_LSU_DFAULT_2,     `CORE_LSU_FP_L_FLAGS,
      `CORE_LSU_FP_L_SX,      `CORE_LSU_LAZY_FAULT_1, `CORE_LSU_LAZY_FAULT_2,
      `CORE_LSU_LAZY_EXIT,    `CORE_LSU_LAZY_S_S0,    `CORE_LSU_LAZY_S_SX,
      `CORE_LSU_LAZY_S_FPSCR :
        lsu_rf_rd_a_en_ex = (instr_detect_exit_ex_i | nvic_int_invoke_i);
      default :
        lsu_rf_rd_a_en_ex = 1'bx;
    endcase

  always @ (state or nvic_dbg_reg_addr_i or instr_detect_exit_ex_i or
            nvic_int_invoke_i or lsu_excp_ex or lsm_reg_ex or
            lsm_reg_nonzero_ex or lsm_nxt_reg_ex or rf_rd_b_addr_ex or
            rf_wr_d_addr_ex)
    case (state)
      `CORE_LSU_INT_S_PC,     `CORE_LSU_DBG_PC_1 :
        lsu_rf_rd_a_addr_ex = 4'b1111;
      `CORE_LSU_DBG_R_1 :
        lsu_rf_rd_a_addr_ex = (nvic_dbg_reg_addr_i[4] ?
                               4'b1101 : nvic_dbg_reg_addr_i[3:0]);
      `CORE_LSU_INT_ENTRY,    `CORE_LSU_INT_ENLCK :
        lsu_rf_rd_a_addr_ex = 4'b0000;
      `CORE_LSU_INT_S_RX :
        lsu_rf_rd_a_addr_ex = lsm_reg_ex[3:0];
      `CORE_LSU_LSS_N,        `CORE_LSU_LSS_P,        `CORE_LSU_LSS_B :
        lsu_rf_rd_a_addr_ex = nvic_int_invoke_i ? 4'b1101 : rf_wr_d_addr_ex;
      `CORE_LSU_LSD_N :
        lsu_rf_rd_a_addr_ex = nvic_int_invoke_i ? 4'b1101 : rf_rd_b_addr_ex;
      `CORE_LSU_LSD_S :
        lsu_rf_rd_a_addr_ex = lsu_excp_ex       ? 4'b1101 : rf_wr_d_addr_ex;
      `CORE_LSU_LTB_D :
        lsu_rf_rd_a_addr_ex = nvic_int_invoke_i ? 4'b1101 : 4'b1111;
      `CORE_LSU_LSM_NONSEQ :
        lsu_rf_rd_a_addr_ex = ((instr_detect_exit_ex_i | nvic_int_invoke_i) ?
                               4'b1101 :
                               (lsm_reg_nonzero_ex ? lsm_reg_ex[3:0] :
                                                     lsm_nxt_reg_ex[3:0]));
      `CORE_LSU_LSM_Rx :
        lsu_rf_rd_a_addr_ex = ((instr_detect_exit_ex_i | lsu_excp_ex) ?
                               4'b1101 : lsm_nxt_reg_ex[3:0]);
      `CORE_LSU_DFAULT_1,     `CORE_LSU_LAZY_ENTRY :
        lsu_rf_rd_a_addr_ex = nvic_int_invoke_i ? 4'b1101 : 4'b1111;
      `CORE_LSU_IDLE,         `CORE_LSU_LSM_INTC1,    `CORE_LSU_LSM_INTC2,
      `CORE_LSU_LTB_N,        `CORE_LSU_LTB_W,        `CORE_LSU_ADD4,
      `CORE_LSU_INT_S_FLAGS,  `CORE_LSU_INT_L_RX,     `CORE_LSU_INT_L_R14,
      `CORE_LSU_INT_L_FLAGS,  `CORE_LSU_INT_RETURN,   `CORE_LSU_INT_SFAULT_1,
      `CORE_LSU_INT_SFAULT_2, `CORE_LSU_INT_UFAULT_1, `CORE_LSU_INT_UFAULT_2,
      `CORE_LSU_DBG_PC_2,     `CORE_LSU_DBG_R_0,      `CORE_LSU_DBG_R_2,
      `CORE_LSU_SLEEP,        `CORE_LSU_FAULT,        `CORE_LSU_DFAULT_2,
      `CORE_LSU_INT_EXIT,     `CORE_LSU_INT_L_PC,     `CORE_LSU_LSS_U,
      `CORE_LSU_LSS_D,        `CORE_LSU_LSD_D,        `CORE_LSU_INT_S_FPSCR,
      `CORE_LSU_FP_S_SX,      `CORE_LSU_FP_L_FLAGS,   `CORE_LSU_FP_L_SX,
      `CORE_LSU_LAZY_S_S0,    `CORE_LSU_LAZY_S_SX,    `CORE_LSU_LAZY_S_FPSCR,
      `CORE_LSU_LAZY_FAULT_1, `CORE_LSU_LAZY_FAULT_2, `CORE_LSU_LAZY_EXIT :
        lsu_rf_rd_a_addr_ex = 4'b1101;
      default :
        lsu_rf_rd_a_addr_ex = 4'bxxxx;
    endcase

  always @ (state or lsu_addracc_wait_reg_i or lsu_fpu_ex or
            nvic_dbg_reg_addr_i or rf_wr_d_addr_wr or nvic_int_invoke_i or
            nvic_int_tail_i or lsu_lfault_ex or int_first_instr_rdy or
            fault_lsu_local_ex_i or lsu_in_nmihf or lsu_isld_ex or lsu_fpu_wr or
            lsu_wb_valid_ex or lsu_wb_after_ex or lsu_wb_valid_wr or
            lsm_reg_nonzero_ex or lsu_isld_wr or instr_lsm_baseinlist_ex or
            rf_rd_a_addr_ex or lsm_rf_d_en_ex or nvic_dbg_reg_wr_i or
            lsu_excp_ex or lsu_ld_excl_ex_i or lsu_st_excl_ex_i or
            nvic_slp_asleep_i or nvic_slp_sleeponexit_i or lsu_exit_valid_ex_i or
            fpu_unstack_ctl_i or fpu_int_cp_fault_i or nvic_fpccr_i)
    case(state)
      `CORE_LSU_DBG_PC_2 :
        begin
          lsu_rf_wr_en_ex     = 1'b1;
          wr_hreadyd_chk_ex   = 1'b0;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_DBG_R_2 :
        begin
          lsu_rf_wr_en_ex     = (nvic_dbg_reg_wr_i &
                                 (nvic_dbg_reg_addr_i[6:5] == 2'b00) &
                                 ((nvic_dbg_reg_addr_i[4] &
                                   (^nvic_dbg_reg_addr_i[1:0])) |
                                  ~nvic_dbg_reg_addr_i[4]));
          wr_hreadyd_chk_ex   = 1'b1;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_INT_S_PC :
        begin
          lsu_rf_wr_en_ex     = ~lsu_exit_valid_ex_i;
          wr_hreadyd_chk_ex   = 1'b1;
          wr_haddraccd_chk_ex = 1'b1;
        end
      `CORE_LSU_INT_ENTRY :
        begin
          lsu_rf_wr_en_ex     = (nvic_int_invoke_i & int_first_instr_rdy &
                                 ~lsu_lfault_ex);
          wr_hreadyd_chk_ex   = 1'b1;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_INT_L_RX,     `CORE_LSU_INT_L_PC :
        begin
          lsu_rf_wr_en_ex     = ~lsu_addracc_wait_reg_i & ~lsu_excp_ex;
          wr_hreadyd_chk_ex   = 1'b1;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_INT_L_R14 :
        begin
          lsu_rf_wr_en_ex     = (((fpu_int_cp_fault_i &
                                   ~fault_lsu_local_ex_i[0]) |
                                  (~fpu_unstack_ctl_i[1] & ~lsu_excp_ex)) &
                                 ~lsu_addracc_wait_reg_i & ~nvic_int_invoke_i);
          wr_hreadyd_chk_ex   = ~fpu_unstack_ctl_i[1] | fpu_int_cp_fault_i;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_INT_L_FLAGS :
        begin
          lsu_rf_wr_en_ex     = (~lsu_addracc_wait_reg_i & ~lsu_excp_ex);
          wr_hreadyd_chk_ex   = 1'b1;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_INT_SFAULT_2 :
        begin
          lsu_rf_wr_en_ex     = lsu_in_nmihf;
          wr_hreadyd_chk_ex   = 1'b0;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_LAZY_FAULT_2 :
        begin
          lsu_rf_wr_en_ex     = lsu_in_nmihf & ~nvic_fpccr_i[4];
          wr_hreadyd_chk_ex   = 1'b0;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_INT_UFAULT_1 :
        begin
          lsu_rf_wr_en_ex     = nvic_int_invoke_i;
          wr_hreadyd_chk_ex   = 1'b0;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_INT_UFAULT_2 :
        begin
          lsu_rf_wr_en_ex     = ~nvic_int_invoke_i;
          wr_hreadyd_chk_ex   = 1'b0;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_INT_EXIT :
        begin
          lsu_rf_wr_en_ex     = ~(nvic_int_invoke_i |
                                  nvic_int_tail_i   |
                                  nvic_slp_sleeponexit_i);
          wr_hreadyd_chk_ex   = 1'b1;
          wr_haddraccd_chk_ex = 1'b1;
        end
      `CORE_LSU_SLEEP :
        begin
          lsu_rf_wr_en_ex     = ~nvic_slp_asleep_i & ~nvic_int_invoke_i;
          wr_hreadyd_chk_ex   = 1'b1;
          wr_haddraccd_chk_ex = 1'b1;
        end
      `CORE_LSU_LSS_N,        `CORE_LSU_LSD_N :
        begin
          lsu_rf_wr_en_ex     = ~lsu_excp_ex & lsu_wb_valid_ex;
          wr_hreadyd_chk_ex   = 1'b1;
          wr_haddraccd_chk_ex = 1'b1;
        end
      `CORE_LSU_LSD_S :
        begin
          lsu_rf_wr_en_ex     = ((lsu_isld_ex & ~lsu_excp_ex & ~lsu_fpu_ex &
                                  ~lsu_addracc_wait_reg_i) |
                                 (lsu_excp_ex &
                                  (lsu_wb_valid_ex |
                                   instr_lsm_baseinlist_ex)));
          wr_hreadyd_chk_ex   = 1'b1;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_LSD_D :
        begin
          lsu_rf_wr_en_ex     = ((lsu_isld_ex & ~lsu_lfault_ex & ~lsu_fpu_ex) |
                                 (lsu_lfault_ex &
                                  (lsu_wb_valid_ex |
                                   instr_lsm_baseinlist_ex)));
          wr_hreadyd_chk_ex   = 1'b1;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_LSS_P :
        begin
          lsu_rf_wr_en_ex     = ((fault_lsu_local_ex_i[0] & lsu_wb_valid_wr) |
                                 (~fault_lsu_local_ex_i[0] &
                                  ~lsu_addracc_wait_reg_i &
                                  (lsu_st_excl_ex_i[0] |
                                   (lsu_isld_wr & ~lsu_fpu_wr &
                                    ~((&rf_wr_d_addr_wr) & lsu_ld_excl_ex_i)))));
          wr_hreadyd_chk_ex   = 1'b1;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_LSS_U,        `CORE_LSU_LSS_D,         `CORE_LSU_LSS_B :
        begin
          lsu_rf_wr_en_ex     = ((fault_lsu_local_ex_i[0] & lsu_wb_valid_wr) |
                                 (lsu_st_excl_ex_i[0] & ~lsu_lfault_ex) |
                                 ((lsu_isld_wr & ~lsu_fpu_wr &
                                   ~fault_lsu_local_ex_i[0] &
                                   ~lsu_addracc_wait_reg_i &
                                   ~((&rf_wr_d_addr_wr) & lsu_ld_excl_ex_i) &
                                   ~(nvic_int_invoke_i &
                                     (rf_wr_d_addr_wr==4'b1101)))));
          wr_hreadyd_chk_ex   = 1'b1;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_LTB_W,        `CORE_LSU_ADD4 :
        begin
          lsu_rf_wr_en_ex     = 1'b1;
          wr_hreadyd_chk_ex   = 1'b0;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_LSM_NONSEQ :
        begin
          lsu_rf_wr_en_ex     = (~lsu_excp_ex &
                                 (// WriteBack if basereg!=PC and first entry
                                  (lsu_wb_valid_ex & ~lsm_reg_nonzero_ex &
                                   ~(lsu_isld_ex & instr_lsm_baseinlist_ex)) |
                                  (lsu_wb_valid_ex & lsu_wb_after_ex &
                                   (rf_rd_a_addr_ex==4'b1101))));
          wr_hreadyd_chk_ex   = 1'b1;
          wr_haddraccd_chk_ex = 1'b1;
        end
      `CORE_LSU_LSM_Rx :
        begin
          lsu_rf_wr_en_ex     = lsm_rf_d_en_ex;
          wr_hreadyd_chk_ex   = 1'b1;
          wr_haddraccd_chk_ex = 1'b0;
        end
      `CORE_LSU_DFAULT_2 :
        begin
          lsu_rf_wr_en_ex     = 1'b1;
          wr_hreadyd_chk_ex   = 1'b1;
          wr_haddraccd_chk_ex = 1'b1;
        end
      `CORE_LSU_IDLE,         `CORE_LSU_LSM_INTC1,    `CORE_LSU_LSM_INTC2,
      `CORE_LSU_LTB_N,        `CORE_LSU_FAULT,        `CORE_LSU_LTB_D,
      `CORE_LSU_INT_S_RX,     `CORE_LSU_INT_S_FLAGS,  `CORE_LSU_INT_RETURN,
      `CORE_LSU_INT_ENLCK,    `CORE_LSU_INT_SFAULT_1, `CORE_LSU_DBG_PC_1,
      `CORE_LSU_DBG_R_0,      `CORE_LSU_DBG_R_1,      `CORE_LSU_DFAULT_1,
      `CORE_LSU_INT_S_FPSCR,  `CORE_LSU_FP_S_SX,      `CORE_LSU_FP_L_FLAGS,
      `CORE_LSU_FP_L_SX,      `CORE_LSU_LAZY_ENTRY,   `CORE_LSU_LAZY_EXIT,
      `CORE_LSU_LAZY_S_S0,    `CORE_LSU_LAZY_S_SX,    `CORE_LSU_LAZY_S_FPSCR,
      `CORE_LSU_LAZY_FAULT_1 :
        begin
          lsu_rf_wr_en_ex     = 1'b0;
          wr_hreadyd_chk_ex   = 1'b0;
          wr_haddraccd_chk_ex = 1'b0;
        end
      default :
        begin
          lsu_rf_wr_en_ex     = 1'bx;
          wr_hreadyd_chk_ex   = 1'bx;
          wr_haddraccd_chk_ex = 1'bx;
        end
    endcase

  assign lsu_rf_wr_d_en_ex      = (lsu_rf_wr_en_ex &
                                   ((state != `CORE_LSU_INT_L_FLAGS) |
                                    lsu_stack_algn_64_ex_i) &
                                   (mtx_ahb_hreadyd_i   | ~wr_hreadyd_chk_ex) &
                                   (mtx_ahb_haddraccd_i | ~wr_haddraccd_chk_ex));

  assign lsu_rf_wr_d_en_fast_ex = (lsu_rf_wr_en_ex &
                                   (mtx_ahb_hreadyd_i   | ~wr_hreadyd_chk_ex) &
                                   (mtx_ahb_haddraccd_i | ~wr_haddraccd_chk_ex));

  assign lsu_rf_wr_z_en_ex      = ((state == `CORE_LSU_LSS_B) & ~lsu_excp_ex &
                                   mtx_ahb_hreadyd_i & mtx_ahb_haddraccd_i);

  always @ (state or nvic_dbg_reg_addr_i or rf_rd_a_addr_ex or
            rf_rd_b_addr_ex or rf_wr_d_addr_ex or nvic_int_invoke_i or
            fault_lsu_local_ex_i or lsu_lfault_ex or lsu_isld_ex or
            lsm_reg_ex or lsu_excp_ex or instr_itblock_ex_i or
            lsu_lsm_base_status_ex or lsm_last_ex or lsu_wb_valid_ex or
            lsu_wb_valid_wr or rf_rd_a_addr_wr or rf_wr_d_addr_wr or
            instr_lsm_baseinlist_ex or lsm_fpu_brestore_ex)
    case(state)
      `CORE_LSU_INT_S_PC,     `CORE_LSU_INT_EXIT,     `CORE_LSU_SLEEP,
      `CORE_LSU_INT_L_FLAGS,  `CORE_LSU_INT_UFAULT_1 :
        lsu_rf_wr_d_addr_ex = 4'b1101;
      `CORE_LSU_INT_ENTRY,    `CORE_LSU_INT_L_R14 :
        lsu_rf_wr_d_addr_ex = 4'b1110;
      `CORE_LSU_DBG_PC_2,     `CORE_LSU_INT_L_PC,     `CORE_LSU_INT_SFAULT_2,
      `CORE_LSU_INT_UFAULT_2, `CORE_LSU_LTB_W,        `CORE_LSU_ADD4,
      `CORE_LSU_DFAULT_2,     `CORE_LSU_LAZY_S_S0,    `CORE_LSU_LAZY_FAULT_2 :
        lsu_rf_wr_d_addr_ex = 4'b1111;
      `CORE_LSU_DBG_R_2 :
        lsu_rf_wr_d_addr_ex = (nvic_dbg_reg_addr_i[4] ?
                               4'b1101 : nvic_dbg_reg_addr_i[3:0]);
      `CORE_LSU_INT_L_RX :
        lsu_rf_wr_d_addr_ex = lsm_reg_ex[3:0];
      `CORE_LSU_LSS_N,        `CORE_LSU_LSD_N,        `CORE_LSU_LSM_NONSEQ:
        lsu_rf_wr_d_addr_ex = rf_rd_a_addr_ex;
      `CORE_LSU_LSM_Rx :
        lsu_rf_wr_d_addr_ex = (((nvic_int_invoke_i & ~lsm_last_ex) |
                                lsu_lfault_ex) &
                               ((lsu_isld_ex & lsu_lsm_base_status_ex[0]) |
                                (lsu_lfault_ex & lsu_wb_valid_ex) |
                                (instr_itblock_ex_i & lsu_wb_valid_ex) |
                                lsm_fpu_brestore_ex) ?
                               rf_rd_a_addr_ex :
                               lsm_reg_ex[3:0]);
      `CORE_LSU_LSD_S :
        lsu_rf_wr_d_addr_ex = ((lsu_excp_ex &
                                (lsu_wb_valid_ex | instr_lsm_baseinlist_ex))) ?
                              rf_rd_a_addr_ex :
                              rf_rd_b_addr_ex;
      `CORE_LSU_LSD_D :
        lsu_rf_wr_d_addr_ex = ((lsu_lfault_ex &
                               (lsu_wb_valid_ex | instr_lsm_baseinlist_ex)) ?
                               rf_rd_a_addr_ex :
                               rf_wr_d_addr_ex);
      `CORE_LSU_LSS_P,        `CORE_LSU_LSS_U,        `CORE_LSU_LSS_D,
      `CORE_LSU_LSS_B :
        lsu_rf_wr_d_addr_ex = ((fault_lsu_local_ex_i[0] & lsu_wb_valid_wr) ?
                               rf_rd_a_addr_wr :
                               rf_wr_d_addr_wr);
      `CORE_LSU_IDLE,         `CORE_LSU_LSM_INTC1,
      `CORE_LSU_LSM_INTC2,    `CORE_LSU_LTB_N,        `CORE_LSU_INT_S_RX,
      `CORE_LSU_INT_S_FLAGS,  `CORE_LSU_INT_RETURN,   `CORE_LSU_INT_ENLCK,
      `CORE_LSU_INT_SFAULT_1, `CORE_LSU_DBG_PC_1,     `CORE_LSU_DBG_R_0,
      `CORE_LSU_DBG_R_1,      `CORE_LSU_FAULT,        `CORE_LSU_DFAULT_1,
      `CORE_LSU_INT_S_FPSCR,  `CORE_LSU_FP_S_SX,      `CORE_LSU_FP_L_FLAGS,
      `CORE_LSU_FP_L_SX,      `CORE_LSU_LAZY_ENTRY,   `CORE_LSU_LAZY_EXIT,
      `CORE_LSU_LAZY_S_SX,    `CORE_LSU_LAZY_S_FPSCR, `CORE_LSU_LAZY_FAULT_1 :
        lsu_rf_wr_d_addr_ex = 4'b0000;
      default :
        lsu_rf_wr_d_addr_ex = 4'bxxxx;
    endcase

  assign lsu_rf_wr_z_addr_ex = rf_rd_a_addr_ex;


  assign lsu_wb_will_occur_ex    = (lsu_wb_valid_ex &
                                    ((state==`CORE_LSU_LSS_N) |
                                     (state==`CORE_LSU_LSS_P)));

  assign lsu_ld_will_occur_ex    = (lsu_isld_wr & ~lsu_fpu_wr &
                                    ((state==`CORE_LSU_LSS_P) |
                                     (state==`CORE_LSU_LSS_B) |
                                     (state==`CORE_LSU_LSS_U) |
                                     (state==`CORE_LSU_LSS_D)));

  assign lsu_last_will_occur_ex  = (lsu_isld_ex  & ~lsu_fpu_ex &
                                    ((state==`CORE_LSU_LSD_D) |
                                     lsm_last_ex));

  assign lsu_entry_will_occur_ex = (state==`CORE_LSU_INT_ENTRY);


  assign lsu_excl_will_occur_ex  = (lsu_st_excl_ex_i[0] &
                                    ((state==`CORE_LSU_LSS_P) |
                                     (state==`CORE_LSU_LSS_U) |
                                     (state==`CORE_LSU_LSS_D)));

  assign lsu_will_wr_rf_ex       = ((~lsu_addracc_wait_reg_i &
                                     (lsu_ld_will_occur_ex |
                                      lsu_last_will_occur_ex)) |
                                    lsu_wb_will_occur_ex |
                                    lsu_entry_will_occur_ex |
                                    lsu_excl_will_occur_ex |
                                    (nvic_int_invoke_i & lsm_fpu_brestore_ex));


  assign lsu_dbl_fault_ex = ((state == `CORE_LSU_DFAULT_1) |
                             (state == `CORE_LSU_DFAULT_2));

  assign lsu_pflush_ex    = (state==`CORE_LSU_DFAULT_2);

  assign lsu_fix_pc_ex    = (// Interrupt
                             (state==`CORE_LSU_INT_S_PC)   |
                             (state==`CORE_LSU_LAZY_ENTRY) |
                             (state==`CORE_LSU_DFAULT_1)   |
                             (state==`CORE_LSU_DBG_PC_1));


  assign lsu_mcyc_invk_msk_ex = ((state==`CORE_LSU_LSD_N)      |
                                 (state==`CORE_LSU_LSD_S)      |
                                 (state==`CORE_LSU_LSM_NONSEQ) |
                                 ((state==`CORE_LSU_LSM_Rx) & ~lsm_last_ex));

  assign lsu_mcyc_ilock_ex    = ((state==`CORE_LSU_LSD_N)      |
                                 (state==`CORE_LSU_LSD_S)      |
                                 (state==`CORE_LSU_LSM_NONSEQ) |
                                 (state==`CORE_LSU_LSM_Rx));

  assign instr_excl_ex_o           = instr_excl_ex;
  assign instr_lsu_ctl_ex_o        = instr_lsu_ctl_ex;
  assign instr_lsm_baseinlist_ex_o = instr_lsm_baseinlist_ex;
  assign lsu_state_ex_o            = state;
  assign lsu_valid_ex_o            = lsu_valid_ex;
  assign lsu_nxt_lsm_fpu_ex_o      = nxt_lsm_fpu_ex;
  assign lsu_lsm_reg_ex_o          = lsm_reg_ex;
  assign lsu_lsm_list_valid_ex_o   = lsm_list_valid_ex;
  assign lsu_lsm_last_ex_o         = lsm_last_ex;
  assign lsm_list_length_de_o      = lsm_list_length_de;
  assign lsm_list_length_ex_o      = lsm_list_length_ex;
  assign lsm_ici_remaining_ex_o    = lsm_ici_remaining_ex;
  assign lsu_stall_ex_o            = stall;
  assign lsu_stall_late_ex_o       = lsu_stall_late_ex;
  assign lsu_excl_mon_ex_o         = lsu_excl_mon_ex;
  assign lsu_pline_valid_de_o      = lsu_pline_valid_de;
  assign lsu_st_imm_nonbuf_ex_o    = lsu_st_imm_nonbuf_ex;
  assign lsu_lsm_base_status_ex_o  = lsu_lsm_base_status_ex;
  assign lsm_fpu_brestore_ex_o     = lsm_fpu_brestore_ex;
  assign lsu_dbl_fault_ex_o        = lsu_dbl_fault_ex;
  assign lsu_stackpush_rdy_ex_o    = lsu_stackpush_rdy_ex;
  assign lsu_pflush_ex_o           = lsu_pflush_ex;
  assign lsu_it_valid_ex_o         = lsu_it_valid_ex;
  assign lsu_it_ex_o               = lsu_it_ex;
  assign lsu_fix_pc_ex_o           = lsu_fix_pc_ex;
  assign lsu_mcyc_invk_msk_ex_o    = lsu_mcyc_invk_msk_ex;
  assign lsu_mcyc_ilock_ex_o       = lsu_mcyc_ilock_ex;
  assign lsu_fp_ls_rf_addr_ex_o    = lsu_fp_ls_rf_addr_ex;
  assign lsu_rf_rd_a_en_fast_ex_o  = lsu_rf_rd_a_en_fast_ex;
  assign lsu_rf_rd_a_en_ex_o       = lsu_rf_rd_a_en_ex;
  assign lsu_rf_rd_a_addr_ex_o     = lsu_rf_rd_a_addr_ex;
  assign lsu_rf_wr_d_en_ex_o       = lsu_rf_wr_d_en_ex;
  assign lsu_rf_wr_d_en_fast_ex_o  = lsu_rf_wr_d_en_fast_ex;
  assign lsu_rf_wr_d_addr_ex_o     = lsu_rf_wr_d_addr_ex;
  assign lsu_rf_wr_z_addr_ex_o     = lsu_rf_wr_z_addr_ex;
  assign lsu_rf_wr_z_en_ex_o       = lsu_rf_wr_z_en_ex;
  assign lsu_will_wr_rf_ex_o       = lsu_will_wr_rf_ex;
  assign lsu_ici_cont_o            = lsu_ici_cont;
  assign lsu_stack_start_o         = lsu_stack_start;
  assign lsu_stack_end_o           = lsu_stack_end;
  assign lsu_addrd_non_buf_o       = lsu_addrd_non_buf;
  assign lsu_main_sp_en_o          = lsu_main_sp_en;
  assign lsu_real_sp_rd_o          = lsu_real_sp_rd;
  assign rf_rd_a_addr_ex_o         = rf_rd_a_addr_ex;
  assign rf_rd_b_addr_ex_o         = rf_rd_b_addr_ex;
  assign instr_rf_rd_c_addr_ex_o   = instr_rf_rd_c_addr_ex;
  assign rf_wr_d_addr_ex_o         = rf_wr_d_addr_ex;
  assign int_entry_o               = int_entry;
  assign int_exit_o                = int_exit;
  assign int_return_o              = int_return;
  assign int_stack_op_o            = int_stack_op;
  assign int_unstacking_o          = int_unstacking;
  assign dbg_ready_o               = dbg_ready;


`ifdef ARM_ASSERT_ON

  reg [8*20:0]     ovl_state_dec;

  always @ (state)
    case (state)
      `CORE_LSU_IDLE:          ovl_state_dec = {"IDLE"};
      `CORE_LSU_LSS_N:         ovl_state_dec = {"LSS_N"};
      `CORE_LSU_LSS_P:         ovl_state_dec = {"LSS_P"};
      `CORE_LSU_LSS_B:         ovl_state_dec = {"LSS_B"};
      `CORE_LSU_LSS_U:         ovl_state_dec = {"LSS_U"};
      `CORE_LSU_LSS_D:         ovl_state_dec = {"LSS_D"};
      `CORE_LSU_LSD_N:         ovl_state_dec = {"LSD_N"};
      `CORE_LSU_LSD_S:         ovl_state_dec = {"LSD_S"};
      `CORE_LSU_LSD_D:         ovl_state_dec = {"LSD_D"};
      `CORE_LSU_LSM_INTC1:     ovl_state_dec = {"LSM_INTC1"};
      `CORE_LSU_LSM_INTC2:     ovl_state_dec = {"LSM_INTC2"};
      `CORE_LSU_LSM_NONSEQ:    ovl_state_dec = {"LSM_NONSEQ"};
      `CORE_LSU_LSM_Rx:        ovl_state_dec = {"LSM_Rx"};
      `CORE_LSU_LTB_N:         ovl_state_dec = {"LTB_N"};
      `CORE_LSU_LTB_D:         ovl_state_dec = {"LTB_D"};
      `CORE_LSU_LTB_W:         ovl_state_dec = {"LTB_W"};
      `CORE_LSU_ADD4:          ovl_state_dec = {"ADD4"};
      `CORE_LSU_INT_S_PC:      ovl_state_dec = {"INT_S_PC"};
      `CORE_LSU_INT_S_RX:      ovl_state_dec = {"INT_S_RX"};
      `CORE_LSU_INT_S_FLAGS:   ovl_state_dec = {"INT_S_FLAGS"};
      `CORE_LSU_INT_L_PC:      ovl_state_dec = {"INT_L_PC"};
      `CORE_LSU_INT_L_RX:      ovl_state_dec = {"INT_L_RX"};
      `CORE_LSU_INT_L_R14:     ovl_state_dec = {"INT_L_R14"};
      `CORE_LSU_INT_L_FLAGS:   ovl_state_dec = {"INT_L_FLAGS"};
      `CORE_LSU_INT_RETURN:    ovl_state_dec = {"INT_RETURN"};
      `CORE_LSU_INT_ENTRY:     ovl_state_dec = {"INT_ENTRY"};
      `CORE_LSU_INT_ENLCK:     ovl_state_dec = {"INT_ENLCK"};
      `CORE_LSU_INT_EXIT:      ovl_state_dec = {"INT_EXIT"};
      `CORE_LSU_INT_SFAULT_1:  ovl_state_dec = {"INT_SFAULT_1"};
      `CORE_LSU_INT_SFAULT_2:  ovl_state_dec = {"INT_SFAULT_2"};
      `CORE_LSU_INT_UFAULT_1:  ovl_state_dec = {"INT_UFAULT_1"};
      `CORE_LSU_INT_UFAULT_2:  ovl_state_dec = {"INT_UFAULT_2"};
      `CORE_LSU_DBG_PC_1:      ovl_state_dec = {"DBG_PC_1"};
      `CORE_LSU_DBG_PC_2:      ovl_state_dec = {"DBG_PC_2"};
      `CORE_LSU_DBG_R_0:       ovl_state_dec = {"DBG_R_0"};
      `CORE_LSU_DBG_R_1:       ovl_state_dec = {"DBG_R_1"};
      `CORE_LSU_DBG_R_2:       ovl_state_dec = {"DBG_R_2"};
      `CORE_LSU_SLEEP:         ovl_state_dec = {"SLEEP"};
      `CORE_LSU_FAULT:         ovl_state_dec = {"FAULT"};
      `CORE_LSU_LAZY_FAULT_1:  ovl_state_dec = {"FP_LAZY_S_FAULT_1"};
      `CORE_LSU_LAZY_FAULT_2:  ovl_state_dec = {"FP_LAZY_S_FAULT_2"};
      `CORE_LSU_DFAULT_1:      ovl_state_dec = {"DFAULT_1"};
      `CORE_LSU_DFAULT_2:      ovl_state_dec = {"DFAULT_2"};
      `CORE_LSU_INT_S_FPSCR:   ovl_state_dec = {"INT_S_FPSCR"};
      `CORE_LSU_LAZY_S_FPSCR:  ovl_state_dec = {"FP_LAZY_S_FPSCR"};
      `CORE_LSU_FP_S_SX:       ovl_state_dec = {"FP_S_SX"};
      `CORE_LSU_LAZY_S_S0:     ovl_state_dec = {"FP_LAZY_S_S0"};
      `CORE_LSU_LAZY_S_SX:     ovl_state_dec = {"FP_LAZY_S_SX"};
      `CORE_LSU_FP_L_FLAGS:    ovl_state_dec = {"FP_L_FLAGS"};
      `CORE_LSU_FP_L_SX:       ovl_state_dec = {"FP_L_SX"};
      `CORE_LSU_LAZY_ENTRY:    ovl_state_dec = {"FPU_LAZY_ENTRY"};
      `CORE_LSU_LAZY_EXIT:     ovl_state_dec = {"FPU_LAZY_EXIT"};
      default:                 ovl_state_dec = {"*UNKNOWN*"};
    endcase

  reg            ovl_instr_atbase_reg;     // instr_atbase_ex registered

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      ovl_instr_atbase_reg <= 1'b0;
    else if (mtx_ahb_hreadyd_i)
      ovl_instr_atbase_reg <= instr_atbase_ex_i;

  assert_zero_one_hot #(0,12,0,"idle_nxt_state controls must be exclusive")
    ovl_idle_sel_excl(hclk, hreset_n, idle_sel);

  assert_never #(0,0,"Abandoned pipelined LSS must go to Idle or LSU state")
    ovl_lssb_check(hclk, hreset_n, ((state==`CORE_LSU_LSS_B) & ~opt_ext_en));

  assert_never #(0,0,"Abandoned pipelined LSS must go to Idle or LSU state")
    ovl_lssp_check(hclk, hreset_n, ((state==`CORE_LSU_LSS_P) & state_we &
                                    ((nxt_state!=`CORE_LSU_IDLE)       &
                                     (nxt_state!=`CORE_LSU_LSS_N)      &
                                     (nxt_state!=`CORE_LSU_LSS_P)      &
                                     (nxt_state!=`CORE_LSU_LSS_B)      &
                                     (nxt_state!=`CORE_LSU_LSS_U)      &
                                     (nxt_state!=`CORE_LSU_LSS_D)      &
                                     (nxt_state!=`CORE_LSU_LSD_N)      &
                                     (nxt_state!=`CORE_LSU_LTB_N)      &
                                     (nxt_state!=`CORE_LSU_ADD4)       &
                                     (nxt_state!=`CORE_LSU_LSM_NONSEQ) &
                                     (nxt_state!=`CORE_LSU_INT_S_PC)   &
                                     (nxt_state!=`CORE_LSU_LAZY_ENTRY) &
                                     (nxt_state!=`CORE_LSU_DBG_PC_1)   &
                                     (nxt_state!=`CORE_LSU_FAULT)      &
                                     (nxt_state!=`CORE_LSU_DFAULT_1))));

  assert_zero_one_hot #(0,5,0,"lsu_request should be exclusive")
    lsu_request_excl(hclk, hreset_n, lsu_request);

  assert_never #(0,0,"lsu_ld_excl_ex must only be set in LSS data phase")
    ovl_lsu_ld_excl_ex(hclk, hreset_n, (lsu_ld_excl_ex_i &
                                    ~((state==`CORE_LSU_LSS_P) |
                                      (state==`CORE_LSU_LSS_U) |
                                      (state==`CORE_LSU_LSS_D))));

  assert_never #(0,0,"lsu_st_excl_ex must only be set in LSS data phase")
    ovl_lsu_st_excl_ex(hclk, hreset_n, (lsu_st_excl_ex_i[0] &
                                    ~((state==`CORE_LSU_LSS_P) |
                                      (state==`CORE_LSU_LSS_U) |
                                      (state==`CORE_LSU_LSS_D))));

  assert_never #(0,0,"Local Monitor must not be set during interrupts")
    ovl_lsu_excl_mon_ex(hclk, hreset_n, (lsu_excl_mon_ex &
                                 ((state==`CORE_LSU_INT_ENTRY) |
                                  (state==`CORE_LSU_INT_ENLCK) |
                                  (state==`CORE_LSU_INT_EXIT)  |
                                  (state==`CORE_LSU_INT_S_PC))));

  assert_never #(0,0,"LSM opcode must have non-zero immediate")
    ovl_lsu_immed(hclk, hreset_n, ((state==`CORE_LSU_LSM_NONSEQ) &
                                   ~(|instr_immed_ex_i[15:0])    &
                                   ~lsu_fpu_ex));

  assert_zero_one_hot #(0,3,0, "int_invoke int_tail nvic_slp_sleeponexit must be excl")
    ovl_int_ctrl_excl(hclk, hreset_n,
                      {nvic_int_invoke_i,nvic_int_tail_i,nvic_slp_sleeponexit_i});

  assert_never #(0,0, "Fault state transition check")
    ovl_lsu_fault(hclk, hreset_n, ((state==`CORE_LSU_FAULT) & state_we &
                                   ~((nxt_state==`CORE_LSU_INT_S_PC) |
                                     (nxt_state==`CORE_LSU_DFAULT_1) |
                                     (nxt_state==`CORE_LSU_IDLE))));

  assert_never #(0,0, "Lazy fault state transition check")
    ovl_lsu_lazy_fault(hclk, hreset_n, ((state==`CORE_LSU_LAZY_FAULT_2) & state_we &
                                        ~((nxt_state==`CORE_LSU_INT_S_PC) |
                                           (nxt_state==`CORE_LSU_IDLE)    |
                                          (nxt_state==`CORE_LSU_DFAULT_1))));

  assert_never_unknown #(0,6,0,"X propagate protect priority LSM encoder")
    ovl_lsm_reg_x(hclk, hreset_n, 1'b1, {lsm_reg_ex, lsm_reg_valid_ex});

  assert_never_unknown #(0,6,0,"X propagate protect default statements")
    ovl_state_x(hclk, hreset_n, 1'b1, state);

  assert_never #(0,0,"lsu_stack_start should not pulse when int_stack_op asserted")
    ovl_int_stack_op(hclk, hreset_n, lsu_stack_start & int_stack_op);

  assert_never #(0,0,"int_stack_op is incorrectly formed")
    ovl_lsu_stack_op(hclk, hreset_n, (int_stack_op ^
                                      ((state==`CORE_LSU_INT_EXIT)     |
                                       (state==`CORE_LSU_INT_SFAULT_1) |
                                       (state==`CORE_LSU_INT_SFAULT_2) |
                                       (state==`CORE_LSU_INT_UFAULT_1) |
                                       (state==`CORE_LSU_INT_UFAULT_2) |
                                       (state==`CORE_LSU_INT_S_PC)     |
                                       (state==`CORE_LSU_INT_S_RX)     |
                                       (state==`CORE_LSU_INT_S_FLAGS)  |
                                       (state==`CORE_LSU_FP_S_SX)      |
                                       (state==`CORE_LSU_INT_S_FPSCR)  |
                                       (state==`CORE_LSU_INT_L_PC)     |
                                       (state==`CORE_LSU_INT_L_RX)     |
                                       (state==`CORE_LSU_FP_L_FLAGS)   |
                                       (state==`CORE_LSU_FP_L_SX)      |
                                       (state==`CORE_LSU_INT_L_R14)    |
                                       (state==`CORE_LSU_INT_L_FLAGS)  |
                                       (state==`CORE_LSU_INT_ENTRY)    |
                                       (state==`CORE_LSU_INT_ENLCK)    |
                                       (state==`CORE_LSU_INT_RETURN)   |
                                       (state==`CORE_LSU_SLEEP))));

  assert_never #(0,0,"fpu_lazy_stack_op is incorrectly formed")
    ovl_lazy_stack(hclk, hreset_n, (fpu_lazy_stack_op_i ^
                                         ((state==`CORE_LSU_LAZY_S_S0)    |
                                          (state==`CORE_LSU_LAZY_S_SX)    |
                                          (state==`CORE_LSU_LAZY_S_FPSCR) |
                                          (state==`CORE_LSU_LAZY_FAULT_1) |
                                          (state==`CORE_LSU_LAZY_FAULT_2) |
                                          (state==`CORE_LSU_LAZY_ENTRY)   |
                                          (state==`CORE_LSU_LAZY_EXIT))));


  assert_never #(0,0,"lsu_stack_start must not assert when core halted")
    ovl_lsu_stack_start_2(hclk, hreset_n, lsu_stack_start & dbg_halted_i);

  assert_next #(0,1,0,0,0,"lsu_stack_start must be pulsed")
    ovl_stack_start_pulsed(hclk, hreset_n, lsu_stack_start, ~lsu_stack_start);

  assert_next #(0,1,0,0,0,"int_entry must be pulsed")
    ovl_intentry_pulsed(hclk, hreset_n, int_entry, ~int_entry);

  assert_next #(0,1,0,0,0,"int_exit must be pulsed")
    ovl_intexit_pulsed(hclk, hreset_n, int_exit, ~int_exit);

  assert_next #(0,1,0,0,0,"int_return must be pulsed")
    ovl_intreturn_pulsed(hclk, hreset_n, int_return, ~int_return);

  assert_never #(0,0,"Port controls should be exclusive")
    ovl_wport_excl(hclk, hreset_n, lsu_wb_will_occur_ex & lsu_ld_will_occur_ex);

  assert_never #(0,0,"PC write is unexpected")
    ovl_pc_wr(hclk, hreset_n, (lsu_rf_wr_d_en_ex & (&lsu_rf_wr_d_addr_ex) &
                               ((state==`CORE_LSU_LSS_N)                   |
                                (state==`CORE_LSU_LSS_P)                   |
                                (state==`CORE_LSU_LSS_B)                   |
                                ((state==`CORE_LSU_LSS_U) & lsu_lfault_ex) |
                                ((state==`CORE_LSU_LSS_D) & lsu_lfault_ex) |
                                (state==`CORE_LSU_LSD_N)                   |
                                (state==`CORE_LSU_LSD_S)                   |
                                (state==`CORE_LSU_LSD_D)                   |
                                (state==`CORE_LSU_LSM_NONSEQ)              |
                                ((state==`CORE_LSU_LSM_Rx) &
                                 (lsu_lfault_ex | ~lsm_last_ex)))));

  assert_never #(0,0,"int_exit asserted at thread level")
    ovl_thread_exit_check(hclk, hreset_n,
                          int_exit & ovl_instr_atbase_reg & instr_atbase_ex_i);

`endif

`include "cm3_dpu_undefs.vp"

endmodule
