module cm3_dpu_regfile
  (// Inputs
   hreset_n, hclk, rd1_addr_i, rd2_addr_i, wr1_en_i, wr1_addr_i, wr1_data_i,
   rd1_data_o, rd2_data_o, r14_data_o
   );

  parameter RESET_ALL_REGS = 0; // Reset all registers if 1

  input          hreset_n;
  input          hclk;
  input [3:0]    rd1_addr_i;                // Read port 1 address
  input [3:0]    rd2_addr_i;                // Read port 2 address
  input          wr1_en_i;                  // Write port 1 enable
  input [3:0]    wr1_addr_i;                // Write port 1 address
  input [31:0]   wr1_data_i;                // Write port 1 data

  output [31:0]  rd1_data_o;                // Read port 1 data
  output [31:0]  rd2_data_o;                // Read port 2 data
  output [31:0]  r14_data_o;                // R14


  wire           full_hreset_n;             // Apply hreset_n to all registers
  wire           reg0_we;                   // Reg  0 write enable
  wire           reg1_we;                   // Reg  1 write enable
  wire           reg2_we;                   // Reg  2 write enable
  wire           reg3_we;                   // Reg  3 write enable
  wire           reg4_we;                   // Reg  4 write enable
  wire           reg5_we;                   // Reg  5 write enable
  wire           reg6_we;                   // Reg  6 write enable
  wire           reg7_we;                   // Reg  7 write enable
  wire           reg8_we;                   // Reg  8 write enable
  wire           reg9_we;                   // Reg  9 write enable
  wire           reg10_we;                  // Reg 10 write enable
  wire           reg11_we;                  // Reg 11 write enable
  wire           reg12_we;                  // Reg 12 write enable
  wire           reg13_we;                  // Reg 13 write enable
  wire           reg14_we;                  // Reg 14 write enable
  wire           reg15_we;                  // Reg 15 write enable
  reg  [31:0]    reg0;                      // Register 0
  reg  [31:0]    reg1;                      // Register 1
  reg  [31:0]    reg2;                      // Register 2
  reg  [31:0]    reg3;                      // Register 3
  reg  [31:0]    reg4;                      // Register 4
  reg  [31:0]    reg5;                      // Register 5
  reg  [31:0]    reg6;                      // Register 6
  reg  [31:0]    reg7;                      // Register 7
  reg  [31:0]    reg8;                      // Register 8
  reg  [31:0]    reg9;                      // Register 9
  reg  [31:0]    reg10;                     // Register 10
  reg  [31:0]    reg11;                     // Register 11
  reg  [31:0]    reg12;                     // Register 12
  reg  [31:2]    reg13;                     // Register 13
  reg  [31:0]    reg14;                     // Register 14
  reg  [31:2]    reg15;                     // Register 15
  reg  [31:0]    rd1_data;                  // Read port 1 data
  reg  [31:0]    rd2_data;                  // Read port 2 data
  wire [31:0]    r14_data;                  // R14

`ifdef ARM_DSM

  wire          dsm_reset_all_regs;

  assign full_hreset_n = dsm_reset_all_regs    ? hreset_n : 1'b1;

`else

  assign full_hreset_n = (RESET_ALL_REGS == 1) ? hreset_n : 1'b1;

`endif


  assign reg0_we  = wr1_en_i & (wr1_addr_i == 4'b0000);
  assign reg1_we  = wr1_en_i & (wr1_addr_i == 4'b0001);
  assign reg2_we  = wr1_en_i & (wr1_addr_i == 4'b0010);
  assign reg3_we  = wr1_en_i & (wr1_addr_i == 4'b0011);
  assign reg4_we  = wr1_en_i & (wr1_addr_i == 4'b0100);
  assign reg5_we  = wr1_en_i & (wr1_addr_i == 4'b0101);
  assign reg6_we  = wr1_en_i & (wr1_addr_i == 4'b0110);
  assign reg7_we  = wr1_en_i & (wr1_addr_i == 4'b0111);
  assign reg8_we  = wr1_en_i & (wr1_addr_i == 4'b1000);
  assign reg9_we  = wr1_en_i & (wr1_addr_i == 4'b1001);
  assign reg10_we = wr1_en_i & (wr1_addr_i == 4'b1010);
  assign reg11_we = wr1_en_i & (wr1_addr_i == 4'b1011);
  assign reg12_we = wr1_en_i & (wr1_addr_i == 4'b1100);
  assign reg13_we = wr1_en_i & (wr1_addr_i == 4'b1101);
  assign reg14_we = wr1_en_i & (wr1_addr_i == 4'b1110);
  assign reg15_we = wr1_en_i & (wr1_addr_i == 4'b1111);


  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg0 <= {32{1'b0}};
    else if (reg0_we)
      reg0 <= wr1_data_i;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg1 <= {32{1'b0}};
    else if (reg1_we)
      reg1 <= wr1_data_i;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg2 <= {32{1'b0}};
    else if (reg2_we)
      reg2 <= wr1_data_i;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg3 <= {32{1'b0}};
    else if (reg3_we)
      reg3 <= wr1_data_i;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg4 <= {32{1'b0}};
    else if (reg4_we)
      reg4 <= wr1_data_i;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg5 <= {32{1'b0}};
    else if (reg5_we)
      reg5 <= wr1_data_i;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg6 <= {32{1'b0}};
    else if (reg6_we)
      reg6 <= wr1_data_i;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg7 <= {32{1'b0}};
    else if (reg7_we)
      reg7 <= wr1_data_i;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg8 <= {32{1'b0}};
    else if (reg8_we)
      reg8 <= wr1_data_i;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg9 <= {32{1'b0}};
    else if (reg9_we)
      reg9 <= wr1_data_i;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg10 <= {32{1'b0}};
    else if (reg10_we)
      reg10 <= wr1_data_i;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg11 <= {32{1'b0}};
    else if (reg11_we)
      reg11 <= wr1_data_i;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg12 <= {32{1'b0}};
    else if (reg12_we)
      reg12 <= wr1_data_i;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg13 <= {30{1'b0}};
    else if (reg13_we)
      reg13 <= wr1_data_i[31:2];

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg14 <= {32{1'b0}};
    else if (reg14_we)
      reg14 <= wr1_data_i;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      reg15 <= {30{1'b0}};
    else if (reg15_we)
      reg15 <= wr1_data_i[31:2];

  always @ (rd1_addr_i or reg0 or reg1 or reg2 or reg3 or reg4 or reg5 or
            reg6 or reg7 or reg8 or reg9 or reg10 or reg11 or reg12 or
            reg13 or reg14 or reg15)
    case (rd1_addr_i)
      4'b0000 : rd1_data = reg0;
      4'b0001 : rd1_data = reg1;
      4'b0010 : rd1_data = reg2;
      4'b0011 : rd1_data = reg3;
      4'b0100 : rd1_data = reg4;
      4'b0101 : rd1_data = reg5;
      4'b0110 : rd1_data = reg6;
      4'b0111 : rd1_data = reg7;
      4'b1000 : rd1_data = reg8;
      4'b1001 : rd1_data = reg9;
      4'b1010 : rd1_data = reg10;
      4'b1011 : rd1_data = reg11;
      4'b1100 : rd1_data = reg12;
      4'b1101 : rd1_data = {reg13,2'b00};
      4'b1110 : rd1_data = reg14;
      4'b1111 : rd1_data = {reg15,2'b00};
      default : rd1_data = 32'bx;
    endcase

  always @ (rd2_addr_i or reg0 or reg1 or reg2 or reg3 or reg4 or reg5 or
            reg6 or reg7 or reg8 or reg9 or reg10 or reg11 or reg12 or
            reg13 or reg14 or reg15)
    case (rd2_addr_i)
      4'b0000 : rd2_data = reg0;
      4'b0001 : rd2_data = reg1;
      4'b0010 : rd2_data = reg2;
      4'b0011 : rd2_data = reg3;
      4'b0100 : rd2_data = reg4;
      4'b0101 : rd2_data = reg5;
      4'b0110 : rd2_data = reg6;
      4'b0111 : rd2_data = reg7;
      4'b1000 : rd2_data = reg8;
      4'b1001 : rd2_data = reg9;
      4'b1010 : rd2_data = reg10;
      4'b1011 : rd2_data = reg11;
      4'b1100 : rd2_data = reg12;
      4'b1101 : rd2_data = {reg13,2'b00};
      4'b1110 : rd2_data = reg14;
      4'b1111 : rd2_data = {reg15,2'b00};
      default : rd2_data = 32'bx;
    endcase

  assign r14_data = reg14;

  assign rd1_data_o = rd1_data;
  assign rd2_data_o = rd2_data;
  assign r14_data_o = r14_data;

endmodule
