`include "cm3_dpu_defs.vp"

module cm3_dpu_status
  (// Inputs
   hreset_n, hclk, rst_pc_valid_ex_i, instr_valid_de_i, instr_br_cond_de_i,
   instr_itctl_de_i, instr_cond_code_de_i, instr_itfold_de_i,
   instr_ls_allow_pline_de_i, instr_ls_single_adv_de_i, instr_valid_ex_i,
   instr_czb_ex_i, instr_itfold_ex_i, instr_stall_ex_i, set_instr_redo_ex_i,
   instr_redo_ex_i, instr_br_t_changer_ex_i, instr_br_pflush_ex_i,
   lsu_flag_wr_ex_i, lsu_psr_ex_i, lsu_it_valid_ex_i, lsu_it_ex_i,
   alu_zero_res_ex_i, alu_flag_wr_ex_i, alu_flag_res_ex_i, alu_ge_res_ex_i,
   alu_mul_flag_res_ex_i, e_mac_flag_wr_ex_i, e_mac_nz_ex_i, e_mac_q_set_ex_i,
   e_simd_q_set_ex_i, e_simd_ge_wr_ex_i, e_simd_ge_res_ex_i, lsu_dbl_fault_ex_i,
   fault_lsu_precise_ex_i, fpu_wr_data_ex_i, fpu_psr_wr_ex_i,
   rf_wr_d_data_erly_ex_i, int_stack_op_i, nvic_int_invoke_i, int_entry_i,
   lsu_stack_start_i, fpu_lazy_stack_op_i, dbg_halted_i,
   instr_itblock_de_o, instr_itccfail_de_o, instr_ccpass_de_o,
   instr_itblock_ex_o, instr_ccpass_ex_o, psr_t_ex_o, psr_ex_o
   );

  parameter      EXT_PRESENT = 0;          // Extensions present if 1
  parameter      FPU_PRESENT = 0;          // FPU present if 1

  input          hreset_n;
  input          hclk;
  input          rst_pc_valid_ex_i;        // PC reset vector fetch completed
  input          instr_valid_de_i;         // Instruction in decode is valid
  input [3:0]    instr_br_cond_de_i;       // Branch condition code
  input [9:0]    instr_itctl_de_i;         // IT control
  input [3:0]    instr_cond_code_de_i;     // Instruction condition code
  input          instr_itfold_de_i;        // IT fold
  input          instr_ls_allow_pline_de_i;// Allow LSS to overlap
  input          instr_ls_single_adv_de_i; // Allow fail/nop underneath LS
  input          instr_valid_ex_i;         // Instruction in execute is valid
  input          instr_czb_ex_i;           // Instruction is CZB
  input          instr_itfold_ex_i;        // IT fold
  input [3:0]    instr_stall_ex_i;         // Instruction stall information
  input          set_instr_redo_ex_i;      // Set instr will be redone
  input          instr_redo_ex_i;          // Instr will be redone
  input          instr_br_t_changer_ex_i;  // Branch changes T bit
  input [2:0]    instr_br_pflush_ex_i;     // Branch pipeline flush
  input          lsu_flag_wr_ex_i;         // LSU flag result write enable
  input [31:0]   lsu_psr_ex_i;             // LSU PSR write value
  input          lsu_it_valid_ex_i;        // LSU write to IT is valid
  input [7:0]    lsu_it_ex_i;              // LSU IT bits to write
  input          alu_zero_res_ex_i;        // ALU result is zero
  input [5:0]    alu_flag_wr_ex_i;         // ALU NZCVQ GE result write enables
  input [4:0]    alu_flag_res_ex_i;        // ALU flag result
  input [3:0]    alu_ge_res_ex_i;          // ALU GE bits result
  input [4:0]    alu_mul_flag_res_ex_i;    // ALU MUL flag result (critical)
  input [4:0]    e_mac_flag_wr_ex_i;       // MAC NZCVQ flag setting
  input [1:0]    e_mac_nz_ex_i;            // MAC CPSR NZ result
  input          e_mac_q_set_ex_i;         // MAC CPSR Q set
  input          e_simd_q_set_ex_i;        // SIMD unit Q flag set
  input          e_simd_ge_wr_ex_i;        // SIMD GE bits write
  input [3:0]    e_simd_ge_res_ex_i;       // SIMD GE bits write
  input          lsu_dbl_fault_ex_i;       // LSU double fault
  input          fault_lsu_precise_ex_i;   // Precise LSU fault
  input [31:0]   fpu_wr_data_ex_i;         // Wr data from FPU (NZCV in 31:28)
  input          fpu_psr_wr_ex_i;          // FPU flag write
  input          lsu_stack_start_i;        // LSU stack start
  input [31:0]   rf_wr_d_data_erly_ex_i;   // RF D port write data (early)
  input          int_stack_op_i;           // LSU is stacking/unstacking
  input          nvic_int_invoke_i;        // Interrupt stack request
  input          int_entry_i;              // Interrupt ISR entered
  input          fpu_lazy_stack_op_i;      // FPU lazy stacking
  input          dbg_halted_i;             // Core halted

  output         instr_itblock_de_o;       // In IT block (De)
  output         instr_itccfail_de_o;      // Condition code fail IT
  output         instr_ccpass_de_o;        // Condition code pass (De)
  output         instr_itblock_ex_o;       // In IT block (Ex)
  output         instr_ccpass_ex_o;        // Condition code pass (Ex)
  output         psr_t_ex_o;               // T bit
  output [31:0]  psr_ex_o;                 // PSR

  wire           opt_ext_en;               // Logic removal for extensions
  wire           opt_fpu_en;               // Logic removal for FPU
  wire           instr_it_de;              // IT in decode
  wire           instr_it_valid_de;        // IT decoded
  reg            instr_itflgscc_ex;        // Flag setting cc pass in IT block
  reg  [3:0]     instr_cond_code_ex;       // Instruction condition code
  reg            instr_itfccpass_ex;       // IT folded onto CC passing instr
  reg            instr_itfnonit_ex;        // IT folded onto non IT block
  wire           instr_it_valid_we;        // IT valid write enable
  wire           nxt_instr_it_valid_ex;    // Next IT valid
  reg            instr_it_valid_ex;        // IT in execute
  wire           t_bit_br_ex;              // Branch T bit write enable
  wire           psr_t_we;                 // T bit write enable
  wire           nxt_psr_t_ex;             // Next T bit
  reg            psr_t_ex;                 // T bit
  wire           instr_it_advance_de;      // Advance IT reg
  wire [4:0]     it_reg_sel_ex;            // IT next value selector
  wire           lsu_it_reg_valid_ex;      // LSU IT value is valid
  wire [7:0]     lsu_it_reg_ex;            // LSU IT value
  wire [7:0]     advance_it_reg_ex;        // IT register advanced
  wire [7:0]     rewind_it_reg_ex;         // IT register rewound
  wire [7:0]     nxt_it_reg_ex;            // Next IT register value
  wire           it_reg_we;                // IT register write enable
  wire           it_skid_we;               // IT skidn reg write enable
  reg  [7:0]     it_reg_ex;                // IT register
  wire [5:0]     nxt_it_skid_ex;           // Next IT register skid info
  reg  [5:0]     it_skid_ex;               // IT register skid info
  wire           instr_itfnonit_de;        // IT folded onto non IT block
  wire           instr_itblock_de;         // In IT block (De)
  wire           instr_itblock_ex;         // In IT block (Ex)
  wire           instr_itccfail_de;        // Condition code fail IT
  wire [4:0]     flags_sel_ex;             // Flags next value selector
  wire           flags_we;                 // Flags write enable
  wire [4:0]     alu_flags_ex;             // ALU flags result
  wire [4:0]     e_mac_flags_ex;           // MAC flags result
  wire [4:0]     simd_flags_ex;            // SIMD flags result
  wire [4:0]     nxt_flags_ex;             // Next flags value
  reg  [4:0]     flags_ex;                 // Flags {N,Z,C,V,Q}
  wire [4:0]     psr_flags_ex;             // PSR flags value (inc Q skid)
  wire           psr_ge_we;                // GE bits write enable
  wire [3:0]     nxt_psr_ge_ex;            // Next GE bits
  reg  [3:0]     ipsr_ge_ex;               // GE bits
  wire [3:0]     psr_ge_ex;                // GE bits
  wire           sel_adv_it_ex;            // Select advanced IT reg value
  wire           sel_rew_it_ex;            // Select rewound IT reg value
  wire           sel_cur_it_ex;            // Select current IT reg value
  wire [7:0]     psr_it_reg_ex;            // IT reg to be used in PSR
  wire [31:0]    psr_ex;                   // PSR
  wire           sel_n_alu_ex;             // Select ALU N flag
  wire           sel_n_mac_ex;             // Select MAC N flag
  wire           sel_n_mul_ex;             // Select mul N flag
  wire           sel_n_fpu_ex;             // Select FPU N flag
  wire           sel_n_cur_ex;             // Select current N flag
  wire           alu_nflag_ex;             // ALU N flag result
  wire           sel_z_alu_ex;             // Select ALU Z flag
  wire           sel_z_mac_ex;             // Select MAC Z flag
  wire           sel_z_mul_ex;             // Select mul Z flag
  wire           sel_z_fpu_ex;             // Select FPU Z flag
  wire           sel_z_cur_ex;             // Select current Z flag
  wire           alu_zflag_ex;             // ALU Z flag result
  wire           sel_c_fpu_ex;             // Select FPU C flag
  wire           sel_c_cur_ex;             // Select current C flag
  wire           alu_cflag_ex;             // ALU C flag result
  wire           sel_v_fpu_ex;             // Select FPU V flag
  wire           sel_v_cur_ex;             // Select current V flag
  wire           alu_vflag_ex;             // ALU V flag result
  wire           redo_it_ex;               // Will redo IT
  wire           use_it_reg_ex;            // Use unadvanced IT in cc
  wire           use_it_adv_ex;            // Use   advanced IT in cc
  wire           use_it_br_de;             // Use branch CC
  wire [3:0]     instr_cc_sel_de;          // Condition code mux select (De)
  wire           instr_ccpass_de;          // Condition code pass (De)
  wire [3:0]     instr_cc_sel_ex;          // Condition code mux select (Ex)
  wire           instr_cc_c_ex;            // Condition code (Ex)
  wire           instr_cc_res_ex;          // CC mux selection/IT fold result
  wire           instr_ccpass_ex;          // Condition code pass (Ex)

`ifdef ARM_DSM

  wire          dsm_fpu_present;

  assign opt_fpu_en = dsm_fpu_present    ? 1'b1 : 1'b0;

`else

  assign opt_fpu_en = (FPU_PRESENT == 1) ? 1'b1 : 1'b0;

`endif

  assign opt_ext_en = (EXT_PRESENT == 1) ? 1'b1 : 1'b0;


  function ccpass;
    input       n;        // N flag
    input       z;        // Z flag
    input       c;        // C flag
    input       v;        // V flag
    input [3:0] cond;     // Condition code value

    reg         m_eq;     // Condition code mnemonic EQ
    reg         m_cs;     // Condition code mnemonic CS
    reg         m_mi;     // Condition code mnemonic MI
    reg         m_vs;     // Condition code mnemonic VS
    reg         m_hi;     // Condition code mnemonic HI
    reg         m_ge;     // Condition code mnemonic GE
    reg         m_gt;     // Condition code mnemonic GT
    reg         m_al;     // Condition code mnemonic AL
    reg         m_ne;     // Condition code mnemonic NE
    reg         m_cc;     // Condition code mnemonic CC
    reg         m_pl;     // Condition code mnemonic PL
    reg         m_vc;     // Condition code mnemonic VC
    reg         m_ls;     // Condition code mnemonic LS
    reg         m_lt;     // Condition code mnemonic LT
    reg         m_le;     // Condition code mnemonic LE
    reg         m_nv;     // Condition code mnemonic NV
    reg [3:0]   flags;    // Combined NZCV
    reg [15:0]  mux;      // Mux select for flags

    begin
      flags = {n,z,c,v};
      m_eq  = (cond == 4'h0);
      m_ne  = (cond == 4'h1);
      m_cs  = (cond == 4'h2);
      m_cc  = (cond == 4'h3);
      m_mi  = (cond == 4'h4);
      m_pl  = (cond == 4'h5);
      m_vs  = (cond == 4'h6);
      m_vc  = (cond == 4'h7);
      m_hi  = (cond == 4'h8);
      m_ls  = (cond == 4'h9);
      m_ge  = (cond == 4'hA);
      m_lt  = (cond == 4'hB);
      m_gt  = (cond == 4'hC);
      m_le  = (cond == 4'hD);
      m_al  = (cond == 4'hE);
      m_nv  = (cond == 4'hF);

      mux = { m_eq | m_cs | m_mi | m_vs | m_ls | m_ge | m_le | m_al | m_nv,
              m_eq | m_cs | m_mi | m_vc | m_ls | m_lt | m_le | m_al | m_nv,
              m_eq | m_cc | m_mi | m_vs | m_ls | m_ge | m_le | m_al | m_nv,
              m_eq | m_cc | m_mi | m_vc | m_ls | m_lt | m_le | m_al | m_nv,
              m_ne | m_cs | m_mi | m_vs | m_hi | m_ge | m_gt | m_al | m_nv,
              m_ne | m_cs | m_mi | m_vc | m_hi | m_lt | m_le | m_al | m_nv,
              m_ne | m_cc | m_mi | m_vs | m_ls | m_ge | m_gt | m_al | m_nv,
              m_ne | m_cc | m_mi | m_vc | m_ls | m_lt | m_le | m_al | m_nv,
              m_eq | m_cs | m_pl | m_vs | m_ls | m_lt | m_le | m_al | m_nv,
              m_eq | m_cs | m_pl | m_vc | m_ls | m_ge | m_le | m_al | m_nv,
              m_eq | m_cc | m_pl | m_vs | m_ls | m_lt | m_le | m_al | m_nv,
              m_eq | m_cc | m_pl | m_vc | m_ls | m_ge | m_le | m_al | m_nv,
              m_ne | m_cs | m_pl | m_vs | m_hi | m_lt | m_le | m_al | m_nv,
              m_ne | m_cs | m_pl | m_vc | m_hi | m_ge | m_gt | m_al | m_nv,
              m_ne | m_cc | m_pl | m_vs | m_ls | m_lt | m_le | m_al | m_nv,
              m_ne | m_cc | m_pl | m_vc | m_ls | m_ge | m_gt | m_al | m_nv };

      ccpass = mux[flags];
    end
  endfunction


  assign instr_it_de       = instr_itctl_de_i[0];

  assign instr_it_valid_de = instr_valid_de_i & instr_it_de;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        instr_itflgscc_ex  <= 1'b0;
        instr_cond_code_ex <= 4'b0000;
        instr_itfccpass_ex <= 1'b0;
        instr_itfnonit_ex  <= 1'b0;
      end
    else if (instr_valid_de_i)
      begin
        instr_itflgscc_ex  <= instr_itctl_de_i[1] & instr_ccpass_de;
        instr_cond_code_ex <= instr_cond_code_de_i;
        instr_itfccpass_ex <= instr_ccpass_de;
        instr_itfnonit_ex  <= instr_itfnonit_de;
      end

  assign instr_it_valid_we     = (instr_valid_de_i |
                                  lsu_flag_wr_ex_i |
                                  lsu_it_valid_ex_i);

  assign nxt_instr_it_valid_ex = ((|nxt_it_reg_ex[3:0]) &
                                  (instr_it_valid_de |
                                   lsu_flag_wr_ex_i  |
                                   lsu_it_valid_ex_i));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      instr_it_valid_ex <= 1'b0;
    else if (instr_it_valid_we)
      instr_it_valid_ex <= nxt_instr_it_valid_ex;


  assign t_bit_br_ex  = ((|instr_br_pflush_ex_i[1:0] & instr_br_t_changer_ex_i) |
                         (instr_br_pflush_ex_i[2] &
                          (int_stack_op_i | ~rst_pc_valid_ex_i)));

  assign psr_t_we     = t_bit_br_ex | lsu_flag_wr_ex_i;

  assign nxt_psr_t_ex = ((t_bit_br_ex & rf_wr_d_data_erly_ex_i[0]) |
                         (lsu_flag_wr_ex_i & lsu_psr_ex_i[24]));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      psr_t_ex <= 1'b1;
    else if (psr_t_we)
      psr_t_ex <= nxt_psr_t_ex;


  assign instr_it_advance_de = (// Instruction based advance
                                (instr_valid_de_i & ~instr_it_de &
                                 ~instr_it_valid_ex & ~nvic_int_invoke_i &
                                 ~instr_redo_ex_i & ~set_instr_redo_ex_i) |
                                (// Advance for stacking
                                 lsu_stack_start_i & ~instr_it_valid_ex &
                                 ~instr_redo_ex_i & ~set_instr_redo_ex_i &
                                 (fpu_lazy_stack_op_i |
                                  lsu_dbl_fault_ex_i  |
                                  ~(|instr_stall_ex_i[1:0]))));

  assign it_reg_sel_ex       = ({// IT instruction
                                 instr_it_valid_de,
                                 lsu_it_valid_ex_i & ~instr_it_valid_de,
                                 (lsu_flag_wr_ex_i & dbg_halted_i),
                                 (instr_it_advance_de & (|it_reg_ex[2:0]) &
                                  (lsu_dbl_fault_ex_i | fpu_lazy_stack_op_i |
                                   ~(|instr_br_pflush_ex_i[1:0]))),
                                 ((instr_it_advance_de &
                                   (it_reg_ex[3:0]==4'b1000)) |
                                  ((|instr_br_pflush_ex_i[1:0]) &
                                   ~lsu_dbl_fault_ex_i & (|it_reg_ex[3:0]) &
                                   instr_valid_ex_i & ~dbg_halted_i))});

  assign lsu_it_reg_valid_ex = ((|lsu_psr_ex_i[11:10]) |
                                (|lsu_psr_ex_i[26:25]));

  assign lsu_it_reg_ex       = ({8{lsu_it_reg_valid_ex}} &
                                { lsu_psr_ex_i[15:10],
                                  lsu_psr_ex_i[26:25] });

  assign advance_it_reg_ex   = { it_reg_ex[7:5], it_reg_ex[3:0], 1'b0 };

  assign rewind_it_reg_ex    = { it_skid_ex[5:1], it_reg_ex[3:1] };

  assign nxt_it_reg_ex       = (// From IT instruction in decode
                                ({8{it_reg_sel_ex[4]}} & instr_itctl_de_i[9:2]) |
                                ({8{it_reg_sel_ex[3]}} & lsu_it_ex_i)           |
                                ({8{it_reg_sel_ex[2]}} & lsu_it_reg_ex)         |
                                ({8{it_reg_sel_ex[1]}} & advance_it_reg_ex));

  assign it_reg_we           = (|it_reg_sel_ex);

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      it_reg_ex  <= {8{1'b0}};
    else if (it_reg_we)
      it_reg_ex  <= nxt_it_reg_ex;

  assign it_skid_we          = (it_reg_we |
                                (instr_valid_de_i & it_skid_ex[0]));

  assign nxt_it_skid_ex      = { ({5{~int_entry_i}} & it_reg_ex[7:3]),
                                 (instr_ls_single_adv_de_i |
                                  instr_ls_allow_pline_de_i) };

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      it_skid_ex <= {6{1'b0}};
    else if (it_skid_we)
      it_skid_ex <= nxt_it_skid_ex;

  assign instr_itfnonit_de  = (instr_itfold_de_i &
                               (nvic_int_invoke_i |
                                (~(|it_reg_ex[2:0]) & ~instr_it_valid_ex &
                                 ~redo_it_ex)));

  assign instr_itblock_de   = ((instr_itblock_ex &
                                ((it_reg_ex[3:0]!=4'b1000) | instr_redo_ex_i)) |
                               instr_it_valid_ex);

  assign instr_itblock_ex   = (// In an IT section (not the IT itself)
                               (|it_reg_ex[3:0]) &
                               ~(instr_it_valid_ex & instr_valid_ex_i));

  assign instr_itccfail_de  = (// in IT block
                               instr_itblock_ex & (it_reg_ex[3:0]!=4'b1000) &
                               ~instr_it_valid_ex &
                               (it_reg_ex[7:5]!=3'b111) &
                               ((instr_cc_res_ex &
                                 (it_reg_ex[4]!=it_reg_ex[3])) |
                                (~instr_cc_res_ex &
                                 (it_reg_ex[4]==it_reg_ex[3]))));


  assign flags_sel_ex   = { fpu_psr_wr_ex_i, // FPU
                            e_simd_q_set_ex_i,
                            ((|e_mac_flag_wr_ex_i[`CORE_N:`CORE_V]) &
                             (instr_itflgscc_ex |
                              (instr_itfnonit_ex & instr_itfold_ex_i) |
                              ~(|it_reg_ex[3:0]))),
                            ((|alu_flag_wr_ex_i[4:0]) &
                             (instr_itflgscc_ex |
                              (instr_itfnonit_ex & instr_itfold_ex_i) |
                              ~(|it_reg_ex[3:0]))),
                            lsu_flag_wr_ex_i };

  assign flags_we       = e_mac_q_set_ex_i | (|flags_sel_ex);

  assign alu_flags_ex   = { (alu_flag_wr_ex_i[`CORE_N] & flags_sel_ex[1]) ?
                            (alu_flag_res_ex_i[`CORE_N] |
                             alu_mul_flag_res_ex_i[`CORE_N]) :
                            psr_flags_ex[`CORE_N],
                            (alu_flag_wr_ex_i[`CORE_Z] & flags_sel_ex[1]) ?
                            (alu_flag_res_ex_i[`CORE_Z] |
                             alu_mul_flag_res_ex_i[`CORE_Z]) :
                            psr_flags_ex[`CORE_Z],
                            (alu_flag_wr_ex_i[`CORE_C] & flags_sel_ex[1]) ?
                            alu_flag_res_ex_i[`CORE_C] : psr_flags_ex[`CORE_C],
                            (alu_flag_wr_ex_i[`CORE_V] & flags_sel_ex[1]) ?
                            alu_flag_res_ex_i[`CORE_V] : psr_flags_ex[`CORE_V],
                            (alu_flag_wr_ex_i[`CORE_Q] & flags_sel_ex[1]) ?
                            alu_flag_res_ex_i[`CORE_Q] : psr_flags_ex[`CORE_Q] };

  assign e_mac_flags_ex = { e_mac_flag_wr_ex_i[`CORE_N] ?
                            e_mac_nz_ex_i[1] : psr_flags_ex[`CORE_N],
                            e_mac_flag_wr_ex_i[`CORE_Z] ?
                            e_mac_nz_ex_i[0] : psr_flags_ex[`CORE_Z],
                            psr_flags_ex[`CORE_C],
                            psr_flags_ex[`CORE_V],
                            e_mac_q_set_ex_i | psr_flags_ex[`CORE_Q] };

  assign simd_flags_ex  = { psr_flags_ex[`CORE_N:`CORE_V],
                            e_simd_q_set_ex_i | psr_flags_ex[`CORE_Q] };

  assign nxt_flags_ex   = (// LSU
                           ({5{flags_sel_ex[0]}} & lsu_psr_ex_i[31:27])    |
                           ({5{flags_sel_ex[1]}} & alu_flags_ex)           |
                           ({5{flags_sel_ex[2]}} & e_mac_flags_ex)         |
                           ({5{flags_sel_ex[3]}} & simd_flags_ex)          |
                           ({5{~(|flags_sel_ex)}} &
                            {flags_ex[`CORE_N:`CORE_V], e_mac_q_set_ex_i}) |
                           ({5{flags_sel_ex[4]}} &
                            {fpu_wr_data_ex_i[31:28],
                             e_mac_q_set_ex_i | flags_ex[0]}));

  always @ (posedge hclk or negedge hreset_n)
    if(!hreset_n)
      flags_ex <= 5'b00000;
    else if (flags_we)
      flags_ex <= nxt_flags_ex;

  assign psr_flags_ex  = { flags_ex[`CORE_N:`CORE_V],
                           e_mac_q_set_ex_i | flags_ex[`CORE_Q] };


  assign psr_ge_we     = ((e_simd_ge_wr_ex_i|
                           lsu_flag_wr_ex_i |
                           alu_flag_wr_ex_i[`CORE_G]) & opt_ext_en);

  assign nxt_psr_ge_ex = ({4{opt_ext_en}} &
                          (// MSR
                           ({4{alu_flag_wr_ex_i[`CORE_G]}} & alu_ge_res_ex_i) |
                           ({4{e_simd_ge_wr_ex_i}} & e_simd_ge_res_ex_i)      |
                           ({4{lsu_flag_wr_ex_i}} & lsu_psr_ex_i[19:16])));

  always @ (posedge hclk or negedge hreset_n)
    if(!hreset_n)
      ipsr_ge_ex <= 4'b0000;
    else if (psr_ge_we)
      ipsr_ge_ex <= nxt_psr_ge_ex;

  assign psr_ge_ex = ipsr_ge_ex & {4{opt_ext_en}};


  assign sel_adv_it_ex = dbg_halted_i & ~instr_it_valid_ex & ~instr_redo_ex_i;

  assign sel_rew_it_ex = (it_skid_ex[0] & fault_lsu_precise_ex_i &
                          nvic_int_invoke_i);

  assign sel_cur_it_ex = ~sel_adv_it_ex & ~sel_rew_it_ex;

  assign psr_it_reg_ex = (({8{sel_adv_it_ex}} & advance_it_reg_ex) |
                          ({8{sel_rew_it_ex}} & rewind_it_reg_ex)  |
                          ({8{sel_cur_it_ex}} & it_reg_ex));

  assign psr_ex = { // NZCVQ
                    psr_flags_ex,
                    psr_it_reg_ex[1:0],
                    psr_t_ex,
                    {4{1'b0}},
                    psr_ge_ex,
                    psr_it_reg_ex[7:2],
                    {10{1'b0}}};


  assign sel_n_alu_ex = alu_flag_wr_ex_i[`CORE_N] & flags_sel_ex[1];
  assign sel_n_mac_ex = (e_mac_flag_wr_ex_i[`CORE_N] & flags_sel_ex[2] &
                         opt_ext_en);
  assign sel_n_mul_ex = (alu_flag_wr_ex_i[`CORE_N]   & flags_sel_ex[1] &
                         ~opt_ext_en);
  assign sel_n_fpu_ex = flags_sel_ex[4] & opt_fpu_en;
  assign sel_n_cur_ex = ~(sel_n_alu_ex | sel_n_mac_ex |
                          sel_n_mul_ex | sel_n_fpu_ex);

  assign alu_nflag_ex = (// Select ALU result
                         (sel_n_alu_ex & alu_flag_res_ex_i[`CORE_N])     |
                         (sel_n_mac_ex & e_mac_nz_ex_i[1]) |
                         (sel_n_mul_ex & alu_mul_flag_res_ex_i[`CORE_N]) |
                         (sel_n_fpu_ex & fpu_wr_data_ex_i[31])           |
                         (sel_n_cur_ex & psr_flags_ex[`CORE_N]));

  assign sel_z_alu_ex = alu_flag_wr_ex_i[`CORE_Z] & flags_sel_ex[1];
  assign sel_z_mac_ex = (e_mac_flag_wr_ex_i[`CORE_Z] & flags_sel_ex[2] &
                         opt_ext_en);
  assign sel_z_mul_ex = (alu_flag_wr_ex_i[`CORE_Z]   & flags_sel_ex[1] &
                         ~opt_ext_en);
  assign sel_z_fpu_ex = flags_sel_ex[4] & opt_fpu_en;
  assign sel_z_cur_ex = ~(sel_z_alu_ex | sel_z_mac_ex |
                          sel_z_mul_ex | sel_z_fpu_ex);

  assign alu_zflag_ex = (// Select ALU result
                         (sel_z_alu_ex & alu_flag_res_ex_i[`CORE_Z])     |
                         (sel_z_mac_ex & e_mac_nz_ex_i[0]) |
                         (sel_z_mul_ex & alu_mul_flag_res_ex_i[`CORE_Z]) |
                         (sel_z_fpu_ex & fpu_wr_data_ex_i[30])           |
                         (sel_z_cur_ex & psr_flags_ex[`CORE_Z]));

  assign sel_c_fpu_ex = flags_sel_ex[4] & opt_fpu_en;
  assign sel_c_cur_ex = ~(sel_c_fpu_ex);

  assign alu_cflag_ex = (// Transfer from FPU
                         (sel_c_fpu_ex & fpu_wr_data_ex_i[29]) |
                         (sel_c_cur_ex & alu_flags_ex[`CORE_C]));

  assign sel_v_fpu_ex = flags_sel_ex[4] & opt_fpu_en;
  assign sel_v_cur_ex = ~sel_v_fpu_ex;

  assign alu_vflag_ex = (// Transfer from FPU
                         (sel_v_fpu_ex & fpu_wr_data_ex_i[28]) |
                         (sel_v_cur_ex & alu_flags_ex[`CORE_V]));

  assign redo_it_ex    = ((set_instr_redo_ex_i | instr_redo_ex_i) &
                          (|it_reg_ex[3:0]));

  assign use_it_reg_ex = ((redo_it_ex | instr_it_valid_ex) &
                          ~nvic_int_invoke_i);

  assign use_it_adv_ex = ((~(redo_it_ex | instr_it_valid_ex) &
                           (|advance_it_reg_ex[3:0])) &
                          ~nvic_int_invoke_i);

  assign use_it_br_de  = ((~(redo_it_ex | instr_it_valid_ex) &
                           ~(|advance_it_reg_ex[3:0])) |
                          nvic_int_invoke_i);

  assign instr_cc_sel_de = (({4{use_it_reg_ex}} & it_reg_ex[7:4])         |
                            ({4{use_it_adv_ex}} & advance_it_reg_ex[7:4]) |
                            ({4{use_it_br_de }} & instr_br_cond_de_i));

  assign instr_ccpass_de = ccpass(alu_nflag_ex,
                                  alu_zflag_ex,
                                  alu_cflag_ex,
                                  alu_vflag_ex,
                                  instr_cc_sel_de);


  assign instr_cc_sel_ex = (((|it_reg_ex[3:0]) & ~instr_it_valid_ex) ?
                            it_reg_ex[7:4] :
                            instr_cond_code_ex);

  assign instr_cc_c_ex   = ccpass(psr_flags_ex[`CORE_N],
                                  psr_flags_ex[`CORE_Z],
                                  psr_flags_ex[`CORE_C],
                                  psr_flags_ex[`CORE_V],
                                  instr_cc_sel_ex);

  assign instr_cc_res_ex = (instr_itfold_ex_i ?
                            instr_itfccpass_ex :
                            instr_cc_c_ex);

  assign instr_ccpass_ex = (instr_czb_ex_i ?
                            (instr_cond_code_ex[0] ^ alu_zero_res_ex_i) :
                            instr_cc_res_ex);

  assign instr_itblock_de_o  = instr_itblock_de;
  assign instr_itccfail_de_o = instr_itccfail_de;
  assign instr_ccpass_de_o   = instr_ccpass_de;
  assign instr_itblock_ex_o  = instr_itblock_ex;
  assign instr_ccpass_ex_o   = instr_ccpass_ex;
  assign psr_t_ex_o          = psr_t_ex;
  assign psr_ex_o            = psr_ex;


`ifdef ARM_ASSERT_ON

  reg            ovl_instr_itccfail_ex;

  always @ (posedge hclk or negedge hreset_n)
    if(!hreset_n)
      ovl_instr_itccfail_ex <= 1'b0;
      else
        ovl_instr_itccfail_ex <= (instr_itccfail_de & instr_valid_de_i &
                                  ~int_entry_i & ~(|instr_br_pflush_ex_i));

  assert_zero_one_hot #(0,5,0,"Flags mux selector must be exclusive")
    ovl_muxsel_exclusive(hclk, hreset_n, flags_sel_ex);

  assert_never #(0,0,
                 "IT decode fail can only be asserted if next opcode is in IT block")
    ovl_itccfail_de_check(hclk, hreset_n, (ovl_instr_itccfail_ex &  ~dbg_halted_i &
                                           ~(instr_itblock_ex | instr_it_valid_ex)));

  assert_zero_one_hot #(0,4,0,"Flag setting controls must be exclusive")
    ovl_flag_wr_check(hclk, hreset_n, {(|alu_flag_wr_ex_i),
                                       (|e_mac_flag_wr_ex_i),
                                       e_simd_ge_wr_ex_i,
                                       e_simd_q_set_ex_i});

`endif

`include "cm3_dpu_undefs.vp"

endmodule
