`include "cm3_dwt_defs.vp"

module cm3_dwt_apb_if
  (// Inputs
   clk, reset_n, dpu_trace_enable_i, ppb_hready_i, ppb_hsel_i, ppb_haddr_i,
   ppb_hwdata_i, ppb_hwrite_i, dpu_ahb_haddrd_i, dpu_ahb_htransd1_i,
   dpu_ahb_hsized_i, dpu_ahb_hwrited_i, dpu_ahb_hwdatad_i, dpu_ahb_hrdatad_i,
   mpu_ahb_hreadyoutd_i, dpu_ahb_haddraccd_i, mpu_canceld_i, dpu_etm_ivalid_i,
   dpu_etm_istall_i, dpu_etm_cancel_i, dpu_etm_pstat_i, dpu_etm_lsu_i,
   dpu_etm_fold_i, dpu_etm_ia_i, dpu_sleeping_i, dpu_sleep_hold_ack_n_i,
   emit_etm_trigger_i, emit_watchpoint_i, emit_pc_sample_i, emit_data_sample_i,
   qualify_data0_i, qualify_data2_i, qualify_data3_i,
   ppb_hreadyout_o, ppb_hresp_o, ppb_hrdata_o, dpu_ahb_hwrite_reg_o,
   dpu_ahb_haddr_reg_o, dpu_ahb_haddrd_masked_o, dpu_ahb_hsize_reg_o,
   dwt_dist_sync_o, dwt_watchpoint_o, dwt_observe_watchpoint_o,
   clear_etm_ivalid_o, allow_pc_match_o, address_phase_o, dwt_pc_cycle_o,
   dwt_pc_sample_match0_o, act_as_data_addr0_o, dwt_emit_range0_o, dwt_mask0_o,
   dwt_function0_o, dwt_comp0_o, dwt_match_on_data1_o, dwt_match_size1_o,
   dwt_emit_range1_o, dwt_mask1_o, dwt_function1_o, dwt_comp1_o,
   act_as_data_addr2_o, dwt_emit_range2_o, dwt_mask2_o, dwt_function2_o,
   dwt_comp2_o, act_as_data_addr3_o, dwt_emit_range3_o, dwt_mask3_o,
   dwt_function3_o, dwt_comp3_o, comp1_good_data_o, sleep_state_o,
   emit_pc_periodic_o, emit_cyc_o, emit_cpi_o, emit_int_o, emit_sleep_o,
   emit_fold_o, emit_lsu_o, int_trace_en_o, watchpoint_data_o
   );

  parameter TRACE_LVL      = 1;  // Trace support level. Between 0 and 3.
  parameter DEBUG_LVL      = 3;  // Debug support level. Between 0 and 3.
  parameter RESET_ALL_REGS = 0;  // Reset all registers if 1.
  parameter OBSERVATION    = 0;  // Allow internal observation if 1.
  parameter CP_PRESENT     = 0;  // Affects ID registers


  input         clk;                      // Clock
  input         reset_n;                  // Reset
  input         dpu_trace_enable_i;       // enable tracing
  input         ppb_hready_i;             // Ready (PPB)

  input         ppb_hsel_i;               // DWT select
  input         ppb_hwrite_i;             // PPB write
  input [12:2]  ppb_haddr_i;              // PPB address
  input [31:0]  ppb_hwdata_i;             // PPB write data
  output        ppb_hreadyout_o;          // DWT ready
  output        ppb_hresp_o;              // DWT response
  output [31:0] ppb_hrdata_o;             // DWT read data

  input [3:0]   emit_etm_trigger_i;
  input [3:0]   emit_watchpoint_i;        // Emit Watchpoint - bit per comparator
  input [3:0]   emit_pc_sample_i;         // Emit Sample PC - bit per comparator
  input [3:0]   emit_data_sample_i;       // Emit Sample Data - bit per comparator

  input         qualify_data0_i;          // Data match qualify
  input         qualify_data2_i;          // Data match qualify
  input         qualify_data3_i;          // Data match qualify

  input [31:0]  dpu_ahb_haddrd_i;         // DCORE address
  input         dpu_ahb_htransd1_i;       // DCORE transaction
  input [1:0]   dpu_ahb_hsized_i;         // DCORE size
  input         dpu_ahb_hwrited_i;        // DCORE write
  input [31:0]  dpu_ahb_hwdatad_i;        // DCORE wdata
  input [31:0]  dpu_ahb_hrdatad_i;        // DCORE rdata
  input         mpu_ahb_hreadyoutd_i;     // DCORE ready
  input         dpu_ahb_haddraccd_i;      // DCORE accept
  input         mpu_canceld_i;            // MPU cancel D

  input         dpu_etm_ivalid_i;         // Instruction Valid
  input         dpu_etm_istall_i;         // ETM opcode stall
  input         dpu_etm_cancel_i;         // Opcode cancelled
  input [1:0]   dpu_etm_pstat_i;          // Pipeline status
  input         dpu_etm_lsu_i;            // Processing LSU operation
  input         dpu_etm_fold_i;           // Folding an instruction
  input [31:1]  dpu_etm_ia_i;             // PC
  input         dpu_sleeping_i;           // Core is sleeping
  input         dpu_sleep_hold_ack_n_i;   // Core is in extended sleep

  output        dpu_ahb_hwrite_reg_o;
  output [1:0]  dpu_ahb_haddr_reg_o;
  output [31:0] dpu_ahb_haddrd_masked_o;  // Masked with dpu_trace_enable_i
  output [1:0]  dpu_ahb_hsize_reg_o;
  output        dwt_dist_sync_o;          // Synchronisation for ITM/TPIU
  output        dwt_watchpoint_o;         // Watchpoint
  output        dwt_observe_watchpoint_o; // Watchpoint observation

  output        clear_etm_ivalid_o;
  output        allow_pc_match_o;
  output        address_phase_o;
  output [31:0] dwt_pc_cycle_o;

  output        dwt_pc_sample_match0_o;
  output        act_as_data_addr0_o;
  output        dwt_emit_range0_o;
  output [3:0]  dwt_mask0_o;
  output [3:0]  dwt_function0_o;
  output [31:0] dwt_comp0_o;

  output        dwt_match_on_data1_o;
  output [1:0]  dwt_match_size1_o;
  output        dwt_emit_range1_o;
  output [3:0]  dwt_mask1_o;
  output [3:0]  dwt_function1_o;
  output [31:0] dwt_comp1_o;

  output        act_as_data_addr2_o;
  output        dwt_emit_range2_o;
  output [3:0]  dwt_mask2_o;
  output [3:0]  dwt_function2_o;
  output [31:0] dwt_comp2_o;

  output        act_as_data_addr3_o;
  output        dwt_emit_range3_o;
  output [3:0]  dwt_mask3_o;
  output [3:0]  dwt_function3_o;
  output [31:0] dwt_comp3_o;

  output        comp1_good_data_o;        // Comp1 can attempt a data match
  output        sleep_state_o;
  output        emit_pc_periodic_o;
  output        emit_cyc_o;
  output        emit_cpi_o;
  output        emit_int_o;
  output        emit_sleep_o;
  output        emit_fold_o;
  output        emit_lsu_o;
  output        int_trace_en_o;
  output [31:0] watchpoint_data_o;

  wire          sleep_state;              // Sleeping state
  reg [6:2]     ahb_addr_reg;             // Registered ppb_haddr_i
  reg           ahb_write_reg;            // Registered ppb_hwrite_i

  wire [18:0]   register_select;          // DWT register address decode
  wire          register_select_pcsr;     // DWT_pc_sample_value address decode
  wire [2:0]    register_select_id2_0;    // ID register address decode
  wire          register_select_id4;      // ID register address decode
  wire [3:0]    register_select_comp;     // ID register address decode
  wire [18:0]   register_we;              // Write strobes for DWT registers
  reg           data_phase;               // Data phase enable (2 cycles)
  wire          data_phase_we;            // Update write enable
  wire [23:0]   periph_id_0_2;            // Value for peripheral ID
  wire [7:0]    periph_id_4;              // Value for peripheral ID
  wire [31:0]   component_id;             // Component Identifier

  wire          unused_address;           // Not a valid register
  reg           unused_address_reg;       // Not a valid register - data phase
  wire          access_we;                // Control phase enable (2 cycles)
  wire          id_bank_select;           // Read from ID register
  reg           id_bank_select_reg;       // Valid ID address - data phase
  wire          dpu_ahb_halted;           // Core is halted
  wire          core_in_execute;          // Core in execute
  wire          core_in_micro;            // Core in microcode

  reg [22:0]    dwt_ctrl;                 // DWT Control Register
  reg [31:0]    dwt_pc_cycle;             // DWT PC Sampler Cycle Counter
  reg [3:0]     dwt_pc_cycle_scale;       // DWT PSCYCCNT post-scaler
  reg  [7:0]    opt_dwt_lsu_count;        // DWT LSU counter
  wire [7:0]    dwt_lsu_count;            // tied to 0 if no trace support
  reg  [7:0]    opt_dwt_fold_count;       // DWT Folded instruction counter
  wire [7:0]    dwt_fold_count;           // tied to 0 if no trace support
  reg  [7:0]    opt_dwt_cpi_count;        // DWT CPI Counter
  wire [7:0]    dwt_cpi_count;            // tied to 0 if no trace support
  reg  [7:0]    opt_dwt_int_count;        // DWT Interrupt Counter
  wire [7:0]    dwt_int_count;            // DWT Interrupt Counter
  reg  [7:0]    opt_dwt_sleep_count;      // DWT Sleep Counter
  wire [7:0]    dwt_sleep_count;          // tied to 0 if no trace support

  reg [31:0]    dwt_comp0;                // DWT Comparator 0
  reg [3:0]     dwt_mask0;                // DWT Mask 0
  reg           dwt_pc_sample_match0;     // DWT PS Count Match 0
  reg           dwt_emit_range0;          // DWT Emit Range 0
  reg [3:0]     dwt_function0;            // DWT Function 0
  reg           dwt_matched0;             // DWT Matched 0
  reg [31:0]    opt_dwt_comp1;            // DWT Comparator 1
  reg [3:0]     opt_dwt_mask1;            // DWT Mask 1
  reg           opt_dwt_emit_range1;      // DWT Emit Range 1
  reg [1:0]     opt_dwt_match_size1;      // DWT Data Size 1
  reg           opt_dwt_match_on_data1;   // DWT Data Match 1
  reg [1:0]     opt_dwt_data1_addr_src0;  // DWT Data Match Addr Src 0
  reg [1:0]     opt_dwt_data1_addr_src1;  // DWT Data Match Addr Src 1
  reg [3:0]     opt_dwt_function1;        // DWT Function 1
  reg           opt_dwt_matched1;         // DWT Matched 1
  reg [31:0]    opt_dwt_comp2;            // DWT Comparator 2
  reg [3:0]     opt_dwt_mask2;            // DWT Mask 2
  reg           opt_dwt_emit_range2;      // DWT Emit Range 2
  reg [3:0]     opt_dwt_function2;        // DWT Function 2
  reg           opt_dwt_matched2;         // DWT Matched 2
  reg [31:0]    opt_dwt_comp3;            // DWT Comparator 3
  reg [3:0]     opt_dwt_mask3;            // DWT Mask 3
  reg           opt_dwt_emit_range3;      // DWT Emit Range 3
  reg [3:0]     opt_dwt_function3;        // DWT Function 3
  reg           opt_dwt_matched3;         // DWT Matched 3

  wire          address_phase;            // Bus transfer address phase
  wire          qualify_data0_we;         // Data match qualify write enable
  reg           opt_qualify_data0_reg;    // Data match qualify - registered
  wire          qualify_data0_reg;        // Data match qualify - registered
  wire          qualify_data1_we;         // Data match qualify write enable
  reg           opt_qualify_data1_reg;    // Data match qualify - registered
  wire          qualify_data1_reg;        // Data match qualify - registered
  wire          qualify_data2_we;         // Data match qualify write enable
  reg           opt_qualify_data2_reg;    // Data match qualify - registered
  wire          qualify_data2_reg;        // Data match qualify - registered
  wire          qualify_data3_we;         // Data match qualify write enable
  reg           opt_qualify_data3_reg;    // Data match qualify - registered
  wire          qualify_data3_reg;        // Data match qualify - registered

  wire          comp1_good_data;          // Comp1 can attempt a data match
  wire          act_as_data_addr0;        // Comp0 is an addr comp for a data comp
  wire          act_as_data_addr2;        // Comp2 is an addr comp for a data comp
  wire          act_as_data_addr3;        // Comp3 is an addr comp for a data comp

  wire          cycle_count_tap_select;   // Taps PSCYCCNT bit 10 if true, or bit 6
  wire          cycle_count_init;         // PSCYCCNT initialised on next cycle
  wire          cycle_count_we;           // PSCYCCNT enable
  wire [31:0]   cycle_count_nxt;          // PSCYCCNT counter next value

  wire          dpu_ahb_halted_we;        // Enable register for dpu_ahb_halted
  reg           dpu_ahb_halted_reg;       // Registered version of dpu_ahb_halted

  wire [3:0]    post_reload_value;        // Reload value for PSPOSTCNT
  wire [3:0]    post_scale_set;           // Initial load value for PSPOSTCNT
  wire          scaled_count_init;        // PSPOSTCNT initialised on next cycle
  wire          scaled_count_we;          // PSPOSTCNT enable
  reg  [3:0]    scaled_count_nxt;         // PSPOSTCNT counter next value
  wire          scaled_count_overflow;    // PSPOSTCNT = 4'b0000
  wire          selected_tap;             // Tap from PSCYCCOUNT
  reg           opt_selected_tap_reg;     // Registered version of selected_tap
  wire          selected_tap_reg;         // Tied to 0 if no trace support
  wire          selected_tap_we;          // enable scaler toggle register
  wire          post_scale_inc;           // enable for post-scaler

  reg           dwt_dist_sync;            // Synchronising pulse for TPIU
  wire [1:0]    dist_sync_tap_select;     // DWT dwt_dist_sync_o tap register
  reg           dist_sync_tap;            // Taps from PSCYCCOUNT
  reg           dist_sync_tap_reg;        // Registered version of dist_sync_tap
  wire          dist_sync_nxt;            // Next value of dwt_dist_sync_o
  wire          dist_sync_tap_we;         // enable dwt_dist_sync_o update

  wire          cpi_count_init;           // cpi_count initialised on next cycle
  wire          cpi_count_inc;            // cpi_count increment
  wire          cpi_count_we;             // cpi_count enable
  wire [7:0]    cpi_count_nxt;            // cpi_count counter next value

  wire          int_count_init;           // int_count initialised on next cycle
  wire          int_count_inc;            // int_count increment
  wire          int_count_we;             // int_count enable
  wire [7:0]    int_count_nxt;            // int_count counter next value

  wire          lsu_count_init;           // lsu_count initialised on next cycle
  wire          lsu_count_inc;            // lsu_count increment
  wire          lsu_count_we;             // lsu_count enable
  wire [7:0]    lsu_count_nxt;            // lsu_count counter next value

  wire          fold_count_init;          // fold_count initialised on next cycle
  wire          fold_count_inc;           // fold_count increment
  wire          fold_count_we;            // fold_count enable
  wire [7:0]    fold_count_nxt;           // fold_count counter next value

  wire          sleep_count_init;         // sleep_count initialised on next cycle
  wire          sleep_count_inc;          // sleep_count increment
  wire          sleep_count_we;           // sleep_count enable
  wire [7:0]    sleep_count_nxt;          // sleep_count counter next value

  wire          pc_event_en;              // PC Sampler event enabled
  wire          cycle_event_en;           // PSCYCCNT=0 event enabled
  wire          sleep_event_en;           // Sleep Count overflow event enabled
  wire          int_event_en;             // Int overhead count overflow event enabled
  wire          cpi_event_en;             // CPI counter overflow event enabled
  wire          int_trace_en;             // Interrupt trace event enabled
  wire          fold_event_en;            // Fold counter overflow event enabled
  wire          lsu_event_en;             // LSU counter overflow event enabled
  wire          cycle_count_en;           // Cycle counter enabled

  wire          emit_pc_periodic;         // Emit PCSample
  wire          emit_cyc;                 // Emit PSCYCCNT underflow event
  wire          emit_cpi;                 // Emit CPI Event
  wire          emit_int;                 // Emit Interrupt event
  wire          emit_sleep;               // Emit Sleep event
  wire          emit_fold;                // Emit fold event
  wire          emit_lsu;                 // Emit LSU event

  wire [31:0]   pc_address;               // PC used for comparison
  wire [31:0]   pc_sample_value;          // PC used for DWT_pc_sample_value
  reg           etm_ivalid_reg;           // Registered dpu_etm_ivalid_i
  wire          clear_etm_ivalid;         // Clear etm_ivalid_reg
  wire          etm_ivalid_reg_we;        // etm_ivalid_reg write enable
  wire          allow_pc_match;           // Allow the PC to be compared

  wire          trans_control_we;         // WriteReg enable
  reg           dpu_ahb_hwrite_reg;       // dpu_ahb_hwrited_i register
  reg [1:0]     dpu_ahb_hsize_reg;        // dpu_ahb_hsized_i register
  reg [1:0]     dpu_ahb_haddr_reg;        // dpu_ahb_haddrd_i[1:0] register
  wire [31:0]   dpu_ahb_haddrd_masked;    // Mask for power saving
  wire [31:0]   watchpoint_data;          // WPdata - mux of read and write data

  wire [31:0]   dwt_comp1;                // DWT Comparator 1
  wire [3:0]    dwt_mask1;                // DWT Mask 1
  wire          dwt_emit_range1;          // DWT Emit Range 1
  wire [1:0]    dwt_match_size1;          // DWT Data Size 1
  wire          dwt_match_on_data1;       // DWT Data Match 1
  wire [1:0]    dwt_data1_addr_src0;      // DWT Data Match Addr Src 0
  wire [1:0]    dwt_data1_addr_src1;      // DWT Data Match Addr Src 1
  wire [3:0]    dwt_function1;            // DWT Function 1
  wire          dwt_matched1;             // DWT Matched 1
  wire [31:0]   dwt_comp2;                // DWT Comparator 2
  wire [3:0]    dwt_mask2;                // DWT Mask 2
  wire          dwt_emit_range2;          // DWT Emit Range 2
  wire [3:0]    dwt_function2;            // DWT Function 2
  wire          dwt_matched2;             // DWT Matched 2
  wire [31:0]   dwt_comp3;                // DWT Comparator 3
  wire [3:0]    dwt_mask3;                // DWT Mask 3
  wire          dwt_emit_range3;          // DWT Emit Range 3
  wire [3:0]    dwt_function3;            // DWT Function 3
  wire          dwt_matched3;             // DWT Matched 3

  wire          matched0_clear;           // dwt_matched0 clear
  wire          matched0_set;             // dwt_matched0 set
  wire          matched0_we;              // dwt_matched0 write enable
  wire          matched1_clear;           // dwt_matched1 clear
  wire          matched1_set;             // dwt_matched1 set
  wire          matched1_we;              // dwt_matched1 write enable
  wire          matched2_clear;           // dwt_matched2 clear
  wire          matched2_set;             // dwt_matched2 set
  wire          matched2_we;              // dwt_matched2 write enable
  wire          matched3_clear;           // dwt_matched3 clear
  wire          matched3_set;             // dwt_matched3 set
  wire          matched3_we;              // dwt_matched3 write enable

  wire          opt_data_match_en;
  wire          opt_full_wpoint_en;
  wire [31:0]   opt_hrdata;
  wire [31:0]   read_data_main;
  wire [7:0]    read_data_id;
  wire          watch_point;
  reg           watch_point_reg;
  wire          opt_dwt_en;
  wire          opt_trace_en;
  wire          opt_reset_all_n;

`ifdef ARM_DSM

  wire          dsm_observation;
  wire [1:0]    dsm_debug_lvl;
  wire [1:0]    dsm_trace_lvl;
  wire          dsm_reset_all_regs;

  assign opt_reset_all_n    = (dsm_reset_all_regs)  ? reset_n : 1'b1;

  assign opt_dwt_en         = (dsm_debug_lvl > 0)   ? 1'b1    : 1'b0;

  assign opt_trace_en       = (dsm_trace_lvl > 0)   ? 1'b1    : 1'b0;

  assign opt_data_match_en  = (dsm_debug_lvl > 2)   ? 1'b1    : 1'b0;

  assign opt_full_wpoint_en = (dsm_debug_lvl > 1)   ? 1'b1    : 1'b0;

`else

  assign opt_reset_all_n    = (RESET_ALL_REGS == 1) ? reset_n : 1'b1;

  assign opt_dwt_en         = (DEBUG_LVL > 0)       ? 1'b1    : 1'b0;

  assign opt_trace_en       = (TRACE_LVL > 0)       ? 1'b1    : 1'b0;

  assign opt_data_match_en  = (DEBUG_LVL > 2)       ? 1'b1    : 1'b0;

  assign opt_full_wpoint_en = (DEBUG_LVL > 1)       ? 1'b1    : 1'b0;

`endif


  assign sleep_state    = dpu_sleeping_i | ~dpu_sleep_hold_ack_n_i;


  assign unused_address = ((ppb_haddr_i[12:7] != `DWT_BASE_SHORT) ||
                           (ppb_haddr_i[6:2]  >= `DWT_TOP_ADDRESS));

  assign id_bank_select = (ppb_haddr_i[12:2] >= `DWT_ID_BASE);

  assign access_we = ppb_hsel_i & ppb_hready_i & dpu_trace_enable_i & opt_dwt_en;

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      begin
        ahb_addr_reg        <= 5'b00000;
        ahb_write_reg       <= 1'b0;
        unused_address_reg  <= 1'b0;
        id_bank_select_reg  <= 1'b0;
      end
    else if (access_we)
      begin
        ahb_addr_reg        <= ppb_haddr_i[6:2];
        ahb_write_reg       <= ppb_hwrite_i;
        unused_address_reg  <= unused_address;
        id_bank_select_reg  <= id_bank_select;
      end

  assign data_phase_we = data_phase | access_we;

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      data_phase <= 1'b0;
    else if (data_phase_we)
      data_phase <= access_we;

  assign register_select[0]  = (ahb_addr_reg == `DWT_CTRL_ADDR)      & ~unused_address_reg;
  assign register_select[1]  = (ahb_addr_reg == `DWT_PSCYCCNT_ADDR)  & ~unused_address_reg;
  assign register_select[2]  = (ahb_addr_reg == `DWT_CPICNT_ADDR)    & ~unused_address_reg;
  assign register_select[3]  = (ahb_addr_reg == `DWT_INTCNT_ADDR)    & ~unused_address_reg;
  assign register_select[4]  = (ahb_addr_reg == `DWT_SLEEPCNT_ADDR)  & ~unused_address_reg;
  assign register_select[5]  = (ahb_addr_reg == `DWT_LSUCNT_ADDR)    & ~unused_address_reg;
  assign register_select[6]  = (ahb_addr_reg == `DWT_FOLDCNT_ADDR)   & ~unused_address_reg;
  assign register_select_pcsr = (ahb_addr_reg == `DWT_PCSR_ADDR)     & ~unused_address_reg;
  assign register_select[7]  = (ahb_addr_reg == `DWT_COMP0_ADDR)     & ~unused_address_reg;
  assign register_select[8]  = (ahb_addr_reg == `DWT_MASK0_ADDR)     & ~unused_address_reg;
  assign register_select[9]  = (ahb_addr_reg == `DWT_FUNCTION0_ADDR) & ~unused_address_reg;
  assign register_select[10] = (ahb_addr_reg == `DWT_COMP1_ADDR)     & ~unused_address_reg & opt_full_wpoint_en;
  assign register_select[11] = (ahb_addr_reg == `DWT_MASK1_ADDR)     & ~unused_address_reg & opt_full_wpoint_en;
  assign register_select[12] = (ahb_addr_reg == `DWT_FUNCTION1_ADDR) & ~unused_address_reg & opt_full_wpoint_en;
  assign register_select[13] = (ahb_addr_reg == `DWT_COMP2_ADDR)     & ~unused_address_reg & opt_full_wpoint_en;
  assign register_select[14] = (ahb_addr_reg == `DWT_MASK2_ADDR)     & ~unused_address_reg & opt_full_wpoint_en;
  assign register_select[15] = (ahb_addr_reg == `DWT_FUNCTION2_ADDR) & ~unused_address_reg & opt_full_wpoint_en;
  assign register_select[16] = (ahb_addr_reg == `DWT_COMP3_ADDR)     & ~unused_address_reg & opt_full_wpoint_en;
  assign register_select[17] = (ahb_addr_reg == `DWT_MASK3_ADDR)     & ~unused_address_reg & opt_full_wpoint_en;
  assign register_select[18] = (ahb_addr_reg == `DWT_FUNCTION3_ADDR) & ~unused_address_reg & opt_full_wpoint_en;

  assign register_select_id2_0[0] = (ahb_addr_reg[5:2] == `DWT_PERIPH_ADDR_0) & id_bank_select_reg;
  assign register_select_id2_0[1] = (ahb_addr_reg[5:2] == `DWT_PERIPH_ADDR_1) & id_bank_select_reg;
  assign register_select_id2_0[2] = (ahb_addr_reg[5:2] == `DWT_PERIPH_ADDR_2) & id_bank_select_reg;
  assign register_select_id4      = (ahb_addr_reg[5:2] == `DWT_PERIPH_ADDR_4) & id_bank_select_reg;
  assign register_select_comp[0]  = (ahb_addr_reg[5:2] == `DWT_PCELL_ADDR_0)  & id_bank_select_reg;
  assign register_select_comp[1]  = (ahb_addr_reg[5:2] == `DWT_PCELL_ADDR_1)  & id_bank_select_reg;
  assign register_select_comp[2]  = (ahb_addr_reg[5:2] == `DWT_PCELL_ADDR_2)  & id_bank_select_reg;
  assign register_select_comp[3]  = (ahb_addr_reg[5:2] == `DWT_PCELL_ADDR_3)  & id_bank_select_reg;

  assign register_we[18:0] = register_select[18:0] & {19{opt_dwt_en & ahb_write_reg & data_phase}};

  assign read_data_main = (({opt_full_wpoint_en  ? 4'b0100  : 4'b0001,
                            opt_trace_en ? 5'b00000 : 5'b11110,
                                        dwt_ctrl[22:0]}     & {32{register_select[0]}}) |
                         (              dwt_pc_cycle[31:0]  & {32{register_select[1]}}) |
                         ({{24{1'b0}},  dwt_cpi_count}      & {32{register_select[2]}}) |
                         ({{24{1'b0}},  dwt_int_count}      & {32{register_select[3]}}) |
                         ({{24{1'b0}},  dwt_sleep_count}    & {32{register_select[4]}}) |
                         ({{24{1'b0}},  dwt_lsu_count}      & {32{register_select[5]}}) |
                         ({{24{1'b0}},  dwt_fold_count}     & {32{register_select[6]}}) |
                         (              pc_sample_value     & {32{register_select_pcsr}})|
                         (              dwt_comp0[31:0]     & {32{register_select[7]}}) |
                         ({{28{1'b0}},  dwt_mask0}          & {32{register_select[8]}}) |
                         ({{7{1'b0}},   dwt_matched0,
                           {16{1'b0}},  dwt_pc_sample_match0,
                           1'b0,        dwt_emit_range0,
                           1'b0,        dwt_function0[3:0]} & {32{register_select[9]}}) |
                         (              dwt_comp1[31:0]     & {32{register_select[10]}})|
                         ({{28{1'b0}},  dwt_mask1}          & {32{register_select[11]}})|
                         ({{7{1'b0}},   dwt_matched1,
                           {6{1'b0}},   dwt_data1_addr_src1[1:0],
                           {2{1'b0}},   dwt_data1_addr_src0[1:0],
                                        dwt_match_size1[1:0],
                                        1'b1,
                                        dwt_match_on_data1,
                           2'b00,       dwt_emit_range1,
                           1'b0,        dwt_function1[3:0]} & {32{register_select[12]}})|
                         (              dwt_comp2[31:0]     & {32{register_select[13]}})|
                         ({{28{1'b0}},  dwt_mask2}          & {32{register_select[14]}})|
                         ({{7{1'b0}},   dwt_matched2,
                           {18{1'b0}},  dwt_emit_range2,
                           1'b0,        dwt_function2[3:0]} & {32{register_select[15]}})|
                         (              dwt_comp3[31:0]     & {32{register_select[16]}})|
                         ({{28{1'b0}},  dwt_mask3}          & {32{register_select[17]}})|
                         ({{7{1'b0}},   dwt_matched3,
                           {18{1'b0}},  dwt_emit_range3,
                           1'b0,        dwt_function3[3:0]} & {32{register_select[18]}}));

  assign periph_id_0_2 = `DWT_PERIPH_VALUE_LO;
  assign periph_id_4   = `DWT_PERIPH_VALUE_HI;
  assign component_id  = `DWT_PCELL_VALUE;

  assign read_data_id = ((periph_id_0_2[7:0]   & {8{register_select_id2_0[0]}})|
                         (periph_id_0_2[15:8]  & {8{register_select_id2_0[1]}})|
                         (periph_id_0_2[23:16] & {8{register_select_id2_0[2]}})|
                         (periph_id_4[7:0]     & {8{register_select_id4}})     |
                         (component_id[7:0]    & {8{register_select_comp[0]}}) |
                         (component_id[15:8]   & {8{register_select_comp[1]}}) |
                         (component_id[23:16]  & {8{register_select_comp[2]}}) |
                         (component_id[31:24]  & {8{register_select_comp[3]}}));

  assign ppb_hreadyout_o  = 1'b1;
  assign ppb_hresp_o      = 1'b0;
  assign opt_hrdata       = read_data_main | {{24{1'b0}},read_data_id};

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      dwt_ctrl[22:0]  <= {23{1'b0}};
    else if(register_we[0])
      dwt_ctrl[22:0]  <= ppb_hwdata_i[22:0];

  assign pc_event_en    = dpu_trace_enable_i & dwt_ctrl[`DWT_CTRL_PCSAMPLEENA];
  assign cycle_event_en = dpu_trace_enable_i & dwt_ctrl[`DWT_CTRL_CYCEVTENA];
  assign sleep_event_en = dpu_trace_enable_i & dwt_ctrl[`DWT_CTRL_SLEEPEVTENA];
  assign int_event_en   = dpu_trace_enable_i & dwt_ctrl[`DWT_CTRL_INTEVTENA];
  assign cpi_event_en   = dpu_trace_enable_i & dwt_ctrl[`DWT_CTRL_CPIEVTENA];
  assign int_trace_en   = dpu_trace_enable_i & dwt_ctrl[`DWT_CTRL_INTTRCENA];
  assign fold_event_en  = dpu_trace_enable_i & dwt_ctrl[`DWT_CTRL_FOLDEVTENA];
  assign lsu_event_en   = dpu_trace_enable_i & dwt_ctrl[`DWT_CTRL_LSUEVTENA];
  assign cycle_count_en = dpu_trace_enable_i & dwt_ctrl[`DWT_CTRL_PSCYCCNTENA];
  assign cycle_count_tap_select = dwt_ctrl[`DWT_CTRL_PSTAPSEL];
  assign post_scale_set         = dwt_ctrl[`DWT_CTRL_PSPOST+`DWT_CTRL_PSPOST_SIZE-1:
                                           `DWT_CTRL_PSPOST];
  assign post_reload_value      = dwt_ctrl[`DWT_CTRL_PSPOST_RELOAD+`DWT_CTRL_PSPOST_SIZE-1:
                                           `DWT_CTRL_PSPOST_RELOAD];
  assign dist_sync_tap_select   = dwt_ctrl[`DWT_CTRL_DSYNC+`DWT_CTRL_DSYNC_SIZE-1:
                                           `DWT_CTRL_DSYNC];

  assign cycle_count_init = register_we[1];

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      dwt_pc_cycle[31:0] <= 32'h00000000;
    else if(cycle_count_we)
      dwt_pc_cycle[31:0] <= cycle_count_nxt;

  assign dpu_ahb_halted  = (dpu_etm_pstat_i == `DWT_PSTAT_IDLE) & ~sleep_state;
  assign core_in_execute = (dpu_etm_pstat_i == `DWT_PSTAT_XEQ) & ~sleep_state;
  assign core_in_micro   = (dpu_etm_pstat_i == `DWT_PSTAT_INT) & ~sleep_state;

  assign cycle_count_we  = ((cycle_count_init |
                             (cycle_count_en & ~dpu_ahb_halted)) & opt_trace_en);
  assign cycle_count_nxt = (cycle_count_init ? ppb_hwdata_i[31:0]:
                                               (dwt_pc_cycle + {32'h00000001}));


  assign scaled_count_init = (register_we[0] &
                              ((ppb_hwdata_i[`DWT_CTRL_PCSAMPLEENA] & ~pc_event_en) |
                               (ppb_hwdata_i[`DWT_CTRL_CYCEVTENA] &
                                ~cycle_event_en & ~pc_event_en)));

  assign selected_tap = (cycle_count_tap_select ?
                         dwt_pc_cycle[`DWT_SCALER_TAP_1]:
                         dwt_pc_cycle[`DWT_SCALER_TAP_0]);

  assign selected_tap_we = (dpu_trace_enable_i & ~dpu_ahb_halted) & opt_trace_en;

  always @(posedge clk or negedge reset_n)
    if (!reset_n)
      opt_selected_tap_reg <= 1'b0;
    else if(selected_tap_we)
      opt_selected_tap_reg <= selected_tap;

  assign selected_tap_reg = opt_trace_en ? opt_selected_tap_reg : 1'b0;

  assign post_scale_inc = ((selected_tap ^ selected_tap_reg) & cycle_count_en &
                           (pc_event_en | cycle_event_en) & ~dpu_ahb_halted);

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      dwt_pc_cycle_scale[3:0] <= 4'b0000;
    else if(scaled_count_we)
      dwt_pc_cycle_scale[3:0] <= scaled_count_nxt;

  assign scaled_count_we = (scaled_count_init | post_scale_inc) & opt_trace_en;

  always @(scaled_count_init or post_scale_set or dwt_pc_cycle_scale or
           post_reload_value)
    case ({scaled_count_init,(dwt_pc_cycle_scale == 4'b0000)})
      2'b00:   scaled_count_nxt = (dwt_pc_cycle_scale - 4'b0001);
      2'b01:   scaled_count_nxt = post_reload_value;
      2'b10,
      2'b11:   scaled_count_nxt = post_scale_set;
      default: scaled_count_nxt = 4'bxxxx;
    endcase

  assign scaled_count_overflow = (dwt_pc_cycle_scale == 4'b0000) & post_scale_inc;

  assign emit_pc_periodic = pc_event_en & scaled_count_overflow;
  assign emit_cyc         = cycle_event_en & ~pc_event_en & scaled_count_overflow;


  assign cpi_count_init = (register_we[0] &
                           ppb_hwdata_i[`DWT_CTRL_CPIEVTENA] & ~cpi_event_en);

  always @(posedge clk or negedge opt_reset_all_n)
    if (!opt_reset_all_n)
      opt_dwt_cpi_count[7:0] <= {8{1'b0}};
    else if (cpi_count_we)
      opt_dwt_cpi_count[7:0] <= cpi_count_nxt;

  assign dwt_cpi_count[7:0] = opt_trace_en ? opt_dwt_cpi_count[7:0] : {8{1'b0}};

  assign cpi_count_inc = (cpi_event_en & core_in_execute & ~dpu_etm_lsu_i &
                          ~dpu_etm_ivalid_i);
  assign cpi_count_we  = (opt_trace_en &
                          (register_we[2] | cpi_count_init | cpi_count_inc));
  assign cpi_count_nxt = (register_we[2] ?
                          ppb_hwdata_i[7:0]:
                          ({8{~cpi_count_init}} & (dwt_cpi_count + 8'h01)));

  assign emit_cpi      = cpi_count_inc & (&dwt_cpi_count);

  assign int_count_init = (register_we[0] &
                       (ppb_hwdata_i[`DWT_CTRL_INTEVTENA] & ~int_event_en));

  always @(posedge clk or negedge opt_reset_all_n)
    if (!opt_reset_all_n)
      opt_dwt_int_count[7:0] <= {8{1'b0}};
    else if (int_count_we & opt_trace_en)
      opt_dwt_int_count[7:0] <= int_count_nxt;

  assign dwt_int_count[7:0] = opt_trace_en ? opt_dwt_int_count[7:0] : {8{1'b0}};

  assign int_count_inc = int_event_en & core_in_micro;
  assign int_count_we  = register_we[3] | int_count_init | int_count_inc;
  assign int_count_nxt = (register_we[3] ?
                          ppb_hwdata_i[7:0] :
                          ({8{~int_count_init}} & (dwt_int_count + 1'b1)));

  assign emit_int      = int_count_inc & (&dwt_int_count);


  assign sleep_count_init = (register_we[0] & ~sleep_event_en &
                             ppb_hwdata_i[`DWT_CTRL_SLEEPEVTENA]);

  always @(posedge clk or negedge opt_reset_all_n)
    if (!opt_reset_all_n)
      opt_dwt_sleep_count[7:0] <= {8{1'b0}};
    else if (sleep_count_we & opt_trace_en)
      opt_dwt_sleep_count[7:0] <= sleep_count_nxt;

  assign dwt_sleep_count[7:0] = (opt_trace_en ? opt_dwt_sleep_count[7:0] :
                                                {8{1'b0}});

  assign sleep_count_inc = sleep_event_en & sleep_state;
  assign sleep_count_we  = register_we[4] | sleep_count_init | sleep_count_inc;
  assign sleep_count_nxt = (register_we[4] ?
                            ppb_hwdata_i[7:0] :
                            ({8{~sleep_count_init}} & (dwt_sleep_count + 1'b1)));

  assign emit_sleep      = sleep_count_inc & (&dwt_sleep_count);

  assign lsu_count_init = (register_we[0] & ppb_hwdata_i[`DWT_CTRL_LSUEVTENA] &
                           ~lsu_event_en);

  always @(posedge clk or negedge opt_reset_all_n)
    if (!opt_reset_all_n)
      opt_dwt_lsu_count[7:0] <= {8{1'b0}};
    else if (lsu_count_we & opt_trace_en)
      opt_dwt_lsu_count[7:0] <= lsu_count_nxt;

  assign dwt_lsu_count[7:0] = opt_trace_en ? opt_dwt_lsu_count[7:0] : {8{1'b0}};

  assign lsu_count_inc = (lsu_event_en & core_in_execute & dpu_etm_lsu_i &
                          ~dpu_etm_ivalid_i);
  assign lsu_count_we  = register_we[5] | lsu_count_init | lsu_count_inc;
  assign lsu_count_nxt = (register_we[5] ?
                          ppb_hwdata_i[7:0] :
                          ({8{~lsu_count_init}} & (dwt_lsu_count + 1'b1)));

  assign emit_lsu      = lsu_count_inc & (&dwt_lsu_count);

  assign fold_count_init = (register_we[0] & ~fold_event_en &
                            ppb_hwdata_i[`DWT_CTRL_FOLDEVTENA]);

  always @(posedge clk or negedge opt_reset_all_n)
    if (!opt_reset_all_n)
      opt_dwt_fold_count[7:0] <= {8{1'b0}};
    else if (fold_count_we & opt_trace_en)
      opt_dwt_fold_count[7:0] <= fold_count_nxt;

  assign dwt_fold_count[7:0] = opt_trace_en ? opt_dwt_fold_count[7:0] : {8{1'b0}};

  assign fold_count_inc = fold_event_en & dpu_etm_fold_i;
  assign fold_count_we  = register_we[6] | fold_count_init | fold_count_inc;
  assign fold_count_nxt = (register_we[6] ?
                           ppb_hwdata_i[7:0] :
                           ({8{~fold_count_init}} & (dwt_fold_count + 1'b1)));

  assign emit_fold      = fold_count_inc & (&dwt_fold_count);


  always @(dist_sync_tap_select or dwt_pc_cycle)
    case (dist_sync_tap_select)
      2'b00:   dist_sync_tap = 1'b0;
      2'b01:   dist_sync_tap = dwt_pc_cycle[`DWT_DSYNC_TAP_1];
      2'b10:   dist_sync_tap = dwt_pc_cycle[`DWT_DSYNC_TAP_2];
      2'b11:   dist_sync_tap = dwt_pc_cycle[`DWT_DSYNC_TAP_3];
      default: dist_sync_tap = 1'bx;
    endcase

  assign dpu_ahb_halted_we = (dpu_ahb_halted ^ dpu_ahb_halted_reg) & opt_trace_en;

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      dpu_ahb_halted_reg <= 1'b0;
    else if (dpu_ahb_halted_we)
      dpu_ahb_halted_reg <= dpu_ahb_halted;

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      dist_sync_tap_reg <= 1'b0;
    else if (cycle_count_we)
      dist_sync_tap_reg <= dist_sync_tap;

  assign dist_sync_nxt = (cycle_count_en & ~dwt_dist_sync &
                          ((dist_sync_tap_reg ^ dist_sync_tap) |
                           (|dist_sync_tap_select &
                            (dpu_ahb_halted & ~dpu_ahb_halted_reg))));

  assign dist_sync_tap_we = (opt_trace_en &
                             (cycle_count_we | dwt_dist_sync  |
                              dpu_ahb_halted_we));

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      dwt_dist_sync <= 1'b0;
    else if (dist_sync_tap_we)
      dwt_dist_sync <= dist_sync_nxt;

  always @(posedge clk or negedge opt_reset_all_n)
    if (!opt_reset_all_n)
      dwt_comp0[31:0] <= {32{1'b0}};
    else if (register_we[7])
      dwt_comp0[31:0] <= ppb_hwdata_i[31:0];

  always @(posedge clk or negedge opt_reset_all_n)
    if (!opt_reset_all_n)
      dwt_mask0[3:0] <= 4'b0000;
    else if (register_we[8])
      dwt_mask0[3:0] <= ppb_hwdata_i[3:0];

  always @(posedge clk or negedge reset_n)
    if (!reset_n)
      begin
        dwt_pc_sample_match0 <= 1'b0;
        dwt_emit_range0      <= 1'b0;
        dwt_function0[3:0]   <= {4{1'b0}};
      end
    else if (register_we[9])
      begin
        dwt_pc_sample_match0 <= ppb_hwdata_i[`DWT_FUNC_PSMATCH];
        dwt_emit_range0      <= ppb_hwdata_i[`DWT_FUNC_EMITRANGE];
        dwt_function0[3:0]   <= ppb_hwdata_i[`DWT_FUNC_FUNCTION + 3:
                                             `DWT_FUNC_FUNCTION];
      end

  assign matched0_clear = register_select[9] & ~ahb_write_reg & data_phase;
  assign matched0_set   = (emit_etm_trigger_i[0] | emit_watchpoint_i[0] |
                           emit_pc_sample_i[0]   | emit_data_sample_i[0]);
  assign matched0_we    = (matched0_clear | matched0_set) & opt_dwt_en;

  always @(posedge clk or negedge reset_n)
    if (!reset_n)
      dwt_matched0 <= 1'b0;
    else if (matched0_we)
      dwt_matched0 <= matched0_set;


  always @(posedge clk or negedge opt_reset_all_n)
    if (!opt_reset_all_n)
      opt_dwt_comp1[31:0] <= {32{1'b0}};
    else if(register_we[10])
      opt_dwt_comp1[31:0] <= ppb_hwdata_i[31:0];

  assign dwt_comp1 = opt_dwt_comp1 & {32{opt_full_wpoint_en}};

  always @(posedge clk or negedge opt_reset_all_n)
    if (!opt_reset_all_n)
      opt_dwt_mask1[3:0] <= 4'b0000;
    else if(register_we[11])
      opt_dwt_mask1[3:0] <= ppb_hwdata_i[3:0];

  assign dwt_mask1 = opt_dwt_mask1 & {4{opt_full_wpoint_en}};

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      begin
        opt_dwt_emit_range1    <= 1'b0;
        opt_dwt_function1[3:0] <= {4{1'b0}};
      end
    else if(register_we[12])
      begin
        opt_dwt_emit_range1    <= ppb_hwdata_i[`DWT_FUNC_EMITRANGE];
        opt_dwt_function1[3:0] <= ppb_hwdata_i[`DWT_FUNC_FUNCTION + 3:
                                               `DWT_FUNC_FUNCTION];
      end

  assign dwt_emit_range1 = opt_dwt_emit_range1 & opt_full_wpoint_en;
  assign dwt_function1   = opt_dwt_function1  & {4{opt_full_wpoint_en}};

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      begin
        opt_dwt_match_size1[1:0]     <= 2'b00;
        opt_dwt_data1_addr_src0[1:0] <= 2'b00;
        opt_dwt_data1_addr_src1[1:0] <= 2'b00;
        opt_dwt_match_on_data1       <= 1'b0;
      end
    else if(register_we[12] & opt_data_match_en)
      begin
        opt_dwt_match_size1[1:0]     <= ppb_hwdata_i[`DWT_FUNC_DATASIZE+
                                                     `DWT_FUNC_DATASIZE_SIZE-1:
                                                     `DWT_FUNC_DATASIZE];
        opt_dwt_data1_addr_src0[1:0] <= ppb_hwdata_i[`DWT_FUNC_DATASRC0+
                                                     `DWT_FUNC_DATASRC0_SIZE-1:
                                                     `DWT_FUNC_DATASRC0];
        opt_dwt_data1_addr_src1[1:0] <= ppb_hwdata_i[`DWT_FUNC_DATASRC1+
                                                     `DWT_FUNC_DATASRC1_SIZE-1:
                                                     `DWT_FUNC_DATASRC1];
        opt_dwt_match_on_data1      <= ppb_hwdata_i[`DWT_FUNC_DATAMATCH];
      end

  assign dwt_match_size1     = opt_dwt_match_size1     & {2{opt_data_match_en}};
  assign dwt_data1_addr_src0 = opt_dwt_data1_addr_src0 & {2{opt_data_match_en}};
  assign dwt_data1_addr_src1 = opt_dwt_data1_addr_src1 & {2{opt_data_match_en}};
  assign dwt_match_on_data1  = opt_dwt_match_on_data1  &    opt_data_match_en;

  assign matched1_clear = register_select[12] & ~ahb_write_reg & data_phase;
  assign matched1_set   = (emit_etm_trigger_i[1] | emit_watchpoint_i[1] |
                           emit_pc_sample_i[1]   | emit_data_sample_i[1]);
  assign matched1_we    = (matched1_clear | matched1_set) & opt_full_wpoint_en;

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      opt_dwt_matched1 <= 1'b0;
    else if (matched1_we)
      opt_dwt_matched1 <= matched1_set;

  assign dwt_matched1 = opt_dwt_matched1 & opt_full_wpoint_en;

  always @(posedge clk or negedge opt_reset_all_n)
    if (!opt_reset_all_n)
      opt_dwt_comp2[31:0] <= {32{1'b0}};
    else if(register_we[13])
      opt_dwt_comp2[31:0] <= ppb_hwdata_i[31:0];

  assign dwt_comp2 = opt_dwt_comp2 & {32{opt_full_wpoint_en}};

  always @(posedge clk or negedge opt_reset_all_n)
    if (!opt_reset_all_n)
      opt_dwt_mask2[3:0] <= 4'b0000;
    else if(register_we[14])
      opt_dwt_mask2[3:0] <= ppb_hwdata_i[3:0];

  assign dwt_mask2 = opt_dwt_mask2 & {4{opt_full_wpoint_en}};

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      begin
        opt_dwt_emit_range2    <= 1'b0;
        opt_dwt_function2[3:0] <= {4{1'b0}};
      end
    else if (register_we[15])
      begin
        opt_dwt_emit_range2    <= ppb_hwdata_i[`DWT_FUNC_EMITRANGE];
        opt_dwt_function2[3:0] <= ppb_hwdata_i[`DWT_FUNC_FUNCTION + 3:
                                               `DWT_FUNC_FUNCTION];
      end

  assign dwt_emit_range2 = opt_dwt_emit_range2 & opt_full_wpoint_en;
  assign dwt_function2   = opt_dwt_function2   & {4{opt_full_wpoint_en}};

  assign matched2_clear = register_select[15] & ~ahb_write_reg & data_phase;
  assign matched2_set   = (emit_etm_trigger_i[2] | emit_watchpoint_i[2] |
                           emit_pc_sample_i[2]   | emit_data_sample_i[2]);
  assign matched2_we    = (matched2_clear | matched2_set) & opt_full_wpoint_en;

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      opt_dwt_matched2 <= 1'b0;
    else if (matched2_we)
      opt_dwt_matched2 <= matched2_set;

  assign dwt_matched2 = opt_dwt_matched2 & opt_full_wpoint_en;

  always @(posedge clk or negedge opt_reset_all_n)
    if (!opt_reset_all_n)
      opt_dwt_comp3[31:0] <= {32{1'b0}};
    else if(register_we[16] & opt_full_wpoint_en)
      opt_dwt_comp3[31:0] <= ppb_hwdata_i[31:0];

  assign dwt_comp3 = opt_dwt_comp3 & {32{opt_full_wpoint_en}};

  always @(posedge clk or negedge opt_reset_all_n)
    if (!opt_reset_all_n)
      opt_dwt_mask3[3:0] <= 4'b0000;
    else if(register_we[17])
      opt_dwt_mask3[3:0] <= ppb_hwdata_i[3:0];

  assign dwt_mask3 = opt_dwt_mask3 & {4{opt_full_wpoint_en}};

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      begin
        opt_dwt_emit_range3    <= 1'b0;
        opt_dwt_function3[3:0] <= {4{1'b0}};
      end
    else if (register_we[18])
      begin
        opt_dwt_emit_range3    <= ppb_hwdata_i[`DWT_FUNC_EMITRANGE];
        opt_dwt_function3[3:0] <= ppb_hwdata_i[`DWT_FUNC_FUNCTION + 3:
                                               `DWT_FUNC_FUNCTION];
      end

  assign dwt_emit_range3 = opt_dwt_emit_range3 & opt_full_wpoint_en;
  assign dwt_function3   = opt_dwt_function3   & {4{opt_full_wpoint_en}};

  assign matched3_clear = register_select[18] & ~ahb_write_reg & data_phase;
  assign matched3_set   = (emit_etm_trigger_i[3] | emit_watchpoint_i[3] |
                           emit_pc_sample_i[3]   | emit_data_sample_i[3]);
  assign matched3_we    = (matched3_clear | matched3_set) & opt_full_wpoint_en;

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      opt_dwt_matched3 <= 1'b0;
    else if (matched3_we)
      opt_dwt_matched3 <= matched3_set;

  assign dwt_matched3 = opt_dwt_matched3 & opt_full_wpoint_en;


  assign comp1_good_data = ((((dwt_data1_addr_src0 == 2'b00) |
                              (dwt_data1_addr_src1 == 2'b00)) & qualify_data0_reg) |
                            (((dwt_data1_addr_src0 == 2'b01) |
                              (dwt_data1_addr_src1 == 2'b01)) & qualify_data1_reg & mpu_ahb_hreadyoutd_i) |
                            (((dwt_data1_addr_src0 == 2'b10) |
                              (dwt_data1_addr_src1 == 2'b10)) & qualify_data2_reg) |
                            (((dwt_data1_addr_src0 == 2'b11) |
                              (dwt_data1_addr_src1 == 2'b11)) & qualify_data3_reg));

  assign act_as_data_addr0 = (((dwt_data1_addr_src0 == 2'b00) |
                               (dwt_data1_addr_src1 == 2'b00)) & dwt_match_on_data1);

  assign act_as_data_addr2 = (((dwt_data1_addr_src0 == 2'b10) |
                               (dwt_data1_addr_src1 == 2'b10)) & dwt_match_on_data1);

  assign act_as_data_addr3 = (((dwt_data1_addr_src0 == 2'b11) |
                               (dwt_data1_addr_src1 == 2'b11)) & dwt_match_on_data1);


  assign clear_etm_ivalid  = ((dpu_etm_istall_i & dpu_etm_cancel_i) |
                              (etm_ivalid_reg & ~dpu_etm_istall_i));
  assign etm_ivalid_reg_we = dpu_etm_ivalid_i | clear_etm_ivalid;

  always @(posedge clk or negedge reset_n)
    if (!reset_n)
      etm_ivalid_reg <= 1'b0;
    else if (etm_ivalid_reg_we & opt_dwt_en)
      etm_ivalid_reg <= dpu_etm_ivalid_i;

  assign allow_pc_match = etm_ivalid_reg & ~dpu_etm_istall_i;

  assign pc_address = {dpu_etm_ia_i, 1'b0};

  assign pc_sample_value = {32{dpu_ahb_halted}} | pc_address;

  assign address_phase    = (mpu_ahb_hreadyoutd_i & dpu_ahb_haddraccd_i &
                             dpu_ahb_htransd1_i & ~mpu_canceld_i);
  assign qualify_data0_we = (opt_data_match_en & dpu_trace_enable_i &
                             (address_phase |
                              (opt_qualify_data0_reg & mpu_ahb_hreadyoutd_i)));

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      opt_qualify_data0_reg <= 1'b0;
    else if (qualify_data0_we)
      opt_qualify_data0_reg <= qualify_data0_i;

  assign qualify_data1_we = (dpu_trace_enable_i & opt_data_match_en &
                             (address_phase |
                              (opt_qualify_data1_reg & mpu_ahb_hreadyoutd_i)));

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      opt_qualify_data1_reg <= 1'b0;
    else if (qualify_data1_we)
      opt_qualify_data1_reg <= address_phase;

  assign qualify_data2_we = (opt_data_match_en & dpu_trace_enable_i &
                             (address_phase |
                              (opt_qualify_data2_reg & mpu_ahb_hreadyoutd_i)));

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      opt_qualify_data2_reg <= 1'b0;
    else if (qualify_data2_we)
      opt_qualify_data2_reg <= qualify_data2_i;

  assign qualify_data3_we = (dpu_trace_enable_i & opt_data_match_en &
                             (address_phase |
                              (opt_qualify_data3_reg & mpu_ahb_hreadyoutd_i)));

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      opt_qualify_data3_reg <= 1'b0;
    else if (qualify_data3_we)
      opt_qualify_data3_reg <= qualify_data3_i;

  assign qualify_data0_reg = opt_qualify_data0_reg & opt_data_match_en;
  assign qualify_data1_reg = opt_qualify_data1_reg & opt_data_match_en;
  assign qualify_data2_reg = opt_qualify_data2_reg & opt_data_match_en;
  assign qualify_data3_reg = opt_qualify_data3_reg & opt_data_match_en;


  assign watch_point      = (|emit_watchpoint_i[3:0]);

  assign trans_control_we = (dpu_trace_enable_i & address_phase &
                             (opt_data_match_en | opt_trace_en));

  always @(posedge clk or negedge opt_reset_all_n)
    if (!opt_reset_all_n)
      begin
        dpu_ahb_hwrite_reg <= 1'b0;
        dpu_ahb_hsize_reg  <= 2'b00;
        dpu_ahb_haddr_reg  <= 2'b00;
      end
    else if(trans_control_we)
      begin
        dpu_ahb_hwrite_reg <= dpu_ahb_hwrited_i;
        dpu_ahb_hsize_reg  <= dpu_ahb_hsized_i;
        dpu_ahb_haddr_reg  <= dpu_ahb_haddrd_i[1:0];
      end

  assign watchpoint_data = (({32{ dpu_ahb_hwrite_reg & dpu_trace_enable_i}} & dpu_ahb_hwdatad_i) |
                            ({32{~dpu_ahb_hwrite_reg & dpu_trace_enable_i}} & dpu_ahb_hrdatad_i));

  assign dpu_ahb_haddrd_masked = dpu_ahb_haddrd_i & {32{dpu_trace_enable_i}};

  always @(posedge clk or negedge reset_n)
    if(!reset_n)
      watch_point_reg <= 1'b0;
    else if (opt_dwt_en)
      watch_point_reg <= watch_point;

`ifdef ARM_DSM
  assign dwt_watchpoint_o         = (dsm_debug_lvl > 0) ? watch_point           : 1'b0;
  assign dwt_observe_watchpoint_o = (dsm_debug_lvl > 0 &
                                     dsm_observation != 0) ? watch_point_reg    : 1'b0;
  assign comp1_good_data_o        = (dsm_debug_lvl > 0) ? comp1_good_data       : 1'b0;
  assign dpu_ahb_hwrite_reg_o     = (dsm_debug_lvl > 0) ? dpu_ahb_hwrite_reg    : 1'b0;
  assign dpu_ahb_haddr_reg_o      = (dsm_debug_lvl > 0) ? dpu_ahb_haddr_reg     : 2'b00;
  assign dpu_ahb_haddrd_masked_o  = (dsm_debug_lvl > 0) ? dpu_ahb_haddrd_masked : {32{1'b0}};
  assign dpu_ahb_hsize_reg_o      = (dsm_debug_lvl > 0) ? dpu_ahb_hsize_reg     : 2'b00;
  assign dwt_dist_sync_o          = (dsm_debug_lvl > 0) ? dwt_dist_sync         : 1'b0;
  assign ppb_hrdata_o             = (dsm_debug_lvl > 0) ? opt_hrdata            : {32{1'b0}};
  assign address_phase_o          = (dsm_debug_lvl > 0) ? address_phase         : 1'b0;
  assign clear_etm_ivalid_o       = (dsm_debug_lvl > 0) ? clear_etm_ivalid      : 1'b0;
  assign allow_pc_match_o         = (dsm_debug_lvl > 0) ? allow_pc_match        : 1'b0;
  assign dwt_pc_cycle_o           = (dsm_debug_lvl > 0) ? dwt_pc_cycle          : {32{1'b0}};
  assign dwt_pc_sample_match0_o   = (dsm_debug_lvl > 0) ? dwt_pc_sample_match0  : 1'b0;
  assign act_as_data_addr0_o      = (dsm_debug_lvl > 0) ? act_as_data_addr0     : 1'b0;
  assign dwt_emit_range0_o        = (dsm_debug_lvl > 0) ? dwt_emit_range0       : 1'b0;
  assign dwt_mask0_o              = (dsm_debug_lvl > 0) ? dwt_mask0             : {4{1'b0}};
  assign dwt_function0_o          = (dsm_debug_lvl > 0) ? dwt_function0         : {4{1'b0}};
  assign dwt_comp0_o              = (dsm_debug_lvl > 0) ? dwt_comp0             : {32{1'b0}};
  assign dwt_match_on_data1_o     = (dsm_debug_lvl > 0) ? dwt_match_on_data1    : 1'b0;
  assign dwt_match_size1_o        = (dsm_debug_lvl > 0) ? dwt_match_size1       : 2'b00;
  assign dwt_emit_range1_o        = (dsm_debug_lvl > 0) ? dwt_emit_range1       : 1'b0;
  assign dwt_mask1_o              = (dsm_debug_lvl > 0) ? dwt_mask1             : {4{1'b0}};
  assign dwt_function1_o          = (dsm_debug_lvl > 0) ? dwt_function1         : {4{1'b0}};
  assign dwt_comp1_o              = (dsm_debug_lvl > 0) ? dwt_comp1             : {32{1'b0}};
  assign act_as_data_addr2_o      = (dsm_debug_lvl > 0) ? act_as_data_addr2     : 1'b0;
  assign dwt_emit_range2_o        = (dsm_debug_lvl > 0) ? dwt_emit_range2       : 1'b0;
  assign dwt_mask2_o              = (dsm_debug_lvl > 0) ? dwt_mask2             : {4{1'b0}};
  assign dwt_function2_o          = (dsm_debug_lvl > 0) ? dwt_function2         : {4{1'b0}};
  assign dwt_comp2_o              = (dsm_debug_lvl > 0) ? dwt_comp2             : {32{1'b0}};
  assign act_as_data_addr3_o      = (dsm_debug_lvl > 0) ? act_as_data_addr3     : 1'b0;
  assign dwt_emit_range3_o        = (dsm_debug_lvl > 0) ? dwt_emit_range3       : 1'b0;
  assign dwt_mask3_o              = (dsm_debug_lvl > 0) ? dwt_mask3             : {4{1'b0}};
  assign dwt_function3_o          = (dsm_debug_lvl > 0) ? dwt_function3         : {4{1'b0}};
  assign dwt_comp3_o              = (dsm_debug_lvl > 0) ? dwt_comp3             : {32{1'b0}};
  assign sleep_state_o            = (dsm_debug_lvl > 0) ? sleep_state           : 1'b0;
  assign emit_pc_periodic_o       = (dsm_debug_lvl > 0) ? emit_pc_periodic      : 1'b0;
  assign emit_cyc_o               = (dsm_debug_lvl > 0) ? emit_cyc              : 1'b0;
  assign emit_cpi_o               = (dsm_debug_lvl > 0) ? emit_cpi              : 1'b0;
  assign emit_int_o               = (dsm_debug_lvl > 0) ? emit_int              : 1'b0;
  assign emit_sleep_o             = (dsm_debug_lvl > 0) ? emit_sleep            : 1'b0;
  assign emit_fold_o              = (dsm_debug_lvl > 0) ? emit_fold             : 1'b0;
  assign emit_lsu_o               = (dsm_debug_lvl > 0) ? emit_lsu              : 1'b0;
  assign int_trace_en_o           = (dsm_debug_lvl > 0) ? int_trace_en          : 1'b0;
  assign watchpoint_data_o        = (dsm_debug_lvl > 0) ? watchpoint_data       : {32{1'b0}};
`else
  assign dwt_watchpoint_o         = (DEBUG_LVL > 0)  ? watch_point           : 1'b0;
  assign dwt_observe_watchpoint_o = (DEBUG_LVL > 0 &
                                     OBSERVATION != 0) ? watch_point_reg     : 1'b0;
  assign comp1_good_data_o        = (DEBUG_LVL > 0)  ? comp1_good_data       : 1'b0;
  assign dpu_ahb_hwrite_reg_o     = (DEBUG_LVL > 0)  ? dpu_ahb_hwrite_reg    : 1'b0;
  assign dpu_ahb_haddr_reg_o      = (DEBUG_LVL > 0)  ? dpu_ahb_haddr_reg     : 2'b00;
  assign dpu_ahb_haddrd_masked_o  = (DEBUG_LVL > 0)  ? dpu_ahb_haddrd_masked : {32{1'b0}};
  assign dpu_ahb_hsize_reg_o      = (DEBUG_LVL > 0)  ? dpu_ahb_hsize_reg     : 2'b00;
  assign dwt_dist_sync_o          = (DEBUG_LVL > 0)  ? dwt_dist_sync         : 1'b0;
  assign ppb_hrdata_o             = (DEBUG_LVL > 0)  ? opt_hrdata            : {32{1'b0}};
  assign address_phase_o          = (DEBUG_LVL > 0)  ? address_phase         : 1'b0;
  assign clear_etm_ivalid_o       = (DEBUG_LVL > 0)  ? clear_etm_ivalid      : 1'b0;
  assign allow_pc_match_o         = (DEBUG_LVL > 0)  ? allow_pc_match        : 1'b0;
  assign dwt_pc_cycle_o           = (DEBUG_LVL > 0)  ? dwt_pc_cycle          : {32{1'b0}};
  assign dwt_pc_sample_match0_o   = (DEBUG_LVL > 0)  ? dwt_pc_sample_match0  : 1'b0;
  assign act_as_data_addr0_o      = (DEBUG_LVL > 0)  ? act_as_data_addr0     : 1'b0;
  assign dwt_emit_range0_o        = (DEBUG_LVL > 0)  ? dwt_emit_range0       : 1'b0;
  assign dwt_mask0_o              = (DEBUG_LVL > 0)  ? dwt_mask0             : {4{1'b0}};
  assign dwt_function0_o          = (DEBUG_LVL > 0)  ? dwt_function0         : {4{1'b0}};
  assign dwt_comp0_o              = (DEBUG_LVL > 0)  ? dwt_comp0             : {32{1'b0}};
  assign dwt_match_on_data1_o     = (DEBUG_LVL > 0)  ? dwt_match_on_data1    : 1'b0;
  assign dwt_match_size1_o        = (DEBUG_LVL > 0)  ? dwt_match_size1       : 2'b00;
  assign dwt_emit_range1_o        = (DEBUG_LVL > 0)  ? dwt_emit_range1       : 1'b0;
  assign dwt_mask1_o              = (DEBUG_LVL > 0)  ? dwt_mask1             : {4{1'b0}};
  assign dwt_function1_o          = (DEBUG_LVL > 0)  ? dwt_function1         : {4{1'b0}};
  assign dwt_comp1_o              = (DEBUG_LVL > 0)  ? dwt_comp1             : {32{1'b0}};
  assign act_as_data_addr2_o      = (DEBUG_LVL > 0)  ? act_as_data_addr2     : 1'b0;
  assign dwt_emit_range2_o        = (DEBUG_LVL > 0)  ? dwt_emit_range2       : 1'b0;
  assign dwt_mask2_o              = (DEBUG_LVL > 0)  ? dwt_mask2             : {4{1'b0}};
  assign dwt_function2_o          = (DEBUG_LVL > 0)  ? dwt_function2         : {4{1'b0}};
  assign dwt_comp2_o              = (DEBUG_LVL > 0)  ? dwt_comp2             : {32{1'b0}};
  assign act_as_data_addr3_o      = (DEBUG_LVL > 0)  ? act_as_data_addr3     : 1'b0;
  assign dwt_emit_range3_o        = (DEBUG_LVL > 0)  ? dwt_emit_range3       : 1'b0;
  assign dwt_mask3_o              = (DEBUG_LVL > 0)  ? dwt_mask3             : {4{1'b0}};
  assign dwt_function3_o          = (DEBUG_LVL > 0)  ? dwt_function3         : {4{1'b0}};
  assign dwt_comp3_o              = (DEBUG_LVL > 0)  ? dwt_comp3             : {32{1'b0}};
  assign sleep_state_o            = (DEBUG_LVL > 0)  ? sleep_state           : 1'b0;
  assign emit_pc_periodic_o       = (DEBUG_LVL > 0)  ? emit_pc_periodic      : 1'b0;
  assign emit_cyc_o               = (DEBUG_LVL > 0)  ? emit_cyc              : 1'b0;
  assign emit_cpi_o               = (DEBUG_LVL > 0)  ? emit_cpi              : 1'b0;
  assign emit_int_o               = (DEBUG_LVL > 0)  ? emit_int              : 1'b0;
  assign emit_sleep_o             = (DEBUG_LVL > 0)  ? emit_sleep            : 1'b0;
  assign emit_fold_o              = (DEBUG_LVL > 0)  ? emit_fold             : 1'b0;
  assign emit_lsu_o               = (DEBUG_LVL > 0)  ? emit_lsu              : 1'b0;
  assign int_trace_en_o           = (DEBUG_LVL > 0)  ? int_trace_en          : 1'b0;
  assign watchpoint_data_o        = (DEBUG_LVL > 0)  ? watchpoint_data       : {32{1'b0}};
`endif


`ifdef ARM_ASSERT_ON

  reg    ovl_count_cpi;
  reg    ovl_count_int;
  reg    ovl_count_lsu;
  reg    ovl_count_slp;
  reg    ovl_cycle_count_init_reg;

  always @(posedge clk or negedge reset_n)
  if( ~reset_n )
  begin
    ovl_count_cpi <= 1'b0;
    ovl_count_int <= 1'b0;
    ovl_count_lsu <= 1'b0;
    ovl_count_slp <= 1'b0;
    ovl_cycle_count_init_reg <= 1'b0;
  end
  else
    begin
      ovl_count_cpi <= cpi_count_inc;
      ovl_count_int <= int_count_inc;
      ovl_count_lsu <= lsu_count_inc;
      ovl_count_slp <= sleep_count_inc;
      ovl_cycle_count_init_reg <= cycle_count_init;
    end

  assert_zero_one_hot #(0,19,0,"register_select must be one-hot")
  onehot_dwtdec
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr (register_select[18:0])
     );

  assert_zero_one_hot #(0,4,0,"Counter enables must be one-hot")
  onehot_counter_en
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr ({ovl_count_cpi,ovl_count_int,ovl_count_lsu,ovl_count_slp})
     );

  wire   ovl_nxt_cpi_count_active;
  wire   ovl_nxt_int_count_active;
  wire   ovl_nxt_slp_count_active;
  wire   ovl_nxt_lsu_count_active;
  wire   ovl_nxt_fold_count_active;
  reg    ovl_cpi_count_active;
  reg    ovl_int_count_active;
  reg    ovl_sleep_count_active;
  reg    ovl_lsu_count_active;
  reg    ovl_fold_count_active;
  wire   ovl_cpi_count_expect_0;
  wire   ovl_int_count_expect_0;
  wire   ovl_sleep_count_expect_0;
  wire   ovl_lsu_count_expect_0;
  wire   ovl_fold_count_expect_0;
  reg    ovl_en_init_reg0;
  reg    ovl_en_init_reg1;

  always @(posedge clk or negedge reset_n)
    if (!reset_n)
      begin
        ovl_en_init_reg0 <= 1'b0;
        ovl_en_init_reg1 <= 1'b0;
      end
    else
      begin
        ovl_en_init_reg0 <= cycle_count_en;
        ovl_en_init_reg1 <= ovl_en_init_reg0;
      end

  assign ovl_nxt_cpi_count_active  = (cpi_count_we   ?
                                      (cpi_event_en |
                                       (register_we[2] & (|ppb_hwdata_i[7:0]))) :
                                      ovl_cpi_count_active);
  assign ovl_nxt_int_count_active  = (int_count_we   ?
                                      (int_event_en |
                                       (register_we[3] & (|ppb_hwdata_i[7:0]))) :
                                      ovl_int_count_active);
  assign ovl_nxt_slp_count_active  = (sleep_count_we ?
                                      (sleep_event_en |
                                       (register_we[4] & (|ppb_hwdata_i[7:0]))) :
                                      ovl_sleep_count_active);
  assign ovl_nxt_lsu_count_active  = (lsu_count_we   ?
                                      (lsu_event_en |
                                       (register_we[5] & (|ppb_hwdata_i[7:0]))) :
                                      ovl_lsu_count_active);
  assign ovl_nxt_fold_count_active = (fold_count_we  ?
                                      (fold_event_en |
                                       (register_we[6] & (|ppb_hwdata_i[7:0]))) :
                                      ovl_fold_count_active);

  always @(posedge clk or negedge reset_n)
    if( ~reset_n )
      begin
        ovl_cpi_count_active   <= 1'b0;
        ovl_int_count_active   <= 1'b0;
        ovl_sleep_count_active <= 1'b0;
        ovl_lsu_count_active   <= 1'b0;
        ovl_fold_count_active  <= 1'b0;
      end
    else
      begin
        ovl_cpi_count_active   <= ovl_nxt_cpi_count_active;
        ovl_int_count_active   <= ovl_nxt_int_count_active;
        ovl_sleep_count_active <= ovl_nxt_slp_count_active;
        ovl_lsu_count_active   <= ovl_nxt_lsu_count_active;
        ovl_fold_count_active  <= ovl_nxt_fold_count_active;
      end

  assign ovl_cpi_count_expect_0   = cpi_event_en   & ~ovl_cpi_count_active;
  assign ovl_int_count_expect_0   = int_event_en   & ~ovl_int_count_active;
  assign ovl_sleep_count_expect_0 = sleep_event_en & ~ovl_sleep_count_active;
  assign ovl_lsu_count_expect_0   = lsu_event_en   & ~ovl_lsu_count_active;
  assign ovl_fold_count_expect_0  = fold_event_en  & ~ovl_fold_count_active;


  assert_never #(0,0,"PSPOSTCNT counter is initialised unexpectedly")
    ovl_pspostcnt_init
      (
       .clk       (clk),
       .reset_n   (reset_n),
       .test_expr (scaled_count_init & (pc_event_en |
                            (~ppb_hwdata_i[`DWT_CTRL_PCSAMPLEENA] & cycle_event_en)))
       );

  assert_never #(0,0,
                 "emit_pc_periodic should never occur when pc_event_en not set")
  ovl_pcsample_event_disabled
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr (!pc_event_en & emit_pc_periodic)
     );

  assert_next #(0,1,1,0,0,
                  "emit_pc_periodic should be a single cycle pulse")
  ovl_emit_pc_pulse
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .start_event (emit_pc_periodic & ovl_en_init_reg1 & ~ovl_cycle_count_init_reg &
                   ~register_we[0]),
     .test_expr (!(emit_pc_periodic & ~ovl_cycle_count_init_reg))
     );

  assert_never #(0,0,
                 "emit_cyc should never occur when PC Sampling is enabled")
  ovl_emit_cyc_disabled1
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr (pc_event_en & emit_cyc)
     );

  assert_never #(0,0,
                 "emit_cyc should never occur when cycle_event_en is not set")
  ovl_emit_cyc_disabled2
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr (!cycle_event_en & emit_cyc)
     );

  assert_never_unknown #(0,32,0,"PSCYCCNT counter failed to initialise")
  ovl_pscyccnt_uninitialised
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .qualifier (cycle_count_we),
     .test_expr (cycle_count_nxt)
     );

  assert_never_unknown #(0,4,0,"PSPOSTCNT counter failed to initialise")
  ovl_pspostcnt_uninitialised
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .qualifier (scaled_count_we),
     .test_expr (scaled_count_nxt)
     );


  assert_never #(0,0,"ovl_cpi_count counter is initialised unexpectedly")
  ovl_cpi_count_init
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr (cpi_count_init & (ppb_hwdata_i[17] & cpi_event_en))
     );

  assert_never #(0,0,
                 "emit_cpi should never occur when CPIEVTENA is not set")
  ovl_cpi_event_disabled
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr (!cpi_event_en & emit_cpi)
     );

  assert_next #(0,1,1,0,0,
                  "emit_cpi should be a single cycle pulse")
  ovl_emit_cpi_pulse
    (
     .clk         (clk),
     .reset_n     (reset_n),
     .start_event (emit_cpi & ~register_we[2]),
     .test_expr   (!emit_cpi)
     );

  assert_never_unknown #(0,8,0,"ovl_cpi_count counter failed to initialise")
  ovl_cpi_count_uninitialised
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .qualifier (cpi_count_we),
     .test_expr (cpi_count_nxt)
     );

  assert_never #(0,0,"ovl_cpi_count not zero after initialisation")
  ovl_cpi_count_not_zeroed
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr (ovl_cpi_count_expect_0 & |dwt_cpi_count)
     );


  assert_never #(0,0,"ovl_int_count counter is initialised unexpectedly")
  ovl_int_count_init
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr (int_count_init & (int_event_en))
     );

  assert_never #(0,0,
                 "emit_int should never occur when INTEVTENA is not set")
  ovl_int_event_disabled
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr (!int_event_en & emit_int)
     );

  assert_next #(0,1,1,0,0,
                  "emit_int should be a single cycle pulse")
  ovl_emit_int_pulse
    (
     .clk         (clk),
     .reset_n     (reset_n),
     .start_event (emit_int & ~register_we[3]),
     .test_expr   (!emit_int)
     );

  assert_never_unknown #(0,8,0,"ovl_int_count counter failed to initialise")
  ovl_int_count_uninitialised
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .qualifier (int_count_we),
     .test_expr (int_count_nxt)
     );

  assert_never #(0,0,"ovl_int_count not zero after initialisation")
  ovl_int_count_not_zeroed
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr (ovl_int_count_expect_0 & (|dwt_int_count))
     );


  assert_never #(0,0,"ovl_sleep_count counter is initialised unexpectedly")
  ovl_sleep_count_init
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr (sleep_count_init & (sleep_event_en))
     );

  assert_never #(0,0,
                 "emit_sleep should never occur when SLEEPEVTENA not set")
  ovl_sleep_event_disabled
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr (!sleep_event_en & emit_sleep)
     );

  assert_next #(0,1,1,0,0,
                  "emit_sleep should be a single cycle pulse")
  ovl_emit_sleep_pulse
    (
     .clk         (clk),
     .reset_n     (reset_n),
     .start_event (emit_sleep & ~register_we[4]),
     .test_expr   (!emit_sleep)
     );

  assert_never_unknown #(0,8,0,"ovl_sleep_count counter failed to initialise")
  ovl_sleep_count_uninitialised
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .qualifier (sleep_count_we),
     .test_expr (sleep_count_nxt)
     );

  assert_never #(0,0,"ovl_sleep_count not zero after initialisation")
  ovl_sleep_count_not_zeroed
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr (ovl_sleep_count_expect_0 & |dwt_sleep_count)
     );




  assert_never_unknown #(0,8,0,"ovl_lsu_count counter failed to initialise")
  ovl_lsu_count_uninitialised
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .qualifier (lsu_count_we & opt_trace_en),
     .test_expr (lsu_count_nxt)
     );

  assert_never #(0,0,"ovl_lsu_count not zero after initialisation")
  ovl_lsu_count_not_zeroed
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr (ovl_lsu_count_expect_0 & |dwt_lsu_count)
     );


  assert_never_unknown #(0,8,0,"ovl_fold_count counter failed to initialise")
  ovl_fold_count_uninitialised
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .qualifier (fold_count_we & opt_trace_en),
     .test_expr (fold_count_nxt)
     );

  assert_never #(0,0,"ovl_fold_count not zero after initialisation")
  ovl_fold_count_not_zeroed
    (
     .clk       (clk),
     .reset_n   (reset_n),
     .test_expr (ovl_fold_count_expect_0 & (|dwt_fold_count))
     );


  assert_never #(0,0,"DWT comparators 1 to 3 cannot match if DEBUG_LVL == 1")
    ovl_cfg_comp1to3
     (
      .clk       (clk),
      .reset_n   (reset_n),
      .test_expr ((DEBUG_LVL == 1) &
                  ((|emit_etm_trigger_i[3:1]) |
                   (|emit_watchpoint_i[3:1]) |
                   (|emit_pc_sample_i[3:1]) |
                   (|emit_data_sample_i[3:1])))
      );

  assert_never #(0,0,"DWT counters cannot be non-zero if TRACE_LVL == 0")
    ovl_cfg_counters
      (
       .clk       (clk),
       .reset_n   (reset_n),
       .test_expr (~opt_trace_en &
                   ((|dwt_pc_cycle)  |
                    (|dwt_pc_cycle_scale) |
                    (|dwt_lsu_count)    |
                    (|dwt_fold_count)   |
                    (|dwt_cpi_count)    |
                    (|dwt_int_count)    |
                    (|dwt_sleep_count)))
       );

`endif

endmodule
`include "cm3_dwt_undefs.vp"
