`include "cm3_dwt_defs.vp"

module cm3_dwt_packet_state
  (// Inputs
   clk, reset_n, event_valid_i, granted_i, itm_dwt_ready_i,
   sampler_status_o
   );

  parameter    TRACE_LVL = 1;


  input         clk;                // Clock
  input         reset_n;            // Reset
  input         event_valid_i;      // Emit
  input         granted_i;          // granted_i
  input         itm_dwt_ready_i;    // Ready from ITM

  output [1:0]  sampler_status_o;   // Sample Status


  wire          ready;              // Ready to emit event
  wire          sampler_status_we;  // Enable sampler_status_o register

  reg  [1:0]    nxt_sampler_status; // Sample state machine - next

  reg  [1:0]    sampler_status;     // Sample state machine
  wire          opt_en;             // Logic removal term

`ifdef ARM_DSM

  wire [1:0]    dsm_trace_lvl;

  assign opt_en = (dsm_trace_lvl > 0) ? 1'b1           : 1'b0;

`else

  assign opt_en = (TRACE_LVL > 0)     ? 1'b1           : 1'b0;

`endif


  assign ready = itm_dwt_ready_i & granted_i;

  always @(sampler_status or event_valid_i or ready)
    begin
      case(sampler_status)
        `ES_INVALID:
          nxt_sampler_status  = event_valid_i ? `ES_VALID   : `ES_INVALID;
        `ES_VALID:
          if(event_valid_i)
            nxt_sampler_status = ready        ? `ES_VALID   : `ES_OFLOW;
          else
            nxt_sampler_status = ready        ? `ES_INVALID : `ES_VALID;
        `ES_OFLOW:
          if(event_valid_i)
            nxt_sampler_status = ready        ? `ES_VALID   : `ES_OFLOW;
          else
            nxt_sampler_status = ready        ? `ES_INVALID : `ES_OFLOW;
        `ES_UNUSED:
          nxt_sampler_status   = `ES_INVALID;
        default:
          nxt_sampler_status   = 2'bxx;
      endcase
    end

  assign sampler_status_we = (opt_en &
                              ((sampler_status != `ES_INVALID) |
                               event_valid_i));

  always @(posedge clk or negedge reset_n)
    if(reset_n == 1'b0)
      sampler_status <= 2'b00;
    else if(sampler_status_we)
      sampler_status <= nxt_sampler_status;

`ifdef ARM_DSM
  assign sampler_status_o = (dsm_trace_lvl > 0) ? sampler_status : 2'b00;
`else
  assign sampler_status_o = (TRACE_LVL > 0)     ? sampler_status : 2'b00;
`endif

`ifdef ARM_ASSERT_ON

  assert_never_unknown #(0,2,0,"Unknown sample status")
  ovl_sampler_status_not_x_or_z
    (
     .clk       (clk),
     .reset_n    (reset_n),
     .qualifier (1'b1),
     .test_expr (sampler_status)
     );

  assert_never #(0,0,"Invalid sample status")
  ovl_sampler_status_o_not_unused
    (
     .clk       (clk),
     .reset_n    (reset_n),
     .test_expr (sampler_status == `ES_UNUSED)
     );

`endif

endmodule
`include "cm3_dwt_undefs.vp"

