module cm3_etm_apb_if
  (
   reset_n, clk, apb_sel_i, apb_enable_i, apb_write_i,
   apb_addr_i,
   register_write_o, apb_addr_reg_o
   );

  parameter TRACE_LVL = 1;
  parameter RESET_ALL_REGS = 0;

  input         reset_n;           // Core clock domain reset
  input         clk;               // Core clock (not gated by etm)
  input         apb_sel_i;         // APB Select
  input         apb_enable_i;      // APB Enable
  input         apb_write_i;       // APB Write
  input [11:2]  apb_addr_i;        // APB address

  output        register_write_o;  // Main Register Write Signal
  output [11:2] apb_addr_reg_o;    // APB address registered


  reg    [11:2] apb_addr_reg;      // APB address register

  wire          opt_etm_en;        // Logic removal enable term
  wire          nxt_register_write;// APB write transfer
  reg           register_write;    // APB write transfer
  wire          opt_reset_n;       // optional reset all registers
  wire          opt_apb_sel;       // Enabled select if component present

`ifdef ARM_DSM
  wire   dsm_reset_all_regs;
  assign opt_reset_n = dsm_reset_all_regs    ? reset_n : 1'b1;
`else
  assign opt_reset_n = (RESET_ALL_REGS == 1) ? reset_n : 1'b1;
`endif

  assign opt_etm_en = (TRACE_LVL > 1) ? 1'b1          : 1'b0;

  assign nxt_register_write = ~apb_enable_i & apb_sel_i & apb_write_i;
  assign opt_apb_sel = apb_sel_i & opt_etm_en;

  always @(posedge clk or negedge opt_reset_n)
    if (!opt_reset_n)
      apb_addr_reg <= {10{1'b1}};
    else if (opt_apb_sel)
        apb_addr_reg <= apb_addr_i;

  always @(posedge clk or negedge reset_n)
    if (!reset_n)
      register_write <= 1'b0;
    else if (opt_apb_sel)
      register_write <= nxt_register_write;

  assign register_write_o = (TRACE_LVL > 1) ? register_write : 1'b0;
  assign apb_addr_reg_o   = (TRACE_LVL > 1) ? apb_addr_reg   : {10{1'b0}};
`ifdef ARM_ASSERT_ON

  assert_never_unknown  #(0,1,0,"PSEL input is x")
  ovl_etm_psel_x(
    .clk       (clk),
    .reset_n   (reset_n),
    .qualifier (1'b1),
    .test_expr (opt_apb_sel));

`endif
endmodule
