module cm3_etm_clk_gate
  (// Inputs
   clk, clk_enable_i, global_disable_i,
   clk_gated_o
  );


  parameter CLKGATE_PRESENT = 0;  // Clock gating present if != 0



  input  clk;
  input  clk_enable_i;
  input  global_disable_i;

  output clk_gated_o;



  wire   clk_en;
  reg    clk_en_latch;
  wire   clk_gated;

  wire   opt_clk_gated;
  wire   opt_mst_clk_en;
  wire   opt_mst_disable;
  wire   opt_clk;



  assign opt_clk_gated   = (CLKGATE_PRESENT != 0) ? clk_gated        : 1'b0;
  assign opt_mst_clk_en  = (CLKGATE_PRESENT != 0) ? clk_enable_i     : 1'b0;
  assign opt_mst_disable = (CLKGATE_PRESENT != 0) ? global_disable_i : 1'b0;
  assign opt_clk         = (CLKGATE_PRESENT != 0) ? clk              : 1'b0;



  assign clk_en = opt_mst_clk_en | opt_mst_disable;

  always @(opt_clk or clk_en)
    if (opt_clk == 1'b0)
      clk_en_latch <= clk_en;

  assign clk_gated = opt_clk & clk_en_latch;

  assign clk_gated_o = opt_clk_gated;

endmodule
