module cm3_etm_event_gen
  (
   mux_in_i,
   mux_sel_i,

   event_o
   );

  parameter TRACE_LVL = 1;


  input [7:0]   mux_in_i;          // resource vector
  input [10:0]  mux_sel_i;         // Event encoding

  output        event_o;           // resulting output of Event Gen Assy

  wire [3:0]    mux_sel_a;         // Resource A select
  wire [3:0]    mux_sel_b;         // Resource B select
  wire [2:0]    bool_fn;           // Boolean function

  wire          opt_event;         // resulting output of Event Gen Assy
  wire          mux_out_a;         // mux A output
  wire          mux_out_b;         // mux B output

  wire [8:0]    mux_filt_a;
  wire [8:0]    mux_filt_b;

  assign        mux_sel_a = mux_sel_i[3:0];
  assign        mux_sel_b = mux_sel_i[7:4];
  assign        bool_fn   = mux_sel_i[10:8];

  assign mux_filt_a = {
                       (mux_sel_a == 4'b11_11), // HardWired
                       (mux_sel_a == 4'b11_01), // ExtIn[1]
                       (mux_sel_a == 4'b11_00), // ExtIn[0]
                       (mux_sel_a == 4'b10_11), // Start/Stop
                       (mux_sel_a == 4'b10_00), // Counter at zero
                       (mux_sel_a == 4'b01_11), // Embedded Ice
                       (mux_sel_a == 4'b01_10), // Embedded Ice
                       (mux_sel_a == 4'b01_01), // Embedded Ice
                       (mux_sel_a == 4'b01_00)};// Embedded Ice

  assign mux_out_a = |({1'b1,mux_in_i} & mux_filt_a);

  assign mux_filt_b = {
                       (mux_sel_b == 4'b11_11), // HardWired
                       (mux_sel_b == 4'b11_01), // ExtIn[1]
                       (mux_sel_b == 4'b11_00), // ExtIn[0]
                       (mux_sel_b == 4'b10_11), // Start/Stop
                       (mux_sel_b == 4'b10_00), // Counter at zero
                       (mux_sel_b == 4'b01_11), // Embedded Ice
                       (mux_sel_b == 4'b01_10), // Embedded Ice
                       (mux_sel_b == 4'b01_01), // Embedded Ice
                       (mux_sel_b == 4'b01_00)};// Embedded Ice

  assign mux_out_b = |({1'b1,mux_in_i} & mux_filt_b);

  assign opt_event = (((bool_fn == 3'b000) &   mux_out_a) |
                        ((bool_fn == 3'b001) &  ~mux_out_a) |
                        ((bool_fn == 3'b010) & ( mux_out_a &  mux_out_b)) |
                        ((bool_fn == 3'b011) & (~mux_out_a &  mux_out_b)) |
                        ((bool_fn == 3'b100) & (~mux_out_a & ~mux_out_b)) |
                        ((bool_fn == 3'b101) & ( mux_out_a |  mux_out_b)) |
                        ((bool_fn == 3'b110) & (~mux_out_a |  mux_out_b)) |
                        ((bool_fn == 3'b111) & (~mux_out_a | ~mux_out_b)));

  assign event_o = (TRACE_LVL > 1) ? opt_event : 1'b0;

endmodule
