`include "cm3_etm_defs.vp"

module cm3_etm_res_control (
  clk, reset_n, apb_addr_i, apb_wdata_i, register_write_i, max_ext_in_i,
  prg_port_mode_i, prg_etm_en_i, prg_prog_bit_i, prg_may_assert_dbgreq_i,
  prg_branch_broadcast_i, prg_stall_processor_i, prg_ts_enable_i, prg_port_size_i,
  prg_etm_power_down_i, rsrc_start_on_i, rsrc_trig_detect_i, trace_flushed_i,
  prg_prog_bit_fall_i, reason_overflow_i, etm_rev_and_i,
  fifo_full_support_reg_i, prg_lock_status_i, prg_claim_tag_i, ext_in_i,
  core_halt_i, atb_atreadym_i, niden_t0_i,
  prg_trigger_event_o, prg_trc_en_start_stop_o, prg_trc_start_o,
  prg_trc_stop_o, prg_trc_en_event_o,
  prg_count_reload_o,
  prg_ts_event_o, read_data_o, atb_atidm_o, stat_reg_write_o,
  stat_reg_written_o, integration_mode_o,
  integration_trigger_we_o, integration_atb_cr0_we_o, prg_fifo_full_level_o
  );

  parameter TRACE_LVL      = 1;
  parameter RESET_ALL_REGS = 0;
  parameter CP_PRESENT     = 0;                  // Affects ID registers

  input                 clk;                     // Gated ETM Clock
  input                 reset_n;                 // ETM Reset

  input [11:2]          apb_addr_i;              // Register Write Address
  input [31:0]          apb_wdata_i;             // Register Write Signal
  input                 register_write_i;        // Register Write Data

  input [1:0]           max_ext_in_i;            // Maximum supported no. of EXTIN's
  input [2:0]           prg_port_mode_i;         // Control reigster port mode
  input                 prg_etm_en_i;            // Control register enable bit
  input                 prg_prog_bit_i;          // Control register programing bit
  input                 prg_may_assert_dbgreq_i; // Control register halt enable
  input                 prg_branch_broadcast_i;  // Control register broadcast enable
  input                 prg_stall_processor_i;   // Control register fifo full enable
  input                 prg_ts_enable_i;         // Control register timestamp enable
  input [3:0]           prg_port_size_i;         // Control register port size
  input                 prg_etm_power_down_i;    // Control register power down
  input                 rsrc_start_on_i;         // Start/Stop block state
  input                 rsrc_trig_detect_i;      // Trigger has been generated
  input                 trace_flushed_i;         // TraceOut is idle
  input                 prg_prog_bit_fall_i;     // Programming Bit falling edge
  input                 reason_overflow_i;       // Pending Overflow
  input [3:0]           etm_rev_and_i;           // ETM RevAnd
  input                 fifo_full_support_reg_i; // System supports FIFO Full stalling
  input                 prg_lock_status_i;       // Lock register input
  input [3:0]           prg_claim_tag_i;         // Claim tag register
  input [1:0]           ext_in_i;                // For Integration Mode
  input                 core_halt_i;             // For Integration Mode
  input                 atb_atreadym_i;          // For Integration Mode
  input                 niden_t0_i;              // Debug enable input

  output [10:0]         prg_trigger_event_o;     // Trigger Event Register
  output                prg_trc_en_start_stop_o; // Value of TrcEn Ctrl 1 Reg[25]
  output [3:0]          prg_trc_start_o;         // Value of TrcEn Start Stop
  output [3:0]          prg_trc_stop_o;          // Value of TrcEn Start Stop
  output [10:0]         prg_trc_en_event_o;      // Value of TrcEn Event Reg
  output [15:0]         prg_count_reload_o;      // Reload value for free running counter
  output [10:0]         prg_ts_event_o;          // timestamp Event Reg

  output [31:0]         read_data_o;             // Reg Read Mux Data
  output [6:0]          atb_atidm_o;             // Value of ATID
  output                stat_reg_write_o;        // Status register has been written

  output                stat_reg_written_o;      // Delayed version of stat_reg_write

  output                integration_mode_o;      // Integration Mode
  output                integration_trigger_we_o;// Int. Mode TRIGOUT Register Enable
  output                integration_atb_cr0_we_o;// Int. Mode ATBCTR2 Register Enable

  output [(`FIFO_DEPTH_SIZE-1):0]   prg_fifo_full_level_o; // Fifo Full level register

  wire              trigger_event_we;            // Trigger Event register write enable
  wire [3:0]        prg_trc_start;               // ETM Trace Enable turn on/off Register
  wire [3:0]        prg_trc_stop;                // ETM Trace Enable turn on/off Register
  wire              prg_trc_en_start_stop;       // ETM Trace Enable Start/Stop Enable
  reg  [1:0]        max_ext_in_reg;              // Reg'd version of max_ext_in_i I/P

  wire              trace_ss_we;                 // Write En for TrcStrtStpEI Reg
  wire              prg_trc_en_event_we;         // Write En for TrcEnEvent Reg
  wire              trc_ss_en_we;                // Write En for TrcEnCtrl1 Reg
  wire              port_size_ok;                // Portsize selected is ok
  wire              port_mode_ok;                // Portmode selected is ok
  wire [6:0]        atb_atidm;                   // ATB ID Register
  wire              atb_atidm_we;                // Write En. for ATID Register
  wire [(`FIFO_DEPTH_SIZE-1):0] prg_fifo_full_level; // Fifo Full level register Register
  wire [(`FIFO_DEPTH_SIZE-1):0] nxt_prg_fifo_full_level; // Fifo Full level register D-input
  wire              fifo_full_level_we;          // Write En. for FFlvl Register
  wire              stat_written_we;
  wire              nxt_stat_reg_written;
  wire              control_rd_en;
  wire              config_code_rd_en;
  wire              trigger_event_rd_en;
  wire              status_rd_en;
  wire              sys_config_rd_en;
  wire              trc_strt_stp_rd_en;
  wire              trc_en_event_rd_en;
  wire              trc_en_ctrl_1_rd_en;
  wire              fifo_full_level_rd_en;
  wire              sync_count_rd_en;
  wire              id_rd_en;
  wire              cnfg_code_ex_rd_en;
  wire              register_write_lk;
  wire              lock_stat_rd_en;
  wire              auth_stat_rd_en;
  wire              prog_bit_ack;                 // ProgBit and Traceout idle
  wire              power_down_status_rd_en;

  wire              int_misc_in_rd_en;
  wire              int_atb_ctr2_rd_en;
  wire              at_idm_rd_en;
  wire              dev_type_rd_en;
  wire              periph_id_0_rd_en;
  wire              periph_id_1_rd_en;
  wire              periph_id_2_rd_en;
  wire              periph_id_3_rd_en;
  wire              periph_id_4_rd_en;
  wire              periph_cid_0_rd_en;
  wire              periph_cid_1_rd_en;
  wire              periph_cid_2_rd_en;
  wire              periph_cid_3_rd_en;
  wire [31:0]       cnfg_code;                    // Configuration code register
  wire [28:0]       control_value;

  wire              count_reload_we;
  wire              count_reload_rd_en;
  reg [15:0]        prg_count_reload;
  reg [2:0]         prg_ts_event_bool;            // timestamp Event Reg
  reg [3:0]         prg_ts_event_resa;            // timestamp Event Reg
  wire[3:0]         nxt_prg_ts_event_resa;        // timestamp Event Reg
  reg [3:0]         prg_ts_event_resb;            // timestamp Event Reg
  wire[3:0]         nxt_prg_ts_event_resb;        // timestamp Event Reg
  wire              ts_event_rd_en;
  wire [16:0]       value_ts_event;
  wire [16:0]       value_trigger_event;
  wire [16:0]       value_trc_en_event;

  wire              integration_mode_we;
  wire              integration_mode_rd_en;
  wire  [31:0]      integration_mode_rd_data;
  wire              integration_mode;


  wire              claim_set_rd_en;
  wire              claim_clr_rd_en;
  wire [31:0]       claim_set_rd_data;
  wire [31:0]       claim_clr_rd_data;

  reg  [2:0]        opt_prg_trigger_event_bool;
  reg  [3:0]        opt_prg_trigger_event_resa;
  wire [3:0]        nxt_prg_trigger_event_resa;
  reg  [3:0]        opt_prg_trigger_event_resb;
  wire [3:0]        nxt_prg_trigger_event_resb;
  reg               opt_prg_trc_en_start_stop;
  reg  [3:0]        opt_prg_trc_start;
  reg  [3:0]        opt_prg_trc_stop;
  reg  [2:0]        opt_prg_trc_en_event_bool;
  reg  [3:0]        opt_prg_trc_en_event_resa;
  wire [3:0]        nxt_prg_trc_en_event_resa;
  reg  [3:0]        opt_prg_trc_en_event_resb;
  wire [3:0]        nxt_prg_trc_en_event_resb;
  wire [31:0]       opt_read_data;
  reg  [6:0]        opt_atb_atidm;
  wire              prg_ts_event_we;
  wire              stat_reg_write;
  reg               opt_stat_reg_written;
  reg               opt_integration_mode;
  wire              integration_trigger_we;
  wire              integration_atb_cr0_we;
  reg [(`FIFO_DEPTH_SIZE-1):0] opt_prg_fifo_full_level;
  wire              opt_etm_en;
  wire              opt_reset_n;


`ifdef ARM_DSM

  wire              dsm_reset_all_regs;

  assign opt_reset_n = dsm_reset_all_regs    ? reset_n : 1'b1;

`else

  assign opt_reset_n = (RESET_ALL_REGS == 1) ? reset_n : 1'b1;

`endif

  assign prg_trc_en_start_stop  = (TRACE_LVL > 1) ? opt_prg_trc_en_start_stop : 1'b0;
  assign prg_trc_start          = (TRACE_LVL > 1) ? opt_prg_trc_start         : 4'b0000;
  assign prg_trc_stop           = (TRACE_LVL > 1) ? opt_prg_trc_stop          : 4'b0000;
  assign atb_atidm              = (TRACE_LVL > 1) ? opt_atb_atidm             : {7{1'b0}};
  assign integration_mode       = (TRACE_LVL > 1) ? opt_integration_mode      : 1'b0;
  assign prg_fifo_full_level    = (TRACE_LVL > 1) ? opt_prg_fifo_full_level   : {`FIFO_DEPTH_SIZE{1'b0}};
  assign opt_etm_en             = (TRACE_LVL > 1) ? 1'b1                      : 1'b0;

  assign register_write_lk = register_write_i & ~prg_lock_status_i;

  assign trigger_event_we = (apb_addr_i[11:2] == `AddrTrgEvnt) &
                              register_write_lk & opt_etm_en;
  assign nxt_prg_trigger_event_resb = {apb_wdata_i[13:12],apb_wdata_i[8:7]};
  assign nxt_prg_trigger_event_resa = {apb_wdata_i[6:5],apb_wdata_i[1:0]};

  always @(posedge clk or negedge opt_reset_n)
    if (!opt_reset_n)
      begin
        opt_prg_trigger_event_bool <= {3{1'b0}};
        opt_prg_trigger_event_resa <= {4{1'b0}};
        opt_prg_trigger_event_resb <= {4{1'b0}};
      end
    else if (trigger_event_we)
      begin
        opt_prg_trigger_event_bool <= apb_wdata_i[16:14];
        opt_prg_trigger_event_resb <= nxt_prg_trigger_event_resb;
        opt_prg_trigger_event_resa <= nxt_prg_trigger_event_resa;
      end

  assign value_trigger_event = {opt_prg_trigger_event_bool,
                                opt_prg_trigger_event_resb[3:2],
                                (opt_prg_trigger_event_resb[3:2]==2'b10) &
                                (opt_prg_trigger_event_resb[1:0] == 2'b11),
                                {2{opt_prg_trigger_event_resb[3] &
                                 (opt_prg_trigger_event_resb[1:0] == 2'b11)}},
                                opt_prg_trigger_event_resb[1:0],
                                opt_prg_trigger_event_resa[3:2],
                                (opt_prg_trigger_event_resa[3:2]==2'b10) &
                                (opt_prg_trigger_event_resa[1:0] == 2'b11),
                                {2{opt_prg_trigger_event_resa[3] &
                                 (opt_prg_trigger_event_resa[1:0] == 2'b11)}},
                                opt_prg_trigger_event_resa[1:0]};


  assign stat_reg_write   = ((apb_addr_i[11:2] == `AddrStatus) & register_write_lk);
  assign stat_written_we      = (stat_reg_write | prg_prog_bit_fall_i) & opt_etm_en;
  assign nxt_stat_reg_written = ~prg_prog_bit_fall_i & (opt_stat_reg_written |
                                                        stat_reg_write);

  always @(posedge clk or negedge reset_n)
    if (!reset_n)
      opt_stat_reg_written <= 1'b0;
    else if (stat_written_we )
      opt_stat_reg_written <= nxt_stat_reg_written;

  assign count_reload_we = register_write_lk &
                           (apb_addr_i[11:2] == `AddrCountReload) &
                             opt_etm_en;

  always @ (posedge clk or negedge opt_reset_n)
    if (!opt_reset_n)
      prg_count_reload <= {16{1'b0}};
    else if (count_reload_we)
      prg_count_reload <= apb_wdata_i[15:0];

  assign trace_ss_we          = register_write_lk &
                                (apb_addr_i[11:2] == `AddrTrcSSEI) & opt_etm_en;
  assign prg_trc_en_event_we  = register_write_lk &
                                (apb_addr_i[11:2] == `AddrTrcEnEvnt)  & opt_etm_en;
  assign trc_ss_en_we         = register_write_lk &
                                (apb_addr_i[11:2] == `AddrTrcEnCtrl1) & opt_etm_en;

  always @ (posedge clk or negedge opt_reset_n)
    if (!opt_reset_n)
      begin
        opt_prg_trc_start <= 4'b0000;
        opt_prg_trc_stop  <= 4'b0000;
      end
    else if (trace_ss_we)
      begin
        opt_prg_trc_start <= apb_wdata_i[3:0];
        opt_prg_trc_stop  <= apb_wdata_i[19:16];
      end

  assign nxt_prg_trc_en_event_resb = {apb_wdata_i[13:12],apb_wdata_i[8:7]};
  assign nxt_prg_trc_en_event_resa = {apb_wdata_i[6:5],apb_wdata_i[1:0]};

  always @(posedge clk or negedge opt_reset_n)
    if (!opt_reset_n)
      begin
        opt_prg_trc_en_event_bool <= {3{1'b0}};
        opt_prg_trc_en_event_resa <= {4{1'b0}};
        opt_prg_trc_en_event_resb <= {4{1'b0}};
      end
    else if (prg_trc_en_event_we)
      begin
        opt_prg_trc_en_event_bool <= apb_wdata_i[16:14];
        opt_prg_trc_en_event_resb <= nxt_prg_trc_en_event_resb;
        opt_prg_trc_en_event_resa <= nxt_prg_trc_en_event_resa;
      end

  assign value_trc_en_event = {opt_prg_trc_en_event_bool,
                                opt_prg_trc_en_event_resb[3:2],
                                (opt_prg_trc_en_event_resb[3:2]==2'b10) &
                                (opt_prg_trc_en_event_resb[1:0] == 2'b11),
                                {2{opt_prg_trc_en_event_resb[3] &
                                 (opt_prg_trc_en_event_resb[1:0] == 2'b11)}},
                                opt_prg_trc_en_event_resb[1:0],
                                opt_prg_trc_en_event_resa[3:2],
                                (opt_prg_trc_en_event_resa[3:2]==2'b10) &
                                (opt_prg_trc_en_event_resa[1:0] == 2'b11),
                                {2{opt_prg_trc_en_event_resa[3] &
                                 (opt_prg_trc_en_event_resa[1:0] == 2'b11)}},
                                opt_prg_trc_en_event_resa[1:0]};

  always @ (posedge clk or negedge opt_reset_n)
    if (!opt_reset_n)
      opt_prg_trc_en_start_stop <= 1'b0;
    else if (trc_ss_en_we)
      opt_prg_trc_en_start_stop <= apb_wdata_i[25];


  assign fifo_full_level_we = register_write_lk &
                              (apb_addr_i[11:2] == `AddrFfLvlCtl) & opt_etm_en;
  assign nxt_prg_fifo_full_level = (apb_wdata_i <{{`FIFO_DEPTH_PAD{1'b0}},`FIFO_SIZE})?
                                     (apb_wdata_i[(`FIFO_DEPTH_SIZE-1):0]):`FIFO_SIZE;

  always @(posedge clk or negedge opt_reset_n)
    if (!opt_reset_n)
      opt_prg_fifo_full_level <= {`FIFO_DEPTH_SIZE{1'b0}};
    else if (fifo_full_level_we)
      opt_prg_fifo_full_level <= nxt_prg_fifo_full_level;


  assign atb_atidm_we = register_write_lk &
                        (apb_addr_i[11:2] == `AddrATID) & opt_etm_en;

  always @(posedge clk or negedge reset_n)
    if (!reset_n)
      opt_atb_atidm <= 7'd0;
    else if (atb_atidm_we)
      opt_atb_atidm[6:0] <= apb_wdata_i[6:0];

  assign prg_ts_event_we  = register_write_lk &
                            (apb_addr_i[11:2] == `AddrTSEvnt)  & opt_etm_en;
  assign nxt_prg_ts_event_resb = {apb_wdata_i[13:12],apb_wdata_i[8:7]};
  assign nxt_prg_ts_event_resa = {apb_wdata_i[6:5],apb_wdata_i[1:0]};

  always @(posedge clk or negedge opt_reset_n)
    if (!opt_reset_n)
      begin
        prg_ts_event_bool <= {3{1'b0}};
        prg_ts_event_resa <= {4{1'b0}};
        prg_ts_event_resb <= {4{1'b0}};
      end
    else if (prg_ts_event_we)
      begin
        prg_ts_event_bool <= apb_wdata_i[16:14];
        prg_ts_event_resb <= nxt_prg_ts_event_resb;
        prg_ts_event_resa <= nxt_prg_ts_event_resa;
      end

  assign value_ts_event = {prg_ts_event_bool,
                           prg_ts_event_resb[3:2],
                           (prg_ts_event_resb[3:2]==2'b10) &
                           (prg_ts_event_resb[1:0] == 2'b11),
                           {2{prg_ts_event_resb[3] &
                              (prg_ts_event_resb[1:0] == 2'b11)}},
                           prg_ts_event_resb[1:0],
                           prg_ts_event_resa[3:2],
                           (prg_ts_event_resa[3:2]==2'b10) &
                           (prg_ts_event_resa[1:0] == 2'b11),
                           {2{prg_ts_event_resa[3] &
                              (prg_ts_event_resa[1:0] == 2'b11)}},
                           prg_ts_event_resa[1:0]};
  always @(posedge clk or negedge reset_n)
    if (!reset_n)
      opt_integration_mode <= 1'b0;
    else if (integration_mode_we)
      opt_integration_mode <= apb_wdata_i[0];

  assign integration_mode_we      = register_write_lk &
                                    (apb_addr_i[11:2] == `AddrITCR) & opt_etm_en;
  assign integration_mode_rd_en   = (apb_addr_i[11:2] == `AddrITCR);
  assign integration_mode_rd_data = { {31{1'b0}}, integration_mode};

  assign integration_trigger_we  = register_write_lk & (apb_addr_i[11:2] == `AddrITTRIGOUT);

  assign integration_atb_cr0_we  = register_write_lk & (apb_addr_i[11:2] == `AddrITATBCTR0);

  assign claim_set_rd_en     = (apb_addr_i == `CM_ETM_CLAIMSET_ADD);
  assign claim_clr_rd_en     = (apb_addr_i == `CM_ETM_CLAIMCLR_ADD);
  assign claim_set_rd_data   = {{28{1'b0}}, `CM_ETM_CLAIMSET_VAL};
  assign claim_clr_rd_data   = {{28{1'b0}}, prg_claim_tag_i[3:0]};

  always @(posedge clk or negedge reset_n)
    if (!reset_n)
      max_ext_in_reg    <= 2'b00;
    else if (opt_etm_en)
      max_ext_in_reg    <= max_ext_in_i;

  assign port_size_ok = (prg_port_size_i == 4'b0001);

  assign port_mode_ok = (prg_port_mode_i == 3'b000);

  assign cnfg_code = {`NumProtocol, 2'b11, `NumContextID, 2'b10,
                     3'b000, max_ext_in_reg, `NumSeq, `NumCntr, 5'b00000,
                     `NumDataCmp, `NumAddCmp};

  assign control_value[28]    = prg_ts_enable_i;
  assign control_value[27:22] = 6'b000000;
  assign control_value[21]    = prg_port_size_i[3];
  assign control_value[20:18] = 3'b000;
  assign control_value[17:16] = prg_port_mode_i[1:0];
  assign control_value[15:14] = 2'b00;
  assign control_value[13]    = prg_port_mode_i[2];
  assign control_value[12]    = 1'b0;
  assign control_value[11]    = prg_etm_en_i;
  assign control_value[10]    = prg_prog_bit_i;
  assign control_value[9]     = prg_may_assert_dbgreq_i;
  assign control_value[8]     = prg_branch_broadcast_i;
  assign control_value[7]     = prg_stall_processor_i;
  assign control_value[6:4]   = prg_port_size_i[2:0];
  assign control_value[3:1]   = 3'b000;
  assign control_value[0]     = prg_etm_power_down_i;

  assign control_rd_en       = apb_addr_i[11:2] ==`AddrCtl;
  assign config_code_rd_en   = apb_addr_i[11:2] ==`AddrCnfgCode;
  assign trigger_event_rd_en = apb_addr_i[11:2] ==`AddrTrgEvnt;
  assign status_rd_en        = apb_addr_i[11:2] ==`AddrStatus;
  assign sys_config_rd_en    = apb_addr_i[11:2] ==`AddrSysConfig;
  assign count_reload_rd_en  = apb_addr_i[11:2] ==`AddrCountReload;
  assign trc_strt_stp_rd_en  = apb_addr_i[11:2] ==`AddrTrcSSEI;
  assign trc_en_event_rd_en  = apb_addr_i[11:2] ==`AddrTrcEnEvnt;
  assign trc_en_ctrl_1_rd_en = apb_addr_i[11:2] ==`AddrTrcEnCtrl1;
  assign fifo_full_level_rd_en = apb_addr_i[11:2] ==`AddrFfLvlCtl;
  assign sync_count_rd_en    = apb_addr_i[11:2] ==`AddrSyncCount;
  assign id_rd_en            = apb_addr_i[11:2] ==`AddrIDReg;
  assign cnfg_code_ex_rd_en  = apb_addr_i[11:2] ==`AddrCnfgCodeEx;
  assign ts_event_rd_en      = apb_addr_i[11:2] ==`AddrTSEvnt;
  assign lock_stat_rd_en     = apb_addr_i[11:2] ==`AddrLStatus;
  assign auth_stat_rd_en     = apb_addr_i[11:2] ==`AddrAStatus;
  assign at_idm_rd_en         = apb_addr_i[11:2] ==`AddrATID;
  assign power_down_status_rd_en = apb_addr_i[11:2] ==`AddrPDSR;
  assign int_misc_in_rd_en   = apb_addr_i[11:2] ==`AddrITMISCIN;
  assign int_atb_ctr2_rd_en  = apb_addr_i[11:2] ==`AddrITATBCTR2;
  assign dev_type_rd_en      = apb_addr_i[11:2] ==`AddrDevTypeReg;
  assign periph_id_0_rd_en   = apb_addr_i[11:2] ==`AddrPeriphID0;
  assign periph_id_1_rd_en   = apb_addr_i[11:2] ==`AddrPeriphID1;
  assign periph_id_2_rd_en   = apb_addr_i[11:2] ==`AddrPeriphID2;
  assign periph_id_3_rd_en   = apb_addr_i[11:2] ==`AddrPeriphID3;
  assign periph_id_4_rd_en   = apb_addr_i[11:2] ==`AddrPeriphID4;
  assign periph_cid_0_rd_en  = apb_addr_i[11:2] ==`AddrPeriphCID0;
  assign periph_cid_1_rd_en  = apb_addr_i[11:2] ==`AddrPeriphCID1;
  assign periph_cid_2_rd_en  = apb_addr_i[11:2] ==`AddrPeriphCID2;
  assign periph_cid_3_rd_en  = apb_addr_i[11:2] ==`AddrPeriphCID3;

  assign prog_bit_ack = trace_flushed_i & prg_prog_bit_i;

  assign opt_read_data =
       {32{control_rd_en}}        & {{3{1'b0}},control_value} |
       {32{config_code_rd_en}}    & cnfg_code  |
       {32{trigger_event_rd_en}}  & {{15{1'b0}},value_trigger_event} |
       {32{status_rd_en}}         & {{28{1'b0}},rsrc_trig_detect_i,rsrc_start_on_i,
                                   prog_bit_ack,reason_overflow_i} |
       {32{sys_config_rd_en}}     & {{14{1'b0}},1'b1,2'b00,3'b000,port_mode_ok,
                                   port_size_ok & port_mode_ok, 1'b0,
                                   fifo_full_support_reg_i,
                                   3'b000, 1'b0,
                                     1'b1, 3'b001} |
       {32{count_reload_rd_en}}   & {{16{1'b0}},prg_count_reload} |
       {32{trc_strt_stp_rd_en}}   & {{12{1'b0}},prg_trc_stop,{12{1'b0}},prg_trc_start} |
       {32{trc_en_event_rd_en}}   & {{15{1'b0}},value_trc_en_event} |
       {32{trc_en_ctrl_1_rd_en}}  & {{6{1'b0}},prg_trc_en_start_stop,1'b0,
                                     {16{1'b0}},{8{1'b0}}}               |
       {32{fifo_full_level_rd_en}}   & {{(32-`FIFO_DEPTH_SIZE){1'b0}},prg_fifo_full_level} |
       {32{sync_count_rd_en}}     & {{21{1'b0}},1'b1,{10{1'b0}}} |
       {32{id_rd_en}}             & ((CP_PRESENT == 0) ? {`ETMID, `ETMIDRel3} : {`ETMID, `ETMIDRel4})|
       {32{cnfg_code_ex_rd_en}}   & `CnfgCodeExVal     |
       {32{ts_event_rd_en}}       & {{15{1'b0}},value_ts_event} |
       ({32{integration_mode_rd_en}}      & integration_mode_rd_data)        |
       ({32{claim_set_rd_en}}     & claim_set_rd_data)    |
       ({32{claim_clr_rd_en}}     & claim_clr_rd_data)    |
       {32{lock_stat_rd_en}}      & {{30{1'b0}},prg_lock_status_i,1'b1} |
       {32{auth_stat_rd_en}}      & {{24{1'b0}},1'b1,niden_t0_i,{6{1'b0}}} |
       {32{at_idm_rd_en}}         & {{25{1'b0}},atb_atidm} |
       {32{power_down_status_rd_en}}       & {{31{1'b0}},1'b1} |
       {32{int_misc_in_rd_en}}    & {{27{1'b0}},core_halt_i,{2{1'b0}},ext_in_i[1:0]} |
       {32{int_atb_ctr2_rd_en}}   & {{31{1'b0}},atb_atreadym_i} |
       {32{dev_type_rd_en}}       & `DevTypeRegVal |
       {32{periph_id_0_rd_en}}    & ((CP_PRESENT == 0) ? `PeriphID0Val3 : `PeriphID0Val4)  |
       {32{periph_id_1_rd_en}}    & `PeriphID1Val  |
       {32{periph_id_2_rd_en}}    & ((CP_PRESENT == 0) ? `PeriphID2Val3 : `PeriphID2Val4)  |
       {32{periph_id_3_rd_en}}    & `PeriphID3Val  |
       {32{periph_id_4_rd_en}}    & `PeriphID4Val  |
       {32{periph_cid_0_rd_en}}   & `PeriphCID0Val |
       {32{periph_cid_1_rd_en}}   & `PeriphCID1Val |
       {32{periph_cid_2_rd_en}}   & `PeriphCID2Val |
       {32{periph_cid_3_rd_en}}   & `PeriphCID3Val ;

  assign read_data_o              = (TRACE_LVL > 1) ? opt_read_data            : {32{1'b0}};
  assign stat_reg_write_o         = (TRACE_LVL > 1) ? stat_reg_write           : 1'b0;
  assign stat_reg_written_o       = (TRACE_LVL > 1) ? opt_stat_reg_written     : 1'b0;
  assign integration_trigger_we_o = (TRACE_LVL > 1) ? integration_trigger_we   : 1'b0;
  assign integration_atb_cr0_we_o = (TRACE_LVL > 1) ? integration_atb_cr0_we   : 1'b0;
  assign prg_trigger_event_o      = (TRACE_LVL > 1) ? {opt_prg_trigger_event_bool,
                                                       opt_prg_trigger_event_resb,
                                                       opt_prg_trigger_event_resa} : {11{1'b0}};
  assign prg_trc_en_start_stop_o  = (TRACE_LVL > 1) ? prg_trc_en_start_stop    : 1'b0;
  assign prg_trc_start_o          = (TRACE_LVL > 1) ? prg_trc_start            : 4'b0000;
  assign prg_trc_stop_o           = (TRACE_LVL > 1) ? prg_trc_stop             : 4'b0000;
  assign prg_trc_en_event_o       = (TRACE_LVL > 1) ? {opt_prg_trc_en_event_bool,
                                                       opt_prg_trc_en_event_resb,
                                                       opt_prg_trc_en_event_resa}  : {11{1'b0}};
  assign atb_atidm_o              = (TRACE_LVL > 1) ? atb_atidm                : {7{1'b0}};
  assign integration_mode_o       = (TRACE_LVL > 1) ? integration_mode         : 1'b0;
  assign prg_fifo_full_level_o    = (TRACE_LVL > 1) ? prg_fifo_full_level      : {`FIFO_DEPTH_SIZE{1'b0}};
  assign prg_count_reload_o       = (TRACE_LVL > 1) ? prg_count_reload         : {16{1'b0}};
  assign prg_ts_event_o           = (TRACE_LVL > 1) ? {prg_ts_event_bool,
                                                       prg_ts_event_resb,
                                                       prg_ts_event_resa}      : {11{1'b0}};



`ifdef ARM_ASSERT_ON
  assert_never_unknown #(0, 1,0, "trigger_event_we must not be X")
  ovl_trigger_event_we_x (
    .clk       (clk),
    .reset_n   (reset_n),
    .qualifier (1'b1),
    .test_expr (trigger_event_we));

  assert_never_unknown #(0, 1,0, "stat_written_we must not be X")
  ovl_stat_written_we_x (
    .clk       (clk),
    .reset_n   (reset_n),
    .qualifier (1'b1),
    .test_expr (stat_written_we));

  assert_never_unknown #(0, 1,0, "count_reload_we must not be X")
  ovl_count_reload_we_x (
    .clk       (clk),
    .reset_n   (reset_n),
    .qualifier (1'b1),
    .test_expr (count_reload_we));

  assert_never_unknown #(0, 1,0, "trace_ss_we must not be X")
  ovl_trace_ss_we_x (
    .clk       (clk),
    .reset_n   (reset_n),
    .qualifier (1'b1),
    .test_expr (trace_ss_we));

  assert_never_unknown #(0, 1,0, "prg_trc_en_event_we must not be X")
  ovl_prg_trc_en_event_we_x (
    .clk       (clk),
    .reset_n   (reset_n),
    .qualifier (1'b1),
    .test_expr (prg_trc_en_event_we));

  assert_never_unknown #(0, 1,0, "trc_ss_en_we must not be X")
  ovl_trc_ss_en_we_x (
    .clk       (clk),
    .reset_n   (reset_n),
    .qualifier (1'b1),
    .test_expr (trc_ss_en_we));

  assert_never_unknown #(0, 1,0, "fifo_full_level_we must not be X")
  ovl_fifo_full_level_we_x (
    .clk       (clk),
    .reset_n   (reset_n),
    .qualifier (1'b1),
    .test_expr (fifo_full_level_we));

  assert_never_unknown #(0, 1,0, "atb_atidm_we must not be X")
  ovl_atb_atidm_we_x (
    .clk       (clk),
    .reset_n   (reset_n),
    .qualifier (1'b1),
    .test_expr (atb_atidm_we));

  assert_never_unknown #(0, 1,0, "prg_ts_event_we must not be X")
  ovl_prg_ts_event_we_x (
    .clk       (clk),
    .reset_n   (reset_n),
    .qualifier (1'b1),
    .test_expr (prg_ts_event_we));

  assert_never_unknown #(0, 1,0, "integration_mode_we must not be X")
  ovl_integration_mode_we_x (
    .clk       (clk),
    .reset_n   (reset_n),
    .qualifier (1'b1),
    .test_expr (integration_mode_we));

`endif

endmodule // CM_Etm_Res_Control
