module cm3_flash_mux (

    HCLK,
    HRESETn,

    HADDRS0,
    HTRANSS0,
    HWRITES0,
    HSIZES0,
    HBURSTS0,
    HPROTS0,
    HWDATAS0,
    HREADYS0,

    HADDRS1,
    HTRANSS1,
    HWRITES1,
    HSIZES1,
    HBURSTS1,
    HPROTS1,
    HWDATAS1,
    HREADYS1,

    HRDATAM0,
    HREADYOUTM0,
    HRESPM0,

    HRDATAS0,
    HREADYOUTS0,
    HRESPS0,

    HRDATAS1,
    HREADYOUTS1,
    HRESPS1,

    HADDRM0,
    HTRANSM0,
    HWRITEM0,
    HSIZEM0,
    HBURSTM0,
    HPROTM0,
    HWDATAM0,
    HREADYMUXM0
    );

    input         HCLK;            // AHB System Clock
    input         HRESETn;         // AHB System Reset

    input  [31:0] HADDRS0;         // Address bus
    input   [1:0] HTRANSS0;        // Transfer type
    input         HWRITES0;        // Transfer direction
    input   [2:0] HSIZES0;         // Transfer size
    input   [2:0] HBURSTS0;        // Burst type
    input   [3:0] HPROTS0;         // Protection control
    input  [31:0] HWDATAS0;        // Write data
    input         HREADYS0;        // Transfer done

    input  [31:0] HADDRS1;         // Address bus
    input   [1:0] HTRANSS1;        // Transfer type
    input         HWRITES1;        // Transfer direction
    input   [2:0] HSIZES1;         // Transfer size
    input   [2:0] HBURSTS1;        // Burst type
    input   [3:0] HPROTS1;         // Protection control
    input  [31:0] HWDATAS1;        // Write data
    input         HREADYS1;        // Transfer done

    input  [31:0] HRDATAM0;        // Read data bus
    input         HREADYOUTM0;     // HREADY feedback
    input         HRESPM0;         // Transfer response

    output [31:0] HRDATAS0;        // Read data bus
    output        HREADYOUTS0;     // HREADY feedback
    output        HRESPS0;         // Transfer response

    output [31:0] HRDATAS1;        // Read data bus
    output        HREADYOUTS1;     // HREADY feedback
    output        HRESPS1;         // Transfer response

    output [31:0] HADDRM0;         // Address bus
    output  [1:0] HTRANSM0;        // Transfer type
    output        HWRITEM0;        // Transfer direction
    output  [2:0] HSIZEM0;         // Transfer size
    output  [2:0] HBURSTM0;        // Burst type
    output  [3:0] HPROTM0;         // Protection control
    output [31:0] HWDATAM0;        // Write data
    output        HREADYMUXM0;     // Transfer done


    wire  [31:0] hrdatas0;        // Read data bus
    wire         hreadyouts0;     // HREADY feedback
    wire         hresps0;         // Transfer response

    wire [31:0]  hrdatas1;        // Read data bus
    wire         hreadyouts1;     // HREADY feedback
    wire         hresps1;         // Transfer response

    wire [31:0]  haddrm0;         // Address bus
    wire  [1:0]  htransm0;        // Transfer type
    wire         hwritem0;        // Transfer direction
    wire  [2:0]  hsizem0;         // Transfer size
    wire  [2:0]  hburstm0;        // Burst type
    wire  [3:0]  hprotm0;         // Protection control
    wire [31:0]  hwdatam0;        // Write data
    wire         hreadymuxm0;     // Transfer done

    wire        sel0;       // Port 0 HSEL signal
    wire [31:0] addr0;      // Port 0 HADDR signal
    wire  [1:0] trans0;     // Port 0 HTRANS signal
    wire        write0;     // Port 0 HWRITE signal
    wire  [2:0] size0;      // Port 0 HSIZE signal
    wire  [2:0] burst0;     // Port 0 HBURST signal
    wire  [3:0] prot0;      // Port 0 HPROTS signal
    wire  [3:0] master0;    // Port 0 HMASTER signal
    wire        mastlock0;  // Port 0 HMASTLOCK signal
    wire        active0;    // Port 0 Active signal
    wire        held_tran0;  // Port 0 HeldTran signal
    wire        readyout0;  // Port 0 Readyout signal
    wire        resp0;      // Port 0 response

    wire        sel1;       // Port 1 HSEL signal
    wire [31:0] addr1;      // Port 1 HADDR signal
    wire  [1:0] trans1;     // Port 1 HTRANS signal
    wire        write1;     // Port 1 HWRITE signal
    wire  [2:0] size1;      // Port 1 HSIZE signal
    wire  [2:0] burst1;     // Port 1 HBURST signal
    wire  [3:0] prot1;      // Port 1 HPROTS signal
    wire  [3:0] master1;    // Port 1 HMASTER signal
    wire        mastlock1;  // Port 1 HMASTLOCK signal
    wire        active1;    // Port 1 Active signal
    wire        held_tran1;  // Port 1 HeldTran signal
    wire        readyout1;  // Port 1 Readyout signal
    wire        resp1;      // Port 1 response

    wire        sel0_to_0;    // Port 0 to 0 HSEL signal
    wire        active0_to_0; // Port 0 to 0 Active signal

    wire        sel1_to_0;    // Port 1 to 0 HSEL signal
    wire        active1_to_0; // Port 1 to 0 Active signal


  input_stage_t u_input_stage_t0
    (
    .hclk          (HCLK),
    .hreset_n      (HRESETn),

    .hsels_i       (1'b1),
    .haddrs_i      (HADDRS0),
    .htranss_i     (HTRANSS0),
    .hwrites_i     (HWRITES0),
    .hsizes_i      (HSIZES0),
    .hbursts_i     (HBURSTS0),
    .hprots_i      (HPROTS0),
    .hmasters_i    (4'b0000),
    .hmastlocks_i  (1'b0),
    .hreadys_i     (HREADYS0),

    .active_i      (active0),
    .readyout_i    (readyout0),
    .resp_i        (resp0),

    .hreadyouts_o  (hreadyouts0),
    .hresps_o      (hresps0),

    .sel_o         (sel0),
    .addr_o        (addr0),
    .trans_o       (trans0),
    .write_o       (write0),
    .size_o        (size0),
    .burst_o       (burst0),
    .prot_o        (prot0),
    .master_o      (master0),
    .mastlock_o    (mastlock0),
    .held_tran_o   (held_tran0)
    );

  input_stage_t u_input_stage_t1
    (
    .hclk          (HCLK),
    .hreset_n      (HRESETn),

    .hsels_i       (1'b1),
    .haddrs_i      (HADDRS1),
    .htranss_i     (HTRANSS1),
    .hwrites_i     (HWRITES1),
    .hsizes_i      (HSIZES1),
    .hbursts_i     (HBURSTS1),
    .hprots_i      (HPROTS1),
    .hmasters_i    (4'b0000),
    .hmastlocks_i  (1'b0),
    .hreadys_i     (HREADYS1),

    .active_i      (active1),
    .readyout_i    (readyout1),
    .resp_i        (resp1),

    .hreadyouts_o  (hreadyouts1),
    .hresps_o      (hresps1),

    .sel_o         (sel1),
    .addr_o        (addr1),
    .trans_o       (trans1),
    .write_o       (write1),
    .size_o        (size1),
    .burst_o       (burst1),
    .prot_o        (prot1),
    .master_o      (master1),
    .mastlock_o    (mastlock1),
    .held_tran_o   (held_tran1)
    );

  matrix_decode_t u_matrix_decode_t0
    (
    .hclk          (HCLK),
    .hreset_n      (HRESETn),

    .hreadys_i     (HREADYS0),
    .sel_i         (sel0),
    .addr_i        (addr0),

    .active0_i     (active0_to_0),
    .readyout0_i   (hreadymuxm0),
    .resp0_i       (HRESPM0),
    .rdata0_i      (HRDATAM0),

    .sel0_o        (sel0_to_0),

    .active_o      (active0),
    .hreadyouts_o  (readyout0),
    .hresps_o      (resp0),
    .hrdatas_o     (hrdatas0)
    );

  matrix_decode_t u_matrix_decode_t1
    (
    .hclk          (HCLK),
    .hreset_n      (HRESETn),

    .hreadys_i     (HREADYS1),
    .sel_i         (sel1),
    .addr_i        (addr1),

    .active0_i     (active1_to_0),
    .readyout0_i   (hreadymuxm0),
    .resp0_i       (HRESPM0),
    .rdata0_i      (HRDATAM0),

    .sel0_o        (sel1_to_0),

    .active_o      (active1),
    .hreadyouts_o  (readyout1),
    .hresps_o      (resp1),
    .hrdatas_o     (hrdatas1)
    );

  output_stage_t u_output_stage_t0
    (
    .hclk          (HCLK),
    .hreset_n      (HRESETn),

    .sel0_i        (sel0_to_0),
    .addr0_i       (addr0),
    .trans0_i      (trans0),
    .write0_i      (write0),
    .size0_i       (size0),
    .burst0_i      (burst0),
    .prot0_i       (prot0),
    .master0_i     (master0),
    .mastlock0_i   (mastlock0),
    .wdata0_i      (HWDATAS0),
    .held_tran0_i  (held_tran0),

    .sel1_i        (sel1_to_0),
    .addr1_i       (addr1),
    .trans1_i      (trans1),
    .write1_i      (write1),
    .size1_i       (size1),
    .burst1_i      (burst1),
    .prot1_i       (prot1),
    .master1_i     (master1),
    .mastlock1_i   (mastlock1),
    .wdata1_i      (HWDATAS1),
    .held_tran1_i  (held_tran1),

    .hreadyoutm_i  (HREADYOUTM0),

    .active0_o     (active0_to_0),
    .active1_o     (active1_to_0),

    .hselm_o       (),
    .haddrm_o      (haddrm0),
    .htransm_o     (htransm0),
    .hwritem_o     (hwritem0),
    .hsizem_o      (hsizem0),
    .hburstm_o     (hburstm0),
    .hprotm_o      (hprotm0),
    .hmasterm_o    (),
    .hmastlockm_o  (),
    .hreadymuxm_o  (hreadymuxm0),
    .hwdatam_o     (hwdatam0)
    );


    assign HRDATAS0    = hrdatas0;
    assign HREADYOUTS0 = hreadyouts0;
    assign HRESPS0     = hresps0;

    assign HRDATAS1    = hrdatas1;
    assign HREADYOUTS1 = hreadyouts1;
    assign HRESPS1     = hresps1;

    assign HADDRM0     = haddrm0;
    assign HTRANSM0    = htransm0;
    assign HWRITEM0    = hwritem0;
    assign HSIZEM0     = hsizem0;
    assign HBURSTM0    = hburstm0;
    assign HPROTM0     = hprotm0;
    assign HWDATAM0    = hwdatam0;
    assign HREADYMUXM0 = hreadymuxm0;

endmodule
