`include "cm3_mpu_defs.vp"

module cm3_mpu_ahb_ctl
  (// Inputs
   hclk, hreset_n, dpu_ahb_haddri_i, mtx_dpu_ahb_haddracci_i, dpu_ahb_hproti_i,
   dpu_ahb_htransi_i, mtx_mpu_ahb_hreadyouti_i, dpu_ahb_haddrd_i,
   mtx_dpu_ahb_haddraccd_i, dpu_ahb_hprotd_i, dpu_ahb_hwrited_i,
   dpu_ahb_htransd_i, mtx_mpu_ahb_hreadyoutd_i, dside_hit_i, iside_hit_i,
   attrsd_i, attrsi_i, mpu_enabled_i, aligned_addrd_i, aligned_writed_i,
   aligned_privd_i, check_alignedd_i, def_mem_map_i, mask_canceld_i,
   nvic_no_def_wbuf_i, priv_faultd_i,
   mpu_mtx_ahb_hproti_o, mpu_dpu_ahb_hreadyi_o, mpu_dpu_ahb_hrespi_o,
   mpu_canceli_o, mpu_mtx_ahb_hprotd_o, mpu_dpu_ahb_hreadyd_o,
   mpu_dpu_ahb_hrespd_o, mpu_canceld_o, mpu_mem_attri_o, mpu_mem_attrd_o,
   mpu_addrd_non_buf_o
  );


  parameter     MPU_PRESENT = 0;   // MPU present (0 = not present; 1 = present)



  input         hclk;                     // AHB clock
  input         hreset_n;                 // AHB reset

  input  [31:0] dpu_ahb_haddri_i;         // I-AHB address in
  input   [1:0] dpu_ahb_htransi_i;        // I-AHB trans in
  input   [1:0] dpu_ahb_hproti_i;         // I-AHB protection in
  input         mtx_dpu_ahb_haddracci_i;  // I-AHB address accept in
  input         mtx_mpu_ahb_hreadyouti_i; // I-AHB ready in
  input  [31:0] dpu_ahb_haddrd_i;         // D-AHB address in
  input   [1:0] dpu_ahb_htransd_i;        // D-AHB trans in
  input   [1:0] dpu_ahb_hprotd_i;         // D-AHB protection in
  input         dpu_ahb_hwrited_i;        // D-AHB write in
  input         mtx_dpu_ahb_haddraccd_i;  // D-AHB address accept in
  input         mtx_mpu_ahb_hreadyoutd_i; // D-AHB ready in
  input         dside_hit_i;              // dpu_ahb_haddrd_i hit
  input         iside_hit_i;              // dpu_ahb_haddri_i hit
  input         mpu_enabled_i;            // Master MPU Enable
  input         def_mem_map_i;            // Enable default memory map
  input   [8:0] attrsd_i;                 // Attributes for the region hit (D)
  input   [9:0] attrsi_i;                 // Attributes for the region hit (I)
  input  [31:5] aligned_addrd_i;          // Aligned addr for unaligned access
  input         aligned_writed_i;         // Aligned write for unaligned access
  input         aligned_privd_i;          // Aligned priv for unaligned access
  input         mask_canceld_i;           // Mask the mpu_canceld signal
  input         check_alignedd_i;         // Need to check an aligned access
  input         nvic_no_def_wbuf_i;       // Disable wr buffers for def mem map
  input         priv_faultd_i;            // D-side privilege access violation

  output  [3:0] mpu_mtx_ahb_hproti_o;     // I-AHB protection out
  output        mpu_dpu_ahb_hreadyi_o;    // I-AHB ready out
  output        mpu_dpu_ahb_hrespi_o;     // I-AHB error response out
  output        mpu_canceli_o;            // I-AHB cancel of an I-Side access
  output  [1:0] mpu_mem_attri_o;          // I-AHB Memory Attributes
  output  [3:0] mpu_mtx_ahb_hprotd_o;     // D-AHB protection out
  output        mpu_dpu_ahb_hreadyd_o;    // D-AHB ready out
  output        mpu_dpu_ahb_hrespd_o;     // D-AHB error response out
  output        mpu_canceld_o;            // D-AHB cancel align/unalign access
  output  [1:0] mpu_mem_attrd_o;          // D-AHB Memory Attributes
  output        mpu_addrd_non_buf_o;      // D-AHB address must not be buffered


  wire    [2:0] attrsd_tex;
  wire          attrsd_s;
  wire          attrsd_c;
  wire          attrsd_b;
  wire    [2:0] attrsi_ap;
  wire          attrsi_xn;
  wire    [2:0] attrsi_tex;
  wire          attrsi_s;
  wire          attrsi_c;
  wire          attrsi_b;

  wire          d_access;
  wire          d_ppb_access;
  wire          d_ppb_align;
  wire          d_fault_enable;
  wire          d_def_active;
  wire          d_def_active_align;
  wire          nxt_d_fault_norm;
  wire          nxt_d_fault_align;
  wire          d_enable_align;
  wire          mpu_canceld;

  wire          d_allow_drive;
  wire          d_outer_cacheable;
  wire          d_outer_bufferable;
  wire          d_allocate;
  wire          d_share;
  wire          d_share_default;
  wire          d_share_align;
  wire          d_alloc_default;
  wire          d_alloc_align;

  wire    [3:2] hprotd_default;
  wire    [3:2] hprotd_align;
  wire    [3:2] hprotd_sel;
  wire    [3:0] ahb_hprotd;
  wire    [1:0] mem_addrd_sel;
  wire    [1:0] mem_attrd;
  wire          addrd_non_buf;

  reg           d_fault;
  wire          d_fault_en;
  wire          nxt_d_fault;
  wire          ahb_hrespd;
  wire          hreadyoutd_err_en;
  wire          nxt_hreadyoutd_err;
  wire          ahb_hreadyd;
  reg           hreadyoutd_err;

  wire          i_access;
  wire          i_ppb_access;
  wire          i_allow_drive;
  wire          i_outer_cacheable;
  wire          i_outer_bufferable;
  wire          i_allocate;
  wire          i_share;
  wire          i_share_default;
  wire          i_alloc_default;

  wire    [3:2] hproti_default;
  wire    [3:0] ahb_hproti;
  wire    [1:0] mem_attri;
  wire          nxt_i_fault;
  wire          mpu_canceli;
  wire          i_upper_fault;
  wire          i_def_xn_fault;
  wire          i_will_i_fault;
  wire          i_will_p_fault;
  wire          nxt_i_access_fault;
  wire          nxt_i_priv_fault;
  wire          i_fault_enable;
  wire          i_def_active;

  reg           i_fault;
  wire          i_fault_en;
  wire          ahb_hrespi;
  reg           hreadyouti_err;
  wire          nxt_hreadyouti_err;
  wire          hreadyouti_err_en;
  wire          ahb_hreadyi;

  wire    [3:0] opt_ahb_hproti;
  wire          opt_ahb_hreadyi;
  wire          opt_ahb_hrespi;
  wire          opt_mpu_canceli;
  wire    [1:0] opt_mem_attri;
  wire    [3:0] opt_ahb_hprotd;
  wire          opt_ahb_hreadyd;
  wire          opt_ahb_hrespd;
  wire          opt_mpu_canceld;
  wire    [1:0] opt_mem_attrd;
  wire          opt_addrd_non_buf;
  wire          opt_mst_en;
  wire          opt_mpu_en;

`ifdef ARM_DSM

  wire          dsm_mpu_present;

  assign opt_mpu_en = (dsm_mpu_present)  ? 1'b1 : 1'b0;

`else

  assign opt_mpu_en = (MPU_PRESENT != 0) ? 1'b1 : 1'b0;

`endif

  assign opt_ahb_hproti    = (opt_mpu_en) ? ahb_hproti    : 4'b0000;
  assign opt_ahb_hreadyi   = (opt_mpu_en) ? ahb_hreadyi   : 1'b0;
  assign opt_ahb_hrespi    = (opt_mpu_en) ? ahb_hrespi    : 1'b0;
  assign opt_mpu_canceli   = (opt_mpu_en) ? mpu_canceli   : 1'b0;
  assign opt_mem_attri     = (opt_mpu_en) ? mem_attri     : 2'b00;
  assign opt_ahb_hprotd    = (opt_mpu_en) ? ahb_hprotd    : 4'b0001;
  assign opt_ahb_hreadyd   = (opt_mpu_en) ? ahb_hreadyd   : 1'b0;
  assign opt_ahb_hrespd    = (opt_mpu_en) ? ahb_hrespd    : 1'b0;
  assign opt_mpu_canceld   = (opt_mpu_en) ? mpu_canceld   : 1'b0;
  assign opt_mem_attrd     = (opt_mpu_en) ? mem_attrd     : 2'b00;
  assign opt_addrd_non_buf = (opt_mpu_en) ? addrd_non_buf : 1'b0;
  assign opt_mst_en        = (opt_mpu_en) ? 1'b1          : 1'b0;

  assign attrsd_tex = attrsd_i[5:3];
  assign attrsd_s   = attrsd_i[2];
  assign attrsd_c   = attrsd_i[1];
  assign attrsd_b   = attrsd_i[0];

  assign attrsi_xn  = attrsi_i[9];
  assign attrsi_ap  = attrsi_i[8:6];
  assign attrsi_tex = attrsi_i[5:3];
  assign attrsi_s   = attrsi_i[2];
  assign attrsi_c   = attrsi_i[1];
  assign attrsi_b   = attrsi_i[0];


  assign d_access     = dpu_ahb_htransd_i[1];

  assign d_ppb_access = (dpu_ahb_haddrd_i[31:20] == `MPUPPBREGION);
  assign d_def_active = ((mpu_enabled_i       & def_mem_map_i &
                          dpu_ahb_hprotd_i[1] & ~check_alignedd_i) |
                         ~mpu_enabled_i);

  assign d_fault_enable = d_access & ~d_ppb_access & ~check_alignedd_i;

  assign nxt_d_fault_norm = (d_fault_enable &
                             ((~dside_hit_i & ~d_def_active) |
                              (priv_faultd_i & dside_hit_i)));

  assign d_ppb_align        = (aligned_addrd_i[31:20] == `MPUPPBREGION);
  assign d_def_active_align = ((mpu_enabled_i & def_mem_map_i &
                                aligned_privd_i & check_alignedd_i) |
                               ~mpu_enabled_i);

  assign d_enable_align = check_alignedd_i & ~d_ppb_align;

  assign nxt_d_fault_align = ((~dside_hit_i & ~d_def_active_align) |
                              (priv_faultd_i & dside_hit_i)) & d_enable_align;

  assign mpu_canceld = ((nxt_d_fault_norm | nxt_d_fault_align) &
                         ~mask_canceld_i & hreadyoutd_err);



  assign d_allow_drive = (mpu_enabled_i & dside_hit_i &
                          ~(d_ppb_access & ~check_alignedd_i) &
                          ~(d_ppb_align  &  check_alignedd_i));

  assign d_outer_bufferable = ((~attrsd_tex[2] & attrsd_b) |
                               ( attrsd_tex[2] & attrsd_tex[0]) |
                               ((attrsd_tex[2:0] == 3'b010) &
                                ~attrsd_c & ~attrsd_b));

  assign d_outer_cacheable  = ((~attrsd_tex[2] & attrsd_c) | attrsd_tex[2] |
                               ((attrsd_tex[2:0] == 3'b001) &
                                ~attrsd_c & ~attrsd_b));

  assign d_share    = ( (attrsd_s | (~attrsd_c & ~|attrsd_tex)) &
                       ~((attrsd_tex == 3'b010) & ~attrsd_c & ~attrsd_b) &
                        (d_allow_drive | d_def_active | d_def_active_align));

  assign d_allocate = ((((attrsd_tex == 3'b000) & attrsd_c) |
                        (attrsd_tex[2:1] == 2'b11)) &
                       (d_allow_drive | d_def_active | d_def_active_align));



  assign hprotd_default[3] = ((~dpu_ahb_haddrd_i[31] &  dpu_ahb_haddrd_i[29]) |
                              (~dpu_ahb_haddrd_i[30] & ~dpu_ahb_haddrd_i[29]));

  assign hprotd_align[3]   = ((~aligned_addrd_i[31] & aligned_addrd_i[29]) |
                              (~aligned_addrd_i[30] & ~aligned_addrd_i[29]));


  assign hprotd_default[2] = ((dpu_ahb_haddrd_i[30] | dpu_ahb_haddrd_i[29]) &
                              ~((&dpu_ahb_haddrd_i[31:29]) &
                                ~(|dpu_ahb_haddrd_i[28:20])));

  assign hprotd_align[2] = ((aligned_addrd_i[30] | aligned_addrd_i[29]) &
                            ~((&aligned_addrd_i[31:29]) &
                              ~(|aligned_addrd_i[28:16])));


  assign d_share_default = (dpu_ahb_haddrd_i[31:29] == 3'b101);
  assign d_share_align   = (aligned_addrd_i[31:29] == 3'b101);


  assign d_alloc_default = ((dpu_ahb_haddrd_i[31:29] == 3'b000) |
                            (dpu_ahb_haddrd_i[31:29] == 3'b100));

  assign d_alloc_align   = ((aligned_addrd_i[31:29] == 3'b000) |
                            (aligned_addrd_i[31:29] == 3'b100));


  assign hprotd_sel[3:2] = check_alignedd_i ? hprotd_align : hprotd_default;

  assign ahb_hprotd[3:2] = (({2{ d_allow_drive}} &
                             {d_outer_cacheable,d_outer_bufferable}) |
                            ({2{~d_allow_drive}} &
                             {2{d_def_active | d_def_active_align}}  &
                              hprotd_sel[3:2]));

  assign ahb_hprotd[1:0]  = dpu_ahb_hprotd_i[1:0];

  assign mem_addrd_sel[1] = check_alignedd_i ? d_share_align : d_share_default;
  assign mem_addrd_sel[0] = check_alignedd_i ? d_alloc_align : d_alloc_default;

  assign mem_attrd = (({2{ d_allow_drive}} & { d_share, d_allocate }) |
                      (mem_addrd_sel & {2{~d_allow_drive}} &
                       {2{d_def_active | d_def_active_align}}));

  assign addrd_non_buf = (((d_def_active | d_def_active_align) &
                           nvic_no_def_wbuf_i & ~dside_hit_i) |
                          ~(|ahb_hprotd[3:2]));


  assign d_fault_en  = (opt_mst_en &
                        ((ahb_hreadyd & mtx_dpu_ahb_haddraccd_i &
                          nxt_d_fault_norm) |
                         nxt_d_fault_align |
                         (ahb_hreadyd & d_fault)));

  assign nxt_d_fault = ((ahb_hreadyd & ~d_fault) | nxt_d_fault_align |
                        (ahb_hreadyd & mtx_dpu_ahb_haddraccd_i &
                         nxt_d_fault_norm));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      d_fault <= 1'b0;
    else if (d_fault_en)
      d_fault <= nxt_d_fault;

  assign ahb_hrespd = d_fault;

  assign hreadyoutd_err_en = (opt_mst_en &
                              (~hreadyoutd_err | nxt_d_fault_align |
                               (ahb_hreadyd & mtx_dpu_ahb_haddraccd_i &
                                nxt_d_fault_norm)));

  assign nxt_hreadyoutd_err = (~(nxt_d_fault_norm | nxt_d_fault_align) |
                               ~hreadyoutd_err);

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      hreadyoutd_err <= 1'b1;
    else if (hreadyoutd_err_en)
      hreadyoutd_err <= nxt_hreadyoutd_err;

  assign ahb_hreadyd = hreadyoutd_err & mtx_mpu_ahb_hreadyoutd_i;


  assign i_access     = dpu_ahb_htransi_i[1] & ~dpu_ahb_hproti_i[0];

  assign i_ppb_access = (dpu_ahb_haddri_i[31:20] == `MPUPPBREGION);
  assign i_def_active = ((mpu_enabled_i & def_mem_map_i & dpu_ahb_hproti_i[1]) |
                         ~mpu_enabled_i);

  assign i_fault_enable = (i_access & ~i_ppb_access &
                           (~i_def_active | iside_hit_i));

  assign nxt_i_access_fault = i_fault_enable & ~iside_hit_i & ~i_def_active;

  assign i_will_i_fault = attrsi_xn & iside_hit_i & i_access;

  assign i_will_p_fault = ((~dpu_ahb_hproti_i[1] & ~attrsi_ap[1] &
                            attrsi_ap[0]) |
                          (~attrsi_ap[2] & ~attrsi_ap[1] & ~attrsi_ap[0]));

  assign i_upper_fault = i_access & (dpu_ahb_haddri_i[31:29] == 3'b111);

  assign i_def_xn_fault = ((i_def_active & i_access &
                            (~mpu_enabled_i | ~iside_hit_i) &
                            ((dpu_ahb_haddri_i[30] & ~dpu_ahb_haddri_i[29]) |
                             (dpu_ahb_haddri_i[31] &  dpu_ahb_haddri_i[29]))));

  assign nxt_i_priv_fault = ((i_fault_enable & i_will_p_fault) |
                             i_will_i_fault | i_upper_fault | i_def_xn_fault);

  assign nxt_i_fault = nxt_i_access_fault | nxt_i_priv_fault;

  assign mpu_canceli = nxt_i_fault & hreadyouti_err;



  assign i_allow_drive = (mpu_enabled_i & iside_hit_i &
                          ~i_ppb_access & ~dpu_ahb_hproti_i[0]);

  assign i_outer_bufferable = ((~attrsi_tex[2] & attrsi_b) |
                               ( attrsi_tex[2] & attrsi_tex[0]) |
                               ((attrsi_tex[2:0] == 3'b010) & ~attrsi_c & ~attrsi_b));

  assign i_outer_cacheable  = ((~attrsi_tex[2] & attrsi_c) | attrsi_tex[2] |
                              ((attrsi_tex[2:0] == 3'b001) &
                               ~attrsi_c & ~attrsi_b));

  assign i_share    =  ((attrsi_s | (~attrsi_c & ~(|attrsi_tex))) &
                        ~((attrsi_tex == 3'b010) & ~attrsi_c & ~attrsi_b) &
                        (i_allow_drive | i_def_active));

  assign i_allocate = (((attrsi_tex == 3'b000) & attrsi_c) |
                       (attrsi_tex[2:1] == 2'b11));


  assign hproti_default[3] = ((~dpu_ahb_haddri_i[31] &  dpu_ahb_haddri_i[29]) |
                              (~dpu_ahb_haddri_i[30] & ~dpu_ahb_haddri_i[29]));

  assign hproti_default[2] = ((dpu_ahb_haddri_i[30] | dpu_ahb_haddri_i[29]) &
                              ~((&dpu_ahb_haddri_i[31:29]) &
                                ~(|dpu_ahb_haddri_i[28:20])));

  assign i_share_default = (dpu_ahb_haddri_i[31:29] == 3'b101);

  assign i_alloc_default = ((dpu_ahb_haddri_i[31:29] == 3'b000) |
                            (dpu_ahb_haddri_i[31:29] == 3'b100));


  assign ahb_hproti[3] = (i_allow_drive ?
                          i_outer_cacheable :
                          ((i_def_active | dpu_ahb_hproti_i[0]) &
                           hproti_default[3]));

  assign ahb_hproti[2] = (i_allow_drive ?
                          i_outer_bufferable :
                          ((i_def_active | dpu_ahb_hproti_i[0]) &
                           hproti_default[2]));

  assign ahb_hproti[1:0] = dpu_ahb_hproti_i[1:0];

  assign mem_attri[0] = (i_allow_drive ?
                         i_allocate :
                         ((i_def_active | dpu_ahb_hproti_i[0]) &
                          i_alloc_default));

  assign mem_attri[1] = (i_allow_drive ?
                         i_share :
                         ((i_def_active | dpu_ahb_hproti_i[0]) &
                          i_share_default));


  assign i_fault_en = (opt_mst_en & ahb_hreadyi &
                      ((mtx_dpu_ahb_haddracci_i & nxt_i_fault) | i_fault));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      i_fault <= 1'b0;
    else if (i_fault_en)
      i_fault <= ~i_fault | (mtx_dpu_ahb_haddracci_i & nxt_i_fault);

  assign ahb_hrespi = i_fault;

  assign hreadyouti_err_en = (opt_mst_en &
                              (~hreadyouti_err |
                               (ahb_hreadyi & mtx_dpu_ahb_haddracci_i &
                                nxt_i_fault)));

  assign nxt_hreadyouti_err = ~nxt_i_fault | ~hreadyouti_err;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      hreadyouti_err <= 1'b1;
    else if (hreadyouti_err_en)
      hreadyouti_err <= nxt_hreadyouti_err;

  assign ahb_hreadyi = hreadyouti_err & mtx_mpu_ahb_hreadyouti_i;

  assign mpu_mtx_ahb_hproti_o  = opt_ahb_hproti;
  assign mpu_dpu_ahb_hreadyi_o = opt_ahb_hreadyi;
  assign mpu_dpu_ahb_hrespi_o  = opt_ahb_hrespi;
  assign mpu_canceli_o         = opt_mpu_canceli;
  assign mpu_mem_attri_o       = opt_mem_attri;
  assign mpu_mtx_ahb_hprotd_o  = opt_ahb_hprotd;
  assign mpu_dpu_ahb_hreadyd_o = opt_ahb_hreadyd;
  assign mpu_dpu_ahb_hrespd_o  = opt_ahb_hrespd;
  assign mpu_canceld_o         = opt_mpu_canceld;
  assign mpu_mem_attrd_o       = opt_mem_attrd;
  assign mpu_addrd_non_buf_o   = opt_addrd_non_buf;


`ifdef ARM_ASSERT_ON

  assert_never #(0,0,"Cannot D-Side fault for normal PPB accesses")
    ovl_normal_ppb(hclk, hreset_n, (opt_mpu_canceld & d_ppb_access &
                                    d_fault_enable));

  assert_never #(0,0,"Cannot D-Side fault for unaligned PPB accesses")
    ovl_unaligned_ppb(hclk, hreset_n, (opt_mpu_canceld & d_ppb_align &
                                       d_enable_align));

  assert_never #(0,0,"Cannot I-Side fault for data fetches")
    ovl_i_vect(hclk, hreset_n, (opt_mpu_canceli & dpu_ahb_hproti_i[0]));

  assert_always #(0,0,"Always I-Side fault for PPB accesses")
    ovl_iside_ppb(hclk, hreset_n,
      ((i_ppb_access & i_access & hreadyouti_err & MPU_PRESENT) ?
        opt_mpu_canceli : 1'b1));

  assert_always #(0,0,"D-AHB ahb_hprotd[0] should always be 1 for any trans")
    ovl_hprotd0(hclk, hreset_n,
      ((dpu_ahb_htransd_i[1] & mtx_dpu_ahb_haddraccd_i &
        mtx_mpu_ahb_hreadyoutd_i) ? opt_ahb_hprotd[0] : 1'b1));

  assert_never #(0,0,"hreadyouti_err must only be deasserted when i_fault is set")
    ovl_hreadyi_err_1(hclk, hreset_n, (~hreadyouti_err & ~i_fault));

  assert_next #(0,1,0,0,0,"hreadyouti_err must only be deasserted for single cycle")
    ovl_hreadyi_err_2(hclk, hreset_n, (i_fault & ~hreadyouti_err), hreadyouti_err);

  assert_never #(0,0,"hreadyoutd_err must only be deasserted when d_fault is set")
    ovl_hreadyd_error_1(hclk, hreset_n, (~hreadyoutd_err & ~d_fault));

  assert_next #(0,1,0,0,0,"hreadyoutd_err must only be deasserted for a single cycle")
    ovl_hreadyd_error_2(hclk, hreset_n,
                        (d_fault & ~hreadyoutd_err), hreadyoutd_err);

  assert_never #(0,0,"Cannot fault vector fetches")
    ovl_vect_fetch(hclk, hreset_n, (nxt_i_fault & dpu_ahb_hproti_i[0]));

  assert_never #(0,0,"Cannot fault PPB accesses via D-AHB")
    ovl_ppb_access_1(hclk, hreset_n, (nxt_d_fault_norm & d_ppb_access));

  assert_never #(0,0,"Cannot fault unaligned PPB accesses via D-AHB")
    ovl_ppb_access_2(hclk, hreset_n, (nxt_d_fault_align & d_ppb_align));

  assert_never #(0,0,"I-AHB accesses to PPB must always fail")
    ovl_ppb_access_3(hclk, hreset_n,
      (i_ppb_access & i_access & mtx_mpu_ahb_hreadyouti_i &
        mtx_dpu_ahb_haddracci_i & ~nxt_i_fault));

  assert_next #(0,1,0,0,0,"canceld must only be asserted for a single cycle")
    ovl_mpu_canceld(hclk, hreset_n,
      (opt_mpu_canceld & mtx_mpu_ahb_hreadyoutd_i & mtx_dpu_ahb_haddraccd_i),
      ~opt_mpu_canceld);

  assert_next #(0,1,0,0,0,"canceli must only be asserted for a single cycle")
    ovl_mpu_canceli(hclk, hreset_n,
      (opt_mpu_canceli & mtx_mpu_ahb_hreadyouti_i & mtx_dpu_ahb_haddracci_i),
      ~opt_mpu_canceli);

  assert_never #(0,0,"check_alignedd should never be set if ahb_hreadyd is high")
    ovl_check_alignedd(hclk, hreset_n,
      (check_alignedd_i & ~d_ppb_align & mtx_mpu_ahb_hreadyoutd_i &
        ~(({dpu_ahb_haddrd_i[31], dpu_ahb_haddrd_i[28:25]} == 5'b00001) &
          (dpu_ahb_haddrd_i[30] ^ dpu_ahb_haddrd_i[29]))));

`endif

endmodule

`include "cm3_mpu_undefs.vp"
