module cm3_mpu_comp
  (// Inputs
   region_comp_en_i, region_perm_srd_i, rsize_mod_3_i, base_addr_i, ahb_haddr_i,
   region_mask_i,
   region_hit_o
  );


  parameter     MPU_PRESENT = 0;   // MPU present (0 = not present; 1 = present)


  input          region_comp_en_i; // Region comparator enable
  input    [7:0] region_perm_srd_i;// Permuted SRD attribute bits
  input    [2:0] rsize_mod_3_i;    // Combined region size mod 3 result
  input   [31:5] base_addr_i;      // Region base address
  input   [31:5] ahb_haddr_i;      // Address on which to match
  input   [31:5] region_mask_i;    // Address mask

  output         region_hit_o;     // Match occurred in this enabled region


  wire           addr_match;       // Address matched
  wire    [31:5] srd_amask;        // Mask to be applied to address for SRD
  wire    [31:5] masked_address;   // Masked address
  wire     [2:0] sr_sel;           // SRD select
  wire           region_hit;       // Got a match with enabled region & subregion
  wire           sub_enable;       // Subregion enable

  wire           opt_mpu_en;

`ifdef ARM_DSM

  wire           dsm_mpu_present;

  assign opt_mpu_en = (dsm_mpu_present)  ? 1'b1 : 1'b0;

`else

  assign opt_mpu_en = (MPU_PRESENT != 0) ? 1'b1 : 1'b0;

`endif



  assign srd_amask      = ~region_mask_i[31:5] & {3'b111, region_mask_i[31:8]};
  assign masked_address = ahb_haddr_i[31:5] & srd_amask[31:5];


  assign sr_sel[2]      = (masked_address[31] | masked_address[28] |
                           masked_address[25] | masked_address[22] |
                           masked_address[19] | masked_address[16] |
                           masked_address[13] | masked_address[10] |
                           masked_address[7]);

  assign sr_sel[1]      = (masked_address[30] | masked_address[27] |
                           masked_address[24] | masked_address[21] |
                           masked_address[18] | masked_address[15] |
                           masked_address[12] | masked_address[9]  |
                           masked_address[6]);

  assign sr_sel[0]      = (masked_address[29] | masked_address[26] |
                           masked_address[23] | masked_address[20] |
                           masked_address[17] | masked_address[14] |
                           masked_address[11] | masked_address[8]  |
                           masked_address[5]);

  assign sub_enable     = ~region_perm_srd_i[sr_sel];

  assign addr_match     = ~(|((ahb_haddr_i ^ base_addr_i) & region_mask_i));
  assign region_hit     = sub_enable & region_comp_en_i & addr_match;

  assign region_hit_o   = opt_mpu_en ? region_hit :  1'b0;

endmodule
