`include "cm3_mpu_defs.vp"

module cm3_mpu_default
  (// Inputs
   hreset_n, hclk, ppb_hsel_i, ppb_hprot_i, ppb_hready_i, dpu_ahb_haddri_i,
   mtx_dpu_ahb_haddracci_i, dpu_ahb_hproti_i, dpu_ahb_htransi_i,
   mtx_mpu_ahb_hreadyouti_i, dpu_ahb_haddrd_i, mtx_dpu_ahb_haddraccd_i,
   dpu_ahb_hprotd_i, dpu_ahb_htransd_i, dpu_ahb_hsized_i,
   mtx_mpu_ahb_hreadyoutd_i, nvic_no_def_wbuf_i,
   ppb_hresp_o, ppb_hrdata_o, ppb_hreadyout_o, mpu_mtx_ahb_hproti_o,
   mpu_dpu_ahb_hreadyi_o, mpu_dpu_ahb_hrespi_o, mpu_canceli_o,
   mpu_mtx_ahb_hprotd_o, mpu_dpu_ahb_hreadyd_o, mpu_dpu_ahb_hrespd_o,
   mpu_canceld_o, mpu_mem_attri_o, mpu_mem_attrd_o, mpu_addrd_non_buf_o
  );


  parameter     MPU_PRESENT = 0;   // MPU present (0 = not present; 1 = present)



  input         hclk;                     // AHB clock
  input         hreset_n;                 // AHB reset

  input         ppb_hsel_i;               // PPB select
  input [3:0]   ppb_hprot_i;              // PPB protection
  input         ppb_hready_i;             // PPB ready in
  output        ppb_hresp_o;              // PPB error response
  output [31:0] ppb_hrdata_o;             // PPB read data
  output        ppb_hreadyout_o;          // PPB ready out

  input [31:0]  dpu_ahb_haddri_i;         // I-AHB address in
  input [1:0]   dpu_ahb_htransi_i;        // I-AHB trans in
  input [1:0]   dpu_ahb_hproti_i;         // I-AHB protection in
  input         mtx_dpu_ahb_haddracci_i;  // I-AHB address accept in
  input         mtx_mpu_ahb_hreadyouti_i; // I-AHB ready in
  output [3:0]  mpu_mtx_ahb_hproti_o;     // I-AHB protection out
  output        mpu_dpu_ahb_hreadyi_o;    // I-AHB ready out
  output        mpu_dpu_ahb_hrespi_o;     // I-AHB error response out

  input [31:0]  dpu_ahb_haddrd_i;         // D-AHB address in
  input [1:0]   dpu_ahb_htransd_i;        // D-AHB trans in
  input [1:0]   dpu_ahb_hsized_i;         // D-AHB size in
  input [1:0]   dpu_ahb_hprotd_i;         // D-AHB protection in
  input         mtx_dpu_ahb_haddraccd_i;  // D-AHB address accept in
  input         mtx_mpu_ahb_hreadyoutd_i; // D-AHB ready in
  input         nvic_no_def_wbuf_i;       // Write buffer disable
  output [3:0]  mpu_mtx_ahb_hprotd_o;     // D-AHB protection out
  output        mpu_dpu_ahb_hreadyd_o;    // D-AHB ready out
  output        mpu_dpu_ahb_hrespd_o;     // D-AHB error response out

  output        mpu_canceli_o;            // I-AHB Cancel access
  output        mpu_canceld_o;            // D-AHB Cancel access

  output [1:0]  mpu_mem_attri_o;          // I-AHB Memory Attributes
  output [1:0]  mpu_mem_attrd_o;          // D-AHB Memory Attributes
  output        mpu_addrd_non_buf_o;      // D-AHB address non-buffable



  wire [3:0]    ahb_hproti;
  wire          ahb_hreadyi;
  wire          ahb_hrespi;
  wire          mpu_canceli;
  wire [1:0]    mem_attri;
  wire [3:0]    ahb_hprotd;
  wire          ahb_hreadyd;
  wire [1:0]    mem_attrd;
  wire          addrd_non_buf;

  wire          region_end;
  wire          c_boundary_end;           // Cacheable
  wire          b_boundary_end;           // Bufferable
  wire          s_boundary_end;           // Shareable
  wire          a_boundary_end;           // Allocate

  wire          unaligned;
  reg           aligned_drive;
  wire          nxt_aligned_drive;
  wire          aligned_drive_en;

  reg           ppb_hresp;
  reg           ppb_hreadyout;
  wire          hresp_en;
  wire          nxt_ppb_hresp;
  wire          user_wr;
  wire          ppb_hreadyout_en;
  wire          nxt_ppb_hreadyout;

  wire          cacheablei;
  wire          cacheabled;
  wire          bufferablei;
  wire          bufferabled;
  wire          shareablei;
  wire          shareabled;
  wire          allocatei;
  wire          allocated;

  reg           cacheabled_algn;
  wire          nxt_cacheabled_algn;
  wire          cacheabled_algn_en;
  reg           bufferabled_algn;
  wire          nxt_bufferabled_algn;
  wire          bufferabled_algn_en;
  reg           shareabled_algn;
  wire          nxt_shareabled_algn;
  wire          shareabled_algn_en;
  reg           allocated_algn;
  wire          nxt_allocated_algn;
  wire          allocated_algn_en;

  wire          default_xn_faulti;
  wire          hreadyouti_err_en;
  wire          nxt_xn_faulti;
  wire          xn_faulti_en;
  wire          nxt_hreadyouti_err;
  reg           xn_faulti;
  reg           hreadyouti_err;

  wire          opt_ppb_hresp;
  wire          opt_ppb_hreadyout;
  wire [3:0]    opt_ahb_hproti;
  wire          opt_ahb_hreadyi;
  wire          opt_ahb_hrespi;
  wire          opt_mpu_canceli;
  wire [1:0]    opt_mem_attri;
  wire [3:0]    opt_ahb_hprotd;
  wire          opt_ahb_hreadyd;
  wire [1:0]    opt_mem_attrd;
  wire          opt_addrd_non_buf;
  wire          opt_mpu_canceld;
  wire          opt_mst_en;
  wire          opt_mpu_en;

`ifdef ARM_DSM

  wire              dsm_mpu_present;

  assign opt_mpu_en = (dsm_mpu_present)  ? 1'b1 : 1'b0;

`else

  assign opt_mpu_en = (MPU_PRESENT != 0) ? 1'b1 : 1'b0;

`endif


  assign opt_ppb_hresp     = (~opt_mpu_en) ? ppb_hresp     : 1'b0;
  assign opt_ppb_hreadyout = (~opt_mpu_en) ? ppb_hreadyout : 1'b1;
  assign opt_ahb_hproti    = (~opt_mpu_en) ? ahb_hproti    : 4'b0000;
  assign opt_ahb_hreadyi   = (~opt_mpu_en) ? ahb_hreadyi   : 1'b0;
  assign opt_ahb_hrespi    = (~opt_mpu_en) ? ahb_hrespi    : 1'b0;
  assign opt_mpu_canceli   = (~opt_mpu_en) ? mpu_canceli   : 1'b0;
  assign opt_mem_attri     = (~opt_mpu_en) ? mem_attri     : 2'b00;
  assign opt_ahb_hprotd    = (~opt_mpu_en) ? ahb_hprotd    : 4'b0001;
  assign opt_ahb_hreadyd   = (~opt_mpu_en) ? ahb_hreadyd   : 1'b0;
  assign opt_mem_attrd     = (~opt_mpu_en) ? mem_attrd     : 2'b00;
  assign opt_addrd_non_buf = (~opt_mpu_en) ? addrd_non_buf : 1'b0;
  assign opt_mst_en        = (~opt_mpu_en) ? 1'b1          : 1'b0;

  assign opt_mpu_canceld   = 1'b0;


  assign nxt_ppb_hreadyout = ~ppb_hreadyout;
  assign ppb_hreadyout_en  = opt_mst_en &
                              ((ppb_hready_i & ppb_hsel_i & ~ppb_hprot_i[1]) |
                               ~opt_ppb_hreadyout);

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ppb_hreadyout <= 1'b1;
    else if (ppb_hreadyout_en)
      ppb_hreadyout <= nxt_ppb_hreadyout;

  assign nxt_ppb_hresp = (~ppb_hresp) | user_wr;
  assign user_wr       = ppb_hready_i & ppb_hsel_i & ~ppb_hprot_i[1];
  assign hresp_en      = opt_mst_en & (ppb_hreadyout & (opt_ppb_hresp | user_wr));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ppb_hresp <= 1'b0;
    else if (hresp_en)
      ppb_hresp <= nxt_ppb_hresp;


  assign unaligned = ((&dpu_ahb_haddrd_i[1:0]) & (dpu_ahb_hsized_i == 2'b01)) |
                     ((|dpu_ahb_haddrd_i[1:0]) & (dpu_ahb_hsized_i == 2'b10));

  assign nxt_aligned_drive = (dpu_ahb_htransd_i[1]     &
                              mtx_dpu_ahb_haddraccd_i  &
                              mtx_mpu_ahb_hreadyoutd_i &
                              unaligned                &
                             ~(dpu_ahb_haddrd_i[31:20] == `MPUPPBREGION));

  assign aligned_drive_en  = (nxt_aligned_drive | aligned_drive) & opt_mst_en;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      aligned_drive <= 1'b0;
    else if (aligned_drive_en)
      aligned_drive <= nxt_aligned_drive;


  assign region_end = (&dpu_ahb_haddrd_i[27:2]);


  assign cacheabled = ((~dpu_ahb_haddrd_i[31] &  dpu_ahb_haddrd_i[29]) |
                       (~dpu_ahb_haddrd_i[30] & ~dpu_ahb_haddrd_i[29]));

  assign cacheablei = ((~dpu_ahb_haddri_i[31] &  dpu_ahb_haddri_i[29]) |
                       (~dpu_ahb_haddri_i[30] & ~dpu_ahb_haddri_i[29]));


  assign c_boundary_end      = (region_end &
                                ((dpu_ahb_haddrd_i[31:28] == 4'b0011) |
                                 (dpu_ahb_haddrd_i[31:28] == 4'b0101) |
                                 (dpu_ahb_haddrd_i[31:28] == 4'b1001) |
                                 (dpu_ahb_haddrd_i[31:28] == 4'b1111)));

  assign nxt_cacheabled_algn = c_boundary_end ^ cacheabled;
  assign cacheabled_algn_en  = opt_mst_en & nxt_aligned_drive;

  always @ (posedge hclk or negedge hreset_n) begin
    if (!hreset_n)
      cacheabled_algn <= 1'b0;
    else if (cacheabled_algn_en)
      cacheabled_algn <= nxt_cacheabled_algn;
  end

  assign ahb_hprotd[3] = aligned_drive ? cacheabled_algn : cacheabled;
  assign ahb_hproti[3] = cacheablei;


  assign bufferabled = ((dpu_ahb_haddrd_i[30] | dpu_ahb_haddrd_i[29]) &
                        ~((&dpu_ahb_haddrd_i[31:29]) &
                          ~(|dpu_ahb_haddrd_i[28:20])));

  assign bufferablei = ((dpu_ahb_haddri_i[30] | dpu_ahb_haddri_i[29]) &
                        ~((&dpu_ahb_haddri_i[31:29]) &
                          ~(|dpu_ahb_haddri_i[28:20])));


  assign b_boundary_end       = (region_end &
                                 ((dpu_ahb_haddrd_i[31:28] == 4'b0001) |
                                  (dpu_ahb_haddrd_i[31:28] == 4'b0111) |
                                  (dpu_ahb_haddrd_i[31:28] == 4'b1001) |
                                  (dpu_ahb_haddrd_i[31:28] == 4'b1101) |
                                  (dpu_ahb_haddrd_i[31:28] == 4'b1111)));

  assign nxt_bufferabled_algn = b_boundary_end ^ bufferabled;
  assign bufferabled_algn_en  = opt_mst_en & nxt_aligned_drive;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      bufferabled_algn <= 1'b0;
    else if (bufferabled_algn_en)
      bufferabled_algn <= nxt_bufferabled_algn;

  assign ahb_hprotd[2] = aligned_drive ? bufferabled_algn : bufferabled;
  assign ahb_hproti[2] = bufferablei;


  assign shareabled = (dpu_ahb_haddrd_i[31:29] == 3'b101);
  assign shareablei = (dpu_ahb_haddri_i[31:29] == 3'b101);


  assign s_boundary_end      = (region_end &
                                ((dpu_ahb_haddrd_i[31:28] == 4'b1001) |
                                 (dpu_ahb_haddrd_i[31:28] == 4'b1011)));

  assign nxt_shareabled_algn = s_boundary_end ^ shareabled;
  assign shareabled_algn_en  = opt_mst_en & nxt_aligned_drive;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      shareabled_algn <= 1'b0;
    else if (shareabled_algn_en)
      shareabled_algn <= nxt_shareabled_algn;

  assign mem_attrd[1] = aligned_drive ? shareabled_algn : shareabled;
  assign mem_attri[1] = shareablei;


  assign allocated = ((dpu_ahb_haddrd_i[31:29] == 3'b000) |
                      (dpu_ahb_haddrd_i[31:29] == 3'b100));
  assign allocatei = ((dpu_ahb_haddri_i[31:29] == 3'b000) |
                      (dpu_ahb_haddri_i[31:29] == 3'b100));


  assign a_boundary_end     = (region_end &
                               ((dpu_ahb_haddrd_i[31:28] == 4'b0001) |
                                (dpu_ahb_haddrd_i[31:28] == 4'b0111) |
                                (dpu_ahb_haddrd_i[31:28] == 4'b1001) |
                                (dpu_ahb_haddrd_i[31:28] == 4'b1111)));

  assign nxt_allocated_algn = a_boundary_end ^ allocated;
  assign allocated_algn_en  = opt_mst_en & nxt_aligned_drive;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      allocated_algn <= 1'b0;
    else if (allocated_algn_en)
      allocated_algn <= nxt_allocated_algn;

  assign mem_attrd[0] = aligned_drive ? allocated_algn : allocated;
  assign mem_attri[0] = allocatei;

  assign addrd_non_buf = nvic_no_def_wbuf_i | ~(|ahb_hprotd[3:2]);


  assign default_xn_faulti = (dpu_ahb_htransi_i[1] &
                              ~dpu_ahb_hproti_i[0] &
                              ((dpu_ahb_haddri_i[30] & ~dpu_ahb_haddri_i[29]) |
                               (dpu_ahb_haddri_i[31] &  dpu_ahb_haddri_i[29])));

  assign xn_faulti_en  = (opt_mst_en &
                          ((ahb_hreadyi & xn_faulti) |
                           (ahb_hreadyi & mtx_dpu_ahb_haddracci_i &
                            default_xn_faulti)));

  assign nxt_xn_faulti = mtx_dpu_ahb_haddracci_i & default_xn_faulti;

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      xn_faulti <= 1'b0;
    else if (xn_faulti_en)
      xn_faulti <= nxt_xn_faulti;

  assign mpu_canceli = default_xn_faulti & hreadyouti_err;

  assign ahb_hrespi = xn_faulti;

  assign hreadyouti_err_en  = (opt_mst_en &
                              (~hreadyouti_err |
                               (ahb_hreadyi & mtx_dpu_ahb_haddracci_i &
                                default_xn_faulti)));

  assign nxt_hreadyouti_err = (~hreadyouti_err |
                               (ahb_hreadyi & mtx_dpu_ahb_haddracci_i &
                                ~default_xn_faulti));

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      hreadyouti_err <= 1'b1;
    else if (hreadyouti_err_en)
      hreadyouti_err <= nxt_hreadyouti_err;

  assign ahb_hreadyi = hreadyouti_err & mtx_mpu_ahb_hreadyouti_i;


  assign ahb_hproti[1:0] = dpu_ahb_hproti_i;
  assign ahb_hprotd[1:0] = dpu_ahb_hprotd_i;

  assign ahb_hreadyd = mtx_mpu_ahb_hreadyoutd_i;


  assign ppb_hrdata_o          = 32'h00000000;    // No read data - always error
  assign ppb_hresp_o           = opt_ppb_hresp;
  assign ppb_hreadyout_o       = opt_ppb_hreadyout;

  assign mpu_mtx_ahb_hproti_o  = opt_ahb_hproti;
  assign mpu_dpu_ahb_hreadyi_o = opt_ahb_hreadyi;
  assign mpu_dpu_ahb_hrespi_o  = opt_ahb_hrespi;

  assign mpu_mtx_ahb_hprotd_o  = opt_ahb_hprotd;
  assign mpu_dpu_ahb_hreadyd_o = opt_ahb_hreadyd;
  assign mpu_dpu_ahb_hrespd_o  = 1'b0;

  assign mpu_canceli_o         = opt_mpu_canceli;
  assign mpu_canceld_o         = opt_mpu_canceld;

  assign mpu_mem_attri_o       = opt_mem_attri;
  assign mpu_mem_attrd_o       = opt_mem_attrd;
  assign mpu_addrd_non_buf_o   = opt_addrd_non_buf;


`ifdef ARM_ASSERT_ON

  reg [3:2] ovl_hprotd_reg;
  reg [1:0] ovl_mem_addrd_reg;

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      begin
        ovl_hprotd_reg[3:2] <= 2'b00;
        ovl_mem_addrd_reg   <= 2'b00;
      end
    else
      begin
        ovl_hprotd_reg[3:2] <= opt_ahb_hprotd[3:2];
        ovl_mem_addrd_reg   <= opt_mem_attrd;
      end

  assert_never #(0,0,"ppb_hreadyout must only be deasserted when ppb_hresp is set")
    ovl_nompu_phreadyout_1(hclk, hreset_n, ~opt_ppb_hreadyout & ~opt_ppb_hresp);

  assert_next #(0,1,0,0,0, "ppb_hreadyout must only be deasserted for a single cycle")
    ovl_nompu_phreadyout_2(hclk, hreset_n, opt_ppb_hresp & ~opt_ppb_hreadyout,
                             opt_ppb_hreadyout);

  assert_always #(0,0,"D-AHB HPROT[0] should always be 1 for any transaction")
    ovl_hprotd0(hclk, hreset_n,
      (dpu_ahb_htransd_i[1] & mtx_dpu_ahb_haddraccd_i & mtx_mpu_ahb_hreadyoutd_i) ?
        opt_ahb_hprotd[0] : 1'b1);

  assert_next #(0,1,0,0,0, "mpu_canceli must only be asserted for a single cycle")
    ovl_mpu_canceli(hclk, hreset_n,
      (opt_mpu_canceli & mtx_mpu_ahb_hreadyouti_i & mtx_dpu_ahb_haddracci_i),
        ~opt_mpu_canceli);

  assert_never #(0,0,"mpu_canceld must never be asserted for the default MPU")
    ovl_mpu_canceld(hclk, hreset_n, opt_mpu_canceld);

  assert_next #(0,1,0,0,0,
    "ahb_hprotd[3] must remain constant for an unaligned access within a region")
     ovl_hprotd3(hclk, hreset_n, nxt_aligned_drive & ~c_boundary_end,
                   (opt_ahb_hprotd[3] == ovl_hprotd_reg[3]));

  assert_next #(0,1,0,0,0,
    "ahb_hprotd[2] must remain constant for an unaligned access within a region")
     ovl_hprotd2(hclk, hreset_n, nxt_aligned_drive & ~b_boundary_end,
                   (opt_ahb_hprotd[2] == ovl_hprotd_reg[2]));

  assert_next #(0,1,0,0,0,
    "mem_attrd must remain constant for an unaligned access within a region")
     ovl_mem_attrd(hclk, hreset_n, nxt_aligned_drive & ~region_end,
                    (opt_mem_attrd == ovl_mem_addrd_reg));

`endif

endmodule

`include "cm3_mpu_undefs.vp"
