module cm3_mpu_full
  (// Inputs
   hreset_n, hclk, pclk, mpu_disable_i, ppb_hsel_i, ppb_hwrite_i, ppb_hsize_i,
   ppb_haddr_i, ppb_hprot_i, ppb_hwdata_i, ppb_hready_i, dpu_ahb_haddri_i,
   mtx_dpu_ahb_haddracci_i, dpu_ahb_hproti_i, dpu_ahb_htransi_i,
   mtx_mpu_ahb_hreadyouti_i, dpu_ahb_haddrd_i, mtx_dpu_ahb_haddraccd_i,
   dpu_ahb_hprotd_i, dpu_ahb_hwrited_i, dpu_ahb_htransd_i, dpu_ahb_hsized_i,
   mtx_mpu_ahb_hreadyoutd_i, nvic_int_mpu_nmihf_i, nvic_no_def_wbuf_i,
   ppb_hresp_o, ppb_hrdata_o, ppb_hreadyout_o, mpu_mtx_ahb_hproti_o,
   mpu_dpu_ahb_hreadyi_o, mpu_dpu_ahb_hrespi_o, mpu_canceli_o,
   mpu_mtx_ahb_hprotd_o, mpu_dpu_ahb_hreadyd_o, mpu_dpu_ahb_hrespd_o,
   mpu_canceld_o, mpu_mem_attri_o, mpu_mem_attrd_o, mpu_addrd_non_buf_o
  );


  parameter     MPU_PRESENT = 0;   // MPU present (0 = not present; 1 = present)



  input         pclk;                     // PPB clock
  input         hclk;                     // AHB clock
  input         hreset_n;                 // AHB reset

  input         mpu_disable_i;            // Disable the MPU (act as default)

  input         ppb_hsel_i;               // PPB select
  input  [11:0] ppb_haddr_i;              // PPB address
  input   [1:0] ppb_hsize_i;              // PPB size
  input   [3:0] ppb_hprot_i;              // PPB protection
  input         ppb_hwrite_i;             // PPB write
  input  [31:0] ppb_hwdata_i;             // PPB write data
  input         ppb_hready_i;             // PPB ready in
  output        ppb_hresp_o;              // PPB error response
  output [31:0] ppb_hrdata_o;             // PPB read data
  output        ppb_hreadyout_o;          // PPB ready out

  input  [31:0] dpu_ahb_haddri_i;         // I-AHB address in
  input   [1:0] dpu_ahb_htransi_i;        // I-AHB trans in
  input   [1:0] dpu_ahb_hproti_i;         // I-AHB protection in
  input         mtx_dpu_ahb_haddracci_i;  // I-AHB address accept in
  input         mtx_mpu_ahb_hreadyouti_i; // I-AHB ready in
  output        mpu_dpu_ahb_hreadyi_o;    // I-AHB ready out, to core
  output        mpu_dpu_ahb_hrespi_o;     // I-AHB error response out, to core
  output  [3:0] mpu_mtx_ahb_hproti_o;     // I-AHB protection out, to matrix

  input  [31:0] dpu_ahb_haddrd_i;         // D-AHB address in
  input   [1:0] dpu_ahb_htransd_i;        // D-AHB trans in
  input   [1:0] dpu_ahb_hsized_i;         // D-AHB size in
  input   [1:0] dpu_ahb_hprotd_i;         // D-AHB protection in
  input         dpu_ahb_hwrited_i;        // D-AHB write in
  input         mtx_dpu_ahb_haddraccd_i;  // D-AHB address accept in
  input         mtx_mpu_ahb_hreadyoutd_i; // D-AHB ready in
  output        mpu_dpu_ahb_hreadyd_o;    // D-AHB ready out, to core
  output        mpu_dpu_ahb_hrespd_o;     // D-AHB error response out, to core
  output  [3:0] mpu_mtx_ahb_hprotd_o;     // D-AHB protection out, to matrix

  input         nvic_int_mpu_nmihf_i;     // In NMI or Hard Fault
  input         nvic_no_def_wbuf_i;       // Write buffer disable

  output        mpu_canceli_o;            // I-AHB Cancel access
  output        mpu_canceld_o;            // D-AHB Cancel access

  output  [1:0] mpu_mem_attri_o;          // I-AHB Memory Attributes
  output  [1:0] mpu_mem_attrd_o;          // D-AHB Memory Attributes
  output        mpu_addrd_non_buf_o;      // D-AHB address non-buffable


  wire    [7:0] rgn_select_oh;
  wire    [7:0] rgn_select_wr;
  wire          rgn_attrs_sel;
  wire          rgn_base_addr_sel;
  wire   [31:0] rgn_nxt_wdata;
  wire    [3:0] rgn_byte_wr;
  wire          mpu_enabled;
  wire          def_mem_map;
  wire          ppb_hresp;
  wire   [31:0] ppb_hrdata;
  wire          ppb_hreadyout;

  wire   [31:5] aligned_addrd;
  wire          aligned_writed;
  wire          aligned_privd;
  wire          check_alignedd;
  wire          mask_canceld;

  wire   [31:0] rgn_hrdata;
  wire    [8:0] attrsd;
  wire    [9:0] attrsi;
  wire          iside_hit;
  wire          dside_hit;
  wire          priv_faultd;

  wire    [3:0] mpu_mtx_ahb_hproti;
  wire    [3:0] mpu_mtx_ahb_hprotd;
  wire          mpu_dpu_ahb_hreadyi;
  wire          mpu_dpu_ahb_hreadyd;
  wire          mpu_dpu_ahb_hrespi;
  wire          mpu_dpu_ahb_hrespd;
  wire          mpu_canceli;
  wire          mpu_canceld;
  wire    [1:0] mpu_mem_attri;
  wire    [1:0] mpu_mem_attrd;
  wire          mpu_addrd_non_buf;

  wire          opt_ppb_hresp;
  wire   [31:0] opt_ppb_hrdata;
  wire          opt_ppb_hreadyout;
  wire    [3:0] opt_mpu_mtx_ahb_hproti;
  wire          opt_mpu_dpu_ahb_hreadyi;
  wire          opt_mpu_dpu_ahb_hrespi;
  wire          opt_mpu_canceli;
  wire    [1:0] opt_mpu_mem_attri;
  wire    [3:0] opt_mpu_mtx_ahb_hprotd;
  wire          opt_mpu_dpu_ahb_hreadyd;
  wire          opt_mpu_dpu_ahb_hrespd;
  wire          opt_mpu_canceld;
  wire    [1:0] opt_mpu_mem_attrd;
  wire          opt_mpu_addrd_non_buf;
  wire          opt_mpu_en;

`ifdef ARM_DSM

  wire          dsm_mpu_present;

  assign opt_mpu_en = (dsm_mpu_present)  ? 1'b1 : 1'b0;

`else

  assign opt_mpu_en = (MPU_PRESENT != 0) ? 1'b1 : 1'b0;

`endif

  assign opt_ppb_hresp           = (opt_mpu_en) ? ppb_hresp           : 1'b0;
  assign opt_ppb_hrdata          = (opt_mpu_en) ? ppb_hrdata          : {32{1'b0}};
  assign opt_ppb_hreadyout       = (opt_mpu_en) ? ppb_hreadyout       : 1'b0;
  assign opt_mpu_mtx_ahb_hproti  = (opt_mpu_en) ? mpu_mtx_ahb_hproti  : 4'b0000;
  assign opt_mpu_dpu_ahb_hreadyi = (opt_mpu_en) ? mpu_dpu_ahb_hreadyi : 1'b0;
  assign opt_mpu_dpu_ahb_hrespi  = (opt_mpu_en) ? mpu_dpu_ahb_hrespi  : 1'b0;
  assign opt_mpu_canceli         = (opt_mpu_en) ? mpu_canceli         : 1'b0;
  assign opt_mpu_mem_attri       = (opt_mpu_en) ? mpu_mem_attri       : 2'b00;
  assign opt_mpu_mtx_ahb_hprotd  = (opt_mpu_en) ? mpu_mtx_ahb_hprotd  : 4'b0000;
  assign opt_mpu_dpu_ahb_hreadyd = (opt_mpu_en) ? mpu_dpu_ahb_hreadyd : 1'b0;
  assign opt_mpu_dpu_ahb_hrespd  = (opt_mpu_en) ? mpu_dpu_ahb_hrespd  : 1'b0;
  assign opt_mpu_canceld         = (opt_mpu_en) ? mpu_canceld         : 1'b0;
  assign opt_mpu_mem_attrd       = (opt_mpu_en) ? mpu_mem_attrd       : 2'b00;
  assign opt_mpu_addrd_non_buf   = (opt_mpu_en) ? mpu_addrd_non_buf   : 1'b0;


  cm3_mpu_ppb_intf #(MPU_PRESENT)
    u_cm3_mpu_ppb_intf
      (// Inputs
       .pclk                      (pclk),
       .hclk                      (hclk),
       .hreset_n                  (hreset_n),
       .mpu_disable_i             (mpu_disable_i),
       .ppb_hready_i              (ppb_hready_i),
       .ppb_hsel_i                (ppb_hsel_i),
       .ppb_hwrite_i              (ppb_hwrite_i),
       .ppb_haddr_i               (ppb_haddr_i),
       .ppb_hsize_i               (ppb_hsize_i),
       .ppb_hwdata_i              (ppb_hwdata_i),
       .ppb_hprot_i               (ppb_hprot_i),
       .rgn_hrdata_i              (rgn_hrdata),
       .nvic_int_mpu_nmihf_i      (nvic_int_mpu_nmihf_i),
       .rgn_select_oh_o           (rgn_select_oh),
       .rgn_select_wr_o           (rgn_select_wr),
       .rgn_base_addr_sel_o       (rgn_base_addr_sel),
       .rgn_nxt_wdata_o           (rgn_nxt_wdata),
       .rgn_attrs_sel_o           (rgn_attrs_sel),
       .rgn_byte_wr_o             (rgn_byte_wr),
       .ppb_hrdata_o              (ppb_hrdata),
       .ppb_hreadyout_o           (ppb_hreadyout),
       .ppb_hresp_o               (ppb_hresp),
       .mpu_enabled_o             (mpu_enabled),
       .def_mem_map_o             (def_mem_map)
      );


  cm3_mpu_align #(MPU_PRESENT)
    u_cm3_mpu_align
      (// Inputs
       .hclk                      (hclk),
       .hreset_n                  (hreset_n),
       .dpu_ahb_haddr_i           (dpu_ahb_haddrd_i),
       .dpu_ahb_hsize_i           (dpu_ahb_hsized_i),
       .dpu_ahb_htrans_i          (dpu_ahb_htransd_i),
       .dpu_ahb_hwrite_i          (dpu_ahb_hwrited_i),
       .dpu_ahb_hprot_i           (dpu_ahb_hprotd_i),
       .mtx_dpu_ahb_haddracc_i    (mtx_dpu_ahb_haddraccd_i),
       .mpu_dpu_ahb_hready_i      (mpu_dpu_ahb_hreadyd),
       .mpu_cancel_i              (mpu_canceld),
       .aligned_addr_o            (aligned_addrd),
       .check_aligned_o           (check_alignedd),
       .aligned_write_o           (aligned_writed),
       .aligned_priv_o            (aligned_privd),
       .mask_cancel_o             (mask_canceld)
      );


  cm3_mpu_regions #(MPU_PRESENT)
    u_cm3_mpu_regions
      (// Inputs
       .hreset_n                  (hreset_n),
       .pclk                      (pclk),
       .hclk                      (hclk),
       .rgn_select_oh_i           (rgn_select_oh),
       .rgn_select_wr_i           (rgn_select_wr),
       .rgn_base_addr_sel_i       (rgn_base_addr_sel),
       .rgn_nxt_wdata_i           (rgn_nxt_wdata),
       .rgn_attrs_sel_i           (rgn_attrs_sel),
       .rgn_byte_wr_i             (rgn_byte_wr),
       .mpu_enabled_i             (mpu_enabled),
       .dpu_ahb_haddri_i          (dpu_ahb_haddri_i),
       .dpu_ahb_haddrd_i          (dpu_ahb_haddrd_i),
       .dpu_ahb_hprotd_i          (dpu_ahb_hprotd_i),
       .dpu_ahb_hwrited_i         (dpu_ahb_hwrited_i),
       .aligned_addrd_i           (aligned_addrd),
       .check_alignedd_i          (check_alignedd),
       .aligned_writed_i          (aligned_writed),
       .aligned_privd_i           (aligned_privd),
       .rgn_hrdata_o              (rgn_hrdata),
       .dside_hit_o               (dside_hit),
       .iside_hit_o               (iside_hit),
       .attrsd_o                  (attrsd),
       .attrsi_o                  (attrsi),
       .priv_faultd_o             (priv_faultd)
      );


  cm3_mpu_ahb_ctl #(MPU_PRESENT)
    u_cm3_mpu_ahb_ctl
      (// Inputs
       .hclk                      (hclk),
       .hreset_n                  (hreset_n),
       .dpu_ahb_haddri_i          (dpu_ahb_haddri_i),
       .dpu_ahb_hproti_i          (dpu_ahb_hproti_i),
       .dpu_ahb_htransi_i         (dpu_ahb_htransi_i),
       .mtx_dpu_ahb_haddracci_i   (mtx_dpu_ahb_haddracci_i),
       .mtx_mpu_ahb_hreadyouti_i  (mtx_mpu_ahb_hreadyouti_i),
       .dpu_ahb_haddrd_i          (dpu_ahb_haddrd_i),
       .dpu_ahb_hprotd_i          (dpu_ahb_hprotd_i),
       .dpu_ahb_hwrited_i         (dpu_ahb_hwrited_i),
       .dpu_ahb_htransd_i         (dpu_ahb_htransd_i),
       .mtx_dpu_ahb_haddraccd_i   (mtx_dpu_ahb_haddraccd_i),
       .mtx_mpu_ahb_hreadyoutd_i  (mtx_mpu_ahb_hreadyoutd_i),
       .dside_hit_i               (dside_hit),
       .iside_hit_i               (iside_hit),
       .attrsd_i                  (attrsd),
       .attrsi_i                  (attrsi),
       .mpu_enabled_i             (mpu_enabled),
       .def_mem_map_i             (def_mem_map),
       .aligned_addrd_i           (aligned_addrd),
       .aligned_writed_i          (aligned_writed),
       .aligned_privd_i           (aligned_privd),
       .check_alignedd_i          (check_alignedd),
       .mask_canceld_i            (mask_canceld),
       .nvic_no_def_wbuf_i        (nvic_no_def_wbuf_i),
       .priv_faultd_i             (priv_faultd),
       .mpu_mtx_ahb_hproti_o      (mpu_mtx_ahb_hproti),
       .mpu_dpu_ahb_hreadyi_o     (mpu_dpu_ahb_hreadyi),
       .mpu_dpu_ahb_hrespi_o      (mpu_dpu_ahb_hrespi),
       .mpu_canceli_o             (mpu_canceli),
       .mpu_mem_attri_o           (mpu_mem_attri),
       .mpu_mtx_ahb_hprotd_o      (mpu_mtx_ahb_hprotd),
       .mpu_dpu_ahb_hreadyd_o     (mpu_dpu_ahb_hreadyd),
       .mpu_dpu_ahb_hrespd_o      (mpu_dpu_ahb_hrespd),
       .mpu_canceld_o             (mpu_canceld),
       .mpu_mem_attrd_o           (mpu_mem_attrd),
       .mpu_addrd_non_buf_o       (mpu_addrd_non_buf)
      );

  assign ppb_hresp_o           = opt_ppb_hresp;
  assign ppb_hrdata_o          = opt_ppb_hrdata;
  assign ppb_hreadyout_o       = opt_ppb_hreadyout;
  assign mpu_mtx_ahb_hproti_o  = opt_mpu_mtx_ahb_hproti;
  assign mpu_dpu_ahb_hreadyi_o = opt_mpu_dpu_ahb_hreadyi;
  assign mpu_dpu_ahb_hrespi_o  = opt_mpu_dpu_ahb_hrespi;
  assign mpu_canceli_o         = opt_mpu_canceli;
  assign mpu_mem_attri_o       = opt_mpu_mem_attri;
  assign mpu_mtx_ahb_hprotd_o  = opt_mpu_mtx_ahb_hprotd;
  assign mpu_dpu_ahb_hreadyd_o = opt_mpu_dpu_ahb_hreadyd;
  assign mpu_dpu_ahb_hrespd_o  = opt_mpu_dpu_ahb_hrespd;
  assign mpu_canceld_o         = opt_mpu_canceld;
  assign mpu_mem_attrd_o       = opt_mpu_mem_attrd;
  assign mpu_addrd_non_buf_o   = opt_mpu_addrd_non_buf;

endmodule
