`include "cm3_mtx_defs.vp"

module cm3_mtx_bit_master
  (// Inputs
   hclk, hreset_n, data_be8_i, dpu_dbg_snapstall_i, dap_ahb_habort_i,
   mpu_canceli_i, mpu_canceld_i, fpb_matched_i, fpb_matched_addri_i,
   fpb_matched_addrd_i, fpb_matched_proti_i, fpb_matched_mem_attri_i,
   fpb_matched_protd_i, fpb_matched_mem_attrd_i, fpb_matched_exreqd_i,
   fpb_matched_sized_i, fpb_remap_addr_i, is_dap_master_i, is_dcore_addr_i,
   is_icore_addr_i, is_dap_addr_i, is_dcore_prot_i, is_icore_prot_i,
   is_dap_prot_i, is_dcore_mem_attr_i, is_icore_mem_attr_i,
   is_dcore_size_i, is_dap_size_i, dpu_ahb_hwdatad_i, dap_ahb_hwdata_i,
   is_dcore_exreq_i, is_dcore_held_trans_i, is_icore_held_trans_i,
   is_dap_held_trans_i, is_dcore_buffered_write_i, dec_dcore_sel_dcode_i,
   dec_dcore_sel_sys_i, dec_dcore_sel_bb_i, dec_icore_sel_sys_i,
   dec_icore_sel_sys_held_i, dec_dap_sel_dcode_i, dec_dap_sel_sys_i,
   dec_dap_sel_bb_i, os_dcore_to_dcode_accept_i, os_dcore_to_sys_accept_i,
   os_dap_to_dcode_accept_i, os_dap_to_dcode_held_req_i, os_dap_to_sys_accept_i,
   os_sys_bm_wait_i, os_dap_to_sys_held_req_i, dec_dcore_active_data_i,
   is_dcore_unalign_i, is_dap_unalign_i, is_dcore_write_i, is_dap_write_i,
   mtx_ahb_hreadyd_i, mtx_ahb_hrespd_i, mtx_ahb_hrdatad_i, mtx_ahb_hreadys_i,
   mtx_ahb_hresps_i, mtx_ahb_hrdatas_i,
   bm_master_o, bm_addr_o, bm_trans_o, bm_cancel_valid_o, bm_write_o, bm_size_o,
   bm_burst_o, bm_prot_o, bm_mem_attr_o, bm_exreq_o, bm_wdata_o,
   bm_state_bb_wr1_o, bm_state_bb_wr2_o, mtx_bm_active_o, bm_dcore_active_o,
   bm_icore_active_o, bm_dap_active_o, bm_dcore_fpb_active_o,
   bm_icore_fpb_active_o, bm_fpb_active_o, bm_dcore_active_sel_o,
   bm_icore_active_sel_o, bm_dap_active_sel_o, bm_dcore_active_data_o,
   bm_dap_active_data_o, bm_dcore_stall_o, bm_icore_stall_o, bm_dap_stall_o,
   bm_dcore_fpb_stall_o, bm_icore_fpb_stall_o, bm_sel_dcode_o,
   bm_sel_dcode_data_o, bm_sel_sys_o, bm_sel_sys_data_o, bm_buff_write_accept_o,
   bm_rdata_o, bm_readyout_o, bm_readyout_sel_o, bm_resp_o,
   mtx_write_buff_abort_o, bm_icore_to_sys_o, bm_mpu_unalign_cancel_o
  );


  parameter DEBUG_LVL      = 3;  // Debug support level, between 0 and 3
  parameter RESET_ALL_REGS = 0;  // Reset all registers if 1
  parameter CONST_AHB_CTRL = 0;  // Constant AHB control info if 1


  input         hclk;                       // AHB system clock
  input         hreset_n;                   // AHB system reset

  input         data_be8_i;                 // BE8 mode
  input         dpu_dbg_snapstall_i;        // SNAPSTALL
  input         dap_ahb_habort_i;           // DAP abort

  input [31:0]  dpu_ahb_hwdatad_i;          // DCore HWDATA
  input [31:0]  dap_ahb_hwdata_i;           // DAP HWDATA

  input         mpu_canceli_i;              // ICore MPU lookup fail
  input         mpu_canceld_i;              // DCore MPU lookup fail

  input         mtx_ahb_hreadyd_i;          // DCode HREADY
  input  [1:0]  mtx_ahb_hrespd_i;           // DCode HRESP
  input [31:0]  mtx_ahb_hrdatad_i;          // DCode HRDATA
  input         mtx_ahb_hreadys_i;          // SYS HREADY
  input  [1:0]  mtx_ahb_hresps_i;           // SYS HRESP
  input [31:0]  mtx_ahb_hrdatas_i;          // SYS HRDATA

  input [1:0]   fpb_matched_i;              // FPB has matched
  input [4:0]   fpb_matched_addri_i;        // Instr address
  input [2:0]   fpb_matched_addrd_i;        // Data address
  input [3:0]   fpb_matched_proti_i;        // Instr prot
  input [3:0]   fpb_matched_protd_i;        // Data prot
  input [1:0]   fpb_matched_mem_attri_i;    // Instr memory attributes
  input [1:0]   fpb_matched_mem_attrd_i;    // Data memory attributes
  input         fpb_matched_exreqd_i;       // Data exclusive request
  input [1:0]   fpb_matched_sized_i;        // Data size
  input [23:0]  fpb_remap_addr_i;           // Remapped address

  input [31:0]  is_icore_addr_i;            // ICore HADDR
  input [3:0]   is_icore_prot_i;            // ICore HPROT
  input [1:0]   is_icore_mem_attr_i;        // ICore memory attributes
  input         is_icore_held_trans_i;      // ICore transaction held
  output        bm_icore_to_sys_o;          // ICore -> SYS access

  input [31:0]  is_dcore_addr_i;            // DCore HADDR
  input [3:0]   is_dcore_prot_i;            // DCore HPROT
  input [1:0]   is_dcore_size_i;            // DCore HSIZE
  input         is_dcore_write_i;           // DCore transfer direction
  input [1:0]   is_dcore_mem_attr_i;        // DCore memory attributes
  input         is_dcore_held_trans_i;      // DCore transaction held
  input [1:0]   is_dcore_buffered_write_i;  // DCore buffered write
  input         is_dcore_exreq_i;           // DCore exclusive request
  input [2:0]   is_dcore_unalign_i;         // DCore unalignment encoding
  output        bm_buff_write_accept_o;     // Buffered write owns Bit Master

  input [31:0]  is_dap_addr_i;              // DAP HADDR
  input [1:0]   is_dap_master_i;            // DAP Master
  input [3:0]   is_dap_prot_i;              // DAP HPROT
  input [1:0]   is_dap_size_i;              // DAP HSIZE
  input         is_dap_write_i;             // DAP transfer direction
  input         is_dap_held_trans_i;        // DAP transaction held
  input [2:0]   is_dap_unalign_i;           // DAP unalignment encoding

  input         dec_dcore_sel_dcode_i;      // DCore -> DCode access
  input         dec_dcore_sel_sys_i;        // DCore -> SYS access
  input         dec_dcore_sel_bb_i;         // DCore -> SYS bit-band access
  input         dec_icore_sel_sys_i;        // ICore -> SYS access
  input         dec_icore_sel_sys_held_i;   // ICore -> SYS access held
  input         dec_dap_sel_dcode_i;        // DAP   -> DCode access
  input         dec_dap_sel_sys_i;          // DAP   -> SYS access
  input         dec_dap_sel_bb_i;           // DAP   -> SYS bit-band access

  output        bm_sel_dcode_o;             // BitMaster -> DCode
  output        bm_sel_dcode_data_o;        // BitMaster -> DCode, data port only
  output        bm_sel_sys_o;               // BitMaster -> SYS
  output        bm_sel_sys_data_o;          // BitMaster -> SYS, data port only
  input         os_dcore_to_dcode_accept_i; // DCore -> DCode active
  input         os_dcore_to_sys_accept_i;   // DCore -> SYS active
  input         os_dap_to_dcode_accept_i;   // DAP   -> DCode active
  input         os_dap_to_dcode_held_req_i; // DAP   -> DCode waited request
  input         os_dap_to_sys_accept_i;     // DAP   -> SYS active
  input         os_dap_to_sys_held_req_i;   // DAP   -> SYS waited request
  input         os_sys_bm_wait_i;           // SYS   -> BM wait

  output [31:0] bm_addr_o;
  output [1:0]  bm_master_o;
  output [1:0]  bm_trans_o;
  output        bm_write_o;
  output [1:0]  bm_size_o;
  output [2:0]  bm_burst_o;
  output [3:0]  bm_prot_o;
  output [31:0] bm_wdata_o;
  output [1:0]  bm_mem_attr_o;
  output        bm_exreq_o;
  output        bm_mpu_unalign_cancel_o;    // MPU unaligned cancel
  output        bm_cancel_valid_o;          // Late transaction cancel allowed

  input         dec_dcore_active_data_i;    // DCore active (data phase)
  output [31:0] bm_rdata_o;
  output        bm_resp_o;
  output        bm_readyout_o;
  output [1:0]  bm_readyout_sel_o;
  output        mtx_write_buff_abort_o;     // Write buffer imprecise abort

  output        bm_dcore_active_o;          // Servicing DCore request
  output [1:0]  bm_dcore_active_sel_o;      // Selected slave for DCore
  output        bm_dcore_active_data_o;     // Servicing DCore request - data
  output        bm_dcore_fpb_active_o;      // Servicing DCore FPB request
  output        bm_dcore_stall_o;           // DCore stalled waiting for Bit Master ownership
  output        bm_dcore_fpb_stall_o;       // Stall FPB data phase from DCore
  output        bm_icore_active_o;          // Servicing ICore request
  output        bm_icore_active_sel_o;      // Selected slave for ICore - SYS only
  output        bm_icore_fpb_active_o;      // Servicing ICore FPB request
  output        bm_icore_stall_o;           // ICore stalled waiting for Bit Master ownership
  output        bm_icore_fpb_stall_o;       // Stall FPB data phase or SYS access from ICore
  output        bm_fpb_active_o;            // Servicing FPB request
  output        bm_dap_active_o;            // Servicing DAP request
  output [1:0]  bm_dap_active_sel_o;        // Selected slave for DAP
  output        bm_dap_active_data_o;       // Servicing DAP request - data
  output        bm_dap_stall_o;             // DAP stall waiting for Bit Master ownership

  output        mtx_bm_active_o;            // Bit Master is active
  output        bm_state_bb_wr1_o;          // Bit Master state is BM_BB_WR_1
  output        bm_state_bb_wr2_o;          // Bit Master state is BM_BB_WR_2


  wire        full_hreset_n;             // Optional reset using RESET_ALL_REGS parameter

  wire        bm_dcore_stall;            // DCore stalled waiting for Bit Master ownership
  wire        bm_icore_stall;            // ICore stalled waiting for Bit Master ownership
  wire        bm_dap_stall;              // DAP stalled waiting for Bit Master ownership
  wire        bm_dcore_fpb_stall;        // Stall FPB access from DCore
  wire        bm_icore_fpb_stall;        // Stall FPB access from ICore

  wire        dcore_unaligned;
  wire        dap_unaligned;
  wire        dap_unalgn_held;
  reg  [3:0]  nxt_dcore_trans;
  reg         nxt_dcore_active;
  reg         nxt_dcore_unaligned;
  reg  [3:0]  nxt_dap_trans;
  reg         nxt_dap_active;
  reg         nxt_dap_unaligned;
  wire        case_trans_wb_req;         // Case items for next transaction type
  wire        case_trans_bb_sys;
  wire        case_trans_unalign_dcode;
  wire        case_trans_unalign_sys;
  wire        case_dap_bb_sys;
  wire        case_dap_unalign_dcode;
  wire        case_dap_unalign_sys;

  wire        bm_buff_write_accept;      // Buffered write owns Bit Master
  wire        nxt_respd_err2;
  wire        nxt_resps_err2;
  wire        nxt_suppress_buff_write;
  reg         suppress_buff_write;

  wire        bm_icore_to_sys;           // ICore -> SYS pipelined access
  wire        bm_icore_to_sys_valid;     // bm_icore_to_sys MPU Lookup pass
  reg         bm_icore_to_sys_valid_reg;

  reg  [3:0]  next_trans;                // Next transaction
  wire [2:0]  next_trans_unalign;        // Type of unaligned transfer
  wire        next_trans_active;         // Next transfer is active
  reg  [2:0]  next_trans_master;         // Master for next transaction
  wire [1:0]  next_trans_slave;          // Destination of next transaction
  wire        addr_ready;                // Address phase ready
  wire        fast_trans_dcode;          // Fast version of next_trans_master[0]
  wire        fast_trans_dcode_dpu;      // DPU's fast version of next_trans_master[0]
  wire        fast_addr_ready;           // Faster version of addr_ready

  reg         unalign_2nd_lookup;        // Second MPU lookup for unaligned transfers
  wire        nxt_unalign_2nd_lookup;
  reg         unalign_cancel;            // Unaligned transfer is cancelled
  wire        nxt_unalign_cancel;
  wire        unalign_cancel_full;       // Valid in full phase of transaction
  wire [1:0]  mpu_masked_trans;          // Trans factoring MPU lookup

  wire        unaligned_boundary;        // Changing word boundary during unaligned transfer
  wire [31:0] addr_mux;                  // Address muxed from masters and buffer
  wire        sel_addr_adder;            // Select address mux adder
  reg  [31:0] addrbuf_addr;              // Buffered address
  reg         addrbuf_write;             // Buffered transfer direction
  reg  [3:0]  addrbuf_prot;              // Buffered prot
  reg  [1:0]  addrbuf_size_bb;           // Size buffer for bit-band
  reg         addrbuf_exreq;             // Buffered exclusive request
  reg  [1:0]  addrbuf_mem_attr;          // Buffered memory attributes
  wire        addrbuf_we;                // Address buffer write enable
  wire        addrbuf_addr_we;           // Enable for address field
  wire        addrbuf_size_bb_we;        // Enable for bit-band size field
  wire [31:0] nxt_addrbuf_addr;          // Next address with adder for unaligned
  wire        nxt_addrbuf_write;         // Next bufferedtransfer direction
  wire [3:0]  nxt_addrbuf_prot;          // Next buffered prot
  wire [1:0]  nxt_addrbuf_size_bb;       // Next buffered size
  wire        nxt_addrbuf_exreq;         // Next buffered exclusive request
  wire [1:0]  nxt_addrbuf_mem_attr;      // Next buffered memory attributes

  reg  [3:0]  unalign_prot;              // Buffered unaligned prot
  reg  [1:0]  unalign_mem_attr;          // Buffered unaligned memory attributes

  wire [31:0] nxt_wdata;                 // Muxed write data from DCore or DAP
  reg  [31:0] write_buff;                // Write buffer
  reg  [31:0] nxt_write_buff;
  reg  [3:0]  write_buff_we;
  wire [1:0]  write_buff_req;            // Write buffer request
  reg         write_buff_hold;           // Hold write buffer value
  wire        nxt_write_buff_hold;
  wire        write_buff_hold_we;
  wire        bm_mpu_unalign_cancel;     // MPU unaligned cancel
  wire [3:0]  masked_write_buff_we;      // Masked write buffer enable
  wire        mtx_write_buff_abort;      // Write buffer imprecise abort

  reg         slv_resp_buff;
  wire        nxt_slv_resp_buff;
  wire        slv_resp_buff_we;

  wire [4:0]  bb_addr;                   // BB byte position
  wire        bb_rd_bit;                 // Extracted read bit for bit-band
  wire        bb_wr_bit;                 // Extracted write bit for bit-band
  reg  [31:0] bb_rdata;                  // Read data for bit-band
  wire [31:0] nxt_bb_wdata;              // Write data for bit-band
  wire [31:0] bb_wdata_mask;             // Mask for merging bit-band write bit

  wire [31:0] bm_wdata;                  // HWDATA signal
  reg  [1:0]  bb_addr_align;             // Address LSBs for aligning bit-band
  reg  [31:0] bm_addr;                   // HADDR signal
  wire [1:0]  bm_master;                 // HMASTER signal
  reg  [1:0]  bm_trans;                  // HTRANS signal
  reg         bm_write;                  // HWRITE signal
  reg  [1:0]  bm_size;                   // HSIZE signal
  wire [2:0]  bm_burst;                  // HBURST signal
  reg  [3:0]  bm_prot;                   // HPROT signal
  reg  [1:0]  bm_mem_attr;               // Memory Attributes
  wire        unalign_no_cross;          // No word boundary cross for unaligned
  wire        bm_cancel_valid;           // Late transaction cancel allowed
  reg         bm_exreq;                  // Exclusive Request

  wire        buffered_resp;             // Combine RESPs for multi-cycle transfers
  reg  [31:0] bm_rdata;                  // HRDATA    -> Master
  wire        bm_readyout;               // HREADYOUT -> Master
  wire [1:0]  bm_readyout_sel;           // HREADYOUT -> Master select lines
  reg         bm_resp;                   // HRESP     -> Master
  wire [31:0] rdata_unalign;             // Merged RDATA for unaligned transfers
  reg  [3:0]  rdata_mask;                // Mask for merging RDATA with write buffer
  wire        active_unalign_bb;         // Unaligned or bit-band in progress and not finished

  wire [31:0] slv_rdata;
  wire        slv_ready;
  wire        slv_resp;

  wire        bm_active;
  wire        bm_dcore_active;
  wire        bm_icore_active;
  wire        bm_dap_active;
  wire        bm_dcore_fpb_active;
  wire        bm_icore_fpb_active;
  wire        bm_fpb_active;
  wire        bm_dcore_active_data;
  wire        bm_dap_active_data;
  wire [1:0]  bm_dcore_active_sel;
  wire        bm_icore_active_sel;
  wire [1:0]  bm_dap_active_sel;

  reg  [3:0]  bm_state;
  reg  [3:0]  i_nxt_bm_state;            // Next state
  wire [3:0]  nxt_bm_state;              // Qualified next state
  wire        clr_bm_state;              // SNAPSTALL or HABORT generated clear
  wire        bm_state_we;               // State enable
  wire        mst_trans_state_we;        // DAP HMASTER state enable
  wire        bm_trans_state_we;
  reg  [2:0]  bm_master_sel;             // Arbitrated master {DAP, ICODE, DCODE} one-hot encoding
  reg  [1:0]  bm_slave_sel;              // Access to DCode
  reg  [2:0]  bm_unalign;                // Unaligned transfer
  wire [2:0]  nxt_bm_master_sel;
  wire [1:0]  nxt_bm_slave_sel;
  wire [2:0]  nxt_bm_unalign;
  wire        bm_sel_dcode;              // BitMaster -> DCode
  wire        bm_sel_dcode_data;         // BitMaster -> DCode, data port only
  wire        bm_sel_sys;                // BitMaster -> SYS
  wire        bm_sel_sys_data;           // BitMaster -> SYS, data port only
  wire        bm_arb_state;              // Arbitration active
  reg  [1:0]  dap_master_reg;            // DAP HMASTER
  wire        dap_master_reg_we;
  wire        bm_state_bb_wr1;           // State machine is in BM_BB_WR_1
  wire        bm_state_bb_wr2;           // State machine is in BM_BB_WR_2

  reg         icore_fpb_pend;            // I-FPB pending
  wire        nxt_icore_fpb_pend;
  wire        icore_fpb_pend_we;
  reg         dcore_fpb_pend;            // D-FPB pending
  wire        nxt_dcore_fpb_pend;
  wire        dcore_fpb_pend_we;

  wire        opt_fpb_en;
  wire        opt_dap_en;
  wire [3:0]  opt_nxt_dap_trans;
  wire        opt_nxt_dap_unaligned;
  wire        opt_nxt_dap_active;
  wire        opt_const_en;

`ifdef ARM_DSM

  wire [1:0]  dsm_debug_lvl;
  wire        dsm_reset_all_regs;
  wire        dsm_const_ahb_ctrl;

  assign full_hreset_n = (dsm_reset_all_regs) ? hreset_n : 1'b1;

  assign opt_fpb_en = (dsm_debug_lvl > 1) ? 1'b1 : 1'b0;

  assign opt_const_en = (dsm_const_ahb_ctrl > 0) ? 1'b1 : 1'b0;

`else

  assign full_hreset_n = (RESET_ALL_REGS == 1) ? hreset_n : 1'b1;

  assign opt_fpb_en = (DEBUG_LVL > 1) ? 1'b1 : 1'b0;

  assign opt_const_en = (CONST_AHB_CTRL > 0) ? 1'b1 : 1'b0;

`endif



  assign opt_nxt_dap_trans     = (DEBUG_LVL > 0) ? nxt_dap_trans     : 4'b0000;
  assign opt_nxt_dap_unaligned = (DEBUG_LVL > 0) ? nxt_dap_unaligned : 1'b0;
  assign opt_nxt_dap_active    = (DEBUG_LVL > 0) ? nxt_dap_active    : 1'b0;

  assign opt_dap_en            = (DEBUG_LVL > 0) ? 1'b1              : 1'b0;



  assign dcore_unaligned = is_dcore_unalign_i != `MTX_UNALIGN_NORMAL;
  assign dap_unaligned   = opt_dap_en & (is_dap_unalign_i != `MTX_UNALIGN_NORMAL);

  assign dap_unalgn_held = ((dap_unaligned | dec_dap_sel_bb_i) &
                            (os_dap_to_dcode_held_req_i |
                             os_dap_to_sys_held_req_i));


  assign bm_dcore_stall = (dec_dcore_sel_bb_i | dcore_unaligned) &
                          ((opt_dap_en & bm_master_sel[2]) |
                           bm_master_sel[1] | dap_unalgn_held);

  assign bm_icore_stall = dec_icore_sel_sys_i   &
                          is_icore_held_trans_i &
                          (((dec_dcore_sel_bb_i | dcore_unaligned) & is_dcore_held_trans_i) |
                           ((dec_dap_sel_bb_i   | dap_unaligned)   & is_dap_held_trans_i) |
                           ((opt_dap_en & bm_master_sel[2]) |
                            bm_master_sel[0]) |
                           bm_buff_write_accept);

  assign bm_dap_stall = opt_dap_en &
                        (dec_dap_sel_bb_i | dap_unaligned) &
                        (((dec_dcore_sel_bb_i | dcore_unaligned) & is_dcore_held_trans_i & ~dap_unalgn_held) |
                         (bm_master_sel[0] | bm_master_sel[1]) | bm_buff_write_accept);

  assign bm_dcore_fpb_stall = opt_fpb_en & (dcore_fpb_pend | (bm_state == `BM_DFPB));
  assign bm_icore_fpb_stall = (bm_state == `BM_ISYS) |
                              (opt_fpb_en & (icore_fpb_pend | (bm_state == `BM_IFPB)));




  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      unalign_2nd_lookup <= 1'b0;
    else
      unalign_2nd_lookup <= nxt_unalign_2nd_lookup;

  assign nxt_unalign_2nd_lookup = nxt_dcore_unaligned & bm_arb_state & addr_ready;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      unalign_cancel <= 1'b0;
    else
      unalign_cancel <= nxt_unalign_cancel;

  assign nxt_unalign_cancel = (unalign_2nd_lookup & mpu_canceld_i) |
                              (unalign_cancel & ~(bm_arb_state & addr_ready));

  assign mpu_masked_trans = unalign_cancel ? `MTX_TRN_IDLE : `MTX_TRN_NONSEQ;

  assign unalign_cancel_full = (unalign_2nd_lookup & mpu_canceld_i) | unalign_cancel;




  assign case_trans_wb_req        = |(write_buff_req);

  assign case_trans_bb_sys        = dec_dcore_sel_bb_i         &
                                    os_dcore_to_sys_accept_i   &
                                    ~mpu_canceld_i;

  assign case_trans_unalign_dcode = dec_dcore_sel_dcode_i      &
                                    os_dcore_to_dcode_accept_i &
                                    ~mpu_canceld_i             &
                                    dcore_unaligned;

  assign case_trans_unalign_sys   = dec_dcore_sel_sys_i        &
                                    os_dcore_to_sys_accept_i   &
                                    ~mpu_canceld_i             &
                                    dcore_unaligned;


  always @ (case_trans_wb_req or case_trans_bb_sys or
            case_trans_unalign_dcode or case_trans_unalign_sys or
            is_dcore_write_i)
    case ({case_trans_wb_req, case_trans_bb_sys,
           case_trans_unalign_dcode, case_trans_unalign_sys})
      4'b1000, 4'b1001, 4'b1010, 4'b1011,
      4'b1100, 4'b1101, 4'b1110, 4'b1111:
        begin
          nxt_dcore_trans     = `BM_WB;
          nxt_dcore_unaligned = 1'b0;
          nxt_dcore_active    = 1'b1;
        end

      4'b0100, 4'b0101, 4'b0110, 4'b0111:
        begin
          nxt_dcore_trans     = is_dcore_write_i ? `BM_BB_WR_1 : `BM_BB_RD;
          nxt_dcore_unaligned = 1'b0;
          nxt_dcore_active    = 1'b1;
        end

      4'b0001, 4'b0010, 4'b0011:
        begin
          nxt_dcore_trans     = `BM_UN_1ST;
          nxt_dcore_unaligned = 1'b1;
          nxt_dcore_active    = 1'b1;
        end

      4'b0000:
        begin
          nxt_dcore_trans     = `BM_IDLE;
          nxt_dcore_unaligned = 1'b0;
          nxt_dcore_active    = 1'b0;
        end

      default:
        begin
          nxt_dcore_trans     = 4'bXXXX;
          nxt_dcore_unaligned = 1'bX;
          nxt_dcore_active    = 1'bX;
        end
    endcase




  assign case_dap_bb_sys        = dec_dap_sel_bb_i         &
                                  os_dap_to_sys_accept_i;

  assign case_dap_unalign_dcode = dec_dap_sel_dcode_i      &
                                  os_dap_to_dcode_accept_i &
                                  dap_unaligned;

  assign case_dap_unalign_sys   = dec_dap_sel_sys_i        &
                                  os_dap_to_sys_accept_i   &
                                  dap_unaligned;


  always @ (case_dap_bb_sys or case_dap_unalign_dcode or
            case_dap_unalign_sys or is_dap_write_i)
    case ({case_dap_bb_sys, case_dap_unalign_dcode, case_dap_unalign_sys})
      3'b100, 3'b101, 3'b110, 3'b111:
        begin
          nxt_dap_trans     = is_dap_write_i ? `BM_BB_WR_1 : `BM_BB_RD;
          nxt_dap_unaligned = 1'b0;
          nxt_dap_active    = 1'b1;
        end

      3'b001, 3'b010, 3'b011:
        begin
          nxt_dap_trans     = `BM_UN_1ST;
          nxt_dap_unaligned = 1'b1;
          nxt_dap_active    = 1'b1;
        end

      3'b000:
        begin
          nxt_dap_trans     = `BM_IDLE;
          nxt_dap_unaligned = 1'b0;
          nxt_dap_active    = 1'b0;
        end

      default:
        begin
          nxt_dap_trans     = 4'bXXXX;
          nxt_dap_unaligned = 1'b0;
          nxt_dap_active    = 1'b1;
        end
    endcase



  assign bm_icore_to_sys = dec_icore_sel_sys_i   &
                           is_icore_held_trans_i &
                           ~dec_icore_sel_sys_held_i;

  assign bm_icore_to_sys_valid = bm_icore_to_sys & ~mpu_canceli_i;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      bm_icore_to_sys_valid_reg <= 1'b0;
    else if (mtx_ahb_hreadys_i)
      bm_icore_to_sys_valid_reg <= bm_icore_to_sys_valid;



  always @ (nxt_dcore_active or nxt_dcore_trans or opt_nxt_dap_active or
            opt_nxt_dap_trans or bm_icore_to_sys or icore_fpb_pend or
            dcore_fpb_pend or opt_fpb_en)
    if (nxt_dcore_active)
      begin
        next_trans        = nxt_dcore_trans;
        next_trans_master = `BM_DCORE;
      end
    else if (opt_nxt_dap_active)
      begin
        next_trans        = opt_nxt_dap_trans;
        next_trans_master = `BM_DAP;
      end
    else if (bm_icore_to_sys)
      begin
        next_trans        = `BM_ISYS;
        next_trans_master = `BM_ICORE;
      end
    else if (opt_fpb_en & icore_fpb_pend)
      begin
        next_trans        = `BM_IFPB;
        next_trans_master = `BM_ICORE;
      end
    else if (opt_fpb_en & dcore_fpb_pend)
      begin
        next_trans        = `BM_DFPB;
        next_trans_master = `BM_DCORE;
      end
    else
      begin
        next_trans        = `BM_IDLE;
        next_trans_master = 3'b000;
      end

  assign next_trans_unalign = ({3{nxt_dcore_active & nxt_dcore_unaligned}} & is_dcore_unalign_i) |
                              ({3{opt_nxt_dap_active & opt_nxt_dap_unaligned}} & is_dap_unalign_i);

  assign next_trans_active  = nxt_dcore_active   |
                              opt_nxt_dap_active |
                              bm_icore_to_sys    |
                              (opt_fpb_en & (icore_fpb_pend | dcore_fpb_pend));

  assign write_buff_req = is_dcore_buffered_write_i &
                          {2{~bm_active}}           &
                          {~mtx_ahb_hreadys_i, ~mtx_ahb_hreadyd_i};

  assign next_trans_slave[0] = // DCore access
                               (dcore_unaligned            &
                                dec_dcore_sel_dcode_i      &
                                os_dcore_to_dcode_accept_i &
                                ~mpu_canceld_i) |
                               (dap_unaligned              &
                                dec_dap_sel_dcode_i        &
                                os_dap_to_dcode_accept_i) |
                               write_buff_req[0];

  assign next_trans_slave[1] = // DCore access
                               (dcore_unaligned          &
                                dec_dcore_sel_sys_i      &
                                os_dcore_to_sys_accept_i &
                                ~mpu_canceld_i) |
                               (dap_unaligned            &
                                dec_dap_sel_sys_i        &
                                os_dap_to_sys_accept_i) |
                               (opt_dap_en               &
                                dec_dap_sel_bb_i         &
                                os_dap_to_sys_accept_i) |
                               (dec_dcore_sel_bb_i       &
                                os_dcore_to_sys_accept_i &
                                ~mpu_canceld_i) |
                               ((opt_fpb_en              &
                                 (icore_fpb_pend | dcore_fpb_pend) | bm_icore_to_sys) &
                                ~next_trans_slave[0]) |
                               write_buff_req[1];

  assign addr_ready = next_trans_slave[0] ? mtx_ahb_hreadyd_i : mtx_ahb_hreadys_i;

  assign fast_trans_dcode = (dcore_unaligned            &
                             dec_dcore_sel_dcode_i      &
                             os_dcore_to_dcode_accept_i &
                             ~mpu_canceld_i) |
                            (dap_unaligned              &
                             dec_dap_sel_dcode_i        &
                             os_dap_to_dcode_accept_i);

  assign fast_trans_dcode_dpu = (dcore_unaligned            &
                                 dec_dcore_sel_dcode_i      &
                                 os_dcore_to_dcode_accept_i) |
                                (dap_unaligned              &
                                 dec_dap_sel_dcode_i        &
                                 os_dap_to_dcode_accept_i);

  assign fast_addr_ready      = ( fast_trans_dcode & mtx_ahb_hreadyd_i) |
                                (~fast_trans_dcode & mtx_ahb_hreadys_i);




  assign slv_ready     = (bm_buff_write_accept |
                          ( bm_slave_sel[0] & mtx_ahb_hreadyd_i) |
                          ( bm_slave_sel[1] & mtx_ahb_hreadys_i & ~os_sys_bm_wait_i) |
                          (~bm_slave_sel[0] & ~bm_slave_sel[1] & fast_addr_ready));

  assign slv_resp      = (bm_slave_sel[0] & mtx_ahb_hrespd_i[0]) |
                         (bm_slave_sel[1] & mtx_ahb_hresps_i[0]);

  assign slv_rdata     = ({32{bm_slave_sel[0]}} & mtx_ahb_hrdatad_i) |
                         ({32{bm_slave_sel[1]}} & mtx_ahb_hrdatas_i);



  assign buffered_resp = // Mask off response for cancelled transactions
                         ~unalign_cancel_full &
                         (slv_resp | slv_resp_buff);

  always @ (bm_state or buffered_resp or slv_resp)
    case (bm_state)
      `BM_UN_1ST, `BM_UN_2ND, `BM_UN_LAST, `BM_BB_WR_1, `BM_BB_WR_2:
        bm_resp = buffered_resp;

      `BM_WB:
        bm_resp = 1'b0;

      `BM_IDLE, `BM_DFPB, `BM_DFPB2, `BM_IFPB, `BM_IFPB2, `BM_ISYS, `BM_ISYS2,
      `BM_BB_RD, `BM_BB_WR_3:
        bm_resp = slv_resp;

      default:
        bm_resp = 1'bX;
    endcase



  always @ (bm_state or bb_rdata or slv_rdata or rdata_unalign or opt_fpb_en)
    case (bm_state)
      `BM_UN_LAST:
        bm_rdata = rdata_unalign;

      `BM_BB_RD:
        bm_rdata = bb_rdata;

      `BM_DFPB, `BM_DFPB2, `BM_IFPB, `BM_IFPB2:
        bm_rdata = opt_fpb_en ? slv_rdata : 32'h00000000;

      `BM_IDLE, `BM_WB, `BM_ISYS, `BM_ISYS2, `BM_UN_1ST, `BM_UN_2ND,
      `BM_BB_WR_1, `BM_BB_WR_2, `BM_BB_WR_3:
        bm_rdata = slv_rdata;

      default:
        bm_rdata = 32'hXXXXXXXX;
    endcase



  always @ (bm_unalign)
    case (bm_unalign)
      `MTX_UNALIGN_HWRD_01: rdata_mask = 4'b1101;
      `MTX_UNALIGN_HWRD_11: rdata_mask = 4'b0111;
      `MTX_UNALIGN_WORD_01: rdata_mask = 4'b0001;
      `MTX_UNALIGN_WORD_10: rdata_mask = 4'b0011;
      `MTX_UNALIGN_WORD_11: rdata_mask = 4'b0100;
      `MTX_UNALIGN_NORMAL:  rdata_mask = 4'b0000;  // Aligned case
      default:              rdata_mask = 4'bXXXX;  // Propagate X
    endcase

  assign rdata_unalign[31:24] = rdata_mask[3] ? slv_rdata[31:24] : write_buff[31:24];
  assign rdata_unalign[23:16] = rdata_mask[2] ? slv_rdata[23:16] : write_buff[23:16];
  assign rdata_unalign[15:8]  = rdata_mask[1] ? slv_rdata[15:8]  : write_buff[15:8];
  assign rdata_unalign[7:0]   = rdata_mask[0] ? slv_rdata[7:0]   : write_buff[7:0];



  assign active_unalign_bb = (bm_state == `BM_UN_1ST) |
                             (bm_state == `BM_UN_2ND) |
                             (bm_state == `BM_BB_WR_1);

  assign bm_readyout       = (~active_unalign_bb &
                              (bm_buff_write_accept |
                               ( bm_slave_sel[0] & mtx_ahb_hreadyd_i) |
                               ( bm_slave_sel[1] & mtx_ahb_hreadys_i) |
                               (~bm_slave_sel[0] & ~bm_slave_sel[1] &
                                (( fast_trans_dcode_dpu & mtx_ahb_hreadyd_i) |
                                 (~fast_trans_dcode_dpu & mtx_ahb_hreadys_i)))));


  assign bm_readyout_sel[1] = (~active_unalign_bb &
                               (bm_buff_write_accept |
                                bm_slave_sel[1] |
                                (~bm_slave_sel[0] & ~bm_slave_sel[1] &
                                 ~fast_trans_dcode_dpu)));

  assign bm_readyout_sel[0] = (~active_unalign_bb &
                               (bm_buff_write_accept |
                                bm_slave_sel[0] |
                                (~bm_slave_sel[0] & ~bm_slave_sel[1] &
                                 fast_trans_dcode_dpu)));


  assign unaligned_boundary = (bm_state == `BM_UN_1ST) | (bm_state == `BM_UN_2ND);

  assign addr_mux = ({32{bm_arb_state & next_trans_master[0]}} & is_dcore_addr_i) |
                    ({32{bm_arb_state & next_trans_master[1]}} & is_icore_addr_i) |
                    ({32{bm_arb_state & next_trans_master[2]}} & is_dap_addr_i)   |
                    ({32{unaligned_boundary}} & addrbuf_addr);


  assign sel_addr_adder = (bm_arb_state &
                            ((next_trans_unalign == `MTX_UNALIGN_HWRD_11) |
                             (next_trans_unalign == `MTX_UNALIGN_WORD_10) |
                             (next_trans_unalign == `MTX_UNALIGN_WORD_11))) |
                          ((bm_state   == `BM_UN_1ST) &
                           (bm_unalign == `MTX_UNALIGN_WORD_01));

  assign nxt_addrbuf_addr     = sel_addr_adder ? (addr_mux + 32'h00000004) : addr_mux;

  assign nxt_addrbuf_size_bb  = ({2{next_trans_master[0]}} & is_dcore_size_i) |
                                ({2{next_trans_master[2]}} & is_dap_size_i);

  assign nxt_addrbuf_write    = (next_trans_master[0] & is_dcore_write_i) |
                                (next_trans_master[2] & is_dap_write_i);

  assign nxt_addrbuf_prot     = ({4{next_trans_master[0]}} & is_dcore_prot_i) |
                                ({4{next_trans_master[1]}} & is_icore_prot_i) |
                                ({4{next_trans_master[2]}} & is_dap_prot_i);

  assign nxt_addrbuf_mem_attr = ({2{next_trans_master[0]}} & is_dcore_mem_attr_i) |
                                ({2{next_trans_master[1]}} & is_icore_mem_attr_i);

  assign nxt_addrbuf_exreq    = next_trans_master[0] & is_dcore_exreq_i;


  assign addrbuf_we = slv_ready &
                      ((i_nxt_bm_state == `BM_ISYS)    |
                       (i_nxt_bm_state == `BM_BB_WR_1) |
                       (i_nxt_bm_state == `BM_UN_1ST));

  assign addrbuf_addr_we = slv_ready &
                           ((i_nxt_bm_state  == `BM_ISYS)    |
                            (i_nxt_bm_state  == `BM_BB_RD)   |
                            (i_nxt_bm_state  == `BM_BB_WR_1) |
                            (i_nxt_bm_state  == `BM_UN_1ST)  |
                            ((i_nxt_bm_state == `BM_UN_2ND) &
                             (bm_unalign     == `MTX_UNALIGN_WORD_01)));

  assign addrbuf_size_bb_we = slv_ready &
                              ((i_nxt_bm_state == `BM_BB_WR_1) |
                               (i_nxt_bm_state == `BM_BB_RD));

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      begin
        addrbuf_write    <= 1'b0;
        addrbuf_prot     <= 4'b0000;
        addrbuf_mem_attr <= 2'b00;
        addrbuf_exreq    <= 1'b0;
      end
    else if (addrbuf_we)
      begin
        addrbuf_write    <= nxt_addrbuf_write;
        addrbuf_prot     <= nxt_addrbuf_prot;
        addrbuf_mem_attr <= nxt_addrbuf_mem_attr;
        addrbuf_exreq    <= nxt_addrbuf_exreq;
      end

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      addrbuf_addr <= 32'h00000000;
    else if (addrbuf_addr_we)
      addrbuf_addr <= nxt_addrbuf_addr;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      addrbuf_size_bb <= 2'b00;
    else if(addrbuf_size_bb_we)
      addrbuf_size_bb <= nxt_addrbuf_size_bb;




  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      begin
        unalign_mem_attr <= 2'b00;
        unalign_prot     <= 4'b0000;
      end
    else if (unalign_2nd_lookup)
      begin
        unalign_mem_attr <= is_dcore_mem_attr_i;
        unalign_prot     <= is_dcore_prot_i;
      end




  assign nxt_respd_err2 = is_dcore_buffered_write_i[0] &
                          mtx_ahb_hrespd_i[0]          &
                          ~mtx_ahb_hreadyd_i;

  assign nxt_resps_err2 = is_dcore_buffered_write_i[1] &
                          mtx_ahb_hresps_i[0]          &
                          ~mtx_ahb_hreadys_i;

  assign nxt_suppress_buff_write = nxt_respd_err2 | nxt_resps_err2;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      suppress_buff_write <= 1'b0;
    else
      suppress_buff_write <= nxt_suppress_buff_write;


  assign bm_buff_write_accept = |(is_dcore_buffered_write_i) &
                                ~bm_active                   &
                                ~suppress_buff_write;


  assign nxt_wdata = ({32{bm_master_sel[0]}}              & dpu_ahb_hwdatad_i) |
                     ({32{opt_dap_en & bm_master_sel[2]}} & dap_ahb_hwdata_i );

  always @ (bm_state or dpu_ahb_hwdatad_i or nxt_wdata or nxt_bb_wdata or
            addrbuf_write or slv_rdata)
    case (bm_state)
      `BM_UN_1ST:
        nxt_write_buff = addrbuf_write ? nxt_wdata : slv_rdata;
      `BM_UN_2ND:
        nxt_write_buff = slv_rdata;

      `BM_BB_WR_1:
        nxt_write_buff = nxt_bb_wdata;

      `BM_IDLE:
        nxt_write_buff = dpu_ahb_hwdatad_i;

      default:
        nxt_write_buff = {32{1'bX}};
    endcase


  always @ (addrbuf_write or bm_state or bm_buff_write_accept or addr_ready or
            bm_unalign or opt_fpb_en)
    case (bm_state)
      `BM_IDLE:
        write_buff_we = {4{bm_buff_write_accept & ~addr_ready}};

      `BM_UN_1ST:
        case (addrbuf_write)
          1'b1: write_buff_we = 4'b1111;
          1'b0:
            case (bm_unalign)
              `MTX_UNALIGN_HWRD_01: write_buff_we = 4'b0010;
              `MTX_UNALIGN_HWRD_11: write_buff_we = 4'b1000;
              `MTX_UNALIGN_WORD_01: write_buff_we = 4'b0010;
              `MTX_UNALIGN_WORD_10: write_buff_we = 4'b1100;
              `MTX_UNALIGN_WORD_11: write_buff_we = 4'b1000;
              default:              write_buff_we = 4'bXXXX;
            endcase
          default: write_buff_we = 4'bXXXX;
        endcase

      `BM_UN_2ND:
        write_buff_we = {4{~addrbuf_write}} &
                        ((bm_unalign == `MTX_UNALIGN_WORD_01) ? 4'b1100 : 4'b0011);

      `BM_UN_LAST:
        write_buff_we = 4'b0000;

      `BM_BB_WR_1:
        write_buff_we = 4'b1111;

      `BM_IFPB, `BM_IFPB2, `BM_DFPB, `BM_DFPB2:
        write_buff_we = 4'b0000;

      `BM_WB, `BM_ISYS, `BM_ISYS2, `BM_BB_RD, `BM_BB_WR_2, `BM_BB_WR_3:
        write_buff_we = 4'b0000;

      default:
        write_buff_we = 4'bXXXX;
    endcase


  assign bm_mpu_unalign_cancel = mpu_canceld_i           &
                                 addrbuf_write           &
                                 dec_dcore_active_data_i &
                                 unalign_2nd_lookup;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      write_buff_hold <= 1'b0;
    else if (write_buff_hold_we)
      write_buff_hold <= nxt_write_buff_hold;

  assign nxt_write_buff_hold = bm_mpu_unalign_cancel & ~slv_ready;
  assign write_buff_hold_we  = bm_mpu_unalign_cancel | slv_ready;

  assign masked_write_buff_we = {4{bm_mpu_unalign_cancel & ~slv_ready}} |
                                (write_buff_we & {4{~write_buff_hold}});

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      write_buff[31:24] <= 8'h00;
    else if (masked_write_buff_we[3])
      write_buff[31:24] <= nxt_write_buff[31:24];

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      write_buff[23:16] <= 8'h00;
    else if (masked_write_buff_we[2])
      write_buff[23:16] <= nxt_write_buff[23:16];

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      write_buff[15:8] <= 8'h00;
    else if (masked_write_buff_we[1])
      write_buff[15:8] <= nxt_write_buff[15:8];

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      write_buff[7:0] <= 8'h00;
    else if (masked_write_buff_we[0])
      write_buff[7:0] <= nxt_write_buff[7:0];


  assign mtx_write_buff_abort = (bm_state == `BM_WB) & slv_resp & slv_ready;




  assign nxt_slv_resp_buff = active_unalign_bb    &
                             ~unalign_cancel_full &
                             (slv_resp | slv_resp_buff);

  assign slv_resp_buff_we  = (i_nxt_bm_state != `BM_IDLE) & slv_ready;

  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      slv_resp_buff <= 1'b0;
    else if (slv_resp_buff_we)
      slv_resp_buff <= nxt_slv_resp_buff;



  assign bb_addr[2:0] = addrbuf_addr[4:2];
  assign bb_addr[3]   = (data_be8_i & (addrbuf_size_bb != 2'b00)) ?
                         ~addrbuf_addr[5] : addrbuf_addr[5];
  assign bb_addr[4]   = (data_be8_i & (addrbuf_size_bb == 2'b10)) ?
                         ~addrbuf_addr[6] : addrbuf_addr[6];

  assign bb_rd_bit = slv_rdata[bb_addr];

  always @ (addrbuf_size_bb or data_be8_i or bb_rd_bit)
    case ({data_be8_i, addrbuf_size_bb})
      3'b0_00, 3'b0_01, 3'b0_10, 3'b1_00:
        bb_rdata = {{31{1'b0}}, bb_rd_bit};
      3'b1_01:
        bb_rdata = {{23{1'b0}}, bb_rd_bit, {8{1'b0}}};
      3'b1_10:
        bb_rdata = {{7{1'b0}}, bb_rd_bit, {24{1'b0}}};
      default:
        bb_rdata = {32{1'bX}};
    endcase

  assign bb_wr_bit = ( data_be8_i & (addrbuf_size_bb == 2'b10) & nxt_wdata[24]) |
                     ( data_be8_i & (addrbuf_size_bb == 2'b01) & nxt_wdata[8])  |
                     ( data_be8_i & (addrbuf_size_bb == 2'b00) & nxt_wdata[0])  |
                     (~data_be8_i & nxt_wdata[0]);

  assign bb_wdata_mask = ~(32'h00000001 << bb_addr); // 1'b0 in bb_addr position
  assign nxt_bb_wdata  = (slv_rdata & bb_wdata_mask) | ({{31{1'b0}}, bb_wr_bit} << bb_addr);



  assign bm_wdata = write_buff[31:0];

  always @ (addrbuf_addr or addrbuf_size_bb)
    case (addrbuf_size_bb[1:0])
      2'b00:   bb_addr_align = addrbuf_addr[6:5];
      2'b01:   bb_addr_align = {addrbuf_addr[6], 1'b0};
      2'b10:   bb_addr_align = 2'b00;
      default: bb_addr_align = 2'bXX;
    endcase

  assign bm_master = ({2{bm_master_sel == `BM_ICORE}} & 2'b10)        |
                     ({2{bm_master_sel == `BM_DAP}}   & dap_master_reg);

  always @ (bm_state or mpu_masked_trans or bm_icore_to_sys_valid_reg or opt_fpb_en)
    case (bm_state)
      `BM_UN_1ST, `BM_UN_2ND, `BM_BB_WR_1:
        bm_trans = mpu_masked_trans;

      `BM_IFPB, `BM_DFPB:
        bm_trans = opt_fpb_en ? `MTX_TRN_NONSEQ : 2'b00;

      `BM_ISYS:
        bm_trans = bm_icore_to_sys_valid_reg ? `MTX_TRN_NONSEQ : `MTX_TRN_IDLE;

      default:
        bm_trans = 2'bXX;
    endcase


  always @ (bm_state or bm_unalign or addrbuf_addr or fpb_remap_addr_i or
            fpb_matched_addri_i or fpb_matched_addrd_i or bb_addr_align or
            opt_fpb_en)
    case (bm_state)
      `BM_UN_1ST:
        case (bm_unalign)
          `MTX_UNALIGN_HWRD_01: bm_addr = {addrbuf_addr[31:2], 2'b10};
          `MTX_UNALIGN_HWRD_11: bm_addr = {addrbuf_addr[31:2], 2'b00};
          `MTX_UNALIGN_WORD_01: bm_addr = {addrbuf_addr[31:2], 2'b10};
          `MTX_UNALIGN_WORD_10: bm_addr = {addrbuf_addr[31:2], 2'b00};
          `MTX_UNALIGN_WORD_11: bm_addr = {addrbuf_addr[31:2], 2'b00};
          default:             bm_addr = {32{1'bX}};
        endcase

      `BM_UN_2ND:
        bm_addr = (bm_unalign == `MTX_UNALIGN_WORD_01) ? {addrbuf_addr[31:2], 2'b00} :
                                                        {addrbuf_addr[31:2], 2'b10};

      `BM_BB_WR_1:
        bm_addr = {addrbuf_addr[31:28], 8'b00000000, addrbuf_addr[24:7], bb_addr_align};

      `BM_IFPB:
        bm_addr = opt_fpb_en ? {3'b001, fpb_remap_addr_i, fpb_matched_addri_i} : {32{1'b0}};
      `BM_DFPB:
        bm_addr = opt_fpb_en ? {3'b001, fpb_remap_addr_i, 2'b11, fpb_matched_addrd_i} : {32{1'b0}};

      `BM_ISYS:
        bm_addr = addrbuf_addr[31:0];

      default:
        bm_addr = 32'hxxxxxxxx;
    endcase


  always @ (bm_state or addrbuf_write or opt_fpb_en)
    case (bm_state)
      `BM_UN_1ST, `BM_UN_2ND:
        bm_write = addrbuf_write;
      `BM_BB_WR_1:
        bm_write = 1'b1;
      `BM_IFPB, `BM_DFPB:
        bm_write = 1'b0;
      `BM_ISYS:
        bm_write = 1'b0;
      default:
        bm_write = 1'bX;
    endcase


  always @ (bm_state or bm_unalign or addrbuf_size_bb or fpb_matched_sized_i or opt_fpb_en)
    case (bm_state)
      `BM_UN_1ST:
        case (bm_unalign)
          `MTX_UNALIGN_HWRD_01: bm_size = `MTX_SZ_BYTE;
          `MTX_UNALIGN_HWRD_11: bm_size = `MTX_SZ_BYTE;
          `MTX_UNALIGN_WORD_01: bm_size = `MTX_SZ_HALF;
          `MTX_UNALIGN_WORD_10: bm_size = `MTX_SZ_HALF;
          `MTX_UNALIGN_WORD_11: bm_size = `MTX_SZ_HALF;
          default:              bm_size = 2'bXX;
        endcase

      `BM_UN_2ND:
        bm_size = `MTX_SZ_BYTE;

      `BM_BB_WR_1:
        bm_size = addrbuf_size_bb;

      `BM_IFPB:
        bm_size = opt_fpb_en ? `MTX_SZ_WORD : 2'b00;
      `BM_ISYS:
        bm_size = `MTX_SZ_WORD;

      `BM_DFPB:
        bm_size = opt_fpb_en ? fpb_matched_sized_i : 2'b00;

      default:
        bm_size = 2'bXX;
    endcase


  assign bm_burst = (bm_state == `BM_ISYS) ? `MTX_BUR_SINGLE : `MTX_BUR_INCR;


  always @ (bm_state or addrbuf_exreq or fpb_matched_exreqd_i or opt_fpb_en)
    case (bm_state)
      `BM_BB_WR_1: bm_exreq = addrbuf_exreq;
      `BM_DFPB:    bm_exreq = opt_fpb_en ? fpb_matched_exreqd_i : 1'b0;

      `BM_IDLE, `BM_WB, `BM_DFPB2, `BM_IFPB, `BM_IFPB2, `BM_ISYS, `BM_ISYS2,
      `BM_UN_1ST, `BM_UN_2ND, `BM_UN_LAST, `BM_BB_RD, `BM_BB_WR_2, `BM_BB_WR_3:
        bm_exreq = 1'b0;

      default:
        bm_exreq = 1'bX;
    endcase


  always @ (bm_state or unalign_no_cross or unalign_2nd_lookup or bm_master_sel or
            opt_fpb_en or addrbuf_prot or addrbuf_mem_attr or is_dcore_prot_i or
            is_dcore_mem_attr_i or unalign_prot or unalign_mem_attr or
            addrbuf_prot or addrbuf_mem_attr or fpb_matched_proti_i or
            fpb_matched_protd_i or fpb_matched_mem_attri_i or
            fpb_matched_mem_attrd_i)
    case (bm_state)
      `BM_UN_1ST:
        case ({unalign_no_cross, unalign_2nd_lookup})
          2'b10, 2'b11:
            begin
              bm_prot     = addrbuf_prot;
              bm_mem_attr = addrbuf_mem_attr;
            end
          2'b01:
            begin
              bm_prot     = is_dcore_prot_i;
              bm_mem_attr = is_dcore_mem_attr_i;
            end
          2'b00:
            begin
              bm_prot     = unalign_prot;
              bm_mem_attr = unalign_mem_attr;
            end
          default:
            begin
              bm_prot     = 4'bXXXX;
              bm_mem_attr = 2'bXX;
            end
        endcase

      `BM_UN_2ND:
        begin
          bm_prot     = bm_master_sel[0] ? unalign_prot     : addrbuf_prot;
          bm_mem_attr = bm_master_sel[0] ? unalign_mem_attr : addrbuf_mem_attr;
        end

      `BM_IFPB:
        begin
          bm_prot     = opt_fpb_en ? fpb_matched_proti_i     : 4'b0000;
          bm_mem_attr = opt_fpb_en ? fpb_matched_mem_attri_i : 2'b00;
        end
      `BM_DFPB:
        begin
          bm_prot     = opt_fpb_en ? fpb_matched_protd_i     : 4'b0000;
          bm_mem_attr = opt_fpb_en ? fpb_matched_mem_attrd_i : 2'b00;
        end

      `BM_BB_WR_1, `BM_ISYS:
        begin
          bm_prot     = addrbuf_prot;
          bm_mem_attr = addrbuf_mem_attr;
        end

      default:
        begin
          bm_prot     = 4'bXXXX;
          bm_mem_attr = 2'bXX;
        end
    endcase

  assign unalign_no_cross = (bm_unalign == `MTX_UNALIGN_HWRD_01) |
                            (bm_unalign == `MTX_UNALIGN_WORD_01) |
                            ~bm_master_sel[0];


  assign bm_cancel_valid  = unalign_2nd_lookup;



  always @ (next_trans or bm_state or bm_unalign)
    case (bm_state)
      `BM_UN_1ST:  i_nxt_bm_state = ((bm_unalign == `MTX_UNALIGN_WORD_01)  |
                                     (bm_unalign == `MTX_UNALIGN_WORD_11)) ?
                                    `BM_UN_2ND : `BM_UN_LAST;
      `BM_UN_2ND:  i_nxt_bm_state = `BM_UN_LAST;
      `BM_BB_WR_1: i_nxt_bm_state = `BM_BB_WR_2;
      `BM_BB_WR_2: i_nxt_bm_state = `BM_BB_WR_3;
      `BM_ISYS:    i_nxt_bm_state = `BM_ISYS2;
      `BM_IFPB:    i_nxt_bm_state = `BM_IFPB2;
      `BM_DFPB:    i_nxt_bm_state = `BM_DFPB2;

      `BM_IDLE,  `BM_UN_LAST, `BM_BB_RD, `BM_BB_WR_3,
      `BM_ISYS2, `BM_IFPB2,   `BM_DFPB2, `BM_WB:
        i_nxt_bm_state = next_trans;

      default:     i_nxt_bm_state = 4'bXXXX;
    endcase

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      bm_state <= `BM_IDLE;
    else if (bm_state_we)
      bm_state <= nxt_bm_state;

  assign clr_bm_state = dpu_dbg_snapstall_i | dap_ahb_habort_i;
  assign nxt_bm_state = {4{~clr_bm_state}}  & i_nxt_bm_state;
  assign bm_state_we  = (slv_ready & (i_nxt_bm_state != bm_state)) | clr_bm_state;


  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        bm_master_sel <= 3'b000;
        bm_slave_sel  <= 2'b00;
        bm_unalign    <= 3'b000;
      end
    else if (bm_trans_state_we)
      begin
        bm_master_sel <= nxt_bm_master_sel;
        bm_slave_sel  <= nxt_bm_slave_sel;
        bm_unalign    <= nxt_bm_unalign;
      end

  assign mst_trans_state_we = slv_ready    &
                              bm_arb_state &
                              ((bm_state != `BM_IDLE) | next_trans_active);

  assign bm_trans_state_we  = mst_trans_state_we | clr_bm_state;

  assign nxt_bm_master_sel = {3{~clr_bm_state}} & next_trans_master;
  assign nxt_bm_slave_sel  = {2{~clr_bm_state}} & next_trans_slave;
  assign nxt_bm_unalign    = {3{~clr_bm_state}} & next_trans_unalign;


  always @ (posedge hclk or negedge full_hreset_n)
    if (!full_hreset_n)
      dap_master_reg <= 2'b00;
    else if (dap_master_reg_we)
      dap_master_reg <= is_dap_master_i;

  assign dap_master_reg_we = opt_dap_en & mst_trans_state_we;


  assign bm_active = (bm_state != `BM_IDLE) | dap_unalgn_held;


  assign bm_arb_state = ( bm_state == `BM_IDLE)                 |
                        ( bm_state == `BM_UN_LAST)              |
                        ( bm_state == `BM_BB_RD)                |
                        ( bm_state == `BM_BB_WR_3)              |
                        ( bm_state == `BM_ISYS2)                |
                        ((bm_state == `BM_IFPB2) & opt_fpb_en)  |
                        ((bm_state == `BM_DFPB2) & opt_fpb_en)  |
                        ( bm_state == `BM_WB);

  assign bm_sel_dcode = bm_slave_sel[0] &
                        ((bm_state == `BM_UN_1ST) |
                         (bm_state == `BM_UN_2ND));

  assign bm_sel_sys   = bm_slave_sel[1] &
                        ((bm_state == `BM_BB_WR_1) |
                         (bm_state == `BM_UN_1ST)  |
                         (bm_state == `BM_UN_2ND)  |
                         (bm_state == `BM_ISYS)    |
                         (opt_fpb_en & (bm_state == `BM_IFPB)) |
                         (opt_fpb_en & (bm_state == `BM_DFPB)));

  assign bm_sel_dcode_data = bm_slave_sel[0] & (bm_state == `BM_WB);
  assign bm_sel_sys_data   = bm_slave_sel[1] & (bm_state == `BM_WB);

  assign bm_state_bb_wr1 = (bm_state == `BM_BB_WR_1);
  assign bm_state_bb_wr2 = (bm_state == `BM_BB_WR_2);



  assign bm_dcore_active_sel[0] = bm_master_sel[0] & bm_slave_sel[0];
  assign bm_dcore_active_sel[1] = bm_master_sel[0] & bm_slave_sel[1];

  assign bm_icore_active_sel    = bm_master_sel[1];

  assign bm_dap_active_sel[0]   = opt_dap_en & bm_master_sel[2] & bm_slave_sel[0];
  assign bm_dap_active_sel[1]   = opt_dap_en & bm_master_sel[2] & bm_slave_sel[1];




  assign bm_dcore_active = bm_master_sel[0] &
                           ((bm_state == `BM_BB_WR_1) |
                            (bm_state == `BM_BB_WR_2) |
                            (bm_state == `BM_UN_1ST)  |
                            (bm_state == `BM_UN_2ND)  |
                            (bm_state == `BM_DFPB));

  assign bm_icore_active = bm_master_sel[1] &
                           (((bm_state == `BM_IFPB) & opt_fpb_en) |
                             (bm_state == `BM_ISYS));

  assign bm_dap_active   = opt_dap_en & bm_master_sel[2] &
                           ((bm_state == `BM_BB_WR_1) |
                            (bm_state == `BM_BB_WR_2) |
                            (bm_state == `BM_UN_1ST)  |
                            (bm_state == `BM_UN_2ND));


  assign bm_dcore_fpb_active = bm_master_sel[0] &
                               ((opt_fpb_en & (bm_state == `BM_DFPB2)) |
                                (bm_state == `BM_WB));

  assign bm_icore_fpb_active = bm_master_sel[1] &
                               ((opt_fpb_en & (bm_state == `BM_IFPB2)) |
                                (bm_state == `BM_ISYS2));

  assign bm_fpb_active       = (opt_const_en &
                                ((bm_state == `BM_DFPB)  |
                                 (bm_state == `BM_DFPB2) |
                                 (bm_state == `BM_IFPB)  |
                                 (bm_state == `BM_IFPB2)));

  assign bm_dcore_active_data = bm_dcore_active & addrbuf_write;
  assign bm_dap_active_data   = bm_dap_active   & addrbuf_write;



  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      icore_fpb_pend <= 1'b0;
    else if (icore_fpb_pend_we)
      icore_fpb_pend <= nxt_icore_fpb_pend;

  assign icore_fpb_pend_we  = opt_fpb_en &
                              (clr_bm_state | fpb_matched_i[0] | icore_fpb_pend);

  assign nxt_icore_fpb_pend = opt_fpb_en    &
                              ~clr_bm_state &
                              (fpb_matched_i[0] |
                               (icore_fpb_pend & ((bm_state != `BM_IFPB) & ~os_sys_bm_wait_i)));


  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      dcore_fpb_pend <= 1'b0;
    else if (dcore_fpb_pend_we)
      dcore_fpb_pend <= nxt_dcore_fpb_pend;

  assign dcore_fpb_pend_we  = opt_fpb_en &
                              (clr_bm_state | fpb_matched_i[1] | dcore_fpb_pend);

  assign nxt_dcore_fpb_pend = opt_fpb_en &
                              ~clr_bm_state &
                              ((fpb_matched_i[1] & ~dcore_unaligned) |
                               (dcore_fpb_pend   & (bm_state != `BM_DFPB)));



  assign mtx_bm_active_o         = bm_active;
  assign bm_dcore_stall_o        = bm_dcore_stall;
  assign bm_icore_stall_o        = bm_icore_stall;
  assign bm_dap_stall_o          = bm_dap_stall;
  assign bm_dcore_fpb_stall_o    = bm_dcore_fpb_stall;
  assign bm_icore_fpb_stall_o    = bm_icore_fpb_stall;
  assign bm_buff_write_accept_o  = bm_buff_write_accept;
  assign bm_icore_to_sys_o       = bm_icore_to_sys;
  assign bm_addr_o               = bm_addr;
  assign bm_trans_o              = bm_trans;
  assign bm_size_o               = bm_size;
  assign bm_write_o              = bm_write;
  assign bm_burst_o              = bm_burst;
  assign bm_prot_o               = bm_prot;
  assign bm_mem_attr_o           = bm_mem_attr;
  assign bm_cancel_valid_o       = bm_cancel_valid;
  assign bm_rdata_o              = bm_rdata;
  assign bm_resp_o               = bm_resp;
  assign bm_exreq_o              = bm_exreq;
  assign bm_readyout_o           = bm_readyout;
  assign bm_readyout_sel_o       = bm_readyout_sel;
  assign bm_wdata_o              = bm_wdata;
  assign bm_master_o             = bm_master;
  assign mtx_write_buff_abort_o  = mtx_write_buff_abort;
  assign bm_mpu_unalign_cancel_o = bm_mpu_unalign_cancel;
  assign bm_sel_dcode_o          = bm_sel_dcode;
  assign bm_sel_sys_o            = bm_sel_sys;
  assign bm_sel_dcode_data_o     = bm_sel_dcode_data;
  assign bm_sel_sys_data_o       = bm_sel_sys_data;
  assign bm_state_bb_wr1_o       = bm_state_bb_wr1;
  assign bm_state_bb_wr2_o       = bm_state_bb_wr2;
  assign bm_dcore_active_sel_o   = bm_dcore_active_sel;
  assign bm_icore_active_sel_o   = bm_icore_active_sel;
  assign bm_dap_active_sel_o     = bm_dap_active_sel;
  assign bm_dcore_active_o       = bm_dcore_active;
  assign bm_icore_active_o       = bm_icore_active;
  assign bm_dap_active_o         = bm_dap_active;
  assign bm_dcore_fpb_active_o   = bm_dcore_fpb_active;
  assign bm_icore_fpb_active_o   = bm_icore_fpb_active;
  assign bm_fpb_active_o         = bm_fpb_active;
  assign bm_dcore_active_data_o  = bm_dcore_active_data;
  assign bm_dap_active_data_o    = bm_dap_active_data;


`ifdef ARM_ASSERT_ON

`include "std_ovl_defines.h"

  reg [8*15:0] ovl_bm_state_mnemonic;

  always @(bm_state)
    case(bm_state)
      `BM_IDLE:    ovl_bm_state_mnemonic = {"BM_IDLE"};
      `BM_UN_1ST:  ovl_bm_state_mnemonic = {"BM_UN_1ST"};
      `BM_UN_2ND:  ovl_bm_state_mnemonic = {"BM_UN_2ND"};
      `BM_UN_LAST: ovl_bm_state_mnemonic = {"BM_UN_LAST"};
      `BM_BB_RD:   ovl_bm_state_mnemonic = {"BM_BB_RD"};
      `BM_BB_WR_1: ovl_bm_state_mnemonic = {"BM_BB_WR_1"};
      `BM_BB_WR_2: ovl_bm_state_mnemonic = {"BM_BB_WR_2"};
      `BM_BB_WR_3: ovl_bm_state_mnemonic = {"BM_BB_WR_3"};
      `BM_ISYS:    ovl_bm_state_mnemonic = {"BM_ISYS"};
      `BM_ISYS2:   ovl_bm_state_mnemonic = {"BM_ISYS2"};
      `BM_IFPB:    ovl_bm_state_mnemonic = {"BM_IFPB"};
      `BM_IFPB2:   ovl_bm_state_mnemonic = {"BM_IFPB2"};
      `BM_DFPB:    ovl_bm_state_mnemonic = {"BM_DFPB"};
      `BM_DFPB2:   ovl_bm_state_mnemonic = {"BM_DFPB2"};
      `BM_WB:      ovl_bm_state_mnemonic = {"BM_WB"};
      default:     ovl_bm_state_mnemonic = {"xxx"};
    endcase

  assert_zero_one_hot #(`OVL_FATAL, 3, `OVL_ASSERT, "bm_master_sel must be exclusive.")
    ovl_master_exclusive(hclk, hreset_n, bm_master_sel);

  assert_zero_one_hot #(`OVL_FATAL, 2, `OVL_ASSERT, "bm_slave_sel must be exclusive.")
    ovl_slave_exclusive(hclk, hreset_n, bm_slave_sel);

  assert_zero_one_hot #(`OVL_FATAL, 2, `OVL_ASSERT, "next_trans_slave select must be exclusive.")
    ovl_next_trans_slave_exclusive(hclk, hreset_n, next_trans_slave);

  assert_always #(`OVL_FATAL, `OVL_ASSERT, "Selected master must be 'b000 when bm_state == BM_IDLE")
    ovl_master_idle_check(hclk, hreset_n,
      ((bm_state == `BM_IDLE) ? (bm_master_sel == 3'b000) : 1'b1));

  assert_always #(`OVL_FATAL, `OVL_ASSERT,
    "Selected master must be non-zero when bm_state != BM_IDLE")
      ovl_master_active_check(hclk, hreset_n,
        ((bm_state != `BM_IDLE) ? (bm_master_sel != 3'b000) : 1'b1));

  assert_always #(`OVL_FATAL, `OVL_ASSERT, "bm_active must be asserted when bm_state != BM_IDLE")
    ovl_bm_active_check(hclk, hreset_n,
      ((bm_state != `BM_IDLE) ? bm_active : 1'b1));

  assert_always #(`OVL_FATAL, `OVL_ASSERT, "addrbuf_write must be 1 when bm_state == BM_BB_WR_1")
    ovl_addrbuf_write_check(hclk, hreset_n,
      ((bm_state == `BM_BB_WR_1) ? addrbuf_write : 1'b1));

  assert_always #(`OVL_FATAL, `OVL_ASSERT, "bm_unalign can't be 'normal' during unaligned states")
    ovl_unalign_check(hclk, hreset_n,
      (((bm_state == `BM_UN_1ST) | (bm_state == `BM_UN_2ND) | (bm_state == `BM_UN_LAST)) ?
        (bm_unalign != 3'b000) : 1'b1));


  assert_never_unknown #(`OVL_FATAL, 4, `OVL_ASSERT, "nxt_dcore_trans goes to x.")
    ovl_nxt_dcore_trans_x (hclk, hreset_n, 1'b1, nxt_dcore_trans);

  assert_never_unknown #(`OVL_FATAL, 4, `OVL_ASSERT, "opt_nxt_dap_trans goes to x.")
    ovl_nxt_dap_trans_x (hclk, hreset_n, 1'b1, opt_nxt_dap_trans);

  assert_never_unknown #(`OVL_FATAL, 4, `OVL_ASSERT, "bm_state goes to x.")
    ovl_bm_state_x (hclk, hreset_n, 1'b1, bm_state);

  assert_never_unknown #(`OVL_FATAL, 2, `OVL_ASSERT, "bm_slave_sel goes to x.")
    ovl_slave_x (hclk, hreset_n, 1'b1, bm_slave_sel);

  assert_never_unknown #(`OVL_FATAL, 3, `OVL_ASSERT, "bm_master_sel goes to x.")
    ovl_master_x (hclk, hreset_n, 1'b1, bm_master_sel);

  assert_never_unknown #(`OVL_FATAL, 3, `OVL_ASSERT, "is_dcore_unalign_i goes to x.")
    ovl_dcore_unalign_x (hclk, hreset_n, 1'b1, is_dcore_unalign_i);

  assert_never_unknown #(`OVL_FATAL, 3, `OVL_ASSERT, "is_dap_unalign_i goes to x.")
    ovl_dap_unalign_x (hclk, hreset_n, 1'b1, is_dap_unalign_i);

  assert_never_unknown #(`OVL_FATAL, 4, `OVL_ASSERT, "rdata_mask goes to X.")
    ovl_rdata_mask_x (hclk, hreset_n, 1'b1, rdata_mask);


  reg ovl_mpu_canceli_state;
  reg ovl_nxt_mpu_canceli_state;

  always @ (mpu_canceli_i or bm_state or i_nxt_bm_state or ovl_mpu_canceli_state)
    case (ovl_mpu_canceli_state)
     1'b0:
       ovl_nxt_mpu_canceli_state = mpu_canceli_i & (i_nxt_bm_state == `BM_ISYS);
     1'b1:
       ovl_nxt_mpu_canceli_state = ((bm_state == `BM_ISYS2) &
                                    mpu_canceli_i           &
                                    (i_nxt_bm_state == `BM_ISYS)) |
                                   (bm_state != `BM_ISYS2);
     default:
       ovl_nxt_mpu_canceli_state = 1'bX;
    endcase

  always @(posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ovl_mpu_canceli_state <= 1'b0;
    else if (mtx_ahb_hreadys_i)
      ovl_mpu_canceli_state <= ovl_nxt_mpu_canceli_state;

  wire [1:0] ovl_cancel_state     = {unalign_cancel, unalign_2nd_lookup};

  wire       BMState_BM_IDLE      = (bm_state == `BM_IDLE);
  wire       BMState_BM_HWRD_01_1 = (bm_state == `BM_UN_1ST)  & (bm_unalign==`MTX_UNALIGN_HWRD_01);
  wire       BMState_BM_HWRD_01_2 = (bm_state == `BM_UN_LAST) & (bm_unalign==`MTX_UNALIGN_HWRD_01);
  wire       BMState_BM_HWRD_11_1 = (bm_state == `BM_UN_1ST)  & (bm_unalign==`MTX_UNALIGN_HWRD_11);
  wire       BMState_BM_HWRD_11_2 = (bm_state == `BM_UN_LAST) & (bm_unalign==`MTX_UNALIGN_HWRD_11);
  wire       BMState_BM_WORD_01_1 = (bm_state == `BM_UN_1ST)  & (bm_unalign==`MTX_UNALIGN_WORD_01);
  wire       BMState_BM_WORD_01_2 = (bm_state == `BM_UN_2ND)  & (bm_unalign==`MTX_UNALIGN_WORD_01);
  wire       BMState_BM_WORD_01_3 = (bm_state == `BM_UN_LAST) & (bm_unalign==`MTX_UNALIGN_WORD_01);
  wire       BMState_BM_WORD_10_1 = (bm_state == `BM_UN_1ST)  & (bm_unalign==`MTX_UNALIGN_WORD_10);
  wire       BMState_BM_WORD_10_2 = (bm_state == `BM_UN_LAST) & (bm_unalign==`MTX_UNALIGN_WORD_10);
  wire       BMState_BM_WORD_11_1 = (bm_state == `BM_UN_1ST)  & (bm_unalign==`MTX_UNALIGN_WORD_11);
  wire       BMState_BM_WORD_11_2 = (bm_state == `BM_UN_2ND)  & (bm_unalign==`MTX_UNALIGN_WORD_11);
  wire       BMState_BM_WORD_11_3 = (bm_state == `BM_UN_LAST) & (bm_unalign==`MTX_UNALIGN_WORD_11);
  wire       BMState_BM_BB_RD     = (bm_state == `BM_BB_RD);
  wire       BMState_BM_BB_WR_1   = (bm_state == `BM_BB_WR_1);
  wire       BMState_BM_BB_WR_2   = (bm_state == `BM_BB_WR_2);
  wire       BMState_BM_BB_WR_3   = (bm_state == `BM_BB_WR_3);
  wire       BMState_BM_IFPB      = (bm_state == `BM_IFPB);
  wire       BMState_BM_IFPB2     = (bm_state == `BM_IFPB2);
  wire       BMState_BM_DFPB      = (bm_state == `BM_DFPB);
  wire       BMState_BM_DFPB2     = (bm_state == `BM_DFPB2);
  wire       BMState_BM_WB        = (bm_state == `BM_WB);
  wire       BMState_BM_ISYS      = (bm_state == `BM_ISYS);
  wire       BMState_BM_ISYS2     = (bm_state == `BM_ISYS2);

`endif

endmodule

`include "cm3_mtx_undefs.vp"
