`include "cm3_mtx_defs.vp"

module cm3_mtx_decode_dcore
  (// Inputs
   hclk, hreset_n, mtx_mpu_ahb_hreadyoutd_i, dpu_dbg_snapstall_i,
   is_dcore_addr_i, is_dcore_unalign_i, os_dcore_to_dcode_accept_i,
   os_dcore_to_dcode_active_i, mtx_ahb_hreadyd_i, mtx_ahb_hrespd_i,
   mtx_exrespd_i, mtx_ahb_hrdatad_i, os_dcore_to_sys_accept_i,
   os_dcore_to_sys_active_i, mtx_ahb_hreadys_i, mtx_ahb_hresps_i,
   mtx_exresps_i, mtx_ahb_hrdatas_i, os_dcore_to_ppb_accept_i,
   os_dcore_to_ppb_active_i, mtx_ppb_hready_i, mtx_ppb_hresp_i,
   mtx_ppb_hrdata_i, bm_state_bb_wr2_i, bm_dcore_active_i,
   bm_dcore_active_sel_i, bm_dcore_active_data_i, bm_dcore_fpb_active_i,
   bm_dcore_stall_i, bm_dcore_fpb_stall_i, bm_readyout_sel_i, bm_resp_i,
   bm_rdata_i,
   dec_dcore_sel_dcode_o, dec_dcore_sel_sys_o, dec_dcore_sel_ppb_o,
   dec_dcore_sel_bb_o, dec_dcore_sel_dcode_held_o, dec_dcore_sel_sys_held_o,
   dec_dcore_sel_ppb_held_o, dec_dcore_accept_o, dec_dcore_active_o,
   dec_dcore_active_data_o, dec_dcore_readyout_o, dec_dcore_resp_o,
   dec_dcore_exresp_o, mtx_dpu_ahb_hrdatad_o
  );


  parameter     BB_PRESENT = 1;             // Bit banding present if 1


  input         hclk;                       // AHB system clock
  input         hreset_n;                   // AHB system reset

  input  [31:0] is_dcore_addr_i;            // DCore HADDR
  input   [2:0] is_dcore_unalign_i;         // Unaligned transaction encoding
  input         mtx_mpu_ahb_hreadyoutd_i;   // HREADY output for DCore port

  input         dpu_dbg_snapstall_i;

  input         os_dcore_to_dcode_accept_i; // DCore -> DCode accepted
  input         os_dcore_to_dcode_active_i; // DCore -> DCode active
  input         mtx_ahb_hreadyd_i;
  input   [1:0] mtx_ahb_hrespd_i;
  input         mtx_exrespd_i;
  input  [31:0] mtx_ahb_hrdatad_i;

  input         os_dcore_to_sys_accept_i;   // DCore -> SYS accepted
  input         os_dcore_to_sys_active_i;   // DCore -> SYS active
  input         mtx_ahb_hreadys_i;
  input   [1:0] mtx_ahb_hresps_i;
  input         mtx_exresps_i;
  input  [31:0] mtx_ahb_hrdatas_i;

  input         os_dcore_to_ppb_accept_i;   // DCore -> PPB accepted
  input         os_dcore_to_ppb_active_i;   // DCore -> PPB active
  input         mtx_ppb_hready_i;
  input   [1:0] mtx_ppb_hresp_i;
  input  [31:0] mtx_ppb_hrdata_i;

  input         bm_state_bb_wr2_i;
  input         bm_dcore_active_i;
  input   [1:0] bm_dcore_active_sel_i;
  input         bm_dcore_active_data_i;
  input         bm_dcore_stall_i;
  input         bm_dcore_fpb_active_i;
  input         bm_dcore_fpb_stall_i;
  input   [1:0] bm_readyout_sel_i;
  input         bm_resp_i;
  input  [31:0] bm_rdata_i;

  output        dec_dcore_sel_dcode_o;      // DCode select
  output        dec_dcore_sel_dcode_held_o; // DCode select held
  output        dec_dcore_sel_sys_o;        // SYS select
  output        dec_dcore_sel_sys_held_o;   // SYS select held
  output        dec_dcore_sel_ppb_o;        // PPB select
  output        dec_dcore_sel_ppb_held_o;   // PPB select held
  output        dec_dcore_sel_bb_o;         // Bit-band select (1st cycle)

  output        dec_dcore_accept_o;         // Transaction accepted from DCore
  output        dec_dcore_active_o;         // Transaction active from DCore
  output        dec_dcore_active_data_o;    // Transaction active, data phase
  output        dec_dcore_readyout_o;       // HREADY from output stage
  output  [1:0] dec_dcore_resp_o;           // HRESP from output stage
  output        dec_dcore_exresp_o;         // EXRESP from output stage
  output [31:0] mtx_dpu_ahb_hrdatad_o;      // HRDATA from output stage



  wire          sel_dcode;                  // DCode address in Code space
  wire          sel_sys;                    // DCode address in System space
  wire          sel_ppb;                    // DCode address in PPB space
  wire          sel_bb;                     // Bit-band access
  wire          mst_bb_en;                  // Logic removal for bit-band
  wire    [2:0] output_port_addr;           // One-hot output port selection

  wire          sel_dcode_held;             // DCore -> DCode held
  wire          sel_sys_held;               // DCore -> SYS held
  wire          sel_ppb_held;               // DCore -> PPB held
  wire          dcore_accept;               // DCore trans accepted by output stage
  wire          dcore_active;               // DCore trans active in output stage
  reg           dcore_to_dcode_accept_reg;  // Registered DCore -> DCode accepted
  reg           dcore_to_sys_accept_reg;    // Registered DCore -> SYS accepted
  reg           dcore_to_ppb_accept_reg;    // Registered DCore -> PPB accepted
  wire          dcore_accept_reg_we;        // Registered accept write enable
  wire          dcore_active_data;          // Combined registered accept

  wire          nxt_sel_bm_data;            // Next data phase to Bit Master
  reg     [3:0] def_output_port_data;       // Default data phase output selection
  wire    [3:0] nxt_def_output_port_data;
  reg     [3:0] output_port_data;           // Data phase output port selection

  wire   [31:0] dcore_rdata;                // DCore HRDATA mux
  wire    [1:0] dcore_resp;                 // DCore HRESP mux
  wire          dcore_exresp;               // DCore EXRESP mux
  wire          dcore_readyout;             // DCore HREADYOUT mux

`ifdef ARM_DSM

  wire          dsm_bb_present;

  assign mst_bb_en = (dsm_bb_present) ? 1'b1 : 1'b0;

`else

  assign mst_bb_en = (BB_PRESENT == 1) ? 1'b1 : 1'b0;

`endif



  assign sel_dcode = ~(|is_dcore_addr_i[31:29]);
  assign sel_ppb   = (is_dcore_addr_i[31:20] == 12'hE00);
  assign sel_sys   = ~sel_dcode & ~sel_ppb;

  assign output_port_addr = {sel_ppb, sel_sys, sel_dcode};

  assign sel_bb = ({is_dcore_addr_i[31], is_dcore_addr_i[28:25]} == 5'b00001) &
                  (is_dcore_addr_i[30] ^ is_dcore_addr_i[29]) & mst_bb_en;




  assign sel_dcode_held = dcore_to_sys_accept_reg  |
                          dcore_to_ppb_accept_reg  |
                          bm_dcore_active_i        |
                          bm_dcore_active_sel_i[1] |
                          bm_dcore_stall_i         |
                          bm_dcore_fpb_stall_i;

  assign sel_sys_held   = dcore_to_dcode_accept_reg |
                          dcore_to_ppb_accept_reg   |
                          bm_dcore_active_i         |
                          bm_dcore_active_sel_i[0]  |
                          bm_dcore_stall_i          |
                          bm_dcore_fpb_stall_i      |
                          bm_state_bb_wr2_i;

  assign sel_ppb_held   = dcore_to_dcode_accept_reg |
                          dcore_to_sys_accept_reg   |
                          bm_dcore_active_i         |
                          bm_dcore_active_sel_i[0]  |
                          bm_dcore_active_sel_i[1]  |
                          bm_dcore_fpb_stall_i;

  assign dcore_accept = (output_port_addr[0] & os_dcore_to_dcode_accept_i) |
                        (output_port_addr[1] & os_dcore_to_sys_accept_i)   |
                        (output_port_addr[2] & os_dcore_to_ppb_accept_i);

  assign dcore_active = (output_port_addr[0] & os_dcore_to_dcode_active_i) |
                        (output_port_addr[1] & os_dcore_to_sys_active_i)   |
                        (output_port_addr[2] & os_dcore_to_ppb_active_i);

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      dcore_to_dcode_accept_reg <= 1'b0;
    else if (dcore_accept_reg_we)
      dcore_to_dcode_accept_reg <= os_dcore_to_dcode_accept_i;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      dcore_to_sys_accept_reg <= 1'b0;
    else if (dcore_accept_reg_we)
      dcore_to_sys_accept_reg <= os_dcore_to_sys_accept_i;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      dcore_to_ppb_accept_reg <= 1'b0;
    else if (dcore_accept_reg_we)
      dcore_to_ppb_accept_reg <= os_dcore_to_ppb_accept_i;

  assign dcore_accept_reg_we = mtx_mpu_ahb_hreadyoutd_i | dpu_dbg_snapstall_i;

  assign dcore_active_data = dcore_to_dcode_accept_reg |
                             dcore_to_sys_accept_reg   |
                             dcore_to_ppb_accept_reg;




  assign nxt_sel_bm_data = ((is_dcore_unalign_i != `MTX_UNALIGN_NORMAL) &
                              ~output_port_addr[2]) |
                           sel_bb                   |
                           bm_dcore_active_data_i;

  always @ (posedge hclk or negedge hreset_n)
      if (!hreset_n)
        def_output_port_data <= 4'b0000;
      else if (dcore_accept_reg_we)
        def_output_port_data <= nxt_def_output_port_data;

  assign nxt_def_output_port_data = nxt_sel_bm_data ? 4'b1000 : {1'b0, output_port_addr};



  always @ (bm_dcore_fpb_stall_i or bm_dcore_fpb_active_i or def_output_port_data)
    case ({bm_dcore_fpb_stall_i, bm_dcore_fpb_active_i})
      2'b10, 2'b11: output_port_data = 4'b0000; // No port will force HREADY low
      2'b01:        output_port_data = 4'b1000; // FPB remap from Bit Master
      2'b00:        output_port_data = def_output_port_data;
      default:      output_port_data = 4'bXXXX;
    endcase



  assign dcore_readyout = (// DCODE
                           ((output_port_data[0] |
                             (output_port_data[3] & bm_readyout_sel_i[0])) &
                            mtx_ahb_hreadyd_i) |
                           ((output_port_data[1] |
                             (output_port_data[3] & bm_readyout_sel_i[1])) &
                            mtx_ahb_hreadys_i) |
                           (output_port_data[2] & mtx_ppb_hready_i));

  assign dcore_resp     = ({2{output_port_data[0]}} & mtx_ahb_hrespd_i) |
                          ({2{output_port_data[1]}} & mtx_ahb_hresps_i) |
                          ({2{output_port_data[2]}} & mtx_ppb_hresp_i)  |
                          ({2{output_port_data[3]}} & {1'b0, bm_resp_i});

  assign dcore_exresp   = (output_port_data[0] & mtx_exrespd_i) |
                          (output_port_data[1] & mtx_exresps_i);

  assign dcore_rdata    = ({32{output_port_data[0]}} & mtx_ahb_hrdatad_i) |
                          ({32{output_port_data[1]}} & mtx_ahb_hrdatas_i) |
                          ({32{output_port_data[2]}} & mtx_ppb_hrdata_i)  |
                          ({32{output_port_data[3]}} & bm_rdata_i);



  assign dec_dcore_sel_dcode_o       = sel_dcode;
  assign dec_dcore_sel_sys_o         = sel_sys;
  assign dec_dcore_sel_ppb_o         = sel_ppb;
  assign dec_dcore_sel_bb_o          = sel_bb;
  assign dec_dcore_sel_dcode_held_o  = sel_dcode_held;
  assign dec_dcore_sel_sys_held_o    = sel_sys_held;
  assign dec_dcore_sel_ppb_held_o    = sel_ppb_held;

  assign dec_dcore_accept_o          = dcore_accept;
  assign dec_dcore_active_o          = dcore_active;
  assign dec_dcore_active_data_o     = dcore_active_data;

  assign dec_dcore_readyout_o        = dcore_readyout;
  assign dec_dcore_resp_o            = dcore_resp;
  assign dec_dcore_exresp_o          = dcore_exresp;
  assign mtx_dpu_ahb_hrdatad_o       = dcore_rdata;



`ifdef ARM_ASSERT_ON

  assert_implication #(0,0,"Code space decode error.")
   ovl_code_mem(hclk, hreset_n,
     ((is_dcore_addr_i >= 32'h00000000) && (is_dcore_addr_i <= 32'h1FFFFFFF)),
       (output_port_addr == 4'b0001));

  assert_implication #(0,0,"SRAM space decode error.")
   ovl_sram_mem(hclk, hreset_n,
     ((is_dcore_addr_i >= 32'h20000000) && (is_dcore_addr_i <= 32'h3FFFFFFF)),
       (output_port_addr == 4'b0010));

  assert_implication #(0,0,"Peripheral space decode error.")
   ovl_peripheral_mem(hclk, hreset_n,
     ((is_dcore_addr_i >= 32'h40000000) && (is_dcore_addr_i <= 32'h5FFFFFFF)),
       (output_port_addr == 4'b0010));

  assert_implication #(0,0,"External RAM decode error.")
   ovl_external_mem(hclk, hreset_n,
     ((is_dcore_addr_i >= 32'h60000000) && (is_dcore_addr_i <= 32'h9FFFFFFF)),
       (output_port_addr == 4'b0010));

  assert_implication #(0,0,"External Device decode error.")
   ovl_external_device_mem(hclk, hreset_n,
     ((is_dcore_addr_i >= 32'hA0000000) && (is_dcore_addr_i <= 32'hDFFFFFFF)),
       (output_port_addr == 4'b0010));

  assert_implication #(0,0,"PPB decode error.")
   ovl_ppb_mem(hclk, hreset_n,
     ((is_dcore_addr_i >= 32'hE0000000) && (is_dcore_addr_i <= 32'hE00FFFFF)),
      (output_port_addr == 4'b0100));

  assert_implication #(0,0,"System space decode error.")
   ovl_system_mem(hclk, hreset_n,
     ((is_dcore_addr_i >= 32'hE0100000) && (is_dcore_addr_i <= 32'hFFFFFFFF)),
       (output_port_addr == 4'b0010));


  assert_zero_one_hot #(0,3,0,"Output stage selection must be exclusive.")
    ovl_output_stage_sel(hclk, hreset_n, {sel_ppb, sel_sys, sel_dcode});


  assert_zero_one_hot #(0,3,0,"output_port_addr must be exclusive.")
    ovl_output_port_addr_exclusive(hclk, hreset_n, output_port_addr);

  assert_never_unknown #(0,3,0,"output_port_addr goes to X.")
    ovl_output_port_addr_x(hclk, hreset_n, 1'b1, output_port_addr);

  assert_zero_one_hot #(0,4,0,"output_port_data must be exclusive.")
    ovl_output_port_data_exclusive(hclk, hreset_n, output_port_data);

  assert_never_unknown #(0,4,0,"output_port_data goes to X.")
    ovl_output_port_data_x(hclk, hreset_n, 1'b1, output_port_data);

`endif

endmodule

`include "cm3_mtx_undefs.vp"
