module cm3_mtx_output_stage_dcode
  (// Inputs
   hclk, hreset_n, mpu_canceld_i, bm_mpu_unalign_cancel_i,
   dec_dcore_sel_dcode_i, dec_dcore_sel_dcode_held_i, is_dcore_master_i,
   is_dcore_addr_i, is_dcore_trans_i, is_dcore_write_i,
   is_dcore_size_unalign_i, is_dcore_burst_i, is_dcore_prot_i,
   is_dcore_exreq_i, dpu_ahb_hwdatad_i, is_dcore_held_trans_i,
   dec_dap_sel_dcode_i, dec_dap_sel_dcode_held_i, is_dap_master_i,
   is_dap_addr_i, is_dap_trans_i, is_dap_write_i, is_dap_size_unalign_i,
   is_dap_burst_i, is_dap_prot_i, dap_ahb_hwdata_i, is_dap_held_trans_i,
   os_icore_to_icode_held_i, bm_sel_dcode_i, bm_sel_dcode_data_i, bm_master_i,
   bm_addr_i, bm_trans_i, bm_cancel_valid_i, bm_write_i, bm_size_i, bm_burst_i,
   bm_prot_i, bm_wdata_i, mtx_ahb_hreadyd_i,
   os_dcore_to_dcode_accept_o, os_dap_to_dcode_accept_o,
   os_dcore_to_dcode_active_o, os_dap_to_dcode_active_o,
   os_dap_to_dcode_held_req_o, mtx_ahb_hmasterd_o, mtx_ahb_haddrd_o,
   mtx_ahb_htransd_o, mtx_ahb_hwrited_o, mtx_ahb_hsized_o, mtx_ahb_hburstd_o,
   mtx_ahb_hprotd_o, mtx_mem_attrd_o, mtx_exreqd_o, mtx_ahb_hwdatad_o
  );

  parameter CONST_AHB_CTRL = 0;  // Constant AHB control info if 1


  input         hclk;                        // AHB system clock
  input         hreset_n;                    // AHB system reset

  input         dec_dcore_sel_dcode_i;       // DCore -> DCode select
  input         dec_dcore_sel_dcode_held_i;  // DCore -> DCode select held
  input         dec_dap_sel_dcode_i;         // DAP -> DCode select
  input         dec_dap_sel_dcode_held_i;    // DAP -> DCode select held
  input         bm_sel_dcode_i;              // Bit Master select
  input         bm_sel_dcode_data_i;         // Bit Master select (data port)

  input  [31:0] is_dcore_addr_i;
  input   [1:0] is_dcore_master_i;
  input   [1:0] is_dcore_trans_i;
  input         is_dcore_write_i;
  input   [1:0] is_dcore_size_unalign_i;
  input   [2:0] is_dcore_burst_i;
  input   [3:0] is_dcore_prot_i;
  input         is_dcore_exreq_i;
  input  [31:0] dpu_ahb_hwdatad_i;
  input         is_dcore_held_trans_i;
  output        os_dcore_to_dcode_accept_o;  // DCore -> DCode accepted
  output        os_dcore_to_dcode_active_o;  // DCore -> DCode active

  input  [31:0] is_dap_addr_i;
  input   [1:0] is_dap_master_i;
  input   [1:0] is_dap_trans_i;
  input         is_dap_write_i;
  input   [1:0] is_dap_size_unalign_i;
  input   [2:0] is_dap_burst_i;
  input   [3:0] is_dap_prot_i;
  input  [31:0] dap_ahb_hwdata_i;
  input         is_dap_held_trans_i;
  output        os_dap_to_dcode_accept_o;    // DAP -> DCode accepted
  output        os_dap_to_dcode_active_o;    // DAP -> DCode active
  output        os_dap_to_dcode_held_req_o;  // DAP -> DCode waited request

  input         os_icore_to_icode_held_i;

  input  [31:0] bm_addr_i;
  input   [1:0] bm_master_i;
  input   [1:0] bm_trans_i;
  input         bm_write_i;
  input   [1:0] bm_size_i;
  input   [2:0] bm_burst_i;
  input   [3:0] bm_prot_i;
  input  [31:0] bm_wdata_i;
  input         bm_cancel_valid_i;

  input         mpu_canceld_i;
  input         bm_mpu_unalign_cancel_i;

  output [31:0] mtx_ahb_haddrd_o;
  output  [1:0] mtx_ahb_hmasterd_o;
  output  [1:0] mtx_ahb_htransd_o;
  output        mtx_ahb_hwrited_o;
  output  [2:0] mtx_ahb_hsized_o;
  output  [2:0] mtx_ahb_hburstd_o;
  output  [3:0] mtx_ahb_hprotd_o;
  output  [1:0] mtx_mem_attrd_o;
  output        mtx_exreqd_o;
  output [31:0] mtx_ahb_hwdatad_o;
  input         mtx_ahb_hreadyd_i;


  wire          dcore_held_req;
  wire          dap_held_req;
  wire          bm_held_req;
  wire          dcore_wait;
  wire          bm_wait;
  wire          dap_wait;
  wire          dcore_req;            // DCore request
  wire          dap_req;              // DAP request
  wire          bm_req;               // Bit Master request
  reg     [2:0] input_port_addr;      // Prioritized input port address

  wire          dcore_to_dcode_accept;
  wire          dap_to_dcode_accept;
  wire          dcore_to_dcode_active;
  wire          dap_to_dcode_active;

  wire   [31:0] ahb_haddr;
  wire    [1:0] ahb_hmaster;
  wire    [1:0] i_ahb_htrans;         // HTRANS before cancel
  wire          ahb_hwrite;
  wire    [2:0] ahb_hsize;
  wire    [2:0] ahb_hburst;
  wire    [3:0] ahb_hprot;
  wire   [31:0] ahb_hwdata;
  wire    [1:0] mem_attr;
  wire          exreq;
  wire          cancel_trans;         // Late cancel signal for timing
  wire    [1:0] ahb_htrans;           // HTRANS with late cancel factored in
  wire          ahb_ctrl_valid_we;
  reg           ahb_ctrl_valid;
  reg     [2:0] ahb_ctrl_master;

  reg     [2:0] input_port_data;      // Input port address for data phase
  reg     [2:0] nxt_input_port_data;
  wire          input_port_data_we;
  wire          dcore_cancel_unalign; // Select write buffer for unaligned cancel

  wire          opt_const_en;

`ifdef ARM_DSM

  wire          dsm_const_ahb_ctrl;

  assign opt_const_en = (dsm_const_ahb_ctrl > 0) ? 1'b1 : 1'b0;

`else

  assign opt_const_en = (CONST_AHB_CTRL > 0) ? 1'b1 : 1'b0;

`endif


  assign dcore_held_req = ahb_ctrl_valid & ahb_ctrl_master[0] & opt_const_en;
  assign dap_held_req   = ahb_ctrl_valid & ahb_ctrl_master[1] & opt_const_en;
  assign bm_held_req    = ahb_ctrl_valid & ahb_ctrl_master[2] & opt_const_en;

  assign dcore_wait     = dap_held_req | bm_held_req | os_icore_to_icode_held_i;
  assign bm_wait        = dap_held_req | dcore_held_req;
  assign dap_wait       = bm_held_req  | dcore_held_req | os_icore_to_icode_held_i;

  assign dcore_req = (is_dcore_held_trans_i & dec_dcore_sel_dcode_i &
                      ~dec_dcore_sel_dcode_held_i & ~dcore_wait);
  assign dap_req   = (is_dap_held_trans_i & dec_dap_sel_dcode_i &
                      (dap_held_req | ~dec_dap_sel_dcode_held_i) & ~dap_wait);
  assign bm_req    = bm_sel_dcode_i & ~bm_wait;

  always @ (bm_req or dcore_req or dap_req)
    case ({bm_req, dcore_req, dap_req})
      3'b100, 3'b101, 3'b110, 3'b111: input_port_addr = 3'b100;  // Bit Master
      3'b010, 3'b011:                 input_port_addr = 3'b001;  // DCore
      3'b001:                         input_port_addr = 3'b010;  // DAP
      3'b000:                         input_port_addr = 3'b000;  // None
      default:                        input_port_addr = 3'bXXX;  // X propagation
    endcase



  assign dcore_to_dcode_accept = input_port_addr[0];
  assign dap_to_dcode_accept   = input_port_addr[1];

  assign dcore_to_dcode_active = ~dec_dcore_sel_dcode_held_i & ~bm_req & mtx_ahb_hreadyd_i & ~dcore_wait;
  assign dap_to_dcode_active   = dap_to_dcode_accept & mtx_ahb_hreadyd_i & ~dap_wait;



  assign ahb_haddr    = ({32{input_port_addr[0]}} & {3'b000, is_dcore_addr_i[28:0]}) |
                        ({32{input_port_addr[1]}} & {3'b000, is_dap_addr_i[28:0]})   |
                        ({32{input_port_addr[2]}} & {3'b000, bm_addr_i[28:0]});

  assign ahb_hmaster  = ({2{input_port_addr[0]}} & is_dcore_master_i) |
                        ({2{input_port_addr[1]}} & is_dap_master_i)   |
                        ({2{input_port_addr[2]}} & bm_master_i);

  assign i_ahb_htrans = ({2{input_port_addr[0]}} & is_dcore_trans_i) |
                        ({2{input_port_addr[1]}} & is_dap_trans_i)   |
                        ({2{input_port_addr[2]}} & bm_trans_i);

  assign ahb_hwrite   = (input_port_addr[0] & is_dcore_write_i) |
                        (input_port_addr[1] & is_dap_write_i)   |
                        (input_port_addr[2] & bm_write_i);

  assign ahb_hsize    = ({3{input_port_addr[0]}} & {1'b0, is_dcore_size_unalign_i}) |
                        ({3{input_port_addr[1]}} & {1'b0, is_dap_size_unalign_i})   |
                        ({3{input_port_addr[2]}} & {1'b0, bm_size_i});

  assign ahb_hburst   = ({3{input_port_addr[0]}} & is_dcore_burst_i) |
                        ({3{input_port_addr[1]}} & is_dap_burst_i)   |
                        ({3{input_port_addr[2]}} & bm_burst_i);

  assign ahb_hprot    = {2'b10, (({2{input_port_addr[0]}} & is_dcore_prot_i[1:0]) |
                                 ({2{input_port_addr[1]}} & is_dap_prot_i[1:0])   |
                                 ({2{input_port_addr[2]}} & bm_prot_i[1:0]))};

  assign mem_attr     = 2'b01;

  assign exreq        = input_port_addr[0] & is_dcore_exreq_i;


  assign cancel_trans = (input_port_addr[2] & mpu_canceld_i & bm_cancel_valid_i) |
                        (input_port_addr[0] & mpu_canceld_i);

  assign ahb_htrans   = {2{~cancel_trans}} & i_ahb_htrans;

  assign ahb_ctrl_valid_we = (opt_const_en &
                              (ahb_ctrl_valid |
                               ((|ahb_htrans) & ~mtx_ahb_hreadyd_i)));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        ahb_ctrl_valid  <= 1'b0;
        ahb_ctrl_master <= 3'b000;
      end
    else if (ahb_ctrl_valid_we)
      begin
        ahb_ctrl_valid  <= (|ahb_htrans) & ~mtx_ahb_hreadyd_i;
        ahb_ctrl_master <= input_port_addr;
      end


  assign dcore_cancel_unalign = bm_mpu_unalign_cancel_i &
                                ~mtx_ahb_hreadyd_i      &
                                input_port_data[0];

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      input_port_data <= 3'b000;
    else if (input_port_data_we)
      input_port_data <= nxt_input_port_data;

  always @ (ahb_hwrite or dcore_cancel_unalign or input_port_addr)
    case ({ahb_hwrite, dcore_cancel_unalign})
      2'b00, 2'b01: nxt_input_port_data = 3'b000;
      2'b10:        nxt_input_port_data = input_port_addr;
      2'b11:        nxt_input_port_data = 3'b100;
      default:      nxt_input_port_data = 3'bXXX;
    endcase

  assign input_port_data_we = dcore_cancel_unalign | mtx_ahb_hreadyd_i;

  assign ahb_hwdata = ({32{ bm_sel_dcode_data_i | input_port_data[2]}} & bm_wdata_i)        |
                      ({32{~bm_sel_dcode_data_i & input_port_data[0]}} & dpu_ahb_hwdatad_i) |
                      ({32{~bm_sel_dcode_data_i & input_port_data[1]}} & dap_ahb_hwdata_i);



  assign os_dcore_to_dcode_accept_o = dcore_to_dcode_accept;
  assign os_dcore_to_dcode_active_o = dcore_to_dcode_active;
  assign os_dap_to_dcode_accept_o   = dap_to_dcode_accept;
  assign os_dap_to_dcode_active_o   = dap_to_dcode_active;
  assign os_dap_to_dcode_held_req_o = dap_held_req;
  assign mtx_ahb_haddrd_o           = ahb_haddr;
  assign mtx_ahb_hmasterd_o         = ahb_hmaster;
  assign mtx_ahb_htransd_o          = ahb_htrans;
  assign mtx_ahb_hwrited_o          = ahb_hwrite;
  assign mtx_ahb_hsized_o           = ahb_hsize;
  assign mtx_ahb_hburstd_o          = ahb_hburst;
  assign mtx_ahb_hprotd_o           = ahb_hprot;
  assign mtx_ahb_hwdatad_o          = ahb_hwdata;
  assign mtx_mem_attrd_o            = mem_attr;
  assign mtx_exreqd_o               = exreq;



`ifdef ARM_ASSERT_ON

  `include "cm3_mtx_defs.v"

  assert_zero_one_hot #(0,3,0,"input_port_addr must be exclusive.")
    ovl_input_port_addr_exclusive(hclk, hreset_n, input_port_addr);

  assert_zero_one_hot #(0,3,0,"input_port_data must be exclusive.")
    ovl_input_port_data_exclusive(hclk, hreset_n, input_port_data);

  assert_always #(0,0,"is_dcore_addr_i[31:29] must be 3'b000 when input_port_addr = 3'b001.")
    ovl_dcore_addr_valid(hclk, hreset_n,
      ((input_port_addr == 3'b001) ? (is_dcore_addr_i[31:29] == 3'b000) : 1'b1));

  assert_always #(0,0,"is_dap_addr_i[31:29] must be 3'b000 when input_port_addr = 3'b010.")
    ovl_dap_addr_valid(hclk, hreset_n,
      ((input_port_addr == 3'b010) ? (is_dap_addr_i[31:29] == 3'b000) : 1'b1));

  assert_never_unknown #(0,3,0,"input_port_addr goes to X.")
    ovl_input_port_addr_x(hclk, hreset_n, 1'b1, input_port_addr);

  assert_never_unknown #(0,3,0,"input_port_data goes to X.")
    ovl_input_port_data_x(hclk, hreset_n, 1'b1, input_port_data);

  assert_never_unknown #(0,32,0,"ahb_haddr goes to X during a transaction.")
    ovl_haddr_x(hclk, hreset_n, (ahb_htrans != `MTX_TRN_IDLE), ahb_haddr);

  assert_never_unknown #(0,1,0,"ahb_hwrite goes to X during a transaction.")
    ovl_hwrite_x(hclk, hreset_n, (ahb_htrans != `MTX_TRN_IDLE), ahb_hwrite);

  assert_never_unknown #(0,3,0,"ahb_hsize goes to X during a transaction.")
    ovl_hsize_x(hclk, hreset_n, (ahb_htrans != `MTX_TRN_IDLE), ahb_hsize);

  assert_never_unknown #(0,3,0,"ahb_hburst goes to X during a transaction.")
    ovl_hburst_x(hclk, hreset_n, (ahb_htrans != `MTX_TRN_IDLE), ahb_hburst);

  assert_never_unknown #(0,4,0,"ahb_hprot goes to X during a transaction.")
    ovl_hprot_x(hclk, hreset_n, (ahb_htrans != `MTX_TRN_IDLE), ahb_hprot);

  assert_never_unknown #(0,2,0,"mem_attr goes to X during a transaction.")
    ovl_mem_attr_x (hclk, hreset_n, (ahb_htrans != `MTX_TRN_IDLE), mem_attr);

  assert_never_unknown #(0,1,0,"exreq goes to X during a transaction.")
    ovl_exreq_x (hclk, hreset_n, (ahb_htrans != `MTX_TRN_IDLE), exreq);

  assert_always #(0,0,"ahb_hprot[3:2] must always be 2'b10.")
    ovl_hprot32_valid(hclk, hreset_n, (ahb_hprot[3:2] == 2'b10));

  assert_always #(0,0,"MEMATTRM must always be 2'b01.")
    ovl_mem_attr_valid(hclk, hreset_n, (mtx_mem_attrd_o == 2'b01));

  `include "cm3_mtx_undefs.v"

`endif

endmodule
