`include "cm3_mtx_defs.vp"

module cm3_mtx_output_stage_sys
  (// Inputs
   hclk, dreset_n, mpu_canceld_i, bm_mpu_unalign_cancel_i, dec_dcore_sel_sys_i,
   dec_dcore_sel_sys_held_i, is_dcore_master_i, is_dcore_addr_i,
   is_dcore_trans_i, is_dcore_write_i, is_dcore_size_i,
   is_dcore_size_unalign_i, is_dcore_burst_i, is_dcore_prot_i,
   is_dcore_mem_attr_i, is_dcore_exreq_i, dpu_ahb_hwdatad_i,
   is_dcore_held_trans_i, dec_dap_sel_sys_i, dec_dap_sel_sys_held_i,
   is_dap_master_i, is_dap_addr_i, is_dap_trans_i, is_dap_write_i,
   is_dap_size_i, is_dap_size_unalign_i, is_dap_burst_i, is_dap_prot_i,
   dap_ahb_hwdata_i, is_dap_held_trans_i, bm_state_bb_wr1_i,
   dec_dcore_sel_bb_i, dec_dap_sel_bb_i, bm_sel_sys_i, bm_sel_sys_data_i,
   bm_master_i, bm_addr_i, bm_trans_i, bm_cancel_valid_i, bm_write_i,
   bm_size_i, bm_burst_i, bm_prot_i, bm_mem_attr_i, bm_exreq_i, bm_wdata_i,
   mtx_ahb_hreadys_i,
   os_dcore_to_sys_accept_o, os_dap_to_sys_accept_o, os_dcore_to_sys_active_o,
   os_dap_to_sys_active_o, os_dap_to_sys_held_req_o, os_sys_bm_wait_o,
   mtx_ahb_hmasters_o, mtx_ahb_hmastlocks_o, mtx_ahb_haddrs_o,
   mtx_ahb_htranss_o, mtx_ahb_hwrites_o, mtx_ahb_hsizes_o, mtx_ahb_hbursts_o,
   mtx_ahb_hprots_o, mtx_mem_attrs_o, mtx_exreqs_o, mtx_ahb_hwdatas_o
  );

  parameter CONST_AHB_CTRL = 0;  // Constant AHB control info if 1


  input         hclk;                      // AHB system clock
  input         dreset_n;                  // Debug reset

  input         dec_dcore_sel_sys_i;       // DCore -> SYS select
  input         dec_dcore_sel_sys_held_i;  // DCore -> SYS select held
  input         dec_dcore_sel_bb_i;        // DCore -> Bit-band
  input         dec_dap_sel_sys_i;         // DAP -> SYS select
  input         dec_dap_sel_sys_held_i;    // DAP -> SYS select held
  input         dec_dap_sel_bb_i;          // DAP -> Bit-band
  input         bm_sel_sys_i;              // Bit Master select
  input         bm_sel_sys_data_i;         // Bit Master select (data port)

  input  [31:0] is_dcore_addr_i;
  input   [1:0] is_dcore_master_i;
  input   [1:0] is_dcore_trans_i;
  input         is_dcore_write_i;
  input   [1:0] is_dcore_size_i;
  input   [1:0] is_dcore_size_unalign_i;
  input   [2:0] is_dcore_burst_i;
  input   [3:0] is_dcore_prot_i;
  input   [1:0] is_dcore_mem_attr_i;
  input         is_dcore_exreq_i;
  input         is_dcore_held_trans_i;
  input  [31:0] dpu_ahb_hwdatad_i;
  output        os_dcore_to_sys_accept_o;  // DCore -> SYS accepted
  output        os_dcore_to_sys_active_o;  // DCore -> SYS active

  input  [31:0] is_dap_addr_i;
  input   [1:0] is_dap_master_i;
  input   [1:0] is_dap_trans_i;
  input         is_dap_write_i;
  input   [1:0] is_dap_size_i;
  input   [1:0] is_dap_size_unalign_i;
  input   [2:0] is_dap_burst_i;
  input   [3:0] is_dap_prot_i;
  input  [31:0] dap_ahb_hwdata_i;
  input         is_dap_held_trans_i;
  output        os_dap_to_sys_accept_o;    // DAP -> SYS accepted
  output        os_dap_to_sys_active_o;    // DAP -> SYS active
  output        os_dap_to_sys_held_req_o;  // DAP -> waited request
  output        os_sys_bm_wait_o;

  input         bm_state_bb_wr1_i;
  input  [31:0] bm_addr_i;
  input   [1:0] bm_master_i;
  input   [1:0] bm_trans_i;
  input         bm_write_i;
  input   [1:0] bm_size_i;
  input   [2:0] bm_burst_i;
  input   [3:0] bm_prot_i;
  input  [31:0] bm_wdata_i;
  input   [1:0] bm_mem_attr_i;
  input         bm_exreq_i;
  input         bm_cancel_valid_i;

  input         mpu_canceld_i;             // DCore MPU lookup fail
  input         bm_mpu_unalign_cancel_i;   // Unaligned MPU cancel

  output [31:0] mtx_ahb_haddrs_o;
  output  [1:0] mtx_ahb_hmasters_o;
  output        mtx_ahb_hmastlocks_o;
  output  [1:0] mtx_ahb_htranss_o;
  output        mtx_ahb_hwrites_o;
  output  [2:0] mtx_ahb_hsizes_o;
  output  [2:0] mtx_ahb_hbursts_o;
  output  [3:0] mtx_ahb_hprots_o;
  output [31:0] mtx_ahb_hwdatas_o;
  output  [1:0] mtx_mem_attrs_o;
  output        mtx_exreqs_o;
  input         mtx_ahb_hreadys_i;


  wire          dcore_held_req;
  wire          dap_held_req;
  wire          bm_held_req;
  wire          dcore_wait;
  wire          bm_wait;
  wire          dap_wait;
  wire          dcore_req;            // DCore request
  wire          dap_req;              // DAP request
  wire          dcore_req_bb;         // DCore request - bit-band access
  wire          dap_req_bb;           // DAP request - bit-band access
  wire          bm_req;               // Bit Master request
  reg     [2:0] input_port_addr;      // Prioritized input port address

  wire          dcore_to_sys_accept;
  wire          dap_to_sys_accept;
  wire          dcore_to_sys_active;
  wire          dap_to_sys_active;

  wire   [31:0] dcore_addr;
  wire    [1:0] dcore_trans;
  wire    [1:0] dcore_size;
  wire    [2:0] dcore_burst;
  wire   [31:0] dap_addr;
  wire    [1:0] dap_trans;
  wire    [1:0] dap_size;
  wire    [2:0] dap_burst;

  wire   [31:0] ahb_haddr;
  wire    [1:0] ahb_hmaster;
  wire          ahb_hmastlock;
  wire    [1:0] i_ahb_htrans;         // HTRANS before cancel
  wire          ahb_hwrite;
  wire    [2:0] ahb_hsize;
  wire    [2:0] ahb_hburst;
  wire    [3:0] ahb_hprot;
  wire   [31:0] ahb_hwdata;
  wire    [1:0] mem_attr;
  wire          exreq;
  wire          cancel_trans;         // Late cancel signal for timing
  wire    [1:0] ahb_htrans;           // HTRANS with late cancel factored in
  wire          ahb_ctrl_valid_we;
  reg           ahb_ctrl_valid;
  reg     [2:0] ahb_ctrl_master;

  wire          bb_write;             // Bit-band write
  reg     [1:0] dcore_addr_align;     // DCore address aligned LSBs for bit-band
  reg     [1:0] dap_addr_align;       // DAP address aligned LSBs for bit-band

  reg     [2:0] input_port_data;      // Input port address for data phase
  reg     [2:0] nxt_input_port_data;
  wire          input_port_data_we;
  wire          dcore_cancel_unalign; // Select write buffer for unaligned cancel

  wire          opt_const_en;

`ifdef ARM_DSM

  wire          dsm_const_ahb_ctrl;

  assign opt_const_en = (dsm_const_ahb_ctrl > 0) ? 1'b1 : 1'b0;

`else

  assign opt_const_en = (CONST_AHB_CTRL > 0) ? 1'b1 : 1'b0;

`endif


  assign dcore_held_req = ahb_ctrl_valid & ahb_ctrl_master[0] & opt_const_en;
  assign dap_held_req   = ahb_ctrl_valid & ahb_ctrl_master[1] & opt_const_en;
  assign bm_held_req    = ahb_ctrl_valid & ahb_ctrl_master[2] & opt_const_en;

  assign dcore_wait     = dap_held_req | bm_held_req;
  assign bm_wait        = dap_held_req | dcore_held_req;
  assign dap_wait       = bm_held_req  | dcore_held_req;

  assign dcore_req      = (is_dcore_held_trans_i & dec_dcore_sel_sys_i &
                           ~dec_dcore_sel_sys_held_i & ~dcore_wait);
  assign dap_req        = (is_dap_held_trans_i & dec_dap_sel_sys_i &
                           (dap_held_req | ~dec_dap_sel_sys_held_i) & ~dap_wait);
  assign bm_req         = bm_sel_sys_i & ~bm_wait;
  assign dcore_req_bb   = is_dcore_held_trans_i & dec_dcore_sel_bb_i;
  assign dap_req_bb     = is_dap_held_trans_i & dec_dap_sel_bb_i;

  always @ (bm_req or dcore_req or dap_req)
    case ({bm_req, dcore_req, dap_req})
      3'b100, 3'b101, 3'b110, 3'b111: input_port_addr = 3'b100;  // Bit Master
      3'b010, 3'b011:                 input_port_addr = 3'b001;  // DCore
      3'b001:                         input_port_addr = 3'b010;  // DAP
      3'b000:                         input_port_addr = 3'b000;  // None
      default:                        input_port_addr = 3'bXXX;  // X propagation
    endcase



  assign dcore_to_sys_accept = input_port_addr[0];
  assign dap_to_sys_accept   = input_port_addr[1];

  assign dcore_to_sys_active = (~dec_dcore_sel_sys_held_i & ~bm_req &
                                mtx_ahb_hreadys_i & ~dcore_wait);
  assign dap_to_sys_active   = (dap_to_sys_accept & mtx_ahb_hreadys_i &
                                ~dap_wait);




  always @ (is_dcore_size_i or is_dcore_addr_i)
    case (is_dcore_size_i[1:0])
       2'b00:   dcore_addr_align = is_dcore_addr_i[6:5];
       2'b01:   dcore_addr_align = {is_dcore_addr_i[6], 1'b0};
       2'b10:   dcore_addr_align = 2'b00;
       default: dcore_addr_align = 2'bxx;
    endcase

  always @ (is_dap_size_i or is_dap_addr_i)
    case (is_dap_size_i[1:0])
       2'b00:   dap_addr_align = is_dap_addr_i[6:5];
       2'b01:   dap_addr_align = {is_dap_addr_i[6], 1'b0};
       2'b10:   dap_addr_align = 2'b00;
       default: dap_addr_align = 2'bxx;
    endcase

  assign bb_write = (dcore_req_bb & is_dcore_write_i &
                     input_port_addr[0] & ~mpu_canceld_i) |
                    (dap_req_bb & is_dap_write_i & input_port_addr[1]);

  assign ahb_hmastlock = bb_write | bm_state_bb_wr1_i;



  assign dcore_addr  = dcore_req_bb ?
                         {is_dcore_addr_i[31:28], 8'h00, is_dcore_addr_i[24:7], dcore_addr_align} :
                         is_dcore_addr_i;
  assign dap_addr    = dap_req_bb ?
                         {is_dap_addr_i[31:28], 8'h00, is_dap_addr_i[24:7], dap_addr_align} :
                         is_dap_addr_i;

  assign dcore_trans = dcore_req_bb ? `MTX_TRN_NONSEQ : is_dcore_trans_i;
  assign dap_trans   = dap_req_bb   ? `MTX_TRN_NONSEQ : is_dap_trans_i;

  assign dcore_size  = dcore_req_bb ? is_dcore_size_i : is_dcore_size_unalign_i;
  assign dap_size    = dap_req_bb   ? is_dap_size_i   : is_dap_size_unalign_i;

  assign dcore_burst = dcore_req_bb ? `MTX_BUR_INCR : is_dcore_burst_i;
  assign dap_burst   = dap_req_bb   ? `MTX_BUR_INCR : is_dap_burst_i;


  assign ahb_hmaster  = ({2{input_port_addr[0]}} & is_dcore_master_i) |
                        ({2{input_port_addr[1]}} & is_dap_master_i)   |
                        ({2{input_port_addr[2]}} & bm_master_i);

  assign ahb_haddr    = ({32{input_port_addr[0]}} & dcore_addr) |
                        ({32{input_port_addr[1]}} & dap_addr)   |
                        ({32{input_port_addr[2]}} & bm_addr_i);

  assign i_ahb_htrans = ({2{input_port_addr[0]}} & dcore_trans) |
                        ({2{input_port_addr[1]}} & dap_trans)   |
                        ({2{input_port_addr[2]}} & bm_trans_i);

  assign ahb_hwrite   = (input_port_addr[0] & ~dcore_req_bb & is_dcore_write_i) |
                        (input_port_addr[1] & ~dap_req_bb & is_dap_write_i)     |
                        (input_port_addr[2] & bm_write_i);

  assign ahb_hsize    = ({3{input_port_addr[0]}} & {1'b0, dcore_size}) |
                        ({3{input_port_addr[1]}} & {1'b0, dap_size})   |
                        ({3{input_port_addr[2]}} & {1'b0, bm_size_i});

  assign ahb_hburst   = ({3{input_port_addr[0]}} & dcore_burst) |
                        ({3{input_port_addr[1]}} & dap_burst)   |
                        ({3{input_port_addr[2]}} & bm_burst_i);

  assign ahb_hprot    = ({4{input_port_addr[0]}} & is_dcore_prot_i) |
                        ({4{input_port_addr[1]}} & is_dap_prot_i)   |
                        ({4{input_port_addr[2]}} & bm_prot_i);

  assign mem_attr     = ({2{input_port_addr[0]}} & is_dcore_mem_attr_i) |
                        ({2{input_port_addr[2]}} & bm_mem_attr_i);

  assign exreq        = (input_port_addr[0] & is_dcore_exreq_i) |
                        (input_port_addr[2] & bm_exreq_i);


  assign cancel_trans = (input_port_addr[2] & mpu_canceld_i & bm_cancel_valid_i) |
                        (input_port_addr[0] & mpu_canceld_i);

  assign ahb_htrans   = {2{~cancel_trans}} & i_ahb_htrans;

  assign ahb_ctrl_valid_we = (opt_const_en &
                              (ahb_ctrl_valid |
                               ((|ahb_htrans) & ~mtx_ahb_hreadys_i)));

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      begin
        ahb_ctrl_valid  <= 1'b0;
        ahb_ctrl_master <= 3'b000;
      end
    else if (ahb_ctrl_valid_we)
      begin
        ahb_ctrl_valid  <= (|ahb_htrans) & ~mtx_ahb_hreadys_i;
        ahb_ctrl_master <= input_port_addr;
      end


  assign dcore_cancel_unalign = bm_mpu_unalign_cancel_i &
                                ~mtx_ahb_hreadys_i      &
                                input_port_data[0];

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      input_port_data <= 3'b000;
    else if (input_port_data_we)
      input_port_data <= nxt_input_port_data;

  always @ (ahb_hwrite or dcore_cancel_unalign or input_port_addr)
    case ({ahb_hwrite, dcore_cancel_unalign})
      2'b00, 2'b01: nxt_input_port_data = 3'b000;
      2'b10:        nxt_input_port_data = input_port_addr;
      2'b11:        nxt_input_port_data = 3'b100;
      default:      nxt_input_port_data = 3'bXXX;
    endcase

  assign input_port_data_we = dcore_cancel_unalign | mtx_ahb_hreadys_i;

  assign ahb_hwdata = ({32{ bm_sel_sys_data_i | input_port_data[2]}} & bm_wdata_i)        |
                      ({32{~bm_sel_sys_data_i & input_port_data[0]}} & dpu_ahb_hwdatad_i) |
                      ({32{~bm_sel_sys_data_i & input_port_data[1]}} & dap_ahb_hwdata_i);



   assign os_dcore_to_sys_accept_o = dcore_to_sys_accept;
   assign os_dap_to_sys_accept_o   = dap_to_sys_accept;
   assign os_dcore_to_sys_active_o = dcore_to_sys_active;
   assign os_dap_to_sys_active_o   = dap_to_sys_active;
   assign os_dap_to_sys_held_req_o = dap_held_req;
   assign os_sys_bm_wait_o         = bm_wait & bm_sel_sys_i;
   assign mtx_ahb_haddrs_o         = ahb_haddr;
   assign mtx_ahb_hmasters_o       = ahb_hmaster;
   assign mtx_ahb_hmastlocks_o     = ahb_hmastlock;
   assign mtx_ahb_htranss_o        = ahb_htrans;
   assign mtx_ahb_hwrites_o        = ahb_hwrite;
   assign mtx_ahb_hsizes_o         = ahb_hsize;
   assign mtx_ahb_hbursts_o        = ahb_hburst;
   assign mtx_ahb_hprots_o         = ahb_hprot;
   assign mtx_ahb_hwdatas_o        = ahb_hwdata;
   assign mtx_mem_attrs_o          = mem_attr;
   assign mtx_exreqs_o             = exreq;


`ifdef ARM_ASSERT_ON

  reg ovl_bb_state;
  reg ovl_nxt_bb_state;

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      ovl_bb_state <= 1'b0;
    else
      ovl_bb_state <= ovl_nxt_bb_state;

  always @ (bb_write or ahb_htrans or mtx_ahb_hreadys_i or ovl_bb_state)
    case (ovl_bb_state)
      1'b0:    ovl_nxt_bb_state = bb_write & mtx_ahb_hreadys_i;
      1'b1:    ovl_nxt_bb_state = ~((ahb_htrans == `MTX_TRN_NONSEQ) & mtx_ahb_hreadys_i);
      default: ovl_nxt_bb_state = 1'bX;
    endcase

  assert_zero_one_hot #(0,3,0,"input_port_addr must be exclusive.")
    ovl_input_port_addr_exclusive(hclk, dreset_n, input_port_addr);

  assert_implication #(0,0,"ahb_hwrite must be 0 for first cycle of bit-band read-modify-write.")
    ovl_bb_hwrite0_check(hclk, dreset_n, bb_write, ~ahb_hwrite);

  assert_implication #(0,0,"ahb_hwrite must be 1 for second cycle of bit-band read-modify-write.")
    ovl_bb_hwrite1_check(hclk, dreset_n, (ovl_bb_state & ~ovl_nxt_bb_state), ahb_hwrite);

  assert_window #(0,0,"ahb_hmastlock must be asserted for read-modify-write sequence.")
    ovl_hmastlock_check(hclk, dreset_n, (bb_write & mtx_ahb_hreadys_i), ahb_hmastlock,
      (ovl_bb_state & ~ovl_nxt_bb_state));

  assert_win_unchange #(0,32,0,"ahb_haddr must be unchanged for read-modify-write sequence.")
   ovl_haddr_unchange(hclk, dreset_n, (bb_write & mtx_ahb_hreadys_i), ahb_haddr,
     (ovl_bb_state & ~ovl_nxt_bb_state));

  assert_win_unchange #(0,3,0,"ahb_hsize must be unchanged for read-modify-write sequence.")
   ovl_hsize_unchange(hclk, dreset_n, (bb_write & mtx_ahb_hreadys_i), ahb_hsize,
     (ovl_bb_state & ~ovl_nxt_bb_state));

  assert_always #(0,0,"ahb_hmastlock must never be asserted if not doing bit-band write.")
    ovl_hmastlock_check2(hclk, dreset_n, ((~bb_write & ~ovl_bb_state) ? ~ahb_hmastlock : 1'b1));

  wire   ovl_bb_read;
  assign ovl_bb_read = ((dcore_req_bb & ~is_dcore_write_i & input_port_addr[0]) |
                        (dap_req_bb & ~is_dap_write_i & input_port_addr[1])) &
                       ~mpu_canceld_i;

  assert_always #(0,0,"ahb_htrans must be NSEQ for bit-band accesses.")
    ovl_bb_trans_check(hclk, dreset_n,
      ((bb_write | ovl_bb_read | ovl_bb_state) ? (ahb_htrans == `MTX_TRN_NONSEQ) : 1'b1));

  assert_always #(0,0,"ahb_hburst must be INCR for bit-band accesses.")
    ovl_bb_burst_check(hclk, dreset_n,
      ((bb_write | ovl_bb_read | ovl_bb_state) ? (ahb_hburst == `MTX_BUR_INCR) : 1'b1));


  assert_never #(0,0,"is_dcore_size_i = DWORD not allowed.")
    ovl_dcore_size_dword(hclk, dreset_n, (is_dcore_size_i[1:0] == 2'b11));

  assert_never #(0,0,"is_dap_size_i = DWORD not allowed.")
    ovl_dap_size_dword(hclk, dreset_n, (is_dap_size_i[1:0] == 2'b11));


  assert_never_unknown #(0,2,0,"is_dcore_size_i goes to X.")
    ovl_dcore_size_x(hclk, dreset_n, 1'b1, is_dcore_size_i);

  assert_never_unknown #(0,2,0,"is_dap_size_i goes to X.")
    ovl_is_dap_size_x(hclk, dreset_n, 1'b1, is_dap_size_i);

  assert_never_unknown #(0,3,0,"input_port_addr goes to X.")
    ovl_input_port_addr_x(hclk, dreset_n, 1'b1, input_port_addr);

  assert_never_unknown #(0,3,0,"input_port_data goes to X.")
    ovl_input_port_data_x(hclk, dreset_n, 1'b1, input_port_data);

  assert_never_unknown #(0,1,0,"ovl_bb_state goes to X.")
    ovl_bb_state_x(hclk, dreset_n, 1'b1, ovl_bb_state);

  assert_never_unknown #(0,2,0,"dcore_addr_align goes to X.")
    ovl_dcore_addr_align_x(hclk, dreset_n, 1'b1, dcore_addr_align);

  assert_never_unknown #(0,2,0,"dap_addr_align goes to X.")
    ovl_dap_addr_align_x(hclk, dreset_n, 1'b1, dap_addr_align);


  assert_never_unknown #(0,32,0,"ahb_haddr goes to X.")
    ovl_haddr_x(hclk, dreset_n, (ahb_htrans != `MTX_TRN_IDLE), ahb_haddr);

  assert_never_unknown #(0,1,0,"ahb_hwrite goes to X.")
    ovl_hwrite_x(hclk, dreset_n, (ahb_htrans != `MTX_TRN_IDLE), ahb_hwrite);

  assert_never_unknown #(0,3,0,"ahb_hsize goes to X.")
    ovl_hsize_x(hclk, dreset_n, (ahb_htrans != `MTX_TRN_IDLE), ahb_hsize);

  assert_never_unknown #(0,3,0,"ahb_hburst goes to X.")
    ovl_hburst_x(hclk, dreset_n, (ahb_htrans != `MTX_TRN_IDLE), ahb_hburst);

  assert_never_unknown #(0,4,0,"ahb_hprot goes to X.")
    ovl_hprot_x(hclk, dreset_n, (ahb_htrans != `MTX_TRN_IDLE), ahb_hprot);

  assert_never_unknown #(0,2,0,"mem_attr goes to X.")
    ovl_mem_attr_x(hclk, dreset_n, (ahb_htrans != `MTX_TRN_IDLE), mem_attr);

  assert_never_unknown #(0,1,0,"exreq goes to X.")
    ovl_exreq_x(hclk, dreset_n, (ahb_htrans != `MTX_TRN_IDLE), exreq);

`endif

endmodule

`include "cm3_mtx_undefs.vp"
