`include "cm3_nvic_defs.vp"

module cm3_nvic
  (// Inputs
   dreset_n, hreset_n, fclk, hclk, pclk, ext_st_clk_i, ext_st_calib_i,
   ext_cp_disable_i, ext_big_end_i, mtx_ppb_hsel_nvic_i, mtx_ppb_hmaster_i,
   mtx_ppb_haddr_i, mtx_ppb_hwrite_i, mtx_ppb_hsize_i, mtx_ppb_hprot_i,
   mtx_ppb_hwdata_i, mtx_ppb_hready_i, dpu_rst_pc_valid_i,
   dpu_ahb_htransi_iss_i, dpu_ahb_haddrd_i, dpu_ahb_haddrd_valid_i,
   dpu_privileged_i, dpu_thread_i, dpu_ivalid_i, dpu_retire_i, dpu_fpccr_we_i,
   dpu_fpcar_nxt_i, dpu_lazy_stk_i, dpu_basepri_we_i, dpu_basepri_nxt_i,
   dpu_fmask_we_i, dpu_fmask_nxt_i, dpu_pmask_we_i, dpu_pmask_nxt_i,
   dpu_raisepri_i, ext_stkalign_init_i, ext_nmi_i, ext_irq_i, mtx_wb_abort_i,
   dpu_bkpt_i, dpu_svc_i, dpu_cp_busy_i, dpu_nxt_bfsr_i, dpu_nxt_ufsr_i,
   dpu_nxt_mfsr_i, dpu_nxt_hfsr_i, ext_aux_fault_i, ext_vect_addr_i,
   ext_vect_addr_en_i, dpu_mcyc_invk_msk_i, dpu_ahb_invk_msk_i, dpu_int_entry_i,
   dpu_int_exit_i, dpu_int_return_i, dpu_int_retisr_i, ext_slp_rxev_i,
   ext_slp_txev_i, ext_slp_hold_req_n_i, dpu_slp_ctl_i, wic_ds_req_n_i,
   ext_dbg_en_i, dwt_watchpoint_i, ext_dbg_req_i, dpu_dbg_halted_i,
   dpu_dbg_ready_i, dpu_dbg_wr_isr_i, ext_dbg_restart_i,
   nvic_ppb_hready_o, nvic_ppb_hrdata_o, nvic_ppb_hresp_o, nvic_int_invoke_o,
   nvic_int_new_arrival_o, nvic_int_addr_o, nvic_int_tail_o,
   nvic_int_nxt_isr_o, nvic_int_curr_isr_o, nvic_int_curr_pri_o,
   nvic_int_stk_actv_o, nvic_int_in_nmihf_o, nvic_int_mpu_nmihf_o,
   nvic_int_stk_algn_o, nvic_int_st_obs_o, nvic_basepri_o, nvic_fmask_o,
   nvic_pmask_o, nvic_data_be8_o, nvic_bfhfnmign_o, nvic_div_0_trp_o,
   nvic_unalign_trp_o, nvic_nonbasethrden_o, nvic_disfold_o, nvic_dismcycint_o,
   nvic_no_def_wbuf_o, nvic_no_ooo_o, nvic_no_lazy_stk_o, nvic_aux_ctrl1_we_o,
   nvic_cpacr_o, nvic_fpccr_o, nvic_fpcar_o, nvic_fpdscr_o, nvic_lockup_o,
   nvic_vect_reset_n_o, nvic_sys_reset_req_o, nvic_slp_sleeponexit_o,
   nvic_slp_asleep_o, nvic_slp_sleeping_o, nvic_slp_sleepdeep_o,
   nvic_slp_hold_ack_n_o, nvic_wic_ds_ack_n_o, nvic_wic_load_o,
   nvic_wic_clear_o, nvic_wic_mask_o, nvic_dbg_halt_req_o, nvic_dbg_mon_req_o,
   nvic_dbg_step_o, nvic_dbg_restarted_o, nvic_dbg_trans_o, nvic_dbg_reg_wr_o,
   nvic_dbg_reg_addr_o, nvic_dbg_snapstall_o, nvic_trc_en_o
   );


  parameter NUM_IRQ         = 16; // Number of interrupts. Between 1 and 240
  parameter LVL_WIDTH       = 3;  // Interrupt priority width. Between 3 and 8
  parameter DEBUG_LVL       = 3;  // Debug support level. Between 0 and 3
  parameter RESET_ALL_REGS  = 0;  // Reset all registers if 1
  parameter WIC_PRESENT     = 0;  // WIC present if 1
  parameter WIC_LINES       = 3;  // Number of WIC lines. Between 3 and 243
  parameter CP_PRESENT      = 0;  // CP present if 1
  parameter EXT_PRESENT     = 0;  // Extensions present if 1


  `define num_ints  (`num_non_irq + NUM_IRQ)

  `define spare     (`max_ints - `num_ints - `num_res_rs)

  `define irq_width (NUM_IRQ  * LVL_WIDTH)
  `define max_width (`max_ints * LVL_WIDTH)


  input                  dreset_n;               // Debug reset
  input                  hreset_n;               // NVIC reset
  input                  fclk;                   // Free running clock
  input                  hclk;                   // AHB-Lite HCLK
  input                  pclk;                   // PPB Clock
  input                  ext_st_clk_i;           // System Tick clock
  input [25:0]           ext_st_calib_i;         // System Tick calibration
  input                  ext_cp_disable_i;       // Disable the CP
  input                  ext_big_end_i;          // Big endian
  input                  mtx_ppb_hsel_nvic_i;    // PPB HSEL
  input [11:0]           mtx_ppb_haddr_i;        // PPB HADDR
  input                  mtx_ppb_hmaster_i;      // PPB HMASTER
  input                  mtx_ppb_hwrite_i;       // PPB HWRITE
  input [1:0]            mtx_ppb_hsize_i;        // PPB HSIZE
  input [3:0]            mtx_ppb_hprot_i;        // PPB HPROT
  input [31:0]           mtx_ppb_hwdata_i;       // PPB HWDATA
  input                  mtx_ppb_hready_i;       // PPB HREADYIN
  input                  dpu_rst_pc_valid_i;     // DPU has been reset
  input                  dpu_ahb_htransi_iss_i;  // In data phase of I-AHB trans
  input [31:0]           dpu_ahb_haddrd_i;       // Data address
  input                  dpu_ahb_haddrd_valid_i; // D addr valid to sample for FAR
  input                  dpu_privileged_i;       // DPU in privileged mode
  input                  dpu_thread_i;           // DPU in thread
  input                  dpu_ivalid_i;           // Decode enable
  input                  dpu_retire_i;           // Opcode retire
  input [1:0]            dpu_fpccr_we_i;         // LSPACT write enable
  input [31:0]           dpu_fpcar_nxt_i;        // FPCAR write value
  input                  dpu_lazy_stk_i;         // Lazy stack in progress
  input                  dpu_basepri_we_i;       // BasePri write enable
  input [7:0]            dpu_basepri_nxt_i;      // Next BasePri
  input                  dpu_fmask_we_i;         // FaultMask write enable
  input                  dpu_fmask_nxt_i;        // Next FaultMask
  input                  dpu_pmask_we_i;         // PriMask write enable
  input                  dpu_pmask_nxt_i;        // Next PriMask
  input                  dpu_raisepri_i;         // Priority raised
  input                  ext_stkalign_init_i;    // Stack alignment reset value
  input                  ext_nmi_i;              // NMI
  input [NUM_IRQ-1:0]    ext_irq_i;              // Interrupt request
  input                  mtx_wb_abort_i;         // BM imprecise write buffer abort
  input                  dpu_bkpt_i;             // BKPT instruction executed
  input                  dpu_svc_i;              // SVC instruction executed
  input                  dpu_cp_busy_i;          // CP busy
  input [5:0]            dpu_nxt_bfsr_i;         // BFSR write value
  input [9:0]            dpu_nxt_ufsr_i;         // UFSR write value
  input [4:0]            dpu_nxt_mfsr_i;         // MFSR write value
  input                  dpu_nxt_hfsr_i;         // HFSR write value
  input [31:0]           ext_aux_fault_i;        // Auxillary FSR input
  input [9:0]            ext_vect_addr_i;        // Vector address overwrite value
  input                  ext_vect_addr_en_i;     // Enable vector address overwrite
  input                  dpu_mcyc_invk_msk_i;    // Multi-cycle op in progress
  input                  dpu_ahb_invk_msk_i;     // Constant AHB invoke mask
  input                  dpu_int_entry_i;        // ISR has been entered
  input                  dpu_int_exit_i;         // ISR has completed
  input                  dpu_int_return_i;       // Interrupt unstacking complete
  input [8:0]            dpu_int_retisr_i;       // ISR returned to
  input                  ext_slp_rxev_i;         // Receive event
  input                  ext_slp_txev_i;         // Transmit event
  input                  ext_slp_hold_req_n_i;   // Hold core in sleep mode
  input [1:0]            dpu_slp_ctl_i;          // [1] = WFE, [0] = WFI
  input                  wic_ds_req_n_i;         // WIC mode request
  input                  ext_dbg_en_i;           // Enable debug
  input                  dwt_watchpoint_i;       // DWT generated watchpoint
  input                  ext_dbg_req_i;          // External debug request
  input                  dpu_dbg_halted_i;       // Core has halted
  input                  dpu_dbg_ready_i;        // Single cycle reply for debug
  input                  dpu_dbg_wr_isr_i;       // Debug over-ride of ISR number
  input                  ext_dbg_restart_i;      // External debug restart request

  output                 nvic_ppb_hready_o;      // PPB HREADY from slave
  output [31:0]          nvic_ppb_hrdata_o;      // PPB HRDATA from slave
  output                 nvic_ppb_hresp_o;       // PPB HRESP from slave
  output                 nvic_int_invoke_o;      // Request interrupt stack
  output                 nvic_int_new_arrival_o; // Late arrival
  output [31:0]          nvic_int_addr_o;        // Vector address for next interrupt
  output                 nvic_int_tail_o;        // Will tail-chain to next interrupt
  output [7:0]           nvic_int_nxt_isr_o;     // Next ISR number to be processed
  output [8:0]           nvic_int_curr_isr_o;    // ISR that is currently active
  output [7:0]           nvic_int_curr_pri_o;    // Current ISR's priority
  output                 nvic_int_stk_actv_o;    // ISR is stacked active
  output [2:0]           nvic_int_in_nmihf_o;    // NMI/HF handler {pend,active}
  output                 nvic_int_mpu_nmihf_o;   // Inform MPU if its in NMI/HF
  output                 nvic_int_stk_algn_o;    // Align stack to double word boundary
  output                 nvic_int_st_obs_o;      // System Tick interrupt observation
  output [7:0]           nvic_basepri_o;         // BasePri
  output                 nvic_fmask_o;           // FaultMask
  output                 nvic_pmask_o;           // PriMask
  output                 nvic_data_be8_o;        // Endian BE8
  output                 nvic_bfhfnmign_o;       // Bus fault ignored in NMI/HF
  output                 nvic_div_0_trp_o;       // Divide by zero fault enabled
  output                 nvic_unalign_trp_o;     // Unaligned fault enable
  output                 nvic_nonbasethrden_o;   // Handler can be thread
  output                 nvic_disfold_o;         // Disable IT folding
  output                 nvic_dismcycint_o;      // Disable int of multi-cycle ops
  output                 nvic_no_def_wbuf_o;     // Disable write buffers
  output                 nvic_no_ooo_o;          // Disable out-of-order
  output                 nvic_no_lazy_stk_o;     // Disable lazy stacking
  output                 nvic_aux_ctrl1_we_o;    // Auxiliary Ctrl write enable
  output [3:0]           nvic_cpacr_o;           // CPACR
  output [31:0]          nvic_fpccr_o;           // FPCCR register
  output [31:0]          nvic_fpcar_o;           // FPCAR register
  output [4:0]           nvic_fpdscr_o;          // FPDSCR register
  output [1:0]           nvic_lockup_o;          // Lock up
  output                 nvic_vect_reset_n_o;    // System reset (internal)
  output                 nvic_sys_reset_req_o;   // System reset (external)
  output                 nvic_slp_sleeponexit_o; // Core is to sleep on ISR exit
  output                 nvic_slp_asleep_o;      // Core is to sleep now
  output                 nvic_slp_sleeping_o;    // Core is sleeping
  output                 nvic_slp_sleepdeep_o;   // Core is to enter deep sleep
  output                 nvic_slp_hold_ack_n_o;  // Core is forced in sleeping state
  output                 nvic_wic_ds_ack_n_o;    // WIC mode acknowledge
  output                 nvic_wic_load_o;        // Load WIC sensitivity list
  output                 nvic_wic_clear_o;       // Clear WIC sensitivity list
  output [WIC_LINES-1:0] nvic_wic_mask_o;        // WIC IRQ/NMI/RXEV sensitivity list
  output                 nvic_dbg_halt_req_o;    // Request the core to stop
  output                 nvic_dbg_mon_req_o;     // Request the core to pause
  output                 nvic_dbg_step_o;        // Debug stepping
  output                 nvic_dbg_restarted_o;   // External debug restart ready
  output                 nvic_dbg_trans_o;       // Debug register transaction
  output                 nvic_dbg_reg_wr_o;      // Debug register read/write
  output [6:0]           nvic_dbg_reg_addr_o;    // Debug register access address
  output                 nvic_dbg_snapstall_o;   // Snap stall to bus matrix
  output                 nvic_trc_en_o;          // Master trace enable


  wire [NUM_IRQ-1:0]     irq_en;                // IRQ enables
  wire [`irq_width-1:0]  irq_lvl;               // Priority level for each interrupt
  wire [LVL_WIDTH-1:0]   shp_mm_lvl;            // Priority level for MemManage Fault
  wire [LVL_WIDTH-1:0]   shp_bf_lvl;            // Priority level for Bus Fault
  wire [LVL_WIDTH-1:0]   shp_uf_lvl;            // Priority level for Undef Fault
  wire [LVL_WIDTH-1:0]   shp_dm_lvl;            // Priority level for Debug Monitor
  wire [LVL_WIDTH-1:0]   shp_svc_lvl;           // Priority level for SVC interrupt
  wire [LVL_WIDTH-1:0]   shp_psv_lvl;           // Priority level for pend SVC interrupt
  wire [LVL_WIDTH-1:0]   shp_st_lvl;            // Priority level for System Tic
  wire [15:0]            shcs_status;           // System Handler status
  wire                   shcs_mm_en;            // MemManage fault enable
  wire                   shcs_bf_en;            // Bus fault enable
  wire [31:0]            mm_bf_far;             // Fault address register
  wire                   shcs_uf_en;            // Usage fault enable
  wire                   int_stk_actv;          // ISR is stacked active
  wire                   int_ret_to_base;       // Return to base
  wire                   nmi_pending;           // NMI is pending
  wire                   dbg_mon_pending;       // Debug monitor is pending
  wire                   dbg_mon_active;        // Debug monitor is active
  wire                   dbg_mon_allow;         // DbgMon will be able to invoke
  wire [7:0]             int_hipend_isr;        // Highest pending vector number
  wire                   psv_pending;           // SVC from pend is pending
  wire                   st_pending;            // SysTick is pending
  wire [`num_ints-1:0]   int_pend_set;          // Set pending
  wire [`num_ints-1:0]   int_pend_clr;          // Clear pending
  wire [`num_ints-1:0]   int_actv_set;          // Set active
  wire [`num_ints-1:0]   int_actv_clr;          // Clear active
  wire                   vect_clr_actv;         // Vector Clear Active
  wire [NUM_IRQ-1:0]     irq_pending;           // Interrupts that are pending
  wire [NUM_IRQ-1:0]     irq_active;            // Interrupts that are active
  wire [31:7]            vto_tbloff;            // Base location of vector table
  wire [3:0]             ppb_we;                // PPB data phase byte write enable
  wire                   ppb_rd;                // PPB data phase read enable
  wire [11:2]            ppb_reg_addr;          // Registered PPB HADDR
  wire                   ppb_hmaster_reg;       // Registered PPB HMASTER
  wire                   scr_deepsleep;         // DeepSleep bit is set
  wire                   scr_wfeonpend;         // Wake up from WFE on newly pended ints
  wire [31:0]            fsr;                   // Fault status register
  wire [31:0]            hfsr;                  // HardFault status register
  wire [31:0]            dfsr;                  // Debug fault status register
  wire [31:0]            afsr;                  // Auxiliary fault status register
  wire                   int_hfault;            // HardFault raised
  wire                   int_escalate;          // Escalate to HF
  wire                   int_lazy_esc;          // Lazy fault escalated
  wire                   dhcs_c_snapstall;      // Snap out of stalled bus transaction
  wire                   dhcs_c_mask_ints;      // Mask interrupts whilst stepping set
  wire                   dhcs_c_halt;           // Halting control
  wire                   dhcs_c_step;           // Halting step enable
  wire                   dhcs_s_reset_st;       // Reset occured since last read
  wire                   dhcs_s_retire_st;      // Instr retire occurred since last read
  wire                   ccr_usersetmpend;      // User can use the software trigger reg
  wire                   ccr_nonbasethrdena;    // Handler can be thread
  wire [31:0]            dbg_dcrd;              // Scratch debug register
  wire                   demc_mon_en;           // Debug monitor enable
  wire                   dbg_mon_pend;          // Debug Monitor pend
  wire                   dbg_mon_step_pend;     // Debug Monitor step pend
  wire                   demc_mon_step;         // Debug Monitor step
  wire                   demc_mon_req;          // Debug Monitor request
  wire [7:0]             dbg_vcatch;            // Vector Catch
  wire                   dbg_dbgen;             // Halting debug enable
  wire                   dhcs_c_debugen;        // Halting debug enable direct from reg
  wire                   dbg_mask_ints;         // Mask interrupts during debug
  wire                   dbg_mon_fault;         // Debug monitor has occurred
  wire [2:0]             airc_prigroup;         // Priority grouping position
  wire                   data_be8;              // Big-endian byte invariant
  wire                   slp_pmask_pend;        // Priority Suppressed Pend
  wire                   scr_sleeponexit;       // Sleep on exit requested
  wire                   stick_cnt_flag;        // SysTick count flag
  wire                   stick_clk_src;         // SysTick clock source
  wire                   stick_int_en;          // Allow pending of SysTick
  wire                   stick_en;              // Enable SysTick counter reload
  wire [23:0]            stick_reload;          // SysTick counter reload value
  wire [23:0]            stick_count;           // SysTick counter value
  wire                   stick_int;             // System Tick interrupt
  wire                   slp_wfe_pend;          // New pend will wake up WFE
  wire                   ccr_unalign_trp;       // Unaligned fault enable
  wire                   int_non_actv_exit;     // Non-active ISR exited
  wire                   int_allow_bf;          // Allow bus fault to cause an invoke
  wire                   dbg_masked_bkpt;       // BKPT will halt
  wire                   dbg_mon_hi_pri_invk;   // DbgMon pri > invoke pri
  wire [4:0]             aux_ctrl;              // Auxiliary Control Register
  wire                   aux_ctrl1_we;          // Auxiliary Ctrl write enable
  wire                   ac_dismcycint;         // aux_ctrl[0]
  wire                   ac_no_def_wbuf;        // aux_ctrl[1]
  wire                   ac_disfold;            // aux_ctrl[2]
  wire                   ac_no_lazy_stk;        // aux_ctrl[3]
  wire                   ac_no_ooo;             // aux_ctrl[4]
  wire                   slp_norm_wake;         // Normal sleep wake up
  wire                   wic_ds_ack_n;          // WIC mode acknowledge
  wire                   wic_load;              // Load WIC sensitivity list
  wire                   wic_clear;             // Clear WIC sensitivity list
  wire [WIC_LINES-1:0]   wic_mask;              // WIC IRQ/NMI/RXEV sensitivity list
  wire                   slp_wfe_valid;         // Sleep is due to WFE (for WIC RXEV)
  wire                   slp_wfi_valid;         // Sleep is due to WFI
  wire                   ppb_hready;            // PPB HREADY from slave
  wire                   ppb_hresp;             // PPB HRESP
  wire [31:0]            ppb_hrdata;            // PPB HRDATA
  wire [8:0]             int_curr_isr;          // Interrupt that is currently active
  wire                   int_tail;              // Will tail-chain into next interrupt
  wire                   int_invoke;            // Preparing to enter interrupt
  wire                   int_new_arrival;       // Higher priority interrupt has occured
  wire [31:0]            int_addr;              // ISR locn for next interrupt
  wire [2:0]             int_in_nmihf;          // NMI/HardFault handler {pend,active}
  wire                   int_mpu_nmihf;         // Inform MPU if its in NMI/HF
  wire                   ccr_bfhfnmign;         // Bus Fault ignored in NMI/HF
  wire                   ccr_stkalign;          // Stack Align
  wire                   ccr_div_0_trp;         // Divide by zero fault enabled
  wire                   slp_sleeping;          // Core is sleeping
  wire                   slp_sleepdeep;         // Core is to enter deep sleep
  wire                   slp_hold_ack_n;        // Core is held in sleeping state
  wire                   slp_sleeponexit;       // Core is to Sleep On Interrupt Exit
  wire                   dbg_halt_req;          // Request the core to stop
  wire                   dbg_mon_req;           // Request the core to pause
  wire [7:0]             int_basepri;           // Base Priority
  wire                   int_fmask;             // Fault Mask
  wire                   int_pmask;             // Priority Mask
  wire                   airc_vectreset;        // System reset (internal)
  wire                   airc_sysresetreq;      // System reset (external)
  wire                   dcrs_rtrans;           // DBG HTRANS
  wire                   dcrs_regwnr;           // Debug register read, register write
  wire [6:0]             dcrs_regsel;           // Debug register address
  wire                   demc_trcena;           // Trace enable
  wire [1:0]             lockup;                // Lock up
  wire [7:0]             int_nxt_isr;           // Next ISR number to be processed
  wire [7:0]             int_curr_pri;          // CurrentInts settable priority
  wire                   dbg_step;              // Debug stepping
  wire                   slp_asleep;            // Core is to sleep Now
  wire                   dbg_restarted;         // External Debug Restart Ready
  wire [3:0]             cpacr;                 // CPACR
  wire [31:0]            fpccr;                 // FPCCR register
  wire [31:0]            fpcar;                 // FPCAR register
  wire [4:0]             fpdscr;                // Default FPSCR register
  wire [3:0]             int_lazy_fpccr;        // Lazy FPCCR ready values
  wire                   int_lazy_dm;           // Lazy debug monitor

  wire                   opt_wic_ds_ack_n;
  wire                   opt_wic_load;
  wire                   opt_wic_clear;
  wire [WIC_LINES-1:0]   opt_wic_mask;
  wire                   opt_ac_no_ooo;
  wire                   opt_ac_no_lazy_stk;
  wire [3:0]             opt_cpacr;
  wire [31:0]            opt_fpccr;
  wire [31:0]            opt_fpcar;
  wire [4:0]             opt_fpdscr;

`ifdef ARM_DSM

  wire                   dsm_wic_present;
  wire                   dsm_cp_present;

  assign opt_wic_ds_ack_n   = dsm_wic_present ? wic_ds_ack_n   : 1'b1;
  assign opt_wic_load       = dsm_wic_present ? wic_load       : 1'b0;
  assign opt_wic_clear      = dsm_wic_present ? wic_clear      : 1'b0;
  assign opt_wic_mask       = dsm_wic_present ? wic_mask       : {WIC_LINES{1'b0}};

  assign opt_ac_no_ooo      = dsm_cp_present  ? ac_no_ooo      : 1'b0;
  assign opt_ac_no_lazy_stk = dsm_cp_present  ? ac_no_lazy_stk : 1'b0;
  assign opt_cpacr          = dsm_cp_present  ? cpacr          : {4{1'b0}};
  assign opt_fpccr          = dsm_cp_present  ? fpccr          : {32{1'b0}};
  assign opt_fpcar          = dsm_cp_present  ? fpcar          : {32{1'b0}};
  assign opt_fpdscr         = dsm_cp_present  ? fpdscr         : {5{1'b0}};

`else

  assign opt_wic_ds_ack_n   = (WIC_PRESENT != 0) ? wic_ds_ack_n   : 1'b1;
  assign opt_wic_load       = (WIC_PRESENT != 0) ? wic_load       : 1'b0;
  assign opt_wic_clear      = (WIC_PRESENT != 0) ? wic_clear      : 1'b0;
  assign opt_wic_mask       = (WIC_PRESENT != 0) ? wic_mask       : {WIC_LINES{1'b0}};

  assign opt_ac_no_ooo      = (CP_PRESENT != 0)  ? ac_no_ooo      : 1'b0;
  assign opt_ac_no_lazy_stk = (CP_PRESENT != 0)  ? ac_no_lazy_stk : 1'b0;
  assign opt_cpacr          = (CP_PRESENT != 0)  ? cpacr          : {4{1'b0}};
  assign opt_fpccr          = (CP_PRESENT != 0)  ? fpccr          : {32{1'b0}};
  assign opt_fpcar          = (CP_PRESENT != 0)  ? fpcar          : {32{1'b0}};
  assign opt_fpdscr         = (CP_PRESENT != 0)  ? fpdscr         : {5{1'b0}};

`endif

  cm3_nvic_ppb_intf #(NUM_IRQ, LVL_WIDTH, DEBUG_LVL, EXT_PRESENT, CP_PRESENT, `irq_width)
    u_cm3_nvic_ppb_intf
      (// Inputs
       .dreset_n                   (dreset_n),
       .hclk                       (hclk),
       .mtx_ppb_haddr_i            (mtx_ppb_haddr_i),
       .mtx_ppb_hprot_i            (mtx_ppb_hprot_i),
       .mtx_ppb_hsel_nvic_i        (mtx_ppb_hsel_nvic_i),
       .mtx_ppb_hmaster_i          (mtx_ppb_hmaster_i),
       .mtx_ppb_hsize_i            (mtx_ppb_hsize_i),
       .mtx_ppb_hwrite_i           (mtx_ppb_hwrite_i),
       .mtx_ppb_hready_i           (mtx_ppb_hready_i),
       .data_be8_i                 (data_be8),
       .ext_cp_disable_i           (ext_cp_disable_i),
       .irq_en_i                   (irq_en),
       .shcs_mm_en_i               (shcs_mm_en),
       .shcs_bf_en_i               (shcs_bf_en),
       .shcs_uf_en_i               (shcs_uf_en),
       .irq_lvl_i                  (irq_lvl),
       .shp_mm_lvl_i               (shp_mm_lvl),
       .shp_bf_lvl_i               (shp_bf_lvl),
       .shp_uf_lvl_i               (shp_uf_lvl),
       .shp_dm_lvl_i               (shp_dm_lvl),
       .shp_svc_lvl_i              (shp_svc_lvl),
       .shp_psv_lvl_i              (shp_psv_lvl),
       .shp_st_lvl_i               (shp_st_lvl),
       .irq_pending_i              (irq_pending),
       .psv_pending_i              (psv_pending),
       .st_pending_i               (st_pending),
       .dbg_mon_pending_i          (dbg_mon_pending),
       .nmi_pending_i              (nmi_pending),
       .irq_active_i               (irq_active),
       .shcs_status_i              (shcs_status),
       .int_hipend_isr_i           (int_hipend_isr),
       .int_curr_isr_i             (int_curr_isr),
       .int_invoke_i               (int_invoke),
       .int_ret_to_base_i          (int_ret_to_base),
       .stick_cnt_flag_i           (stick_cnt_flag),
       .stick_clk_src_i            (stick_clk_src),
       .stick_int_en_i             (stick_int_en),
       .stick_en_i                 (stick_en),
       .stick_reload_i             (stick_reload),
       .stick_count_i              (stick_count),
       .stick_no_ref_i             (ext_st_calib_i[25]),
       .stick_skew_i               (ext_st_calib_i[24]),
       .stick_one_ms_i             (ext_st_calib_i[23:0]),
       .fsr_i                      (fsr),
       .hfsr_i                     (hfsr),
       .dfsr_i                     (dfsr),
       .afsr_i                     (afsr),
       .mm_bf_far_i                (mm_bf_far),
       .aux_ctrl_i                 (aux_ctrl),
       .vto_tbloff_i               (vto_tbloff),
       .cpacr_i                    (opt_cpacr),
       .fpccr_i                    (opt_fpccr),
       .fpcar_i                    (opt_fpcar),
       .fpdscr_i                   (opt_fpdscr),
       .airc_prigroup_i            (airc_prigroup),
       .ccr_div_0_trp_i            (ccr_div_0_trp),
       .ccr_unalign_trp_i          (ccr_unalign_trp),
       .ccr_bfhfnmign_i            (ccr_bfhfnmign),
       .ccr_stkalign_i             (ccr_stkalign),
       .ccr_nonbasethrdena_i       (ccr_nonbasethrdena),
       .ccr_usersetmpend_i         (ccr_usersetmpend),
       .scr_sleeponexit_i          (scr_sleeponexit),
       .scr_deepsleep_i            (scr_deepsleep),
       .scr_wfeonpend_i            (scr_wfeonpend),
       .slp_asleep_i               (slp_asleep),
       .lockup_i                   (lockup),
       .demc_mon_en_i              (demc_mon_en),
       .demc_mon_step_i            (demc_mon_step),
       .demc_mon_req_i             (demc_mon_req),
       .demc_trcena_i              (demc_trcena),
       .dhcs_s_reset_st_i          (dhcs_s_reset_st),
       .dhcs_s_retire_st_i         (dhcs_s_retire_st),
       .dhcs_c_snapstall_i         (dhcs_c_snapstall),
       .dhcs_c_mask_ints_i         (dhcs_c_mask_ints),
       .dhcs_c_halt_i              (dhcs_c_halt),
       .dhcs_c_step_i              (dhcs_c_step),
       .dhcs_c_debugen_i           (dhcs_c_debugen),
       .dcrs_rtrans_i              (dcrs_rtrans),
       .dbg_vcatch_i               (dbg_vcatch),
       .dbg_dcrd_i                 (dbg_dcrd),
       .dpu_dbg_halted_i           (dpu_dbg_halted_i),
       .dpu_dbg_ready_i            (dpu_dbg_ready_i),
       .ppb_hrdata_o               (ppb_hrdata),
       .ppb_hready_o               (ppb_hready),
       .ppb_hresp_o                (ppb_hresp),
       .ppb_we_o                   (ppb_we),
       .ppb_rd_o                   (ppb_rd),
       .ppb_hmaster_reg_o          (ppb_hmaster_reg),
       .ppb_reg_addr_o             (ppb_reg_addr));

  cm3_nvic_reg #(NUM_IRQ, LVL_WIDTH, DEBUG_LVL, RESET_ALL_REGS, WIC_PRESENT,
                 CP_PRESENT, `irq_width, `num_ints)
    u_cm3_nvic_reg
      (// Inputs
       .dreset_n                   (dreset_n),
       .hreset_n                   (hreset_n),
       .fclk                       (fclk),
       .hclk                       (hclk),
       .pclk                       (pclk),
       .ppb_hmaster_reg_i          (ppb_hmaster_reg),
       .ppb_we_i                   (ppb_we),
       .ppb_rd_i                   (ppb_rd),
       .ppb_reg_addr_i             (ppb_reg_addr),
       .mtx_ppb_hwdata_i           (mtx_ppb_hwdata_i),
       .ext_big_end_i              (ext_big_end_i),
       .ext_st_clk_i               (ext_st_clk_i),
       .ext_st_noref_i             (ext_st_calib_i[25]),
       .ext_cp_disable_i           (ext_cp_disable_i),
       .ext_stkalign_init_i        (ext_stkalign_init_i),
       .dpu_rst_pc_valid_i         (dpu_rst_pc_valid_i),
       .dpu_ahb_haddrd_i           (dpu_ahb_haddrd_i),
       .dpu_ahb_haddrd_valid_i     (dpu_ahb_haddrd_valid_i),
       .dpu_ahb_htransi_iss_i      (dpu_ahb_htransi_iss_i),
       .dpu_privileged_i           (dpu_privileged_i),
       .dpu_thread_i               (dpu_thread_i),
       .dpu_ivalid_i               (dpu_ivalid_i),
       .dpu_retire_i               (dpu_retire_i),
       .dpu_slp_ctl_i              (dpu_slp_ctl_i),
       .dpu_fpccr_we_i             (dpu_fpccr_we_i),
       .dpu_fpcar_nxt_i            (dpu_fpcar_nxt_i),
       .dpu_svc_i                  (dpu_svc_i),
       .dpu_bkpt_i                 (dpu_bkpt_i),
       .dpu_cp_busy_i              (dpu_cp_busy_i),
       .dpu_lazy_stk_i             (dpu_lazy_stk_i),
       .dpu_nxt_hfsr_i             (dpu_nxt_hfsr_i),
       .dpu_nxt_mfsr_i             (dpu_nxt_mfsr_i),
       .dpu_nxt_bfsr_i             (dpu_nxt_bfsr_i),
       .dpu_nxt_ufsr_i             (dpu_nxt_ufsr_i),
       .ext_aux_fault_i            (ext_aux_fault_i),
       .dwt_watchpoint_i           (dwt_watchpoint_i),
       .mtx_wb_abort_i             (mtx_wb_abort_i),
       .int_lazy_fpccr_i           (int_lazy_fpccr),
       .int_lazy_dm_i              (int_lazy_dm),
       .int_allow_bf_i             (int_allow_bf),
       .int_escalate_i             (int_escalate),
       .int_lazy_esc_i             (int_lazy_esc),
       .int_hfault_i               (int_hfault),
       .int_invoke_i               (int_invoke),
       .dpu_int_entry_i            (dpu_int_entry_i),
       .dpu_int_exit_i             (dpu_int_exit_i),
       .int_nxt_isr_i              (int_nxt_isr),
       .int_in_nmihf_i             (int_in_nmihf),
       .int_non_actv_exit_i        (int_non_actv_exit),
       .ext_slp_rxev_i             (ext_slp_rxev_i),
       .ext_slp_txev_i             (ext_slp_txev_i),
       .ext_slp_hold_req_n_i       (ext_slp_hold_req_n_i),
       .slp_norm_wake_i            (slp_norm_wake),
       .slp_pmask_pend_i           (slp_pmask_pend),
       .slp_wfe_pend_i             (slp_wfe_pend),
       .slp_sleeponexit_i          (slp_sleeponexit),
       .wic_ds_ack_n_i             (wic_ds_ack_n),
       .wic_ds_req_n_i             (wic_ds_req_n_i),
       .ext_dbg_en_i               (ext_dbg_en_i),
       .ext_dbg_req_i              (ext_dbg_req_i),
       .ext_dbg_restart_i          (ext_dbg_restart_i),
       .dpu_dbg_ready_i            (dpu_dbg_ready_i),
       .dpu_dbg_halted_i           (dpu_dbg_halted_i),
       .dbg_mon_active_i           (dbg_mon_active),
       .dbg_mon_fault_i            (dbg_mon_fault),
       .dbg_mon_allow_i            (dbg_mon_allow),
       .dbg_mon_step_pend_i        (dbg_mon_step_pend),
       .dbg_mon_hi_pri_invk_i      (dbg_mon_hi_pri_invk),
       .dbg_masked_bkpt_i          (dbg_masked_bkpt),
       .data_be8_o                 (data_be8),
       .aux_ctrl_o                 (aux_ctrl),
       .aux_ctrl1_we_o             (aux_ctrl1_we),
       .ccr_div_0_trp_o            (ccr_div_0_trp),
       .ccr_unalign_trp_o          (ccr_unalign_trp),
       .ccr_bfhfnmign_o            (ccr_bfhfnmign),
       .ccr_stkalign_o             (ccr_stkalign),
       .ccr_usersetmpend_o         (ccr_usersetmpend),
       .ccr_nonbasethrdena_o       (ccr_nonbasethrdena),
       .cpacr_o                    (cpacr),
       .fpccr_o                    (fpccr),
       .fpcar_o                    (fpcar),
       .fpdscr_o                   (fpdscr),
       .airc_vectreset_o           (airc_vectreset),
       .airc_sysresetreq_o         (airc_sysresetreq),
       .airc_prigroup_o            (airc_prigroup),
       .vto_tbloff_o               (vto_tbloff),
       .lockup_o                   (lockup),
       .irq_en_o                   (irq_en),
       .shcs_mm_en_o               (shcs_mm_en),
       .shcs_bf_en_o               (shcs_bf_en),
       .shcs_uf_en_o               (shcs_uf_en),
       .irq_lvl_o                  (irq_lvl),
       .shp_mm_lvl_o               (shp_mm_lvl),
       .shp_bf_lvl_o               (shp_bf_lvl),
       .shp_uf_lvl_o               (shp_uf_lvl),
       .shp_dm_lvl_o               (shp_dm_lvl),
       .shp_svc_lvl_o              (shp_svc_lvl),
       .shp_psv_lvl_o              (shp_psv_lvl),
       .shp_st_lvl_o               (shp_st_lvl),
       .int_pend_set_o             (int_pend_set),
       .int_pend_clr_o             (int_pend_clr),
       .int_actv_set_o             (int_actv_set),
       .int_actv_clr_o             (int_actv_clr),
       .vect_clr_actv_o            (vect_clr_actv),
       .stick_cnt_flag_o           (stick_cnt_flag),
       .stick_clk_src_o            (stick_clk_src),
       .stick_int_en_o             (stick_int_en),
       .stick_en_o                 (stick_en),
       .stick_reload_o             (stick_reload),
       .stick_count_o              (stick_count),
       .stick_int_o                (stick_int),
       .fsr_o                      (fsr),
       .hfsr_o                     (hfsr),
       .dfsr_o                     (dfsr),
       .afsr_o                     (afsr),
       .mm_bf_far_o                (mm_bf_far),
       .scr_sleeponexit_o          (scr_sleeponexit),
       .scr_deepsleep_o            (scr_deepsleep),
       .scr_wfeonpend_o            (scr_wfeonpend),
       .slp_wfe_valid_o            (slp_wfe_valid),
       .slp_wfi_valid_o            (slp_wfi_valid),
       .slp_hold_ack_n_o           (slp_hold_ack_n),
       .slp_asleep_o               (slp_asleep),
       .slp_sleeping_o             (slp_sleeping),
       .slp_sleepdeep_o            (slp_sleepdeep),
       .wic_load_o                 (wic_load),
       .wic_clear_o                (wic_clear),
       .demc_mon_en_o              (demc_mon_en),
       .demc_mon_req_o             (demc_mon_req),
       .demc_mon_step_o            (demc_mon_step),
       .demc_trcena_o              (demc_trcena),
       .dhcs_c_debugen_o           (dhcs_c_debugen),
       .dhcs_c_halt_o              (dhcs_c_halt),
       .dhcs_c_step_o              (dhcs_c_step),
       .dhcs_s_reset_st_o          (dhcs_s_reset_st),
       .dhcs_s_retire_st_o         (dhcs_s_retire_st),
       .dhcs_c_mask_ints_o         (dhcs_c_mask_ints),
       .dhcs_c_snapstall_o         (dhcs_c_snapstall),
       .dbg_mon_pend_o             (dbg_mon_pend),
       .dbg_mon_req_o              (dbg_mon_req),
       .dbg_dbgen_o                (dbg_dbgen),
       .dbg_vcatch_o               (dbg_vcatch),
       .dbg_halt_req_o             (dbg_halt_req),
       .dbg_step_o                 (dbg_step),
       .dbg_mask_ints_o            (dbg_mask_ints),
       .dbg_restarted_o            (dbg_restarted),
       .dbg_dcrd_o                 (dbg_dcrd),
       .dcrs_rtrans_o              (dcrs_rtrans),
       .dcrs_regwnr_o              (dcrs_regwnr),
       .dcrs_regsel_o              (dcrs_regsel));

  cm3_nvic_main #(NUM_IRQ, LVL_WIDTH, WIC_PRESENT, WIC_LINES, CP_PRESENT,
                `irq_width, `max_width, `num_ints, `spare)
    u_cm3_nvic_main
      (// Inputs
       .hreset_n                   (hreset_n),
       .fclk                       (fclk),
       .hclk                       (hclk),
       .ext_nmi_i                  (ext_nmi_i),
       .ext_irq_i                  (ext_irq_i),
       .dwt_watchpoint_i           (dwt_watchpoint_i),
       .mtx_wb_abort_i             (mtx_wb_abort_i),
       .dpu_svc_i                  (dpu_svc_i),
       .dpu_bkpt_i                 (dpu_bkpt_i),
       .stick_int_i                (stick_int),
       .dpu_nxt_hfsr_i             (dpu_nxt_hfsr_i),
       .dpu_nxt_mfsr_i             (dpu_nxt_mfsr_i),
       .dpu_nxt_bfsr_i             (dpu_nxt_bfsr_i),
       .dpu_nxt_ufsr_i             (dpu_nxt_ufsr_i),
       .int_pend_set_i             (int_pend_set),
       .int_pend_clr_i             (int_pend_clr),
       .int_actv_set_i             (int_actv_set),
       .int_actv_clr_i             (int_actv_clr),
       .vect_clr_actv_i            (vect_clr_actv),
       .irq_en_i                   (irq_en),
       .shcs_mm_en_i               (shcs_mm_en),
       .shcs_bf_en_i               (shcs_bf_en),
       .shcs_uf_en_i               (shcs_uf_en),
       .irq_lvl_i                  (irq_lvl),
       .shp_mm_lvl_i               (shp_mm_lvl),
       .shp_bf_lvl_i               (shp_bf_lvl),
       .shp_uf_lvl_i               (shp_uf_lvl),
       .shp_dm_lvl_i               (shp_dm_lvl),
       .shp_svc_lvl_i              (shp_svc_lvl),
       .shp_psv_lvl_i              (shp_psv_lvl),
       .shp_st_lvl_i               (shp_st_lvl),
       .dpu_basepri_we_i           (dpu_basepri_we_i),
       .dpu_basepri_nxt_i          (dpu_basepri_nxt_i),
       .dpu_fmask_we_i             (dpu_fmask_we_i),
       .dpu_fmask_nxt_i            (dpu_fmask_nxt_i),
       .dpu_pmask_we_i             (dpu_pmask_we_i),
       .dpu_pmask_nxt_i            (dpu_pmask_nxt_i),
       .dpu_raisepri_i             (dpu_raisepri_i),
       .dpu_int_entry_i            (dpu_int_entry_i),
       .dpu_int_exit_i             (dpu_int_exit_i),
       .dpu_int_return_i           (dpu_int_return_i),
       .dpu_int_retisr_i           (dpu_int_retisr_i),
       .dpu_lazy_stk_i             (dpu_lazy_stk_i),
       .dpu_retire_i               (dpu_retire_i),
       .dpu_mcyc_invk_msk_i        (dpu_mcyc_invk_msk_i),
       .dpu_ahb_invk_msk_i         (dpu_ahb_invk_msk_i),
       .ac_dismcycint_i            (ac_dismcycint),
       .airc_prigroup_i            (airc_prigroup),
       .vto_tbloff_i               (vto_tbloff),
       .ccr_bfhfnmign_i            (ccr_bfhfnmign),
       .fpccr_i                    (fpccr),
       .ext_vect_addr_i            (ext_vect_addr_i),
       .ext_vect_addr_en_i         (ext_vect_addr_en_i),
       .scr_wfeonpend_i            (scr_wfeonpend),
       .scr_sleeponexit_i          (scr_sleeponexit),
       .scr_deepsleep_i            (scr_deepsleep),
       .slp_wfe_valid_i            (slp_wfe_valid),
       .slp_wfi_valid_i            (slp_wfi_valid),
       .slp_hold_ack_n_i           (slp_hold_ack_n),
       .slp_asleep_i               (slp_asleep),
       .wic_ds_req_n_i             (wic_ds_req_n_i),
       .ext_dbg_req_i              (ext_dbg_req_i),
       .dhcs_c_debugen_i           (dhcs_c_debugen),
       .dpu_dbg_wr_isr_i           (dpu_dbg_wr_isr_i),
       .dbg_dbgen_i                (dbg_dbgen),
       .dbg_mask_ints_i            (dbg_mask_ints),
       .dbg_mon_pend_i             (dbg_mon_pend),
       .demc_mon_en_i              (demc_mon_en),
       .dbg_mon_req_i              (dbg_mon_req),
       .irq_pending_o              (irq_pending),
       .nmi_pending_o              (nmi_pending),
       .psv_pending_o              (psv_pending),
       .st_pending_o               (st_pending),
       .irq_active_o               (irq_active),
       .int_hfault_o               (int_hfault),
       .int_escalate_o             (int_escalate),
       .int_lazy_esc_o             (int_lazy_esc),
       .int_invoke_o               (int_invoke),
       .int_new_arrival_o          (int_new_arrival),
       .int_addr_o                 (int_addr),
       .int_tail_o                 (int_tail),
       .int_curr_isr_o             (int_curr_isr),
       .int_curr_pri_o             (int_curr_pri),
       .int_hipend_isr_o           (int_hipend_isr),
       .int_nxt_isr_o              (int_nxt_isr),
       .int_stk_actv_o             (int_stk_actv),
       .int_ret_to_base_o          (int_ret_to_base),
       .int_non_actv_exit_o        (int_non_actv_exit),
       .int_in_nmihf_o             (int_in_nmihf),
       .int_mpu_nmihf_o            (int_mpu_nmihf),
       .int_allow_bf_o             (int_allow_bf),
       .int_lazy_fpccr_o           (int_lazy_fpccr),
       .int_lazy_dm_o              (int_lazy_dm),
       .int_basepri_o              (int_basepri),
       .int_fmask_o                (int_fmask),
       .int_pmask_o                (int_pmask),
       .shcs_status_o              (shcs_status),
       .slp_pmask_pend_o           (slp_pmask_pend),
       .slp_wfe_pend_o             (slp_wfe_pend),
       .slp_sleeponexit_o          (slp_sleeponexit),
       .slp_norm_wake_o            (slp_norm_wake),
       .wic_ds_ack_n_o             (wic_ds_ack_n),
       .wic_mask_o                 (wic_mask),
       .dbg_masked_bkpt_o          (dbg_masked_bkpt),
       .dbg_mon_fault_o            (dbg_mon_fault),
       .dbg_mon_pending_o          (dbg_mon_pending),
       .dbg_mon_active_o           (dbg_mon_active),
       .dbg_mon_allow_o            (dbg_mon_allow),
       .dbg_mon_step_pend_o        (dbg_mon_step_pend),
       .dbg_mon_hi_pri_invk_o      (dbg_mon_hi_pri_invk));

  assign ac_dismcycint          = aux_ctrl[0];
  assign ac_no_def_wbuf         = aux_ctrl[1];
  assign ac_disfold             = aux_ctrl[2];
  assign ac_no_lazy_stk         = aux_ctrl[3];
  assign ac_no_ooo              = aux_ctrl[4];

  assign nvic_ppb_hready_o      = ppb_hready;
  assign nvic_ppb_hrdata_o      = ppb_hrdata;
  assign nvic_ppb_hresp_o       = ppb_hresp;
  assign nvic_int_invoke_o      = int_invoke;
  assign nvic_int_new_arrival_o = int_new_arrival;
  assign nvic_int_addr_o        = int_addr;
  assign nvic_int_tail_o        = int_tail;
  assign nvic_int_nxt_isr_o     = int_nxt_isr;
  assign nvic_int_curr_isr_o    = int_curr_isr;
  assign nvic_int_curr_pri_o    = int_curr_pri;
  assign nvic_int_stk_actv_o    = int_stk_actv;
  assign nvic_int_in_nmihf_o    = int_in_nmihf;
  assign nvic_int_mpu_nmihf_o   = int_mpu_nmihf;
  assign nvic_int_stk_algn_o    = ccr_stkalign;
  assign nvic_int_st_obs_o      = stick_int;
  assign nvic_basepri_o         = int_basepri;
  assign nvic_fmask_o           = int_fmask;
  assign nvic_pmask_o           = int_pmask;
  assign nvic_data_be8_o        = data_be8;
  assign nvic_bfhfnmign_o       = ccr_bfhfnmign;
  assign nvic_div_0_trp_o       = ccr_div_0_trp;
  assign nvic_unalign_trp_o     = ccr_unalign_trp;
  assign nvic_nonbasethrden_o   = ccr_nonbasethrdena;
  assign nvic_disfold_o         = ac_disfold;
  assign nvic_dismcycint_o      = ac_dismcycint;
  assign nvic_no_def_wbuf_o     = ac_no_def_wbuf;
  assign nvic_no_ooo_o          = opt_ac_no_ooo;
  assign nvic_no_lazy_stk_o     = opt_ac_no_lazy_stk;
  assign nvic_aux_ctrl1_we_o    = aux_ctrl1_we;
  assign nvic_cpacr_o           = opt_cpacr;
  assign nvic_fpccr_o           = opt_fpccr;
  assign nvic_fpcar_o           = opt_fpcar;
  assign nvic_fpdscr_o          = opt_fpdscr;
  assign nvic_lockup_o          = lockup;
  assign nvic_vect_reset_n_o    = airc_vectreset;
  assign nvic_sys_reset_req_o   = airc_sysresetreq;
  assign nvic_slp_sleeponexit_o = slp_sleeponexit;
  assign nvic_slp_asleep_o      = slp_asleep;
  assign nvic_slp_sleeping_o    = slp_sleeping;
  assign nvic_slp_sleepdeep_o   = slp_sleepdeep;
  assign nvic_slp_hold_ack_n_o  = slp_hold_ack_n;
  assign nvic_wic_ds_ack_n_o    = opt_wic_ds_ack_n;
  assign nvic_wic_load_o        = opt_wic_load;
  assign nvic_wic_clear_o       = opt_wic_clear;
  assign nvic_wic_mask_o        = opt_wic_mask;
  assign nvic_dbg_halt_req_o    = dbg_halt_req;
  assign nvic_dbg_mon_req_o     = dbg_mon_req;
  assign nvic_dbg_step_o        = dbg_step;
  assign nvic_dbg_restarted_o   = dbg_restarted;
  assign nvic_dbg_trans_o       = dcrs_rtrans;
  assign nvic_dbg_reg_wr_o      = dcrs_regwnr;
  assign nvic_dbg_reg_addr_o    = dcrs_regsel;
  assign nvic_dbg_snapstall_o   = dhcs_c_snapstall;
  assign nvic_trc_en_o          = demc_trcena;

endmodule

`include "cm3_nvic_undefs.vp"
