module cm3_nvic_cell
  (// Inputs
   a_lvl_i, b_lvl_i, a_num_i, b_num_i,
   out_lvl_o, out_num_o
   );

  parameter LVL_WIDTH       = 3;                    // Interrupt priority width
  parameter NUM_WIDTH       = 0;

  input [LVL_WIDTH-1:0]     a_lvl_i;                // input a priority level
  input [LVL_WIDTH-1:0]     b_lvl_i;                // input b priority level
  input [NUM_WIDTH:0]       a_num_i;                // input a int number
  input [NUM_WIDTH:0]       b_num_i;                // input b int number

  output [LVL_WIDTH-1:0]    out_lvl_o;              // priority level of winner
  output [NUM_WIDTH+1:0]    out_num_o;              // int number of winner

  wire                      a_lt_b;                 // a_lvl < b_lvl
  wire                      force_a;                // Force A if B not en but A is
  wire                      force_b;                // Force B if A is not enabled
  wire                      a_l_b_76;               // a_lvl[7:6] < b_lvl[7:6]
  wire                      a_e_b_76;               // a_lvl[7:6] = b_lvl[7:6]
  wire                      a_l_b_54;               // a_lvl[5:4] < b_lvl[5:4]
  wire                      a_e_b_54;               // a_lvl[5:4] = b_lvl[5:4]
  wire                      a_l_b_32;               // a_lvl[3:2] < b_lvl[3:2]
  wire                      a_e_b_32;               // a_lvl[3:2] = b_lvl[3:2]
  wire                      a_l_b_10;               // a_lvl[1:0] < b_lvl[1:0]

  wire [LVL_WIDTH-1:0]      out_lvl;                // priority level of winner
  wire [NUM_WIDTH+1:0]      out_num;                // int number of winner

  reg  [7:0]                full_a_lvl;             // Max supported level width
  reg  [7:0]                full_b_lvl;             // Max supported level width


  always @ (a_lvl_i or b_lvl_i)
    begin
      full_a_lvl                = 8'b00000000;
      full_b_lvl                = 8'b00000000;
      full_a_lvl[7:8-LVL_WIDTH] = a_lvl_i;
      full_b_lvl[7:8-LVL_WIDTH] = b_lvl_i;
    end

  assign force_a  =  a_num_i[0] & ~b_num_i[0];
  assign force_b  = ~a_num_i[0];

  assign a_l_b_76 = ((~full_a_lvl[7] & (full_b_lvl[7] | (~full_a_lvl[6] & full_b_lvl[6]))) |
                     (~full_a_lvl[6] & full_b_lvl[7] & full_b_lvl[6]));

  assign a_e_b_76 = full_a_lvl[7:6] == full_b_lvl[7:6];

  assign a_l_b_54 = ((~full_a_lvl[5] & (full_b_lvl[5] | (~full_a_lvl[4] & full_b_lvl[4]))) |
                     (~full_a_lvl[4] & full_b_lvl[5] & full_b_lvl[4]));

  assign a_e_b_54 = full_a_lvl[5:4] == full_b_lvl[5:4];

  assign a_l_b_32 = ((~full_a_lvl[3] & (full_b_lvl[3] | (~full_a_lvl[2] & full_b_lvl[2]))) |
                     (~full_a_lvl[2] & full_b_lvl[3] & full_b_lvl[2]));

  assign a_e_b_32 = full_a_lvl[3:2] == full_b_lvl[3:2];

  assign a_l_b_10 = ((~full_a_lvl[1] & (full_b_lvl[1] | (~full_a_lvl[0] & full_b_lvl[0]))) |
                     (~full_a_lvl[0] & full_b_lvl[1] & full_b_lvl[0]));

  assign a_lt_b   = (force_a |
                     (~force_b & ((a_l_b_76) |
                                  (a_e_b_76 & a_l_b_54) |
                                  (a_e_b_76 & a_e_b_54 & a_l_b_32) |
                                  (a_e_b_76 & a_e_b_54 & a_e_b_32 & a_l_b_10))));

  assign out_lvl  = (({LVL_WIDTH{a_lt_b}} & a_lvl_i) |
                     ({LVL_WIDTH{~a_lt_b}} & b_lvl_i));

  assign out_num[NUM_WIDTH:0] = (({NUM_WIDTH+1{ a_lt_b}} & a_num_i) |
                                 ({NUM_WIDTH+1{~a_lt_b}} & b_num_i));

  assign out_num[NUM_WIDTH+1] = a_lt_b;

  assign out_lvl_o = out_lvl;
  assign out_num_o = out_num;

endmodule
