`include "cm3_nvic_defs.vp"

module cm3_nvic_main
  (// Inputs
   hreset_n, fclk, hclk, ext_nmi_i, ext_irq_i, dwt_watchpoint_i, mtx_wb_abort_i,
   dpu_svc_i, dpu_bkpt_i, stick_int_i, dpu_nxt_hfsr_i, dpu_nxt_mfsr_i,
   dpu_nxt_bfsr_i, dpu_nxt_ufsr_i, int_pend_set_i, int_pend_clr_i,
   int_actv_set_i, int_actv_clr_i, vect_clr_actv_i, irq_en_i, shcs_mm_en_i,
   shcs_bf_en_i, shcs_uf_en_i, irq_lvl_i, shp_mm_lvl_i, shp_bf_lvl_i,
   shp_uf_lvl_i, shp_dm_lvl_i, shp_svc_lvl_i, shp_psv_lvl_i, shp_st_lvl_i,
   dpu_basepri_we_i, dpu_basepri_nxt_i, dpu_fmask_we_i, dpu_fmask_nxt_i,
   dpu_pmask_we_i, dpu_pmask_nxt_i, dpu_raisepri_i, dpu_int_entry_i,
   dpu_int_exit_i, dpu_int_return_i, dpu_int_retisr_i, dpu_lazy_stk_i,
   dpu_retire_i, dpu_mcyc_invk_msk_i, dpu_ahb_invk_msk_i, ac_dismcycint_i,
   airc_prigroup_i, vto_tbloff_i, ccr_bfhfnmign_i, fpccr_i, ext_vect_addr_en_i,
   ext_vect_addr_i, scr_wfeonpend_i, scr_sleeponexit_i, scr_deepsleep_i,
   slp_wfe_valid_i, slp_wfi_valid_i, slp_hold_ack_n_i, slp_asleep_i,
   wic_ds_req_n_i, ext_dbg_req_i, dhcs_c_debugen_i, dpu_dbg_wr_isr_i,
   dbg_dbgen_i, dbg_mask_ints_i, dbg_mon_pend_i, demc_mon_en_i, dbg_mon_req_i,
   irq_pending_o, nmi_pending_o, psv_pending_o, st_pending_o, irq_active_o,
   int_hfault_o, int_escalate_o, int_lazy_esc_o, int_invoke_o, int_new_arrival_o,
   int_addr_o, int_tail_o, int_curr_isr_o, int_curr_pri_o, int_hipend_isr_o,
   int_nxt_isr_o, int_stk_actv_o, int_ret_to_base_o, int_non_actv_exit_o,
   int_in_nmihf_o, int_mpu_nmihf_o, int_allow_bf_o, int_lazy_fpccr_o,
   int_lazy_dm_o, int_basepri_o, int_fmask_o, int_pmask_o, shcs_status_o,
   slp_pmask_pend_o, slp_wfe_pend_o, slp_sleeponexit_o, slp_norm_wake_o,
   wic_ds_ack_n_o, wic_mask_o, dbg_masked_bkpt_o, dbg_mon_fault_o,
   dbg_mon_pending_o, dbg_mon_active_o, dbg_mon_allow_o, dbg_mon_step_pend_o,
   dbg_mon_hi_pri_invk_o
   );


  parameter NUM_IRQ         = 16;                   // Number of interrupts
  parameter LVL_WIDTH       = 3;                    // Interrupt priority width
  parameter WIC_PRESENT     = 0;                    // WIC present if 1
  parameter WIC_LINES       = 3;                    // Number of WIC lines
  parameter CP_PRESENT      = 0;                    // CP present if 1
  parameter IRQ_WIDTH       = NUM_IRQ*LVL_WIDTH;    // Total priority width
  parameter MAX_WIDTH       = `max_ints*LVL_WIDTH;  // Max priority width
  parameter NUM_INTS        = `num_non_irq+NUM_IRQ; // Total supported ints
  parameter SPARE           = `max_ints-NUM_INTS-`num_res_rs;

  input                     hreset_n;               // NVIC reset
  input                     fclk;                   // NVIC Clock (free-running)
  input                     hclk;                   // NVIC Clock (Clock gated)
  input                     ext_nmi_i;              // NMI
  input [NUM_IRQ-1:0]       ext_irq_i;              // Interrupt request
  input                     dwt_watchpoint_i;       // DWT generated watchpoint
  input                     mtx_wb_abort_i;         // Write buffer abort
  input                     dpu_svc_i;              // SVC has been executed
  input                     dpu_bkpt_i;             // BKPT has been executed
  input                     stick_int_i;            // System Tick interrupt
  input                     dpu_nxt_hfsr_i;         // Vector fetch hard fault
  input [4:0]               dpu_nxt_mfsr_i;         // MemManage fault type
  input [5:0]               dpu_nxt_bfsr_i;         // Bus fault type
  input [9:0]               dpu_nxt_ufsr_i;         // Usage fault type
  input [NUM_INTS-1:0]      int_pend_set_i;         // Set pending
  input [NUM_INTS-1:0]      int_pend_clr_i;         // Clear pending
  input [NUM_INTS-1:0]      int_actv_set_i;         // Set active
  input [NUM_INTS-1:0]      int_actv_clr_i;         // Clear active
  input                     vect_clr_actv_i;        // Vector clear active
  input [NUM_IRQ-1:0]       irq_en_i;               // IRQ enables
  input                     shcs_mm_en_i;           // MemManage fault enable
  input                     shcs_bf_en_i;           // Bus fault enable
  input                     shcs_uf_en_i;           // Usage fault enable
  input [IRQ_WIDTH-1:0]     irq_lvl_i;              // Priority for each IRQ
  input [LVL_WIDTH-1:0]     shp_mm_lvl_i;           // Priority for MemManage
  input [LVL_WIDTH-1:0]     shp_bf_lvl_i;           // Priority for Bus Fault
  input [LVL_WIDTH-1:0]     shp_uf_lvl_i;           // Priority for Undef Fault
  input [LVL_WIDTH-1:0]     shp_dm_lvl_i;           // Priority for Debug Mon
  input [LVL_WIDTH-1:0]     shp_svc_lvl_i;          // Priority for SVC
  input [LVL_WIDTH-1:0]     shp_psv_lvl_i;          // Priority for pend SVC
  input [LVL_WIDTH-1:0]     shp_st_lvl_i;           // Priority for System Tick
  input                     dpu_basepri_we_i;       // Base priority write enable
  input [7:0]               dpu_basepri_nxt_i;      // Next base priority
  input                     dpu_fmask_we_i;         // Fault mask write enable
  input                     dpu_fmask_nxt_i;        // Next fault mask
  input                     dpu_pmask_we_i;         // Pri mask write enable
  input                     dpu_pmask_nxt_i;        // Next pri mask
  input                     dpu_raisepri_i;         // Raise priority
  input                     dpu_int_entry_i;        // ISR has been entered
  input                     dpu_int_exit_i;         // ISR has completed
  input                     dpu_int_return_i;       // Stack restoration done
  input [8:0]               dpu_int_retisr_i;       // ISR returned to
  input                     dpu_lazy_stk_i;         // Lazy stack in progress
  input                     dpu_retire_i;           // Opcode is retiring
  input                     dpu_mcyc_invk_msk_i;    // Multi-cycle op executing
  input                     dpu_ahb_invk_msk_i;     // Constant AHB invoke mask
  input                     ac_dismcycint_i;        // Don't int multi-cyc ops
  input [2:0]               airc_prigroup_i;        // Priority grouping position
  input [31:7]              vto_tbloff_i;           // Base location of vector table
  input                     ccr_bfhfnmign_i;        // Ignore bus fault in NMI/HF
  input [31:0]              fpccr_i;                // FPCCR value
  input                     ext_vect_addr_en_i;     // Enable vector table remap
  input [9:0]               ext_vect_addr_i;        // Vector table remap value
  input                     scr_wfeonpend_i;        // Wake WFE on pended ints
  input                     scr_sleeponexit_i;      // Sleep on exit requested
  input                     scr_deepsleep_i;        // deepsleep set
  input                     slp_wfe_valid_i;        // Sleep due to WFE
  input                     slp_wfi_valid_i;        // Sleep due to WFI
  input                     slp_hold_ack_n_i;       // Core held in sleep mode
  input                     slp_asleep_i;           // DPU is sleeping
  input                     wic_ds_req_n_i;         // WIC mode request
  input                     ext_dbg_req_i;          // External debug request
  input                     dhcs_c_debugen_i;       // Halting debug enable in dhcs
  input                     dpu_dbg_wr_isr_i;       // Debug over-ride of ISR number
  input                     dbg_dbgen_i;            // Halting debug enable
  input                     dbg_mask_ints_i;        // Mask interrupts during debug
  input                     dbg_mon_pend_i;         // Debug monitor has been pended
  input                     demc_mon_en_i;          // Debug monitor enable
  input                     dbg_mon_req_i;          // Request the core to pause

  output [NUM_IRQ-1:0]      irq_pending_o;          // Pending IRQs
  output                    nmi_pending_o;          // NMI is pending
  output                    psv_pending_o;          // SVC from pend is pending
  output                    st_pending_o;           // SysTick is pending
  output [NUM_IRQ-1:0]      irq_active_o;           // Active IRQs
  output                    int_hfault_o;           // HardFault raised
  output                    int_escalate_o;         // Escalated to HF
  output                    int_lazy_esc_o;         // Lazy fault escalated
  output                    int_invoke_o;           // Perform hardware stacking
  output                    int_new_arrival_o;      // New high priority interrupt
  output [31:0]             int_addr_o;             // ISR location for interrupt
  output                    int_tail_o;             // Will tail-chain
  output [8:0]              int_curr_isr_o;         // Current ISR number
  output [7:0]              int_curr_pri_o;         // Current ISR priority
  output [7:0]              int_hipend_isr_o;       // Highest pending interrupt
  output [7:0]              int_nxt_isr_o;          // Next ISR number
  output                    int_stk_actv_o;         // Stack is active
  output                    int_ret_to_base_o;      // Return to base
  output                    int_non_actv_exit_o;    // Non-active ISR exited
  output [2:0]              int_in_nmihf_o;         // NMI/HF {pend,active}
  output                    int_mpu_nmihf_o;        // Inform MPU if its in NMI/HF
  output                    int_allow_bf_o;         // Allow bus fault interrupt
  output [3:0]              int_lazy_fpccr_o;       // Lazy FPCCR ready values
  output                    int_lazy_dm_o;          // Lazy debug monitor
  output [7:0]              int_basepri_o;          // Base priority
  output                    int_fmask_o;            // Fault mask
  output                    int_pmask_o;            // Priority mask
  output [15:0]             shcs_status_o;          // System Handler status
  output                    slp_pmask_pend_o;       // PriMask suppressed pend
  output                    slp_wfe_pend_o;         // New pend wake up for WFE
  output                    slp_sleeponexit_o;      // Will sleep on exit
  output                    slp_norm_wake_o;        // Normal sleep wake up
  output                    wic_ds_ack_n_o;         // WIC mode acknowledge
  output [WIC_LINES-1:0]    wic_mask_o;             // WIC mask sensitivity list
  output                    dbg_masked_bkpt_o;      // BKPT will halt
  output                    dbg_mon_fault_o;        // Debug monitor has occurred
  output                    dbg_mon_pending_o;      // Debug monitor is pending
  output                    dbg_mon_active_o;       // Debug monitor is active
  output                    dbg_mon_allow_o;        // DbgMon will be able to invoke
  output                    dbg_mon_step_pend_o;    // DbgMon step has been requested
  output                    dbg_mon_hi_pri_invk_o;  // DbgMon pri > invoke pri

  reg                       int_invoke;             // Perform hardware stacking
  reg                       int_tail;               // Will tail-chain
  reg                       int_new_arrival;        // New high priority interrupt
  reg                       int_fmask;              // Fault mask
  reg                       int_pmask;              // Priority mask
  reg                       int_ret_to_base;        // next return is to thread
  reg                       slp_norm_wake;          // Normal sleep wake up

  wire [`max_ints-1:0]      int_en;                 // Interrupt Enable
  wire [`max_ints  :0]      int_en_tmp;             // Int Enable with MSB always 0
  wire [NUM_INTS-1:0]       int_pend_s;             // Pend Source
  wire [NUM_INTS-1:0]       int_pend_v;             // Pend Vector
  wire [`max_ints:0]        raw_ints_pend;          // Internal Pend Vector (masked)
  wire [`max_ints-1:0]      ints_pend;              // Pend Vector (masked)
  wire [NUM_INTS-1:0]       int_actv_v;             // Reduced Active Vector
  wire [`max_ints:0]        ints_actv;              // Full Active Vector
  wire [`max_ints-1:0]      mask_ints_actv;         // Full Active Vector (masked)

  wire [NUM_INTS-1:0]       opt_int_actv_set;       // Internal active set
  wire [NUM_INTS-1:0]       int_actv_set_i;         // Set active
  wire [NUM_INTS-1:0]       int_actv_clr_i;         // Clear active
  reg  [`max_ints-1:0]      isr_entry;              // One-hot entry to an ISR
  wire [NUM_INTS-1:0]       int_isr_entry;          // Reduced width isr_entry
  wire [`max_ints-1:0]      isr_exit;               // One-hot exit from an ISR
  wire [NUM_INTS-1:0]       int_isr_exit;           // Reduced width isr_exit

  wire                      int_pend;               // An interrupt is pending
  wire                      int_actv;               // An interrupt is active
  wire                      nxt_int_ret_to_base;    // Next int_ret_to_base

  wire [LVL_WIDTH-1:0]      int_pend_lvl;           // Highest priority pending level
  wire [7:0]                int_pend_num;           // ISR number to match int_pend_lvl
  wire [LVL_WIDTH-1:0]      int_stack_lvl;          // Highest priority stacked level
  wire                      core_at_base;           // In thread

  wire                      prem_pend_hi_pri_curr;  // Pending pri > current pri
  wire                      prem_pend_hi_pri_stack; // Pending pri > stack pri
  wire                      prem_pend_hi_pri_invk;  // Pending pri > invoking pri

  wire                      preempt_invoke;         // Will perform a pre-emptive invoke
  wire                      masked_p_invoke;        // Masked preempt invoke based
  wire                      tail_invoke;            // Will perform a tail invoke
  wire                      set_invoke;             // Set the int_invoke signal
  wire                      clr_invoke;             // Clear the int_invoke signal
  wire                      nxt_int_invoke;         // Next int_invoke
  wire                      nxt_int_tail;           // Next int_tail
  wire                      nxt_int_new_arrival;    // Next new_arrival

  wire                      mem_manage;             // Local MemManage Handler
  wire                      bus_fault;              // Local Bus Handler
  wire                      usage_fault;            // Local Usage Handler
  wire                      dbg_mon_fault;          // Debug Monitor
  wire                      pend_sv;                // SVC from pend Handler
  wire                      sv_call;                // SVC Handler
  wire                      int_escalate;           // Escalate to hard fault
  wire                      int_lazy_esc;           // Lazy fault escalated
  wire [4:0]                local_fault_sel;        // Masked DbgMon LocalFault signal
  wire                      dbg_mon_non_esc;        // A non-escalatable debug mon pend
  wire                      non_esc_bus_f;          // A non-escalatable bus fault pend
  wire                      asynch_d_mon;           // Asynchronus debug event occurred

  reg  [7:0]                int_nxt_isr;            // Next ISR number
  reg  [LVL_WIDTH-1:0]      int_nxt_lvl;            // Priority of int_nxt_isr
  wire                      int_nxt_isr_we;         // int_nxt_isr write enable
  wire                      int_nxt_lvl_we;         // int_nxt_lvl write enable

  reg                       force_int_addr;         // force int_addr to be vectaddr
  reg                       force_int_pulse;        // force int_addr pulse
  wire                      force_addr_we;          // write enable for force_int_addr
  wire                      force_pulse_we;         // write enable for force_int_pulse
  wire                      nxt_force_int_addr;     // next force_int_addr
  wire                      nxt_force_int_pulse;    // next force_int_addr
  wire [7:0]                lower_int_addr;         // lower portion of int_addr

  wire                      wic_ds_ack_n;           // WIC mode acknowledge
  wire [WIC_LINES-1:0]      wic_mask;               // WIC mask sensitivity list
  wire                      opt_wic_en;             // WIC logic removal term
  wire                      opt_wic_ds_ack_n;       // WIC mode acknowledge
  wire [WIC_LINES-1:0]      opt_wic_mask;           // WIC mask sensitivity list

  reg                       in_pop;                 // Currently popping stack frame
  wire                      set_in_pop;             // Set in_pop signal
  wire                      clr_in_pop;             // Clear in_pop signal
  wire                      in_pop_we;              // in_pop register write enable
  reg                       fault_pending;          // Additional tracking of HF status
  wire                      f_pending_we;           // fault_pending write enable
  wire                      hfault_raised;          // HardFault has occurred
  wire                      hfault_active;          // HardFault is active
  wire                      nmi_active;             // NMI is active
  wire [MAX_WIDTH:0]        int_ints_lvl;           // Internal ints_lvl
  wire [MAX_WIDTH-1:0]      ints_lvl;               // Priority for each interrupt
  wire                      fault_raised;           // A fault has been raised
  wire [LVL_WIDTH-1:0]      fault_lvl;              // Level of the fault raised
  wire                      fault_comp_en;          // Raised fault is enabled
  wire                      imp_bus_fault_esc;      // Imprecise bus fault escalted
  wire                      fault_hi_pri_curr;      // Raised fault pri > current pri
  wire                      fault_hi_pri_stack;     // Raised fault pri > stack pri
  wire                      fault_hi_pri_base;      // Raised fault pri > base pri
  wire                      always_escalate;        // Always escalate for certain cases
  wire                      norm_esc;               // Normal escalation cases
  wire                      dbg_mon_allow;          // DbgMon will be able to invoke
  wire                      dbg_mon_int_allow;      // dbg_mon_allow except int_pmask
  wire                      dbg_mon_ext_allow;      // dbg_mon_allow except int_pmask
  wire                      dbg_mon_exit;           // Sleep on exit case for dbg mon
  wire                      dbg_wake_up;            // int_pmask suppressed debug events
  wire                      int_no_basepri;         // int_basepri is disabled
  reg  [LVL_WIDTH-1:0]      curr_set_lvl;           // Current active ISR level
  wire [8:0]                int_int_curr_pri;       // Expanded curr_set_lvl
  wire                      dbg_mon_hi_pri_curr;    // DbgMon pri > current pri
  wire                      dbg_mon_hi_pri_stack;   // DbgMon pri > stack pri
  wire                      dbg_mon_hi_pri_invk;    // DbgMon pri > invoke pri
  wire [8:0]                pad_int_basepri;        // Padded int_basepri
  reg  [LVL_WIDTH-1:0]      reg_int_basepri;        // Width reduced int_basepri
  wire [LVL_WIDTH-1:0]      opt_int_basepri;        // Width reduced int_basepri
  wire [LVL_WIDTH-1:0]      int_basepri_lvl;        // Effective width reduced basepri
  wire                      int_fmask_clr;          // int_fmask clear
  wire                      int_fmask_we;           // int_fmask write enable
  wire                      nxt_int_fmask;          // Next int_fmask
  wire                      int_mask_en;            // mask write enables
  wire                      int_pmask_we;           // int_pmask write enable
  wire                      int_basepri_we;         // int_basepri write enable
  wire                      fmask_wr_valid;         // int_fmask write enable
  wire [7:0]                prem_mask;              // Pre-emptive priority mask
  wire                      will_pre_empt;          // Pending int will cause a preemption
  wire                      nxt_slp_pmask_pend;     // Next PriMask suppressed pend
  reg                       slp_pmask_pend;         // PriMask suppressed pend
  wire                      int_pmask_disabled;     // Pending ints pri masked by int_pmask
  wire                      int_allow_bf;           // Allow bus fault interrupt
  wire                      curr_void;              // Current ISR number is not active
  wire                      use_curr_void;          // curr_void can be used in indicators
  reg                       int_non_actv_exit;      // Exited a non-active ISR
  wire                      nxt_int_non_actv_exit;  // Next non_active_exit
  wire                      int_non_actv_exit_we;   // write enable for non_active_exit
  reg  [`max_ints-1:0]      curr_select;            // One-hot select of current active ISR
  wire                      int_curr_isr_we;        // write enable for current int register
  wire [8:0]                nxt_int_curr_isr;       // next value for current int register
  reg  [8:0]                int_curr_isr;           // current int register
  wire                      hfault_pending;         // HardFault is pending
  wire                      nmi_nvic_dpu_lockup;    // Corner case control for NMI lock up
  wire                      nxt_int_stk_actv;       // About to enter a handler 2 deep
  wire                      int_pmask_sleep;        // Don't sleep due to int_pmask'd interrupt
  wire                      nxt_slp_sleeponexit;    // next value for sleep in exit reg
  reg                       slp_sleeponexit;        // Will sleep on exit
  wire                      pend_lt_stack;          // Pend pri < stack pri
  wire [31:0]               int_addr;               // ISR location for interrupt
  wire                      nmi_pending;            // NMI is pending
  wire                      dbg_mon_pending;        // Debug monitor is pending
  wire                      dbg_mon_active;         // Debug monitor is active
  wire                      psv_pending;            // SVC from pend is pending
  wire                      st_pending;             // SysTick is pending
  wire [NUM_IRQ-1:0]        irq_pending;            // Pending IRQs
  wire [7:0]                int_hipend_isr;         // Highest pending vector number
  wire [NUM_IRQ-1:0]        irq_active;             // Active IRQs
  wire [2:0]                int_in_nmihf;           // NMI/HF {pend,active}
  wire                      int_mpu_nmihf;          // Inform MPU if its in NMI/HF
  wire [15:0]               shcs_status;            // System Handler status
  wire                      int_stk_actv;           // Stack is active
  wire [7:0]                int_basepri;            // Base priority
  wire                      int_hfault;             // HardFault raised
  wire                      slp_wfe_pend;           // New pend wake up for WFE
  wire                      dbg_mon_step_pend;      // DbgMon step requested
  wire                      dbg_masked_bkpt;        // BKPT will halt
  wire [7:0]                int_curr_pri;           // Current ISR priority
  wire                      bf_hi_pri_stack;        // Bus fault pri > stack pri
  wire                      bf_hi_pri_curr;         // Bus fault pri > current pri
  wire                      bf_hi_pri_base;         // Bus fault pri > base pri
  wire                      mm_hi_pri_stack;        // MemManage pri > stack pri
  wire                      mm_hi_pri_curr;         // MemManage pri > current pri
  wire                      mm_hi_pri_base;         // MemManage pri > base pri
  wire [3:0]                int_lazy_fpccr;         // Lazy FPCCR ready values
  wire                      lazy_mm;                // Lazy MemManage pend
  wire                      lazy_bf;                // Lazy bus fault pend
  wire                      lazy_hf;                // Lazy hard fault pend
  wire                      lazy_dm;                // Lazy debug monitor pend
  wire                      opt_cp_en;              // CP logic removal

  integer                   l0;
  integer                   l1;
  integer                   l2;

`ifdef ARM_DSM

  wire [3:0]                dsm_lvl_width;
  wire                      dsm_wic_present;
  wire                      dsm_cp_present;
  reg  [7:0]                dsm_pmask;
  integer                   k;

  assign opt_wic_ds_ack_n = dsm_wic_present ? wic_ds_ack_n  : 1'b1;
  assign opt_wic_mask     = dsm_wic_present ? wic_mask      : {WIC_LINES{1'b0}};
  assign opt_wic_en       = dsm_wic_present ? 1'b1          : 1'b0;
  assign opt_cp_en        = dsm_cp_present  ? 1'b1          : 1'b0;

  always @ (dsm_lvl_width)
    for (k = 0; k < 8; k = k + 1)
      dsm_pmask[k] = ((dsm_lvl_width == 8) | (k > (7 - dsm_lvl_width))) ? 1'b1 : 1'b0;

`else

  assign opt_wic_ds_ack_n = (WIC_PRESENT != 0) ? wic_ds_ack_n : 1'b1;
  assign opt_wic_mask     = (WIC_PRESENT != 0) ? wic_mask     : {WIC_LINES{1'b0}};
  assign opt_wic_en       = (WIC_PRESENT != 0) ? 1'b1         : 1'b0;
  assign opt_cp_en        = (CP_PRESENT != 0)  ? 1'b1         : 1'b0;

`endif


  function a_lvl_lt_b_lvl;

    input [LVL_WIDTH-1:0]   a_lvl;
    input [LVL_WIDTH-1:0]   b_lvl;
    input [7:0]             mask;

    reg   [8:0]             full_a_lvl; // a_lvl padded out to maximum
    reg   [8:0]             full_b_lvl; // b_lvl padded out to maximum
    reg   [7:0]             mask_a_lvl; // Masked full_a_lvl
    reg   [7:0]             mask_b_lvl; // Masked full_b_lvl
    reg                     a_l_b_76;   // a_lvl[7:6] < b_lvl[7:6]
    reg                     a_e_b_76;   // a_lvl[7:6] = b_lvl[7:6]
    reg                     a_l_b_54;   // a_lvl[5:4] < b_lvl[5:4]
    reg                     a_e_b_54;   // a_lvl[5:4] = b_lvl[5:4]
    reg                     a_l_b_32;   // a_lvl[3:2] < b_lvl[3:2]
    reg                     a_e_b_32;   // a_lvl[3:2] = b_lvl[3:2]
    reg                     a_l_b_10;   // a_lvl[1:0] < b_lvl[1:0]

    begin

      full_a_lvl = {{9-LVL_WIDTH{1'b0}}, a_lvl} << (8 - LVL_WIDTH);
      full_b_lvl = {{9-LVL_WIDTH{1'b0}}, b_lvl} << (8 - LVL_WIDTH);

      mask_a_lvl = full_a_lvl[7:0] & mask;
      mask_b_lvl = full_b_lvl[7:0] & mask;

      a_l_b_76   = ((~mask_a_lvl[7] & (mask_b_lvl[7] |
                                       (~mask_a_lvl[6] & mask_b_lvl[6]))) |
                    (~mask_a_lvl[6] & mask_b_lvl[7] & mask_b_lvl[6]));

      a_e_b_76   = (mask_a_lvl[7:6] == mask_b_lvl[7:6]);

      a_l_b_54   = ((~mask_a_lvl[5] & (mask_b_lvl[5] |
                                       (~mask_a_lvl[4] & mask_b_lvl[4]))) |
                    (~mask_a_lvl[4] & mask_b_lvl[5] & mask_b_lvl[4]));

      a_e_b_54   = (mask_a_lvl[5:4] == mask_b_lvl[5:4]);

      a_l_b_32   = ((~mask_a_lvl[3] & (mask_b_lvl[3] |
                                       (~mask_a_lvl[2] & mask_b_lvl[2]))) |
                    (~mask_a_lvl[2] & mask_b_lvl[3] & mask_b_lvl[2]));

      a_e_b_32   = (mask_a_lvl[3:2] == mask_b_lvl[3:2]);

      a_l_b_10   = ((~mask_a_lvl[1] & (mask_b_lvl[1] |
                                       (~mask_a_lvl[0] & mask_b_lvl[0]))) |
                    (~mask_a_lvl[0] & mask_b_lvl[1] & mask_b_lvl[0]));

      a_lvl_lt_b_lvl = ((a_l_b_76) |
                        (a_e_b_76 & a_l_b_54) |
                        (a_e_b_76 & a_e_b_54 & a_l_b_32) |
                        (a_e_b_76 & a_e_b_54 & a_e_b_32 & a_l_b_10));

    end
  endfunction

  assign prem_mask       = (8'b11111110 << airc_prigroup_i);


  assign int_no_basepri  = ~(|opt_int_basepri);

`ifdef ARM_DSM
  assign int_basepri_lvl = ({LVL_WIDTH{int_no_basepri}} | opt_int_basepri) & dsm_pmask;
`else
  assign int_basepri_lvl = {LVL_WIDTH{int_no_basepri}} | opt_int_basepri;
`endif

  assign int_allow_bf    = ~(|int_in_nmihf[1:0] & ccr_bfhfnmign_i);


  assign int_ints_lvl    = { {((SPARE*LVL_WIDTH) + 1){1'b0}},
                             irq_lvl_i,
                             shp_st_lvl_i,
                             shp_psv_lvl_i,
                             {LVL_WIDTH{1'b0}},
                             shp_dm_lvl_i,
                             shp_svc_lvl_i,
                             {LVL_WIDTH{1'b0}},
                             {LVL_WIDTH{1'b0}},
                             {LVL_WIDTH{1'b0}},
                             {LVL_WIDTH{1'b0}},
                             shp_uf_lvl_i,
                             shp_bf_lvl_i,
                             shp_mm_lvl_i,
                             {LVL_WIDTH{1'b0}},
                             {LVL_WIDTH{1'b0}},
                             {LVL_WIDTH{1'b0}},
                             int_basepri_lvl };

  assign ints_lvl        = int_ints_lvl[MAX_WIDTH-1:0];

  assign local_fault_sel = { (dpu_svc_i),                           // SVCall
                             (~dbg_dbgen_i & dpu_bkpt_i),           // DbgMon Fault
                             (|dpu_nxt_ufsr_i) | int_non_actv_exit, // Usage Fault
                             (int_allow_bf &                        // BusFault (precise)
                              ((|dpu_nxt_bfsr_i[4:3]) | (|dpu_nxt_bfsr_i[1:0]))),
                             ((|dpu_nxt_mfsr_i[3:0])) };            // MemManage


  assign fault_raised    = (|local_fault_sel);

  assign fault_lvl       = ( ({LVL_WIDTH{local_fault_sel[0]}} & shp_mm_lvl_i) |
                             ({LVL_WIDTH{local_fault_sel[1]}} & shp_bf_lvl_i) |
                             ({LVL_WIDTH{local_fault_sel[2]}} & shp_uf_lvl_i) |
                             ({LVL_WIDTH{local_fault_sel[3]}} & shp_dm_lvl_i) |
                             ({LVL_WIDTH{local_fault_sel[4]}} & shp_svc_lvl_i));

  assign fault_comp_en   = ((local_fault_sel[0] & shcs_mm_en_i) |
                            (local_fault_sel[1] & shcs_bf_en_i) |
                            (local_fault_sel[2] & shcs_uf_en_i) |
                            (local_fault_sel[3] & demc_mon_en_i) |
                            (local_fault_sel[4]));


  assign imp_bus_fault_esc  = ((mtx_wb_abort_i | dpu_nxt_bfsr_i[2]) &
                               ~shcs_bf_en_i);

  assign fault_hi_pri_stack = (a_lvl_lt_b_lvl(fault_lvl,int_stack_lvl,prem_mask) |
                               (~int_stk_actv & int_no_basepri));

  assign fault_hi_pri_curr  = (a_lvl_lt_b_lvl(fault_lvl,curr_set_lvl,prem_mask) |
                               (int_no_basepri & core_at_base) |
                               use_curr_void | in_pop);

  assign fault_hi_pri_base  = (int_no_basepri |
                               a_lvl_lt_b_lvl(fault_lvl,opt_int_basepri,prem_mask));

  assign always_escalate    = int_pmask | int_fmask | ~fault_comp_en;

  assign norm_esc           = (// A fault has been raised but..
                               fault_raised &
                               (// int_pmask/int_fmask is set or fault not enabled
                                always_escalate |
                                ~fault_hi_pri_stack |
                                ~fault_hi_pri_base |
                                ~fault_hi_pri_curr));

  assign int_escalate       = imp_bus_fault_esc | norm_esc;
  assign int_lazy_esc       = (dpu_lazy_stk_i &
                               ((dpu_nxt_bfsr_i[5] & ~fpccr_i[6]) |
                                (dpu_nxt_mfsr_i[4] & ~fpccr_i[5])));

  assign lazy_mm            = fpccr_i[5] & dpu_nxt_mfsr_i[4];
  assign lazy_bf            = fpccr_i[6] & dpu_nxt_bfsr_i[5];
  assign lazy_hf            = ((fpccr_i[4] | ~(|int_in_nmihf[1:0])) &
                               ((dpu_nxt_mfsr_i[4] & ~fpccr_i[5]) |
                                (dpu_nxt_bfsr_i[5] & ~fpccr_i[6])));
  assign lazy_dm            = (dpu_lazy_stk_i & fpccr_i[8] & dwt_watchpoint_i &
                               ~dbg_dbgen_i);

  assign int_hfault         = ((~(|int_in_nmihf[1:0]) &
                                (int_escalate | dpu_nxt_hfsr_i)) | lazy_hf);
  assign mem_manage         = (local_fault_sel[0] & ~norm_esc) | lazy_mm;
  assign bus_fault          = (local_fault_sel[1] & ~norm_esc) | lazy_bf;
  assign usage_fault        = (local_fault_sel[2] & ~norm_esc);
  assign dbg_mon_non_esc    = (local_fault_sel[3] & ~norm_esc) | lazy_dm;
  assign sv_call            = (local_fault_sel[4] & ~norm_esc);

  assign pend_sv            = int_pend_set_i[7];
  assign non_esc_bus_f      = shcs_bf_en_i & (mtx_wb_abort_i | dpu_nxt_bfsr_i[2]);

  assign dbg_mon_hi_pri_curr = (a_lvl_lt_b_lvl(shp_dm_lvl_i,curr_set_lvl,prem_mask) |
                                (int_no_basepri & core_at_base) |
                                use_curr_void);

  assign dbg_mon_hi_pri_stack = (a_lvl_lt_b_lvl(shp_dm_lvl_i,int_stack_lvl,prem_mask) |
                                 (~int_stk_actv & int_no_basepri));

  assign dbg_mon_hi_pri_invk  = (a_lvl_lt_b_lvl(shp_dm_lvl_i,int_nxt_lvl,prem_mask) &
                                 int_invoke & (int_nxt_isr!=8'h0C));

  assign dbg_mon_int_allow    = (demc_mon_en_i & ~dhcs_c_debugen_i &
                                 ~(|int_in_nmihf[1:0]) & dbg_mon_hi_pri_stack &
                                 (dbg_mon_hi_pri_curr | in_pop));

  assign dbg_mon_ext_allow    = (demc_mon_en_i & ~dbg_dbgen_i &
                                 ~(|int_in_nmihf[1:0]) & dbg_mon_hi_pri_stack &
                                 (dbg_mon_hi_pri_curr | in_pop));

  assign dbg_mon_allow        = dbg_mon_ext_allow & ~int_pmask;

  assign dbg_mon_exit         = (demc_mon_en_i & ~dhcs_c_debugen_i &
                                 (int_fmask_clr | ~int_fmask) &
                                 opt_wic_en & dbg_mon_hi_pri_stack);

  assign asynch_d_mon         = (dbg_mon_allow &
                                 ((dwt_watchpoint_i & ~dpu_lazy_stk_i) |
                                  ext_dbg_req_i));

  assign dbg_mon_step_pend    = (dbg_mon_allow & dbg_mon_req_i &
                                 (dpu_retire_i |
                                  (dpu_int_entry_i & int_nxt_isr!=8'h0C)));

  assign dbg_mon_fault        = (dbg_mon_non_esc | dbg_mon_pend_i |
                                 asynch_d_mon    | dbg_mon_step_pend);

  assign dbg_wake_up          = (dbg_mon_ext_allow & int_pmask &
                                 ((dbg_mon_req_i &
                                   (dpu_retire_i | dpu_int_entry_i)) |
                                  dwt_watchpoint_i | ext_dbg_req_i));


  assign f_pending_we = int_hfault | int_isr_entry[`NVIC_HDFNUM];

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      fault_pending <= 1'b0;
    else if (f_pending_we)
      fault_pending <= ~int_isr_entry[`NVIC_HDFNUM];

  assign hfault_raised = fault_pending | int_hfault;

  assign int_pend_s = { ext_irq_i, stick_int_i, pend_sv, dbg_mon_fault, sv_call,
                        usage_fault, bus_fault | non_esc_bus_f, mem_manage,
                        hfault_raised, ext_nmi_i };


  assign opt_int_actv_set = (int_actv_set_i |
                             { {NUM_IRQ{1'b0}}, 8'b00000000, nmi_nvic_dpu_lockup});

  cm3_nvic_int_state #(NUM_INTS)
    u_cm3_nvic_int_state
      (// Inputs
       .hreset_n                   (hreset_n),
       .fclk                       (fclk),
       .hclk                       (hclk),
       .int_pend_set_i             (int_pend_set_i),
       .int_pend_clr_i             (int_pend_clr_i),
       .int_actv_set_i             (opt_int_actv_set),
       .int_actv_clr_i             (int_actv_clr_i),
       .int_pend_i                 (int_pend_s),
       .dpu_int_entry_i            (dpu_int_entry_i),
       .isr_entry_i                (int_isr_entry),
       .dpu_int_exit_i             (dpu_int_exit_i),
       .isr_exit_i                 (int_isr_exit),
       .pend_state_o               (int_pend_v),
       .actv_state_o               (int_actv_v),
       .slp_wfe_pend_o             (slp_wfe_pend)
       );

  assign int_actv = (|int_actv_v);


  cm3_nvic_preempt #(WIC_PRESENT, WIC_LINES, NUM_IRQ, LVL_WIDTH, IRQ_WIDTH, NUM_INTS)
    u_cm3_nvic_preempt
      (// Inputs
       .fclk                       (fclk),
       .hreset_n                   (hreset_n),
       .irq_en_i                   (irq_en_i),
       .irq_lvl_i                  (irq_lvl_i),
       .int_pend_v_i               (int_pend_v),
       .int_actv_v_i               (int_actv_v),
       .nmi_active_i               (nmi_active),
       .curr_set_lvl_i             (curr_set_lvl),
       .core_at_base_i             (core_at_base),
       .prem_mask_i                (prem_mask),
       .int_basepri_i              (opt_int_basepri),
       .int_no_basepri_i           (int_no_basepri),
       .int_fmask_i                (int_fmask),
       .int_fmask_clr_i            (int_fmask_clr),
       .int_pmask_i                (int_pmask),
       .int_stk_actv_i             (int_stk_actv),
       .dpu_int_exit_i             (dpu_int_exit_i),
       .scr_wfeonpend_i            (scr_wfeonpend_i),
       .scr_deepsleep_i            (scr_deepsleep_i),
       .slp_wfe_valid_i            (slp_wfe_valid_i),
       .slp_wfi_valid_i            (slp_wfi_valid_i),
       .slp_asleep_i               (slp_asleep_i),
       .wic_ds_req_n_i             (wic_ds_req_n_i),
       .dbg_mon_int_allow_i        (dbg_mon_int_allow),
       .dbg_mon_exit_i             (dbg_mon_exit),
       .wic_ds_ack_n_o             (wic_ds_ack_n),
       .wic_mask_o                 (wic_mask));

  assign shcs_status = { int_pend_v[`NVIC_SVCNUM], int_pend_v[`NVIC_BEFNUM],
                         int_pend_v[`NVIC_MEMNUM], int_pend_v[`NVIC_USGNUM],
                         int_actv_v[`NVIC_TCKNUM], int_actv_v[`NVIC_PSVNUM],
                         1'b0,                     int_actv_v[`NVIC_DBMNUM],
                         int_actv_v[`NVIC_SVCNUM], 3'b000,
                         int_actv_v[`NVIC_USGNUM], 1'b0,
                         int_actv_v[`NVIC_BEFNUM], int_actv_v[`NVIC_MEMNUM] };


  assign int_en_tmp = { {(SPARE+1){1'b0}},                       // Padding
                        irq_en_i & {NUM_IRQ{~dbg_mask_ints_i}},  // IRQ
                        ~dbg_mask_ints_i,                        // SysTick
                        ~dbg_mask_ints_i,                        // PendSV
                        1'b0,                                    // Rsvd
                        ~dbg_mask_ints_i,                        // DbgMonitor
                        1'b1,                                    // SVCall
                        4'b0000,                                 // Rsvd
                        shcs_uf_en_i,                            // UsageFault
                        shcs_bf_en_i,                            // BusFault
                        shcs_mm_en_i,                            // MemManage
                        1'b1,                                    // HardFault
                        1'b1,                                    // NMI
                        1'b0,                                    // Rsvd
                        1'b1 };                                  // Reset

  assign int_en     = int_en_tmp[`max_ints-1:0];

  assign raw_ints_pend = ({{SPARE+1{1'b0}},
                           int_pend_v[`NVIC_IRQBASE+NUM_IRQ-1:`NVIC_IRQBASE],
                           int_pend_v[`NVIC_TCKNUM],
                           int_pend_v[`NVIC_PSVNUM],
                           1'b0,
                           int_pend_v[`NVIC_DBMNUM],
                           int_pend_v[`NVIC_SVCNUM],
                           1'b0,
                           1'b0,
                           1'b0,
                           1'b0,
                           int_pend_v[`NVIC_USGNUM],
                           int_pend_v[`NVIC_BEFNUM],
                           int_pend_v[`NVIC_MEMNUM],
                           int_pend_v[`NVIC_HDFNUM],
                           int_pend_v[`NVIC_NMINUM],
                           1'b0,
                           1'b0 });

  assign ints_pend     = raw_ints_pend[`max_ints-1:0] & int_en;


  cm3_nvic_tree #(LVL_WIDTH, MAX_WIDTH)
    u_cm3_nvic_pend_tree
      (// Inputs
       .int_state_i                (ints_pend[`max_ints-1:0]),
       .int_lvl_i                  (ints_lvl[MAX_WIDTH-1:0]),
       .int_num_o                  (int_pend_num),
       .int_hi_lvl_o               (int_pend_lvl)
       );

  assign int_pend      = (|ints_pend[`max_ints-1:0]);


  assign ints_actv     = ({{SPARE+1{1'b0}},
                           int_actv_v[`NVIC_IRQBASE+NUM_IRQ-1:`NVIC_IRQBASE],
                           int_actv_v[`NVIC_TCKNUM],
                           int_actv_v[`NVIC_PSVNUM],
                           1'b0,
                           int_actv_v[`NVIC_DBMNUM],
                           int_actv_v[`NVIC_SVCNUM],
                           1'b0,
                           1'b0,
                           1'b0,
                           1'b0,
                           int_actv_v[`NVIC_USGNUM],
                           int_actv_v[`NVIC_BEFNUM],
                           int_actv_v[`NVIC_MEMNUM],
                           int_actv_v[`NVIC_HDFNUM],
                           int_actv_v[`NVIC_NMINUM],
                           1'b0,
                           (|opt_int_basepri) });

  assign mask_ints_actv = ints_actv[`max_ints-1:0] & ~curr_select;


  cm3_nvic_tree #(LVL_WIDTH, MAX_WIDTH)
    u_cm3_nvic_actv_tree
      (// Inputs
       .int_state_i                (mask_ints_actv[`max_ints-1:0]),
       .int_lvl_i                  (ints_lvl[MAX_WIDTH-1:0]),
       .int_num_o                  (),
       .int_hi_lvl_o               (int_stack_lvl)
       );

  assign int_stk_actv        = (|mask_ints_actv[`max_ints-1:1]);

  assign nxt_int_stk_actv    = ~core_at_base & dpu_int_entry_i;

  assign nxt_int_ret_to_base = ~core_at_base & ~int_stk_actv;

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      int_ret_to_base <= 1'b0;
    else
      int_ret_to_base <= nxt_int_ret_to_base;


  always @ (ints_lvl or int_curr_isr)
    for (l2 = 0; l2 < LVL_WIDTH; l2 = l2 + 1)
      curr_set_lvl[l2] = ints_lvl[(LVL_WIDTH*int_curr_isr[7:0])+l2];

  assign int_int_curr_pri = {{(9 - LVL_WIDTH){1'b0}}, curr_set_lvl } << (8 - LVL_WIDTH);
  assign int_curr_pri     = int_int_curr_pri[7:0];


  assign prem_pend_hi_pri_curr  = (// NMI over HFault and settable
                                   (nmi_pending & ~nmi_active) |
                                   (hfault_pending & ~hfault_active & ~nmi_active) |
                                   (// Settable level comparison
                                    (a_lvl_lt_b_lvl(int_pend_lvl,curr_set_lvl,prem_mask) |
                                     (int_no_basepri & core_at_base)) &
                                   ~(|int_in_nmihf)));

  assign pend_lt_stack          = a_lvl_lt_b_lvl(int_pend_lvl,int_stack_lvl,prem_mask);

  assign prem_pend_hi_pri_stack = ((~int_stk_actv & int_no_basepri &
                                    ~((nmi_active | in_pop) & int_fmask)) |
                                   (nmi_pending) |
                                   (hfault_pending & ~hfault_active) |
                                   (pend_lt_stack &
                                    ~(hfault_active & (nmi_active | in_pop))));

  assign prem_pend_hi_pri_invk  = (// NMI over HFault and settable
                                   (nmi_pending) |
                                   (hfault_pending &
                                    ~(nmi_pending |
                                      (int_invoke & int_nxt_isr == 8'b00000010))) |
                                   (int_pend &
                                    a_lvl_lt_b_lvl(int_pend_lvl,int_nxt_lvl,prem_mask)));

  assign int_pmask_disabled     = (|int_pend_num[7:2]) & int_pmask;


  assign will_pre_empt          = (int_pend & prem_pend_hi_pri_stack &
                                   (prem_pend_hi_pri_curr |
                                    (use_curr_void & ~in_pop) |
                                    (in_pop & ~nmi_active &
                                     (~int_fmask | nmi_pending)) |
                                    (dpu_int_exit_i & prem_pend_hi_pri_curr)));

  assign nxt_slp_pmask_pend     = will_pre_empt | dbg_wake_up;

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      slp_pmask_pend <= 1'b0;
    else
      slp_pmask_pend <= nxt_slp_pmask_pend;

  assign preempt_invoke  = (will_pre_empt & ~int_pmask_disabled &
                            ~dpu_ahb_invk_msk_i &
                            ~(dpu_mcyc_invk_msk_i & ac_dismcycint_i));

  assign masked_p_invoke = preempt_invoke & slp_hold_ack_n_i;

  assign tail_invoke     = int_tail & dpu_int_exit_i;

  assign set_invoke      = tail_invoke | masked_p_invoke;

  assign clr_invoke      = ((dpu_int_entry_i & ~int_nxt_isr_we)                     |
                            (dbg_mask_ints_i &  int_nxt_isr_we & int_pend_num > 13) |
                            (dbg_mask_ints_i & ~int_nxt_isr_we & int_nxt_isr  > 13));

  assign nxt_int_invoke  = (set_invoke | int_invoke) & ~clr_invoke;

  assign nxt_int_tail    = (int_pend & ~int_pmask_disabled & int_actv &
                            ~use_curr_void & prem_pend_hi_pri_stack &
                            ~preempt_invoke & ~int_invoke & ~core_at_base &
                            ~dpu_int_exit_i & ~in_pop & ~clr_invoke &
                            slp_hold_ack_n_i & ~dpu_ahb_invk_msk_i);


  assign int_nxt_isr_we  = ((int_nxt_isr != int_pend_num) &
                            ~(dpu_int_exit_i & int_tail) &
                            (~int_invoke | prem_pend_hi_pri_invk));

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      int_nxt_isr <= {8{1'b0}};
    else if (int_nxt_isr_we)
      int_nxt_isr <= int_pend_num;

  assign int_nxt_lvl_we = (int_nxt_isr_we |
                           ((int_nxt_isr == int_pend_num) &
                            (int_nxt_lvl != int_pend_lvl)));

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      int_nxt_lvl <= {LVL_WIDTH{1'b0}};
    else if (int_nxt_lvl_we)
      int_nxt_lvl <= int_pend_lvl;

  always @ (dpu_int_entry_i or int_nxt_isr)
    for (l0 = 0; l0 < `max_ints; l0 = l0 + 1)
      isr_entry[l0] = (int_nxt_isr == l0) & dpu_int_entry_i;

  assign int_isr_entry = { isr_entry[NUM_IRQ+15:14],
                           isr_entry[12:11],
                           isr_entry[6:2]};

  assign isr_exit              = (curr_select &
                                  {`max_ints{dpu_int_exit_i & int_actv}});

  assign int_isr_exit          = { isr_exit[NUM_IRQ+15:14],
                                   isr_exit[12:11],
                                   isr_exit[6:2] };

  assign nxt_int_non_actv_exit = dpu_int_exit_i & curr_void;
  assign int_non_actv_exit_we  = dpu_int_exit_i | int_non_actv_exit;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      int_non_actv_exit <= 1'b0;
    else if (int_non_actv_exit_we)
      int_non_actv_exit <= nxt_int_non_actv_exit;

  assign nxt_int_curr_isr = (vect_clr_actv_i ? 9'b000000000 :
                             (dpu_int_entry_i ? { 1'b0, int_nxt_isr } :
                              ((dpu_int_return_i | dpu_dbg_wr_isr_i) ?
                               dpu_int_retisr_i : int_curr_isr)));

  assign int_curr_isr_we  = (vect_clr_actv_i | dpu_int_entry_i |
                             dpu_int_return_i | dpu_dbg_wr_isr_i);

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      int_curr_isr <= 9'b000000000;
    else if (int_curr_isr_we)
      int_curr_isr <= nxt_int_curr_isr;

  assign core_at_base     = ~(|int_curr_isr);

  always @ (int_curr_isr)
    for (l1 = 0; l1 < `max_ints; l1 = l1 + 1)
      curr_select[l1] = (int_curr_isr[8:0] == l1);

  assign curr_void        = (~core_at_base &
                             (~ints_actv[int_curr_isr[7:0]] | int_curr_isr[8]));
  assign use_curr_void    = curr_void & ~hfault_active & ~nmi_active;

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      begin
        int_invoke    <= 1'b0;
        int_tail      <= 1'b0;
        slp_norm_wake <= 1'b0;
      end
    else
      begin
        int_invoke    <= nxt_int_invoke;
        int_tail      <= nxt_int_tail;
        slp_norm_wake <= preempt_invoke;
      end

  assign int_in_nmihf        =  { hfault_pending |
                                  nmi_pending    |
                                  (int_nxt_isr == 8'h02),
                                  hfault_active,
                                  nmi_active };

  assign int_mpu_nmihf       = (dpu_lazy_stk_i ? ~fpccr_i[4] :
                                                 (hfault_active | nmi_active));

  assign nmi_nvic_dpu_lockup = hfault_active & in_pop & dpu_nxt_ufsr_i[2];


  assign nxt_force_int_addr  = (ext_vect_addr_en_i &
                                (int_nxt_isr[7:3] == 5'b00010) &
                                int_invoke & ~int_nxt_isr_we);

  assign nxt_force_int_pulse = nxt_force_int_addr & ~force_int_addr;

  assign force_addr_we       = nxt_force_int_addr ^ force_int_addr;
  assign force_pulse_we      = nxt_force_int_addr | force_int_pulse;

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      force_int_addr <= 1'b0;
    else if (force_addr_we)
      force_int_addr <= nxt_force_int_addr;

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      force_int_pulse <= 1'b0;
    else if (force_pulse_we)
      force_int_pulse <= nxt_force_int_pulse;

  assign lower_int_addr      = ((force_int_addr & ~force_int_pulse) ?
                                ext_vect_addr_i[9:2] : int_nxt_isr);

  assign int_addr            = { vto_tbloff_i[31:10],
                                 ({ vto_tbloff_i[9:7], 5'b00000 } |
                                  lower_int_addr),
                                 2'b00 };

  assign nmi_pending         = int_pend_v[`NVIC_NMINUM];
  assign hfault_pending      = int_pend_v[`NVIC_HDFNUM];
  assign psv_pending         = int_pend_v[`NVIC_PSVNUM];
  assign dbg_mon_pending     = int_pend_v[`NVIC_DBMNUM];
  assign dbg_mon_active      = int_actv_v[`NVIC_DBMNUM];
  assign st_pending          = int_pend_v[`NVIC_TCKNUM];
  assign irq_pending         = int_pend_v[`NVIC_IRQBASE+NUM_IRQ-1:`NVIC_IRQBASE];
  assign nmi_active          = int_actv_v[`NVIC_NMINUM];
  assign hfault_active       = int_actv_v[`NVIC_HDFNUM] | int_fmask;
  assign irq_active          = int_actv_v[`NVIC_IRQBASE+NUM_IRQ-1:`NVIC_IRQBASE];
  assign int_hipend_isr      = int_nxt_isr;

  assign set_in_pop          = dpu_int_exit_i & ~int_invoke;

  assign clr_in_pop          = dpu_int_return_i | set_invoke;

  assign in_pop_we           = set_in_pop | clr_in_pop;

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      in_pop <= 1'b0;
    else if (in_pop_we)
      in_pop <= ~clr_in_pop;

  assign nxt_int_new_arrival = ((masked_p_invoke & ~int_invoke & ~clr_invoke) |
                                (int_nxt_isr_we & int_invoke)                 |
                                (tail_invoke & int_nxt_isr_we)                |
                                force_int_pulse);

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      int_new_arrival <= 1'b0;
    else
      int_new_arrival <= nxt_int_new_arrival;


  assign int_mask_en         = ((dpu_raisepri_i & ~(set_invoke | int_invoke)) |
                                ~dpu_raisepri_i);

  assign int_basepri_we      = dpu_basepri_we_i & int_mask_en;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      reg_int_basepri <= {LVL_WIDTH{1'b0}};
    else if (int_basepri_we)
      reg_int_basepri <= dpu_basepri_nxt_i[7:8-LVL_WIDTH];

`ifdef ARM_DSM
  assign opt_int_basepri     = reg_int_basepri & dsm_pmask;
`else
  assign opt_int_basepri     = reg_int_basepri;
`endif

  assign pad_int_basepri     = {{(9 - LVL_WIDTH){1'b0}}, opt_int_basepri } << (8 - LVL_WIDTH);
  assign int_basepri         = pad_int_basepri[7:0];


  assign int_fmask_clr       = (int_fmask & dpu_int_exit_i &
                                ((|int_curr_isr[8:2]) | int_curr_isr[0]));

  assign fmask_wr_valid      = dpu_fmask_we_i & int_mask_en;
  assign int_fmask_we        = fmask_wr_valid | int_fmask_clr;
  assign nxt_int_fmask       = dpu_fmask_nxt_i & ~int_fmask_clr;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      int_fmask <= 1'b0;
    else if (int_fmask_we)
      int_fmask <= nxt_int_fmask;

  assign int_pmask_we        = dpu_pmask_we_i & int_mask_en;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      int_pmask <= 1'b0;
    else if (int_pmask_we)
      int_pmask <= dpu_pmask_nxt_i;

  assign dbg_masked_bkpt     = dpu_bkpt_i & (~preempt_invoke | ~dbg_dbgen_i);


  assign int_pmask_sleep     = (int_pend & int_pmask &
                                (~int_fmask | ~nmi_active) &
                                (int_no_basepri |
                                 pend_lt_stack));

  assign nxt_slp_sleeponexit = (scr_sleeponexit_i &
                                ~(nxt_int_invoke  | nxt_int_tail |
                                  int_pmask_sleep | int_stk_actv |
                                  nxt_int_stk_actv));

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      slp_sleeponexit <= 1'b0;
    else
      slp_sleeponexit <= nxt_slp_sleeponexit;



  assign bf_hi_pri_stack   = (a_lvl_lt_b_lvl(shp_bf_lvl_i,int_stack_lvl,prem_mask) |
                              (~int_stk_actv & int_no_basepri));

  assign bf_hi_pri_curr    = (a_lvl_lt_b_lvl(shp_bf_lvl_i,curr_set_lvl,prem_mask) |
                              (int_no_basepri & core_at_base) |
                              use_curr_void |
                              in_pop);

  assign bf_hi_pri_base    = (a_lvl_lt_b_lvl(shp_bf_lvl_i,opt_int_basepri,prem_mask) |
                              int_no_basepri);


  assign mm_hi_pri_stack   = (a_lvl_lt_b_lvl(shp_mm_lvl_i,int_stack_lvl,prem_mask) |
                              (~int_stk_actv & int_no_basepri));

  assign mm_hi_pri_curr    = (a_lvl_lt_b_lvl(shp_mm_lvl_i,curr_set_lvl,prem_mask) |
                              (int_no_basepri & core_at_base) |
                              use_curr_void |
                              in_pop);

  assign mm_hi_pri_base    = (a_lvl_lt_b_lvl(shp_mm_lvl_i,opt_int_basepri,prem_mask) |
                              int_no_basepri);


  assign int_lazy_fpccr[0] = ~(|int_in_nmihf[1:0]);

  assign int_lazy_fpccr[1] = (shcs_mm_en_i & ~int_pmask & ~(|int_in_nmihf[1:0]) &
                              mm_hi_pri_stack & mm_hi_pri_curr & mm_hi_pri_base);

  assign int_lazy_fpccr[2] = (int_allow_bf & shcs_bf_en_i & ~int_pmask &
                              ~(|int_in_nmihf[1:0]) & bf_hi_pri_stack &
                              bf_hi_pri_curr & bf_hi_pri_base);

  assign int_lazy_fpccr[3] = dbg_mon_allow;

  assign irq_pending_o         = irq_pending;
  assign nmi_pending_o         = nmi_pending;
  assign psv_pending_o         = psv_pending;
  assign st_pending_o          = st_pending;
  assign irq_active_o          = irq_active;
  assign int_hfault_o          = int_hfault;
  assign int_escalate_o        = int_escalate;
  assign int_lazy_esc_o        = int_lazy_esc;
  assign int_invoke_o          = int_invoke;
  assign int_new_arrival_o     = int_new_arrival;
  assign int_addr_o            = int_addr;
  assign int_tail_o            = int_tail;
  assign int_curr_isr_o        = int_curr_isr;
  assign int_curr_pri_o        = int_curr_pri;
  assign int_hipend_isr_o      = int_hipend_isr;
  assign int_nxt_isr_o         = int_nxt_isr;
  assign int_stk_actv_o        = int_stk_actv;
  assign int_ret_to_base_o     = int_ret_to_base;
  assign int_non_actv_exit_o   = int_non_actv_exit;
  assign int_in_nmihf_o        = int_in_nmihf;
  assign int_mpu_nmihf_o       = int_mpu_nmihf;
  assign int_allow_bf_o        = int_allow_bf;
  assign int_lazy_fpccr_o      = int_lazy_fpccr & {4{opt_cp_en}};
  assign int_lazy_dm_o         = lazy_dm & opt_cp_en;
  assign int_basepri_o         = int_basepri;
  assign int_fmask_o           = int_fmask;
  assign int_pmask_o           = int_pmask;
  assign shcs_status_o         = shcs_status;
  assign slp_pmask_pend_o      = slp_pmask_pend;
  assign slp_wfe_pend_o        = slp_wfe_pend;
  assign slp_sleeponexit_o     = slp_sleeponexit;
  assign slp_norm_wake_o       = slp_norm_wake;
  assign wic_ds_ack_n_o        = opt_wic_ds_ack_n;
  assign wic_mask_o            = opt_wic_mask;
  assign dbg_masked_bkpt_o     = dbg_masked_bkpt;
  assign dbg_mon_fault_o       = dbg_mon_fault;
  assign dbg_mon_pending_o     = dbg_mon_pending;
  assign dbg_mon_active_o      = dbg_mon_active;
  assign dbg_mon_allow_o       = dbg_mon_allow;
  assign dbg_mon_step_pend_o   = dbg_mon_step_pend;
  assign dbg_mon_hi_pri_invk_o = dbg_mon_hi_pri_invk;


`ifdef ARM_ASSERT_ON

  reg                       ovl_int_exit_q;
  reg                       ovl_int_return_q;
  reg                       ovl_int_curr_isr_valid;
  reg                       ovl_int_curr_isr_valid_q;
  reg                       ovl_int_entry_q;
  reg                       ovl_int_invoke_q;
  reg                       ovl_int_tail_q;

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      ovl_int_curr_isr_valid <= 1'b0;
    else if (dpu_int_entry_i | (dpu_int_return_i & int_actv))
      ovl_int_curr_isr_valid <= 1'b1;
    else if (dpu_int_exit_i)
      ovl_int_curr_isr_valid <= 1'b0;

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      begin
        ovl_int_entry_q          <= 1'b0;
        ovl_int_exit_q           <= 1'b0;
        ovl_int_return_q         <= 1'b0;
        ovl_int_curr_isr_valid_q <= 1'b0;
        ovl_int_invoke_q         <= 1'b0;
        ovl_int_tail_q           <= 1'b0;
      end
    else
      begin
        ovl_int_entry_q          <= dpu_int_entry_i;
        ovl_int_exit_q           <= dpu_int_exit_i;
        ovl_int_return_q         <= dpu_int_return_i;
        ovl_int_curr_isr_valid_q <= ovl_int_curr_isr_valid;
        ovl_int_invoke_q         <= int_invoke;
        ovl_int_tail_q           <= int_tail;
      end

  assert_never #(0,0,"dpu_int_entry_i must be pulsed.")
    ovl_nvic_int_entry(fclk, hreset_n, dpu_int_entry_i & ovl_int_entry_q);

  assert_never #(0,0,"dpu_int_exit_i must be pulsed.")
    ovl_nvic_int_exit(fclk, hreset_n, dpu_int_exit_i & ovl_int_exit_q);

  assert_never #(0,0,"dpu_int_return_i must be pulsed.")
    ovl_nvic_int_return(fclk, hreset_n, dpu_int_return_i & ovl_int_return_q);

  assert_never #(0,0,"int_pend_num is out of range.")
    ovl_int_pend_num_check(fclk, hreset_n, int_pend_num > (NUM_INTS + `num_res_rs));

  assert_zero_one_hot #(0,5,0,"Local Faults must be exclusive.")
    ovl_local_fault_sel_check(fclk, hreset_n, local_fault_sel);

  assert_never
  #(0,0, "int_new_arrival must be set in 1st cycle of invoke if not tailing")
    ovl_new_arrival_check(hclk, hreset_n, int_invoke & ~ovl_int_tail_q &
                          ~ovl_int_invoke_q & ~int_new_arrival);

  assert_never #(0,0,"dpu_int_entry_i must not be assrted if int_new_arrival is asserted")
    ovl_new_arrival_check2(hclk, hreset_n, dpu_int_entry_i & int_new_arrival);

  assert_never #(0,0,"Handler entry cannot be to 0 or 1.")
    ovl_entry_check(hclk, hreset_n, dpu_int_entry_i & ~(|int_nxt_isr[7:1]));

  assert_never #(0,0,"dpu_int_entry_i must not be assrted if int_invoke is not asserted")
    ovl_entry_check2(hclk, hreset_n, dpu_int_entry_i & ~int_invoke);

  assert_never #(0,0,"int_new_arrival must not be assrted if int_invoke is not asserted")
    ovl_new_arrival_check3(hclk, hreset_n, int_new_arrival & ~int_invoke);

`endif

endmodule

`include "cm3_nvic_undefs.vp"
