`include "cm3_nvic_defs.vp"

module cm3_nvic_ppb_intf
  (// Inputs
   dreset_n, hclk, mtx_ppb_haddr_i, mtx_ppb_hprot_i, mtx_ppb_hsel_nvic_i,
   mtx_ppb_hmaster_i, mtx_ppb_hsize_i, mtx_ppb_hwrite_i, mtx_ppb_hready_i,
   data_be8_i, ext_cp_disable_i, irq_en_i, shcs_mm_en_i, shcs_bf_en_i,
   shcs_uf_en_i, irq_lvl_i, shp_mm_lvl_i, shp_bf_lvl_i, shp_uf_lvl_i,
   shp_dm_lvl_i, shp_svc_lvl_i, shp_psv_lvl_i, shp_st_lvl_i, irq_pending_i,
   psv_pending_i, st_pending_i, dbg_mon_pending_i, nmi_pending_i, irq_active_i,
   shcs_status_i, int_hipend_isr_i, int_curr_isr_i, int_invoke_i,
   int_ret_to_base_i, stick_cnt_flag_i, stick_clk_src_i, stick_int_en_i,
   stick_en_i, stick_reload_i, stick_count_i, stick_no_ref_i, stick_skew_i,
   stick_one_ms_i, fsr_i, hfsr_i, dfsr_i, afsr_i, mm_bf_far_i, aux_ctrl_i,
   vto_tbloff_i, cpacr_i, fpccr_i, fpcar_i, fpdscr_i, airc_prigroup_i,
   ccr_div_0_trp_i, ccr_unalign_trp_i, ccr_bfhfnmign_i, ccr_stkalign_i,
   ccr_nonbasethrdena_i, ccr_usersetmpend_i, scr_sleeponexit_i,
   scr_deepsleep_i, scr_wfeonpend_i, slp_asleep_i, lockup_i, demc_mon_en_i,
   demc_mon_step_i, demc_mon_req_i, demc_trcena_i, dhcs_s_reset_st_i,
   dhcs_s_retire_st_i, dhcs_c_snapstall_i, dhcs_c_mask_ints_i, dhcs_c_halt_i,
   dhcs_c_step_i, dhcs_c_debugen_i, dcrs_rtrans_i, dbg_vcatch_i, dbg_dcrd_i,
   dpu_dbg_halted_i, dpu_dbg_ready_i,
   ppb_hrdata_o, ppb_hready_o, ppb_hresp_o, ppb_we_o, ppb_rd_o,
   ppb_hmaster_reg_o, ppb_reg_addr_o
   );

  parameter NUM_IRQ         = 16;                   // Number of interrupts
  parameter LVL_WIDTH       = 3;                    // Interrupt priority width
  parameter DEBUG_LVL       = 3;                    // Debug level
  parameter EXT_PRESENT     = 0;                    // Extensions present if 1
  parameter CP_PRESENT      = 0;                    // CP present if 1
  parameter IRQ_WIDTH       = NUM_IRQ * LVL_WIDTH;  // Total priority width

  input                     dreset_n;               // Debug reset
  input                     hclk;                   // NVIC Clock (clock gated)
  input [11:0]              mtx_ppb_haddr_i;        // PPB HADDR
  input [3:0]               mtx_ppb_hprot_i;        // PPB HPROT
  input                     mtx_ppb_hsel_nvic_i;    // PPB HSEL
  input                     mtx_ppb_hmaster_i;      // PPB HMASTER
  input [1:0]               mtx_ppb_hsize_i;        // PPB HSIZE
  input                     mtx_ppb_hwrite_i;       // PPB HWRITE
  input                     mtx_ppb_hready_i;       // PPB HREADYIN
  input                     data_be8_i;             // Big-endian byte invariant
  input                     ext_cp_disable_i;       // Disable the CP
  input [NUM_IRQ-1:0]       irq_en_i;               // IRQ enables
  input                     shcs_mm_en_i;           // MemManage fault enable
  input                     shcs_bf_en_i;           // Bus fault enable
  input                     shcs_uf_en_i;           // Usage fault enable
  input [IRQ_WIDTH-1:0]     irq_lvl_i;              // IRQ priority levels
  input [LVL_WIDTH-1:0]     shp_mm_lvl_i;           // MemManage Fault priority
  input [LVL_WIDTH-1:0]     shp_bf_lvl_i;           // Bus Fault priority
  input [LVL_WIDTH-1:0]     shp_uf_lvl_i;           // Usage Fault priority
  input [LVL_WIDTH-1:0]     shp_dm_lvl_i;           // Debug Monitor priority
  input [LVL_WIDTH-1:0]     shp_svc_lvl_i;          // SVC priority
  input [LVL_WIDTH-1:0]     shp_psv_lvl_i;          // Pend SVC priority
  input [LVL_WIDTH-1:0]     shp_st_lvl_i;           // System Tick priority
  input [NUM_IRQ-1:0]       irq_pending_i;          // IRQs that are pending
  input                     psv_pending_i;          // SVC from pend is pending
  input                     st_pending_i;           // SysTick is pending
  input                     dbg_mon_pending_i;      // Debug monitor pending
  input                     nmi_pending_i;          // NMI is pending
  input [NUM_IRQ-1:0]       irq_active_i;           // Interrupts that are active
  input [15:0]              shcs_status_i;          // System Handler status
  input [7:0]               int_hipend_isr_i;       // Highest pending vector number
  input [8:0]               int_curr_isr_i;         // Current active ISR number
  input                     int_invoke_i;           // Perform hardware stacking
  input                     int_ret_to_base_i;      // Return to base
  input                     stick_cnt_flag_i;       // SysTick count flag
  input                     stick_clk_src_i;        // SysTick clock source
  input                     stick_int_en_i;         // Allow pending of SysTick
  input                     stick_en_i;             // Enable SysTick counter reload
  input [23:0]              stick_reload_i;         // SysTick counter reload value
  input [23:0]              stick_count_i;          // SysTick counter value
  input                     stick_no_ref_i;         // No SysTick reference clock
  input                     stick_skew_i;           // SysTick skew not 10ms
  input [23:0]              stick_one_ms_i;         // SysTick 10ms timing
  input [31:0]              fsr_i;                  // Fault status register
  input [31:0]              hfsr_i;                 // HardFault status register
  input [31:0]              dfsr_i;                 // Debug fault status register
  input [31:0]              afsr_i;                 // Auxiliary fault status register
  input [31:0]              mm_bf_far_i;            // Fault address register
  input [4:0]               aux_ctrl_i;             // Auxiliary Control Reigster
  input [31:7]              vto_tbloff_i;           // Vector table offset
  input [3:0]               cpacr_i;                // CPACR
  input [31:0]              fpccr_i;                // FPCCR register
  input [31:0]              fpcar_i;                // FPCAR register
  input [4:0]               fpdscr_i;               // Default FPSCR register
  input [2:0]               airc_prigroup_i;        // Priority grouping position
  input                     ccr_div_0_trp_i;        // Divide by zero fault enable
  input                     ccr_unalign_trp_i;      // Unaligned fault enable
  input                     ccr_bfhfnmign_i;        // Ignore bus fault in NMI/HF
  input                     ccr_stkalign_i;         // Stack align
  input                     ccr_nonbasethrdena_i;   // Handler can be thread
  input                     ccr_usersetmpend_i;     // User can software trigger
  input                     scr_sleeponexit_i;      // Sleep on exit requested
  input                     scr_deepsleep_i;        // Deep sleep bit is set
  input                     scr_wfeonpend_i;        // Wake up WFE on new pends
  input                     slp_asleep_i;           // DPU is sleeping
  input [1:0]               lockup_i;               // Core is locked up
  input                     demc_mon_en_i;          // Debug monitor enable
  input                     demc_mon_step_i;        // Debug monitor step
  input                     demc_mon_req_i;         // Monitor requested
  input                     demc_trcena_i;          // Trace enable
  input                     dhcs_s_reset_st_i;      // Reset occured after read
  input                     dhcs_s_retire_st_i;     // Instr retired after read
  input                     dhcs_c_snapstall_i;     // Snap stalled transaction
  input                     dhcs_c_mask_ints_i;     // Mask interrupts for step
  input                     dhcs_c_halt_i;          // Halting control
  input                     dhcs_c_step_i;          // Halting step enable
  input                     dhcs_c_debugen_i;       // Halting debug enable
  input                     dcrs_rtrans_i;          // DBG register trans
  input [7:0]               dbg_vcatch_i;           // Vector catch
  input [31:0]              dbg_dcrd_i;             // Scratch debug register
  input                     dpu_dbg_halted_i;       // Core is halted
  input                     dpu_dbg_ready_i;        // Single cycle reply for debug

  output [31:0]             ppb_hrdata_o;           // PPB HRDATA
  output                    ppb_hready_o;           // PPB HREADYOUT
  output                    ppb_hresp_o;            // PPB HRESP
  output [3:0]              ppb_we_o;               // PPB data phase byte write enable
  output                    ppb_rd_o;               // PPB data phase read enable
  output [11:2]             ppb_reg_addr_o;         // Registered PPB HADDR
  output                    ppb_hmaster_reg_o;      // Registered PPB HMASTER


  reg                       ppb_hready;              // PPB HREADYOUT
  reg                       ppb_hresp;               // PPB HRESP
  reg [3:0]                 ppb_we;                  // PPB data phase byte write
  reg                       ppb_rd;                  // PPB data phase read enable
  reg [11:2]                ppb_reg_addr;            // Registered PPB HADDR
  reg                       ppb_hmaster_reg;         // Registered PPB HMASTER

  reg  [31:0]               ppb_hrdata;              // Read data mux
  wire [`max_ints-1:0]      pad_irq_en;              // irq_en padded
  wire [`max_ints-1:0]      irq_pending;             // irq_pending padded
  wire [`max_ints-1:0]      irq_active;              // irq_active padded
  reg  [(`max_ints*8)-1:0]  pad_irq_lvl;             // irq_lvl padded
  reg  [31:0]               pad_local_lvl_0;         // Local fault levels padded
  reg  [31:0]               pad_local_lvl_1;         // Local fault levels padded
  reg  [31:0]               pad_local_lvl_2;         // Local fault levels padded
  wire                      ppb_hready_we;           // ppb_hready write enable
  wire                      ppb_hresp_we;            // ppb_hresp write enable
  wire                      ppb_en;                  // PPB access enable
  wire                      ppb_trans;               // PPB transaction occurred
  wire                      next_ppb_rd;             // Next ppb_rd
  wire [3:0]                next_ppb_we;             // Byte lanes write mask
  wire                      priv_access;             // Privileged access
  wire [31:4]               cpu_id;                  // CPU ID value
  reg  [3:0]                rev_px_value;            // Revision value register
  wire                      mst_cp_en;               // Master CP enable
  wire                      opt_dbg_en;              // Removal of debug
  wire                      opt_ext_en;              // Removal of extensions
  wire                      opt_cp_en;               // Removal of CP

  integer                   l0;
  integer                   l1;

`ifdef ARM_DSM

  wire [7:0]                dsm_num_irq;
  wire [3:0]                dsm_lvl_width;
  wire [1:0]                dsm_debug_lvl;
  wire                      dsm_cp_present;

  assign opt_dbg_en = (dsm_debug_lvl    > 0) ? 1'b1 : 1'b0;

  assign opt_cp_en  = (dsm_cp_present  != 0) ? 1'b1 : 1'b0;

`else

  assign opt_dbg_en = (DEBUG_LVL > 0)        ? 1'b1 : 1'b0;

  assign opt_cp_en  = (CP_PRESENT != 0)      ? 1'b1 : 1'b0;

`endif

  assign opt_ext_en = (EXT_PRESENT != 0)     ? 1'b1 : 1'b0;

  assign mst_cp_en  = opt_cp_en & ~ext_cp_disable_i;


  assign priv_access     = (mtx_ppb_hprot_i[1] |
                            ((mtx_ppb_haddr_i[11:2] == `NVIC_SWINTREG) &
                             ccr_usersetmpend_i));


  assign ppb_trans      = mtx_ppb_hsel_nvic_i & priv_access & mtx_ppb_hready_i;

  assign next_ppb_rd    = ~mtx_ppb_hwrite_i & ppb_trans;

  assign next_ppb_we[0] = (ppb_trans & mtx_ppb_hwrite_i &
                           (mtx_ppb_hsize_i==2'b10 |
                            mtx_ppb_hsize_i==2'b01 & mtx_ppb_haddr_i[1]==1'b0 |
                            mtx_ppb_hsize_i==2'b00 & mtx_ppb_haddr_i[1:0]==2'b00));

  assign next_ppb_we[1] = (ppb_trans & mtx_ppb_hwrite_i &
                           (mtx_ppb_hsize_i==2'b10 |
                            mtx_ppb_hsize_i==2'b01 & mtx_ppb_haddr_i[1]==1'b0 |
                            mtx_ppb_hsize_i==2'b00 & mtx_ppb_haddr_i[1:0]==2'b01));

  assign next_ppb_we[2] = (ppb_trans & mtx_ppb_hwrite_i &
                           (mtx_ppb_hsize_i==2'b10 |
                            mtx_ppb_hsize_i==2'b01 & mtx_ppb_haddr_i[1]==1'b1 |
                            mtx_ppb_hsize_i==2'b00 & mtx_ppb_haddr_i[1:0]==2'b10));

  assign next_ppb_we[3] = (ppb_trans & mtx_ppb_hwrite_i &
                           (mtx_ppb_hsize_i==2'b10 |
                            mtx_ppb_hsize_i==2'b01 & mtx_ppb_haddr_i[1]==1'b1 |
                            mtx_ppb_hsize_i==2'b00 & mtx_ppb_haddr_i[1:0]==2'b11));

  assign ppb_en         = ppb_trans | ((ppb_rd | (|ppb_we)) & mtx_ppb_hready_i);

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      begin
        ppb_rd          <= 1'b0;
        ppb_we          <= 4'b0000;
        ppb_reg_addr    <= 10'b0000000000;
        ppb_hmaster_reg <= 1'b0;
      end
    else if (ppb_en)
      begin
        ppb_rd          <= next_ppb_rd;
        ppb_we          <= next_ppb_we;
        ppb_reg_addr    <= mtx_ppb_haddr_i[11:2];
        ppb_hmaster_reg <= mtx_ppb_hmaster_i;
      end


`ifdef ARM_DSM
  always @ (irq_lvl_i or dsm_lvl_width or dsm_num_irq)
    begin
      pad_irq_lvl = {(`max_ints*8){1'b0}};
      for (l0 = 0; l0 < NUM_IRQ; l0 = l0 + 1)
        for (l1 = 1; l1 < LVL_WIDTH + 1; l1 = l1 + 1)
          pad_irq_lvl[((l0+1)*8)-l1] = (((l1 <= dsm_lvl_width) & (l0 < dsm_num_irq)) ?
                                        irq_lvl_i[((l0+1)*LVL_WIDTH)-l1] : 1'b0);
    end
`else
  always @ (irq_lvl_i)
    begin
      pad_irq_lvl = {(`max_ints*8){1'b0}};
      for (l0 = 0; l0 < NUM_IRQ; l0 = l0 + 1)
        for (l1 = 1; l1 < LVL_WIDTH + 1; l1 = l1 + 1)
          pad_irq_lvl[((l0+1)*8)-l1] = irq_lvl_i[((l0+1)*LVL_WIDTH)-l1];
    end
`endif

  always @ (shp_uf_lvl_i or shp_bf_lvl_i or shp_mm_lvl_i)
    begin
      pad_local_lvl_0                  = {32{1'b0}};
      pad_local_lvl_0[23:24-LVL_WIDTH] = shp_uf_lvl_i;
      pad_local_lvl_0[15:16-LVL_WIDTH] = shp_bf_lvl_i;
      pad_local_lvl_0[ 7: 8-LVL_WIDTH] = shp_mm_lvl_i;
    end

  always @ (shp_svc_lvl_i)
    begin
      pad_local_lvl_1                  = {32{1'b0}};
      pad_local_lvl_1[31:32-LVL_WIDTH] = shp_svc_lvl_i;
    end

  always @ (shp_st_lvl_i or shp_psv_lvl_i or shp_dm_lvl_i)
    begin
      pad_local_lvl_2                  = {32{1'b0}};
      pad_local_lvl_2[31:32-LVL_WIDTH] = shp_st_lvl_i;
      pad_local_lvl_2[23:24-LVL_WIDTH] = shp_psv_lvl_i;
      pad_local_lvl_2[ 7: 8-LVL_WIDTH] = shp_dm_lvl_i;
    end

  assign pad_irq_en  = {{`max_ints-NUM_IRQ{1'b0}}, irq_en_i[NUM_IRQ-1:0]     };
  assign irq_pending = {{`max_ints-NUM_IRQ{1'b0}}, irq_pending_i[NUM_IRQ-1:0]};
  assign irq_active  = {{`max_ints-NUM_IRQ{1'b0}}, irq_active_i[NUM_IRQ-1:0] };


`ifdef ARM_CMS_BUILD
  always @ (ppb_reg_addr or irq_pending or irq_active or pad_irq_en or
            pad_irq_lvl or fsr_i or mm_bf_far_i or shcs_status_i or
            shcs_mm_en_i or dpu_dbg_ready_i or pad_local_lvl_2 or
            shcs_bf_en_i or shcs_uf_en_i or nmi_pending_i or int_invoke_i or
            int_hipend_isr_i or dfsr_i or afsr_i or vto_tbloff_i or
            ccr_div_0_trp_i or slp_asleep_i or dpu_dbg_halted_i or data_be8_i or
            ccr_unalign_trp_i or ccr_bfhfnmign_i or ccr_stkalign_i or
            ccr_nonbasethrdena_i or scr_sleeponexit_i or scr_deepsleep_i or
            scr_wfeonpend_i or dcrs_rtrans_i or dhcs_c_snapstall_i or
            dhcs_c_mask_ints_i or dhcs_c_halt_i or dhcs_c_step_i or
            dhcs_s_reset_st_i or st_pending_i or dhcs_s_retire_st_i or hfsr_i or
            lockup_i or ccr_usersetmpend_i or dhcs_c_debugen_i or
            psv_pending_i or ppb_rd or demc_mon_en_i or demc_mon_step_i or
            demc_mon_req_i or demc_trcena_i or dbg_vcatch_i or opt_dbg_en or
            dbg_dcrd_i or int_curr_isr_i or int_ret_to_base_i or
            airc_prigroup_i or stick_cnt_flag_i or stick_clk_src_i or
            stick_int_en_i or stick_en_i or stick_reload_i or stick_count_i or
            stick_no_ref_i or stick_skew_i or stick_one_ms_i or
            dbg_mon_pending_i or pad_local_lvl_0 or pad_local_lvl_1 or
            aux_ctrl_i or cpu_id or rev_px_value or cpacr_i or fpccr_i or
            fpcar_i or fpdscr_i or opt_ext_en or mst_cp_en or dsm_num_irq)
`else
  always @ (ppb_reg_addr or irq_pending or irq_active or pad_irq_en or
            pad_irq_lvl or fsr_i or mm_bf_far_i or shcs_status_i or
            shcs_mm_en_i or dpu_dbg_ready_i or pad_local_lvl_2 or
            shcs_bf_en_i or shcs_uf_en_i or nmi_pending_i or int_invoke_i or
            int_hipend_isr_i or dfsr_i or afsr_i or vto_tbloff_i or
            ccr_div_0_trp_i or slp_asleep_i or dpu_dbg_halted_i or data_be8_i or
            ccr_unalign_trp_i or ccr_bfhfnmign_i or ccr_stkalign_i or
            ccr_nonbasethrdena_i or scr_sleeponexit_i or scr_deepsleep_i or
            scr_wfeonpend_i or dcrs_rtrans_i or dhcs_c_snapstall_i or
            dhcs_c_mask_ints_i or dhcs_c_halt_i or dhcs_c_step_i or
            dhcs_s_reset_st_i or st_pending_i or dhcs_s_retire_st_i or hfsr_i or
            lockup_i or ccr_usersetmpend_i or dhcs_c_debugen_i or
            psv_pending_i or ppb_rd or demc_mon_en_i or demc_mon_step_i or
            demc_mon_req_i or demc_trcena_i or dbg_vcatch_i or opt_dbg_en or
            dbg_dcrd_i or int_curr_isr_i or int_ret_to_base_i or
            airc_prigroup_i or stick_cnt_flag_i or stick_clk_src_i or
            stick_int_en_i or stick_en_i or stick_reload_i or stick_count_i or
            stick_no_ref_i or stick_skew_i or stick_one_ms_i or
            dbg_mon_pending_i or pad_local_lvl_0 or pad_local_lvl_1 or
            aux_ctrl_i or cpu_id or rev_px_value or cpacr_i or fpccr_i or
            fpcar_i or fpdscr_i or opt_ext_en or mst_cp_en)
`endif

  begin
    case (ppb_reg_addr & {10{ppb_rd}})
      `NVIC_INTCTRLTYPE:
`ifdef ARM_DSM
        ppb_hrdata        = ((dsm_num_irq-1)>>5);
`else
        ppb_hrdata        = ((NUM_IRQ-1)>>5);
`endif
      `NVIC_AUXCTRL:
        ppb_hrdata        = {{16{1'b0}},                   // [31:15]
                             {6{1'b0}}, aux_ctrl_i[4:3],   // [15: 8]
                             {5{1'b0}}, aux_ctrl_i[2:0]};  // [ 7: 0]
      `NVIC_SYSTICKCS:
        ppb_hrdata        = { {15{1'b0}}, stick_cnt_flag_i, {13{1'b0}},
                              stick_clk_src_i, stick_int_en_i, stick_en_i };
      `NVIC_SYSTICKRL:
        ppb_hrdata        = {8'h00, stick_reload_i};
      `NVIC_SYSTICKCV:
        ppb_hrdata        = {8'h00, stick_count_i};
      `NVIC_SYSTICKCB:
        ppb_hrdata        = {stick_no_ref_i, stick_skew_i, 6'b000000, stick_one_ms_i};
      `NVIC_INTSET0, `NVIC_INTCLR0:
        ppb_hrdata        = pad_irq_en[31:0];
      `NVIC_INTSET1, `NVIC_INTCLR1:
        ppb_hrdata        = pad_irq_en[63:32];
      `NVIC_INTSET2, `NVIC_INTCLR2:
        ppb_hrdata        = pad_irq_en[95:64];
      `NVIC_INTSET3, `NVIC_INTCLR3:
        ppb_hrdata        = pad_irq_en[127:96];
      `NVIC_INTSET4, `NVIC_INTCLR4:
        ppb_hrdata        = pad_irq_en[159:128];
      `NVIC_INTSET5, `NVIC_INTCLR5:
        ppb_hrdata        = pad_irq_en[191:160];
      `NVIC_INTSET6, `NVIC_INTCLR6:
        ppb_hrdata        = pad_irq_en[223:192];
      `NVIC_INTSET7, `NVIC_INTCLR7:
        ppb_hrdata        = pad_irq_en[255:224];
      `NVIC_INTPENDSET0, `NVIC_INTPENDCLR0:
        ppb_hrdata        = irq_pending[31:0];
      `NVIC_INTPENDSET1, `NVIC_INTPENDCLR1:
        ppb_hrdata        = irq_pending[63:32];
      `NVIC_INTPENDSET2, `NVIC_INTPENDCLR2:
        ppb_hrdata        = irq_pending[95:64];
      `NVIC_INTPENDSET3, `NVIC_INTPENDCLR3:
        ppb_hrdata        = irq_pending[127:96];
      `NVIC_INTPENDSET4, `NVIC_INTPENDCLR4:
        ppb_hrdata        = irq_pending[159:128];
      `NVIC_INTPENDSET5, `NVIC_INTPENDCLR5:
        ppb_hrdata        = irq_pending[191:160];
      `NVIC_INTPENDSET6, `NVIC_INTPENDCLR6:
        ppb_hrdata        = irq_pending[223:192];
      `NVIC_INTPENDSET7, `NVIC_INTPENDCLR7:
        ppb_hrdata        = irq_pending[255:224];
      `NVIC_INTACTIVE0:
        ppb_hrdata        = irq_active[31:0];
      `NVIC_INTACTIVE1:
        ppb_hrdata        = irq_active[63:32];
      `NVIC_INTACTIVE2:
        ppb_hrdata        = irq_active[95:64];
      `NVIC_INTACTIVE3:
        ppb_hrdata        = irq_active[127:96];
      `NVIC_INTACTIVE4:
        ppb_hrdata        = irq_active[159:128];
      `NVIC_INTACTIVE5:
        ppb_hrdata        = irq_active[191:160];
      `NVIC_INTACTIVE6:
        ppb_hrdata        = irq_active[223:192];
      `NVIC_INTACTIVE7:
        ppb_hrdata        = irq_active[255:224];
      `NVIC_INTPRILVL0:
        ppb_hrdata        = pad_irq_lvl[31:0];
      `NVIC_INTPRILVL1:
        ppb_hrdata        = pad_irq_lvl[63:32];
      `NVIC_INTPRILVL2:
        ppb_hrdata        = pad_irq_lvl[95:64];
      `NVIC_INTPRILVL3:
        ppb_hrdata        = pad_irq_lvl[127:96];
      `NVIC_INTPRILVL4:
        ppb_hrdata        = pad_irq_lvl[159:128];
      `NVIC_INTPRILVL5:
        ppb_hrdata        = pad_irq_lvl[191:160];
      `NVIC_INTPRILVL6:
        ppb_hrdata        = pad_irq_lvl[223:192];
      `NVIC_INTPRILVL7:
        ppb_hrdata        = pad_irq_lvl[255:224];
      `NVIC_INTPRILVL8:
        ppb_hrdata        = pad_irq_lvl[287:256];
      `NVIC_INTPRILVL9:
        ppb_hrdata        = pad_irq_lvl[319:288];
      `NVIC_INTPRILVL10:
        ppb_hrdata        = pad_irq_lvl[351:320];
      `NVIC_INTPRILVL11:
        ppb_hrdata        = pad_irq_lvl[383:352];
      `NVIC_INTPRILVL12:
        ppb_hrdata        = pad_irq_lvl[415:384];
      `NVIC_INTPRILVL13:
        ppb_hrdata        = pad_irq_lvl[447:416];
      `NVIC_INTPRILVL14:
        ppb_hrdata        = pad_irq_lvl[479:448];
      `NVIC_INTPRILVL15:
        ppb_hrdata        = pad_irq_lvl[511:480];
      `NVIC_INTPRILVL16:
        ppb_hrdata        = pad_irq_lvl[543:512];
      `NVIC_INTPRILVL17:
        ppb_hrdata        = pad_irq_lvl[575:544];
      `NVIC_INTPRILVL18:
        ppb_hrdata        = pad_irq_lvl[607:576];
      `NVIC_INTPRILVL19:
        ppb_hrdata        = pad_irq_lvl[639:608];
      `NVIC_INTPRILVL20:
        ppb_hrdata        = pad_irq_lvl[671:640];
      `NVIC_INTPRILVL21:
        ppb_hrdata        = pad_irq_lvl[703:672];
      `NVIC_INTPRILVL22:
        ppb_hrdata        = pad_irq_lvl[735:704];
      `NVIC_INTPRILVL23:
        ppb_hrdata        = pad_irq_lvl[767:736];
      `NVIC_INTPRILVL24:
        ppb_hrdata        = pad_irq_lvl[799:768];
      `NVIC_INTPRILVL25:
        ppb_hrdata        = pad_irq_lvl[831:800];
      `NVIC_INTPRILVL26:
        ppb_hrdata        = pad_irq_lvl[863:832];
      `NVIC_INTPRILVL27:
        ppb_hrdata        = pad_irq_lvl[895:864];
      `NVIC_INTPRILVL28:
        ppb_hrdata        = pad_irq_lvl[927:896];
      `NVIC_INTPRILVL29:
        ppb_hrdata        = pad_irq_lvl[959:928];
      `NVIC_INTPRILVL30:
        ppb_hrdata        = pad_irq_lvl[991:960];
      `NVIC_INTPRILVL31:
        ppb_hrdata        = pad_irq_lvl[1023:992];
      `NVIC_INTPRILVL32:
        ppb_hrdata        = pad_irq_lvl[1055:1024];
      `NVIC_INTPRILVL33:
        ppb_hrdata        = pad_irq_lvl[1087:1056];
      `NVIC_INTPRILVL34:
        ppb_hrdata        = pad_irq_lvl[1119:1088];
      `NVIC_INTPRILVL35:
        ppb_hrdata        = pad_irq_lvl[1151:1120];
      `NVIC_INTPRILVL36:
        ppb_hrdata        = pad_irq_lvl[1183:1152];
      `NVIC_INTPRILVL37:
        ppb_hrdata        = pad_irq_lvl[1215:1184];
      `NVIC_INTPRILVL38:
        ppb_hrdata        = pad_irq_lvl[1247:1216];
      `NVIC_INTPRILVL39:
        ppb_hrdata        = pad_irq_lvl[1279:1248];
      `NVIC_INTPRILVL40:
        ppb_hrdata        = pad_irq_lvl[1311:1280];
      `NVIC_INTPRILVL41:
        ppb_hrdata        = pad_irq_lvl[1343:1312];
      `NVIC_INTPRILVL42:
        ppb_hrdata        = pad_irq_lvl[1375:1344];
      `NVIC_INTPRILVL43:
        ppb_hrdata        = pad_irq_lvl[1407:1376];
      `NVIC_INTPRILVL44:
        ppb_hrdata        = pad_irq_lvl[1439:1408];
      `NVIC_INTPRILVL45:
        ppb_hrdata        = pad_irq_lvl[1471:1440];
      `NVIC_INTPRILVL46:
        ppb_hrdata        = pad_irq_lvl[1503:1472];
      `NVIC_INTPRILVL47:
        ppb_hrdata        = pad_irq_lvl[1535:1504];
      `NVIC_INTPRILVL48:
        ppb_hrdata        = pad_irq_lvl[1567:1536];
      `NVIC_INTPRILVL49:
        ppb_hrdata        = pad_irq_lvl[1599:1568];
      `NVIC_INTPRILVL50:
        ppb_hrdata        = pad_irq_lvl[1631:1600];
      `NVIC_INTPRILVL51:
        ppb_hrdata        = pad_irq_lvl[1663:1632];
      `NVIC_INTPRILVL52:
        ppb_hrdata        = pad_irq_lvl[1695:1664];
      `NVIC_INTPRILVL53:
        ppb_hrdata        = pad_irq_lvl[1727:1696];
      `NVIC_INTPRILVL54:
        ppb_hrdata        = pad_irq_lvl[1759:1728];
      `NVIC_INTPRILVL55:
        ppb_hrdata        = pad_irq_lvl[1791:1760];
      `NVIC_INTPRILVL56:
        ppb_hrdata        = pad_irq_lvl[1823:1792];
      `NVIC_INTPRILVL57:
        ppb_hrdata        = pad_irq_lvl[1855:1824];
      `NVIC_INTPRILVL58:
        ppb_hrdata        = pad_irq_lvl[1887:1856];
      `NVIC_INTPRILVL59:
        ppb_hrdata        = pad_irq_lvl[1919:1888];
      `NVIC_INTPRILVL60:
        ppb_hrdata        = pad_irq_lvl[1951:1920];
      `NVIC_INTPRILVL61:
        ppb_hrdata        = pad_irq_lvl[1983:1952];
      `NVIC_INTPRILVL62:
        ppb_hrdata        = pad_irq_lvl[2015:1984];
      `NVIC_INTPRILVL63:
        ppb_hrdata        = pad_irq_lvl[2047:2016];
      `NVIC_CPUIDBASEREG:
        ppb_hrdata        = {cpu_id[31:4], rev_px_value[3:0]};
      `NVIC_INTCTRLSTAT:
        ppb_hrdata        = { nmi_pending_i, 2'b00,
                              psv_pending_i,
                              1'b0,
                              st_pending_i,
                              2'b00,
                              int_invoke_i & dpu_dbg_halted_i,
                                (|irq_pending),
                              2'b00, int_hipend_isr_i, int_ret_to_base_i, 1'b0,
                              1'b0, int_curr_isr_i };
      `NVIC_INTTBLOFF:
        ppb_hrdata        = { vto_tbloff_i, 7'b0000000 };
      `NVIC_APPIRC:
        ppb_hrdata        = { 16'hFA05, data_be8_i, 4'b0000, airc_prigroup_i, 8'h00};
      `NVIC_SYSCTRL:
        ppb_hrdata        = {{27{1'b0}}, scr_wfeonpend_i, 1'b0, scr_deepsleep_i,
                             scr_sleeponexit_i, 1'b0};
      `NVIC_CONFIGCTRL:
        ppb_hrdata        = {{22{1'b0}}, ccr_stkalign_i, ccr_bfhfnmign_i,
                             3'b000, ccr_div_0_trp_i, ccr_unalign_trp_i, 1'b0,
                             ccr_usersetmpend_i, ccr_nonbasethrdena_i};
      `NVIC_SYSHNDLRPRI0:
        ppb_hrdata        = pad_local_lvl_0;
      `NVIC_SYSHNDLRPRI1:
        ppb_hrdata        = pad_local_lvl_1;
      `NVIC_SYSHNDLRPRI2:
        ppb_hrdata        = pad_local_lvl_2;
      `NVIC_SYSHNDLREN:
        ppb_hrdata        = {{13{1'b0}}, shcs_uf_en_i, shcs_bf_en_i,
                             shcs_mm_en_i, shcs_status_i};
      `NVIC_FAULTSTAT:
        ppb_hrdata        = fsr_i[31:0];
      `NVIC_HARDFAULTSTAT:
        ppb_hrdata        = hfsr_i[31:0];
      `NVIC_DBGFAULTSTAT:
        ppb_hrdata        = dfsr_i[31:0];
      `NVIC_BUSFAULTADDR:
        ppb_hrdata        = mm_bf_far_i;
      `NVIC_MEMFAULTADDR:
        ppb_hrdata        = mm_bf_far_i;
      `NVIC_AUXFAULTSTAT:
        ppb_hrdata        = afsr_i[31:0];
      `NVIC_PFR0:
        ppb_hrdata        = 32'h00000030;
      `NVIC_PFR1:
        ppb_hrdata        = 32'h00000200;
      `NVIC_DFR0:
        ppb_hrdata        = opt_dbg_en ? 32'h00100000 : 32'h00000000;
      `NVIC_MMFR0:
        ppb_hrdata        = 32'h00100030;
      `NVIC_MMFR1:
        ppb_hrdata        = 32'h00000000;
      `NVIC_MMFR2:
        ppb_hrdata        = 32'h01000000;
      `NVIC_ISAR0:
        ppb_hrdata        = 32'h01101110;
      `NVIC_ISAR1:
        ppb_hrdata        = opt_ext_en ? 32'h02112000 : 32'h02111000;
      `NVIC_ISAR2:
        ppb_hrdata        = opt_ext_en ? 32'h21232231 : 32'h21112231;
      `NVIC_ISAR3:
        ppb_hrdata        = opt_ext_en ? 32'h01111131 : 32'h01111110;
      `NVIC_ISAR4:
        ppb_hrdata        = 32'h01310132;
      `NVIC_CPACR:
        ppb_hrdata        = {{8{1'b0}}, cpacr_i & {4{mst_cp_en}}, {20{1'b0}}};
      `NVIC_FPSID:
        ppb_hrdata        = 32'h`NVIC_FPU_REVISION;
      `NVIC_FPCCR:
        ppb_hrdata        = mst_cp_en ? fpccr_i : {32{1'b0}};
      `NVIC_FPCAR:
        ppb_hrdata        = mst_cp_en ? fpcar_i : {32{1'b0}};
      `NVIC_FPDSCR:
        ppb_hrdata        = {{5{1'b0}},
                             mst_cp_en ? fpdscr_i : {5{1'b0}},
                             {22{1'b0}}};
      `NVIC_MVFR0:
        ppb_hrdata        = mst_cp_en ? `NVIC_MVFR0_VALUE : {32{1'b0}};
      `NVIC_MVFR1:
        ppb_hrdata        = mst_cp_en ? `NVIC_MVFR1_VALUE : {32{1'b0}};
      `NVIC_ICDSC:
        ppb_hrdata        = { 6'b000000, dhcs_s_reset_st_i, dhcs_s_retire_st_i,
                              4'b0000, lockup_i[0], slp_asleep_i,
                              dpu_dbg_halted_i, dpu_dbg_ready_i & ~dcrs_rtrans_i,
                              {10{1'b0}}, dhcs_c_snapstall_i, 1'b0,
                              dhcs_c_mask_ints_i, dhcs_c_step_i, dhcs_c_halt_i,
                              dhcs_c_debugen_i };
      `NVIC_ICDRD:
        ppb_hrdata        = dbg_dcrd_i;
      `NVIC_ICDEMC:
        ppb_hrdata        = { 4'h0, 3'b000, demc_trcena_i, 4'h0, demc_mon_req_i,
                              demc_mon_step_i, dbg_mon_pending_i, demc_mon_en_i,
                              5'b00000, dbg_vcatch_i[7:1], 3'b000,
                              dbg_vcatch_i[0] };
      `NVIC_PERIPHID4:
        ppb_hrdata        = 32'h00000004;
      `NVIC_PERIPHID5:
        ppb_hrdata        = 32'h00000000;
      `NVIC_PERIPHID6:
        ppb_hrdata        = 32'h00000000;
      `NVIC_PERIPHID7:
        ppb_hrdata        = 32'h00000000;
      `NVIC_PERIPHID0:
        ppb_hrdata        = mst_cp_en ? 32'h0000000C : 32'h00000000;
      `NVIC_PERIPHID1:
        ppb_hrdata        = 32'h000000B0;
      `NVIC_PERIPHID2:
        ppb_hrdata        = 32'h0000000B;
      `NVIC_PERIPHID3:
        ppb_hrdata        = 32'h00000000;
      `NVIC_PCELLID0:
        ppb_hrdata        = 32'h0000000D;
      `NVIC_PCELLID1:
        ppb_hrdata        = 32'h000000E0;
      `NVIC_PCELLID2:
        ppb_hrdata        = 32'h00000005;
      `NVIC_PCELLID3:
        ppb_hrdata        = 32'h000000B1;
      default:
        ppb_hrdata        = {32{1'b0}};
    endcase
  end

  assign ppb_hready_we = (~ppb_hready |
                          (mtx_ppb_hready_i & mtx_ppb_hsel_nvic_i &
                           ~priv_access));

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      ppb_hready <= 1'b1;
    else if (ppb_hready_we)
      ppb_hready <= ~ppb_hready;

  assign ppb_hresp_we = (ppb_hready &
                         (ppb_hresp |
                          (mtx_ppb_hready_i & mtx_ppb_hsel_nvic_i &
                           ~priv_access)));

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      ppb_hresp <= 1'b0;
    else if (ppb_hresp_we)
      ppb_hresp <= mtx_ppb_hsel_nvic_i & ~priv_access;

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      rev_px_value <= 4'b0001;
    else
      rev_px_value <= rev_px_value;

  assign cpu_id[31:4]      = opt_ext_en ? 28'h410FC24 : 28'h412FC23;

  assign ppb_hrdata_o      = ppb_hrdata;
  assign ppb_hready_o      = ppb_hready;
  assign ppb_hresp_o       = ppb_hresp;
  assign ppb_we_o          = ppb_we;
  assign ppb_rd_o          = ppb_rd;
  assign ppb_hmaster_reg_o = ppb_hmaster_reg;
  assign ppb_reg_addr_o    = ppb_reg_addr;


`ifdef ARM_ASSERT_ON

  assert_never #(0,0,"PHREADYOUT must only be deasserted when PHRESP is set")
    ovl_hreadyout_check1(hclk, dreset_n, ~ppb_hready & ~ppb_hresp);

  assert_next #(0,1,0,0,0, "PHREADYOUT must only be deasserted for a single cycle")
    ovl_hreadyout_check2(hclk, dreset_n, ppb_hresp & ~ppb_hready, ppb_hready);

`endif

endmodule

`include "cm3_nvic_undefs.vp"
