`include "cm3_nvic_defs.vp"

module cm3_nvic_reg
  (// Inputs
   dreset_n, hreset_n, fclk, hclk, pclk, ppb_hmaster_reg_i, ppb_we_i,
   ppb_rd_i, ppb_reg_addr_i, mtx_ppb_hwdata_i, ext_big_end_i,
   ext_st_clk_i, ext_st_noref_i, ext_cp_disable_i, ext_stkalign_init_i,
   dpu_rst_pc_valid_i, dpu_ahb_haddrd_i, dpu_ahb_haddrd_valid_i,
   dpu_ahb_htransi_iss_i, dpu_privileged_i, dpu_thread_i, dpu_ivalid_i,
   dpu_retire_i, dpu_slp_ctl_i, dpu_fpccr_we_i, dpu_fpcar_nxt_i, dpu_svc_i,
   dpu_bkpt_i, dpu_cp_busy_i, dpu_lazy_stk_i, dpu_nxt_hfsr_i, dpu_nxt_mfsr_i,
   dpu_nxt_bfsr_i, dpu_nxt_ufsr_i, ext_aux_fault_i, dwt_watchpoint_i,
   mtx_wb_abort_i, int_lazy_fpccr_i, int_lazy_dm_i, int_allow_bf_i,
   int_escalate_i, int_lazy_esc_i, int_hfault_i, int_invoke_i, dpu_int_entry_i,
   dpu_int_exit_i, int_nxt_isr_i, int_in_nmihf_i, int_non_actv_exit_i,
   ext_slp_rxev_i, ext_slp_txev_i, ext_slp_hold_req_n_i, slp_norm_wake_i,
   slp_pmask_pend_i, slp_wfe_pend_i, slp_sleeponexit_i, wic_ds_ack_n_i,
   wic_ds_req_n_i, ext_dbg_en_i, ext_dbg_req_i, ext_dbg_restart_i,
   dpu_dbg_ready_i, dpu_dbg_halted_i, dbg_mon_active_i, dbg_mon_fault_i,
   dbg_mon_allow_i, dbg_mon_step_pend_i, dbg_mon_hi_pri_invk_i,
   dbg_masked_bkpt_i,
   data_be8_o, aux_ctrl_o, aux_ctrl1_we_o, ccr_div_0_trp_o, ccr_unalign_trp_o,
   ccr_bfhfnmign_o, ccr_stkalign_o, ccr_usersetmpend_o, ccr_nonbasethrdena_o,
   cpacr_o, fpcar_o, fpccr_o, fpdscr_o, airc_vectreset_o, airc_sysresetreq_o,
   airc_prigroup_o, vto_tbloff_o, lockup_o, irq_en_o, shcs_mm_en_o,
   shcs_bf_en_o, shcs_uf_en_o, irq_lvl_o, shp_mm_lvl_o, shp_bf_lvl_o,
   shp_uf_lvl_o, shp_dm_lvl_o, shp_svc_lvl_o, shp_psv_lvl_o, shp_st_lvl_o,
   int_pend_set_o, int_pend_clr_o, int_actv_set_o, int_actv_clr_o,
   vect_clr_actv_o, stick_cnt_flag_o, stick_clk_src_o, stick_int_en_o,
   stick_en_o, stick_reload_o, stick_count_o, stick_int_o, fsr_o, hfsr_o,
   dfsr_o, afsr_o, mm_bf_far_o, scr_sleeponexit_o, scr_deepsleep_o,
   scr_wfeonpend_o, slp_wfe_valid_o, slp_wfi_valid_o, slp_hold_ack_n_o,
   slp_asleep_o, slp_sleeping_o, slp_sleepdeep_o, wic_load_o, wic_clear_o,
   demc_mon_en_o, demc_mon_req_o, demc_mon_step_o, demc_trcena_o,
   dhcs_c_debugen_o, dhcs_c_halt_o, dhcs_c_step_o, dhcs_s_reset_st_o,
   dhcs_s_retire_st_o, dhcs_c_mask_ints_o, dhcs_c_snapstall_o, dbg_mon_pend_o,
   dbg_mon_req_o, dbg_dbgen_o, dbg_vcatch_o, dbg_halt_req_o, dbg_step_o,
   dbg_mask_ints_o, dbg_restarted_o, dbg_dcrd_o, dcrs_rtrans_o, dcrs_regwnr_o,
   dcrs_regsel_o
  );

  parameter NUM_IRQ         = 16;                   // Number of interrupts
  parameter LVL_WIDTH       = 3;                    // Interrupt priority width
  parameter DEBUG_LVL       = 3;                    // Debug level
  parameter RESET_ALL_REGS  = 0;                    // Reset all registers
  parameter WIC_PRESENT     = 0;                    // WIC present
  parameter CP_PRESENT      = 0;                    // CP present if 1
  parameter IRQ_WIDTH       = NUM_IRQ * LVL_WIDTH;  // Total priority width
  parameter NUM_INTS        = `num_non_irq+NUM_IRQ; // Total supported ints

  input                     dreset_n;               // Debug reset
  input                     hreset_n;               // NVIC reset
  input                     fclk;                   // NVIC Clock (free-running)
  input                     hclk;                   // NVIC Clock (Clock gated)
  input                     pclk;                   // PPB Clock (Clock gated)
  input                     ppb_hmaster_reg_i;      // Registered PMASTER
  input [3:0]               ppb_we_i;               // PPB data phase byte write
  input                     ppb_rd_i;               // PPB data phase read enable
  input [11:2]              ppb_reg_addr_i;         // Registered ppb_haddr
  input [31:0]              mtx_ppb_hwdata_i;       // PPB write data
  input                     ext_big_end_i;          // Big endianess
  input                     ext_st_clk_i;           // System Tick Clock
  input                     ext_st_noref_i;         // No SysTick reference clock
  input                     ext_cp_disable_i;       // CP disable
  input                     ext_stkalign_init_i;    // Stack alignment reset value
  input                     dpu_rst_pc_valid_i;     // Core has been reset
  input [31:0]              dpu_ahb_haddrd_i;       // Core D-bus (Bus Fault)
  input                     dpu_ahb_haddrd_valid_i; // Core D-bus is valid (non-uCode)
  input                     dpu_ahb_htransi_iss_i;  // In data phase of I-AHB trans
  input                     dpu_privileged_i;       // DPU in privileged mode
  input                     dpu_thread_i;           // DPU in thread
  input                     dpu_ivalid_i;           // Decode enable
  input                     dpu_retire_i;           // Opcode is retiring
  input [1:0]               dpu_slp_ctl_i;          // [1] = WFE, [0] = WFI
  input [1:0]               dpu_fpccr_we_i;         // LSPACT write enable
  input [31:0]              dpu_fpcar_nxt_i;        // FPCAR write value
  input                     dpu_svc_i;              // SVC has been executed
  input                     dpu_bkpt_i;             // BKPT instruction executed
  input                     dpu_cp_busy_i;          // CP busy
  input                     dpu_lazy_stk_i;         // Lazy stacking
  input                     dpu_nxt_hfsr_i;         // Vector fetch hard fault
  input [4:0]               dpu_nxt_mfsr_i;         // MemManage fault type
  input [5:0]               dpu_nxt_bfsr_i;         // Bus fault type
  input [9:0]               dpu_nxt_ufsr_i;         // Usage fault type
  input [31:0]              ext_aux_fault_i;        // Auxiliary fsr pulse inputs
  input                     dwt_watchpoint_i;       // DWT generated watchpoint
  input                     mtx_wb_abort_i;         // Write buffer abort
  input [3:0]               int_lazy_fpccr_i;       // Lazy FPCCR ready values
  input                     int_lazy_dm_i;          // Lazy debug monitor
  input                     int_allow_bf_i;         // Allow bus fault to invoke
  input                     int_escalate_i;         // Escalate to HF
  input                     int_lazy_esc_i;         // Lazy fault escalated
  input                     int_hfault_i;           // HardFault raised
  input                     int_invoke_i;           // Perform hardware stacking
  input                     dpu_int_entry_i;        // ISR has been entered
  input                     dpu_int_exit_i;         // ISR has completed
  input [7:0]               int_nxt_isr_i;          // Nxt ISR number to be processed
  input [2:0]               int_in_nmihf_i;         // NMI/HardFault handler {pend,active}
  input                     int_non_actv_exit_i;    // Non-active ISR exited
  input                     ext_slp_rxev_i;         // Receive Event
  input                     ext_slp_txev_i;         // Transmit Event
  input                     ext_slp_hold_req_n_i;   // Hold core in sleep mode
  input                     slp_norm_wake_i;        // Normal sleep awake
  input                     slp_pmask_pend_i;       // PriMask suppressed pend
  input                     slp_wfe_pend_i;         // New pend will wake up WFE
  input                     slp_sleeponexit_i;      // Will sleep on exit
  input                     wic_ds_ack_n_i;         // WIC mode acknowledge
  input                     wic_ds_req_n_i;         // WIC mode request
  input                     ext_dbg_en_i;           // Debug disable
  input                     ext_dbg_req_i;          // External debug request pin
  input                     ext_dbg_restart_i;      // External debug restart request
  input                     dpu_dbg_ready_i;        // Single cycle reply for debug
  input                     dpu_dbg_halted_i;       // Core is halted
  input                     dbg_mon_active_i;       // Debug monitor is active
  input                     dbg_mon_fault_i;        // Debug monitor has occurred
  input                     dbg_mon_allow_i;        // DbgMon will be able to invoke
  input                     dbg_mon_step_pend_i;    // Debug Monitor step pend
  input                     dbg_mon_hi_pri_invk_i;  // dbg_mon pri > invoke pri
  input                     dbg_masked_bkpt_i;      // BKPT will halt

  output                    data_be8_o;             // Big-endian byte invariant
  output [4:0]              aux_ctrl_o;             // Auxiliary Control Register
  output                    aux_ctrl1_we_o;         // Auxiliary Ctrl write enable
  output                    ccr_div_0_trp_o;        // Divide by zero fault enable
  output                    ccr_unalign_trp_o;      // Unaligned fault enable
  output                    ccr_bfhfnmign_o;        // Ignore bus fault in NMI/HF
  output                    ccr_stkalign_o;         // Stack Align
  output                    ccr_usersetmpend_o;     // User can use software trigger
  output                    ccr_nonbasethrdena_o;   // Handler can be thread
  output [3:0]              cpacr_o;                // CPACR
  output [31:0]             fpcar_o;                // FPCAR register
  output [31:0]             fpccr_o;                // FPCCR register
  output [4:0]              fpdscr_o;               // Default FPSCR register
  output                    airc_vectreset_o;       // System reset (internal)
  output                    airc_sysresetreq_o;     // System reset (external)
  output [2:0]              airc_prigroup_o;        // Priority grouping position
  output [31:7]             vto_tbloff_o;           // Vector table offset
  output [1:0]              lockup_o;               // Core is locked up
  output [NUM_IRQ-1:0]      irq_en_o;               // Individual IRQ enables
  output                    shcs_mm_en_o;           // MemManage fault enable
  output                    shcs_bf_en_o;           // Bus fault enable
  output                    shcs_uf_en_o;           // Usage fault enable
  output [IRQ_WIDTH-1:0]    irq_lvl_o;              // IRQ priority levels
  output [LVL_WIDTH-1:0]    shp_mm_lvl_o;           // MemManage Fault priority
  output [LVL_WIDTH-1:0]    shp_bf_lvl_o;           // Bus Fault priority
  output [LVL_WIDTH-1:0]    shp_uf_lvl_o;           // Usage Fault priority
  output [LVL_WIDTH-1:0]    shp_dm_lvl_o;           // Debug Monitor priority
  output [LVL_WIDTH-1:0]    shp_svc_lvl_o;          // SVC priority
  output [LVL_WIDTH-1:0]    shp_psv_lvl_o;          // Pend SVC priority
  output [LVL_WIDTH-1:0]    shp_st_lvl_o;           // System Tick priority
  output [NUM_INTS-1:0]     int_pend_set_o;         // Set pending
  output [NUM_INTS-1:0]     int_pend_clr_o;         // Clear pending
  output [NUM_INTS-1:0]     int_actv_set_o;         // Set active
  output [NUM_INTS-1:0]     int_actv_clr_o;         // Clear active
  output                    vect_clr_actv_o;        // Vector Clear Active
  output                    stick_cnt_flag_o;       // SysTick count flag
  output                    stick_clk_src_o;        // SysTick clock source
  output                    stick_int_en_o;         // Allow pending of SysTick
  output                    stick_en_o;             // Enable SysTick counter reload
  output [23:0]             stick_reload_o;         // SysTick counter reload value
  output [23:0]             stick_count_o;          // SysTick counter value
  output                    stick_int_o;            // System Tick interrupt
  output [31:0]             fsr_o;                  // Fault status register
  output [31:0]             hfsr_o;                 // HardFault status register
  output [31:0]             dfsr_o;                 // Debug fault status register
  output [31:0]             afsr_o;                 // Auxiliary fault status register
  output [31:0]             mm_bf_far_o;            // Fault address register
  output                    scr_sleeponexit_o;      // Sleep on exit requested
  output                    scr_deepsleep_o;        // Deep sleep bit is set
  output                    scr_wfeonpend_o;        // Wake up from WFE on new pends
  output                    slp_wfe_valid_o;        // Sleep is due to WFE
  output                    slp_wfi_valid_o;        // Sleep is due to WFI
  output                    slp_hold_ack_n_o;       // Hold core in sleep mode ack
  output                    slp_asleep_o;           // DPU is sleeping
  output                    slp_sleeping_o;         // Core is sleeping
  output                    slp_sleepdeep_o;        // Core is deep sleeping
  output                    wic_load_o;             // Load WIC sensitivity list
  output                    wic_clear_o;            // Clear WIC sensitivity list
  output                    demc_mon_en_o;          // Debug monitor enable from reg
  output                    demc_mon_req_o;         // Debug Monitor request
  output                    demc_mon_step_o;        // Debug Monitor step
  output                    demc_trcena_o;          // Trace enable
  output                    dhcs_c_debugen_o;       // Halting debug enable from reg
  output                    dhcs_c_halt_o;          // Halting control
  output                    dhcs_c_step_o;          // Halting step enable
  output                    dhcs_s_reset_st_o;      // Reset occured since read
  output                    dhcs_s_retire_st_o;     // Instr retired since read
  output                    dhcs_c_mask_ints_o;     // Mask interrupts whilst step
  output                    dhcs_c_snapstall_o;     // Snap stalled bus trans
  output                    dbg_mon_pend_o;         // Debug Monitor pend
  output                    dbg_mon_req_o;          // Request the core to pause
  output                    dbg_dbgen_o;            // Halting debug enable
  output [7:0]              dbg_vcatch_o;           // Vector catch
  output                    dbg_halt_req_o;         // Request the core to stop
  output                    dbg_step_o;             // Debug stepping
  output                    dbg_mask_ints_o;        // Mask interrupts during debug
  output                    dbg_restarted_o;        // External debug restart req
  output [31:0]             dbg_dcrd_o;             // Scratch debug register
  output                    dcrs_rtrans_o;          // DBG HTRANS
  output                    dcrs_regwnr_o;          // Debug register rd/wr
  output [6:0]              dcrs_regsel_o;          // Register address, R0-R15

  reg  [2:0]                airc_prigroup;          // Priority grouping position
  reg                       data_be8;               // Big-endian byte invariant
  reg                       stick_cnt_flag;         // SysTick count flag
  reg                       stick_int_en;           // Allow pending of SysTick
  reg                       stick_en;               // Enable SysTick counter reload
  reg  [23:0]               stick_reload;           // SysTick counter reload value
  reg  [23:0]               stick_count;            // SysTick counter value
  reg                       stick_int;              // System Tick interrupt
  reg                       airc_vectreset;         // System reset (internal)
  reg                       airc_sysresetreq;       // System reset (external)
  reg  [31:0]               afsr;                   // Auxiliary fault status reg

  wire                      airc_we;

  wire                      airc_vectreset_en;
  wire                      nxt_airc_vectreset;

  reg  [NUM_IRQ-1:0]        irq_en_set;
  reg  [NUM_IRQ-1:0]        irq_en_clr;
  wire [NUM_IRQ-1:0]        irq_int_we;
  wire                      irq_int_cg_en;
  wire [NUM_IRQ-1:0]        nxt_irq_en;
  reg  [NUM_IRQ-1:0]        irq_en_reg;
  wire [NUM_IRQ-1:0]        irq_en;

  wire [(4*LVL_WIDTH)-1:0]  irq_wdata;
  reg  [NUM_IRQ-1:0]        irq_pri_lvl_we;
  reg  [IRQ_WIDTH-1:0]      irq_lvl_reg;
  wire [IRQ_WIDTH-1:0]      irq_lvl;
  reg  [IRQ_WIDTH-1:0]      nxt_irq_lvl;
  wire                      irq_lvl_we;

  wire                      int_ctrl_state_we;
  wire                      nmi_pend_set;
  wire                      psv_pend_set;
  wire                      psv_pend_clr;
  wire                      st_pend_set;
  wire                      st_pend_clr;

  wire [3:0]                vto_tbloff_we;
  reg  [31:7]               vto_tbloff;
  wire [31:7]               nxt_vto_tbloff;

  wire                      sys_ctrl_we;
  reg                       scr_deepsleep;
  reg                       scr_sleeponexit;
  reg                       scr_wfeonpend;

  wire                      slp_wfe_new_pend;
  reg                       slp_event_reg;
  wire                      slp_event_reg_set;
  wire                      slp_event_reg_clr;
  wire                      slp_event_reg_we;

  reg                       slp_normal;
  reg                       slp_forced;
  reg  [1:0]                slp_sleep_type;
  wire                      slp_normal_en;
  wire [1:0]                nxt_slp_normal;
  wire                      slp_wfe_valid;
  wire                      slp_wfi_valid;
  wire                      slp_forced_en;
  wire                      slp_forced_clr;
  wire                      slp_forced_set;
  reg                       slp_hold_ack_q;
  wire                      slp_hold_ack_n;
  wire                      slp_asleep;
  wire                      slp_sleeping;
  wire                      slp_sleepdeep;

  wire [1:0]                config_ctrl_we;
  reg                       ccr_nonbasethrdena;
  reg                       ccr_usersetmpend;
  reg                       ccr_div_0_trp;
  reg                       ccr_unalign_trp;
  reg                       ccr_bfhfnmign;
  wire                      ccr_stkalign_we;
  wire                      nxt_ccr_stkalign;
  reg                       ccr_stkalign;

  wire                      shp_mm_lvl_we;
  wire                      shp_bf_lvl_we;
  wire                      shp_dm_lvl_we;
  wire                      shp_svc_lvl_we;
  wire                      shp_psv_lvl_we;
  wire                      shp_st_lvl_we;
  wire                      shp_uf_lvl_we;
  reg  [LVL_WIDTH-1:0]      shp_mm_lvl;
  reg  [LVL_WIDTH-1:0]      shp_bf_lvl;
  reg  [LVL_WIDTH-1:0]      shp_dm_lvl;
  reg  [LVL_WIDTH-1:0]      shp_svc_lvl;
  reg  [LVL_WIDTH-1:0]      shp_psv_lvl;
  reg  [LVL_WIDTH-1:0]      shp_st_lvl;
  reg  [LVL_WIDTH-1:0]      shp_uf_lvl;
  wire [LVL_WIDTH-1:0]      opt_shp_mm_lvl;
  wire [LVL_WIDTH-1:0]      opt_shp_bf_lvl;
  wire [LVL_WIDTH-1:0]      opt_shp_uf_lvl;
  wire [LVL_WIDTH-1:0]      opt_shp_dm_lvl;
  wire [LVL_WIDTH-1:0]      opt_shp_svc_lvl;
  wire [LVL_WIDTH-1:0]      opt_shp_psv_lvl;
  wire [LVL_WIDTH-1:0]      opt_shp_st_lvl;

  wire [2:0]                shcs_we;
  reg                       shcs_mm_en;
  reg                       shcs_bf_en;
  reg                       shcs_uf_en;

  wire [31:0]               fsr;
  reg  [18:0]               fsr_min;
  wire [18:0]               nxt_fsr;
  wire [18:0]               clr_fsr;
  wire [18:0]               set_fsr;
  wire                      fsr_we;

  wire [31:0]               hfsr;
  reg  [2:0]                hfsr_min;
  wire [2:0]                nxt_hfsr;
  wire [2:0]                clr_hfsr;
  wire [2:0]                set_hfsr;
  wire                      hfsr_we;

  wire [31:0]               dfsr;
  wire                      mask_dfsr;
  wire                      mask_async_dfsr;
  wire [4:0]                dfsr_write;
  wire                      set_ext_dfsr;
  wire                      ext_dfsr_we;
  wire                      nxt_ext_dfsr;
  reg                       ext_dfsr;
  wire                      set_vc_dfsr;
  wire                      vc_dfsr_we;
  wire                      nxt_vc_dfsr;
  reg                       vc_dfsr;
  wire                      set_trap_dfsr;
  wire                      trap_dfsr_we;
  wire                      nxt_trap_dfsr;
  reg                       trap_dfsr;
  wire                      set_bkpt_dfsr;
  wire                      bkpt_dfsr_we;
  wire                      nxt_bkpt_dfsr;
  reg                       bkpt_dfsr;
  wire                      set_halt_dfsr;
  wire                      halt_dfsr_we;
  wire                      nxt_halt_dfsr;
  reg                       halt_dfsr;

  wire [31:0]               set_afsr;
  wire [31:0]               clr_afsr;
  wire [31:0]               nxt_afsr;
  wire                      afsr_we;

  wire                      fault_addr;
  wire [3:0]                fault_addr_wr;
  wire                      fault_addr_up;
  wire                      fault_addr_we;
  reg  [31:0]               mm_bf_far;
  wire [31:0]               nxt_fa;

  reg  [2:0]                aux_ctrl0;
  wire                      aux_ctrl0_we;
  reg  [1:0]                aux_ctrl1;
  wire                      aux_ctrl1_we;
  wire [4:0]                aux_ctrl;

  wire                      icdsc_we;
  wire                      nxt_dhcs_c_debugen;
  reg                       opt_dhcs_c_debugen;
  wire                      dbg_dbgen;
  wire                      dhcs_c_debugen;
  reg                       opt_dhcs_c_halt;
  wire                      dhcs_c_halt;
  wire                      dhcs_c_halt_ws;
  wire                      dhcs_c_halt_set;
  wire                      dhcs_c_halt_clr;
  wire                      dhcs_c_halt_we;
  wire                      nxt_dhcs_c_halt;
  wire                      set_halt_req;
  wire                      nxt_dbg_halt_req;
  wire                      dbg_halt_req_we;
  reg                       dbg_halt_req_reg;
  wire                      dbg_halt_req;
  reg                       opt_dhcs_c_step;
  wire                      dhcs_c_step;
  wire                      nxt_dhcs_c_mask_ints;
  reg                       opt_dhcs_c_mask_ints;
  wire                      dhcs_c_mask_ints;
  wire                      dhcs_c_snapstall_we;
  reg                       opt_dhcs_c_snapstall;
  wire                      dhcs_c_snapstall;
  wire                      nxt_dbg_mask_ints;
  reg                       opt_dbg_mask_ints;
  wire                      dbg_mask_ints;
  reg                       opt_dhcs_s_reset_st;
  wire                      dhcs_s_reset_st;
  wire                      nxt_dhcs_s_retire_st;
  wire                      dhcs_s_retire_st_we;
  reg                       opt_dhcs_s_retire_st;
  wire                      dhcs_s_retire_st;
  reg  [1:0]                lockup;
  wire                      set_lockup;
  wire                      clr_lockup;
  wire                      lockup_we;
  wire                      nxt_icdsc_rd_clr;
  reg                       opt_icdsc_rd_clr;
  wire                      icdsc_rd_clr;
  wire                      icdsc_rd_clr_we;
  wire [7:0]                vector_evt;
  wire [1:0]                vector_catch;

  wire                      icdrs_we;
  wire                      dcrs_rtrans;
  reg                       opt_dcrs_rtrans;
  wire                      dcrs_rtrans_clr;
  wire                      dcrs_rtrans_we;
  reg                       opt_dcrs_regwnr;
  reg  [6:0]                opt_dcrs_regsel;
  wire                      dcrs_regwnr;
  wire [6:0]                dcrs_regsel;

  wire [3:0]                dcrd_we;
  reg  [31:0]               dbg_dcrd;
  wire [31:0]               dbg_tmp;
  wire [31:0]               nxt_dbg_tmp;

  wire [3:0]                icdemc_we;
  reg                       demc_vc_corereset;
  reg                       demc_vc_mmerr;
  reg                       demc_vc_nocperr;
  reg                       demc_vc_chkerr;
  reg                       demc_vc_staterr;
  reg                       demc_vc_buserr;
  reg                       demc_vc_interr;
  reg                       demc_vc_hard_err;
  wire [7:0]                dbg_vcatch;
  reg                       demc_mon_en;
  reg                       demc_mon_step;
  reg                       demc_mon_req;
  reg                       dbg_mon_req;
  wire                      dbg_mon_allow_step;
  wire                      dbg_mon_req_we;
  wire                      nxt_dbg_mon_req;
  wire                      dbg_mon_req_set;
  wire                      dbg_mon_req_clr;
  reg                       opt_demc_trcena;
  wire                      demc_trcena;
  wire                      dbg_mon_pend;
  wire                      dbg_mon_p_clr;
  wire                      dbg_step;
  wire                      dbg_step_req;
  wire                      dbg_allow_step;
  wire                      vect_clr_actv;

  wire                      stick_ctrl_en;
  wire                      stick_cnt_flag_set;
  wire                      stick_cnt_flag_clr;
  wire                      stick_cnt_flag_en;
  wire [2:0]                stick_reload_en;
  wire                      sys_tick_en;
  reg                       sys_tick_q;
  reg                       sys_tick_qq;
  reg                       sys_tick_qqq;
  wire                      stickref_pulse;
  wire                      stick_count_set;
  wire                      stick_count_clr;
  wire                      stick_count_we;
  wire [1:0]                stick_is_zero;
  wire                      stick_to_zero;
  wire [23:0]               nxt_stick_count;
  wire                      stick_int_we;
  wire                      nxt_stick_int;
  wire [255:0]              hwdata_vec;
  reg                       stick_clk_src_reg;
  wire                      stick_clk_src;

  wire [3:0]                fpccr_we;
  reg                       fpccr_aspen;
  reg                       fpccr_lspen;
  reg                       fpccr_monrdy;
  reg                       fpccr_bfrdy;
  reg                       fpccr_mmrdy;
  reg                       fpccr_hfrdy;
  reg                       fpccr_thread;
  reg                       fpccr_user;
  reg                       fpccr_lspact;
  wire                      nxt_fpccr_monrdy;
  wire                      fpccr_monrdy_we;
  wire                      nxt_fpccr_bfrdy;
  wire                      fpccr_bfrdy_we;
  wire                      nxt_fpccr_mmrdy;
  wire                      fpccr_mmrdy_we;
  wire                      nxt_fpccr_hfrdy;
  wire                      fpccr_hfrdy_we;
  wire                      nxt_fpccr_thread;
  wire                      fpccr_thread_we;
  wire                      nxt_fpccr_user;
  wire                      fpccr_user_we;
  wire                      nxt_fpccr_lspact;
  wire                      fpccr_lspact_we;
  wire [31:0]               fpccr;

  reg  [28:0]               fpcar_min;
  wire [28:0]               nxt_fpcar_ppb;
  wire [28:0]               nxt_fpcar_min;
  wire [3:0]                fpcar_we;
  wire                      fpcar_min_we;
  wire [31:0]               fpcar;

  wire [1:0]                fpdscr_we;
  reg  [4:0]                fpdscr;

  reg                       dbg_restart_q;        // Sampled dbg_restart
  reg                       dbg_restarted_reg;    // Register for handshake output
  wire                      nxt_dbg_restarted;    // nxt state for iDbgRestarted
  wire                      dbg_restarted;        // Handshake output
  wire                      restart_done;         // Handshaking completed
  wire                      ext_halt_clr;         // Halt clear by external
  reg                       dbg_restart_flag;     // Flag to limit one unhalt per handshake
  wire                      nxt_dbg_restart_flag; // nxt state of dbg_restart_flag

  wire [NUM_INTS-1:0]       int_pend_set;
  wire [NUM_INTS-1:0]       int_pend_clr;
  wire [NUM_INTS-1:0]       int_actv_set;
  wire [NUM_INTS-1:0]       int_actv_clr;
  reg  [NUM_IRQ-1:0]        int_set_pend_irq;
  wire [NUM_IRQ-1:0]        set_pend_irq;
  wire [255:0]              sw_int_reg_pend;
  reg  [NUM_IRQ-1:0]        irq_pend_clr;

  wire                      cpacr_we;
  reg  [3:0]                cpacr;

  integer                   l0;
  integer                   l1;
  integer                   l2;
  integer                   l3;
  integer                   l4;
  integer                   l5;
  integer                   l6;

  wire                      opt_dbg_en;
  wire                      opt_wic_en;
  wire                      opt_wic_load;
  wire                      opt_wic_clear;
  wire                      opt_cp_en;
  wire [31:0]               opt_fpccr;
  wire [31:0]               opt_fpcar;
  wire [4:0]                opt_fpdscr;
  wire [3:0]                opt_cpacr;

  wire                      mst_cp_en;
  wire                      wic_load;
  reg                       wic_clear;
  wire                      nxt_wic_clear;
  wire                      wic_clear_we;
  wire                      full_dreset_n;

  reg                       reset_init;

`ifdef ARM_DSM

  wire [7:0]                dsm_num_irq;
  wire [3:0]                dsm_lvl_width;
  wire [1:0]                dsm_debug_lvl;
  wire                      dsm_reset_all_regs;
  wire                      dsm_wic_present;
  wire                      dsm_cp_present;
  reg  [239:0]              dsm_imask;
  reg  [7:0]                dsm_pmask;
  wire [(240*8-1):0]        dsm_lmask;
  integer                   j;
  integer                   k;

  assign full_dreset_n = dsm_reset_all_regs  ? dreset_n    : 1'b1;
  assign opt_dbg_en    = (dsm_debug_lvl > 0) ? 1'b1        : 1'b0;
  assign opt_wic_en    = dsm_wic_present     ? 1'b1        : 1'b0;
  assign opt_wic_load  = dsm_wic_present     ? wic_load    : 1'b0;
  assign opt_wic_clear = dsm_wic_present     ? wic_clear   : 1'b0;
  assign opt_cp_en     = dsm_cp_present      ? 1'b1        : 1'b0;
  assign opt_fpccr     = dsm_cp_present      ? fpccr       : {32{1'b0}};
  assign opt_fpcar     = dsm_cp_present      ? fpcar       : {32{1'b0}};
  assign opt_fpdscr    = dsm_cp_present      ? fpdscr      : {5{1'b0}};
  assign opt_cpacr     = dsm_cp_present      ? cpacr       : 4'b0000;

  always @ (dsm_num_irq)
    for (j = 0; j < 240; j = j + 1)
      dsm_imask[j] = (j < dsm_num_irq);

  always @ (dsm_lvl_width)
    for (k = 0; k < 8; k = k + 1)
      dsm_pmask[k] = ((dsm_lvl_width == 8) | (k > (7 - dsm_lvl_width)));

  assign dsm_lmask     = {240{dsm_pmask}};

`else

  assign full_dreset_n = (RESET_ALL_REGS == 1) ? dreset_n  : 1'b1;

  assign opt_dbg_en    = (DEBUG_LVL > 0)       ? 1'b1      : 1'b0;

  assign opt_wic_en    = (WIC_PRESENT != 0)    ? 1'b1      : 1'b0;
  assign opt_wic_load  = (WIC_PRESENT != 0)    ? wic_load  : 1'b0;
  assign opt_wic_clear = (WIC_PRESENT != 0)    ? wic_clear : 1'b0;

  assign opt_cp_en     = (CP_PRESENT != 0)     ? 1'b1      : 1'b0;
  assign opt_fpccr     = (CP_PRESENT != 0)     ? fpccr     : {32{1'b0}};
  assign opt_fpcar     = (CP_PRESENT != 0)     ? fpcar     : {32{1'b0}};
  assign opt_fpdscr    = (CP_PRESENT != 0)     ? fpdscr    : {5{1'b0}};
  assign opt_cpacr     = (CP_PRESENT != 0)     ? cpacr     : 4'b0000;

`endif

  assign mst_cp_en     = opt_cp_en & ~ext_cp_disable_i;

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      reset_init <= 1'b0;
    else if (~reset_init)
      reset_init <= 1'b1;

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      data_be8   <= 1'b0;
    else if (~reset_init)
      data_be8   <= ext_big_end_i;


  always @ (ppb_we_i or ppb_reg_addr_i)
    for (l0 = 0; l0 < NUM_IRQ; l0 = l0 + 1)
      irq_en_set[l0] = (ppb_we_i[(l0 % 32)/8] & (ppb_reg_addr_i == `NVIC_INTSET0 + (l0/32)));

  always @ (ppb_we_i or ppb_reg_addr_i)
    for (l1 = 0; l1 < NUM_IRQ; l1 = l1 + 1)
      irq_en_clr[l1] = (ppb_we_i[(l1 % 32)/8] & (ppb_reg_addr_i == `NVIC_INTCLR0 + (l1/32)));

  assign irq_int_we = irq_en_set | irq_en_clr;

  assign hwdata_vec = {8{mtx_ppb_hwdata_i[31:0]}};

  assign nxt_irq_en = (// Setting
                       (irq_en_set & (irq_en_reg | hwdata_vec[NUM_IRQ-1:0]))  |
                       (irq_en_clr & (irq_en_reg & ~hwdata_vec[NUM_IRQ-1:0])) |
                       (~irq_int_we & irq_en_reg));

  assign irq_int_cg_en = |irq_int_we;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      irq_en_reg <= {NUM_IRQ{1'b0}};
    else if (irq_int_cg_en)
      irq_en_reg <= nxt_irq_en;

`ifdef ARM_DSM
  assign irq_en = irq_en_reg & dsm_imask;
`else
  assign irq_en = irq_en_reg;
`endif


  always @ (ppb_we_i or ppb_reg_addr_i or mtx_ppb_hwdata_i)
    for (l2 = 0; l2 < NUM_IRQ; l2 = l2 + 1)
      int_set_pend_irq[l2] = (ppb_we_i[(l2 % 32)/8] &
                              (ppb_reg_addr_i == `NVIC_INTPENDSET0 + (l2/32)) &
                              mtx_ppb_hwdata_i[l2 % 32]);

  assign sw_int_reg_pend = ({{255{1'b0}},
                             ((|ppb_we_i) & (ppb_reg_addr_i==`NVIC_SWINTREG) &
                              (~(|mtx_ppb_hwdata_i[31:8])))} <<
                            (mtx_ppb_hwdata_i[7:0] & {8{|ppb_we_i}}));

  assign set_pend_irq = int_set_pend_irq | sw_int_reg_pend[NUM_IRQ-1:0];

  always @ (ppb_we_i or ppb_reg_addr_i or mtx_ppb_hwdata_i)
    for (l3 = 0; l3 < NUM_IRQ; l3 = l3 + 1)
      irq_pend_clr[l3] = (ppb_we_i[(l3 % 32)/8] &
                          (ppb_reg_addr_i == `NVIC_INTPENDCLR0 + (l3/32)) &
                          mtx_ppb_hwdata_i[l3 % 32]);


  always @ (ppb_we_i or ppb_reg_addr_i)
    for (l4 = 0; l4 < NUM_IRQ; l4 = l4 + 1)
      irq_pri_lvl_we[l4] = (ppb_we_i[l4 % 4] &
                            (ppb_reg_addr_i == `NVIC_INTPRILVL0 + (l4/4)));

  assign irq_wdata = { mtx_ppb_hwdata_i[31:32-LVL_WIDTH],
                       mtx_ppb_hwdata_i[23:24-LVL_WIDTH],
                       mtx_ppb_hwdata_i[15:16-LVL_WIDTH],
                       mtx_ppb_hwdata_i[7:8-LVL_WIDTH] };

  always @ (irq_pri_lvl_we or irq_wdata or irq_lvl)
    for (l5 = 0; l5 < NUM_IRQ; l5 = l5 + 1)
      for (l6 = 0; l6 < LVL_WIDTH; l6 = l6 + 1)
        nxt_irq_lvl[(l5*LVL_WIDTH)+l6] =
              (irq_pri_lvl_we[l5] ? irq_wdata[((l5%4)*LVL_WIDTH)+l6] :
                                    irq_lvl[(l5*LVL_WIDTH)+l6]);

  assign irq_lvl_we = (|irq_pri_lvl_we);

  always @ (posedge pclk or negedge hreset_n)
    if (!hreset_n)
      irq_lvl_reg <= {(NUM_IRQ*LVL_WIDTH){1'b0}};
    else if (irq_lvl_we)
      irq_lvl_reg <= nxt_irq_lvl;

`ifdef ARM_DSM
  assign irq_lvl = irq_lvl_reg & dsm_lmask;
`else
  assign irq_lvl = irq_lvl_reg;
`endif


  assign int_ctrl_state_we = ppb_we_i[3] & (ppb_reg_addr_i==`NVIC_INTCTRLSTAT);

  assign nmi_pend_set      = int_ctrl_state_we & mtx_ppb_hwdata_i[31];
  assign psv_pend_set      = int_ctrl_state_we & mtx_ppb_hwdata_i[28];
  assign psv_pend_clr      = int_ctrl_state_we & mtx_ppb_hwdata_i[27];
  assign st_pend_set       = int_ctrl_state_we & mtx_ppb_hwdata_i[26];
  assign st_pend_clr       = int_ctrl_state_we & mtx_ppb_hwdata_i[25];


  assign vto_tbloff_we     = ppb_we_i & {4{ppb_reg_addr_i==`NVIC_INTTBLOFF}};

  assign nxt_vto_tbloff    = { vto_tbloff_we[3] ? mtx_ppb_hwdata_i[31:24] : vto_tbloff[31:24],
                               vto_tbloff_we[2] ? mtx_ppb_hwdata_i[23:16] : vto_tbloff[23:16],
                               vto_tbloff_we[1] ? mtx_ppb_hwdata_i[15:8]  : vto_tbloff[15:8],
                               vto_tbloff_we[0] ? mtx_ppb_hwdata_i[7]     : vto_tbloff[7]};

  always @ (posedge pclk or negedge hreset_n)
    if (!hreset_n)
      vto_tbloff <= {25{1'b0}};
    else if (|vto_tbloff_we)
      vto_tbloff <= nxt_vto_tbloff;


  assign airc_we = ((&ppb_we_i) & (ppb_reg_addr_i==`NVIC_APPIRC) &
                    (mtx_ppb_hwdata_i[31:16]==`NVIC_INTCTRLKEY));

  assign airc_vectreset_en  = airc_we | ~airc_vectreset;
  assign nxt_airc_vectreset = ~mtx_ppb_hwdata_i[0] | ~airc_vectreset;

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      airc_vectreset <= 1'b1;
    else if (airc_vectreset_en)
      airc_vectreset <= nxt_airc_vectreset;

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      begin
        airc_sysresetreq <= 1'b0;
        airc_prigroup    <= 3'b000;
      end
    else if (airc_we)
      begin
        airc_sysresetreq <= mtx_ppb_hwdata_i[2];
        airc_prigroup    <= mtx_ppb_hwdata_i[10:8];
      end


  assign sys_ctrl_we = ppb_we_i[0] & (ppb_reg_addr_i==`NVIC_SYSCTRL);

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      begin
        scr_sleeponexit <= 1'b0;
        scr_deepsleep   <= 1'b0;
        scr_wfeonpend   <= 1'b0;
      end
    else if (sys_ctrl_we)
      begin
        scr_sleeponexit <= mtx_ppb_hwdata_i[1];
        scr_deepsleep   <= mtx_ppb_hwdata_i[2];
        scr_wfeonpend   <= mtx_ppb_hwdata_i[4];
      end


  assign slp_wfe_new_pend  = scr_wfeonpend & slp_wfe_pend_i;

  assign slp_event_reg_set = (~slp_event_reg & ~dpu_slp_ctl_i[1] &
                              ~slp_sleep_type[1] &
                              (ext_slp_rxev_i   | ext_slp_txev_i  |
                               slp_wfe_new_pend | dpu_int_entry_i |
                               dpu_int_exit_i   | dhcs_c_halt));
  assign slp_event_reg_clr = slp_event_reg & dpu_slp_ctl_i[1];
  assign slp_event_reg_we  = slp_event_reg_set | slp_event_reg_clr;

  always @ (negedge hreset_n or posedge fclk)
    if (!hreset_n)
      slp_event_reg <= 1'b0;
    else if (slp_event_reg_we)
      slp_event_reg <= slp_event_reg_set;


  assign slp_normal_en = (// sleeponexit when last interrupt pops to reset_isr
                          (slp_sleeponexit_i & dpu_int_exit_i)                    |
                          (dpu_slp_ctl_i[0] & ~slp_normal & ~int_invoke_i &
                           ~slp_pmask_pend_i)                                     |
                          (dpu_slp_ctl_i[1] & ~slp_normal & ~int_invoke_i &
                           ~ext_slp_rxev_i & ~ext_slp_txev_i & ~slp_wfe_new_pend &
                           ~slp_event_reg)                                        |
                          ((ext_slp_rxev_i | ext_slp_txev_i) & slp_sleep_type[1]) |
                          (slp_wfe_new_pend & slp_sleep_type[1])                  |
                          (slp_norm_wake_i & slp_normal)                          |
                          (slp_sleep_type[0] & slp_pmask_pend_i)                  |
                          (dhcs_c_halt & slp_normal));

  assign nxt_slp_normal = ({1'b0, slp_sleeponexit_i & dpu_int_exit_i} |
                           dpu_slp_ctl_i[1:0]);

  always @ (negedge hreset_n or posedge fclk)
    if (!hreset_n)
      begin
        slp_sleep_type <= 2'b00;
        slp_normal     <= 1'b0;
      end
    else if (slp_normal_en)
      begin
        slp_sleep_type <= nxt_slp_normal;
        slp_normal     <= |nxt_slp_normal;
      end

  assign slp_wfe_valid = nxt_slp_normal[1] & slp_normal_en;

  assign slp_wfi_valid = nxt_slp_normal[0] & slp_normal_en;

  assign slp_forced_set = (slp_sleeping & ~ext_slp_hold_req_n_i & ~slp_forced &
                           ~int_invoke_i & ~(slp_normal_en & ~(|nxt_slp_normal)));

  assign slp_forced_clr = (slp_forced &
                           (ext_slp_hold_req_n_i | int_invoke_i));

  assign slp_forced_en  = slp_forced_set | slp_forced_clr;

  always @ (negedge hreset_n or posedge fclk)
    if (!hreset_n)
      slp_forced <= 1'b0;
    else if (slp_forced_en)
      slp_forced <= slp_forced_set;

  always @ (negedge hreset_n or posedge fclk)
    if (!hreset_n)
      slp_hold_ack_q <= 1'b0;
    else if (~slp_hold_ack_n^slp_hold_ack_q)
      slp_hold_ack_q <= ~slp_hold_ack_n;

  assign slp_asleep     = ((slp_normal & ~int_invoke_i) |
                           ~slp_hold_ack_n | slp_hold_ack_q);

  assign slp_sleeping   = (slp_normal & ~int_invoke_i &
                           ~dpu_ahb_htransi_iss_i & ~dpu_cp_busy_i);
  assign slp_sleepdeep  = scr_deepsleep & slp_sleeping;
  assign slp_hold_ack_n = ~(slp_forced & ~int_invoke_i);

  assign wic_load       = (// Load WIC if core is in WIC mode
                           opt_wic_en & ~wic_ds_ack_n_i & ~wic_ds_req_n_i &
                           slp_normal_en & scr_deepsleep & (|nxt_slp_normal));

  assign nxt_wic_clear  = (// Clear WIC if core is in WIC mode
                           opt_wic_en & ~wic_ds_ack_n_i & scr_deepsleep &
                           (// and is coming out of deep sleep and not staying
                            (slp_normal & slp_normal_en & ~(|nxt_slp_normal) &
                             (~slp_forced | slp_forced_clr)) |
                            (~slp_normal & slp_forced & slp_forced_clr) |
                            (slp_normal & slp_forced & slp_forced_clr &
                             (slp_normal_en & ~(|nxt_slp_normal)))));

  assign wic_clear_we   = (opt_wic_en & (nxt_wic_clear | wic_clear));

  always @ (negedge hreset_n or posedge fclk)
    if (!hreset_n)
      wic_clear <= 1'b0;
    else if (wic_clear_we)
      wic_clear <= nxt_wic_clear;


  assign config_ctrl_we = ppb_we_i[1:0] & {2{ppb_reg_addr_i==`NVIC_CONFIGCTRL}};

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      begin
        ccr_nonbasethrdena <= 1'b0;
        ccr_usersetmpend   <= 1'b0;
        ccr_unalign_trp    <= 1'b0;
        ccr_div_0_trp      <= 1'b0;
      end
    else if (config_ctrl_we[0])
      begin
        ccr_nonbasethrdena <= mtx_ppb_hwdata_i[0];
        ccr_usersetmpend   <= mtx_ppb_hwdata_i[1];
        ccr_unalign_trp    <= mtx_ppb_hwdata_i[3];
        ccr_div_0_trp      <= mtx_ppb_hwdata_i[4];
      end

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      ccr_bfhfnmign <= 1'b0;
    else if (config_ctrl_we[1])
      ccr_bfhfnmign <= mtx_ppb_hwdata_i[8];

  assign ccr_stkalign_we  = config_ctrl_we[1] | ~reset_init;
  assign nxt_ccr_stkalign = (config_ctrl_we[1] ? mtx_ppb_hwdata_i[9] :
                                                 ext_stkalign_init_i);

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      ccr_stkalign <= 1'b1;
    else if (ccr_stkalign_we)
      ccr_stkalign <= nxt_ccr_stkalign;

  assign aux_ctrl0_we = ppb_we_i[0] & {ppb_reg_addr_i==`NVIC_AUXCTRL};
  assign aux_ctrl1_we = ppb_we_i[1] & {ppb_reg_addr_i==`NVIC_AUXCTRL} & mst_cp_en;

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      aux_ctrl0 <= 3'b000;
    else if (aux_ctrl0_we)
      aux_ctrl0 <= mtx_ppb_hwdata_i[2:0];

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      aux_ctrl1 <= 2'b00;
    else if (aux_ctrl1_we)
      aux_ctrl1 <= mtx_ppb_hwdata_i[9:8];

  assign aux_ctrl = { aux_ctrl1[1] & mst_cp_en,    // [4] DISOOO
                      aux_ctrl1[0] & mst_cp_en,    // [3] DISLAZY
                      aux_ctrl0[2],                // [2] DISFOLD
                      aux_ctrl0[1],                // [1] DISDEFWBUF
                      aux_ctrl0[0]};               // [0] DISMCYCINT


  assign shp_mm_lvl_we = ppb_we_i[0] & (ppb_reg_addr_i==`NVIC_SYSHNDLRPRI0);

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      shp_mm_lvl <= {LVL_WIDTH{1'b0}};
    else if (shp_mm_lvl_we)
      shp_mm_lvl <= mtx_ppb_hwdata_i[7:8-LVL_WIDTH];

  assign shp_dm_lvl_we = ppb_we_i[0] & (ppb_reg_addr_i==`NVIC_SYSHNDLRPRI2);

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      shp_dm_lvl <= {LVL_WIDTH{1'b0}};
    else if (shp_dm_lvl_we)
      shp_dm_lvl <= mtx_ppb_hwdata_i[7:8-LVL_WIDTH];

  assign shp_bf_lvl_we = ppb_we_i[1] & (ppb_reg_addr_i==`NVIC_SYSHNDLRPRI0);

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      shp_bf_lvl <= {LVL_WIDTH{1'b0}};
    else if (shp_bf_lvl_we)
      shp_bf_lvl <= mtx_ppb_hwdata_i[15:16-LVL_WIDTH];

  assign shp_uf_lvl_we = ppb_we_i[2] & (ppb_reg_addr_i==`NVIC_SYSHNDLRPRI0);

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      shp_uf_lvl <= {LVL_WIDTH{1'b0}};
    else if (shp_uf_lvl_we)
      shp_uf_lvl <= mtx_ppb_hwdata_i[23:24-LVL_WIDTH];

  assign shp_psv_lvl_we = ppb_we_i[2] & (ppb_reg_addr_i==`NVIC_SYSHNDLRPRI2);

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      shp_psv_lvl <= {LVL_WIDTH{1'b0}};
    else if (shp_psv_lvl_we)
      shp_psv_lvl <= mtx_ppb_hwdata_i[23:24-LVL_WIDTH];

  assign shp_svc_lvl_we = ppb_we_i[3] & (ppb_reg_addr_i==`NVIC_SYSHNDLRPRI1);

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      shp_svc_lvl <= {LVL_WIDTH{1'b0}};
    else if (shp_svc_lvl_we)
      shp_svc_lvl <= mtx_ppb_hwdata_i[31:32-LVL_WIDTH];

  assign shp_st_lvl_we = ppb_we_i[3] & (ppb_reg_addr_i==`NVIC_SYSHNDLRPRI2);

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      shp_st_lvl <= {LVL_WIDTH{1'b0}};
    else if (shp_st_lvl_we)
      shp_st_lvl <= mtx_ppb_hwdata_i[31:32-LVL_WIDTH];

`ifdef ARM_DSM
  assign opt_shp_mm_lvl  = shp_mm_lvl  & dsm_pmask;
  assign opt_shp_dm_lvl  = shp_dm_lvl  & dsm_pmask;
  assign opt_shp_bf_lvl  = shp_bf_lvl  & dsm_pmask;
  assign opt_shp_uf_lvl  = shp_uf_lvl  & dsm_pmask;
  assign opt_shp_psv_lvl = shp_psv_lvl & dsm_pmask;
  assign opt_shp_svc_lvl = shp_svc_lvl & dsm_pmask;
  assign opt_shp_st_lvl  = shp_st_lvl  & dsm_pmask;
`else
  assign opt_shp_mm_lvl  = shp_mm_lvl;
  assign opt_shp_dm_lvl  = shp_dm_lvl;
  assign opt_shp_bf_lvl  = shp_bf_lvl;
  assign opt_shp_uf_lvl  = shp_uf_lvl;
  assign opt_shp_psv_lvl = shp_psv_lvl;
  assign opt_shp_svc_lvl = shp_svc_lvl;
  assign opt_shp_st_lvl  = shp_st_lvl;
`endif


  assign shcs_we = ppb_we_i[2:0] & {3{ppb_reg_addr_i==`NVIC_SYSHNDLREN}};

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      begin
        shcs_mm_en <= 1'b0;
        shcs_bf_en <= 1'b0;
        shcs_uf_en <= 1'b0;
      end
    else if (shcs_we[2])
      begin
        shcs_mm_en <= mtx_ppb_hwdata_i[16];
        shcs_bf_en <= mtx_ppb_hwdata_i[17];
        shcs_uf_en <= mtx_ppb_hwdata_i[18];
      end


  assign clr_fsr = {
    {2{ppb_we_i[3] & ppb_reg_addr_i==`NVIC_FAULTSTAT}} & mtx_ppb_hwdata_i[25:24],
    {4{ppb_we_i[2] & ppb_reg_addr_i==`NVIC_FAULTSTAT}} & mtx_ppb_hwdata_i[19:16],
    {7{ppb_we_i[1] & ppb_reg_addr_i==`NVIC_FAULTSTAT}} &
    {mtx_ppb_hwdata_i[15], mtx_ppb_hwdata_i[13:8]},
    {6{ppb_we_i[0] & ppb_reg_addr_i==`NVIC_FAULTSTAT}} &
    {mtx_ppb_hwdata_i[7], mtx_ppb_hwdata_i[5:3], mtx_ppb_hwdata_i[1:0]}};

  assign set_fsr = {// UFSR
                    dpu_nxt_ufsr_i[9:8], dpu_nxt_ufsr_i[3],
                    dpu_nxt_ufsr_i[2] | int_non_actv_exit_i, dpu_nxt_ufsr_i[1:0],
                    dpu_nxt_bfsr_i[1],
                    {dpu_nxt_bfsr_i[5:3],
                     dpu_nxt_bfsr_i[2] | mtx_wb_abort_i,
                     dpu_nxt_bfsr_i[1:0]},
                    dpu_nxt_mfsr_i[1], dpu_nxt_mfsr_i[4:0]};

  assign nxt_fsr = set_fsr | (~clr_fsr & fsr_min);

  assign fsr_we  = (|clr_fsr) | (|set_fsr);

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      fsr_min <= {19{1'b0}};
    else if (fsr_we)
      fsr_min <= nxt_fsr;

  assign fsr = {// UFSR
                6'b000000, fsr_min[18:17], 4'b0000, fsr_min[16:13],
                fsr_min[12], 1'b0, fsr_min[11:6],
                fsr_min[5], 1'b0, fsr_min[4:2], 1'b0, fsr_min[1:0]};



  assign clr_afsr = {
    {8{ppb_we_i[3] & ppb_reg_addr_i==`NVIC_AUXFAULTSTAT}} & mtx_ppb_hwdata_i[31:24],
    {8{ppb_we_i[2] & ppb_reg_addr_i==`NVIC_AUXFAULTSTAT}} & mtx_ppb_hwdata_i[23:16],
    {8{ppb_we_i[1] & ppb_reg_addr_i==`NVIC_AUXFAULTSTAT}} & mtx_ppb_hwdata_i[15:8],
    {8{ppb_we_i[0] & ppb_reg_addr_i==`NVIC_AUXFAULTSTAT}} & mtx_ppb_hwdata_i[7:0]};

  assign set_afsr = ext_aux_fault_i;

  assign nxt_afsr = set_afsr | (~clr_afsr & afsr);

  assign afsr_we  = (|clr_afsr) | (|set_afsr);

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      afsr <= {32{1'b0}};
    else if (afsr_we)
      afsr <= nxt_afsr;


  assign clr_hfsr = {
    {2{ppb_we_i[3] & ppb_reg_addr_i==`NVIC_HARDFAULTSTAT}} & mtx_ppb_hwdata_i[31:30],
    {1{ppb_we_i[0] & ppb_reg_addr_i==`NVIC_HARDFAULTSTAT}} & mtx_ppb_hwdata_i[1]};

  assign set_hfsr = {
    int_hfault_i & (dpu_bkpt_i & ~dbg_mon_allow_i),
    ((int_hfault_i & (dbg_mon_allow_i | ~dpu_bkpt_i) & int_escalate_i) | int_lazy_esc_i),
    dpu_nxt_hfsr_i};

  assign nxt_hfsr = (set_hfsr | (~clr_hfsr & hfsr_min));

  assign hfsr_we  = (|clr_hfsr) | (|set_hfsr);

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      hfsr_min <= 3'b000;
    else if (hfsr_we)
      hfsr_min <= nxt_hfsr;

  assign hfsr = { hfsr_min[2:1], {28{1'b0}}, hfsr_min[0], 1'b0 };


  assign mask_dfsr       = (// Halting event
                            ( dbg_dbgen & ~dpu_dbg_halted_i) |
                            (~dbg_dbgen & demc_mon_en & dbg_mon_fault_i &
                             ~dbg_mon_active_i));

  assign mask_async_dfsr = (// Halting event
                            ( dbg_dbgen & ~dpu_dbg_halted_i) |
                            (~dbg_dbgen & demc_mon_en &
                             (dwt_watchpoint_i | ext_dbg_req_i) &
                             dbg_mon_allow_i & ~dbg_mon_active_i));

  assign dfsr_write      = ({5{ppb_we_i[0] & ppb_reg_addr_i==`NVIC_DBGFAULTSTAT}} &
                            mtx_ppb_hwdata_i[4:0]);

  assign set_ext_dfsr    = mask_async_dfsr & ext_dbg_req_i;
  assign ext_dfsr_we     = dfsr_write[4] | set_ext_dfsr;
  assign nxt_ext_dfsr    = (set_ext_dfsr | (~dfsr_write[4] & ext_dfsr));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      ext_dfsr <= 1'b0;
    else if (ext_dfsr_we)
      ext_dfsr <= nxt_ext_dfsr;

  assign set_vc_dfsr     = (dbg_dbgen & ~dpu_dbg_halted_i &
                            (vector_catch[0] | (dpu_int_entry_i & vector_catch[1])));
  assign vc_dfsr_we      = (dfsr_write[3] | set_vc_dfsr) & opt_dbg_en;
  assign nxt_vc_dfsr     = ((set_vc_dfsr) | (~dfsr_write[3] & vc_dfsr)) & opt_dbg_en;

  always @ (posedge hclk or negedge dreset_n)
    if (!dreset_n)
      vc_dfsr <= 1'b0;
    else if (vc_dfsr_we)
      vc_dfsr <= nxt_vc_dfsr;

  assign set_trap_dfsr   = ((mask_async_dfsr & dwt_watchpoint_i &
                             (dbg_dbgen | ~dpu_lazy_stk_i)) |
                            int_lazy_dm_i);
  assign trap_dfsr_we    = (dfsr_write[2] | set_trap_dfsr) & opt_dbg_en;
  assign nxt_trap_dfsr   = ((set_trap_dfsr) | (~dfsr_write[2] & trap_dfsr)) & opt_dbg_en;

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      trap_dfsr <= 1'b0;
    else if (trap_dfsr_we)
      trap_dfsr <= nxt_trap_dfsr;

  assign set_bkpt_dfsr   = dbg_masked_bkpt_i;
  assign bkpt_dfsr_we    = dfsr_write[1] | set_bkpt_dfsr;
  assign nxt_bkpt_dfsr   = ((set_bkpt_dfsr) | (~dfsr_write[1] & bkpt_dfsr));

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      bkpt_dfsr <= 1'b0;
    else if (bkpt_dfsr_we)
      bkpt_dfsr <= nxt_bkpt_dfsr;

  assign set_halt_dfsr   = ((icdsc_we & (&mtx_ppb_hwdata_i[1:0])) |
                            (mask_dfsr & (dbg_step_req | dbg_mon_step_pend_i)));
  assign halt_dfsr_we    = dfsr_write[0] | set_halt_dfsr;
  assign nxt_halt_dfsr   = (set_halt_dfsr | (~dfsr_write[0] & halt_dfsr));

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      halt_dfsr <= 1'b0;
    else if (halt_dfsr_we)
      halt_dfsr <= nxt_halt_dfsr;

  assign dfsr = {{27{1'b0}}, ext_dfsr, vc_dfsr, trap_dfsr, bkpt_dfsr, halt_dfsr};



  assign fault_addr    = ((ppb_reg_addr_i==`NVIC_BUSFAULTADDR) |
                          (ppb_reg_addr_i==`NVIC_MEMFAULTADDR));

  assign fault_addr_wr = {4{fault_addr}} & ppb_we_i;

  assign fault_addr_up = dpu_ahb_haddrd_valid_i & ~(nxt_fsr[12] | nxt_fsr[5]);

  assign nxt_fa        = {
    fault_addr_wr[3] ? mtx_ppb_hwdata_i[31:24] : (fault_addr_up ?  dpu_ahb_haddrd_i[31:24] : mm_bf_far[31:24]),
    fault_addr_wr[2] ? mtx_ppb_hwdata_i[23:16] : (fault_addr_up ?  dpu_ahb_haddrd_i[23:16] : mm_bf_far[23:16]),
    fault_addr_wr[1] ? mtx_ppb_hwdata_i[15:8]  : (fault_addr_up ?  dpu_ahb_haddrd_i[15:8]  : mm_bf_far[15:8]),
    fault_addr_wr[0] ? mtx_ppb_hwdata_i[7:0]   : (fault_addr_up ?  dpu_ahb_haddrd_i[7:0]   : mm_bf_far[7:0])};

  assign fault_addr_we = (|fault_addr_wr) | fault_addr_up;

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      mm_bf_far <= {32{1'b0}};
    else if (fault_addr_we)
      mm_bf_far <= nxt_fa;


  assign icdsc_we             = (ppb_we_i[0] & opt_dbg_en &
                                 (ppb_reg_addr_i==`NVIC_ICDSC) &
                                 (mtx_ppb_hwdata_i[31:16]==`NVIC_DEBUGKEY) &
                                 (ppb_we_i[3:1] == 3'b111));

  assign nxt_dhcs_c_debugen   = (ppb_hmaster_reg_i ? mtx_ppb_hwdata_i[0] :
                                                     dhcs_c_debugen);
  assign nxt_dhcs_c_mask_ints = ((dpu_dbg_halted_i & ~dhcs_c_halt_clr) ?
                                 mtx_ppb_hwdata_i[3] : dhcs_c_mask_ints);
  assign nxt_dbg_mask_ints    = nxt_dhcs_c_debugen & nxt_dhcs_c_mask_ints;

  always @ (negedge dreset_n or posedge pclk)
    if (!dreset_n)
      begin
        opt_dhcs_c_debugen   <= 1'b0;
        opt_dhcs_c_step      <= 1'b0;
        opt_dhcs_c_mask_ints <= 1'b0;
        opt_dbg_mask_ints    <= 1'b0;
      end
    else if (icdsc_we)
      begin
        opt_dhcs_c_debugen   <= nxt_dhcs_c_debugen;
        opt_dhcs_c_step      <= mtx_ppb_hwdata_i[2];
        opt_dhcs_c_mask_ints <= nxt_dhcs_c_mask_ints;
        opt_dbg_mask_ints    <= nxt_dbg_mask_ints;
      end

  assign dhcs_c_snapstall_we = (icdsc_we & opt_dbg_en & dhcs_c_halt_ws &
                                ~dhcs_s_retire_st);

  always @ (negedge dreset_n or posedge pclk)
    if (!dreset_n)
      opt_dhcs_c_snapstall <= 1'b0;
    else if (dhcs_c_snapstall_we)
      opt_dhcs_c_snapstall <= mtx_ppb_hwdata_i[5];

  assign dhcs_c_debugen   = opt_dhcs_c_debugen   & opt_dbg_en;
  assign dbg_dbgen        = (opt_dhcs_c_debugen  & opt_dbg_en &
                             (ext_dbg_en_i | dbg_halt_req));
  assign dhcs_c_step      = opt_dhcs_c_step      & opt_dbg_en;
  assign dhcs_c_mask_ints = opt_dhcs_c_mask_ints & opt_dbg_en & ext_dbg_en_i;
  assign dhcs_c_snapstall = opt_dhcs_c_snapstall & opt_dbg_en;
  assign dbg_mask_ints    = opt_dbg_mask_ints    & opt_dbg_en;

  assign vector_evt = ({8{dbg_dbgen}} & {{7{int_invoke_i}},1'b1} &
                       {// HARDERR
                        (int_nxt_isr_i==8'h03 & (hfsr[30] | hfsr[1])),
                        ((int_nxt_isr_i==8'h03 & (hfsr[1]))   |
                         (int_nxt_isr_i==8'h04 & (|fsr[4:3])) |
                         (int_nxt_isr_i==8'h05 & (|fsr[12:11]))),
                        (int_nxt_isr_i==8'h05 & (|fsr[10:8])),
                        (int_nxt_isr_i==8'h06 & (|fsr[18:16])),
                        (int_nxt_isr_i==8'h06 & (|fsr[25:24])),
                        (int_nxt_isr_i==8'h06 & (fsr[19])),
                        (int_nxt_isr_i==8'h04 & (|fsr[1:0])),
                        (~dpu_rst_pc_valid_i & reset_init)});

  assign vector_catch    =  (// Handler events
                             {(|(dbg_vcatch[7:1] & vector_evt[7:1])),
                              (dbg_vcatch[0] & vector_evt[0])});


  assign dhcs_c_halt_ws  = (// Write to register
                            icdsc_we & (&mtx_ppb_hwdata_i[1:0]) &
                            (ppb_hmaster_reg_i | dbg_dbgen));

  assign dhcs_c_halt_set = (// Enabled debug event
                            (dbg_dbgen &
                             (dbg_masked_bkpt_i    |
                              dwt_watchpoint_i     |
                              ext_dbg_req_i        |
                              dbg_step_req         |
                              vector_catch[0]      |
                              (dpu_int_entry_i & vector_catch[1]))) |
                            dhcs_c_halt_ws);

  assign dhcs_c_halt_clr = (icdsc_we & ~(&mtx_ppb_hwdata_i[1:0])) | ext_halt_clr;

  assign dhcs_c_halt_we  = (dhcs_c_halt_set | dhcs_c_halt_clr) & opt_dbg_en;

  assign nxt_dhcs_c_halt = ~dhcs_c_halt_clr;

  always @ (negedge hreset_n or posedge fclk)
    if (!hreset_n)
      opt_dhcs_c_halt <= 1'b0;
    else if (dhcs_c_halt_we)
      opt_dhcs_c_halt <= nxt_dhcs_c_halt;

  assign dhcs_c_halt      = opt_dhcs_c_halt & opt_dbg_en;

  assign set_halt_req     = ((ext_dbg_en_i | dpu_dbg_halted_i) &
                             slp_hold_ack_n & ~slp_forced_set &
                             (dhcs_c_halt_set | dhcs_c_halt | vector_catch[1]));
  assign nxt_dbg_halt_req = set_halt_req & ~dhcs_c_halt_clr & opt_dbg_en;

  assign dbg_halt_req_we  = (opt_dbg_en &
                             (nxt_dbg_halt_req != dbg_halt_req_reg) &
                             ~(dbg_halt_req & ~dpu_dbg_halted_i &
                               dpu_rst_pc_valid_i & ~int_invoke_i));

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      dbg_halt_req_reg <= 1'b0;
    else if (dbg_halt_req_we)
      dbg_halt_req_reg <= nxt_dbg_halt_req;

  assign dbg_halt_req      = dbg_halt_req_reg & opt_dbg_en;

  assign dbg_allow_step    = dbg_dbgen & dhcs_c_step & opt_dbg_en;
  assign dbg_step_req      = (opt_dbg_en & dbg_allow_step &
                              ~dhcs_c_halt_clr &
                              (dpu_ivalid_i | dpu_retire_i));

   assign nxt_icdsc_rd_clr = (opt_dbg_en & ppb_rd_i &
                              (ppb_reg_addr_i==`NVIC_ICDSC));

  assign icdsc_rd_clr_we   = (opt_dbg_en &
                             (icdsc_rd_clr | nxt_icdsc_rd_clr));

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      opt_icdsc_rd_clr <= 1'b0;
    else if (icdsc_rd_clr_we)
      opt_icdsc_rd_clr <= nxt_icdsc_rd_clr;

  assign icdsc_rd_clr = opt_icdsc_rd_clr & opt_dbg_en;

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      opt_dhcs_s_reset_st <= 1'b1;
    else if (icdsc_rd_clr)
      opt_dhcs_s_reset_st <= 1'b0;

  assign dhcs_s_reset_st = opt_dbg_en ? opt_dhcs_s_reset_st   : 1'b1;

  assign nxt_dhcs_s_retire_st = dpu_retire_i;
  assign dhcs_s_retire_st_we  = (dpu_retire_i | icdsc_rd_clr) & opt_dbg_en;

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      opt_dhcs_s_retire_st <= 1'b0;
    else if (dhcs_s_retire_st_we)
      opt_dhcs_s_retire_st <= nxt_dhcs_s_retire_st;

  assign dhcs_s_retire_st = opt_dbg_en ? opt_dhcs_s_retire_st : 1'b1;


  assign set_lockup = ((|int_in_nmihf_i[1:0]) &
                       ((int_allow_bf_i & ((|dpu_nxt_bfsr_i[1:0]) | dpu_nxt_bfsr_i[3])) |
                        (|dpu_nxt_ufsr_i[9:0])                                          |
                        (|dpu_nxt_mfsr_i[2:0])                                          |
                        (dpu_svc_i)                                                     |
                        (dpu_bkpt_i & ~dbg_dbgen)                                       |
                        ((dpu_nxt_bfsr_i[5] | dpu_nxt_mfsr_i[4]) & ~fpccr[4])           |
                        (int_in_nmihf_i[2] & (dpu_nxt_mfsr_i[3] |
                                              (dpu_nxt_bfsr_i[4] & int_allow_bf_i)))));

  assign clr_lockup = dpu_retire_i | dpu_dbg_halted_i;

  assign lockup_we  = set_lockup | clr_lockup;

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      lockup <= 2'b00;
    else if (lockup_we)
      lockup <= { dpu_lazy_stk_i, set_lockup };


  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      dbg_restart_q <= 1'b0;
    else if (opt_dbg_en)
      dbg_restart_q <= ext_dbg_restart_i;

  assign restart_done      = ~dpu_dbg_halted_i & ~dbg_restarted_reg & ~dbg_restart_q;

  assign nxt_dbg_restarted = restart_done | (~dbg_restart_q & dbg_restarted_reg);

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      dbg_restarted_reg <= 1'b1;
    else if (opt_dbg_en)
      dbg_restarted_reg <= nxt_dbg_restarted;

  assign nxt_dbg_restart_flag = (((dbg_restart_q & dbg_restarted_reg) |
                                  (dbg_restart_flag & ~dbg_restarted_reg)) &
                                 ~ext_halt_clr & dpu_dbg_halted_i);

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      dbg_restart_flag <= 1'b0;
    else if (opt_dbg_en)
      dbg_restart_flag <= nxt_dbg_restart_flag;

  assign ext_halt_clr  = (dhcs_c_halt & ~dcrs_rtrans_we & ~dbg_restarted_reg &
                          ~dbg_restart_q & dbg_restart_flag & opt_dbg_en);

  assign dbg_restarted = opt_dbg_en ? dbg_restarted_reg : 1'b1;


  assign icdrs_we        = ((&ppb_we_i) & (ppb_reg_addr_i==`NVIC_ICDRS) &
                            dpu_dbg_halted_i & dpu_dbg_ready_i);

  assign dcrs_rtrans_clr = opt_dcrs_rtrans & (dpu_dbg_ready_i | ~dpu_dbg_halted_i);

  assign dcrs_rtrans_we  = opt_dbg_en & (icdrs_we | dcrs_rtrans_clr);

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      opt_dcrs_rtrans <= 1'b0;
    else if (dcrs_rtrans_we)
      opt_dcrs_rtrans <= icdrs_we;

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      begin
        opt_dcrs_regwnr <= 1'b0;
        opt_dcrs_regsel <= 7'b0000000;
      end
    else if (icdrs_we & opt_dbg_en)
      begin
        opt_dcrs_regwnr <= mtx_ppb_hwdata_i[16];
        opt_dcrs_regsel <= mtx_ppb_hwdata_i[6:0];
      end

  assign dcrs_rtrans = opt_dcrs_rtrans & opt_dbg_en & ext_dbg_en_i;
  assign dcrs_regwnr = opt_dcrs_regwnr & opt_dbg_en;
  assign dcrs_regsel = opt_dcrs_regsel & {7{opt_dbg_en}};


  assign dcrd_we     = ppb_we_i & {4{ppb_reg_addr_i==`NVIC_ICDRD}} & {4{opt_dbg_en}};

  assign nxt_dbg_tmp = { dcrd_we[3] ? mtx_ppb_hwdata_i[31:24] : dbg_tmp[31:24],
                         dcrd_we[2] ? mtx_ppb_hwdata_i[23:16] : dbg_tmp[23:16],
                         dcrd_we[1] ? mtx_ppb_hwdata_i[15:8]  : dbg_tmp[15:8],
                         dcrd_we[0] ? mtx_ppb_hwdata_i[7:0]   : dbg_tmp[7:0]};

  always @ (posedge pclk or negedge full_dreset_n)
    if (!full_dreset_n)
      dbg_dcrd <= {32{1'b0}};
    else if (|dcrd_we)
      dbg_dcrd <= nxt_dbg_tmp;

  assign dbg_tmp = dbg_dcrd & {32{opt_dbg_en}};


  assign icdemc_we = ppb_we_i & {4{ppb_reg_addr_i==`NVIC_ICDEMC}};

  always @ (negedge dreset_n or posedge pclk)
    if (!dreset_n)
      begin
        demc_vc_corereset <= 1'b0;
        demc_vc_mmerr     <= 1'b0;
        demc_vc_nocperr   <= 1'b0;
        demc_vc_chkerr    <= 1'b0;
        demc_vc_staterr   <= 1'b0;
      end
    else if (icdemc_we[0] & opt_dbg_en)
      begin
        demc_vc_corereset <= mtx_ppb_hwdata_i[0];
        demc_vc_mmerr     <= mtx_ppb_hwdata_i[4];
        demc_vc_nocperr   <= mtx_ppb_hwdata_i[5];
        demc_vc_chkerr    <= mtx_ppb_hwdata_i[6];
        demc_vc_staterr   <= mtx_ppb_hwdata_i[7];
      end

  always @ (negedge dreset_n or posedge pclk)
    if (!dreset_n)
      begin
        demc_vc_buserr    <= 1'b0;
        demc_vc_interr    <= 1'b0;
        demc_vc_hard_err  <= 1'b0;
      end
    else if (icdemc_we[1] & opt_dbg_en)
      begin
        demc_vc_buserr    <= mtx_ppb_hwdata_i[8];
        demc_vc_interr    <= mtx_ppb_hwdata_i[9];
        demc_vc_hard_err  <= mtx_ppb_hwdata_i[10];
      end

  assign dbg_vcatch = ({8{opt_dbg_en}} &
                      { demc_vc_hard_err, demc_vc_interr,
                        demc_vc_buserr,   demc_vc_staterr,
                        demc_vc_chkerr,   demc_vc_nocperr,
                        demc_vc_mmerr,    demc_vc_corereset });

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      begin
        demc_mon_en       <= 1'b0;
        demc_mon_step     <= 1'b0;
        demc_mon_req      <= 1'b0;
      end
    else if (icdemc_we[2])
      begin
        demc_mon_en       <= mtx_ppb_hwdata_i[16];
        demc_mon_step     <= mtx_ppb_hwdata_i[18];
        demc_mon_req      <= mtx_ppb_hwdata_i[19];
      end

  assign dbg_mon_pend  = icdemc_we[2] &  mtx_ppb_hwdata_i[17];

  assign dbg_mon_p_clr = icdemc_we[2] & ~mtx_ppb_hwdata_i[17];

  always @ (negedge dreset_n or posedge pclk)
    if (!dreset_n)
      opt_demc_trcena <= 1'b0;
    else if (icdemc_we[3] & opt_dbg_en)
      opt_demc_trcena <= mtx_ppb_hwdata_i[24];

  assign demc_trcena = opt_demc_trcena & opt_dbg_en;


  assign dbg_mon_allow_step = dbg_mon_allow_i & demc_mon_step;

  assign dbg_mon_req_set    = (dbg_mon_allow_step &
                               (// Instruction
                                (dpu_ivalid_i &
                                 ~(dpu_int_entry_i & ~dbg_mon_hi_pri_invk_i)) |
                                (int_invoke_i & dbg_mon_hi_pri_invk_i)));

  assign dbg_mon_req_clr    = (dbg_mon_req &
                               ((int_invoke_i & ~dbg_mon_hi_pri_invk_i) |
                                ~dbg_mon_allow_i));

  assign dbg_mon_req_we     = dbg_mon_req_set | dbg_mon_req_clr;
  assign nxt_dbg_mon_req    = dbg_mon_req_set & ~dbg_mon_req_clr;

  always @ (negedge hreset_n or posedge hclk)
    if (!hreset_n)
      dbg_mon_req <= 1'b0;
    else if (dbg_mon_req_we)
      dbg_mon_req <= nxt_dbg_mon_req;

  assign dbg_step = (demc_mon_en & ~dbg_dbgen & demc_mon_step) | dbg_allow_step;


  assign int_pend_set = { set_pend_irq, st_pend_set, psv_pend_set, 1'b0,
                          shcs_we[1] &  mtx_ppb_hwdata_i[15],
                          shcs_we[1] &  mtx_ppb_hwdata_i[12],
                          shcs_we[1] &  mtx_ppb_hwdata_i[14],
                          shcs_we[1] &  mtx_ppb_hwdata_i[13],
                          1'b0, nmi_pend_set };

  assign int_pend_clr = { irq_pend_clr, st_pend_clr, psv_pend_clr, dbg_mon_p_clr,
                          shcs_we[1] & ~mtx_ppb_hwdata_i[15],
                          shcs_we[1] & ~mtx_ppb_hwdata_i[12],
                          shcs_we[1] & ~mtx_ppb_hwdata_i[14],
                          shcs_we[1] & ~mtx_ppb_hwdata_i[13],
                          2'b00 };


  assign int_actv_set = { {NUM_IRQ{1'b0}},
                          {3{shcs_we[1]}} & { mtx_ppb_hwdata_i[11:10],
                                              mtx_ppb_hwdata_i[8] },
                          {4{shcs_we[0]}} & { mtx_ppb_hwdata_i[7],
                                              mtx_ppb_hwdata_i[3],
                                              mtx_ppb_hwdata_i[1:0] },
                          2'b00 };

  assign int_actv_clr  = (// System Handler register write
                          {{NUM_IRQ{1'b0}},
                           {3{shcs_we[1]}} & ~{ mtx_ppb_hwdata_i[11:10],
                                                mtx_ppb_hwdata_i[8] },
                           {4{shcs_we[0]}} & ~{ mtx_ppb_hwdata_i[7],
                                                mtx_ppb_hwdata_i[3],
                                                mtx_ppb_hwdata_i[1:0] },
                           2'b00 } |
                          {NUM_INTS{vect_clr_actv}});

  assign vect_clr_actv = ((&ppb_we_i) & mtx_ppb_hwdata_i[1] &
                          (ppb_reg_addr_i==`NVIC_APPIRC) &
                          (mtx_ppb_hwdata_i[31:16]==`NVIC_INTCTRLKEY));


  assign stick_ctrl_en = ppb_we_i[0] & (ppb_reg_addr_i==`NVIC_SYSTICKCS);

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      begin
        stick_en          <= 1'b0;
        stick_int_en      <= 1'b0;
        stick_clk_src_reg <= 1'b0;
      end
    else if (stick_ctrl_en)
      begin
        stick_en          <= mtx_ppb_hwdata_i[0];
        stick_int_en      <= mtx_ppb_hwdata_i[1];
        stick_clk_src_reg <= mtx_ppb_hwdata_i[2];
      end

  assign stick_clk_src      = ext_st_noref_i | stick_clk_src_reg;

  assign stick_cnt_flag_set = stick_count_set & stick_to_zero & ~stick_count_clr;

  assign stick_cnt_flag_clr = (ppb_rd_i & (ppb_reg_addr_i==`NVIC_SYSTICKCS) &
                               ~ppb_hmaster_reg_i);

  assign stick_cnt_flag_en  = (stick_cnt_flag_set |
                               stick_cnt_flag_clr |
                               stick_count_clr);

  always @ (negedge hreset_n or posedge fclk)
    if (!hreset_n)
      stick_cnt_flag <= 1'b0;
    else if (stick_cnt_flag_en)
      stick_cnt_flag <= stick_cnt_flag_set;


  assign stick_reload_en = ppb_we_i[2:0] & {3{ppb_reg_addr_i==`NVIC_SYSTICKRL}};

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      stick_reload[7:0] <= 8'b00000000;
    else if (stick_reload_en[0])
      stick_reload[7:0] <= mtx_ppb_hwdata_i[7:0];

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      stick_reload[15:8] <= 8'b00000000;
    else if (stick_reload_en[1])
      stick_reload[15:8] <= mtx_ppb_hwdata_i[15:8];

  always @ (negedge hreset_n or posedge pclk)
    if (!hreset_n)
      stick_reload[23:16] <= 8'b00000000;
    else if (stick_reload_en[2])
      stick_reload[23:16] <= mtx_ppb_hwdata_i[23:16];


  assign sys_tick_en = stick_en & ~stick_clk_src;

  always @ (negedge hreset_n or posedge fclk)
    if (!hreset_n)
      begin
        sys_tick_q   <= 1'b0;
        sys_tick_qq  <= 1'b0;
        sys_tick_qqq <= 1'b0;
      end
    else if (sys_tick_en)
      begin
        sys_tick_q   <= ext_st_clk_i;
        sys_tick_qq  <= sys_tick_q;
        sys_tick_qqq <= sys_tick_qq;
      end

  assign stickref_pulse = sys_tick_qq & ~sys_tick_qqq;

  assign stick_count_set = (stick_en & (stick_clk_src | stickref_pulse) &
                            ~stick_count_clr & ~dpu_dbg_halted_i);

  assign stick_count_clr = (|ppb_we_i) & (ppb_reg_addr_i==`NVIC_SYSTICKCV);

  assign stick_is_zero   = {~(|stick_count[23:1]), ~stick_count[0]};

  assign stick_to_zero   = stick_is_zero[1] & stick_count[0];

  assign nxt_stick_count = (stick_count_clr ? 24'h000000 :
                            ((&stick_is_zero) ? stick_reload :
                                                (stick_count - 24'h000001)));

  assign stick_count_we  = stick_count_set | stick_count_clr;

  always @ (negedge hreset_n or posedge fclk)
    if (!hreset_n)
      stick_count <= {24{1'b0}};
    else if (stick_count_we)
      stick_count <= nxt_stick_count;

  assign stick_int_we  = nxt_stick_int | stick_int;
  assign nxt_stick_int = stick_int_en & stick_count_set & stick_to_zero;

  always @ (negedge hreset_n or posedge fclk)
    if (!hreset_n)
      stick_int <= 1'b0;
    else if (stick_int_we)
      stick_int <= nxt_stick_int;


  assign cpacr_we = mst_cp_en & ppb_we_i[2] & (ppb_reg_addr_i==`NVIC_CPACR);

   always @ (negedge hreset_n or posedge fclk)
    if (!hreset_n)
      cpacr <= 4'b0000;
    else if (cpacr_we)
      cpacr <= mtx_ppb_hwdata_i[23:20];


  assign fpccr_we = ({4{mst_cp_en}} & ppb_we_i[3:0] &
                     {4{ppb_reg_addr_i==`NVIC_FPCCR}});

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      begin
        fpccr_aspen <= 1'b1;
        fpccr_lspen <= 1'b1;
      end
    else if (fpccr_we[3])
      begin
        fpccr_aspen <= mtx_ppb_hwdata_i[31];
        fpccr_lspen <= mtx_ppb_hwdata_i[30];
      end

  assign nxt_fpccr_monrdy = fpccr_we[1] ? mtx_ppb_hwdata_i[8] : int_lazy_fpccr_i[3];
  assign fpccr_monrdy_we  = fpccr_we[1] | dpu_fpccr_we_i[1];

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fpccr_monrdy  <= 1'b0;
    else if (fpccr_monrdy_we & opt_cp_en)
      fpccr_monrdy  <= nxt_fpccr_monrdy;

  assign nxt_fpccr_bfrdy = fpccr_we[0] ? mtx_ppb_hwdata_i[6] : int_lazy_fpccr_i[2];
  assign fpccr_bfrdy_we  = fpccr_we[0] | dpu_fpccr_we_i[1];

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fpccr_bfrdy   <= 1'b0;
    else if (fpccr_bfrdy_we & opt_cp_en)
      fpccr_bfrdy   <= nxt_fpccr_bfrdy;

  assign nxt_fpccr_mmrdy = fpccr_we[0] ? mtx_ppb_hwdata_i[5] : int_lazy_fpccr_i[1];
  assign fpccr_mmrdy_we  = fpccr_we[0] | dpu_fpccr_we_i[1];

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fpccr_mmrdy   <= 1'b0;
    else if (fpccr_mmrdy_we & opt_cp_en)
      fpccr_mmrdy   <= nxt_fpccr_mmrdy;

  assign nxt_fpccr_hfrdy = fpccr_we[0] ? mtx_ppb_hwdata_i[4] : int_lazy_fpccr_i[0];
  assign fpccr_hfrdy_we  = fpccr_we[0] | dpu_fpccr_we_i[1];

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fpccr_hfrdy   <= 1'b0;
    else if (fpccr_hfrdy_we & opt_cp_en)
      fpccr_hfrdy   <= nxt_fpccr_hfrdy;

  assign nxt_fpccr_thread = fpccr_we[0] ? mtx_ppb_hwdata_i[3] : dpu_thread_i;
  assign fpccr_thread_we  = fpccr_we[0] | dpu_fpccr_we_i[1];

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fpccr_thread  <= 1'b0;
    else if (fpccr_thread_we & opt_cp_en)
      fpccr_thread  <= nxt_fpccr_thread;

  assign nxt_fpccr_user  = fpccr_we[0] ? mtx_ppb_hwdata_i[1] : ~dpu_privileged_i;
  assign fpccr_user_we   = fpccr_we[0] | dpu_fpccr_we_i[1];

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fpccr_user    <= 1'b0;
    else if (fpccr_user_we & opt_cp_en)
      fpccr_user    <= nxt_fpccr_user;

  assign nxt_fpccr_lspact = fpccr_we[0] ? mtx_ppb_hwdata_i[0] : ~fpccr_lspact;
  assign fpccr_lspact_we  = fpccr_we[0] | dpu_fpccr_we_i[0];

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fpccr_lspact  <= 1'b0;
    else if (fpccr_lspact_we & opt_cp_en)
      fpccr_lspact  <= nxt_fpccr_lspact;

  assign fpccr = { fpccr_aspen,    //   [31] : ASPEN
                   fpccr_lspen,    //   [30] : LSPEN
                   {21{1'b0}},     // [29:9] : Reserved (RAZ)
                   fpccr_monrdy,   //    [8] : MONRDY
                   1'b0,           //    [7] : Reserved (RAZ)
                   fpccr_bfrdy,    //    [6] : BFRDY
                   fpccr_mmrdy,    //    [5] : MMRDY
                   fpccr_hfrdy,    //    [4] : HFRDY
                   fpccr_thread,   //    [3] : THREAD
                   1'b0,           //    [2] : Reserved (RAZ)
                   fpccr_user,     //    [1] : USER
                   fpccr_lspact};  //    [0] : LSPACT


  assign fpcar_we     = {4{mst_cp_en}} & ppb_we_i & {4{ppb_reg_addr_i==`NVIC_FPCAR}};
  assign fpcar_min_we = (|fpcar_we) | dpu_fpccr_we_i[1];

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fpcar_min <= {29{1'b0}};
    else if (fpcar_min_we)
      fpcar_min <= nxt_fpcar_min;

  assign nxt_fpcar_ppb = {(fpcar_we[3] ? mtx_ppb_hwdata_i[31:24] : fpcar_min[28:21]),
                          (fpcar_we[2] ? mtx_ppb_hwdata_i[23:16] : fpcar_min[20:13]),
                          (fpcar_we[1] ? mtx_ppb_hwdata_i[15: 8] : fpcar_min[12: 5]),
                          (fpcar_we[0] ? mtx_ppb_hwdata_i[ 7: 3] : fpcar_min[ 4: 0])};

  assign nxt_fpcar_min = dpu_fpccr_we_i[1] ? dpu_fpcar_nxt_i[31:3] : nxt_fpcar_ppb;

  assign fpcar         = { fpcar_min, 3'b000 };


  assign fpdscr_we = ({2{mst_cp_en}} & ppb_we_i[3:2] &
                      {2{ppb_reg_addr_i==`NVIC_FPDSCR}});

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fpdscr[4:2] <= 3'b000;
    else if (fpdscr_we[1])
      fpdscr[4:2] <= mtx_ppb_hwdata_i[26:24];

  always @ (posedge hclk or negedge hreset_n)
    if (!hreset_n)
      fpdscr[1:0] <= 2'b00;
    else if (fpdscr_we[0])
      fpdscr[1:0] <= mtx_ppb_hwdata_i[23:22];

  assign data_be8_o           = data_be8;
  assign aux_ctrl_o           = aux_ctrl;
  assign aux_ctrl1_we_o       = aux_ctrl1_we;
  assign ccr_div_0_trp_o      = ccr_div_0_trp;
  assign ccr_unalign_trp_o    = ccr_unalign_trp;
  assign ccr_bfhfnmign_o      = ccr_bfhfnmign;
  assign ccr_stkalign_o       = ccr_stkalign;
  assign ccr_usersetmpend_o   = ccr_usersetmpend;
  assign ccr_nonbasethrdena_o = ccr_nonbasethrdena;
  assign cpacr_o              = opt_cpacr;
  assign fpcar_o              = opt_fpcar;
  assign fpccr_o              = opt_fpccr;
  assign fpdscr_o             = opt_fpdscr;
  assign airc_vectreset_o     = airc_vectreset;
  assign airc_sysresetreq_o   = airc_sysresetreq;
  assign airc_prigroup_o      = airc_prigroup;
  assign vto_tbloff_o         = vto_tbloff;
  assign lockup_o             = lockup;
  assign irq_en_o             = irq_en;
  assign shcs_mm_en_o         = shcs_mm_en;
  assign shcs_bf_en_o         = shcs_bf_en;
  assign shcs_uf_en_o         = shcs_uf_en;
  assign irq_lvl_o            = irq_lvl;
  assign shp_mm_lvl_o         = opt_shp_mm_lvl;
  assign shp_dm_lvl_o         = opt_shp_dm_lvl;
  assign shp_bf_lvl_o         = opt_shp_bf_lvl;
  assign shp_uf_lvl_o         = opt_shp_uf_lvl;
  assign shp_svc_lvl_o        = opt_shp_svc_lvl;
  assign shp_psv_lvl_o        = opt_shp_psv_lvl;
  assign shp_st_lvl_o         = opt_shp_st_lvl;
  assign int_pend_set_o       = int_pend_set;
  assign int_pend_clr_o       = int_pend_clr;
  assign int_actv_set_o       = int_actv_set;
  assign int_actv_clr_o       = int_actv_clr;
  assign vect_clr_actv_o      = vect_clr_actv;
  assign stick_cnt_flag_o     = stick_cnt_flag;
  assign stick_clk_src_o      = stick_clk_src;
  assign stick_int_en_o       = stick_int_en;
  assign stick_en_o           = stick_en;
  assign stick_reload_o       = stick_reload;
  assign stick_count_o        = stick_count;
  assign stick_int_o          = stick_int;
  assign fsr_o                = fsr;
  assign hfsr_o               = hfsr;
  assign dfsr_o               = dfsr;
  assign afsr_o               = afsr;
  assign mm_bf_far_o          = mm_bf_far;
  assign scr_sleeponexit_o    = scr_sleeponexit;
  assign scr_deepsleep_o      = scr_deepsleep;
  assign scr_wfeonpend_o      = scr_wfeonpend;
  assign slp_wfe_valid_o      = slp_wfe_valid;
  assign slp_wfi_valid_o      = slp_wfi_valid;
  assign slp_hold_ack_n_o     = slp_hold_ack_n;
  assign slp_asleep_o         = slp_asleep;
  assign slp_sleeping_o       = slp_sleeping;
  assign slp_sleepdeep_o      = slp_sleepdeep;
  assign wic_load_o           = opt_wic_load;
  assign wic_clear_o          = opt_wic_clear;
  assign demc_mon_en_o        = demc_mon_en;
  assign demc_mon_req_o       = demc_mon_req;
  assign demc_mon_step_o      = demc_mon_step;
  assign demc_trcena_o        = demc_trcena;
  assign dhcs_c_debugen_o     = dhcs_c_debugen;
  assign dhcs_c_halt_o        = dhcs_c_halt;
  assign dhcs_c_step_o        = dhcs_c_step;
  assign dhcs_s_reset_st_o    = dhcs_s_reset_st;
  assign dhcs_s_retire_st_o   = dhcs_s_retire_st;
  assign dhcs_c_mask_ints_o   = dhcs_c_mask_ints;
  assign dhcs_c_snapstall_o   = dhcs_c_snapstall;
  assign dbg_mon_pend_o       = dbg_mon_pend;
  assign dbg_mon_req_o        = dbg_mon_req;
  assign dbg_dbgen_o          = dbg_dbgen;
  assign dbg_vcatch_o         = dbg_vcatch;
  assign dbg_halt_req_o       = dbg_halt_req;
  assign dbg_step_o           = dbg_step;
  assign dbg_mask_ints_o      = dbg_mask_ints;
  assign dbg_restarted_o      = dbg_restarted;
  assign dbg_dcrd_o           = dbg_dcrd;
  assign dcrs_rtrans_o        = dcrs_rtrans;
  assign dcrs_regwnr_o        = dcrs_regwnr;
  assign dcrs_regsel_o        = dcrs_regsel;


`ifdef ARM_ASSERT_ON

  reg                       ovl_last_dbg_restart;
  reg                       ovl_last_dbg_restarted;
  reg                       ovl_last_dhcs_c_halt;
  reg                       ovl_slp_sleeping_q;
  reg                       ovl_slp_sleepdeep_q;
  reg                       ovl_sleep_hold_req_n_q;
  reg                       wic_loaded;
  reg                       ovl_dbg_en;
  reg                       ovl_sys_ctrl_we_q;
  reg                       ovl_sys_ctrl_we_qq;
  reg                       ovl_wic_ds_req_n_q;
  reg                       ovl_wic_ds_ack_n_q;


  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      begin
        ovl_slp_sleeping_q     <= 1'b0;
        ovl_slp_sleepdeep_q    <= 1'b0;
        ovl_sleep_hold_req_n_q <= 1'b1;
        ovl_last_dbg_restart   <= 1'b0;
        ovl_last_dbg_restarted <= 1'b1;
        ovl_last_dhcs_c_halt   <= 1'b0;
        ovl_dbg_en             <= 1'b0;
        ovl_sys_ctrl_we_q      <= 1'b0;
        ovl_sys_ctrl_we_qq     <= 1'b0;
        ovl_wic_ds_req_n_q     <= 1'b0;
        ovl_wic_ds_ack_n_q     <= 1'b0;
      end
    else
      begin
        ovl_slp_sleeping_q     <= slp_sleeping;
        ovl_slp_sleepdeep_q    <= slp_sleepdeep;
        ovl_sleep_hold_req_n_q <= ext_slp_hold_req_n_i;
        ovl_last_dbg_restart   <= ext_dbg_restart_i;
        ovl_last_dbg_restarted <= dbg_restarted_reg;
        ovl_last_dhcs_c_halt   <= dhcs_c_halt;
        ovl_dbg_en             <= ext_dbg_en_i;
        ovl_sys_ctrl_we_q      <= sys_ctrl_we;
        ovl_sys_ctrl_we_qq     <= ovl_sys_ctrl_we_q | (ovl_sys_ctrl_we_qq & slp_normal);
        ovl_wic_ds_req_n_q     <= wic_ds_req_n_i;
        ovl_wic_ds_ack_n_q     <= wic_ds_ack_n_i;
      end

  always @ (posedge fclk or negedge hreset_n)
    if (!hreset_n)
      wic_loaded <= 1'b0;
    else if (opt_wic_load | opt_wic_clear)
      wic_loaded <= opt_wic_load;

  assert_next #(0,1,0,0,0,"dcrs_rtrans must be pulsed")
    ovl_dcrs_rtrans_pulsed(hclk, hreset_n, dcrs_rtrans, ~dcrs_rtrans);

  assert_next #(0,1,0,0,0,"stick_int must be pulsed")
    ovl_stick_int_pulsed(fclk, hreset_n, stick_int, ~stick_int);

  assert_never #(0,0,"Tick Count stick_count_clr must not count down or interrupt")
    ovl_stick_count_check(hclk, hreset_n, stick_count_clr & |nxt_stick_count);

  assert_never #(0,0,"slp_sleep_type should be exclusive")
    ovl_sleep_type_check_1(hclk, hreset_n, (&slp_sleep_type));

  assert_never #(0,0,"slp_sleep_type should not be set without slp_normal")
    ovl_sleep_type_check_2(fclk, hreset_n,
                           (slp_normal & ~(|slp_sleep_type)) |
                           ((|slp_sleep_type) & ~slp_normal));

  assert_zero_one_hot #(0,2,0,"set_hfsr[2:1] controls should be exclusive")
    ovl_set_hfsr_excl(hclk, hreset_n,
                      (set_hfsr[2:1] &
                       {~(dwt_watchpoint_i | ext_dbg_req_i), 1'b1}));

  assert_never #(0,0,"dhcs_c_halt cannot be set without dbg_halt_req unless forced to sleep")
    ovl_dhcs_c_halt_check(hclk, hreset_n, (dhcs_c_halt & ~dbg_halt_req &
                                           ovl_dbg_en & slp_hold_ack_n &
                                           ~slp_hold_ack_q & ~slp_forced));

  assert_never #(0,0,"debug monitor request has been incorrectly formed")
    ovl_mon_req_check(hclk, hreset_n,
                      (dbg_mon_req & dpu_int_entry_i &
                       ((int_nxt_isr_i==8'h0C) | ~dbg_mon_hi_pri_invk_i)));

  assert_never_unknown #(0,32,0,"Invalid value on ext_aux_fault_i")
    ovl_aux_fault_check(hclk, hreset_n, 1'b1, ext_aux_fault_i);

  assert_never #(0,0,"nxt_afsr has been incorrectly formed")
    ovl_aux_fsr_check(hclk, hreset_n,
                      ((nxt_afsr & ext_aux_fault_i) != ext_aux_fault_i));

  assert_implication #(0,0,"afsr_we has been incorrectly formed")
    ovl_afsr_we_check(hclk, hreset_n, !(|{ext_aux_fault_i,ppb_we_i}), !afsr_we);

  assert_next #(0,1,0,0,0,"ext_halt_clr must be pulsed")
    ovl_ext_halt_clr_pulsed(hclk, hreset_n, ext_halt_clr, ~ext_halt_clr);

  assert_never #(0,0,"ext_halt_clr has been incorrectly activated")
    ovl_ext_halt_clr(hclk, hreset_n, (ext_halt_clr &
                                      (ovl_last_dbg_restart | dbg_restarted)));

  assert_implication #(0,1,"Constraint for DBGRESTARTED_check_1")
    ovl_dbg_restart_1(hclk, hreset_n, (ovl_last_dbg_restart & ~ext_dbg_restart_i),
                      ~ovl_last_dbg_restarted);

  assert_never #(0,0,"dbg_restarted_reg goes low when ext_dbg_restart_i is low")
    ovl_dbg_restarted_1(hclk, hreset_n, (~dbg_restarted_reg &
                                         ovl_last_dbg_restarted &
                                         ~ext_dbg_restart_i));

  assert_implication #(0,1,"Constraint for DBGRESTARTED_check_2")
    ovl_dbg_restart_2(hclk, hreset_n, (~ovl_last_dbg_restart & ext_dbg_restart_i),
                      ovl_last_dbg_restarted);

  assert_never #(0,0,"dbg_restarted_reg goes low when ext_dbg_restart_i is high")
    ovl_dbg_restarted_2(hclk, hreset_n, (dbg_restarted_reg &
                                         ~ovl_last_dbg_restarted &
                                         ext_dbg_restart_i));

  assert_never #(0,0,"Deep sleep must not be driven without sleep")
    ovl_sleep_check(fclk, hreset_n, slp_sleepdeep & ~slp_sleeping);

  assert_never #(0,0,"slp_hold_ack_n can only follow a high ovl_slp_sleeping_q")
    ovl_sleep_ack_1(fclk, hreset_n, (~slp_hold_ack_n & ~slp_hold_ack_q &
                                     ~ovl_slp_sleeping_q));

  assert_never #(0,0,"slp_hold_ack_n must stay high until sleep_hold_req dropped or halt")
    ovl_sleep_ack_2(fclk, hreset_n, (~ovl_sleep_hold_req_n_q & slp_hold_ack_q &
                                     ~ovl_last_dhcs_c_halt & slp_hold_ack_n));

  assert_never #(0,0,"If sleep_hold_req goes low, slp_hold_ack_n must follow")
    ovl_sleep_req(fclk, hreset_n, ovl_sleep_hold_req_n_q & ~slp_hold_ack_n);

  assert_never #(0,0,"Must never be asleep and halted at the same time")
    ovl_sleep_halt(fclk, hreset_n, ((slp_sleeping | ~slp_hold_ack_n) &
                                    dpu_dbg_halted_i));

  assert_never #(0,0,"Never enter WIC mode deep sleep without loading WIC first")
    ovl_wic_load(fclk, hreset_n, (slp_sleepdeep & ~ovl_slp_sleepdeep_q &
                                  ~ovl_wic_ds_req_n_q & ~ovl_wic_ds_ack_n_q &
                                  ~(ovl_sys_ctrl_we_q | ovl_sys_ctrl_we_qq) & ~wic_loaded));


  assert_never #(0,0,"Core cannot halt if DEBUG_LVL == 0")
    ovl_cfg_halt(hclk, hreset_n, ((DEBUG_LVL == 0) &
                                  (dpu_dbg_halted_i | dhcs_c_halt)));

  assert_never #(0,0,"Core cannot perform register transfer if DEBUG_LVL == 0")
    ovl_cfg_reg_trans(hclk, hreset_n, (DEBUG_LVL == 0) & dcrs_rtrans);

`endif

endmodule

`include "cm3_nvic_undefs.vp"
