module cm3_rom_table (/*AUTOARG*/
  PCLK, PRESETn, PSEL, PENABLE, PADDR, PWRITE,
  PRDATA
  );

  parameter DEBUG_LVL = 3;
  parameter TRACE_LVL = 1;

  input         PCLK;           //APB Clock
  input         PRESETn;        //APB Reset
  input         PSEL;           //APB Select
  input         PENABLE;        //APB Enable
  input  [11:2] PADDR;          //APB Address
  input         PWRITE;         //APB Write


  output [31:0] PRDATA;         // APB Read Data

  wire          addr_reg_we;
  wire    [5:0] nxt_addr_reg;
  reg     [5:0] addr_reg;

  reg    [31:0] apb_rdata;


  assign PRDATA  = (DEBUG_LVL > 0) ? apb_rdata : {32{1'b0}};

  assign addr_reg_we  = PSEL & ~PWRITE;
  assign nxt_addr_reg = {&PADDR[11:6], ~|PADDR[11:6],PADDR[5:2]};

  always @ (posedge PCLK or negedge PRESETn)
    if (!PRESETn)
      addr_reg <= 6'b000000;
    else if (addr_reg_we)
      addr_reg <= nxt_addr_reg;

  always @ (addr_reg)
    case (addr_reg)
      6'b0_1_0000 : apb_rdata = 32'hFFF0F003;
      6'b0_1_0001 : apb_rdata = 32'hFFF02003;
      6'b0_1_0010 : apb_rdata = 32'hFFF03003;
      6'b0_1_0011 : apb_rdata = (TRACE_LVL > 0) ? 32'hFFF01003 : 32'hFFF01002;

      6'b0_1_0100 : apb_rdata = (TRACE_LVL > 0) ? 32'hFFF41003 : 32'hFFF41002;
      6'b0_1_0101 : apb_rdata = (TRACE_LVL > 1) ? 32'hFFF42003 : 32'hFFF42002;



      6'b1_0_0100 : apb_rdata = 32'h00000004;
      6'b1_0_0101 : apb_rdata = 32'h00000000;
      6'b1_0_0110 : apb_rdata = 32'h00000000;
      6'b1_0_0111 : apb_rdata = 32'h00000000;
      6'b1_0_1000 : apb_rdata = 32'h000000C3;
      6'b1_0_1001 : apb_rdata = 32'h000000B4;
      6'b1_0_1010 : apb_rdata = 32'h0000000B;
      6'b1_0_1011 : apb_rdata = 32'h00000000;

      6'b1_0_0011 : apb_rdata = 32'h00000001;
      6'b1_0_1100 : apb_rdata = 32'h0000000D;
      6'b1_0_1101 : apb_rdata = 32'h00000010;
      6'b1_0_1110 : apb_rdata = 32'h00000005;
      6'b1_0_1111 : apb_rdata = 32'h000000B1;

      default     : apb_rdata = 32'h00000000;
    endcase

endmodule
