module cm3_sync
  (// Inputs
   reset_n, clk, d_async_i,
   q_o
  );


  parameter SYNC_PRESENT = 1;



  input         clk;         // Clock to which q_o will be synchronized
  input         reset_n;     // Reset for synchronization registers
  input         d_async_i;   // Signal to be synchronized
  output        q_o;         // Synchronized output



  reg           d_sync1;     // 1st synchroniser DFF
  reg           q;           // 2nd synchroniser DFF

  wire          opt_q;



  assign opt_q = (SYNC_PRESENT != 0) ? q : d_async_i;



  always @ (posedge clk or negedge reset_n)
    if (!reset_n)
      begin
       q       <= 1'b0;
       d_sync1 <= 1'b0;
      end
    else
      begin
        q       <= d_sync1;
        d_sync1 <= d_async_i;
      end



  assign q_o = opt_q;



`ifdef ARM_ASSERT_ON

  assert_never #(2,0,
    "Glitching input - valid waveform, but undesirable in handshake design.")
      ovl_cm3_sync_glitch(clk, reset_n,
        ((d_sync1 != d_async_i) & (d_sync1 != q)));

  assert_never_unknown #(0,1,0,
    "Input X not allowed - halt propogation at synchroniser level.")
      ovl_cm3_sync_xprop(clk, reset_n, 1'b1, d_async_i);

`endif

endmodule