module cm3_tpiu_atb_sync (
  clk, reset_n, read_pointer1_gray_async_i, read_pointer2_gray_async_i,
  sync_ack_async_i, trigger_ack_async_i, tpiu_baud_async_i,
  trace_out_idle_async_i,
  read_pointer1_gray_o, read_pointer2_gray_o, sync_ack_o, trigger_ack_o,
  tpiu_baud_o, trace_out_idle_o
  );

  parameter TRACE_LVL = 1;


  input         clk;        // ATB clock
  input         reset_n;     // ATB asychronous reset active low
  input [2:0]   read_pointer1_gray_async_i;    // Read pointer from the read clock domain
  input [2:0]   read_pointer2_gray_async_i;    // Read pointer from the read clock domain
  input         sync_ack_async_i;
  input         trigger_ack_async_i;
  input         tpiu_baud_async_i;
  input         trace_out_idle_async_i;

  output [2:0]  read_pointer1_gray_o;// Read pointer to the write clock domain
  output [2:0]  read_pointer2_gray_o;// Read pointer to the write clock domain
  output        sync_ack_o;
  output        trigger_ack_o;
  output        tpiu_baud_o;
  output        trace_out_idle_o;

  wire [2:0]    read_pointer1_gray;
  wire [2:0]    read_pointer2_gray;
  wire          sync_ack;
  wire          trigger_ack;
  wire          tpiu_baud;
  wire          trace_out_idle;

  assign read_pointer1_gray_o   = (TRACE_LVL > 0) ? read_pointer1_gray   : 3'b000;
  assign read_pointer2_gray_o   = (TRACE_LVL > 0) ? read_pointer2_gray   : 3'b000;
  assign sync_ack_o        = (TRACE_LVL > 0) ? sync_ack       : 1'b0;
  assign trigger_ack_o     = (TRACE_LVL > 0) ? trigger_ack       : 1'b0;
  assign tpiu_baud_o       = (TRACE_LVL > 0) ? tpiu_baud : 1'b0;
  assign trace_out_idle_o  = (TRACE_LVL > 0) ? trace_out_idle : 1'b0;

  cm3_cdc_capt_sync
    u_sync_ack_async
      (
       .clk         (clk),
       .reset_n     (reset_n),
       .async_i     (sync_ack_async_i),
       .sync_o      (sync_ack)
       );

  cm3_cdc_capt_sync
    u_trigger_ack_async
      (
       .clk         (clk),
       .reset_n     (reset_n),
       .async_i     (trigger_ack_async_i),
       .sync_o      (trigger_ack)
       );

  cm3_cdc_capt_sync
    u_read_pointer1_gray0
      (
       .clk         (clk),
       .reset_n     (reset_n),
       .async_i     (read_pointer1_gray_async_i[0]),
       .sync_o      (read_pointer1_gray[0])
       );

  cm3_cdc_capt_sync
    u_read_pointer1_gray1
      (
       .clk         (clk),
       .reset_n     (reset_n),
       .async_i     (read_pointer1_gray_async_i[1]),
       .sync_o      (read_pointer1_gray[1])
       );

  cm3_cdc_capt_sync
    u_read_pointer1_gray2
      (
       .clk         (clk),
       .reset_n     (reset_n),
       .async_i     (read_pointer1_gray_async_i[2]),
       .sync_o      (read_pointer1_gray[2])
       );

  cm3_cdc_capt_sync
    u_read_pointer2_gray0
      (
       .clk         (clk),
       .reset_n     (reset_n),
       .async_i     (read_pointer2_gray_async_i[0]),
       .sync_o      (read_pointer2_gray[0])
       );

  cm3_cdc_capt_sync
    u_read_pointer2_gray1
      (
       .clk         (clk),
       .reset_n     (reset_n),
       .async_i     (read_pointer2_gray_async_i[1]),
       .sync_o      (read_pointer2_gray[1])
       );

  cm3_cdc_capt_sync
    u_read_pointer2_gray2
      (
       .clk         (clk),
       .reset_n     (reset_n),
       .async_i     (read_pointer2_gray_async_i[2]),
       .sync_o      (read_pointer2_gray[2])
       );

  cm3_cdc_capt_sync
    u_tpiu_baud
      (
       .clk         (clk),
       .reset_n     (reset_n),
       .async_i     (tpiu_baud_async_i),
       .sync_o      (tpiu_baud)
       );

  cm3_cdc_capt_sync
    u_trace_out_idle
      (
       .clk         (clk),
       .reset_n     (reset_n),
       .async_i     (trace_out_idle_async_i),
       .sync_o      (trace_out_idle)
       );

endmodule
