`include "cm3_tpiu_defs.vp"

module cm3_tpiu_trace_sync
  (
  clk, reset_n, write_pointer1_gray_async_i, write_pointer2_gray_async_i,
  sync_valid_async_i, trigger_valid_async_i, baud_div_reset_async_i,
  write_pointer1_gray_o, write_pointer2_gray_o, sync_valid_o,
  trigger_valid_o, baud_div_reset_o
  );

  parameter TRACE_LVL = 1;


  input         clk;                          // Trace clock
  input         reset_n;                      // Trace asychronous reset active low

  input [2:0]   write_pointer1_gray_async_i;  // Write pointer from the ATclk clock domain
  input [2:0]   write_pointer2_gray_async_i;  // Write pointer from the ATclk clock domain
  input         sync_valid_async_i;
  input         trigger_valid_async_i;
  input         baud_div_reset_async_i;

  output [2:0]  write_pointer1_gray_o;   // Write pointer to the read clock domain
  output [2:0]  write_pointer2_gray_o;   // Write pointer to the read clock domain
  output        sync_valid_o;
  output        trigger_valid_o;
  output        baud_div_reset_o;

  wire [2:0]    write_pointer1_gray;
  wire [2:0]    write_pointer2_gray;
  wire          sync_valid;
  wire          trigger_valid;
  wire          baud_div_reset;


  cm3_cdc_capt_sync
    u_sync_valid_async
      (
       .clk       (clk),
       .reset_n   (reset_n),
       .async_i   (sync_valid_async_i),
       .sync_o    (sync_valid)
       );

  cm3_cdc_capt_sync
    u_trigger_valid_async
      (
       .clk       (clk),
       .reset_n   (reset_n),
       .async_i   (trigger_valid_async_i),
       .sync_o    (trigger_valid)
       );

  cm3_cdc_capt_sync
    u_write_pointer1_gray0
      (
       .clk       (clk),
       .reset_n   (reset_n),
       .async_i   (write_pointer1_gray_async_i[0]),
       .sync_o    (write_pointer1_gray[0])
       );

  cm3_cdc_capt_sync
    u_write_pointer1_gray1
      (
       .clk       (clk),
       .reset_n   (reset_n),
       .async_i   (write_pointer1_gray_async_i[1]),
       .sync_o    (write_pointer1_gray[1])
       );

  cm3_cdc_capt_sync
    u_write_pointer1_gray2
      (
       .clk       (clk),
       .reset_n   (reset_n),
       .async_i   (write_pointer1_gray_async_i[2]),
       .sync_o    (write_pointer1_gray[2])
       );

  cm3_cdc_capt_sync
    u_write_pointer2_gray0
      (
       .clk       (clk),
       .reset_n   (reset_n),
       .async_i   (write_pointer2_gray_async_i[0]),
       .sync_o    (write_pointer2_gray[0])
       );

  cm3_cdc_capt_sync
    u_write_pointer2_gray1
      (
       .clk       (clk),
       .reset_n   (reset_n),
       .async_i   (write_pointer2_gray_async_i[1]),
       .sync_o    (write_pointer2_gray[1])
       );

  cm3_cdc_capt_sync
    u_write_pointer2_gray2
      (
       .clk       (clk),
       .reset_n   (reset_n),
       .async_i   (write_pointer2_gray_async_i[2]),
       .sync_o    (write_pointer2_gray[2])
       );

  cm3_cdc_capt_sync
    u_baud_div_reset
      (
       .clk       (clk),
       .reset_n   (reset_n),
       .async_i   (baud_div_reset_async_i),
       .sync_o    (baud_div_reset)
       );


  assign write_pointer1_gray_o = (TRACE_LVL > 0) ? write_pointer1_gray : 3'b000;
  assign write_pointer2_gray_o = (TRACE_LVL > 0) ? write_pointer2_gray : 3'b000;
  assign sync_valid_o          = (TRACE_LVL > 0) ? sync_valid          : 1'b0;
  assign trigger_valid_o       = (TRACE_LVL > 0) ? trigger_valid       : 1'b0;
  assign baud_div_reset_o      = (TRACE_LVL > 0) ? baud_div_reset      : 1'b0;

endmodule
