module DAPSWJDP
  (/*AUTOARG*/
  nTRST, nPOTRST, SWCLKTCK, SWDITMS, TDI, DAPRESETn, DAPCLK,
  DAPCLKEN, DAPRDATA, DAPREADY, DAPSLVERR, nCDBGPWRDN, CDBGPWRUPACK,
  CSYSPWRUPACK, CDBGRSTACK,
  SWDO, SWDOEN, TDO, nTDOEN, DAPADDR, DAPWRITE, DAPENABLE, DAPABORT,
  DAPSEL, DAPWDATA, CDBGPWRUPREQ, CSYSPWRUPREQ, CDBGRSTREQ, JTAGNSW,
  JTAGTOP
  );

  parameter DP_PRESENT     = 1; // DP logic is included if 1
  parameter JTAG_PRESENT   = 1; // Include JTAG as well as SW if 1
  parameter RESET_ALL_REGS = 0; // Reset all register if 1, even if not needed


  input         nTRST;      // JTAG TAP Reset
  input         nPOTRST;    // JTAG/Serial Wire Power on reset
  input         SWCLKTCK;   // SW/JTAG Clock
  input         SWDITMS;    // SW Debug Data In / JTAG Test Mode Select
  input         TDI;        // JTAG TAP Data In / Alternative input function
  input         DAPRESETn;  // DAP Reset
  input         DAPCLK;     // DAP Clock
  input         DAPCLKEN;   // DAP Clock Enable
  input  [31:0] DAPRDATA;   // DAP Read Data
  input         DAPREADY;   // DAP Data bus Ready
  input         DAPSLVERR;  // AP slave error response
  input         nCDBGPWRDN;     // Debug infrastructure power-down control
  input         CDBGPWRUPACK;   // Debug Power Domain up acknowledge
  input         CSYSPWRUPACK;   // System Power Domain up acknowledge
  input         CDBGRSTACK;     // Debug reset acknowledge to reset controller

  output        SWDO;       // SW Data Out
  output        SWDOEN;     // SW Data Out Enable
  output        TDO;        // JTAG TAP Data Out
  output        nTDOEN;     // TDO enable
  output [31:0] DAPADDR;    // DAP Address
  output        DAPWRITE;   // DAP Bus Write
  output        DAPENABLE;  // DAP Enable Transaction
  output        DAPABORT;   // DAP Abort
  output        DAPSEL;     // DAP transaction select
  output [31:0] DAPWDATA;   // DAP Write Data
  output        CDBGPWRUPREQ;   // Debug Power Domain up request
  output        CSYSPWRUPREQ;   // System Power Domain up request
  output        CDBGRSTREQ;     // Debug reset request to reset controller
  output        JTAGNSW;    // JTAG/not Serial Wire Mode
  output        JTAGTOP;    // JTAG-DP TAP controller in one of four top states

  wire          nTRST;      // JTAG Reset
  wire          SWCLKTCK;   // SW/JTAG Clock
  wire          TDI;        // TAP Data In
  wire          DAPRESETn;  // DAP Reset
  wire          DAPCLK;     // DAP Clock
  wire   [31:0] DAPRDATA;   // DAP Read Data
  wire          DAPREADY;   // DAP Data bus Ready
  wire          SWDO;       // SW Data Out
  wire   [31:0] DAPADDR;    // DAP Address
  wire          DAPWRITE;   // DAP Bus Write
  wire          DAPENABLE;  // DAP Enable Transaction
  wire   [31:0] DAPWDATA;   // DAP Write Data


  wire          JBusreq;
  wire          JBuswrite;
  wire   [31:0] JBusaddr;
  wire          JBusabort;
  wire   [31:0] JBuswdata;
  wire    [1:0] JBusmode;
  wire    [3:0] JBusmask;
  wire   [11:0] JBustrncnt;

  wire          SBusreq;
  wire          SBuswrite;
  wire   [31:0] SBusaddr;
  wire          SBusabort;
  wire   [31:0] SBuswdata;
  wire    [1:0] SBusmode;
  wire    [3:0] SBusmask;
  wire   [11:0] SBustrncnt;

  wire          JTAGTLR;        // TAP in TLR
  wire          TMS;            // TMS to JTAG-DP
  wire          JTAGNSW;        // JTAG/SWD select
  wire          DBGDI;          // SWD data input
  wire          WIRERESET;
  wire          SCDbgPwrUpReq;
  wire          SCSysPwrUpReq;
  wire          SCDbgRstReq;
  wire          JCDbgPwrUpReq;
  wire          JCSysPwrUpReq;
  wire          JCDbgRstReq;

  wire          Busreq;         // local wire
  wire          BusreqC;        // clamp wire
  wire          BusreqD;        // sync  wire
  wire          Busack;         // local wire
  wire          BusackC;        // clamp wire
  wire          BusackT;        // sync  wire
  wire          Buswrite;       // local wire
  wire          BuswriteC;      // clamp wire
  wire          Busabort;       // local wire
  wire          BusabortC;      // clamp wire
  wire          BusabortD;      // sync  wire
  wire   [31:0] Busaddr;        // local wire
  wire   [31:0] BusaddrC;       // clamp wire
  wire   [31:0] Busrdata;       // local wire
  wire   [31:0] BusrdataC;      // clamp wire
  wire   [31:0] Buswdata;       // local wire
  wire   [31:0] BuswdataC;      // clamp wire
  wire          Buscmp;         // local wire
  wire          BuscmpC;        // clamp wire
  wire          Buserror;       // local wire
  wire          BuserrorC;      // clamp wire
  wire   [11:0] Busendcnt;      // local wire
  wire   [11:0] BusendcntC;     // clamp wire
  wire    [1:0] Busmode;        // local wire
  wire    [1:0] BusmodeC;       // clamp wire
  wire    [3:0] Busmask;        // local wire
  wire    [3:0] BusmaskC;       // clamp wire
  wire   [11:0] Bustrncnt;      // local wire
  wire   [11:0] BustrncntC;     // clamp wire
  wire          CdbgpwrupackS;  // sync  wire
  wire          CsyspwrupackS;  // sync  wire
  wire          CdbgrstackS;    // sync  wire

  wire          iSWDO;
  wire          iSWDOEN;
  wire          iTDO;
  wire          inTDOEN;
  wire [31:0]   iDAPADDR;
  wire          iDAPWRITE;
  wire          iDAPENABLE;
  wire          iDAPABORT;
  wire          iDAPSEL;
  wire [31:0]   iDAPWDATA;
  wire          iCDBGPWRUPREQ;
  wire          iCSYSPWRUPREQ;
  wire          iCDBGRSTREQ;
  wire          iJTAGNSW;
  wire          iJTAGTOP;

  assign SWDO         = (DP_PRESENT != 0) ? iSWDO         : 1'b0;
  assign SWDOEN       = (DP_PRESENT != 0) ? iSWDOEN       : 1'b0;
  assign TDO          = (DP_PRESENT != 0) ? iTDO          : 1'b0;
  assign nTDOEN       = (DP_PRESENT != 0) ? inTDOEN       : 1'b1;
  assign DAPADDR      = (DP_PRESENT != 0) ? iDAPADDR      : {32{1'b0}};
  assign DAPWRITE     = (DP_PRESENT != 0) ? iDAPWRITE     : 1'b0;
  assign DAPENABLE    = (DP_PRESENT != 0) ? iDAPENABLE    : 1'b0;
  assign DAPABORT     = (DP_PRESENT != 0) ? iDAPABORT     : 1'b0;
  assign DAPSEL       = (DP_PRESENT != 0) ? iDAPSEL       : 1'b0;
  assign DAPWDATA     = (DP_PRESENT != 0) ? iDAPWDATA     : {32{1'b0}};
  assign CDBGPWRUPREQ = (DP_PRESENT != 0) ? iCDBGPWRUPREQ : 1'b0;
  assign CSYSPWRUPREQ = (DP_PRESENT != 0) ? iCSYSPWRUPREQ : 1'b0;
  assign CDBGRSTREQ   = (DP_PRESENT != 0) ? iCDBGRSTREQ   : 1'b0;
  assign JTAGNSW      = (DP_PRESENT != 0) ? iJTAGNSW      : 1'b0;
  assign JTAGTOP      = (DP_PRESENT != 0) ? iJTAGTOP      : 1'b0;




  DAPSwjWatcher #(DP_PRESENT, JTAG_PRESENT)
    uDAPSwjWatcher
   (/*AUTOINST*/
    .JTAGNSW                            (iJTAGNSW),
    .DBGDI                              (DBGDI),
    .TMS                                (TMS),
    .SWCLKTCK                           (SWCLKTCK),
    .SWDITMS                            (SWDITMS),
    .nPOTRST                            (nPOTRST),
    .WIRERESET                          (WIRERESET),
    .JTAGTLR                            (JTAGTLR));

  DAPSwDpProtocol  #(DP_PRESENT, RESET_ALL_REGS)
    uDAPSwDpProtocol
   (
    .DBGDO                              (iSWDO),
    .DBGDOEN                            (iSWDOEN),
    .BUSREQ                             (SBusreq),
    .BUSWRITE                           (SBuswrite),
    .BUSABORT                           (SBusabort),
    .BUSADDR                            (SBusaddr[31:0]),
    .BUSWDATA                           (SBuswdata[31:0]),
    .BUSMODE                            (SBusmode[1:0]),
    .BUSMASK                            (SBusmask[3:0]),
    .BUSTRNCNT                          (SBustrncnt[11:0]),
    .CDBGPWRUPREQ                       (SCDbgPwrUpReq),
    .CSYSPWRUPREQ                       (SCSysPwrUpReq),
    .CDBGRSTREQ                         (SCDbgRstReq),
    .WIRERESET                          (WIRERESET),
    .DBGRESETn                          (nPOTRST),
    .DBGCLK                             (SWCLKTCK),
    .DBGDI                              (DBGDI),
    .BUSACK                             (BusackT),
    .BUSRDATA                           (BusrdataC[31:0]),
    .BUSCMP                             (BuscmpC),
    .BUSERROR                           (BuserrorC),
    .BUSENDCNT                          (BusendcntC[11:0]),
    .CDBGPWRUPACK                       (CdbgpwrupackS),
    .CSYSPWRUPACK                       (CsyspwrupackS),
    .CDBGRSTACK                         (CdbgrstackS));

  DAPJtagDpProtocol  #(DP_PRESENT, JTAG_PRESENT)
    uDAPJtagDpProtocol
   (
    .TDO                                (iTDO),
    .nTDOEN                             (inTDOEN),
    .BUSREQ                             (JBusreq),
    .BUSWRITE                           (JBuswrite),
    .BUSABORT                           (JBusabort),
    .BUSADDR                            (JBusaddr[31:0]),
    .BUSWDATA                           (JBuswdata[31:0]),
    .BUSMODE                            (JBusmode[1:0]),
    .BUSMASK                            (JBusmask[3:0]),
    .BUSTRNCNT                          (JBustrncnt[11:0]),
    .CDBGPWRUPREQ                       (JCDbgPwrUpReq),
    .CSYSPWRUPREQ                       (JCSysPwrUpReq),
    .CDBGRSTREQ                         (JCDbgRstReq),
    .JTAGTLR                            (JTAGTLR),
    .JTAGTOP                            (iJTAGTOP),
    .nPOTRST                            (nPOTRST),
    .nTRST                              (nTRST),
    .TCK                                (SWCLKTCK),
    .TMS                                (TMS),
    .TDI                                (TDI),
    .BUSACK                             (BusackT),
    .BUSRDATA                           (BusrdataC[31:0]),
    .BUSCMP                             (BuscmpC),
    .BUSERROR                           (BuserrorC),
    .BUSENDCNT                          (BusendcntC[11:0]),
    .CDBGPWRUPACK                       (CdbgpwrupackS),
    .CSYSPWRUPACK                       (CsyspwrupackS),
    .CDBGRSTACK                         (CdbgrstackS));

  DAPDpIMux #(DP_PRESENT, JTAG_PRESENT)
    uDAPDpIMux
   (
    .DBGCLK                             (SWCLKTCK),
    /*AUTOINST*/
    .Busreq                             (Busreq),
    .Buswrite                           (Buswrite),
    .Busaddr                            (Busaddr[31:0]),
    .Busabort                           (Busabort),
    .Buswdata                           (Buswdata[31:0]),
    .Busmode                            (Busmode[1:0]),
    .Busmask                            (Busmask[3:0]),
    .Bustrncnt                          (Bustrncnt[11:0]),
    .CDBGPWRUPREQ                       (iCDBGPWRUPREQ),
    .CSYSPWRUPREQ                       (iCSYSPWRUPREQ),
    .CDBGRSTREQ                         (iCDBGRSTREQ),
    .nPOTRST                            (nPOTRST),
    .JTAGNSW                            (JTAGNSW),
    .JBusreq                            (JBusreq),
    .JBuswrite                          (JBuswrite),
    .JBusaddr                           (JBusaddr[31:0]),
    .JBusabort                          (JBusabort),
    .JBuswdata                          (JBuswdata[31:0]),
    .JBusmode                           (JBusmode[1:0]),
    .JBusmask                           (JBusmask[3:0]),
    .JBustrncnt                         (JBustrncnt[11:0]),
    .JCDbgPwrUpReq                      (JCDbgPwrUpReq),
    .JCSysPwrUpReq                      (JCSysPwrUpReq),
    .JCDbgRstReq                        (JCDbgRstReq),
    .SBusreq                            (SBusreq),
    .SBuswrite                          (SBuswrite),
    .SBusaddr                           (SBusaddr[31:0]),
    .SBusabort                          (SBusabort),
    .SBuswdata                          (SBuswdata[31:0]),
    .SBusmode                           (SBusmode[1:0]),
    .SBusmask                           (SBusmask[3:0]),
    .SBustrncnt                         (SBustrncnt[11:0]),
    .SCDbgPwrUpReq                      (SCDbgPwrUpReq),
    .SCSysPwrUpReq                      (SCSysPwrUpReq),
    .SCDbgRstReq                        (SCDbgRstReq));

  DAPSwDpApbIf  #(DP_PRESENT)
    uDAPDpApbIf
   (
    .DAPSEL                             (iDAPSEL),
    .DAPENABLE                          (iDAPENABLE),
    .DAPWRITE                           (iDAPWRITE),
    .DAPABORT                           (iDAPABORT),
    .DAPADDR                            (iDAPADDR[31:0]),
    .DAPWDATA                           (iDAPWDATA[31:0]),
    .BUSACK                             (Busack),
    .BUSRDATA                           (Busrdata[31:0]),
    .BUSCMP                             (Buscmp),
    .BUSERROR                           (Buserror),
    .BUSENDCNT                          (Busendcnt[11:0]),
    .DAPRESETn                          (DAPRESETn),
    .DAPCLK                             (DAPCLK),
    .DAPCLKEN                           (DAPCLKEN),
    .DAPRDATA                           (DAPRDATA[31:0]),
    .DAPREADY                           (DAPREADY),
    .DAPSLVERR                          (DAPSLVERR),
    .BUSREQ                             (BusreqD),
    .BUSWRITE                           (BuswriteC),
    .BUSADDR                            (BusaddrC[31:0]),
    .BUSABORT                           (BusabortD),
    .BUSWDATA                           (BuswdataC[31:0]),
    .BUSMODE                            (BusmodeC[1:0]),
    .BUSMASK                            (BusmaskC[3:0]),
    .BUSTRNCNT                          (BustrncntC[11:0]));

  DAPDpApbSync #(DP_PRESENT)
    uDAPDpApbSync
   (
    .DAPRESETn      (DAPRESETn),
    .DAPCLK         (DAPCLK),
    .DAPCLKEN       (DAPCLKEN),
    .Busreq         (BusreqC),
    .Busabort       (BusabortC),

    .BusreqD        (BusreqD),
    .BusabortD      (BusabortD)
    );

  DAPSwDpSync #(DP_PRESENT)
    uDAPSwDpSync
   (
    .DBGRESETn      (nPOTRST),
    .DBGCLK         (SWCLKTCK),
    .Busack         (BusackC),
    .CDBGPWRUPACK   (CDBGPWRUPACK),
    .CSYSPWRUPACK   (CSYSPWRUPACK),
    .CDBGRSTACK     (CDBGRSTACK),

    .BusackT        (BusackT),
    .CdbgpwrupackS  (CdbgpwrupackS),
    .CsyspwrupackS  (CsyspwrupackS),
    .CdbgrstackS    (CdbgrstackS)
    );


`ifdef ARM_IEMSupport

  DAPDpApbIfClamp #(DP_PRESENT)
    uDAPDpApbIfClamp
   (
    .nCDBGPWRDN     (nCDBGPWRDN),
    .Busreq         (Busreq),
    .Busack         (Busack),
    .Buswrite       (Buswrite),
    .Busabort       (Busabort),
    .Busaddr        (Busaddr),
    .Busrdata       (Busrdata),
    .Buswdata       (Buswdata),
    .Buscmp         (Buscmp),
    .Buserror       (Buserror),
    .Busendcnt      (Busendcnt),
    .Busmode        (Busmode),
    .Busmask        (Busmask),
    .Bustrncnt      (Bustrncnt),

    .BusreqC        (BusreqC),
    .BusackC        (BusackC),
    .BuswriteC      (BuswriteC),
    .BusabortC      (BusabortC),
    .BusaddrC       (BusaddrC),
    .BusrdataC      (BusrdataC),
    .BuswdataC      (BuswdataC),
    .BuscmpC        (BuscmpC),
    .BuserrorC      (BuserrorC),
    .BusendcntC     (BusendcntC),
    .BusmodeC       (BusmodeC),
    .BusmaskC       (BusmaskC),
    .BustrncntC     (BustrncntC)
    );

`else  // !ifdef ARM_IEMSupport

  assign BusreqC    = Busreq;
  assign BusackC    = Busack;
  assign BuswriteC  = Buswrite;
  assign BusabortC  = Busabort;
  assign BusaddrC   = Busaddr;
  assign BusrdataC  = Busrdata;
  assign BuswdataC  = Buswdata;
  assign BuscmpC    = Buscmp;
  assign BuserrorC  = Buserror;
  assign BusendcntC = Busendcnt;
  assign BusmodeC   = Busmode;
  assign BusmaskC   = Busmask;
  assign BustrncntC = Bustrncnt;

`endif // !ifdef ARM_IEMSupport

endmodule
