module matrix_decode_t (
    hclk,
    hreset_n,

    hreadys_i,
    sel_i,
    addr_i,

    active0_i,
    readyout0_i,
    resp0_i,
    rdata0_i,

    sel0_o,

    active_o,
    hreadyouts_o,
    hresps_o,
    hrdatas_o
    );

    input         hclk;             // AHB System Clock
    input         hreset_n;         // AHB System Reset
    input         hreadys_i;        // Transfer done
    input         sel_i;            // HSEL input
    input [31:0]  addr_i;           // HADDR input

    input         active0_i;        // O/P stage0 Active signal
    input         readyout0_i;      // HREADYOUT input
    input         resp0_i;          // HRESP input
    input  [31:0] rdata0_i;         // HRDATA input

    output        sel0_o;           // HSEL output

    output        active_o;         // Combinatorial Active O/P
    output        hreadyouts_o;     // HREADY feedback output
    output        hresps_o;         // Transfer response
    output [31:0] hrdatas_o;        // Read Data


    reg           sel0;             // HSEL output

    reg           active;           // Combinatorial Active O/P signal
    reg           hreadyouts;       // Combinatorial HREADYOUT signal
    reg           hresps;           // Combinatorial HRESPS signal
    reg    [31:0] hrdatas;          // Read data bus

    reg     [3:0] addr_out_port;    // Address output ports
    reg     [3:0] data_out_port;    // Data output ports


  always @ (addr_i)
    begin : p_addr_out_port_comb
      case (addr_i [27:24])

        4'b0000 : addr_out_port = 4'b0000;
        default : addr_out_port = 4'b0000;
      endcase // case(Addr [27:24])
    end // block: p_AddrOutPortComb


  always @ (sel_i or addr_out_port)
    begin : p_sel_comb
      sel0  = 1'b0 ;
      if  (sel_i)
      begin
        case (addr_out_port)
          4'b0000 : sel0  = 1'b1 ;
          default: begin
          end
        endcase // case(AddrOutPort)
      end // if (Sel)
    end // block: p_SelComb

  always @ (active0_i or
            addr_out_port)
    begin : p_active_comb
      case (addr_out_port )

        4'b0000 : active  = active0_i ;
        default : active  = active0_i ;
      endcase // case(AddrOutPort )
    end // block: p_ActiveComb

  always @ (posedge hclk or negedge hreset_n)
    begin : p_data_out_port_seq
      if  (hreset_n == 1'b0)
        data_out_port  <= 4'b0000;
      else
        begin
          if  (hreadys_i)
            data_out_port  <= addr_out_port ;
        end
    end // block: p_DataOutPortSeq

  always @ (readyout0_i or
            data_out_port)
  begin : p_ready_comb
    case (data_out_port)

      4'b0000 : hreadyouts  = readyout0_i ;
      default : hreadyouts  = readyout0_i;
    endcase // case(DataOutPort)
  end // block: p_ReadyComb


  always @ (resp0_i or
            data_out_port)
  begin : p_resp_comb
    case (data_out_port)
      4'b0000 : hresps  = resp0_i ;
      default: hresps  = resp0_i;
    endcase // case(DataOutPort)
  end // block: p_RespComb

  always @ (rdata0_i or
            data_out_port)
  begin : p_rdata_comb
    case (data_out_port )

      4'b0000 : hrdatas  = rdata0_i ;
      default : hrdatas  = rdata0_i ;
    endcase // case(DataOutPort )
  end // block: p_RdataComb

  assign sel0_o        = sel0;
  assign active_o      = active;
  assign hreadyouts_o  = hreadyouts;
  assign hresps_o      = hresps;
  assign hrdatas_o     = hrdatas;

endmodule
